#!/usr/bin/perl

=head1 PBMT configuration

=begin html
This script allows you to configure the parameter configuration file of PBMT in an interactive way.

=end html

=cut

#==============================================================================
#-- File:         Config.pl
#-- Date:         24.09.2007
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to configure PBMT
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2007/19/24 : First implementation
#--   1.0.1 - 2007/09/25 : ANTms65540: Encrypted parameters may contain '=' characters.  Adapt parsing to this
#--   1.0.2 - 2007/09/27 : Adapted to completely generate param.cfg.  It is no longer present as a default file.
#--   1.0.3 - 2008/05/14 : chmod 666 of param.cfg once it has been written.
#--   1.0.4 - 2008/05/26 : ANTmt08304.  Use default values for a parameter commented out in param.cfg.
#--   1.0.5 - 2008/06/24 : ANTmt13249 (or clone).  Include actual and new CDE file name in the default options.
#--   1.0.6 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--                        Don't leave an empty param.cfg file when interupting the update process.
#--   1.0.7 - 2009/11/11 : introduction of undocumented option (-cdedir) for ATX
#--   1.0.8 - 2010/03/04 : PBMT x86 Changes - ANTmt88407
#--   3.0.1 - 2011/01/11 : LRM updates for lib-perl path
#--   3.0.2 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.3 - 2011/05/02 : ALU01178887
#--   3.0.4 - 2013/04/29 : ALU01880147
#--   3.0.5 - 2016/07/15 : ALU02237966
#--   3.0.6 - 2016/09/28 : ALU02278259


=head1 VERSION

=begin html
Current version: "Config.pl - 3.0.06- 2016/09/28"

=end html

=cut

my $VERSION = "Config.pl - 3.0.06 - 2016/09/28";
#==============================================================================

use strict;
require 5.6.0;
use warnings 'all';

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li>Utils</li>
      <li>version</li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
#==============================================================================
#== Find the lib dir
use Cwd qw( realpath);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone"; 
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 

#==============================================================================

use Utils;
use Crypt::Lite;
use version qw(:ALL);
use ToolCommon();
use PBMTGlobal;
use PbmtConfig();
use ErrorCodes qw(:EXIT_CODES);
#==============================================================================
#== Prototypes
#==============================================================================
sub FileExists();
sub ParseCommandLine();

my $ownPid = $$;
my $warnCount = 0;
my $makeNew = 0;

=begin html

=head1 FUNCTIONS

=head2 FileExists()

=begin html
Returns 1 in case the parameter configuration file already exists in the installation directory, 0 in case it is not there.

=end html

=cut

sub FileExists() {
  my $dirName = dirname($0);
  return (-e "$dirName/param.cfg");
}

=head2 ParseCommandLine()

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine() {
  my ($debugOn, $useDefaults);
  my $cdeDirOpt = undef; 
  my $options = GetOptions('debug', \$debugOn, 'default=s', \$useDefaults, 'cdedir=s',\$cdeDirOpt); 
  my %params = ();

  if (! $options) {
    print STDERR "Incorrect parameter usage\n";
    exit 0;
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  $params{DEBUG} = defined($debugOn) ? 1 : 0;
  if (defined($useDefaults)) {
    my @arr = split(':', $useDefaults);
    my $arrLength = scalar @arr;
    if ($arrLength < 5) {
      print STDERR "Invalid default setting.  Expecting cliuser:clipwd:tl1user:tl1pwd:snmpcommunity[:actual CDE file name:new CDE filename]\n";
      exit 0;
    }
    $params{DEFAULT} = $useDefaults;
    $params{CDEDIROPT} = $cdeDirOpt;
  }
  return \%params;
}

# Main code

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

$SIG{"__WARN__"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();
  if (!($msgStr =~ /Zlib/)) {
    print STDERR "Warning [package '$package', file: '$fileName', line:$line]: $msgStr\n";
    $warnCount++;
  }
};
$SIG{"__DIE__"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  exit EXIT_DIED_CODE;
};
$SIG{"INT"} = sub {
  my $dirName = dirname($0);

  if ($makeNew) {
    my $deleted = unlink("$dirName/param.cfg");
    print STDERR "\nReceived ^C, removed param.cfg under construction (deleted '$deleted' file)\n";
  } else {
    my $deleted = unlink("$dirName/param.cfg");
    if (!rename("$dirName/param.cfg.sav", "$dirName/param.cfg")) {
      print STDERR "\nReceived ^C and failed to rename $dirName/param.cfg (no param.cfg file left any longer): $! (deleted '$deleted' file)\n";
    } else {
      print STDERR "\nReceived ^C and restored saved param.cfg (deleted '$deleted' file)\n";
    }
  }
  exit EXIT_SIGNAL_CODE;
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  exit EXIT_DIED_CODE;
};
$SIG{"PIPE"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    print STDERR "SIGPIPE received (ignored): $msgStr";
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	print STDERR sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined");
	$i++;
      } else {
	$framePresent = 0;
      }
    }
  }
  exit EXIT_DIED_CODE;
};

#-- Read parameter config file and adjust tool defaults if needed -- BB--

STDOUT->autoflush(1);
STDERR->autoflush(1);

$params = ParseCommandLine();
PbmtConfig::AdjustToolVars();

my $umask = umask();
print STDERR sprintf("umask = %o\n", $umask) if $params->{DEBUG};

if ($params->{DEFAULT}) {
  PbmtConfig::DefaultInstall();
} else {
  print sprintf("%s -- Version: %s::%s (%s)\n", $0, PBMT_VERSION, $VERSION, PbmtConfig::GetVersion());
  print "PBMT parameter file configuration utility.\n";
  if (FileExists()) {
    print "\n";
    # File already exists.  Build list of new parameters and prompt for these
    PbmtConfig::UpdateConfig();
  } else {
    # New install: run the complete initialisation
    $makeNew = 1;
    PbmtConfig::NewInstall();
  }
}

exit 0;
