#!/usr/bin/perl

=head1 PBMT SW minimizer

=begin html
This PERL module implements minimizing of a SW package as stored on an ftp server.  This script is based on the
on-line SW reduction module.

=end html

=cut

#################################################################################
#-- File:         MinimizeSWP.pl                                                #
#-- Date:         23.03.2006                                                    #
#-- Company:      Wipro Technologies                                            #
#-- Author:       Nabanita Nayak                                                #
#################################################################################     
#-- Function:     Script to reduce the OSWP (for R3.0 onwards) by offline       #
#                 and download it onto the sandisk                              #
#-- MinimizeSWP:                                                                #
#                 Read the descriptor file, reduce it offline according to the  #
#                 plugged in boards and download it to the sandisk.             #
#                                                                               #
#                                                                               #
#################################################################################
#--   1.0.1  - 16112006: ANTms19172 - don't copy a board when blank line has been
#--                      read from the config file.
#--   1.0.2  - 26032007 : Ignore case in board config file: convert to UC.
#--   1.0.3  - 31052007 : ANTms49179, skip blank lines in board description section
#--   1.0.4  - 03062007 : ANTms49521, SWP file corrupted.  Re-arranged SWP parsing.
#--   1.0.5  - 04062008 : ANTmt10004. Remove leading space when check "END" for noard
#--                                   section in SWP file.
#--   1.0.6  - 26052009 : ANTmt59954.
#--   1.0.7  - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.8  - 2010/03/04 : ANTmg01050 - Total_MiB label change from Ex.x.y.z to Vx.x.x.y.z
#--   3.0.00 - 2010/05/24 : Version converted to 3.0.00 for Converged PBMT
#--   3.0.01 - 2010/06/23 : Converged PBMT Changes
#--                       : x86 Changes
#--   3.0.02 - 2011/01/07 : LRM updates for lib-perl path
#--   3.0.03 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.04 - 2011/05/02 : ALU01178887
#--   3.0.05 - 2011/10/24 : logfile command line parameter can include a directory
#--   3.0.06 - 2012/05/11 : ALU01789702 - World writeable directories need sticky bit set 
#--                       : ALU01790810
#--   3.0.07 - 2013/04/29 : ALU01880147
#--   3.0.08 - 2014/02/05 : ALU01956151 
#--   3.0.09 - 2015/08/04 : ALU02104915
#--   3.0.10 - 2015/08/10 : ALU02112322
#--   3.0.11 - 2015/08/21 : ALU02115198
#--   3.0.12 - 2015/10/28 : ALU02139820
#--   3.0.13 - 2015/11/25 : ALU02146451
#--   3.0.14 - 2016/04/28 : ALU02206611
#--   3.0.15 - 2016/05/13 : ALU02202972, ALU02213657
#--   3.0.16 - 2016/05/23 : ALU02215557
#--   3.0.17 - 2016/07/15 : ALU02237966
#--   3.0.18 - 2016/07/29 : ALU02250240
#--   3.0.19 - 2016/09/28 : ALU02278259
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

##################################################################################

=head1 VERSION

=begin html
Current version: "MinimizeSWP.pl -- 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "MinimizeSWP.pl -- 3.1.0 -- 2018/06/20";
#==============================================================================


=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>Getopt::Long</li>
      <li>FileHandle</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Sys::Hostname</li>
      <li>File::Basename</li>
      <li>File::Find</li>
      <li>File::Copy</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use  strict;
require 5.6.0;
use warnings 'all';

use Getopt::Long;
use FileHandle;
use POSIX ":sys_wait_h";
use Sys::Hostname;
use File::Find;
use File::Copy;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir -- 1.0.6
use Cwd qw( realpath getcwd);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}
use lib "$scriptDir/lib"; 

use PBMTGlobal;
use Logger qw(:ALL);
use Net::SNMP;
use Crypt::Lite;
use Utils;
use ErrorCodes qw(:EXIT_CODES :SWMM);
use ToolCommon qw(:CONSTANTS);
use version qw(:ALL);
use PbmtConfig;
use Capability;
use SNMPComm;
#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$);
sub GetSpace($);
sub GetColumn($$$$$$);
sub AdjustToolVars($);

# Constants

# Common variables
my $MM_CHAR = "Z";
my $DESCRIPTOR = "DESCRIPTOR-FILE";
my $BOARDSECTION = "DESCRIPTION";
my $ENDSECTION = "END";
my $NOT_STARTED = 0;
my $HEADER_STARTED = 1;
my $BOARD_STARTED = 2;
my $boardName = "";
my @phases = ("Not started", "Header section started", "Board section started");
my $minAssign = 0; 

my %config;

my %toolData = (
                TFTPDIR =>         "/var/opt/aws/equipment",
                NT_COMMUNITY =>    "public",
              );

=head1 USAGE

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] MinimizeSWP.pl --isam <ip> | --config <file> --swp <path-to-oswp-descriptor> --logfile <file> \ <br>
                          [--v] [--debug] [--target &lt;dir&gt;] [--minset]</tt>

=end html

=head2 Commmand options

=begin html
<table>
<table>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>swp</td><td>path to oswp descriptor file (relative to TFTP root directory)</td></tr>
  <tr><td>config</td><td>board configuration file to be used when creating new OSWP/SWP</td></tr>
  <tr><td>debug</td><td>Additional debugging required or not</td></tr>
  <tr><td>target</td><td>Store new OSWP and SWP in the location as specified here (relative to tftp root directory)</td></tr>
  <tr><td>minset</td><td>force boards to be in the minimum set, i.e. all SW will be downloaded (default: not in minimum set)</td></tr>
</table>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 [--v] --isam <ip> | --config <filename> --swp <path-to-oswp-descriptor>\\\n";
  print STDERR "       [--name <oswp-descriptor-prefix>] [--debug] --logfile <file> [--target <dir>] [--minset]\\\n";
  print STDERR "       [--comm <SNMP community string>] [--v2]  \\ \n";
  print STDERR "       [--v3auth <user>:<password>:<authProtocol>] [-v3priv <privProtocol>:<password>] \\ \n";
  print STDERR "       [--bytpe  <value>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         v      : print version and exit\n";
  print STDERR "         swp    : path to oswp descriptor file\n";
  print STDERR "         config : file name of file containing requested board mnemonics\n";
  print STDERR "         name   : OSWP descriptor filename prefix (first 4 alphanumeric characters Eg.UVSE)\n";
  print STDERR "         target : target directory\n";
  print STDERR "         minset : force boards to be in the minimum set, i.e. all SW will be downloaded (default: not in minimum set)\n";
  print STDERR "         comm     : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is \"public\"\n";
  print STDERR "         v2       : Use SNMPv2c\n";
  print STDERR "         v3auth   : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                    The password should be at least 8 characters long.\n";
  print STDERR "                    Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                    All fields are mandatory when option is used.\n";
  print STDERR "                    e.g. --v3auth user:password:: will result in MD5 as protocol.\n";
  print STDERR "         v3priv   : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                    Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  print STDERR "         btype    : Build type to choose XDescriptor; Possible values are SNMP, YANG\n";

  exit CMD_INCOMPL_CODE;
}

=head2 ParseCommandLine()

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my $config = shift @_;
  my ($isam, $version, $btype, $boardFile, $logFile, $verbose, $swp, $target, $minSet, $name, $snmpV2, $communityString, $v3Auth, $v3Priv);
  my $options = GetOptions('isam=s', \$isam,
			   'v', \$version,
			   'config=s', \$boardFile,
			   'logfile=s',\$logFile,
			   'btype=s',\$btype,
			   'debug',\$verbose,
			   'swp=s',\$swp,
			   'target=s', \$target,
			   'name=s', \$name,
			   'minset', \$minSet,
			   'v2', \$snmpV2,
                           'comm=s', \$communityString,
			   'v3auth=s', \$v3Auth,
	                   'v3priv=s', \$v3Priv
                         );
  my %params;
  #-- check options
  if (! $options) {
    print STDERR "Failed to parse options\n";
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($isam) && !defined($boardFile)) {
    print STDERR "Either an IP address or board config file has to be specified\n";
    Usage();
  }
  if (!defined($logFile)) {
    print STDERR "ERROR: no logfile given\n";
    Usage();	
  }
  if (!defined($swp)) {
    print STDERR sprintf("ERROR: no OSWP specified\n");
    Usage();
  }
  my $baseName = basename($swp);
  my $dirName = dirname($swp);
  my $regExp = REGEXP;
  if (!($baseName =~ /$regExp/)) {  # -- 1.0.4
    print STDERR sprintf("Overall software package name (%s) not conforming to regexp (%s).  Check param.cfg!!!\n",
			 $baseName, $regExp);
    exit PARAM_CFG_NOT_OK_CODE;
  }
  if( defined($btype) ) {
      if( $btype ne "SNMP" and $btype ne "YANG" ) {
        print STDERR "ERROR: Invalid value passed for --bytpe; possible values are SNMP, YANG\n";
        Usage();
      } else {
        $params{BTYPE} = $btype;
      }
  }
  $params{BOARDFILE} = $boardFile if (defined($boardFile));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if(defined($verbose));
  $params{OSWP} = $swp if(defined($swp));
  $params{LOGFILE} = $logFile;
  if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
    my $logdir = $1;
    if (! -d $logdir) {
      if (!mkdir( $logdir, 0777)) {
        print STDERR "Cannot create directory '$logdir': $!\n";
        exit CREATE_DIR_FAILED_CODE;
      }
    }
  }
 
  #if($name !~ /[:alphanum:]{4}/) {
  if(defined($name)) {
	  if($name !~ /^[a-zA-Z0-9]{4}$/) {
	    print STDERR "OSWP descriptor prefix not in prescribed format\n"; 
            Usage(); 
	  } else {
	    #print STDERR "OSWP descriptor prefix s OK ...\n"; 
	    $params{OSWPPREFIX} = $name if (defined($name));
	  } 
  }

  $params{TARGET} = $target if (defined($target));
  my $oswp2Consider = $swp;
  $params{NOTFTP} = 0;
  # First try the file as specfied (could be an absolute path when using ftp e.g.
  if (! -r $oswp2Consider) {
    # Next try the SWP relative to the login home directory
    $oswp2Consider = "$ENV{HOME}/$swp";
    $oswp2Consider =~ s/\/\//\//g;
    if (! -r $oswp2Consider) {
      # Finally check as tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $oswp2Consider =~ s/\/\//\//g;
      if (! -r  $oswp2Consider) {
	print STDERR "ERROR: Accessing OSWP '$oswp2Consider' failed: $!\n";
	Usage();
      } else {
	$params{OSWP_HOME} = $toolData{TFTPDIR};
	$params{NOTFTP} = 0;
      }
    } else {
      $params{OSWP_HOME} = $ENV{HOME};
      $params{NOTFTP} = 1;
    }
  } else {
    $params{OSWP_HOME} = "";
    $params{NOTFTP} = 1;
  }
  $params{OSWP_FULL} = $oswp2Consider;
  $regExp = SWPSYNTAX;
  if ($swp =~ /$regExp/) {
    print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_INV_CHARS_CODE));
    exit OSWP_FILE_INV_CHARS_CODE;
  }
  my $tftpInfo = Utils::FindTftpInfo(undef, $oswp2Consider);
  if (!defined($tftpInfo->{PRIMIP}) || ($tftpInfo->{PRIMIP} eq "")) {
    print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_NO_IP_CODE));
    exit OSWP_FILE_NO_IP_CODE;
  }


  
  my $newSwpBaseName = basename($tftpInfo->{NAME});
  $newSwpBaseName =~ s/(....).(.......)/$1$MM_CHAR$2/;
  $params{NEWSWPPATH} = defined($target) ? "$params{OSWP_HOME}/$target" : "$params{OSWP_HOME}/" . dirname($tftpInfo->{NAME});
  $params{PRIMTFTPIP} = $tftpInfo->{PRIMIP};
  $params{SECTFTPIP} = $tftpInfo->{SECIP};
  $params{SWPNAME} = $tftpInfo->{NAME};
  $params{NEWSWPNAME} = $newSwpBaseName;
  $params{MINSET} = defined($minSet) ? 1 : 0;
  my $newOswpName = basename($swp);
  $newOswpName =~ s/(....).(.......)/$1$MM_CHAR$2/;
  $params{NEWOSWPPATH} = defined($target) ? "$params{OSWP_HOME}/$target" : dirname($oswp2Consider);
  $params{NEWOSWP} = $newOswpName;
  my $prefixNewOswp = $newOswpName; 
  $prefixNewOswp =~ s/(....)(........)/$params{OSWPPREFIX}$2/   if(defined($params{OSWPPREFIX})); ;
  $params{PREFIXNEWOSWP} = $prefixNewOswp ;
  if (defined($isam)) {
    my @addr = gethostbyname($isam);
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $params{ISAM} = $isam;
    $params{ISAMIP} = $ipAddr;
  }
  my $oswpPath;
  $oswpPath = "$params{OSWP_HOME}/$params{SWPNAME}";
  if($params{NOTFTP} == 1) {
    if($params{OSWP_HOME} ne "") {
        $oswpPath = "$params{SWPNAME}";
        $params{OSWP_HOME} = "";
    }
  }
  $params{SWPPATH} = $oswpPath;

  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
  if (!defined($snmpV2)) { #ALU02028455 #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }

  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
        if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA512|SHA256|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
          $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
          $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
          $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
          if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
             $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
        } else {
          print STDERR "SNMPv3 authentication protocol can only be SHA or SHA256 or SHA512 or MD5.\n";
          Usage();
        }
      } else {
        print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
        Usage();
      }

    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
          $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n", scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
    $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "";
    if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
       $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {
     my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
        if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
            (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
          $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
          $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
        } else {
          print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
          Usage();
        }
      } else {
        print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
        Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
      if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
           $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
           $params{SNMP}{V3}{PRIV}{PASSWORD} = defined($toolData{SNMPV3_PRIV_PASSWORD}) ? $toolData{SNMPV3_PRIV_PASSWORD} : "";
      }
  } #ALU01524313-indhu-15/12/2011
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }

  return \%params;
}

=head2 PrintLibVersions()

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut
sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw( 
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability 
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

=head2 PrintOptionsSummary()

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$) {
  my($argLogFH, $logOnScreen) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{BOARDFILE}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Inclde boards in minimum set" , 
			       ($params->{MINSET}) ? "Minset Enabled" :  "Minset Disabled" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Board config file" , $params->{BOARDFILE} ) ) if defined($params->{BOARDFILE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "XDescriptor build type" , $params->{BTYPE} ) ) if defined($params->{BTYPE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "OSWP Descriptor file" , $params->{OSWP} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- SWP Name" , $params->{SWPNAME} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- DB Version" , $params->{DBVER_TGTREL} ) ) if defined($params->{DBVER_TGTREL});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Target OSWP Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Provided target path" , $params->{TARGET} ) ) if( defined $params->{TARGET} );
  if(defined($params->{OSWPPREFIX})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Prefix for new OSWP Name" , $params->{OSWPPREFIX} ) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Target OSWP Descriptor file" , "$params->{NEWOSWPPATH}/$params->{PREFIXNEWOSWP}" ) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Target OSWP Descriptor file" , "$params->{NEWOSWPPATH}/$params->{NEWOSWP}" ) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Target SWP Name" , "$params->{NEWSWPPATH}/$params->{NEWSWPNAME}" ) );
  } 

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Transaction IP Address" , 
			       defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} :  $params->{PRIMTFTPIP}) );
            
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "FTP Details :-") );
  if (! $params->{NOTFTP}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "TFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- TFTP (HOME) Directory" , $toolData{TFTPDIR}) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "SFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );

}

sub DynamicOSWPVariant($$$) {
    my ($execLogFH, $config, $ntType) = @_;
    if( defined $params->{SWPPATH} and $params->{SWPPATH} ne "" ) {
        if( defined $config->{NT_SWP_MAPFILE} and $config->{NT_SWP_MAPFILE} ne "" ) {
           my $ntDescriptorMapFile = dirname($params->{SWPPATH}) . "/" . $config->{NT_SWP_MAPFILE};
           if( ! ( -f $ntDescriptorMapFile ) ) {
                Logger::LogNotify($execLogFH, "NT SWP Mapping file :$ntDescriptorMapFile: not exists\n");
                return 0;
           }
           Capability::SetVariantCheckSkip(1);
           Logger::LogTrace($execLogFH, "NT Descriptor mapping file($ntDescriptorMapFile) process started..\n");
           my( $status, $ntDescriptorFile) = Utils::FetchNTDescriptor($ntDescriptorMapFile, $ntType);
           Logger::LogTrace($execLogFH, "Details : $status, $ntType, $ntDescriptorFile\n");
           if( $status != 0 ) {
                   Logger::LogError($execLogFH, "ERROR: NTDescriptor Mapfile : $ntDescriptorFile\n");
                   exit EC_CONF_FILE_READ_FAILED;
           }
           Logger::LogNotify($execLogFH, "XDescriptor build type:$params->{BTYPE}:\n") if defined($params->{BTYPE});
           my $regExp = REGEXP;
           if (!($ntDescriptorFile =~ /$regExp/)) {  # -- 1.0.4
                   my $errMesg = "Software package name ($ntDescriptorFile, read from NT Descriptor Mapfile) not conforming to regexp (".REGEXP.") as per param.cfg\n";
                   Logger::LogError($execLogFH, "ERROR : $errMesg"); 
                   exit PARAM_CFG_NOT_OK_CODE;
           }
           Logger::LogNotify($execLogFH, "NT DescriptorFile($ntDescriptorFile) found..\n");
           my( $oswpVariantU,$oswpVariantL, $oswpRelease) = ("","","");
           my( $ntDescriptorVariantU,$ntDescriptorVariantL, $ntDescriptorRelease ) = ("","","");
           my( $oswpRegexpMatch, $ntDescriptorRegexpMatch ) = (0,0);
           my $baseOSWP = basename($params->{OSWP});
           my $regexpVariantRelease = VARIANT_RELEASE_REGEXP;
           if( $baseOSWP =~ /$regexpVariantRelease/ ) {
                $oswpVariantU = $1; 
                $oswpVariantL = $2; 
                $oswpRelease = $3;
                $oswpRegexpMatch = 1;
           }
           if( $ntDescriptorFile =~ /$regexpVariantRelease/ ) {
                $ntDescriptorVariantU = $1;
                $ntDescriptorVariantL = $2;
                $ntDescriptorRelease = $3;
                $ntDescriptorRegexpMatch = 1;
           }
           if( $oswpRegexpMatch and $ntDescriptorRegexpMatch ) {
              if( $baseOSWP !~ /(AA|AN|$ntDescriptorVariantU$ntDescriptorVariantL)(\d\d\.\d\d\d)/ ) {
                 Logger::LogError($execLogFH, "ntDescriptor($ntDescriptorFile), OSWP($baseOSWP)Variant Release Details are not same"); 
                  exit EC_NTTYPE_OSWP_FAILED;
              } else {
	          Logger::LogTrace($execLogFH, "OSWP($baseOSWP), ntDescriptor($ntDescriptorFile) match the regex..\n");
              }
           }
           my( $swpVariantU,$swpVariantL, $swpRelease, $swpRegexpMatch, ) = ("","", 0);
           ($ntDescriptorVariantU,$ntDescriptorVariantL, $ntDescriptorRelease,$ntDescriptorRegexpMatch ) = ("","", 0);
           my $baseSWP = basename($params->{SWPNAME});
           if( $baseSWP =~ /$regexpVariantRelease/ ) {
                $swpVariantU = $1;
                $swpVariantL = $2;
                $swpRelease = $3;
                $swpRegexpMatch = 1;
           }
           if( $ntDescriptorFile =~ /$regexpVariantRelease/ ) {
                $ntDescriptorVariantU = $1;
                $ntDescriptorVariantL = $2;
                $ntDescriptorRelease = $3;
                $ntDescriptorRegexpMatch = 1;
           }
           if( $swpRegexpMatch and $ntDescriptorRegexpMatch ) {
              if( $baseSWP =~ /($ntDescriptorVariantU$ntDescriptorVariantL)(\d\d\.\d\d\d)$/ ) {
	        Logger::LogNotify($execLogFH, "NT DescriptorFile($ntDescriptorFile) and SWP basename($baseSWP) are same, No need to update..\n");
	        return 0;
              } else {
	          Logger::LogNotify($execLogFH, "SWPNAME($baseSWP), ntDescriptor($ntDescriptorFile) are not same, Need to update..\n");
              }
              my $newSWPNAME = "";
              if( dirname($params->{SWPNAME}) ne "" and dirname($params->{SWPNAME}) ne "." ) {
                $newSWPNAME = dirname($params->{SWPNAME}) . "/";
              }
              $newSWPNAME .=  $ntDescriptorFile;
              my $newSWPFile = $newSWPNAME;
              if( $params->{NOTFTP} == 0 ) {
                    $newSWPFile = $toolData{TFTPDIR} . "/" . $newSWPNAME;
              }
              if( ! -e $newSWPFile ) {
                   FamilyCommon::IncrErrorCount("ERROR : NT-SWP Map, SWP file ($newSWPFile) not exists\n",
                       EC_NTTYPE_SWPNAME_FAILED,
                       undef,
                       1);
                   return 1;
              }

              if( defined $params->{OSWP_FULL} and $params->{OSWP_FULL} ne "" ) {
                 Logger::LogTrace($execLogFH, "Updated new OSWP NT:$ntType:, based:$newSWPNAME:\n");
                 #$params->{OSWP} = $newOSWP;
                 $params->{SWPNAME} = $newSWPNAME;
                 Logger::LogInfo($execLogFH, "NT-SWP Map : SWP name updated as :$params->{SWPNAME}:\n" ); 
                 return 0;
              }
           } else {
		Capability::SetVariantCheckSkip(0);
           }

        }
    }
    return 0;
}


sub GetSpace($) {
  my ($path) = @_;
  my $size = 0;

  find sub { $size += -s },$path;
  return $size;
} 

=head2 GetColumn()

=begin html
Get a column defined by $oid using the opened SNMP session context $session for the NE defined by $asam.  $verbose indicates whether additional
debugging needs to be switched on (in which case $desc represents a description of the OID being used in this get_table request.  Debug info is
written to the file identified by $execLogFH).<br>
A reference to the values read is returned in case of success otherwise the undefined value is returned.

=end html

=cut

sub GetColumn($$$$$$) {
  my ($execLogFH, $ip, $session, $oid, $descr, $debug) = @_;   # -- 2.0.0

  if (defined($descr)) {
    Logger::LogDebug($execLogFH, "Getting $descr ($ip) ...\n");
  }
  my $result = $session->get_table( $oid);
  if( !defined( $result)) {
    Logger::LogError($execLogFH, "Get $descr FAILED ($ip)\n  ERROR: " .  $session->error() ."\n");   # -- 2.0.0
    return undef;
  }
  return $result;
}

=head2 AdjustToolVars()

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file).

=end html

=cut

sub AdjustToolVars($) {
  my $config = shift @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
                                                                                 ($config->{NT_COMMUNITY} ne ""));
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});

}

#Main Program
$execLogFH = undef;

#-- Read parameter config file and adjust tool defaults if needed -- BB--
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
%config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}


AdjustToolVars(\%config);
$params = ParseCommandLine(\%config);

my $logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", $$));

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1);
Net::SNMP->snmp_debug(0x3e, $execLogFH) if ($params->{DEBUG});   # send SNMP debug info to the child logfile first

my @a_boards; 
my %a_boards_minset;
#-- open snmp session

if (defined($params->{ISAM})) {
    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $params->{ISAM} ); 
    if( $snmpStatus ) {
	      FamilyCommon::IncrErrorCount( "FAILED : ".SNMPComm::GetError(),
	        		                    EC_SNMP_OPEN_FAIL, ": ".SNMPComm::GetError(), 1);
	      next;
    }
    my $mibVersion = SNMPComm::GetValue( 'MIB VERSIO', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         FamilyCommon::IncrErrorCount(sprintf("  FAILED to obtain MIB version: %s\n", SNMPComm::GetError() ),
	       		   EC_NO_MIB_VERSION, sprintf(": %s", SNMPComm::GetError() ), 1);
         next;
    }
    ToolCommon::SetDebug($execLogFH);

  ######## Getting Planned And Iserted Boards######## 

  Logger::LogInfo($execLogFH, "Getting planned and inserted boards ...\n");

  my $plannedBoards = GetColumn($execLogFH, $params->{ISAMIP}, $session,
				PLANNEDBOARD_5X_OID, "PlannedBoards", $params->{DEBUG});    # -- 2.0.0
  my $insertedBoards = GetColumn($execLogFH, $params->{ISAMIP}, $session,
				 INSERTEDBOARD_5X_OID, "Inserted Boards", $params->{DEBUG});   # -- 2.0.0
  my $activeStatus = GetColumn($execLogFH, $params->{ISAMIP}, $session,
			       ACTIVEOSWP_5X_OID, "ActiveOSWPState", $params->{DEBUG});   # -- 2.0.0

  if (defined($plannedBoards) && defined($insertedBoards) && defined($activeStatus)) {  # -- 2.0.0
    my @oid;
    my $slot;
    my $cnt=0;
    foreach (Net::SNMP::oid_lex_sort(keys(%{$plannedBoards}))) {                                                                    
      @oid = split( /\./, $_);
      $slot = $oid[-1];
      my $fullInsertedOidStr = INSERTEDBOARD_5X_OID . ".$slot";
      my $fullPlannedOIdStr = PLANNEDBOARD_5X_OID . ".$slot";
      next if( ($insertedBoards->{$fullInsertedOidStr} eq "EMPTY" ||                                  
		$insertedBoards->{$fullInsertedOidStr} eq "NOT_ALLOWED" ||
		$insertedBoards->{$fullInsertedOidStr} eq "UNKNOWN") &&
	       ($plannedBoards->{$fullPlannedOIdStr} eq "ANY_BOARD" ||                            
		$plannedBoards->{$fullPlannedOIdStr} eq "NOT_PLANNED"));                          

      if ($plannedBoards->{$fullPlannedOIdStr} ne "ANY_BOARD" &&
	  $plannedBoards->{$fullPlannedOIdStr} ne "NOT_PLANNED") {
	$a_boards[$cnt] = $plannedBoards->{$fullPlannedOIdStr};
	$cnt++;
      }
      if ($insertedBoards->{$fullInsertedOidStr} ne "EMPTY" && 
	  $insertedBoards->{$fullInsertedOidStr} ne "NOT_ALLOWED" &&
	  $insertedBoards->{$fullInsertedOidStr} ne "UNKNOWN" &&
	  $insertedBoards->{$fullInsertedOidStr} ne $plannedBoards->{$fullPlannedOIdStr}) {
	$a_boards[$cnt] = $insertedBoards->{$fullInsertedOidStr};
	$cnt++;
      }
      chomp();                                          
    }
  }
} else {
  Logger::LogInfo($execLogFH, "Getting boardlist from config file ...\n");
  if (! open(CONFFILE, $params->{BOARDFILE}) ) {
    Logger::LogError($execLogFH, "Can't open $params->{BOARDFILE}: $!\n");
    exit EC_CONF_FILE_READ_FAILED;
  } else {
    my $boardName = undef; 
    while (<CONFFILE>) {
      chomp;
      push @a_boards, uc($_) if ($_ ne "");
      
      if(defined($_) && ($_ =~ /\w+\-\w\s*$/)) {
         (uc($_) =~ /NT/) ? ($a_boards_minset{uc($_)} =  0) :  ($a_boards_minset{uc($_)} =  1);
      }
      if(defined($_) && ($_ =~ /(\w+\-\w)\s*\,\s*(\w)/)) {
        my $brd = $1; 
        my $ind = $2; 
        ($_ !~ /NT/) ? ($a_boards_minset{uc($brd)} = (uc($ind) eq 'N') ?  0 :  1) : ($a_boards_minset{uc($brd)} = 0) ;  
        $minAssign = 1; 
      } 

    }
    close(CONFFILE);
  }
}

my $ntCount = 0;  
my $ntType = "";
my %ntTypes = ();
Logger::LogDebug($execLogFH, "The available boards are:\n");
my $index = 0;
while ($index < scalar(@a_boards)) {
  Logger::LogDebug($execLogFH, "'$a_boards[$index]'\n");
  if( ($a_boards[$index] =~ /NT/ ) and ( $a_boards[$index] =~ /^\s*(\w+\-\w+)\s*/ ) ) {
      next if( $1 eq "" );
      $ntType = $1;
      if( defined $ntTypes{$ntType} ) {
            $ntTypes{$ntType}=$ntTypes{$ntType}+1;
      } else {
            $ntTypes{$ntType}=1;
      }
  }
  $index++;
}
my $boardList = join(" ", @a_boards);

$ntCount = scalar(keys %ntTypes);
my $ntBoardList = join(" ", keys %ntTypes );
Logger::LogDebug($execLogFH, "NT board count = $ntCount, NT board list = $ntBoardList\n");

if( defined $params->{SWPPATH} and $params->{SWPPATH} ne "" ) {
    if( defined $config{NT_SWP_MAPFILE} and $config{NT_SWP_MAPFILE} ne "" ) {
       my $ntDescriptorMapFile = dirname($params->{SWPPATH}) . "/" . $config{NT_SWP_MAPFILE};
       if( -f $ntDescriptorMapFile  ) {
          Logger::LogTrace($execLogFH, "NT-SWP mapping file($ntDescriptorMapFile) descriptor check started..\n");
          foreach $ntType ( keys %ntTypes ) {
               my( $status, $ntDescriptorFile) = Utils::FetchNTDescriptor($ntDescriptorMapFile, $ntType);
               Logger::LogTrace($execLogFH, "Details : $status, $ntType, $ntDescriptorFile\n");
               if( $status != 0 ) {
                       Logger::LogInfo($execLogFH, "ERROR: NTDescriptor Mapfile : $ntDescriptorFile\n" );
                       exit EC_CONF_FILE_READ_FAILED;
               }
               $ntTypes{ $ntType } = $ntDescriptorFile;
               Logger::LogDebug($execLogFH, "ntType:$ntType:, nt-swp Map file :$ntDescriptorFile:\n");
          }
          my %uniqueNTSWPDescriptor = map { $_ => 1 } (values %ntTypes);
          if( scalar(keys %uniqueNTSWPDescriptor) > 1 ) {
                Logger::LogError($execLogFH, "More than one different family NT's are found to Minimize\n" );
                exit EC_CREATE_TARGET_DIR_FAILED;
          }
      }
   }
}

Capability::ResetGlobals();
my $oswpDynamicVariant = DynamicOSWPVariant($execLogFH, \%config, $ntType );
if( $oswpDynamicVariant ) {
     next;
}
my $newSwpBaseName = basename($params->{NEWSWPNAME});
my $SwpBaseName = basename($params->{SWPNAME});
$SwpBaseName =~ s/(....).(.......)/$1$MM_CHAR$2/;
#$params->{NEWSWPPATH} =~ s/$newSwpBaseName$/$SwpBaseName/;
$params->{NEWSWPNAME} =~ s/$newSwpBaseName$/$SwpBaseName/;

#Reducing SWP descriptor file to desired boardlist
Logger::LogInfo($execLogFH, "Reducing SWP descriptor file to desired boardlist ...\n" );

	
my $i = 0;
my $j = 0;
my $each;
my $completePath = "/";
if (defined($params->{TARGET})) {
  my @pathSplitted = split(/\//, $params->{NEWSWPPATH});
  foreach my $pathPart (@pathSplitted) {
    if ($pathPart ne "") {
      $completePath .= $completePath eq "/" ? $pathPart : "/$pathPart";
      ToolCommon::CreateDirectory( $execLogFH, $completePath  );
    }
  }
  $completePath .= "/$params->{NEWSWPNAME}";
  ToolCommon::CreateDirectory( $execLogFH, $completePath  );
} else {
  $completePath = "$params->{NEWSWPPATH}";
}
if (! open(MAIN, "$params->{OSWP_HOME}/$params->{SWPNAME}")) {
  Logger::LogError($execLogFH, "Could not open $params->{OSWP_HOME}/$params->{SWPNAME} for retrieving information: $!\n");
  exit EC_OPEN_SWP_FAIL;
}
if(defined($params->{OSWPPREFIX})) {
	if (! open(NEW, ">$completePath/$params->{PREFIXNEWOSWP}")) {
	  Logger::LogError($execLogFH, "Could not open $completePath/$params->{PREFIXNEWOSWP} for writing information: $! \n");
	  exit EC_CREATE_SWP_FAIL;
	}
} else {
	if (! open(NEW, ">$completePath/$params->{NEWSWPNAME}")) {
	  Logger::LogError($execLogFH, "Could not open $completePath/$params->{NEWSWPNAME} for writing information: $! \n" );
	  exit EC_CREATE_SWP_FAIL;
	}
}

my $phase = $NOT_STARTED;
my $line;
my $boardDefined = 0;
my $str1 = undef;
my $str2 = undef; 

while (defined($line = <MAIN>)) {
  chomp $line;
  Logger::LogDebug($execLogFH, "$line\n");
  if ($line =~ /$DESCRIPTOR/) {
    $phase = $HEADER_STARTED;
  } elsif ($line =~ /$BOARDSECTION/) {
    $phase = $BOARD_STARTED;
  } elsif ($line =~ /$ENDSECTION/) {
    print NEW "$line\n\n" if $boardDefined;
    $phase = $NOT_STARTED, $boardDefined = 0;
    $boardName = "";
  }
  Logger::LogDebug($execLogFH, "phase = $phases[$phase]\n");
  if ($phase == $HEADER_STARTED) {
    if ($line =~ /$DESCRIPTOR/) {
      print NEW "DESCRIPTOR-FILE " . basename($params->{NEWSWPNAME}) . "\n";
    } else {
      print NEW "$line\n";
    }
  } elsif ($phase == $BOARD_STARTED) {
    if ($line =~ /\s*$BOARDSECTION\s+(\w+\-\w+)/) {
      $boardName = $1;
      $boardDefined = (index($boardList, $boardName) != -1) ? 1 : 0;
    }
    Logger::LogDebug($execLogFH, sprintf("board %s defined: %s\n", $boardName, $boardDefined ? "Yes" : "No"));
    if ($boardDefined) {
      if ($line =~ /(\s*MINIMUMSET\s*:\s*)(\w*)(.*)/) {
	if ($params->{MINSET}) {
            if($minAssign) {
               $str1 = $1; 
               $str2 = $3; 
               (defined($a_boards_minset{uc($boardName)}) && !($a_boards_minset{uc($boardName)})) ? 
               print NEW "$line\n" :   print NEW "$str1" . "YES" . "$str2\n"; 
            } else {
               ($boardName !~ /NT/) ?  print NEW "$1" . "YES" . "$3\n" : print NEW "$line\n";
            }
	} else {
	  print NEW "$line\n";
	}
      } else {
	print NEW "$line\n" if ($line ne "");
      }
    }
  }
}
print NEW "END\n";
close(MAIN);
close(NEW);
           
#################Creating OSWP descriptor file 

Logger::LogTrace($execLogFH, "Changing OSWP descriptor file ...\n", 1);

my $newPrefixOswp = undef; 
if(defined($params->{OSWPPREFIX})) {
      $newPrefixOswp =  $params->{NEWOSWP};
      $newPrefixOswp =~ s/(....)(........)/$params->{OSWPPREFIX}$2/;
	if (! open (OVERALL, ">$params->{NEWOSWPPATH}/$newPrefixOswp")) {
	  Logger::LogError($execLogFH, "Could not open targetfile $params->{NEWOSWPPATH}/$newPrefixOswp for writing: $!\n");
	  exit EC_CREATE_OSWP_FAIL;
	}
} else {
	if (! open (OVERALL, ">$params->{NEWOSWPPATH}/$params->{NEWOSWP}")) {
	  Logger::LogError($execLogFH, "Could not open targetfile $params->{NEWOSWPPATH}/$params->{NEWOSWP} for writing: $!\n");
	  exit EC_CREATE_OSWP_FAIL;
	}
}
if (!open(MAIN,"$params->{OSWP_FULL}") ) {
  Logger::LogError($execLogFH, "Could not open targetfile $params->{OSWP_FULL} for reading: $!\n");
  exit EC_OPEN_OSWP_FAIL;
}
while (my $line = <MAIN>) {
  chomp $line;
  $line =~ s/\r//g;
  Logger::LogDebug($execLogFH, "Line read: $line\n");

  if ($line =~ /ASAM-CORE/) {
    my $t = defined($params->{OSWPPREFIX}) ? "$completePath/$newPrefixOswp" : "$completePath/$params->{NEWSWPNAME}";
    if( $params->{NOTFTP} == 0 ){
        $t =~ s/$toolData{TFTPDIR}\///g;
    }
    print OVERALL "        ASAM-CORE        : $t $params->{PRIMTFTPIP} $params->{SECTFTPIP};\n";
  } elsif ($line =~ /OVERALL-DESCRIPTOR-FILE/) {
   	 print OVERALL "OVERALL-DESCRIPTOR-FILE " . $params->{NEWOSWP} . "\n";
  } else {
    print OVERALL "$line\n";
  }
}
close(MAIN);
close(OVERALL);

if (defined($params->{TARGET})) {
  my $swPathOnly = "$params->{OSWP_HOME}/$params->{SWPNAME}";
  my $base = basename($swPathOnly);
  $swPathOnly =~ s/\/$base$//;
  my $swpfile = defined($params->{OSWPPREFIX}) ? "$completePath/$newPrefixOswp" : "$completePath/$params->{NEWSWPNAME}";
  Logger::LogDebug($execLogFH, "Copy SW files (using $swpfile)...\n" );
  if (! open(NEW, "$swpfile")) {
    Logger::Error($execLogFH, "Could not open $completePath/$swpfile: $! \n");
    exit EC_OPEN_SWP_FAIL;
  } else {
    while (my $line = <NEW>) {
      chomp $line;
      $line =~ s/\r//g;
      if ($line =~ /(TYPE-\w)\s*:\s*(\w+\.\w+).*/) {
	if (!copy("$swPathOnly/$2", "$completePath/$2")) {
	  Logger::LogError($execLogFH, "Copy from '$swPathOnly/$2' to '$completePath/$2' failed: $!\n");
	  exit EC_COPY_FAILED;
	} else {
	  Logger::LogTrace($execLogFH, "Copy from '$swPathOnly/$2' to '$completePath/$2'\n");
	}
      } 
    } 
    if( defined $config{NT_SWP_MAPFILE} and $config{NT_SWP_MAPFILE} ne "" ) {
      my $ntMapFile = $config{NT_SWP_MAPFILE};
      if( -f "$swPathOnly/$ntMapFile" ) {
        if (!copy("$swPathOnly/$ntMapFile", "$completePath/$ntMapFile")) {
          Logger::LogError($execLogFH, "Copy from '$swPathOnly/$ntMapFile' to '$completePath/$ntMapFile' failed: $!\n");
          exit EC_COPY_FAILED;
        } else {
          Logger::LogTrace($execLogFH, "Copy from '$swPathOnly/$ntMapFile' to '$completePath/$ntMapFile'\n");
        }
      }
    }
    close(NEW);
  }
}
Logger::LogInfo($execLogFH, "Execution (SW Minimization) finished....\n"); 
close($execLogFH);

exit SUCCESS_CODE;   # -- 2.0.0

