#!/usr/bin/perl
#==============================================================================
#-- File:         ChgNwVlan.pl
#-- Date:         18.03.2008
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to change the NW Vlan of an E-IBridge
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent AND 2007
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/06/03 : First implementation (RCR ANTms78906/ANTmt09937
#--   1.0.1 - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.0.2 - 2008/09/05 : ANTmt09937.  New SHUB VLAN parameters in R3.7
#--   1.0.3 - 2008/09/15 : ANTmt23460.  Reject reconfiguration zhen NW VLAN is used
#--                        as PVID on an SHUB bridge port.
#--   1.0.4 - 2008/10/27 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.0.5 - 2009/01/26 : ANTmt39757.
#--   1.0.6 - 2009/03/10 : ANTmt47355. New option for bridge port
#--   1.0.7 - 2009/05/30 : ANTmt60596.  layer2-terminated VLAN also accecpted on SHUB.
#--   1.0.8 - 2009/06/11 : Allow SSH for CLI
#--   1.0.9 - 2009/06/29 : Some cleanup and best practises.
#--   1.0.10 - 2009/08/31: ANTmt62118.  Reconfigure MCAST SRC when defined on VLAN to be changed.
#--                        Added --sleep: delay introduced before deleting the VLAN.
#--   1.0.11 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.12 - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session
#--   1.0.13 - 2010/03/04 : ANTmg01050 - Total_MiB label change from Ex.x.y.z to Vx.x.x.y.z
#--   1.0.14 - 2010/03/15 : PBMT x86 Changes - ANTmt88407
#--   1.0.15 - 2011/01/11 : LRM updates for lib-perl path
#--   1.0.16 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   1.0.17 - 2011/05/02 : ALU01178887
#--   1.0.18 - 2012/05/11 : ALU01789702 - World writeable directories need sticky bit set 
#--   1.0.19 - 2012/06/13 : Change permissions for backup db file
#--   1.0.20 - 2013/04/29 : ALU01880147
#--   1.0.21 - 2016/03/16 : ALU02190562
#--   1.0.22 - 2016/07/15 : ALU02237966
#--   1.0.23 - 2016/09/28 : ALU02278259


my $VERSION = "ChgNwVlan.pl - 1.0.23 - 2016/09/28";
#==============================================================================
use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 
#==============================================================================
use Crypt::Lite;
use Capability;
use IsamCli;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :CNWV);
use SNMPComm;
use Utils;
use version qw(:ALL);
#==============================================================================
#== Constants
#==============================================================================
my $HOSTSFILE =               "/etc/hosts";
my $TFTPDIR =                 "/var/opt/aws/common/tftpd/data";
my $TFTPWORKDIR =             "tmp";
my $MAXSESSIONS =             5;
my $USERNAME =                "";
my $PASSWORD =                "";
my $NODEPREFIX =              "AS-";
my $TELNETTIMEOUT =           10;
my $STATUSFILE =              "CNWVresult.txt";
my $NT_COMMUNITY =            "public";
my $RETRIES =                 5;
my $NO_PING =                 0;
#==============================================================================
#== Vars
#==============================================================================
my $warnLogFile;
my %config;

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub Usage {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> --from <vlanid> --to <vlanid> \\ \n";
  print STDERR "       [--sessions <nr>] [--seplog] [--noaction] [--confinfo] [--noping] \\ \n";
  print STDERR "       [--rollback]  [--telnetto <minutes>] [--username <user>] \\ \n";
  print STDERR "       [--password <password>] [--v] [--debug] [--bufsize n] [--sleep n] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         from     : id of the old network VLAN\n";
  print STDERR "         to       : id of the new network VLAN\n";
  print STDERR "         sessions : number of parallel ISAMs to be treated (MAX=$MAXSESSIONS)\n";
  print STDERR "         username : ISAM username\n";
  print STDERR "         password : ISAM password\n";
  print STDERR "         seplog   : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noaction : just check, do not change the type\n";
  print STDERR "         debug    : extra logging for debugging purposes\n";
  print STDERR "         confinfo : print out all retrieved configration info\n";
  print STDERR "         rollback : roll back to original configuration\n";
  print STDERR "         bufsize  : size of CLI command return buffer (default is 15Mb)\n";
  print STDERR "         telnetto : timeout value for telnet session (in minutes)\n";
  print STDERR "         noping   : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "         sleep    : introduce a delay of n seconds before sending the VLAN deletion command.  Default is 5.\n";
  exit CMD_INCOMPL_CODE;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub ParseCommandLine {
  my ($isam, $fromVlan, $toVlan, $username, $password, $sessions, $debug, $hosts, $logfile, $version, $sepLog, $noaction, $confInfo,
      $rollBack, $bufSize, $telnetTO, $defPrio, $maxCc, $noPing, $sleep);
  my %params;
  my $options = GetOptions('isam=s', \$isam,
			   'noaction', \$noaction,
			   'fromvlan=i', \$fromVlan,
			   'tovlan=i', \$toVlan,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'username=s', \$username,
			   'password=s', \$password,
			   'logfile=s', \$logfile,
			   'v', \$version,
			   'seplog', \$sepLog,
			   'confinfo', \$confInfo,
			   'rollback', \$rollBack,
			   'telnetto=i', \$telnetTO,
			   'bufsize=i', \$bufSize,
			   'noping', \$noPing);

  #-- check options
  if (! $options) {
    Usage();
  }
  if (defined($version)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logfile)) {
    print STDERR "ERROR: no logfile given\n";
    Usage();	
  }
  if (!defined($fromVlan)) {
    print STDERR "ERROR: No from-network-VLAN ID provided\n";
    Usage();
  }
  if (!defined($toVlan)) {
    print STDERR "ERROR: No to-network-VLAN ID provided\n";
    Usage();
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: Hostsfile $hosts does not exist\n";
    Usage();
  }
  if (defined($username)) {
    $params{USERNAME} = $username;
  } else {
    $params{USERNAME} = $USERNAME;
  }
  if (defined($password)) {
    $params{PASSWORD} = $password;
  } else {
    $params{PASSWORD} = $PASSWORD;
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $MAXSESSIONS) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
	"($sessions > $MAXSESSIONS)\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $HOSTSFILE;
  }
  $params{UMASK_CHMOD} = ( (! defined $config{UMASK_OVERRIDE}) or $config{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = defined($debug) ? 1 : 0;
  $params{FROMVLAN} = $fromVlan;
  $params{TOVLAN} = $toVlan;
  $params{LOGFILE} = $logfile;
  $params{SEPLOG} = defined($sepLog) ? 1 : 0;
  $params{NOACTION} = defined($noaction) ? 1 : 0;
  $params{CONFINFO} = defined($confInfo) ? 1 : 0;
  $params{ROLLBACK} = defined($rollBack) ? 1 : 0;
  $params{BUFSIZE} = defined($bufSize) ? $bufSize : 15;
  $params{BUFSIZE} = 100 if ($params{BUFSIZE} > 100);
  $params{BUFSIZE} *= 1048576; # Convert to MBytes
  $params{TELNETTIMEOUT} = defined($telnetTO) ? $telnetTO * 60 : 600; # default 10 minutes
  $params{NOPING} = $NO_PING;
  $params{NOPING} = 1 if defined($noPing);
  $params{SLEEP} = defined($sleep) ? $sleep : 5;
  return \%params;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub PrintLibVersions {
  my ($fh, $out) = @_;

  if (defined($fh)) {
    ToolCommon::Log($fh, "Library versions:\n", $out);
    ToolCommon::Log($fh, sprintf("  %s\n", IsamCli::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ToolCommon::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", FamilyCommon::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", Capability::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ErrorCodes::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", Utils::GetVersion()), $out);
  } else {
    print STDERR "Library versions:\n";
    print STDERR sprintf("  %s\n", IsamCli::GetVersion());
    print STDERR sprintf("  %s\n", ToolCommon::GetVersion());
    print STDERR sprintf("  %s\n", FamilyCommon::GetVersion());
    print STDERR sprintf("  %s\n", Capability::GetVersion());
    print STDERR sprintf("  %s\n", ErrorCodes::GetVersion());
    print STDERR sprintf("  %s\n", Utils::GetVersion());
  }
}

sub PrintOptionsSummary {
  my($fh, $out) = @_;

  ToolCommon::Log($fh, sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION), $out);
  ToolCommon::Log($fh, "Options:\n", $out);
  ToolCommon::Log($fh, sprintf("   Reconfigure from VLAN %d to VLAN %d\n", $params->{FROMVLAN}, $params->{TOVLAN}), $out);
  ToolCommon::Log($fh, "   Nr Sessions                             : $params->{SESSIONS}\n", $out);
  ToolCommon::Log($fh, "   Hosts file                              : $params->{HOSTS}\n", $out) if (!defined($params->{ISAM}));
  ToolCommon::Log($fh, "   Only One Isam                           : $params->{ISAM}\n", $out) if (defined($params->{ISAM}));
  ToolCommon::Log($fh, sprintf("   Ping                                    : %s\n", $params->{NOPING} ? "not allowed" : "allowed"), $out);
  ToolCommon::Log($fh, sprintf("   Telnet CLI command buffer size          : %d Mbytes\n", $params->{BUFSIZE} / 1048576), $out);
  ToolCommon::Log($fh, sprintf("   Log file per node                       : %s\n", $params->{SEPLOG} ? "Yes" : "No"), $out);
  ToolCommon::Log($fh, sprintf("   Telnet timeout                          : %d minutes\n", $params->{TELNETTIMEOUT} / 60), $out);
  ToolCommon::Log($fh, "   Debugging Enabled\n", $out) if $params->{DEBUG};
  ToolCommon::Log($fh, "   No reconfiguration, only testing\n", $out) if $params->{NOACTION};
  ToolCommon::Log($fh, "   Print retrieved CLI configuration information\n", $out) if $params->{CONFINFO};
  ToolCommon::Log($fh, "   Rollback original configuration\n", $out) if $params->{ROLLBACK};
  PrintLibVersions($fh, $out);
}

#------------------------------------------------------------------------------
#-- Parse SHUB VLAN configuration
#--
#-- This function is called to parse a VLAN as defined on the SHUB.
#-- Hash structure as follows:
#-- %vlanConfig{SHUB}{ID}
#--                  {NAME}
#--                  {MODE}
#--                  {ALLOWMACMOVE}
#--                  {USER2COMM}
#--                  {EGRESS}{LT/NT}
#--                  {UNTAG}{LT/NT}
#--
#-- $fh: pointer to a file handle for logging purposes
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $vlanConfig: pointer to a hash for storing the VLAN related info.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------

sub ParseShubVlanConfig {
  my ($fh, $response, $cliResponseLines, $vlanConfig, $releaseNum) = @_;
  my $vlanId;
  my $shubPort;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+vlan\s+shub\s+id\s+([^\s]+)/) {
	$vlanId = $1;
	push @{$cliResponseLines}, $line if ($line =~ /configure/);
	ToolCommon::Log($fh, "line read: $line\n") if ($params->{DEBUG});
	$line =~ s/configure\s+vlan\s+shub\s+id\s+([^\s]+)//;
	$vlanConfig->{SHUB}{ID} = $vlanId;
	ToolCommon::Log($fh, sprintf("  Stored vlanConfig->{SHUB}{ID} as '%d'\n", $vlanConfig->{SHUB}{ID})) if ($params->{DEBUG});
	if ($line =~ s/name\s+([^\s]+)//) {
	  $vlanConfig->{SHUB}{NAME} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanConfig->{SHUB}{NAME} as '%s'\n", $vlanConfig->{SHUB}{NAME})) if ($params->{DEBUG});
	}
	if ($line =~ s/mode\s+([^\s]+)//) {
	  $vlanConfig->{SHUB}{MODE} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanConfig->{SHUB}{MODE} as '%s'\n", $vlanConfig->{SHUB}{MODE})) if ($params->{DEBUG});
	}
	if ($line =~ s/mac\-move\-allow//) {
	  $vlanConfig->{SHUB}{ALLOWMACMOVE} = 1;
	  ToolCommon::Log($fh, "  Stored vlanConfig->{SHUB}{ALLOWMACMOVE}\n") if ($params->{DEBUG});
	}
	if ($line =~ s/mac-learning-off//) {
	  $vlanConfig->{SHUB}{MACLEARNOFF} = 1;
	  ToolCommon::Log($fh, "  Stored vlanConfig->{SHUB}{MACLEARNOFF}\n") if ($params->{DEBUG});
	}
	if ($line =~ s/user2\-comm\s+([^\s]+)//) {
	  $vlanConfig->{SHUB}{USER2COMM} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanConfig->{SHUB}{USER2COMM} as '%s'\n", $vlanConfig->{SHUB}{USER2COMM})) if ($params->{DEBUG});
	}
	if ($line =~ s/egress\-port\s+([^\s]+)//) {
	  $shubPort = $1;
	  $vlanConfig->{SHUB}{EGRESS}{$shubPort} = 1;
	  ToolCommon::Log($fh, "  Stored vlanConfig->{SHUB}{EGRESS}{$shubPort}\n") if ($params->{DEBUG});
	}
	if ($line =~ s/untag\-port\s+([^\s]+)//) {
	  $shubPort = $1;
	  $vlanConfig->{SHUB}{UNTAG}{$shubPort} = 1;
	  ToolCommon::Log($fh, "  Stored vlanConfig->{SHUB}{UNTAG}{$shubPort}\n") if ($params->{DEBUG});
	}
	$line =~ s/\s//g;
	if ($line ne "") {
	  FamilyCommon::SetAndLogError($fh, "Command not completely parsed, left: $line");
	  return 0;
	}
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "'$line' not according to expected syntax");
	return 0;
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse NT VLAN configuration
#--
#-- This function is called to parse a VLAN as defined on the NT.
#-- Hash structure as follows:
#-- %vlanConfig{NT}{ID}
#--                {NAME}
#--                {MODE}
#--                {PRIO}
#--                {BCFRMS}
#--                {PROTFLT}
#--                {PPPOERLYTAG}
#--                {OPT82}
#--                {CIRCIDDHCP}
#--                {REMIDDHCP}
#--                {DHCPLINERATE}
#--                {PPPOELINERATE}
#--                {CIRCIDPPPOE}
#--                {REMIDPPPOE}
#--                {VMACTRANS}
#--                {VMACDNFLT}
#--                {SECFWD}
#--                {AGINGTIME}
#--
#-- $fh: pointer to a file handle for logging purposes
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $vlanConfig: pointer to a hash for storing the VLAN related info.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------

sub ParseNtVlanConfig {
  my ($fh, $response, $cliResponseLines, $vlanConfig, $releaseNum) = @_;
  my $vlanId;
  my $ok = 1;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+vlan\s+id\s+([^\s]+)/) {
	ToolCommon::Log($fh, "line read: $line\n") if ($params->{DEBUG});
	push @{$cliResponseLines}, $line;
	$vlanId = $1;
	$vlanConfig->{NT}{ID} = $vlanId;
	$line =~ s/configure\s+vlan\s+id\s+[^\s]+//;
	ToolCommon::Log($fh, sprintf("Stored VLAN ID as '%s'\n", $vlanConfig->{NT}{ID})) if ($params->{DEBUG});
	if ($line =~ s/name\s+([^\s]+)//) {
	  $vlanConfig->{NT}{NAME} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN name as '%s'\n", $vlanConfig->{NT}{NAME})) if ($params->{DEBUG});
	}
	if ($line =~ s/mode\s+([^\s]+)//) {
	  $vlanConfig->{NT}{MODE} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN mode as '%s'\n", $vlanConfig->{NT}{MODE})) if ($params->{DEBUG});
	}
	if ($line =~ s/priority\s+(\d+)//) {
	  $vlanConfig->{NT}{PRIO} = $1;
	  ToolCommon::Log($fh, "Stored VLAN priority\n") if ($params->{DEBUG});
	}
	if ($line =~ s/broadcast\-frames//) {
	  $vlanConfig->{NT}{BCFRMS} = 1;
	  ToolCommon::Log($fh, "Stored VLAN broadcast frames\n") if ($params->{DEBUG});
	}
	if ($line =~ s/protocol\-filter\s+([^\s]+)//) {
	  $vlanConfig->{NT}{PROTFLT} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN protocol filter as '%s'\n", $vlanConfig->{NT}{PROTFLT})) if ($params->{DEBUG});
	}
	if ($line =~ s/pppoe\-relay\-tag\s+(\w+)//) {
	  $vlanConfig->{NT}{PPPOERLYTAG} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN PPPoE relay tag as '%s'\n", $vlanConfig->{NT}{PPPOERLYTAG})) if ($params->{DEBUG});
	}
	if ($line =~ s/dhcp\-opt\-82//) {
	  $vlanConfig->{NT}{OPT82} = 1;
	  ToolCommon::Log($fh, "Stored VLAN DHCP option 82\n") if ($params->{DEBUG});
	}
	if ($line =~ s/circuit\-id\-dhcp\s+([^\s]+)//) {
	  $vlanConfig->{NT}{CIRCIDDHCP} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN circuit-id DHCP as '%s'\n", $vlanConfig->{NT}{CIRCIDDHCP})) if ($params->{DEBUG});
	}
	if ($line =~ s/remote\-id\-dhcp\s+([^\s]+)//) {
	  $vlanConfig->{NT}{REMIDDHCP} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN remote-id DHCP as '%s'\n", $vlanConfig->{NT}{REMIDDHCP})) if ($params->{DEBUG});
	}
	if ($line =~ s/dhcp-linerate\s+([^\s]+)//) {
	  $vlanConfig->{NT}{DHCPLINERATE} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN DHCP linerate as '%s'\n", $vlanConfig->{NT}{DHCPLINERATE})) if ($params->{DEBUG});
	}
	if ($line =~ s/pppoe\-linerate\s+([^\s]+)//) {
	  $vlanConfig->{NT}{PPPOELINERATE} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN PPPoE linerate as '%s'\n", $vlanConfig->{NT}{PPPOELINERATE})) if ($params->{DEBUG});
	}
	if ($line =~ s/circuit\-id\-pppoe\s+([^\s]+)//) {
	  $vlanConfig->{NT}{CIRCIDPPPOE} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN circuit-id PPPoE as '%s'\n", $vlanConfig->{NT}{CIRCIDPPPOE})) if ($params->{DEBUG});
	}
	if ($line =~ s/remote\-id\-pppoe\s+([^\s]+)//) {
	  $vlanConfig->{NT}{REMIDPPPOE} = $1;
	  ToolCommon::Log($fh, sprintf("Stored VLAN remote-id PPPoE as '%s'\n", $vlanConfig->{NT}{REMIDPPPOE})) if ($params->{DEBUG});
	}
	if ($line =~ s/vmac\-translation//) {
	  $vlanConfig->{NT}{VMACTRANS} = 1;
	  ToolCommon::Log($fh, "Stored VLAN VMAC translation\n") if ($params->{DEBUG});
	}
	if ($line =~ s/vmac\-dnstr\-filter//) {
	  $vlanConfig->{NT}{VMACDNFLT} = 1;
	  ToolCommon::Log($fh, "Stored VLAN VMAC downstream filter\n") if ($params->{DEBUG});
	}
	if ($line =~ s/secure\-forwarding//) {
	  $vlanConfig->{NT}{SECFWD} = 1;
	  ToolCommon::Log($fh, "Stored VLAN secure forwarding\n") if ($params->{DEBUG});
	}
	if ($line =~ s/aging\-time//) {
	  $vlanConfig->{NT}{AGINGTIME} = 1;
	  ToolCommon::Log($fh, "Stored VLAN againg timing\n") if ($params->{DEBUG});
	}
	$line =~ s/\s//g;
	if ($line ne "") {
	  FamilyCommon::SetAndLogError($fh, "Command not completely parsed, left: $line");
	  return 0;
	}
      } else {
	if (! ($line =~ /\#[-]+$/) &&
	    ! ($line =~ /[\$%\#>]\s*$/)) {
	  $ok = 0;
	  FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	}
      }
    }
  }
  return $ok;
}

#------------------------------------------------------------------------------
#-- Parse bridge port info
#--
#-- This function is called to parse all bridge ports whaving netwrok-vlan set to the
#-- value of the VLAN ID associated to the CC engine.
#--
#-- Hash structure as follows:
#-- %vlanPortConfig{$bridgePort}{$vlan}{NETWORKVLAN}
#--                                    {VLANSCOPE}
#--                                    {QOS}
#-- $fh: pointer to a file handle for logging purposes
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $bridgeConfigStorage: pointer to a hash for storing the bridge related info.
#-- $vlanPortStorage: pointer to a hash for storing the vlan-port related info corresponding to the network vlan ID
#-- $pppCcClientPorts: pointer to a hash containing already retrieved PPPoX-relay CC client-ports
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseNwVLANBridgePortConfig {
  my ($fh, $cli, $response, $cliResponseLines, $bridgeConfigStorage, $vlanPortStorage, $releaseNum) = @_;
  my $portId = "";
  my $vlanId;
  my @fullConfig = ();

  foreach my $line (@{$response}) {
    $vlanId = undef;
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+bridge\s+port\s+([^\s]+)\s+vlan-id\s+([^\s]+)/) {
	$portId = $1;
	my $checkVlanId = $2;
	# Only when the network VLAN is an exact match (match vlan-id 10 als returns vlan-id 100 and 1000...
	if ($checkVlanId == $params->{FROMVLAN}) {
	  @fullConfig = IsamCli::SendCommand($cli, "info configure bridge port $portId flat");
	  if (defined($fullConfig[0])) {
	    if (IsamCli::CommandError(join("", @fullConfig))) {
	      FamilyCommon::SetAndLogError($fh, sprintf("FAILED to retrieve bridge info: %s", IsamCli::GetError()));
	      return 0;
	    } else {
	      foreach my $line (@fullConfig) {
		chomp $line;
		if ($line =~ /configure/) {
		  push @{$cliResponseLines}, $line;
		  ToolCommon::Log($fh, "line read: $line\n") if ($params->{DEBUG});
		  $vlanId = ParseBridgeConfigLine($fh, $line, $portId, $bridgeConfigStorage, $vlanPortStorage, $vlanId);
		  return 0 if (defined($vlanId) && ($vlanId == -1));
		}
	      }
	    }
	  } else {
	    FamilyCommon::SetAndLogError($fh, sprintf("Empty CLI response received: %s", IsamCli::GetError()));
	    return 0;
	  }
	}
      } else {
	if (! ($line =~ /\#[-]+$/) &&
	    ! ($line =~ /[\$%\#>]\s*$/)) {
	  FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	  return 0;
	}
      }
    }
  }
  return 1;
}

sub ParseBridgeConfigLine {
  my ($fh, $line, $portId, $bridgeConfigStorage, $vlanPortStorage, $vlanRead) = @_;
  my $vlanId = undef;

  $vlanId = $vlanRead if (defined($vlanRead));
  $line =~ s/configure\s+bridge\s+port\s+$portId//;
  if ($line =~ s/pvid\s+([^\s]+)//) {
    $bridgeConfigStorage->{$portId}{PVID} = $1;
    ToolCommon::Log($fh, sprintf("Stored bridge PVID for %s as '%d'\n",
				 $portId, $bridgeConfigStorage->{$portId}{PVID})) if ($params->{DEBUG});
  }
  if ($line =~ s/default\-priority\s+([^\s]+)//) {
    $bridgeConfigStorage->{$portId}{PRIO} = $1;
    ToolCommon::Log($fh, sprintf("Stored bridge priority for %s as '%d'\n", $portId, $bridgeConfigStorage->{$portId}{PRIO}))
      if ($params->{DEBUG});
  }
  if ($line =~ s/mac\-learn\-off//) {
    $bridgeConfigStorage->{$portId}{MACLEARNOFF} = 1;
    ToolCommon::Log($fh, sprintf("Stored bridge MAC learning off for %s\n", $portId)) if ($params->{DEBUG});
  }
  if ($line =~ s/max\-unicast\-mac\s+(\d+)//) {
    $bridgeConfigStorage->{$portId}{MAXUNICASTMAC} = $1;
    ToolCommon::Log($fh, sprintf("Stored bridge max-unicast-mac for %s as '%d'\n", $portId,
				 $bridgeConfigStorage->{$portId}{MAXUNICASTMAC})) if ($params->{DEBUG});
  }
  if ($line =~ s/accept\-frame\-type\s+([^\s]+)//) {
    $bridgeConfigStorage->{$portId}{ACCEPTFRMTYPE} = $1;
    ToolCommon::Log($fh, sprintf("Stored bridge accept-frame-type for %s as '%s'\n", $portId,
				 $bridgeConfigStorage->{$portId}{ACCEPTFRMTYPE})) if ($params->{DEBUG});
  }
  if ($line =~ s/prio\-regen\-prof\s+([^\s]+)//) {
    $bridgeConfigStorage->{$portId}{PRIOREGENPROF} = $1;
    ToolCommon::Log($fh, sprintf("Stored bridge priority regeneration profile for %s as '%s'\n", $portId,
				 $bridgeConfigStorage->{$portId}{PRIOREGENPROF})) if ($params->{DEBUG});
  }
  if ($line =~ s/max\-committed-mac\s+(\d+)//) {
    $bridgeConfigStorage->{$portId}{MAXCOMMAC} = $1;
    ToolCommon::Log($fh, sprintf("Stored bridge max-committed-mac for %s as '%d'\n", $portId,
				 $bridgeConfigStorage->{$portId}{MAXCOMMAC})) if ($params->{DEBUG});
  }
  # Parsing of VLAN port specifics start here
  if ($line =~ s/vlan\-id\s+([^\s]+)//) {
    $vlanId = $1;
    # Need this one to make sure that at least $vlanPortStorage->{$pvc}{$vlanId} gets defined.
    $vlanPortStorage->{$portId}{$vlanId}{VLANID} = $vlanId if (!defined($vlanPortStorage->{$portId}{$vlanId}{VLANID}));
    $vlanPortStorage->{$portId}{$vlanId}{VLANSCOPE} = "network" if (!defined($vlanPortStorage->{$portId}{$vlanId}{VLANSCOPE}));
    ToolCommon::Log($fh, sprintf("Stored '%s' as VLAN id for %s, default scope\n", $vlanId, $portId)) if ($params->{DEBUG});
  }
  if ($line =~ s/qos\-profile\s+([^\s]+)//) {
    # qos-profile can be on bridge or on vlan-port level.  In case of vlan port, $vlanId will have been set above.
    if (defined($vlanId)) {
      $vlanPortStorage->{$portId}{$vlanId}{QOSPROF} = $1;
      ToolCommon::Log($fh, sprintf("Stored vlan-port qos-profile for %s:%s as '%s'\n", $portId, $vlanId,
				   $vlanPortStorage->{$portId}{$vlanId}{QOSPROF}))
	if ($params->{DEBUG});
    } else {
      $bridgeConfigStorage->{$portId}{QOSPROF} = $1;
      ToolCommon::Log($fh, sprintf("Stored bridge qos-profile for %s as '%s'\n", $portId, $bridgeConfigStorage->{$portId}{QOSPROF}))
	if ($params->{DEBUG});
    }
  }
  if ($line =~ s/tag\s+([^\s]+)//) {
    $vlanPortStorage->{$portId}{$vlanId}{TAG} = $1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port tag for %s:%s as '%s'\n", $portId, $vlanId, $vlanPortStorage->{$portId}{$vlanId}{TAG}))
      if ($params->{DEBUG});
  }
  if ($line =~ s/network\-vlan\s+([^\s]+)//) {
    $vlanPortStorage->{$portId}{$vlanId}{NETWORKVLAN} = $1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port network VLAN for %s:%s as '%d'\n", $portId, $vlanId,
				 $vlanPortStorage->{$portId}{$vlanId}{NETWORKVLAN})) if ($params->{DEBUG});
  }
  if ($line =~ s/vlan\-scope\s+([^\s]+)//) {
    $vlanPortStorage->{$portId}{$vlanId}{VLANSCOPE} = $1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port VLAN scope for %s:%s as '%s'\n", $portId, $vlanId,
				 $vlanPortStorage->{$portId}{$vlanId}{VLANSCOPE})) if ($params->{DEBUG});
  }
  if ($line =~ s/qos\s+([^\s]+)//) {
    $vlanPortStorage->{$portId}{$vlanId}{QOS} = $1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port QOS for %s:%s as '%s'\n", $portId, $vlanId, $vlanPortStorage->{$portId}{$vlanId}{QOS}))
      if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-best\-effort//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIOBESTEFF} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-best-effort for %s:%s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-background//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIOBACKGRND} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-background for %s:%s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-spare//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIOSPARE} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-spare for %s:%s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-exc\-effort//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIOEXCEFF} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-exc-effort for %s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-ctrl\-load//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIOCTRLLOAD} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-ctrl-load for %s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-less\-100ms//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIOLESS100MS} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-less-100ms for %s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-less\-10ms//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIOLESS10MS} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-less-10ms for %s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/prior\-nw\-ctrl//) {
    $vlanPortStorage->{$portId}{$vlanId}{PRIONWCTRL} = 1;
    ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-nw-ctrl for %s\n", $portId, $vlanId)) if ($params->{DEBUG});
  }
  if ($line =~ s/static\-user\s+ip\-address\s+(\d+\.\d+\.\d+\.\d+\/\d+)//) {
    my $ipAddr = $1;
    $vlanPortStorage->{$portId}{$vlanId}{IPADDR}{$ipAddr} = 1;
    ToolCommon::Log($fh, sprintf("  Stored VLAN-port static IP interface for %s:%s as %s\n", $portId, $vlanId,
				 $ipAddr)) if ($params->{DEBUG});
  }
  $line =~ s/\s//g;
  if ($line ne "") {
    FamilyCommon::SetAndLogError($fh, "Command not completely parsed, left: $line");
    return -1;
  }
  return $vlanId;
}

#------------------------------------------------------------------------------
#-- Get configuration of involved CC engine bridge and VLAN ports
#--
#-- Function is call for every bridge port having vlan-port with the value of the VLAN ID
#-- associated to the CC engine.
#--
#-- Hash structure as follows:
#-- %bridgeConfig{$pvc}{PVID}
#--                    {PRIO}
#--                    {MACLEARNOFF}
#--                    {MAXUNICASTMAC}
#--                    {ACCEPTFRMTYPE}
#--                    {QOSPROF}
#--                    {PRIOREGENPROF}
#-- %vlanPortConfig{$pvc}{$vlan}{VLANID}
#--                             {TAG}
#--                             {NETWORKVLAN}
#--                             {VLANSCOPE}
#--                             {QOS}
#--                             {QOSPROF}
#--                             {PRIOBESTEFF}
#--                             {PRIOBACKGRND}
#--                             {PRIOSPARE}
#--                             {PRIOEXCEFF}
#--                             {PRIOCTRLLOAD}
#--                             {PRIOLESS100MS}
#--                             {PRIOLESS10MS}
#--                             {PRIONWCTRL}
#--                             {STATIC_USER}{IP}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $bridgeConfigStorage: pointer to a hash for storing the bridge related info.
#-- $vlanPortStorage: pointer to a hash for storing the vlan-port related info corresponding to the network vlan ID
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub GetBridgePortConfig {
  my ($fh, $cli, $response, $cliResponseLines, $bridgeConfigStorage, $vlanPortStorage, $releaseNum) = @_;
  my @fullConfig = ();
  my $rspStr = "";
  my $portId;
  my $vlanId;
  my $nwVlanId;

  foreach my $line (@{$response}) {
    $vlanId = undef;
    # Loop on all the matching bridge ports
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+bridge\s+port\s+([^\s]+)\s+vlan-id\s+([^\s]+).*network-vlan\s+([^\s]+)/) {
	$portId = $1, $nwVlanId = $3;;
	# Only when the network VLAN is an exact match (match vlan-id 10 als returns vlan-id 100 and 1000...
	if ($nwVlanId == $params->{FROMVLAN}) {
	  @fullConfig = IsamCli::SendCommand($cli, "info configure bridge port $portId flat");
	  if (defined($fullConfig[0])) {
	    if (IsamCli::CommandError(join("", @fullConfig))) {
	      FamilyCommon::SetAndLogError($fh, sprintf("FAILED to retrieve bridge info: %s", IsamCli::GetError()));
	      return 0;
	    } else {
	      foreach my $line (@fullConfig) {
		chomp $line;
		if ($line =~ /configure/) {
		  push @{$cliResponseLines}, $line;
		  ToolCommon::Log($fh, "line read: $line\n") if ($params->{DEBUG});
		  $vlanId = ParseBridgeConfigLine($fh, $line, $portId, $bridgeConfigStorage, $vlanPortStorage, $vlanId);
		  return 0 if (defined($vlanId) && ($vlanId == -1));
		}
	      }
	    }
	  } else {
	    FamilyCommon::SetAndLogError($fh, sprintf("Empty CLI response received: %s", IsamCli::GetError()));
	    return 0;
	  }
	}
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "'$line' not according to expected syntax");
	return 0;
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse VLAN port interface info
#--
#-- This function is called to parse all bridge ports whaving netwrok-vlan set to the
#-- value of the VLAN ID associated to the CC engine.
#--
#-- Hash structure as follows:
#-- %vlanPortConfig{$bridgePort}{$vlan}{NETWORKVLAN}
#--                                    {VLANSCOPE}
#--                                    {QOS}
#-- $fh: pointer to a file handle for logging purposes
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $vlanPortStorage: pointer to a hash for storing the vlan-port related info corresponding to the network vlan ID
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseVlanPortInterface {
  my ($fh, $cli, $response, $cliResponseLines, $vlanPortStorage, $releaseNum) = @_;
  my $portId = "";
  my $vlanId;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+interface\s+port\s+vlan-port:([^\s]+)/) {
	$portId = $1;
	$portId =~ s/:(\d+)$//;
	$vlanId = $1;
	push @{$cliResponseLines}, $line;
	ToolCommon::Log($fh, "line read: $line (port: $portId, vlan: $vlanId)\n") if ($params->{DEBUG});
	if (defined($vlanPortStorage->{$portId})) {
	  if (defined($vlanPortStorage->{$portId}{$vlanId})) {
	    if ($line =~ /user\s+([^\s]+)/) {
	      $vlanPortStorage->{$portId}{$vlanId}{CUSTID} = $1;
	      ToolCommon::Log($fh, sprintf("  Stored vlanPortConfig->{$portId}{$vlanId}{CUSTID} as '%s'\n",
					   $vlanPortStorage->{$portId}{$vlanId}{CUSTID})) if ($params->{DEBUG});
	    }
	  }
	}
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse MCAST SRC info
#--
#-- This function is called to parse the MCAST SRC settings associated to the VLAN
#-- being reconfigured.
#--
#-- Hash structure as follows:
#-- %mastSrcConfig{$src}{}
#-- $fh: pointer to a file handle for logging purposes
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $mcastSrcStorage: pointer to a hash for storing the mcast src related info corresponding to the network vlan ID
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseMcastSrc {
  my ($fh, $cli, $response, $cliResponseLines, $mcastSrcStorage, $releaseNum) = @_;
  my $vlanId;
  my ($srcId, $srcIp);
  my @cmdResponse;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure mcast channel (\d+\.\d+\.\d+\.\d+) src-ip-addr \d+\.\d+\.\d+\.\d+ vlan-id ([^\s]+)/) {
      $srcId = $1;
      $vlanId = $2;
      if ($vlanId == $params->{FROMVLAN}) {
	@cmdResponse = IsamCli::SendCommand($cli, "info configure mcast channel $srcId flat");
	if (defined($cmdResponse[0])) {
	  if (IsamCli::CommandError(join("", @cmdResponse))) {
	    FamilyCommon::SetAndLogError($fh, sprintf("FAILED to retrieve bridge info: %s", IsamCli::GetError()));
	    return 0;
	  } else {
	    foreach my $cmdLine (@cmdResponse) {
	      chomp $cmdLine;
	      if ($cmdLine =~ /configure/) {
		push @{$cliResponseLines}, $cmdLine;
		ToolCommon::Log($fh, "line read: $cmdLine (mcast src: $srcId, vlan: $vlanId)\n") if ($params->{DEBUG});
		if ($cmdLine =~ s/configure mcast channel $srcId src-ip-addr (\d+\.\d+\.\d+\.\d+)//) {
		  $srcIp = $1;
		}
		if ($cmdLine =~ s/vlan-id ([^\s]+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{VLAN} = $1;
		  ToolCommon::Log($fh, sprintf("Stored vlan-id for %s - %s as %s\n", $srcId, $srcIp, $mcastSrcStorage->{$srcId}{$srcIp}{VLAN}))
		    if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/dis-fast-change//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{FST_CHG} = 1;
		  ToolCommon::Log($fh, sprintf("Stored dis-fast-change for %s - %s\n", $srcId, $srcIp)) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/pkg-mem-bitmap [^\s]+//) {
		  # Discard, repeated in packagemember
		}
		if ($cmdLine =~ s/name ([^\s]+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{NAME} = 1;
		  ToolCommon::Log($fh, sprintf("Stored name for %s - %s as %s\n", $srcId, $srcIp,
					       $mcastSrcStorage->{$srcId}{$srcIp}{NAME})) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/guaranteed-serv//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{GUAR_SRV} = 1;
		  ToolCommon::Log($fh, sprintf("Stored guaranteed service for %s - %s\n", $srcId, $srcIp)) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/peak-bit-rate (\d+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{PEAK} = $1;
		  ToolCommon::Log($fh, sprintf("Stored peak-bit-rate for %s - %s as %d\n", $srcId, $srcIp,
					       $mcastSrcStorage->{$srcId}{$srcIp}{PEAK})) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/service-name ([^\s]+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{SRV_NAME} = $1;
		  ToolCommon::Log($fh, sprintf("Stored service-name for %s - %s as %s\n", $srcId, $srcIp,
					       $mcastSrcStorage->{$srcId}{$srcIp}{SRV_NAME})) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/preview-duration (\d+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{PRV_DUR} = $1;
		  ToolCommon::Log($fh, sprintf("Stored preview-duration for %s - %s as %d\n", $srcId, $srcIp,
					       $mcastSrcStorage->{$srcId}{$srcIp}{PRV_DUR})) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/preview-number (\d+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{PRV_NUM} = $1;
		  ToolCommon::Log($fh, sprintf("Stored preview-number for %s - %s as %d\n", $srcId, $srcIp,
					       $mcastSrcStorage->{$srcId}{$srcIp}{PRV_NUM})) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/preview-blackout (\d+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{PRV_BO} = $1;
		  ToolCommon::Log($fh, sprintf("Stored preview-blackout for %s - %s as %d\n", $srcId, $srcIp,
					       $mcastSrcStorage->{$srcId}{$srcIp}{PRV_BO})) if ($params->{DEBUG});
		}
		if ($cmdLine =~ s/packagemember ([^\s]+)//) {
		  $mcastSrcStorage->{$srcId}{$srcIp}{PKGMEM} = $1;
		  ToolCommon::Log($fh, sprintf("Stored packagemember for %s - %s as %s\n", $srcId, $srcIp,
					       $mcastSrcStorage->{$srcId}{$srcIp}{PKGMEM})) if ($params->{DEBUG});
		}
		$cmdLine =~ s/\s//g;
		if ($cmdLine ne "") {
		  FamilyCommon::SetAndLogError($fh, "Command not completely parsed, left: $cmdLine");
		  return 0;
		}
	      }
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, sprintf("Empty CLI response received: %s", IsamCli::GetError()));
	  return 0;
	}
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
# Generate vlan port deletion rollback command using the configuration
# previously retrieved.
#------------------------------------------------------------------------------
sub RollBackBridgeVlanPort {
  my ($bridgePort, $vlanId, $vlanPortConfig, $releaseNum) = @_;
  my $rollBackCmd = "configure bridge port $bridgePort vlan-id $vlanId";

  $rollBackCmd .= " tag $vlanPortConfig->{$bridgePort}{$vlanId}{TAG}" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{TAG});
  $rollBackCmd .= " network-vlan $vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN}"
    if defined($vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN});
  $rollBackCmd .= " vlan-scope $vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE}"
    if defined($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE});
  $rollBackCmd .= " qos-profile $vlanPortConfig->{$bridgePort}{$vlanId}{QOSPROF}" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{QOSPROF});
  $rollBackCmd .= " qos $vlanPortConfig->{$bridgePort}{$vlanId}{QOS}" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{QOS});
  $rollBackCmd .= " prior-best-effort" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOBESTEFF});
  $rollBackCmd .= " prior-background" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOBACKGRND});
  $rollBackCmd .= " prior-spare" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOSPARE});
  $rollBackCmd .= " prior-exc-effort" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOEXCEFF});
  $rollBackCmd .= " prior-ctrl-load" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOCTRLLOAD});
  $rollBackCmd .= " prior-less-100ms" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOLESS100MS});
  $rollBackCmd .= " prior-less-10ms" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOLESS10MS});
  $rollBackCmd .= " prior-nw-ctrl" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIONWCTRL});
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate VLAN creation command using the VLAN configuration previously
# retrieved.
# Part 1: re-create the VLAN
#configure vlan id 248 mode residential-bridge no name no priority no broadcast-frames no protocol-filter no pppoe-relay-tag secure-forwarding no aging-time
#------------------------------------------------------------------------------
sub CreateNtVlanPart1 {
  my ($vlanStorage, $vlanId, $releaseNum) = @_;
  my $cmd = "configure vlan id $vlanId mode $vlanStorage->{NT}{MODE}";

  $cmd .= " name $vlanStorage->{NT}{NAME}" if (defined($vlanStorage->{NT}{NAME}));
  $cmd .= " prioriy $vlanStorage->{NT}{PRIO}" if (defined($vlanStorage->{NT}{PRIO}));
  $cmd .= " broadcast-frames " if (defined($vlanStorage->{NT}{BCFRMS}));
  $cmd .= " protocol-filter $vlanStorage->{NT}{PROTFLT}" if (defined($vlanStorage->{NT}{PROTFLT}));
  $cmd .= " pppoe-relay-tag $vlanStorage->{NT}{PPPOERLYTAG}" if (defined($vlanStorage->{NT}{PPPOERLYTAG}));
  $cmd .= " secure-forwarding" if (defined($vlanStorage->{NT}{SECFWD}));
  $cmd .= " aging-time" if (defined($vlanStorage->{NT}{AGINGTIME}));
  return $cmd;
}

#------------------------------------------------------------------------------
# Generate VLAN creation command using the VLAN configuration previously
# retrieved.
# Part 2: re-create the network interface adding the options not available
# at creation time
#configure vlan id 248 no vmac-translation no vmac-dnstr-filter dhcp-opt-82 circuit-id-dhcp physical-id remote-id-dhcp customer-id no dhcp-linerate no pppoe-linerate no circuit-id-pppoe no remote-id-ppe
#------------------------------------------------------------------------------
sub CreateNtVlanPart2 {
  my ($vlanStorage, $vlanId, $releaseNum) = @_;
  my $cmd = "configure vlan id $vlanId";

  $cmd .= " dhcp-opt-82 " if (defined($vlanStorage->{NT}{OPT82}));
  $cmd .= " circuit-id-dhcp $vlanStorage->{NT}{CIRCIDDHCP}" if (defined($vlanStorage->{NT}{CIRCIDDHCP}));
  $cmd .= " remote-id-dhcp $vlanStorage->{NT}{REMIDDHCP}" if (defined($vlanStorage->{NT}{REMIDDHCP}));
  $cmd .= " dhcp-linerate $vlanStorage->{NT}{DHCPLINERATE}" if (defined($vlanStorage->{NT}{DHCPLINERATE}));
  $cmd .= " pppoe-linerate $vlanStorage->{NT}{PPPOELINERATE}" if (defined($vlanStorage->{NT}{PPPOELINERATE}));
  $cmd .= " circuit-id-pppoe $vlanStorage->{NT}{CIRCIDPPPOE}" if (defined($vlanStorage->{NT}{CIRCIDPPPOE}));
  $cmd .= " remote-id-pppoe $vlanStorage->{NT}{REMIDPPPOE}" if (defined($vlanStorage->{NT}{REMIDPPPOE}));
  $cmd .= " vmac-translation" if (defined($vlanStorage->{NT}{VMACTRANS}));
  $cmd .= " vmac-dnstr-filter" if (defined($vlanStorage->{NT}{VMACDNFLT}));
  return ($cmd ne "configure vlan id $vlanId") ? $cmd : "";
}

#------------------------------------------------------------------------------
# Generate VLAN deletion rollback command using the VLAN configuration previously
# retrieved.
# Part 1: create the VLAN
#------------------------------------------------------------------------------
sub CreateShubVlanPart1 {
  my ($vlanStorage, $vlanId, $releaseNum) = @_;
  my $rollBackCmd = "configure vlan shub id $vlanId mode $vlanStorage->{SHUB}{MODE}";

  $rollBackCmd .= " name $vlanStorage->{SHUB}{NAME}" if (defined($vlanStorage->{SHUB}{NAME}));
  $rollBackCmd .= " mac-move-allow" if (defined($vlanStorage->{SHUB}{ALLOWMACMOVE}));
  $rollBackCmd .= " mac-learning-off" if (defined($vlanStorage->{SHUB}{MACLEARNOFF}));
  $rollBackCmd .= " user2-comm $vlanStorage->{SHUB}{USER2COMM}" if (defined($vlanStorage->{SHUB}{USER2COMM}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate MCAST SRC creation part 1 (all except packagememeber list)
#------------------------------------------------------------------------------
sub CreateMcastSrcPart1 {
  my ($mcastSrcConfig, $srcId, $srcIp, $releaseNum, $vlanId) = @_;
  my $cmd = "configure mcast channel $srcId src-ip-addr $srcIp";
  $cmd .= sprintf(" vlan-id %s", defined($vlanId) ? $vlanId : $mcastSrcConfig->{$srcId}{$srcIp}{VLAN});
  $cmd .= " dis-fast-change" if defined($mcastSrcConfig->{$srcId}{$srcIp}{FST_CHG});
  $cmd .= " name $mcastSrcConfig->{$srcId}{$srcIp}{NAME}" if defined($mcastSrcConfig->{$srcId}{$srcIp}{NAME});
  $cmd .= " guaranteed-serv" if defined($mcastSrcConfig->{$srcId}{$srcIp}{GUAR_SRV});
  $cmd .= " peak-bit-rate $mcastSrcConfig->{$srcId}{$srcIp}{PEAK}" if defined($mcastSrcConfig->{$srcId}{$srcIp}{PEAK});
  $cmd .= " service-name $mcastSrcConfig->{$srcId}{$srcIp}{SRV_NAME}" if defined($mcastSrcConfig->{$srcId}{$srcIp}{SRV_NAME});
  $cmd .= " preview-duration $mcastSrcConfig->{$srcId}{$srcIp}{PRV_DUR}" if defined($mcastSrcConfig->{$srcId}{$srcIp}{PRV_DUR});
  $cmd .= " preview-number $mcastSrcConfig->{$srcId}{$srcIp}{PRV_NUM}" if defined($mcastSrcConfig->{$srcId}{$srcIp}{PRV_NUM});
  $cmd .= " preview-blackout $mcastSrcConfig->{$srcId}{$srcIp}{PRV_BO}" if defined($mcastSrcConfig->{$srcId}{$srcIp}{PRV_BO});
  return $cmd;
}

#------------------------------------------------------------------------------
# Generate MCAST SRC creation part 2 (packagememeber list)
#------------------------------------------------------------------------------
sub CreateMcastSrcPart2 {
  my ($mcastSrcConfig, $srcId, $srcIp, $releaseNum) = @_;

  return "configure mcast channel $srcId src-ip-addr $srcIp packagemember $mcastSrcConfig->{$srcId}{$srcIp}{PKGMEM}";
}


#------------------------------------------------------------------------------
# Prepare the deletion of old RB:
# 1. Delete vlan-port association:
#    1.1 In case of a network-scope vlan: delete vlan-id
#    1.2 In case of a local-scope VLAN with a network VLAN: delete network-VLAN
# 2. Delete MCAST sources
# 3. Delete RB VLAN
# 4. Delete L2-terminated RB on SHUB
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $vlanConfig: pointer to a hash of the old RB VLAN.
# $bridgeConfig: pointer to a hash of the bridge configuration.
# $vlanPortConfig: pointer to a hash for the VLAN port settings
# $mcastSrcConfig: pointer to a hash for the MCAST SRC config settings.
# $cliCommandStorage: pointer to an array holding all the unconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollback commands of the unconfiguration.
#------------------------------------------------------------------------------
sub PrepareDeletion {
  my ($fh, $vlanConfig, $bridgeConfig, $vlanPortConfig, $mcastSrcConfig, $cliCommandStorage, $rollBackCmds, $releaseId) = @_;
  my $cmd = "";
  my $rollBackCmd = "";

  # 1. Delete the vlan-ports
  foreach my $bridgePort (sort { my $a1 = $a; $a1 =~ s/\//\./g;
				 my $b1 = $b; $b1 =~ s/\//\./g;
				 pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
			  keys %{$vlanPortConfig}) {
    foreach my $vlanId (sort keys %{$vlanPortConfig->{$bridgePort}}) {
      if ((defined($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE}) &&
	   ($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE} eq "network")) ||
	  (!defined($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE}))) {
	if (defined($bridgeConfig->{$bridgePort}{PVID})) {
	  # Remove PVID if present
	  $cmd = "configure bridge port $bridgePort no pvid";
	  $rollBackCmd = "configure bridge port $bridgePort pvid $bridgeConfig->{$bridgePort}{PVID}";
	  $rollBackCmds->{$cmd}[0] = $rollBackCmd;
	  ToolCommon::Log($fh, "1.1  Stored '$cmd'\n")if ($params->{DEBUG});
	  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	  push @{$cliCommandStorage}, $cmd;
	}
	# 1.1 VLAN scope is network -> remove vlan-id
	$cmd = "configure bridge port $bridgePort no vlan-id $vlanId";
	$rollBackCmd = RollBackBridgeVlanPort($bridgePort, $vlanId, $vlanPortConfig, $releaseId);
	push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	ToolCommon::Log($fh, "1.2  Stored '$cmd'\n")if ($params->{DEBUG});
	ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	push @{$cliCommandStorage}, $cmd;
	if (defined($vlanPortConfig->{$bridgePort}{$vlanId}{CUSTID})) {
	  $rollBackCmd = "configure interface port vlan-port:$bridgePort:$vlanId user $vlanPortConfig->{$bridgePort}{$vlanId}{CUSTID}";
	  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	}
	if (defined($vlanPortConfig->{$bridgePort}{$vlanId}{IPADDR})) {
	  foreach my $ipAddr (keys %{$vlanPortConfig->{$bridgePort}{$vlanId}{IPADDR}}) {
	    $rollBackCmd = "configure bridgpe port $bridgePort vlan-id $vlanId static-user ip-address $ipAddr";
	    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	    ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});	    
	  }
	}
      } elsif (defined($vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN})) {
	if ($vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN} == $params->{FROMVLAN}) {
	  # 1.2 VLAN scope is local with network-vlan translation
	  $cmd = "configure bridge port $bridgePort vlan-id $vlanId no network-vlan";
	  $rollBackCmd = "configure bridge port $bridgePort vlan-id $vlanId network-vlan $vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN}";
	  $rollBackCmds->{$cmd}[0] = $rollBackCmd;
	  ToolCommon::Log($fh, "1.2  Stored '$cmd'\n")if ($params->{DEBUG});
	  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	  push @{$cliCommandStorage}, $cmd;
	}
      }
    }
  }
  # 2. Delete MCAST src
  foreach my $srcId (sort keys %{$mcastSrcConfig}) {
    foreach my $srcIp (sort keys %{$mcastSrcConfig->{$srcId}}) {
      $cmd = "configure mcast no channel $srcId src-ip-addr $srcIp";
      ToolCommon::Log($fh, "2    Stored '$cmd'\n")if ($params->{DEBUG});
      foreach my $srcIp (sort keys %{$mcastSrcConfig->{$srcId}}) {
	$rollBackCmd = CreateMcastSrcPart1($mcastSrcConfig, $srcId, $srcIp, $releaseId);
	push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	$rollBackCmd = CreateMcastSrcPart2($mcastSrcConfig, $srcId, $srcIp, $releaseId);
	push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
      }
    }
    push @{$cliCommandStorage}, $cmd;
  }
  # 3. Delete NT-RB
  $cmd = "configure vlan no id $params->{FROMVLAN}";
  ToolCommon::Log($fh, "3    Stored '$cmd'\n")if ($params->{DEBUG});
  $rollBackCmd = CreateNtVlanPart1($vlanConfig, $params->{FROMVLAN}, $releaseId);
  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  $rollBackCmd = CreateNtVlanPart2($vlanConfig, $params->{FROMVLAN}, $releaseId);
  if ($rollBackCmd ne "") {
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "   Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  }
  push @{$cliCommandStorage}, $cmd;
  # 4. Delete SHUB RB
  $cmd = "configure vlan shub no id $params->{FROMVLAN}";
  ToolCommon::Log($fh, "4    Stored '$cmd'\n")if ($params->{DEBUG});
  push @{$cliCommandStorage}, $cmd;
  $rollBackCmd = CreateShubVlanPart1($vlanConfig, $params->{FROMVLAN}, $releaseId);
  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  foreach my $port (keys %{$vlanConfig->{SHUB}{EGRESS}}) {
    $rollBackCmd = "configure vlan shub id $params->{FROMVLAN} egress-port $port";
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  }
  foreach my $port (keys %{$vlanConfig->{SHUB}{UNTAG}}) {
    $rollBackCmd = "configure vlan shub id $params->{FROMVLAN} untag-port $port";
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  }
}

sub AddVlanPortOptions {
  my ($vlanPortConfig, $portId, $vlanId, $releaseId) = @_;
  my $options = "";

  $options .= " qos-profile $vlanPortConfig->{$portId}{$vlanId}{QOSPROF}" if defined($vlanPortConfig->{$portId}{$vlanId}{QOSPROF});
  $options .= " tag $vlanPortConfig->{$portId}{$vlanId}{TAG}" if defined($vlanPortConfig->{$portId}{$vlanId}{TAG});
  $options .= " qos $vlanPortConfig->{$portId}{$vlanId}{QOS}" if defined($vlanPortConfig->{$portId}{$vlanId}{QOS});
  $options .= " prior-best-effort" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOBESTEFF});
  $options .= " prior-background" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOBACKGRND});
  $options .= " prior-spare" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOSPARE});
  $options .= " prior-exc-effort" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOEXCEFF});
  $options .= " prior-ctrl-load" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOCTRLLOAD});
  $options .= " prior-less-100ms" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOLESS100MS});
  $options .= " prior-less-10ms" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOLESS10MS});
  $options .= " prior-nw-ctrl" if defined($vlanPortConfig->{$portId}{$vlanId}{PRIONWCTRL});
  # static ip addresses handled seperately as there can be multiple
  return $options;
}

sub DumpBridgeVlanPortConfig {
  my ($fh, $portId, $vlanId, $bridgeConfig, $vlanPortConfig) = @_;

  ToolCommon::Log($fh, sprintf("Dumping config for bridge port '%s'...\n", $portId));
  ToolCommon::Log($fh, "  pvid '$bridgeConfig->{$portId}{PVID}'\n") if defined($bridgeConfig->{$portId}{PVID});
  ToolCommon::Log($fh, "  default-priority '$bridgeConfig->{$portId}{PRIO}'\n") if defined($bridgeConfig->{$portId}{PRIO});
  ToolCommon::Log($fh, "  mac-learn-off\n") if defined($bridgeConfig->{$portId}{MACLEARNOFF});
  ToolCommon::Log($fh, "  max-unicast-mac '$bridgeConfig->{$portId}{MAXUNICASTMAC}'\n") if defined($bridgeConfig->{$portId}{MAXUNICASTMAC});
  ToolCommon::Log($fh, "  accept-frame-type '$bridgeConfig->{$portId}{ACCEPTFRMTYPE}'\n") if defined($bridgeConfig->{$portId}{ACCEPTFRMTYPE});
  ToolCommon::Log($fh, "  qos-profile '$bridgeConfig->{$portId}{QOSPROF}'\n") if defined($bridgeConfig->{$portId}{QOSPROF});
  ToolCommon::Log($fh, "  prio-regen-prof '$bridgeConfig->{$portId}{PRIOREGENPROF}'\n") if defined($bridgeConfig->{$portId}{PRIOREGENPROF});
  ToolCommon::Log($fh, sprintf("Dumping config for VLAN port '%s:%s'...\n", $portId, $vlanId));
  ToolCommon::Log($fh, "  qos-profile '$vlanPortConfig->{$portId}{$vlanId}{QOSPROF}'\n") if defined($vlanPortConfig->{$portId}{$vlanId}{QOSPROF});
  ToolCommon::Log($fh, "  tag '$vlanPortConfig->{$portId}{$vlanId}{TAG}'\n") if defined($vlanPortConfig->{$portId}{$vlanId}{TAG});
  ToolCommon::Log($fh, "  network-vlan '$vlanPortConfig->{$portId}{$vlanId}{NETWORKVLAN}'\n")
    if defined($vlanPortConfig->{$portId}{$vlanId}{NETWORKVLAN});
  ToolCommon::Log($fh, "  vlan-scope '$vlanPortConfig->{$portId}{$vlanId}{VLANSCOPE}'\n")
    if defined($vlanPortConfig->{$portId}{$vlanId}{VLANSCOPE});
  ToolCommon::Log($fh, "  qos '$vlanPortConfig->{$portId}{$vlanId}{QOS}'\n") if defined($vlanPortConfig->{$portId}{$vlanId}{QOS});
  ToolCommon::Log($fh, "  prior-best-effort\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOBESTEFF});
  ToolCommon::Log($fh, "  prior-background\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOBACKGRND});
  ToolCommon::Log($fh, "  prior-spare\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOSPARE});
  ToolCommon::Log($fh, "  prior-exc-effort\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOEXCEFF});
  ToolCommon::Log($fh, "  prior-ctrl-load\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOCTRLLOAD});
  ToolCommon::Log($fh, "  prior-less-100ms\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOLESS100MS});
  ToolCommon::Log($fh, "  prior-less-10ms\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIOLESS10MS});
  ToolCommon::Log($fh, "  prior-nw-ctrl\n") if defined($vlanPortConfig->{$portId}{$vlanId}{PRIONWCTRL});
  
}
#------------------------------------------------------------------------------
# Prepare creation of new RB and re-assign all vlan-ports
# 1. Create new RB:
#    1.1 On NT (same configuration as old)
#    1.2 On SHUB (same configuration as old)
# 2. Recreate vlan ports:
#    2.1 Local scope VLAN: add new network-vlan and static IP users if any
#    2.2 Network scope VLAN: add new netword-scope vlan-id and static IP users if any
# 3. Set pvid one was present
# 4. Adjust customer IDs if any
# 5. Re-create MCAST SRC if any
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $vlanConfig: pointer to a hash of the CC engine.
# $bridgeConfig: pointer to a hash of the bridge configuration.
# $mcastSrcConfig: pointer to a hash for the MCAST SRC config settings.
# $cliCommandStorage: pointer to an array holding all the reconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollkbac commands of the reconfiguration.
#------------------------------------------------------------------------------
sub PrepareCreation {
  my ($fh, $vlanConfig, $bridgeConfig, $vlanPortConfig, $mcastSrcConfig, $cliCommandStorage, $rollBackCmds, $releaseId) = @_;
  my $cmd = "";
  my $rollBackCmd = "";

  # 1.1 Create new RB on NT
  $cmd = CreateNtVlanPart1($vlanConfig, $params->{TOVLAN}, $releaseId);
  ToolCommon::Log($fh, "1.1  Stored '$cmd'\n")if ($params->{DEBUG});
  push @{$cliCommandStorage}, $cmd;
  $rollBackCmd = "configure vlan no id $params->{TOVLAN}";
  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  $cmd = CreateNtVlanPart2($vlanConfig, $params->{TOVLAN}, $releaseId);
  if ($cmd ne "") {
    push @{$cliCommandStorage}, $cmd;
    ToolCommon::Log($fh, "     Stored command '$cmd'\n")if ($params->{DEBUG});
  }
  # 1.2 Create new RB on NT
  $cmd = CreateShubVlanPart1($vlanConfig, $params->{TOVLAN}, $releaseId);
  ToolCommon::Log($fh, "1.2  Stored '$cmd'\n")if ($params->{DEBUG});
  push @{$cliCommandStorage}, $cmd;
  $rollBackCmd = "configure vlan shub no id $params->{TOVLAN}";
  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  foreach my $port (keys %{$vlanConfig->{SHUB}{EGRESS}}) {
    $cmd = "configure vlan shub id $params->{TOVLAN} egress-port $port";
    push @{$cliCommandStorage}, $cmd;
    ToolCommon::Log($fh, "     Stored command '$cmd'\n")if ($params->{DEBUG});
  }
  foreach my $port (keys %{$vlanConfig->{SHUB}{UNTAG}}) {
    $cmd = "configure vlan shub id $params->{TOVLAN} untag-port $port";
    push @{$cliCommandStorage}, $cmd;
    ToolCommon::Log($fh, "     Stored command '$cmd'\n")if ($params->{DEBUG});
  }
  foreach my $bridgePort (sort { my $a1 = $a; $a1 =~ s/\//\./g;
				 my $b1 = $b; $b1 =~ s/\//\./g;
				 pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
			  keys %{$vlanPortConfig}) {
    foreach my $vlanId (sort keys %{$vlanPortConfig->{$bridgePort}}) {
      DumpBridgeVlanPortConfig($fh, $bridgePort, $vlanId, $bridgeConfig, $vlanPortConfig) if $params->{DEBUG};
      if (defined($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE}) &&
	  ($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE} eq "local")) {
	if ($vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN} == $params->{FROMVLAN}) {
	  # 2.1 VLAN scope is local with network-vlan translation
	  $cmd = "configure bridge port $bridgePort vlan-id $vlanId network-vlan $params->{TOVLAN} vlan-scope local";
	  $cmd .= AddVlanPortOptions($vlanPortConfig, $bridgePort, $vlanId, $releaseId);
	  $rollBackCmd = "configure bridge port $bridgePort vlan-id $vlanId no network-vlan $params->{TOVLAN}";
	  $rollBackCmds->{$cmd}[0] = $rollBackCmd;
	  ToolCommon::Log($fh, "2.1  Stored '$cmd'\n")if ($params->{DEBUG});
	  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	  push @{$cliCommandStorage}, $cmd;
	  if (defined($bridgeConfig->{$bridgePort}{PVID})) {
	    # Change PVID if present
	    $cmd = "configure bridge port $bridgePort pvid $bridgeConfig->{$bridgePort}{PVID}";
	    $rollBackCmd = "configure bridge port $bridgePort no pvid";
	    $rollBackCmds->{$cmd}[0] = $rollBackCmd;
	    ToolCommon::Log($fh, "3.   Stored '$cmd'\n")if ($params->{DEBUG});
	    ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	    push @{$cliCommandStorage}, $cmd;
	  }
	  if (defined($vlanPortConfig->{$bridgePort}{$vlanId}{IPADDR})) {
	    foreach my $ipAddr (keys (%{$vlanPortConfig->{$bridgePort}{$vlanId}{IPADDR}})) {
	      $cmd = "configure bridge port $bridgePort vlan-id $vlanId static-user ip-addr $ipAddr";
	      $rollBackCmd = "configure bridge port $bridgePort vlan-id $vlanId static-user no ip-addr $ipAddr";
	      $rollBackCmds->{$cmd}[0] = $rollBackCmd;
	      ToolCommon::Log($fh, "4.2  Stored '$cmd'\n")if ($params->{DEBUG});
	      ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	      push @{$cliCommandStorage}, $cmd;
	    }
	  }
	}
      } elsif ((defined($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE}) &&
		($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE} eq "network")) ||
	       (!defined($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE}))) {
	# 2.2 VLAN scope is network -> set vlan-id
	$cmd = "configure bridge port $bridgePort vlan-id $params->{TOVLAN} no network-vlan";
	$cmd .= AddVlanPortOptions($vlanPortConfig, $bridgePort, $vlanId, $releaseId);
	$rollBackCmd = "configure bridge port $bridgePort no vlan-id $params->{TOVLAN}";
	$rollBackCmds->{$cmd}[0] = $rollBackCmd;
	ToolCommon::Log($fh, "2.2  Stored '$cmd'\n")if ($params->{DEBUG});
	ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	push @{$cliCommandStorage}, $cmd;
	if (defined($bridgeConfig->{$bridgePort}{PVID})) {
	  # Change PVID if present
	  $cmd = "configure bridge port $bridgePort pvid $params->{TOVLAN}";
	  $rollBackCmd = "configure bridge port $bridgePort no pvid";
	  $rollBackCmds->{$cmd}[0] = $rollBackCmd;
	  ToolCommon::Log($fh, "3.   Stored '$cmd'\n")if ($params->{DEBUG});
	  ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	  push @{$cliCommandStorage}, $cmd;
	}
	if (defined($vlanPortConfig->{$bridgePort}{$vlanId}{IPADDR})) {
	  foreach my $ipAddr (keys (%{$vlanPortConfig->{$bridgePort}{$vlanId}{IPADDR}})) {
	    $cmd = "configure bridge port $bridgePort vlan-id $params->{TOVLAN} static-user ip-addr $ipAddr";
	    $rollBackCmd = "configure bridge port $bridgePort vlan-id $params->{TOVLAN} static-user no ip-addr $ipAddr";
	    $rollBackCmds->{$cmd}[0] = $rollBackCmd;
	    ToolCommon::Log($fh, "4.2  Stored '$cmd'\n")if ($params->{DEBUG});
	    ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	    push @{$cliCommandStorage}, $cmd;
	  }
	}
      } elsif (defined($vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN})) {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "Step 2 - Unexpected condition - should not occur");
	DumpVlanPortConfig($fh, $bridgePort, $vlanId, $bridgeConfig, $vlanPortConfig);
	return 0;
      }
      # 4 Configure user-id if present
      if (defined($vlanPortConfig->{$bridgePort}{$vlanId}{CUSTID})) {
	$cmd = "configure interface port vlan-port:$bridgePort:$params->{TOVLAN} user $vlanPortConfig->{$bridgePort}{$vlanId}{CUSTID}";
	ToolCommon::Log($fh, "4.1  Stored '$cmd'\n")if ($params->{DEBUG});
	push @{$cliCommandStorage}, $cmd;
      }
    }
  }
  # 5. Configure MCAST src if any
  foreach my $srcId (sort keys %{$mcastSrcConfig}) {
    foreach my $srcIp (sort keys %{$mcastSrcConfig->{$srcId}}) {
      $rollBackCmd = "configure mcast no channel $srcId $srcIp";
      $cmd = CreateMcastSrcPart1($mcastSrcConfig, $srcId, $srcIp, $releaseId, $params->{TOVLAN});
      ToolCommon::Log($fh, "5    Stored '$cmd'\n")if ($params->{DEBUG});
      push @{$cliCommandStorage}, $cmd;
      $rollBackCmds->{$cmd}[0] = $rollBackCmd;
      ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
      $cmd = CreateMcastSrcPart2($mcastSrcConfig, $srcId, $srcIp, $releaseId);
      ToolCommon::Log($fh, "5    Stored '$cmd'\n")if ($params->{DEBUG});
      push @{$cliCommandStorage}, $cmd;
      $rollBackCmds->{$cmd}[0] = $rollBackCmd;
      ToolCommon::Log($fh, "     Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#  Backup system database for safety (may be needed if reconfiguration failed
# dramatically and CLI restoration failed.
#------------------------------------------------------------------------------
sub BackupDatabase {
  my ($fh, $cli, $isam, $debug, $parentFh, $logfileName, $count) = @_;
  my $time1 = time();
  my $oswpInfo;
  my $ok = 1;
  my $tftpIpAddress = "";
  my $fileName = "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
  my $cmd = "$config{RM} $fileName > CNWVrm.txt 2>&1;$config{TOUCH} $fileName";
  my $sysret = system($cmd);

  ToolCommon::Log($fh, "\nCMD = $cmd\n") if ($debug);
  if ($sysret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($sysret);
    FamilyCommon::SetAndLogError($fh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** $cmd FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "Touching $fileName OK\n") if $debug;
  }
  if ($params->{UMASK_CHMOD} and ! chmod(0666, "$fileName")) {
    FamilyCommon::SetAndLogError($fh, "chmod(666,$fileName) FAILED: $!", "  FAILED: $!\n");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** chmod(666,$fileName) FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  }
  if ($ok) {
    $oswpInfo = IsamCli::GetOswpInfo($cli, REGEXP);
    if (!defined($oswpInfo)) {
      FamilyCommon::SetAndLogError($fh, sprintf("Failed to retrieve OSWP info: %s", IsamCli::GetError()));
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Active OSWP: $oswpInfo->{ACTIVE}{INDEX}\n") if $debug;
      my @response = IsamCli::SendCommand($cli, "info configure software-mngt oswp flat", undef);
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::SetAndLogError($fh, sprintf("Retrieving oswp configuration failed: %s", IsamCli::GetError()));
	$ok = 0;
      } else {
	foreach my $line (@response) {
	  chomp $line;
	  ToolCommon::Log($fh, "Line read: $line\n") if $debug;
	  if ($line =~ /configure\s+software\-mngt\s+oswp\s+(\d+)\s+primary\-file\-server\-id\s+([^\s]+)/) {
	    my $oswpNum = $1;
	    my $ipAddress = $2;
	    if ($oswpInfo->{ACTIVE}{INDEX} == $oswpNum) {
	      $tftpIpAddress = $ipAddress;
	      last;
	    }
	  } else {
	    if (! ($line =~ /\#[-]+$/) &&
		! ($line =~ /[\$%\#>]\s*$/)) {
	      # Should not happen
	      FamilyCommon::SetAndLogError($fh, "OSWP config info not according to expected syntax: $line");
	      $ok = 0;
	      last;
	    }
	  }
	}
	if (($tftpIpAddress ne "") && ($tftpIpAddress ne "0.0.0.0")) {
	  my $ret = IsamCli::UploadDb($cli, 'actual-active',
				      $tftpIpAddress, "$TFTPWORKDIR/$isam", 1);
	  if (!defined($ret) || !$ret)  {
	    FamilyCommon::SetAndLogError($fh, sprintf("  FAILED (%s)\n", IsamCli::GetError()));
	    $ok = 0;
	  } else {
	    my $dbSize = -s "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
	    if ($dbSize == 0) {
	      FamilyCommon::SetAndLogError($fh, sprintf("  Size of database file 0\n"));
	      $ok = 0;
	    } else {
	      ToolCommon::Log($fh, sprintf("  OK, transferred %d bytes in %d seconds\n", $dbSize, time() - $time1));
	      ToolCommon::Log($fh, "Database backup stored as $fileName\n", 1);
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, "No TFTP IP address could be obtained\n");
	  $ok = 0;
	}
      }
    }
  }
  my @response = IsamCli::SendCommand($cli, "exit all");  # Set prompt to top node
  return $ok;
}

#------------------------------------------------------------------------------
# Rollback configuration in case of error (or forced as command option)
#------------------------------------------------------------------------------
sub RollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;
  my @response;

  if ($params->{ROLLBACK}) {
    ToolCommon::Log($fh, "==================================\n");
    ToolCommon::Log($fh, "The following will be rolled back:\n");
    ToolCommon::Log($fh, "==================================\n");
  } else {
    ToolCommon::Log($fh, "Start rollback...\n");
  }
  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "  Rolling back '$rollBackCmd'...\n");
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	if ($params->{ROLLBACK}) {
	  ToolCommon::Log($fh, "    '$cmd'\n");
	} else {
	  ToolCommon::Log($fh, sprintf("    %s '%s'\n", $params->{NOACTION} ? "Simulating" : "Sending", $cmd));
	}
	if (! $params->{NOACTION}) {
	  sleep $params->{SLEEP} if ($cmd =~ /configure vlan no id/);  # give deletion of network itf a bit of time
	  @response = IsamCli::SendCommand($cli, $cmd);
	  return if ((IsamCli::CommandError(join("", @response))) && !($cmd =~ /mcast no channel/));
#	  sleep 2;
	}
      }
    }
  }
  ToolCommon::Log($fh, "  Done\n");
}

#------------------------------------------------------------------------------
# Debugging: dump the content of the rollback command buffer.
#------------------------------------------------------------------------------
sub DumpRollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;

  ToolCommon::Log($fh, "==================================\n");
  ToolCommon::Log($fh, "CLI commands in case of roll back:\n");
  ToolCommon::Log($fh, "==================================\n");

  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "Rolling back '$rollBackCmd'...\n");
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	ToolCommon::Log($fh, sprintf("  Simulating '%s'\n", $cmd));
      }
    }
  }
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------
my $error = 0;
my @failedNodes;
my $nodeStatusLog = undef;
# To avoid too many function parameters...  I know it's not that nice...
my $childFh = undef;
my $parentFh = undef;
my $childCount = undef;
my $failedNode = undef;
my $cliSession2Close = undef;
my $snmpSession2Close = undef;

# Increment error count, write failure to log files and close node status report log.
sub IncrErrorCount {
  my ($msg, $errCode, $addInfo, $close) = @_;
  my $str = ErrorCodes::ErrorString($errCode);
  my $newNode = 1;
  $str .= " $addInfo" if (defined($addInfo));

  $error++;
  foreach (@failedNodes) {
    $newNode = 0 if ($failedNode eq $_);
  }
  push(@failedNodes, $failedNode) if $newNode;
  ToolCommon::Log($childFh, $msg) if ($msg ne "");
  my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime(time()))[0,1,2,3,4,5];
  ToolCommon::Log($parentFh, sprintf("Child $childCount: %4d-%02d-%02d %02d:%02d:%02d: Failed for %s: %s\n",
				     $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $failedNode, $str), 1);
  if (defined($close)) {
    if (defined($nodeStatusLog)) {
      print $nodeStatusLog "$str\n";
      close $nodeStatusLog;
      $nodeStatusLog = undef;
    }
    if (defined($cliSession2Close)) {
      IsamCli::Transaction($cliSession2Close, 'release'); # ignore possible error - will time out eventually
      IsamCli::AllowAlarms($cliSession2Close);
      IsamCli::Disconnect($cliSession2Close);
      $cliSession2Close = undef;
    }
    if (defined($snmpSession2Close)) {
      SNMPComm::SetValue( 'RELEASE OWNER', TRANSACTOWNER, "0.0.0.0" );
      $snmpSession2Close->close();
      $snmpSession2Close = undef;
    }
#    $childFh = undef;
#    $failedNode = undef;
  }
}

sub HandleIsams {
  my ($fh, $isams, $count, $config) = @_;
  my $isam;
  my ($time1, $time2);
  my $startTime;
  my $nodeLogFh;
  my $nodeLogFile;
  my $failedNode;
  my $cli = undef;
  my $releaseNum;
  my %vlanConfig;
  my %bridgeConfig;
  my %vlanPortConfig;
  my %mcastSrcConfig;
  my @configToDelete;
  my @configToCreate;
  my @configRetrieved = ();
  my %rollBackCommandConfig = ();
  my @rollBackCommands = ();
  my $cliCommandsSent = 0;
  my $reconfigStart;
  my $ret;
  my $deletionOk = 1;
  my $creationOk = 1;

  FamilyCommon::SetOwnPid($$);
  $parentFh = $fh; # Store for IncrErrorCount
  $childCount = $count; # Store for IncrErrorCount
  my $newLog = "$params->{LOGFILE}_$count";
  if (! open(LOGCHILD, ">$newLog")) {
    ToolCommon::Log($fh, "Child $count: cannot open $newLog for writing: $!\n", 1);
    CORE::exit(CREATE_FILE_FAIL_CODE)
  }
  LOGCHILD->autoflush(1);
  my $newfh = \*LOGCHILD;
  ToolCommon::Log($fh, "Child $count: Logging continuous in $newLog...\n", 1);

  PrintOptionsSummary($newfh, undef);

  foreach $isam (@{$isams}) {
    $failedNode = $params->{NONAME} ? $isam->[0] : $isam->[1];
    FamilyCommon::SetFailedNode($failedNode); # Store for IncrErrorCount
    FamilyCommon::SetWarnLogFile($newfh);   # Make sure warnings and 
    FamilyCommon::SetChildLogFile($newfh);  # logging is sent to the child process' logfile.
    # Switch debugging (if any) to an opened filehandle...
    Utils::SetDebug($newfh) if ($params->{DEBUG});
    IsamCli::SetDebug($newfh)  if ($params->{DEBUG});
    Capability::SetDebug($newfh) if ($params->{DEBUG});
    ToolCommon::SetDebug($newfh);

    ToolCommon::Log($newfh, "#===================================================\n");
    ToolCommon::Log($newfh, "#== $isam->[1]\n");
    ToolCommon::Log($newfh, "#===================================================\n");
    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Started $isam->[1]...\n",
		    $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second), 1);

    my $dir = "$TFTPDIR/$TFTPWORKDIR/$isam->[1]";
    $dir =~ s/\/\//\//g;
    if ( ! -d $dir) {
      ToolCommon::Log($newfh, "Creating working dir $dir...");
      if (!mkdir( $dir, 0777)) {
	ToolCommon::Log($newfh, "  FAILED\n");
	ToolCommon::Log($fh, "Child $count: **FATAL** Creating working dir $dir for $isam->[1] failed, please verify $newLog.\n", 1);
	CORE::exit(CREATE_DIR_FAILED_CODE);
      }
      ToolCommon::Log($newfh, "  OK\n");
    }
    if (! chmod(01777, "$dir")) {
        ToolCommon::Log($fh,"Failed to chmod 1777 for directory '$dir'\n");
    }

    if ($params->{SEPLOG}) {
      $nodeLogFile = sprintf("$dir/commit_%04d-%02d-%02d_%02d:%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
      if (! open(LOGNODE, ">$nodeLogFile")) {  # -- 1.2.5 START
	ToolCommon::Log($fh, "Child $count: cannot open $nodeLogFile for writing: $!\n", 1);
	CORE::exit(CREATE_FILE_FAIL_CODE)
      } else {
	ToolCommon::Log($fh, "Child $count: NE specific logging to $nodeLogFile...\n", 1);
      }
      LOGNODE->autoflush(1);
      $nodeLogFh = \*LOGNODE;
      Utils::SetDebug($nodeLogFh) if ($params->{DEBUG});
      IsamCli::SetDebug($nodeLogFh)  if ($params->{DEBUG});
      Capability::SetDebug($nodeLogFh) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFh);
      FamilyCommon::SetWarnLogFile($nodeLogFh);   # Make sure warnings and 
      FamilyCommon::SetChildLogFile($nodeLogFh);  # logging is sent to the child process' logfile.
    } else {
      $nodeLogFh = $newfh;
    }
    $warnLogFile = $nodeLogFh;
    $childFh = $nodeLogFh; # Store for IncrErrorCount
    $failedNode = $isam->[1]; # Store for IncrErrorCount

    if (! open(STATUS_LOG, ">$dir/$STATUSFILE")) {
      ToolCommon::Log($fh, "Child $count: cannot create $dir/$STATUSFILE for $isam->[1]: $!\n", 1);
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    }
    STATUS_LOG->autoflush(1);
    $nodeStatusLog = \*STATUS_LOG;

    ### Is Isam reachable through Ping command
    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }

    $params->{SNMP}{V1}{NT_COMMUNITY} = $NT_COMMUNITY;
    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
        FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError(), undef );
        next;
    }
    my $mibVersion = SNMPComm::GetValue('MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         next if( FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError(), undef ) );
    }
    $snmpSession2Close = $session; # Store for IncrErrorCount

      if (($mibVersion =~ /3FE21961EAAA_E(\d+)\.(\d+)\.\d+\.\d+/)      ||  # ETSI
          ($mibVersion =~ /3FE21961EAAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/) ||  # ETSI  (ISAM MIB label Extn)
	  ($mibVersion =~ /3FE21961ANAA_E(\d+)\.(\d+)\.\d+\.\d+/)      ||  # ANSI 
	  ($mibVersion =~ /3FE21961ANAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/)) {  # ANSI  (ISAM MIB label Extn)
	my $major = $1; my $minor = $2;
	$releaseNum = Capability::ConvertRelease($major, $minor);
	if (Capability::ReleaseCompare($releaseNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
	  FamilyCommon::IncrErrorCount(sprintf("  ERROR: Need at least R3.5 to do this reconfiguration (current release : %s)\n",
					       Capability::NtDbVersionInfo(Capability::IsamFamily(), $releaseNum)),
				       EC_RELEASE_TOO_LOW, undef, 1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      } else {
	FamilyCommon::IncrErrorCount("  ERROR: unsupported MIB version ($mibVersion)\n",
				     EC_INV_MIB_VERSION,
				     undef,
				     1);
	next;
      }

    ToolCommon::Log($nodeLogFh, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
				   EC_GET_ACC_BITS_FAILED,
				   undef,
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
      # Check whether this IP address is found in the know hosts file(s).
      if (! ToolCommon::SshPossible($isam->[0], $params->{DEBUG}, $config)) {
	FamilyCommon::IncrErrorCount(sprintf("No SSH access possible by PBMT: %s", ToolCommon::GetError()),
				     EC_NO_SSH_POSS,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	next;
      } else {
	$params->{USESSH} = 1;
	ToolCommon::Log($nodeLogFh, "No CLI access via Telnet possible, forced usage of SSH\n", 1);
      }
    }

    #-- Open CLI Session
    $cli = IsamCli::Connect($isam->[0], $params->{TELNETTIMEOUT},
			    $params->{DEBUG} ? $nodeLogFh : undef);
    if (!defined($cli)) {
      FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for $isam->[1]: %s\n", IsamCli::GetError()),
				   EC_OPENCLI_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    }
    $cliSession2Close = $cli;

    my $prev = $cli->max_buffer_length($params->{BUFSIZE});
    ToolCommon::Log($nodeLogFh, sprintf("Increased input buffer length from %d to %d bytes\n", $prev, $params->{BUFSIZE}));

    ToolCommon::Log($nodeLogFh, "Prepare CLI environment...");
    my @response = IsamCli::SendCommand($cli, "environment inhibit-alarms");
    if (IsamCli::CommandError(join("", @response))) {
      FamilyCommon::IncrErrorCount(sprintf("Alarm inhibit failed: %s\n", IsamCli::GetError()),
				   EC_INHIBIT_ALRM_FAILED,
				   undef,
				   1);
      next;
    }
    @response = IsamCli::SendCommand($cli, "environment print no-more");
    if (IsamCli::CommandError(join("", @response))) {
      FamilyCommon::IncrErrorCount(sprintf("Disabling operator interaction failed: %s\n", IsamCli::GetError()),
				   EC_PRNT_NOMORE_FAILED,
				   undef,
				   1);
      next;
    }
    @response = IsamCli::SendCommand($cli, "environment mode batch");
    if (IsamCli::CommandError(join("", @response))) {
      FamilyCommon::IncrErrorCount(sprintf("Enabling CLI batch mode failed: %s\n", IsamCli::GetError()),
				   EC_BATCHMODE_FAILED,
				   undef,
				   1);
      next;
    }
    if (!IsamCli::SetSessionTimeout($cli, 120)) {
      FamilyCommon::IncrErrorCount(sprintf("  Set session timeout FAILED: %s\n", IsamCli::GetError()),
				   EC_SET_TIMEOUT_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    # Here we go....
    @configRetrieved = ();
    # Check VLAN involvement in SHUB filters
    ToolCommon::Log($nodeLogFh, "Check SHUB filters...");
    @response = IsamCli::SendCommand($cli, "info configure system shub filter flat | match exact \"vlan-id $params->{FROMVLAN}\"");
    if (defined($response[0]) && ($response[0] =~ /^configure/)) {
      FamilyCommon::IncrErrorCount("  FAILED: from-VLAN used in SHUB filters\n",
				   EC_VLAN_USED,
				   undef,
				   1);
      next;
    } else {
      if (defined($response[0])) {
	ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				     EC_EMPTY_RESPONSE,
				     undef,
				     1);
	next;
      }
    }
    # Check existance of new NW VLAN (SHUB) - exit if existing.
    ToolCommon::Log($nodeLogFh, "Check existance of to-VLAN on SHUB...");
    @response = IsamCli::SendCommand($cli, "info configure vlan shub id $params->{TOVLAN} flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	ToolCommon::Log($nodeLogFh, sprintf("Command err (%s)\n", IsamCli::GetError()));
	if (index(join("", @response), "does not exist") == -1) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_SHUB_VLAN_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
	}
      } else {
	ToolCommon::Log($nodeLogFh, "No command err\n");
	FamilyCommon::IncrErrorCount("  FAILED: exists\n",
				     EC_TO_SHUBVLAN_EXISTS,
				     undef,
				     1);
	next;
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }
   # Check existance of new NW VLAN (NT) - exit if existing.
    ToolCommon::Log($nodeLogFh, "Check existance of to-VLAN on NT...");
    @response = IsamCli::SendCommand($cli, "info configure vlan id $params->{TOVLAN} flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(join("", @response), "does not exist") == -1) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_NT_VLAN_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
	}
      } else {
	FamilyCommon::IncrErrorCount("  FAILED: exists\n",
				     EC_TO_NTVLAN_EXISTS,
				     undef,
				     1);
	next;
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }
    # Check route-dest on SHUB
    ToolCommon::Log($nodeLogFh, "Check existance of route-dest on SHUB...");
    @response = IsamCli::SendCommand($cli,
				     "info configure ip shub vrf flat | match exact:\"route-dest\" | ".
				     "match exact:\"vlan-id $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_SHUBVRF_ERROR,
				     undef,
				     1);
	next;
      } else {
	if (join("", @response) =~ /configure/) {
	  FamilyCommon::IncrErrorCount("  FAILED (used in route-dest on SHUB)\n",
				       EC_USED_SHUB_ROUTEDEST,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }
    
    # Check usage in VRF on NT
    ToolCommon::Log($nodeLogFh, "Check usage of from-VLAN in NT VRF...");
    @response = IsamCli::SendCommand($cli, "info configure ip vrf flat | match exact:\"network-itf $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_NTVRF_ERROR,
				     undef,
				     1);
	next;
      } else {
	if (join("", @response) =~ /configure/) {
	  FamilyCommon::IncrErrorCount("  FAILED (used in VRF on NT)\n",
				       EC_USED_NT_VRF,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }
    
    # Check usage in PPP CC client-ports
    ToolCommon::Log($nodeLogFh, "Check cross-connect client-ports...");
    @response = IsamCli::SendCommand($cli,
				     "info configure pppox-relay cross-connect client-port flat | match exact:\"vlan-id $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_CCPORT_ERROR,
				     undef,
				     1);
	next;
      } else {
	if (join("", @response) =~ /configure/) {
	  FamilyCommon::IncrErrorCount("  FAILED (used in CC client-ports on NT)\n",
				       EC_USED_CCPORT,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    # Check usage in VLAN port-protocol
    ToolCommon::Log($nodeLogFh, "Check VLAN port-protocol...");
    @response = IsamCli::SendCommand($cli,
				     "info configure vlan port-protocol | match exact:\"vlan-id $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_VLANPP_ERROR,
				     undef,
				     1);
	next;
      } else {
	if (join("", @response) =~ /configure/) {
	  FamilyCommon::IncrErrorCount("  FAILED (used in VLAN port-protocol on NT)\n",
				       EC_USED_VLANPP,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    # Check usage of PVID on SHUB side.  Exit if found.  ANTmt23460
    ToolCommon::Log($nodeLogFh, "Check for PVID on SHUB network VLAN-port...");
    @response = IsamCli::SendCommand($cli,
				     "info configure bridge shub port flat | match exact:\"pvid $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_SHUB_BRG_ERROR,
				     undef,
				     1);
	next;
      } else {
	if (join("", @response) =~ /configure bridge shub port/) {
	  FamilyCommon::IncrErrorCount("  FAILED (used as PVID on SHUB bridge port)\n",
				       EC_USED_SHUBPVID,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK (does not exist)\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    # Retrieve configuration of old NW VLAN.. 2 parts: NT and SHUB
    %vlanConfig = ();
    ToolCommon::Log($nodeLogFh, "Retrieve VLAN info for from-VLAN $params->{FROMVLAN} info...");
    @response = IsamCli::SendCommand($cli, "info configure vlan shub id $params->{FROMVLAN} flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(join("", @response), "does not exist") != -1) {
	  FamilyCommon::IncrErrorCount("  FAILED: from-VLAN does not exist on SHUB\n",
				       EC_NO_SUCH_SHUB_VLAN,
				       undef,
				       1);
	  next;
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_SHUB_VLAN_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	ToolCommon::Log($nodeLogFh, "Parsing SHUB VLAN configuration...");
	if ( !ParseShubVlanConfig($nodeLogFh, \@response, \@configRetrieved, \%vlanConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_SHUB_VLAN_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  if (($vlanConfig{SHUB}{MODE} eq "residential-bridge") ||
	      ($vlanConfig{SHUB}{MODE} eq "layer2-terminated")) {
	    @response = IsamCli::SendCommand($cli, "info configure vlan id $params->{FROMVLAN} flat");
	    if (defined($response[0])) {
	      if (IsamCli::CommandError(join("", @response))) {
		if (index(join("", @response), "does not exist") != -1) {
		  FamilyCommon::IncrErrorCount("  FAILED: from-VLAN does not exist on NT\n",
					       EC_NO_SUCH_NT_VLAN,
					       undef,
					       1);
		  next;
		} else {
		  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					       EC_INFO_CONF_NT_VLAN_ERROR,
					       undef,
					       1);
		  next;
		}
	      } else {
		ToolCommon::Log($nodeLogFh, "  OK\n");
		ToolCommon::Log($nodeLogFh, "Parsing NT VLAN configuration...");
		if ( !ParseNtVlanConfig($nodeLogFh, \@response, \@configRetrieved, \%vlanConfig, $releaseNum)) {
		  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
					       EC_NT_VLAN_PARSE_ERROR,
					       undef,
					       1);
		  next;
		} else {
		  if ($vlanConfig{NT}{MODE} eq "residential-bridge") {
		    ToolCommon::Log($nodeLogFh, "  OK\n");
		  } else {
		    FamilyCommon::IncrErrorCount("  incorrect RB type: residential-bridge expected\n",
						 EC_INV_NT_RB_TYPE,
						 " (configured $vlanConfig{NT}{MODE})",
						 1);
		    next;
		  }
		}
	      }
	    } else {
	      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
					   EC_EMPTY_RESPONSE,
					   undef,
					   1);
	      next;
	    }
	  } else {
	    FamilyCommon::IncrErrorCount("  incorrect VLAN mode: residential-bridge or layer2-terminated expected\n",
					 EC_INV_SHUB_RB_TYPE,
					 " (configured $vlanConfig{SHUB}{MODE})",
					 1);
	    next;
	  }
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    # Retrieve bridge ports using from-VLAN as network-vlan
    ToolCommon::Log($nodeLogFh, "Retrieve bridge port info (vlan-id)...");
    @response = IsamCli::SendCommand($cli, "info configure bridge flat | match exact:\"vlan-id $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_BRIDGEPORT_ERROR,
				     undef,
				     1);
	next;
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	%bridgeConfig = ();
	%vlanPortConfig = ();
	ToolCommon::Log($nodeLogFh, "Parsing bridge port configuration (vlan-id)...");
	if (ParseNwVLANBridgePortConfig($nodeLogFh, $cli,
					\@response, \@configRetrieved, \%bridgeConfig, \%vlanPortConfig, $releaseNum)) {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_BRIDGEPORT_PARSE_ERROR,
				       undef,
				       1);
	  next;
	}
      }
    }    

    # Retrieve bridge ports using from VLAN as network-vlan for a VLAN-ID with local scope
    ToolCommon::Log($nodeLogFh, "Retrieve bridge port list (network-vlan)...");
    @response = IsamCli::SendCommand($cli, "info configure bridge flat | match exact:\"network-vlan $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_BRIDGEPORT_ERROR,
				     undef,
				     1);
	next;
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	ToolCommon::Log($nodeLogFh, "Retrieve bridge port configuration (network-vlan)...");
	if ( !GetBridgePortConfig($nodeLogFh, $cli, \@response, \@configRetrieved, \%bridgeConfig, \%vlanPortConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_BRIDGEPORT_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    }

    ToolCommon::Log($nodeLogFh, "Retrieve VLAN-port customer IDs...");
    @response = IsamCli::SendCommand($cli,
				     "info configure interface flat | match exact:\"vlan-port\" " .
				     "| match exact:\":$params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_VLANPORT_ERROR,
				     undef,
				     1);
	next;
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	ToolCommon::Log($nodeLogFh, "Parse VLAN port customer IDs...");
	if ( !ParseVlanPortInterface($nodeLogFh, $cli, \@response, \@configRetrieved, \%vlanPortConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_VLANPORT_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    }

    %mcastSrcConfig = ();
    # Check whether VLAN is used in MCAST
    ToolCommon::Log($nodeLogFh, "Check for usage in MCAST...");
    @response = IsamCli::SendCommand($cli,
				     "info configure mcast channel flat | match exact:\"vlan-id $params->{FROMVLAN}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_MCAST_ERROR,
				     undef,
				     1);
	next;
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	ToolCommon::Log($nodeLogFh, "Parse MCAST SRC...");
	if ( !ParseMcastSrc($nodeLogFh, $cli, \@response, \@configRetrieved, \%mcastSrcConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_MCASTSRC_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'\n", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    if ($params->{CONFINFO}) {
      ToolCommon::Log($nodeLogFh, "====================================================================\n");
      ToolCommon::Log($nodeLogFh, "The following configuration info has been read, parsed and stored...\n");
      ToolCommon::Log($nodeLogFh, "====================================================================\n");
      foreach (@configRetrieved) {
	ToolCommon::Log($nodeLogFh, "  $_\n");
      }
    }

    ToolCommon::Log($nodeLogFh, "Backup NT Database...");
    if (! BackupDatabase($nodeLogFh, $cli, $isam->[1], $params->{DEBUG}, $parentFh, $newLog, $count)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_BACKUPDB_FAILED,
				   undef,
				   1);
      next;
    }

    @configToDelete = ();
    %rollBackCommandConfig = ();
    PrepareDeletion($nodeLogFh, \%vlanConfig, \%bridgeConfig, \%vlanPortConfig, \%mcastSrcConfig,
		    \@configToDelete, \%rollBackCommandConfig, $releaseNum);
    @configToCreate = ();
    if (PrepareCreation($nodeLogFh, \%vlanConfig, \%bridgeConfig, \%vlanPortConfig, \%mcastSrcConfig,
			\@configToCreate, \%rollBackCommandConfig, $releaseNum)) {
      if ($params->{CONFINFO}) {
	ToolCommon::Log($nodeLogFh, "==============================\n");
	ToolCommon::Log($nodeLogFh, "The following will be deleted:\n");
	ToolCommon::Log($nodeLogFh, "==============================\n");
	foreach (@configToDelete) {
	  ToolCommon::Log($nodeLogFh, "  $_\n");
	}
	ToolCommon::Log($nodeLogFh, "==============================\n");
	ToolCommon::Log($nodeLogFh, "The following will be created:\n");
	ToolCommon::Log($nodeLogFh, "==============================\n");
	foreach (@configToCreate) {
	  ToolCommon::Log($nodeLogFh, "  $_\n");
	}
      }
      @rollBackCommands = ();

    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership...\n");
    return undef if( ToolCommon::AcquireTransaction() );
    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership... DONE\n");


      ToolCommon::Log($nodeLogFh, "Start deletion of resources...\n") if (! $params->{NOACTION});
      $reconfigStart = time();
      foreach my $cliCmd (@configToDelete) {
	ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd));
	if (! $params->{NOACTION}) {
	  $cliCommandsSent++;
	  sleep $params->{SLEEP} if ($cliCmd =~ /configure vlan no id/);  # give deletion of network itf a bit of time
	  @response = IsamCli::SendCommand($cli, $cliCmd);
	  if (defined($response[0])) {
	    if (IsamCli::CommandError(join("", @response))) {
	      $deletionOk = 0;
	      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					   EC_DEL_CONFIG_ERROR,
					   undef,
					   undef);
	      last;
	    } else {
	      if (defined($rollBackCommandConfig{$cliCmd})) {
		push @rollBackCommands, $cliCmd;
		ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						    join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	      }
	      #	  sleep 2;
	    }
	  } else {
	    $deletionOk = 0;
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_DEL_CONFIG_ERROR,
					 undef,
					 undef);
	    if (IsamCli::ConnectionReset()) {
	      ToolCommon::Log($fh,
			      "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	      sleep 1;
	      CORE::exit(EXIT_WARN_CODE);
	    }
	    last;
	  }
	} else {
	  if (defined($rollBackCommandConfig{$cliCmd})) {
	    push @rollBackCommands, $cliCmd;
	    ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd'\ -> '%s'n",
						join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	  }
	}
      }
      if (! $params->{NOACTION}) {
	if (! $deletionOk) {
	  if (!RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
					 EC_ROLLBACK_ERROR,
					 undef,
					 undef);
	  }
	  next;
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  Creation preparation FAILED: %s\n", FamilyCommon::GetError()),
				   EC_PREPARE_CREATE_ERROR,
				   undef,
				   undef);
      next;
    }
    ToolCommon::Log($nodeLogFh, "  Done\n") if (! $params->{NOACTION});
    sleep 10;

    ToolCommon::Log($nodeLogFh, "Start creation of resources...\n") if (! $params->{NOACTION});
    foreach my $cliCmd (@configToCreate) {
      ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd));
      if (! $params->{NOACTION}) {
	$cliCommandsSent++;
	@response = IsamCli::SendCommand($cli, $cliCmd);
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_CREATE_CONFIG_ERROR,
					 undef, 
					 undef);
	    $creationOk = 0;
	    last;
	  } else {
	    if (defined($rollBackCommandConfig{$cliCmd})) {
	      push @rollBackCommands, $cliCmd;
	      ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						  join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	    }
	    #sleep 2;
	  }
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_CREATE_CONFIG_ERROR,
				       undef,
				       undef);
	  if (IsamCli::ConnectionReset()) {
	    ToolCommon::Log($fh,
			    "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	    sleep 1;
	    CORE::exit(EXIT_WARN_CODE);
	  }
	  $creationOk = 0;
	  last;
	}
      } else {
	if (defined($rollBackCommandConfig{$cliCmd})) {
	  push @rollBackCommands, $cliCmd;
	  ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
					      join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	}
      }
    }
    if (! $creationOk) {
      if (! RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				     EC_ROLLBACK_ERROR,
				     undef,
				     undef);
      }
      next;
    }
    ToolCommon::Log($nodeLogFh, "  Done\n") if (! $params->{NOACTION});
    ToolCommon::Log($nodeLogFh, sprintf("%d CLI commands sent in %d seconds to accomplish this reconfiguration\n",
					$cliCommandsSent, time() - $reconfigStart));

    # We're done, see what's next
    if ($params->{ROLLBACK}) {
      RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig);
    } else {
      DumpRollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)  if $params->{DEBUG};
    }

    # Release transaction
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership... DONE");

    IsamCli::Disconnect($cli);
    $session->close();
    close LOGNODE if ($params->{SEPLOG});  # -- 1.2.5

    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Finished $isam->[1]... in %d seconds\n",
				 $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $endTime - $startTime), 1);

    if (defined($nodeStatusLog)) {
      print $nodeStatusLog sprintf("%s\n", ErrorCodes::ExitString(SUCCESS_CODE));
      close $nodeStatusLog;
    }
  }
  if ($error) {
    ToolCommon::Log($fh, sprintf("Child $count: $error error(s) occured for the following ISAMs, please verify %s\n",
				 $params->{SEPLOG} ? $nodeLogFile : "$params->{LOGFILE}_$count."), 1);
    foreach (@failedNodes) {
      ToolCommon::Log($fh, "  $_\n", 1);
    }
  }
  ToolCommon::Log($fh, "Child $count: finished...\n", 1);
  CORE::exit($error ? EXIT_WARN_CODE : SUCCESS_CODE);
}

#------------------------------------------------------------------------------
#-- Adjust Tool defaults if needed (i.e. when defined in the parameter config
#-- file
#------------------------------------------------------------------------------
sub AdjustToolVars {
  my $crypt = Crypt::Lite->new('debug' => 0);

  $HOSTSFILE = $config{'HOSTSFILE'} if defined ($config{'HOSTSFILE'});
  $TFTPDIR = $config{'TFTPDIR'} if defined($config{'TFTPDIR'});
  $TFTPWORKDIR = $config{'TFTPWORKDIR'} if defined($config{'TFTPWORKDIR'});
  $MAXSESSIONS = $config{'MAXTFTPSESSION'} if defined($config{'MAXTFTPSESSION'});
  $USERNAME = $crypt->decrypt($config{'DEFAULTCLIUSER'}, CYPHER) if (defined($config{'DEFAULTCLIUSER'}) &&
								     ($config{'DEFAULTCLIUSER'} ne ""));
  $PASSWORD = $crypt->decrypt($config{'DEFAULTCLIPWD'}, CYPHER) if (defined($config{'DEFAULTCLIPWD'}) &&
								    ($config{'DEFAULTCLIPWD'} ne ""));
  $NT_COMMUNITY = $crypt->decrypt($config{NT_COMMUNITY}, CYPHER) if (defined($config{NT_COMMUNITY}) &&
								     ($config{NT_COMMUNITY} ne ""));
  $NODEPREFIX = $config{'NODEPREFIX'} if defined($config{'NODEPREFIX'});
  $TELNETTIMEOUT = $config{'TELNETTIMEOUT'} if defined($config{TELNETTIMEOUT});
  $STATUSFILE = $config{CCCE_STATUS_FILE} if defined($config{CCCE_STATUS_FILE});
  $RETRIES = $config{RETRIES} if defined($config{RETRIES});
  $NO_PING = uc($config{'NO_PING'}) eq "N" ? 0 : 1 if defined($config{'NO_PING'});
}

#==============================================================================
#== Main program
#==============================================================================
my $count = 0;
my @pids;
my $pid;
my $warnCount = 0;

sub CleanUp {
  my ($signal, $exitCode) = @_;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string;
  ToolCommon::Log($warnLogFile, $string) if defined($warnLogFile);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      ToolCommon::Log($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR sprintf("%s::%s\n", PBMT_VERSION, $VERSION);
    print STDERR "\nCaught Interupt (^C), aborting children...";
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	kill($pid);
      }
    }
  }
  exit(EXIT_SIGNAL_CODE);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my $level = $config{warn_level};
  $level = shift if (1 < scalar(@_));
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }

  if (!($msgStr =~ /Zlib/)) {
    print $warnLogFile "Warning: $msgStr" if (defined($warnLogFile));
    print STDERR "Warning: $msgStr";
    $warnCount++;
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  ToolCommon::Log($warnLogFile, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  ToolCommon::Log($warnLogFile, "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};

#-- Read parameter config file and adjust tool defaults if needed 
%config = Utils::ReadConfig('param.cfg');
AdjustToolVars();

#-- Handle command line
$params = ParseCommandLine();

STDOUT->autoflush(1);
my $logFile;
if (!open($logFile, ">$params->{LOGFILE}")) {
  print STDERR "Cannot open $params->{LOGFILE} for writing: $!\n";
  exit CREATE_FILE_FAIL_CODE;
}
$logFile->autoflush(1);
FamilyCommon::SetWarnLogFile($logFile);

#-- Print summary of options
PrintOptionsSummary($logFile, 1);

Utils::SetDebug($logFile) if ($params->{DEBUG});  # -- 1.2.2

#-- Create work dir
my $tmpDir = "$TFTPDIR/$TFTPWORKDIR";
if (! -d $tmpDir) {
  if (!mkdir( $tmpDir, 0777)) {
    ToolCommon::Log($logFile, "Cannot create directory $tmpDir: $!\n", 1);
    exit CREATE_DIR_FAILED_CODE;
  }
  if (! chmod(01777, "$tmpDir")) {
    ToolCommon::Log($logFile,"Failed to chmod 1777 for directory '$tmpDir'\n",1);
  }
}

#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  my @addr = gethostbyname($params->{ISAM});
  my @ip = unpack( "C4", $addr[4]);
  my $ipAddr = join( '.', @ip );
  $isamsRef = [ [$ipAddr, $params->{ISAM}] ];
} else {
  ToolCommon::Log($logFile, "Reading Hostfile...\n", 1);
  $isamsRef = ToolCommon::ReadHostsFile($logFile, $params->{DEBUG}, $params->{HOSTS}, $NODEPREFIX);
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

while ($count < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $count) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      ToolCommon::Log($logFile, "Child $count: started as $pid\n", 1);
      push(@pids, $pid);
    } else {
      HandleIsams($logFile, \@isamPart, $count, \%config);
      exit SUCCESS_CODE;
    }
  }
  $count++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    $exitCode = $? >> 8;
    $signal = $? & 255;
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      ToolCommon::Log($logFile, sprintf("Child %d: exit code: '%d' (%s), signal: '%d'\n",
				     $childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal), 1);
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  ToolCommon::Log($logFile, "All childs finished, exiting\n", 1);
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
close $logFile;
exit $overAllExitCode;
