#!/usr/bin/perl
#==============================================================================
#-- File:         cleanUp.pl
#-- Date:         05.11.2005
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to clean up the contents of the working directory created
#--               as a result of the network migration.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2006/12/18 : First implementation
#--   1.0.1 - 2007/06/18 : ANTms51632 allow multiple users for same ISAM.
#--   1.0.2 - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.0.3 - 2009/03/30 : ANTmt50185
#--   1.0.5 - 2010/03/04 : PBMT x86 Changes - ANTmt88407
#--   1.0.6 - 2010/10/15 : ALU00808688 new LRM option --id (suppresses the --logfile option).  Must not be used in combination with --hosts
#--   1.0.7 - 2011/01/11 : LRM updates for lib-perl path
#--   1.0.8 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   1.0.9 - 2011/05/02 : ALU01178887
#--  1.0.10 - 2011/10/24 : logfile command line parameter can include a directory
#--  1.0.11 - 2013/04/29 : ALU01880147
#--  1.0.12 - 2016/07/15 : ALU02237966
#--  1.0.13 - 2016/09/28 : ALU02278259


my $VERSION = "cleanUp.pl - 1.0.13 - 2016/09/28";
#==============================================================================

use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use File::Copy;
use Getopt::Long;
use Socket;
use POSIX ":sys_wait_h";

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 
use PbmtConfig;
use Utils;
use Capability;
use ToolCommon;
use PBMTGlobal;
use Logger;
use ErrorCodes qw(:EXIT_CODES);
use version qw(:ALL);

#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars
#==============================================================================
my $TFTPDIR =     "/global/aws_var/aws/equipment";
my $TFTPWORKDIR = "tmp";
my $STATUSFILE =  "result.txt";
my $ID_FILENAME = "CLUP";
my $HOSTSFILE =   "/etc/hosts";
my $NODEPREFIX =  "AS-";

my %config;

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub Usage {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file>  --logfile <file> | --id <id> \\ \n";
  print STDERR "       [--debug] [--v] [--multi] [--noname]\n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         id       : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                    The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         noname   : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         multi    : include Unix username in NE logging directory path\n";
  exit CMD_INCOMPL_CODE;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub ParseCommandLine {
  my ($isam, $debug, $hosts, $logFile, $version, $multi, $noName, $id);
  my %params;
  my $options = GetOptions('isam=s', \$isam,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'logfile=s', \$logFile,
			   'v', \$version,
			   'multi', \$multi,
			   'noname', \$noName,
                           'id=s', \$id);

  #-- check options
  if (! $options) {
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();	
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: Reading hosts file '$hosts' failed: $!\n";
    Usage();
  }
  #-- fill hash, set defaults if needed
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $HOSTSFILE;
  }
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if( defined($debug));
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
  } else {
    $params{LOGFILE} = $logFile;
    if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
      my $logdir = $1;
      if (! -d $logdir) {
       if (!mkdir( $logdir, 0777)) {
         print STDERR "Cannot create directory '$logdir': $!\n";
         exit CREATE_DIR_FAILED_CODE;
       }
      }
    }
  }
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{NONAME} = defined($noName) ? 1 : 0;
  $params{SEPLOG} = 0 if defined($id);
  return \%params;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }

  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );

  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw( 
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli 
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

sub PrintOptionsSummary {
  my($argLogFH, $logOnScreen) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------
my $nodeStatusLog = undef;

sub HandleIsams {
  my ($execLogFH, $isams) = @_;
  my $isam;
  my $ret;
  my $errors = 0;
  my @failedNodes;
  my $userName = $params->{MULTI} ? getpwuid($<) : "";

  Utils::SetDebug($execLogFH) if ($params->{DEBUG});
  Capability::SetDebug($execLogFH) if ($params->{DEBUG});
  ToolCommon::SetDebug($execLogFH);
  foreach $isam (@{$isams}) {
    ToolCommon::Log($execLogFH, "#===================================================\n");
    ToolCommon::Log($execLogFH, sprintf("#== %s\n", $params->{NONAME} ? $isam->[0] : $isam->[1]));
    ToolCommon::Log($execLogFH, "#===================================================\n");
    my $dir = sprintf("$TFTPDIR/$TFTPWORKDIR/%s/$userName", $params->{NONAME} ? $isam->[0] : $isam->[1]);
    $dir =~ s/\/\//\//g;
    if ( ! -d $dir) {
      ToolCommon::Log($execLogFH, "$dir does not exist, skipping...");
      next;
    }
    if (!system("$config{RM} -Rf $dir/*")) {
      if (!rmdir($dir)) {
	ToolCommon::Log($execLogFH, "Removing directory $dir failed: $!\n");
	$errors++;
	push(@failedNodes, $params->{NONAME} ? $isam->[0] : $isam->[1]);
      }
    } else {
      ToolCommon::Log($execLogFH, "$config{RM} -Rf $dir/* failed, directory still exists: $!\n");
      $errors++;
      push(@failedNodes, $params->{NONAME} ? $isam->[0] : $isam->[1]);
    }
  }
  if ($errors) {
    ToolCommon::Log($execLogFH, "$errors error(s) occured for the following ISAMs, please verify logfile.\n", 1);
    foreach (@failedNodes) {
      ToolCommon::Log($execLogFH, "  $_\n", 1);
    }
  }
  ToolCommon::Log($execLogFH, "Finished...\n", 1);
  return ($errors ? EXIT_WARN_CODE : SUCCESS_CODE);
}



#------------------------------------------------------------------------------
#-- Adjust Tool defaults if needed (i.e. when defined in the parameter config
#-- file
#------------------------------------------------------------------------------
sub AdjustToolVars {

  $HOSTSFILE = $config{'HOSTSFILE'} if defined ($config{'HOSTSFILE'});
  $TFTPDIR = $config{'TFTPDIR'} if defined($config{'TFTPDIR'});
  $TFTPWORKDIR = $config{'TFTPWORKDIR'} if defined($config{'TFTPWORKDIR'});
  $STATUSFILE = $config{CLUP_STATUS_FILE} if defined($config{CLUP_STATUS_FILE});
}

#==============================================================================
#== Main program
#==============================================================================
my $count = 0;
my @pids;
my $pid;
my $warnCount = 0;

sub CleanUp {
  if (scalar @pids) {
    print STDERR sprintf("%s::%s\n", PBMT_VERSION, $VERSION);
    print STDERR "\nCaught Interupt (^C)...\n";
    foreach (@pids) {
      print STDERR "Killing process $_...\n";
      kill($_);
    }
  }
  exit(EXIT_SIGNAL_CODE);
}

$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my $level = $config{warn_level};
  $level = shift if (1 < scalar(@_));
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }

  if (!($msgStr =~ /Zlib/)) {
    print $execLogFH "Warning: $msgStr" if (defined($execLogFH));
    print STDERR "Warning: $msgStr";
    $warnCount++;
  }
};

#-- Read parameter config file and adjust tool defaults if needed 
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
%config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit(PARAM_CFG_NEEDS_UPDATE_CODE);
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars();

#-- Handle command line
$params = ParseCommandLine();

STDOUT->autoflush(1);
my $logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1);

Utils::SetDebug($execLogFH) if ($params->{DEBUG});  # -- 1.2.2

#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  ToolCommon::Log($execLogFH, "Reading Hostfile...\n", 1);
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $NODEPREFIX);
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

exit (HandleIsams($execLogFH, \@isams));
