#!/usr/bin/perl
#==============================================================================
#-- File:         convertCcEngine.pl
#-- Date:         24.01.2008
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to convert a CC engine to use MAC@ concentration
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent AND 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/01/24 : First implementation (RCR ANTms41225)
#--   1.0.1 - 2008/03/10 : ANTms87729.  Cope with all KPN configuration possibilities.
#--   1.0.2 - 2008/06/03 : ANTmt09938 - include transaction set and release.
#--   1.0.3 - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.0.4 - 2008/10/27 : ANTmt29255.  Some customers do not allow the usage of ping.
#--           2008/10/28 : ANTmt29410.  Make sure only bridge ports having an association with
#--                        the network vlan are considered (check on the bridgeport VLAN ID)
#--   1.0.5 - 2009/03/10 : ANTmt18506
#--   1.0.6 - 2009/05/07 : ANTmt56708
#--   1.0.7 - 2009/05/29 : ANTmt57839
#--   1.0.8 - 2009/06/11 : Allow SSH for CLI
#--   1.0.9 - 2009/06/29 : Some cleanup and best practises.
#--   1.0.10 - 2009/07/03: Changed scope for KPN.
#--   1.0.11 - 2009/07/23: ANTmt69142 Error in above fix.
#--   1.0.12 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.13 - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session
#--   1.0.14 - 2010/03/04 : ANTmg01050 - Total_MiB label change from Ex.x.y.z to Vx.x.x.y.z
#--   1.0.15 - 2010/03/15 : PBMT x86 Changes - ANTmt88407
#--   1.0.16 - 2011/01/11 : LRM updates for lib-perl path
#--   1.0.17 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   1.0.18 - 2011/05/02 : ALU01178887
#--   1.0.19 - 2012/05/11 : ALU01789702
#--   1.0.20 - 2012/06/13 : Change permissions for backup db file
#--   1.0.21 - 2013/04/29 : ALU01880147
#--   1.0.22 - 2016/07/15 : ALU02237966
#--   1.0.23 - 2016/09/28 : ALU02278259


my $VERSION = "convertCcEngine.pl - 1.0.23 - 2016/09/28";
#==============================================================================
use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 
#==============================================================================
use Crypt::Lite;
use Capability;
use IsamCli;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :CCCE);
use SNMPComm;
use Utils;
use version qw(:ALL);
#==============================================================================
#== Constants
#==============================================================================
my $HOSTSFILE =               "/etc/hosts";
my $TFTPDIR =                 "/var/opt/aws/common/tftpd/data";
my $TFTPWORKDIR =             "tmp";
my $MAXSESSIONS =             5;
my $USERNAME =                "";
my $PASSWORD =                "";
my $NODEPREFIX =              "AS-";
my $TELNETTIMEOUT =           10;
my $STATUSFILE =              "CCCEresult.txt";
my $NT_COMMUNITY =            "public";
my $RETRIES =                 5;
my $NO_PING =                 0;
#==============================================================================
#== Vars
#==============================================================================
my %config;

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub Usage {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> --vlan <vlanid> [-efm] \\ \n";
  print STDERR "       --defPrio <priority> [-maxcc n] [--sessions <nr>] [--seplog] [--noaction] [--confinfo] \\ \n";
  print STDERR "       [--rollback]  [--telnetto <minutes>] [--username <user>] \\ \n";
  print STDERR "       [--password <password>] [--v] [--debug] [--bufsize n] [--noping]\n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         vlan     : id of the network VLAN\n";
  print STDERR "         efm      : also include non-ADSL ports\n";
  print STDERR "         defPrio  : Default priority of new to be created PPPoX-relay cross-connect client-ports\n";
  print STDERR "         maxcc    : Maximum number of cross-connections for a new to be created PPPoX-relay cross-connect client-port\n";
  print STDERR "         sessions : number of parallel ISAMs to be treated (MAX=$MAXSESSIONS)\n";
  print STDERR "         username : ISAM username\n";
  print STDERR "         password : ISAM password\n";
  print STDERR "         seplog   : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noaction : just check, do not change the type\n";
  print STDERR "         debug    : extra logging for debugging purposes\n";
  print STDERR "         confinfo : print out all retrieved configration info\n";
  print STDERR "         rollback : roll back to original configuration\n";
  print STDERR "         bufsize  : size of CLI command return buffer (default is 15Mb)\n";
  print STDERR "         telnetto : timeout value for telnet session (in minutes)\n";
  print STDERR "         noping   : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  exit CMD_INCOMPL_CODE;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub ParseCommandLine {
  my ($isam, $nwVlanId, $username, $password, $sessions, $debug, $hosts, $logfile, $version, $sepLog, $noaction, $confInfo,
      $rollBack, $bufSize, $telnetTO, $defPrio, $maxCc, $noPing, $efm);
  my %params;
  my $options = GetOptions('isam=s', \$isam,
			   'noaction', \$noaction,
			   'vlan=i', \$nwVlanId,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'username=s', \$username,
			   'password=s', \$password,
			   'logfile=s', \$logfile,
			   'defPrio=i', \$defPrio,
			   'maxcc=i', \$maxCc,
			   'v', \$version,
			   'seplog', \$sepLog,
			   'confinfo', \$confInfo,
			   'rollback', \$rollBack,
			   'telnetto=i', \$telnetTO,
			   'bufsize=i', \$bufSize,
			   'noping', \$noPing,
			   'efm', \$efm);

  #-- check options
  if (! $options) {
    Usage();
  }
  if (defined($version)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logfile)) {
    print STDERR "ERROR: no logfile given\n";
    Usage();	
  }
  if (!defined($nwVlanId)) {
    print STDERR "ERROR: No network VLAN ID provided\n";
    Usage();
  }
  if (!defined($defPrio)) {
    print STDERR "ERROR: No default priority for new to be created PPPoX-relay cross-connect client-ports provided\n";
    Usage();
  }
  if (defined($maxCc) && $maxCc > MAXCC_MACCONC) {
    print STDERR sprintf("ERROR: maximum higher then what is supported for MAC@ concentration (%s)\n", MAXCC_MACCONC);
    Usage();
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: Hostsfile $hosts does not exist\n";
    Usage();
  }
  if (defined($username)) {
    $params{USERNAME} = $username;
  } else {
    $params{USERNAME} = $USERNAME;
  }
  if (defined($password)) {
    $params{PASSWORD} = $password;
  } else {
    $params{PASSWORD} = $PASSWORD;
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $MAXSESSIONS) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
	"($sessions > $MAXSESSIONS)\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $HOSTSFILE;
  }
  $params{UMASK_CHMOD} = ( (! defined $config{UMASK_OVERRIDE}) or $config{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = defined($debug) ? 1 : 0;
  $params{NWVLANID} = $nwVlanId;
  $params{LOGFILE} = $logfile;
  $params{SEPLOG} = defined($sepLog) ? 1 : 0;
  $params{NOACTION} = defined($noaction) ? 1 : 0;
  $params{CONFINFO} = defined($confInfo) ? 1 : 0;
  $params{ROLLBACK} = defined($rollBack) ? 1 : 0;
  $params{BUFSIZE} = defined($bufSize) ? $bufSize : 15;
  $params{BUFSIZE} = 100 if ($params{BUFSIZE} > 100);
  $params{BUFSIZE} *= 1048576; # Convert to MBytes
  $params{TELNETTIMEOUT} = defined($telnetTO) ? $telnetTO * 60 : 600; # default 10 minutes
  if (defined($maxCc)) {
    $params{MAXCC} = $maxCc;
  }
  $params{DEFPRIO} = $defPrio;
  $params{NOPING} = $NO_PING;
  $params{NOPING} = 1 if defined($noPing);
  $params{EFM} = defined($efm) ? 1 : 0;
  return \%params;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub PrintLibVersions {
  my ($fh, $out) = @_;

  if (defined($fh)) {
    ToolCommon::Log($fh, "Library versions:\n", $out);
    ToolCommon::Log($fh, sprintf("  %s\n", IsamCli::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ToolCommon::GetVersion()), $out);
  } else {
    print STDERR "Library versions:\n";
    print STDERR sprintf("  %s\n", IsamCli::GetVersion());
    print STDERR sprintf("  %s\n", ToolCommon::GetVersion());
  }
}

sub PrintOptionsSummary {
  my($fh, $out) = @_;

  ToolCommon::Log($fh, sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION), $out);
  ToolCommon::Log($fh, "Options:\n", $out);
  ToolCommon::Log($fh, "   Nr Sessions                             : $params->{SESSIONS}\n", $out);
  ToolCommon::Log($fh, "   Hosts file                              : $params->{HOSTS}\n", $out) if (!defined($params->{ISAM}));
  ToolCommon::Log($fh, "   Only One Isam                           : $params->{ISAM}\n", $out) if (defined($params->{ISAM}));
  ToolCommon::Log($fh, sprintf("   Ping                                    : %s\n", $params->{NOPING} ? "not allowed" : "allowed"), $out);
  ToolCommon::Log($fh, "   Network VLAN ID                         : $params->{NWVLANID}\n", $out);
  ToolCommon::Log($fh, "   Default priority for new CC client-ports: $params->{DEFPRIO}\n", $out);
  ToolCommon::Log($fh, "   max-cc for new  CC client-ports         : $params->{MAXCC}\n", $out) if (defined($params->{MAXCC}));
  ToolCommon::Log($fh, sprintf("   Telnet CLI command buffer size          : %d Mbytes\n", $params->{BUFSIZE} / 1048576), $out);
  ToolCommon::Log($fh, sprintf("   Log file per node                       : %s\n", $params->{SEPLOG} ? "Yes" : "No"), $out);
  ToolCommon::Log($fh, sprintf("   Telnet timeout                          : %d minutes\n", $params->{TELNETTIMEOUT} / 60), $out);
  ToolCommon::Log($fh, "   Debugging Enabled\n", $out) if $params->{DEBUG};
  ToolCommon::Log($fh, "   No reconfiguration, only testing\n", $out) if $params->{NOACTION};
  ToolCommon::Log($fh, "   Print retrieved CLI configuration information\n", $out) if $params->{CONFINFO};
  ToolCommon::Log($fh, "   Rollback original configuration\n", $out) if $params->{ROLLBACK};
  PrintLibVersions($fh, $out);
}

#------------------------------------------------------------------------------
#-- Parse PPPoX-relay CC engine configuration.
#-- Hash structure as follows:
#-- %pppCcEngine{VLANID}{ID}
#--                     {NAME}
#--                     {SERVICENAME}
#--                     {MAXPAYLOADTAG}
#--                     {LOCK}
#--                     {MONITOR}
#--                     {DSLIWFTAG}
#--                     {MACCONC}
#--
#-- $fh: pointer to a file handle for logging purposes
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete VRF structure.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseEngineConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseId) = @_;
  my $vlanId = 0;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+pppox\-relay\s+cross\-connect\s+engine\s+(\d+)/) {
	ToolCommon::Log($fh, "[ParseEngineConfig] line read: $line\n") if ($params->{DEBUG});
	push @{$cliResponseLines}, $line;
	$vlanId = $1;
	$line =~ s/configure\s+pppox\-relay\s+cross\-connect\s+engine\s+\d+//;
	# First remove the no-form parameters
	$line =~ s/no name//;
	$line =~ s/no service-name//;
	$line =~ s/no max\-payload\-tag//;
	$line =~ s/no monitor//;
	$line =~ s/no lock//;
	$line =~ s/no dslf\-iwf\-tag//;
	$line =~ s/no mac\-addr\-conc//;
	# now start parsing....
	if (!defined($configStorage->{$vlanId}{ID})) {
	  $configStorage->{$vlanId}{ID} = $vlanId;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcEngine->{$vlanId}{ID} as '%d\n",
				       $configStorage->{$vlanId}{ID})) if ($params->{DEBUG});
	}
	if ($line =~ s/name\s+([^\s]+)//) {
	  my $engineName = $1;
	  $configStorage->{$vlanId}{NAME} = $engineName;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcEngine->{$vlanId}{NAME} as '%s'\n",
				       $configStorage->{$vlanId}{NAME})) if ($params->{DEBUG});
	}
	if ($line =~ s/service\-name\s+([^\s]+)//) {
	  $configStorage->{$vlanId}{SERVICENAME} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcEngine->{$vlanId}{SERVICENAME} as '%s'\n",
				       $configStorage->{$vlanId}{SERVICENAME})) if ($params->{DEBUG});
	}
	if ($line =~ s/max\-payload\-tag\s+([^\s]+)//) {
	  $configStorage->{$vlanId}{MAXPAYLOADTAG} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vrfConfig->{$vlanId}{MAXPAYLOADTAG} as '%s'\n",
				       $configStorage->{$vlanId}{MAXPAYLOADTAG})) if ($params->{DEBUG});
	}
	if ($line =~ s/monitor//) {
	  $configStorage->{$vlanId}{MONITOR} = 1;
	  ToolCommon::Log($fh, "  Stored vrfConfig->{$vlanId}{MONITOR}\n") if ($params->{DEBUG});
	}
	if ($line =~ s/mac\-addr\-conc//) {
	  $configStorage->{$vlanId}{MACCONC} = 1;
	  ToolCommon::Log($fh, "  Stored vrfConfig->{$vlanId}{MACCONC}\n") if ($params->{DEBUG});
	}
	if ($line =~ s/lock//) {
	  $configStorage->{$vlanId}{LOCK} = $1;
	  ToolCommon::Log($fh, "  Stored vrfConfig->{$vlanId}{LOCK}\n") if ($params->{DEBUG});
	}
	if ($line =~ s/dslf-iwf-tag//) {
	  $configStorage->{$vlanId}{DSLIWFTAG} = 1;
	  ToolCommon::Log($fh, "  Stored vrfConfig->{$vlanId}{DSLIWFTAG}\n") if ($params->{DEBUG});
	}
	$line =~ s/\s//g;
	if ($line ne "") {
	  FamilyCommon::SetAndLogError($fh, "[ParseEngineConfig] Command not completely parsed, left: $line");
	  return 0;
	}
      } else {
	if (! ($line =~ /\#[-]+$/) &&
	    ! ($line =~ /[\$%\#>]\s*$/)) {
	  FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	  return 0;
	}
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse PPPoX-relay CC client-port configuration.
#-- Hash structure as follows:
#-- %pppCcClientPorts{bridgeport}{VLANID}
#--                              {DEFPRIO}
#--                              {MAXCC}
#--                              {QOSPROF}
#--
#-- There is one line per client-port in the response.
#--
#-- $fh: pointer to a file handle for logging purposes
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete VLAN structure.
#-- $releaseId: release indentification (for possible future usage).
# configure pppox-relay cross-connect client-port 1/1/7/1:8:35 vlan-id 100 default-priority 1
#------------------------------------------------------------------------------
sub ParseClientPortConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseId) = @_;
  my $bridgePort = "";

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+pppox-relay\s+cross-connect\s+client-port\s+([^\s]+)/) {
	ToolCommon::Log($fh, "[ParseClientPortConfig] line read: $line\n") if ($params->{DEBUG});
	push @{$cliResponseLines}, $line;
	$bridgePort = $1;
	$line =~ s/configure\s+pppox-relay\s+cross-connect\s+client-port\s+([^\s]+)//;
	if ($line =~ s/vlan\-id\s+([^\s]+)//) {
	  $configStorage->{$bridgePort}{VLANID} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{VLANID} as '%s'\n",
				       $configStorage->{$bridgePort}{VLANID})) if ($params->{DEBUG});
	}
	if ($line =~ s/default\-priority\s+(\d+)//) {
	  $configStorage->{$bridgePort}{DEFPRIO} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{DEFPRIO} as '%s'\n",
				       $configStorage->{$bridgePort}{DEFPRIO})) if ($params->{DEBUG});
	}
	if ($line =~ s/max\-cc\s+(\d+)//) {
	  $configStorage->{$bridgePort}{MAXCC} = $1;
#	  if ($configStorage->{$bridgePort}{MAXCC} > MAXCC_MACCONC) {
#	    FamilyCommon::SetAndLogError($fh, "[ParseClientPortConfig] max-cc of client-port $bridgePort too high for new configuration");
#	    return 0;
#	  } else {
	    ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{MAXCC} as '%s'\n",
					 $configStorage->{$bridgePort}{MAXCC})) if ($params->{DEBUG});
#	  }
	}
	if ($line =~ s/qos\-profile\s+([^\s]+)//) {
	  $configStorage->{$bridgePort}{QOSPROF} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{QOSPROF} as '%s'\n",
				       $configStorage->{$bridgePort}{QOSPROF})) if ($params->{DEBUG});
	}
	$line =~ s/\s//g;
	if ($line ne "") {
	  FamilyCommon::SetAndLogError($fh, "[ParseClientPortConfig] Command not completely parsed, left: $line");
	  return 0;
	}
      } else {
	if (! ($line =~ /\#[-]+$/) &&
	    ! ($line =~ /[\$%\#>]\s*$/)) {
	  FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	  return 0;
	}
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse bridge port info
#--
#-- This function is called to parse all bridge ports whaving netwrok-vlan set to the
#-- value of the VLAN ID associated to the CC engine.
#--
#-- Hash structure as follows:
#-- %bridgeConfig{$bridgePort}{CREATECC}: 1 indicates that a new CC clien-port has to be created for this bridge port
#--                           {DELBRIDGE}: indicates that this bridge needs to be removed when reconfiguring
#--                           {RECREATE}: 
#-- %vlanPortConfig{$bridgePort}{$vlan}{NETWORKVLAN}
#--                                    {VLANSCOPE}
#--                                    {QOS}
#-- $fh: pointer to a file handle for logging purposes
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $bridgeConfigStorage: pointer to a hash for storing the bridge related info.
#-- $vlanPortStorage: pointer to a hash for storing the vlan-port related info corresponding to the network vlan ID
#-- $pppCcClientPorts: pointer to a hash containing already retrieved PPPoX-relay CC client-ports
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseNwVLANBridgePortConfig {
  my ($fh, $response, $cliResponseLines, $bridgeConfigStorage, $vlanPortStorage, $pppCcClientPorts, $releaseNum) = @_;
  my $portId = "";
  my $vlanId;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+bridge\s+port\s+([^\s]+)/) {
	$portId = $1;
	push @{$cliResponseLines}, $line if ($line =~ /configure/);
	ToolCommon::Log($fh, "[ParseNwVLANBridgePortConfig] line read: $line\n") if ($params->{DEBUG});
	$line =~ s/configure\s+bridge\s+port\s+([^\s]+)//;
	if (! defined($pppCcClientPorts->{$portId}) && ! defined($bridgeConfigStorage->{$portId})) {
	  # Bridge port on EFM interface.  CC client-port needed in new config;
	  $bridgeConfigStorage->{$portId}{CREATECC} = 1;
	  ToolCommon::Log($fh, "  Stored create client-port indication for '$portId'\n") if ($params->{DEBUG});
	  if ($portId =~ /\d+:\d+/) {
	    # Bridge to be deleted in case of ATM interface
	    $bridgeConfigStorage->{$portId}{DELBRIDGE} = 1;
	    ToolCommon::Log($fh, "  Stored delete bridge port indication for '$portId'\n") if ($params->{DEBUG});
	  }
	} else {
	  $bridgeConfigStorage->{$portId}{DELBRIDGE} = 0;
	  ToolCommon::Log($fh, "  Bridge port '$portId' will no longer be present after reconfiguration\n") if ($params->{DEBUG});
	}
	if ($line =~ /vlan-id\s+(\d+)/) {
	  $vlanId = $1;
	}
	if ($line =~ /network\-vlan\s+([^\s]+)/) {
	  $vlanPortStorage->{$portId}{$vlanId}{NETWORKVLAN} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanPortConfig->{$portId}{$vlanId}{NETWORKVLAN} as '%s'\n",
				       $vlanPortStorage->{$portId}{$vlanId}{NETWORKVLAN})) if ($params->{DEBUG});
	}
	if ($line =~ /vlan\-scope\s+([^\s]+)/) {
	  $vlanPortStorage->{$portId}{$vlanId}{VLANSCOPE} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanPortConfig->{$portId}{$vlanId}{VLANSCOPE} as '%s'\n",
				       $vlanPortStorage->{$portId}{$vlanId}{VLANSCOPE})) if ($params->{DEBUG});
	}
	if ($line =~ /qos\s+([^\s]+)/) {
	  $vlanPortStorage->{$portId}{$vlanId}{QOS} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored bridge QOS for %s:%s as '%s'\n", $portId, $vlanId, $vlanPortStorage->{$portId}{$vlanId}{QOS}))
	    if ($params->{DEBUG});
	}
	if ($line =~ s/qos\-profile\s+([^\s]+)//) {
	  $vlanPortStorage->{$portId}{$vlanId}{QOSPROF} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanPortStorage->{$portId}{$vlanId}{QOSPROF} as '%s'\n",
				       $vlanPortStorage->{$portId}{$vlanId}{QOSPROF})) if ($params->{DEBUG});
	}
      }
    }
  }
}

#------------------------------------------------------------------------------
#-- Get configuration of involved CC engine bridge and VLAN ports
#--
#-- Function is call for every bridge port having vlan-port with the value of the VLAN ID
#-- associated to the CC engine.
#--
#-- Hash structure as follows:
#-- %bridgeConfig{$pvc}{PVID}
#--                    {PRIO}
#--                    {MACLEARNOFF}
#--                    {MAXUNICASTMAC}
#--                    {ACCEPTFRMTYPE}
#--                    {QOSPROF}
#--                    {PRIOREGENPROF}
#--                    {DELBRIDGE}: indicates that this bridge needs to be removed when reconfiguring.
#--                    {RECREATE}: indicates that this bridge needs to be re-created when rolling back.
#-- %vlanPortConfig{$pvc}{$vlan}{VLANID}
#--                             {TAG}
#--                             {NETWORKVLAN}
#--                             {VLANSCOPE}
#--                             {QOS}
#--                             {QOSPROF}
#--                             {PRIOBESTEFF}
#--                             {PRIOBACKGRND}
#--                             {PRIOSPARE}
#--                             {PRIOEXCEFF}
#--                             {PRIOCTRLLOAD}
#--                             {PRIOLESS100MS}
#--                             {PRIOLESS10MS}
#--                             {PRIONWCTRL}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $bridgeConfigStorage: pointer to a hash for storing the bridge related info.
#-- $vlanPortStorage: pointer to a hash for storing the vlan-port related info corresponding to the network vlan ID
#-- $clientPorts: all pppox CC client-ports retrieved.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub GetBridgePortConfig {
  my ($fh, $cli, $response, $cliResponseLines, $bridgeConfigStorage, $vlanPortStorage, $clientPorts, $nwVlanId, $releaseNum) = @_;
  my @fullConfig = ();
  my $rspStr = "";
  my ($pvc, $vlanId);
  my $bridgeVlan = 0;

  foreach my $line (@{$response}) {
    $vlanId = undef;
    # Loop on all the matching bridge ports
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+bridge\s+port\s+([^\s]+)/) {
	$pvc = $1;
	if ($line =~ /\svlan-id\s(\d+)/) {
	  $bridgeVlan = $1;
	  if ($bridgeVlan == $nwVlanId) {
	    if (!defined($bridgeConfigStorage->{$pvc})  && !defined($clientPorts->{$pvc})) {
	      # Bridge not present yet, so get the full configuration
	      @fullConfig = IsamCli::SendCommand($cli, "info configure bridge port $pvc flat");
	      if (defined($fullConfig[0])) {
		if (IsamCli::CommandError(join("", @fullConfig))) {
		  FamilyCommon::SetAndLogError($fh, sprintf("FAILED to retrieve bridge info: %s", IsamCli::GetError()));
		  return 0;
		} else {
		  foreach my $line (@fullConfig) {
		    chomp $line;
		    if ($line =~ /configure/) {
		      push @{$cliResponseLines}, $line if ($line =~ /configure/);
		      ToolCommon::Log($fh, "[GetBridgePortConfig] line read: $line\n") if ($params->{DEBUG});
		      $line =~ s/configure\s+bridge\s+port\s+$pvc//;
		      if ($line =~ s/pvid\s+([^\s]+)//) {
			my $pvId = $1;
			if ($pvc =~ /\d+:\d+/) {
			  # Only for ATM.  Delete bridge 
			  $bridgeConfigStorage->{$pvc}{PVID} = $pvId;
			  $bridgeConfigStorage->{$pvc}{DELBRIDGE} = 1;
			  $bridgeConfigStorage->{$pvc}{RECREATE} = 1;  # for Rollback in the deletion phase.  
			  $bridgeConfigStorage->{$pvc}{CREATECC} = 1;  # Create CC clientport without having a bridge port but on top of pvc
			  ToolCommon::Log($fh, sprintf("Stored bridge PVID for %s as '%d' (marked for port deletion and CC creation)\n",
						       $pvc, $bridgeConfigStorage->{$pvc}{PVID})) if ($params->{DEBUG});
			}
		      }
		      if ($line =~ s/default\-priority\s+([^\s]+)//) {
			$bridgeConfigStorage->{$pvc}{PRIO} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge priority for %s as '%d'\n", $pvc, $bridgeConfigStorage->{$pvc}{PRIO}))
			  if ($params->{DEBUG});
		      }
		      if ($line =~ s/mac\-learn\-off//) {
			$bridgeConfigStorage->{$pvc}{MACLEARNOFF} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge MAC learning off for %s\n", $pvc)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/max\-unicast\-mac\s+(\d+)//) {
			$bridgeConfigStorage->{$pvc}{MAXUNICASTMAC} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge max-unicast-mac for %s as '%d'\n", $pvc,
						     $bridgeConfigStorage->{$pvc}{MAXUNICASTMAC})) if ($params->{DEBUG});
		      }
		      if ($line =~ s/accept\-frame\-type\s+([^\s]+)//) {
			$bridgeConfigStorage->{$pvc}{ACCEPTFRMTYPE} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge accept-frame-type for %s as '%s'\n", $pvc,
						     $bridgeConfigStorage->{$pvc}{ACCEPTFRMTYPE})) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prio\-regen\-prof//) {
			$bridgeConfigStorage->{$pvc}{PRIOREGENPROF} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge priority regeneration profile for %s as '%s'\n", $pvc,
						     $bridgeConfigStorage->{$pvc}{PRIOREGENPROF})) if ($params->{DEBUG});
		      }
		      # Parsing of VLAN port specifics start here
		      if ($line =~ s/vlan\-id\s+([^\s]+)//) {
			$vlanId = $1;
			# Need this one to make sure that at least $vlanPortStorage->{$pvc}{$vlanId} gets defined.
			$vlanPortStorage->{$pvc}{$vlanId}{VLANID} = $vlanId;
			$vlanPortStorage->{$pvc}{$vlanId}{VLANSCOPE} = "network";
			ToolCommon::Log($fh, sprintf("Stored '%s' as VLAN id for %s, default scope\n", $vlanId, $pvc)) if ($params->{DEBUG});
			if ($vlanId == $params->{NWVLANID}) {
			  $bridgeConfigStorage->{$pvc}{CREATECC} = 1;  # VLAN scope and presence of network-vlan will have impact on whether
			  # the CC clienport is realy created: only in case of network scope
			  # or local scope and presence of network-vlan (vlan translation).
			  ToolCommon::Log($fh, sprintf("Stored create CC for %s:%s\n", $pvc, $vlanId)) if ($params->{DEBUG});
			}
		      }
		      if ($line =~ s/qos\-profile\s+([^\s]+)//) {
			# qos-profile can be on bridge or on vlan-port level.  In case of vlan port, $vlanId will have been set above.
			if (defined($vlanId)) {
			  $vlanPortStorage->{$pvc}{$vlanId}{QOSPROF} = $1;
			  ToolCommon::Log($fh, sprintf("Stored vlan-port qos-profile for %s as '%s'\n", $pvc,
						       $vlanPortStorage->{$pvc}{$vlanId}{QOSPROF}))
			    if ($params->{DEBUG});
			} else {
			  $bridgeConfigStorage->{$pvc}{QOSPROF} = $1;
			  ToolCommon::Log($fh, sprintf("Stored bridge qos-profile for %s as '%s'\n", $pvc, $bridgeConfigStorage->{$pvc}{QOSPROF}))
			    if ($params->{DEBUG});
			}
		      }
		      if ($line =~ s/tag\s+([^\s]+)//) {
			$vlanPortStorage->{$pvc}{$vlanId}{TAG} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge tag for %s:%s as '%s'\n", $pvc, $vlanId, $vlanPortStorage->{$pvc}{$vlanId}{TAG}))
			  if ($params->{DEBUG});
		      }
		      if ($line =~ s/network\-vlan\s+([^\s]+)//) {
			$vlanPortStorage->{$pvc}{$vlanId}{NETWORKVLAN} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge network VLAN for %s:%s as '%d'\n", $pvc, $vlanId,
						     $vlanPortStorage->{$pvc}{$vlanId}{NETWORKVLAN})) if ($params->{DEBUG});
		      }
		      if ($line =~ s/vlan\-scope\s+([^\s]+)//) {
			$vlanPortStorage->{$pvc}{$vlanId}{VLANSCOPE} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge VLAN scope for %s:%s as '%s'\n", $pvc, $vlanId,
						     $vlanPortStorage->{$pvc}{$vlanId}{VLANSCOPE})) if ($params->{DEBUG});
		      }
		      if ($line =~ s/qos\s+([^\s]+)//) {
			$vlanPortStorage->{$pvc}{$vlanId}{QOS} = $1;
			ToolCommon::Log($fh, sprintf("Stored bridge QOS for %s:%s as '%s'\n", $pvc, $vlanId, $vlanPortStorage->{$pvc}{$vlanId}{QOS}))
			  if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-best\-effort//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIOBESTEFF} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-best-effort for %s:%s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-background//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIOBACKGRND} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-background for %s:%s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-spare//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIOSPARE} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-spare for %s:%s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-exc\-effort//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIOEXCEFF} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-exc-effort for %s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-ctrl\-load//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIOCTRLLOAD} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-ctrl-load for %s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-less\-100ms//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIOLESS100MS} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-less-100ms for %s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-less\-10ms//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIOLESS10MS} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-less-10ms for %s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      if ($line =~ s/prior\-nw\-ctrl//) {
			$vlanPortStorage->{$pvc}{$vlanId}{PRIONWCTRL} = 1;
			ToolCommon::Log($fh, sprintf("Stored bridge prior-nw-ctrl for %s\n", $pvc, $vlanId)) if ($params->{DEBUG});
		      }
		      $line =~ s/\s//g;
		      if ($line ne "") {
			FamilyCommon::SetAndLogError($fh, "[GetBridgeConfig] Command not completely parsed, left: $line");
			return 0;
		      }
		    }
		  }
		}
	      } else {
		FamilyCommon::SetAndLogError($fh, sprintf("Empty CLI response received: %s", IsamCli::GetError()));
		return 0;
	      }
	    } else {
	      ToolCommon::Log($fh, "Bridge port '$pvc' already parsed, skipping...\n") if ($params->{DEBUG});
	    }
	  } else {
	    ToolCommon::Log($fh, "Bridge port '$pvc' associated with VLAN $bridgeVlan, not with $nwVlanId, skipping...\n") if ($params->{DEBUG});
	  }
	} else {
	  # Should not happen...
	  SetAndLogError($fh, "'$line' does not have 'vlan-id'");
	  return 0;
	}
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "'$line' not according to expected syntax");
	return 0;
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse VLAN port-protocol configuration.
#-- Hash structure as follows:
#-- %vlanPortProtocolConfig{bridgeport}{$vlanId}{PROTGRP}
#--                                             {PRIO}
#-- %bridgeConfig{$bridgeport}{DELBRIDGE}: indicates that this bridge needs to be removed when reconfiguring.
#--                           {RECREATE}: indicates that this bridge needs to be re-created when rolling back.
#--
#-- There is one line per VLAN port-protocol in the response.
#--
#-- $fh: pointer to a file handle for logging purposes
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete VLAN structure.
#-- $bridgeConfigStorage: pointer to a hash containing the bridge port configuration.
#-- $releaseId: release indentification (for possible future usage).
# configure vlan port-protocol 1/1/4/7:8:35 protocol-group pppoe vlan-id 10 priority 0
# configure vlan port-protocol 1/1/5/1 protocol-group pppoe vlan-id 10 priority 0
#------------------------------------------------------------------------------
sub ParseVlanPortProtocolConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $bridgePortConfig, $releaseId) = @_;
  my $bridgePort = "";
  my $vlanId;
  my $atmPvc;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ /configure\s+vlan\s+port\-protocol\s+([^\s]+)/) {
	$bridgePort = $1;
	$atmPvc = $bridgePort =~ /\d+:\d+/ ? 1 : 0;
	push @{$cliResponseLines}, $line;
	$line =~ s/configure\s+vlan\s+port\-protocol\s+([^\s]+)//;
	# First pick up VLAN id
	if ($line =~ s/vlan\-id\s+([^\s]+)//) {
	  $vlanId = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanId as '%s'\n", $vlanId)) if ($params->{DEBUG});
	}
	# Now the other settings....
	if ($line =~ s/protocol\-group\s+([^\s]+)//) {
	  $configStorage->{$bridgePort}{$vlanId}{PROTGRP} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanPortProtocolConfig->{$bridgePort}{$vlanId}{PROTGRP} as '%s'\n",
				       $configStorage->{$bridgePort}{$vlanId}{PROTGRP})) if ($params->{DEBUG});
	}
	if ($line =~ s/priority\s+(\d+)//) {
	  $configStorage->{$bridgePort}{$vlanId}{PRIO} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored vlanPortProtocolConfig->{$bridgePort}{$vlanId}{PRIO} as '%s'\n",
				       $configStorage->{$bridgePort}{$vlanId}{PRIO})) if ($params->{DEBUG});
	}
	if ((! defined($bridgePortConfig->{$bridgePort}{DELBRIDGE})) && $atmPvc) {
	  ToolCommon::Log($fh, "  Stored delete bridge port and create CC indication for '$bridgePort'\n") if ($params->{DEBUG});
	  $bridgePortConfig->{$bridgePort}{DELBRIDGE} = 1; # Get rid of bridge
	  $bridgePortConfig->{$bridgePort}{RECREATE} = 1;  # For rollback in the deletion part
	  $bridgePortConfig->{$bridgePort}{CREATECC} = 1;  # Create CC client port on top of PVC
	} else {
	  ToolCommon::Log($fh, "  Bridge port '$bridgePort' already marked for deletion\n")
	    if ($bridgePortConfig->{$bridgePort}{DELBRIDGE} && $params->{DEBUG});
	}
	$line =~ s/\s//g;
	if ($line ne "") {
	  FamilyCommon::SetAndLogError($fh, "[ParseVlanPortProtocolConfig] Command not completely parsed, left: $line");
	  return 0;
	}
      } else {
	if (! ($line =~ /\#[-]+$/) &&
	    ! ($line =~ /[\$%\#>]\s*$/)) {
	  FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	  return 0;
	}
      }
    }
  }
  return 1;
}
#------------------------------------------------------------------------------
# Generate PPPoX CC clientport deletion rollback command using the client-port
# configuration previously retrieved.
#------------------------------------------------------------------------------
sub RollBackCcPortDeletion {
  my ($bridgePort, $clientPorts, $releaseNum) = @_;

  my $rollBackCmd = "configure pppox-relay cross-connect client-port $bridgePort vlan-id $clientPorts->{$bridgePort}{VLANID}";
  $rollBackCmd .= " default-priority $clientPorts->{$bridgePort}{DEFPRIO}" if (defined($clientPorts->{$bridgePort}{DEFPRIO}));
  $rollBackCmd .= " max-cc $clientPorts->{$bridgePort}{MAXCC}" if (defined($clientPorts->{$bridgePort}{MAXCC}));
  $rollBackCmd .= " qos-profile $clientPorts->{$bridgePort}{QOSPROF}" if (defined($clientPorts->{$bridgePort}{QOSPROF}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate PPPoX CC engine deletion rollback command using the CC engine
# configuration previously retrieved.
# Part 1
#------------------------------------------------------------------------------
sub RollBackCcEngineDeletionPart1 {
  my ($vlanId, $ccEngine, $releaseNum) = @_;

  my $rollBackCmd = "configure pppox-relay cross-connect engine $vlanId";
  $rollBackCmd .= " name $ccEngine->{$vlanId}{NAME}" if (defined($ccEngine->{$vlanId}{NAME}));
  $rollBackCmd .= " service-name $ccEngine->{$vlanId}{SERVICENAME}" if (defined($ccEngine->{$vlanId}{SERVICENAME}));
  $rollBackCmd .= " max-payload-tag $ccEngine->{$vlanId}{MAXPAYLOADTAG}" if (defined($ccEngine->{$vlanId}{MAXPAYLOADTAG}));
  $rollBackCmd .= " lock" if (defined($ccEngine->{$vlanId}{LOCK}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate PPPoX CC engine deletion rollback command using the CC engine
# configuration previously retrieved.
# Part 2
#------------------------------------------------------------------------------
sub RollBackCcEngineDeletionPart2 {
  my ($vlanId, $ccEngine, $releaseNum) = @_;
  my $cmd = defined($ccEngine->{$vlanId}{MONITOR}) ? "configure pppox-relay cross-connect engine $vlanId monitor" : "";

  if ($cmd eq "") {
    $cmd .=  defined($ccEngine->{$vlanId}{DSLIWFTAG}) ? "configure pppox-relay cross-connect engine $vlanId dslf-iwf-tag" : "";
  } else {
    $cmd = defined($ccEngine->{$vlanId}{DSLIWFTAG}) ? " dslf-iwf-tag" : "";
  }
  if ($cmd eq "") {
    $cmd .= defined($ccEngine->{$vlanId}{MACCONC}) ? "configure pppox-relay cross-connect engine $vlanId mac-addr-conc" : "";
  } else {
    $cmd .= " mac-addr-conc" if defined($ccEngine->{$vlanId}{MACCONC});
  }
  return $cmd;
}

#------------------------------------------------------------------------------
# Generate bridge port deletion rollback command using the configuration
# previously retrieved.  Part 1
#------------------------------------------------------------------------------
sub RollBackBridgeDeletionPart1 {
  my ($bridgePort, $bridgePortConfig, $releaseNum) = @_;
  my $rollBackCmd = "configure bridge port $bridgePort";

  $rollBackCmd .= " default-priority $bridgePortConfig->{$bridgePort}{PRIO}" if defined($bridgePortConfig->{$bridgePort}{PRIO});
  $rollBackCmd .= " mac-learn-off" if defined($bridgePortConfig->{$bridgePort}{MACLEARNOFF});
  $rollBackCmd .= " max-unicast-mac $bridgePortConfig->{$bridgePort}{MAXUNICASTMAC}" if defined($bridgePortConfig->{$bridgePort}{MAXUNICASTMAC});
  $rollBackCmd .= " accept-frame-type $bridgePortConfig->{$bridgePort}{ACCEPTFRMTYPE}" if defined($bridgePortConfig->{$bridgePort}{ACCEPTFRMTYPE});
  $rollBackCmd .= " qos-profile $bridgePortConfig->{$bridgePort}{QOSPROF}" if defined($bridgePortConfig->{$bridgePort}{QOSPROF});
  $rollBackCmd .= " prio-regen-prof $bridgePortConfig->{$bridgePort}{PRIOREGENPROF}" if defined($bridgePortConfig->{$bridgePort}{PRIOREGENPROF});
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate bridge port deletion rollback command using the configuration
# previously retrieved.  Part 1
#------------------------------------------------------------------------------
sub RollBackBridgeDeletionPart2 {
  my ($bridgePort, $bridgePortConfig, $releaseNum) = @_;

  return defined($bridgePortConfig->{$bridgePort}{PVID}) ? "configure bridge port $bridgePort pvid $bridgePortConfig->{$bridgePort}{PVID}" : "";
}

#------------------------------------------------------------------------------
# Generate vlan port deletion rollback command using the configuration
# previously retrieved.
#------------------------------------------------------------------------------
sub RollBackBridgeVlanPort {
  my ($bridgePort, $vlanId, $vlanPortConfig, $releaseNum) = @_;
  my $rollBackCmd = "configure bridge port $bridgePort vlan-id $vlanId";

  $rollBackCmd .= " tag $vlanPortConfig->{$bridgePort}{$vlanId}{TAG}" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{TAG});
  $rollBackCmd .= " network-vlan $vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN}"
    if defined($vlanPortConfig->{$bridgePort}{$vlanId}{NETWORKVLAN});
  $rollBackCmd .= " vlan-scope $vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE}"
    if defined($vlanPortConfig->{$bridgePort}{$vlanId}{VLANSCOPE});
  $rollBackCmd .= " qos-profile $vlanPortConfig->{$bridgePort}{$vlanId}{QOSPROF}" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{QOSPROF});
  $rollBackCmd .= " qos $vlanPortConfig->{$bridgePort}{$vlanId}{QOS}" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{QOS});
  $rollBackCmd .= " prior-best-effort" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOBESTEFF});
  $rollBackCmd .= " prior-background" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOBACKGRND});
  $rollBackCmd .= " prior-spare" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOSPARE});
  $rollBackCmd .= " prior-exc-effort" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOEXCEFF});
  $rollBackCmd .= " prior-ctrl-load" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOCTRLLOAD});
  $rollBackCmd .= " prior-less-100ms" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOLESS100MS});
  $rollBackCmd .= " prior-less-10ms" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIOLESS10MS});
  $rollBackCmd .= " prior-nw-ctrl" if defined($vlanPortConfig->{$bridgePort}{$vlanId}{PRIONWCTRL});
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate vlan port protocol deletion rollback command using the configuration
# previously retrieved.
#------------------------------------------------------------------------------
sub RollBackBridgeVlanPortProtocol {
  my ($bridgePort, $vlanId, $vlanPortProtocolConfig, $releaseNum) = @_;
  my $rollBackCmd = "configure vlan port-protocol $bridgePort protocol-group $vlanPortProtocolConfig->{$bridgePort}{$vlanId}{PROTGRP}";

  $rollBackCmd .= " vlan-id $vlanId priority $vlanPortProtocolConfig->{$bridgePort}{$vlanId}{PRIO}";
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Prepare the CC-engine reconfiguration:
# 1. Clear out CC engine before it can be deleted: delete all client-ports
# 2. Delete CC-engine
# 3. Delete bridge ports in case of ATM PVC
# 4. Delete 
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $ccEngine: pointer to a hash of the CC engine.
# $clientPorts: pointer to a hash of all the clientports associated with that engine.
# $bridgeConfig: pointer to a hash of the bridge configuration.
# $vlanPortConfig: pointer to a hash for the VLAN port settings
# $vlanPortProtocolConfig: pointer to a hash for the VLAN port protocol settings
# $cliCommandStorage: pointer to an array holding all the unconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollkbac commands of the unconfiguration.
#------------------------------------------------------------------------------
sub PrepareDeletion {
  my ($fh, $ccEngine, $clientPorts, $bridgeConfig, $vlanPortConfig, $vlanPortProtocolConfig, $cliCommandStorage,
      $rollBackCmds, $releaseId) = @_;
  my $cmd = "";
  my $rollBackCmd = "";

  # 1. Delete the client-ports
  ToolCommon::Log($fh, "[PrepareDeletion]\n") if ($params->{DEBUG});
  foreach my $bridgePort (sort { my $a1 = $a; $a1 =~ s/\//\./g;
				 my $b1 = $b; $b1 =~ s/\//\./g;
				 pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
			  keys %{$clientPorts}) {
    $cmd = "configure pppox-relay cross-connect no client-port $bridgePort", 
    $rollBackCmd = RollBackCcPortDeletion($bridgePort, $clientPorts, $releaseId);
    $rollBackCmds->{$cmd}[0] = $rollBackCmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
    push @{$cliCommandStorage}, $cmd;
  }
  # 2. Delete CC engine
  foreach my $vlanId (keys %{$ccEngine}) {
    $cmd = "configure pppox-relay cross-connect no engine $vlanId";
    $rollBackCmd = RollBackCcEngineDeletionPart1($vlanId, $ccEngine, $releaseId);
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
    $rollBackCmd = RollBackCcEngineDeletionPart2($vlanId, $ccEngine, $releaseId);
    if ($rollBackCmd ne "") {
      ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
      ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
    }
    push @{$cliCommandStorage}, $cmd;
  }
  # 3. Delete bridge ports for ATM-PVC bridge ports converted to CC client-ports.
  #    Bridge port deletion propagates to deletion of vlan ports and vlan port-protocol settings (when defined)
  foreach my $bridge (sort { my $a1 = $a; $a1 =~ s/\//\./g;
			     my $b1 = $b; $b1 =~ s/\//\./g;
			     pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
		      keys %{$bridgeConfig}) {
    if (defined($bridgeConfig->{$bridge}{DELBRIDGE}) && ($bridgeConfig->{$bridge}{DELBRIDGE})) {
      $cmd = "configure bridge no port $bridge";
      ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
      push @{$cliCommandStorage}, $cmd;
      if ($bridgeConfig->{$bridge}{RECREATE}) {
	push @{$rollBackCmds->{$cmd}}, RollBackBridgeDeletionPart1($bridge, $bridgeConfig, $releaseId);
	foreach my $vlanId (sort keys %{$vlanPortConfig->{$bridge}}) {
	  push @{$rollBackCmds->{$cmd}}, RollBackBridgeVlanPort($bridge, $vlanId, $vlanPortConfig, $releaseId);
	  $rollBackCmd = RollBackBridgeDeletionPart2($bridge, $bridgeConfig, $releaseId);
	  push @{$rollBackCmds->{$cmd}}, $rollBackCmd if ($rollBackCmd ne "");
	}
	if (defined($vlanPortProtocolConfig->{$bridge})) {
	  foreach my $vlanId (sort keys %{$vlanPortProtocolConfig->{$bridge}}) {
	    push @{$rollBackCmds->{$cmd}}, RollBackBridgeVlanPortProtocol($bridge, $vlanId, $vlanPortProtocolConfig, $releaseId);
	  }
	}
	if ($params->{DEBUG}) {
	  foreach my $rollBackCmd (@{$rollBackCmds->{$cmd}}) {
	    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	  }
	}
      }
    }
  }
}

#------------------------------------------------------------------------------
# Generate cross-connect engine creation command.  Part 1
#------------------------------------------------------------------------------
sub PrepareEngineCreationPart1 {
  my ($vlanId, $ccEngineConfig, $concentrate, $releaseId) = @_;
  my $cmd = "configure pppox-relay cross-connect engine $vlanId";

  $cmd .= " mac-addr-conc" if $concentrate;
  $cmd .= " name $ccEngineConfig->{$vlanId}{NAME}" if defined($ccEngineConfig->{$vlanId}{NAME});
  $cmd .= " service-name $ccEngineConfig->{$vlanId}{SERVICENAME}" if defined($ccEngineConfig->{$vlanId}{SERVICENAME});
  $cmd .= " max-payload-tag $ccEngineConfig->{$vlanId}{MAXPAYLOADTAG}" if defined($ccEngineConfig->{$vlanId}{MAXPAYLOADTAG});
  $cmd .= " dslf-iwf-tag" if defined($ccEngineConfig->{$vlanId}{DSLIWFTAG});
  $cmd .= " lock" if defined($ccEngineConfig->{$vlanId}{LOCK});
  return $cmd;
}

#------------------------------------------------------------------------------
# Generate cross-connect engine creation command.  Part 1
#------------------------------------------------------------------------------
sub PrepareEngineCreationPart2 {
  my ($vlanId, $ccEngineConfig, $releaseId) = @_;

  return defined($ccEngineConfig->{$vlanId}{MONITOR}) ? "configure pppox-relay cross-connect engine $vlanId monitor" : "";
}

#------------------------------------------------------------------------------
# Generate cross-connect client-port creation command.
#------------------------------------------------------------------------------
sub PrepareClientPortCreation {
  my ($bridgePort, $clientPorts, $releaseId) = @_;
  my $cmd = "configure pppox-relay cross-connect client-port $bridgePort vlan-id $clientPorts->{$bridgePort}{VLANID}";

  $cmd .= " default-priority $clientPorts->{$bridgePort}{DEFPRIO}";
  $cmd .= " max-cc $clientPorts->{$bridgePort}{MAXCC}" if defined($clientPorts->{$bridgePort}{MAXCC});
  $cmd .= " qos-profile $clientPorts->{$bridgePort}{QOSPROF}" if defined($clientPorts->{$bridgePort}{QOSPROF});
  return $cmd;
}


#------------------------------------------------------------------------------
#  Create a new pppox-relay cc client-port.
#------------------------------------------------------------------------------
sub CreateNewCc {
  my ($bridgePort, $userVlan, $bridgePorts, $vlanPorts, $releaseId) = @_;
  my $cmd;
  
  if (defined($userVlan)) {
    if (defined($vlanPorts->{$bridgePort})) {
      if (defined($vlanPorts->{$bridgePort}{$userVlan})) {
	if (defined($vlanPorts->{$bridgePort}{$userVlan}{QOS})) {
	  if ($vlanPorts->{$bridgePort}{$userVlan}{QOS} =~ /priority:(\d+)/) {
	    $cmd = sprintf("configure pppox-relay cross-connect client-port vlan-port:%s:%s vlan-id %s default-priority %s",
			   $bridgePort, $userVlan, $params->{NWVLANID}, $1);
	  } else {
	    $cmd = sprintf("configure pppox-relay cross-connect client-port vlan-port:%s:%s vlan-id %s default-priority %s",
			   $bridgePort, $userVlan, $params->{NWVLANID}, $params->{DEFPRIO});
	  }
	} else {
	  $cmd = sprintf("configure pppox-relay cross-connect client-port vlan-port:%s:%s vlan-id %s default-priority %s",
			 $bridgePort, $userVlan, $params->{NWVLANID}, $params->{DEFPRIO});
	}
      }
    } else {
      $cmd = sprintf("configure pppox-relay cross-connect client-port vlan-port:%s:%s vlan-id %s default-priority %s",
		     $bridgePort, $userVlan, $params->{NWVLANID}, $params->{DEFPRIO});
    }
    $cmd .= " qos-profile $vlanPorts->{$bridgePort}{$userVlan}{QOSPROF}" if (defined($vlanPorts->{$bridgePort}{$userVlan}{QOSPROF}));
  } else {
    $cmd = "configure pppox-relay cross-connect client-port $bridgePort vlan-id $params->{NWVLANID} default-priority $params->{DEFPRIO}";
    $cmd .= " qos-profile $bridgePorts->{$bridgePort}{QOSPROF}" if (defined($bridgePorts->{$bridgePort}{QOSPROF}));
  }
  $cmd .= " max-cc $params->{MAXCC}" if defined($params->{MAXCC});
  return $cmd;
}

#------------------------------------------------------------------------------
# Prepare creation of CC engine with MAC@ concentration:
# 1. Recreate CC engine with MAC@ concentration on
# 2. Recreate CC client-ports existing before
# 3. Ensure that the remaining vlan-ports have a local vlan-scope and no network vlan defined.
# 4. Create new CC client-ports for bridge ports having PVID set of having a vlan-port protocol defined for the NW vlan
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $ccEngine: pointer to a hash of the CC engine.
# $clientPorts: pointer to a hash of all the clientports associated with that engine.
# $bridgeConfig: pointer to a hash of the bridge configuration.
# $cliCommandStorage: pointer to an array holding all the reconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollkbac commands of the reconfiguration.
#------------------------------------------------------------------------------
sub PrepareCreation {
  my ($fh, $ccEngine, $clientPorts, $bridgePorts, $vlanPorts, $vlanPortProtocolConfig, $cliCommandStorage,
      $rollBackCmds, $releaseId) = @_;
  my $cmd = "";
  my $rollBackCmd = "";

  ToolCommon::Log($fh, "[PrepareCreation]\n") if ($params->{DEBUG});
  # 1. Recreate PPP CC engine
  ToolCommon::Log($fh, "[PrepareCreation] Step 1\n") if ($params->{DEBUG});
  foreach my $vlanId (sort keys %{$ccEngine}) {
    $cmd = PrepareEngineCreationPart1($vlanId, $ccEngine, 1, $releaseId);
    push @{$cliCommandStorage}, $cmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    $rollBackCmd = "configure pppox-relay cross-connect no engine $vlanId";
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    $cmd = PrepareEngineCreationPart2($vlanId, $ccEngine, $releaseId);
    if ($cmd ne "") {
      push @{$cliCommandStorage}, $cmd;
      ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    }
  }
  ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  # 2. Recreate existing CC ports.
  ToolCommon::Log($fh, "[PrepareCreation] Step 2\n") if ($params->{DEBUG});
  foreach my $clientPort (sort { my $a1 = $a; $a1 =~ s/\//\./g;
				 my $b1 = $b; $b1 =~ s/\//\./g;
				 pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
			  keys %{$clientPorts}) {
    $cmd = PrepareClientPortCreation($clientPort, $clientPorts, $releaseId);
    push @{$cliCommandStorage}, $cmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n") if ($params->{DEBUG});
    $rollBackCmd = "configure pppox-relay cross-connect no client-port $clientPort";
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  }
  # 3. Ensure correct VLAN port settings
  ToolCommon::Log($fh, "[PrepareCreation] Step 3\n") if ($params->{DEBUG});
  foreach my $bridgePort (sort { my $a1 = $a; $a1 =~ s/\//\./g;
				 my $b1 = $b; $b1 =~ s/\//\./g;
				 pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
			  keys %{$vlanPorts}) {
    ToolCommon::Log($fh, "  bridgeport $bridgePort\n") if ($params->{DEBUG});
    if (!($bridgePort =~ /\d+:\d+/)) {
      # Only for EFM ports.  Non-ATM bridge ports will no longer be present after reconfiguration
      foreach my $vlanId (sort keys %{$vlanPorts->{$bridgePort}}) {
	if (($vlanId == $params->{NWVLANID}) ||
	    (defined($vlanPorts->{$bridgePort}{$vlanId}{NETWORKVLAN}) && ($vlanPorts->{$bridgePort}{$vlanId}{NETWORKVLAN} == $params->{NWVLANID}))) {
	  if ((!defined($vlanPorts->{$bridgePort}{$vlanId}{VLANSCOPE})) ||
	      (defined($vlanPorts->{$bridgePort}{$vlanId}{VLANSCOPE}) && ($vlanPorts->{$bridgePort}{$vlanId}{VLANSCOPE} eq "network"))) {
	    $cmd = "configure bridge port $bridgePort vlan-id $vlanId vlan-scope local";
	    push @{$cliCommandStorage}, $cmd;
	    ToolCommon::Log($fh, "  Stored '$cmd'\n") if ($params->{DEBUG});
	    $rollBackCmd = "configure bridge port $bridgePort vlan-id $vlanId no vlan-scope";
	    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	  } else {
	    # VLAN scope is local
	    if (defined($vlanPorts->{$bridgePort}{$vlanId}{NETWORKVLAN})) {
	      # Network VLAN has been defined and vlan scope is local -> VLAN translation
	      $cmd = "configure bridge port $bridgePort vlan-id $vlanId no network-vlan";
	      push @{$cliCommandStorage}, $cmd;
	      ToolCommon::Log($fh, "  Stored '$cmd'\n") if ($params->{DEBUG});
	      $rollBackCmd = "configure bridge port $bridgePort vlan-id $vlanId vlan-scope local network-vlan $params->{NWVLANID}";
	      push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	      ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	    }
	  }
	  if (defined($vlanPortProtocolConfig->{$bridgePort}{$vlanId})) {
	    $cmd = "configure vlan no port-protocol $bridgePort protocol-group $vlanPortProtocolConfig->{$bridgePort}{$vlanId}{PROTGRP}";
	    push @{$cliCommandStorage}, $cmd;
	    ToolCommon::Log($fh, "  Stored '$cmd'\n") if ($params->{DEBUG});
	    $rollBackCmd = RollBackBridgeVlanPortProtocol($bridgePort, $vlanId, $vlanPortProtocolConfig, $releaseId);
	    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	  }
	  # 4. Create new CC client-ports
	  # New client port is needed:
	  # ATM: bridge marked for deletion and marked for CC creation, see else clause
	  # EFM: - bridge marked for CC creation AND
	  #      - vlanid is equal to input network VLAN id and vlan-scope is network OR
	  #      - vlanid is local scope and network-vlan is equal to input network VLAN id (VLAN translation)
	  if (defined($bridgePorts->{$bridgePort}{CREATECC})) {
	    # vlanid is network VLAN
	    if ((($vlanId == $params->{NWVLANID}) &&
		 (defined($vlanPorts->{$bridgePort}{$vlanId}{VLANSCOPE}) && ($vlanPorts->{$bridgePort}{$vlanId}{VLANSCOPE} eq "network"))) ||
		# VLAN translation
		(defined($vlanPorts->{$bridgePort}{$vlanId}{VLANSCOPE}) && ($vlanPorts->{$bridgePort}{$vlanId}{VLANSCOPE} eq "local") &&
		 defined($vlanPorts->{$bridgePort}{$vlanId}{NETWORKVLAN}))) {
	      $cmd = CreateNewCc($bridgePort, $vlanId, $bridgePorts, $vlanPorts, $releaseId);
	      push @{$cliCommandStorage}, $cmd;
	      ToolCommon::Log($fh, "  Stored '$cmd'\n") if ($params->{DEBUG});
	      $rollBackCmd = "configure pppox-relay cross-connect no client-port vlan-port:$bridgePort:$vlanId";
	      push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	      ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	    }
	  }
	}
      }
    } else {
      # create new CC ATM case.
      if (defined($bridgePorts->{$bridgePort}{CREATECC})) {
	if (!defined($clientPorts->{$bridgePort})) {
	  $cmd =  CreateNewCc($bridgePort, undef, $bridgePorts, $vlanPorts, $releaseId);
	  push @{$cliCommandStorage}, $cmd;
	  ToolCommon::Log($fh, "  Stored '$cmd'\n") if ($params->{DEBUG});
	  $rollBackCmd = "configure pppox-relay cross-connect no client-port $bridgePort";
	  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	}
      }
    }
  }
}

#------------------------------------------------------------------------------
#  Backup system database for safety (may be needed if reconfiguration failed
# dramatically and CLI restoration failed.
#------------------------------------------------------------------------------
sub BackupDatabase {
  my ($fh, $cli, $isam, $debug, $parentFh, $logfileName, $count) = @_;
  my $time1 = time();
  my $oswpInfo;
  my $ok = 1;
  my $tftpIpAddress = "";
  my $fileName = "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
  my $cmd = "$config{RM} $fileName > CCCErm.txt 2>&1;$config{TOUCH} $fileName";
  my $sysret = system($cmd);

  ToolCommon::Log($fh, "\nCMD = $cmd\n") if ($debug);
  if ($sysret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($sysret);
    FamilyCommon::SetAndLogError($fh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** $cmd FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "Touching $fileName OK\n") if $debug;
  }
  if ($params->{UMASK_CHMOD} and ! chmod(0666, "$fileName")) {
    FamilyCommon::SetAndLogError($fh, "chmod(666,$fileName) FAILED: $!", "  FAILED: $!\n");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** chmod(666,$fileName) FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  }
  if ($ok) {
    $oswpInfo = IsamCli::GetOswpInfo($cli, REGEXP);
    if (!defined($oswpInfo)) {
      FamilyCommon::SetAndLogError($fh, sprintf("Failed to retrieve OSWP info: %s", IsamCli::GetError()));
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Active OSWP: $oswpInfo->{ACTIVE}{INDEX}\n") if $debug;
      my @response = IsamCli::SendCommand($cli, "info configure software-mngt oswp flat", undef);
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::SetAndLogError($fh, sprintf("Retrieving oswp configuration failed: %s", IsamCli::GetError()));
	$ok = 0;
      } else {
	foreach my $line (@response) {
	  chomp $line;
	  ToolCommon::Log($fh, "Line read: $line\n") if $debug;
	  if ($line =~ /configure\s+software\-mngt\s+oswp\s+(\d+)\s+primary\-file\-server\-id\s+([^\s]+)/) {
	    my $oswpNum = $1;
	    my $ipAddress = $2;
	    if ($oswpInfo->{ACTIVE}{INDEX} == $oswpNum) {
	      $tftpIpAddress = $ipAddress;
	      last;
	    }
	  } else {
	    if (! ($line =~ /\#[-]+$/) &&
		! ($line =~ /[\$%\#>]\s*$/)) {
	      # Should not happen
	      FamilyCommon::SetAndLogError($fh, "OSWP config info not according to expected syntax: $line");
	      $ok = 0;
	      last;
	    }
	  }
	}
	if (($tftpIpAddress ne "") && ($tftpIpAddress ne "0.0.0.0")) {
	  my $ret = IsamCli::UploadDb($cli, 'actual-active',
				      $tftpIpAddress, "$TFTPWORKDIR/$isam", 1);
	  if (!defined($ret) || !$ret)  {
	    FamilyCommon::SetAndLogError($fh, sprintf("  FAILED (%s)\n", IsamCli::GetError()));
	    $ok = 0;
	  } else {
	    my $dbSize = -s "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
	    if ($dbSize == 0) {
	      FamilyCommon::SetAndLogError($fh, sprintf("  Size of database file 0\n"));
	      $ok = 0;
	    } else {
	      ToolCommon::Log($fh, sprintf("  OK, transferred %d bytes in %d seconds\n", $dbSize, time() - $time1));
	      ToolCommon::Log($fh, "Database backup stored as $fileName\n", 1);
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, "No TFTP IP address could be obtained\n");
	  $ok = 0;
	}
      }
    }
  }
  my @response = IsamCli::SendCommand($cli, "exit all");  # Set prompt to top node
  return $ok;
}

#------------------------------------------------------------------------------
# Rollback configuration in case of error (or forced as command option)
#------------------------------------------------------------------------------
sub RollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;
  my @response;

  if ($params->{ROLLBACK}) {
    ToolCommon::Log($fh, "==================================\n");
    ToolCommon::Log($fh, "The following will be rolled back:\n");
    ToolCommon::Log($fh, "==================================\n");
  } else {
    ToolCommon::Log($fh, "Start rollback...\n");
  }
  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "  Rolling back '$rollBackCmd'...\n");
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	if ($params->{ROLLBACK}) {
	  ToolCommon::Log($fh, "    '$cmd'\n");
	} else {
	  ToolCommon::Log($fh, sprintf("    %s '%s'\n", $params->{NOACTION} ? "Simulating" : "Sending", $cmd));
	}
	if (! $params->{NOACTION}) {
	  @response = IsamCli::SendCommand($cli, $cmd);
	  return if (IsamCli::CommandError(join("", @response)));
#	  sleep 2;
	}
      }
    }
  }
  ToolCommon::Log($fh, "  Done\n");
}

#------------------------------------------------------------------------------
# Debugging: dump the content of the rollback command buffer.
#------------------------------------------------------------------------------
sub DumpRollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;

  ToolCommon::Log($fh, "==================================\n");
  ToolCommon::Log($fh, "CLI commands in case of roll back:\n");
  ToolCommon::Log($fh, "==================================\n");

  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "Rolling back '$rollBackCmd'...\n");
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	ToolCommon::Log($fh, sprintf("  Simulating '%s'\n", $cmd));
      }
    }
  }
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------
sub HandleIsams {
  my ($fh, $isams, $count, $config) = @_;
  my $isam;
  my ($time1, $time2);
  my $startTime;
  my $nodeLogFh;
  my $nodeLogFile;
  my $nodeStatusLog;
  my $cli = undef;
  my $releaseNum;
  my %pppCcEngine;
  my %pppCcClientPorts;
  my %bridgeConfig;
  my %vlanPortConfig;
  my %vlanPortProtocolConfig;
  my @configToDelete;
  my @configToCreate;
  my @configRetrieved = ();
  my %rollBackCommandConfig = ();
  my @rollBackCommands = ();
  my $cliCommandsSent = 0;
  my $reconfigStart;
  my $ret;

  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetParentLogFile($fh);
  FamilyCommon::SetChildNum($count);
  my $newLog = "$params->{LOGFILE}_$count";
  my $newfh;
  if (! open($newfh, ">$newLog")) {
    ToolCommon::Log($fh, "Child $count: cannot open $newLog for writing: $!\n", 1);
    CORE::exit(CREATE_FILE_FAIL_CODE)
  }
  $newfh->autoflush(1);
  ToolCommon::Log($fh, "Child $count: Logging continuous in $newLog...\n", 1);

  PrintOptionsSummary($newfh, undef);

  foreach $isam (@{$isams}) {
    FamilyCommon::SetFailedNode($isam->[1]);
    # Switch debugging (if any) to an opened filehandle...
    Utils::SetDebug($newfh) if ($params->{DEBUG});
    IsamCli::SetDebug($newfh)  if ($params->{DEBUG});
    Capability::SetDebug($newfh) if ($params->{DEBUG});
    ToolCommon::SetDebug($newfh);

    ToolCommon::Log($newfh, "#===================================================\n");
    ToolCommon::Log($newfh, "#== $isam->[1]\n");
    ToolCommon::Log($newfh, "#===================================================\n");
    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Started $isam->[1]...\n",
		    $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second), 1);

    my $dir = "$TFTPDIR/$TFTPWORKDIR/$isam->[1]";
    $dir =~ s/\/\//\//g;
    if ( ! -d $dir) {
      ToolCommon::Log($newfh, "Creating working dir $dir...");
      if (!mkdir( $dir, 0777)) {
	ToolCommon::Log($newfh, "  FAILED\n");
	ToolCommon::Log($fh, "Child $count: **FATAL** Creating working dir $dir for $isam->[1] failed, please verify $newLog.\n", 1);
	CORE::exit(CREATE_DIR_FAILED_CODE);
      }
      ToolCommon::Log($newfh, "  OK\n");
    }
    if (! chmod(01777, "$dir")) {
       ToolCommon::Log($fh,"Failed to chmod 1777 for directory '$dir'\n",1);
    }

    if ($params->{SEPLOG}) {
      $nodeLogFile = sprintf("$dir/commit_%04d-%02d-%02d_%02d:%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
      if (! open($nodeLogFh, ">$nodeLogFile")) {  # -- 1.2.5 START
	ToolCommon::Log($fh, "Child $count: cannot open $nodeLogFile for writing: $!\n", 1);
	CORE::exit(CREATE_FILE_FAIL_CODE)
      } else {
	ToolCommon::Log($fh, "Child $count: NE specific logging to $nodeLogFile...\n", 1);
      }
      $nodeLogFh->autoflush(1);
      Utils::SetDebug($nodeLogFh) if ($params->{DEBUG});
      IsamCli::SetDebug($nodeLogFh)  if ($params->{DEBUG});
      Capability::SetDebug($nodeLogFh) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFh);
    } else {
      $nodeLogFh = $newfh;
    }
    FamilyCommon::SetWarnLogFile($nodeLogFh);
    FamilyCommon::SetChildLogFile($nodeLogFh);

    if (! open($nodeStatusLog, ">$dir/$STATUSFILE")) {
      ToolCommon::Log($fh, "Child $count: cannot create $dir/$STATUSFILE for $isam->[1]: $!\n", 1);
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    }
    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    ### Is Isam reachable through Ping command
    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }

    $params->{SNMP}{V1}{NT_COMMUNITY} = $NT_COMMUNITY;
    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
        FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError(), undef );
        next;
    }
    my $mibVersion = SNMPComm::GetValue('MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         next if( FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError(), undef ) );
    }
      if (($mibVersion =~ /3FE21961EAAA_E(\d+)\.(\d+)\.\d+\.\d+/)       ||  # ETSI
          ($mibVersion =~ /3FE21961EAAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/)  ||  # ETSI  (ISAM MIB label Extn)
	  ($mibVersion =~ /3FE21961ANAA_E(\d+)\.(\d+)\.\d+\.\d+/)       ||  # ANSI
	  ($mibVersion =~ /3FE21961ANAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/)) {  
	my $major = $1; my $minor = $2;
	$releaseNum = Capability::ConvertRelease($major, $minor);
	if (Capability::ReleaseCompare($releaseNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
	  FamilyCommon::IncrErrorCount(sprintf("  ERROR: Need at least R3.5 to do this reconfiguration (current release : %s)\n",
					       Capability::NtDbVersionInfo(Capability::IsamFamily(), $releaseNum)),
				       EC_RELEASE_TOO_LOW,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      } else {
	FamilyCommon::IncrErrorCount("  ERROR: unsupported MIB version ($mibVersion)\n",
				     EC_INV_MIB_VERSION,
				     "($mibVersion)",
				     1);
	next;
      }
    ToolCommon::Log($nodeLogFh, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
				   EC_GET_ACC_BITS_FAILED,
				   undef,
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
      # Check whether this IP address is found in the know hosts file(s).
      if (! ToolCommon::SshPossible($isam->[0], $params->{DEBUG}, $config)) {
	FamilyCommon::IncrErrorCount(sprintf("No SSH access possible by PBMT: %s", ToolCommon::GetError()),
				     EC_NO_SSH_POSS,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	next;
      } else {
	$params->{USESSH} = 1;
	ToolCommon::Log($nodeLogFh, "No CLI access via Telnet possible, forced usage of SSH\n", 1);
      }
    }

    #-- Open CLI Session
    $cli = ToolCommon::OpenCli($isam, $nodeLogFh, $params->{TELNETTIMEOUT});
    if (!defined($cli)) {
      FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for $isam->[1]: %s\n", IsamCli::GetError()),
				   EC_OPENCLI_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    }
    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount

    my $prev = $cli->max_buffer_length($params->{BUFSIZE});
    ToolCommon::Log($nodeLogFh, sprintf("Increased input buffer length from %d to %d bytes\n", $prev, $params->{BUFSIZE}));

    if (!IsamCli::SetSessionTimeout($cli, 120)) {
      FamilyCommon::IncrErrorCount(sprintf("  Set session timeout FAILED: %s\n", IsamCli::GetError()),
				   EC_SET_TIMEOUT_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    ToolCommon::Log($nodeLogFh, "Backup NT Database...");
    if (! BackupDatabase($nodeLogFh, $cli, $isam->[1], $params->{DEBUG}, FamilyCommon::GetParentLogFile(), $newLog, $count)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_BACKUPDB_FAILED,
				   undef,
				   1);
      next;
    }

    # Here we go....
    @configRetrieved = ();
    %pppCcEngine = ();
    ToolCommon::Log($nodeLogFh, "Retrieve pppox-relay cross-connect engine $params->{NWVLANID} info...");
    my @response = IsamCli::SendCommand($cli, "info configure pppox-relay cross-connect engine $params->{NWVLANID} detail flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") != -1) {
	  FamilyCommon::IncrErrorCount("  FAILED: cross=connect engine does not exist\n",
				       EC_NO_SUCH_ENGINE,
				       "($params->{NWVLANID})",
				       1);
	  next;
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_ENGINE_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	  ToolCommon::Log($nodeLogFh, "Parsing PPPoX-relay CC engine configuration...");
	if ( !ParseEngineConfig($nodeLogFh, \@response, \@configRetrieved, \%pppCcEngine, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_ENGINE_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  if ($pppCcEngine{$params->{NWVLANID}}{MACCONC}) {
	    # MAC@ concentration already enabled no need to continue.
	    FamilyCommon::IncrErrorCount("MAC address concentration already enabled\n",
					 EC_MAC_CONC_ENABLED,
					 undef,
					 1);
	    next;
	  }
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    ToolCommon::Log($nodeLogFh, "Retrieve PPPoX-relay CC client-port info...");
    @response = IsamCli::SendCommand($cli,
				     "info configure pppox-relay cross-connect client-port flat | match exact:\"vlan-id $params->{NWVLANID}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") != -1) {
	  FamilyCommon::IncrErrorCount("  FAILED: does not exist\n",
				       EC_NO_SUCH_CLIENTPORT,
				       "($params->{NWVLANID})",
				       1);
	  next;
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_CLIENTPORT_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	%pppCcClientPorts = ();
	ToolCommon::Log($nodeLogFh, "Parsing client port configuration...");
	if ( !ParseClientPortConfig($nodeLogFh, \@response, \@configRetrieved, \%pppCcClientPorts, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_CLIENTPORT_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    if ($params->{EFM}) {
      ToolCommon::Log($nodeLogFh, "Retrieve bridge port info (network-vlan)...");
      @response = IsamCli::SendCommand($cli, "info configure bridge flat | match exact:\"network-vlan $params->{NWVLANID}\"");
      if (defined($response[0])) {
	if (IsamCli::CommandError(join("", @response))) {
	  if (index(IsamCli::GetError(), "does not exist") != -1) {
	    FamilyCommon::IncrErrorCount("  FAILED: does not exist\n",
					 EC_NO_SUCH_BRIDGEPORT,
					 "($params->{NWVLANID})", 1);
	    next;
	  } else {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_INFO_CONF_BRIDGEPORT_ERROR,
					 undef,
					 1);
	    next;
	  }
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	  %bridgeConfig = ();
	  %vlanPortConfig = ();
	  ToolCommon::Log($nodeLogFh, "Parsing bridge port configuration (network-vlan)...");
	  ParseNwVLANBridgePortConfig($nodeLogFh, \@response, \@configRetrieved, \%bridgeConfig, \%vlanPortConfig,
				      \%pppCcClientPorts, $releaseNum);
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				     EC_EMPTY_RESPONSE,
				     undef,
				     1);
	next;
      }

      ToolCommon::Log($nodeLogFh, "Retrieve bridge port list (vlan-id)...");
      @response = IsamCli::SendCommand($cli, "info configure bridge flat | match exact:\"vlan-id $params->{NWVLANID}\"");
      if (defined($response[0])) {
	if (IsamCli::CommandError(join("", @response))) {
	  if (index(IsamCli::GetError(), "does not exist") != -1) {
	    FamilyCommon::IncrErrorCount("  FAILED: does not exist\n",
					 EC_NO_SUCH_BRIDGEPORT,
					 "($params->{NWVLANID})",
					 1);
	    next;
	  } else {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_INFO_CONF_BRIDGEPORT_ERROR,
					 undef,
					 1);
	    next;
	  }
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	  ToolCommon::Log($nodeLogFh, "Retrieve bridge port configuration (vlan-id)...");
	  if ( !GetBridgePortConfig($nodeLogFh, $cli, \@response, \@configRetrieved, \%bridgeConfig, \%vlanPortConfig, \%pppCcClientPorts,
				    $releaseNum)) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
					 EC_BRIDGEPORT_PARSE_ERROR,
					 undef,
					 1);
	    next;
	  } else {
	    ToolCommon::Log($nodeLogFh, "  OK\n");
	  }
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				     EC_EMPTY_RESPONSE,
				     undef,
				     1);
	next;
      }

      ToolCommon::Log($nodeLogFh, "Retrieve VLAN port-protocol info...");
      @response = IsamCli::SendCommand($cli, "info configure vlan port-protocol flat | match exact:\"vlan-id $params->{NWVLANID}\"");
      if (defined($response[0])) {
	if (IsamCli::CommandError(join("", @response))) {
	  if (index(IsamCli::GetError(), "does not exist") != -1) {
	    FamilyCommon::IncrErrorCount("  FAILED: does not exist\n",
					 EC_NO_SUCH_VLAN_PP,
					 "($params->{NWVLANID})",
					 1);
	    next;
	  } else {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_INFO_CONF_VLAN_PP_ERROR,
					 undef,
					 1);
	    next;
	  }
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	  %vlanPortProtocolConfig = ();
	  ToolCommon::Log($nodeLogFh, "Parsing VLAN port-protocol configuration...");
	  if ( !ParseVlanPortProtocolConfig($nodeLogFh, \@response, \@configRetrieved, \%vlanPortProtocolConfig, \%bridgeConfig,
					    $releaseNum)) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
					 EC_VLAN_PP_PARSE_ERROR,
					 undef,
					 1);
	    next;
	  } else {
	    ToolCommon::Log($nodeLogFh, "  OK\n");
	  }
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				     EC_EMPTY_RESPONSE,
				     undef,
				     1);
	next;
      }
    }
    if ($params->{CONFINFO}) {
      ToolCommon::Log($nodeLogFh, "====================================================================\n");
      ToolCommon::Log($nodeLogFh, "The following configuration info has been read, parsed and stored...\n");
      ToolCommon::Log($nodeLogFh, "====================================================================\n");
      foreach (@configRetrieved) {
	ToolCommon::Log($nodeLogFh, "  $_\n");
      }
    }

    @configToDelete = ();
    %rollBackCommandConfig = ();
    PrepareDeletion($nodeLogFh, \%pppCcEngine, \%pppCcClientPorts, \%bridgeConfig, \%vlanPortConfig, \%vlanPortProtocolConfig,
		    \@configToDelete, \%rollBackCommandConfig, $releaseNum);
    @configToCreate = ();
    PrepareCreation($nodeLogFh, \%pppCcEngine, \%pppCcClientPorts, \%bridgeConfig, \%vlanPortConfig, \%vlanPortProtocolConfig,
		    \@configToCreate, \%rollBackCommandConfig, $releaseNum);
    if ($params->{CONFINFO}) {
      ToolCommon::Log($nodeLogFh, "==============================\n");
      ToolCommon::Log($nodeLogFh, "The following will be deleted:\n");
      ToolCommon::Log($nodeLogFh, "==============================\n");
      foreach (@configToDelete) {
	ToolCommon::Log($nodeLogFh, "  $_\n");
      }
      ToolCommon::Log($nodeLogFh, "==============================\n");
      ToolCommon::Log($nodeLogFh, "The following will be created:\n");
      ToolCommon::Log($nodeLogFh, "==============================\n");
      foreach (@configToCreate) {
	ToolCommon::Log($nodeLogFh, "  $_\n");
      }
    }
    my $deletionOk = 1;
    my $creationOk = 1;
    @rollBackCommands = ();

    # Now start the reconfiguration... first delete what has to be deleted.
    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership...\n");
    return undef if( ToolCommon::AcquireTransaction() );
    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership... DONE\n");

    ToolCommon::Log($nodeLogFh, "Start deletion of resources...\n") if (! $params->{NOACTION});
    $reconfigStart = time();
    foreach my $cliCmd (@configToDelete) {
      ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd));
      if (! $params->{NOACTION}) {
	$cliCommandsSent++;
	@response = IsamCli::SendCommand($cli, $cliCmd);
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    $deletionOk = 0;
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_DEL_CONFIG_ERROR,
					 undef,
					 undef);
	    last;
	  } else {
	    if (defined($rollBackCommandConfig{$cliCmd})) {
	      push @rollBackCommands, $cliCmd;
	      ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						  join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	    }
	    #	  sleep 2;
	  }
	} else {
	  $deletionOk = 0;
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_DEL_CONFIG_ERROR,
				       undef,
				       undef);
	  if (IsamCli::ConnectionReset()) {
	    ToolCommon::Log($fh,
			    "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	    sleep 1;
	    CORE::exit(EXIT_WARN_CODE);
	  }
	  last;
	}
      } else {
	if (defined($rollBackCommandConfig{$cliCmd})) {
	  push @rollBackCommands, $cliCmd;
	  ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd'\ -> '%s'n",
					      join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	}
      }
    }
    if (! $params->{NOACTION}) {
      if (! $deletionOk) {
	if (!RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_ROLLBACK_ERROR,
				       undef,
				       1);
	}
	next;
      }
    }
    ToolCommon::Log($nodeLogFh, "  Done\n") if (! $params->{NOACTION});
    sleep 10;

    ToolCommon::Log($nodeLogFh, "Start creation of resources...\n") if (! $params->{NOACTION});
    foreach my $cliCmd (@configToCreate) {
      ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd));
      if (! $params->{NOACTION}) {
	$cliCommandsSent++;
	@response = IsamCli::SendCommand($cli, $cliCmd);
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_CREATE_CONFIG_ERROR,
					 undef,
					 undef);
	    $creationOk = 0;
	    last;
	  } else {
	    if (defined($rollBackCommandConfig{$cliCmd})) {
	      push @rollBackCommands, $cliCmd;
	      ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						  join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	    }
	    #sleep 2;
	  }
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_CREATE_CONFIG_ERROR,
				       undef,
				       undef);
	  if (IsamCli::ConnectionReset()) {
	    ToolCommon::Log($fh,
			    "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	    sleep 1;
	    CORE::exit(EXIT_WARN_CODE);
	  }
	  $creationOk = 0;
	  last;
	}
      } else {
	if (defined($rollBackCommandConfig{$cliCmd})) {
	  push @rollBackCommands, $cliCmd;
	  ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
					      join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	}
      }
    }
    if (! $creationOk) {
      if (! RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				     EC_ROLLBACK_ERROR,
				     undef,
				     1);
      }
      next;
    }
    ToolCommon::Log($nodeLogFh, "  Done\n") if (! $params->{NOACTION});
    ToolCommon::Log($nodeLogFh, sprintf("%d CLI commands sent in %d seconds to accomplish this reconfiguration\n",
					$cliCommandsSent, time() - $reconfigStart));

    # We're done, see what's next
    if ($params->{ROLLBACK}) {
      RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig);
    } else {
      DumpRollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)  if $params->{DEBUG};
    }

    # Release transaction
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership... DONE");

    IsamCli::Disconnect($cli);
    $session->close();
    close $nodeLogFh if ($params->{SEPLOG});  # -- 1.2.5

    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Finished $isam->[1]... in %d seconds\n",
				 $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $endTime - $startTime), 1);
    $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
    if (defined($nodeStatusLog)) {
      print $nodeStatusLog
	sprintf("%s\n",
		FamilyCommon::GetErrorCount() ? ErrorCodes::ExitString(EXIT_WARN_CODE) : ErrorCodes::ExitString(SUCCESS_CODE));
      FamilyCommon::CloseNodeStatusLog();
    }
  }
  if (FamilyCommon::GetErrorCount()) {
    ToolCommon::Log($fh, sprintf("Child %d: %d error(s) occured for the following ISAMs, please verify %s\n",
				 $count, FamilyCommon::GetErrorCount(),
				 $params->{SEPLOG} ? $nodeLogFile : "$params->{LOGFILE}_$count."), 1);
    my $failedNodes = FamilyCommon::GetFailedNodeList();
    foreach my $node (sort keys %$failedNodes) {
      ToolCommon::Log($fh, "  $node\n", 1);
    }
  }
  close $nodeLogFh if (! $params->{SEPLOG});
  ToolCommon::Log($fh, "Child $count: finished...\n", 1);
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}

#------------------------------------------------------------------------------
#-- Adjust Tool defaults if needed (i.e. when defined in the parameter config
#-- file
#------------------------------------------------------------------------------
sub AdjustToolVars {
  my $crypt = Crypt::Lite->new('debug' => 0);

  $HOSTSFILE = $config{'HOSTSFILE'} if defined ($config{'HOSTSFILE'});
  $TFTPDIR = $config{'TFTPDIR'} if defined($config{'TFTPDIR'});
  $TFTPWORKDIR = $config{'TFTPWORKDIR'} if defined($config{'TFTPWORKDIR'});
  $MAXSESSIONS = $config{'MAXTFTPSESSION'} if defined($config{'MAXTFTPSESSION'});
  $USERNAME = $crypt->decrypt($config{'DEFAULTCLIUSER'}, CYPHER) if (defined($config{'DEFAULTCLIUSER'}) &&
								     ($config{'DEFAULTCLIUSER'} ne ""));
  $PASSWORD = $crypt->decrypt($config{'DEFAULTCLIPWD'}, CYPHER) if (defined($config{'DEFAULTCLIPWD'}) &&
								    ($config{'DEFAULTCLIPWD'} ne ""));
  $NT_COMMUNITY = $crypt->decrypt($config{NT_COMMUNITY}, CYPHER) if (defined($config{NT_COMMUNITY}) &&
								     ($config{NT_COMMUNITY} ne ""));
  $NODEPREFIX = $config{'NODEPREFIX'} if defined($config{'NODEPREFIX'});
  $TELNETTIMEOUT = $config{'TELNETTIMEOUT'} if defined($config{TELNETTIMEOUT});
  $STATUSFILE = $config{CCCE_STATUS_FILE} if defined($config{CCCE_STATUS_FILE});
  $RETRIES = $config{RETRIES} if defined($config{RETRIES});
  $NO_PING = uc($config{'NO_PING'}) eq "N" ? 0 : 1 if defined($config{'NO_PING'});
}

#==============================================================================
#== Main program
#==============================================================================
my $count = 0;
my @pids;
my $pid;
my $warnCount = 0;

sub CleanUp {
  if (scalar @pids) {
    print STDERR sprintf("%s::%s\n", PBMT_VERSION, $VERSION);
    print STDERR "\nCaught Interupt (^C), aborting children...";
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	kill($pid);
      }
    }
  }
  exit(EXIT_SIGNAL_CODE);
}

$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }

  if (!($msgStr =~ /Zlib/)) {
    print $warnLogFile "Warning: $msgStr" if (defined($warnLogFile));
    print STDERR "Warning: $msgStr";
    $warnCount++;
  }
};

#-- Read parameter config file and adjust tool defaults if needed 
%config = Utils::ReadConfig('param.cfg');
AdjustToolVars();

#-- Handle command line
$params = ParseCommandLine();

STDOUT->autoflush(1);
my $logFile;
if (!open($logFile, ">$params->{LOGFILE}")) {
  print STDERR "Cannot open $params->{LOGFILE} for writing: $!\n";
  exit CREATE_FILE_FAIL_CODE;
}
$logFile->autoflush(1);
FamilyCommon::SetWarnLogFile($logFile);

#-- Print summary of options
PrintOptionsSummary($logFile, 1);

Utils::SetDebug($logFile) if ($params->{DEBUG});  # -- 1.2.2

#-- Create work dir
my $tmpDir = "$TFTPDIR/$TFTPWORKDIR";
if (! -d $tmpDir) {
  if (!mkdir( $tmpDir, 0777)) {
    ToolCommon::Log($logFile, "Cannot create directory $tmpDir: $!\n", 1);
    exit CREATE_DIR_FAILED_CODE;
  }
}
if (! chmod(01777, "$tmpDir")) {
    ToolCommon::Log($logFile,"Failed to chmod 1777 for directory '$tmpDir' \n",1);
}

#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  my @addr = gethostbyname($params->{ISAM});
  my @ip = unpack( "C4", $addr[4]);
  my $ipAddr = join( '.', @ip );
  $isamsRef = [ [$ipAddr, $params->{ISAM}] ];
} else {
  ToolCommon::Log($logFile, "Reading Hostfile...\n", 1);
  $isamsRef = ToolCommon::ReadHostsFile($logFile, $params->{DEBUG}, $params->{HOSTS}, $NODEPREFIX);
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

while ($count < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $count) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      ToolCommon::Log($logFile, "Child $count: started as $pid\n", 1);
      push(@pids, $pid);
    } else {
      HandleIsams($logFile, \@isamPart, $count, \%config);
      exit SUCCESS_CODE;
    }
  }
  $count++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    $exitCode = $? >> 8;
    $signal = $? & 255;
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      ToolCommon::Log($logFile, sprintf("Child %d: exit code: '%d' (%s), signal: '%d'\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal), 1);
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  ToolCommon::Log($logFile, "All childs finished, exiting\n", 1);
  close $logFile;
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
