#!/usr/bin/perl
#==============================================================================
#-- File:         ipAware2Ibridge.pl
#-- Date:         06.06.2007
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to convert an IP-aware bridge to an I-Bridge
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent AND 2007
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2007/06/06 : First implementation (RCR ANTms41225)
#--   1.0.1 - 2008/02/22 : ANTms85481.  Typo in VLAN reconfiguration command for DHCP.
#--                        Seen when retesting with R3.5
#--   1.0.2 - 2008/02/25 : Error in precreation logic.  A generated command was stored
#--                        before it was actually generated (as such we stored the same
#--                        command twice (the last create bridge port)
#--   1.0.3 - 2008/04/15 : ANTms87260.  Adapt to allow VLAN ports and non-ATM bridge
#--                        ports (remove all ATM related code)
#--   1.0.4 - 2008/04/25 : ANTmt04759.  IGMP channel configuration gets lost when a brigeport
#--                        or VLAN port is removed from the VRF.  Therefor we need to retrieve
#--                        the IGMP channel information of the involved ports and reconfigure
#--                        the IGMP channel if there was one defined originally.
#--   1.0.5 - 2008/06/03 : ANTmt09938 include set and release transaction
#--   1.0.6 - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.0.7 - 2008/07/03 : In the end no SHUB recofig is needed.
#--   1.0.8 - 2008/09/05 : Ignore blank line in CLI response
#--                        Ignore first line in CLI response if not according to expected syntax:
#--                        could be a remainder of the echoed command.
#--   1.0.9 - 2008/10/27 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.0.10 - 2008/12/01: ANTmt33842.  Only check VLAN ports associated to this VRF: a bridge port may
#--                        have multiple vlanports associated to multiple VRFs.
#--                        Added physical-id mapping for the dchp remote-id option.
#--   1.0.11 - 2009/01/19: ANTmt37968.  No more support for show pppoe session from R3.7 onwards.
#--   1.0.12 - 2009/03/10: ANTmt18506
#--   1.0.13 - 2009/05/07: ANTmt56637. Reset bcast-vlan of DHCP-relay before deletion of VRF 
#--   1.0.14 - 2009/05/26: remove sleep between CLI commands.
#--   1.0.15 - 2009/06/11: Allow SSH for CLI
#--   1.0.16 - 2009/06/29: Some cleanup and best practices.
#--   1.0.17 - 2009/08/24: ANTmt72969. Workaround for problem caused by ANTmt38211) and delay submit of
#--                        VLAN delete command.
#--   1.0.18 - 2009/08/27: ANTmt73667.  Allow specification of a VLAN MAC ageing time.
#--   1.0.19 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.20 - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session
#--   1.0.21 - 2010/03/04 : ANTmg01050.	Total_MiB label change from Ex.x.y.z to Vx.x.x.y.z 
#--   1.0.22 - 2010/03/15 : PBMT x86 Changes - ANTmt88407
#--   1.0.23 - 2010/03/25 : Add option bridgeportmac <value>
#--   1.0.24 - 2010/06/07 : ALU00417675
#--   1.0.25 - 2011/01/11 : LRM updates for lib-perl path
#--   1.0.26 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   1.0.27 - 2011/05/02 : ALU01178887
#--   1.0.28 - 2012/05/11 : ALU01789702 - World writeable directories need sticky bit set
#--   1.0.29 - 2012/06/13 : Change permissions for backup db file
#--   1.0.30 - 2013/04/29 : ALU01880147
#--   1.0.31 - 2016/07/15 : ALU02237966
#--   1.0.32 - 2016/09/28 : ALU02278259


my $VERSION = "ipAware2Ibridge.pl - 1.0.32 - 2016/09/28";
#==============================================================================
use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 
#==============================================================================
use Crypt::Lite;
use Capability;
use IsamCli;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :IBRG);
use Utils;
use SNMPComm;
use version qw(:ALL);
#==============================================================================
#== Constants
#==============================================================================

my $HOSTSFILE =               "/etc/hosts";
my $TFTPDIR =                 "/var/opt/aws/common/tftpd/data";
my $TFTPWORKDIR =             "tmp";
my $MAXSESSIONS =             5;
my $USERNAME =                "";
my $PASSWORD =                "";
my $NODEPREFIX =              "AS-";
my $TELNETTIMEOUT =           10;
my $STATUSFILE =              "IBRGresult.txt";
my $MAX_QOS_SESS_PROFNAME_L = 64;
my $NT_COMMUNITY =            "public";
my $RETRIES =                 5;
my $NO_PING =                 0;

my %optionTranslation = ('DHCP' => {CIRCID => {'customer-id' => 'customer-id',
					       'physical-id' => 'physical-id',
					       'ccsa-format' => 'ccsa-format',
					       'notadd' => 'disable'},
				    REMID => {'customer-id' => 'customer-id',
					      'physical-id' => 'physical-id',
					      'notadd' => 'disable'}
				   });
#==== VRF parsing ===
my $VRF_DEFINTION  = 1;
my $VRF_NETW_ITF   = 2;
my $VRF_GTWY_ITF   = 3;
my $VRF_USER_ITF   = 4;
my $VRF_ROUTE_DEST = 5;

#==============================================================================
#== Vars
#==============================================================================
my %config;
#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub Usage {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "\nUsage: $0 --isam <ip> | --hosts <file> --logfile <file> --netwVRF <vrfid> [--ageing <seconds>] \\ \n";
  print STDERR "       --profsufx <text> [--bridgeportmac <n>] [--sessions <nr>] [--seplog] [--noaction] [--confinfo] \\ \n";
  print STDERR "       [--rollback]  [--telnetto <minutes>] [--passipoe] [--username <user>] \\ \n";
  print STDERR "       [--password <password>] [--v] [--debug] [--bufsize n] [--noping] [--sleep n]\n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         netwVRF  : id of the network Virtual Router Function (1..127)\n";
  print STDERR "         ageing   : the number of seconds to be used for VLAN MAC ageing when re-creating the VLAN\n";
  print STDERR "         passipoe : I-bridge will only pass IPoE traffic (PPPoE traffic will not be forwarded to the network\n";
  print STDERR "         profsufx : QoS session profile suffix.  In case a user interface associated to this VRF is configured\n";
  print STDERR "                    with a QoS session profile this profile is reconfigured on the VLAN port.  The logical\n";
  print STDERR "                    flowtype has to be generic.  In case a new QoS session profile has to be created with\n";
  print STDERR "                    identical contents but with logical flow type generic, the same profile name will be used\n";
  print STDERR "                    concatenated with the text provided with this option.\n";
  print STDERR "                    Note that truncation may occur in case the maximum profile name length (64 chars) is exceeded.\n";
  print STDERR "    bridgeportmac : the maximum number of uni-cast-mac value.\n";
  print STDERR "         sessions : number of parallel ISAMs to be treated (MAX=$MAXSESSIONS)\n";
  print STDERR "         username : ISAM username\n";
  print STDERR "         password : ISAM password\n";
  print STDERR "         seplog   : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noaction : just check, do not change the type\n";
  print STDERR "         debug    : extra logging for debugging purposes\n";
  print STDERR "         confinfo : print out all retrieved configration info\n";
  print STDERR "         rollback : roll back to original configuration\n";
  print STDERR "         bufsize  : size of CLI command return buffer (default is 15Mb)\n";
  print STDERR "         telnetto : timeout value for telnet session (in minutes)\n";
  print STDERR "         noping   : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "         sleep    : introduce a delay of n seconds before sending the VLAN deletion command.  Default is 5.\n";
  exit CMD_INCOMPL_CODE;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub ParseCommandLine {
  my ($isam, $nwVrfId, $username, $password, $sessions, $debug, $hosts, $logfile, $version, $sepLog, $noaction, $confInfo, $passIpoe,
      $rollBack, $bufSize, $telnetTO, $profSufx, $noPing, $sleep, $ageing, $maxUnicastMac);
  my %params;
  my $options = GetOptions('i|isam=s', \$isam,
			   'noaction', \$noaction,
			   'netwVRF=i', \$nwVrfId,
			   's|sessions=i', \$sessions,
			   'd|debug', \$debug,
			   'h|hosts=s', \$hosts,
			   'u|username=s', \$username,
			   'p|password=s', \$password,
			   'l|logfile=s', \$logfile,
			   'passipoe', \$passIpoe,
			   'v', \$version,
			   'seplog', \$sepLog,
			   'confinfo', \$confInfo,
			   'rollback', \$rollBack,
			   't|elnetto=i', \$telnetTO,
			   'bufsize=i', \$bufSize,
			   'profsufx=s', \$profSufx,
			   'b|bridgeportmac=i', \$maxUnicastMac,
			   'noping', \$noPing,
			   'sleep=i', \$sleep,
			   'ageing=i', \$ageing);

  #-- check options
  if (! $options) {
    Usage();
  }
  if (defined($version)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logfile)) {
    print STDERR "\nERROR: no logfile given\n";
    Usage();	
  }
  if (!defined($nwVrfId)) {
    print STDERR "\nERROR: No network VRF ID provided\n";
    Usage();
  } else {
    if (($nwVrfId < 1) || ($nwVrfId > 127)) {
      print STDERR "\nERROR: Incorrect value for a VRF ID.  Value must be between 1 and 127\n";
      Usage();
    }
  }
  if (!defined($profSufx)) {
    print STDERR "\nERROR: No QoS session profile name suffix provided\n";
    Usage();
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "\nERROR: Hostsfile $hosts does not exist\n";
    Usage();
  }
  if (defined($username)) {
    $params{USERNAME} = $username;
  } else {
    $params{USERNAME} = $USERNAME;
  }
  if (defined($password)) {
    $params{PASSWORD} = $password;
  } else {
    $params{PASSWORD} = $PASSWORD;
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $MAXSESSIONS) {
      print STDERR "\n ERROR: # sessions is larger than the maximum ($sessions > $MAXSESSIONS)\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $HOSTSFILE;
  }
  $params{UMASK_CHMOD} = ( (! defined $config{UMASK_OVERRIDE}) or $config{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = defined($debug) ? 1 : 0;
  $params{NWVRFID} = $nwVrfId;
  $params{LOGFILE} = $logfile;
  $params{PASSIPOE} = defined($passIpoe) ? 1 : 0;
  $params{SEPLOG} = defined($sepLog) ? 1 : 0;
  $params{NOACTION} = defined($noaction) ? 1 : 0;
  $params{CONFINFO} = defined($confInfo) ? 1 : 0;
  $params{ROLLBACK} = defined($rollBack) ? 1 : 0;
  $params{BUFSIZE} = defined($bufSize) ? $bufSize : 15;
  $params{BUFSIZE} = 100 if ($params{BUFSIZE} > 100);
  $params{BUFSIZE} *= 1048576; # Convert to MBytes
  $params{TELNETTIMEOUT} = defined($telnetTO) ? $telnetTO * 60 : 600; # default 10 minutes
  $params{PROFSUFX} = $profSufx;
  $params{NOPING} = $NO_PING;
  $params{NOPING} = 1 if defined($noPing);
  $params{SLEEP} = defined($sleep) ? $sleep : 5;
  $params{AGEING} = $ageing if defined($ageing);
  $params{MAXUNICASTMAC} = $maxUnicastMac if defined($maxUnicastMac);
  return \%params;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub PrintLibVersions {
  my ($fh, $out) = @_;

  if (defined($fh)) {
    ToolCommon::Log($fh, "Library versions:\n", $out);
    ToolCommon::Log($fh, sprintf("  %s\n", IsamCli::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ToolCommon::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", FamilyCommon::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", Capability::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ErrorCodes::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", Utils::GetVersion()), $out);
  } else {
    print STDERR "Library versions:\n";
    print STDERR sprintf("  %s\n", IsamCli::GetVersion());
    print STDERR sprintf("  %s\n", ToolCommon::GetVersion());
    print STDERR sprintf("  %s\n", FamilyCommon::GetVersion());
    print STDERR sprintf("  %s\n", Capability::GetVersion());
    print STDERR sprintf("  %s\n", ErrorCodes::GetVersion());
    print STDERR sprintf("  %s\n", Utils::GetVersion());
  }
}

sub PrintOptionsSummary {
  my($fh, $out) = @_;

  ToolCommon::Log($fh, sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION), $out);
  ToolCommon::Log($fh, "Options:\n", $out);
  ToolCommon::Log($fh, "   Nr Sessions                     : $params->{SESSIONS}\n", $out);
  ToolCommon::Log($fh, "   Hosts file                      : $params->{HOSTS}\n", $out) if (!defined($params->{ISAM}));
  ToolCommon::Log($fh, "   Only One Isam                   : $params->{ISAM}\n", $out) if (defined($params->{ISAM}));
  ToolCommon::Log($fh, sprintf("   Ping                            : %s\n", $params->{NOPING} ? "not allowed" : "allowed"), $out);
  ToolCommon::Log($fh, "   Network VRF ID                  : $params->{NWVRFID}\n", $out);
  ToolCommon::Log($fh, "   Delay before deleting VLAN      : $params->{SLEEP} seconds\n", $out);
  ToolCommon::Log($fh, "   QoS session profile name suffix : $params->{PROFSUFX}\n", $out);
  ToolCommon::Log($fh, "   Max-unicast-mac value           : $params->{MAXUNICASTMAC}\n", $out) if defined($params->{MAXUNICASTMAC});
  ToolCommon::Log($fh, "   Only pass IPoE traffic\n", $out) if $params->{PASSIPOE};
  ToolCommon::Log($fh, sprintf("   Telnet CLI command buffer size  : %d Mbytes\n", $params->{BUFSIZE} / 1048576), $out);
  ToolCommon::Log($fh, sprintf("   Log file per node               : %s\n", $params->{SEPLOG} ? "Yes" : "No"), $out);
  ToolCommon::Log($fh, sprintf("   Telnet timeout                  : %d minutes\n", $params->{TELNETTIMEOUT} / 60), $out);
  ToolCommon::Log($fh, "   Debugging Enabled\n", $out) if $params->{DEBUG};
  ToolCommon::Log($fh, "   No reconfiguration, only testing\n", $out) if $params->{NOACTION};
  ToolCommon::Log($fh, "   Print retrieved CLI configuration information\n", $out) if $params->{CONFINFO};
  ToolCommon::Log($fh, "   Rollback original configuration\n", $out) if $params->{ROLLBACK};
  PrintLibVersions($fh, $out);
}

#------------------------------------------------------------------------------
#-- Parse VRF configuration.
#-- Hash structure as follows:
#-- %vrfConfig{VRFDEF}{ID}
#--                   {NAME}
#--                   {DESC}
#--                   {CONTACT}
#--                   {IP}
#--                   {DEFTTL}
#--                   {REASSEM}
#--                   {MODE}
#-- %vrfConfig{NWITF}{$vlanId}{VLANID}
#--                           {NUMBERED}
#--                           {ARPPOLICY}
#--                           {IP}
#-- %vrfConfig{USRITF}{$port}{ATM} if ATM port
#--                          {BRIDGE} if bridge port
#--                          {VLAN} if VLAN port
#--                          {QOS}
#-- %vrfConfig{GWYITF}{$ipAddress}{IP}
#-- %vrfConfig{ROUTEDEST}{$destination}{IP}
#--                                    {NEXTHOP}
#--
#-- $fh: pointer to a file handle for logging purposes
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete VRF structure.
#-- $qosProfileStorage: pointer to a hash for storing the Qos-profiles used on user interfaces
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseVrfConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $qosSessionProfileStorage, $releaseId) = @_;
  my $vrfId = 0;
  my $ok = 1;
  my $netwItfCount = 0;
  my $prevNwVlan = 0;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure\s+ip\s+vrf\s+(\d+)/) {
      ToolCommon::Log($fh, "[ParseVrfConfig] line read: $line\n") if ($params->{DEBUG});
      push @{$cliResponseLines}, $line;
      $vrfId = $1;
      $line =~ s/configure\s+ip\s+vrf\s+\d+//;
      if (!defined($configStorage->{VRFDEF}{ID})) {
	$configStorage->{VRFDEF}{ID} = $vrfId;
	ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{ID} as '%d\n",
				     $configStorage->{VRFDEF}{ID})) if ($params->{DEBUG});
      }
      if ($line =~ s/name\s+([^\s]+)//) {
	my $vrfName = $1;
	$configStorage->{VRFDEF}{NAME} = $vrfName;
	ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{NAME} as '%s'\n",
				     $configStorage->{VRFDEF}{NAME})) if ($params->{DEBUG});
	if ($line =~ s/desc\s+([^\s]+)//) {
	    $configStorage->{VRFDEF}{DESC} = $2;
	    ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{DESC} as '%s'\n",
				       $configStorage->{VRFDEF}{DESC})) if ($params->{DEBUG});
		      }
	if ($line =~ s/contact\s+([^\s]+)//) {
	  $configStorage->{VRFDEF}{CONTACT} = $1;
	  ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{CONTACT} as '%s'\n",
				       $configStorage->{VRFDEF}{CONTACT})) if ($params->{DEBUG});
	}
	if ($line =~ s/my-ipaddress\s+(\d+\.\d+\.\d+\.\d+)//) {
	  $configStorage->{VRFDEF}{IP} = $1;
	  ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{IP} as '%s'\n",
				       $configStorage->{VRFDEF}{IP})) if ($params->{DEBUG});
	}
	if ($line =~ s/default\-ttl\s+(\d+)//) {
	  $configStorage->{VRFDEF}{DEFTTL} = $1;
	  ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{DEFTTL} as '%d'\n",
				       $configStorage->{VRFDEF}{DEFTTL})) if ($params->{DEBUG});
	}
	if ($line =~ s/reassem\-timeout\s+(\d+)//) {
	  $configStorage->{VRFDEF}{REASSEM} = $1;
	  ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{REASSEM} as '%d'\n",
				       $configStorage->{VRFDEF}{REASSEM})) if ($params->{DEBUG});
	}
	if ($line =~ s/mode\s+(\w+)//) {
	  $configStorage->{VRFDEF}{MODE} = $1;
	  ToolCommon::Log($fh, sprintf("  [VRFDEF] Stored vrfConfig->{VRFDEF}{MODE} as '%s'\n",
				       $configStorage->{VRFDEF}{MODE})) if ($params->{DEBUG});
	}
      } elsif ($line =~ /network\-itf/) {
	if (($line =~ s/network\-itf\s+(\d+)//) ||
	    ($line =~ s/network\-itf\s+(stacked:\d+:\d+)//)) {
	  my $vlanId = $1;
	  $netwItfCount++ if ($vlanId != $prevNwVlan);
	  if (!defined($configStorage->{NWITF}{$vlanId}{VLANID})) {
	    $configStorage->{NWITF}{$vlanId}{VLANID} = $vlanId;
	    $configStorage->{NWITF}{$vlanId}{NUMBERED} = "no unnumbered";
	    ToolCommon::Log($fh, sprintf("  [NWITF] Stored vrfConfig->{NWITF}{$vlanId}{VLANID} as '%d'\n",
					 $configStorage->{NWITF}{$vlanId}{VLANID})) if ($params->{DEBUG});
	  }
	  if ($line =~ s/arp\-policy\s+([not\-trusted]+)//) {
	    $configStorage->{NWITF}{$vlanId}{ARPPOLICY} = $1;
	    ToolCommon::Log($fh, sprintf("  [NWITF] Stored vrfConfig->{NWITF}{$vlanId}{ARPPOLICY} as '%s'\n",
					 $configStorage->{NWITF}{$vlanId}{ARPPOLICY})) if ($params->{DEBUG});	    
	  }
	  if (($line =~ s/(no\s+unnumbered)//) ||
	      ($line =~ s/(unnumbered)//)) {
	    $configStorage->{NWITF}{$vlanId}{NUMBERED} = $1;
	    ToolCommon::Log($fh, sprintf("  [NWITF] Stored vrfConfig->{NWITF}{$vlanId}{NUMBERED} as '%s'\n",
					 $configStorage->{NWITF}{$vlanId}{NUMBERED})) if ($params->{DEBUG});
	  }
	  if ($line =~ s/ip\-address\s+(\d+\.\d+\.\d+\.\d+\/\d+)//) {
	    $configStorage->{NWITF}{$vlanId}{IP} = $1;
	    ToolCommon::Log($fh, sprintf("  [NWITF] Stored vrfConfig->{NWITF}{$vlanId}{IP} as '%s'\n",
					 $configStorage->{NWITF}{$vlanId}{IP})) if ($params->{DEBUG});
	  }
	}
      } elsif ($line =~ s/user\-itf//) {
	my $port = "";
	if (($line =~ s/port\s+(bridge\-port:\d+\/\d+\/\d+\/\d+:\d+:\d+)//) ||  # ATM format
	    ($line =~ s/port\s+(bridge\-port:\d+\/\d+\/\d+\/\d+)//)) {          # ETH format
	  $port = $1;
	  $configStorage->{USRITF}{$port}{BRIDGE} = 1;
	  ToolCommon::Log($fh, sprintf("  [USRITF BRIDGE] Stored vrfConfig->{USRITF}{$port}{BRIDGE} as '%d'\n",
				       $configStorage->{USRITF}{$port}{BRIDGE})) if ($params->{DEBUG});
	}
	if (($line =~ s/port\s+(atm\-port:\d+\/\d+\/\d+\/\d+:\d+:\d+)//) ||
	    ($line =~ s/port\s+(\d+\/\d+\/\d+\/\d+:\d+:\d+)//)) {
	  $port = $1;
	  $configStorage->{USRITF}{$port}{ATM} = 1;
	  ToolCommon::Log($fh, sprintf("  [USRITF ATM] Stored vrfConfig->{USRITF}{$port}{ATM} as '%d'\n",
				       $configStorage->{USRITF}{$port}{ATM})) if ($params->{DEBUG});
	}
	if ($line =~ s/port\s+(vlan\-port:[^\s]+)//) {
	  $port = $1;   # Both ATM and ETH formats captured here.
	  $configStorage->{USRITF}{$port}{VLAN} = 1;
	  ToolCommon::Log($fh, sprintf("  [USRITF VLAN] Stored vrfConfig->{USRITF}{$port}{VLAN} as '%d'\n",
				       $configStorage->{USRITF}{$port}{VLAN})) if ($params->{DEBUG});
	}
	if ($line =~ s/qos\-profile\s+name:([^\s]+)//) {
	  $configStorage->{USRITF}{$port}{QOS} = $1;
	  ToolCommon::Log($fh, sprintf("  [USRITF] Stored vrfConfig->{USRITF}{$port}{QOS} as '%s'\n",
				       $configStorage->{USRITF}{$port}{QOS})) if ($params->{DEBUG});
	  $qosSessionProfileStorage->{$1}{FLOWTYPE} = "unknown" if (!defined($qosSessionProfileStorage->{$1}{TYPE}));
	}
	if ($line =~ s/arp\-proxy\-stats//) {
	  $configStorage->{USRITF}{$port}{ARPPROXYSTATS} = 1;
	  ToolCommon::Log($fh, "  [USRITF VLAN] Stored arp-proxy-stats\n") if ($params->{DEBUG});
	}
      } elsif ($line =~ s/gateway\-itf//) {
	if ($line =~ s/ip\-address\s+(\d+\.\d+\.\d+\.\d+\/\d+)//) {
	  $configStorage->{GWYITF}{$1}{IP} = $1;
	  ToolCommon::Log($fh, sprintf("  [GWYITF] Stored vrfConfig->{GWYITF}{IP} as '%s'\n",
				       $configStorage->{GWYITF}{$1}{IP})) if ($params->{DEBUG});
	}
      } elsif ($line =~ /route\-dest/) {
	my $routeDest = "";
	if ($line =~ s/route-dest\s+(\d+\.\d+\.\d+\.\d+\/\d+)//) {
	  $routeDest = $1;
	  $configStorage->{ROUTEDEST}{$routeDest}{IP} = $1;
	  ToolCommon::Log($fh, sprintf("  [ROUTEDEST] Stored vrfConfig->{ROUTEDEST}{$routeDest}{IP} as '%s'\n",
				       $configStorage->{ROUTEDEST}{$routeDest}{IP})) if ($params->{DEBUG});
	}
	if ($line =~ s/next-hop\s+([^\s]+)//) {
	  $configStorage->{ROUTEDEST}{$routeDest}{NEXTHOP} = $1;
	  ToolCommon::Log($fh, sprintf("  [ROUTEDEST] Stored as vrfConfig->{ROUTEDEST}{$routeDest}{NEXTHOP} '%s'\n",
				       $configStorage->{ROUTEDEST}{$routeDest}{NEXTHOP})) if ($params->{DEBUG});
	}
      }
      $line =~ s/\s//g;
      if ($line ne "") {
	FamilyCommon::SetAndLogError($fh, "[ParseVrfConfig] Command not completely parsed, left: $line");
	return 0;
      }
    } else {
      if (! ($line =~ /\#[-]+$/) &&
	  ! ($line =~ /[\$%\#>]\s*$/)) {
	$ok = 0;
	FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
      }
    }
  }
  if ($netwItfCount > 1) {
    $ok = 0;
    FamilyCommon::SetAndLogError($fh, "  FAILED: multiple Network-itf defined on IP-aware bridge");
  }
  return $ok;
}

#------------------------------------------------------------------------------
#-- Parse VLAN configuration.
#-- Hash structure as follows:
#-- %vlanConfig{VLANDEF}{ID}
#--                     {NAME}
#--                     {MODE}
#--                     {PRIO}
#--                     {BCFRMS}
#--                     {PROTFLT}
#--                     {PPPOERLYTAG}
#--                     {OPT82}
#--                     {CIRCIDDHCP}
#--                     {REMIDDHCP}
#--                     {DHCPLINERATE}
#--                     {PPPOELINERATE}
#--                     {CIRCIDPPPOE}
#--                     {REMIDPPPOE}
#--                     {SECFWD}
#--                     {AGING}
#--                     {VMACTRANS}
#--                     {VMACDNFLT}
#-- $fh: pointer to a file handle for logging purposes
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete VLAN structure.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseVlanConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseId) = @_;
  my $vlanId = 0;
  my $ok = 1;
  my $firstLine = 1;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure\s+vlan\s+id\s+([^\s]+)/) {
      ToolCommon::Log($fh, "[ParseVlanConfig] line read: $line\n") if ($params->{DEBUG});
      $firstLine = 0;
      push @{$cliResponseLines}, $line;
      $vlanId = $1;
      $configStorage->{VLANDEF}{ID} = $vlanId if (!defined($configStorage->{VLANDEF}{ID}));
      $line =~ s/configure\s+vlan\s+id\s+[^\s]+//;
      ToolCommon::Log($fh, sprintf("Stored VLAN ID as '%s'\n", $configStorage->{VLANDEF}{ID})) if ($params->{DEBUG});
      if ($line =~ s/name\s+([^\s]+)//) {
	my $vlanName = $1;
	$configStorage->{VLANDEF}{NAME} = $vlanName;
	ToolCommon::Log($fh, sprintf("Stored VLAN name as '%s'\n", $configStorage->{VLANDEF}{NAME})) if ($params->{DEBUG});
      }
      if ($line =~ s/mode\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{MODE} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN mode as '%s'\n", $configStorage->{VLANDEF}{MODE})) if ($params->{DEBUG});
      }
      if ($line =~ s/priority\s+(\d+)//) {
	$configStorage->{VLANDEF}{PRIO} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN priority as '%s'\n", $configStorage->{VLANDEF}{PRIO})) if ($params->{DEBUG});
      }
      if ($line =~ s/broadcast\-frames//) {
	$configStorage->{VLANDEF}{BCFRMS} = 1;
	ToolCommon::Log($fh, "Stored VLAN broadcast frames\n") if ($params->{DEBUG});
      }
      if ($line =~ s/protocol\-filter\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{PROTFLT} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN protocol filter as '%s'\n", $configStorage->{VLANDEF}{PROTFLT})) if ($params->{DEBUG});
      }
      if ($line =~ s/pppoe\-relay\-tag\s+(\w+)//) {
	$configStorage->{VLANDEF}{PPPOERLYTAG} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN PPPoE relay tag as '%s'\n", $configStorage->{VLANDEF}{PPPOERLYTAG})) if ($params->{DEBUG});
      }
      if ($line =~ s/dhcp\-opt\-82//) {
	$configStorage->{VLANDEF}{OPT82} = 1;
	ToolCommon::Log($fh, "Stored VLAN DHCP option 82\n") if ($params->{DEBUG});
      }
      if ($line =~ s/circuit\-id\-dhcp\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{CIRCIDDHCP} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN circuit-id DHCP as '%s'\n", $configStorage->{VLANDEF}{CIRCIDDHCP})) if ($params->{DEBUG});
      }
      if ($line =~ s/remote\-id\-dhcp\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{REMIDDHCP} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN remote-id DHCP as '%s'\n", $configStorage->{VLANDEF}{REMIDDHCP})) if ($params->{DEBUG});
      }
      if ($line =~ s/dhcp-linerate\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{DHCPLINERATE} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN DHCP linerate as '%s'\n", $configStorage->{VLANDEF}{DHCPLINERATE})) if ($params->{DEBUG});
      }
      if ($line =~ s/pppoe\-linerate\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{PPPOELINERATE} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN PPPoE linerate as '%s'\n", $configStorage->{VLANDEF}{PPPOELINERATE})) if ($params->{DEBUG});
      }
      if ($line =~ s/circuit\-id\-pppoe\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{CIRCIDPPPOE} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN circuit-id PPPoE as '%s'\n", $configStorage->{VLANDEF}{CIRCIDPPPOE})) if ($params->{DEBUG});
      }
      if ($line =~ s/remote\-id\-pppoe\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{REMIDPPPOE} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN remote-id PPPoE as '%s'\n", $configStorage->{VLANDEF}{REMIDPPPOE})) if ($params->{DEBUG});
      }
      if ($line =~ s/secure\-forwarding//) {
	$configStorage->{VLANDEF}{SECFWD} = 1;
	ToolCommon::Log($fh, "Stored VLAN secure forward\n") if ($params->{DEBUG});
      }
      if ($line =~ s/aging\-time\s+([^\s]+)//) {
	$configStorage->{VLANDEF}{AGING} = $1;
	ToolCommon::Log($fh, sprintf("Stored VLAN aging time as \n", $configStorage->{VLANDEF}{AGING})) if ($params->{DEBUG});
      }
      if ($line =~ s/vmac\-translation//) {
	$configStorage->{VLANDEF}{VMACTRANS} = 1;
	ToolCommon::Log($fh, "Stored VLAN vmac translation\n") if ($params->{DEBUG});
      }
      if ($line =~ s/vmac\-dnstr\-filter//) {
	$configStorage->{VLANDEF}{VMACDNFLT} = 1;
	ToolCommon::Log($fh, "Stored VLAN vmac translation\n") if ($params->{DEBUG});
      }
      $line =~ s/\s//g;
      if ($line ne "") {
	FamilyCommon::SetAndLogError($fh, "[ParseVlanConfig] Command not completely parsed, left: $line");
	return 0;
      }
    } else {
      if (($line ne "") &&
	  ! $firstLine &&
	  ! ($line =~ /\#[-]+$/) &&
	  ! ($line =~ /[\$%\#>]\s*$/)) {
	$ok = 0;
	SetAndLogError($fh, "Line not according to expected syntax: $line");
      }
      $firstLine = 0 if $firstLine;
    }
  }
  return $ok;
}

#------------------------------------------------------------------------------
#-- Parse DHCP relay-agent configuration.
#-- Hash structure as follows:
#-- %dhcpRelayAgentConfig{VRFID}
#--                      {BCASTVLAN}
#--                      {OPT82}
#--                      {CIRCID}
#--                      {REMID}
#--                      {CLIENTHWADDR}
#--                      {LINERATE}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete DHCP relay-agent structure.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseDhcpRelayAgentConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseId) = @_;
  my $vrfId = 0;
  my $ok = 1;
  my $firstLine = 1;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure\s+dhcp-relay\s+agent\s+(\d+)/) {
      ToolCommon::Log($fh, "[ParseDhcpRelayAgentConfig] line read: $line\n") if ($params->{DEBUG});
      $firstLine = 0;
      push @{$cliResponseLines}, $line;
      $vrfId = $1;
      $line =~ s/configure\s+dhcp-relay\s+agent\s+\d+//;
      $configStorage->{VRFID} = $vrfId if (!defined($configStorage->{VRFID}));
      ToolCommon::Log($fh, sprintf("Stored DHCP relay-agent VFR ID as '%d'\n", $configStorage->{VRFID})) if ($params->{DEBUG});
      if ($line =~ s/bcast\-vlan\s+([^\s]+)//) {
	$configStorage->{BCASTVLAN} = $1;
	ToolCommon::Log($fh, sprintf("Stored DHCP relay-agent BCAST VLAN as '%s'\n",
				     $configStorage->{BCASTVLAN})) if ($params->{DEBUG});
      }
      if ($line =~ s/option\-82//) {
	$configStorage->{OPT82} = 1;
	ToolCommon::Log($fh, "Stored DHCP relay-agent option-82\n") if ($params->{DEBUG});
      }
      if ($line =~ s/circuit\-id\s+([^\s]+)//) {
	$configStorage->{CIRCID} = $1;
	ToolCommon::Log($fh, sprintf("Stored DHCP relay-agent circuit-id as '%s'\n", $configStorage->{CIRCID})) if ($params->{DEBUG});
      }
      if ($line =~ s/remote\-id\s+([^\s]+)//) {
	$configStorage->{REMID} = $1;
	ToolCommon::Log($fh, sprintf("Stored DHCP relay-agent remote-id as '%s'\n", $configStorage->{REMID})) if ($params->{DEBUG});
      }
      if ($line =~ s/client\-hw-\address\s+([^\s]+)//) {
	$configStorage->{CLIENTHWADDR} = $1;
	ToolCommon::Log($fh, sprintf("Stored DHCP relay-agent client-hw-address as '%s'\n",
				     $configStorage->{CLIENTHWADDR})) if ($params->{DEBUG});
      }
      if ($line =~ s/dhcp\-linerate\s+([^\s]+)//) {
	$configStorage->{LINERATE} = $1;
	ToolCommon::Log($fh, sprintf("Stored DHCP relay-agent dhcp-linerate as '%s'\n",
				     $configStorage->{LINERATE})) if ($params->{DEBUG});
      }
      $line =~ s/\s//g;
      if ($line ne "") {
	FamilyCommon::SetAndLogError($fh, "[ParseDhcpRelayAgentConfig] Command not completely parsed, left: $line");
	return 0;
      }
    } else {
      if (($line ne "") &&
	  ! $firstLine &&
	  ! ($line =~ /\#[-]+$/) &&
	  ! ($line =~ /[\$%\#>]\s*$/)) {
	$ok = 0;
	SetAndLogError($fh, "Line not according to expected syntax: $line");
      }
      $firstLine = 0 if $firstLine;
    }
  }
  return $ok;
}

#------------------------------------------------------------------------------
#-- Parse vlan-bind configuration.
#-- Hash structure as follows:
#-- %vlanBindConfig{$vlanId}{LAYER3ITFID}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete DHCP relay-agent structure.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseVlanBindConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseId) = @_;
  my ($vlanId, $layer3ItfId);
  my $ok = 1;
  my $firstLine = 1;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure\s+ip\s+vlan\-bind\s+(\d+)\s+layer3\-itf\s+([^\s]+)/) {
      ToolCommon::Log($fh, "[ParseVlanBindConfig] line read: $line\n") if ($params->{DEBUG});
      $firstLine = 0;
      push @{$cliResponseLines}, $line;
      $vlanId = $1, $layer3ItfId = $2;
      $line =~ s/configure\s+ip\s+vlan\-bind\s+\d+\s+layer3\-itf\s+[^\s]+//;
      if (defined($configStorage->{$vlanId}{$layer3ItfId})) {
	FamilyCommon::SetAndLogError($fh, sprintf("  FAILED: already binding defined between VLAN '%d' and L3ITF '%s'", $vlanId, $layer3ItfId));
	$ok = 0;
      } else {
	$configStorage->{$vlanId}{$layer3ItfId} = 1;
	ToolCommon::Log($fh, sprintf("  [VLANBIND] Stored VLAN binding between VLAN '%d' and L3ITF '%s'\n", $vlanId, $layer3ItfId))
	  if ($params->{DEBUG});
      }
      $line =~ s/\s//g;
      if ($line ne "") {
	FamilyCommon::SetAndLogError($fh, "[ParseVlanBindConfig] Command not completely parsed, left: $line");
	return 0;
      }      
    } else {
      if (($line ne "") &&
	  ! $firstLine &&
	  ! ($line =~ /\#[-]+$/) &&
	  ! ($line =~ /[\$%\#>]\s*$/)) {
	$ok = 0;
	SetAndLogError($fh, "Line not according to expected syntax: $line");
      }
      $firstLine = 0 if $firstLine;
    }
  }
  return $ok;
}

#------------------------------------------------------------------------------
#-- Parse pppoe session info.
#-- Hash structure as follows:
#-- %pppoeSession{$port}{LOGICSESS}
#--                     {SESSID}
#--                     {LOCMAC}
#--                     {REMMAC}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the pppoe session ports.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParsePppoeSessions {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseId) = @_;
  my ($port, $logicSession, $sessionId, $locMac, $remMac);

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /^([^\s]+)\s+(\d+)\s+([^\s]+)\s+([^\s]+)/) {
      $port = $1, $sessionId = $2, $locMac = $3, $remMac = $4;
      $port =~ s/:(\d+)$//;
      $logicSession = $1;
      ToolCommon::Log($fh, "[ParsePppoeSessions] line read: $line\n") if ($params->{DEBUG});
      $configStorage->{$port}{LOGICSESS} = $logicSession;
      $configStorage->{$port}{SESSID} = $2;
      $configStorage->{$port}{LOCMAC} = $3;
      $configStorage->{$port}{REMMAC} = $3;
      ToolCommon::Log($fh, sprintf("  [PPPOE SESS] Stored for port %s: session id: '%s' (logic %d), local and remote mac: '%s' and '%s'\n",
				   $port, $configStorage->{$port}{SESSID}, $configStorage->{$port}{LOGICSESS}, $configStorage->{$port}{LOCMAC},
				   $configStorage->{$port}{REMMAC})) if ($params->{DEBUG});
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Get configuration of involved user-itf bridges.
#--
#-- Hash structure as follows:
#-- %bridgeConfig{$pvc}{PVID}
#--                    {PRIO}
#--                    {MACLEARNOFF}
#--                    {MAXUNICASTMAC}
#--                    {ACCEPTFRMTYPE}
#--                    {QOSPROF}
#--                    {PRIOREGENPROF}
#-- %vlanPortConfig{$pvc}{$vlan}{VLANID}
#--                             {TAG}
#--                             {NETWORKVLAN}
#--                             {VLANSCOPE}
#--                             {QOS}
#--                             {QOSPROF}
#--                             {PRIOBESTEFF}
#--                             {PRIOBACKGRND}
#--                             {PRIOSPARE}
#--                             {PRIOEXCEFF}
#--                             {PRIOCTRLLOAD}
#--                             {PRIOLESS100MS}
#--                             {PRIOLESS10MS}
#--                             {PRIONWCTRL}
#--                             {IPADDR}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $vrfConfig: pointer to VRF configuration
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $bridgePortStorage: pointer to a hash for storing the bridge port configuration.
#-- $vlanPortStorage: pointer to a hash for storing the VLAN port configuration.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub GetBridgeConfig {
  my ($fh, $cli, $vrfConfig, $cliResponseLines, $bridgePortStorage, $vlanPortStorage, $releaseNum) = @_;
  my @response = ();
  my $rspStr = "";
  my ($port, $portId, $vlanId);

  foreach $portId (keys %{$vrfConfig->{USRITF}}) {
    $vlanId = undef;
    if (defined($vrfConfig->{USRITF}{$portId}{BRIDGE}) || defined($vrfConfig->{USRITF}{$portId}{VLAN})) {
      if (($portId =~ /vlan\-port:([^\s]+)/) || ($portId =~ /bridge\-port:([^\s]+)/)) {
	$port = $1;
	$port =~ s/:\d+$// if ($portId =~ /vlan\-port:/);
	@response = IsamCli::SendCommand($cli, "info configure bridge port $port flat");
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    FamilyCommon::SetAndLogError($fh, sprintf("FAILED to retrieve bridge info: %s", IsamCli::GetError()));
	    return 0;
	  } else {
	    foreach my $line (@response) {
	      chomp $line;
	      if ($line =~ /configure\s+bridge/) {
		push @{$cliResponseLines}, $line if ($line =~ /^configure/);
		ToolCommon::Log($fh, "[GetBridgeConfig] line read: $line\n") if ($params->{DEBUG});
		$line =~ s/configure\s+bridge\s+port\s+$port//;
		if ($line =~ s/pvid\s+([^\s]+)//) {
		  $bridgePortStorage->{$port}{PVID} = $1;
		  ToolCommon::Log($fh, sprintf("Stored bridge PVID for %s as '%d'\n", $port, $bridgePortStorage->{$port}{PVID}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/default\-prioriy\s+([^\s]+)//) {
		  $bridgePortStorage->{$port}{PRIO} = $1;
		  ToolCommon::Log($fh, sprintf("Stored bridge priority for %s as '%d'\n", $port, $bridgePortStorage->{$port}{PRIO}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/mac\-learn\-off//) {
		  $bridgePortStorage->{$port}{MACLEARNOFF} = 1;
		  ToolCommon::Log($fh, sprintf("Stored bridge MAC learning off for %s\n", $port)) if ($params->{DEBUG});
		}
		if ($line =~ s/max\-unicast\-mac\s+(\d+)//) {
		  $bridgePortStorage->{$port}{MAXUNICASTMAC} = $1;
		  ToolCommon::Log($fh, sprintf("Stored bridge max-unicast-mac for %s as '%d'\n", $port, $bridgePortStorage->{$port}{MAXUNICASTMAC}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/accept\-frame\-type\s+([^\s]+)//) {
		  $bridgePortStorage->{$port}{ACCEPTFRMTYPE} = $1;
		  ToolCommon::Log($fh, sprintf("Stored bridge accept-frame-type for %s as '%s'\n", $port, $bridgePortStorage->{$port}{ACCEPTFRMTYPE}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/prio\-regen\-prof//) {
		  $bridgePortStorage->{$port}{PRIOREGENPROF} = $1;
		  ToolCommon::Log($fh, sprintf("Stored bridge priority regeneration profile for %s as '%s'\n", $port,
					       $bridgePortStorage->{$port}{PRIOREGENPROF})) if ($params->{DEBUG});
		}
		# Parsing of VLAN port specifics start here
		if ($line =~ s/vlan\-id\s+([^\s]+)//) {
		  $vlanId = $1;
		  $vlanPortStorage->{$port}{$vlanId}{VLANID} = $vlanId;
		  $vlanPortStorage->{$port}{$vlanId}{VLANSCOPE} = "network";
		  ToolCommon::Log($fh, sprintf("Stored bridge VLAN for %s:%s as '%s'\n", $port, $vlanId,
					       $vlanPortStorage->{$port}{$vlanId}{VLANID}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/qos\-profile\s+([^\s]+)//) {
		  if (defined($vlanId)) {
		    $vlanPortStorage->{$port}{$vlanId}{QOSPROF} = $1;
		    ToolCommon::Log($fh, sprintf("Stored vlan-port qos-profile for %s:%s as '%s'\n", $port, $vlanId,
						 $vlanPortStorage->{$port}{$vlanId}{QOSPROF}))
		      if ($params->{DEBUG});
		  } else {
		    $bridgePortStorage->{$port}{QOSPROF} = $1;
		    ToolCommon::Log($fh, sprintf("Stored bridge qos-profile for %s as '%s'\n", $port, $bridgePortStorage->{$port}{QOSPROF}))
		      if ($params->{DEBUG});
		  }
		}
		if ($line =~ s/tag\s+([^\s]+)//) {
		  $vlanPortStorage->{$port}{$vlanId}{TAG} = $1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port tag for %s:%s as '%s'\n", $port, $vlanId, $vlanPortStorage->{$port}{$vlanId}{TAG}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/network\-vlan\s+([^\s]+)//) {
		  $vlanPortStorage->{$port}{$vlanId}{NETWORKVLAN} = $1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port network VLAN for %s:%s as '%d'\n", $port, $vlanId,
					       $vlanPortStorage->{$port}{$vlanId}{NETWORKVLAN}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/vlan-scope\s+([^\s]+)//) {
		  $vlanPortStorage->{$port}{$vlanId}{VLANSCOPE} = $1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port VLAN scope for %s:%s as '%s'\n", $port, $vlanId,
					       $vlanPortStorage->{$port}{$vlanId}{VLANSCOPE}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/qos\s+([^\s]+)//) {
		  $vlanPortStorage->{$port}{$vlanId}{QOS} = $1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port QOS for %s:%s as '%s'\n", $port, $vlanId, $vlanPortStorage->{$port}{$vlanId}{QOS}))
		    if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-best\-effort//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIOBESTEFF} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-best-effort for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-background//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIOBACKGRND} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-background for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-spare//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIOSPARE} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-spare for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-exc\-effort//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIOEXCEFF} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-exc-effort for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-ctrl\-load//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIOCTRLLOAD} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-ctrl-load for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-less\-100ms//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIOLESS100MS} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-less-100ms for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-less\-10ms//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIOLESS10MS} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port prior-less-10ms for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/prior\-nw\-ctrl//) {
		  $vlanPortStorage->{$port}{$vlanId}{PRIONWCTRL} = 1;
		  ToolCommon::Log($fh, sprintf("Stored bridge prior-nw-ctrl for %s:%s\n", $port, $vlanId)) if ($params->{DEBUG});
		}
		if ($line =~ s/static\-user\s+ip\-address\s+(\d+\.\d+\.\d+\.\d+\/\d+)//) {
		  my $ipAddr = $1;
		  $vlanPortStorage->{$port}{$vlanId}{IPADDR}{$ipAddr} = 1;
		  ToolCommon::Log($fh, sprintf("Stored VLAN-port static IP interface for %s:%s as %s\n", $port, $vlanId,
					       $ipAddr)) if ($params->{DEBUG});
		}
		$line =~ s/\s//g;
		if ($line ne "") {
		  FamilyCommon::SetAndLogError($fh, "[GetBridgeConfig] Command not completely parsed, left: $line");
		  return 0;
		}
	      }
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, sprintf("Empty CLI response received: %s", IsamCli::GetError()));
	  return 0;
	}
      } else {
	FamilyCommon::SetAndLogError($fh, "Port not to expected syntax: $portId");
	return 0;
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Get IGMP channel configuration of involved user-itf bridges.
#--
#-- Hash structure as follows:
#-- %igmpChConfig{$port}
#--                             {TAG}
#--                             {NETWORKVLAN}
#--                             {VLANSCOPE}
#--                             {QOS}
#--                             {QOSPROF}
#--                             {PRIOBESTEFF}
#--                             {PRIOBACKGRND}
#--                             {PRIOSPARE}
#--                             {PRIOEXCEFF}
#--                             {PRIOCTRLLOAD}
#--                             {PRIOLESS100MS}
#--                             {PRIOLESS10MS}
#--                             {PRIONWCTRL}
#--                             {IPADDR}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $vrfConfig: pointer to VRF configuration
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $bridgePortStorage: pointer to a hash for storing the bridge port configuration.
#-- $vlanPortStorage: pointer to a hash for storing the VLAN port configuration.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub GetIgmpChlConfig {
  my ($fh, $cli, $vrfConfig, $cliResponseLines, $igmpChStorage, $releaseNum) = @_;
  my @response = ();
  my $rspStr = "";
  my ($port, $portId, $vlanId);

  foreach $portId (keys %{$vrfConfig->{USRITF}}) {
    $vlanId = undef;
    if (defined($vrfConfig->{USRITF}{$portId}{BRIDGE}) || defined($vrfConfig->{USRITF}{$portId}{VLAN})) {
      if (($portId =~ /vlan\-port:([^\s]+)/) || ($portId =~ /bridge\-port:([^\s]+)/)) {
	$port = $1;
	if ($portId =~ /vlan\-port:/) {
	  $port =~ s/:(\d+)$//;
	  $vlanId = $1;
	} else {
	  $vlanId = undef;
	}
	if (!defined($vlanId)) {
	  @response = IsamCli::SendCommand($cli, "info configure igmp channel $port flat");
	} else {
	  @response = IsamCli::SendCommand($cli, "info configure igmp channel vlan:$port:$vlanId flat");
	}
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    # No IGMP channel defined for this user-itf, ignore and go for the next
	  } else {
	    foreach my $line (@response) {
	      chomp $line;
	      if ($line =~ /^configure\s+igmp/) {
		push @{$cliResponseLines}, $line if ($line =~ /^configure/);
		ToolCommon::Log($fh, "[GetIgmpChlConfig] line read: $line\n") if ($params->{DEBUG});
		if (defined($vlanId)) {
		  $line =~ s/configure\s+igmp\s+channel\s+vlan:$port:$vlanId//;
		  $igmpChStorage->{$portId}{VLAN} = $vlanId;
		} else {
		  $line =~ s/configure\s+igmp\s+channel\s+$port//;
		}
		if ($line =~ s/perm\-pkg\-bitmap\s+([^\s]+)//) {
		  $igmpChStorage->{$portId}{PERMPKGBITMAP} = $1;
		  ToolCommon::Log($fh, sprintf("Stored permission package bitmap for %s as '%s'\n", $portId,
					       $igmpChStorage->{$portId}{PERMPKGBITMAP})) if ($params->{DEBUG});
		}
		if ($line =~ s/max\-num\-group\s+(\d+)//) {
		  $igmpChStorage->{$portId}{MAXNUMGRP} = $1;
		  ToolCommon::Log($fh, sprintf("Stored maximum number of groups for %s as '%d'\n", $portId,
					       $igmpChStorage->{$portId}{MAXNUMGRP})) if ($params->{DEBUG});
		}
		if ($line =~ s/igmp\-version\s+([^\s]+)//) {
		  $igmpChStorage->{$portId}{IGMPVERS} = $1;
		  ToolCommon::Log($fh, sprintf("Stored IGMP version for %s as '%s'\n", $portId,
					       $igmpChStorage->{$portId}{IGMPVERS})) if ($params->{DEBUG});
		}
		if ($line =~ s/fullview\-packages\s+([^\s]+)//) {
		  $igmpChStorage->{$portId}{FULLVIEWPKG} = $1;
		  ToolCommon::Log($fh, sprintf("Stored fullview packages for %s as '%s'\n", $portId,
					       $igmpChStorage->{$portId}{FULLVIEWPKG})) if ($params->{DEBUG});
		}
		if ($line =~ s/preview\-packages\s+([^\s]+)//) {
		  $igmpChStorage->{$portId}{PREVIEWPKG} = $1;
		  ToolCommon::Log($fh, sprintf("Stored preview packages for %s as '%s'\n", $portId,
					       $igmpChStorage->{$portId}{PREVIEWPKG})) if ($params->{DEBUG});
		}
		$line =~ s/\s//g;
		if ($line ne "") {
		  FamilyCommon::SetAndLogError($fh, "[GetIgmpChlConfig] Command not completely parsed, left: $line");
		  return 0;
		}
	      }
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, sprintf("Empty CLI response received: %s", IsamCli::GetError()));
	  return 0;
	}
      } else {
	FamilyCommon::SetAndLogError($fh, "Port not to expected syntax: $portId");
	return 0;
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Get QoS session profile config of involved user-itf ATM ports.
#-- Hash structure as follows:
#-- %qosSessionProfileConfig{$profName}{FLOWTYPE}
#--                                    {UPPOLICER}
#--                                    {DNPOLICER}
#--                                    {UPMARKER}
#--                                    {UPPOLICY}
#--                                    {DNPOLICY}
#--                                    {NEWNAME}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the QoS session profiles (name
#--                 has already been stored during the VRF user-itf parsing).
#-- $releaseNum: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub GetQosSessionProfiles {
  my ($fh, $cli, $cliResponseLines, $configStorage, $releaseNum) = @_;
  my $ok = 1;
  my @response = ();
  my $rspStr = "";
  my $firstLine = 1;

  foreach my $profName (keys %{$configStorage}) {
    @response = IsamCli::SendCommand($cli, "info configure qos profiles session $profName flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::SetAndLogError($fh, sprintf("  FAILED: %s\n", IsamCli::GetError()));
	$ok = 0;
      } else {
	foreach my $line (@response) {
	  chomp $line;
	  if ($line =~ /configure\s+qos\s+profiles\s+session\s+([^\s]+)/) {
	    $profName = $1;
	    if (defined($configStorage->{$profName}{FLOWTYPE})) {
	      push @{$cliResponseLines}, $line;
	      ToolCommon::Log($fh, "[GetQosSessionProfiles] line read: $line\n") if ($params->{DEBUG});
	      $firstLine = 0;
	      $line =~ s/configure\s+qos\s+profiles\s+session\s+[^\s]+//;
	      if ($line =~ s/logical\-flow\-type\s+([^\s]+)//) {
		$configStorage->{$profName}{FLOWTYPE} = $1;
		ToolCommon::Log($fh, sprintf("  [QOSSESS] stored configStorage->{$profName}{FLOWTYPE} as '%s'\n",
					     $configStorage->{$profName}{FLOWTYPE})) if $params->{DEBUG};
		if ($configStorage->{$profName}{FLOWTYPE} ne "generic") {
		  my $newName = $profName . $params->{PROFSUFX};
		  $newName = substr($newName, 0, $MAX_QOS_SESS_PROFNAME_L) if length($newName) > $MAX_QOS_SESS_PROFNAME_L;
		  if ($newName eq $profName) {
		    $newName = substr($profName, 0, $MAX_QOS_SESS_PROFNAME_L - length($params->{PROFSUFX}));
		    $newName .= $params->{PROFSUFX};
		  }
		  $configStorage->{$profName}{NEWNAME} = $newName;
		  ToolCommon::Log($fh, sprintf("  [QOSSESS] stored configStorage->{$profName}{NEWNAME} as '%s'\n",
					       $configStorage->{$profName}{NEWNAME})) if $params->{DEBUG};
		}
	      }
	      if ($line =~ s/up\-policer\s+name:([^\s]+)//) {
		$configStorage->{$profName}{UPPOLICER} = $1;
		ToolCommon::Log($fh, sprintf("  [QOSSESS] stored configStorage->{$profName}{UPPOLICER} as '%s'\n",
					     $configStorage->{$profName}{UPPOLICER})) if $params->{DEBUG};
	      }
	      if ($line =~ s/down\-policer\s+name:([^\s]+)//) {
		$configStorage->{$profName}{DNPOLICER} = $1;
		ToolCommon::Log($fh, sprintf("  [QOSSESS] stored configStorage->{$profName}{DNPOLICER} as '%s'\n",
					     $configStorage->{$profName}{DNPOLICER})) if $params->{DEBUG};
	      }
	      if ($line =~ s/up\-marker\s+name:([^\s]+)//) {
		$configStorage->{$profName}{UPMARKER} = $1;
		ToolCommon::Log($fh, sprintf("  [QOSSESS] stored configStorage->{$profName}{UPMARKER} as '%s'\n",
					     $configStorage->{$profName}{UPMARKER})) if $params->{DEBUG};
	      }
	      if ($line =~ s/up\-policy\s+([^\s]+)//) {
		$configStorage->{$profName}{UPPOLICY} = $1;
		ToolCommon::Log($fh, sprintf("  [QOSSESS] stored configStorage->{$profName}{UPPOLICY} as '%s'\n",
					     $configStorage->{$profName}{UPPOLICY})) if $params->{DEBUG};
	      }
	      if ($line =~ s/down\-policy\s+([^\s]+)//) {
		$configStorage->{$profName}{DNPOLICY} = $1;
		ToolCommon::Log($fh, sprintf("  [QOSSESS] stored configStorage->{$profName}{DNPOLICY} as '%s'\n",
					     $configStorage->{$profName}{DNPOLICY})) if $params->{DEBUG};
	      }
	      $line =~ s/\s//g;
	      if ($line ne "") {
		FamilyCommon::SetAndLogError($fh, "[GetQosSessionProfiles] Command not completely parsed, left: $line");
		return 0;
	      }
	    } else {
	      ToolCommon::Log($fh, "  [QOSSESS] skipping session profile '$profName'\n") if ($params->{DEBUG});
	    }
	  } else {
	    if (($line ne "") &&
		! $firstLine &&
		! ($line =~ /\#[-]+$/) &&
		! ($line =~ /[\$%\#>]\s*$/)) {
	      $ok = 0;
	      SetAndLogError($fh, "Line not according to expected syntax: $line");
	    }
	    $firstLine = 0 if $firstLine;
	  }
	}
      }
    } else {
      FamilyCommon::SetAndLogError($fh, sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()));
      $ok = 0;
    }
  }
  return $ok;
}
#------------------------------------------------------------------------------
#-- Check validity of the VRF configuration.  Reject if
#-- 1. VLAN not layer2-terminated
#-- 2. VRF user-itf port is vlan-port (should be ATM or bridge port)
#-- 3. Invalid netmask used in route-dest (should be 32 or 0)
#-- 4. AAL5-ENCAP-TYPE can only be llc-snap (default setting) of vc-mux-bridged-8023
#-- 5. Network-vlan occurs only once in the vlanBind info
#-- 6. Layer3-itf associated to network vlan-binding should occur only once.
#-- 7. A VRF user interface has an active pppoe session.
#-- 8. VLAN ports have local vlan scope and do not have a network VLAN when configured in IP aware bridge
#------------------------------------------------------------------------------
sub ConfigValid {
  my ($fh, $vrfConfig, $vlanConfig, $bridgeConfig, $vlanPortConfig, $vlanBindConfig, $pppoeSessions, $releaseNum) = @_;
  my $ok = 1;
  my $netwIfCount = 0;
  my $bridgeWithPvidFound = 0;
  my $wrongEncapTypeFound = 0;
  my $vlanPortNwScopeFound = 0;
  my $vlanPortNwVlanFound = 0;

  if (!defined($vlanConfig->{VLANDEF}{ID})) {
    FamilyCommon::SetAndLogError($fh, "  FAILED: No VLAN definition has been retrieved");
    $ok = 0;
  } else {
    # Check network-itf VLAN: should not be stacked
    foreach my $vlanId (keys %{$vrfConfig->{NWITF}}) {
      if ($vlanId == $vlanConfig->{VLANDEF}{ID}) {
	if ($vlanConfig->{VLANDEF}{MODE} ne "layer2-terminated") {
	  $ok = 0;
	  FamilyCommon::SetAndLogError($fh, "  FAILED: VLAN $vlanId not layer2-terminated ($vlanConfig->{VLANDEF}{MODE})");
	  last;
	} else {
	  ToolCommon::Log($fh, "  Network VLAN is layer2-terminated: OK\n");
	  $netwIfCount++;
	}
      } else {
	$netwIfCount++;
      }
    }
    foreach my $pvc (keys %{$bridgeConfig}) {
      if (defined($bridgeConfig->{$pvc}{PVID})) {
	$ok = 0, $bridgeWithPvidFound++;
	FamilyCommon::SetAndLogError($fh, "bridge $pvc has pvid");
      }
    }
    if (!$bridgeWithPvidFound) {
      ToolCommon::Log($fh, "  No bridge ports with PVID found: OK\n");
    }
    foreach my $ipAddress (keys %{$vrfConfig->{ROUTEDEST}}) {
      if ($ipAddress =~ /\d+\.\d+\.\d+\.\d+\/(\d+)/) {
	my $mask = $1;
	if (($mask != 0) && ($mask != 32)) {
	  $ok = 0;
	  FamilyCommon::SetAndLogError($fh, "Invalid netmask ($mask) used in $ipAddress (should be 32 or 0): deletion will fail");
	}
      } else {
	# Should not happen...
	$ok = 0;
	FamilyCommon::SetAndLogError($fh, "IP address ($ipAddress) not according to expected syntax");
      }
    }
  }
  my @l3Interfaces = (keys %{$vlanBindConfig->{$vlanConfig->{VLANDEF}{ID}}});
  my $numOfInterfaces = scalar @l3Interfaces;
  if ($numOfInterfaces > 1) {
    $ok = 0;
    FamilyCommon::SetAndLogError($fh, "Only one VLAN-Layer3 Interface binding should be present fo the network-VLAN ($numOfInterfaces found)");
  } else {
    ToolCommon::Log($fh, "  Network VRF has only 1 Layer3 interface binding: OK\n");
  }
  my $netwVlanL3ItfId = $l3Interfaces[0];
  my $l3ItfRefs = 0;
  foreach my $vlanId (keys %{$vlanBindConfig}) {
    foreach my $vlanL3Itf (keys %{$vlanBindConfig->{$vlanId}}) {
      $l3ItfRefs++ if ($vlanL3Itf == $l3Interfaces[0]);
    }
  }
  if ($l3ItfRefs > 1) {
    $ok = 0;
    FamilyCommon::SetAndLogError($fh, "Layer 3 interface of network VRF referred to by multiple VRFs");
    foreach my $vlanId (keys %{$vlanBindConfig}) {
      foreach my $vlanL3Itf (keys %{$vlanBindConfig->{$vlanId}}) {
	ToolCommon::Log($fh, "  Binding between '$vlanId' and Layer3 interface '$vlanL3Itf'\n");
      }
    }
  } else {
    ToolCommon::Log($fh, "  Network VRF Layer3 interface only refered once: OK\n");
  }
  foreach my $port (keys %{$pppoeSessions}) {
    if (defined($vrfConfig->{USRITF}{$port}{ATM}) || defined($vrfConfig->{USRITF}{$port}{BRIDGE}) || defined($vrfConfig->{USRITF}{$port}{VLAN})) {
      $ok = 0;
      FamilyCommon::SetAndLogError($fh, "User interface $port involved in dynamic pppoe session");
    }
  }
  foreach my $port (keys %{$vlanPortConfig}) {
    foreach my $vlanId (keys %{$vlanPortConfig->{$port}}) {
      my $vlanPort = "vlan-port:$port:$vlanId";
      ToolCommon::Log($fh, "Probing for vlan-port '$vlanPort'\n") if $params->{DEBUG};
      if (defined($vlanPortConfig->{$port}{$vlanId}{NETWORKVLAN})) {
	if (defined($vrfConfig->{USERITF}{VLAN})) {
	  $ok = 0, $vlanPortNwVlanFound++;
	  FamilyCommon::SetAndLogError($fh, "VLAN port $port:$vlanId has network VLAN ($vlanPortConfig->{$port}{$vlanId}{NETWORKVLAN})");
	}
      }
      if (defined($vlanPortConfig->{port}{$vlanId}{VLANSCOPE}) && ($vlanPortConfig->{port}{$vlanId}{VLANSCOPE} eq "network")) {
	if (defined($vrfConfig->{USERITF}{VLAN})) {
	  $ok = 0, $vlanPortNwScopeFound++;
	  FamilyCommon::SetAndLogError($fh, "VLAN scope for $port:$vlanId is network, should be local scope");
	}
      }
    }
  }
  if (!$vlanPortNwVlanFound) {
    ToolCommon::Log($fh, "  No bridge VLAN ports with network VLAN found: OK\n");
  }
  if (!$vlanPortNwScopeFound) {
    ToolCommon::Log($fh, "  No bridge VLAN ports with network-scope found: OK\n");
  }
  return $ok;
}


#------------------------------------------------------------------------------
#  Get Full release info
#------------------------------------------------------------------------------
sub GetFullReleaseInfo {
  my ($fh, $cli, $debug) = @_;
  my $oswpInfo;
  my $oswpName;
  my $fullRelNum = 0;
  my $regExp = REGEXP;

    $oswpInfo = IsamCli::GetOswpInfo($cli, REGEXP);
    if (!defined($oswpInfo)) {
		FamilyCommon::SetAndLogError($fh, sprintf("Failed to retrieve OSWP info: %s", IsamCli::GetError()));
		ToolCommon::Log($fh, "Failed to retrieve OSWP info\n") if $debug;
		$fullRelNum = 0;
    } else {
		ToolCommon::Log($fh, "Active OSWP index : $oswpInfo->{ACTIVE}{INDEX}\n") if $debug;
		ToolCommon::Log($fh, "Active OSWP name  : $oswpInfo->{ACTIVE}{NAME}\n") if $debug;
		
		$oswpName = $oswpInfo->{ACTIVE}{NAME}; 

		if (basename($oswpName) =~ /$regExp/) {
			  ToolCommon::Log($fh, "Converting to Release '$1' '$2' '$3' \n") if $debug;
		      $fullRelNum = Capability::ConvertRelease($1, $2, $3);
	    }	
    }
  my @response = IsamCli::SendCommand($cli, "exit all");  
  return $fullRelNum;
}

#------------------------------------------------------------------------------
# Generate DHCP Relay interface rollback command using the 
# configuration previously retrieved.
#------------------------------------------------------------------------------
sub RollBackDHCPRelayDeletion {
  my ($dhcpAgentStorage,  $releaseNum) = @_;

  my $rollBackCmd  = "configure dhcp-relay agent $dhcpAgentStorage->{VRFID}";
     $rollBackCmd .= " bcast-vlan $dhcpAgentStorage->{BCASTVLAN}"; 
	 $rollBackCmd .= " option-82" if (defined($dhcpAgentStorage->{OPT82})); 
	 $rollBackCmd .= " circuit-id $dhcpAgentStorage->{CIRCID}" if (defined($dhcpAgentStorage->{CIRCID})); 
	 $rollBackCmd .= " remote-id $dhcpAgentStorage->{REMID}" if (defined($dhcpAgentStorage->{REMID})); 
	 $rollBackCmd .= " client-hw-address $dhcpAgentStorage->{CLIENTHWADDR}" if (defined($dhcpAgentStorage->{CLIENTHWADDR}));
	 $rollBackCmd .= " dhcp-linerate $dhcpAgentStorage->{LINERATE}" if (defined($dhcpAgentStorage->{LINERATE}));
	 return $rollBackCmd; 
}

#------------------------------------------------------------------------------
# Generate VRF deletion rollback command using the VRF configuration previously
# retrieved.
#------------------------------------------------------------------------------
sub RollBackVrfDeletion {
  my ($vrfStorage, $releaseNum) = @_;

  my $rollBackCmd = "configure ip vrf $vrfStorage->{VRFDEF}{ID}";
  $rollBackCmd .= " name $vrfStorage->{VRFDEF}{NAME}" if (defined($vrfStorage->{VRFDEF}{NAME}));
  $rollBackCmd .= " desc $vrfStorage->{VRFDEF}{DESC}" if (defined($vrfStorage->{VRFDEF}{DESC}));
  $rollBackCmd .= " contact $vrfStorage->{VRFDEF}{CONTACT}" if (defined($vrfStorage->{VRFDEF}{CONTACT}));
  $rollBackCmd .= " my-ipaddress $vrfStorage->{VRFDEF}{IP}" if (defined($vrfStorage->{VRFDEF}{IP}));
  $rollBackCmd .= " default-ttl $vrfStorage->{VRFDEF}{DEFTTL}" if (defined($vrfStorage->{VRFDEF}{DEFTTL}));
  $rollBackCmd .= " reassem-timeout $vrfStorage->{VRFDEF}{REASSEM}" if (defined($vrfStorage->{VRFDEF}{REASSEM}));
  $rollBackCmd .= " mode $vrfStorage->{VRFDEF}{MODE}" if (defined($vrfStorage->{VRFDEF}{MODE}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate VRF route destination deletion rollback command using the VRF
# configuration previously retrieved.
#------------------------------------------------------------------------------
sub RollBackRouteDestDeletion {
  my ($vrfStorage, $ipAddress, $releaseNum) = @_;

  my $rollBackCmd = sprintf("configure ip vrf %d route-dest %s next-hop %s",
			    $vrfStorage->{VRFDEF}{ID}, $vrfStorage->{ROUTEDEST}{$ipAddress}{IP}, $vrfStorage->{ROUTEDEST}{$ipAddress}{NEXTHOP});
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate VRF network interface deletion rollback command using the VRF
# configuration previously retrieved.
# Part 1: re-create the network interface
#------------------------------------------------------------------------------
sub RollBackNetwItfDeletionPart1 {
  my ($vrfStorage, $vlanId, $releaseNum) = @_;
  my $rollBackCmd = sprintf("configure ip vrf %d network-itf %d", $vrfStorage->{VRFDEF}{ID}, $vlanId);

  $rollBackCmd .= " arp-policy $vrfStorage->{NWITF}{$vlanId}{ARPPOLICY}" if (defined($vrfStorage->{NWITF}{$vlanId}{ARPPOLICY}));
  $rollBackCmd .= " $vrfStorage->{NWITF}{$vlanId}{NUMBERED}" if (defined($vrfStorage->{NWITF}{$vlanId}{NUMBERED}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate VRF network interface deletion rollback command using the VRF
# configuration previously retrieved.
# Part 2: re-create the network interface adding the options not available
# at creation time
#------------------------------------------------------------------------------
sub RollBackNetwItfDeletionPart2 {
  my ($vrfStorage, $vlanId, $releaseNum) = @_;

  return ((defined($vrfStorage->{NWITF}{$vlanId}{IP}))
    ? sprintf("configure ip vrf %d network-itf %d ip-address %s", $vrfStorage->{VRFDEF}{ID}, $vlanId, $vrfStorage->{NWITF}{$vlanId}{IP})
    : "");
}

#------------------------------------------------------------------------------
# Generate VRF user interface deletion rollback command using the VRF
# configuration previously retrieved.
#------------------------------------------------------------------------------
sub RollBackUsrItfDeletion {
  my ($vrfStorage, $port, $releaseNum) = @_;
  my $rollBackCmd = sprintf("configure ip vrf %d user-itf port %s", $vrfStorage->{VRFDEF}{ID}, $port);

  $rollBackCmd .= " qos-profile name:$vrfStorage->{USRITF}{$port}{QOS}" if (defined($vrfStorage->{USRITF}{$port}{QOS}));
  $rollBackCmd .= " arp-proxy-stats" if (defined($vrfStorage->{USRITF}{$port}{ARPPROXYSTATS}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate VLAN deletion rollback command using the VLAN configuration previously
# retrieved.
# Part 1: re-create the VLAN
#------------------------------------------------------------------------------
sub RollBackVlanDeletionPart1 {
  my ($vlanStorage, $releaseNum) = @_;
  my $rollBackCmd = "configure vlan id $vlanStorage->{VLANDEF}{ID} mode $vlanStorage->{VLANDEF}{MODE}";

  $rollBackCmd .= " name $vlanStorage->{VLANDEF}{NAME}" if (defined($vlanStorage->{VLANDEF}{NAME}));
  $rollBackCmd .= " prioriy $vlanStorage->{VLANDEF}{PRIO}" if (defined($vlanStorage->{VLANDEF}{PRIO}));
  $rollBackCmd .= " broadcast-frames " if (defined($vlanStorage->{VLANDEF}{BCFRMS}));
  $rollBackCmd .= " protocol-filter $vlanStorage->{VLANDEF}{PROTFLT}" if (defined($vlanStorage->{VLANDEF}{PROTFLT}));
  $rollBackCmd .= " pppoe-relay-tag $vlanStorage->{VLANDEF}{PPPOERLYTAG}" if (defined($vlanStorage->{VLANDEF}{PPPOERLYTAG}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate VLAN deletion rollback command using the VLAN configuration previously
# retrieved.
# Part 2: re-create the network interface adding the options not available
# at creation time
#------------------------------------------------------------------------------
sub RollBackVlanDeletionPart2 {
  my ($vlanStorage, $releaseNum) = @_;
  my $rollBackCmd = "configure vlan id $vlanStorage->{VLANDEF}{ID}";

  $rollBackCmd .= " dhcp-opt82 $vlanStorage->{VLANDEF}{OPT82}" if (defined($vlanStorage->{VLANDEF}{OPT82}));
  $rollBackCmd .= " circuit-id-dhcp $vlanStorage->{VLANDEF}{CIRCIDDHCP}" if (defined($vlanStorage->{VLANDEF}{CIRCIDDHCP}));
  $rollBackCmd .= " remote-id-dhcp $vlanStorage->{VLANDEF}{REMIDDHCP}" if (defined($vlanStorage->{VLANDEF}{REMIDDHCP}));
  $rollBackCmd .= " dhcp-linerate $vlanStorage->{VLANDEF}{DHCPLINERATE}" if (defined($vlanStorage->{VLANDEF}{DHCPLINERATE}));
  $rollBackCmd .= " pppoe-linerate $vlanStorage->{VLANDEF}{PPPOELINERATE}" if (defined($vlanStorage->{VLANDEF}{PPPOELINERATE}));
  $rollBackCmd .= " circuit-id-pppoe $vlanStorage->{VLANDEF}{CIRCIDPPPOE}" if (defined($vlanStorage->{VLANDEF}{CIRCIDPPPOE}));
  $rollBackCmd .= " remote-id-pppoe $vlanStorage->{VLANDEF}{REMIDPPPOE}" if (defined($vlanStorage->{VLANDEF}{REMIDPPPOE}));
  $rollBackCmd .= " secure-forwarding" if (defined($vlanStorage->{VLANDEF}{SECFWD}));
  $rollBackCmd .= " aging-time $vlanStorage->{VLANDEF}{AGING}" if (defined($vlanStorage->{VLANDEF}{AGING}));
  $rollBackCmd .= " vmac-translation" if (defined($vlanStorage->{VLANDEF}{VMACTRANS}));
  $rollBackCmd .= " vmac-dnstr-filter" if (defined($vlanStorage->{VLANDEF}{VMACDNFLT}));
  return ($rollBackCmd ne "configure vlan id $vlanStorage->{VLANDEF}{ID}") ? $rollBackCmd : "";
}

#------------------------------------------------------------------------------
# Generate PVC deletion rollback command using the VLAN configuration previously
# retrieved.
#------------------------------------------------------------------------------
sub RollBackPvcDeletion {
  my ($pvcStorage, $pvc, $releaseNum) = @_;
  my $rollBackCmd = "configure atm pvc $pvc";

  $rollBackCmd .= " aal5-encap-type $pvcStorage->{$pvc}{AAL5ENCAP}" if (defined($pvcStorage->{$pvc}{AAL5ENCAP}));
  $rollBackCmd .= " admin-down" if (defined($pvcStorage->{$pvc}{ADMINDOWN}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Generate IGMP deletion rollback or IGMP createion command using the IGMP configuration
# previously retrieved.
# Part 1: create the IGMP channel
#------------------------------------------------------------------------------
sub CreateIgmpChPart1 {
  my ($igmpChStorage, $portId, $vlanId, $extendedPort, $releaseNum) = @_;
  my $cmd;

  if (defined($vlanId)) {
    $cmd = "configure igmp channel vlan:$portId:$vlanId max-num-group $igmpChStorage->{$extendedPort}{MAXNUMGRP}";
  } else {
    $cmd = "configure igmp channel $portId max-num-group $igmpChStorage->{$extendedPort}{MAXNUMGRP}";
  }
  $cmd .= " perm-pkg-bitmap $igmpChStorage->{$extendedPort}{PERMPKGBITMAP}" if (defined($igmpChStorage->{$extendedPort}{PERMPKGBITMAP}));
  $cmd .= " igmp-version $igmpChStorage->{$extendedPort}{IGMPVERS}" if (defined($igmpChStorage->{$extendedPort}{IGMPVERS}));
  return $cmd;
}

#------------------------------------------------------------------------------
# Generate IGMP deletion rollback or IGMP createion command using the IGMP configuration
# previously retrieved.
# Part 2: create the IGMP channel preview package
#------------------------------------------------------------------------------
sub CreateIgmpChPart2 {
  my ($igmpChStorage, $portId, $vlanId, $extendedPort, $releaseNum) = @_;
  my $cmd;

  if (defined($igmpChStorage->{$extendedPort}{PREVIEWPKG})) {
    if (defined($vlanId)) {
      $cmd = "configure igmp channel vlan:$portId:$vlanId preview-packages $igmpChStorage->{$extendedPort}{PREVIEWPKG}";
    } else {
      $cmd = "configure igmp channel $portId preview-packages $igmpChStorage->{$extendedPort}{PREVIEWPKG}";
    }
  } else {
    $cmd = "";
  }
  return $cmd;
}

#------------------------------------------------------------------------------
# Generate IGMP deletion rollback or IGMP createion command using the IGMP configuration
# previously retrieved.
# Part 3: create the IGMP channel fullview package
#------------------------------------------------------------------------------
sub CreateIgmpChPart3 {
  my ($igmpChStorage, $portId, $vlanId, $extendedPort, $releaseNum) = @_;
  my $cmd;

  if (defined($igmpChStorage->{$extendedPort}{FULLVIEWPKG})) {
    if (defined($vlanId)) {
      $cmd = "configure igmp channel vlan:$portId:$vlanId fullview-packages $igmpChStorage->{$extendedPort}{FULLVIEWPKG}";
    } else {
      $cmd = "configure igmp channel $portId fullview-packages $igmpChStorage->{$extendedPort}{FULLVIEWPKG}";
    }
  } else {
    $cmd = "";
  }
  return $cmd;
}


#------------------------------------------------------------------------------
# Prepare the IP-aware bridge deletion:
#   Reset bcast vlan associated to DHCP relay for releases > 3701
# 1 Clear out VRF before it can be deleted:
#   1.1. Delete all route-dest resources
#   1.2. Delete all user-itf resources
#   1.3. Delete all gateway-itf resources
#   1.4. Delete network itf resources
# 2 Delete VRF
# 3 Delete VLAN
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $vrfStorage: pointer to a hash of complete VRF structure.
# $vlanStorage: pointer to a hash of the complete VLAN structure.
# $igmpChStorage: pointer to a hash of the IGMP channel configuration
# $dhcpAgentStorage: pointer to a hash of the complete DHCP relay-agent structure.
# $pvcStorage: pointer to a hash of the ATM PVC configuration.
# $cliCommandStorage: pointer to an array holding all the unconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollkbac commands of the unconfiguration.
#------------------------------------------------------------------------------
sub PrepareDeletion {
  my ($fh, $cli, $vrfStorage, $vlanStorage, $igmpChStorage, $dhcpAgentStorage, $cliCommandStorage, $rollBackCmds, $releaseId) = @_;
  my $cmd = "";
  my $rollBackCmd = "";

  ToolCommon::Log($fh, "[PrepareDeletion]\n") if ($params->{DEBUG});

  # Reset bcast vlan associated with DHCP relay 
  ToolCommon::Log($fh,"[Prepare to modify DHCP relay info]\n") if ($params->{DEBUG});
  my $fullRelease = GetFullReleaseInfo($fh, $cli, $params->{DEBUG}); 

	if (Capability::ReleaseCompare($fullRelease, Capability::ConvertRelease(3,7,1), REGEXP, 1,1) > 0) {
         ToolCommon::Log($fh,"Release compare with releasenum '$fullRelease'\n")if ($params->{DEBUG});
	  if($dhcpAgentStorage->{VRFID}) {
		$cmd = sprintf("configure dhcp-relay agent %d bcast-vlan 0",$dhcpAgentStorage->{VRFID}); 
		ToolCommon::Log($fh,"Stored '$cmd'\n")if ($params->{DEBUG});
		push @{$cliCommandStorage}, $cmd;
		$rollBackCmd = RollBackDHCPRelayDeletion($dhcpAgentStorage, $releaseId);
		if($rollBackCmd ne "") {
			push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
		ToolCommon::Log($fh,"Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
		}
	  }
        }

  # Clear out VRF before it can be deleted:
  # 1.1 Delete all route-dest resources
  foreach my $ipAddress (keys %{$vrfStorage->{ROUTEDEST}}) {
    if ($ipAddress =~ /\d+\.\d+\.\d+\.\d+\/(\d+)/) {
      my $mask = $1;
      if (($mask == 32) || ($mask == 0)) {
	$cmd = sprintf("configure ip vrf %d no route-dest %s next-hop %s",
		       $vrfStorage->{VRFDEF}{ID}, $vrfStorage->{ROUTEDEST}{$ipAddress}{IP}, $vrfStorage->{ROUTEDEST}{$ipAddress}{NEXTHOP});
	$rollBackCmd = RollBackRouteDestDeletion($vrfStorage, $ipAddress, $releaseId);
	$rollBackCmds->{$cmd}[0] = $rollBackCmd;
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	push @{$cliCommandStorage}, $cmd;
      }
    }
  }
  # 1.2. Delete all user-itf resources
  foreach my $port (keys %{$vrfStorage->{USRITF}}) {
    # Loop on all user ports
    $cmd = sprintf("configure ip vrf %d user-itf no port %s", $vrfStorage->{VRFDEF}{ID}, $port);
    $rollBackCmd = RollBackUsrItfDeletion($vrfStorage, $port, $releaseId);
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
    push @{$cliCommandStorage}, $cmd;
    my ($portId, $vlanId);
    if (($port =~ /bridge\-port:(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) ||
	($port =~ /bridge\-port:(\d+\/\d+\/\d+\/\d+)/)) {
      $portId = $1, $vlanId = undef;
    } elsif (($port =~ /vlan\-port:(\d+\/\d+\/\d+\/\d+:\d+:\d+):(\d+)/) ||
	     ($port =~ /vlan\-port:(\d+\/\d+\/\d+\/\d+):(\d+)/)) {
      $portId = $1, $vlanId = $2;
    } else {
      $portId = undef, $vlanId = undef;
      # ATM ports -> skip: do not have IGMP
    }
    if (defined($portId)) {
      if (defined($igmpChStorage->{$port}{MAXNUMGRP})) {
	# An IGMP channel was associated to this IP interface, roll it back
	$rollBackCmd = CreateIgmpChPart1($igmpChStorage, $portId, $vlanId, $port, $releaseId);
	push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	$rollBackCmd = CreateIgmpChPart2($igmpChStorage, $portId, $vlanId, $port, $releaseId);
	if ($rollBackCmd ne "") {
	  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	  ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	}
	$rollBackCmd = CreateIgmpChPart3($igmpChStorage, $portId, $vlanId, $port, $releaseId);
	if ($rollBackCmd ne "") {
	  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	  ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	}
      }
    }
  }
  # 1.3. Delete all gateway-itf resources
  foreach my $ipAddress (keys %{$vrfStorage->{GWYITF}}) {
    $cmd = sprintf("configure ip vrf %d gateway-itf no ip-address %s", $vrfStorage->{VRFDEF}{ID}, $ipAddress);
    $rollBackCmd = sprintf("configure ip vrf %d gateway-itf ip-address %s", $vrfStorage->{VRFDEF}{ID}, $ipAddress);
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
    push @{$cliCommandStorage}, $cmd;
  }
  # 1.4. Delete network itf resources
  foreach my $vlanId (keys %{$vrfStorage->{NWITF}}) {
    # Loop on all vlans stored for this VRF
    if (defined($vrfStorage->{NWITF}{$vlanId}{IP})) {
      $cmd = sprintf("configure ip vrf %d network-itf %d no ip-address %s", $vrfStorage->{VRFDEF}{ID}, $vlanId,
		     $vrfStorage->{NWITF}{$vlanId}{IP});
      push @{$cliCommandStorage}, $cmd;
      $rollBackCmd = RollBackNetwItfDeletionPart2($vrfStorage, $vlanId, $releaseId);
      if ($rollBackCmd ne "") {
	push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
      }
    }
    $cmd = sprintf("configure ip vrf %d no network-itf %s", $vrfStorage->{VRFDEF}{ID}, $vlanId);
    push @{$cliCommandStorage}, $cmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    $rollBackCmd = RollBackNetwItfDeletionPart1($vrfStorage, $vlanId, $releaseId);
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
    $rollBackCmd = RollBackNetwItfDeletionPart2($vrfStorage, $vlanId, $releaseId);
  }
  # 2. Delete VRF
  $cmd = "configure ip no vrf $vrfStorage->{VRFDEF}{ID}";
  $rollBackCmd = RollBackVrfDeletion($vrfStorage, $releaseId);
  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
  ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  push @{$cliCommandStorage}, $cmd;
  # 3. Delete VLAN
  $cmd = "admin ip no vlan-itf-delete $vlanStorage->{VLANDEF}{ID}";
  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
  push @{$cliCommandStorage}, $cmd;
  $cmd = "configure vlan no id $vlanStorage->{VLANDEF}{ID}";
  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
  $rollBackCmd = RollBackVlanDeletionPart1($vlanStorage, $releaseId);
  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
  ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  $rollBackCmd = RollBackVlanDeletionPart2($vlanStorage, $releaseId);
  if ($rollBackCmd ne "") {
    push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
    ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  }
  push @{$cliCommandStorage}, $cmd;
}

#------------------------------------------------------------------------------
# Generate VLAN creation rollback command to restore the VLAN in its
# original state.
# Part 1: Create the VLAN
#------------------------------------------------------------------------------
sub CreateVLANpart1 {
  my ($vlanConfig, $releaseId) = @_;
  my $cmd = sprintf("configure vlan id %s mode residential-bridge %s", $vlanConfig->{VLANDEF}{ID},
		    Capability::ReleaseCompare($releaseId, Capability::ConvertRelease(3, 4),
					       REGEXP, 0) > 0 ? "secure-forwarding" : ""); # secure-forwarding for Enhanced I-brdige (only from 3.5 onwards)

  $cmd .= " protocol-filter pass-ipoe" if $params->{PASSIPOE};
  return $cmd;
}

#------------------------------------------------------------------------------
# Generate VLAN creation rollback command to restore the VLAN in its
# original state.
# Part 1: Add the options not available at creation time
#------------------------------------------------------------------------------
sub CreateVLANpart2 {
  my ($vlanConfig, $dhcpAgentConfig, $releaseId, $newAge) = @_;
  my $cmd = sprintf("configure vlan id %s", $vlanConfig->{VLANDEF}{ID});

  $cmd .= " dhcp-opt-82" if (defined($dhcpAgentConfig->{OPT82}));
  $cmd .= " circuit-id-dhcp $optionTranslation{DHCP}{CIRCID}{$dhcpAgentConfig->{CIRCID}}" if (defined($dhcpAgentConfig->{CIRCID}));
  $cmd .= " remote-id-dhcp $optionTranslation{DHCP}{REMID}{$dhcpAgentConfig->{REMID}}" if (defined($dhcpAgentConfig->{REMID}));
  $cmd .= " dhcp-linerate $dhcpAgentConfig->{LINERATE}" if (defined($dhcpAgentConfig->{LINERATE}));
  $cmd .= " pppoe-linerate $vlanConfig->{VLANDEF}{PPPOELINERATE}" if (defined($vlanConfig->{VLANDEF}{PPPOELINERATE}));
  $cmd .= " circuit-id-pppoe $vlanConfig->{VLANDEF}{CIRCIDPPPOE}" if (defined($vlanConfig->{VLANDEF}{CIRCIDPPPOE}));
  $cmd .= " remote-id-pppoe $vlanConfig->{VLANDEF}{REMIDPPPOE}" if (defined($vlanConfig->{VLANDEF}{REMIDPPPOE}));
  $cmd .= " secure-forwarding" if (defined($vlanConfig->{VLANDEF}{SECFWD}));
  if (defined($newAge)) {
    $cmd .= " aging-time $newAge";
  } else {
    $cmd .= " aging-time $vlanConfig->{VLANDEF}{AGING}" if (defined($vlanConfig->{VLANDEF}{AGING}));
  }
  $cmd .= " vmac-translation" if (defined($vlanConfig->{VLANDEF}{VMACTRANS}));
  $cmd .= " vmac-dnstr-filter" if (defined($vlanConfig->{VLANDEF}{VMACDNFLT}));
  return ($cmd ne sprintf("configure vlan id %s", $vlanConfig->{VLANDEF}{ID}) ? $cmd : "");
}

sub CreateQoSSessionProfilePart1 {
  my ($qosSessionProfiles, $profName, $releaseId) = @_;
  my $cmd = "configure qos profiles session $qosSessionProfiles->{$profName}{NEWNAME} logical-flow-type generic";

  $cmd .= " up-policer $qosSessionProfiles->{$profName}{UPPOLICER}" if (defined($qosSessionProfiles->{$profName}{UPPOLICER}));
  $cmd .= " down-policer $qosSessionProfiles->{$profName}{DNPOLICER}" if (defined($qosSessionProfiles->{$profName}{DNPOLICER}));
  $cmd .= " up-marker $qosSessionProfiles->{$profName}{UPMARKER}" if (defined($qosSessionProfiles->{$profName}{UPMARKER}));
  return $cmd;
}

sub CreateQoSSessionProfilePart2 {
  my ($qosSessionProfiles, $profName, $releaseId) = @_;

  return (defined($qosSessionProfiles->{$profName}{UPPOLICY}) ?
	  "configure qos profiles session $qosSessionProfiles->{$profName}{NEWNAME} up-policy $qosSessionProfiles->{$profName}{UPPOLICY}" :
	  "");
}

sub CreateQoSSessionProfilePart3 {
  my ($qosSessionProfiles, $profName, $releaseId) = @_;

  return (defined($qosSessionProfiles->{$profName}{DNPOLICY}) ?
	  "configure qos profiles session $qosSessionProfiles->{$profName}{NEWNAME} down-policy $qosSessionProfiles->{$profName}{DNPOLICY}" :
	  "");
}
#------------------------------------------------------------------------------
# Prepare creation of I-bridge:
# 1. Create (enhanced) I-bridge (re-creation of VLAN)
# 2. Create new QoS session profiles for non generic profiles 
# 3. Configure bridge port for atm-port user interfaces
# 4. attach all bridge ports to the i-bridge
# 5. Set PVID for bridge ports without VLAN before reconfiguration
# 6. attach all vlan-ports (if any) to the i-bridge
# 7. configure static routes of involded user interfaces
# 8. Recreate IGMP channels
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $vrfConfig: pointer to a hash of complete VRF structure.
# $vlanConfig: pointer to a hash of the complete VLAN structure.
# $dhcpAgentConfig: pointer to a hash of the complete DHCP relay-agent structure.
# $bridgeConfig: pointer to a hash of the complete bridge port structure.
# $vlanPortConfig: pointer to a hash of the VLAN port configuration
# $igmpChConfig: pointer to a hash of the IGMP channel configuration
# $qosSessionProfileStorage: pointer to a hash of the QoS session profiles.
# $cliCommandStorage: pointer to an array holding all the reconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollkbac commands of the reconfiguration.
#------------------------------------------------------------------------------
sub PrepareCreation {
  my ($fh, $vrfConfig, $vlanConfig, $dhcpAgentConfig, $bridgeConfig, $vlanPortConfig, $igmpChConfig,
      $qosSessionProfileConfig, $cliCommandStorage, $rollBackCmds, $releaseId) = @_;
  my $cmd = "";
  my $ok = 1;
  my $rollBackCmd = "";

  # 1 Create (enhanced) I-bridge (re-creation of VLAN)
  ToolCommon::Log($fh, "[PrepareCreation] $releaseId\n") if ($params->{DEBUG});
  $cmd = CreateVLANpart1($vlanConfig, $releaseId);
  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
  push @{$cliCommandStorage}, $cmd;
  $rollBackCmd = "configure vlan no id $vlanConfig->{VLANDEF}{ID}";
  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
  ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
  $cmd = CreateVLANpart2($vlanConfig, $dhcpAgentConfig, $releaseId, $params->{AGEING});
  push @{$cliCommandStorage}, $cmd if ($cmd ne "");
  # 2. Create Generic QoS session profiles (if needed)
  foreach my $profName (keys %{$qosSessionProfileConfig}) {
    if (defined($qosSessionProfileConfig->{$profName}{NEWNAME})) {
      $cmd = CreateQoSSessionProfilePart1($qosSessionProfileConfig, $profName, $releaseId);
      push @{$cliCommandStorage}, $cmd;
      push @{$rollBackCmds->{$cmd}}, "configure qos profiles session no $qosSessionProfileConfig->{$profName}{NEWNAME}";
      $cmd = CreateQoSSessionProfilePart2($qosSessionProfileConfig, $profName, $releaseId);
      push @{$cliCommandStorage}, $cmd if ($cmd ne "");
      $cmd = CreateQoSSessionProfilePart3($qosSessionProfileConfig, $profName, $releaseId);
      push @{$cliCommandStorage}, $cmd if ($cmd ne "");
    }
  }
  # 3. Configure bridge port for atm-port user interfaces
  foreach my $port (keys %{$vrfConfig->{USRITF}}) {
    if (defined($vrfConfig->{USRITF}{$port}{ATM})) {
      if ($port =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) {
	my $pvc = $1;
	$cmd = "configure bridge port $pvc";
	push @{$rollBackCmds->{$cmd}}, "configure bridge no port $pvc";
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	push  @{$cliCommandStorage}, $cmd;
	# No bridge defined for ATM ports, store it for later usage
	$bridgeConfig->{$pvc}{VLANID} = $vlanConfig->{VLANDEF}{ID};
	$vrfConfig->{USRITF}{$port}{BRIDGE} = 1;
      } else {
	FamilyCommon::SetAndLogError($fh, "port $port not according to expected syntax");
	$ok = 0;
      }
    }
    if (defined($params->{MAXUNICASTMAC})) {
      if (($port =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) || ($port =~ (/(\d+\/\d+\/\d+\/\d+)/))) {
	$cmd = "configure bridge port $1 max-unicast-mac $params->{MAXUNICASTMAC}";
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	push  @{$cliCommandStorage}, $cmd;
      } else {
	FamilyCommon::SetAndLogError($fh, "port $port not according to expected syntax (max-unicast-mac)");
	$ok = 0;
      }
    }
  }
  # 4. attach all bridge ports to the i-bridge
  foreach my $port (keys %{$vrfConfig->{USRITF}}) {
    if (defined($vrfConfig->{USRITF}{$port}{BRIDGE})) {
      if (($port =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) || ($port =~ (/(\d+\/\d+\/\d+\/\d+)/))) {
	my $pvc = $1;
	if (defined($bridgeConfig->{$pvc}{VLANID})) {
	  if ($bridgeConfig->{$pvc}{VLANID} != $vlanConfig->{VLANDEF}{ID}) {
	    $cmd = sprintf("configure bridge port %s vlan-id %d network-vlan %d vlan-scope local",
			   $pvc, $bridgeConfig->{$pvc}{VLANID}, $vlanConfig->{VLANDEF}{ID});
	  } else {
	    $cmd = sprintf("configure bridge port %s vlan-id %d",
			   $pvc, $vlanConfig->{VLANDEF}{ID});
	  }
	} else {
	  $cmd = sprintf("configure bridge port %s vlan-id %d",
			 $pvc, $vlanConfig->{VLANDEF}{ID});
	}
	if (defined($vrfConfig->{USRITF}{$port}{QOS})) {
	  if (defined($qosSessionProfileConfig->{$vrfConfig->{USRITF}{$port}{QOS}}{NEWNAME})) {
	    $cmd .= " qos-profile name:$qosSessionProfileConfig->{$vrfConfig->{USRITF}{$port}{QOS}}{NEWNAME}";
	  } else {
	    $cmd .= " qos-profile name:$vrfConfig->{USRITF}{$port}{QOS}";
	  }
	}
	my $rollbackCmd;
	if (! defined($vrfConfig->{USRITF}{$port}{ATM})) {
	  if (defined($bridgeConfig->{$pvc}{VLANID})) {
	    $rollbackCmd = sprintf("configure bridge port %s vlan-id %d no network-vlan", $pvc, $bridgeConfig->{$pvc}{VLANID});
	  } else {
	    $rollbackCmd = sprintf("configure bridge port %s no vlan-id %d", $pvc, $vlanConfig->{VLANDEF}{ID});
	  }
	}
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	push @{$cliCommandStorage}, $cmd;
	push @{$rollBackCmds->{$cmd}}, $rollbackCmd;
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "port $port not according to expected syntax");
	$ok = 0;
      }
    } elsif (defined($vrfConfig->{USRITF}{$port}{VLAN})) {
      if (($port =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+):(\d+)/) || ($port =~ /(\d+\/\d+\/\d+\/\d+):(\d+)/)) {
	my $pvc = $1;
	my $vlanId = $2;
	$cmd = sprintf("configure bridge port %s vlan-id %d network-vlan %d vlan-scope local", $pvc, $vlanId, $vlanConfig->{VLANDEF}{ID});
	push @{$rollBackCmds->{$cmd}}, sprintf("configure bridge port %s vlan-id %d no network-vlan", $pvc, $vlanId);
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	push  @{$cliCommandStorage}, $cmd;
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "port $port not according to expected syntax");
	$ok = 0;
      }
    }
  }
  # 5. Set PVID for bridge ports without VLAN before reconfiguration
  foreach my $port (keys %{$vrfConfig->{USRITF}}) {
    if (defined($vrfConfig->{USRITF}{$port}{BRIDGE})) {
      if (($port =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) || ($port =~ /(\d+\/\d+\/\d+\/\d+)/)) {
	my $pvc = $1;
	if (! defined($bridgeConfig->{$pvc}{VLANID})) {
	  $cmd = sprintf("configure bridge port %s pvid %d", $pvc, $vlanConfig->{VLANDEF}{ID});
	  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	  push @{$cliCommandStorage}, $cmd;
	  push @{$rollBackCmds->{$cmd}}, sprintf("configure bridge port %s no pvid", $pvc);
	}
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "port $port not according to expected syntax");
	$ok = 0;
      }
    }
    # Not for VLAN ports
  }

  # 6. Attach vlan-ports to EIB
  # 7. configure static routes of involded user interfaces
  foreach my $ipAddress (keys %{$vrfConfig->{ROUTEDEST}}) {
    my $nextHop = $vrfConfig->{ROUTEDEST}{$ipAddress}{NEXTHOP};
    if (index($nextHop, "direct:user") != -1) {
      # Either user-port or user-bridgeport
      if (($nextHop =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) || ($nextHop =~ /(\d+\/\d+\/\d+\/\d+)/)) {
	my $pvc = $1;
	if (defined($bridgeConfig->{$pvc}{VLANID})) {
	  $cmd = sprintf("configure bridge port %s vlan-id %d static-user ip-address %s", $pvc, $bridgeConfig->{$pvc}{VLANID}, $ipAddress);
	  $rollBackCmd = sprintf("configure bridge port %s vlan-id %d static-user no ip-address %s",
				 $pvc, $bridgeConfig->{$pvc}{VLANID}, $ipAddress);
	  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	  push  @{$cliCommandStorage}, $cmd;
	} else {
	  $cmd = sprintf("configure bridge port %s vlan-id %d static-user ip-address %s", $pvc, $vlanConfig->{VLANDEF}{ID}, $ipAddress);
	  $rollBackCmd = sprintf("configure bridge port %s vlan-id %d static-user no ip-address %s",
				 $pvc, $vlanConfig->{VLANDEF}{ID}, $ipAddress);
	  push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	  push  @{$cliCommandStorage}, $cmd;
	}
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "port $nextHop not according to expected syntax");
	$ok = 0;
      }
    } elsif (index($nextHop, "direct:vlan") != -1) {
      # VLAN ports
      if (($nextHop =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+):(\d+)/) || ($nextHop =~ /(\d+\/\d+\/\d+\/\d+):(\d+)/)) {
	my $pvc = $1;
	my $vlanId = $2;
	$cmd = sprintf("configure bridge port %s vlan-id %d static-user ip-address %s", $pvc, $vlanId, $ipAddress);
	$rollBackCmd = sprintf("configure bridge port %s vlan-id %d static-user no ip-address %s",
			       $pvc, $vlanId, $ipAddress);
	push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	push  @{$cliCommandStorage}, $cmd;
      } else {
	# Should not happen...
	FamilyCommon::SetAndLogError($fh, "port $nextHop not according to expected syntax");
	$ok = 0;
      }
    }
  }
  # 8. Recreate IGMP channels
  foreach my $port (keys %{$igmpChConfig}) {
    my ($portId, $vlanId);
    if (($port =~ /bridge\-port:(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) ||
	($port =~ /bridge\-port:(\d+\/\d+\/\d+\/\d+)/)) {
      $portId = $1, $vlanId = undef;
    } elsif (($port =~ /vlan\-port:(\d+\/\d+\/\d+\/\d+:\d+:\d+):(\d+)/) ||
	     ($port =~ /vlan\-port:(\d+\/\d+\/\d+\/\d+):(\d+)/)) {
      $portId = $1, $vlanId = $2;
    } else {
      $portId = undef, $vlanId = undef;
      # ATM ports -> skip: do not have IGMP
    }
    if (defined($portId)) {
      if (defined($igmpChConfig->{$port}{MAXNUMGRP})) {
	$cmd = CreateIgmpChPart1($igmpChConfig, $portId, $vlanId, $port, $releaseId);
	$rollBackCmd = defined($vlanId) ? "configure igmp no channel vlan:$portId:$vlanId" : "configure igmp no channel $portId";
	push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	ToolCommon::Log($fh, "  Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
	push @{$cliCommandStorage}, $cmd;
	$cmd = CreateIgmpChPart2($igmpChConfig, $portId, $vlanId, $port, $releaseId);
	if ($cmd ne "") {
	  push @{$cliCommandStorage}, $cmd;
	  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	}
	$cmd = CreateIgmpChPart3($igmpChConfig, $portId, $vlanId, $port, $releaseId);
	if ($cmd ne "") {
	  push @{$cliCommandStorage}, $cmd;
	  ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
	}
      }
    }
  }
  return $ok;
}


#------------------------------------------------------------------------------
#  Backup system database for safety (may be needed if reconfiguration failed
# dramatically and CLI restoration failed.
#------------------------------------------------------------------------------
sub BackupDatabase {
  my ($fh, $cli, $isam, $debug, $parentFh, $logfileName, $count) = @_;
  my $time1 = time();
  my $oswpInfo;
  my $ok = 1;
  my $tftpIpAddress = "";
  my $fileName = "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
  my $cmd = "$config{RM} $fileName > ipAwrm.txt 2>&1;$config{TOUCH} $fileName";
  my $sysret = system($cmd);

  ToolCommon::Log($fh, "\nCMD = $cmd\n") if ($debug);
  if ($sysret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($sysret);
    FamilyCommon::SetAndLogError($fh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** $cmd FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "Touching $fileName OK\n") if $debug;
  }
  if ($params->{UMASK_CHMOD} and ! chmod(0666, "$fileName")) {
    FamilyCommon::SetAndLogError($fh, "chmod(666,$fileName) FAILED: $!", "  FAILED: $!\n");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** chmod(666,$fileName) FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  }
  if ($ok) {
    $oswpInfo = IsamCli::GetOswpInfo($cli, REGEXP);
    if (!defined($oswpInfo)) {
      FamilyCommon::SetAndLogError($fh, sprintf("Failed to retrieve OSWP info: %s", IsamCli::GetError()));
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Active OSWP: $oswpInfo->{ACTIVE}{INDEX}\n") if $debug;
      my @response = IsamCli::SendCommand($cli, "info configure software-mngt oswp flat", undef);
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::SetAndLogError($fh, sprintf("Retrieving oswp configuration failed: %s", IsamCli::GetError()));
	$ok = 0;
      } else {
	foreach my $line (@response) {
	  chomp $line;
	  ToolCommon::Log($fh, "Line read: $line\n") if $debug;
	  if ($line =~ /configure\s+software\-mngt\s+oswp\s+(\d+)\s+primary\-file\-server\-id\s+([^\s]+)/) {
	    my $oswpNum = $1;
	    my $ipAddress = $2;
	    if ($oswpInfo->{ACTIVE}{INDEX} == $oswpNum) {
	      $tftpIpAddress = $ipAddress;
	      last;
	    }
	  } else {
	    if (! ($line =~ /\#[-]+$/) &&
		! ($line =~ /[\$%\#>]\s*$/)) {
	      # Should not happen
	      FamilyCommon::SetAndLogError($fh, "OSWP config info not according to expected syntax: $line");
	      $ok = 0;
	      last;
	    }
	  }
	}
	if (($tftpIpAddress ne "") && ($tftpIpAddress ne "0.0.0.0")) {
	  my $ret = IsamCli::UploadDb($cli, 'actual-active',
				      $tftpIpAddress, "$TFTPWORKDIR/$isam", 1);
	  if (!defined($ret) || !$ret)  {
	    FamilyCommon::SetAndLogError($fh, sprintf("  FAILED (%s)\n", IsamCli::GetError()));
	    $ok = 0;
	  } else {
	    my $dbSize = -s "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
	    if ($dbSize == 0) {
	      FamilyCommon::SetAndLogError($fh, sprintf("  Size of database file 0\n"));
	      $ok = 0;
	    } else {
	      ToolCommon::Log($fh, sprintf("  OK, transferred %d bytes in %d seconds\n", $dbSize, time() - $time1));
	      ToolCommon::Log($fh, "Database backup stored as $fileName\n", 1);
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, "No TFTP IP address could be obtained\n");
	  $ok = 0;
	}
      }
    }
  }
  my @response = IsamCli::SendCommand($cli, "exit all");  # Set prompt to top node
  return $ok;
}

#------------------------------------------------------------------------------
# Rollback configuration in case of error (or forced as command option)
#------------------------------------------------------------------------------
sub RollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;
  my @response;

  if ($params->{ROLLBACK}) {
    ToolCommon::Log($fh, "==================================\n", 1);
    ToolCommon::Log($fh, "The following will be rolled back:\n", 1);
    ToolCommon::Log($fh, "==================================\n", 1);
  } else {
    ToolCommon::Log($fh, "Start rollback...\n", 1);
  }
  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "Rolling back '$rollBackCmd'...\n") if $params->{DEBUG};
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	if ($params->{ROLLBACK}) {
	  ToolCommon::Log($fh, "  '$cmd'\n", 1);
	} else {
	  ToolCommon::Log($fh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating" : "Sending", $cmd), 1);
	}
	if (! $params->{NOACTION}) {
	  sleep $params->{SLEEP} if ($cmd =~ /configure vlan no id/);  # give deletion of network itf a bit of time
	  @response = IsamCli::SendCommand($cli, $cmd);
	  return 0 if (IsamCli::CommandError(join("", @response)));
	}
      }
    }
  }
  ToolCommon::Log($fh, "  Done\n");
  return 1;
}

#------------------------------------------------------------------------------
# Debugging: dump the content of the rollback command buffer.
#------------------------------------------------------------------------------
sub DumpRollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;

  ToolCommon::Log($fh, "==================================\n", 1);
  ToolCommon::Log($fh, "CLI commands in case of roll back:\n", 1);
  ToolCommon::Log($fh, "==================================\n", 1);

  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "Rolling back '$rollBackCmd'...\n");
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	ToolCommon::Log($fh, sprintf("  Simulating '%s'\n", $cmd), 1);
      }
    }
  }
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------
sub HandleIsams {
  my ($fh, $isams, $count, $config) = @_;
  my $isam;
  my ($time1, $time2);
  my $startTime;
  my $nodeLogFh;
  my $nodeLogFile;
  my $nodeStatusLog;
  my $cli = undef;
  my %vrfConfig = ();
  my %qosSessionProfileStorage = ();
  my %pppoeSessions = ();
  my %vlanConfig = ();
  my %dhcpRelayAgentConfig = ();
  my %bridgeConfig = ();
  my %vlanPortConfig = ();
  my %vlanBindConfig = ();
  my %igmpChConfig = ();
  my $releaseNum;
  my @configToDelete;
  my @configToCreate;
  my @configRetrieved = ();
  my %rollBackCommandConfig = ();
  my @rollBackCommands = ();
  my $reconfigStart;
  my $cliCommandsSent;
  my $ret;

  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetParentLogFile($fh);
  FamilyCommon::SetChildNum($count);
  my $newLog = "$params->{LOGFILE}_$count";
  my $newfh;
  if (! open($newfh, ">$newLog")) {
    ToolCommon::Log($fh, "Child $count: cannot open $newLog for writing: $!\n", 1);
    CORE::exit(CREATE_FILE_FAIL_CODE)
  }
  $newfh->autoflush(1);
  ToolCommon::Log($fh, "Child $count: Logging continuous in $newLog...\n", 1);

  PrintOptionsSummary($newfh, undef);

  foreach $isam (@{$isams}) {
    FamilyCommon::SetFailedNode($isam->[1]);
    # Switch debugging (if any) to an opened filehandle...
    Utils::SetDebug($newfh) if ($params->{DEBUG});
    IsamCli::SetDebug($newfh)  if ($params->{DEBUG});
    Capability::SetDebug($newfh) if ($params->{DEBUG});
    ToolCommon::SetDebug($newfh);

    ToolCommon::Log($newfh, "#===================================================\n");
    ToolCommon::Log($newfh, "#== $isam->[1]\n");
    ToolCommon::Log($newfh, "#===================================================\n");
    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Started $isam->[1]...\n",
		    $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second), 1);

    my $dir = "$TFTPDIR/$TFTPWORKDIR/$isam->[1]";
    $dir =~ s/\/\//\//g;
    if ( ! -d $dir) {
      ToolCommon::Log($newfh, "Creating working dir $dir...");
      if (!mkdir( $dir, 0777)) {
	ToolCommon::Log($newfh, "  FAILED\n");
	ToolCommon::Log($fh, "Child $count: **FATAL** Creating working dir $dir for $isam->[1] failed, please verify $newLog.\n", 1);
	CORE::exit(CREATE_DIR_FAILED_CODE);
      }
      ToolCommon::Log($newfh, "  OK\n");
    }
    if (! chmod(01777, "$dir")) {
       ToolCommon::Log($fh,  "Failed to chmod 1777 for directory '$dir'\n",1);
    }

    if ($params->{SEPLOG}) {
      $nodeLogFile = sprintf("$dir/commit_%04d-%02d-%02d_%02d:%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
      if (! open($nodeLogFh, ">$nodeLogFile")) {  # -- 1.2.5 START
	ToolCommon::Log($fh, "Child $count: cannot open $nodeLogFile for writing: $!\n", 1);
	CORE::exit(CREATE_FILE_FAIL_CODE)
      } else {
	ToolCommon::Log($fh, "Child $count: NE specific logging to $nodeLogFile...\n", 1);
      }
      $nodeLogFh->autoflush(1);
      Utils::SetDebug($nodeLogFh) if ($params->{DEBUG});
      IsamCli::SetDebug($nodeLogFh)  if ($params->{DEBUG});
      Capability::SetDebug($nodeLogFh) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFh);
    } else {
      $nodeLogFh = $newfh;
    }
    FamilyCommon::SetWarnLogFile($nodeLogFh);
    FamilyCommon::SetChildLogFile($nodeLogFh);

    if (! open($nodeStatusLog, ">$dir/$STATUSFILE")) {
      ToolCommon::Log($fh, "Child $count: cannot create $dir/$STATUSFILE for $isam->[1]: $!\n", 1);
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE);
    }
    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    ### Is Isam reachable through Ping command
    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }

    $params->{SNMP}{V1}{NT_COMMUNITY} = $NT_COMMUNITY;
    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
        FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError(), undef );
        next;
    }
    my $mibVersion = SNMPComm::GetValue('MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         next if( FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError(), undef ) );
    }
      if (($mibVersion =~ /3FE21961EAAA_E(\d+)\.(\d+)\.\d+\.\d+/)      ||  # ETSI
          ($mibVersion =~ /3FE21961EAAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/) ||  # ETSI  (ISAM MIB label Extn)
	  ($mibVersion =~ /3FE21961ANAA_E(\d+)\.(\d+)\.\d+\.\d+/)      ||  # ANSI
	  ($mibVersion =~ /3FE21961ANAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/))  {  # ANSI (ISAM MIB label Extn)
	my $major = $1; my $minor = $2; 
	$releaseNum = Capability::ConvertRelease($major, $minor);
	if (Capability::ReleaseCompare($releaseNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
	  FamilyCommon::IncrErrorCount(sprintf("  ERROR: Need at least R3.5 to do this reconfiguration (current release : %s)\n",
					       Capability::NtDbVersionInfo(Capability::IsamFamily(), $releaseNum)),
				       EC_RELEASE_TOO_LOW,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      } else {
	FamilyCommon::IncrErrorCount("  ERROR: unsupported MIB version ($mibVersion)\n",
				     EC_INV_MIB_VERSION,
				     "($mibVersion)",
				     1);
	next;
      }

    ToolCommon::Log($nodeLogFh, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
				   EC_GET_ACC_BITS_FAILED,
				   undef,
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
      # Check whether this IP address is found in the know hosts file(s).
      if (! ToolCommon::SshPossible($isam->[0], $params->{DEBUG}, $config)) {
	FamilyCommon::IncrErrorCount(sprintf("No SSH access possible by PBMT: %s", ToolCommon::GetError()),
				     EC_NO_SSH_POSS,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	next;
      } else {
	$params->{USESSH} = 1;
	ToolCommon::Log($nodeLogFh, "No CLI access via Telnet possible, forced usage of SSH\n", 1);
      }
    }

    #-- Open CLI Session
    $cli = ToolCommon::OpenCli($isam, $nodeLogFh, $params->{TELNETTIMEOUT});
    if (!defined($cli)) {
      FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for $isam->[1]: %s\n", IsamCli::GetError()),
				   EC_OPENCLI_FAILED,
				   sprintf(": %s", IsamCli::GetError()), 1);
      next;
    }
    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount

    my $prev = $cli->max_buffer_length($params->{BUFSIZE});
    ToolCommon::Log($nodeLogFh, sprintf("Increased input buffer length from %d to %d bytes\n", $prev, $params->{BUFSIZE}));
    
    if (!IsamCli::SetSessionTimeout($cli, 120)) {
      FamilyCommon::IncrErrorCount(sprintf("  Set session timeout FAILED: %s\n", IsamCli::GetError()),
				   EC_SET_TIMEOUT_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    ToolCommon::Log($nodeLogFh, "Backup NT Database...");
    if (! BackupDatabase($nodeLogFh, $cli, $isam->[1], $params->{DEBUG}, FamilyCommon::GetParentLogFile(), $newLog, $count)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_BACKUPDB_FAILED,
				   undef,
				   1);
      next;
    }

    @configRetrieved = ();

    %vrfConfig = ();
    %qosSessionProfileStorage = ();
    ToolCommon::Log($nodeLogFh, "Retrieve VRF $params->{NWVRFID} info...");
    my @response = IsamCli::SendCommand($cli, "info configure ip vrf $params->{NWVRFID} flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") != -1) {
	  FamilyCommon::IncrErrorCount("  FAILED: VRF does not exist\n",
				       EC_NO_SUCH_VRF,
				       "($params->{NWVRFID})",
				       1);
	  next;
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_VRF_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	  ToolCommon::Log($nodeLogFh, "Parsing VRF configuration...");
	if ( !ParseVrfConfig($nodeLogFh, \@response, \@configRetrieved, \%vrfConfig, \%qosSessionProfileStorage, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_VRF_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    ToolCommon::Log($nodeLogFh, "Retrieve Qos session profile info...");
    if (! GetQosSessionProfiles($nodeLogFh, $cli, \@configRetrieved, \%qosSessionProfileStorage, $releaseNum)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				   EC_INFO_CONF_QOS_ERROR,
				   undef,
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    my @vlans = keys (%{$vrfConfig{NWITF}});
    ToolCommon::Log($nodeLogFh, "Retrieve VLAN (NT) $vlans[0] info...");
    @response = IsamCli::SendCommand($cli, "info configure vlan id $vlans[0] flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") != -1) {
	  FamilyCommon::IncrErrorCount("  FAILED: VLAN does not exist\n",
				       EC_NO_SUCH_VLAN,
				       "($vlans[0])",
				       1);
	  next;
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_VLAN_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	%vlanConfig = ();
	ToolCommon::Log($nodeLogFh, "Parsing VLAN configuration...");
	if ( !ParseVlanConfig($nodeLogFh, \@response, \@configRetrieved, \%vlanConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_VLAN_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }
    %pppoeSessions = ();
    if (Capability::ReleaseCompare($releaseNum, Capability::ConvertRelease(3, 7), REGEXP, 0) < 0) {
      ToolCommon::Log($nodeLogFh, "Retrieve pppoe session info...");
      @response = IsamCli::SendCommand($cli, "show pppoe session");
      if (defined($response[0])) {
	if (IsamCli::CommandError(join("", @response))) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_SHOW_PPPOESESS_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	  ToolCommon::Log($nodeLogFh, "Parsing pppoe session info...");
	  if ( !ParsePppoeSessions($nodeLogFh, \@response, \@configRetrieved, \%pppoeSessions, $releaseNum)) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
					 EC_PPPOESESS_PARSE_ERROR,
					 undef,
					 1);
	    next;
	  } else {
	    ToolCommon::Log($nodeLogFh, "  OK\n");
	  }
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				     EC_EMPTY_RESPONSE,
				     undef,
				     1);
	next;
      }
    }

    %dhcpRelayAgentConfig = ();
    ToolCommon::Log($nodeLogFh, "Retrieve DHCP relay-agent for $params->{NWVRFID} info...");
    @response = IsamCli::SendCommand($cli, "info configure dhcp-relay agent $params->{NWVRFID} flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") == -1) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_DHCPRELAY_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	ToolCommon::Log($nodeLogFh, "Parsing DHCP relay-agent configuration...");
	if ( !ParseDhcpRelayAgentConfig($nodeLogFh, \@response, \@configRetrieved, \%dhcpRelayAgentConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_DHCPRELAY_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    %vlanBindConfig = ();
    ToolCommon::Log($nodeLogFh, "Retrieve vlan-bind info...");
    @response = IsamCli::SendCommand($cli, "info configure ip vlan-bind flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") == -1) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_VLANBIND_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	ToolCommon::Log($nodeLogFh, "Parsing vlan-bind configuration...");
	if ( !ParseVlanBindConfig($nodeLogFh, \@response, \@configRetrieved, \%vlanBindConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_VLANBIND_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    %bridgeConfig = ();
    %vlanPortConfig = ();
    ToolCommon::Log($nodeLogFh, "Retrieving bridge configuration of involved user-itf ports...");
    if (!GetBridgeConfig($nodeLogFh, $cli, \%vrfConfig, \@configRetrieved, \%bridgeConfig, \%vlanPortConfig, $releaseNum)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_GET_BRIDGEINFO_ERROR,
				   undef,
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK!\n");
    }

    %igmpChConfig = ();
    ToolCommon::Log($nodeLogFh, "Retrieving bridge configuration of involved user-itf ports...");
    if (!GetIgmpChlConfig($nodeLogFh, $cli, \%vrfConfig, \@configRetrieved, \%igmpChConfig, $releaseNum)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_GET_IGMPCH_INFO_ERROR,
				   undef,
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK!\n");
    }

    ToolCommon::Log($nodeLogFh, "Checking configuration to be migrated...\n");
    if (ConfigValid($nodeLogFh, \%vrfConfig, \%vlanConfig, \%bridgeConfig, \%vlanPortConfig,
		    \%vlanBindConfig, \%pppoeSessions, $releaseNum)) {
      @configToDelete = ();
      %rollBackCommandConfig = ();
      @configToCreate = ();
      PrepareDeletion($nodeLogFh, $cli, \%vrfConfig, \%vlanConfig, \%igmpChConfig, \%dhcpRelayAgentConfig,
		      \@configToDelete, \%rollBackCommandConfig, $releaseNum);
      if ($params->{CONFINFO}) {
	ToolCommon::Log($nodeLogFh, "====================================================================\n", 1);
	ToolCommon::Log($nodeLogFh, "The following configuration info has been read, parsed and stored...\n", 1);
	ToolCommon::Log($nodeLogFh, "====================================================================\n", 1);
	foreach (@configRetrieved) {
	  ToolCommon::Log($nodeLogFh, "  $_\n", 1);
	}
      }
      PrepareCreation($nodeLogFh, \%vrfConfig, \%vlanConfig, \%dhcpRelayAgentConfig, \%bridgeConfig, \%vlanPortConfig, \%igmpChConfig,
		      \%qosSessionProfileStorage, \@configToCreate, \%rollBackCommandConfig, $releaseNum);
      if ($params->{CONFINFO}) {
	ToolCommon::Log($nodeLogFh, "==============================\n", 1);
	ToolCommon::Log($nodeLogFh, "The following will be deleted:\n", 1);
	ToolCommon::Log($nodeLogFh, "==============================\n", 1);
	foreach (@configToDelete) {
	  ToolCommon::Log($nodeLogFh, "  $_\n", 1);
	}
	ToolCommon::Log($nodeLogFh, "==============================\n", 1);
	ToolCommon::Log($nodeLogFh, "The following will be created:\n", 1);
	ToolCommon::Log($nodeLogFh, "==============================\n", 1);
	foreach (@configToCreate) {
	  ToolCommon::Log($nodeLogFh, "  $_\n", 1);
	}
      }

      my $deletionOk = 1;
      my $creationOk = 1;
      @rollBackCommands = ();
      # Here we go....
    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership...\n");
    return undef if( ToolCommon::AcquireTransaction() );
    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership... DONE\n");

      ToolCommon::Log($nodeLogFh, "Start deletion of resources...") if (! $params->{NOACTION});
      $reconfigStart = time();
      $cliCommandsSent = 0;
      foreach my $cliCmd (@configToDelete) {
	ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd)) if $params->{DEBUG};
	if (! $params->{NOACTION}) {
	  sleep $params->{SLEEP} if ($cliCmd =~ /configure vlan no id/);  # give deletion of network itf a bit of time
	  @response = IsamCli::SendCommand($cli, $cliCmd);
	  if (defined($response[0])) {
	    if (IsamCli::CommandError(join("", @response)) && ! ($cliCmd =~ /admin ip/)) {
	      $deletionOk = 0;
	      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					   EC_DEL_CONFIG_ERROR,
					   undef,
					   undef);
	      last;
	    } else {
	      $cliCommandsSent++;
	      if (! ($cliCmd =~ /admin ip/)) {
		push @rollBackCommands, $cliCmd;
		ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						    join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	      }
#	      sleep 2;
	    }
	  } else {
	    $deletionOk = 0;
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_DEL_CONFIG_ERROR,
					 undef,
					 undef);
	    if (IsamCli::ConnectionReset()) {
	      ToolCommon::Log($fh,
			      "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	      sleep 1;
	      CORE::exit(EXIT_WARN_CODE);
	    }
	    last;
	  }
	} else {
	  push @rollBackCommands, $cliCmd;
	  ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd'\ -> '%s'n",
					      join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	}
      }
      if (! $params->{NOACTION}) {
	if (! $deletionOk) {
	  RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig);
	  next;
	}
      }
      ToolCommon::Log($nodeLogFh, "  OK\n") if (! $params->{NOACTION});
      sleep 10;

      ToolCommon::Log($nodeLogFh, "Start creation of resources...") if (! $params->{NOACTION});
      foreach my $cliCmd (@configToCreate) {
	ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd)) if $params->{DEBUG};
	if (! $params->{NOACTION}) {
	  @response = IsamCli::SendCommand($cli, $cliCmd);
	  if (defined($response[0])) {
	    if (IsamCli::CommandError(join("", @response))) {
	      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					   EC_CREATE_CONFIG_ERROR,
					   undef,
					   undef);
	      $creationOk = 0;
	      last;
	    } else {
	      $cliCommandsSent++;
	      push @rollBackCommands, $cliCmd;
	      if (defined($rollBackCommandConfig{$cliCmd})) {
		ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						    join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	      }
#	      sleep 2;
	    }
	  } else {
	    $creationOk = 0;
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_CREATE_CONFIG_ERROR,
					 undef,
					 undef);
	    if (IsamCli::ConnectionReset()) {
	      ToolCommon::Log($fh,
			      "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	      sleep 1;
	      CORE::exit(EXIT_WARN_CODE);
	    }
	    last;
	  }
	} else {
	  push @rollBackCommands, $cliCmd;
	  ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
					      join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	}
      }
      if (! $creationOk) {
	if (! RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_ROLLBACK_ERROR,
				       undef,
				       1);
	}
	next;
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_INVALID_CONFIG,
				   undef,
				   1);
      next;
    }
    ToolCommon::Log($nodeLogFh, "  Done\n") if (! $params->{NOACTION});
    ToolCommon::Log($nodeLogFh, sprintf("%d CLI commands sent in %d seconds to accomplish this reconfiguration\n",
					$cliCommandsSent, time() - $reconfigStart));

    if ($params->{ROLLBACK}) {
      RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig);
    } else {
      DumpRollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)  if $params->{DEBUG};
    }

   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership... DONE");

    IsamCli::Disconnect($cli);
    $session->close();
    close $nodeLogFh if ($params->{SEPLOG});  # -- 1.2.5

    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Finished $isam->[1]... in %d seconds\n",
				 $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $endTime - $startTime), 1);
    $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
    if (defined($nodeStatusLog)) {
      print $nodeStatusLog
	sprintf("%s\n",
		FamilyCommon::GetErrorCount() ? ErrorCodes::ExitString(EXIT_WARN_CODE) : ErrorCodes::ExitString(SUCCESS_CODE));
      close $nodeStatusLog;
      FamilyCommon::CloseNodeStatusLog();
    }
  }
  if (FamilyCommon::GetErrorCount()) {
    ToolCommon::Log($fh, sprintf("Child %d: %d error(s) occured for the following ISAMs, please verify %s\n",
				 $count, FamilyCommon::GetErrorCount(),
				 $params->{SEPLOG} ? $nodeLogFile : "$params->{LOGFILE}_$count."), 1);
    my $failedNodes = FamilyCommon::GetFailedNodeList();
    foreach my $node (sort keys %$failedNodes) {
      ToolCommon::Log($fh, "  $node\n", 1);
    }
  }
  close $nodeLogFh if (! $params->{SEPLOG});
  ToolCommon::Log($fh, "Child $count: finished...\n", 1);
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}


#------------------------------------------------------------------------------
#-- Adjust Tool defaults if needed (i.e. when defined in the parameter config
#-- file
#------------------------------------------------------------------------------
sub AdjustToolVars {
  my $crypt = Crypt::Lite->new('debug' => 0);

  $HOSTSFILE = $config{'HOSTSFILE'} if defined ($config{'HOSTSFILE'});
  $TFTPDIR = $config{'TFTPDIR'} if defined($config{'TFTPDIR'});
  $TFTPWORKDIR = $config{'TFTPWORKDIR'} if defined($config{'TFTPWORKDIR'});
  $MAXSESSIONS = $config{'MAXTFTPSESSION'} if defined($config{'MAXTFTPSESSION'});
  $USERNAME = $crypt->decrypt($config{'DEFAULTCLIUSER'}, CYPHER) if (defined($config{'DEFAULTCLIUSER'}) &&
								     ($config{'DEFAULTCLIUSER'} ne ""));
  $PASSWORD = $crypt->decrypt($config{'DEFAULTCLIPWD'}, CYPHER) if (defined($config{'DEFAULTCLIPWD'}) &&
								    ($config{'DEFAULTCLIPWD'} ne ""));
  $NT_COMMUNITY = $crypt->decrypt($config{NT_COMMUNITY}, CYPHER) if (defined($config{NT_COMMUNITY}) &&
								     ($config{NT_COMMUNITY} ne ""));
  $NODEPREFIX = $config{'NODEPREFIX'} if defined($config{'NODEPREFIX'});
  $TELNETTIMEOUT = $config{'TELNETTIMEOUT'} if defined($config{TELNETTIMEOUT});
  $STATUSFILE = $config{IBRG_STATUS_FILE} if defined($config{IBRG_STATUS_FILE});
  $RETRIES = $config{RETRIES} if defined($config{RETRIES});
  $NO_PING = uc($config{'NO_PING'}) eq "N" ? 0 : 1 if defined($config{'NO_PING'});
}

#==============================================================================
#== Main program
#==============================================================================
my $count = 0;
my @pids;
my $pid;
my $warnCount = 0;

sub CleanUp {
  if (scalar @pids) {
    print STDERR sprintf("%s::%s\n", PBMT_VERSION, $VERSION);
    print STDERR "\nCaught Interupt (^C), aborting children...";
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	kill($pid);
      }
    }
  }
  exit(EXIT_SIGNAL_CODE);
}

$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }

  if (!($msgStr =~ /Zlib/)) {
    print $warnLogFile "Warning: $msgStr" if (defined($warnLogFile));
    print STDERR "Warning: $msgStr";
    $warnCount++;
  }
};

#-- Read parameter config file and adjust tool defaults if needed 
%config = Utils::ReadConfig('param.cfg');
AdjustToolVars();

#-- Handle command line
$params = ParseCommandLine();

STDOUT->autoflush(1);
my $logFile;
if (!open($logFile, ">$params->{LOGFILE}")) {
  print STDERR "Cannot open $params->{LOGFILE} for writing: $!\n";
  exit CREATE_FILE_FAIL_CODE;
}
$logFile->autoflush(1);
FamilyCommon::SetWarnLogFile($logFile);

#-- Print summary of options
PrintOptionsSummary($logFile, 1);

Utils::SetDebug($logFile) if ($params->{DEBUG});  # -- 1.2.2

#-- Create work dir
my $tmpDir = "$TFTPDIR/$TFTPWORKDIR";
if (! -d $tmpDir) {
  if (!mkdir( $tmpDir, 0777)) {
    ToolCommon::Log($logFile, "Cannot create directory $tmpDir: $!\n", 1);
    exit CREATE_DIR_FAILED_CODE;
  }
}
if (! chmod(01777, "$tmpDir")) {
    ToolCommon::Log($logFile,  "Failed to chmod 1777 for directory '$tmpDir'\n",1);
}

#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  my @addr = gethostbyname($params->{ISAM});
  my @ip = unpack( "C4", $addr[4]);
  my $ipAddr = join( '.', @ip );
  $isamsRef = [ [$ipAddr, $params->{ISAM}] ];
} else {
  ToolCommon::Log($logFile, "Reading Hostfile...\n", 1);
  $isamsRef = ToolCommon::ReadHostsFile($logFile, $params->{DEBUG}, $params->{HOSTS}, $NODEPREFIX);
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

while ($count < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $count) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      ToolCommon::Log($logFile, "Child $count: started as $pid\n", 1);
      push(@pids, $pid);
    } else {
      HandleIsams($logFile, \@isamPart, $count, \%config);
      exit SUCCESS_CODE;
    }
  }
  $count++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    $exitCode = $? >> 8;
    $signal = $? & 255;
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      ToolCommon::Log($logFile, sprintf("Child %d: exit code: '%d' (%s), signal: '%d'\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal), 1);
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  ToolCommon::Log($logFile, "All childs finished, exiting\n", 1);
  close $logFile;
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
