=head1 PBMT capability module

=begin html
This PERL module defines the capabilities of the PBMT tools in terms of:
<ul>
  <li>supported NT types</li>
  <li>known releases</li>
  <li>migration and upgrade restrictions</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         Capability.pm
#-- Date:         16.03.2006
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel 2006
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.2.0   - 2006/06/03 : First implementation
#--   1.2.3   - 2006/06/08 : ENTC-C NT SW file starts with MP1J i.s.o L5YY
#--   1.2.4   - 2006/06/14 : Added streams 243 and 245 to the capability model.
#--   1.2.5.1 - 2006/09/29 : Details for NANT and HYNT boards, ECNT-C 512MB
#--   1.2.5.2 - 2006/10/05 : ANTms16720 - support for R2.4.16
#--   1.4.0   - 2006/10/06 : ANTms17249: R3.1/R3.2 mainstream support
#--   1.4.2   - 2006/10/09 : ANTms17315: info about migration or upgrade
#--                          issues.
#--   1.4.3   - 2006/10/18 : Timestamp logging
#--   1.4.3.5 - 2006/11/15 : Prepare for R3.2
#--   1.4.3.6 - 2006/11/20 : Mainstream release now associated with R3.3
#--   1.5.0   - 2006/12/12 : - Add NT OFLMT capability checks
#--                          - R3.2 can be delivered as R3.2 (320 stream) or
#--                            R3.1.1 (311 stream)
#--   1.5.1   - 2006/12/19 : Call capability accesor to determine SHUB NT type for
#--                          offfline migration.
#--   1.6.0   - 2007/04/16 : Print stream related info
#--   1.7.0   - 2007/05/02 : ANTms45130 - associate R3.4 with mainstream.
#--   1.7.1   - 2007/05/22 : ANTms45130 - implementation incomplete
#--   1.7.2   - 2007/05/23 : ANTms45206 prepare for R3.5 and NANT-B
#--   1.7.3   - 2007/06/29 : Remove '\' after a comma.
#--   1.7.4   - 2007/08/10 : Added POD documentation
#--                          New NT type in R3.5: AGNT-A
#--   1.7.5   - 2007/10/23 : ANTms45206 SHUB config file was missing a parameter.
#--   1.7.6   - 2007/11/30 : ANTms74803: Problem introduced as reported in ANTms70072
#--                          introduces a new R3.3 and R3.4 DB with version B....
#--   1.7.7   - 2007/12/09 : First check whether migrations are excluded prior to any
#--                          other migration-possible check
#--   1.7.8   - 2007/12/13 : Release streams have now been fixed to R3.3.04 and R3.4.02
#--   1.7.9   - 2007/12/18 : ANTms77027.  NT OFLMT 3.4.01 converts 64 packages as member into
#--                                       1024 packages on the condition that the highest package
#--                                       member is 64.  As such the check in PBMT is adapted to
#--                                       accept this too.
#--   1.8.0   - 2007/12/18 : ISAM-V migration restrictions preparation
#--   1.8.1   - 2008/01/22 : accept streams 335 and 337 with baseline 334
#--   1.8.2   - 2008/03/13 : ANTms88054.  Accept stream 343..345
#--   1.8.3   - 2008/04/29 : ANTmt04599.  NANT-A TYPE-B SW name changed between R3.2 and R3.3
#--                                       ==> implement logic in TYPE-B SW naming (OSWP reduction).
#--   1.8.4   - 2008/05/23 : ANTmt08354.  Include release stream 338
#--   1.8.5   - 2008/07/23 : ANTmt17439.  AGNT-A is an ECNT-C board for SHUB OFLMT.
#--   1.8.6   - 2008/08/22 : ANTmt16940.  Include internal LT number to CLI mapping and LT MAC @ limitations.
#--   1.8.7   - 2008/10/17 : ANTmt16940.  Somce LTs have 4096 as limit for static MAC @.  This must be 1024.
#--   1.8.8   - 2008/08/23 : ANTmt44696.  R3.7.10
#--   1.8.9   - 2009/03/06 : ANTmt46670.  ANSI stream still identified as 'AA'
#--   1.8.10  - 2009/03/13 : ANTmt47687.  SHUB OFLMT V1.49
#--   1.8.11  - 2009/03/26 : ANTmt50138.  SHUB OFLMT V1.50
#--   1.8.12  - 2009/04/13 : ANTmt51710.  SHUB OFLMT V1.51
#--   1.8.13  - 2009/04/13 : ANTmt54321.  NT OFLMT V16.0.1 for R4.1
#--   1.8.14  - 2009/04/29 : ANTmt55717.  AGNT-A belongs to both ISAM streams so had to introduce ISAM_BOTH.
#--                          ANTmt56626.  SHUB OFLMT 1.52 (R4.0) - NT 15.0.2 (R4.0)
#--   1.8.15  - 2009/05/06 : ANTmt56869.  Accidently removed ECNT-C from the R4.0 stream whilew cleaning up test code.
#--   1.8.16  - 2009/05/07 : ANTmt57304.  New version of the IVPS OFLMT
#--                          ECNT-C is ISAM_ANSI family
#--   1.8.17  - 2009/05/25 : ANSI stream split off.  ANSI OSWP/SWP is not identified by 'AN'.
#--   1.8.18  - 2009/05/28 : ANTmt60512: NT DB Version for FGN4.1 is Nx.yyy rather then Ax.yyy
#--   1.8.19  - 2009/05/29 : ANTmt60738: no migration for R4.1 xVPS.
#--   1.8.20  - 2009/06/02 : ANTmt61239 - Added 1540LS NE family
#--   1.8.21  - 2009/06/11 : 7354RU NT type is RVSA-A and RVSA-B
#--   1.8.22  - 2009/06/26 : New NT OFLMT version for 4.1
#--                          ANTmt65332.  Introduction of the NTT ETSI family
#--             2009/07/09 : ANTmt67067.  Support for 1.53 and 1.54 of LANX OFLMT
#--   1.8.23  - 2009/07/13 : ANTmt65332
#--   1.8.24  - 2009/07/15 : New OFLMT for xVPS migration
#--   1.8.25  - 2009/07/24 : ANTmt71238: ECNT-E conversion changes
#--   1.8.26  - 2009/08/24 : MIG_EXCLUDED_TO is a hash.  Adjusted index-call in MigrationPossible.
#--   1.8.27  - 2009/09/02 : Highest NT OFLMT is 16.3.0 (R4.1)
#--   1.8.28  - 2009/09/15 : Support build 349
#--   1.8.29  - 2009/09/17 : ANTmt76101. Highest SHUB OFLMT is 1.55
#--   1.8.30  - 2009/09/25 : R4.1 DB version of xVPS is A4.100
#--   1.8.31  - 2009/09/28 : Error in IVPS DB version for some R3.4 MR.
#--   1.8.32  - 2009/11/19 : ANTmt84725 - support for FGN42
#--   			     ANTmt86148: 7354RU Changes and ANTmt83962 fix
#--   1.8.33  - 2010/03/04 : ANTmg01050
#--   1.8.34  - 2010/03/15 : x86 platform Changes
#--   1.8.35  - 2010/04/13 : ALU00257699 - support for LANX migration tool 1.58. 
#                            Support for NT OFLMT tool V17.0.0
#                            Support for Mini-NT (NRNT-A) & NANT-E board
#--   1.8.36  - 2010/06/02 : ALU00386912 (Corrected by R Janakiraman)
#--   3.0.00  - 2010/04/20 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01  - 2010/04/20 : Converged PBMT changes
#--   3.0.02  - 2010/09/15 : ALU00753972, ALU00179369,  
#--   3.0.03  - 2010/09/29 : Support for NT OFLMT tool V18.2.4
#                            ALU00782559 - PBMT restricts check for R4010 i.e. 4106 & 4109 may NOT be taken by R4010
#--   3.0.04  - 2010/12/15 : ALU00909611 - Support for R43
#--   3.0.05  - 2011/01/20 : undone changes tried for R4220 and R4230
#--   3.0.06  - 2011/01/21 : added support for Lanx migration tool 1.59
#--   3.0.07  - 2011/02/04 : added support for Lanx migration tool 1.60
#--   3.0.08  - 2011/02/11 : added support for Lanx migration tool 1.61
#--   3.0.09  - 2011/02/22 : ALU01088438 
#--   3.0.10  - 2011/03/19 : Highest SHUB OFLMT is 1.62
#--   3.0.11  - 2011/04/01 : Support for NT OFLMT tool 20.4.1
#                          : RVSA-C NT HW type added for 7354
#--   3.0.12  - 2011/04/05 : Change highest supported NT OFLMT tool version for ANSI stream 
#--   3.0.13  - 2011/04/19 : Highest IVPS OFLMT tool updated
#--   3.0.14  - 2011/04/25 : ALU01031457 - Support for R4220 and R4230
#--   3.0.15  - 2011/05/04 : Highest NT OFLMT tool updated for ANSI stream
#--   3.0.16  - 2011/05/05 : Minimized sw download failure for R4230 due to variant mismatch
#--   3.0.17  - 2011/05/10 : ALU01213090 
#--   3.0.18  - 2011/05/17 : ALU01152509
#--   3.0.19  - 2011/05/27 : ALU01237623 
#--   3.0.20  - 2011/06/09 : Highest SHUB OFLMT is 1.65
#--   3.0.21  - 2011/06/17 : ALU01261528 
#--   3.0.22  - 2011/06/22 : ALU01273279 
#--   3.0.23  - 2011/07/05 : ALU01300398 
#--   3.0.24  - 2011/07/27 : To support R43/R4301 SB runs with integration Build version numbers
#--   3.0.25  - 2011/09/22 : ALU01393253
#--   3.0.26  - 2011/09/24 : Highest IVPS OFLMT tool updated 
#--   3.0.27  - 2011/09/27 : Addition of FGLT-A to the list of Hicap LT's
#--   3.0.28  - 2011/11/03 : ALU01595196
#--   3.0.29  - 2011/11/10 : changes to resolve upgrade restriction failure in 4106
#--   3.0.30  - 2011/11/17 : ALU01607596 - support for R4.4
#--   3.0.31  - 2011/11/22 : ALU01630848
#--   3.0.32  - 2011/12/14 : checking for shelftypes and profile modes 
#--                        : Highest SHUB OFLMT 1.67
#--   3.0.33  - 2012/02/22 : ALU01755206
#--   3.0.34  - 2012/03/26 : ALU01753789 
#--   3.0.35  - 2012/04/06 : ALU01787464  
#--   3.0.36  - 2012/05/14 : ALU01790650, ALU01790654 - support in R4401/R4402 for RLAB  
#--   3.0.37  - 2012/06/26 : Changes to facilitate R4302g release migration
#--   3.0.38  - 2012/07/14 : ALU01804629 
#--   3.0.39  - 2012/07/24 : ALU01808034
#--   3.0.40  - 2012/08/16 : Highest oflmt version change
#--   3.0.41  - 2012/08/28 : ALU01814545 
#--   3.0.42  - 2012/09/20 : ALU01824012
#--   3.0.43  - 2012/09/27 : change SHUB version for R45
#--   3.0.44  - 2012/11/17 : ALU01836902
#--   3.0.45  - 2012/11/25 : ALU01844317
#--   3.0.46  - 2012/12/17 : to resolve migration issue in 4404 builds
#--   3.0.47  - 2013/01/10 : ALU01854309
#--   3.0.48  - 2013/02/18 : ALU01864146
#--   3.0.49  - 2013/02/22 : ALU01865738
#--   3.0.50  - 2013/03/07 : Highest shub version updated
#--   3.0.51  - 2013/05/02 : ALU01879100
#--   3.0.52  - 2013/05/07 : Highest oflmt and shub version updated
#--   3.0.53  - 2013/05/15 : ALU01883817
#--   3.0.54  - 2013/06/24 : ALU01885436
#--   3.0.55  - 2013/07/16 : ALU01896304 (to support FDT builds for RLAB in R4601)
#--   3.0.56  - 2013/07/29 : ALU01898394
#--   3.0.57  - 2013/09/05 : ALU01910032
#--   3.0.58  - 2013/09/13 : to resolve SB run issue in 44.9xx FDT builds
#--   3.0.59  - 2013/09/25 : ALU01893590
#--   3.0.60  - 2013/11/06 : ALU01927702
#--   3.0.61  - 2013/11/20 : ALU01932194
#--   3.0.62  - 2013/12/10 : ALU01937650
#--   3.0.63  - 2013/12/23 : ALU01940963
#--   3.0.64  - 2014/01/10 : ALU01944901
#--   3.0.65  - 2014/02/06 : change for FGN4310 stream builds
#--   3.0.66  - 2014/03/08 : PBMT support for R51(ALU01955636) & R4407(ALU01951877) 
#--   3.0.67  - 2014/03/20 : NGVR support 
#--   3.0.68  - 2014/03/25 : support for new db version in R5001 (ALU01963317)
#--   3.0.69  - 2014/04/17 : ALU01968792
#--   3.0.70  - 2014/04/29 : ALU01967668 : PBMT adaptation for NT-B Support
#--   3.0.71  - 2014/05/14 : ALU01975944 : PBMT support for ANSI VSRM, ALU01972034
#--   3.0.72  - 2014/05/16 : Highest NT OFLMT tool version supported updated for ETSI
#--   3.0.73  - 2014/07/21 : ALU01998650 - Support for R5002 build stream 
#--   3.0.74  - 2014/08/04 : ALU02001379 
#--   3.0.75  - 2014/09/12 : Highest oflmt version updated
#--   3.0.76  - 2014/09/19 : ALU02013699
#--   3.0.77  - 2014/10/13 : ALU02018215: Support for R52
#--   3.0.78  - 2014/10/18 : Remove R52 entry of AGNT-A & NANT-A from NO_MIG 
#--   3.0.79  - 2014/11/07 : ALU02016191
#--   3.0.80  - 2014/12/20 : ALU02035773
#--   3.0.81  - 2015/01/05 : ALU02036710
#--   3.0.82  - 2015/02/28 : ALU02028785
#--   3.0.83  - 2015/03/26 : FGN44 FDT build support 44.6xx 
#--   3.0.84  - 2015/03/31 : ALU02063854: Support for R53
#--   3.0.85  - 2015/04/01 : ALU02063854: Lanx 1.76 support added 
#--   3.0.86  - 2015/04/13 : ALU02066921: PBMT capability support for SRNT-C,SRNT-E 
#--   3.0.87  - 2015/04/23 : Support for 5001e and 5200a releases
#--   3.0.88  - 2015/05/15 : ALU02080736: Lanx 1.77 support added 
#--   3.0.89  - 2015/06/05 : ALU02083290
#--   3.0.90  - 2015/08/04 : changes for SX-16VP
#--   3.0.91  - 2015/09/03 : Support for new db version in R5301
#--   3.0.92  - 2015/09/09 : changes for SX-16F 
#--   3.0.93  - 2015/09/15 : ALU02126210
#--   3.0.94  - 2015/09/18 : Support for new DB version in FGN4401
#--   3.0.95  - 2015/09/23 : ALU02129706
#--   3.0.96  - 2015/09/24 : Support for new Lanx version in ISR54
#--   3.0.97  - 2016/01/07 : support for FANT-G board in Capability model
#--   3.0.98  - 2016/01/27 : ALU02147557
#--   3.0.99  - 2016/02/15 : ALU02175729
#--   3.1.00  - 2016/04/08 : ALU02198672
#--   3.1.01  - 2016/05/04 : ALU02196748
#--   3.1.02  - 2016/05/06 : ALU02206611, R5203 support
#--   3.1.03  - 2016/05/13 : ALU02202972, ALU02213657
#--   3.1.04  - 2016/06/13 : ALU02231273
#--   3.1.05  - 2016/07/12 : ALU02244120
#--   3.1.06  - 2016/07/15 : ALU02237966
#--   3.1.07  - 2016/09/09 : Support for ISR56 migration
#--   3.1.08  - 2016/09/16 : ALU02271670
#--   3.1.09  - 2016/10/13 : ALU02282385
#--   3.1.10  - 2016/10/21 : ALU02285181 
#--   3.1.11  - 2016/11/28 : ALU02285181
#--   3.1.12  - 2016/12/06 : ALU02307515
#--   3.1.13  - 2016/12/19 : ALU02311659
#--   3.1.14  - 2017/01/23 : ALU02324103
#--   3.1.15  - 2017/02/08 : Support to ISR5601
#--   3.1.16  - 2017/04/06 : ALU02347016
#--   3.1.17  - 2017/06/27 : ALU02360205
#--   3.1.18  - 2017/07/28 : Support for ISR5602 db change
#--   3.1.19  - 2017/08/09 : Support for ISR5701 db change
#--   3.1.20  - 2017/08/29 : ALU02392474 - Support for CFNT-C from R5701
#--   3.1.21  - 2017/09/13 : ALU02397238
#--   3.1.22  - 2017/09/29 : R58 Changes
#--   3.1.23  - 2018/02/01 : ALU02431642
#--   3.1.24  - 2018/02/01 : ALU02437753
#--   3.1.25  - 2018/03/15 : ALU02444650
#--   3.1.26  - 2018/03/28 : R60 Changes
#--   3.1.27  - 2018/05/11 : Lanx Version 184
#--   3.1.28  - 2018/07/11 : R60 New Board support
#--   3.1.29  - 2018/08/09 : ALU02494467
#--   3.1.30  - 2018/10/01 : R61 Changes
#--   3.1.31  - 2019/02/07 : Lanx Version 186

=head1 VERSION

=begin html
Current version: "Capability.pm - 3.1.31 - 2019/02/07"

=end html

=cut

my $VERSION = "Capability.pm - 3.1.31 - 2019/02/07";
#==============================================================================

package Capability;

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use File::Basename;
use IsamCli;
use ToolCommon qw(:CONSTANTS);
use Logger;
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetError();
sub SelectFH($);
sub SetDebug($);
sub ReleaseName($$);
sub NtLoadPrefix($$);
sub SwMgntIniSize($$);
sub ShubBoardType($);
sub AltSwMgntIniSize($$);
sub SwParSize($);
sub NtSupported($);
sub ReleaseCompare($$$$;$);
sub ConvertRelease($$;$$);
sub DotRelease($);
sub BaseRelease($);
sub NtDbVersion($$;$);
sub XvpsDbVersion($$);
sub NtDbVersionInfo($$);
sub ShubDbVersion($$);
sub ShubReleaseNum($$);
sub DetermineRelNum($$);
sub DetermineNtType($);
sub DetermineXvpsType($);
sub NtOflmtVersionString($);
sub NtOflmtBuildString($);
sub ShubOflmtVersionString($);
sub HighestShubOflmtVersion($);
sub HighestNtOflmtVersion($);
sub HighestNtOflmtBuild($);
sub HighestXvpsOflmtVersion($);
sub InterfaceBase($);
sub GetReleaseFromList($$);
sub GetCDEMigrFromRelease($);
sub SelectFamily($$);
sub IsIsamFamily($$);
sub IsIsamBothFamily($$);
sub IsIsamEtsiExtendedFamily($$);
sub IsIsamEtsiNttFamily($$);
sub IsIsamEtsiFamily($$);
sub IsIsamAnsiFamily($$);
sub IsIsamHiCapEtsiFamily($$);
sub IsIsamHiCapAnsiFamily($$);
sub Is7354RUFamily($$);
sub IsIsamLSFamily($$);
sub IsIsam7367Family($$);
sub IsIsam7367_SXFamily($$);
sub IsIsam7363Family($$);
sub ReductionChars($$);
sub ReductionChar($);
sub OswpVariant($$;$);
sub GetOswpVariant($$);
sub OswpReduced($$);
sub LogNeFamily($$);
sub IsamLS2IsamRelease($);
sub IsamHiCapEtsiFamily();
sub IsamHiCapAnsiFamily();
sub IsamFamily();
sub IsamLSFamily();
sub IsamEtsiFamily();
sub IsamEtsiNttFamily();
sub IsamEtsiExtendedFamily();
sub IsamAnsiFamily();
sub IsamBothFamily();
sub RU7354Family();
sub Isam7367Family();
sub Isam7367_SXFamily();
sub Isam7363Family();
sub IsamOswpVariant($);
sub IsamLSOswpVariant($);
sub IsamEtsiOswpVariant($);
sub IsamEtsiNttOswpVariant($);
sub IsamAnsiOswpVariant($);
sub IsamHiCapEtsiOswpVariant($);
sub Isam7367OswpVariant($);
sub Isam7367_SXOswpVariant($);
sub Isam7363OswpVariant($);
sub FamilyIdInList($$$);
sub FamilyFromOswp($;$);
sub BuildNtOflCap($);
sub NtOflCapDefined($$);
sub SafeNtIcs($);
sub SafeShubOflmt($$$);
sub McastPackageCountOK($$$);
sub GetMcastSrcOID($);
sub GetFirstLtOffset($$);
sub GetLtInternalNum($$$);
sub ConvertLtInternalNum($$$);
sub MaxMacPerLsm($);
sub ReadSWP($$$);
sub FindShubSwName($$$$$$);
sub ShubSwName($$);
sub PrintStreamInfo();
sub GetTransactValue($); 
sub GetReleaseFromDBVersion($);
sub IsMiscLT($);
sub IsANSIShelf($);
sub IsNEPRUShelf($);
sub ChangeEqptType($$);
sub IsTypeBased($); 
sub SetVariantCheckSkip($);
sub ResetGlobals();
#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;

my $MAINSTREAM_ASSOCIATION = 340;

my $CRITICALFILESIZE_R1 = 1024;
my $CRITICALFILESIZE_R2 = 1024;
my $MORECRITICALSIZE = 1000;            # When migrating a downgraded R2.3 we have to
                                        # reduce the critical size due to an extra line
                                        # in the SwMgnt.ini file
my $INFINITESIZE = -1;
my $CRITICALFILESIZE_R3 = 1024;
my $CRITICALFILESIZE_R4 = 1024;
my $UNSUPPORTED = 0;
my $IMPOSSIBLE = '__IMPOSSIBLE__';
my $NANTB_IFBASE = 448;
my $OTHER_IFBASE = 256;
# -- 1.4.1
my $INCOMP_RELSTREAM = 'Incompatible release streams with respect to feature compatibility, please check with RSC';
my $FTP_ISSUE = 'No upgrade possible due to file transfer problem';
my $NO_MIG_PATH = 'No migration path defined';
# -- 1.4.1

# NE families
my $ISAM = "ISAM-7302";
my $ISAM_HICAP_ETSI = "ISAM-7302-HICAP-ETSI";
my $ISAM_HICAP_ANSI = "ISAM-7302-HICAP-ANSI";
my $ISAM_BOTH = "ISAM-7302-ETSI-ANSI";
my $ISAM_ANSI = "ISAM-7330-ANSI";
my $ISAM_ETSI = "ISAM-7302-ETSI";
my $ISAM_ETSI_NTT = "ISAM-7302-ETSI-NTT";
my $ISAM_ETSI_EXT = "ISAM-7302-ETSI-EXT";  # includes NTT build
my $ISAM_PIZZA = "ISAM-7354RU";
my $ISAM_GPON = "ISAM-7342";
my $ISAM_LS = "ISAM-1540";
my $ISAM_TISEM = "ISAM-7367";
my $ISAM_ANSI_VSRM = "ISAM-7367-SX";
my $ISAM_NGVR = "ISAM-7363";

# EQPT types
my $XD = "XD";
my $FD = "FD";
my $FX = "FX";
my $PIZZA = "7354RU";
my $TISEM = "7367";
my $ANSI_VSRM = "7367-SX";
my $NGVR = "7363";

my @neFamilies = ($ISAM,              # "Old ISAM", i.e. till R3.7
		  $ISAM_BOTH,         # From R4.0 onwards for NTs that are both in the ETSI and ANSI stream
		  $ISAM_HICAP_ETSI,   # From R3.7.10 onwards for NANT-D
		  $ISAM_HICAP_ANSI,   # From R3.7.10 onwards
		  $ISAM_PIZZA,        # Small Pizza box 7354RU
		  $ISAM_ANSI,         # From R4.0 onwards for the XD NTs ECNT-A and ECNT-C
		  $ISAM_ETSI,         # From R4.0 onwards for the FD NT NANT-A (pure ETSI build only)
		  $ISAM_ETSI_NTT,     # From R4.0 onwards for the FD NT NANT-A (pure NTT build only)
		  $ISAM_ETSI_EXT,     # From R4.0 onwards for the FD NANT-A include NTT build
		  $ISAM_GPON,         # For the GPON NE family
		  $ISAM_LS,           # For 1540LS ISAM variant
		  $ISAM_TISEM,        # For 7367 box
		  $ISAM_ANSI_VSRM,    # For 7367-SX box
		  $ISAM_NGVR);       # For 7363 box

my $VARIANT_RE = "....(\\w\\w)......";

my $MAXSTATICUNICASTMACPERLT = 1024;    # the Capability model as %ltLimits.
my $ECNTA_MINSWPARSIZE = MINIMALSWPARSIZE;
my $ECNTC_MINSWPARSIZE = MINIMALSWPARSIZE_ECNTC;

# Shelf Types
my $NEP   = "NFXS-A"; 
my $NEPRU = "NFXS-B"; 
#my $TISEM_SHELF        TI SEM shelf (optional)

# Profile Mode Types
my $LEGACY   = "LEGACY-BASED"; 
my $TYPE     = "TYPE-BASED"; 
my $POSITION = "POSITION-BASED"; 


# -- 1.4.1
my $ERR_MESG_INCOMP_RELSTREAM = 'Incompatible release streams with respect to feature compatibility, please check with RSC';
my $ERR_MESG_FTP_ISSUE = 'No upgrade possible due to file transfer problem';
my $ERR_MESG_INV_MIG_PATH = 'Migration paths are not valid/available';

### Upgrade operation restriction details
my %UpgradeRestricted = (
    'A2.400'  => [
                   {
                     'SOURCE' => { 'FROM' => '24.000', 'TO' => '24.099' },
                     'TARGET' => { 'FROM' => '24.000', 'TO' => '24.999' },
                     'BOARD'  => [ 'ECNT-A', 'ECNT-B', 'ECNT-C' ],
                     'ISSUE'  => $ERR_MESG_FTP_ISSUE,
                   },
                   {
                     'SOURCE' => { 'FROM' => '24.100', 'TO' => '24.299' },
                     'TARGET' => { 'FROM' => '24.400', 'TO' => '24.499' },
                     'BOARD'  => [ 'ECNT-A', 'ECNT-B', 'ECNT-C' ],
                     'ISSUE'  => $ERR_MESG_INCOMP_RELSTREAM,
                   },
                   {
                     'SOURCE' => { 'FROM' => '24.100', 'TO' => '24.299' },
                     'TARGET' => { 'FROM' => '24.600', 'TO' => '24.699' },
                     'BOARD'  => [ 'ECNT-A', 'ECNT-B', 'ECNT-C' ],
                     'ISSUE'  => $ERR_MESG_INCOMP_RELSTREAM,
                   },
                   {
                     'SOURCE' => { 'FROM' => '24.100', 'TO' => '24.299' },
                     'TARGET' => { 'FROM' => '24.800', 'TO' => '24.999' },
                     'BOARD'  => [ 'ECNT-A', 'ECNT-B', 'ECNT-C' ],
                     'ISSUE'  => $ERR_MESG_INCOMP_RELSTREAM,
                   },
                 ],
   );


### Migrate operation restriction details
my %MigrateRestricted = (
    'C4.302'  => { 
                   'TARGET' => [ 'A4.400', 'B4.401' ],
                   'BOARD'  => [ 'AGNT-A', 'NANT-A', 'NANT-D', 'NRNT-A', 'NANT-E', 'FANT-F' ],
                   'ISSUE'  => $ERR_MESG_INV_MIG_PATH,
                 },
    'B4.401'  => { 
                   'TARGET' => [ 'ALL' ],
                   'BOARD'  => [ 'AGNT-A', 'NANT-A' ],
                   'ISSUE'  => $ERR_MESG_INV_MIG_PATH,
                 },
    'C5.602'  => { 
                   'TARGET' => [ 'A5.700' ],
                   'BOARD'  => [ 'ALL' ],
                   'ISSUE'  => $ERR_MESG_INV_MIG_PATH,
                 }
   );


=head1 CAPABILITY MODEL

=begin html
The capability model is defined by one hash-table containing:
<ul>
  <li>Supported NT types</li>
  <li>The known releases for these NT types</li>
  <li>The migration and SW upgrade restrictions applicable for these release</li>
  <li>Various dat associated to the release (e.g. NT and SHUB DB release names)</li>
</ul>
The following entries are/may be present per NT type:
<table>
  <tr>
    <td>SW_PAR</td>
    <td>specifies the minimum SW partition size on the NT SAN disk.  ECNT-A had variants that make SW reduction necessary to allow download
        of a new package.  An undef value indicates that this NT type is NOT supported by PBMT</td>
  </tr>
  <tr>
    <td>NO_UPG</td>
    <td>indicates what SW upgrades are to be avoided.  $IMPOSSIBLE means a dead end upgrade</td>
  </tr>
  <tr>
    <td>NO_MIG</td>
    <td>indicates what SW migrations are to be avoided.  $IMPOSSIBLE means a dead end upgrade</td>
  </tr>
  <tr>
    <td>NO_MIG_V</td>
    <td>indicates what ISAM-V SW migrations are to be avoided.  $IMPOSSIBLE means a dead end upgrade</td>
  </tr>
  <tr>
    <td>PREFIX</td>
    <td>The SW prefix of the NT build.  This entry is used to determine the running release in case no OSWP information was returned via CLI.
        In certain conditions (e.g. error cases of the NT) this can happen.  We then try via an alternative way to determine the release info.
        As there can be more then one prefix (e.g. NANT-A) this entry is actually a hash.  If only one prefix is possible it will be identified
        as "ALL", otherwise the stream (e.g. 320, 330) will be used as a qualifier.</td>
  </tr>
  <tr>
    <td>BRDCODE</td>
    <td>The board code internally used by the SW.  This entry is needed to determine the NT type when migrating a DB using the --ofldb option.</td>
  </tr>
  <tr>
    <td>SAFE_ICS</td>
    <td>Optional entry defining the safe ICS of a certain NT.  For ECNT-A vairous variants with different ICS codes were put into production.  Certain
        ICS codes had potential issues and replacement is needed in certain conditions.</td>
  </tr>
  <tr>
    <td>SHUB_TYPE</td>
    <td>A string to used when migrating the SHUB database.  It identifies the NT type for the SHUB OLFMT.</td>
  </tr>
  <tr>
    <td>IF_BASE</td>
    <td>A number identifying the number of IP interfaces supported on the SHUB.  This has been introduced in the migration to R3.5</td>
  </tr>
  <tr>
    <td>Release</td>
    <td>A release identification in its translated form (a 3-digit version).  For every entry the following values are defined:
        <ul>
          <li>CRITICAL: the critical SwMgnt.ini file size for a board flashed with R2.2 ISAM SW</li>
          <li>ALT-SIZE: the critical SwMgnt.ini file size for a board flashed with > R2.2 ISAM SW</li>
          <li>REL_NAME: a string used when displaying the NE release</li>
        </ul>
       As long as a board is known in a certain release it needs to entered here.
    </td>
  </tr>
  <tr>
    <td>Family</td>
    <td>A hash identifying the NE family this NE belongs to.  The hash entry contains the following data:
      <ul>
        <li>A release identification.  This can be "ALL" to denote all releases or a specific release e.g. 400 defining R4.0.0</li>
        <li>the NE family associated to that release</li>
      </ul>
    </td>
</table>

=end html

=cut

our %ntCapabilities = (
              ### LS-Family/Isam1540
              'ALL'   => {
                   'CRITICAL' => $INFINITESIZE,
                   'ALT_SIZE' => $INFINITESIZE,
              },
		      'GEB3-B' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'NANU'},
				   'BRDCODE' => 21837,
				   'SHUB_TYPE' => 'ECNTC',
				   'EQPT_TYPE' => $FD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM_LS},
			  },
		      'RVSA-A' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12359,                              # -- To be modified
				   'EQPT_TYPE' => $PIZZA,
				   'FAMILY' => {'ALL' => $ISAM_PIZZA},
			  },
		      'RVSA-B' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12359,                              # -- To be modified
				   'EQPT_TYPE' => $PIZZA,
				   'FAMILY' => {'ALL' => $ISAM_PIZZA},
			 },
             'RVSA-C' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12359,                              # -- To be modified
				   'EQPT_TYPE' => $PIZZA,
				   'FAMILY' => {'ALL' => $ISAM_PIZZA},
			  },
               ### Others
              'EANT-A' => {'SW_PAR' => undef},
		      'EBNT-A' => {'SW_PAR' => undef},
		      'ECNT-A' => {
                   'SW_PAR' => $ECNTA_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'L5YY'},
				   'BRDCODE' => 12355,
				   'SAFE_ICS' => 13,
				   'SHUB_TYPE' => 'ECNTA',
				   'EQPT_TYPE' => $XD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM,
                                '[N-W][4-9].[0-9][0-9][0-9]' => $ISAM_ANSI },
              },
		      'ECNT-B' => {
                   'SW_PAR' => $ECNTA_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'L5YY'},
				   'BRDCODE' => 12356,
				   'SHUB_TYPE' => 'ECNTA',
				   'EQPT_TYPE' => $XD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM},
              },
		      'ECNT-C' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- 1.2.3
				   'BRDCODE' => 12359,
				   'SHUB_TYPE' => 'ECNTC',
				   'EQPT_TYPE' => $XD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM,
                                '[N-W][4-9].[0-9][0-9][0-9]' => $ISAM_ANSI },
              },
		      'HYNT-A' => {
                   'SW_PAR' => $ECNTA_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'L5YY'},
				   'BRDCODE' => 12357,
				   'SHUB_TYPE' => 'ECNTA',
				   'EQPT_TYPE' => $XD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM},
              },
		      'HYNT-B' => {
                   'SW_PAR' => $ECNTA_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'L5YY'},
				   'BRDCODE' => 12358,
				   'SHUB_TYPE' => 'ECNTA',
				   'EQPT_TYPE' => $XD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM},
              },
		      'NANT-A' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'3100' => 'MUK4',
						'3200' => 'MUK4',
						'ALL' => 'MP1J'},
				   'BRDCODE' => 12360,
				   'SHUB_TYPE' => 'ECNTC',
				   'EQPT_TYPE' => $FD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM_ETSI_EXT,
                                '[A-Z][0-3].[0-9][0-9][0-9]' => $ISAM },
              },
		      'NANT-B' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'M16X'},
				   'BRDCODE' => 12361,
				   'SHUB_TYPE' => 'ECNTC',
				   'EQPT_TYPE' => $FD,
				   'IF_BASE' => $NANTB_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM},
              },
		      'AGNT-A' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => '????'},
				   'BRDCODE' => 12364,
				   'SHUB_TYPE' => 'ECNTC',
				   'EQPT_TYPE' => $FD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {
                                '[A-J][4-9].[0-9][0-9][0-9]' => $ISAM_ETSI,
                                '[N-W][4-9].[0-9][0-9][0-9]' => $ISAM_ANSI,
                                '[A-Z][0-3].[0-9][0-9][0-9]' => $ISAM, 
                                },
			  },
		      # May have to be split up in ipdAnsi and ipdEtsi capabilities
		      'NANT-D' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12365,
				   'EQPT_TYPE' => $FD,
				   'FAMILY' => {'ALL' => $ISAM_HICAP_ETSI},
			  },
		      'NRNT-A' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12366,
                   'SHUB_TYPE' => 'ECNTC',
				   'EQPT_TYPE' => $FD,
				   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM_ETSI_EXT},
		      },
		      'NANT-E' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12367,
				   'EQPT_TYPE' => $FD,
				   'FAMILY' => {'ALL' => $ISAM_HICAP_ETSI},
		      },
	          'FANT-F' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12368,
				   'EQPT_TYPE' => $FX,
				   'FAMILY' => {'ALL' => $ISAM_HICAP_ETSI},   
			  },
	          'FANT-G' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 16728,
				   'EQPT_TYPE' => $FX,
				   'FAMILY' => {'ALL' => $ISAM_HICAP_ETSI},   
			  },
	          'FANT-H' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 17745,
				   'EQPT_TYPE' => $FX,
				   'FAMILY' => {'ALL' => $ISAM_HICAP_ETSI},   
			  },
		      'ECNT-E' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12363,
				   'SHUB_TYPE' => 'ECNTC',
				   'EQPT_TYPE' => $XD,
                   'IF_BASE' => $OTHER_IFBASE,
				   'FAMILY' => {'ALL' => $ISAM_ANSI},
			  },
		      'ECNT-F' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,
				   'PREFIX' => {'ALL' => 'MP1J'},                   # -- To be modified
				   'BRDCODE' => 12363,
				   'EQPT_TYPE' => $FD,
				   'FAMILY' => {'ALL' => $ISAM_HICAP_ANSI},
			  },
		      'NRNT-I' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 17235,   
				   'EQPT_TYPE' => $TISEM,
				   'FAMILY' => {'ALL' => $ISAM_TISEM},
			  },
		      'SRNT-A' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 12369,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
		      'SRNT-B' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 14672,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
		      'SRNT-C' => {
                   'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 14679,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
		      'SRNT-D' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16707,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
		      'SRNT-E' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16705,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
		      'SRNT-F' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16711,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
			  'SRNT-G' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16716,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
			  'SRNT-H' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16718,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
			  'SRNT-I' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16722,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
			  'SRNT-J' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16967,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
			  'SRNT-K' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,    
				   'PREFIX' => {'ALL' => 'MP1J'},     
				   'BRDCODE' => 16730,   
				   'EQPT_TYPE' => $ANSI_VSRM,
				   'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
              'SRNT-P' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                    'PREFIX' => {'ALL' => 'MP1J'},
                    'BRDCODE' => 16986,
                    'EQPT_TYPE' => $ANSI_VSRM,
                    'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
			  'SRNT-N' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                    'PREFIX' => {'ALL' => 'MP1J'},
                    'BRDCODE' => 17744,
                    'EQPT_TYPE' => $ANSI_VSRM,
                    'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			  },
			  'RANT-A' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                    'PREFIX' => {'ALL' => 'MP1J'},
                    'BRDCODE' => 14163,
                    'EQPT_TYPE' => $NGVR,
                    'FAMILY' => {'ALL' => $ISAM_NGVR},
			   },
			   'RANT-B' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 16965,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			   },
			   'RANT-C' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17479,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			   },
			   'DRNT-C' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17488,
                     'EQPT_TYPE' => $ANSI_VSRM,
                     'FAMILY' => {'ALL' => $ISAM_ANSI_VSRM},
			   },
			   'CFNT-A' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 16725,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			   },
			   'CFNT-B' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 16973,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			   },
               'CFNT-C' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 16976,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			   },
			   'RPNI-A' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17237,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			  },
			  'RANI-A' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17238,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
              },
			  'RPNI-B' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17239,
                     'EQPT_TYPE' => $NGVR,
              },
			  'RPNI-C' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17485,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
              },
			  'RANI-B' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17240,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
              },
			  'RANI-C' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17484,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
             },
			 'RANI-D' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 16966,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
             },
			'RANI-E' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17732,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
            },
			'RANI-F' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17730,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
            },
			'RANI-G' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17729,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
            },
			'RGNI-B' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17241,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
            },
			'RPOW-A' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17712,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			},
			'RACT-A' => {'SW_PAR' => $ECNTC_MINSWPARSIZE,
                     'PREFIX' => {'ALL' => 'MP1J'},
                     'BRDCODE' => 17720,
                     'EQPT_TYPE' => $NGVR,
                     'FAMILY' => {'ALL' => $ISAM_NGVR},
			},
  );

my %xvpsCapabilities = (
            'IPVS-A' => {'BRDCODE' => 12877},
			'IVPS-B' => {'BRDCODE' => 12888},
			'NVPS-A' => {'BRDCODE' => 13384},
			'NVPS-C' => {'BRDCODE' => 14425},
			'VVPS-A' => {'BRDCODE' => 17739}   );

my %releaseInfo = (
		   $ISAM_LS => {
                'A3.400' => {
					  'ISAM_REL' => 3400,
					  'SHUB' => 'R3.4',
					  'SHUB_REL' => '3400',
                           },
				'A3.700' => {
					  'ISAM_REL' => 3700,
					  'SHUB' => 'R3.7',
					  'SHUB_NAME' => 'MT87CA30',
					  'SHUB_REL' => '3700',
					  'PRT' => 'R3.0'
                         }
			       },
            $ISAM => {
			     'A3.400' => {
				       'SHUB' => 'R3.4',
				       'SHUB_NAME' => 'MA6YAA34',
				       'SHUB_REL' => '3400',
				       'PRT' => 'R3.4'},
			     'B3.450' => {
				       'SHUB' => 'R3.4',
				       'SHUB_NAME' => 'MA6YAA34',
				       'SHUB_REL' => '3400',
				       'PRT' => 'R3.4.02'},
			     'A3.500' => {
				       'SHUB' => 'R3.5',
				       'SHUB_NAME' => 'MA6YAA35',
				       'SHUB_REL' => '3500',
				       'PRT' => 'R3.5',
				       'TOOLS' => {'SHUB' => 136} },
			     'A3.700' => {
				       'SHUB' => 'R3.7',
				       'SHUB_REL' => '3700',
				       'SHUB_NAME' => 'MA6YAA37',
				       'PRT' => 'R3.7'} },
		   $ISAM_BOTH => {
				  'B4.100' => {
					    'SHUB' => 'R4.1',
					    'SHUB_NAME' => 'MA6YAA41',
					    'SHUB_REL' => '4100',
					    'PRT' => 'R4.0.10'},
				  'B4.220' => {
					    'SHUB' => 'R4.2',
					    'SHUB_NAME' => 'MA6YAA42',
					    'SHUB_REL' => '4200',
					    'PRT' => 'R4.2.20'},
				  'C4.230' => {
					    'SHUB' => 'R4.2',
					    'SHUB_NAME' => 'MA6YAA42',
					    'SHUB_REL' => '4200',
					    'PRT' => 'R4.2.30'},
				  'B4.301' => {
					    'SHUB' => 'R4.3',
					    'SHUB_NAME' => 'MA6YAA43',
					    'SHUB_REL' => '4300',
					    'PRT' => 'R4.3.01'},
				  'B4.301' => {
					    'SHUB' => 'R4.3',
					    'SHUB_NAME' => 'MA6YAA43',
					    'SHUB_REL' => '4300',
					    'PRT' => 'R4.3.06'},
				  'C4.302' => {
					    'SHUB' => 'R4.3',
					    'SHUB_NAME' => 'MA6YAA43',
					    'SHUB_REL' => '4300',
					    'PRT' => 'R4.3.20'},
				  'A4.400' => {
					    'SHUB' => 'R4.3',
					    'SHUB_NAME' => 'MA6YAA43',
					    'SHUB_REL' => '4300',
					    'PRT' => 'R4.4'},
				  'B4.401' => {
					    'SHUB' => 'R4.3',
					    'SHUB_NAME' => 'MA6YAA43',
					    'SHUB_REL' => '4300',
					    'PRT' => 'R4.4.01'},
				  'A5.000' => {
					    'SHUB' => 'R5.0',
					    'SHUB_NAME' => 'MA6YAA50',
					    'SHUB_REL' => '5000',
					    'IVPS' => 'A5.000',
					    'PRT' => 'R5.0.00'},
				  'B5.001'  => {
					    'SHUB' => 'R5.0',
					    'SHUB_NAME' => 'MA6YAA50',
					    'SHUB_REL' => '5000',
					    'IVPS' => 'A5.000',
					    'PRT' => 'R5.0.01'},
				 },
		   $ISAM_ETSI => {
                  'A4.000'  => {
					    'SHUB' => 'R4.0',
					    'SHUB_NAME' => 'MA6YAA40',
					    'SHUB_REL' => '4000',
					    'IVPS' => 'A4.000',
					    'PRT' => 'R4.0'},
                 },
		   $ISAM_ETSI_NTT => {
                  'A3.700' => {
						'SHUB' => 'R3.7',
						'SHUB_NAME' => 'MA6YAA37',
						'SHUB_REL' => '3700',
						'IVPS' => 'A3.700',
						'PRT' => 'R4.0.01'},
				   'A4.000' => {
						'SHUB' => 'R4.0',
						'SHUB_NAME' => 'MA6YAA40',
						'SHUB_REL' => '4000',
						'IVPS' => 'A4.000',
						'PRT' => 'R4.0.02'},
                  },
		   $ISAM_ETSI_EXT => {
                   'A4.000' => {
						'SHUB' => 'R4.0',
						'SHUB_NAME' => 'MA6YAA40',
						'SHUB_REL' => '4000',
						'IVPS' => 'A4.000',
						'PRT' => 'R4.0'},
				  },
		   $ISAM_ANSI => {
                    'A4.000'  => {
					    'SHUB' => 'F4.0',
					    'SHUB_NAME' => 'MA6YAA40',
					    'SHUB_REL' => '4000',
					    'IVPS' => 'A4.000',
					    'PRT' => 'FGN4.0'},
				    'N4.100'  => {
					    'SHUB' => 'F4.1',
					    'SHUB_NAME' => 'MA6YAA41',
					    'SHUB_REL' => '4100',
					    'IVPS' => 'A4.100',
					    'PRT' => 'FGN4.1'},
				     'N4.200' => {
					    'SHUB' => 'F4.2',
					    'SHUB_NAME' => 'MA6YAA42',
					    'SHUB_REL' => '4200',
					    'IVPS' => 'A4.200',
					    'PRT' => 'FGN4.2'},
				     'N4.300'  => {
					    'SHUB' => 'F4.3',
					    'SHUB_NAME' => 'MA6YAA43',
					    'SHUB_REL' => '4300',
					    'IVPS' => 'A4.300',
					    'PRT' => 'FGN4.3'},
				     'N4.400' => {
					    'SHUB' => 'F4.4',
					    'SHUB_NAME' => 'MA6YAA44',
					    'SHUB_REL' => '4400',
					    'IVPS' => 'A4.400',
					    'PRT' => 'FGN4.4'},
				     'P4.401' => {
					    'SHUB' => 'F4.4',
					    'SHUB_NAME' => 'MA6YAA44',
					    'SHUB_REL' => '4400',
					    'IVPS' => 'A4.400',
					    'PRT' => 'FGN4.4.01'},
				     'N4.500' => {
					    'SHUB' => 'F4.5',
					    'SHUB_NAME' => 'MA6YAA45',
					    'SHUB_REL' => '4500',
					    'IVPS' => 'A4.500',
					    'PRT' => 'FGN4.5.00'}
                   },
		   $ISAM_HICAP_ETSI => {
                    'D3.700' => {
						  'IVPS' => 'D3.700',
						  'PRT' => 'R3.7.10'},
					'A4.000' => {
						  'IVPS' => 'A4.000',
						  'PRT' => 'R4.0.02'},
					 'A4.100' => {
						  'IVPS' => 'A4.100',
						  'PRT' => 'R4.1.00'},
				     'B4.100' => {
						  'IVPS' => 'A4.100',
						  'PRT' => 'R4.0.10'},						  
					 'A4.200' => {
						  'IVPS' => 'A4.200',
						  'PRT' => 'R4.2'},
                   },
		   $ISAM_HICAP_ANSI => {
                     'D4.100' => {
						  'IVPS' => 'D4.100',
						  'PRT' => 'FGN4.1.02'} 
                   },
		   $ISAM_PIZZA => {
                     'DB1.005' => {
					     'PRT' => 'R1.0'},
				     'DB1.101' => {
					     'PRT' => 'R1.1'}
				  },
		   $ISAM_TISEM => {
                    'B4.601' => {
				          'PRT' => 'R4.6.01'},
		            'A5.000' => {
				          'PRT' => 'R5.0.00'},
				  },
		   $ISAM_ANSI_VSRM => {
                    'A5.000' => {
				              'PRT' => 'R5.0.00'},
				 },
		   $ISAM_NGVR => {
                   'A5.000' => {
				          'PRT' => 'R5.0.00'},
                 },
		  );

my %neCapabilities = (
             $ISAM => {
                    'SW_VARIANT' => {'ALL' => 'AA'},
			     	'RED_VARIANT' => {'ALL' => 'RA'},
			     	'FAMILY_ID' => {'ALL' => 'A'},
			     	'RED_CHAR' => 'R',
			     	'MIN_VARIANT' => {'ALL' => 'ZA'},
			     	'MIN_CHAR' => 'Z',
			     	'HIGHEST_SHUB_OFLMT' => 186,    # 1.86
			     	'HIGHEST_NT_OFLMT' => 4550,     # 45.5.0
			     	'HIGHEST_NT_OFLMT_BLD' => 6101599,     # 6101.599
			     	'HIGHEST_IVPS_OFLMT' => 710},   # 7.1.0
		      $ISAM_ETSI => {'SW_VARIANT' => {'4230' => 'AZ',
                                                      '4220' => 'AY',
                                                      '4320' => 'AW',
                                                       'ALL' => 'AA'},
				     'RED_VARIANT' => {'4230' => 'RZ',
                                                       '4220' => 'RY',
                                                       '4320' => 'RW',
                                                        'ALL' => 'RA'},
				     'FAMILY_ID' => {'4230' => 'Z',
                                                     '4220' => 'Y',
                                                     '4320' => 'W',
                                                      'ALL' => 'A'},
				     'RED_CHAR' => 'R',
				     'MIN_VARIANT' => {'4230' => 'ZZ',
                                                       '4220' => 'ZY',
                                                       '4320' => 'ZW',
                                                        'ALL' => 'ZA'},
				     'MIN_CHAR' => 'Z',
				     'HIGHEST_SHUB_OFLMT' => 186,    # 1.86
				     'HIGHEST_NT_OFLMT' => 4550,     # 45.5.0
				     'HIGHEST_NT_OFLMT_BLD' => 6101599,     # 6001.599
				     'HIGHEST_IVPS_OFLMT' => 710},   # 7.1.0
		      $ISAM_ETSI_NTT => {'SW_VARIANT' => {'3700' => 'AA',
							  'ALL' => 'AC'},
					 'RED_VARIANT' => {'3700' => 'AC',
							   'ALL' => 'RC'},
					 'FAMILY_ID' => {'3700' => 'A',
							 'ALL' => 'C'},
					 'RED_CHAR' => 'R',
					 'MIN_VARIANT' => {'3700' => 'ZA',
							   'ALL' => 'ZC'},
					 'MIN_CHAR' => 'Z',
					 'HIGHEST_SHUB_OFLMT' => 186,    # 1.86
					 'HIGHEST_NT_OFLMT' => 4550,     # 45.5.0
				         'HIGHEST_NT_OFLMT_BLD' => 6101599,     # 6001.599
					 'HIGHEST_IVPS_OFLMT' => 710},   # 7.1.0
		      $ISAM_ANSI => {'SW_VARIANT' => {'ALL' => 'AN'},
				     'RED_VARIANT' => {'ALL' => 'RN'},
				     'FAMILY_ID' => {'ALL' => 'N'},
				     'RED_CHAR' => 'R',
				     'MIN_VARIANT' => {'ALL' => 'ZN'},
				     'MIN_CHAR' => 'Z',
				     'HIGHEST_SHUB_OFLMT' => 186,    # 1.86
				     'HIGHEST_NT_OFLMT' => 4440000,     # 4440.000
				     'HIGHEST_IVPS_OFLMT' => 710},   # 7.1.0
		      $ISAM_HICAP_ETSI => {'SW_VARIANT' => {'4230' => 'AZ',
                                                            '4220' => 'AY',
                                                            '4320' => 'AW',
                                                            '3700' => 'AD',
							    'ALL' => 'AA'},
					   'RED_VARIANT' => {'4230' => 'RZ',
                                                             '4220' => 'RY',
                                                             '4320' => 'RW',
                                                             '3700' => 'RD',
							     'ALL' => 'RA'},
					   'FAMILY_ID' => {'4230' => 'Z',
                                                           '4220' => 'Y',
                                                           '4320' => 'W',
                                                           '3700' => 'D',
							   'ALL' => 'A'},
					   'RED_CHAR' => 'R',
					   'MIN_VARIANT' => {                    '4230' => 'ZZ',
                                                             '4220' => 'ZY',
                                                             '4320' => 'ZW',
                                                             '3700' => 'ZD',
							     'ALL' => 'ZA'},
					   'MIN_CHAR' => 'Z',
					   'HIGHEST_NT_OFLMT' => 4550,    # 45.5.0
				           'HIGHEST_NT_OFLMT_BLD' => 6101599,     # 6001.599
					   'HIGHEST_IVPS_OFLMT' => 710},  # 7.1.0
		      $ISAM_LS => {'SW_VARIANT' => {'ALL' => 'DA'},
				   'RED_VARIANT' => {'ALL' => 'RA'},
				   'FAMILY_ID' => {'ALL' => 'A'},
				   'RED_CHAR' => 'R',
				   'MIN_VARIANT' => {'ALL' => 'ZA'},
				   'MIN_CHAR' => 'Z',
				   'HIGHEST_SHUB_OFLMT' => 186,    # 1.86
				   'HIGHEST_NT_OFLMT' => 4550,     # 45.5.0
				   'HIGHEST_NT_OFLMT_BLD' => 6101599,     # 6001.599
				   'HIGHEST_IVPS_OFLMT' => 710},   # 7.1.0
             $ISAM_TISEM  =>  {'SW_VARIANT' => {'ALL' => 'AA'},
				   'RED_VARIANT' => {'ALL' => 'RA'},
				   'FAMILY_ID' => {'ALL' => 'A'},
				   'RED_CHAR' => 'R',
				   'MIN_VARIANT' => {'ALL' => 'ZA'},
				   'MIN_CHAR' => 'Z',
				   'HIGHEST_NT_OFLMT' => 4550,     # 45.5.0
				   'HIGHEST_NT_OFLMT_BLD' => 6101599},     # 6001.599
		     $ISAM_NGVR  =>  {'SW_VARIANT' => {'ALL' => 'AA'},
                                   'RED_VARIANT' => {'ALL' => 'RA'},
                                   'FAMILY_ID' => {'ALL' => 'A'},
                                   'RED_CHAR' => 'R',
                                   'MIN_VARIANT' => {'ALL' => 'ZA'},
                                   'MIN_CHAR' => 'Z',
                                   'HIGHEST_NT_OFLMT' => 4550,     # 45.5.0
                                   'HIGHEST_NT_OFLMT_BLD' => 6101599},     # 6001.599
		     $ISAM_ANSI_VSRM  =>  {'SW_VARIANT' => {'ALL' => 'AA'},
                                   'RED_VARIANT' => {'ALL' => 'RA'},
                                   'FAMILY_ID' => {'ALL' => 'A'},
                                   'RED_CHAR' => 'R',
                                   'MIN_VARIANT' => {'ALL' => 'ZA'},
                                   'MIN_CHAR' => 'Z',
                                   'HIGHEST_NT_OFLMT' => 4550,     # 45.5.0
                                   'HIGHEST_NT_OFLMT_BLD' => 6101599},     # 6001.599
		      # No reduction or minimization for the PIZZA box
		     );

#defined for Rel V5.x
my %CDERelDBVersMap = (      
#to DBvers => From DBvers
"V5.200" =>  "V5.001",
"B5.301" =>  "V5.200"
);

# Add to LT number to get LT numbering inline with legacy or position based numbering.  Legacy mode is used
# to calculate ifIndexes.

my %profileModes = ($XD => {'TYPE-BASED' => {'OFFSET' => 3,
					     'LTMAP' => {'1' => 4,
							 '2' => 5,
							 '3' => 6,
							 '4' => 7,
							 '5' => 8,
							 '6' => 9,
							 '7' => 10,
							 '8' => 11,
							 '9' => 12,
							 '10' => 13,
							 '11' => 14,
							 '12' => 15,
							 '13' => 16,
							 '14' => 17,
							 '15' => 18,
							 '16' => 19},
					     'INVMAP' => {'4' => 1,
							  '5' => 2,
							  '6' => 3,
							  '7' => 4,
							  '8' => 5,
							  '9' => 6,
							  '10' => 7,
							  '11' => 8,
							  '12' => 9,
							  '13' => 10,
							  '14' => 11,
							  '15' => 12,
							  '16' => 13,
							  '17' => 14,
							  '18' => 15,
							  '19' => 16}},
			    'POSITION-BASED' => {'OFFSET' => 0,
						 'LTMAP' => {'4' => 4,
							     '5' => 5,
							     '6' => 6,
							     '7' => 7,
							     '8' => 8,
							     '9' => 9,
							     '10' => 10,
							     '11' => 11,
							     '12' => 12,
							     '13' => 13,
							     '14' => 14,
							     '15' => 15,
							     '16' => 16,
							     '17' => 17,
							     '18' => 18,
							     '19' => 19},
						 'INVMAP' => {'4' => 4,
							      '5' => 5,
							      '6' => 6,
							      '7' => 7,
							      '8' => 8,
							      '9' => 9,
							      '10' => 10,
							      '11' => 11,
							      '12' => 12,
							      '13' => 13,
							      '14' => 14,
							      '15' => 15,
							      '16' => 16,
							      '17' => 17,
							      '18' => 18,
							      '19' => 19}},
			    'LEGACY-BASED' => {'OFFSET' => 0,
					       'LTMAP' => {'4' => 4,
							   '5' => 5,
							   '6' => 6,
							   '7' => 7,
							   '8' => 8,
							   '9' => 9,
							   '10' => 10,
							   '11' => 11,
							   '12' => 12,
							   '13' => 13,
							   '14' => 14,
							   '15' => 15,
							   '16' => 16,
							   '17' => 17,
							   '18' => 18,
							   '19' => 19},
					       'INVMAP' => {'4' => 4,
							    '5' => 5,
							    '6' => 6,
							    '7' => 7,
							    '8' => 8,
							    '9' => 9,
							    '10' => 10,
							    '11' => 11,
							    '12' => 12,
							    '13' => 13,
							    '14' => 14,
							    '15' => 15,
							    '16' => 16,
							    '17' => 17,
							    '18' => 18,
							    '19' => 19}}},
		    $FD => {'TYPE-BASED' => {'OFFSET' => 3,
					     'LTMAP' => {'1' => 4,
							 '2' => 5,
							 '3' => 6,
							 '4' => 7,
							 '5' => 8,
							 '6' => 9,
							 '7' => 10,
							 '8' => 11,
							 '9' => 12,
							 '10' => 13,
							 '11' => 14,
							 '12' => 15,
							 '13' => 16,
							 '14' => 17,
							 '15' => 18,
							 '16' => 19,
							 '17' => 20,
							 '18' => 21},
					     'INVMAP' => {'4' => 1,
							  '5' => 2,
							  '6' => 3,
							  '7' => 4,
							  '8' => 5,
							  '9' => 6,
							  '10' => 7,
							  '11' => 8,
							  '12' => 9,
							  '13' => 10,
							  '14' => 11,
							  '15' => 12,
							  '16' => 13,
							  '17' => 14,
							  '18' => 15,
							  '19' => 16,
							  '20' => 17,
							  '21' => 18}},
			    'POSITION-BASED' => {'OFFSET' => 3,
						 'LTMAP' => {'1' => 4,
							     '2' => 5,
							     '3' => 6,
							     '4' => 7,
							     '5' => 8,
							     '6' => 9,
							     '7' => 10,
							     '8' => 11,
							     '10' => 20,
							     '11' => 21,
							     '12' => 12,
							     '13' => 13,
							     '14' => 14,
							     '15' => 15,
							     '16' => 16,
							     '17' => 17,
							     '18' => 18,
							     '19' => 19},
						 'INVMAP' => {'4' => 1,
							      '5' => 2,
							      '6' => 3,
							      '7' => 4,
							      '8' => 5,
							      '9' => 6,
							      '10' => 7,
							      '11' => 8,
							      '20' => 10,
							      '21' => 11,
							      '12' => 12,
							      '13' => 13,
							      '14' => 14,
							      '15' => 15,
							      '16' => 16,
							      '17' => 17,
							      '18' => 18,
							      '19' => 19}},
			    'LEGACY-BASED' => {'OFFSET' => 3,
					       'LTMAP' => {'1' => 4,
							   '2' => 5,
							   '3' => 6,
							   '4' => 7,
							   '5' => 8,
							   '6' => 9,
							   '7' => 10,
							   '8' => 11,
							   '10' => 20,
							   '11' => 21,
							   '12' => 12,
							   '13' => 13,
							   '14' => 14,
							   '15' => 15,
							   '16' => 16,
							   '17' => 17,
							   '18' => 18,
							   '19' => 19},
					       'INVMAP' => {'4' => 1,
							    '5' => 2,
							    '6' => 3,
							    '7' => 4,
							    '8' => 5,
							    '9' => 6,
							    '10' => 7,
							    '11' => 8,
							    '20' => 10,
							    '21' => 11,
							    '12' => 12,
							    '13' => 13,
							    '14' => 14,
							    '15' => 15,
							    '16' => 16,
							    '17' => 17,
							    '18' => 18,
							    '19' => 19}}},
		   );

my %ltSlotNum = ( $FD => { 'NFXS-A' => {'LTMAP' => {'1' => 1,'2' =>2,'3'=>3,'4' => 4 , '5'=>5 ,'6'=>6 ,'7'=>7 ,'8'=>8 , '9'=>12, '10'=>13 ,'11'=>14,'12'=>15,
                                                        '13'=>16,'14'=>17,'15'=>18,'16'=>19}, 
                                        'INVMAP' => {'1'=>1,'2'=>2,'3'=>3,'4'=>4,'5'=>5,'6'=>6,'7'=>7,'8'=>8,'12'=>9,'13'=>10,'14'=>11,'15'=>12,
                                                        '16'=>13,'17'=>14,'18'=>15,'19'=>16}} , 
                           'NFXS-B' => {'LTMAP'=>{'1'=>4,'2'=>5,'3'=>6,'4'=>7,'5'=>8,'6'=>9,'7'=>10,'8'=>11,'9'=>2,'10'=>3},
                                        'INVMAP'=>{'4'=>1,'5'=>2,'6'=>3,'7'=>4,'8'=>5,'9'=>6,'10'=>7,'11'=> 8,'2'=>9,'3'=>10}}}, 
                  $FX => { 'NFXS-E' => {'LTMAP' => {'1' => 5,'2' =>6,'3'=>7,'4' => 8 , '5'=>9 ,'6'=>10 ,'7'=>11 ,'8'=>12}, 
                                        'INVMAP' => {'5'=>1 ,'6'=>2 ,'7'=>3 ,'8'=>4 , '9'=>5, '10'=>6 ,'11'=>7,'12'=>8}},
                           'NFXS-F' => {'LTMAP'=>{'1'=>5,'2'=>6,'3'=>7,'4'=>8},
                                        'INVMAP' => {'5'=>1,'6'=>2,'7'=>3,'8'=>4}},
                           'NFXS-D' => {'LTMAP' => {'1' => 1,'2' =>2,'3'=>3,'4' => 4 , '5'=>5 ,'6'=>6 ,'7'=>7 ,'8'=>8 , '9'=>12, '10'=>13 ,'11'=>14,'12'=>15,
                                                        '13'=>16,'14'=>17,'15'=>18,'16'=>19}, 
                                        'INVMAP' => {'1'=>1,'2'=>2,'3'=>3,'4'=>4,'5'=>5,'6'=>6,'7'=>7,'8'=>8,'12'=>9,'13'=>10,'14'=>11,'15'=>12,
                                                        '16'=>13,'17'=>14,'18'=>15,'19'=>16}}},
);

my %ltLimits = ('MAX_MACS_PER_LSM' => {'EALT-A' => 488,
				       'EALT-B' => 488,
				       'EBLT-A' => 488,
				       'EBLT-B' => 488,
				       'EBLT-C' => 1024,
				       'EBLT-D' => 1024,
				       'EBLT-E' => 1024,
				       'EBLT-F' => 1024,
				       'EBLT-K' => 1024,
				       'ASEM-B' => 1024,
				       'EVLT-A' => 1024,
				       'EVLT-C' => 1024,
				       'EVLT-D' => 1024,
				       'EVLT-E' => 1024,
				       'EBLT-J' => 488,
				       'EBLT-G' => 1024,
				       'EBLT-H' => 1024,
				       'EVLT-F' => 1024,
				       'IWSM-A' => 8264,
				       'VSEM-A' => 1024,
				       'EBLT-L' => 1024,
				       'RVLT-A' => 1024,
				       'RVLT-B' => 1024,
				       'EBLT-M' => 1024,
				       'EBLT-N' => 1024,
				       'NALT-B' => 488,
				       'NALT-C' => 1024,
				       'NALT-D' => 1024,
				       'SMLT-H' => 1024,
				       'SMLT-J' => 1024,
				       'SMLT-K' => 1024,
				       'NVLT-A' => 1024,
				       'NVLT-B' => 1024,
				       'NSLT-A' => 488,
				       'NVLT-C' => 1024,
				       'NVLT-D' => 1024,
				       'NELT-A' => 1024,
				       'NALT-A' => 1024}
	       );

my %mcastSrcLimits = ('MCSRC_PKG'      => {'R34' => '.1.3.6.1.4.1.637.61.1.29.1.1.6',
					   'R35' => '.1.3.6.1.4.1.637.61.1.29.67.1.4'},
		      'MCSRC_PKGLIMIT' => 20
		     );

my %ntOflClones = ();
my %ntOflCap = ();

my @HiCapLT = ("NGLT-A", "NGLT-B", "FGLT-A"); 

my @MiscLT = ("VSEM-C", "VSEM-D", "VSEM-A"); 

my @ANSIShelf = ("ARAM-D", "ARAM-E"); 

my $NT_ERAMA = "AGNT-A"; 

my $DEBUG = undef;

my $ERROR = "";

my $SkipOSWPVariantCheck = 0;

my $NE_FAMILY = "";

sub ResetGlobals() {
     SetVariantCheckSkip(0);
     $NE_FAMILY = "";
}

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 GetError()

=begin html
Return the error encountered by the Capability library.

=end html

=cut

sub GetError() {
  return $ERROR;
}

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 SetDebug()

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
#  print $DEBUG "Testing...\n";
}

sub SetVariantCheckSkip($) {
   my ($value) = @_;
   $SkipOSWPVariantCheck = $value;
}

sub IsUpgradeRestricted($$$$$) {
  my ($ntType, $dbVersionActive, $dbVersionTarget, $oswpActive, $oswpTarget ) = @_;
  Logger::LogDebug($nodeLogFH, "OSWP-NAME  : $oswpActive => $oswpTarget\n");
  Logger::LogDebug($nodeLogFH, "DB-Version : $dbVersionActive => $dbVersionTarget\n");
  my ($restricted, $reason) = ( 0, "");
  $ntType = uc($ntType);
  if ($dbVersionActive ne $dbVersionTarget){
      $restricted = 1;
      $reason = "DB Version is not same for active and target\n";
      return ($restricted, $reason);
  }
  if (not defined $UpgradeRestricted{$dbVersionActive} ) {
      return ($restricted, $reason);
  }
  my ($buildActive, $buildTarget) = ("", "");
  if ($oswpActive =~ /.*(\d{2}\.\d{3})$/) {
      $buildActive = $1;
  }
  if ($oswpTarget =~ /.*(\d{2}\.\d{3})$/) {
      $buildTarget = $1;
  }
  ## 'NANT-A', 'A3.400', '34.110', '34.620'
  foreach my $RestrictedClause ( @{$UpgradeRestricted{$dbVersionActive}} ) {
      if ( (($RestrictedClause->{SOURCE}{FROM} le $buildActive) and ($RestrictedClause->{SOURCE}{TO} ge $buildActive)) and
           (($RestrictedClause->{TARGET}{FROM} le $buildTarget) and ($RestrictedClause->{TARGET}{TO} ge $buildTarget))
         ) {
            if ( grep { /^($ntType|ALL)$/ } @{$RestrictedClause->{BOARD}} ) {
               $reason = $RestrictedClause->{ISSUE};
               $restricted = 1;
               last;
            }
       }
  }
  return ($restricted, $reason);
}

sub IsMigrateRestricted($$$$$) {
  my ($ntType, $dbVersionActive, $dbVersionTarget, $oswpActive, $oswpTarget ) = @_;
  Logger::LogDebug($nodeLogFH, "OSWP-NAME  : $oswpActive => $oswpTarget\n");
  Logger::LogDebug($nodeLogFH, "DB-Version : $dbVersionActive => $dbVersionTarget\n");
  $ntType = uc($ntType);
  my $dbVersionActiveNum = substr($dbVersionActive,1);
  my $dbVersionTargetNum = substr($dbVersionTarget,1);
  if ($dbVersionActiveNum ge $dbVersionTargetNum) {
      return (1, "Active version is higher than Target version");
  }
  if (not defined $MigrateRestricted{$dbVersionActive} ) {
      return (0, "" );
  }
  ## 'NANT-A', 'A3.400', '34.110', '34.620' 
  my $RestrictedClause = $MigrateRestricted{$dbVersionActive};
  if ( grep { /^($dbVersionTarget|ALL)$/ } @{$RestrictedClause->{TARGET}} ) {
       if ( grep { /^($ntType|ALL)$/ } @{$RestrictedClause->{BOARD}} ) {
          return (1, $RestrictedClause->{ISSUE} );
       }
  }
  return (0, "");
}

sub ReleaseName($$) {
  my ($ntType, $relNum) = @_;

  $ntType = uc($ntType);
  my $rn = defined($ntCapabilities{$ntType}{$relNum}) ? $relNum : BaseRelease($relNum);
  ToolCommon::Log($DEBUG, "Determine release name for NT type '$ntType', '$relNum' ($rn)\n") if defined($DEBUG);
  return (defined($ntCapabilities{$ntType}{$rn}{REL_NAME}) ? $ntCapabilities{$ntType}{$rn}{REL_NAME} : "Unknown release");
}

sub NtLoadPrefix($$) {
  my ($ntType, $relNum) = @_;

  $ntType = uc($ntType);
  ToolCommon::Log($DEBUG, "Determine NT load prefix for NT type '$ntType', '$relNum'\n") if defined($DEBUG);
  return (defined($ntCapabilities{$ntType}{PREFIX}{$relNum}) ? $ntCapabilities{$ntType}{PREFIX}{$relNum} :
	  $ntCapabilities{$ntType}{PREFIX}{'ALL'});
}

=head2 SwMgntIniSize()

=begin html
Return the size of the SwMgnt.ini file associated to a certain release.<br>The following parameters are used:
<table>
  <tr><td>ntType</td><td>the NT type involved in the migration (string)</td></tr>
  <tr><td>activeRelNum</td><td>the release currently active</td></tr>
</table>

=end html

=cut

sub SwMgntIniSize($$) {
  my ($ntType, $dbVersion) = @_;

  $ntType = uc($ntType);
  my $criticalSize = 0;
  if (defined($ntCapabilities{$ntType}{$dbVersion}{'CRITICAL'})) {
      $criticalSize = $ntCapabilities{$ntType}{$dbVersion}{'CRITICAL'};
  } elsif ( defined($ntCapabilities{$ntType}) and defined($ntCapabilities{'ALL'}{'CRITICAL'}) ) {
      $criticalSize = $ntCapabilities{'ALL'}{'CRITICAL'};
  }
  ToolCommon::Log($DEBUG, sprintf("SwMgntIniSize for NT type '%s', DB-Version '%s' : %s\n",
				  $ntType, $dbVersion, $criticalSize)) if defined($DEBUG);
  return $criticalSize;
}

=head2 ShubBoardType()

=begin html
Return the SHUB board type string for Offline migration for NT type $ntType

=end html

=cut

sub ShubBoardType($) {
  my ($ntType) = @_;

  $ntType = uc($ntType);
  ToolCommon::Log($DEBUG, sprintf("\nReturn SHUB board type for NT type '%s': %s\n",
				  $ntType, $ntCapabilities{$ntType}{'SHUB_TYPE'})) if defined($DEBUG);
  return $ntCapabilities{$ntType}{'SHUB_TYPE'};
}

=head2 AltSwMgntIniSize()

=begin html
Return the alternative size of the SwMgnt.ini file related to a certain release (in case the NT has been running R2.3 or R2.4 already: SwMgnt.ini file
contains extra line).<br>The following parameters are used:
<table>
  <tr><td>ntType</td><td>the NT type involved in the migration (string)</td></tr>
  <tr><td>activeRelNum</td><td>the release currently active</td></tr>
</table>

=end html

=cut

sub AltSwMgntIniSize($$) {
  my ($ntType, $dbVersion) = @_;

  $ntType = uc($ntType);
  my $altSize = 0;
  if (defined($ntCapabilities{$ntType}{$dbVersion}{'ALT_SIZE'})) {
      $altSize = $ntCapabilities{$ntType}{$dbVersion}{'ALT_SIZE'};
  } elsif ( defined($ntCapabilities{$ntType}) and defined($ntCapabilities{'ALL'}{'ALT_SIZE'}) ) {
      $altSize = $ntCapabilities{'ALL'}{'ALT_SIZE'};
  }

  ToolCommon::Log($DEBUG, sprintf("Alt SwMgntIniSize for NT type '%s', DB-Version '%s' :%s\n",
				  $ntType, $dbVersion, $altSize)) if defined($DEBUG);
  return $altSize;
}

=head2 SwParSize()

=begin html
Return the size of the SW partition associated to NY type $ntType.  The SW partition size should be at a certain minumum to hold R2.3 SW packages.
This function is used in case of ECNT-A only.

=end html

=cut

sub SwParSize($) {
  my ($ntType) = @_;

  $ntType = uc($ntType);
  ToolCommon::Log($DEBUG, sprintf("\nReturn SW partition for NT type '%s': %s\n",
				  $ntType, $ntCapabilities{$ntType}{'SW_PAR'})) if defined($DEBUG);
  return $ntCapabilities{$ntType}{'SW_PAR'};
}

=head2 NtSupported()

=begin html
Check whether the NT $ntType is supported by the toolset or not.

=end html

=cut

sub NtSupported($) {
  my ($ntType) = @_;

  $ntType = uc($ntType);
  return defined($ntCapabilities{$ntType}{'SW_PAR'}) ? 1 : 0;
}

=head2 ReleaseCompare()

=begin html
Compare releases to see whether they are equal, come before or come after.<br>The following parameters are used:
<table>
  <tr><td>current</td><td>the current release</td></tr>
  <tr><td>other</td><td>the new release</td></tr>
  <tr><td>regExp</td><td>the regular expression used to parse the SWP name</td></tr>
  <tr><td>suffix</td><td>the release suffix (the digit following the maintenance release indication - no longer used but left in to avoid code
      updates of all the scripts calling this function)</td></tr>
  <tr><td>full</td><td>convert to the full (3 digits deterministic) or the base (2 digits deterministic, third is 0) release number</td></tr>
</table>
Expected syntax for the release indication, either:
<ul>
  <li>a decimal representation (e.g. 20 for 2.0, 31 for 3.1) NOT PASSED AS STRING!!!</li>
  <li>an OSWP name meeting the $REGEXP syntax</li>
</ul>
The call returns:
<ul>
  <li>-1: Release smaller (older)</li>
  <li>0: Release identical</li>
  <li>+1: Release greater (more recent)</li>
</ul>

=end html

=cut

sub ReleaseCompare($$$$;$) {
  my ($current, $other, $regExp, $suffix, $full) = @_;
  my ($currNum, $otherNum);
  my $compare;
  if ($current =~ /$regExp/) {
    if ($full) {
      $currNum = ConvertRelease($1, $2, $3);
    } else {
      $currNum = ConvertRelease($1, $2);
    }
  } else {
    $currNum = $full ? $current : BaseRelease($current);
  }
  if ($other =~ /$regExp/) {
    if ($full) {
      $otherNum = ConvertRelease($1, $2, $3);
    } else {
      $otherNum = ConvertRelease($1, $2);
    }
  } else {
    $otherNum = $full ? $other : BaseRelease($other);
  }
  if ($currNum == $otherNum) {
    $compare = 0;
  } elsif ($currNum < $otherNum) {
    $compare = -1;
  } else {
    $compare = 1;
  }
  ToolCommon::Log($DEBUG, " Compare '$current' and '$other' : $compare\n") if defined($DEBUG);
  return $compare;
}

sub CompareValues($$;$) {
    my ($dbVersion, $valueToCompare, $length ) = @_;
    if ( $dbVersion =~ /^[A-Z]\d\.\d+$/ ) {
         $dbVersion = substr($dbVersion,1);
    }
    $dbVersion =~ s/\.//;
    if (length($dbVersion) > length($valueToCompare) ) {
        $dbVersion = substr($dbVersion,0,length($valueToCompare));
    }
    print "DB:$dbVersion:\n";
    return ( "$dbVersion" cmp "$valueToCompare" );

}

=head2 ConvertRelease()

=begin html
Convert a release number in the form of major, minor, suffix into a numeric value and return it.

=end html

=cut

sub ConvertRelease($$;$$) {
  my ($major, $minor, $suffix, $dbVersion) = @_;
  my $relNum; 

 if(!defined($dbVersion)){
    $relNum = ($major*100 + $minor*10)*10;

    if ($relNum != 9900) {
       $relNum += $suffix if (defined($suffix));
    } else {
       # -- 1.4.0
       $relNum = $MAINSTREAM_ASSOCIATION;
    }
  } else {
     my $release = GetReleaseFromDBVersion($dbVersion); 
     $relNum = $release ; 
     $relNum += $suffix if (defined($suffix));
  }
  #ToolCommon::Log($DEBUG, sprintf("\nCapability::ConvertRelease: %d\n", $relNum)) if defined($DEBUG);
  return ($relNum);
}

=head2 DotRelease()

=begin html
For logging purposes: return the release $relNum as a string in dotted notation.

=end html

=cut

sub DotRelease($) {
  my ($relNum) = @_;

  if ($relNum > 100) {
    return (sprintf("%d.%d", int($relNum / 100), $relNum % 100));
  } else {
    return (sprintf("%d.%d", int($relNum / 10), $relNum % 10));
  }
}

=head2 BaseRelease()

=begin html
Return the base release of $fullRelNum (ignoring the suffix - set to 0)

=end html

=cut

sub BaseRelease($) {
  my ($fullRelNum) = @_;
  my $baseRelease = 0; 

  #added for 4.0.10 			                   #added for R4.3.01
  if ( (($fullRelNum == 4107) || ($fullRelNum == 4108) ) || (($fullRelNum >= 4302) && ($fullRelNum <= 4306) ) 
         || (($fullRelNum >= 4402) && ($fullRelNum <= 4408)) || (($fullRelNum >= 4602) && ($fullRelNum <= 4606)) 
         || (($fullRelNum >= 5002) && ($fullRelNum <= 5006)) || (($fullRelNum >= 5303) && ($fullRelNum <= 5307)) 
         || (($fullRelNum >= 5503) && ($fullRelNum <= 5507)) || (($fullRelNum >= 5603) && ($fullRelNum <= 5606)) 
	     || (($fullRelNum >= 5702) && ($fullRelNum <= 5706)) 
     ) {
      $baseRelease = $fullRelNum;
  } else {
      $baseRelease = int($fullRelNum / 10) * 10;
  }  

  #ToolCommon::Log($DEBUG, "$fullRelNum -> $baseRelease\n") if (defined($DEBUG));
  return $baseRelease;
}

=head2 XvpsDbVersion()

=begin html
Return the xVPS DB version associated to $relNum.

=end html

=cut

sub XvpsDbVersion($$) {
  my ($neFamily, $dbVersion) = @_;
  my $version = undef;

  if( IsXvpsFamily($neFamily) ) {
       if (defined($releaseInfo{$neFamily}{$dbVersion}) and defined($releaseInfo{$neFamily}{$dbVersion}{IVPS}) ) {
             return $releaseInfo{$neFamily}{$dbVersion}{IVPS};
       } else {
          my $dbVersionMajorRel = substr($dbVersion,1,3);
          if(  $dbVersionMajorRel ge "5.7" ) {
                $version =  $dbVersion;
          } else {
                $version = "A$dbVersionMajorRel"."00";
          }
       }
  }
  ToolCommon::Log($DEBUG, sprintf("\n%s XvpsDbVersion for %s is %s\n",
				  $neFamily, $dbVersion, defined($version) ? $version : "not defined" )) if (defined($DEBUG));
  return $version;
}

=head2 NtDbVersionInfo()

=begin html
Return the NT DB version associated to $relNum.  Used when printing DB verion-related information.

=end html

=cut

sub NtDbVersionInfo($$) {
  my ($neFamily, $dbVersion) = @_;
  my $info = "";
  if (defined($releaseInfo{$neFamily}{$dbVersion}) and defined($releaseInfo{$neFamily}{$dbVersion}{PRT}) ) {
        $info = $releaseInfo{$neFamily}{$dbVersion}{PRT};
  } else {
     my $major = substr($dbVersion,1,3);
     my $minor = substr($dbVersion,4,2);
     if ($minor eq "00") {
        $info = "R$major";
     } else {
        $info = "R$major.$minor";
     }
  }
  ToolCommon::Log($DEBUG, sprintf("\n%s NtDbVersion Info for %s is %s\n",
				  $neFamily, $dbVersion, defined($info) ? $info : "not defined")) if (defined($DEBUG));
  return $info;
}

=head2 ShubDbVersion()

=begin html
Return the SHUB DB version associated to $relNum.

=end html

=cut

sub ShubDbVersion($$) {
  my ($neFamily, $dbVersion) = @_;
  my $version = undef;

  if( IsXvpsFamily($neFamily) ) {
       if (defined($releaseInfo{$neFamily}{$dbVersion}) and defined($releaseInfo{$neFamily}{$dbVersion}{SHUB}) ) {
             $version = $releaseInfo{$neFamily}{$dbVersion}{SHUB};
       } else {
             my $dbVersionMajorRel = substr($dbVersion,1,3);
             $version = "R$dbVersionMajorRel";
       }
  }
  ToolCommon::Log($DEBUG, sprintf("%s ShubDbVersion for %s is %s\n",
				  $neFamily, $dbVersion, defined($version) ? $version : "not defined")) if (defined($DEBUG));
  return $version;
}

=head2 ShubReleaseNum()

=begin html
Return the "real"  SHUB DB version associated to $relNum.

=end html

=cut

sub ShubReleaseNum($$) {
  my ($neFamily, $dbVersion) = @_;
  my $baseRelNum;
  my $shubRel = undef;
  
  if( IsXvpsFamily($neFamily) ) {
       if (defined($releaseInfo{$neFamily}{$dbVersion}) and defined($releaseInfo{$neFamily}{$dbVersion}{SHUB_REL}) ) {
             $shubRel = $releaseInfo{$neFamily}{$dbVersion}{SHUB_REL};
       } else {
          my $dbVersionRel = substr($dbVersion,1);
          $dbVersionRel =~ s/\.//;
          $shubRel = "$dbVersionRel";
       }
  }
  ToolCommon::Log($DEBUG, sprintf("\n%s ShubRelease Number for %s is %s\n",
				  $neFamily, $dbVersion, defined($shubRel) ? $shubRel : "not defined")) if (defined($DEBUG));
  return $shubRel;
}

=head2 DetermineRelNum()

=begin html
Return the release number associated to the NT DB version provided in $dbVersion.  0 is returned wne no match is found.
=end html

=cut

sub DetermineRelNum($$) {
  my ($neFamily, $dbVersion) = @_;

  ToolCommon::Log($DEBUG, "Determine release for ($neFamily, $dbVersion)") if (defined($DEBUG));
  foreach my $dbVersionValue (keys %{$releaseInfo{$neFamily}}) {
   ToolCommon::Log($DEBUG, "\n  $dbVersionValue -> '$releaseInfo{$neFamily}{$dbVersionValue}{NT}'") if (defined($DEBUG));   
    if ($dbVersion eq $releaseInfo{$neFamily}{$dbVersionValue}{NT}) {
       return $dbVersionValue;
    }
  }
  return 0;
}

=head2 DetermineNtType()

=begin html
Return the NT type associated to the boardcode provided in $boardCode.  undef is returned wne no match is found.

=end html

=cut

sub DetermineNtType($) {
  my ($boardCode) = @_;

  foreach my $ntType (sort keys %ntCapabilities) {
    return $ntType if (defined($ntCapabilities{$ntType}{BRDCODE}) && ($boardCode == $ntCapabilities{$ntType}{BRDCODE}));
  }
  ToolCommon::Log($DEBUG, sprintf("Could not determine NT type for board code %d\n", $boardCode)) if (defined($DEBUG));
  return undef;
}

=head2 DetermineXvpsType()

=begin html
Return the XVPS type associated to the boardcode provided in $boardCode.  undef is returned wne no match is found.

=end html

=cut

sub DetermineXvpsType($) {
  my ($boardCode) = @_;

  foreach my $xvpsType (sort keys %xvpsCapabilities) {
    return $xvpsType if (defined($xvpsCapabilities{$xvpsType}{BRDCODE}) && ($boardCode == $xvpsCapabilities{$xvpsType}{BRDCODE}));
  }
  ToolCommon::Log($DEBUG, sprintf("Could not determine XVPS type for board code %d\n", $boardCode)) if (defined($DEBUG));
  return undef;
}

=head2 NtOflmtVersionString($)

=begin html
Convert an NT OFLMT version number into a string and return it to the caller.

=end html

=cut

sub NtOflmtVersionString($) {
  my ($versionNbr) = @_;
  my $major = defined($versionNbr) ?  $versionNbr / 100 : 0;
  my $minor = defined($versionNbr) ? ($versionNbr % 100) / 10 : 0;
  my $sub = defined($versionNbr) ? ($versionNbr % 1000) % 10 : 0;
  return (sprintf("%d.%d.%d", $major, $minor, $sub));
}

=head2 NtOflmtBuildString($)

=begin html
Convert an NT OFLMT Build number into a string and return it to the caller.

=end html

=cut

sub NtOflmtBuildString($) {
  my ($buildNbr) = @_;
  my $release = defined($buildNbr) ?  $buildNbr / 1000 : 0;
  my $bld = defined($buildNbr) ? ($buildNbr % 1000) : 0;
  return (sprintf("%d.%d", $release, $bld));
}

=head2 ShubOflmtVersionString($)

=begin html
Convert an SHUB OFLMT version number into a string and return it to the caller.

=end html

=cut

sub ShubOflmtVersionString($) {
  my ($versionNbr) = @_;
  my $major = defined($versionNbr) ?  $versionNbr / 100 : 0;
  my $minor = defined($versionNbr) ? ($versionNbr % 100) : 0;
  return (sprintf("%d.%d", $major, $minor));
}

=head2 HighestShubOflmtVersion($)

=begin html
Returns the highest supported SHUB Offline migrationtool version for this NE family.

=end html

=cut

sub HighestShubOflmtVersion($) {
  my ($neFamily) = @_;

  return defined($neCapabilities{$neFamily}{HIGHEST_SHUB_OFLMT}) ? $neCapabilities{$neFamily}{HIGHEST_SHUB_OFLMT} : 0;
}

=head2 HighestNtOflmtVersion($)

=begin html
Returns the highest supported NT Offline migrationtool version for this NE family.

=end html

=cut

sub HighestNtOflmtVersion($) {
  my ($neFamily) = @_;

  return defined($neCapabilities{$neFamily}{HIGHEST_NT_OFLMT}) ? $neCapabilities{$neFamily}{HIGHEST_NT_OFLMT} : 0;
}

=head2 HighestNtOflmtBuild($)

=begin html
Returns the highest supported NT Offline migrationtool build number for this NE family.

=end html

=cut

sub HighestNtOflmtBuild($) {
  my ($neFamily) = @_;

  return defined($neCapabilities{$neFamily}{HIGHEST_NT_OFLMT_BLD}) ? $neCapabilities{$neFamily}{HIGHEST_NT_OFLMT_BLD} : 0;
}

=head2 HighestXvpsOflmtVersion($)

=begin html
Returns the highest supported xVPS Offline migrationtool version for this NE family.

=end html

=cut

sub HighestXvpsOflmtVersion($) {
  my ($neFamily) = @_;

  ToolCommon::Log($DEBUG, "NE family: $neFamily\n") if defined($DEBUG);
  return defined($neCapabilities{$neFamily}{HIGHEST_IVPS_OFLMT}) ? $neCapabilities{$neFamily}{HIGHEST_IVPS_OFLMT} : 0;
}

=head2 InterfaceBase()

=begin html
Returns the interface base for the board identified by $ntType.

=end html

=cut

sub InterfaceBase($) {
  my ($ntType) = @_;

  $ntType = uc($ntType);
  return $ntCapabilities{$ntType}{IF_BASE};
}

=head2 GetReleaseFromList($$)

=begin html
Return the release hash entry from a list using the NT type and release ID.  Either returns "ALL" or a real release version

=end html

=cut

sub GetReleaseFromList($$) {
  my ($releaseNum, $list) = @_;
  my @keys = sort keys (%{$list});
  my $target = "";

  if (scalar @keys > 1) {
    my $cnt = 0;
    while (($cnt < scalar @keys) && ($target eq "")) {
      $target = $keys[$cnt] if (index($keys[$cnt], $releaseNum) != -1);
      $cnt++;
    }
    $target = "ALL" if ($target eq "");
  } else {
    $target = "ALL";
  }
  ToolCommon::Log($DEBUG, "Selected $target for '$releaseNum'\n") if defined($DEBUG);
  return $target;
}

sub IsXvpsFamily($) {
  my ($neFamily ) = @_;
  my $possibleXvpsFamilies =  IsamLSFamily() ."|". IsamFamily() ."|". IsamBothFamily() ."|". IsamEtsiFamily();
  $possibleXvpsFamilies .= "|" . IsamEtsiNttFamily() ."|". IsamEtsiExtendedFamily() ."|". IsamAnsiFamily() ."|". IsamHiCapEtsiFamily();
  if( $neFamily =~ /^($possibleXvpsFamilies)$/ ) {
     return 1;
  } else {
     return 0;
  }
}

sub IsShubFamily($) {
  my ($neFamily ) = @_;
  my $possibleShubFamilies =  IsamLSFamily() ."|". IsamFamily() ."|". IsamBothFamily() ."|". IsamEtsiFamily();
  $possibleShubFamilies .= "|" . IsamEtsiNttFamily() ."|". IsamEtsiExtendedFamily() ."|". IsamAnsiFamily();
  if( $neFamily =~ /^($possibleShubFamilies)$/ ) {
     return 1;
  } else {
     return 0;
  }
}

=head2 SelectFamily($$)

=begin html
This function selects the NE family for this NT type and release.  Release can be specified as (checked in that order):
<ul>
  <li>a release stream number (base number)</li>
  <li>A DB version string</li>
  <li>a MIB version</li>
  <li>an SWP name</li>
</ul>

=end html

=cut

sub SelectFamily($$) {
  my ($ntType, $dbVersion) = @_;
  my $family = undef;
  my $regExp = REGEXP;
  my $DBREGEXP = "\[ABCDNPV](\\d)\.(\\d)\(\\d)\\d";  # TODO:change to alphabet the first char 

  Logger::LogDebug($nodeLogFH, "ntType: '$ntType', DB-Versionn : '$dbVersion'\n");
  if ($dbVersion !~ /$DBREGEXP/) {
    $ERROR = "Invalid format in DB-Version : $dbVersion";
    return $family;
  }
  foreach my $dbVersionFormat ( keys %{$ntCapabilities{$ntType}{FAMILY}} ) {
         if( defined $dbVersionFormat and $dbVersion =~ /$dbVersionFormat/ ) {
            $family = $ntCapabilities{$ntType}{FAMILY}{$dbVersionFormat};
         }
  }
  if( !defined $family and defined $ntCapabilities{$ntType}{FAMILY}{'ALL'} ) {
        $family =  $ntCapabilities{$ntType}{FAMILY}{'ALL'};
  }
  if( defined $family ) {
      $NE_FAMILY = $family;
      $params->{'BOARD-FAMILY'} = $family;
      ToolCommon::Log($DEBUG, "ntType: '$ntType', DB-Versionn : '$dbVersion', Family : $family\n") if defined($DEBUG);
  }
  return $family;
}

=head2 IsIsamFamily()

=begin html
This function returns 1 in case the NT type is an ISAM NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamEtsiFamily()

=begin html
This function returns 1 in case the NT type is an ISAM ETSI stream NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamEtsiFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_ETSI ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamEtsiNttFamily()

=begin html
This function returns 1 in case the NT type is an ISAM ETSI NTT stream NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamEtsiNttFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_ETSI_NTT ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamAnsiFamily()

=begin html
This function returns 1 in case the NT type is an ISAM ANSI stream NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamAnsiFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_ANSI ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamBothFamily()

=begin html
This function returns 1 in case the NT type belongs to both ETSI and ANSI stream, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamBothFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_BOTH ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamEtsiExtendedFamily()

=begin html
This function returns 1 in case the NT type belongs to the extended ETSI stream (includes NTT build), 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamEtsiExtendedFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_ETSI_EXT ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamHiCapEtsiFamily()

=begin html
This function returns 1 in case the NT type is a HiCap ISAM ETSI stream NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamHiCapEtsiFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_HICAP_ETSI ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamHiCapEtsiFamily()

=begin html
This function returns 1 in case the NT type is a HiCap ISAM ANSI stream NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamHiCapAnsiFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_HICAP_ANSI ? 1 : 0;
  }
  return $result;
}

=head2 Is7534RUFamily()

=begin html
This function returns 1 in case the NT type is a 7354RU NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub Is7354RUFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_PIZZA ? 1 : 0;
  }
  return $result;
}

=head2 IsIsam7367Family()

=begin html
This function returns 1 in case the NT type belongs to 7367 family, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsam7367Family($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_TISEM ? 1 : 0;
  }
  return $result;
}

=head2 IsIsam7367_SXFamily()

=begin html
This function returns 1 in case the NT type belongs to 7367 SX family, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsam7367_SXFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_ANSI_VSRM ? 1 : 0;
  }
  return $result;
}

=head2 IsIsam7363Family()

=begin html
This function returns 1 in case the NT type belongs to 7363 family, 0 if not and
 undef if the NT type is unknown.

=end html

=cut

sub IsIsam7363Family($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_NGVR ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamLSFamily($$)

=begin html
This function returns 1 in case the NT type is a ISAM Litespan NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

sub IsIsamLSFamily($$) {
  my ($ntType, $release) = @_;
  my $result = 0;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    $result = $family eq $ISAM_LS ? 1 : 0;
  }
  return $result;
}

=head2 IsIsamLSFamily($$)

=begin html
This function returns 1 in case the NT type is a ISAM Litespan NT, 0 if not and undef if the NT type is unknown.

=end html

=cut

=head2 ReductionChars()

=begin html
Return the 2 characters for an online reduced SWP/OSWP for the NE family identified by $neFamily.<br>
$neFamily should be obtained by either calling IsamFamily() or IpdIsamFamily() of the Capability library.

=end html

=cut

sub ReductionChars($$) {
  my ($neFamily, $releaseNum) = @_;
  if (($neFamily eq $ISAM_BOTH) || ($neFamily eq $ISAM_ETSI_EXT)) {
    $neFamily =  FamilyFromOswp(basename($params->{OSWP}), undef);
  }
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$neFamily}{RED_VARIANT});

  return($neCapabilities{$neFamily}{RED_VARIANT}{$target});
}

=head2 ReductionChar()

=begin html
Return the first character for an online reduced SWP/OSWP for the NE family identified by $neFamily.<br>
$neFamily should be obtained by either calling IsamFamily() or IpdIsamFamily() of the Capability library.

=end html

=cut

sub ReductionChar($) {
  my ($neFamily) = @_;
  if (($neFamily eq $ISAM_BOTH) || ($neFamily eq $ISAM_ETSI_EXT)) {
    $neFamily =  FamilyFromOswp(basename($params->{OSWP}), undef);
  }

  return($neCapabilities{$neFamily}{RED_CHAR});
}

=head2 OswpVariant()

=begin html
Check whether the OSWP variant characters are as expected for this NE family.  Returns 1 if so, 0 in case of error or incorrect variant.

=end html

=cut

sub OswpVariant($$;$) {
  my ($oswpName, $neFamily,$paramsValue) = @_;
  my $ret = 0;
  my $regExp = REGEXP;
  my $releaseNum; 
  my $dbVer; 
  ### DEVEL-NOTE :
  ### From R5401, No need to perform OSWP variant check, Since build NT Descriptor mapping found the dynamic variant
  if( $SkipOSWPVariantCheck ) {
        $ret = 1;
        return $ret;
  }
  if (($neFamily eq $ISAM_BOTH) || ($neFamily eq $ISAM_ETSI_EXT)) {
    $neFamily = defined($paramsValue) ? FamilyFromOswp($oswpName,$paramsValue) : FamilyFromOswp($oswpName, undef);
  }
  if (basename($oswpName) =~ $VARIANT_RE) {
    my $variant = $1;
    if (basename($oswpName) =~ /$regExp/) {
      $releaseNum = defined($paramsValue) ? GetReleaseNum($oswpName) : ((($1 * 100) + ($2 * 10)) * 10);
      my $target = GetReleaseFromList($releaseNum, $neCapabilities{$neFamily}{RED_VARIANT});
      ToolCommon::Log($DEBUG, "Variant of $oswpName ($neFamily) is $variant\n") if defined($DEBUG);
      if (($variant eq $neCapabilities{$neFamily}{SW_VARIANT}{$target}) or
	  ($variant eq $neCapabilities{$neFamily}{RED_VARIANT}{$target}) or
	  ($variant eq $neCapabilities{$neFamily}{MIN_VARIANT}{$target})) {
	$ret = 1;
      } else {
	$ERROR = $variant;
      }
    }
  } else {
    $ERROR = "OSWP name '$oswpName' not according to expected syntax";
  }
  return $ret;
}

=head2 GetOswpVariant($$)

=begin html
Return the OSWP variant characters in the OSWP name for this NE family.

=end html

=cut

sub GetOswpVariant($$) {
  my ($neFamily, $releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$neFamily}{RED_VARIANT});
  my $ret = 0;

  return sprintf("%s, %s (online reduction) or %s (SW minimizing)",
		 $neCapabilities{$neFamily}{SW_VARIANT}{$target},
		 $neCapabilities{$neFamily}{RED_VARIANT}{$target},
		 $neCapabilities{$neFamily}{MIN_VARIANT}{$target})
}

=head2 OswpReduced ()

=begin html
This function returns true in case the OSWP has been online-reduced for the given NT type, 0 is not.

=end html

=cut

sub OswpReduced($$) {
  my ($swpName, $ntType) = @_;
  my $redChar;
  my $reduced = 0;
  
  if( $SkipOSWPVariantCheck ) {
    if (basename($swpName) =~ /....(.)./) {
	 if( $1 eq "R" ) {
	  $reduced = 1;
          ToolCommon::Log($DEBUG, "SkipVariantCheck : Reduction char in swp name '$swpName': '$1'.  Reduced: $reduced\n") if (defined($DEBUG));
	  return $reduced;
	 } else {
	  return $reduced;
         }
    }
	
  }

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $swpName);
  if(defined($family)) {
   if (($family eq $ISAM_BOTH) || ($family eq $ISAM_ETSI_EXT)) {
    $family = FamilyFromOswp($swpName);
   }
   if (defined($neCapabilities{$family})) {
    if (basename($swpName) =~ /....(.)./) {
      $redChar = $1;
      $reduced = 1 if ($neCapabilities{$family}{RED_CHAR} eq $redChar);
      ToolCommon::Log($DEBUG, "Reduction char in swp name '$swpName': '$redChar'.  Reduced: $reduced\n") if (defined($DEBUG));
    }
    return $reduced;
   }
  }
 return $reduced;
}

=head2 LogNeFamily()

=begin html
Log the NE family associated with the ntType.

=end html

=cut

sub LogNeFamily($$) {
  my ($ntType, $release) = @_;

  $ntType = uc($ntType);
  my $family = SelectFamily($ntType, $release);
  if (defined($family)) {
    if (defined($DEBUG)) {
      ToolCommon::Log($DEBUG, sprintf("NE family: %s\n", $family));
    }
  } else {
    ToolCommon::Log($DEBUG, sprintf("Could not determine NE family for NT '$ntType' and release '$release'\n")) if defined($DEBUG);
  }
  return $family;
}

=head2 IsamLS2IsamRelease

=begin html
Convert the ISAM LS release into an ISAM release

=end html

=cut

sub IsamLS2IsamRelease($) {
  my ($fromRelease) = @_;
  my $baseFrom;
  my $isamRelease = undef;

  ToolCommon::Log($DEBUG, sprintf("Keys before: %s\n", join(" ", keys %{$releaseInfo{$ISAM_LS}}))) if defined($DEBUG);
  $baseFrom = defined($releaseInfo{$ISAM_LS}{$fromRelease}) ? $fromRelease : BaseRelease($fromRelease);
  ToolCommon::Log($DEBUG, sprintf("Keys after: %s\n", join(" ", keys %{$releaseInfo{$ISAM_LS}}))) if defined($DEBUG);
  if (defined($releaseInfo{$ISAM_LS}{$baseFrom}) ){
    $isamRelease = defined($releaseInfo{$ISAM_LS}{$baseFrom}{ISAM_REL}) ? $releaseInfo{$ISAM_LS}{$baseFrom}{ISAM_REL} : undef;
  }
  ToolCommon::Log($DEBUG, sprintf("Converted ISAM LS release '%s' (%s) to ISAM release '%s'\n",
				  $fromRelease, $baseFrom, defined($isamRelease) ? $isamRelease : 'not defined')) if defined($DEBUG);
  return $isamRelease;
}

=head2 Isam2IsamLSRelease

=begin html
Convert the ISAM LS release into an ISAM release

=end html

=cut

sub Isam2IsamLSRelease($) {
  my ($fromRelease) = @_;
  my $baseFrom = BaseRelease($fromRelease);
  my $isamLSRelease = undef;
  my @keys = (sort keys %{$releaseInfo{$ISAM_LS}});

  ToolCommon::Log($DEBUG, "Keys: @keys\n") if defined($DEBUG);
  foreach my $relNum (@keys) {
    ToolCommon::Log($DEBUG, "Probing release $relNum\n") if defined($DEBUG);
    if ($releaseInfo{$ISAM_LS}{$relNum}{ISAM_REL} == $fromRelease) {
      $isamLSRelease = $relNum;
      last;
    }
  }
  if (! defined($isamLSRelease)) {
    @keys = sort keys %{$releaseInfo{$ISAM_LS}};
    ToolCommon::Log($DEBUG, "Probing for match using base-release $baseFrom...\n") if defined($DEBUG);
    ToolCommon::Log($DEBUG, "Keys: @keys\n") if defined($DEBUG);
    foreach my $relNum (@keys) {
      ToolCommon::Log($DEBUG, "Probing base-release $relNum\n") if defined($DEBUG);
      if ($releaseInfo{$ISAM_LS}{$relNum}{ISAM_REL} == $baseFrom) {
        $isamLSRelease = $relNum;
        last;
      }
    }
  }
  ToolCommon::Log($DEBUG, sprintf("Converted ISAM release '%s' (%s) to ISAM LS release '%s'\n",
				  $fromRelease, $baseFrom, defined($isamLSRelease) ? $isamLSRelease : 'not defined')) if defined($DEBUG);
  return $isamLSRelease;
}

=head2 IsamHiCapEtsiFamily()

=begin html
This functions allows to specify the ISAM HICAP NE ETSI family in calls to the capability model.

=end html

=cut

sub IsamHiCapEtsiFamily() {
  return $ISAM_HICAP_ETSI;
}

=head2 IsamHiCapAnsiFamily()

=begin html
This functions allows to specify the ISAM HICAP NE ANSI family in calls to the capability model.

=end html

=cut

sub IsamHiCapAnsiFamily() {
  return $ISAM_HICAP_ANSI;
}

=head2 IsamFamily()

=begin html
This functions allows to specify the ISAM NE family in calls to the capability model.

=end html

=cut

sub IsamFamily() {
  return $ISAM;
}

=head2 IsamEtsiFamily()

=begin html
This functions allows to specify the ISAM ETSI NE family in calls to the capability model.

=end html

=cut

sub IsamEtsiFamily() {
  return $ISAM_ETSI;
}

=head2 IsamAnsiFamily()

=begin html
This functions allows to specify the ISAM ANSI NE family in calls to the capability model.

=end html

=cut

sub IsamAnsiFamily() {
  return $ISAM_ANSI;
}

=head2 IsamBothFamily()

=begin html
This functions allows to specify the combined ISAM ETSI and ANSI NE family in calls to the capability model.

=end html

=cut

sub IsamBothFamily() {
  return $ISAM_BOTH;
}

=head2 IsamEtsiExtendedFamily()

=begin html
This functions allows to specify the extended ISAM ETSI NE family in calls to the capability model.

=end html

=cut

sub IsamEtsiExtendedFamily() {
  return $ISAM_ETSI_EXT;
}

=head2 IsamEtsiNttFamily()

=begin html
This functions allows to specify the ISAM ETSI NTT NE family in calls to the capability model.

=end html

=cut

sub IsamEtsiNttFamily() {
  return $ISAM_ETSI_NTT;
}

=head2 IsamLSFamily()

=begin html
This functions allows to specify the ISAM Litespan NE family in calls to the capability model.

=end html

=cut

sub IsamLSFamily() {
  return $ISAM_LS;
}

=head2 7354RUFamily()

=begin html
This functions allows to specify the 7354RU NE family in calls to the capability model.

=end html

=cut

sub RU7354Family() {
  return $ISAM_PIZZA;
}

=head2 Isam7367Family()

=begin html
This functions allows to specify the TISEM family in calls to the capability model.

=end html

=cut

sub Isam7367Family() {
  return $ISAM_TISEM;
}

=head2 Isam7367_SXFamily()

=begin html
This functions allows to specify the ANSI VSRM SX family in calls to the capability model.

=end html

=cut

sub Isam7367_SXFamily() {
  return $ISAM_ANSI_VSRM;
}

=head2 Isam7363Family()

=begin html
This functions allows to specify the NGVR family in calls to the capability mode
l.

=end html

=cut

sub Isam7363Family() {
  return $ISAM_NGVR;
}

=head2 IsamOswpVariant($)

=begin html
This function returns the ISAM NE OSWP variant in calls to the capability model.

=end html

=cut

sub IsamOswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM}{SW_VARIANT});

  return $neCapabilities{$ISAM}{'SW_VARIANT'}{$target};
}

=head2 IsamHiCapEtsiOswpVariant($)

=begin html
This function returns the HICAP ISAM NE OSWP variant in calls to the capability model.

=end html

=cut

sub IsamHiCapEtsiOswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_HICAP_ETSI}{SW_VARIANT});

  return $neCapabilities{$ISAM_HICAP_ETSI}{'SW_VARIANT'}{$target};
}

=head2 IsamEtsiOswpVariant($)

=begin html
This function returns the ISAM ETSI NE OSWP variant in calls to the capability model.

=end html

=cut

sub IsamEtsiOswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_ETSI}{SW_VARIANT});

  return $neCapabilities{$ISAM_ETSI}{'SW_VARIANT'}{$target};
}

=head2 IsamEtsiNttOswpVariant($)

=begin html
This function returns the ISAM ETSI NTT NE OSWP variant in calls to the capability model.

=end html

=cut

sub IsamEtsiNttOswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_ETSI_NTT}{SW_VARIANT});

  return $neCapabilities{$ISAM_ETSI_NTT}{'SW_VARIANT'}{$target};
}

=head2 IsamLSOswpVariant($)

=begin html
This function returns the ISAM Litespan NE OSWP variant in calls to the capability model.

=end html

=cut

sub IsamLSOswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_LS}{SW_VARIANT});

  return $neCapabilities{$ISAM_LS}{'SW_VARIANT'}{$target};
}

=head2 IsamAnsiOswpVariant($)

=begin html
This function returns the ISAM ANSI NE OSWP variant in calls to the capability model.

=end html

=cut

sub IsamAnsiOswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_ANSI}{SW_VARIANT});

  return $neCapabilities{$ISAM_ANSI}{'SW_VARIANT'}{$target};
}

=head2 Isam7367OswpVariant($)

=begin html
This function returns the ISAM 7367 OSWP variant in calls to the capability model.

=end html

=cut

sub Isam7367OswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_TISEM}{SW_VARIANT});

  return $neCapabilities{$ISAM_TISEM}{'SW_VARIANT'}{$target};
}

=head2 Isam7367_SXOswpVariant($)

=begin html
This function returns the ISAM 7367 SX OSWP variant in calls to the capability model.

=end html

=cut

sub Isam7367_SXOswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_ANSI_VSRM}{SW_VARIANT});

  return $neCapabilities{$ISAM_ANSI_VSRM}{'SW_VARIANT'}{$target};
}

=head2 Isam7363OswpVariant($)

=begin html
This function returns the ISAM 7363 OSWP variant in calls to the capability mode
l.

=end html

=cut

sub Isam7363OswpVariant($) {
  my ($releaseNum) = @_;
  my $target = GetReleaseFromList($releaseNum, $neCapabilities{$ISAM_NGVR}{SW_VARIANT});

  return $neCapabilities{$ISAM_NGVR}{'SW_VARIANT'}{$target};
}

=head2 FamilyIdInList($$$)

=begin html
Verify whether a given NE family ID is in the provided list using the release number provided.

=end html

=cut

sub FamilyIdInList($$$) {
  my ($familyId, $releaseNum, $list) = @_;
  my $target = GetReleaseFromList($releaseNum, $list);

  return ($list->{$target} eq $familyId) ? 1 : 0;
}

=head2 FamilyFromOswp($;$) 

=begin html
Return the true ISAM family from the OSWP name.<br>
Note that only ISAM 7302 is convered in this function, so no support for GPON or 7354RU.  This may come as needed.  Currently only required for
ISAM 7302 since NT types can belong to both ISAM streams.

=end html

=cut

sub FamilyFromOswp($;$) {
  my ($oswpName,$paramsValue) = @_;
  $oswpName = basename($oswpName);
  my $familyChars = substr($oswpName, 4, 2);  # Starts counting from 0 not 1....
  my $familyId = substr($oswpName, 5, 1);
  my $oswpChar = substr($oswpName, 4, 1);
  my $family = undef;
  my $regExp = REGEXP;
  my $releaseNum;
  my $dbVer = undef; 

  ToolCommon::Log($DEBUG, "familyChars: '$familyChars', familyId: '$familyId', oswpChar: '$oswpChar'\n") if defined($DEBUG);
  if (basename($oswpName) =~ /$regExp/) {
    $releaseNum = defined($paramsValue) ? GetReleaseNum($oswpName) : ((($1 * 100) + ($2 * 10)) * 10);

    if (($familyChars eq "ZZ") || ($familyChars eq "RP")) {
      # IDs used till V2.0, i.e. prior to stream split.  This chars in the OSWP mean that we're dealing with an "old" ISAM NE
      if($releaseNum < 4000) {
          $family = $ISAM;
         return $family; 
      }
    } 
    if (FamilyIdInList($familyId, $releaseNum, $neCapabilities{$ISAM_ANSI}{FAMILY_ID})) {
      #if ($familyId eq $neCapabilities{$ISAM_ANSI}{FAMILY_ID}) {
	$family = $ISAM_ANSI;
#      } elsif ($familyId eq $neCapabilities{$ISAM_ETSI}{FAMILY_ID}) {
    } elsif (FamilyIdInList($familyId, $releaseNum, $neCapabilities{$ISAM_ETSI}{FAMILY_ID})) {
	$family = $ISAM_ETSI;
#      } elsif ($familyId eq $neCapabilities{$ISAM_ETSI_NTT}{FAMILY_ID}) {
    } elsif (FamilyIdInList($familyId, $releaseNum, $neCapabilities{$ISAM_ETSI_NTT}{FAMILY_ID})) {
	$family = $ISAM_ETSI_NTT;
#      } elsif ($familyId eq $neCapabilities{$ISAM_HICAP_ETSI}{FAMILY_ID}) {
    } elsif (FamilyIdInList($familyId, $releaseNum, $neCapabilities{$ISAM_HICAP_ETSI}{FAMILY_ID})) {
	$family = $ISAM_HICAP_ETSI;
    }
    if(!defined($family)) {
        if($releaseNum == 4200) {
           if(($familyId eq 'Y') || ($familyId eq 'Z')) {
              $family = $ISAM_ETSI;
           }
        } elsif ($releaseNum == 4300) {
           if($familyId eq 'W') {
              $family = $ISAM_ETSI;
           }
        } elsif ( $NE_FAMILY ne "" ) {
	   $family = $NE_FAMILY;
        }
    }

    ToolCommon::Log($DEBUG, sprintf("Found NE family '%s'\n", defined($family) ? $family : "not found")) if defined($DEBUG);
  }
  return $family;
}

=head2 BuildNtOflCap()

=begin html
Build NT migration capabilities based on the migration paths defined in the NT OFLMT config file.  This is done to determine whether migration is
possible as early as possible.

=end html

=cut

sub BuildNtOflCap($) {
  my ($configFile) = @_;
  my $err = SUCCESS_CODE;
  my $migCapRead = 0;
  my $migCapStarted = 0;
  my $migCloneStarted = 0;

  if (open(CONF, $configFile)) {
    while (! $migCapRead) {
      my $line = <CONF>;
      if ($line =~ /migrpaths\s=\s\[/) {
	$migCapStarted = 1;
      } elsif (($line =~ /\]/) && $migCapStarted) {
	$migCapRead = 1;
      } elsif (($line =~ /\]/) && $migCloneStarted) {
	$migCloneStarted = 0;
      } elsif ($line =~ /cloningpaths\s=\s\[/) {
	$migCloneStarted = 1;
      } elsif ($migCloneStarted && ($line =~ /\{(\w\d\.\d\d\d),(\w\d\.\d\d\d)\}/)) {
	chomp($line);
	ToolCommon::Log($DEBUG, "$line\n", undef, 1) if (defined($DEBUG));
	my $from = $1;
	my $to = $2;
	$ntOflClones{$to} = "$from";
	ToolCommon::Log($DEBUG, sprintf("Defining clone %s similar as %s\n", $to, $ntOflClones{$to}), undef, 1) if (defined($DEBUG));	
      } elsif ($migCapStarted && ($line =~ /\{(\w\d\.\d\d\d),(\w\d\.\d\d\d)\}/)) {
	chomp($line);
	ToolCommon::Log($DEBUG, "$line\n", undef, 1) if (defined($DEBUG));
	my $from = $1;
	my $to = $2;
	$ntOflCap{$from} = "$to";
	ToolCommon::Log($DEBUG, sprintf("Defining migration from %s to %s\n", $from, $ntOflCap{$from}), undef, 1) if (defined($DEBUG));
      }
    }
    close(CONF);
    foreach my $rel (sort keys %ntOflCap) {
      foreach my $fixRel (sort keys %ntOflCap) {
	ToolCommon::Log($DEBUG, "rel: $rel, fixRel: $fixRel\n", undef, 1) if (defined($DEBUG));
	if ($fixRel ne $rel) {
	  if (index($ntOflCap{$rel}, "$fixRel") != -1) {
	    $ntOflCap{$rel} .= " $ntOflCap{$fixRel}";
	    ToolCommon::Log($DEBUG, sprintf("Adding migration for %s: %s\n", $rel, $ntOflCap{$rel}), undef, 1) if (defined($DEBUG));
	  }
	}
      }
    }
    if (defined($DEBUG)) {
      ToolCommon::Log($DEBUG, "NT OFLMT capabilities:\n", undef, 1);
      for my $rel (sort keys %ntOflCap) {
	ToolCommon::Log($DEBUG, sprintf("%s -> %s\n", $rel, $ntOflCap{$rel}), undef, 1);
      }
    }
  } else {
    $err = FILE_OPEN_FAIL_CODE;
  }
  return($err);
}

=head2 NtOflCapDefined()

=begin html
Determine whether NT OFLMT is possible between $from and $to.  1 is returned when supported, undef if not.

=end html

=cut

sub NtOflCapDefined($$) {
  my ($from, $to) = @_;

  ToolCommon::Log($DEBUG, "Verify $from -> $to\n", undef, 1) if (defined($DEBUG));
  if (!defined($ntOflCap{$from})) {
    # Check whether this release has some clones and whether cloning between from and to has been defined
    if (defined($ntOflClones{$to})) {
      if (index($ntOflClones{$to}, $from) == -1) {
	return undef;
      }
    }
  } else {
    if (index($ntOflCap{$from}, $to) == -1) {
      if (defined($ntOflClones{$to})) {
	if (index($ntOflClones{$to}, $from) == -1) {
	  return undef;
	}
      }
    }
  }
  return 1;
}

=head2 SafeNtIcs()

=begin html
Return the safe ICS code associated to $ntType.

=end html

=cut

sub SafeNtIcs($) {
  my ($ntType) = @_;

  $ntType = uc($ntType);
  return $ntCapabilities{$ntType}{SAFE_ICS};
}

=head2 SafeShubOflmt()

=begin html
Checks whether the version of the SHUB offline migration tools is ok for the release specified.  The release and tool version number have been
"liniarized".<br>
Returns 1 if safe, 0 if not.

=end html

=cut

sub SafeShubOflmt($$$) {
  my ($neFamily, $relNum, $version) = @_;
  my $ok = 1;
  my $baseRelNum;

  if (! defined($releaseInfo{$neFamily}{$relNum})) {
    $baseRelNum = BaseRelease($relNum);
  } else {
    $baseRelNum = $relNum;
  }
  if (defined($releaseInfo{$neFamily}{$baseRelNum}{TOOLS})) {
    $ok = 0 if ($version < $releaseInfo{$neFamily}{$baseRelNum}{TOOLS}{SHUB});
  }
  return $ok;
}

=head2 McastPackageCountOK()

=begin html
Check whether the amount of packages in a MCAST src is not exceeding the maximum.  Returns 0 if not ok.
Note:<br>
<ul>
  <li>Maximum till R2.4: 64</li>
  <li>Maximum from R2.4: 1024</li>
  <li>Maximum from R3.4: 20 or 1024 (64 will be converted internally to 1024 by the offline migration tool).</li>
</ul>

=end html

=cut

my $TILL_R24 = 64;
my $FROM_R24 = 1024;

sub McastPackageCountOK($$$) {
  my ($packages, $highestPkgNum, $srcAddr) = @_;
  my $maxPkg = 0;
  my $err = "";

  if ((($packages != $TILL_R24) && ($packages != $FROM_R24)) && ($packages > $mcastSrcLimits{MCSRC_PKGLIMIT})) {
    $err = sprintf("Excess packages for multicast source %s (counted %d, only %d allowed)\n",
                   $srcAddr, $packages, $mcastSrcLimits{MCSRC_PKGLIMIT});
  } elsif (($packages == $TILL_R24) && ($highestPkgNum > $TILL_R24)) {
    $err = sprintf("%d packages counted for multicast source %s but highest package number too high ".
		   "(%d detected while only %d allowed as highest package number in this case)\n", $TILL_R24, $srcAddr, $highestPkgNum, $TILL_R24);
  } else {
    # OK.
  }
  return $err;
}

=head2 GetMcastSrcOID()

=begin html
Returns the MCAST SRC OID associated to the release identified by $relNum.

=end html

=cut

sub GetMcastSrcOID($) {
  my ($relNum) = @_;

  return ReleaseCompare($relNum, Capability::ConvertRelease(3, 4), REGEXP, 0) <= 0 ?
    $mcastSrcLimits{MCSRC_PKG}{R34} : $mcastSrcLimits{MCSRC_PKG}{R35};
}

=head2 GetFirstLtOffset()

=begin html
Returns an offset to be used when calculating LT ifIndexes.  It takes the equipment type and CLI mode into account.

=end html

=cut

sub GetFirstLtOffset($$) {
  my ($profileMode, $ntType) = @_;
  my $ret = undef;

  $ntType = uc($ntType);
  ToolCommon::Log($DEBUG, "Determine LT offset for profile-mode $profileMode and NT $ntType\n") if defined($DEBUG);
  if (defined($profileModes{$ntCapabilities{$ntType}{EQPT_TYPE}}{uc($profileMode)}{OFFSET})) {
    $ret = $profileModes{$ntCapabilities{$ntType}{EQPT_TYPE}}{uc($profileMode)}{OFFSET};
  }
  return $ret;
}

=head2 GetLtInternalNum()

=begin html
Returns the internal LT number based on the CLI profile mode, NT type and LT number read.

=end html

=cut

sub GetLtInternalNum($$$) {
  my ($profileMode, $ntType, $ltNum) = @_;
  my $ret = undef;

  $ntType = uc($ntType);
  ToolCommon::Log($DEBUG, "Determine internal LT number for profile-mode $profileMode, NT $ntType and LT $ltNum\n") if defined($DEBUG);
  if (defined($profileModes{$ntCapabilities{$ntType}{EQPT_TYPE}}{uc($profileMode)}{LTMAP}{$ltNum})) {
    $ret = $profileModes{$ntCapabilities{$ntType}{EQPT_TYPE}}{uc($profileMode)}{LTMAP}{$ltNum};
  }
  return $ret;
}

=head2 ConvertLtInternalNum()

=begin html
Returns the CLI LT number LT number based on the CLI profile mode, NT type and internal LT number read.

=end html

=cut

sub ConvertLtInternalNum($$$) {
  my ($profileMode, $ntType, $ltNum) = @_;
  my $ret = undef;

  ToolCommon::Log($DEBUG, "Determine internal LT number for profile-mode $profileMode, NT $ntType and LT $ltNum\n") if defined($DEBUG);
  if (defined($profileModes{$ntCapabilities{$ntType}{EQPT_TYPE}}{uc($profileMode)}{INVMAP}{$ltNum})) {
    $ret = $profileModes{$ntCapabilities{$ntType}{EQPT_TYPE}}{uc($profileMode)}{INVMAP}{$ltNum};
  }
  return $ret;
}


=head2 ConvertLtslot()

=begin html
Returns the LT number for lt slot when configured in TYPE-BASED numbering in profile mode

=end html

=cut

sub ConvertLTslot($$@) {
  my ($shelfType, $ntType, @feltbLts) = @_;
  my @ret = ();
  my $ltSlot = "";
  my @ltsplit = ();
  foreach (@feltbLts) {
	$ltSlot = $_;
	@ltsplit = split ("\/", $ltSlot);
	if (defined($ltSlotNum{$ntCapabilities{$ntType}{EQPT_TYPE}}{$shelfType}{LTMAP}{$ltsplit[2]})) {
		$ltsplit[2] = $ltSlotNum{$ntCapabilities{$ntType}{EQPT_TYPE}}{$shelfType}{LTMAP}{$ltsplit[2]};
	}
	$ltSlot = join ("/",@ltsplit);
	push @ret, $ltSlot;
	}
  return \@ret;
}




=head2 MaxMacPerLsm()

=begin html
Return the maximum number of MAC addresses per LSM for this LT type.<br>
In case the LT type is unknown to the model we return 1024 ($MAXSTATICUNICASTMACPERLT)

=end html

=cut

sub MaxMacPerLsm($) {
  my ($ltType) = @_;
  my $limit = defined($ltLimits{MAX_MACS_PER_LSM}{uc($ltType)}) ? $ltLimits{MAX_MACS_PER_LSM}{uc($ltType)} : $MAXSTATICUNICASTMACPERLT;

  ToolCommon::Log($DEBUG, "Return maximum number of MAC addresses per LT for LT type $ltType: $limit\n") if defined($DEBUG);
  return $limit;
}

=head2 ReadSWP()

=begin html
Read the contents of the SWP file to look for the complete name of the SHUB SW.<br>
The following parameters are used:
<table>
  <tr><td>swpName</td><td>name of the SW package file</td></tr>
  <tr><td>shubSwPrefix</td><td>SHUB SW prefix to look for</td></tr>
  <tr><td>tftpDir</td><td>TFTP top directory setting</td></tr>
</table>
Returns the name of the SHUB SW or an empty string in case it was not found.

=end html

=cut

sub ReadSWP($$$) {
  my ($swpName, $shubSwPrefix, $tftpDir) = @_;
  my $fileName = "";
  my $line;

  if (open(SWP, "$tftpDir/$swpName")) {
    while (defined($line = <SWP>) && ($fileName eq "")) {
      chomp $line;
      if ($line =~ /^\s*TYPE\-H\s*\:\s*($shubSwPrefix\.\d\d\d).*/) {
	$fileName = $1;
      }
    }
    close(SWP);
  }
  return $fileName;
}

=head2 FindShubSwName()

=begin html
Return the name SHUB SW file.<br>
The following parameters are used:
<table>
  <tr><td>swpDirPref</td><td>Path/name of SWP</td></tr>
  <tr><td>regExp</td><td>Regular expression as to determine the release of the SWP</td></tr>
  <tr><td>tftpDir</td><td>TFTP top directory</td></tr>
  <tr><td>shubSwPath</td><td>Path where SHUB SW can be found (as defined in param.cfg)</td></tr>
  <tr><td>shubSwName</td><td>SHUB SW name (as defined in param.cfg)</td></tr>
  <tr><td>neFamily</td><td>NE family</td></tr>
</table>
Returns the full name of the SHUB SW or an empty string in case it was not found.

=end html

=cut

sub FindShubSwName($$$$$$) {
  my ($swpDirPref, $regExp, $tftpDir, $shubSwPath, $shubSwName, $neFamily) = @_;
  my $releaseId = "";
  my $fileName = "";
  my $swDir = dirname($swpDirPref);
  my $dbVersion = $params->{DBVER_TGTREL};
  if (basename($swpDirPref) =~ /$regExp/) {
       $releaseId = ConvertRelease($1, $2, undef);
       ToolCommon::Log($DEBUG, "TFTPDIR - $tftpDir, SWPNAME - $swpDirPref, NE-FAMILY-$neFamily, DB-VERSION - $dbVersion'\n") if defined($DEBUG);
       if ($releaseId == 9900) { # -- 1.4.0
            $fileName = "Notneeded";
            ToolCommon::Log($DEBUG, "fileName: '$fileName'\n") if defined($DEBUG);
            return $fileName;
       }
       if (defined($shubSwPath) && ($shubSwPath ne "")) {
	       if (defined(ShubSwName($dbVersion, $neFamily ))) {
	         $swDir = $shubSwPath;
	         if (defined($shubSwName) && ($shubSwName ne "")) {
	           $fileName = $swDir . "/" . $shubSwName;
	         } else {
	           $fileName = $swDir . "/" . ShubSwName($dbVersion, $neFamily );
	         }
	       }
       } else {
	        if (defined ShubSwName($dbVersion, $neFamily ) ) {
	          if ($releaseId > 2200) {
	            $fileName = ReadSWP($swpDirPref, ShubSwName($dbVersion, $neFamily ), $tftpDir);
	            if ($fileName ne "") {
	              $fileName = $swDir . "/" . $fileName;
	            }
	          } else {
	            $fileName = $swDir . "/" . ShubSwName($dbVersion, $neFamily );
	          }
	        }
       }
  }
  ToolCommon::Log($DEBUG, "fileName: '$fileName'\n") if defined($DEBUG);
  return $fileName;
}

sub ShubSwName($$) {
  my ($dbVersion, $neFamily) = @_;

  return undef if ( ! IsShubFamily($neFamily) );

  if( defined($releaseInfo{$neFamily}{$dbVersion}) and defined($releaseInfo{$neFamily}{$dbVersion}{SHUB_NAME}) ) {
      return $releaseInfo{$neFamily}{$dbVersion}{SHUB_NAME};
  }
  if ( $dbVersion =~ /[A-Z](.*)/ ) {
       $dbVersion = $1;
       $dbVersion =~ s/\.//;
  }
  my $release = substr($dbVersion,0,2);
  return "MA6YAA$release";
}


=head2 PrintStreamInfo()

=begin html
Print the capabilities of the PBMT toolset.  Executed when --cap option is ued.

=end html

=cut

sub PrintStreamInfo() {

  print STDERR "NT Details:\n";
  my $str = "";
  $str = sprintf("%10s %10s %10s %20s => %s:\n", 'NT_BOARD', 'BOARD-CODE', 'SW_PAR', 'FAMILY' , 'DB_VERSION-REGEX' );
  print STDERR $str;
  foreach my $board (sort keys %ntCapabilities) {
      foreach my $family ( sort keys %{$ntCapabilities{$board}{FAMILY}} ) {
	  $str = sprintf("%10s %10s %10s %20s => %s:\n",
	        $board,
	        defined($ntCapabilities{$board}{BRDCODE}) ? $ntCapabilities{$board}{BRDCODE} : "NA",
	        defined($ntCapabilities{$board}{SW_PAR}) ? $ntCapabilities{$board}{SW_PAR} : "NA",
            $ntCapabilities{$board}{FAMILY}{$family}, $family  );
            print STDERR $str;
      }
  }
}

=head2 GetTransactValue()

=begin html
Get the Transaction timeout value depending on the release

=end html

=cut

my $TRASACT_TIME_R24 =  3600000; 

sub GetTransactValue($) {
  my ($relNum) = @_;
  my $transact_time = TRANSACT_TIME; 

  if (ReleaseCompare($relNum, ConvertRelease(2, 4), REGEXP, 0) <= 0) {
	# fix for ANTmt83962 - for releases <= R2.4 unit of Transaction Timeout is in millisecs
	$transact_time = $TRASACT_TIME_R24;    
  }
  ToolCommon::Log($DEBUG, "Transaction Timeout set to: $transact_time\n") if ($params->{DEBUG});
  return $transact_time; 
}

=head2 CheckHiCapLTPresent()

=begin html
Check whether the HiCap LT (GPON LT) is present in the NE
Returns 1 in case HiCap LT is present and 0 otherwise

=end html

=cut

sub CheckHiCapLTPresent($) {
  my ($LT) = @_;
  ## Check passed LT is HiCap one (NGLT-A, NGLT-B, FGLT-A, FGLT-B, FWLT-A, FWLT-B)
  if( $LT =~ /^[A-Z]([A-Z]{3})-[A-Z]$/ ) {
	return 1 if( $1 eq "GLT" or $1 eq "WLT" );
  }
  return 0; 
}

=head2 CheckGponLTExists()

=begin html
Check whether the HiCap LT (GPON LT) is enabled
Returns 1 in case HiCap LT is in enabled state in the NE and 0 otherwise

=end html

=cut

sub CheckGponLTExists($) {    
  my ($cli) = @_; 
  my $GponLTEnabled = 0;
  my $ok = 1;     # return 1 if GPON LT is enabled
  my $ret;

  $ERROR = "";

  ToolCommon::Log($DEBUG, "Check whether any HICAP LT connected to NE and is ENABLED...\n") if defined($DEBUG);

  my $ltInfo = IsamCli::GetLtConfigState($cli);
  if (!defined($ltInfo)) {
    $ERROR = sprintf("Retrieving LT configuration state failed: %s", IsamCli::GetError());
    $ok = 0; 
  } else {
           foreach my $lt (keys %{$ltInfo}) {
                        $ret = CheckHiCapLTPresent(uc($ltInfo->{$lt}{TYPE}));
                        if($ret) {
                                if($ltInfo->{$lt}{ENABLED} eq "YES") {
                                        ToolCommon::Log($DEBUG, "HICAP LT $ltInfo->{$lt}{TYPE} present in Slot $lt and is Enabled...\n") if defined($DEBUG);
  					$GponLTEnabled = 1;
                                } else {
                                        ToolCommon::Log($DEBUG, "HICAP LT $ltInfo->{$lt}{TYPE} present in Slot $lt but NOT Enabled...\n") if defined($DEBUG);
                                }
                        }
           }
           if(!$GponLTEnabled) {
                        ToolCommon::Log($DEBUG, "HICAP LT not present in NE or HICAP LT not Enabled...\n") if defined($DEBUG);
			$ok = 0; 
           }
  }
  return $ok; 
}

sub GetReleaseFromDBVersion($) {
  my ($dbVersion) = @_;
  my $releaseNum = undef;

  my $dbVersionRel = substr($dbVersion,1);
  $dbVersionRel =~ s/\.//;
  return $dbVersionRel;
}

sub GetCDEMigrFromRelease($) {
  my ($toDbVersion) = @_;
  my $fromDBversion = undef;

  if(defined($CDERelDBVersMap{$toDbVersion})) {
     $fromDBversion = $CDERelDBVersMap{$toDbVersion};
  }
  return $fromDBversion;
}

sub GetReleaseNum($) {
  my ($oswpName) = @_;
  my $releaseNum = undef; 

  #my $dbVer = (basename(->{OSWP}) eq $oswpName) ? $params->{DBVER_TGTREL} : $params->{DBVER_SRCREL} if(defined($params));
  my $dbVer = $params->{DBVER_TGTREL} if(defined($params));
  $releaseNum = GetReleaseFromDBVersion($dbVer) if(defined($dbVer)); 

  return $releaseNum;
}

=head2 ChangeEqptType($$)

=begin html
Change Equipment Type for AGNT-A (ERAM-A) when used 
in ANSI stream to XD equipment type (default is FD type)

=end html

=cut

sub ChangeEqptType($$)
{
  my ($profileMode, $ntType) = @_;
  if((uc($profileMode) =~ /$LEGACY/) && (uc($ntType) eq $NT_ERAMA)) {
       ToolCommon::Log($DEBUG, "AGNT-A used in ANSI stream, changing EQPT type\n") if defined($DEBUG);
       $ntCapabilities{$ntType}{EQPT_TYPE} = $XD; 
  }
  return 1; 
}

=head2 IsMiscLT()

=begin html
Check for additional LT's which do not have LT suffix in them
eg. vsem-c, vsem-d do not have lt suffix in contrast to
eblt-c, nalt-c etc.  which match regex /lt/
Returns 1 in case these LT's are present and 0 otherwise

=end html

=cut

sub IsMiscLT($) {
  my ($LT) = @_;

  for (my $count =0; $count < scalar @MiscLT; $count++) {
	return 1 if($LT eq $MiscLT[$count]); 
  }
  return 0; 
}

sub IsANSIShelf($) {
  my ($shelfType) = @_;

  return 0 if(!defined($shelfType) || ($shelfType eq "")); 

  for (my $count =0; $count < scalar @ANSIShelf; $count++) {
	return 1 if(uc($shelfType) eq $ANSIShelf[$count]); 
  }
  return 0; 
}

sub IsNEPRUShelf($) {
  my ($shelfType) = @_;

  return 0 if(!defined($shelfType) || ($shelfType eq "")); 

  if(uc($shelfType) eq $NEPRU) {
     return 1; 
  }

  return 0; 
}

sub IsTypeBased($) {
  my ($profileMode) = @_;

  return 0 if(!defined($profileMode) || ($profileMode eq "")); 

  if(uc($profileMode) =~ /$TYPE/) {
     return 1; 
  }

  return 0; 
}

#-- End of package
1;
