package CapabilityReEngr;

use strict;
use warnings 'all';
use PBMTGlobal;
use Logger;

my %ReleaseDBVer = ();
my %CloningPaths = ();
my %MigrationPaths = ();
my $DBToolConfig = 0;

=begin html
Check whether migration is possible.<br>The following parameters are used:
<table>
  <tr><td>ntType</td><td>the NT type involved in the migration (string)</td></tr>
  <tr><td>from</td><td>the release currently active</td></tr>
  <tr><td>to:</td><td>the new release</td></tr>
</table>
Releasenumbers need to have been converted using <tt>ConvertRelease</tt>.<br>
It returns whether migration is possible and a reason in case it is not possible.

=end html

=cut

sub MigrationPossible($$$) {
  my ($ntType, $sourceDbVersion, $targetDbVersion) = @_;
  my ($possible, $reason) = (1,"");
  $ntType = uc($ntType);
  Logger::LogNotify( $nodeLogFH, "Migration Path : Source '$sourceDbVersion' => Target '$targetDbVersion' \n");

  if( not $DBToolConfig ) {
      $possible = 0;
      $reason = "Migration path check config not exists";
  } elsif( !defined $ReleaseDBVer{$sourceDbVersion} ) {
        $possible = 0;
        $reason = "active release is not in Migrate path";
  } elsif( !defined $ReleaseDBVer{$targetDbVersion} ) {
        $possible = 0;
        $reason = "target release is not in Migrate path";
  } elsif( $ReleaseDBVer{$sourceDbVersion} gt $ReleaseDBVer{$targetDbVersion}  ) {
      $possible = 0;
      $reason = "Target release is lower than active ";
  }

  if( !$possible ) {
     return ($possible, $reason);
  }

  my $hopCount = 0;
  my $fromDbVersion = $sourceDbVersion;
  my $toDbVersion = "";
  $reason = sprintf(" Source Release %s '%s'; Target Release %s '%s'\n",
                     $ReleaseDBVer{$sourceDbVersion}, $sourceDbVersion, $ReleaseDBVer{$targetDbVersion}, $targetDbVersion );
  while ( $possible ) {
      if( not defined $MigrationPaths{$fromDbVersion} ) {
           $possible = 0;
           $reason = "Migration paths are not valid/available";
           last;
      }
	  $hopCount++;
      $toDbVersion = $MigrationPaths{$fromDbVersion};
      Logger::LogNotify( $nodeLogFH, "Migration Path : $fromDbVersion => $toDbVersion\n");
	  if( $toDbVersion eq $targetDbVersion ) {
	     last;
      }
      my $targetDbVersionRel = substr($targetDbVersion,1);
      my $toDbVersionRel     = substr($toDbVersion,1);
      if ( $toDbVersionRel gt $targetDbVersionRel ) {
         my $value = defined($CloningPaths{$fromDbVersion}) ? $CloningPaths{$fromDbVersion} : "";
	     if( defined $CloningPaths{$fromDbVersion} and ($CloningPaths{$fromDbVersion} eq $targetDbVersion) ) {
             Logger::LogNotify( $nodeLogFH, "Migration Path : $fromDbVersion => $CloningPaths{$fromDbVersion} (Cloned)\n");
             last;
         } else {
	         $possible = 0;
	         $reason = "Migration paths are not valid/available";
	         last;
         }
      }
      $fromDbVersion = $toDbVersion;
  }
  return ($possible, $reason);
}

=head2 UpgradePossible()

=begin html
Check whether SW upgrade is possible.<br>The following parameters are used:
<table>
  <tr><td>ntType</td><td>the NT type involved in the migration (string)</td></tr>
  <tr><td>from</td><td>the release currently active</td></tr>
  <tr><td>to:</td><td>the new release</td></tr>
</table>
Releasenumbers need to have been converted using <tt>ConvertRelease</tt>.<br>
It returns whether SW upgrade is possible and a reason in case it is not possible.

=end html

=cut

sub UpgradePossible($$$) {
  my ($ntType, $fromDbVersion, $toDbVersion) = @_;
  my ($possible, $reason) = (1,"");
  $ntType = uc($ntType);
  Logger::LogNotify( $nodeLogFH, "Upgrade Path : Actvie '$fromDbVersion' => Target '$toDbVersion' \n");
  if( ! $DBToolConfig ) {
      $possible = 0;
      $reason = "Upgrade path check config not exists";
  } elsif( !defined $ReleaseDBVer{$fromDbVersion} ) {
	$possible = 0;
	$reason = "Active release is not in Upgrade path";
  } elsif( !defined $ReleaseDBVer{$toDbVersion} ) {
	$possible = 0;
	$reason = "Target release is not in Upgrade path";
  } elsif( $ReleaseDBVer{$fromDbVersion} ne $ReleaseDBVer{$toDbVersion}  ) {
      $possible = 0;
      $reason = "Target release different from Active Release";
  }

  if( !$possible ) {
     return ($possible, $reason);
  }
  if( $fromDbVersion eq $toDbVersion ) {
     $possible = 1;
     $reason = "Active and Target DB Versions are same";
  } else {
     $reason = "Active and Target DB Versions are not same";
  }
  return ($possible, $reason);
}

sub GetDbVerToRelease($) {
   my( $dbVersion ) = (@_ );
   if( defined $ReleaseDBVer{ $dbVersion } ) {
       return(0, $ReleaseDBVer{ $dbVersion });
   }
   return(1, "");
 }


sub ParseDBToolConfig($) {
   my ($dbToolConfig) = @_;
   if( ! defined $dbToolConfig ) {
      print STDERR "Error : Required dbtool config haven't passed to parse\n";
      return 1;
   }
   if( ! -f $dbToolConfig ) {
      print STDERR "Error : dbtool config $dbToolConfig is not a file\n";
      return 1;
   }
   my ($configFH,$line) = (undef,"");
   my ($parseFile, $parseReleaseDBVer, $parseCloningPaths, $parseMigrationPaths ) = (0,0,0,0);

   if (!open($configFH, "<$dbToolConfig")) {
     print STDERR "Cannot open $dbToolConfig for reading: $!\n";
     return 1;
   }
   %ReleaseDBVer = ();
   %CloningPaths = ();
   %MigrationPaths = ();
   my ( $key,$value) = ("","");
   while ( $line = <$configFH> ) {
	$parseFile = 1 if( $line =~ /^START_PATH_CFG/ );
	$parseFile = 0 if( $line =~ /^END_PATH_CFG/ );

        next if( !$parseFile or $line =~ /^\s*$/ );

        if( $line =~/^releasetodbversion\s*=\s*\[/ ) {
	   $parseReleaseDBVer = 1;
        }
        if( $line =~/^cloningpaths\s*=\s*\[/ ) {
	   $parseCloningPaths= 1;
        }
        if( $line =~/^migrpaths\s*=\s*\[/ ) {
	   $parseMigrationPaths= 1;
        }
        if( $line =~ /^\s*\]\s*$/ ) {
	   $parseReleaseDBVer = 0 if ($parseReleaseDBVer);
	   $parseCloningPaths = 0 if ($parseCloningPaths);
	   $parseMigrationPaths = 0 if ($parseMigrationPaths);
        }
        if( $line =~ /^\s*\{([A-Z0-9\.]+),([A-Z0-9\.]+)\},?\s*$/) {
	       $key = $1; $value = $2;
               $ReleaseDBVer{$value} = $key if($parseReleaseDBVer);
               $CloningPaths{$key} = $value if($parseCloningPaths);
               $MigrationPaths{$key} = $value if($parseMigrationPaths);
        }
   }
   close($configFH);
   $DBToolConfig = 1;
   $params->{'StripCapabilityChecks'} = 1;
   return 0;
}

sub CompareVersion($$) {
    my ($dbVersionSource, $dbVersionTarget ) = @_;
    my $fromVersion = substr($dbVersionSource,1);
    my $toVersion   = substr($dbVersionTarget,1);
    return $fromVersion cmp $toVersion;
}

1;
