package CommitIsam;

=head1 HiCap ISAM 7302/7330-specific commit module

=begin html
This library contains the functions dedicated to perform a SW commit for a HiCap 7302/7330 ISAM.

=end html

=cut

#==============================================================================
#-- File:         CommitIsam.pm
#-- Date:         2018/06/28
#-- Company:      Nokia
#-- Author:       Dhineshkumar Manikannan
#------------------------------------------------------------------------------
#-- Function:     Script to commit the active OSWP
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Nokia
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Nokia AND 2018
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   3.0.0  - 2018/06/28 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "CommitIsam.pm - 3.0.0 -- 2018/06/28"

=end html

=cut

my $VERSION = "CommitIsam.pm - 3.0.0 -- 2018/06/28";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use File::stat;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;

use Net::SNMP;
use IsamCli;
use Crypt::Lite;
use Utils;
use Capability;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :COMT);
use ToolCommon;
use PBMTGlobal;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Commit($);
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $STATUSFILE =       "result.txt";
my $toolData;
my $config;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Commit($$)

=begin html
This function is called to perform SW commisioning of a node of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

sub Commit($) {
  my ($isam) = @_;
  my ($newMajor, $newMinor, $newSuffix, $newFullRelNum);
  my $shubSwName;
  my $activeNt;
  my $cli = undef;
  my $ret;
  my $dir = FamilyCommon::GetNeStorage();
  my $nodeLogFh = FamilyCommon::GetChildLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();

  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  #-- Open CLI Session
  $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFh, $toolData->{TELNETTIMEOUT});
  if (!defined($cli)) {
     FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, ToolCommon::GetError() );
     return;
  }
  #-- STEP 0: Acquire transaction
  Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership... DONE\n");

  #-- STEP 1: Check Active OSWP is committed
  ToolCommon::Log($nodeLogFh, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($cli);
  return undef if( not defined($ntInfo) );
  $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';

  my $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}), undef,
					 $toolData->{RETRIES});
  if (! defined($oswpInfo)) {
     FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, "" );
     return;
  }

  if (! $params->{FORCECOMMIT}) {
    #-- STEP 2: Check active SW is expected SW
    my ($restartOswpInfo, $oswpInfoIsEmpty);
    ($ret, $oswpInfoIsEmpty, $restartOswpInfo) = FamilyCommon::CheckActiveSoftwareIsNew($nodeLogFh, $cli, $snmpSession,
											$params->{OSWP}, $dir, $isam,
											$ntInfo->{$activeNt}{TYPE});
    if (!$ret) {
      FamilyCommon::ReportStatus( EC_NEW_SW_NOT_ACT, FamilyCommon::GetError() );
      return;
    }
  }

  #-- STEP 3: If not committed -> commit OSWP -> removes passive OSWP
  ToolCommon::Log($nodeLogFh, "Checking Active Software is committed...");
  if ($oswpInfo->{ACTIVE}{COMMITED} ne "committed") {
    ToolCommon::Log($nodeLogFh, "  not committed.\n");
    ToolCommon::Log($nodeLogFh, "Committing Active OSWP...");
    $ret = IsamCli::CommitSWP($cli, $oswpInfo->{ACTIVE}{INDEX});
    if (!defined($ret) || !$ret) {
      FamilyCommon::ReportStatus( EC_COMMIT_FAILED, IsamCli::GetError() );
      return;
    } else {
      ToolCommon::Log($nodeLogFh, " OK\n");
    }
  } else {
    ToolCommon::Log($nodeLogFh, " OK, already commited\n");
  }
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership... DONE");
   FamilyCommon::CloseCliSession2Close();
}

1;
