package DM;

=head1 DM database file interface

=begin html
This PERL module implements the access to DM database files

=end html

=cut

#==============================================================================
#-- File:         DM.pm
#-- Date:         18.03.2002
#-- Company:      Alcatel Bell - TEC
#-- Author:       Donny Janssens, WD25
#------------------------------------------------------------------------------
#-- Function:     Library to read DM Tables
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#--               There is no warranty about it, use of this script is at own
#--               risk
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel BND 2002
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 18032002 : Start First implementation
#==============================================================================

=head1 VERSION

=begin html
Current version: "DM.pm -- 1.0.0 - 2002/03/18"

=end html

=cut

use strict;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetError();
sub SetSeparator($);
sub _ConvertBytesToDec($);
sub _ConvertBytesToHex($);
sub _ConvertBytesToStr($;$);
sub _CvtField($$$$);
sub _ReadField($$$);
sub _AlignAtBoundary($$$);
sub EmergencyClose();
sub ReadTableDefinition($);
sub ReadFile($);
sub PrintTblHeader();
sub PrintHeader($);
sub PrintRecHeading();
sub GetFieldType($);
sub GetInitValue($);
sub PrintRecordDescription($;$);
sub DumpRecord($);
sub PrintRecord($$$);
sub PrintRecordFieldNames($);
sub PrintOneRecord($$);
sub DumpOneRecord($$);
sub PrintRecords($$);
sub DumpRecords($$);
sub PrintTable($$$);
sub HeaderDefinitionDifferent($$);
sub RecordDefinitionDifferent($$);

#==============================================================================
my %types = (0 => "Unsigned Char (1 Byte)",
	     1 => "Char (1 Byte)",
	     2 => "Short (2 Byte)",
	     3 => "Unsigned Short (2 Byte)",
	     4 => "Unsigned Int (4 Byte)",
	     5 => "Int (4 Byte)",
	     6 => "Unsigned Long (4 Byte)",
	     7 => "Long (4 Byte)",
	     8 => "String (Variable Length)",
	     9 => "Time");

my %dmTypes = (0 => "UNSIGNED_CHAR",
	       1 => "CHAR",
	       2 => "SHORT",
	       3 => "UNSIGNED_SHORT",
	       4 => "UNSIGNED_INT",
	       5 => "INT",
	       6 => "UNSIGNED_LONG",
	       7 => "LONG",
	       8 => "STRING"
	      );

my %persistencyLevels = (0 => "Reset Safe",
			 1 => "Disk",
			 2 => "Disk Protected",
			 3 => "Volatile (Memory)");

my %dmPersLevels = (0 => "Resetsafe",
		    1 => "disk",
		    2 => "Diskprotected",
		    3 => "Diskrdonly"
		   );

my $separator = ",";
my $fileHdl = undef;
my $fileHdlFile = "";

my $fileErr = 0;

=head1 FUNCTIONS

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $fileErr;
}

=head2 SetSeparator($)

=begin html
Set the separator to be used by print functions (when producing csv files) to the value specified by $sep.

=end html

=cut

sub SetSeparator($) {
  my ($sep) = @_;
  $separator = $sep;
}

=head2 _ConvertBytesToDec($)

=begin html
Private function to convert the bytes passed in $var into decimal.

=end html

=cut

sub _ConvertBytesToDec($) {
  my ($var) = @_;
  return hex(_ConvertBytesToHex($var));
}

=head2 _ConvertBytesToHex($)

=begin html
Private function to convert the bytes passed in $var into hexadecimal.

=end html

=cut

sub _ConvertBytesToHex($) {
  my ($var) = @_;
  my $hexStr = unpack("H*",$var);
  return $hexStr;
}

=head2 _ConvertBytesToStr($;$)

=begin html
Private function to convert the bytes passed in $var into a string.

=end html

=cut

sub _ConvertBytesToStr($;$) {
  my ($var, $givenLength) = @_;
  my $length = length($var);
  my $index = index($var, "\0");
  my $str;

  if (!defined($givenLength)) {
    $str = substr($var, 0, $index);
  } else {
    $str = substr($var, 0, $givenLength);
  }
  return $str;
}

=head2 _CvtField($$$$)

=begin html
Private function to convert the value of a DB field.<br>
The following parameters are used:
<table>
  <tr><td>buffer</td><td>buffer holding the data read from file</td></tr>
  <tr><td>offset</td><td>position in the buffer identifying the start of the field</td></tr>
  <tr><td>length</td><td>the length of the field</td></tr>
  <tr><td>type</td><td>the type of the field</td></tr>
</table>
Returns the converted field or undef in an unknown type was specified.

=end html

=cut

sub _CvtField($$$$) {
  my ($buffer, $offset, $length, $type) = @_;
  my $val = substr($buffer, $offset, $length);
  my $value;

  if ($type eq "STR") {
    $value = _ConvertBytesToStr($val, $length);
  } elsif ($type eq "DEC") {
    $value = _ConvertBytesToDec($val);
  } elsif ($type eq "HEX") {
    $value = _ConvertBytesToHex($val);
  } else {
    #-- undefined type
    return undef;
  }
#  print "Offset: $offset, length: $length, value: $value\n";
  return ($value);
}

=head2 _ReadField($$$)

=begin html
Private function to a field from a DB file.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>a filehandle associated with the file being read</td></tr>
  <tr><td>length</td><td>the length of the field</td></tr>
  <tr><td>type</td><td>the type of the field</td></tr>
</table>
Returns the converted field or undef in an unknown type was specified.

=end html

=cut

sub _ReadField($$$) {
  my ($fh, $length, $type) = @_;
  my $buffer;
  my $value;
  my $retCode = sysread( $fh, $buffer, $length);

  return undef if (!$retCode);

  if ($type eq "STR") {
    $value = _ConvertBytesToStr($buffer);
  } elsif ($type eq "DEC") {
    $value = _ConvertBytesToDec($buffer);
  } elsif ($type eq "HEX") {
    $value = _ConvertBytesToHex($buffer);
  } else {
    #-- undefined type
    return undef;
  }
  return $value;
}

=head2 _AlignAtBoundary($$$)

=begin html
Private function to align the buffer to the next field:
<ul>
  <li>If the type is a Int, Long or String align at 4 bytes</li>
  <li>If type is a Short align at 2 bytes</li>
</ul>
.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>a filehandle associated with the file being read</td></tr>
  <tr><td>length</td><td>the length of the field</td></tr>
  <tr><td>type</td><td>the type of the field</td></tr>
</table>
Returns the number of bytes read (as number and the contents of the padding as string).

=end html

=cut

sub _AlignAtBoundary($$$) {
  my ($fh, $type, $totalRead) = @_;
  my $bytesRead = 0;
  my $padding;

  #-- If the type is a Int, Long or String align at 4 bytes
  #-- If type is a Short align at 2 bytes
  if (($type == 4 || $type == 5 || $type == 6 || $type == 7 || $type == 8) && ($totalRead % 4) != 0) {
    $bytesRead = 4 - ( $totalRead % 4);
  } elsif (($type == 2 || $type == 3) && ($totalRead % 2) != 0) {
    $bytesRead = 1;
  }
  if ($bytesRead != 0) {
    sysread($fh, $padding, $bytesRead);
  }
  return ($bytesRead, $padding);
}


=head2 EmergencyClose()

=begin html
Close the opened file in case of error in the program using the module.

=end html

=cut

sub EmergencyClose() {
  print STDERR "Emergency close $fileHdlFile\n";
  close($fileHdl) if (defined($fileHdl));
}

=head2 ReadTableDefinition($)

=begin html
Read the defintion part of this DB file.
Returns undef in case of error or a couple in case of success:
<ul>
  <li>pointer to a hash containing the file header definition</li>
  <li>pointer to an array of hashes.  Each array entry corresponds to a field of the DB record defined in this DB file.</li>
</ul>

=end html

=cut

sub ReadTableDefinition($) {
  my ($file) = @_;
  my %fileHeader;
  my @fields = ();
  my $eof = 0;
  my $junk;

  if (!open(DM, $file)) {
    $fileErr = $!;
    return;
  }
  $fileHdl = \*DM, $fileHdlFile = $file;

  #-- read Table Header
  $fileHeader{DUMMY} = _ReadField(\*DM, 2, "DEC");
  $fileHeader{TABLE_ID} =  _ReadField(\*DM, 2, "DEC");
  $fileHeader{OFFSETFIRSTRECORD} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{MAGICNUMBER} = _ReadField(\*DM, 4, "HEX");
  $fileHeader{CHECKSUM} = _ReadField(\*DM, 4, "HEX");
  $fileHeader{TABLENAME} = _ReadField(\*DM, 16, "STR");
  $fileHeader{RECORDLEN} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{MAXNBRRECS} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{PERSLVL} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{PERSFLGS} =  _ReadField(\*DM, 4, "DEC");

  #-- Read Record Definition
  my $name = "start";
  my $fieldTotalLength = 0;

  while ( !$eof && $name ne "__sc__") {
    my %fieldDef;
    $fieldDef{PRIMKEY} = _ReadField(\*DM, 4, "DEC");
    if (!defined($fieldDef{PRIMKEY})) {
      $eof = 1;
      next;
    }
    $fieldDef{NAME} = _ReadField(\*DM, 8, "STR");
    $fieldDef{TYPE} = _ReadField(\*DM, 4, "DEC");
    $fieldDef{LENGTH} = _ReadField(\*DM, 4, "DEC");
#    print "Name: $fieldDef{NAME}, Type: $types{$fieldDef{TYPE}}, length: $fieldDef{LENGTH}\n";
    $fieldDef{USEINITVALUE} = _ReadField(\*DM, 1, "DEC");
    sysread(DM, $junk, 3);
    $fieldDef{INITVALUE} = _ReadField(\*DM, 4, "DEC");
    $fieldDef{FIELDOFFSET} = _ReadField(\*DM, 2, "DEC");
    $fieldDef{NOTUSED} = _ReadField(\*DM, 2, "DEC");
    $name = $fieldDef{NAME};
    if ($fieldDef{NAME} ne "__sc__" && $fieldDef{NAME} ne "__oi__") {
      $fieldTotalLength += $fieldDef{LENGTH};
#      print "fieldTotalLength: $fieldTotalLength\n";
      push(@fields, \%fieldDef);
    }
  }
  close(DM);
  $fileHdl = undef, $fileHdlFile = "";
  return (\%fileHeader, \@fields);
}

=head2 ReadFile($)

=begin html
Read a DB file.
Returns a list:
<ul>
  <li>pointer to a hash containing the file header definition</li>
  <li>pointer to an array of hashes.  Each array entry corresponds to a field of the DB record defined in this DB file.</li>
  <li>pointer to an array of hashes.  Each array entry corresponds to the contents of a field of the DB record defined in this DB file.</li>
  <li>the amount of records in the table</li>
  <li>the amount non-empty records in the table</li>
</ul>

=end html

=cut

sub ReadFile($) {
  my ($file) = @_;
  my %fileHeader;
  my @fields = ();
  my @records = ();
  my $buffer;
  my $eof = 0;
  my $junk;
  my $first = 1;
  my $firstRec;

#  print "$file\n";
  if (!open(DM, $file)) {
    $fileErr = $!;
    return;
  }
  binmode DM;
  $fileHdl = \*DM, $fileHdlFile = $file;

  #-- read Table Header
  $fileHeader{DUMMY} = _ReadField(\*DM, 2, "DEC");
  $fileHeader{TABLE_ID} =  _ReadField(\*DM, 2, "DEC");
  $fileHeader{OFFSETFIRSTRECORD} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{MAGICNUMBER} = _ReadField(\*DM, 4, "HEX");
  $fileHeader{CHECKSUM} = _ReadField(\*DM, 4, "HEX");
#    $fileHeader{RELPROTECTINFO} = _ReadField( \*DM, 12, "HEX");
  $fileHeader{TABLENAME} = _ReadField(\*DM, 16, "STR");
  $fileHeader{RECORDLEN} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{MAXNBRRECS} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{PERSLVL} = _ReadField(\*DM, 4, "DEC");
  $fileHeader{PERSFLGS} =  _ReadField(\*DM, 4, "DEC");

  #-- Read Record Definition
  my $name = "start";
  my $fieldTotalLength = 0;

  while ( !$eof && $name ne "__sc__") {
    my %fieldDef;
    $fieldDef{PRIMKEY} = _ReadField(\*DM, 4, "DEC");
    if (!defined($fieldDef{PRIMKEY})) {
      $eof = 1;
      next;
    }
    $fieldDef{NAME} = _ReadField(\*DM, 8, "STR");
    $fieldDef{TYPE} = _ReadField(\*DM, 4, "DEC");
    $fieldDef{LENGTH} = _ReadField(\*DM, 4, "DEC");
#    print "Name: $fieldDef{NAME}, Type: $types{$fieldDef{TYPE}}, length: $fieldDef{LENGTH}\n";
    $fieldDef{USEINITVALUE} = _ReadField(\*DM, 1, "DEC");
    sysread(DM, $junk, 3);
    $fieldDef{INITVALUE} = _ReadField(\*DM, 4, "DEC");
    $fieldDef{FIELDOFFSET} = _ReadField(\*DM, 2, "DEC");
    $fieldDef{NOTUSED} = _ReadField(\*DM, 2, "DEC");
    $name = $fieldDef{NAME};
    if ($fieldDef{NAME} ne "__sc__" && $fieldDef{NAME} ne "__oi__") {
      $fieldTotalLength += $fieldDef{LENGTH};
#      print "fieldTotalLength: $fieldTotalLength\n";
      push(@fields, \%fieldDef);
    }
  }

  #-- read records
  my $field;
  my $type;
  $eof = 0;
  my $locBuf;
  my $locLen;
  my $recCnt = 0;
  my $actualCnt = 0;

  while (!$eof) {
    my %rec;
    $locLen = sysread(DM, $locBuf, $fileHeader{RECORDLEN});
#    print "Read $locLen bytes\n";
#    print DumpRecord($locBuf) ."\n";
    if ($locLen > 0) {
      $rec{_TABLEID_} = _CvtField($locBuf, 0, 2, "DEC");
#      if (!defined($rec{TABLEID})) {
#	$eof = 1;
#	next;
#      }
      $rec{_RECORDID_} = _CvtField($locBuf, 2, 2, "DEC");
      foreach $field (@fields) {
	if ($field->{TYPE} == 8) {
	  $type = "STR";
	} elsif ($field->{TYPE} < 8) {
	  $type = "DEC";
	}
	$rec{$field->{NAME}} = _CvtField($locBuf, $field->{FIELDOFFSET}, $field->{LENGTH}, $type);
      }
      if ($rec{_RECORDID_} != 65535) {
	push(@records, \%rec);
	$actualCnt++;
      }
      $recCnt++;
    } else {
      $eof = 1;
    }
  }
  close(DM);
  $fileHdl = undef, $fileHdlFile = "";
  return (\%fileHeader, \@fields, \@records, $recCnt, $actualCnt);
}


=head2 PrintTblHeader()

=begin html
Print the table definition (field names only)

=end html

=cut

sub PrintTblHeader() {
  return "TlbID" . $separator. "Name" . $separator . "Offset1stRec" . $separator . "MagicNum" . $separator . "Chcksum" . $separator . "RecLen" .
    $separator . "MaxRecs" . $separator . "PersLvl" . $separator . "PersFlgs";
}

=head2 PrintHeader($)

=begin html
Print the table definition (field values only)

=end html

=cut

sub PrintHeader($) {
  my ($fileHeader) = @_;

  return "$fileHeader->{TABLE_ID}" . $separator . "$fileHeader->{TABLENAME}" . $separator . "$fileHeader->{OFFSETFIRSTRECORD}" . $separator .
    "$fileHeader->{MAGICNUMBER}" . $separator . "$fileHeader->{CHECKSUM}" . $separator . "$fileHeader->{RECORDLEN}" . $separator .
      "$fileHeader->{MAXNBRRECS}" . $separator . "$persistencyLevels{$fileHeader->{PERSLVL}}" . $separator . "$fileHeader->{PERSFLGS}";
}

=head2 PrintRecHeading()

=begin html
Print the record heading (field names only)

=end html

=cut

sub PrintRecHeading() {
  return "Count" . $separator . "Primkey" . $separator . "Name" . $separator . "Type" . $separator . "Length" . $separator . "InitVal Used" . $separator .
    "InitValue" . $separator . "Field Offset";
}

=head2 GetFieldType($)

=begin html
Return the type of $field.

=end html

=cut

sub GetFieldType($) {
  my ($field) = @_;
  my $typeStr = $types{$field->{TYPE}};

  if ($field->{TYPE} == 8) {
    $typeStr = substr($typeStr, 0, -1) . ", $field->{LENGTH})";
  }
  return $typeStr;
}

sub DmFieldType {
  my ($fieldType) = @_;
  return $dmTypes{$fieldType};
}

sub DmPersLevel{
  my ($level) = @_;
  return $dmPersLevels{$level};
}

sub DmFieldValue{
  my ($type, $value) = @_;
  my $valStr = "0x";

  if ($type < 2) { # Char and unsigned char
    $valStr .= sprintf("%02x", $value);
  } elsif ($type < 4) { # short and unsigned short
    $valStr .= sprintf("%04x", $value);
  } elsif ($type < 8) { # int and long (signed and unsigned)
    $valStr .= sprintf("%08x", $value);
  } elsif ($type < 9) {
    my @valArr = split(//, $value);
    foreach my $char (@valArr) {
      $valStr .= sprintf("%02x", ord($char));
    }
  } else {
    # time not yet supported
  }
  return $valStr;
}


=head2 GetInitValue($)

=begin html
Return the init-value of $field.

=end html

=cut

sub GetInitValue($) {
  my ($field) = @_;

  return $field->{USEINITVALUE} ? "Yes $field->{INITVALUE}" : "No";
}

=head2 PrintRecordDescription($;$)

=begin html
Print the record heading (field values only)

=end html

=cut

sub PrintRecordDescription($;$) {
  my ($fields, $prefix) = @_;
  my $count = 1;
  my $field;
  my $str = "";

  foreach $field (@{$fields}) {
    $str .= $prefix if (defined($prefix));
    $str .= "$count" . $separator . "$field->{PRIMKEY}" . $separator . "$field->{NAME}" . $separator . "$types{$field->{TYPE}}" . $separator .
      "$field->{LENGTH}" . $separator . "$field->{USEINITVALUE}" . $separator . "$field->{INITVALUE}" . $separator . "$field->{FIELDOFFSET}\n";
    $count++;
  }
  return $str;
}

=head2 DumpRecord($)

=begin html
Dump a record as hexadecimal values.

=end html

=cut

sub DumpRecord($) {
  my ($record) = @_;
  my $string = _ConvertBytesToHex($record);
  my @splitted = split(//, $string);
  my $str = "";
  my $i = 0;
  my $cnt = 0;

  foreach (@splitted) {
    $str .= $_;
    $i++, $cnt++;
    if ($i == 2) {
      $str .= " ";
      $i = 0;
    }
    if ($cnt % 64 == 0) {
      $str .= "\n";
    }
  }
  $str .= "\n";
  return $str;
}

=head2 PrintRecord($$$)

=begin html
Print a record as 'Name: value' lines.

=end html

=cut

sub PrintRecord($$$) {
  my ($recordNr, $fields, $records) = @_;
  my $field;
  my $rec;
  my $str = "";
  $recordNr--;

  $rec = $records->[$recordNr];
  #-- print record
  $str .=  "Record $rec->{RECORDID} in Table $rec->{TABLEID}:\n";
  foreach $field (@{$fields}) {
    $str .= "  $field->{NAME}: $rec->{$field->{NAME}}\n";
  }
  return $str;
}

=head2 PrintRecordFieldNames($)

=begin html
Print record field names.

=end html

=cut

sub PrintRecordFieldNames($) {
  my ($fields) = @_;
  my $str = "";

  foreach my $field (@{$fields}) {
    $str .= "$field->{NAME}" . $separator;
  }
  $str = substr($str, 0, -1);
  return $str
}

=head2 PrintOneRecord($$)

=begin html
Alternative form to print a record $record as 'Name: value' lines.

=end html

=cut

sub PrintOneRecord($$) {
  my ($fields, $record) = @_;
  my $str =  sprintf("[Rec %05d] ", $record->{_RECORDID_});

  foreach my $field (@{$fields}) {
    $str .= "$record->{$field->{NAME}}" . $separator;
  }
  $str = substr($str, 0, -1);
  return $str;
}

=head2 DumpOneRecord($$)

=begin html
Alternative form to dump one record contained in $record according to the syntax specified by $fields.

=end html

=cut

sub DumpOneRecord($$) {
  my ($fields, $record) = @_;
  my $str = sprintf("[Rec %05d] ", $record->{_RECORDID_});
  my $length;

  foreach my $field (@{$fields}) {
    $length = $field->{LENGTH};
    if ($field->{TYPE} == 8) {
      # string
      my $i = 0;
      while ($length > 0) {
	$str .= sprintf("%02x ", ord(substr($record->{$field->{NAME}}, $i, 1)));
	$i++;
	$length--;
      }
    } else {
      # decimal
      while ($length > 0) {
	$str .= sprintf("%02x ", ($record->{$field->{NAME}} & (0xFF << (($length-1) << 3))) >> (($length-1) << 3));
	$length--;
      }
    }
    $str = substr($str, 0, -1) . $separator;
  }
  $str = substr($str, 0, -1);
  return $str;
}

=head2 PrintRecords($$)

=begin html
Print all records of $records identified by the syntax in $fields.

=end html

=cut

sub PrintRecords($$) {
  my ($fields, $records) = @_;
  my $field;
  my $rec;
  my $str = "";

  $str = PrintRecordFieldNames($fields) . "\n'";
  foreach $rec (@{$records}) {
    $str .= PrintOneRecord($fields, $rec) . "'\n'";
    #-- print record
  }
  $str = substr($str, 0, -1);
  return $str;
}

=head2 DumpRecords($$)

=begin html
Dump all records of $records identified by the syntax in $fields.

=end html

=cut

sub DumpRecords($$) {
  my ($fields, $records) = @_;
  my $field;
  my $rec;
  my $str = "";

  $str = PrintRecordFieldNames($fields) . "\n";
  foreach $rec (@{$records}) {
    $str .= DumpOneRecord($fields, $rec) . "\n";
    #-- print record
  }
  $str = substr($str, 0, -1);
  return $str;
}

=head2 PrintTable($$$)

=begin html
Print a complete DB table:
<ul>
  <li>the table header</li>
  <li>the field definition</li>
  <li>the field contents</li>
</ul>

=end html

=cut

sub PrintTable($$$) {
  my ($header, $fields, $records) = @_;
  print PrintHeader( $header);
  print PrintRecordDescription($fields);
  print PrintRecords($fields, $records);
}

=head2 HeaderDefinitionDifferent($$)

=begin html
Indicates whether table defintions contained in $head1 and $head2 are different.  The differences are returned in a string (an empty string indicates
that the definitions are equal.

=end html

=cut

sub HeaderDefinitionDifferent($$) {
  my ($head1, $head2) = @_;
  my $head1Length;
  my $head2Length;
  my $str = "";

  if (defined($head1) && defined($head2)) {
    $head1Length = scalar(keys %{$head1});
    $head2Length = scalar(keys %{$head2});
    if ($head1Length != $head2Length) {
      $str = "Number of fields in file header differs\n\t     " . PrintTblHeader() . "\n\tDB1: ";
      $str .= PrintHeader($head1) . "\n\tDB2: " . PrintHeader($head2) . "\n";
    } else {
      if ($head1->{RECORDLEN}  != $head2->{RECORDLEN}) {
	$str .= "File header differs - record length: $head1->{RECORDLEN} vs $head2->{RECORDLEN}\n";
      }
      if ($head1->{MAXNBRRECS} != $head2->{MAXNBRRECS}) {
	$str .= "File header differs - max nbr recs : $head1->{MAXNBRRECS} vs $head2->{MAXNBRRECS}\n";
      }
      if ($head1->{PERSLVL}    != $head2->{PERSLVL}) {
	$str .= "File header differs - persistlevel : $head1->{PERSLVL} vs $head2->{PERSLVL}\n";
      }
      if ($head1->{PERSFLGS}   != $head2->{PERSFLGS}) {
	$str .= "File header differs - persistflags : $head1->{PERSFLGS} vs $head2->{PERSFLGS}\n";
      }
    }
  }
  return $str;
}

=head2 RecordDefinitionDifferent($$)

=begin html
Indicates whether ercord defintions contained in $fields1 and $fields2 are different.  The differences are returned in a string (an empty string
indicates that the definitions are equal.

=end html

=cut

sub RecordDefinitionDifferent($$) {
  my ($fields1, $fields2) = @_;
  my $field1Length;
  my $field2Length;
  my $str = "";
  my $fieldCount = 0;

  if (defined($fields1) && defined($fields2)) {
    $field1Length = scalar(@{$fields1});
    $field2Length = scalar(@{$fields2});
    if ($field1Length != $field2Length) {
      $str = "Number of fields in record definition differs\n" . PrintRecHeading() . "\n";
      foreach my $field (@{$fields1}) {
	$str .= PrintRecordDescription($fields1, "\tDB1: ");
      } 
      foreach my $field (@{$fields2}) {
	$str .= PrintRecordDescription($fields2, "\tDB2: ");
      }
    } else {
      $fieldCount = 0;
      while ($fieldCount < $field1Length) {
	if ($fields1->[$fieldCount]{NAME}         ne $fields2->[$fieldCount]{NAME}) {
	  $str = "Record definition shuffled\n";
	  $str .= PrintRecordDescription($fields1, "\tDB1: ") . PrintRecordDescription($fields2, "\tDB2: ") . "\n";
	} else {
	  if ($fields1->[$fieldCount]{TYPE}         != $fields2->[$fieldCount]{TYPE}) {
	    $str = sprintf("%s : Record definition type differs  : %s vs. %s\n",
			   $fields1->[$fieldCount]{NAME}, $types{$fields1->[$fieldCount]{TYPE}}, $types{$fields2->[$fieldCount]{TYPE}});
	  }
	  if ($fields1->[$fieldCount]{LENGTH}       != $fields2->[$fieldCount]{LENGTH}) {
	    $str = sprintf("%s: Record definition length differs: %d vs. %d\n",
			   $fields1->[$fieldCount]{NAME}, $fields1->[$fieldCount]{LENGTH}, $fields2->[$fieldCount]{LENGTH});
	  }
	  if ($fields1->[$fieldCount]{USEINITVALUE} != $fields2->[$fieldCount]{USEINITVALUE}) {
	    $str = sprintf("%s: Record definition use init diffs: %d vs. %d\n",
			   $fields1->[$fieldCount]{NAME}, $fields1->[$fieldCount]{USEINITVALUE}, $fields2->[$fieldCount]{USEINITVALUE});
	  }
	  if ($fields1->[$fieldCount]{INITVALUE}    != $fields2->[$fieldCount]{INITVALUE}) {
	    if ($fields1->[$fieldCount]{USEINITVALUE} && $fields2->[$fieldCount]{USEINITVALUE}) {
	    $str = sprintf("%s: Record definition init val diffs: %d vs. %d\n",
			   $fields1->[$fieldCount]{NAME}, $fields1->[$fieldCount]{INITVALUE}, $fields2->[$fieldCount]{INITVALUE});
	    }
	  }
	}
	$fieldCount++;
      }
    }
  }
  return $str;
}

#-- end of library
1;
