package DownloadIsam1540;

=head1 ISAM 7302/7330 SW download module

=begin html
This library contains the functions dedicated to perform prechecks for a 7302/7330 Litespan ISAM.

=end html

=cut

#==============================================================================
#-- File:         DownloadIsam1540.pm
#-- Date:         06.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Donny Janssens for the initial version
#--               Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to download new OSWP to Litespan ISAMs
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2009/05/07 : First implementation (based on copy of ETSI ISAM)
#--   1.0.1 - 2010/04/02 : ANTmt91104 / ALU00100007 PBMT: SW dir permissions not set when not reducing
#--   1.0.2 - 2010/04/26 : ALU00259276 - Boardlist support check in SWP skipped if force download option is used
#--   1.0.3 - 2011/02/02 : ALU01006984 - Undo workaround done for ALU00100007 since issue is solved in NT SW
#--   1.0.4 - 2016/07/15 : ALU02237966
#--   1.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "DownloadIsam1540.pm - 1.1.0 -- 2018/06/20";

=end html

=cut

my $VERSION = "DownloadIsam1540.pm - 1.1.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP<a/></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use File::stat;
use File::Basename;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;

use Net::TFTP;
use Net::SNMP;

use IsamCli;
use Crypt::Lite;
use Utils;  # -- BB --
require Octopus;
use Capability;
use ToolCommon;
use PBMTGlobal;
require FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :SWDL);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetMinimumSet($);
sub DownloadMissingFiles($$$$);
sub Download($$);
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $config;
my $toolData;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 GetMinimumSet($$)

=begin html
Determine the minimum set of boards in the SWP taken from '$params'.  The list is returned as a pointer to a hash.

=end html

=cut

sub GetMinimumSet($) {
  my ($logFile) = @_;
  my $swpFile;
  my $line;
  my $board = undef;
  my %minSet = ();

  if (open($swpFile, $params->{NOTFTP} ? $params->{SWPNAME} : "$toolData->{TFTPDIR}/$params->{SWPNAME}")) {
    while ($line = <$swpFile>) {
      chomp $line;
      $line =~ s/\r//g;
      $line =~ s/\n//g;
      ToolCommon::Log($logFile, "SWPfile line read: $line\n") if $params->{DEBUG};
      if (defined($board)) {
	if ($line =~ /^\s*MINIMUMSET\s*:\s*([^\s]*)/) {
	  my $inMinSet = $1;
	  if (index(uc($inMinSet), "YES") != -1) {
	    $minSet{$board} = 1;
	  }
	  $board = undef;
	}
      } else {
	if ($line =~ /^\s*DESCRIPTION\s+([^\s]+)/) {
	  $board = $1;
	}
      }
    }
    close $swpFile;
  } else {
    my $t = sprintf("Failed to open %s: $!", $params->{NOTFTP} ? $params->{SWPNAME} : "$toolData->{TFTPDIR}/$params->{SWPNAME}");
    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
    return undef;
  }
  return \%minSet;
}

=head2 DownloadMissingFiles($$$$$)

=begin html
Download files that were not downloaded during the regular download (can only happen for boards that are new in a certain release and not yet
known by the currently active release.  We have to parse the SWP file and only include boards that are in the minimum set.<br>
The following parameters are used:
<table>
  <tr><td>logFile</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>Pointer to an opened CLI session</td></tr>
  <tr><td>swList</td><td>List of the SW files for this new SW package</td></tr>
  <tr><td>params</td><td>the parameters provided to the script</td></tr>
</table>

=end html

=cut

sub DownloadMissingFiles($$$$) {
  my ($logFile, $isamCliSession, $swList, $isam) = @_;
  my $diskInfo;
  my $ok = 0;
  my $additionalSpaceNeeded = 0;
  my %missingFiles = ();

  # First retrieve the disk occupation
  $diskInfo = IsamCli::ReadDiskSpace($isamCliSession);
  if (defined($diskInfo)) {
    foreach my $package (keys %{$swList}) {
      foreach my $fileName (keys %{$swList->{$package}}) {
	if ($swList->{$package}{$fileName}{STATE} eq "not-available") {
          if (! defined($missingFiles{$fileName})) {
  	    my $fullName = sprintf("%s/%s/%s", $params->{OSWP_HOME}, dirname($params->{SWPNAME}), $fileName);
	    $fullName =~ s|\/\/|\/|g;
	    $additionalSpaceNeeded += -s $fullName;
	    ToolCommon::Log($logFile, "Determine size of $fullName; additional space required: $additionalSpaceNeeded\n") if $params->{DEBUG};
	    $missingFiles{$fileName} = $fullName;
          }
	}
      }
    }
    my $percentage = 100 - ((($diskInfo->{AVAIL} - $additionalSpaceNeeded) / $diskInfo->{TOTAL}) * 100);
    ToolCommon::Log($logFile, sprintf("Disk threshold before TFTP: %d%%, after TFTP: %d%%\n",
				      100 - (($diskInfo->{AVAIL} / $diskInfo->{TOTAL}) * 100), $percentage)) if $params->{DEBUG};
    if ($percentage < 95) {
      ToolCommon::Log($logFile, "Disk threshold OK, start download of missing files...") if $params->{DEBUG};
      my $tftp = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});
      if (defined($tftp)) {
	$tftp->binary();
	foreach my $fileName (keys %missingFiles) {
	  my $destinationFile = "/pub/Sw/Download/$fileName";
	  ToolCommon::Log($logFile, sprintf("tftp put %s %s\n", $missingFiles{$fileName}, $destinationFile)) if $params->{DEBUG};
	  my $time1 = time();
	  if ($tftp->put($missingFiles{$fileName}, $destinationFile) ne "") {
	    my $t = sprintf("Failed to tftp file $fileName: %s", $tftp->error());
	    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
	    return $ok;
	  } else {
	    ToolCommon::Log($logFile, sprintf("  OK, transfered %d bytes in %d seconds\n", $tftp->bytesTransfered(), time() - $time1))
	      if $params->{DEBUG};
	  }
	}
	ToolCommon::Log($logFile, "  OK\n") if $params->{DEBUG};
	$ok = 1;
      } else {
	my $t = "Failed to open a tftp session";
	FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
      }
    } else {
      my $t = "Operation discontinued: disk threshold would be exceeded ($percentage exceeds 95%)";
      FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
    }
  } else {
    my $t = sprintf("Failed to retrieve disk info: %s", IsamCli::GetError());
    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
  }
  return $ok;
}

=head2 Download($$$)

=begin html
This function is called to perform a SW download of a node of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>ntType</td><td>the type of NT inserted</td></tr>
>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Download($$) {
  my ($isam, $ntType) = @_;
  my $isamCliSession = undef;
  my $swParSize;
  my ($newMajor, $newMinor, $newSuffix, $newRelNum, $newFullRelNum, $activeRelNum, $fullActiveRelNum);
  my $shubSwName;
  my ($time1, $time2);
  my $activeNt;
  my $possible = 0;
  my $reason = "";
  my $ret;
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childCount = FamilyCommon::GetChildNum();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();

  if (not defined($params->{OSWP})) {
      FamilyCommon::ReportStatus( EC_OSWP_MISSING, undef );
      return 0;
  }
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  # Check OSWP for variant
  my $new = basename($params->{SWPNAME});
  my $regExp = REGEXP;
  if ($new =~ /$regExp/) {   # -- Start 1.0.4
    $newMajor = $1;
    $newMinor = $2;
    $newSuffix = $3;
    $newFullRelNum = Capability::ConvertRelease($newMajor, $newMinor, $newSuffix);
    $newRelNum = Capability::IsamLS2IsamRelease(Capability::ConvertRelease($newMajor, $newMinor));
    if (not defined($newRelNum)) {
      FamilyCommon::ReportStatus( EC_REL_CVT_FAILED, "Could not retrieve ISAM release number" );
      return 0;
    }
  }                          # -- End 1.0.4
  ToolCommon::Log($nodeLogFH, "Check OSWP variant...");
  if (! Capability::OswpVariant($params->{SWPNAME}, Capability::IsamLSFamily())) {
    FamilyCommon::ReportStatus(EC_WRONG_VARIANT, sprintf("ISAM variant for %s, expecting '%s'\n", basename($params->{SWPNAME}),
					             Capability::GetOswpVariant(Capability::IsamLSFamily(), $newRelNum) ) );
    return 0;
  }
  #-- Open CLI Session
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $isamCliSession = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($isamCliSession)) {
      FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, IsamCli::GetError() );
      return 0;
  }
  FamilyCommon::SetCliSession2Close($isamCliSession);  # Store for IncrErrorCount
  ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $isamCliSession->errmode)) if ($params->{DEBUG});

  #-- STEP 0: acquire transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");


  #-- STEP 1: NT type information and check Active OSWP is committed
  #-- Get NT type information
  Logger::LogTrace($nodeLogFH, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($isamCliSession);
  return 0 if (not defined($ntInfo) );

  my $ltConfig = IsamCli::GetSlotConfig($isamCliSession);
  if (!defined($ltConfig)) {
      FamilyCommon::ReportStatus( EC_GET_LT_FAILED, IsamCli::GetError() );
      return 0;
  } else {
       if (! $params->{FORCEDL}) {
	    my %ltHandled = ();
	    my $swMissing = "";
	    foreach my $ltNum (sort keys %$ltConfig) {
	      if (! defined($ltHandled{$ltConfig->{$ltNum}})) {
		if (! FamilyCommon::BoardInSwp($nodeLogFH, uc($ltConfig->{$ltNum}))) {
		  $swMissing .= sprintf("%s ", uc($ltConfig->{$ltNum}));
		}
		$ltHandled{$ltConfig->{$ltNum}} = 1;
	      }
	    }
	    if ($swMissing ne "") {
            FamilyCommon::ReportStatus( EC_LT_NOT_SUPPORTED, "(detected for $swMissing)");
	        return 0;
	    }
        } else {
             ToolCommon::Log($nodeLogFH, "Forced download, continue without checking for SW support for Boards in SWP ...\n");
        }
  }

  $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';
  my $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE}, undef,
					 $toolData->{RETRIES});
  if (! defined($oswpInfo)) {
    FamilyCommon::IncrErrorCount("",
				 EC_GET_OSWP_FAILED,
				 sprintf(": %s", ToolCommon::GetError()),
				 1);
    return 0;
  }
  #-- Verify migration path
  $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'},
						 $oswpInfo->{'ACTIVE'}{'SUFFIX'});
  $activeRelNum = Capability::IsamLS2IsamRelease(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
									    $oswpInfo->{'ACTIVE'}{'MINOR'}));

  if (! defined($activeRelNum)) {
    FamilyCommon::IncrErrorCount("Could not convert active release to ISAM release number",
				 EC_REL_CVT_FAILED,
				 undef,
				 1);
    return 0;
  }

  my $newSwpIndex = $oswpInfo->{PASSIVE}{INDEX};
  if (! $params->{FORCEDL}) {
    if ($oswpInfo->{'ACTIVE'}{'DBVERSION'} ne $params->{DBVER_TGTREL} ) {
      #-- Verify Migrate path
      Logger::LogTrace($nodeLogFH, "Migrate restrictions...\n");
      my ($restricted, $reason) = Capability::IsMigrateRestricted( $ntInfo->{$activeNt}{TYPE}, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Migrate restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    } else {
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntInfo->{$activeNt}{TYPE}, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    }
  } else {
    ToolCommon::Log($nodeLogFH, "Forced download, continue without checks...\n");
  }

  if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3,2), REGEXP, 0) < 0) {
    # From R3.1 onwards the same OSWP can be redownloaded so we have to allow it.
    ToolCommon::Log($nodeLogFH, "Checking new software already present...");
    if (basename($oswpInfo->{ACTIVE}{NAME}) eq (basename($params->{OSWP})) || # -- 1.2.3
	(basename($oswpInfo->{PASSIVE}{NAME}) eq basename($params->{OSWP}) && $oswpInfo->{PASSIVE}{AVAIL} eq 'enabled')) { #-- 1.2.3
      FamilyCommon::AdvanceToNext("Already present... skipping\n");
      return;
    }
    ToolCommon::Log($nodeLogFH, " OK\n");
  }

  ToolCommon::Log($nodeLogFH, sprintf("ISAM running R%d.%d, get active osp...", $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{MINOR}));
  my $showShubConf = Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(1,1), REGEXP, 0) == 0
    ? "" : "config-file";
  my $activeConfig = IsamCli::GetLanxActiveOSWP($isamCliSession, $showShubConf);
  
  if (!defined($activeConfig) || ($activeConfig eq "")) {
    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s... skipping!!!\n", IsamCli::GetError()),
				 EC_GET_SHUB_CONF_FAILED,
				 sprintf(": %s", IsamCli::GetError()),
				 1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFH, "  OK\n");
  }
  if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) == 0) &&
      ($activeConfig ne "tftp")) {
    FamilyCommon::IncrErrorCount("SHUB not booting from tftp, change setting first and try again... skipping\n",
				 EC_SHUB_NO_TFTP_BOOT,
				 "(set to $activeConfig)",
				 1);
    return 0;
  }

  ToolCommon::Log($nodeLogFH, "Checking Active Software is committed...");
  if ($oswpInfo->{ACTIVE}{COMMITED} ne "committed") {
    #-- STEP 2: If not committed -> commit to remove passive OSWP
    ToolCommon::Log($nodeLogFH, "  not committed.\n");
    ToolCommon::Log($nodeLogFH, "Committing Active OSWP...");
    $ret = IsamCli::CommitSWP($isamCliSession, $oswpInfo->{ACTIVE}{INDEX});
    if (!defined($ret) || ! $ret) {
      FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				   EC_COMMIT_FAIL,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return 0;
    } else {
      ToolCommon::Log($nodeLogFH, " OK\n");
    }
  } else {
    ToolCommon::Log($nodeLogFH, " OK\n");
  }
  #-- STEP 3: Abort any active downloads
  ToolCommon::Log($nodeLogFH, "Aborting Active Downloads...");
  if (uc($oswpInfo->{PASSIVE}{NAME}) ne NO_OSWP) {
    if (!defined(IsamCli::AbortDownloadSWP($isamCliSession, $newSwpIndex))) {
      FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				   EC_ABORT_DL_FAIL,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return 0;
    } else {
      #-- no error checking as it fails if no active one
      ToolCommon::Log($nodeLogFH, " OK\n");
      sleep 2;
    }
  } else {
    ToolCommon::Log($nodeLogFH, " OK (no passive OSWP)\n");
  }

  # Release transaction - Reduce SWP is performing SNMP sets
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  #    if (((uc($ntInfo->{$activeNt}{TYPE}) eq 'ECNT-A') && ($ntInfo->{$activeNt}{ICS} < 10)) || ($ntInfo->{$activeNt}{ICS} eq "")) {
  $swParSize = ToolCommon::GetSwParSize();
  if( not defined $swParSize or $swParSize < 0 ) {
     return 0 if( FamilyCommon::ReportStatus( EC_SW_RED_FAILED, SNMPComm::GetError(), undef ) );
  }
  if (! $params->{NOREDUCE}) {
    #-- STEP 3bis: Reduce current SWP if migrating from R2.4 or higher -- 1.0.2
    if (((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) > 0) &&
	 (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 1), REGEXP, 0) <= 0)) ||
	# or SW partition size not according to R2.2 standards..  -- 1.0.4
	($swParSize < MINIMALSWPARSIZE) ||
	# or active release is 2.3 and new release is 3.0 or higher.  -- 1.2.1
	(((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) == 0) &&
	  (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 0), REGEXP, 0) > 0))) ||
	($params->{FORCEREDUCE})) {
      if (($swParSize < MINIMALSWPARSIZE) &&
	  (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) > 0)) {
	FamilyCommon::IncrErrorCount("Board not fitted to accomodate this release, please upgrade NT HW to ICS10 or higher... skipping\n",
				     EC_PAR_SIZE_TOO_SMALL,
				     "(SW partition size: $swParSize)",
				     1);
	return 0;
      }
      if (((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 4), REGEXP, 0) >= 0) ||
	   (((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) >= 0) &&
	     (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 0), REGEXP, 0) > 0)))) &&
	  (! $oswpInfo->{ACTIVE}{REDUCED})) {
	ToolCommon::Log($nodeLogFH, sprintf("swParsize: '%s', minimum: '%s'\n",
					    $swParSize, MINIMALSWPARSIZE)) if $params->{DEBUG};
	if (($swParSize <= MINIMALSWPARSIZE) || ($params->{FORCEREDUCE})) {
	  # No need to reduce if SW Partition size is bigger then the boundary for ECNT-A ICS10 or > (e.g. ECNT-C with 512 Mb SAN)
	  if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 1), REGEXP, 0) <= 0) ||
	      ($params->{FORCEREDUCE})) {
	    # Exception: when forced reduction is requested we'll do it in any case          -- 1.5.1.2
	    $ret = Octopus::ReduceActiveSWP($isam, $snmpSession, $childCount, $dir,
					    Capability::ReductionChar(Capability::IsamLSFamily()), $dirName);
	  }
	  if (defined($ret) && !$ret) {
	    FamilyCommon::IncrErrorCount("",
					 EC_SW_RED_FAILED,
					 sprintf(": %s", Octopus::GetError()),
					 1);
	    return 0;
	  }
	  sleep(10);  # wait for the OSWP to stabilize.  We want some info back on the new  GetOswpInfo.
	  #-- STEP 3bis': Get passive SW index again (has been changed while reducing the SWP)
	  $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE},
					      undef, $toolData->{RETRIES});
	  if (! defined($oswpInfo)) {
	    FamilyCommon::IncrErrorCount("",
					 EC_GET_OSWP_FAILED,
					 "(after SW reduction)",
					 1);
	    return 0;
	  }
	}
      }
      $newSwpIndex = defined($oswpInfo->{PASSIVE}{INDEX}) ? $oswpInfo->{PASSIVE}{INDEX} : undef;
    }
    #-- END STEP 3bis -- 1.0.2
  }

  if (!defined($newSwpIndex)) {
    FamilyCommon::IncrErrorCount("Obtaining SWP number of passive OSWP failed... skipping\n",
				 EC_NO_PASS_SWPIDX_FOUND,
				 undef,
				 1);
    return 0;
  }
  Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
  return undef if( ToolCommon::IsTransferOnGoing() );

  # Again take the transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  ToolCommon::Log($nodeLogFH, "Configure Download Parameters...");
  #-- STEP 4: Configure Download Parameters
  $ret = IsamCli::ConfigureSWPDownload($isamCliSession, $newSwpIndex,
				       $params->{PRIMTFTPIP},
				       'complete-set');
  if (!defined($ret) || ! $ret) {
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				 EC_CONFIG_DL_FAILED,
				 undef,
				 1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFH, " OK\n");
  }

  #-- STEP 5: Download SWP
  # -- 1.0.4 START
  # Lanx SW has to be downloaded separately if the active SW is R2.0 or R2.1:
  # once to the passive conf
  # once to the NT
  my $newRelCmp = Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(2,2), REGEXP, 0);
  my $actRelCmp = Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2,2), REGEXP, 0);
  $shubSwName = Capability::FindShubSwName($params->{SWPNAME}, REGEXP, $toolData->{TFTPDIR}, $toolData->{SHUBSWPATH}, $toolData->{SHUBSWNAME},
					   Capability::IsamLSFamily());
  if (($shubSwName eq "") && (uc($ntInfo->{NTA}{TYPE}) eq 'ECNT-A') &&   # 1.2.4.2
      ($actRelCmp < 0)) {
    # Direct upgrade from R2.0/R2.1 to R2.2 or higher (<= R2.4) only required for ECNT-A.
    FamilyCommon::IncrErrorCount("FAILED, could not determine SHUB SW name from '$params->{SWPNAME}', '$params->{OSWP}'... skipping!!!\n",
				 EC_NO_SHUB_SW,
				 "(OSWP:  $params->{SWPNAME}, SWP: $params->{OSWP})",
				 1);
    return 0;
  }
  #    }
  if (($newRelCmp <= 0) || ($actRelCmp <= 0)) {
    # New release is 2.2 or lower or Active release is R2.2 or lower: download SHUB SW to SHUB
    my $newConfig;
    if ($activeConfig eq "config-1") {
      $newConfig = "config2";
    } elsif ($activeConfig eq "config-2") {
      $newConfig = "config1";
    } else {
      # config is already set to tftp (system is running R2.2
      $newConfig = $activeConfig;
    }
    if (($newConfig ne "tftp") || ($params->{DLSHUB})) {
      # SW retrieved from NT SW so no need to transfer to SHUB
      ToolCommon::Log($nodeLogFH, sprintf("ISAM running R%d.%d, update SHUB SW...", $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{MINOR}));
      $time1 = time();
      if (!IsamCli::DownloadShubSW($isamCliSession, $shubSwName, $params->{PRIMTFTPIP}, $showShubConf)) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED, SHUB SW download failed: %s\n... skipping!!!\n", IsamCli::GetError()),
				     EC_SHUB_SW_DL_FAILED,
				     sprintf(": %s", IsamCli::GetError()),
				     1);
	return 0;
      } else {
	$time2 = time();
	ToolCommon::Log($nodeLogFH, sprintf("  OK, downloaded in %d seconds.\n", $time2 - $time1));
	sleep 10;
      }
    }
    #     }
  }
  # Also download to NT using TFTP
  # To be certain: remove any Lanx.Z present /Sw left there from a possible previous DL
  # It will make this tftp to fail if a Lanx.Z is present in /Sw
  # Only download new SHUB SW if new SW is R2.2
  if (Capability::ReleaseCompare($newRelNum,
				 Capability::ConvertRelease(2, 2), REGEXP, 0) == 0) {
    if ($actRelCmp <= 0) {
      if (! Octopus::RemoveNtShubSw($params->{DEBUG}, $dir, $isam, $dirName)) {  # Also enables write on /Sw for Downgrade on R2.3 or R2.4
	FamilyCommon::IncrErrorCount("Removing SHUB SW failed, check logs... skipping\n",
				     EC_DEL_SHUB_SW_FAILED,
				     sprintf(": %s", Octopus::GetError()),
				     1);
	return 0;
      }
    } else {
      if (! Octopus::EnableSwWrite($params->{DEBUG}, $dir, $isam, $dirName)) {
	FamilyCommon::IncrErrorCount("Enable write on /Sw failed, check logs... skipping\n",
				     EC_ENABLE_WR_FAILED,
				     sprintf(": %s", Octopus::GetError()),
				     1);
	return 0;
      }
    }
    # TFTP errors: errno values are returned
    my $tftp = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});
    ToolCommon::Log($nodeLogFH, $actRelCmp > 0 ? "Transfer SHUB SW to /Sw..." : "Transfer SHUB SW to /pub/Sw/Download...");
    if (defined($tftp)) {
      $tftp->binary();
      my $sourceFile = $toolData->{TFTPDIR} . "/" . $shubSwName;
      my $destinationFile = $actRelCmp > 0 ? "/Sw/Lanx.Z" : "/pub/Sw/Download/Lanx.Z";
      ToolCommon::Log($nodeLogFH, sprintf("tftp put %s %s\n", $sourceFile, $destinationFile)) if $params->{DEBUG};
      $time1 = time();
      if ($tftp->put($sourceFile, $destinationFile) ne "") {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s... skipping\n", $tftp->error()),
				     EC_TFTP_PUT_FAILED,
				     sprintf("(to /pub/Sw/Download/Lanx.Z): %s", $tftp->error()),
				     1);
	return 0;
      } else {
	$time2 = time();
	ToolCommon::Log($nodeLogFH, sprintf("  OK, transfered %d bytes in %d seconds\n", $tftp->bytesTransfered(), $time2 - $time1));
	sleep 30;  # Make sure SWP download goes well...
      }
      if (($actRelCmp > 0) && 
	  ((uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") &&
	   (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY"))){
	# Sync to passive if redundant
	if (Octopus::SyncLanxSw($params->{DEBUG}, $dir, $isam, $dirName) < 0) {
	  FamilyCommon::IncrErrorCount("",
				       EC_SYNC_TO_STBY_FAILED,
				       sprintf(": %s", Octopus::GetError()),
				       1);
	  
	  return 0;
	} else {
	  sleep 30; # Wait for Lanx.Z to be synced
	}
      }
    } else {
      FamilyCommon::IncrErrorCount("  FAILED to open tftp session\n",
				   EC_TFTP_OPEN_FAILED,
				   undef,
				   1);
      return 0;
    }
  }
  # -- 1.0.4 END

  ToolCommon::Log($nodeLogFH, "Downloading OSWP-$newSwpIndex ...");
  $time1 = time();
  $ret = IsamCli::DownloadSWP($isamCliSession, $newSwpIndex, $params->{OSWP});
  if (!defined($ret) || ! $ret) {
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				 EC_DL_SWP_FAILED,
				 sprintf(": %s", IsamCli::GetError()),
				 1);
    return 0;
  } else {
    $time2 = time();
    ToolCommon::Log($nodeLogFH, sprintf("  OK, transfered in %d seconds\n", $time2 - $time1));
  }
  $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE}, undef, $toolData->{RETRIES});
  if (!defined($oswpInfo)) {
    FamilyCommon::IncrErrorCount("",
				 EC_GET_OSWP_FAILED,
				 sprintf("(after SWP download): %s", ToolCommon::GetError()),
				 1);
    return 0;
  }

  # Check for any files marked as not-available.  Workaround for ANTmt25573.  Using the MINIMUMSET option it is expected that SW for
  # boards not yet known to the NT are downloaded too.  It is not the case.  So we'll tftp the files that are not available to the
  # /pub/Sw directory.
  # We will only do this for NTs running the intelligent download and after verifying that there is still enough room on the disk.
  if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3,1), REGEXP, 0) >= 0) {
    ToolCommon::Log($nodeLogFH, "Check for files marked as not-available but included in the minimum-set...");
    my $swList = IsamCli::ListSwpFiles($isamCliSession, basename($params->{SWPNAME}));
    my $filesMissing = 0;
    my $boardsInMinSet;
    if (defined($swList)) {
      $boardsInMinSet = GetMinimumSet($nodeLogFH);
      if (defined($boardsInMinSet)) {
	foreach my $package (keys %{$swList}) {
	  foreach my $fileName (keys %{$swList->{$package}}) {
	    ToolCommon::Log($nodeLogFH, sprintf("package: %s, file: %s\n", $package, $fileName)) if $params->{DEBUG};
	    if ($swList->{$package}{$fileName}{STATE} eq "not-available") {
	      if (defined($boardsInMinSet->{$swList->{$package}{$fileName}{BOARD}})) {
		ToolCommon::Log($nodeLogFH, "Including $fileName in to-be-downloaded list\n") if $params->{DEBUG};
		$filesMissing = 1;
	      } else {
		# Remove files from the list that are not included in the minimum set.
		ToolCommon::Log($nodeLogFH, "Removing $fileName from SW list\n") if $params->{DEBUG};
		delete $swList->{$package}{$fileName};
	      }
	    }
	  }
	}
	if ($filesMissing) {
	  # Got at least one, so download using tftp
	  if (! DownloadMissingFiles($nodeLogFH, $isamCliSession, $swList, $isam)) {
	    FamilyCommon::IncrErrorCount("",
					 EC_TFTP_MISSING_FAILED,
					 "",
					 1);
	    return 0;
	  } else {
	    ToolCommon::Log($nodeLogFH, "  OK files sent\n");
	  }
	} else {
	  ToolCommon::Log($nodeLogFH, "  Ok, no files to be sent\n");
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED to parse the SWP file: %s", FamilyCommon::GetError()),
				     EC_PARSE_SWP_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return 0;
      }
    } else {
      FamilyCommon::IncrErrorCount("  FAILED",
				   EC_LIST_SWFILES_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return 0;
    }
  }
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  IsamCli::AllowAlarms($isamCliSession);
  FamilyCommon::CloseCliSession2Close();
  return 1;
}

1;
