package DownloadIsamIHUB;

=head1 IHUB ISAM 7302/7330 SW download module

=begin html
This library contains the functions dedicated to perform prechecks for a 7302/7330 ISAM.

=end html

=cut

#==============================================================================
#-- File:         DownloadIsamIHUB.pm
#-- Date:         2018/06/20
#-- Company:      Nokia
#-- Author:       Dhineshkumar Manikannan
#------------------------------------------------------------------------------
#-- Function:     Script to download new OSWP to IHUB ISAMs
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   3.1.0   - 2018/06/20 : First implementation (Clubed all IHUB board)
#--   3.1.1   - 2019/03/24 : ALU02559738


=head1 VERSION

=begin html
Current version: "DownloadIsamIHUB.pm - 3.1.1 -- 2019/03/24"

=end html

=cut

my $VERSION = "DownloadIsamIHUB.pm - 3.1.1 -- 2019/03/24";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP<a/></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use File::stat;
use File::Basename;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;

use Net::TFTP;
use Net::SNMP;

use IsamCli;
use Crypt::Lite;
use Utils;  # -- BB --
require Octopus;
use Capability;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
require FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :SWDL);
use ToolCommon();
use PBMTGlobal;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Download($$);
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $config;
my $toolData;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 Download($$$)

=begin html
This function is called to perform a SW download of a node of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>ntType</td><td>the type of NT inserted</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Download($$) {
  my ($isam, $ntType) = @_;
  my $isamCliSession = undef;
  my $swParSize;
  my $shubSwName;
  my ($time1, $time2);
  my $activeNt;
  my $possible = 0;
  my $reason = "";
  my $ret;
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childCount = FamilyCommon::GetChildNum();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();
  my $dbVersion = $params->{DBVER_TGTREL};
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;

  if (not defined($params->{OSWP})) {
    FamilyCommon::ReportStatus( EC_OSWP_MISSING, undef, undef );
    return 0;
  }
  if (not defined($params->{DBVER_TGTREL})) {
    FamilyCommon::ReportStatus( EC_DB_VER_UNDEFINED, undef, undef );
    return 0;
  }
  Logger::LogTrace($nodeLogFH, "Check OSWP variant...");
  if (! Capability::OswpVariant($params->{SWPNAME}, $params->{'BOARD-FAMILY'}, 1)) {
    FamilyCommon::ReportStatus( EC_WRONG_VARIANT, "Expecting ". Capability::GetOswpVariant( $params->{'BOARD-FAMILY'}, 'ALL') );
    return 0;
  }
  #-- Open CLI Session
  Logger::LogTrace($nodeLogFH, "Open CLI...");
  $isamCliSession = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($isamCliSession)) {
      FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, ToolCommon::GetError(), undef );
      return 0;
  }
  FamilyCommon::SetCliSession2Close($isamCliSession);  # Store for IncrErrorCount

  ### Acquire config-lock ownership
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return 0 if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");


  #-- STEP 1: NT type information and check Active OSWP is committed
  #-- Get NT type information
  Logger::LogTrace($nodeLogFH, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($isamCliSession);
  return 0 if (not defined($ntInfo) ); 
  $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';

  my $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE}, undef,
					 $toolData->{RETRIES});
  if (! defined($oswpInfo)) {
       FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED, ToolCommon::GetError(), undef );
       return 0;
  }

  # Check whether the NT type is present in the SWP 
  $ntType = uc($ntInfo->{$activeNt}{TYPE}); 
  if (!FamilyCommon::BoardInSwp($nodeLogFH, $ntType)) {
       return undef if( FamilyCommon::ReportStatus( EC_NT_NOT_IN_SWP, FamilyCommon::GetError() ) );
  }
  ### No need to perform Board Availability check for 7367 and 7367_SX
  my $possibleBoards = Capability::Isam7367_SXFamily() ."|". Capability::Isam7367Family();
  my $ltConfig = IsamCli::GetSlotConfig($isamCliSession);
  if (not defined($ltConfig)) {
      return undef if( FamilyCommon::ReportStatus( EC_GET_LT_FAILED, IsamCli::GetError() ) );
  } elsif ($params->{'BOARD-FAMILY'} !~ /^($possibleBoards)$/) {
      my %ltHandled = ();
      my $swMissing = "";
      foreach my $ltNum (sort keys %$ltConfig) {
         next if ( defined($ltHandled{$ltConfig->{$ltNum}}) );
         next if ( FamilyCommon::BoardInSwp($nodeLogFH, uc($ltConfig->{$ltNum})));
         $swMissing .= sprintf(" %s", uc($ltConfig->{$ltNum}));
         $ltHandled{$ltConfig->{$ltNum}} = 1;
      }
      if ($swMissing ne "") {
         return undef if( FamilyCommon::ReportStatus( EC_LT_NOT_SUPPORTED, $swMissing ) );
      }
  }
   my ($newMajor, $newMinor, $newSuffix, $newFullRelNum, $newRelNum );
   my $new = basename($params->{SWPNAME});
   my $regExp = REGEXP;
   if ($new =~ /$regExp/) {   # -- Start 1.0.4
     $newMajor = $1;
     $newMinor = $2;
     $newSuffix = $3;
     $newFullRelNum = Capability::ConvertRelease($newMajor, $newMinor, $newSuffix, $dbVersion); # -- 1.2.0
     $newRelNum =  Capability::ConvertRelease($newMajor, $newMinor, 0, $dbVersion);
   }                          # -- End 1.0.4

  my $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'},
						 $oswpInfo->{'ACTIVE'}{'SUFFIX'},
                         $oswpInfo->{'ACTIVE'}{'DBVERSION'} );
  my $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
					     $oswpInfo->{'ACTIVE'}{'MINOR'},
                                             0,
                         $oswpInfo->{'ACTIVE'}{'DBVERSION'} );

  my $newSwpIndex = $oswpInfo->{PASSIVE}{INDEX};
  if (! $params->{FORCEDL}) {
    if ($oswpInfo->{'ACTIVE'}{'DBVERSION'} ne $params->{DBVER_TGTREL} ) {
      #-- Verify Migrate path
      Logger::LogTrace($nodeLogFH, "Migrate restrictions...\n");
      my ($restricted, $reason) = Capability::IsMigrateRestricted( $ntInfo->{$activeNt}{TYPE}, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Migrate restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    } else {
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntInfo->{$activeNt}{TYPE}, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    }
  } else {
    ToolCommon::Log($nodeLogFH, "Forced download, continue without checks...\n");
  }

  my $dbVersionActive = $oswpInfo->{'ACTIVE'}{'DBVERSION'};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveNum = substr($dbVersionActive,1);
  my $dbVersionTargetNum = substr($dbVersionTarget,1);

  if ($dbVersionActive lt "A3.200" ) {
    # From R3.1 onwards the same OSWP can be redownloaded so we have to allow it.
    Logger::LogTrace($nodeLogFH, "Checking new software already present...");
    if ( (basename($oswpInfo->{ACTIVE}{NAME}) eq basename($params->{OSWP}) ) || # -- 1.2.3
	     ( (basename($oswpInfo->{PASSIVE}{NAME}) eq basename($params->{OSWP}) ) && ($oswpInfo->{PASSIVE}{AVAIL} eq 'enabled') )
       ) { #-- 1.2.3
         FamilyCommon::AdvanceToNext("Already present... skipping\n");
         return;
    }
  }
  
  #Check any download is ongoing. and If any download is ongoing 
  #wait for file transfer to complete.
  Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
  return undef if( ToolCommon::IsTransferOnGoing() );

 # Check file transfer settings
  if ($dbVersionActive ge "A3.300" ) {
      Logger::LogTrace($nodeLogFH, "Check security filetransfer settings...");
      my $fileXfrSettings = IsamCli::GetFileTransferSettings($isamCliSession);
      if (! defined($fileXfrSettings)) {
        FamilyCommon::ReportStatus( EC_GET_FILEXFR_FAILED, IsamCli::GetError(), undef );
        return 0;
      }
  }


  Logger::LogTrace($nodeLogFH, "Checking Active Software is committed...");
  if ($oswpInfo->{ACTIVE}{COMMITED} ne "committed") {
    #-- STEP 2: If not committed -> commit to remove passive OSWP
    Logger::LogTrace($nodeLogFH, "Committing Active OSWP...");
    $ret = IsamCli::CommitSWP($isamCliSession, $oswpInfo->{ACTIVE}{INDEX});
    if (!defined($ret) || ! $ret) {
      FamilyCommon::ReportStatus( EC_COMMIT_FAIL, IsamCli::GetError(), undef );
      return 0;
    }
  }
  #-- STEP 3: Abort any active downloads
  Logger::LogTrace($nodeLogFH, "Aborting Active Downloads...");
  if (uc($oswpInfo->{PASSIVE}{NAME}) ne NO_OSWP) {
    if (!defined(IsamCli::AbortDownloadSWP($isamCliSession, $newSwpIndex))) {
      FamilyCommon::ReportStatus(EC_ABORT_DL_FAIL, IsamCli::GetError(), undef);
      return 0;
    }
    #-- no error checking as it fails if no active one
    sleep 6;
  } else {
    Logger::LogTrace($nodeLogFH, "No passive OSWP found to abort existing\n");
  }

  # Release transaction - Reduce SWP is performing SNMP sets
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
  return 0 if( ToolCommon::ReleaseTransaction() );
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

 if ($params->{FORCEREDUCE} ) {
    $possibleBoards = Capability::Isam7367_SXFamily() ."|". Capability::Isam7367Family();
    if ($params->{'BOARD-FAMILY'} =~ /^($possibleBoards)$/ ) {
       FamilyCommon::ReportStatus(EC_REDUCE_OPR_NOTALLOW,undef );
       return 0;
    }
    # Exception: when forced reduction is requested we'll do it in any case
    # Check for waiting-for-sw in case LT software not present 
    my $ltNoSWFound = ToolCommon::CheckLTSWPresence($isamCliSession);
    if ( %$ltNoSWFound ) {
      my @ltTmpArr;
      foreach my $lt (keys %{$ltNoSWFound}) 
      {
        Logger::LogNotify($nodeLogFH, "No software found for ISAM board : $ltNoSWFound->{$lt}{TYPE}!!\n");
	    push @ltTmpArr, $ltNoSWFound->{$lt}{TYPE};
      }
      FamilyCommon::ReportStatus(EC_NO_LT_SWP_FOUND, sprintf("Board - %s : No board Software found under /Sw \n", join(", ", @ltTmpArr)), undef );
      return 0;
    }

    $ret = Octopus::ReduceActiveSWP($isam, $snmpSession, $childCount, $dir,
				    Capability::ReductionChar(Capability::IsamFamily()), $dirName,$isamCliSession);
    if (defined($ret) && !$ret) {
      FamilyCommon::ReportStatus( EC_SW_RED_FAILED, Octopus::GetError(), undef );
      return 0;
    }
    sleep(20);  # wait for the OSWP to stabilize.  We want some info back on the new  GetOswpInfo.
    #-- STEP 3bis': Get passive SW index again (has been changed while reducing the SWP)
    $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE}, undef, $toolData->{RETRIES});
    if (! defined($oswpInfo)) {
      FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, "(after SW reduction)", undef );
      return 0;
    }
    $newSwpIndex = defined($oswpInfo->{PASSIVE}{INDEX}) ? $oswpInfo->{PASSIVE}{INDEX} : undef;
  }



  # Again take the transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return 0 if(  ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

 
  Logger::LogTrace($nodeLogFH, "Configure Download Parameters...");
  #-- STEP 4: Configure Download Parameters
  $ret = IsamCli::ConfigureSWPDownload($isamCliSession, $newSwpIndex,
				       $params->{PRIMTFTPIP},
				       'complete-set');
  if (!defined($ret) || ! $ret) {
    FamilyCommon::ReportStatus( EC_CONFIG_DL_FAILED, IsamCli::GetError(), undef );
    return 0;
  }

  ### DEVEL-NOTES : AUTO_VERIFY AFTER DOWNLOAD
  ### 1 - Check it is already configured by admin
  ###     YES - Blindly continue next step
  ###     NO  - Configure auto-verify
  ### 2 - Download Oswp
  ### 3 - Disable auto-verify, If it is configured through step-1
  my $autoVerifyStatus = 0;
  if ( ( $ntType !~ /^FANT/ ) and
       (Capability::ReleaseCompare($fullActiveRelNum, Capability::ConvertRelease(6,0,3), REGEXP, 1,1) >= 0 ) and
       (Capability::ReleaseCompare($newFullRelNum, Capability::ConvertRelease(6,0,3), REGEXP, 1,1) >= 0 ) 
     ) {
      $autoVerifyStatus = IsamCli::IsConfigured($isamCliSession, "info configure software-mngt oswp $newSwpIndex detail", "auto-verify enabled");
      if( $autoVerifyStatus == 1 ) {
          $autoVerifyStatus = 0;
      } elsif (defined $params->{AUTOVERIFY} and $params->{AUTOVERIFY}) {
          $autoVerifyStatus = DownloadAutoVerify($isamCliSession, $newSwpIndex, "enabled");
          return $autoVerifyStatus if ($autoVerifyStatus == 0 );
      }
  }
  Logger::LogTrace($nodeLogFH, "Downloading OSWP...");
  $time1 = time();
  my $downloadError = "";
  my $returnDownloadStatus = IsamCli::DownloadSWP($isamCliSession, $newSwpIndex, $params->{OSWP});
  if ($returnDownloadStatus) {
    $time2 = time();
    Logger::LogTrace($nodeLogFH, sprintf("  OK, transfered in %d seconds\n", $time2 - $time1));
  } else {
     $downloadError = IsamCli::GetError();
  }
  if ($autoVerifyStatus == 1) {
      $autoVerifyStatus = DownloadAutoVerify($isamCliSession, $newSwpIndex, "disabled");
      return $autoVerifyStatus if ($autoVerifyStatus == 0 );
  }
  if ($returnDownloadStatus == 0) {
    FamilyCommon::ReportStatus( EC_DL_SWP_FAILED, $downloadError, undef );
    return 0;
  }
  $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE}, undef, $toolData->{RETRIES});
  if (!defined($oswpInfo)) {
    FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, "(after SWP download)", undef );
    return 0;
  }

  sleep(20); 
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
  return 0 if( ToolCommon::ReleaseTransaction() );
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  IsamCli::AllowAlarms($isamCliSession);
  FamilyCommon::CloseCliSession2Close();
  return 1;
}

sub DownloadAutoVerify($$$) {
  my($isamCliSession, $oswpIndex, $autoVerify ) = @_;
  my $returnAutoVerify = IsamCli::ConfigureSWPDownloadVerify($isamCliSession, $oswpIndex, $autoVerify);
  if ($returnAutoVerify) {
    FamilyCommon::ReportStatus( EC_CONFIG_DL_FAILED, IsamCli::GetError(), undef );
    return 0;
  }
  return 1;
}

1;
