package DownloadIsamSHUB;

=head1 ISAM 7302/7330 SW download module

=begin html
This library contains the functions dedicated to perform prechecks for a 7302/7330 ISAM.

=end html

=cut

#==============================================================================
#-- File:         DownloadIsamSHUB.pm
#-- Date:         06.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Dhineshkumar Manikannan for code optimization
#                 Donny Janssens for the initial version
#--               Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to download new OSWP to ANSI ISAMs
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2005/11/06 : First implementation
#--   1.0.1 - 2005/12/15 : Added unix ping in stead of udp ping.  Caused by
#--                        BDFhw70276
#--                        Added a regexp for determining major and minor
#--                        release ids.
#--   1.0.2 - 2006/01/16 : Include a call to ReduceSwp.pl for R2.4 upgrades
#--   1.0.3 - 2006/01/20 : Node prefix configurable.
#--                        AMS can't do node name to IP address convertion (
#--                        ip address/node name not stored in /etc/hosts)
#--   1.0.4 - 2006/02/13 : SW DL from active SW R2.1 possible.
#--   1.0.5 - 2006/02/27 : SW DL from active SW R2.0 possible.
#--   1.2.0 - 2006/03/16 : Intoduction of the capability model
#--   1.2.1 - 2006/03/19 : SW reduction needs to be done in case active release
#--                        is R2.3 and new release is R3.0 or higher (BDFhw92378)
#--                        Verify environment
#--   1.2.2 - 2006/05/31 : Using --debug results in failure (undefined value).
#--   1.2.3 - 2006/06/08 : ECNT-C NT SW file has other prefix the ENCT-A
#--                        Compare basenames of OSWP instead of full name.
#--   1.2.4.1 - 2006/07/25 : ANTms06883 - remove CLI logfile frmo /tmp on exit
#--                          of emergency exit
#--   1.2.4.2 - 2006/08/21 : ANTms08417 - do not check on SHUB SW size for releases
#--                          after R2.2
#--   1.2.5.1 - 2006/08/23 : ANTms10868 - create log file per NE in stead of
#--                          appending all in the child logfile.
#--             2009/09/25 : Set to type-based CLI positioning.
#--             2006/09/27 : ANTms15715: take transaction before backing up
#--                          NT database.
#--             2006/09/29 : Include program warnings in log files.
#--   1.4.2   - 2006/10/09 : ANTms17315: info about migration or upgrade
#--                          issues.
#--   1.4.3   - 2006/10/18 : Timestamp logging
#--   1.5 1.1 - 2007/01/09 : ANTms30214 - Do not abort passive OSWP is set to NO_OSWP.
#--   1.5.1.2 - 2007/01/29 : Include --force.  Force an online SW reduction.
#--   1.5.1.3 - 2007/02/02 : Allow download of the same OSWP from R3.1 onwards.
#--   1.5.1.4 - 2007/05/04 : ANTms49911: Small correction in Ctrl-c handling (don't kill pid
#--                          that have already exited)
#--   1.5.2 - 2007/06/18   : ANTms51632 allow multiple users for same ISAM.
#--   1.5.3   - 2007/06/22 : ANTms52362 - Make sure node status result file is never
#--                          empty (Added AdvanceToNext).
#--   1.5.4   - 2007/06/26 : ANTms51632 allow multiple users for same ISAM.
#--                          Corrected incorrect check.
#--   1.5.5   - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.5.6   - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.5.7   - 2007/08/22 : ANTms60869: Make NT community string configurable
#--   1.5.8   - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.5.9   - 2008/03/04 : - ANTms86893, error in IP regular expression
#--                          - Ignore unknown stdby NT when status is not operational.
#--   1.5.10  - 2008/04/29 : ANTmt04599.  NANT-A TYPE-B SW name changed between R3.2 and R3.3
#--                                       Implement logic for SW name retrieval in case of
#--                                       'reduced' SW
#--   1.5.11  - 2008/05/06 : ANTmt05829.  Check ip address immediately after parsing the OSWP file
#--                                       in ParseCommandLine.
#--   1.5.12  - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.5.13  - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--             2008/07/25 : Add check on file transfer protocol being used in the ISAM.
#--   1.5.14  - 2008/10/01 : ANTmt25573.  Include a workaround for files marked as not available after SW download.
#--   1.6.0   - 2008/10/03 : ANTmt44696.  R3.7.10 and IPD ISAM family.
#--   1.6.1   - 2009/04/02 : Check SW support in the SWP for all LTs in the system
#--   1.6.2   - 2009/04/20 : ANTmt54050. LT Board Type not displayed during generation of error message for SW support unavailability
#                            pertaining to missing LT boards.
#--   1.6.3   - 2009/04/22 : ANTmt54584
#--   1.6.4   - 2009/04/24 : Release transaction if TO = 127.0.0.1
#--   1.6.5   - 2009/06/02
#--   1.6.6   - 2010/04/02 : ANTmt91104 / ALU00100007 PBMT: SW dir permissions not set when not reducing
#--   1.6.7   - 2010/04/26 : ALU00259276 - Boardlist support check in SWP skipped if force download option is used
#--   1.6.8   - 2011/02/02 : ALU01006984 - Undo workaround done for ALU00100007 since issue is solved in NT SW 
#--   3.0.1   - 2011/03/29 : ALU01031457 - Support for R4220 & R4230
#--   3.0.2   - 2012/12/07 : ALU01809526 - Fix for SW Mgnt Busy error in redundant setup
#--   3.0.3   - 2013/07/03 : ALU01882966
#--   3.0.4   - 2013/08/23 : change to resolve RLAB issue for transaction release failure
#--   3.0.5   - 2013/10/24 : ALU01913513
#--   3.0.6   - 2014/06/12 : Sleep for 20 seconds before releasing the transaction 
#--   3.0.7   - 2015/03/26 : ALU02055435  
#--   3.0.8   - 2015/05/04 : ALU02062224  
#--   3.0.9   - 2015/07/29 : ALU02106765
#--   3.0.10  - 2015/01/07 : ALU02163612
#--   3.0.11  - 2016/07/15 : ALU02237966
#--   3.1.0   - 2018/06/20 : ALU02460863 & ALU02471697


=head1 VERSION

=begin html
Current version: "DownloadIsamSHUB.pm - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "DownloadIsamSHUB.pm - 3.1.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP<a/></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use File::stat;
use File::Basename;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;

use Net::TFTP;
use Net::SNMP;

use IsamCli;
use Crypt::Lite;
use Utils;  # -- BB --
require Octopus;
use Capability;
use ToolCommon;
use PBMTGlobal;
require FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :SWDL);
use ErrorCodes qw(:SWRD);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetMinimumSet($);
sub DownloadMissingFiles($$$$);
sub Download($$);
sub CheckSwPresenceInIsam($$$);
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $config;
my $toolData;

#=============================================================================
my $swArrRef;
my $swRet;
my $possibleBoards = "";

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 GetMinimumSet($$)

=begin html
Determine the minimum set of boards in the SWP taken from '$params'.  The list is returned as a pointer to a hash.

=end html

=cut

sub GetMinimumSet($) {
  my ($logFile) = @_;
  my $swpFile;
  my $line;
  my $board = undef;
  my %minSet = ();

  if (open($swpFile, $params->{NOTFTP} ? $params->{SWPNAME} : "$toolData->{TFTPDIR}/$params->{SWPNAME}")) {
      while ($line = <$swpFile>) {
          chomp $line;
          $line =~ s/\r//g;
          $line =~ s/\n//g;
          Logger::LogDebug($logFile, "SWPfile line read: $line\n");
	      if ($line =~ /^\s*MINIMUMSET\s*:\s*([^\s]*)/) {
	          my $inMinSet = $1;
	          if (defined $board and (uc($inMinSet) eq "YES") ) {
	            $minSet{$board} = 1;
	          }
	          $board = undef;
          } elsif ($line =~ /^\s*DESCRIPTION\s+([^\s]+)/) {
	           $board = $1;
	         
          }
      }
      close $swpFile;
  } else {
    my $t = sprintf("Failed to open %s: $!", $params->{NOTFTP} ? $params->{SWPNAME} : "$toolData->{TFTPDIR}/$params->{SWPNAME}");
    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
    return undef;
  }
  return \%minSet;
}

=head2 DownloadMissingFiles($$$$$)

=begin html
Download files that were not downloaded during the regular download (can only happen for boards that are new in a certain release and not yet
known by the currently active release.  We have to parse the SWP file and only include boards that are in the minimum set.<br>
The following parameters are used:
<table>
  <tr><td>logFile</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>Pointer to an opened CLI session</td></tr>
  <tr><td>swList</td><td>List of the SW files for this new SW package</td></tr>
  <tr><td>params</td><td>the parameters provided to the script</td></tr>
</table>

=end html

=cut

sub DownloadMissingFiles($$$$) {
  my ($logFile, $cli, $swList, $isam) = @_;
  my $diskInfo;
  my $ok = 0;
  my $additionalSpaceNeeded = 0;
  my %missingFiles = ();
  my $path = FamilyCommon::GetToolDirName();
  # First retrieve the disk occupation
  $diskInfo = IsamCli::ReadDiskSpace($cli);
  if (not defined($diskInfo)) {
    my $t = sprintf("Failed to retrieve disk info: %s", IsamCli::GetError());
    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
    return $ok;
  }
    foreach my $package (keys %{$swList}) {
      foreach my $fileName (keys %{$swList->{$package}}) {
	      if ($swList->{$package}{$fileName}{STATE} eq "not-available") {
                if (not defined($missingFiles{$fileName})) {
  	                 my $fullName = sprintf("%s/%s/%s", $params->{OSWP_HOME}, dirname($params->{SWPNAME}), $fileName);
	                 $fullName =~ s|\/\/|\/|g;
	                 $additionalSpaceNeeded += -s $fullName;
	                 Logger::LogDebug($logFile, "Determine size of $fullName; additional space required: $additionalSpaceNeeded\n");
	                 $missingFiles{$fileName} = $fullName;
                }
	      }
      }
    }
    my $percentage = 100 - ((($diskInfo->{AVAIL} - $additionalSpaceNeeded) / $diskInfo->{TOTAL}) * 100);
    Logger::LogDebug($logFile, sprintf("Disk threshold before TFTP: %d%%, after TFTP: %d%%\n",
				      100 - (($diskInfo->{AVAIL} / $diskInfo->{TOTAL}) * 100), $percentage));
    if ($percentage >= 95) {
      my $t = "Operation discontinued: disk threshold would be exceeded ($percentage exceeds 95%)";
      FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
      return $ok;
    }
    Logger::LogDebug($logFile, "Disk threshold OK, start download of missing files...");

    if(not $params->{NOTFTP}) {
        my $tftpSession = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});
        if (not defined($tftpSession)) {
    	    my $t = "Failed to open a tftp session";
    	    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
            return $ok;
        }
    	$tftpSession->binary();
    	foreach my $fileName (keys %missingFiles) {
    	    my $destinationFile = "/pub/Sw/Download/$fileName";
    	    Logger::LogTrace($logFile, "Placing file '$missingFiles{$fileName}' to '$destinationFile' through TFTP\n");
    	    my $time1 = time();
    	    my $returnStatus = $tftpSession->put($missingFiles{$fileName}, $destinationFile);
    	    if ($returnStatus == 0) {
    	         Logger::LogDebug($logFile, sprintf("  OK, transfered %d bytes in %d seconds\n", $tftpSession->bytesTransfered(), time() - $time1));
                 next;
            }
    	    my $errStr = $tftpSession->error();
    	    if ($errStr=~ /no write permission/) {
    	         Logger::LogDebug($logFile, "Check the SW file '$fileName' present under /Sw via Octopus..\n");
                 if(CheckSwPresenceInIsam($isam, $path, $fileName))  {
    	              Logger::LogTrace($logFile, "SW file '$fileName' already present under /Sw via Octopus..\n");
                      next;
                 }
            }
            my $mesg = "Placing file '$missingFiles{$fileName}' to '$destinationFile' failed '$errStr'";
            Logger::LogError($logFile, "$mesg\n", undef, $Logger::LogOnlyHandle);
            FamilyCommon::SetAndLogError($logFile, $mesg, "$mesg\n");
            return $ok; 
    	}
    } else {
         foreach my $fileName (keys %missingFiles) {
             my $destinationFile1 = "/pub/Sw/Download/$fileName";
             my $ret = ToolCommon::SFTPClientWrap( $logFile, $config, $path, "$missingFiles{$fileName}|$destinationFile1", 'put', $isam->[0] );
             if( $ret == 0 ) {
    	         Logger::LogTrace($logFile, "File '$fileName' has placed successfully through SFTP\n");
                 next;
             }
    	     Logger::LogDebug($logFile, "Check the SW file '$fileName' present under /Sw via Octopus..\n");
             if(CheckSwPresenceInIsam($isam, $path, $fileName))  {
    	         Logger::LogTrace($logFile, "SW file '$fileName' already present under /Sw via Octopus..\n");
                 next;
             }
             my $mesg = "Placing file '$missingFiles{$fileName}' to '$destinationFile1' failed";
             Logger::LogError($logFile, "$mesg\n", undef, $Logger::LogOnlyHandle);
             FamilyCommon::SetAndLogError($logFile, $mesg, "$mesg\n");
             return $ok; 
        }
     }
     $ok = 1;
     return $ok;
}

sub CheckSwPresenceInIsam($$$) {
  my ($isam,$path,$swFile) = @_;
  my $matchFound = 0;
  my $dir = FamilyCommon::GetNeStorage();
  ($swRet, $swArrRef ) = Octopus::SwFilePresent($params->{DEBUG}, $dir, $isam, $path) if !defined($swArrRef);
  if ( $swRet < 0 ) {
      FamilyCommon::ReportStatus( EC_DIR_SW_LIST_FAILED, Octopus::GetError() );
      return ($matchFound);
  }
  if(defined($swArrRef)) { 
    foreach my $arrElem (@{$swArrRef}) {
      if ($arrElem eq $swFile ) {
        return 1; 
      }
    }
  }
  return ($matchFound);
}

=head2 Download($$$)

=begin html
This function is called to perform a SW download of a node of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>ntType</td><td>the type of NT inserted</td></tr>
>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Download($$) {
  my ($isam, $ntType) = @_;
  my $isamCliSession = undef;
  my $swParSize;
  my ($newMajor, $newMinor, $newSuffix, $newRelNum, $newFullRelNum, $activeRelNum, $fullActiveRelNum);
  my $shubSwName;
  my ($time1, $time2);
  my $activeNt;
  my $possible = 0;
  my $reason = "";
  my $ret;
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childCount = FamilyCommon::GetChildNum();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();
  my $dbVersion = $params->{DBVER_TGTREL};
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;

  if (not defined($params->{OSWP})) {
      FamilyCommon::ReportStatus( EC_OSWP_MISSING, undef );
      return 0;
  }
  if (not defined($params->{DBVER_TGTREL})) {
      FamilyCommon::ReportStatus( EC_DB_VER_UNDEFINED, undef );
      return 0;
  }
  Logger::LogTrace($nodeLogFH, "Check OSWP variant...");
  if (! Capability::OswpVariant($params->{SWPNAME}, $params->{'BOARD-FAMILY'}, 1)) {
    FamilyCommon::ReportStatus( EC_WRONG_VARIANT, "Expecting ". Capability::GetOswpVariant( $params->{'BOARD-FAMILY'}, 'ALL') );
    return 0;
  }
  #-- Open CLI Session
  Logger::LogTrace($nodeLogFH, "Open CLI...");
  $isamCliSession = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($isamCliSession)) {
      FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, ToolCommon::GetError(), undef );
      return 0;
  }
  FamilyCommon::SetCliSession2Close($isamCliSession);  # Store for IncrErrorCount

  #-- STEP 0: acquire transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");


  #-- STEP 1: NT type information and check Active OSWP is committed
  #-- Get NT type information
  Logger::LogTrace($nodeLogFH, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($isamCliSession);
  return 0 if (not defined($ntInfo));
  $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';
  my $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE}, undef,
					 $toolData->{RETRIES});
  if (! defined($oswpInfo)) {
       FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED, ToolCommon::GetError(), undef );
       return 0;
  }

  my $ltConfig = IsamCli::GetSlotConfig($isamCliSession);
  if (not defined($ltConfig)) {
      FamilyCommon::ReportStatus( EC_GET_LT_FAILED, IsamCli::GetError() );
      return 0;
  }
  my %ltHandled = ();
  my $swMissing = "";
  foreach my $ltNum (sort keys %$ltConfig) {
     next if ( defined($ltHandled{$ltConfig->{$ltNum}}) );
     next if ( FamilyCommon::BoardInSwp($nodeLogFH, uc($ltConfig->{$ltNum})));
     $swMissing .= sprintf(" %s", uc($ltConfig->{$ltNum}));
     $ltHandled{$ltConfig->{$ltNum}} = 1;
  }
  if ($swMissing ne "") {
     return undef if( FamilyCommon::ReportStatus( EC_LT_NOT_SUPPORTED, $swMissing ) );
  }

   my $new = basename($params->{SWPNAME});
   my $regExp = REGEXP;
   if ($new =~ /$regExp/) {   # -- Start 1.0.4
     $newMajor = $1;
     $newMinor = $2;
     $newSuffix = $3;
     $newFullRelNum = Capability::ConvertRelease($newMajor, $newMinor, $newSuffix, $dbVersion); # -- 1.2.0
     $newRelNum =  Capability::ConvertRelease($newMajor, $newMinor, 0, $dbVersion);
   }                          # -- End 1.0.4

  $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'},
						 $oswpInfo->{'ACTIVE'}{'SUFFIX'},
                         $oswpInfo->{'ACTIVE'}{'DBVERSION'});
  $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
					     $oswpInfo->{'ACTIVE'}{'MINOR'},
                                             0,
                         $oswpInfo->{'ACTIVE'}{'DBVERSION'});

  my $newSwpIndex = $oswpInfo->{PASSIVE}{INDEX};
  if (! $params->{FORCEDL}) {
    if ($oswpInfo->{'ACTIVE'}{'DBVERSION'} ne $params->{DBVER_TGTREL} ) {
      #-- Verify Migrate path
      Logger::LogTrace($nodeLogFH, "Migrate restrictions...\n");
      my ($restricted, $reason) = Capability::IsMigrateRestricted( $ntInfo->{$activeNt}{TYPE}, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Migrate restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    } else {
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntInfo->{$activeNt}{TYPE}, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    }
  } else {
    ToolCommon::Log($nodeLogFH, "Forced download, continue without checks...\n");
  }

  my $dbVersionActive = $oswpInfo->{'ACTIVE'}{'DBVERSION'};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveNum = substr($dbVersionActive,1);
  my $dbVersionTargetNum = substr($dbVersionTarget,1);

  if ($dbVersionActive lt "A3.200" ) {
    # From R3.1 onwards the same OSWP can be redownloaded so we have to allow it.
    Logger::LogTrace($nodeLogFH, "Checking new software already present...");
    if ( (basename($oswpInfo->{ACTIVE}{NAME}) eq basename($params->{OSWP}) ) || # -- 1.2.3
	     ( (basename($oswpInfo->{PASSIVE}{NAME}) eq basename($params->{OSWP}) ) && ($oswpInfo->{PASSIVE}{AVAIL} eq 'enabled') )
       ) { #-- 1.2.3
         FamilyCommon::AdvanceToNext("Already present... skipping\n");
         return;
    }
  }

  my $showShubConf = ($dbVersionActive eq "A1.100" ) ? "" : "config-file";
  Logger::LogTrace( $nodeLogFH, "Fetch LAnx OSWP details\n");
  my $activeConfig = IsamCli::GetLanxActiveOSWP($isamCliSession, $showShubConf);
  if (not defined($activeConfig) || ($activeConfig eq "")) {
	  FamilyCommon::ReportStatus( EC_GET_SHUB_CONF_FAILED, IsamCli::GetError() );
      return 0;
  }
  if( ($dbVersionActive eq "A2.200") and ($activeConfig ne "tftp") ) {
	  FamilyCommon::ReportStatus( EC_SHUB_NO_TFTP_BOOT, "Current is $activeConfig, Not as tftp" );
      return 0;
  }

  my $reduce = 1;
  Logger::LogTrace($nodeLogFH, "Checking Active Software is committed...");
  if ($oswpInfo->{ACTIVE}{COMMITED} ne "committed") {
    #-- STEP 2: If not committed -> commit to remove passive OSWP
    $reduce = 0 if $oswpInfo->{ACTIVE}{NAME} =~ /GPRN/;
    Logger::LogTrace($nodeLogFH, "Committing Active OSWP...");
    $ret = IsamCli::CommitSWP($isamCliSession, $oswpInfo->{ACTIVE}{INDEX});
    if (!defined($ret) || ! $ret) {
      FamilyCommon::ReportStatus( EC_COMMIT_FAIL, IsamCli::GetError(), undef );
      return 0;
    }
  }
  #-- STEP 3: Abort any active downloads
  Logger::LogTrace($nodeLogFH, "Aborting Active Downloads...");
  if (uc($oswpInfo->{PASSIVE}{NAME}) ne NO_OSWP) {
    if (!defined(IsamCli::AbortDownloadSWP($isamCliSession, $newSwpIndex))) {
      FamilyCommon::ReportStatus(EC_ABORT_DL_FAIL, IsamCli::GetError(), undef);
      return 0;
    }
    #-- no error checking as it fails if no active one
    sleep 6;
  } else {
    Logger::LogTrace($nodeLogFH, "No passive OSWP found to abort existing\n");
  }

  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
  return undef if( ToolCommon::ReleaseTransaction() );
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  #    if (((uc($ntInfo->{$activeNt}{TYPE}) eq 'ECNT-A') && ($ntInfo->{$activeNt}{ICS} < 10)) || ($ntInfo->{$activeNt}{ICS} eq "")) {
  $swParSize = ToolCommon::GetSwParSize();
  if( not defined $swParSize or $swParSize < 0 ) {
     FamilyCommon::ReportStatus( EC_SW_RED_FAILED, SNMPComm::GetError(), undef );
     return 0;
  }
  if (not $params->{NOREDUCE}) {
	 ### STEP 3b is : REDUCE Enabled or SW partition size not according to R2.2 standards..  -- 1.0.4
     if( $params->{FORCEREDUCE} || ($swParSize < MINIMALSWPARSIZE) ||
     ### Reduce current SWP if migrating from R2.4 or higher -- 1.0.2
          (($dbVersionActiveNum gt "2.300") and ($dbVersionActiveNum le "3.100")) ||
	 ### or active release is 2.3 and new release is 3.0 or higher.  -- 1.2.1
          (($dbVersionActiveNum eq "2.300") and ($dbVersionTargetNum ge "3.000"))
        ) {
         if (($swParSize < MINIMALSWPARSIZE) && ($dbVersionActiveNum gt "2.300") ) {
             FamilyCommon::ReportStatus( EC_PAR_SIZE_TOO_SMALL, "(SW partition size: $swParSize, Board not fitted to accomdate this release" );
	         return 0;
         }
         if ( (($dbVersionActiveNum ge "2.400") ||
              ( ($dbVersionActiveNum ge "2.200") and ($dbVersionTargetNum gt "3.000") ) ) &&
              ( not $oswpInfo->{ACTIVE}{REDUCED}) ) {
	             Logger::LogDebug($nodeLogFH, sprintf("swParsize: '%s', minimum: '%s'\n", $swParSize, MINIMALSWPARSIZE));
	             if (($swParSize <= MINIMALSWPARSIZE) || ($params->{FORCEREDUCE})) {
	               # No need to reduce if SW Partition size is bigger then the boundary for ECNT-A ICS10 or > (e.g. ECNT-C with 512 Mb SAN)
	                 # Exception: when forced reduction is requested we'll do it in any case          -- 1.5.1.2
                     if (($dbVersionActiveNum le "3.100") || ($params->{FORCEREDUCE}) ) {
                          # Exception: when forced reduction is requested we'll do it in any case          -- 1.5.1.2
                          # Check for waiting-for-sw in case LT software not present
                          my $ltNoSWFound = ToolCommon::CheckLTSWPresence($isamCliSession);
                          if ( %$ltNoSWFound ) {
                             my @ltTmpArr;
                             foreach my $lt (keys %{$ltNoSWFound}) {
                                 Logger::LogTrace($nodeLogFH, "No software found for ISAM board : $ltNoSWFound->{$lt}{TYPE}!!\n");
                                 push @ltTmpArr, $ltNoSWFound->{$lt}{TYPE};
                             }
                             FamilyCommon::ReportStatus( EC_NO_LT_SWP_FOUND, "Undef /SW ".join(", ", @ltTmpArr) );
	                         return 0;
                          }
	                      $ret = Octopus::ReduceActiveSWP($isam, $snmpSession, $childCount, $dir,
                                                          Capability::ReductionChar( $params->{'BOARD-FAMILY'}), $dirName, $isamCliSession) if $reduce;
	                      if (defined($ret) && !$ret) {
                            FamilyCommon::ReportStatus( EC_SW_RED_FAILED, Octopus::GetError() );
	                        return 0;
	                      }
	                      sleep(20);  # wait for the OSWP to stabilize.  We want some info back on the new  GetOswpInfo.
                      }
	               #-- STEP 3bis': Get passive SW index again (has been changed while reducing the SWP)
	               $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE},
	             				      undef, $toolData->{RETRIES});
	               if (! defined($oswpInfo)) {
                     FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, "After SW Reduction");
	                 return 0;
	               }
                   $newSwpIndex = defined($oswpInfo->{PASSIVE}{INDEX}) ? $oswpInfo->{PASSIVE}{INDEX} : undef;
	             }
        }
    }
    #-- END STEP 3bis -- 1.0.2
  }

  if (not defined($newSwpIndex)) {
      FamilyCommon::ReportStatus( EC_NO_PASS_SWPIDX_FOUND, undef);
      return 0;
  }
  Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
  return undef if( ToolCommon::IsTransferOnGoing() );

  # Again take the transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");


  Logger::LogTrace($nodeLogFH, "Configure Download Parameters...");
  #-- STEP 4: Configure Download Parameters
  $ret = IsamCli::ConfigureSWPDownload($isamCliSession, $newSwpIndex,
				       $params->{PRIMTFTPIP},
				       'complete-set');
  if (not defined($ret) || not $ret) {
      FamilyCommon::ReportStatus( EC_CONFIG_DL_FAILED, IsamCli::GetError() );
      return 0;
  }

  #-- STEP 5: Download SWP
  # -- 1.0.4 START
  # Lanx SW has to be downloaded separately if the active SW is R2.0 or R2.1:
  # once to the passive conf
  # once to the NT
  $shubSwName = Capability::FindShubSwName($params->{SWPNAME}, REGEXP, $toolData->{TFTPDIR}, $toolData->{SHUBSWPATH}, $toolData->{SHUBSWNAME},
                       $params->{'BOARD-FAMILY'} );
  if ( ($shubSwName eq "") and (uc($ntInfo->{NTA}{TYPE}) eq 'ECNT-A')  and ($dbVersionActiveNum le "2.200") ) {
    # Direct upgrade from R2.0/R2.1 to R2.2 or higher (<= R2.4) only required for ECNT-A.
      FamilyCommon::ReportStatus( EC_NO_SHUB_SW, "(OSWP:  $params->{SWPNAME}, SWP: $params->{OSWP})" );
      return 0;
  }
  #    }
  if ( ($dbVersionTargetNum le "2.200") and ($dbVersionActiveNum le "2.200") ) {
    # New release is 2.2 or lower or Active release is R2.2 or lower: download SHUB SW to SHUB
    my $newConfig;
    if ($activeConfig eq "config-1") {
        $newConfig = "config2";
    } elsif ($activeConfig eq "config-2") {
        $newConfig = "config1";
    } else {
        # config is already set to tftp (system is running R2.2
        $newConfig = $activeConfig;
    }
    if (($newConfig ne "tftp") || ($params->{DLSHUB})) {
      # SW retrieved from NT SW so no need to transfer to SHUB
      Logger::LogTrace($nodeLogFH, sprintf("ISAM running '%s - %s', Update SHUB SW...", $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{DBVERSION}));
      $time1 = time();
      if (!IsamCli::DownloadShubSW($isamCliSession, $shubSwName, $params->{PRIMTFTPIP}, $showShubConf)) {
          FamilyCommon::ReportStatus( EC_SHUB_SW_DL_FAILED, IsamCli::GetError() );
	      return 0;
      } else {
	      $time2 = time();
	      Logger::LogTrace($nodeLogFH, sprintf("  OK, downloaded in %d seconds.\n", $time2 - $time1));
	      sleep 10;
      }
    }
  }
  # Also download to NT using TFTP
  # To be certain: remove any Lanx.Z present /Sw left there from a possible previous DL
  # It will make this tftp to fail if a Lanx.Z is present in /Sw
  # Only download new SHUB SW if new SW is R2.2
  if($dbVersionTargetNum eq "2.200") {
       if ($dbVersionActiveNum le "2.200") {
         if (! Octopus::RemoveNtShubSw($params->{DEBUG}, $dir, $isam, $dirName)) {  # Also enables write on /Sw for Downgrade on R2.3 or R2.4
             FamilyCommon::ReportStatus( EC_DEL_SHUB_SW_FAILED, Octopus::GetError() );
	         return 0;
         }
       } else {
         if (! Octopus::EnableSwWrite($params->{DEBUG}, $dir, $isam, $dirName)) {
             FamilyCommon::ReportStatus( EC_ENABLE_WR_FAILED, Octopus::GetError() );
	         return 0;
         }
       }
       # TFTP errors: errno values are returned
       my $tftpSession = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});
       if (not defined($tftpSession)) {
           FamilyCommon::ReportStatus( EC_TFTP_OPEN_FAILED, undef);
           return 0;
       }
       $tftpSession->binary();
       my $sourceFile = $toolData->{TFTPDIR} . "/" . $shubSwName;
       my $destinationFile =  ($dbVersionActiveNum ge "2.200") ? "/Sw/Lanx.Z" : "/pub/Sw/Download/Lanx.Z";
       Logger::LogDebug($nodeLogFH, sprintf("tftp put %s %s\n", $sourceFile, $destinationFile));
       $time1 = time();
       if ($tftpSession->put($sourceFile, $destinationFile) ne "") {
         FamilyCommon::ReportStatus( EC_TFTP_PUT_FAILED, "to put $destinationFile" . Octopus::GetError() );
	     return 0;
       } else {
	      $time2 = time();
	      Logger::LogTrace($nodeLogFH, sprintf("  OK, transfered %d bytes in %d seconds\n", $tftpSession->bytesTransfered(), $time2 - $time1));
	      sleep 30;  # Make sure SWP download goes well...
       }
       if ( ($dbVersionActiveNum ge "2.200") && ((uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") &&  (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY"))){
	       # Sync to passive if redundant
	       if (Octopus::SyncLanxSw($params->{DEBUG}, $dir, $isam, $dirName) < 0) {
                FamilyCommon::ReportStatus( EC_SYNC_TO_STBY_FAILED, Octopus::GetError() );
	            return 0;
	       } else {
	            sleep 30; # Wait for Lanx.Z to be synced
	       }
       }
  }
  # -- 1.0.4 END

  $time1 = time();
  Logger::LogTrace($nodeLogFH, "Downloading OSWP-$newSwpIndex ...");
  $ret = IsamCli::DownloadSWP($isamCliSession, $newSwpIndex, $params->{OSWP});
  if ( !defined($ret) || !$ret) {
      FamilyCommon::ReportStatus( EC_DL_SWP_FAILED, IsamCli::GetError() );
      return 0;
  } else {
      $time2 = time();
      Logger::LogTrace($nodeLogFH, sprintf("  OK, transfered in %d seconds\n", $time2 - $time1));
  }
  $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, $ntInfo->{$activeNt}{TYPE}, undef, $toolData->{RETRIES});
  if (not defined($oswpInfo)) {
      FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, "after SWP download ".ToolCommon::GetError() );
      return 0;
  }

  # Check for any files marked as not-available.  Workaround for ANTmt25573.  Using the MINIMUMSET option it is expected that SW for
  # boards not yet known to the NT are downloaded too.  It is not the case.  So we'll tftp the files that are not available to the
  # /pub/Sw directory.
  # We will only do this for NTs running the intelligent download and after verifying that there is still enough room on the disk.
  if ( ($dbVersionActiveNum ge "3.100") ) {
        Logger::LogTrace($nodeLogFH, "Check for files marked as not-available but included in the minimum-set...");
        my $filesMissing = 0;
        my $boardsInMinSet;
        my $swList = IsamCli::ListSwpFiles($isamCliSession, basename($params->{SWPNAME}));
        if (not defined($swList)) {
            FamilyCommon::ReportStatus( EC_LIST_SWFILES_FAILED, IsamCli::GetError() );
            return 0;
        }
        $boardsInMinSet = GetMinimumSet($nodeLogFH);
        if (not defined($boardsInMinSet)) {
             FamilyCommon::ReportStatus( EC_PARSE_SWP_FAILED, FamilyCommon::GetError() );
             return 0;
        }
	    foreach my $package (keys %{$swList}) {
	        foreach my $fileName (keys %{$swList->{$package}}) {
	            Logger::LogDebug($nodeLogFH, sprintf("package: %s, file: %s\n", $package, $fileName));
	            if ($swList->{$package}{$fileName}{STATE} eq "not-available") {
	              if (defined($boardsInMinSet->{$swList->{$package}{$fileName}{BOARD}})) {
	    	           Logger::LogDebug($nodeLogFH, "Including $fileName in to-be-downloaded list\n");
	    	           $filesMissing = 1;
	              } else {
	    	           # Remove files from the list that are not included in the minimum set.
	    	           Logger::LogDebug($nodeLogFH, "Removing $fileName from SW list\n");
	    	           delete $swList->{$package}{$fileName};
	              }
	            }
	        }
	    }
	    if ($filesMissing) {
	      # Got at least one, so download using tftp
	      if (! DownloadMissingFiles($nodeLogFH, $isamCliSession, $swList, $isam)) {
             FamilyCommon::ReportStatus( EC_MISSING_FAILED, undef );
	         return 0;
	      } else {
	         Logger::LogTrace($nodeLogFH, "  OK files sent\n");
	      }
	    } else {
	      Logger::LogTrace($nodeLogFH, "  Ok, no files to be sent\n");
	    }
  }

  sleep(20); 
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
  return undef if( ToolCommon::ReleaseTransaction() );
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  IsamCli::AllowAlarms($isamCliSession);
  FamilyCommon::CloseCliSession2Close();
  return 1;
}

1;
