package FamilyCommon;

=head1 PBMT family-common routines

=begin html
This PERL module provides routines common to the PBMT 'ISAM' family.  This package has been created
to facilitate ISAM-NE specific scenarios for migration, upgrade, download, commit,....

=end html

=cut

#==============================================================================
#-- File:         FamilyCommon.pm
#-- Date:         27.08.2008
#-- Company:      Alcatel Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Routines common to the PBMT NE family
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0  -  2008/08/27  : Start First implementation
#--   1.0.1  -  2009/03/09  : Read asamSystemInitProgress in WaitForRestart.  R3.7
#--                           allows snmpget on sysuptime sooner but returns no write
#--                           access on the transactionowner OID.
#--   1.0.2  -  2009/10/09  : SNMPv3 returns empty strings in case of problems.  Convert
#--                           the expected number into a string in WaitForRestart and change 
#-                            the value to 0 if we get an empty string in stead of a number.
#--   3.0.00 - 2010/04/21   : Converted version to 3.0.00 for converged PBMT
#--   3.0.01 - 2010/05/24   : Converged PBMT Changes
#--                           Added x86 changes
#--   3.0.02 - 2011/01/11   : LRM support added
#--   3.0.03 - 2011/03/29   : ALU01031457 - Support for R4220 & R4230
#--   3.0.04 - 2012/05/11   : ALU01789702 - World writeable directories need sticky bit set 
#--   3.0.05 - 2012/06/11   : Change permissions for backup db file
#--   3.0.06 - 2012/08/02   : to eliminate warnings related to use of getlogin call
#--   3.0.07 - 2012/11/28   : Impact due to RCR ALU00123215 - Upgrade on StandBy NT
#--   3.0.08 - 2013/04/05   : SNMP retry mechanism and error handling altered in ActivateSwp 
#--   3.0.09 - 2013/05/31   : ALU01881027
#--   3.0.10 - 2013/06/18   : oid not properly updated in WaitForUpgradeViaStdbyNT
#--   3.0.11 - 2014/08/20   : subroutine WaitForNePingAble added for rollback 
#--   3.0.12 - 2014/10/10   : ALU02016489
#--   3.0.13 - 2016/03/05   : ALU02188504
#--   3.0.14 - 2016/04/04   : ALU02197279
#--   3.0.15 - 2016/07/15   : ALU02237966
#--   3.0.16 - 2017/07/17   : ALU02373989
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697



=head1 VERSION

=begin html
Current version: "FamilyCommon.pm - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "FamilyCommon.pm - 3.1.0 -- 2018/06/20";
#==============================================================================

use strict;
use File::Basename;
use User::pwent;

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use Cwd qw( realpath getcwd);
require Utils;
use PBMTGlobal;
use Logger qw(:ALL);
use ToolCommon qw(:CONSTANTS);
require IsamCli;
require IsamTL1;
require perlTL1::SshChannel;
use ErrorCodes qw(:ALL);
use SNMPComm;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub DetermineSwMgntSize($$$$$$$$$);
sub CreateTmpDirFile($$$$$$$);
sub CreateBUFile($$$$$$$);
sub WaitForUpgradeViaStdbyNT($$$$);
sub WaitForNePingAble($$);
sub WaitForRestart($$$$);
sub CheckActiveSoftwareIsNew($$$$$$$);
sub BoardInSwp($$);
sub PrepareLogging($$$$);
sub SetAndLogError($$;$);
sub GetError();
sub ClearError();
sub IncrErrorCount($$$$);
sub ToolExit($);
sub AdvanceToNext($);
sub SetOwnPid($);
sub GetOwnPid();
sub ClearOwnPid();
sub SetWarnLogFile($);
sub GetWarnLogFile();
sub CloseWarnLogFile();
sub SetParentLogFile($);
sub GetParentLogFile();
sub SetChildLogFile($);
sub GetChildLogFile();
sub CloseChildLogFile();
sub SetNodeStatusLog($);
sub GetNodeStatusLog();
sub CloseNodeStatusLog();
sub CreateNodeStatusLog($$);
sub UpdateNodeStatusLog($;$);
sub ClearCliSession2Close();
sub SetCliSession2Close($);
sub CloseCliSession2Close();
sub GetCliSession2Close();
sub ClearTL1Session2Close();
sub SetTL1Session2Close($);
sub CloseTL1Session2Close();
sub GetTL1Session2Close();
sub ClearSshTL1Session2Close();
sub SetSshTL1Session2Close($);
sub CloseSshTL1Session2Close();
sub GetSshTL1Session2Close();
sub ClearSnmpSession2Close();
sub SetSnmpSession2Close($);
sub CloseSnmpSession2Close();
sub GetSnmpSession2Close();
sub ClearFailedNode();
sub SetFailedNode($);
sub NodeIsFailed();
sub GetFailedNode();
sub GetErrorCount();
sub GetFailedNodeList();
sub GetWarningCount();
sub GetWarnedNodeList();
sub SetChildNum($);
sub GetChildNum();
sub SetToolDirName($);
sub GetToolDirName();
sub SetSigPipeReceived();
sub ClearSigPipeReceived();
sub GetSigPipeReceived();
sub SetOldTransactionTO($);
sub GetOldTransactionTO();
sub SetConfigData($);
sub GetConfigData();
sub SetWarningLevels($);
sub GetWarningLevels();
sub SetToolData($);
sub GetToolData();
sub SetNeStorage($);
sub GetNeStorage();
sub IncrWarnCount($$$);
sub SetFailontlistFile($);
sub GetFailontListFile();

#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars
#==============================================================================
my $config;
my $warnLevels = undef;
my $toolData = undef;
my $neStorage = undef;
my $ERROR = "";
my $overAllWarnCode = 0;
my $failedontlistfile;
my $oswpStatus = 0;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}


sub SetOSWPStatus($) {
   my( $oswpStatusValue ) = @_;
   $oswpStatus = $oswpStatusValue;
}

sub GetOSWPStatus() {
   return $oswpStatus;
}

=head2 DetermineSwMgntSize($$$$$$$$$)

=begin html
Determine the size of the SwMgnt.ini file.  Migrations from R2.2 to a higher release is at risk when this file is too big ( &gt; 1024).  This
problem was solved in the NT SW in R2.3.  In order to secure the migration from R2.2 we retrieve this file from the ISAM using tftp and check the
size.  In case the board was flashed with R2.3 an alternative filesize is used because one more variable has been defined, not exiting in R2.2<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>criticalSize</td><td>maximum filesize for an NT flashed with SW &lt; R2.3</td></tr>
  <tr><td>altSize</td><td>maximum filesize for an NT flashed with SW &gt; R2.2</td></tr>
  <tr><td>otherOswp</td><td>name of the passive OSWP</td></tr>
  <tr><td>oswp</td><td>name of the active OSWP</td></tr>
  <tr><td>path</td><td>PBMT tool installation path</td></tr>
</table>

=end html

=cut

sub DetermineSwMgntSize($$$$$$$$$) {
  my ($fh, $debug, $dir, $isam, $criticalSize, $altSize, $otherOswp, $oswp, $path) = @_;
  my ($time1, $time2);
  my $newSize = 0;
  my $continue = 1;
  my $ok = 0;
  my $useAltSize = 0;

  ToolCommon::Log($fh, "Enable read in /Dbase...");
  if (! Octopus::EnableDbaseRead($debug, $dir, $isam, $path)) {
    FamilyCommon::SetAndLogEror($fh, sprintf("  FAILED: %s\n", Octopus::GetError()), sprintf("Failed: %s", Octopus::GetError()));
  } else {
    ToolCommon::Log($fh, "  OK\n");
    ToolCommon::Log($fh, "Retrieve SwMgnt.ini...");
    my $tftp = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});  # -- 1.1.6
    if (defined($tftp)) {
      $time1 = time();
      if ($tftp->get("/Dbase/SwMgnt.ini", "$dir/SwMgnt.ini") ne "") {
	my $tftpErr = $tftp->error();
	FamilyCommon::SetAndLogError($fh, "tftp get /Dbase/SwMgnt.ini failed: $tftpErr", "  FAILED tftp get /Dbase/SwMgnt.ini failed: $tftpErr");
      } else {
	$time2 = time();
	my $bytes = $tftp->bytesTransfered();
	if ($bytes > 0) {
	  ToolCommon::Log($fh, sprintf("  OK transfered %d bytes in %d seconds\n", $bytes, $time2 - $time1));
	  ToolCommon::Log($fh, "  Filename = $dir/SwMgnt.ini\n") if $debug;
	  ToolCommon::Log($fh, "Determine SwMgnt.ini file size...");
	  my $swIni;
	  if (!open($swIni, "$dir/SwMgnt.ini")) {
	    FamilyCommon::SetAndLogError($fh, "Failed to open $dir/SwMgnt.ini: $!", "  FAILED: could not open $dir/SwMgnt.ini: $!\n");
	  } else {
	    my $line;
	    while (defined($line = <$swIni>) && $continue) {
	      chomp $line;
	      $line =~ s/\r//g;
	      ToolCommon::Log($fh, "Line read: $line\n") if $debug;
	      if ($line =~ /ProtGroupLocked/) {
		$useAltSize = 1;
	      } elsif ($line =~ /\[Oswp$otherOswp\]/) {
		$continue = 0;
		my $empty = <$swIni>;
		my $name = <$swIni>;
		if (defined($empty) && defined($name)) {
		  chomp($empty), $empty =~ s/\r//g;
		  chomp($name), $name =~ s/\r//g;
		  ToolCommon::Log($fh, "'$empty', '$name', new oswp: '$oswp'\n") if ($debug);
		  if ($empty =~ /OSWP_EMPTY/) {
		    # New length:
		    # 2 times new OSWP: stored in Name and path.
		    # 4 bytes extra: OSWP_EMPTY -> OSWP_NOT_EMPTY
		    # subtract size of basename($oswp) since path only contains the path.
		    $newSize = $bytes + (length($oswp)*2) + 4 - length(basename($oswp));
		    ToolCommon::Log($fh, "Changing SwMgnt.ini (empty OSWP) file size from $bytes to $newSize bytes\n") if $debug;
		  } else {
		    if ($name =~ /Name\s=\s(.*)/) {
		      my $oldName = $1;
		      my $oldPathLength = length($oldName) - length(basename($oldName));
		      ToolCommon::Log($fh, sprintf("Length stored name: %d, stored path: %d\n", length($oldName), $oldPathLength)) if $debug;
		      $newSize = $bytes - length($oldName) - $oldPathLength + (length($oswp) * 2) - length(basename($oswp));
		      ToolCommon::Log($fh, "Changing SwMgnt.ini file size from $bytes to $newSize bytes\n") if $debug;
		    } else {
		      FamilyCommon::SetAndLogError($fh, "Failed to find 'Name = ' clause", "Did not find 'Name = ' clause\n");
		    }
		  }
		} else {
		  my $part = defined($empty) ? "Name" : "Empty";
		  FamilyCommon::SetAndLogError($fh, "OSWP definition not complete, missing $part",
					       "OSWP defintion not complete, missing $part\n");
		}
	      }
	    }
	    close($swIni);
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, "Empty file $dir/SwMgnt.ini retrieved");
	}
      }
    } else {
      FamilyCommon::SetAndLogError("Failed to open tftp session", "  FAILED - could not open tftp session\n");
    }
  }
  if ($newSize > 0) {
    my $chkSize = $useAltSize ? $altSize : $criticalSize;
    if ($newSize > $chkSize) {
      FamilyCommon::SetAndLogError($fh, "Size could get too high (new $newSize vs $chkSize)",
				   "  FAILED: size could get too high (new $newSize vs $chkSize)\n");
    }
  }
  return ($ok);
}

=head2 CreateTmpDirFile($$$$$$$$)

=begin html
Create working directory and empty backup file.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>parentFh</td><td>the file handle of the parent script.  When fatal errors are occuring logging to the main log file will also be performed</td></tr>
  <tr><td>childFh</td><td>the file handle of the parent script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>dir</td><td>the directory name where the empty backup files have to be created</td></tr>
  <tr><td>userName</td><td>Unix user name in case of multi-user operation (command line option) empty otherwise</td></tr>
  <tr><td>logFileName</td><td>the name of the log file associated to this child process</td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
  <tr><td>params</td><td>pointer to command parameter hash</td></tr>
  <tr><td>suppressTrace</td><td>Suppress tracing (1) or not (0)</td></tr>
</table>

=end html

=cut

sub CreateTmpDirFile($$$$$$$) {
  my ($parentFh, $childFh, $dir, $userName, $logfileName, $count, $suppressTrace) = @_;

  ToolCommon::CreateDirectory( $parentFh, $dir );
  if (! -w $dir) {
    SetAndLogError($childFh,
  	     sprintf("Working dir '%s' can't be written by user %s",
  		     $dir,
                           getpwuid($<)));
    ToolCommon::Log($parentFh, sprintf("Child %s: **FATAL** working dir '%s' can't be written by user %s\n", $count, $dir, getpwuid($<)), 1);
    return 0;

  }
  if ($params->{MULTI}) {
      $dir .= "/$userName";
      ToolCommon::CreateDirectory( $parentFh, $dir );
	  if (! -w $dir) {
          SetAndLogError($childFh, sprintf("Working dir '%s' can't be written by user '%s'\n", $dir, getpwuid($<)));
	      ToolCommon::Log($parentFh, sprintf("Child %s: **FATAL** working dir '%s' can't be written by user %s\n", $count, $dir, getpwuid($<)), 1);
          return 0;
	  }
    }
    if ($params->{WIPE}) {
      if (!system("$config->{RM} -Rf $dir/* > $dir/FamComWipe.txt 2>&1")) {
	     ToolCommon::Log($childFh, "$config->{RM} -Rf $dir/* failed: $!");
      }
    }
  return 1;
}

=head2 CreateBUFile($$$$$$$)

=begin html
Create empty backup files.  The TFTP deamon on AWS expects the file to be present which means that the script needs to create an empty database
file before it can be retrieved.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>parentFh</td><td>the file handle of the parent script.  When fatal errors are occuring logging to the main log file will also be performed</td></tr>
  <tr><td>childFh</td><td>the file handle of the parent script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>dir</td><td>the directory name where the empty backup files have to be created</td></tr>
  <tr><td>fileName</td><td>the name of the empty file to be created</td></tr>
  <tr><td>logFileName</td><td>the name of the log file associated to this child process</td></tr>
  <tr><td>count</td><td>the child number (there may be multiple child processes active)</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled or not</td></tr>
</table>

=end html

=cut

sub CreateBUFile($$$$$$$) {
  my ($parentFh, $childFh, $dir, $fileName, $logfileName, $count, $debug) = @_;
  my $ok = 1;
  my $cmd = "$config->{RM} $dir/$fileName >$neStorage/MIGRbuFile.txt 2>&1;$config->{TOUCH} $dir/$fileName >$neStorage/MIGRtouch.txt 2>&1";
  my $sysret = system($cmd);

  ToolCommon::Log($childFh, "CMD = $cmd\n") if ($debug);
  if ($sysret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($sysret);
    FamilyCommon::SetAndLogError($childFh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** $cmd FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  } else {
    ToolCommon::Log($childFh, "Touching $dir/$fileName OK\n") if $debug;
  }
  if ( $params->{UMASK_CHMOD} and ! chmod(0666, "$dir/$fileName")) {
    FamilyCommon::SetAndLogError($childFh, "chmod(666,$dir/$fileName) FAILED: $!", "  FAILED: $!\n");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** chmod(666,$dir/$fileName) FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  }
  return $ok;
}

sub WaitForUpgradeViaStdbyNT($$$$)
{
  my ($fh, $session, $asam, $oswpInfo) = @_;
  my ($time1, $timeDiff, $sleepTime, $startTime, $endTime);
  my $ok = 0; 
  $sleepTime = 15; 

  my $passiveIdx = $oswpInfo->{'PASSIVE'}{'INDEX'}; 
  my $restart = 0; 
  my $actstdby = 0; 
  my $ACTIVEOSWP_5X_OID = ".1.3.6.1.4.1.637.61.1.24.1.1.1.4";
  my $oid = "$ACTIVEOSWP_5X_OID\.$passiveIdx"; 
  $startTime = time(); 

  $session->retries(1); 

  ToolCommon::Log($fh, sprintf("Waiting for upgrade via Standby NT ...\n"));
      my ($error,$error_status) = ("",0);
      my ($snmp_err_count, $oswp_active_count) = (0,0);
  #-- check for upgrade via standby NT
      while(!$restart) {
        if (defined($session)) {
           my $oswpActStatus = ToolCommon::DoSafeGet($session, $oid, $toolData->{RETRIES});
           #----
           ### DEVEL-NOTE : session->error() return always null when message as noSuchObject.
           ###              through session->error_status(), we detect error through non zero value
           $error = $session->error();
	   $error_status = $session->error_status();
           $error = $session->error();
	   $error_status = $session->error_status();
	   if (($error =~ /noSuchName|noSuchObject|Timeout|No response/) || ($error =~ /genError/) || ($error_status != 0 )) {
	          sleep 15;
                  $snmp_err_count++;
                  $restart = 1 if( $snmp_err_count == 200 );
	   } else {
		  if(defined($oswpActStatus) && ($oswpActStatus == 4)) {
		      sleep 15; 
		      $actstdby = 1;
                      $oswp_active_count++;
                      $restart = 1 if( $oswp_active_count == 200 );
		  } else {
		      $restart = 1; 
                      $ok = 1; 
		      $endTime = time(); 
		      ToolCommon::Log($fh, sprintf("Activate via Standby NT OK (after %d seconds)\n", $endTime - $startTime)) if ($actstdby);
		  }
          }
        } 
      }
  $session->retries(3); 
  return $ok; 
}



=head2 WaitForRestart($$$$$)

=begin html
Wait for NT restart by starting a ping loop.  When the node is alive an SNMP-get loop is started to await the availability of the management
interface (this routine is also called after an SHUB reboot but without the snmp ping loop).<br>
We try to stay as close as possible to the MAXWAITTIME by obtaining the time from the wallclock when performing a sleep between two ping/snmp
get attempts.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>session</td><td>an opened SNMP session</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>asam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>shub</td><td>Restarting SHUB (defined) or NT (not defined)</td></tr>
</table>

=end html

=cut

sub WaitForRestart($$$$) {
  my ($fh, $session, $asam, $shub) = @_;
  my ($retryWaitTime, $maxWaitTime, $startTime, $endTime) = ( 60, $toolData->{MAXWAITTIME}, 0, 0 );
  Logger::LogTrace($nodeLogFH, sprintf("Waiting for Restart : ", defined($shub) ? "SHUB" : "NT"));
  $startTime = time();
  my $sleepTime   = 10;
  if( defined $params->{BTYPE} and $params->{BTYPE} ne "SNMP" ) {
        $sleepTime = $toolData->{MAXWAITTIME} - 30;
        $maxWaitTime = 30;
  }
  Logger:LogTrace($nodeLogFH, "Sleeping for $sleepTime seconds to check for status\n");
  sleep($sleepTime);
  my $retryTotalTime = $sleepTime;
  my $systemInitValue = 0;
  while ( $retryTotalTime < $maxWaitTime) {
      if( not ToolCommon::Ping($config->{PING}, $asam->[0]) ) {  # -- 1.1.3 -- 1.1.6
          sleep( $retryWaitTime );
          $retryTotalTime += $retryWaitTime;
          next;
       }
       $systemInitValue = SNMPComm::GetValue('SYSTEM INIT', SYSTEMINITPROGRESSOID, 2  );
	   if (defined($systemInitValue)) {
	     my $valString = sprintf("%s", $systemInitValue);
	     if (not ($valString =~ /\d+/)) {
	       $systemInitValue = 0;
	     }
	     last if ($systemInitValue == 100);
       }
       sleep( $retryWaitTime );
       $retryTotalTime += $retryWaitTime;
  }
  $endTime = time();
  if( defined $systemInitValue and $systemInitValue == 100 ) {
      Logger::LogNotify( $nodeLogFH, sprintf("ISAM Brought up successfully (after %d seconds)\n", ($endTime-$startTime)) );
      return 1;
  } else {
      Logger::LogError( $nodeLogFH, sprintf("ISAM not brought up and timed out (after %d seconds )\n", ($endTime-$startTime)) );
      return 0;
  }
}


=head2 CheckActiveSoftwareIsNew($$$$$$$$)

=begin html
Check whether the new SW is also the active SW.  Returns 3 values:<br>
<ul>
  <li>the first value: 0 in case of error (new SW not active is also error), 1 in case of success 2 in case of special condition (check SHUB).</li>
  <li>the second value indicates whether the show sowftware-mngt returned an empty list (1) or not (0).  A value of -1 means no information could
      be retrieved (e.g. node is down)</li>
  <li>the latest OSWP information</li>
</ul>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>snmpSession</td><td>an opened SNMP session</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>swp</td><td>SW packange name</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>ntType</td><td>NT HW type</td></tr>
</table>

=end html

=cut

sub CheckActiveSoftwareIsNew($$$$$$$) {
  my ($newfh, $cli, $snmpSession, $swp, $dir, $isam, $ntType) = @_;
  my $oswpInfoEmpty = -1;
  my $oswpInfo;
  my $ok = 1;

  $ERROR = "";
  ToolCommon::Log($newfh, "Checking Active Software == New Software...");
  $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntType), \$oswpInfoEmpty);
  if (!defined($oswpInfo)) {
    my $t = ToolCommon::GetError();
    SetAndLogError($newfh, $t, "  $t\n");
    $ok = 0;
  } else {
    if (basename($oswpInfo->{'ACTIVE'}{'NAME'}) ne basename($swp)) {
      if (($oswpInfoEmpty > 0) && ($oswpInfo->{FRMW_OSWP} == $oswpInfo->{ACT_PENDING})) {
	ToolCommon::Log($newfh, "  OK (special)\n");
	$ok = 2;
      } else {
	my $t = "Failed (Active is '$oswpInfo->{ACTIVE}{NAME}')";
	SetAndLogError($newfh, $t, "  $t\n");
	$ok = 0;
      }
    } else {
      ToolCommon::Log($newfh, "  OK\n");
    }
  }
  return ($ok, $oswpInfoEmpty, $oswpInfo);
}

=head2 BoardInSwp($$$)

=begin html
This function determines whether the specified board is found in the SWP file.  1 is returned in case the board is foudn, 0 if not found or in case
of error.

=end html

=cut

sub BoardInSwp($$) {
  my ($fh, $boardName) = @_;
  my $hdl;
  my $line;
  my $ok = 0;
  my $swpFilePath = $params->{NOTFTP} ? $params->{SWPNAME} : "$toolData->{TFTPDIR}/$params->{SWPNAME}";

  $boardName = uc($boardName);
  $swpFilePath =~ s/\/\//\//g;
  if ( not -f $swpFilePath ) {
      $ERROR = "SWP file '$swpFilePath' not exist/not a file to find Board '$boardName'";
      return $ok;
  }
  my $boardContentCmd = "$config->{SED} -ne '/^ *DESCRIPTION *$boardName/,/^ *END/p' $swpFilePath";
  if ( system("$boardContentCmd | $config->{GREP} $boardName > /dev/null 2>&1") ){
       Logger::LogError( $nodeLogFH, "CMD Failed '$boardContentCmd |$config->{GREP} $boardName > /dev/null 2&1'", undef, $Logger::LogOnlyHandle );
       $ERROR = "Unable to found Board '$boardName' from SWP file '$swpFilePath'";
       return $ok;
  } else {
      my $boardContent = `$boardContentCmd`;
      if( $? != 0 ) {
          $ERROR = "Unable to fetch Board '$boardName' conetnt from SWP file '$swpFilePath'";
          return $ok;
      }
      my @boardContents = split( /\n/, $boardContent );
      foreach ( @boardContents ) {
          Logger::LogNotify( $nodeLogFH, "Board Content : $_\n");
      }
      $ok = 1;
      return $ok;
  }
}

=head2 PrepareLogging($$$$$)

=begin html
This function determines where logfiles and files uploaded from the NE are to be stored.  This setting depends on the filetransfer protocol selected
in the ISAM:
<ul>
  <li>In case of tftp: all files transfered between ISAM and tftp server are based from the TFTP root directory (installation dependant)</li>
  <li>In case of ftp/sftp: all files transfered between ISAM and (s)ftp server are relative from the home directory of the unix user</li>
</ul>
In case the multi-user option has been selected (--multi), the user name is reflected in the path too in the tftp directory setting.<br>
The function returns 3 parameters:
<ul>
  <li>dir: a absolute path where the logging has to be stored (excluding the 'username' setting as the directory may still have to be created.
      The program calling this function still has to take the unix username into account if further using this setting as location offset (i.e. it
      has to append the unix username to this string</li>
  <li>tftpDir: the relative path of the above (i.e. omitting either the unix user home directory part or the tftp root directory part</li>
  <li>fxrData: a structure holding the filetransfer settings as retrieved from the ISAM as well as home-directory settings for NT and xVSP fxr.
</ul>
The calling function has to append the 'username' setting to both dir and relativeDir in case the multi-user opion was selected.  The reason for this
is that this fuction is not creating the directories (needs to be done by the caller).<br>
In case of failure (undef, undef, undef) is returned to the caller.

=end html

=cut

sub PrepareLogging($$$$) {
  my ($session, $newfh, $userName, $isamID) = @_;
  my $fxrData;
  my $dir;
  my $relativeDir;
  
  ToolCommon::Log($newfh, sprintf("Check filetransfer settings (user %s)\n", defined($userName) ? $userName : "not defined")) if $params->{DEBUG};
  $fxrData = ToolCommon::GetFxrSettings($session, $toolData->{RETRIES});
  if (!defined($fxrData)) {
    my $t = sprintf("FAILED to determine file transfer protocol: %s", ToolCommon::GetError());
    SetAndLogError($newfh, $t, "  $t\n");
    return (undef, undef, undef);
  }
  # in case the ISAM is not using tftp as filetransfer protocol
  ToolCommon::Log($newfh, sprintf("Node using %s as filetransfer protocol\n",
				  FXRPROTOCOL_NAME($fxrData->{PROTOCOL}))) if $params->{DEBUG};
  if ((($fxrData->{PROTOCOL} == TFTP_FXR) && $params->{NOTFTP}) ||
      (($fxrData->{PROTOCOL} != TFTP_FXR) && ! $params->{NOTFTP})) {
    my $t = sprintf("ERROR: assumed filetransfer protocol (%s) not compatible with protocol configured in the node (%s)",
		    $params->{NOTFTP} ? "ftp/sftp" : "tftp", FXRPROTOCOL_NAME($fxrData->{PROTOCOL}));
    SetAndLogError($newfh, $t, "  $t\n");
    return (undef, undef, undef);
  }
  if ($fxrData->{PROTOCOL} != TFTP_FXR) {
    # we're not using tftp, get the unix user even if --multi was not provided.
    my $localUser = getpwuid($<);
    ToolCommon::Log($newfh, sprintf("Check if server is fileserver (user %s)\n", $localUser)) if $params->{DEBUG};
    my $fxrServer = ToolCommon::FxrServerIsThisNode($fxrData, $config);
    if (defined($fxrServer)) {
      if ($fxrServer eq "") {
	if ($fxrData->{PROTOCOL} == FTP_FXR) {
	  my $t = "ERROR: this server is not a filetransfer server";
	  SetAndLogError($newfh, $t, "  $t\n");
	  return (undef, undef, undef);
	} else {
	  # treat SFTP as tftp
	  $dir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
	  $relativeDir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
	}
      } else {
	# Filetransfer protcol is not tftp.  Relect in fxrData structure.
	if ($fxrData->{SERVERS}{$fxrServer} ne $localUser) {
	  my $pw = getpwnam($fxrData->{SERVERS}{$fxrServer});
	  # The file transfer user is not the same as the user logged on so create directories for storing file transfer related files.
##	  $dir = $pw->dir . "/$toolData->{TFTPWORKDIR}/$isamID";
	  $dir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
#	  $dir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID/$localUser" if (!$params->{ID});
	  #$dir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
	  ToolCommon::Log($newfh, "File transfer home location: '$dir'\n") if $params->{DEBUG};
##	  $relativeDir = "$toolData->{TFTPWORKDIR}/$isamID/$userName";
	  $relativeDir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
#	  $relativeDir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID/$localUser" if (!$params->{ID});
	} else {
##	  $dir = "$ENV{HOME}/$toolData->{TFTPWORKDIR}/$isamID";
	  $dir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
#	  $dir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID/$localUser" if (!$params->{ID});
##	  $relativeDir = "$toolData->{TFTPWORKDIR}/$isamID/$userName";
	  $relativeDir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
#	  $relativeDir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID/$localUser" if (!$params->{ID});
	}
      }
    } else {
      my $t = sprintf("failed to determine whether this machine is a filetransfer server: %s", ToolCommon::GetError());
      SetAndLogError($newfh, $t, "  $t\n");
      return (undef, undef, undef);
    }
  } else {
    $dir = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamID";
    $relativeDir = "$toolData->{TFTPWORKDIR}/$isamID";
  }
  # Set filetransfer home directory.  In case of ftp/sftp usage, the filetransfer user may be different from the user running this script
  $fxrData->{HOMEDIR} = $userName ne "" ? "$dir/$userName" : $dir;
  $fxrData->{HOMEDIR} =~ s/\/\//\//g;       # On unix this does not make a difference but let's convert double forward slashes into a single one
  $fxrData->{XVPS_HOMEDIR} = "$relativeDir/$userName";  # Only tftp for xVPS DB transfer so store it for later usage.  Only applicable for R3.3/R3.4
                                                        # where xVPS DB is uploaded from xVPS.  From R3.5 it is included in the NT backup.
  return ($dir, $relativeDir, $fxrData);
}

=head2 SetAndLogError($$;$), GetError(), ClearError()

=begin html
Print and set the detected error for further processing.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>errStr</td><td>Error message to be written in the child process log file</td></tr>
  <tr><td>logInfo</td><td>Error message to be written in the NE result log file</td></tr>
</table>

=end html

=cut

sub SetAndLogError($$;$) {
  my ($fh, $errStr, $logInfo) = @_;

  $ERROR = $errStr;
  ToolCommon::Log($fh, defined($logInfo) ? $logInfo : "$errStr\n");

}

sub GetError() { return $ERROR;}
sub ClearError() { $ERROR = ""; }

# To avoid too many function parameters...  I know it's not that nice...
my %failedNodes;
my %warnNodes;
my $nodeStatusLog = undef;
my $oldTransactionTO = undef;
my $childFh = undef;
my $parentFh = undef;
my $childCount = undef;
my $failedNode = undef;
my $cliSession2Close = undef;
my $tl1Session2Close = undef;
my $sshTl1Session2Close = undef;
my $snmpSession2Close = undef;

=head2 IncrErrorCount($$$$)

=begin html
Function to keep track of the number of errors detected by this child run.  It is using the following global variables:
<ul>
  <li>@failedNodes</li>
  <li>$nodeStatusLog</li>
  <li>$oldTransactionTO</li>
  <li>$childFh</li>
  <li>$parentFh</li>
  <li>$childCount</li>
  <li>$failedNode</li>
  <li>$cliSession2Close</li>
  <li>$snmpSession2Close</li>
</ul>
The following emergency recoveries are carried out in case an error is detected for a certain ISAM (take into account that we do not check whether
this was successful):
<ul>
  <li>Reset CLI transaction timeout value</li>
  <li>Allow alarms on CLI again</li>
  <li>Release the CLI transaction</li>
  <li>Close the CLI session and release the telnet session</li>
  <li>Reset SNMP transaction owner</li>
  <li>Close SNMP session</li>
</ul>
<br>The following parameters are used:
<table>
  <tr><td>msg</td><td>Error message</td></tr>
  <tr><td>errCode</td><td>error code (as defined in ErrorCodes.pm)</td></tr>
  <tr><td>addInfo</td><td>an optional additional message</td></tr>
</table>

=end html

=cut

my $errorCount = 0;
my $warningCount = 0;
sub IncrErrorCount($$$$) {
  my ($msg, $errCode, $addInfo, $close) = @_;
  my $str = ErrorCodes::ErrorString($errCode);
  $str .= " $addInfo" if (defined($addInfo));

  # Important note:
  # A call to FamilyCommon::SetWarningLevels is required when warning and error levels are associated to the error code being
  # passed to IncrErrorCount (is the case for - currently the only 2 - preCheck and statusCheckCmp)
  #
  if (not(defined($warnLevels)) or (defined($warnLevels) and (ErrorCodes::WarningLevelIsError($warnLevels->{$errCode})))) {
    $errorCount++;
    if (defined($failedNodes{$failedNode})) {
      $failedNodes{$failedNode}++;
    } else {
      $failedNodes{$failedNode} = 1;
    }
    $str .= " [Error]" if (defined($warnLevels));
  } elsif (defined($warnLevels) and (ErrorCodes::WarningLevelIsWarning($warnLevels->{$errCode}))) {
    $warningCount++;
    if (defined($warnNodes{$failedNode})) {
      $warnNodes{$failedNode}++;
    } else {
      $warnNodes{$failedNode} = 1;
    }
    $str .= " [Warning]" if (defined($warnLevels));
#    ToolCommon::Log($fh, "Incrementing warning count for '$failedNode' to $warnNodes{$failedNode}' for '$errCodeString'\n");
  }
  LogError( $childFh, $msg) if( $msg ne "");
  #ToolCommon::Log($childFh, $msg) if ($msg ne "");
  if (defined($nodeStatusLog)) {
    print $nodeStatusLog "$str\n";
    if (defined($close)) {
      my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime(time()))[0,1,2,3,4,5];
      ToolCommon::Log($parentFh, sprintf("Child $childCount: %4d-%02d-%02d %02d:%02d:%02d: Failed for %s: %s\n",
					 $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $failedNode, $str), 1);
      print $nodeStatusLog sprintf("%s\n", ErrorCodes::ExitString(EXIT_CHILD_FATAL_CODE))
	if ErrorCodes::IsPreCheckError($errCode);
      close $nodeStatusLog;
      $nodeStatusLog = undef;
    }
  } else {
    print STDERR "$str\n";
    if (defined($close)) {
      my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime(time()))[0,1,2,3,4,5];
      print STDERR sprintf("%s%4d-%02d-%02d %02d:%02d:%02d: Failed for %s: %s\n",
			   defined($childCount) ? "Child $childCount: " : "",
			   $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $failedNode, $str);
      print STDERR sprintf("%s\n", ErrorCodes::ExitString(EXIT_CHILD_FATAL_CODE)) if ErrorCodes::IsPreCheckError($errCode);
    }
  }
  if (defined($close)) {
    if (defined($cliSession2Close)) {
      IsamCli::SetTransactionTimeout($cliSession2Close, $oldTransactionTO) if (defined($oldTransactionTO));
      IsamCli::AllowAlarms($cliSession2Close);
      IsamCli::Transaction($cliSession2Close, 'release'); # ignore possible error - will time out eventually
      IsamCli::Disconnect($cliSession2Close);
      $cliSession2Close = undef;
    }
    if (defined($snmpSession2Close)) {
      SNMPComm::SetValue( 'RELEASE OWNER', TRANSACTOWNER, "0.0.0.0" );
      $snmpSession2Close->close();
      $snmpSession2Close = undef;
    }
    if (defined($tl1Session2Close)) {
      IsamTL1::AllowAlarms($tl1Session2Close);
      IsamTL1::Transaction($tl1Session2Close, 'release'); # ignore possible error - will time out eventually
      IsamTL1::Disconnect($tl1Session2Close);
      $tl1Session2Close = undef;
    }
    if (defined($sshTl1Session2Close)) {
      SshChannel::Disconnect($sshTl1Session2Close);
      $sshTl1Session2Close = undef;
    }
  }
#  $childFh = undef;
#  $failedNode = undef;
}
sub ReportStatus($$;$) {
  my ( $errCode, $errMesg, $logMesg ) = @_;
  my $mesgToUpdate = "'".ErrorCodes::ErrorString($errCode)."'";
  $mesgToUpdate .= ": $errMesg" if (defined($errMesg));


  # Important note:
  # A call to FamilyCommon::SetWarningLevels is required when warning and error levels are associated to the error code being
  # passed to IncrErrorCount (is the case for - currently the only 2 - preCheck and statusCheckCmp)
  #
  $logMesg = ( defined $logMesg and $logMesg ne "" ) ? "$logMesg => $mesgToUpdate\n" : "$mesgToUpdate\n";
  if( not defined($warnLevels) or not defined $warnLevels->{$errCode} ) {
       $errorCount++;
       ++$failedNodes{$failedNode};
       Logger::LogError( $execLogFH, "Session ($childCount), Node ($failedNode) : $logMesg\n");
       Logger::LogError( $nodeLogFH, "$logMesg\n", undef, $Logger::LogOnlyHandle );
  } elsif ( ErrorCodes::WarningLevelIsIgnore($warnLevels->{$errCode}) ) {
       Logger::LogNotify( $execLogFH, "(IGNORED NOW) $logMesg\n");
       Logger::LogNotify( $nodeLogFH, "(IGNORED NOW) $logMesg\n");
       return 0;
  } elsif ( ErrorCodes::WarningLevelIsWarning($warnLevels->{$errCode}) ) {
       $warningCount++;
       ++$warnNodes{$failedNode};
       Logger::LogWarn( $execLogFH, "Session ($childCount), Node ($failedNode) : $logMesg\n");
       Logger::LogWarn( $nodeLogFH, "$logMesg\n", undef, $Logger::LogOnlyHandle );
       $mesgToUpdate .= " [Warning]";
       if (defined($nodeStatusLog)) {
           print $nodeStatusLog "$mesgToUpdate\n";
       } else {
           print STDERR "$mesgToUpdate\n";
       }
      return 0;
  } elsif ( ErrorCodes::WarningLevelIsError($warnLevels->{$errCode}) ) {
       $errorCount++;
       ++$failedNodes{$failedNode};
       Logger::LogError( $execLogFH, "Session ($childCount), Node ($failedNode) : $logMesg\n");
       Logger::LogError( $nodeLogFH, "$logMesg\n", undef, $Logger::LogOnlyHandle );
       $mesgToUpdate .= " [Error]";
  }
  if (defined($nodeStatusLog)) {
       print $nodeStatusLog "$mesgToUpdate\n";
       print $nodeStatusLog sprintf("%s\n", ErrorCodes::ExitString(EXIT_CHILD_FATAL_CODE)) if ErrorCodes::IsPreCheckError($errCode);
       close $nodeStatusLog;
       $nodeStatusLog = undef;
  } else {
       print STDERR "$mesgToUpdate\n";
       print STDERR sprintf("%s\n", ErrorCodes::ExitString(EXIT_CHILD_FATAL_CODE)) if ErrorCodes::IsPreCheckError($errCode);
  }
  if (defined($cliSession2Close)) {
       IsamCli::SetTransactionTimeout($cliSession2Close, $oldTransactionTO) if (defined($oldTransactionTO));
       IsamCli::AllowAlarms($cliSession2Close);
       IsamCli::Transaction($cliSession2Close, 'release'); # ignore possible error - will time out eventually
       IsamCli::Disconnect($cliSession2Close);
       $cliSession2Close = undef;
  }
  if (defined($snmpSession2Close)) {
       SNMPComm::SetValue( 'RELEASE OWNER', TRANSACTOWNER, "0.0.0.0" );
       $snmpSession2Close->close();
       $snmpSession2Close = undef;
  }
  if (defined($tl1Session2Close)) {
       IsamTL1::AllowAlarms($tl1Session2Close);
       IsamTL1::Transaction($tl1Session2Close, 'release'); # ignore possible error - will time out eventually
       IsamTL1::Disconnect($tl1Session2Close);
       $tl1Session2Close = undef;
  }
  if (defined($sshTl1Session2Close)) {
       SshChannel::Disconnect($sshTl1Session2Close);
       $sshTl1Session2Close = undef;
  }
  return 1;
#  $childFh = undef;
#  $failedNode = undef;
}

=head2 IncrWarnCount($$$)

=begin html
Function to keep track of the number of warnings detected by this child run. The warning string is logged to the result file also
It is using the following global variables:
<ul>
  <li>%warnNodes</li>
  <li>$nodeStatusLog</li>
  <li>$overAllWarnCode</li>
  <li>$warningCount</li>
</ul>
<br>The following parameters are used:
<table>
  <tr><td>msg</td><td>Warning message</td></tr>
  <tr><td>errCode</td><td>error code (as defined in ErrorCodes.pm)</td></tr>
  <tr><td>addInfo</td><td>an optional additional message</td></tr>
</table>

=end html

=cut

sub IncrWarnCount($$$) {
  my ($msg, $errCode, $addInfo) = @_;
  my $str = ErrorCodes::ErrorString($errCode);
  $str .= " $addInfo" if (defined($addInfo));

    $overAllWarnCode = $errCode;
    $warningCount++;
    if (defined($warnNodes{$failedNode})) {
      $warnNodes{$failedNode}++;
    } else {
      $warnNodes{$failedNode} = 1;
    }

  $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
  if (defined($nodeStatusLog)) {
    print $nodeStatusLog sprintf("%s\n", $str);
  }
}


=head2 ToolExit($)

=begin html
Perform emergency recovery and exit this child process because of some fatal condition.It is using the following global variables:
<ul>
  <li>$cliSession2Close</li>
  <li>$snmpSession2Close</li>
</ul>
The following emergency recoveries are carried out (take into account that we do not check whether this was successful):
<ul>
  <li>Reset CLI transaction timeout value</li>
  <li>Allow alarms on CLI again</li>
  <li>Release the CLI transaction</li>
  <li>Close the CLI session and release the telnet session</li>
  <li>Reset SNMP transaction owner</li>
  <li>Close SNMP session</li>
</ul>
<br>The following parameters are used:
<table>
  <tr><td>exitCode</td><td>the tool exit code</td></tr>
</table>

=end html

=cut

sub ToolExit($) {
  my ($exitCode) = @_;

  if (defined($cliSession2Close)) {
    IsamCli::SetTransactionTimeout($cliSession2Close, $oldTransactionTO) if (defined($oldTransactionTO));
    IsamCli::AllowAlarms($cliSession2Close);
    IsamCli::Transaction($cliSession2Close, 'release'); # ignore possible error - will time out eventually
    IsamCli::Disconnect($cliSession2Close);
    $cliSession2Close = undef;
  }
  if (defined($snmpSession2Close)) {
    SNMPComm::SetValue( 'RELEASE OWNER', TRANSACTOWNER, "0.0.0.0" );
    $snmpSession2Close->close();
    $snmpSession2Close = undef;
  }
  if (defined($tl1Session2Close)) {
    IsamTL1::AllowAlarms($tl1Session2Close);
    IsamTL1::Transaction($tl1Session2Close, 'release'); # ignore possible error - will time out eventually
    IsamTL1::Disconnect($tl1Session2Close);
    $tl1Session2Close = undef;
  }
  if (defined($sshTl1Session2Close)) {
    SshChannel::Disconnect($sshTl1Session2Close);
    $sshTl1Session2Close = undef;
  }
  print STDERR sprintf("[PID %s] - exiting with %s\n", GetOwnPid(), $exitCode);
  CORE::exit($exitCode);
}

=head2 AdvanceToNext($)

=begin html
Perform recovery for this ISAM in case we need to skip further operations.  It is using the following global variables:
<ul>
  <li>$nodeStatusLog</li>
  <li>$oldTransactionTO</li>
  <li>$childFh</li>
  <li>$parentFh</li>
  <li>$childCount</li>
  <li>$failedNode</li>
  <li>$cliSession2Close</li>
  <li>$snmpSession2Close</li>
</ul>
The following emergency recoveries are carried out (take into account that we do not check whether this was successful):
<ul>
  <li>Reset CLI transaction timeout value</li>
  <li>Allow alarms on CLI again</li>
  <li>Release the CLI transaction</li>
  <li>Close the CLI session and release the telnet session</li>
  <li>Reset SNMP transaction owner</li>
  <li>Close SNMP session</li>
</ul>
<br>The following parameters are used:
<table>
</table>

=end html

=cut

sub AdvanceToNext($) {
  my ($msg) = @_;
  my $str = ErrorCodes::ExitString(SUCCESS_CODE);

  ToolCommon::Log($childFh, $msg) if ($msg ne "");
  my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime(time()))[0,1,2,3,4,5];
  if (defined($parentFh)) {
    ToolCommon::Log($parentFh, sprintf("Child $childCount: %4d-%02d-%02d %02d:%02d:%02d: Finished %s: %s\n",
				       $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $failedNode, $str), 1);
  } else {
    print STDERR sprintf("Child $childCount: %4d-%02d-%02d %02d:%02d:%02d: Finished %s: %s\n",
			 $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $failedNode, $str);
  }
  if (defined($nodeStatusLog)) {
    print $nodeStatusLog "$str\n";
    close $nodeStatusLog;
    $nodeStatusLog = undef;
  } else {
    print STDERR "$str\n";
  }
  if (defined($cliSession2Close)) {
    IsamCli::SetTransactionTimeout($cliSession2Close, $oldTransactionTO) if (defined($oldTransactionTO));
    IsamCli::Transaction($cliSession2Close, 'release'); # ignore possible error - will time out eventually
    IsamCli::AllowAlarms($cliSession2Close);
    IsamCli::Disconnect($cliSession2Close);
    $cliSession2Close = undef;
  }
  if (defined($snmpSession2Close)) {
    SNMPComm::SetValue( 'RELEASE OWNER', TRANSACTOWNER, "0.0.0.0" );
    $snmpSession2Close->close();
    $snmpSession2Close = undef;
  }
  if (defined($tl1Session2Close)) {
    IsamTL1::AllowAlarms($tl1Session2Close);
    IsamTL1::Transaction($tl1Session2Close, 'release'); # ignore possible error - will time out eventually
    IsamTL1::Disconnect($tl1Session2Close);
    $tl1Session2Close = undef;
  }
}
#-----------------
# Get, Set, Clear and Close functions
# own PID

=head2 SetOwnPid($), GetOwnPid(), ClearOwnPid()

=begin html
Functions to set, retrieve or clear the ProcessID of the active PBMT tool.

=end html

=cut

my $ownPid;
sub SetOwnPid($) {
  my ($pid) = @_;

  $ownPid = $pid;
}
sub GetOwnPid() { return $ownPid; }
sub ClearOwnPid() { $ownPid = undef; }

=head2 SetWarnLogFile($), GetWarnLogFile(), CloseWarnLogFile()

=begin html
Functions to set, retrieve or close the warning logfile maintained by the active PBMT tool.

=end html

=cut

# Warn logfile
my $warnLogFile = undef;
sub SetWarnLogFile($) {
  my ($handle) = @_;

  $warnLogFile = $handle;
#  ToolCommon::Log($warnLogFile, "Set warnLogFile: $handle\n");
}
sub GetWarnLogFile() { return $warnLogFile; }
sub CloseWarnLogFile() {
  close $warnLogFile if (defined($warnLogFile));
  $warnLogFile = undef;
}

=head2 SetParentLogFile($), GetParentLogFile(), CloseParentLogFile()

=begin html
Functions to set, retrieve or close the logfile maintained by the active PBMT parent tool.

=end html

=cut

# Parent logfile
sub SetParentLogFile($) {
  my ($handle) = @_;

  $parentFh = $handle;
#  ToolCommon::Log($parentFh, "Set parentFh: $handle\n");
}
sub GetParentLogFile() { return $parentFh; }

=head2 SetChildLogFile($), GetChildLogFile(), CloseChildLogFile()

=begin html
Functions to set, retrieve or close the logfile maintained by the active PBMT child tool.

=end html

=cut

# Child logfile
sub SetChildLogFile($) {
  my ($handle) = @_;

  $childFh = $handle;
#  ToolCommon::Log($childFh, "Set childFh: $handle\n");
}
sub GetChildLogFile() { return $childFh; }
sub CloseChildLogFile() {
  close $childFh if (defined($childFh));
  $childFh = undef;
}

=head2 SetNodeStatusLog($), GetNodeStatusLog(), CloseNodeStatusLog()

=begin html
Functions to set, retrieve or close the status result file maintained by the active PBMT tool.

=end html

=cut

# Node status logfile
sub SetNodeStatusLog($) {
  my ($handle) = @_;

  $nodeStatusLog = $handle;
#  ToolCommon::Log($nodeStatusLog, "Set nodeStatusLog: $handle\n");
}
sub GetNodeStatusLog() { return $nodeStatusLog; }
sub CloseNodeStatusLog() {
  close $nodeStatusLog if (defined($nodeStatusLog));
  $nodeStatusLog = undef;
}

sub CreateNodeStatusLog($$) {
    my ($nodeStatusLogFile, $logLabel) = @_;
    my $isamNode = GetFailedNode();
    my $nodeStatusLog = undef; ## Local dummy
    $logLabel = "Node ($isamNode)" if( not defined($logLabel) );
    if (! open($nodeStatusLog, ">$nodeStatusLogFile")) {
      Logger::LogError( $execLogFH, "$logLabel : Unable to create status file '$nodeStatusLogFile' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE);
      return; ## DUMMY for safer side
    } 
    Logger::LogInfo($nodeLogFH, sprintf("$logLabel : Result status file '%s%s'\n",
                              ($nodeStatusLogFile  =~ /^\//) ? "" : getcwd()."/",  $nodeStatusLogFile ) );
    $nodeStatusLog->autoflush(1);
    SetNodeStatusLog( $nodeStatusLog );
    return $nodeStatusLog;
}

sub UpdateNodeStatusLog($;$) {
    my ($exitCode, $logMesg, $dontClose ) = @_;
    my $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
    if (defined($nodeStatusLog)) {
      $logMesg = (defined $logMesg ) ? "'$logMesg'" : '';
      print $nodeStatusLog sprintf("%s %s\n", ErrorCodes::ExitString($exitCode), $logMesg );
      Logger::LogNotify( $nodeLogFH, sprintf("STATUS : %s %s\n", ErrorCodes::ExitString($exitCode), $logMesg ) );
      FamilyCommon::CloseNodeStatusLog() if( not defined $dontClose );
    }
}

=head2 ClearCliSession2Close(), SetCliSession2Close($), GetCliSession2Close(), CloseCliSession2Close()

=begin html
Functions to clear, set, retrieve or close the CLI session setup towards the ISAM by a PBMT tool.

=end html

=cut

# CLI session to be closed at error
sub ClearCliSession2Close() { $cliSession2Close = undef; }
sub SetCliSession2Close($) {
  my ($session) = @_;

  $cliSession2Close = $session;
}
sub CloseCliSession2Close() {
  IsamCli::Disconnect($cliSession2Close) if (defined($cliSession2Close));
  $cliSession2Close = undef;
}
sub GetCliSession2Close() { return $cliSession2Close; }

=head2 ClearSnmpSession2Close(), SetSnmpSession2Close($), GetSnmpSession2Close(), CloseSnmpSession2Close()

=begin html
Functions to clear, set, retrieve or close the SNMP session setup towards the ISAM by a PBMT tool.

=end html

=cut

=head2 ClearTL1Session2Close(), SetTL1Session2Close($), GetTL1Session2Close(), CloseTl1Session2Close()

=begin html
Functions to clear, set, retrieve or close the TL1/UDP session setup towards the ISAM by a PBMT tool.

=end html

=cut

# TL1 session to be closed at error
sub ClearTL1Session2Close() { $tl1Session2Close = undef; }
sub SetTL1Session2Close($) {
  my ($session) = @_;

  $tl1Session2Close = $session;
}
sub CloseTL1Session2Close() {
  IsamTL1::Disconnect($tl1Session2Close) if (defined($tl1Session2Close));
  $tl1Session2Close = undef;
}
sub GetTL1Session2Close() { return $tl1Session2Close; }

=head2 ClearSshTL1Session2Close(), SetSshTL1Session2Close($), GetSshTL1Session2Close(), CloseSshTl1Session2Close()

=begin html
Functions to clear, set, retrieve or close the TL1/SSH session setup towards the ISAM by a PBMT tool.

=end html

=cut

# TL1/SSH session to be closed at error
sub ClearSshTL1Session2Close() { $sshTl1Session2Close = undef; }
sub SetSshTL1Session2Close($) {
  my ($session) = @_;

  $sshTl1Session2Close = $session;
}
sub CloseSshTL1Session2Close() {
  SshChannel::Disconnect($sshTl1Session2Close) if (defined($sshTl1Session2Close));
  $sshTl1Session2Close = undef;
}
sub GetSshTL1Session2Close() { return $sshTl1Session2Close; }

=head2 ClearSnmpSession2Close(), SetSnmpSession2Close($), GetSnmpSession2Close(), CloseSnmpSession2Close()

=begin html
Functions to clear, set, retrieve or close the SNMP session setup towards the ISAM by a PBMT tool.

=end html

=cut

# SNMP session to be closed at error
sub ClearSnmpSession2Close() { $snmpSession2Close = undef; }
sub SetSnmpSession2Close($) {
  my ($session) = @_;

  $snmpSession2Close = $session;
  SNMPComm::SetSession($session);
}
sub CloseSnmpSession2Close() {
  $snmpSession2Close->close() if (defined($snmpSession2Close));
  $snmpSession2Close = undef;
}
sub GetSnmpSession2Close() { return $snmpSession2Close; }
# Failed node

=head2 ClearFailedNode(), SetFailedNode($), GetFailedNode(), GetFailedNodeList(), GetWarnedNodeList(), GetErrorCount(), GetWarningCount(), NodeIsFailed()

=begin html
Functions to
<ul>
  <li>clear, set or get the failed node (general rule: the failed node is the ISAM currently being "worked" upon)</li>
  <li>Retrieve a list of nodes for which errors or warnings have been generated during the PBMT run</li>
  <li>Retrieve the amount of errors or warnings that have been encountered by a PBMT tool</li>
  <li>Determine whether an error has been detectedt for the node being worked on</li>
</ul>

=end html

=cut

sub ClearFailedNode() { $failedNode = undef; }
sub SetFailedNode($) {
  my ($id) = @_;

  $failedNode = $id;
}
sub NodeIsFailed() {
  return (defined($failedNodes{$failedNode}) ? 1 : 0);
}
sub GetFailedNode() { return $failedNode; }
# Failed node list
sub GetErrorCount() { return $errorCount; }
sub GetFailedNodeList() { return \%failedNodes; }
# Warning node list
sub GetWarningCount() { return $warningCount; }
sub GetWarnedNodeList() { return \%warnNodes; }

=head2 SetChildNum($), GetChildNum()

=begin html
Functions to set or retrieve the number associated to this child process.

=end html

=cut

# Child number
sub SetChildNum($) {
  my ($childNum) = @_;

  $childCount = $childNum;
}
sub GetChildNum() { return $childCount;}

=head2 SetToolDirName($), GetToolDirName()

=begin html
Functions to set or retrieve or close the installation directory of the PBMT tools.

=end html

=cut

# Tool directory name
my $toolDirName = undef;
sub SetToolDirName($) {
  my ($name) = @_;

  $toolDirName = $name;
}
sub GetToolDirName() { return $toolDirName };

=head2 SetSigPipeReceived(), ClearSigPipeReceived(), GetSigPipeReceived()

=begin html
Functions to set, clear or retrieve the indication whether a SIGPIPE has been caught by the active PBMT tool.

=end html

=cut

# SIGPIPE handling
my $sigPipeReceived = 0;
sub SetSigPipeReceived() { $sigPipeReceived = 1;}
sub ClearSigPipeReceived() { $sigPipeReceived = 0; }
sub GetSigPipeReceived() { return $sigPipeReceived; }

=head2 SetOldTransactionTO($), GetOldTransactionTO()

=begin html
Functions to set or retrieve the original transaction timeout value as configured in the ISAM.

=end html

=cut

# Transaction timeout
sub SetOldTransactionTO($) { 
  my ($value) = @_;
  $oldTransactionTO = $value;
}
sub GetOldTransactionTO() { return $oldTransactionTO; }

=head2 SetConfigData($), GetConfigData(), SetWarningLevels($), SetToolData($)

=begin html
Functions to store a pointer to
<ul>
  <li>the PBMT configuration data (mainly equivalent to what has been read from the param.cfg)
  <li>the warning levels associated to the prechecking or status checking</li>
  <li>the data used by a certain PBMT tool class (e.g. the mgiration process) as shared between the script and its supporting modules</li>
</ul>

=end html

=cut

# Config data pointer
sub SetConfigData($) {
  my ($configData) = @_;

  $config = $configData;
}
sub GetConfigData() { return $config; }
# Warning level pointer
sub SetWarningLevels($) {
  my ($warningLevels) = @_;

  $warnLevels = $warningLevels;
}
sub GetWarningLevels() { return $warnLevels; }

sub SetToolData($) {
  my ($toolDataPtr) = @_;

  $toolData = $toolDataPtr;
}
sub GetToolData() { return $toolData; }

=head2 SetNeStorage($), GetNeStorage()

=begin html
Functions to set or retrieve the directory name associated to the temporary storage for this NE.

=end html

=cut

sub SetNeStorage($) {
  my ($dir) = @_;
  $neStorage = $dir;
}

sub GetNeStorage() {
  return $neStorage;
}

=head2 SetFailontlistFile($)

=begin html
Function to store the handle to the ont status file in the global variable $failedontlistfile.
<ul>
  <li>The ont status file contains the list of ONTs that failed during migration</li>
</ul>
<br>
The following parameters are used:
<table>
  <tr><td>handle</td><td>handle to the ont status file.
</table>

=end html

=cut

sub SetFailontlistFile($){
  my ($handle) = @_;
  $failedontlistfile = $handle;
}

=head2 GetFailontListFile($)

=begin html
Function to return the handle to the ONT status file
<ul>
  <li>The SetFailontlistFile routine should have been called before invoking this routine</li>
</ul>
<br>
There are no parameters to this routine
=end html

=cut

sub GetFailontListFile() {
  return $failedontlistfile;
}

sub WaitForRestartConnect($$$) {
  my ($fh, $asam, $shub) = @_;
  my $notReachable = 1;
  my $waitedSeconds = 10;
  my $pingOk = 0;
  my $ok = 1;
  my $firstSnmp = 1;
  my ($time1, $timeDiff, $sleepTime, $startTime, $endTime);

  $startTime = time();
  if ($params->{NOPING}) {
    ToolCommon::Log($fh, sprintf("Waiting for %s restart...Trying to check if connection can be opened", defined($shub) ? "SHUB" : "NT"));
    sleep(120);
    $pingOk = 1;
  } else {
    ToolCommon::Log($fh, sprintf("Waiting for %s restart...Ping", defined($shub) ? "SHUB" : "NT"));
  }
  
  sleep(10);

  while ($notReachable && $waitedSeconds < $toolData->{MAXWAITTIME}) {
    ToolCommon::Log($fh, ".", undef, 1);
    $time1 = time();
    if (!$pingOk) {
       if(ToolCommon::Ping($config->{PING}, $asam->[0])) {  # -- 1.1.3 -- 1.1.6
           $endTime = time();
           #-- ok Asam is reachable by ping
           ToolCommon::Log($fh, sprintf("OK (after %d seconds)... Trying to check if connection can be opened \n", $endTime - $startTime));
           $startTime = time();
           $pingOk = 1;
       }
    } else {
       ToolCommon::Log($fh, "\n", undef, 1);
       my  $cliSession = ToolCommon::OpenCliWrap($asam, $fh, $toolData->{TELNETTIMEOUT});

       if($cliSession){
          $endTime = time();
          $notReachable = 0;
          sleep(5);
          IsamCli::Disconnect($cliSession);
       }
    }
    $timeDiff = time() - $time1;
    if ($notReachable) {
      $sleepTime = 10 - $timeDiff;
      sleep($sleepTime) if ($sleepTime > 0);
      $waitedSeconds += $timeDiff > 10 ? $timeDiff : 10;
    }
  }
  if ($notReachable) {
    SetAndLogError($fh, "ISAM not back (timeout)", "  FAILED (timeout)\n");
    $ok = 0;
  }
  ToolCommon::Log($fh, sprintf("OK (after %d seconds)", $endTime - $startTime)) if ($ok);
  ToolCommon::Log($fh, "\n", undef, 1);
  return $ok;
}


sub WaitForNePingAble($$) {

  my ($fh, $isam) = @_;
  my $notReachable = 1;
  my $waitedSeconds = 10;
  my $pingOk = 0;
  my $uptime;
  my $ok = 1;
  my ($time1, $timeDiff, $sleepTime, $startTime, $endTime);

  $startTime = time();
  sleep(10);

  ToolCommon::Log($fh, sprintf("Waiting for NT restart...Ping\n"));


  while ($notReachable && $waitedSeconds < $toolData->{MAXWAITTIME}) {
    ToolCommon::Log($fh, ".", undef, 1);
    $time1 = time();
    if (!$pingOk) {
      if(ToolCommon::Ping($config->{PING}, $isam->[0])) {  # -- 1.1.3 -- 1.1.6
        $endTime = time();
        $pingOk = 1;
        $notReachable = 0;
      }
   }
   ToolCommon::Log($fh, "\n", undef, 1) if $params->{DEBUG};  #just to have a clean log file with new line

   $timeDiff = time() - $time1;
    if ($notReachable) {
      $sleepTime = 10 - $timeDiff;
      sleep($sleepTime) if ($sleepTime > 0);
      $waitedSeconds += $timeDiff > 10 ? $timeDiff : 10;
    }
  } #while


  if ($notReachable) {
    SetAndLogError($fh, "  FAILED (timeout)\n");
    $ok = 0;
  } else {
   ToolCommon::Log ($fh,  " Ping OK \n");
   }
  return $ok;
 }

1;
