package FileTransfer;

#==============================================================================
#-- Copyright     :    All rights reserved. Passing on and copying of this document,
#--                    use and communication Of its contents not permitted without
#--                    written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- File          :    StageONT7342.pm
#-- Date          :    2009/07/16 
#-- Company       :    Alcatel-Lucent
#-- Author        :    Palaniappan Ramasamy
#------------------------------------------------------------------------------
#-- Function      :    Script to perform File Transfer
#------------------------------------------------------------------------------
#-- Disclaimer    :    This script contains references to proprietary information
#--                    which is owned by Alcatel- Lucent.
#------------------------------------------------------------------------------
#-- Copyright     :    All rights reserved. Passing on and copying of this document,
#--                    use and communication Of its contents not permitted without
#--                    written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- Requirements  :
#------------------------------------------------------------------------------
#-- History       :
#-- 1.0.1         :    2009/07/16 : First Implementation.
#-- 1.0.2         :    2009/08/07 : DDTs tickets resolved 
#--                                   BDFam52649
#-- 1.0.3         :    2009/08/21 : DDTs tickets resolved 
#--                                   BDFam52780
#-- 1.0.4         :    2009/09/07 : DDTs tickets resolved 
#--                                   BDFam53861
#--                                   BDFam54235
#-- 1.0.5         :    2009/10/01 : DDTs tickets resolved
#                                      BDFam54235
#-- 3.0.00        :    2010/04/21 : Converted version to 3.0.00 for converged PBMT
#-- 3.0.01        :    2011/01/12 : LRM adaptations for lib-perl path
#-- 3.0.02        :    2011/03/27 : Linux compatibility changes
#-- 3.0.03        :    2011/05/05 : ALU01178887
#-- 3.0.04        :    2013/04/29 : ALU01880147
#-- 3.0.05        :    2013/06/14 : ALU01877212
#-- 3.0.06        :    2016/09/28 : ALU02278259
#------------------------------------------------------------------------------

use Cwd;
use File::Basename;
use Config;

BEGIN {
  my $dir = dirname($0);
  my $path;
  my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif( $dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib");
  unshift(@INC, "$path/lib-perl/$perlLibLocation");

  if ($Config{archname} =~ /sun4/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$path/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
}

use Net::Telnet;
use IO::Pty;

my $VERSION = "FileTransfer.pm - 3.0.05 - 2013/06/14";
#==============================================================================

sub GetVersion() {
  return $VERSION;
}

sub OpenTelnet($$;$$$$);
sub closeTelnet($);
sub GetError();
sub PUT($$$);

my $ERROR = "Transfer timed out";                         # BDFam52780

sub GetError(){
  return $ERROR;
}

sub OpenTelnet($$;$$$$){
  my ($mode, $host, $user, $passwd, $timeout, $logFile) = @_;

  my $ssh = undef;
  my $prompt; 
  my $pty;

  if(!$mode){
    $ERROR = "Invalid Input";
    return undef;
  }
 
  if(uc($mode) eq 'SFTP'){
    $prompt = '/sftp>/'; 

    if ( (!$user) || (!$passwd) ){
      $ERROR = "Invalid Input";
      return undef;
    }

    $pty = &spawn("sftp  $user\@$host");  # spawn() defined below

    if (! $pty){
      $ERROR = "Spawn failed";
      return undef;
    }

    ## Create a Net::Telnet object to perform I/O on ssh's tty.
    $ssh = new Net::Telnet (-fhopen => $pty,
                            -timeout => defined($timeout) ? $timeout : 10,
                            -prompt => $prompt,
                            -telnetmode => 0,
                            -cmd_remove_mode => 1,
                            -output_record_separator => "\r",
                             # dump log for debug
                            -input_log => defined($logFile) ? $logFile : "/tmp/telnet_ssh_$host.trace"
                            );

    ## Login to remote host.
    if (! ($ssh->waitfor(-match => '/password: ?$/i',
                  -errmode => "return")) ){
      if ( $ssh->lastline =~ /^DSA key fingerprint/ ) {
        $ssh->print('yes');
        if (! ($ssh->waitfor(-match => '/password: ?$/i',
                  -errmode => "return")) ) {
          $ERROR = $ssh->lastline;
          return undef;
        }
      } else {
        $ERROR = $ssh->lastline;
        return undef;
      }
    }

    $ssh->print($passwd);

    if (! ($ssh->waitfor(-match => $ssh->prompt,
		  -errmode => "return")) ) {
      $ERROR = $ssh->lastline;
      return undef;
    }
  }

  if ( uc($mode) eq 'TFTP') {
    $prompt = '/tftp>/';

    $pty = &spawn("tftp $host");  # spawn() defined below

    if (! $pty){
      $ERROR = "Spawn failed";
      return undef;
    }

    ## Create a Net::Telnet object to perform I/O on ssh's tty.
    $ssh = new Net::Telnet (-fhopen => $pty,
                            -timeout => defined($timeout) ? $timeout : 10,
                            -prompt => $prompt,
                            -telnetmode => 0,
                            -cmd_remove_mode => 1,
                            -output_record_separator => "\r",
                            -input_log => defined($logFile) ? $logFile : "/tmp/telnet_ssh_$host.trace"
                            );

    if (! ($ssh->waitfor(-match => $ssh->prompt,
                  -errmode => "return")) ) {
      $ERROR = $ssh->lastline;
      return undef;
    }
  }

  return $ssh;
}

sub closeTelnet($){
  my ($ssh) = @_;

  $ssh->close if ($ssh);
}

sub PUT($$$){
  my ($ssh,$source,$dest) = @_;

  if ( (!$ssh) || (!$source) || (!$dest) ){
    $ERROR = "Invalid Input";
    return undef;
  }
  my $t = $ssh->timeout;

  $ssh->timeout(1200);
  my @lines = $ssh->cmd(String => "put $source $dest",
                        Errmode => 'return');                               # BDFam52780 

  my $i = 1;
  my $lengthOfArray =@lines;                                               #BDFam54235 
  if (!$lengthOfArray){
    $ERROR = "/Broken pipe/";
    $i=0;
  }else{                                                                        #BDFam54235 ends here
  foreach my $type (@lines) {
    if ( ($type =~ /Failure/) ||
         ($type =~ /not found/) ||
         ($type =~ /Error/) || 
         ($type =~ /Permission denied/) ||     # BDFam53861
         ($type =~ /Transfer timed out/) ||    # BDFam52649
         ($type =~ /no write permission/) ||   # BDFam54235
         ($type =~ /Connection reset/) ||      # BDFam54235
         ($type =~ /Broken pipe/) ||           # BDFam54235
         ($type =~ /No such file/) ) {
    $ERROR = $type;
    $i = 0;
    }
  }
  }
  $ssh->timeout($t);

  return $i;
}

sub spawn {
    my(@cmd) = @_;
    my($pid, $pty, $tty, $tty_fd);
    
    ## Create a new pseudo terminal.
    use IO::Pty ();
    $^W = 0;
    $pty = new IO::Pty or return undef;
    
    ## Execute the program in another process.
    unless ($pid = fork) {  # child process
        die "problem spawning program: $!\n" unless defined $pid;
        #return undef unless defined $pid;
	
	## Disassociate process from existing controlling terminal.
	use POSIX ();
	POSIX::setsid
            or return undef;
	
	## Associate process with a new controlling terminal.
	$tty = $pty->slave;
	$pty->make_slave_controlling_terminal();
	$tty_fd = $tty->fileno;
	close $pty;
	
	## Make stdio use the new controlling terminal.
	open STDIN, "<&$tty_fd" or return undef;
	open STDOUT, ">&$tty_fd" or return undef;
        open STDERR, ">&STDOUT" or return undef;
	close $tty;
	
	## Execute requested program.
	exec @cmd or return undef;
    } # end child process
    
    $^W = 1;
    $pty;
} # end sub spawn

1;
