package Isam7342TL1;

=head1 PBMT ISAM 7342 TL1 accessor module

=begin html
This PERL module contains the TL1 accesses made to the ISAM 7342.

=end html

=cut

#==============================================================================
#-- File:         Isam7342TL1.pm
#-- Date:         27.11.2008
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Library with ISAM CLI commands
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2005/11/05   : Start First implementation
#--   1.0.1 - 2016/07/15   : ALU02237966
#--   1.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "Isam7342TL1.pm - 1.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "IsamCli.pm - 1.1.0 -- 2018/06/20";
#==============================================================================

use strict;
use warnings 'all';
use File::Basename;

use vars qw ($tmpLogFile);

use perlTL1::UdpChannel;
use ToolCommon qw(:CONSTANTS);
use ErrorCodes qw(:EXIT_CODES :COMMON);#New
use PBMTGlobal;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub SelectFH($);
sub SetDebug($);
sub GetError();
sub Connect($$$$;$);
sub Disconnect($);
sub SendCommand($$);
sub Transaction($$);
sub InhibitAlarms($);
sub AllowAlarms($);
sub GetOswpInfo($$);
sub CommitSWP($$);
sub AbortDownloadSWP($$);
sub DownloadSWP($$$$$);
#==============================================================================
#== Variables
#==============================================================================
my $DEBUG = undef;
my $ERROR = "";

=head1 FUNCTIONS

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 SetDebug()

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
#  print $DEBUG "Testing...\n";
}

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}

=head2 Connect($$$$;$)

=begin html
Open a TL1 session and establish TL1 login.  The opened telnet session is returned to the caller or undef in case of failure.<br>
The following parameters are used:
<table>
  <tr><td>ip</td><td>IP address of the NE</td></tr>
  <tr><td>udpPort</td><td>UDP port to be used for the TL1 session</td></tr>
  <tr><td>user</td><td>TL1 user name</td></tr>
  <tr><td>passwd</td><td>TL1 password</td></tr>
  <tr><td>logFile</td><td>file handle for logging of TELNET activity (undef if not needed)</td></tr>
</table>

=end html

=cut

sub Connect($$$$;$) {
  my ($isam, $udpPort, $userName, $passWord, $logFile) = @_;
  my $session = new UdpChannel;
  my $cmdResponse;
  my $ok = 0;

  $ok = $session->OpenChannel($isam, $udpPort, $logFile);
  if ($ok) {
    SendCommand($session, "LOGOFF;");
    $cmdResponse = SendCommand($session, sprintf("ACT-USER::%s:::%s;", $userName, $passWord));
    if (defined($cmdResponse)) {
      $ok = 1
    }
  }
  return $ok ? $session : undef;
}

=head2 Disconnect($)

=begin html
Logout from the NE and close the TL1 session specified by $session.

=end html

=cut

sub Disconnect($) {
  my ($session) = @_;

  SendCommand($session, "LOGOFF;");
  $session->CloseChannel();
}


=head2 SendCommand($$)

=begin html
Send a TL-1 command.<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>pointer to an opened TL-1 session</td></tr>
  <tr><td>cmd</td><td>TL-1 command to be sent</td></tr>
</table>

=end html

=cut

sub SendCommand($$) {
  my ($session, $cmd) = @_;
  my $retry = 1;
  my $cmdSends = 0;
  my $ok;
  my $response;

  while ($retry) {
    $cmd .= ";" if (not ($cmd =~/;$/)); # Add a semi-colon if one is not appended to the end
    $response = $session->SendTL1Command($cmd);
    if (!defined($response)) {
      $ERROR = $session->GetError();
      $ok = 0;
    } else {
      if (($response =~ /DENY/) && ($response =~ /SARB/)) {
	# All resources busy, try again
	select(undef, undef, undef, 0.1); # 100 msec delay
	$cmdSends++;
	$retry = 0 if ($cmdSends > 10);
	$ERROR = "Could not send command, all resources busy";
	$ok = 0;
      } else {
	$ok = !($response =~ /COMPLD/) ? 0 : 1;
	$ERROR = $response;
	$retry = 0;
      }
    }
  }
  $response =~ s/\r//g;
  return $ok ? $response : undef;
}

=head2 Transaction($$)

=begin html
Acquire or release command transaction as specified by $action using TL1 session $session.<br>Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub Transaction($$) {
  my ($session, $type) = @_;
  my $response;
  if ($type eq "acquire") {
    $response = SendCommand($session, "STA-CMDSSN::COM::;");
    if( !defined $response ) {
        FamilyCommon::ReportStatus( EC_CONFIG_LOCK_ACQUIRE_FAILED, GetError(), undef );
    } else {
        Logger::LogTrace( $nodeLogFH, "Acquired ownership for configuration-lock\n");
    }
  } else {
    $response = SendCommand($session, "STP-CMDSSN::COM::;");
    if( !defined $response ) {
        FamilyCommon::ReportStatus( EC_CONFIG_LOCK_RELEASE_FAILED, GetError(), undef );
    } else {
        Logger::LogTrace( $nodeLogFH, "Released ownership for configuration-lock\n");
    }
  }

  return defined($response) ? 1 : 0;
}

=head2 InhibitAlarms($)

=begin html
Inhibit generation of alarms using TL1 session $session.

=end html

=cut

sub InhibitAlarms($) {
  my ($session) = @_;
  my $response = SendCommand($session, "inh-msg-ALL::ALL:::CR&MJ&MN&NA;");

  return defined($response) ? 1 : 0;
}

=head2 AllowAlarms($)

=begin html
Allow generation of alarms using TL1 session $session.

=end html

=cut

sub AllowAlarms($) {
  my ($session) = @_;
  my $response = SendCommand($session, "alw-msg-ALL::ALL:::CR&MJ&MN&NA;");

  return defined($response) ? 1 : 0;
}

=head2 GetOswpInfo($$)

=begin html
Retrieve OSWP info using the opened TL1 session.  $regExp is used to determine the SW version numbers.
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: "ACTIVE" and "PASSIVE".  For each
entry the following is given:
<ul>
  <li>INDEX: the associated OSWP index</li>
  <li>NAME: the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>COMMITED: the OSWP SW commited status</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>

=end html

=cut

sub GetOswpInfo($$) {
  my ($session, $regExp) = @_;
  my $cmdResponse = SendCommand($session, "rept-opstat-oswp::all::;");
  my $index = undef;
  my $name = undef;
  my $status = undef;
  my $availability = undef;
  my $swpIndex;
  my $otherSwpIndex;
  my %oswpByNum;
  my %oswpInfo = ('ACTIVE' => { 'INDEX' => 0,
				'NAME' => NO_OSWP,
				'COMMITED' => "",
				'AVAIL' => "",
				'MAJOR' => 0,
				'MINOR' => 0,
				'SUFFIX' => 0,
				'MAINT' => 0 },
		  'PASSIVE' => { 'INDEX' => 0,
				 'NAME' => NO_OSWP,
				 'COMMITED' => "",
				 'AVAIL' => "",
				 'MAJOR' => 0,
				 'MINOR' => 0,
				 'SUFFIX' => 0,
				 'MAINT' => 0});
  my @active;
  my @passive;
  my $major = undef;    # -- 1.0.4
  my $minor = undef;    # -- 1.0.4
  my $maintRel = undef; # -- 1.2.0
  my $suffix = undef;   # -- 1.2.0

  if(!defined($cmdResponse)) {
    return undef;
  }
  my @response = split(/\n/, $cmdResponse);
  my ($actOswp, $passOswp);
  foreach my $line (@response) {
    if ($line =~ /\"OSWP-(\d):(\w+),(\w+)\"/) {
      my $oswpNum = $1;
      my $fieldName = $2;
      my $fieldConts = $3;
      $oswpByNum{$oswpNum}{$fieldName} = $fieldConts;
      $actOswp = $oswpNum if (($fieldName eq "ACTSTAT") && ($fieldConts eq "ACTIVE"));
    }
  }
  $passOswp = $actOswp == 1 ? 2 : 1;
  $oswpInfo{'ACTIVE'}{'INDEX'} = $actOswp;
  $oswpInfo{'ACTIVE'}{'COMMITED'} = lc($oswpByNum{$actOswp}{COMMITSTAT});
  $oswpInfo{'ACTIVE'}{'AVAIL'} = lc($oswpByNum{$actOswp}{AVAILSTAT});
  $oswpInfo{'PASSIVE'}{'INDEX'} = $passOswp;
  $oswpInfo{'PASSIVE'}{'COMMITED'} = lc($oswpByNum{$passOswp}{COMMITSTAT});
  $oswpInfo{'PASSIVE'}{'AVAIL'} = lc($oswpByNum{$passOswp}{AVAILSTAT});
  $cmdResponse = SendCommand($session, "rept-opstat-oswpdl::all::;");
  if(!defined($cmdResponse)) {
    return undef;
  }
  @response = split(/\n/, $cmdResponse);
  foreach my $line (@response) {
    if ($line =~ /\"OSWP-(\d):PACKNAME,(.*)\"/) {
      my $oswpNum = $1;
      my $oswpName = basename($2);
      if ($oswpName =~ /$regExp/) {  # -- 1.0.4
	$major = $1;                                  # -- 1.0.4
	$minor = $2;                                  # -- 1.0.4
	$suffix = $3;                                 # -- 1.2.0
	$maintRel = $4;                               # -- 1.2.0
      }                                               # -- 1.0.4
      if ($oswpNum == $actOswp) {
	$oswpInfo{'ACTIVE'}{'MAJOR'} = $major;
	$oswpInfo{'ACTIVE'}{'MINOR'} = $minor;
	$oswpInfo{'ACTIVE'}{'SUFFIX'} = $suffix;
	$oswpInfo{'ACTIVE'}{'MAINT'} = $maintRel;
	$oswpInfo{'ACTIVE'}{'NAME'} = $oswpName;
      } else {
	$oswpInfo{'PASSIVE'}{'MAJOR'} = $major;
	$oswpInfo{'PASSIVE'}{'MINOR'} = $minor;
	$oswpInfo{'PASSIVE'}{'SUFFIX'} = $suffix;
	$oswpInfo{'PASSIVE'}{'MAINT'} = $maintRel;
	$oswpInfo{'PASSIVE'}{'NAME'} = $oswpName;
      }
    }
  }
  return \%oswpInfo;
}

=head2 WaitForDownload($$)

=begin html
Wait for DB or SW download completion<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>oswpIndex</td><td>OSWP index</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForDownload($$) {
  my ($session, $oswpIndex) = @_;
  #-- check for completed download (or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $progress = undef;
  my $error;
  my $fileServer = "";
  my $errorFile = "";
  my $toReturn = undef;

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = SendCommand($session, sprintf("rept-opstat-oswpdl::oswp-%d", $oswpIndex));
      if (!defined($response)) {
	if ($retries > 4) {
	  # 5 retries, seems that the first ones may yield no response...
	  $ERROR = sprintf("Maximum command retransmissions tried for rept-opstat-oswpdl::oswp-%d", $oswpIndex);
	  $toReturn = 0;
	  return;
	}
	$retries++;
	sleep 2;
	next;
      }
      $retries = 0;
      $progress = undef;
      $fileServer = "";
      $errorFile = "";
      $error = "NOERROR";
      my @responseArray = split(/\n/, $response);
      foreach (@responseArray) {
#   "OSWP-2:DLPROGRESS,EMPTY"
#   "OSWP-2:DLERROR,NOERROR"
#   "OSWP-2:COMPSET,0"
#   "OSWP-2:AVAILCOMPSET,0"
#   "OSWP-2:TOTALDISK,151539712"
#   "OSWP-2:FREEDISK,118624256"
#   "OSWP-2:DLSERVPRI,172.31.212.116"
#   "OSWP-2:DLSERVSEC,0.0.0.0"
#   "OSWP-2:PACKNAME,NO_OSWP"
	if (/OSWP\-\d:DLPROGRESS,([\w-]+)/) {
	  $progress = $1;
	}
	if (/OSWP\-\d:DLERROR,([\w-]+)/) {
	  $error = $1
	}
	if (/OSWP\-\d:ERRFILE,(.*)/) {
	  $errorFile = $1;
	  $errorFile =~ s/\s//g;
          $errorFile =~ s/\"//g;
	}
	if (/OSWP\-\d:SERVID,(\d+\.\d+\.\d+\.\d+)/) {
	  $fileServer = $1;
	}
	if (/^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/) {
	  # Alarm received, continue
	  $progress = "DOWNLOADING";
	  $error = "NOERROR";
	}
      }
      ToolCommon::Log($DEBUG, sprintf("progress: %s, error: %s\n", defined($progress) ? $progress : "undef",
				      defined($error) ? $error : "undef")) if defined($DEBUG);
      if (defined($progress) && defined($error)) {
	if ($progress eq "ENABLED" && $error eq "NOERROR") {
	  $toReturn = 1;
	  return 1;
	} elsif ($progress eq "DOWNLOADING" && 
		 $error eq "NOERROR") {
	  sleep 2;
	} elsif ($progress eq "DISABLED") {
	    $ERROR = "Download failed. Error: $error (failed file: '$errorFile', server: '$fileServer'), availability: $progress";
	    $toReturn = 0;
	  return 0;
	} else {
	  $ERROR = sprintf("Internal error: Unused progress value (%s).\nFull response: %s\n", $progress, $response);
	  $toReturn = 0;
	  return 0;
	}
      } else {
	$ERROR = sprintf("No progress information available.\nFull response: %s\n", $response);
	$toReturn = 0;
	return 0;
      }
    }
  };
  ToolCommon::Log($DEBUG, "eval block ended ($evalRet)\n") if defined($DEBUG);
  if ($gotSigPipe) {
    ToolCommon::Log($DEBUG, "Local sigpipe handler called\n") if defined($DEBUG);
    $ERROR = "SIGPIPE (WaitForDownload)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 CommitSWP($$)

=begin html
Commit OSWP with index $swpIndex using the opened TL! session $session.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub CommitSWP($$) {
  my ($session, $swpIndex) = @_;
  my $response = SendCommand($session, sprintf("ed-oswp::oswp-%d::::ACTIVATION=,COMMITMENT=true,ABORT=,AUTOACTIVE=;", $swpIndex));

  return defined($response) ? 1 : 0;
}

=head2 AbortDownloadSWP($$)

=begin html
Abort download of OSWP with index $swpIndex using the opened TL1 session $session.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub AbortDownloadSWP($$) {
  my ($session, $swpIndex) = @_;
  my $response = SendCommand($session, sprintf("ed-oswp::oswp-%d::::ACTIVATION=,COMMITMENT=,ABORT=true,AUTOACTIVE=", $swpIndex));

  return defined($response) ? 1 : 0;
}

=head2 DownloadSWP($$$$$)

=begin html
Download OSWP and wait for completion.<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>swpIndex</td><td>OSWP index (1 or 2)</td></tr>
  <tr><td>oswpName</td><td>name of the OSWP</td></tr>
  <tr><td>primFtpIp</td><td>IP address of primary filetransfer server</td</tr>
  <tr><td>secFtpIp</td><td>IP address of primary filetransfer server</td</tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub DownloadSWP($$$$$) {
  my ($session, $swpIndex, $oswpName, $primFtpIp, $secFtpIp) = @_;
#  $oswpName =~ s/^\/// if ($oswpName =~ /^\//);
  my $response = SendCommand($session,
			     sprintf("dl-oswp:::::%s,%s,%s;", $primFtpIp, $secFtpIp, $oswpName));

  return 0 if (! defined($response));
  sleep 2;
  Transaction($session, "release");  # 1.2.5
  sleep 2;
  return WaitForDownload($session, $swpIndex);
}

#-- End of library
1;
