package Isam7354RUCli;

=head1 PBMT 7354RU ISAM CLI accessor module

=begin html
This PERL module contains the CLI accesses made to the ISAM 7354RU.

=end html

=cut

#==============================================================================
#-- File:         Isam7354RUCli.pm
#-- Date:         05.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Library with CLI commands for 7354RU that are different from
#--               7302
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/10/30   : Start First implementation
#--   1.0.1 - 2009/11/23   : ANTmt86148: 7354RU changes
#--   1.0.2 - 2016/07/15   : ALU02237966


=head1 VERSION

=begin html
Current version: "Isam7354RUCli.pm - 1.0.2 -- 2016/07/15"

=end html

=cut

my $VERSION = "Isam7354RUCli.pm - 1.0.2 -- 2016/07/15";
#==============================================================================

use strict;
use warnings 'all';

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN: <a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2101-DFZZA">Net::Telnet</a></li>
  <li>PBMT: <a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
</ul>

=end html

=cut

use vars qw ($tmpLogFile);

use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
require IsamCli;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub SelectFH($);
sub SetDebug($);
sub GetError();
sub ConfigureSWPDownload($$$);
sub WaitForDownload($$);
sub WaitForAbort($$);
sub WaitForCommit($$);
sub DownloadSWP($$$);
sub AbortDownloadSWP($$);
sub CommitSWP($$);
#==============================================================================
#== Variables
#==============================================================================

my $DEBUG = undef;

#==============================================================================
#== Constants
#==============================================================================
my $INVALID_TOKEN = "invalid token";
my $INTERNAL_ERROR = "Internal processing error";
my $ERR = "Error : ";
my $TELNET_CLOSED = "telnet session is closed/no longer opened";

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 SelectFH($)

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 SetDebug($)

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
#  print $DEBUG "Testing...\n";
}

my $ERROR = "";

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}

=head2 ConfigureSWPDownload($$$)

=begin html
Configure an OSWP for SW download.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>swpIndex</td><td>OSWP index (1 or 2)</td></tr>
  <tr><td>tftpserver</td><td>IP address of TFTP server</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub ConfigureSWPDownload($$$) {
  my( $telnet, $swpIndex, $tftpserver) = @_;
  my $cmd = "configure software-mngt oswp $swpIndex primary-file-server-id $tftpserver";
  my @response = IsamCli::SendCommand($telnet, $cmd);
  $ERROR = IsamCli::GetError();

  return 0 if (($ERROR =~ /command timed\-out/) || ($ERROR =~ /$TELNET_CLOSED/));
  foreach (@response) {
    if (/$ERR/ || /$INVALID_TOKEN/ || /$INTERNAL_ERROR/) {
      $ERROR = "$cmd\n" . join("", @response);
      return 0;
    }
  }
  return 1;
}

=head2 WaitForDownload($$)

=begin html
Wait for DB or SW download completion<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>showCmd</td><td>command section to deal with release differences</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForDownload($$) {
  my ($telnet, $showCmd) = @_;
  #-- check for completed download (or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $progress = undef;
  my $error;
  my $fileServer = "";
  my $errorFile = "";
  my $toReturn = undef;

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = IsamCli::RetrieveInfo($telnet, "show software-mngt $showCmd detail", undef);
      if (!defined($response) || $telnet->timed_out()) {
	if ($retries > 4) {
	  # 5 retries, seems that the first ones may yield no response...
	  $ERROR = "Maximum command retransmissions tried for 'show software-mngt $showCmd detail'";
	  $toReturn = 0;
	  return;
	}
	$retries++;
	sleep 2;
	next;
      }
      $retries = 0;
      $progress = undef;
      $fileServer = "";
      $errorFile = "";
      foreach (@{$response}) {
	if (/\s+download-progress\s:\s([\w-]+)/) {
	  $progress = $1;
	}
	if (/\s+availability\s:\s([\w-]+)/) {
	  $progress = $1;
	}
	if (/.*\s+download-error\s:\s([\w-]+)/) {
	  $error = $1
	}
	if (/\s+err-file-name\s:\s(.*)/) {
	  $errorFile = $1;
	  $errorFile =~ s/\s//g;
	}
	if (/\s+err-file-server\s:\s(.*)/) {
	  $fileServer = $1;
	  $fileServer =~ s/\s//g;
	}
	if (/^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/) {
	  # Alarm received, continue
	  $progress = "downloading";
	  $error = "no-error";
	}
      }
      ToolCommon::Log($DEBUG, sprintf("progress: %s, error: %s\n", defined($progress) ? $progress : "undef",
				      defined($error) ? $error : "undef")) if defined($DEBUG);
      if (defined($progress) && defined($error)) {
	if (($progress eq "download-success" || $progress eq "enabled") &&
	   $error eq "no-error") {
	  $toReturn = 1;
	  return 1;
	} elsif (($progress eq "download-ongoing" || $progress eq "downloading") && 
		 $error eq "no-error") {
	  sleep 2;
	} elsif (($progress eq "download-fail") || ($progress eq "disabled") || ($progress eq "empty")) {
	  if ($progress eq "disabled" || ($progress eq "empty")) {
	    $ERROR = "Download failed. Error: $error (failed file: '$errorFile', server: '$fileServer'), availability: $progress";
	  } else {
	    $ERROR = "Download failed. Error: $error, availability: $progress";
	  }
	  $toReturn = 0;
	  return 0;
	} else {
	  $ERROR = sprintf("Internal error: Unused progress value (%s).\nFull response: %s\n", $progress, join("", @{$response}));
	  $toReturn = 0;
	  return 0;
	}
      } else {
	$ERROR = sprintf("No progress information available.\nFull response: %s\n", join("", @{$response}));
	$toReturn = 0;
	return 0;
      }
    }
  };
  ToolCommon::Log($DEBUG, "eval block ended ($evalRet)\n") if defined($DEBUG);
  if ($gotSigPipe) {
    ToolCommon::Log($DEBUG, "Local sigpipe handler called\n") if defined($DEBUG);
    $ERROR = "SIGPIPE (WaitForDownload)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 WaitForAbort($$)

=begin html
Wait for OSWP abort to complete<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>showCmd</td><td>command section to deal with release differences</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForAbort($$) {
  my ($telnet, $showCmd) = @_;
  #-- check for completed download (or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $progress = undef;
  my $error;
  my $toReturn = undef;

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = IsamCli::RetrieveInfo($telnet, "show software-mngt $showCmd detail", undef);
      if (!defined($response) || $telnet->timed_out()) {
	if ($retries > 4) {
	  # 5 retries, seems that the first ones may yield no response...
	  $ERROR = "Maximum command retransmissions tried for 'show software-mngt $showCmd detail'";
	  $toReturn = 0;
	  return;
	}
	$retries++;
	sleep 2;
	next;
      }
      $retries = 0;
      $progress = undef;
      foreach (@{$response}) {
	if (/\s+availability\s:\s([\w-]+)/) {
	  $progress = $1;
	}
	if (/^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/) {
	  # Alarm received, continue
	  $progress = "downloading";
	  $error = "no-error";
	}
      }
      ToolCommon::Log($DEBUG, sprintf("progress: %s\n", defined($progress) ? $progress : "undef")) if defined($DEBUG);
      if (defined($progress)) {
	if ($progress eq "enabled") {
	  $toReturn = 1;
	  return 1;
	} elsif ($progress eq "aborting") {
	  sleep 2;
	} else {
	  $ERROR = sprintf("Internal error: Unused progress value (%s).\nFull response: %s\n", $progress, join("", @{$response}));
	  $toReturn = 0;
	  return 0;
	}
      } else {
	$ERROR = sprintf("No progress information available.\nFull response: %s\n", join("", @{$response}));
	$toReturn = 0;
	return 0;
      }
    }
  };
  ToolCommon::Log($DEBUG, "eval block ended ($evalRet)\n") if defined($DEBUG);
  if ($gotSigPipe) {
    ToolCommon::Log($DEBUG, "Local sigpipe handler called\n") if defined($DEBUG);
    $ERROR = "SIGPIPE (WaitForAbotr)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 WaitForCommit($$)

=begin html
Wait for OSWP commit to complete<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>showCmd</td><td>command section to deal with release differences</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForCommit($$) {
  my ($telnet, $showCmd) = @_;
  #-- check for completed download (or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $progress = undef;
  my $error;
  my $toReturn = undef;

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = IsamCli::RetrieveInfo($telnet, "show software-mngt $showCmd detail", undef);
      if (!defined($response) || $telnet->timed_out()) {
	if ($retries > 4) {
	  # 5 retries, seems that the first ones may yield no response...
	  $ERROR = "Maximum command retransmissions tried for 'show software-mngt $showCmd detail'";
	  $toReturn = 0;
	  return;
	}
	$retries++;
	sleep 2;
	next;
      }
      $retries = 0;
      $progress = undef;
      foreach (@{$response}) {
	if (/\s+commit-status\s:\s([\w-]+)/) {
	  $progress = $1;
	}
	if (/^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/) {
	  # Alarm received, continue
	  $progress = "committing";
	  $error = "no-error";
	}
      }
      ToolCommon::Log($DEBUG, sprintf("progress: %s\n", defined($progress) ? $progress : "undef")) if defined($DEBUG);
      if (defined($progress)) {
	if ($progress eq "committed") {
	  $toReturn = 1;
	  return 1;
	} elsif ($progress eq "committing") {
	  sleep 2;
	} else {
	  $ERROR = sprintf("Internal error: Unused progress value (%s).\nFull response: %s\n", $progress, join("", @{$response}));
	  $toReturn = 0;
	  return 0;
	}
      } else {
	$ERROR = sprintf("No progress information available.\nFull response: %s\n", join("", @{$response}));
	$toReturn = 0;
	return 0;
      }
    }
  };
  ToolCommon::Log($DEBUG, "eval block ended ($evalRet)\n") if defined($DEBUG);
  if ($gotSigPipe) {
    ToolCommon::Log($DEBUG, "Local sigpipe handler called\n") if defined($DEBUG);
    $ERROR = "SIGPIPE (WaitForCommit)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 DownloadSWP($$$)

=begin html
Download OSWP and wait for completion.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>swpIndex</td><td>OSWP index (1 or 2)</td></tr>
  <tr><td>oswpName</td><td>name of the OSWP</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub DownloadSWP($$$) {
  my ($telnet, $swpIndex, $oswpName) = @_;
#  $oswpName =~ s/^\/// if ($oswpName =~ /^\//);
  my @response = IsamCli::SendCommand($telnet, 
				      "admin software-mngt oswp $swpIndex download $oswpName");

  return 0 if (($ERROR =~ /command timed\-out/) || ($ERROR =~ /$TELNET_CLOSED/));
  foreach (@response) {
    if (/$ERR/ || /$INVALID_TOKEN/ || /$INTERNAL_ERROR/) {
      $ERROR = "admin software-mngt oswp $swpIndex download $oswpName\n" . join("", @response);
      return 0;
    }
  }
  sleep 2;
  IsamCli::Transaction($telnet, "release");  # 1.2.5
  sleep 2;
  return WaitForDownload($telnet, "oswp $swpIndex");
}

=head2 AbortDownloadSWP($$)

=begin html
Abort download of OSWP with index $swpIndex using the opened cli session $telnet.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub AbortDownloadSWP($$) {
  my ($telnet, $swpIndex) = @_;
  my @response = IsamCli::SendCommand($telnet, "admin software-mngt oswp $swpIndex abort-download");

  return 0 if (($ERROR =~ /command timed\-out/) || ($ERROR =~ /$TELNET_CLOSED/));
  foreach (@response) {
    if (/$ERR/ || /$INVALID_TOKEN/ || /$INTERNAL_ERROR/) {
      $ERROR = "admin software-mngt oswp $swpIndex abort-download\n" . join("", @response);
      return 0;
    }
  }
  return WaitForAbort($telnet, "oswp $swpIndex");
}

=head2 CommitSWP($$)

=begin html
Commit OSWP with index $swpIndex using the opened cli session $telnet.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub CommitSWP($$) {
  my ($telnet, $swpIndex) = @_;
  my @response = IsamCli::SendCommand($telnet, "admin software-mngt oswp $swpIndex commit");

  return 0 if (($ERROR =~ /command timed\-out/) || ($ERROR =~ /$TELNET_CLOSED/));
  foreach (@response) {
    if (/$ERR/ || /$INVALID_TOKEN/ || /$INTERNAL_ERROR/) {
      $ERROR = "admin software-mngt oswp $swpIndex commit\n" . join("", @response);
      return 0;
    }
  }
  return WaitForCommit($telnet, "oswp $swpIndex");
}


#-- End of library
1;
