package IsamCli;

=head1 PBMT ISAM CLI accessor module

=begin html
This PERL module contains the CLI accesses made to the ISAM.

=end html

=cut

#==============================================================================
#-- File:         IsamCli.pm
#-- Date:         05.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Donny Janssens
#--               Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Library with ISAM CLI commands
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2005/11/05   : Start First implementation
#--   1.0.1 - 2005/11/13   : Move trace file to /tmp/telnet_$ip.trace
#--   1.0.2 - 2005/11/18   : Add function for shub database save
#--   1.0.3 - 2005/12/14   : Added flush of telnet buffers (to clear alarm junk)
#--   1.0.4 - 2005/12/16   : Prepared for R2.4
#--   1.0.5 - 2006/02/07   : - Multiple NT boards.
#--                          - Migration from R2.0/R2.1: Get LANX active config
#--   1.0.6 - 2006/03/13   : Use admin software-mngt shub sw-package in stead of
#--                          admin software-mngt shub copy
#--   1.2.0 - 2006/03/16   : NT capability model
#--   1.2.4.1 - 2006/07/25 : ANTms06883 - remove CLI logfile frmo /tmp on exit
#--                          of emergency exit
#--   1.2.5   - 2006/08/21 : ANTms10579 - CLI output of 'show equipment slot'
#--                          has changed.
#--                          ANTms10531 - Release transaction after start of SW DL
#--   1.2.5.1 - 2006/09/25 : CLI type based position numbering
#--             2006/10/02 : - Additional robustness on info retrieval commands.
#--                          - turn off the CLI 'more' prompting mode.
#--   1.2.5.3 - 2006/10/06 : print incomplete response in case no trailing '==..==='
#--                          is found.
#--   1.4.3   - 2006/10/18 : Timestamp logging
#--   1.4.3.4 - 2006/11/13 : ANTms22321 - Transaction timeour values in units of msecs.
#--   1.5.0   - 2006/12/15 : Added function to download either dm_complete.tar or dm.tar
#--   1.5.1   - 2006/12/19 : transaction timeout value can be a word (indefinite).
#--   1.5.1.1 - 2007/01/10 : Remove spinning wheel chars from response if present
#--                          (after the facts FR: ANTms28676)
#--   1.5.1.2 - 2007/01/22 : ANTms30899: WaitForDownload changed:
#--                          - reset retry count with every show oswp command
#--                          - up to 5 retries
#--                          - include sleep 2 before retrying
#--   1.5.1.3 - 2007/01/26 : Report availability state after SW DL failure.
#--   1.5.1.4 - 2007/02/22 : ANTms31773 - add file-error and file-server info
#--                          in case of DL failure.
#--   1.6.1   - 2007/02/28 : ANTms35659 - Added code to handel SFP provisioning when
#--                          migrating from R 2.4 or 2.5 to R 3.3 only
#--                          [Implemented by Haq Furqan]
#--   1.6.2   - 2007/03/15 : ANTmsnnnnn - FD may not have an nt-b position
#--   1.6.3   - 2007/04/25 : ANTms44293 - Include LT status information in GetLtConfig
#--   1.6.4   - 2007/05/04 : ANTms45601 - SFP configuration after migration should be type-based
#--   1.6.5   - 2007/05/29 : ANTms48813 - remove leading '/' in from sw mgnt commands involving tftp
#--   1.7.0   - 2007/05/03 : ANTms49896 - Forward TTL=0 packets for a VRF
#--   1.7.1   - 2007/07/04 : ANTms45205 - Error in applique retrieval.
#--                                       Use direction info of "info configure equipment external-link-assign"
#--   1.7.2   - 2007/08/10 : End wait for download if show software-mgnt does not return any info at all.
#--   1.7.3   - 2007/08/23 : ANTms60501 - output of info configure system transaction has changed:
#--                                       the timout value is no longer returned in case of a default value.
#--                                       Changed to info configure system transaction detail.
#--           - 2007/08/21 : Parse OSWP info in GetOswpInfo (in R3.4 alarms are appended to the output
#--                          resulting in the fact that the command is not ending on '=====...==='
#--   1.7.4   - 2007/09/03 : ANTms62398 - Remove event messages from all CLI outputs.
#--                          ANTms62454: When the node is up too soon, the workaround
#--                          is triggered but the Telnet session is closed.  We now
#--                          specifically check on this (added ConnectionReset().
#--   1.7.5   - 2007/09/05 : ANTms62648: Workaround not OK for redundant systems.
#--   1.7.6   - 2008/01/10 : ANTms63550: Added a new function to configure access operatorId through cli
#--   1.7.7   - 2008/02/08 : ANTms83783: Check whether both NT and SHUB are running the new SW
#--   1.7.8   - 2008/03/10 : ANTms87133: WaitForDownload stops after 5 timeouts (due to permananent
#--                                      link failure.
#--   1.7.9   - 2008/04/16 : Added checking on opened telnet session.
#--   1.7.10  - 2008/04/22 : Slight modification vsd 1.7.9
#--   1.7.11  - 2008/05/29 : ANTmt09138.  LT numbering starts from 1 in type-based
#--                          operator profiles.  Incorrect LT number is used to calculate
#--                          an ifIndex.
#--   1.7.12  - 2008/06/14 : Add Telnet debugging
#--   1.7.13  - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--             2008/07/25 : Add check on file transfer protocol being used in the ISAM.
#--   1.7.14  - 2008/10/01 : Workaround for ANTmt25573.
#--   1.7.15  - 2008/10/21 : ANTmt28533  Include workaround for a prompt without a prompt terminator.
#--                          In some case, the complete prompt becomes too long resulting in the prompt
#--                          terminator to be chopped off.  The RE search fails and logging in fails.
#--                          CHeck for a timeout in such a case and change the prompt to default.
#--   1.7.16  - 2009/03/06 : Check on $DEBUG defined missing in ConnectSSH
#--   1.7.17  - 2009/04/20 : ANTmt54050. For Legacy systems corrected retrieval of LT slot information. Prevent the retrieval
#--                          of Special board information.
#--   1.7.18  - 2009/04/22 : Cope with an empty reponse when retrieving DB upload progress and try a couple of times before giving up.
#--                          ANTmt52885: print $cmd as '%s' rather then using $cmd since %-directives may get converted (unintentionally
#--                          as in the prompt settingdirectives for example).
#--   1.7.19  - 2009/04/30 : not-allowed was returned as planned-type.  Turns out to be a problem in configdata.xml but decided to keep this
#--                          robustness improvement.
#--   1.7.20  - 2009/05/06 : ANTmt54684.  access-operator-id must be access-oper-id.  Added GetAccessOperId function.
#--   1.7.21  - 2009/05/19 : ANTmt59034.  Set timeout on Telnet-session for SSH
#--   1.7.22  - 2009/05/20 : Errmode return for SSH.
#--   1.7.23  - 2009/06/22 : Use same login algorithm for telnet as for SSH.
#--   1.7.24  - 2009/07/16 : ANTmt68267 - Serial number info is sometimes enclosed between double quotes and sometimes it is not.
#--   1.7.25  - 2009/08/04 : Error when determining CLI profile mode.  Mode gets fixed on legacy-mode
#--   1.7.26  - 2009/08/04 : Retrieve shelf configuration
#--   1.7.27  - 2009/08/24 : Clear $ERROR in SendCommand.
#--   1.7.28  - 2009/08/24 : ANTmt72875: LANX SW version for ANSI stream has changed.
#--   1.7.29  - 2009/09/10 : ANTmt75645 - reworked connection management.
#--   1.7.30  - 2009/10/08 : SSH 'telnet sessions' do not have a host associated.  Consequently connection contexts can be retrieved in case of
#--                          SSH.
#--   1.7.31  - 2009/10/26 : Catch 'key change' condition when logging in using SSH to produce a better error message.
#--   1.7.32  - 2009/10/30 : ANTmt82134: Return legacy-mode as CLI numbering in case the release does not support the feature.
#--   1.7.33  - 2009/12/01 : ANTmt85862
#--   1.7.34  - 2010/03/08 : PBMT x86 Changes - ANTmt88407
#--   1.7.35  - 2010/04/02 : ALU00194284 - Intermittant telnet session establishment problem
#--   1.7.36  - 2010/05/20 : ALU00215292 - Maximum number of sessions reached during CLI session establishment
#--   3.0.00  - 2010/06/06 : Version changed for Converged PBMT
#--   3.0.01  - 2010/07/06 : Converged PBMT Changes
#--   3.0.02  - 2010/08/04 : ALU00735537 - PBMT tries to tftp OSWP file although it exists for another board
#--   3.0.03  - 2010/10/18 : ALU00820178 - NANT-D Redundancy, migration does not happen
#--   3.0.04  - 2010/10/27 : ALU00837446 - download Script not working in R4.2 For ERAM-A 
#--   3.0.05  - 2010/11/03 : NT Synchronization timeout (NT_SYNC_TIMEOUT) set to 600sec 
#--   3.0.06  - 2011/01/12 : LRM updates for lib-perl path
#--   3.0.07  - 2011/02/09 : ALU01033842 
#--   3.0.08  - 2011/03/27 : Linux compatibility changes
#--   3.0.09  - 2011/04/15 : ALU01185809
#--   3.0.10  - 2011/05/17 : ALU01152509
#--   3.0.11  - 2011/05/24 : ALU01178887
#--   3.0.12  - 2011/06/16 : Set U2UHairpin command introduction
#--   3.0.13  - 2011/06/24 : Modification for subroutine SetU2UHairpin
#--   3.0.14  - 2011/11/14 : ALU01605960
#--   3.0.15  - 2011/12/12 : parsing done for LT boards with suffix LS viz. NALS-x 
#--   3.0.16  - 2012/03/29 : Telnet prompt pattern changed for 7354RU to resolve SSH failure 
#--   3.0.17  - 2013/02/15 : consider "Permission denied" response during SSH connection establishment as a login failure
#--   3.0.18  - 2013/04/26 : ALU01871009, ALU01880147
#--   3.0.19  - 2013/06/14 : ALU01877212
#--   3.0.20  - 2013/09/26 : ALU01893590
#--   3.0.21  - 2013/10/17 : ALU01915648
#--   3.0.22  - 2013/10/24 : ALU01913513
#--   3.0.23  - 2014/07/15 : ALU01980121
#--   3.0.24  - 2014/08/19 : ALU01997836
#--   3.0.25  - 2014/10/20 : ALU02007896
#--   3.0.26  - 2014/11/04 : ALU02020212
#--   3.0.27  - 2014/01/27 : ALU02042831: SW Reduce not displaying proper error when Control card waiting for SW
#--   3.0.28  - 2015/02/13 : ALU02026412 - Autoback configuration check for Auto backup progress
#--   3.0.29  - 2015/03/13 : ALU02053783
#--   3.0.30  - 2015/04/23 : ALU02071478
#--   3.0.31  - 2015/04/29 : ALU02069878
#--   3.0.32  - 2015/05/04 : ALU02064630
#--   3.0.33  - 2015/05/12 : ALU02070858
#--   3.0.34  - 2015/06/05 : ALU02083844
#--   3.0.35  - 2015/06/25 : ALU02095460
#--   3.0.36  - 2015/07/01 : ALU02093886
#--   3.0.37  - 2015/07/29 : ALU02091141, ALU02106765
#--   3.0.38  - 2015/08/10 : ALU02109766, ALU02112322
#--   3.0.39  - 2015/11/02 : ALU02142016
#--   3.0.40  - 2015/11/13 : ALU02145791
#--   3.0.41  - 2015/11/27 : ALU02145791
#--   3.0.42  - 2015/12/14 : ALU02156544
#--   3.0.43  - 2016/01/18 : ALU02166695 - PBMT v3.2.61
#--   3.0.44  - 2016/01/28 : ALU02171179
#--   3.0.45  - 2016/02/02 : ALU02168421, ALU02167196 
#--   3.0.46  - 2016/02/09 : ALU02174208
#--   3.0.47  - 2016/02/28 : ALU02182678
#--   3.0.48  - 2016/03/16 : ALU02190562, ALU02190558
#--   3.0.49  - 2016/04/22 : ALU02196118
#--   3.0.50  - 2016/06/13 : ALU02231273
#--   3.0.51  - 2016/06/14 : ALU02224709
#--   3.0.52  - 2016/06/24 : ALU02237966
#--   3.0.53  - 2016/07/27 : ALU02240675
#--   3.0.54  - 2016/09/28 : ALU02278259
#--   3.0.55  - 2017/09/27 : to resolve some warning
#--   3.0.56  - 2017/10/24 : ALU02400566 
#--   3.0.57  - 2018/02/12 : Adding e1-sip-termination to emergency call alarm list 
#--   3.0.58  - 2018/02/27 : Adding secondary-sip-term to emergency call alarm list and ALU02444650
#--   3.1.01  - 2018/06/13 : Logger Functionality and Common ErrorCode
#--   3.2.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.2.1  - 2018/09/10 :  ONT Migration via CLI
#--   3.2.2  - 2018/11/19 :  ALU02510303
#--   3.2.3  - 2018/12/03 :  checks added for ONT ctrl table retrieval 
#--   3.2.4  - 2018/12/05 :  solve Stageont issue related to download of all builds from map file
#--   3.2.5  - 2019/02/28 :  ALU02552653

=head1 VERSION

=begin html
Current version: "IsamCli.pm - 3.2.5 -- 2019/02/28"

=end html

=cut

my $VERSION = "IsamCli.pm - 3.2.5 -- 2019/02/28";
#==============================================================================

use strict;
use warnings 'all';
use File::Basename;
use Data::Dumper;
use Cwd;

BEGIN {
  use Config;
  my $dir = dirname($0);
  my $path;
  my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }
  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif( $dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }

  unshift(@INC, "$path/lib-perl/$perlLibLocation");
  if ($Config{archname} =~ /sun4/) {
  		unshift(@INC, "$path/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
  		unshift(@INC, "$path/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
  		unshift(@INC, "$path/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }

  use IO::Pty ();

}

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN: <a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2101-DFZZA">Net::Telnet</a></li>
  <li>PBMT: <a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
</ul>

=end html

=cut

use vars qw ($tmpLogFile %isamConnectionList);

END {
  unlink($tmpLogFile) if (defined($tmpLogFile) && ($tmpLogFile ne ""));
  $tmpLogFile = "";
  foreach my $session (keys %isamConnectionList) {
    cleanConnection($session);
  }
}
#-- 1.2.4.1 -- END

use Net::Telnet;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:COMMON);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub SelectFH($);
sub SetDebug($);
sub GetError();
sub Spawn(@);
sub ConnectSSH($$;$);
sub Connect($;$$$);
sub TelnetNoDataRead($);
sub TelnetConnectionLost($);
sub Disconnect(;$);
sub SendCommand($$;$);
sub RetrieveInfo($$;$);
sub CommandError($);
sub GetNtExtendedInfo($;$$);
sub GetNtInfo($);
sub GetNtIcs($$);
sub GetPEState($);
sub GetOswpInfo($$);
sub GetDbVersionInfo($);
sub GetLtConfig($);
sub GetSlotConfig($);
sub GetLtConfigState($);
sub GetShelfConfig($);
sub GetLanxActiveOSWP($$);
sub UploadDb($$$$$);
sub CommitSWP($$);
sub AbortDownloadSWP($$);
sub ConfigureSWPDownload($$$$);
sub ConfigureSWPDownloadVerify($$$);
sub WaitForDownload($$);
sub DownloadCompleteDb($$$);
sub DownloadDb($$$$);
sub DownloadSWP($$$);
sub WaitForShubCopy($$$);
sub GetLanxDbFile($$$$);
sub ChangeShubConfig($$);
sub RebootShub($);
sub DownloadShubSW($$$$);
sub GetLanxSwVersion($);
sub Transaction($$);
sub ShubDbaseSave($);
sub InhibitAlarms($);
sub AllowAlarms($);
sub GetConfig($$);
sub GetConfigFile($$$);
sub DeleteCmdFile($$);
sub SetSessionTimeout($$);
sub GetShelfType($);
sub InhibitMore($);
sub SetTransactionTimeout($$);
sub GetTransactionTimeout($);
sub GetSfpAssignment($);
sub ConfigureSfpAssignment($$);
sub PrintSfpAssignmentTable($);
sub GetSfpConfig($);
sub ConfigureSfp($$);
sub PrintSfpConfig($);
sub ForwardTtl0Packets($$);
sub ConfigureAccessOprId($$);
sub GetAccessOperId($);
sub NtAppliqueInfo($);
sub ConnectionReset();
sub ConnectionStillOpen($);
sub EmptyCliResponse();
sub GetProfileSlotNumberMode($$);
sub GetFileTransferSettings($);
sub ReadDiskSpace($);
sub ListSwpFiles($$);
sub ListSwpDescFiles($$);
sub ListCliSessions($);
sub WaitForNTSync($;$); 
sub GetVlanUniqHostId($); 
sub GetAutoBkpConfig($); 
sub GetSecureDebugSetting($);
sub ConnectSecDebugViaSSH($$;$);
sub SetCliConnection($);
sub GetCliConnection();
sub reEstablishConnection($);
sub cleanConnection($);
sub AlarmStatus($$);
sub ResetGlobals();
sub GetONTSwVer($);
sub deleteONTSw($$);

#==============================================================================
#== Variables
#==============================================================================
my $DEBUG = undef;
my $tmpLogFile = "";
my $ERROR = "";
my $responseEmpty = 0;
my %isamConnectionList = ();
my $MAXEMPTYRESP = 10;
my $NT_SYNC_TIMEOUT = 600;    ## default sync time for NT set to 600 sec
my $slotTypeNT = ""; ## if there is only one NT supported.
our $neId = ""; ## NE's ID
 
#==============================================================================
#== Constants
#==============================================================================
my $CONN_RESET = "Connection reset by peer|Broken pipe|Connection reset";
my $FILE_CLOSED = "write error: filehandle isn\'t open";
my $INVALID_TOKEN = "invalid token";
my $INTERNAL_ERROR = "Internal processing error";
my $TELNET_CLOSED = "telnet session is closed/no longer opened";
my $TIMED_OUT = "command timed\-out";
my $GENERAL_ERROR = "General error";
my $GPONMGNT_ERROR = "GPON MGT error";
my $ERR = "^Error : ";
my $DOWNLINK = "remote-lt";
#-----  SSH login phases
my $INIT = 0;
my $KEYFP = 1;
my $PASSWD = 2;
my $PASSPHRASE = 3;
my $LOGGEDIN = 4;
my $LOGINERROR = 5;

my %sshLoginPhases = (0 => "Init",
                      1 => "Key fingerprint",
		      2 => "Password",
		      3 => "Passphrase",
		      4 => "Logged in",
		      5 => "Error");
 
#------ SSH login status
my $NOTLOGGEDIN = 0;
my $LOGINOK = 1;
my $LOGINFAIL = 2;

my $CLISession     = undef;
my $SESSION_SSH    = "SSH";
my $SESSION_DEBUG  = "SSH-DEBUG";
my $SESSION_TELNET = "TELNET";

our @emergencyCallList = ( "e1-sip-termination",
 						  "secondary-sip-term", 
						  "sip-termination",
						  "ont",
						  "sfu-sup",
						  "soho-sup",
						  "mdu-sup");



=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 SetDebug()

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
#  print $DEBUG "Testing...\n";
}

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}


sub ResetGlobals() {
   $slotTypeNT = "";
}

=head2 Spawn(@)

=begin html
Spawn a program.  The function returns the pty associated to the spawned program or undef in case of failure.

=end html

=cut

sub Spawn(@) {
  my(@cmd) = @_;
  my($pid, $pty, $tty, $tty_fd);

  $pty = undef;
  ## Create a new pseudo terminal.
  use IO::Pty ();
  $^W = 0;
  $pty = new IO::Pty;
  if ($pty) {
    ## Execute the program in another process.
    $pid = fork();
    if (defined($pid)) {
      if (! $pid) {
	# In child process
	## Disassociate process from existing controlling terminal.
	use POSIX ();
	if (POSIX::setsid) {
	  ## Associate process with a new controlling terminal.
	  $tty = $pty->slave;
	  $pty->make_slave_controlling_terminal();
	  $tty_fd = $tty->fileno;
	  close $pty;
	  ## Make stdio use the new controlling terminal.
	  if (open STDIN, "<&$tty_fd") {
	    if (open STDOUT, ">&$tty_fd") {
	      if (open STDERR, ">&STDOUT") {
		close $tty;
		## Execute requested program.
		if (! exec @cmd) {
		  $ERROR = "Spawn failure: problem executing $cmd[0]: $!";
		  $pty = undef;
		}
	      } else {
		$ERROR = "Spawn failure: Redirecting STDERR failed: $!";
		$pty = undef;
	      }
	    } else {
	      $ERROR = "Spawn failure: Redirecting STDOUT failed: $!";
	      $pty = undef;
	    }
	  } else {
	    $ERROR = "Spawn failure: Redirecting STDIN failed: $!";
	    $pty = undef;
	  }
	} else {
	  $ERROR = "Spawn failure: setsid failed: $!";
	}
      } else {
	# In parent.
	ToolCommon::Log($DEBUG, "Spawned '@cmd' as $pid\n") if defined($DEBUG);
      }
    } else {
      $ERROR = "Spawn failure: failed to create child process";
      $pty = undef;
    }
  } else {
    $ERROR = "Spawn failure: Allocating pty failed: $!";
  }
  $^W = 1;
  return ($pty, $tty, $pid);
}

=head2 HandleLoginInit($$$$)

=begin html
During init following can be received:
<ul>
  <li>'Connection closed': entering the ISAM using SSH is refused</li>
  <li>Prompt for login:<br>
      This happens when logging in using telnet and a prompt for a username is received.</li>
  <li>Prompt for key fingerprint:<br>
      This happens when the node is not in the known hosts file.  We will have to check the received fingerprint before going ay further.</li>
  <li>Prompt for a password:<br>
      This happens when the node is in the known hosts file and the ISAM accepts the SSH session by prompting for the password of the
      username that was passed through.  Happens with SSH using username and password.</li>
  <li>Prompt for a new password:<br>
      This happens when the password is expired.  Login fails.</li>
  <li>Prompt for a passphrase:<br>
      This happens when the node is in the known hosts file and the configuration has been setup to use key files and the private keyfile
      is protected by a passphrase.</li>
  <li>A regular prompt:<br>
      This happens when using keyfiles and no passphrase needs to be entered.  We are now logged on.</li>
  <li>"last login" line:<br>
      This happens when using keyfiles and no passphrase needs to be entered but the prompt is so long that we could not determine the
      end of the prompt.  We are now logged on.</li>
</ul>

=end html

=cut

sub HandleLoginInit($$$$) {
  my ($isamCliSession, $isamContent, $thisPhase, $loginConfirmKey) = @_;
  my $nextPhase = $thisPhase;
  my $loggedIn = $NOTLOGGEDIN;
  $ERROR = "";

  if( ($isamContent =~ /Connection (closed|refused)/) ||
     ($isamContent =~ /Connection to/ && $isamContent =~ /closed/) ) {
      $ERROR = "Connection refused/closed, please check configuration settings";
  } elsif ($isamContent =~ /key verification failed/) {
      $ERROR = "Connection refused due to host key change, please check configuration settings";
  } elsif ($isamContent =~ /can\'t be established/ and ($isamContent =~ /key fingerprint is/) ) {
      my $hostAutoUpdate = (defined $params->{UPD_KNOWN_HOST_FILE} and $params->{UPD_KNOWN_HOST_FILE}) ? 1 : 0;
	  if ($hostAutoUpdate) {
         Logger::LogNotify( $nodeLogFH, "Known hosts file automatic update is enabled\n");
	     $isamCliSession->print("yes\n");
	     sleep(2);
	  } else {     
	     $ERROR = "Received unexpected key fingerprint, Host file need to update";
	  }
  } elsif ($isamContent =~ /enter new password: $/) {
      # Check this one priotr to the 'normal' password prompt!!
      $ERROR = "Password has expired, please set password first";
  } elsif ($isamContent =~ /login:\s$/) {
      my $userName = defined $params->{USERNAME} ? $params->{USERNAME} : undef;
      if (defined($userName)) {
        Logger::LogDebug( $nodeLogFH, "Sending username...\n");
        $isamCliSession->print("$userName");
        $nextPhase = $PASSWD;
      } else {
        $ERROR = "Username required but none specified";
      }
  } elsif ($isamContent =~  /password: $/) {
      my $passwd = defined $params->{PASSWORD} ? $params->{PASSWORD} : undef;
      if (defined($passwd)) {
         Logger::LogDebug( $nodeLogFH, "Sending password...\n");
         $isamCliSession->print("$passwd\n");
         $nextPhase = $PASSWD;
      } else {
         $ERROR = "Password required but none specified";
      }
  } elsif ($isamContent =~ /Enter passphrase/) {
     my $passwd = defined $params->{PASSWORD} ? $params->{PASSWORD} : undef;
     if (defined($passwd)) {
        Logger::LogDebug( $nodeLogFH, "Sending password as passphrase...\n");
        $isamCliSession->print("$passwd\n");
        $nextPhase = $PASSPHRASE;
     } else {
       $ERROR = "Passphrase required but none specified";
     }
  } elsif ($isamContent =~ /\n$loginConfirmKey\n/ ){
      $loggedIn = $LOGINOK;
      $nextPhase = $LOGGEDIN;
      Logger::LogDebug( $nodeLogFH, "Confirmed, Login successful\n");
  } elsif ($isamContent =~ /last login : |[\$%\#\>]\s*$/) {
     ## We've received a prompt/last-login, so we're successfully logged in using keyfiles
     my $found_type = ( $& eq "last login :" ) ? "last-login" : "prompt";
     Logger::LogDebug( $nodeLogFH, "Found $found_type, Yet to Confirm\n" );
     $isamCliSession->print("echo $loginConfirmKey\n"); 
     sleep(2);
  } else {
    # ignore and continue
  }
  if( $ERROR ne "" ) {
      Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
      $loggedIn = $LOGINFAIL;
      $nextPhase = $LOGINERROR;
  }
  return($loggedIn, $nextPhase);
}

=head2 HandleLoginPassCode($$$$;$)

=begin html
The SSH session was provided with either a password or a passphrase and we should receive the system prompt now.
In error situations a password prompt may be received: the NT has a key file but for some reason it is gone on the server
from where we try to logon

=end html

=cut

sub HandleLoginPassCode($$$$) {
  my ($isamCliSession, $isamContent, $thisPhase,$loginConfirmKey) = @_;
  my $nextPhase = $thisPhase;
  my $loggedIn = $NOTLOGGEDIN;
  my $passPhraseFailed = 0;
  $ERROR = "";

  if ( $isamContent =~ /Login incorrect|Permission denied/ ) {
       $ERROR = "Login failed: incorrect username or password specified";
  } elsif ($isamContent =~   /Max\. Sessions Reached/) {
       $ERROR = "Connection refused by remote host. Maximum Session Limit Reached";
  } elsif (($isamContent =~ /Connection to/) && ($isamContent =~ /closed/)) {
    if ($isamContent =~ /closed by remote/) {
      $ERROR = "Connection refused by remote host.";
    } else {
      if (! $passPhraseFailed) {
	      $ERROR = sprintf("Connection refused, invalid %s provided", $nextPhase == $PASSWD ? "password" : "passphrase");
      } else {
	      $ERROR = "Connection refused, invalid passphrase provided, tried as password but also failed";
      }
    }
  } elsif ($isamContent =~ /\n$loginConfirmKey\n/ ){
    $nextPhase = $LOGGEDIN;
    $loggedIn = $LOGINOK;
    Logger::LogDebug( $nodeLogFH, "Confirmed, Login successful\n");
  } elsif ($isamContent =~ /last login : |[\$%\#\>]\s*$/) {
     ## We've received a prompt/last-login, so we're successfully logged in using keyfiles
     my $found_type = ( $& eq "last login :" ) ? "last-login" : "prompt";
     Logger::LogDebug( $nodeLogFH, "Found $found_type, Yet to Confirm\n" );
     $isamCliSession->print("echo $loginConfirmKey\n"); 
     sleep(2);
  } elsif ( $isamContent =~ /major alarm occurred.*Successful Management login/ ) {
       ### After successful login only, We will receive alarms from ISAM.
       ### This has considered to avoid when login prompt is totally difference not ending with '$%>#' and 
       ### wait till timeout reaches to proceed
       $nextPhase = $LOGGEDIN;
       $loggedIn = $LOGINOK;
       Logger::LogDebug( $nodeLogFH, "Confirmed, Login alarm received\n");
  } elsif ($isamContent =~ /password: $/) {
    my $passwd = defined $params->{PASSWORD} ? $params->{PASSWORD} : undef;
    if (defined($passwd)) {
      Logger::LogDebug( $nodeLogFH, "Sending password...\n");
      $isamCliSession->print("$passwd\n");
      $passPhraseFailed = 1 if ($nextPhase == $PASSPHRASE);
      $nextPhase = $PASSWD;
    } else {
      $ERROR = "Password required but none specified";
    }
  } elsif ($isamContent =~ /Enter passphrase/) {
    my $passwd = defined $params->{PASSWORD} ? $params->{PASSWORD} : undef;
    if (defined($passwd)) {
      Logger::LogDebug( $nodeLogFH, "Sending password as passphrase...\n");
      $isamCliSession->print("$passwd\n");
      $nextPhase = $PASSPHRASE;
    } else {
      $ERROR = "Passphrase required but none specified";
    }
  } else {
  }
  if( $ERROR ne "" ) {
      Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
      $loggedIn = $LOGINFAIL;
      $nextPhase = $LOGINERROR;
  }
  return($loggedIn, $nextPhase);
}


sub DoKnownHostCheck() {
    my $hostAutoUpd = (defined $params->{UPD_KNOWN_HOST_FILE} and $params->{UPD_KNOWN_HOST_FILE}) ? 1 : 0;
    my $homeDir = $ENV{HOME};
    my ($systemKnownHosts, $userKnownHosts ) = ( "", "" );
    my ($systemReturnCode, $systemError, $userReturnCode, $userError) = ( 0, "", 0, "" );
    if (defined($params->{LOC_KNOWNHOSTS})) {
        $userKnownHosts = $params->{LOC_KNOWNHOSTS};
        if ($userKnownHosts !~ m/^[\/]/) {
            $userKnownHosts = "$homeDir/$userKnownHosts";
        }
        Logger::LogDebug( $nodeLogFH, "KnownHostCheck of user specific known hosts file\n");
        ($userReturnCode, $userError) = ToolCommon::KnownHostCheck($hostAutoUpd, $userKnownHosts);
    }
    if ( $userReturnCode and defined($params->{SYS_KNOWNHOSTS})) {
        $systemKnownHosts = $params->{SYS_KNOWNHOSTS};
        if ($systemKnownHosts !~ m/^[\/]/) {
            $systemKnownHosts = "$homeDir/$systemKnownHosts";
        }
        Logger::LogDebug( $nodeLogFH, "KnownHostCheck of system specific known hosts file\n");
        ($systemReturnCode, $systemError) = ToolCommon::KnownHostCheck($hostAutoUpd, $systemKnownHosts);
    }
    if( $systemReturnCode ) {
        $ERROR = ( $userReturnCode ) ? "Issue in user specific known hosts : $userError " :
                                       "Issue in system specific known hosts : $systemError ";
        return 1;
                               
    }
    return 0;
}

################################################################################
### Check we have ISAM session already with us
################################################################################
sub IsSessionExists($$) {
    my( $sessionType,$ipAddress ) = @_;
    my ($host, $type, $session) = ("", "", undef );
    foreach $session (sort keys %isamConnectionList) {
        $host = (defined $isamConnectionList{$session}{HOST}) ? $isamConnectionList{$session}{HOST} : "";
        $type = (defined $isamConnectionList{$session}{TYPE}) ? $isamConnectionList{$session}{TYPE} : "";
        Logger::LogDebug( $nodeLogFH, "isamConnection session:$session:, host:$host:, type:$type:'\n");
        if( ($host eq $ipAddress) and ($type eq $sessionType) ) {
            $ERROR = "Already we hold an ISAM CLI-$sessionType Session/Connection";
            return (1, $session);
        }
    }
    return (0,$session);;
}


sub GetSSHSession($$$$) {
    my( $sessionType, $ipAddress, $timeout, $logFile ) = ( @_ );
    my $homeDir = $ENV{HOME};
    my $config = FamilyCommon::GetConfigData();
    my ($systemKnownHosts, $userKnownHosts ) = ( "", "" );
    my $inheritance = Net::Telnet::whatamI(undef);
    foreach (@$inheritance) {
       Logger::LogDebug( $nodeLogFH, "Telnet is a $_\n" );
    }
    if (defined($params->{LOC_KNOWNHOSTS})) {
        $userKnownHosts = $params->{LOC_KNOWNHOSTS};
        if ($userKnownHosts !~ m/^[\/]/) {
            $userKnownHosts = "$homeDir/$userKnownHosts";
        }
    }
    if ( defined($params->{SYS_KNOWNHOSTS})) {
        $systemKnownHosts = $params->{SYS_KNOWNHOSTS};
        if ($systemKnownHosts !~ m/^[\/]/) {
            $systemKnownHosts = "$homeDir/$systemKnownHosts";
        }
    }
    my $prompt = '/[\$%#>]\s*$/';      #default prompt for 7302/7330
    if(defined($params->{ISAM7354RU})) {
      $prompt = '/[\$%#>]\s+$/';       # prompt changed to resolve SSH issue in 7354  
    }
    ### SSH command formation
    my @sshSpawnParameters = ( $config->{SSH}, "-o UserKnownHostsFile=$userKnownHosts", "-o GlobalKnownHostsFile=$systemKnownHosts");
    if( $params->{DEBUG} ) {
        push @sshSpawnParameters, "-vv";
    }
    if( $sessionType eq $SESSION_DEBUG ) {
        my $port = ( defined $params->{DEBUGSSHPORT} ) ? $params->{DEBUGSSHPORT} : 30;
        push @sshSpawnParameters, ( "-p", $port );
    }
    push @sshSpawnParameters, ( "-l", $params->{USERNAME}, $ipAddress );

    Logger::LogDebug( $nodeLogFH, "Establish ISAM CLI-$sessionType session ...\n"); 
    my ($pty, $tty, $pid) = &Spawn(  @sshSpawnParameters );
    ## Create a Net::Telnet object to perform I/O on ssh's tty.
    if( not defined($logFile) ) {
        $logFile = $tmpLogFile = "/tmp/isam_ssh_$ipAddress.trace";
    }
    my $isamCliSession = new Net::Telnet (
                        -fhopen => $pty,
			            -timeout => defined($timeout) ? $timeout : 10,
			            -prompt => $prompt,
			            -telnetmode => 0,
			            -errmode => 'return',
			            -cmd_remove_mode => 1,
			            -input_log => $logFile, 
			            -output_record_separator => "\r"
                     );
    if( not defined $isamCliSession) {
        $ERROR = "Establish an ISAM CLI-$sessionType Session/Connection is failed";
        return (1, $isamCliSession ); 
    }
    Logger::LogNotify( $nodeLogFH, "Establish ISAM CLI-$sessionType session ...DONE\n"); 
	$isamConnectionList{$isamCliSession}{TYPE}     = $sessionType;
	$isamConnectionList{$isamCliSession}{HOST}     = $ipAddress;
    $isamConnectionList{$isamCliSession}{SESSION} = $isamCliSession;
	$isamConnectionList{$isamCliSession}{TTY}      = $tty;
	$isamConnectionList{$isamCliSession}{PTY}      = $pty;
	$isamConnectionList{$isamCliSession}{PID}      = $pid;
    $isamConnectionList{$isamCliSession}{PARAMS}   = $params;
    $isamConnectionList{$isamCliSession}{TIMEOUT}  = $timeout;
    $isamConnectionList{$isamCliSession}{LOGFH}    = $logFile;
    return (0, $isamCliSession ); 
}

################################################################################
################################################################################
sub ConnectISAM($$) {
    my ($sessionType, $isamCliSession) = @_;
    my ($isamContent, $loggedIn, $loginPhase) = ("", $NOTLOGGEDIN, $INIT);
    Logger::LogDebug( $nodeLogFH, "Log-In to ISAM CLI-$sessionType session ...\n");
    my $loginConfirmKey = "!>>>LOGGEDIN-CONFIRM-KEY-".time()."-".rand(100)."<<<<<!";
    my $noLogin = 0;
    my $isamContentAsOfNow = "";
    my $user = undef;

    while ( not $loggedIn ) {
	    $isamContent = $isamCliSession->get();
        if( $loginPhase == $INIT and $sessionType eq $SESSION_TELNET ) {
		    $isamContent = $isamContentAsOfNow . $isamContent;
            $isamContentAsOfNow = $isamContent;
            $user = defined $params->{USERNAME} ? $params->{USERNAME} : undef;
        }
	    if (not defined($isamContent)) {
	      if ($isamCliSession->eof()) {
	          $ERROR = "No data was read after opening ssh session";
              Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
              $noLogin = 1;
	      } else {
	          if ($isamCliSession->timed_out()) {
	            $ERROR = "Timed out while waiting for some alive indication";
                Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
	          } else {
                $ERROR = "An error occured while processing data :" . $isamCliSession->errmsg;
                Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
	            $noLogin = 1;
	          }
	      }
	      $loggedIn = $loginPhase == $PASSWD ? $LOGINOK : $LOGINFAIL;  # To deal with a prompt without a terminator
	    } else {
           #Logger::LogDebug( $nodeLogFH,"\n", $Logger::LogOnlyMessage );
           #Logger::LogDebug( $nodeLogFH,"Login phase: '$sshLoginPhases{$loginPhase}'; Data Received:\n $isamContent\n--->> END <<---\n");
	       if ($loginPhase == $INIT) {
	         ($loggedIn, $loginPhase) = HandleLoginInit($isamCliSession, $isamContent, $loginPhase, $loginConfirmKey);
	       } elsif (($loginPhase == $PASSWD) || ($loginPhase == $PASSPHRASE)) {
	         ($loggedIn, $loginPhase) = HandleLoginPassCode($isamCliSession, $isamContent, $loginPhase, $loginConfirmKey);
	       }
	    }
    }
    if (($loggedIn > $LOGINOK) || $noLogin) {
        Logger::LogDebug( $nodeLogFH,"\n", $Logger::LogOnlyMessage );
        if( defined $isamContent ) {
           Logger::LogDebug( $nodeLogFH,"Login phase: '$sshLoginPhases{$loginPhase}'; Data Received:\n $isamContent\n--->> END <<---\n");
        } else {
           Logger::LogDebug( $nodeLogFH,"Login phase: '$sshLoginPhases{$loginPhase}' - $ERROR\n");
        }
        cleanConnection($isamCliSession);
	    $isamCliSession = undef;
    } else {
	  # Logged in now.  Set the prompt
      my @lines = $isamCliSession->getlines(-timeout => 5);
      ToolCommon::Log($DEBUG, "last input: '@lines'\n") if defined($DEBUG);

	  $isamCliSession->buffer_empty();
	  my @response = $isamCliSession->cmd("environment prompt \"%d%c \"");
	  if (CommandError(join("", @response))) {
	    $ERROR = sprintf("Setting environment prompt failed: %s", join("", @response));
        cleanConnection($isamCliSession);
	    $isamCliSession = undef;
      }
    }
    Logger::LogNotify( $nodeLogFH, "Log-In to ISAM CLI-$sessionType session ...DONE\n") if (defined $isamCliSession);
    return($isamCliSession);
}

=head2 ConnectSSH($$$;$)

=begin html
Open an ssh session using a telnet socket on the local machine and establish CLI login.
The opened CLI session is returned to the caller or undef in case of failure.<br>
The following parameters are used:
<table>
  <tr><td>ip</td><td>IP address of the NE</td></tr>
  <tr><td>params</td><td>pointer to a hash of command paramater values</td></tr>
  <tr><td>timeout</td><td>TELNET timeout value</td></tr>
  <tr><td>logFile</td><td>file handle for logging of TELNET activity (undef if not needed)</td></tr>
</table>

=end html

=cut

sub ConnectSSH($$;$) {
    my ($ipAddress, $timeout, $logFile) = @_;
    my $sessionType = $SESSION_SSH;
    $neId = $ipAddress;
    $ERROR = "";

    my ($status, $isamCliSession ) = (0, undef );

    $status = DoKnownHostCheck();
    return undef if( $status );

    ($status, $isamCliSession) = IsSessionExists($sessionType, $ipAddress );
    return undef if( $status );

    ($status, $isamCliSession ) = GetSSHSession ( $sessionType, $ipAddress, $timeout, $logFile );
    return undef if( $status );


    $isamCliSession = ConnectISAM($sessionType, $isamCliSession );
    return($isamCliSession);
}

sub ConnectSecDebugViaSSH($$;$) {
    my ($ipAddress, $timeout, $logFile) = @_;
    my $sessionType = $SESSION_DEBUG;
    $neId = $ipAddress;
    $ERROR = "";

    my( $status, $isamCliSession ) = (0, undef);

    $status = DoKnownHostCheck();
    return undef if( $status );

    ($status, $isamCliSession) = IsSessionExists($sessionType, $ipAddress );
    return undef if( $status );

    ($status, $isamCliSession ) = GetSSHSession ( $sessionType, $ipAddress, $timeout, $logFile );
    return undef if( $status );

    $isamCliSession = ConnectISAM($sessionType, $isamCliSession );
    return($isamCliSession);
}

=head2 Connect($$$;$$$)

=begin html
Open telnet session and establish CLI login.  The opened telnet session is returned to the caller or undef in case of failure.<br>
The following parameters are used:
<table>
  <tr><td>ip</td><td>IP address of the NE</td></tr>
  <tr><td>user</td><td>CLI user name</td></tr>
  <tr><td>passwd</td><td>CLI password</td></tr>
  <tr><td>timeout</td><td>TELNET timeout value</td></tr>
  <tr><td>logFile</td><td>file handle for logging of TELNET activity (undef if not needed)</td></tr>
</table>

=end html

=cut

my $OPEN_MAX = 5;
my $CONN_MAX = 5;
sub Connect($;$$$) {
    my ($ipAddress, $timeout, $logFile, $port) = @_;
    my $sessionType = $SESSION_TELNET;
    $neId = $ipAddress;
    $ERROR = "";

    my ($status, $isamCliSession) = IsSessionExists($sessionType, $ipAddress );
    return undef if( $status );

    my $inheritance = Net::Telnet::whatamI(undef);
    foreach (@$inheritance) {
      Logger::LogDebug( $nodeLogFH, "Telnet is a $_\n" );
    }
    Logger::LogDebug( $nodeLogFH, "Establish $sessionType session ...\n"); 
    if( not defined($logFile) ) {
        $logFile = $tmpLogFile = "/tmp/isam_telnet_$ipAddress.trace";
    }
    ### Get blank telnet session/stream
    my $tries = 0;
    do {
        if( $tries >= $CONN_MAX ) {
           $ERROR = "Establish an ISAM CLI-$sessionType session failed at telnet stream processing";
           return undef;   ### DEVEL-NOTE : Why can't we return old session  ## FIX-ME FIXME
        }
        $tries++;
        sleep(5);
        $isamCliSession = new Net::Telnet (
                             Timeout => defined($timeout) ? $timeout : 10,
          		             Errmode => 'return',
          		             Input_log => $logFile, 
          		             Port => defined($port) ? $port : 23,
          		             Prompt => '/[\$%#>]\s*$/'
                           );  # -- 1.0.4 to cope with optional trailing white space(s)
    } while (not defined($isamCliSession) );
    Logger::LogDebug( $nodeLogFH, "Establish $sessionType session ...DONE\n"); 
    $tries = 0;
    ### Try to open/connect/telnet to ISAM
    Logger::LogDebug( $nodeLogFH, "Establish ISAM CLI-$sessionType session ...\n"); 
    do {
        if( $tries >= $OPEN_MAX ) {
            $ERROR = "CLI-$sessionType session failed to open connection : " . $isamCliSession->errmsg();
            return undef;   ### DEVEL-NOTE : Why can't we return old session  ## FIX-ME FIXME
        }
        sleep(5);
        $tries++;
        $status = $isamCliSession->open($ipAddress);
    } while( not defined $status or not $status );
    Logger::LogNotify( $nodeLogFH, "Establish ISAM CLI-$sessionType session ...DONE\n"); 

    ToolCommon::Log($DEBUG, "Storing connection data for TELNET session to '$ipAddress'\n") if defined($DEBUG);
    $isamConnectionList{$isamCliSession}{HOST} = $ipAddress;
    $isamConnectionList{$isamCliSession}{TYPE} = $sessionType;
    $isamConnectionList{$isamCliSession}{SESSION} = $isamCliSession;
    $isamConnectionList{$isamCliSession}{PARAMS}  = $params;
    $isamConnectionList{$isamCliSession}{TIMEOUT} = $timeout;
    $isamConnectionList{$isamCliSession}{LOGFH}   = $logFile;

    $isamCliSession = ConnectISAM($sessionType, $isamCliSession );
    return($isamCliSession);

}

=head2 TelnetConnectionLost($)

=begin html
This function closes a local telnet connection in case the connection to the remote is lost (e.g. due to a SIGPIPE).

=end html

=cut

sub TelnetConnectionLost($) {
  my ($telnet) = @_;

  my $host = DetermineSessionHost($telnet);
  if ($host ne "") {
    if ( $isamConnectionList{$telnet} ) {
	cleanConnection($telnet);
    } else {
      ToolCommon::Log($DEBUG, "No internal connection established to '$host'\n") if defined($DEBUG);
      $telnet->close();  # call the close anyhow..
    }
  } else {
    $ERROR = "No connection context found for this CLI session";
  }
}

=head TelnetNoDataRead($)

=begin html
This function returns true in case no data was read from the telnet connection in an eof condition.

=end html

=cut

sub TelnetNoDataRead($) {
  my ($errStr) = @_;

  if ($errStr =~ /No data was read after opening/ || $errStr =~ /Timed out while waiting for some alive indication/) {
    return 1;
  }
  return 0;
}

sub cleanConnection($) {
  my ($isamConnection) = @_;
  if( defined($isamConnectionList{$isamConnection}) ) {
       if( defined $isamConnectionList{$isamConnection}{PTY} ) {
	   close $isamConnectionList{$isamConnection}{PTY};
       }
       if( defined $isamConnectionList{$isamConnection}{TTY} ) {
	   close $isamConnectionList{$isamConnection}{TTY};
       }
       SetCliConnection(undef);
       $isamConnection->close();
       sleep 5;
       if ($tmpLogFile ne "") {
         my $deleted = unlink($tmpLogFile);
         if ($deleted == 1) {
           $tmpLogFile = "";
         } else {
           $ERROR = "FAILED to delete '$tmpLogFile': $!";
         }
       }
       delete $isamConnectionList{$isamConnection};
  }
}

=head2 Disconnect($)

=begin html
Logout from the NE and close the telnet session specified by $telnet.

=end html

=cut

sub DetermineSessionHost($) {
  my ($isamConnection) = @_;
  my $host = "";
  my $type = "";
  if( !defined $isamConnection ) {
    return $host;
  }
  $host = $isamConnection->host();
  if( $host ne "" ) {
    return $host;
  }
  
  if( defined($isamConnectionList{$isamConnection}) ) {
	if( defined($isamConnectionList{$isamConnection}{HOST}) ) {
	   $host = $isamConnectionList{$isamConnection}{HOST};
	}
	if( defined($isamConnectionList{$isamConnection}{TYPE}) ) {
	   $type = $isamConnectionList{$isamConnection}{TYPE};
	}
  }
  Logger::LogTrace($nodeLogFH, "Determine session:'$isamConnection', host:$host:, type:$type:'\n");
  return $host;
}

sub Disconnect(;$) {
  my ($telnet) = @_;
  my $cmd = "";
 
  my $cli = GetCliConnection();
  if( not defined $cli ) {
    #$ERROR = "Unable to fetch CLI connection";
    #ToolCommon::Log($DEBUG, "ERROR :$ERROR\n" ) if defined($DEBUG);
    #return undef;
    ## FIX-ME : Through some log message to scan when this situation occurs
    $cli = $telnet;
  }
  if( not defined $cli ){
      $ERROR = "Not (yet) connected";
      return;
  }
  my $host = DetermineSessionHost($cli);
  if ($host ne "") {
    ToolCommon::Log($DEBUG, "Disconnect received for context '$cli', host '$host'\n") if defined($DEBUG);
    if (defined($isamConnectionList{$cli})) {
      my @lines = $cli->cmd(-string => "logout", -Timeout => 1); # Change default timeout to 1 second since we will waiting here anyway
      my $inputEmpty = 0;
      while (! $inputEmpty) {
          @lines = $cli->getlines(-timeout => 5);
          ToolCommon::Log($DEBUG, "Reading from input <@lines>\n") if defined($DEBUG);
          if (scalar(@lines) == 0) {
            ToolCommon::Log($DEBUG, sprintf("No lines read, EOF: %s, Time-out: %s\n",
				  $cli->eof() ? "Yes": "No", $cli->timed_out() ? "Yes" : "No")) if defined($DEBUG);
            $inputEmpty = 1 if $cli->timed_out() || $cli->eof();
          }
      }
      cleanConnection($cli);
    } else {
      $ERROR = "Not (yet) connected";
    }
  } else {
    $ERROR = "No connection context found for this CLI session";
  }
}

=head2 SendCommand($$;$)

=begin html
Send a CLI command to the ISAM.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>telnet session context</td></tr>
  <tr><td>cmd</td><td>CLI command to be sent</td></tr>
  <tr><td>checkError</td><td>check the response when defined.  In this case undef will be returned to the caller otherwise the parsing for errors
          is left to the caller</td></tr>
</table>
The command is retransmitted in case an alarm has been received during the command execution.<br>
The response received is returned to the caller.

=end html

=cut

sub SendCommand($$;$) {
  my ($telnet, $cmd, $checkError) = @_;
  my $rspReceived = 0;
  my @lines;
  my $alarmOccured;
  my @parsedResponse;
  my $i = 0;
  my $cli = GetCliConnection();
  if( ! defined $cli ) {
    #$ERROR = "Unable to fetch CLI connection";
    #ToolCommon::Log($DEBUG, "ERROR :$ERROR\n" ) if defined($DEBUG);
    #return undef;
    ## FIX-ME : Through some log message to scan when this situation occurs
    $cli = $telnet;
  }
  $ERROR = "";

  my $host = DetermineSessionHost($cli);
  if ($host eq "") {
    $ERROR = "No connection found to execute CLI Command '$cmd'";
    Logger::LogError($nodeLogFH,  "$ERROR\n", undef, $Logger::LogOnlyHandle );
    return undef;
  }
  $responseEmpty = 0;
  Logger::LogTrace( $nodeLogFH, "CLI Command : $cmd\n" );
  while (not $rspReceived) {
    @parsedResponse = ();
    $alarmOccured = 0;
    $cli->buffer_empty();
    my $bufferdata = $cli->get(Timeout=>0);
    if( defined $bufferdata ) {  ## HANDLE CLI BUFFET ## TERMINAL TIMEOUT
        Logger::LogDebug($nodeLogFH,  "Buffer data :$bufferdata:\n" );
        if ( $bufferdata =~ /$host closed|Broken pipe|closed by remote host|logging out !$/) {
           Logger::LogNotify($nodeLogFH,  ">>>Connection was closed by Remote host<<<:\n" );
        }
        $cli = reEstablishConnection($cli) ;
        if( not defined $cli ) {
           $ERROR = "Unable to reestablish CLI connection";
           Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
           return undef;
        }
        $cli->buffer_empty();
    }  
    @lines = $cli->cmd( $cmd );
    Logger::LogDebug( $nodeLogFH, "\n", $Logger::LogOnlyMessage );
    $i = 0;
    foreach (@lines) {
         if ($_ =~ /[\-\|\\\/\s]\x1B\x5B1D/) {
	         # spinning wheel present: remove from input
	         $_ =~ s/\-\x1B\x5B1D//g;
	         $_ =~ s/\|\x1B\x5B1D//g;
	         $_ =~ s/\/\x1B\x5B1D//g;
	         $_ =~ s/\\\x1B\x5B1D//g;
	         $_ =~ s/\s\x1B\x5B1D//g;
         }
         chomp($_);
         #Logger::LogDebug($nodeLogFH,  sprintf("(%02d)%s\n", length($line), $line));
         if ($_ =~ /^\d\d.\d\d.\d\d[\s,]\d\d.\d\d.\d\d.*/ ) {
	         $alarmOccured++;
         } elsif (length($_) > 0) {
	         $parsedResponse[$i++] = "$_\n";
         }
    }
    Logger::LogTrace( $nodeLogFH, "Encountered $alarmOccured alarms\n") if ( $alarmOccured );
    my $responseAsString = join ('', @parsedResponse );
    if ( $responseAsString =~ /($ERR|$INVALID_TOKEN|$INTERNAL_ERROR|$CONN_RESET|$FILE_CLOSED|$TIMED_OUT).*/m ) {
           my $matchedError = $&;
           Logger::LogTrace($nodeLogFH,  sprintf("(%02d)%s\n", length($_), $_)) foreach ( @parsedResponse);
           $ERROR = "CLI Command '$cmd' failed with error : '$matchedError'\n";
           Logger::LogError($nodeLogFH, "$ERROR", undef, $Logger::LogOnlyHandle );
           return undef;
    }
    my $sessionErrMsg = $cli->errmsg();
    if( $sessionErrMsg ne "" ) {
       Logger::LogTrace($nodeLogFH,  sprintf("(%02d)%s\n", length($_), $_)) foreach ( @parsedResponse);
       $ERROR = "CLI Command '$cmd' failed with error : $sessionErrMsg\n";
       Logger::LogError($nodeLogFH, "$ERROR", undef, $Logger::LogOnlyHandle );
       return undef;
    }
    if( $i == 0 ) {
       Logger::LogTrace($nodeLogFH,  sprintf("(%02d)%s\n", length($_), $_)) foreach ( @parsedResponse);
       $ERROR = "CLI Command '$cmd' failed for no lines copied\n";
       Logger::LogError($nodeLogFH, "$ERROR", undef, $Logger::LogOnlyHandle );
       return undef;
    }
    $rspReceived = 1;
    if (not ($cmd =~ /^configure/) and not $alarmOccured ) {
            ### DEVEL-NOTE :
            ###  We found that 'info configure' command get dumped into CLI O/P for SSH stream,
            ###  In conversion script, we have handled parsing as per TELNET stream so  we failed to ignore in 'info configure' o/p parsing.
            if( ($cmd =~ /^info configure/) ) {
               if((defined($parsedResponse[0])) &&  ($parsedResponse[0] =~ /^$cmd/) ) {
                  shift @parsedResponse;
               }
            }
    }
  }  ## END WHILE LOOP
  Logger::LogTrace($nodeLogFH,  sprintf("CLI Command O/P has %d lines\n", scalar(@parsedResponse)) );
  return @parsedResponse;
}

=head2 RetrieveInfo($$$)

=begin html
Generic info retrieval command.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>telnet session context</td></tr>
  <tr><td>cmd</td><td>CLI command to be sent</td></tr>
  <tr><td>returnResponseOnErr</td>
      <td>controls the result returned to the caller:
      <ul>
        <li>defined: the received response is returned in case of error</li>
        <li>not defined:  undef is returned to the caller in case of error.</li>
      </ul>
      </td>
  </tr>
</table>
In case of success the command response is returned.

=end html

=cut

sub RetrieveInfo($$;$) {
  my ($telnet, $cmd, $returnResponseOnErr) = @_;
  $ERROR = "";
  my @rspReceived = SendCommand($telnet, $cmd);
  if ($ERROR ne "") {
      return undef;
  }
  my $rspLength = @rspReceived ? scalar(@rspReceived) : 0;
  my $last = defined $rspReceived[$rspLength-2] ? $rspReceived[$rspLength-2] : "Not-As-Expected";
  chomp($last);
  $last =~ s/\r//g;
  if (not ($last =~ /^(=+|#-+)/) && ( $cmd !~ / flat[\$\s]/ and $cmd !~ /\|\s+match exact:/) ) {
      Logger::LogTrace($nodeLogFH, sprintf("(%02d)%s\n", length($_), $_)) foreach ( @rspReceived);
      Logger::LogNotify($nodeLogFH,"last line : '$last'\n");
      $ERROR = "CLI Command '$cmd' has incomplete response to extract data\n";
      Logger::LogError($nodeLogFH, "$ERROR", undef, $Logger::LogOnlyHandle );
      return undef;
  }
  return \@rspReceived;
}

sub IsConfigured($$$) {
  my ($cliSession, $cmd, $fieldExists ) = @_;
  my $rspReceived = RetrieveInfo($cliSession, $cmd);
  if ($ERROR ne "") {
      return 0;
  }
  foreach my $line (@{$rspReceived}) {
    $line =~ s/^\s+//;
    $line = " $line";
    if (($line =~ / $fieldExists /) or ($line =~ / $fieldExists$/)) {
       return 1;
    }
  }
  return 2;
}

=head2 CommandError($)

=begin html
Return TRUE in case a command has failed, FALSE otherwise.

=end html

=cut

sub CommandError($) {
  my ($rspStr) = @_;

  return ((($rspStr =~ /$ERR/) || ($rspStr =~ /$INVALID_TOKEN/) || ($rspStr =~ /$INTERNAL_ERROR/) || ($rspStr =~ /$GENERAL_ERROR/) || ($rspStr =~ /$GPONMGNT_ERROR/) ||
	   # Deal with connection resets -> action undefined...
	   ($rspStr =~ /$CONN_RESET/) || ($rspStr =~ /$FILE_CLOSED/)) ? 1 : 0);
}

=head2 GetNtExtendedInfo($;$)

=begin html
Retrieve extended NT info using the opened isamCliSession session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: NTA, NTB.  For every NT the following
is specified:
<ul>
  <li>TYPE: the NT type</li>
  <li>STATUS: the NT status</li>
  <li>ICS: the NT ICS code</li>
  <li>SERIAL: the NT serial number</li>
  <li>ACTIVE: active or passive indication</li>
</ul>

=end html

=cut

sub GetNtExtendedInfo($;$$) {  # -- 1.2.0
    my ($isamCliSession, $noIcs, $ntNotSync) = @_;
    my $extInfo;
    my $ntInfo;

    $ntInfo = GetNtInfo($isamCliSession);
    if (not defined($ntInfo)) {
      FamilyCommon::ReportStatus( EC_GET_NT_INFO_FAILED, "Failed to get NT info : $ERROR" );
      return undef;
    }
	$ntInfo->{NTA}{ICS} = '';
	$ntInfo->{NTA}{SERIAL} = '';
	$ntInfo->{NTB}{ICS} = '';
	$ntInfo->{NTB}{SERIAL} = '';
    if( not defined($noIcs) ) {
        if ( $ntInfo->{NTA}{STATUS} eq 'PRESENT' ) {
	         $extInfo = GetNtIcs($isamCliSession, 'NTA');
	         if (not defined($extInfo)) {
                return undef if( FamilyCommon::ReportStatus( EC_GET_NT_INFO_FAILED, "Failed to retrieve NTA extended info: $ERROR" ) );
             } else {
	            $ntInfo->{NTA}{ICS} = $extInfo->{ICS};
	            $ntInfo->{NTA}{SERIAL} = $extInfo->{SERIAL};
             }
        }
        if ( $ntInfo->{NTB}{STATUS} eq 'PRESENT' ) {
	         $extInfo = GetNtIcs($isamCliSession, 'NTB');
	         if (not defined($extInfo)) {
                return undef if( FamilyCommon::ReportStatus( EC_GET_NT_INFO_FAILED, "Failed to retrieve NTB extended info: $ERROR" ) );
             } else {
	            $ntInfo->{NTB}{ICS} = $extInfo->{ICS};
	            $ntInfo->{NTB}{SERIAL} = $extInfo->{SERIAL}
             }
        }
    }
    if ( ($ntInfo->{NTA}{STATUS} eq 'PRESENT') && ($ntInfo->{NTB}{STATUS} eq 'PRESENT') and 
         ($params->{'BOARD-FAMILY'} ne Capability::Isam7363Family()) ) {
        my $protElemState = IsamCli::GetPEState($isamCliSession);
        if (not defined($protElemState)) {
            return undef if( FamilyCommon::ReportStatus( EC_GET_NT_INFO_FAILED, "Failed to retrieve protection info: $ERROR" ) );
        } else {
	        $ntInfo->{NTA}{ACTIVE} = $protElemState->{NTA};
	        $ntInfo->{NTB}{ACTIVE} = $protElemState->{NTB};
        }
    } else {
         if ($ntInfo->{NTA}{STATUS} eq 'PRESENT') {
	         $ntInfo->{NTA}{ACTIVE} = 'ACTIVE';
	         $ntInfo->{NTB}{ACTIVE} = '';
         } else {
	         $ntInfo->{NTB}{ACTIVE} = 'ACTIVE';
	         $ntInfo->{NTA}{ACTIVE} = '';
         }
    }
    Logger::LogTrace( $nodeLogFH, sprintf("NTA - %-8s - %-8s - %-8s (ICS :%s) (SERIAL : %s)\n", 
                                          uc($ntInfo->{NTA}{TYPE}),
                                          $ntInfo->{NTA}{STATUS},
                                          ( $ntInfo->{NTA}{ACTIVE} eq '' ) ? 'NA' : $ntInfo->{NTA}{ACTIVE},
                                          $ntInfo->{NTA}{ICS}, $ntInfo->{NTA}{SERIAL} ) );
    Logger::LogTrace( $nodeLogFH, sprintf("NTB - %-8s - %-8s - %-8s (ICS :%s) (SERIAL : %s)\n", 
                                          uc($ntInfo->{NTB}{TYPE}),
                                          $ntInfo->{NTB}{STATUS},
                                          ( $ntInfo->{NTB}{ACTIVE} eq '' ) ? 'NA' : $ntInfo->{NTB}{ACTIVE},
                                          $ntInfo->{NTB}{ICS}, $ntInfo->{NTB}{SERIAL} ) );


  if ( (uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") and (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY") and 
       ($ntInfo->{NTA}{TYPE} ne $ntInfo->{NTB}{TYPE}) and  ($params->{'BOARD-FAMILY'} ne Capability::Isam7363Family()) ) {
         return undef if( FamilyCommon::ReportStatus( EC_INCOMPAT_NT, "(NT-A: '$ntInfo->{NTA}{TYPE}', NT-B: '$ntInfo->{NTB}{TYPE}')" ) );
      }
  if ( (uc($ntInfo->{NTA}{TYPE}) ne "EMPTY") and 
       (uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") and 
       (not Capability::NtSupported($ntInfo->{NTA}{TYPE})) ) {
       return undef if( FamilyCommon::ReportStatus( EC_NT_TYPE_UNKNOWN, "NT-A type '$ntInfo->{NTA}{TYPE}'" ) );
  }
  if ( (uc($ntInfo->{NTB}{TYPE}) ne "EMPTY") and 
       (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY") and 
       (not Capability::NtSupported($ntInfo->{NTB}{TYPE})) ) {
       return undef if( FamilyCommon::ReportStatus( EC_NT_TYPE_UNKNOWN, "NT-B type '$ntInfo->{NTB}{TYPE}'", undef ) );
  }
  return $ntInfo;
}

=head2 GetNtInfo($)

=begin html
Retrieve basic NT info using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: NTA, NTB.  For every NT the following
is specified:
<ul>
  <li>TYPE: the NT type</li>
  <li>STATUS: the NT status</li>
</ul>

=end html

=cut

sub GetNtInfo($) {
  my ($telnet) = @_;
  my %ntInfo = ('NTA' => { 'TYPE' => "EMPTY",
			   'STATUS' => "EMPTY"},
		'NTB' => { 'TYPE' => "EMPTY",
			   'STATUS' => "EMPTY"});
  my $rspLength;
#  $slotTypeNT = "";
  my $response = RetrieveInfo($telnet, "show equipment slot", undef);

  if (defined($response)) {
    my ($position, $type, $enabled);
    my $index;
    foreach (@{$response}) {
      $position = 0;
      $type = "";
      $enabled = "";
      $index = "";
      if (/^1\/1\/([2|3])\s*(\w+\-\w|\w+)\s*(\w+)\s*.*/) {
	#slot           type                 enabled error-status           availability 
	#--------------------------------------------------------------------------------
	#1/1/1          AACU-C               no      no-planned-board       available    
	#1/1/2          ECNT-A               yes     no-error               available    
	#1/1/3          EMPTY                no      no-error               dependency   
	# or for R2.2:
	#1/1/3                               no      no-error               dependency   
	$position = $1, $type = $2, $enabled = $3;
	$index = $position == 2 ? 'NTA' : 'NTB';
	$ntInfo{$index}{'TYPE'} = uc($type) ne 'NO' ? $type : "EMPTY";
	$ntInfo{$index}{'STATUS'} = uc($enabled) eq "YES" ? "PRESENT" : "EMPTY";
      } elsif (/^nt\-([a|b])\s*(\w+\-\w|\w+)\s*(\w+)\s*.*/) { # 1.2.5
	#slot           type                 enabled error-status           availability 
	#--------------------------------------------------------------------------------
	#acu:1/1        AACU-C               no      no-planned-board       available    
	#nt-a           ECNT-A               yes     no-error               available    
	#nt-b           EMPTY                no      no-error               dependency   
	$position = $1, $type = $2, $enabled = $3;
	$index = uc($position) eq 'A' ? 'NTA' : 'NTB';
	$ntInfo{$index}{'TYPE'} = uc($type) ne 'NO' ? $type : "EMPTY";
	$ntInfo{$index}{'STATUS'} = uc($enabled) eq "YES" ? "PRESENT" : "EMPTY";
      } elsif (/^nt\s*(\w+\-\w|\w+)\s*(\w+)\s*.*/) { 
        #ERAM-A providing slot as nt instead of nt-a starting R42 
        #slot     |actual-type|enabled|error-status          |availability |restrt-cnt
        #---------+-----------+-------+----------------------+-------------+-------------
        #nt        agnt-a      yes     no-error               available     0
        $type = $1, $enabled = $2;
	$index = 'NTA'; 
	$ntInfo{$index}{'TYPE'} = uc($type) ne 'NO' ? $type : "EMPTY";
	$ntInfo{$index}{'STATUS'} = uc($enabled) eq "YES" ? "PRESENT" : "EMPTY";
	$slotTypeNT = 'nt';
      }
    }
    return \%ntInfo;
  } else {
    return undef;
  }
  return undef;
}

=head2 GetNtIcs($$)

=begin html
Retrieve NT ICS code for slot position $pos using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise the ICS code and board serial are returned.<br>
Added R3.1 onwards: scripts will use type-based notation so:
<ul>
  <li>first try using type-based notation</li>
  <li>If that fails use pre-R3.1 notation</li>
</ul>

=end html

=cut

sub GetNtIcs($$) {
  my ($telnet, $pos) = @_;
  my $slotPos = $pos eq 'NTA' ? 2 : 3;
  my $slotTypePos = $pos eq 'NTA' ? 'nt-a' : 'nt-b';
  my $respStr = "";
  my $response = "";
  my %extInfo;
  if ((defined($slotTypeNT)) && ($slotTypeNT eq 'nt')) {
	  $response = RetrieveInfo($telnet, "show equipment slot $slotTypeNT detail", undef);
		   if (!defined($response)) {
		     return undef;
		   }
  } else {
  $response = RetrieveInfo($telnet, "show equipment slot $slotTypePos detail", 1);
  return undef if (!defined $response);
  foreach (@{$response}) {
    $respStr .= $_;
  }
  if ($respStr =~ /$INVALID_TOKEN/) {
    $response = RetrieveInfo($telnet, "show equipment slot 1/1/$slotPos detail", undef);   
		   if (!defined($response)) {
		     return undef;
		   }
    }
  }
  foreach (@{$response}) {
    if (/\s*.*\-ics\s:\s(\d*).*/) {
      $extInfo{ICS} = $1;
    }
    if (/\s*.serial\-no\s:\s([^\s]+)/) {
      $extInfo{SERIAL} = $1;
    }
    if (/\s*.serial\-no\s:\s\"([^\s]+)/) {
      $extInfo{SERIAL} = $1;
    }
    return \%extInfo if (defined($extInfo{ICS}) && defined($extInfo{SERIAL}))
  } 
  $ERROR = "no ics data found for $slotTypePos\n'" . join("", @{$response}) . "'";
  return undef;
}

=head2 GetPEState($)

=begin html
Retrieve NT redundancy info using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: NTA, NTB and specifies its
redundancy status ("ACTIVE" or "STANDBY").

=end html

=cut

#------------------------------------------------------------------------------
#-- Retrieve protection element state
#--   $telnet: TELNET session context
#-- Returns the protection-element info or undef in case of failure.
#------------------------------------------------------------------------------
sub GetPEState($) {
  my ($telnet) = @_;
  my ($position, $index, $state);
  my %peState = ('NTA' => '',
		 'NTB' => '');
  my $response = RetrieveInfo($telnet, "show equipment protection-element", undef);

  if(!defined($response)) {
    return undef;
  }
  foreach (@{$response}) {
    if (/^1\/1\/([2|3])\s*(.*)/) {
      $position = $1, $state = $2;
      $index = $position == 2 ? 'NTA' : 'NTB';
      if (uc($state) =~ /PROVIDING/) {
	$peState{$index} = 'ACTIVE';
      } elsif (uc($state) =~ /STANDBY/) {
	$peState{$index} = 'STANDBY';
      }
    } elsif (/^nt\-([a|b])\s*(.*)/) { # 1.2.5
      $position = $1, $state = $2;
      $index = uc($position) eq 'A' ? 'NTA' : 'NTB';
      if (uc($state) =~ /PROVIDING/) {
	$peState{$index} = 'ACTIVE';
      } elsif (uc($state) =~ /STANDBY/) {
	$peState{$index} = 'STANDBY';
      }
    }
  }
  return \%peState;
}                # -- 1.2.0

=head2 GetOswpInfo($$)

=begin html
Retrieve OSWP info using the opened telnet session $telnet.  $regExp is used to determine the SW version numbers.
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: "ACTIVE" and "PASSIVE".  For each
entry the following is given:
<ul>
  <li>INDEX: the associated OSWP index</li>
  <li>NAME: the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>COMMITED: the OSWP SW commited status</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>

=end html

=cut

sub GetOswpInfo($$) {
  my ($telnet, $regExp) = @_;
  my $index = undef;
  my $name = undef;
  my $status = undef;
  my $availability = undef;
  my $swpIndex;
  my $otherSwpIndex;
  my %oswpInfo = ('ACTIVE' => { 'INDEX' => 0,
				'NAME' => NO_OSWP,
				'COMMITED' => "",
				'AVAIL' => "",
				'MAJOR' => 0,
				'MINOR' => 0,
				'SUFFIX' => 0,
				'MAINT' => 0 },
		  'PASSIVE' => { 'INDEX' => 0,
				 'NAME' => NO_OSWP,
				 'COMMITED' => "",
				 'AVAIL' => "",
				 'MAJOR' => 0,
				 'MINOR' => 0,
				 'SUFFIX' => 0,
				 'MAINT' => 0});
  my @active;
  my @passive;
  my $major = undef;    # -- 1.0.4
  my $minor = undef;    # -- 1.0.4
  my $maintRel = undef; # -- 1.2.0
  my $suffix = undef;   # -- 1.2.0
  my $line = "";

  my $response = RetrieveInfo($telnet, "show software-mngt oswp detail", 1);
  return undef if(not defined($response));
  my $rspLength = scalar @{$response};
  my $lineNo = 0;
  while ($lineNo < $rspLength) {
    $line = $response->[$lineNo];
    chomp $line;
    $lineNo++;
    if ($line =~ /index\s:\s(\d)/) {
        $index = $1;
        next;
    }
    if ($line =~ /name\s:\s+([^\s]+)/) {
        next if( not defined $index );
	    $name = $1;
	    $line = $response->[$lineNo++];
	    chomp $line;
	    if (not ($line =~ /.+\w+\s:\s.+/) ) {
	      if ($line =~ /\s+([\w\/\.]+)/) {
	        $name .= $1;
	      }
	    }
	    $name = basename($name);
    }
    if ($line =~ /availability\s:\s([\w-]+)\s+act-status\s:\s([\w-]+)/) {
         next if (not defined($index)) ;
	     $availability = $1;
	     $status = $2;
    }
    if ($line =~ /commit-status\s:\s([\w-]+)/) {
           my $commit = $1;
           next if ( (not defined($index)) or (not defined( $name)) or (not defined( $status)) );
	       if ($name =~ /$regExp/) {  # -- 1.0.4
	         $major = $1;                                  # -- 1.0.4
	         $minor = $2;                                  # -- 1.0.4
	         $suffix = $3;                                 # -- 1.2.0
	         $maintRel = $4;                               # -- 1.2.0
	       }                                               # -- 1.0.4
	       if ($status eq "active") {
	         $swpIndex = 'ACTIVE';
	         $otherSwpIndex = 'PASSIVE';
	       } else {
	         $swpIndex = 'PASSIVE';
	         $otherSwpIndex = 'ACTIVE';
	       }
	       $oswpInfo{$swpIndex}{'INDEX'} = $index;
	       if ($oswpInfo{$otherSwpIndex}{INDEX} == 0) {
	         $oswpInfo{$otherSwpIndex}{INDEX} = ($index == 1) ? 2 : 1;
	       }
	       $oswpInfo{$swpIndex}{'NAME'} = $name;
	       $oswpInfo{$swpIndex}{'COMMITED'} = defined($commit) ? $commit : "";
	       $oswpInfo{$swpIndex}{'AVAIL'} = defined($availability) ? $availability : "";
	       $oswpInfo{$swpIndex}{'MAJOR'} = defined($major) ? $major : 0;
	       $oswpInfo{$swpIndex}{'MINOR'} = defined($minor) ? $minor : 0;
	       $oswpInfo{$swpIndex}{'SUFFIX'} = defined($suffix) ? $suffix : 0;
	       $oswpInfo{$swpIndex}{'MAINT'} = defined($maintRel) ? $maintRel : 0;
	       $index = $name = $status = $availability =  undef;
	       $major = $minor = $suffix = $maintRel = undef;                                  # -- 1.0.4
    }
  }
  return \%oswpInfo;
}


=head2 GetDbVersionInfo($)

=begin html
Retrieve the actual NT DB version of the 2 OSWP using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: "1", "2" and specifies its DB
version.

=end html

=cut

sub GetDbVersionInfo($) {
  my ($telnet) = @_;
  my %dbVersion = ("1" => "", "2" => "");

  my $response = RetrieveInfo($telnet, "show software-mngt database", undef);
  if( not defined($response)) {
      Logger::LogError( $nodeLogFH, "Failed to retrive DB Version details\n", undef, $Logger::LogOnlyHandle );
      return undef;
  }
  foreach (@{$response}) {
    #first        1           empty                previous    A2.200                
    #second       1           empty                actual      A2.200                
    if (/\w*\s*(\d)\s*.*\s*actual\s*(\w+\d+\.\d+)/) {
        $dbVersion{$1} = $2;
    }
  }
  foreach my $key (sort keys %dbVersion ) {
      Logger::LogTrace( $nodeLogFH, "DB Details : OSWP-No '$key', Version '$dbVersion{$key}'\n");
  }
  return(\%dbVersion);
}

=head2 GetLtConfig($)

=begin html
Retrieve the actual LT configuration using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  The hask key is the slot position and holds the detected
LT-type

=end html

=cut

sub GetLtConfig($) {
  my ($telnet) = @_;
  my %ltConfig;
  my ($slot, $type, $id);
  my $response = RetrieveInfo($telnet, "show equipment slot", undef);

  if (!defined($response)) {
    return undef;
  }
  foreach (@{$response}) {
    # 1/1/5          eblt-e      no      waiting-for-sw         available    
    if (/^(\d+\/\d+\/\d+)\s*(\w+)\-(\w).*/) {
      $slot = $1, $type = $2, $id = $3;
      $ltConfig{$slot} = "$type-$id" if (($type =~ /lt|ls/) && (length($type) > 3));
    } elsif (/^lt:(\d+\/\d+\/\d+)\s*(\w+)\-(\w).*/) { # 1.2.5
      $slot = $1, $type = $2, $id = $3;
      $ltConfig{$slot} = "$type-$id" if (($type =~ /lt|ls/) && (length($type) > 3));
    }
  }
  return(\%ltConfig);
}


sub GetFeltBSlotDetails($)
{
  my ($session) = @_;
  my ($response, $ltNo) = (undef, "");
  my @feltbLts= ();
  $response = RetrieveInfo($session, "info configure equipment slot flat", undef);

  if (!defined($response)) {
    return "";
  }
  foreach (@{$response}) {
    if ($_ =~ /\s+planned-type\s+felt-b\s+/ and $_ =~/\s+capab-profile\s+ethernet_uplink\s+/) {
       my @details = split( /\s+/, $_ );
       $ltNo= $details[3];
       push @feltbLts, $ltNo;
    }
  }
  return \@feltbLts;
}

=head2 GetSecureDebugSetting($)

=begin html
=head2 GetSecureDebugSetting($)

=begin html
Retrieves Secure debug setting configuration using the opened telnet session $telnet.
The value returned as 1 in case Secure setting (ssh or udp-ssh) else it return as 0

=end html

=cut

sub GetSecureDebugSetting($) {
  my ($telnet) = @_;
  my ($slot, $type, $id);
  my @response = SendCommand($telnet, "info configure system security ssh access detail");
  return undef if ( $ERROR ne "" );
  my $respStr = "";
  my $cliDebugSetting = "";
  foreach my $line (@response) {
    $respStr .= $line;
    if ( $line =~ /(^\s*debug)\s+(\w+\-\w+)/ || $line =~/(^\s*debug)\s+(\w+)/ ) {
        $cliDebugSetting = $2;
    }
  }
  if ( $cliDebugSetting eq "ssh" || $cliDebugSetting eq "udp-ssh" ) {
   return 1;
  } 
  return 0;
}

=head2 GetSlotConfig($)

=begin html
=head2 GetSlotConfig($)

=begin html
Retrieve the planned equipment slot configuration using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  The hask key is the slot position and holds the detected
LT-type

=end html

=cut

sub GetSlotConfig($) {
  my ($telnet) = @_;
  my %slotConfig;
  my ($slot, $type, $id);
  my $response = RetrieveInfo($telnet, "info configure equipment slot flat", undef);

  if (!defined($response)) {
    return undef;
  }
  foreach (@{$response}) {
    #configure equipment slot lt:1/1/4 planned-type eblt-c unlock capab-profile 8vc_default
    if (/^configure equipment slot (\d+\/\d+\/\d+) planned-type (\w+)\-(\w).*/) {
      $slot = $1, $type = $2, $id = $3;
      if ($slot =~ /(\d+)\/(\d+)\/(\d+)/) { 
	my $slotNum = $3;
	if ($slotNum > 1) {
	  $slotConfig{$slot} = "$type-$id" if (length($type) > 3);
	}
      }
    } elsif (/^configure equipment slot lt:(\d+\/\d+\/\d+) planned-type (\w+)\-(\w).*/) { # 1.2.5
      $slot = $1, $type = $2, $id = $3;
      $slotConfig{$slot} = "$type-$id" if (length($type) > 3);
    }
  }
  return(\%slotConfig);
}


sub GetAlalrmDetails($) {
  my ($telnet) = @_;
  my $alrm_list;

  my $response = RetrieveInfo($telnet, "show alarm current table", undef);
  if (!defined($response)) {
    return undef;
  }
  my @alarmDetails = ();
  foreach (@{$response}) {
    if( /^\d+(\s+)([^\s]+)/ ) {
        my $alrmType = $2; 
        next if($alrmType =~ /mgmt\-sec\-access/); 
        push @alarmDetails, $alrmType if(!grep( /$alrmType/, @alarmDetails)) ;
    }
  }
  #my %uniqueAlarms = map { $_ => 1 } @alarmDetails;
  #foreach my $alarmType ( keys %uniqueAlarms ) {
    # my $response = RetrieveInfo($telnet, "show alarm current $alarmType", undef);
  #   $uniqueAlarms{$alarmType} = AlarmCurrTableDetail($telnet, $alarmType); 
    # if (!defined($response)) {
    #   return undef;
    # }
  #}
  
  return \@alarmDetails;
}


sub AlarmCurrTableDetail($$) {
  my ($telnet, $alarmType) = @_;
  my $response = RetrieveInfo($telnet, "show alarm current $alarmType", undef);
  if (!defined($response)) {
    return undef;
  }

  my $alarmOutput = "";
  my (@var_value);
  my $list; 

  foreach my $line (@{$response}) {
       next if(($line =~ /additional\-info/) || ($line =~ /last\-updated\-on/) || 
               ($line =~ /SessionId \:/) || ($line =~ /Type \:/));
       if($line =~ /[^\s]+\s+:/) {
           chomp $line;
           $alarmOutput .= $line;
  #     } elsif (($line =~ /^[\-]+/) || ($line =~ /^[\=]+/) || ($line =~ /^$alarmType/)) {
       } elsif (($line =~ /^[\=]+/) || ($line =~ /^$alarmType/)) {
          if($alarmOutput ne "") {
             my ($h_alm) = {};
             $alarmOutput =~ s/ : /=/g;
             (@var_value) = $alarmOutput =~ /([^ ]+)=([^ ]*)/g;
             if(@var_value % 2 != 0) {
                #return undef; 
                return 1; 
             } 
                $h_alm->{"blk"} = {@var_value};
                my $almidx = $h_alm->{blk}{'index'};
                $h_alm->{"idx"} = $almidx; 
              
             my ($idx, $h) = ($almidx, $h_alm) ;
             push(@{$list}, $h); 
             $alarmOutput = "";
          }
       }
  } 

  $list ? return $list : return 1;
}

=head2 AlarmStatus($$) {

=begin html
Retrieves the current status of the alarms (@emergencyCallList) which matches the key ($key) using the opened telnet session ($telnet). This function 
returns 1 in case if any of the emergency call alarms in the list @emergencyCallList is active and 0 if the alarm is not active and 
the entry is not present.

=end html

=cut

sub AlarmStatus($$) {
	my ($telnet, $key) = @_;
	my @rspReceived = ();
  
    foreach my $alarm_types ( @emergencyCallList ) {
		@rspReceived = SendCommand($telnet, "show alarm current $alarm_types | match exact:$key");
        return 0 if( $ERROR ne "" );
        if ( scalar(@rspReceived) > 0) {
			foreach my $line(@rspReceived) {
				chomp $line;
				#  subs-inittimeout : no                     emergency-call : yes
				if ($line =~ /(emergency-call|ont-gen-emergency-call|sfu-sup-emergency-call|soho-sup-emergency-cal|mdu-sup-emergency-call|emergency-call-sec)\s*:\s*([^\s]+)/) {
					if ($2 =~ /^yes/) {
   						ToolCommon::Log($DEBUG, "Emergency call alarm is active.\n ") if defined($DEBUG);
						return 1;
					}
				}
			}
	     }
	}
    return 0;
}


=head2 GetLtConfigState($)

=begin html
Retrieve the actual LT configuration and state using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  The hask key is the slot position and holds the detected
LT-type ("TYPE") and LT status ("ENABLED").

=end html

=cut

sub GetLtConfigState($) {
  my ($telnet) = @_;
  my %ltConfig;
  my $err_state;
  my ($slot, $type, $enabled);
  my $response = RetrieveInfo($telnet, "show equipment slot", undef);

  if (!defined($response)) {
    return undef;
  }
  foreach (@{$response}) {
    # 1/1/5          eblt-e      no      waiting-for-sw         available    
    if (/^(\d+\/\d+\/\d+)\s*(\w+\-\w)\s*(\w+)\s*.*/) {
      $slot = $1, $type = $2, $enabled = $3;
      if (($type =~ /lt|ls/) || (ToolCommon::CheckMiscLT($type))) {
      	$ltConfig{$slot}{TYPE} = $type;
      	$ltConfig{$slot}{ENABLED} = uc($enabled);
      }
    } elsif ((/^lt:(\d+\/\d+\/\d+)\s*(\w+\-\w)\s*(\w+)\s*(\w+).*/) || 
     (/^ctrl:(\d+\/\d+)\s*(\w+\-\w)\s*(\w+)\s*(\w+).*/)) { # 1.2.5
      $slot = $1, $type = $2, $enabled = $3;
      if ((/^lt:(\d+\/\d+\/\d+)\s*(\w+\-\w)\s*(\w+)\s*(\bwaiting-for-sw\b).*/) || (/^ctrl:(\d+\/\d+)\s*(\w+\-\w)\s*(\w+)\s*(\bwaiting-for-sw\b).*/)) { # 1.2.5
        $err_state = $4;
        if ( uc($err_state) eq "WAITING-FOR-SW") {
	  $ltConfig{$slot}{TYPE} = $type;
	  $ltConfig{$slot}{ENABLED} = $enabled;
	  $ltConfig{$slot}{ERRSTATE} = uc($err_state); 
        }
      }
      if (($type =~ /lt|ls/) || (ToolCommon::CheckMiscLT($type))) {
	$ltConfig{$slot}{TYPE} = $type;
	$ltConfig{$slot}{ENABLED} = uc($enabled);
      }
      $err_state = undef;
    }
  }
  return(\%ltConfig);
}

=head2 GetShelfConfig($)

=begin html
Retrieve the actual shelf configuration using the opened telnet session $telnet.
The undefined value is returned in case of error otherwise a reference to hash is returned.  The hask key is the slot position and holds the detected
LT-type.

=end html

=cut

sub GetShelfConfig($) {
  my ($telnet) = @_;
  my %shelfConfig;   
  my ($slot, $type);
  my $response = RetrieveInfo($telnet, "show equipment shelf", undef);

  if (!defined($response)) {
    return undef;
  }
  foreach my $line (@{$response}) {
    chomp $line;
    # 1/1   nfxs-a      yes     no-error               available    
    if ($line =~ /(\d+\/\d+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)/) {
      $shelfConfig{$1} = $2;
    }
  }
  return(\%shelfConfig);
}
  

=head2 GetLanxActiveOSWP($$)

=begin html
Return the active SWP configuration of the SHUB (config-1, config-2 or tftp) using the opened cli session $telnet.  the undefined value is returned
in case or error.<br>
$command is used to cope with the different syntax of the various ISAM releases.

=end html

=cut

sub GetLanxActiveOSWP($$) { # -- 1.1.5 BEGIN
  my ($telnet, $command) = @_;
  my $response = RetrieveInfo($telnet, "show software-mngt shub $command detail", undef);
  my $config = "";

  if (!defined($response)) {
    return undef;
  }
  foreach (@{$response}) {
    if (/(current\-osp)\s*\:\s*(config\-\d)/) {
      $config = $2;
    } elsif ((/(current\-osp)\s*\:\s*(tftp)/) || (/(current\-osp)\s*\:\s*(tffp)/)) {
      $config = "tftp";
    } else {
      $ERROR = "show software-mngt shub $command detail\n" . join("", @{$response});
    }
  }
  return $config;
}                       # - 1.1.5 END

=head2 UploadDb($$$$$)

=begin html
Upload database and wait for completion.<br>
The following parameters are used:
<table>
  <tr><td>db</td><td>which DB to be uploaded: either actual-active, preferred-active, previous-active, failed-active, actual-not-active,
                     preferred-not-active, previous-not-active, failed-not-active</td></tr>
  <tr><td>ip</td><td>IP address of TFTP server</td></tr>
  <tr><td>dir</td><td>target directory of database upload (relative to TFTP system-top directory setting - unix)</td></tr>
  <tr>complete</td><td>dm_complete (1) or dm.tar (0)?</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub UploadDb($$$$$) {
  my ($telnet, $db, $ipAddress, $dir, $complete) = @_;
#  $dir =~ s/^\/// if ($dir =~ /^\//);
  $dir =~ s/\/$// if ($dir =~ /\/$/);
  my $emptyResponses = 0;
  my $toolData = FamilyCommon::GetToolData();
  my $dbFileName = sprintf("$dir/%s", $complete ? "dm_complete.tar" : "dm.tar");
  if ( ! $params->{NOTFTP} ) {
     $dbFileName = $toolData->{TFTPDIR}."/$dbFileName";
  }
  if ( !$params->{UMASK_CHMOD} and ! chmod(0666, $dbFileName) ){
     LogWarn( $nodeLogFH,  "Failed to chmod 0666 for directory '$dbFileName' : $!\n");
  }
  my $cmd = sprintf("admin software-mngt database upload $db:$ipAddress:$dir/%s", $complete ? "dm_complete.tar" : "dm.tar");
  my @response = SendCommand($telnet, $cmd);
  return 0 if ($ERROR ne "");
  my $retries = 0;
  sleep 10;
  #-- check for completed upload (or error)
  my $uploadProgress = 1;
  while ($uploadProgress) {
    my $response = RetrieveInfo($telnet, "show software-mngt upload-download detail", undef);
    if (!defined($response) or $ERROR ne "") {
      return 0 if ($retries > 3); # retry a couple of times, seems that the first one may yield no response...
      $retries++;
      next;
    }
    $retries = 0;
    my ($progress, $error ) = (undef, undef );
    foreach (@{$response}) {
      if (/\s+upload-progress\s:\s([\w-]+)/) {
	      $progress = $1;
      }
      if (/.*\s+upload-error\s:\s([\w-]+)/) {
	      $error = $1
      }
      if (/ ^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/) {
	      # Alarm read, skip it an go for a new command
	      $progress = "upload-ongoing";
	      $error = "no-error";
      }
    }
    #	print "Progress=$progress,Error=$error\n";
    if (defined($progress) && defined($error)) {
      $ERROR = "";
      if ($progress eq "upload-success" && $error eq "no-error") {
         $uploadProgress = 0;
      } elsif ($progress eq "upload-ongoing" && $error eq "no-error") {
           sleep 10;
      } elsif ($progress eq "upload-fail") {
	       $ERROR = "DB Upload failed: $error";
           $uploadProgress = 0;
      } else {
	       $ERROR = "Internal error: Unknown progress value";
           $uploadProgress = 0;
      }
    } else {
	  $ERROR = "No progress information available.\nFull response: " . join("", @{$response});
      $emptyResponses++;
      $uploadProgress = 0 if ($emptyResponses > $MAXEMPTYRESP);
    }
  }
  my $umaskForFile =  "0".(777 - sprintf("%O", umask));
  if ( !$params->{UMASK_CHMOD} and !chmod( oct($umaskForFile), "$dbFileName") ) {
     LogWarn( $nodeLogFH,  "Failed to chmod $umaskForFile for directory '$dbFileName' : $!\n");
  }
  if ($ERROR ne "" ) {
     return 0;
  } else {
     return 1;
  }
}

=head2 CommitSWP($$)

=begin html
Commit OSWP with index $swpIndex using the opened cli session $telnet.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub CommitSWP($$) {
  my ($telnet, $swpIndex) = @_;
  my @response = SendCommand($telnet, "admin software-mngt oswp $swpIndex commit");
  if ( $ERROR ne "" ) {
     return 0;
  } else {
     return 1;
  }
}

=head2 AbortDownloadSWP($$)

=begin html
Abort download of OSWP with index $swpIndex using the opened cli session $telnet.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub AbortDownloadSWP($$) {
  my ($telnet, $swpIndex) = @_;
  my @response = SendCommand($telnet, "admin software-mngt oswp $swpIndex abort-download");
  if ( $ERROR ne "" ) {
     return 0;
  } else {
     return 1;
  }
}

=head2 ConfigureSWPDownload($$$$)

=begin html
Configure an OSWP for SW download.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>swpIndex</td><td>OSWP index (1 or 2)</td></tr>
  <tr><td>tftpserver</td><td>IP address of TFTP server</td></tr>
  <tr><td>downloadset</td><td>the download set: min-set, actual-set or complete-set</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

my $DL_ATTEMPTS = 5;
my $DL_SLEEP = 20;

sub ConfigureSWPDownload($$$$) {
  my( $telnet, $swpIndex, $tftpserver, $downloadset) = @_;
  my $cmd = "configure software-mngt oswp $swpIndex primary-file-server-id $tftpserver";
  my $tries = 0;
  while ($tries < $DL_ATTEMPTS) {
      my @response = SendCommand($telnet, $cmd);
      if ($ERROR ne "" ) {
         if ( $ERROR =~ /busy : please retry later/ || $ERROR =~ /resource is currently/ ) {
            $tries++;
            if ($tries >= $DL_ATTEMPTS) {
              $ERROR = "retries exhausted: $cmd\n" . join("", @response);
              return 0;
            } else {
              sleep $DL_SLEEP;
            }
          } else {
            return 0;
          }
      } else {
         $tries = $DL_ATTEMPTS + 1;
      }
  }
  sleep 2;
  return 1;
}

sub ConfigureSWPDownloadVerify($$$) {
  my( $cliSession, $oswpIndex, $autoVerifyState) = @_;
  my $cmd = "configure software-mngt oswp $oswpIndex auto-verify $autoVerifyState";
  my @response = SendCommand($cliSession, $cmd);
  if ($ERROR ne "" ) {
     $ERROR = "Autoverify ($autoVerifyState) after download is failed : $ERROR";
     return 1;
  }
  return 0;
}


=head2 WaitForDownload($$)

=begin html
Wait for DB or SW download completion<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>showCmd</td><td>command section to deal with release differences</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForDownload($$) {
  my ($telnet, $showCmd) = @_;
  #-- check for completed download (or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $progress = undef;
  my $error;
  my $fileServer = "";
  my $errorFile = "";
  my $toReturn = undef;
  my $emptyResponses = 0;

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = RetrieveInfo($telnet, "show software-mngt $showCmd detail", undef);
      if (!defined($response)) {
	if ($retries > 4) {
	  # 5 retries, seems that the first ones may yield no response...
	  $ERROR = "Maximum command retransmissions tried for 'show software-mngt $showCmd detail'";
	  $toReturn = 0;
	  return 0;
	}
	$retries++;
	sleep 2;
	next;
      }
      $retries = 0;
      $progress = undef;
      $fileServer = "";
      $errorFile = "";
      foreach (@{$response}) {
	if (/\s+download-progress\s:\s([\w-]+)/) {
	  $progress = $1;
	}
	if (/\s+availability\s:\s([\w-]+)/) {
	  $progress = $1;
	}
	if (/.*\s+download-error\s:\s([\w-]+)/) {
	  $error = $1
	}
	if (/\s+err-file-name\s:\s(.*)/) {
	  $errorFile = $1;
	  $errorFile =~ s/\s//g;
	}
	if (/\s+err-file-server\s:\s(.*)/) {
	  $fileServer = $1;
	  $fileServer =~ s/\s//g;
	}
	if (/^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/) {
	  # Alarm received, continue
	  $progress = "downloading";
	  $error = "no-error";
	}
      }
      ToolCommon::Log($DEBUG, sprintf("progress: %s, error: %s\n", defined($progress) ? $progress : "undef",
				      defined($error) ? $error : "undef")) if defined($DEBUG);
      if (defined($progress) && defined($error)) {
	if (($progress eq "download-success" || $progress eq "enabled") &&
	   $error eq "no-error") {
	  $toReturn = 1;
	  return 1;
	} elsif (($progress eq "download-ongoing" || $progress eq "downloading")) {# && 
#		 $error eq "no-error") {
	  sleep 30;
	} elsif (($progress eq "download-fail") || ($progress eq "disabled")) {
	  if ($progress eq "disabled") {
	    $ERROR = "Download failed. Error: $error (failed file: '$errorFile', server: '$fileServer'), availability: $progress";
	  } else {
	    $ERROR = "Download failed. Error: $error, availability: $progress";
	  }
	  $toReturn = 0;
	  return 0;
	} else {
	  $ERROR = sprintf("Internal error: Unused progress value (%s).\nFull response: %s\n", $progress, join("", @{$response}));
	  $toReturn = 0;
	  return 0;
	}
      } else {
	$emptyResponses++;
	if ($emptyResponses > $MAXEMPTYRESP) {
	  $ERROR = sprintf("No progress information available.\nFull response: %s\n", join("", @{$response}));
	  $toReturn = 0;
	  return 0;
	}
      }
    }
  };
  ToolCommon::Log($DEBUG, "eval block ended ($evalRet)\n") if defined($DEBUG);
  if ($gotSigPipe) {
    ToolCommon::Log($DEBUG, "Local sigpipe handler called\n") if defined($DEBUG);
    $ERROR = "SIGPIPE (WaitForDownload)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 DownloadCompleteDb($$$)

=begin html
Download dm_complete database and wait till completion.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>ip</td><td>IP address of TFTP server</td></tr>
  <tr><td>dir</td><td>directory containing database to be downloaded (relative to TFTP system-top directory setting - unix)</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.<br>
This routine is obsolete but is still provided for backwards compatibility.

=end html

=cut

sub DownloadCompleteDb($$$) {
    my ($telnet, $ipAddress, $dir) = @_;
    $dir =~ s/^\/// if ($dir =~ /^\//);
    my @response;
    my $gotSigPipe = 0;
    my $tries = 0;
    my $toolData = FamilyCommon::GetToolData();
    my $dbFileName = "$dir/dm_complete.tar";
    if ( ! $params->{NOTFTP} ) {
       $dbFileName = $toolData->{TFTPDIR}."/$dbFileName";
    }
    if ( !$params->{UMASK_CHMOD} and ! chmod( 0666, $dbFileName) ){
       LogWarn( $nodeLogFH,  "Failed to chmod 0666 for directory '$dbFileName' : $!\n");
    }
    my $cmd = "admin software-mngt database download $ipAddress:$dir/dm_complete.tar";

    while ($tries < $DL_ATTEMPTS) {
         eval {
           local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
           @response = SendCommand($telnet, $cmd);
         };
         if ($gotSigPipe) {
           ToolCommon::Log($DEBUG, "Local sigpipe handler called\n") if defined($DEBUG);
           $ERROR = "SIGPIPE (DownloadCompleteDb)";
           kill 13, $$;
         }
        if ($ERROR ne "" ) {
           if ( $ERROR =~ /busy : please retry later/ || $ERROR =~ /resource is currently/ ) {
              $tries++;
              if ($tries >= $DL_ATTEMPTS) {
                $ERROR = "retries exhausted: $cmd\n" . join("", @response);
                return 0;
              } else {
                sleep $DL_SLEEP;
              }
            } else {
              return 0;
            }
        } else {
           $tries = $DL_ATTEMPTS + 1;
        }
    }
    sleep 2;
    my $status = WaitForDownload($telnet, 'upload-download');
    my $umaskForFile = "0".( 777 - sprintf("%O", umask) );
    if ( !$params->{UMASK_CHMOD} and ! chmod( oct($umaskForFile), $dbFileName) ){
       LogWarn( $nodeLogFH,  "Failed to chmod $umaskForFile for directory '$dbFileName' : $!\n");
    }
    return $status;
}

=head2 DownloadDb($$$$)

=begin html
Download database and wait till completion.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>ip</td><td>IP address of TFTP server</td></tr>
  <tr><td>dir</td><td>directory containing database to be downloaded (relative to TFTP system-top directory setting - unix)</td></tr>
  <tr><td>complete</td><td>dm_complete (1) or dm.tar (0)?</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub DownloadDb($$$$) {
  my ($telnet, $ipAddress, $dir, $complete) = @_;
#  $dir =~ s/^\/// if ($dir =~ /^\//);
  $dir =~ s/\/$//;
  my $toolData = FamilyCommon::GetToolData();
  my $dbFileName = sprintf("$dir/%s", $complete ? "dm_complete.tar" : "dm.tar");
  if ( ! $params->{NOTFTP} ) {
     $dbFileName = $toolData->{TFTPDIR}."/$dbFileName";
  }
  if ( !$params->{UMASK_CHMOD} and ! chmod( 0666, $dbFileName) ){
     LogWarn( $nodeLogFH,  "Failed to chmod 0666 for directory '$dbFileName' : $!\n");
  }
  my $cmd = sprintf("admin software-mngt database download $ipAddress:$dir/%s", $complete ? "dm_complete.tar" : "dm.tar");
  my @response;
  my $tries = 0;

  while ($tries < $DL_ATTEMPTS) {
    @response = SendCommand($telnet, $cmd);
    if ($ERROR ne "" ) {
       if ( $ERROR =~ /busy : please retry later/ || $ERROR =~ /resource is currently/ ) {
          $tries++;
          if ($tries >= $DL_ATTEMPTS) {
            $ERROR = "retries exhausted: $cmd\n" . join("", @response);
            return 0;
          } else {
            sleep $DL_SLEEP;
          }
        } else {
          return 0;
        }
    } else {
       $tries = $DL_ATTEMPTS + 1;
    }
  }
  sleep 2;
  my $status = WaitForDownload($telnet, 'upload-download');
  my $umaskForFile = "0".( 777 - sprintf("%O", umask) );
  if ( !$params->{UMASK_CHMOD} and ! chmod( oct($umaskForFile), $dbFileName) ){
     LogWarn( $nodeLogFH,  "Failed to chmod $umaskForFile for directory '$dbFileName' : $!\n");
  }
  return $status;
}

=head2 DownloadSWP($$$)

=begin html
Download OSWP and wait for completion.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>swpIndex</td><td>OSWP index (1 or 2)</td></tr>
  <tr><td>oswpName</td><td>name of the OSWP</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub DownloadSWP($$$) {
  my ($telnet, $swpIndex, $oswpName) = @_;
  my $tries = 0;
  my @response;

  while ($tries < $DL_ATTEMPTS) {
      my $cmd = "admin software-mngt oswp $swpIndex download $oswpName";
      @response = SendCommand($telnet, $cmd ); 
      if ($ERROR ne "" ) {
         if ( $ERROR =~ /busy : please retry later/ || $ERROR =~ /resource is currently/ ) {
            $tries++;
            if ($tries >= $DL_ATTEMPTS) {
              $ERROR = "retries exhausted: $cmd\n" . join("", @response);
              return 0;
            } else {
              sleep $DL_SLEEP;
            }
          } else {
            return 0;
          }
      } else {
         $tries = $DL_ATTEMPTS + 1;
      }
  }
  sleep 2;
  Transaction($telnet, "release");  # 1.2.5
  sleep 2;
  return WaitForDownload($telnet, "oswp $swpIndex");
}

=head2 WaitForShubCopy($$$)

=begin html
Wait for SHUB copy command to complete.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>config</td><td>cli command section to deal with different syntaxes</td></tr>
  <tr><td>string</td><td>either 'copy-status' or 'sw-download-status' (release differences)</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.<br>
Note that this command is only needed for ISAM release &lt; R2.2 or &lt;= R2.2 when booting from own config. Not needed when &gt; R2.2

=end html

=cut

sub WaitForShubCopy($$$) {                 # -- 1.0.5 START
  my ($telnet, $config, $string) = @_;
  #-- check for completed download (or error)
  my $completed = 0;
  my $progress = undef;
  my $ok = 0;
  my $tries = 0;

  sleep(5);
  while (!$completed) {
    my $response = RetrieveInfo($telnet, "show software-mngt shub $config detail", undef);
    if(!defined($response)) {
      return 0;
    }
    foreach (@{$response}) {
      if (/\s+$string\s:\s([\w]+).*/) {
	$progress = $1;
      } elsif ( /^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/ ) {
	# Line contains 0.0.0.0 in the downloaded sw field.
	$progress = "inprogress";
      } elsif (/no response/) {
	$tries++;
	$progress = $tries < 5 ? "inprogress" : "";
      }
    }
    if (defined($progress)) {
      if ($progress eq "successful") {
	$completed = 1;
	$ok = 1;
      } elsif ($progress eq "inprogress") {
	sleep 2;
      } elsif ($progress eq "download-fail") {
	$ERROR = "Copy or download failed";
	$completed = 1;
      } elsif ($progress eq "failed") {
	$ERROR = "Copy or download failed";
	$completed = 1;
      } else {
	$ERROR = sprintf("Internal error (Unknown progress value: %s", join("", @{$response}));
	$completed = 1;
      }
    } else {
      $ERROR = sprintf("Internal error (unknown response received: %s", join("", @{$response}));
      $completed = 1;
    }
  }
  return $ok;
}

=head2 GetLanxDbFile($$$$)

=begin html
Copy file from SHUB and wait till completion.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>from</td><td>the source location</td></tr>
  <tr><td>to</td><td>the destination</td></tr>
  <tr><td>config</td><td>'config-file' or '' (to deal with different command syntaxes)</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.<br>
Note that this command is only needed for ISAM release &lt; R2.2 or &lt;= R2.2 when booting from own config. Not needed when &gt; R2.2

=end html

=cut

sub GetLanxDbFile($$$$) {
  my ($telnet, $from, $to, $config) = @_;
  $to =~ s/^\/// if ($to =~ /^\//);
  my @response = SendCommand($telnet, "admin software-mngt shub copy from shub:$from to $to");
  if ( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
  sleep 2;
  return WaitForShubCopy($telnet, $config, "copy-status");
}

=head2 ChangeShubConfig($$)

=begin html
Change the requested SHUB condifguration to $config ('config-1', 'config-2' or 'tftp') using telnet session $telnet.
Returns 1 in case of success or 0 in case of failure.<br>
Note that this command is only needed for ISAM release &lt; R2.2 or &lt;= R2.2 when booting from own config. Not needed when &gt; R2.2

=end html

=cut

sub ChangeShubConfig($$) {
  my ($telnet, $config) = @_;
  my @response = SendCommand($telnet, "admin software-mngt shub select-config $config");
  if ( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 RebootShub($)

=begin html
Reboot the SHUB using telnet session $telnet.
Returns 1 in case of success or 0 in case of failure.<br>
Note that this command is only needed for ISAM release &lt; R2.2 or &lt;= R2.2 when booting from own config. Not needed when &gt; R2.2

=end html

=cut

sub RebootShub($) {
  my ($telnet) = @_;
  my @response = SendCommand($telnet, "admin equipment reboot-shub systemreset");
  if ( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 DownloadShubSW($$$$)

=begin html
Download SHUB SW and wait for completion.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
  <tr><td>swName</td><td>SHUB SW file name</td></tr>
  <tr><td>tftpIp</td><td>IP address of TFTP server</td></tr>
  <tr><td>config</td><td>'config-file' or '' to deal with changed command syntax</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.<br>
Note that this command is only needed for ISAM release &lt; R2.2 or &lt;= R2.2 when booting from own config. Not needed when &gt; R2.2


=end html

=cut

sub DownloadShubSW($$$$) {
  my ($telnet, $swName, $tftpIp, $config) = @_;
  $swName =~ s/^\/// if ($swName =~ /^\//);
  my @response = SendCommand($telnet,
    "admin software-mngt shub sw-package $tftpIp:$swName download");
  if ( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }

  sleep 5;
  return WaitForShubCopy($telnet, $config, "sw-download-status");
}

=head2 GetLanxSwVersion($)

=begin html
Retrieve SHUB SW version using telnet session $telnet.<br>Returns SHUB SW version in case of success, '' in case of failure

=end html

=cut

sub GetLanxSwVersion($) {
  my ($telnet) = @_;
  my $response = RetrieveInfo($telnet, "show system shub entry version", undef);
  my $version = "";
  my $rspLine = "";

  if (!defined($response)) {
    return "";
  }
  foreach (@{$response}) {
    if (/fw-ver\s*:\s*(LANX_R.*)/) {
      $version = $1;
    } elsif (/sw-release-num\s*:\s*(LANX_R.*)/) {
      $version = $1;
    } elsif (/sw-release-num\s*:\s*(LANX_F.*)/) {
      # ANSI load indentification
      $version = $1;
    } elsif (/fw-ver\s*:\s*(Lanx_R.*)/) {
      $version = uc($1);
    } elsif (/sw-release-num\s*:\s*(Lanx_R.*)/) {
      $version = uc($1);
    } elsif (/sw-release-num\s*:\s*(Lanx_F.*)/) {
      # ANSI load indentification
      $version = uc($1);
    } elsif (/sw-release-num\s*:\s*(R.*)/) {
      $version = $1;
    }
  }
  $ERROR = join("", @{$response}) if ($version eq "");
  return $version;
}

=head2 Transaction($$)

=begin html
Acquire or release command transaction as specified by $action using telnet session $telnet.<br>Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub Transaction($$) {
  my ($telnet, $action ) = @_;
  my $cmd = "admin transaction configure-lock $action";
  my $tries = 0;
  while ($tries < $DL_ATTEMPTS) {
      my @response = SendCommand($telnet, $cmd);
      if ($ERROR ne "" ) {
         if ( $ERROR =~ /busy : please retry later/ || $ERROR =~ /resource is currently/ ) {
    	    $tries++;
    	    if ($tries >= $DL_ATTEMPTS) {
    	      $ERROR = "retries exhausted: $cmd\n" . join("", @response);
    	      return 0;
    	    } else {
    	      sleep $DL_SLEEP;
    	    }
    	  } else {
    	    return 0;
    	  }
      } else {
	     $tries = $DL_ATTEMPTS + 1;
      }
  }
  sleep 2;
  return 1;
}


sub ActivateSwp($$) {
  my ($telnet, $oswpIndex) = @_;
  my $cmd = "admin software-mngt oswp $oswpIndex activate with-linked-db";
  my $tries = 0;
  while ($tries < $DL_ATTEMPTS) {
      my @response = SendCommand($telnet, $cmd);
      if ($ERROR ne "") {
         if ( $ERROR =~ /busy : please retry later/ || $ERROR =~ /resource is currently/ ) {
            $tries++;
            if ($tries >= $DL_ATTEMPTS) {
              $ERROR = "$cmd retries exhausted: $ERROR\n";
              return 0;
            } else {
              sleep $DL_SLEEP;
            }
         } else {
            $ERROR = "$cmd failed with $ERROR\n";
            return 0;
         }
      } else {
         $tries = $DL_ATTEMPTS + 1;
      }
  }
  sleep 2;
  return 1;
}

  
=head2 ShubDbaseSave($)

=begin html
Save SHUB database using telnet session $telnet.<br>Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub ShubDbaseSave($) {
  my ($telnet) = @_;
  my @response = SendCommand($telnet, 
			     "admin software-mngt shub database save");
  if( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 InhibitAlarms($)

=begin html
Inhibit generation of CLI alarms using telnet session $telnet.<br>No error indication is returned as this command is not supported by all releases.

=end html

=cut

sub InhibitAlarms($) {
  my ($telnet) = @_;

  my @response = SendCommand($telnet, "environment inhibit-alarms");
  # Don't check response - it's not supported in all releases.
}

=head2 AllowAlarms($)

=begin html
Allow generation of CLI alarms using telnet session $telnet.<br>No error indication is returned as this command is not supported by all releases.

=end html

=cut

sub AllowAlarms($) {
  my ($telnet) = @_;

  my @response = SendCommand($telnet, "environment no inhibit-alarms");
  # Don't check response - it's not supported in all releases.
}

=head2 GetConfig($$)

=begin html
Generic configuration retrieval command using telnet session $telnet and sending $configPart.<br>
The response as received is returned (all parsing and error checking is left to the caller).<br>
Introduced in the context of the TI DB corruption for providing a workaround by uploading the complete configuration and generating a replay file.
It is not called otherwise.

=end html

=cut

sub GetConfig($$) {
  my ($telnet, $configPart) = @_;
  my @response = SendCommand($telnet, "$configPart");
  return @response;
}

=head2 GetConfigFile($$$)

=begin html
Generic configuration retrieval command using telnet session $telnet and sending $configPart while redirecting the output to a file $fileName on the
NT disk.  File upload is left to the caller.<br>
Introduced in the context of the TI DB corruption for providing a workaround by uploading the complete configuration and generating a replay file.
It is not called otherwise.

=end html

=cut

sub GetConfigFile($$$) {
  my ($telnet, $configPart, $fileName) = @_;
  my @response = SendCommand($telnet, "$configPart | tee $fileName | discard");
  if( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 DeleteCmdFile($$)

=begin html
Delete file $fileName from the NT disk using telnet session $telnet.<br>
Returns 1 in case of success, 0 in case of failure.<br>
Introduced in the context of the TI DB corruption for providing a workaround by uploading the complete configuration and generating a replay file.
It is not called otherwise.

=end html

=cut

sub DeleteCmdFile($$) {
  my ($telnet, $fileName) = @_;
  my @response = SendCommand($telnet, "delete $fileName");
  if( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 SetSessionTimeout($$)

=begin html
Set the NT transactin timer to value $toValue using telnet session $telnet.<br>
Returns 1 in case of success, 0 in case of failure.
=end html

=cut

sub SetSessionTimeout($$) {
  my ($telnet, $toValue) = @_;
  my @response = SendCommand($telnet, "environment terminal-timeout timeout:$toValue");
  if( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 GetShelfType($)

=begin html
Retrieve the shelf type of shelf 1/1.  Returns the shelf-type, 0 (in case of time-out), undef in case of error.<br>
Introduced in the context of the TI DB corruption for providing a workaround by uploading the complete configuration and generating a replay file.
It is not called otherwise.

=end html

=cut

sub GetShelfType($) {
  my ($telnet) = @_;
  my @response = SendCommand($telnet, "info configure equipment shelf 1/1 flat", 1);
  return 0 if( $ERROR ne "" );
  my $respStr = "";
  my $shelfType = "";
  if (! scalar(@response)) {
    $ERROR = "info configure equipment shelf 1/1 flat yields no response\n";
    return undef;
  }
  foreach (@response) {
	$respStr .= $_;
	if ($respStr =~ /\s+planned-type\s(\w+\-\w)/) {
		$shelfType = $1;
     }
  }
  if (!defined($shelfType)) {
    $ERROR = "info configure shelf 1/1 flat yields unrecognized type\n@response";
	return 0;
  }
  return $shelfType;
}

=head2 InhibitMore($)

=begin html
Disable the 'more' prompting in case of info retrieval commands using telnet session $telnet (when the output is more then a page, the CLI by default
waits for operator input).<br>
Returns 1 in case of success, 0 in case of failure.

=end html

=cut

sub InhibitMore($) {
  my ($telnet) = @_;
  my @response = SendCommand($telnet, "environment print no-more");
  if( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 SetTransactionTimeout($$)

=begin html
Set the CLI transaction timeout to $seconds using telnet session $telnet.<br>
Returns 1 in case of success, 0 in case of failure.

=end html

=cut

sub SetTransactionTimeout($$) {
  my ($telnet, $seconds) = @_;
  my $toVal = $seconds;
  my @response = SendCommand($telnet, "configure system transaction time-out-value $toVal");
  if( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
  return 1;
}

=head2 GetTransactionTimeout($)

=begin html
Set the CLI transaction timeout to $seconds using telnet session $telnet.<br>
Returns the timeout value in case of success, undef in case of failure (0 means indefinite).

=end html

=cut

sub GetTransactionTimeout($) {
  my ($telnet) = @_;
  my $toVal = undef;
  my @response = SendCommand($telnet, "info configure system transaction detail");
  if( $ERROR ne "" ) {
      return 0;
  }
  my $respStr = "";
  foreach (@response) {
    $respStr .= $_;
  }
  if ($respStr =~ /time-out-value (\d+).*/) {
    $toVal = $1;
  } elsif ($respStr =~ /time-out-value indefinite/) {
    $toVal = 0;
  } elsif ($respStr =~ /no time-out-value \# value=(\d+).*/) {
    $toVal = $1;
  } else {
    $ERROR = "Unexpected response for 'info configure system transaction detail'\n" . join("", @response);
  }
  return $toVal;
}

=head2 GetSfpAssignment($)

=begin html
Return the SFP assignment table using telnet session $telnet.<br>
Returns 0 in case of failure or a pointer to an assignment hash containing the following data for each entry:
<ul>
  <li>EXPSLOT: the expansion slot number</li>
  <li>SFPNUMBER: the SFP number</li>
</ul>

=end html

=cut

sub GetSfpAssignment($) {
  my ($telnet) = @_;
  my ($sfpNumber, $expSlot);
  my $numOfSfpAssignments = 0;

  # --------------------------------------------------------------------------------
  # external-link-host
  # --------------------------------------------------------------------------------
  #       faceplate-number : 5                           shub-port : 22
  #        downlink-status : uplink
  #               exp-slot : 0/0/0                   host-sfp-type : unknown
  #         cabling-status : no-mismatch
  #            phy-address :
  # --------------------------------------------------------------------------------
  my $pattern1 = '\s*faceplate-number\s*\:\s*(\d*)\s*.*';
  my $pattern2 = '\s+exp-slot\s+\:\s+(\w\/\w/\w)\s*.*';

  my %sfpAssignments = (''=>{'SFPNUMBER'=>'','EXPSLOT'=>''},'TOTAL'=>0);
  my @strings =  IsamCli::SendCommand($telnet,"show equipment external-link-host detail");
  return 0 if( $ERROR ne "" );
  foreach(@strings) {
    if (/$pattern1/) {
      $sfpNumber = $1;
    } elsif (/$pattern2/) {
      $expSlot = $1;
      $sfpAssignments{$numOfSfpAssignments}{'EXPSLOT'} = $expSlot;
      $sfpAssignments{$numOfSfpAssignments}{'SFPNUMBER'} = $sfpNumber;
      $sfpAssignments {'TOTAL'} = $numOfSfpAssignments;
      $numOfSfpAssignments++;
    }
  }
  return (\%sfpAssignments);
}
 
=head2 ConfigureSfpAssignment($$)

=begin html
Configure the SFP as defined by $sfpAssignmentTable using telnet session $telnet.<br>
Returns 0 in case of failure, 1 in case of success.

=end html

=cut

sub ConfigureSfpAssignment($$) {
  my ($telnet, $sfpAssignmentTable) = @_;

  for (my $count = 0; $count < $sfpAssignmentTable->{'TOTAL'}; $count++) {
    my $sfpNumber = $sfpAssignmentTable->{$count}{'SFPNUMBER'};
    my $expSlot = $sfpAssignmentTable->{$count}{'EXPSLOT'};
    if (!($expSlot eq "0/0/0")) {
        substr($expSlot, 4) = substr($expSlot,4) + 3;
        my $cmd = "configure equipment external-link-assign  $sfpNumber remote $expSlot";
        my @response =  IsamCli::SendCommand($telnet,"$cmd");
        return 0 if( $ERROR ne "" );
    }
  }
  return 1;
}
 
=head2 PrintSfpAssignmentTable($)

=begin html
Print the content of the SFP assignment contained $sfpAssignmentTable.<br>
Returns 1.

=end html

=cut

sub PrintSfpAssignmentTable($) {
  my ($sfpAssignmentTable) = @_;

  if (defined($DEBUG)) {
    ToolCommon::Log($DEBUG, "|-------------------------------|\n");
    ToolCommon::Log($DEBUG, "|-------SfpAssignmentTable------|\n");
    ToolCommon::Log($DEBUG, "|-------------------------------|\n");
    ToolCommon::Log($DEBUG, "|faceplate-number    exp-slot   |\n");
    ToolCommon::Log($DEBUG, "|-------------------------------|\n");

    for (my $count = 0; $count < $sfpAssignmentTable->{'TOTAL'}; $count++) {
      my $sfpNumber = $sfpAssignmentTable->{$count}{'SFPNUMBER'};
      my $expSlot = $sfpAssignmentTable->{$count}{'EXPSLOT'};
      ToolCommon::Log($DEBUG, "|......$sfpNumber..........|...$expSlot....|\n");
    }
    ToolCommon::Log($DEBUG, "|-------------------------------|\n");
  }
  return 1;
}

=head2 GetSfpConfig($)

=begin html
Return the SFP host assignment table using telnet session $telnet.<br>
Returns undef in case of failure or a pointer to an assignment hash containing the following data for each entry (hash key is the SFP number):
<ul>
  <li>EXPSLOT: the expansion slot number</li>
  <li>DIRECTION: uplink or downlink</li>
</ul>
Used in the context of NCNC-C to NCNC-E concvertion.

=end html

=cut

sub GetSfpConfig($) {
  my ($telnet) = @_;
  my %fullSfpConfig;
  my @strings = IsamCli::SendCommand($telnet, "info configure equipment external-link-assign flat");
  return undef if( $ERROR ne "" ); 
  foreach my $line (@strings) {
    chomp $line;
    if ($line =~ /configure\s+equipment\s+external-link-assign\s+(\d+)\s+remote\s+(\d+\/\d+\/\d+)/) {
      $fullSfpConfig{$1}{EXPSLOT} = $2;
      $fullSfpConfig{$1}{DIRECTION} = $DOWNLINK;
    }
  }
  return (\%fullSfpConfig);
}

=head2 ConfigureSfp($$)

=begin html
Reconfiguire the NTIO using the CLI session $telnet.  The SFP assignment is expected in a hash containing the following data for each entry (hash key
is the SFP number):
<ul>
  <li>EXPSLOT: the expansion slot number</li>
  <li>DIRECTION: uplink or downlink</li>
</ul>
Returns 0 in case of failure, 1 in case of success.<br>
Used in the context of NCNC-C to NCNC-E concvertion.

=end html

=cut

sub ConfigureSfp($$) {
  my ($telnet, $sfpConfig) = @_;

  foreach my $key (sort keys %{$sfpConfig}) {
    if (defined($sfpConfig->{$key}{DIRECTION})) {
      if (($key >= 2) && ($key <= 7)) {
	     if (! defined(SendCommand($telnet, "configure equipment external-link-host $key direction $sfpConfig->{$key}{DIRECTION}", 1))) {
	       return 0;
	     }
         return 0 if( $ERROR ne "" );
      }
      if ($sfpConfig->{$key}{DIRECTION} eq $DOWNLINK) {
	     if (! defined(SendCommand($telnet, "configure equipment external-link-assign $key remote $sfpConfig->{$key}{EXPSLOT}", 1))) {
	       return 0;
	     }
         return 0 if( $ERROR ne "" );
      }
    }
  }
  return 1;
}

=head2 PrintSfpConfig($)

=begin html
Print the SFP assignment. $sfpConfig is expected as a hash containing the following data for each entry (hash key is the SFP number):
<ul>
  <li>EXPSLOT: the expansion slot number</li>
  <li>DIRECTION: uplink or downlink</li>
</ul>
Returns 1.<br>
Used in the context of NCNC-C to NCNC-E concvertion.

=end html

=cut

sub PrintSfpConfig($) {
  my ($sfpConfig) = @_;

  if (defined($DEBUG)) {
    ToolCommon::Log($DEBUG, "|----------------------------------------|\n");
    ToolCommon::Log($DEBUG, "|----------------SfpConfig---------------|\n");
    ToolCommon::Log($DEBUG, "|----------------------------------------|\n");
    ToolCommon::Log($DEBUG, "|faceplate-number | exp-slot | direction |\n");
    ToolCommon::Log($DEBUG, "|----------------------------------------|\n");
    foreach my $key (keys %{$sfpConfig}) {
      ToolCommon::Log($DEBUG, sprintf("| %15.2d |   %s  | %10s|\n", $key, $sfpConfig->{$key}{EXPSLOT},
				      defined($sfpConfig->{$key}{DIRECTION}) ? $sfpConfig->{$key}{DIRECTION} : ""));
    }
    ToolCommon::Log($DEBUG, "|----------------------------------------|\n");
  }
  return 1;
}

=head2 ForwardTtl0Packets($$)

=begin html
Enable TTL0 forwarding for the SHUB VRF $vrfId using CLI command session $telnet.<br>
Returns 1 in case of success, 0 in case of error.

=end html

=cut

sub ForwardTtl0Packets($$) {
  my ($telnet, $vrfId) = @_;

  my $respStr = "";
  my $vrfName = undef;
  my $vrfMode = undef;
  my $ERROR = "";
  my @response = SendCommand($telnet, "info configure ip shub vrf $vrfId flat");
  return 0 if( $ERROR ne "" );

  foreach my $line (@response) {
    chomp($line);
    if ($line =~ /configure\sip\sshub\svrf\s$vrfId\s(.*)\sname\s(.*)/) {
        $vrfName = $2;
        $vrfMode = $1;
    }
  }
  if (defined($vrfName)) {
      @response = SendCommand($telnet, "configure ip shub vrf $vrfId name $vrfName $vrfMode forward-ttl-zero");
      return 0 if( $ERROR ne "" );
  } else {
      $ERROR = "info configure ip shub vrf $vrfId flat missing required info (VRF name and mode)\n" . join("", @response);
  }
  return ($ERROR eq "" ? 1 : 0);
}

=head2 ConfigureAccessOprId($$)

=begin html
Configures access operatorId through ISAM CLI.
Returns 1 in case of success, 0 in case of error.<br>
<table>
  <tr><td>telnet</td><td>telnet session context</td></tr>
  <tr><td>operatorId</td><td>An unsigned int assigned by ALU. Range 0...65535 </td></tr>
</table>

=end html

=cut

#--------------------------------------------
#   configureAccessOprId
#--------------------------------------------
sub ConfigureAccessOprId($$) {
  my ($telnet, $operatorId) = @_;
  my $cmd = "configure system access-oper-id $operatorId";

  my @response =  IsamCli::SendCommand($telnet,"$cmd");
  if( $ERROR ne "" ) {
      return 0;
  } else {
      return 1;
  }
}

=head2 GetAccessOperId($)

=begin html
Return the access operator ID configured for this node.  Returns undef in case of failure.

=end html

=cut

sub GetAccessOperId($) {
  my ($cli) = @_;
  my @response = SendCommand($cli, "info configure system flat | match exact:\"access-oper-id\"");
  return undef if( $ERROR ne "" );
  my $respStr = "";
  my $accOperId = 0;
  foreach (@response) {
    if (/^.+\saccess-oper-id\s(\d+)/) {
      $accOperId = $1;
    }
  }
  return ($ERROR eq "" ? $accOperId : undef);

}

=head2 NtAppliqueInfo($)

=begin html
Return NT relation applique info using CLI session $cli.<br>
Returns undef in case of failure a a pointer to a hash containing the NT applique info.  Hash key is the applique-type/slot combination.

=end html

=cut

sub NtAppliqueInfo($) {
  my ($cli) = @_;
  my @response = SendCommand($cli, "info configure equipment applique flat");
  return undef if( $ERROR ne "" );
  my $respStr = "";
  my %ntIoInfo = ();
  foreach (@response) {
    if (/^.*(iont:\d+\/\d+)\s+planned\-type\s+(\w+\-\w)/) {
        $ntIoInfo{$1} = $2;
    }
  }
  return ($ERROR eq "" ? \%ntIoInfo : undef);
}

=head2 ConnectionReset()

=begin html
Returns 1 in case the last CLI error indicates that the Telnet session has been closed.

=end html

=cut

sub ConnectionReset() {

  return $ERROR =~ /$CONN_RESET/ ? 1 : 0;
}

=head2 ConnectionStillOpen($)

=begin html
Returns 1 in case the telnet session is still open.

=end html

=cut

sub ConnectionStillOpen($) {
  my ($telnet) = @_;

  my @lines = $telnet->cmd("echo test");
  return ($telnet->errmsg() =~ /$CONN_RESET/) ? 0 : 1;
}

=head2 EmptyCliResponse()

=begin html
Returns indication about CLI response being empty or not

=end html

=cut

sub EmptyCliResponse() {
  return $responseEmpty;
}

=head2 GetProfileSlotNumberMode($$)

=begin html
Returns the CLI mode used by this operator.  The CLI mode determines LT slot numbers used in certain configuration results.

=end html

=cut

sub GetProfileSlotNumberMode($$) {
  my ($telnet, $operatorName) = @_;
  my $profileName =  undef;
  my $slotNumberMode = "legacy-based";

  my $response = RetrieveInfo($telnet, "info configure system security operator $operatorName detail", 1);
  if( !defined $response and $ERROR ne "" ) {
     if ( $ERROR  =~ /instance does not exist/ ) {     ## Added for RADIUS server Authentication
        my $cliSessions = ListCliSessions($telnet); 
        foreach my $sessionId (sort keys %{$cliSessions}) {
           if(defined($cliSessions->{$sessionId}{OPER}) && (($cliSessions->{$sessionId}{OPER}) =~ /$operatorName/)) {
              return $cliSessions->{$sessionId}{MODE};
           } 
        }
     }
     return undef; 
  }
  foreach my $line (@{$response}) {
    if ($line =~ /profile\s([^\s]+)/) {
      $profileName = $1;
      last;
    }
  }
  if (defined($profileName)) {
      $response = RetrieveInfo($telnet, "info configure system security profile $profileName detail", 1);
      return undef if( !defined $response and $ERROR ne "" );
      foreach my $line (@{$response}) {
        if ($line =~ /^\s*no slot\-numbering\s.\svalue=([^\s]+)/) {
	         # no slot-numbering # value=legacy-based
	         $slotNumberMode = $1;
	         last;
        } elsif ($line =~ /^\s*slot\-numbering\s([^\s]+)/) {
	         $slotNumberMode = $1;
	         last;
        }
      }
  } else {
    $ERROR = "info configure system security operator $operatorName detail did not return an operator profile name\n" . join("", @{$response});
  }
  #  $ERROR = "No slot-numbering info found in reponse\n"  . join("", @{$response}) if (!defined($slotNumberMode));
  # We will return legacy-based as we assume the feature is not yet supported by the release.
  return $slotNumberMode;
}

=head2 GetFileTransferSettings($)

=begin html
Retrieve settings with respect to file transfer protocol being used by the node.  The result is returned in a hash structure with the following keys:
<ul>
  <li>PROTOCOL: the protocol used.  No means tftp</li>
  <li>SERVER: a hash of ftp or sftp servers with the following key:
  <ul>
    <li>&gt;IP address&lt;: the user name to be used when using sftp or ftp to this IP address</li>
  </ul>
</ul>
The undefined value is returned in case of error.

=end html

=cut

sub GetFileTransferSettings($) {
  my ($telnet) = @_;
  my $response = RetrieveInfo($telnet, "info configure system security filetransfer flat", 1);
  return undef if( !defined $response and $ERROR ne "" );
  #ISAM default file transfer protocol setting is 'tftp'
  my %protocolSettings = ('PROTOCOL' => 'tftp');

  foreach my $line (@{$response}) {
    if ($line =~ /protocol\s([^\s]+)$/) {
      $protocolSettings{PROTOCOL} = $1;
    } elsif ($line =~ /server\s(\d+\.\d+\.\d+\.\d+)\suser\-name\s([^\s]+)/) {
      $protocolSettings{SERVER}{$1} = $2;
    }
  }
  return \%protocolSettings;
}

=head2 ReadDiskSpace ($)

=begin html
Retrieve the total and available disk space on the system.  Returns undef in case of failure.

=end html

=cut

sub ReadDiskSpace($) {
  my ($telnet) = @_;
  my $response = RetrieveInfo($telnet, "show software-mngt upload-download", 1);
  return undef if( !defined $response and $ERROR ne "" );
  my %diskData = ('TOTAL' => 0,
		  'AVAIL' => 0);

  foreach my $line (@{$response}) {
    if ($line =~ /disk\-space\s:\s(\d+)\s+free\-space\s:\s(\d+)/) {
      $diskData{TOTAL} = $1;
      $diskData{AVAIL} = $2;
    }
  }
  return \%diskData;
}

=head2 ListSwpFiles($$)

=begin html
Returns the list of diskfiles associated with a SW package.  In case no SW package was provided, all SW files are returned.  Returns undef
in case of failure.

=end html

=cut

sub ListSwpFiles($$) {
  my ($telnet, $swPackage) = @_;
  my $cmd = defined($swPackage) ?
  sprintf("show software-mngt swp-disk-file | match exact:%s", basename($swPackage)) :
      "show software-mngt swp-disk-file";
  my $response = RetrieveInfo($telnet, $cmd, 1);
  return undef if( !defined $response and $ERROR ne "" );
  my %swpFiles = ();
  foreach my $line (@{$response}) {
    if ($line =~ /^([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)/) {
      my $package = $1;
      my $fileName = $2;
      my $board = $3;
      my $fileType = $4;
      my $state = $5;
      if(defined($swpFiles{$package}{$fileName}{STATE}) && ($swpFiles{$package}{$fileName}{STATE} eq "available")) {
         $state = "available";
      }
      $swpFiles{$package}{$fileName}{BOARD} = $board;
      $swpFiles{$package}{$fileName}{TYPE} = $fileType;
      $swpFiles{$package}{$fileName}{STATE} = $state;
    }
  }
  return \%swpFiles;
}
=head2 ListSwpDescFiles($$)

=begin html
Returns the list of diskfiles associated with a SW package.  In case no SW package was provided, all SW files are returned.  Returns undef
in case of failure.

=end html

=cut

sub ListSwpDescFiles($$) {
  my ($telnet, $swPackage) = @_;
  #my $cmd = defined($swPackage) ?
  #  sprintf("show software-mngt descriptor-files | match exact:%s", basename($swPackage)) :
  #     "show software-mngt descriptor-files";
  my $cmd = "show software-mngt descriptor-files";
  my $response = RetrieveInfo($telnet, $cmd, 1);
  return undef if( !defined $response and $ERROR ne "" );
  my %swpDescFiles = ();
  foreach my $line (@{$response}) {
    if ($line =~ /^([^\s]+)\s+([^\s]+)\s+([^\s]+)/) {
      my $index = $1;
      my $type = $2;
      my $fileSize = $3;
      $swpDescFiles{$index}{TYPE} = $type;
      $swpDescFiles{$index}{SIZE} = $fileSize;
      ### my $lineNo = basename($index); 
      ##print ("Index:<$lineNo> Type:<$type> FileSize:<$fileSize>\n");
      #if(defined($swpFiles{$package}{$fileName}{STATE}) && ($swpFiles{$package}{$fileName}{STATE} eq "available")) {
         #$state = "available";
      #}
      ###$swpDescFiles{$lineNo}{TYPE} = $type;
      ###$swpDescFiles{$lineNo}{SIZE} = $fileSize;
    }
  }
  return \%swpDescFiles;
}

=head2 ListCliSessions($) 

=begin html
Return a reference to a list of active CLI sessions as a hash:<br>
<ul>
  <li>$list->{$id}{OPER} = operator name</li>
  <li>$list->{$id}{FROM} = IP address</li>
  <li>$list->{$id}{TYPE} = connection type (telnet, ssh, craft)</li>
  <li>$list->{$id}{STAT} = session state</li>
  <li>$list->{$id}{LOGIN} = login time</li>
  <li>$list->{$id}{IDLE} = idle time</li>
  <li>$list->{$id}{MODE} = CLI mode</li>
</ul>

=end html

=cut

sub ListCliSessions($) {
  my ($telnet, $swPackage) = @_;
  my $response = RetrieveInfo($telnet, "show session", 1);
  my %sessionInfo = ();

  foreach my $line (@{$response}) {
    if ($line =~ /$ERR/ || $line =~ /$INVALID_TOKEN/ || $line =~ /$INTERNAL_ERROR/ || $line =~ /$FILE_CLOSED/) {
      $ERROR = "\n" . join("show session", @{$response});
      return undef;
    } elsif ($line =~ /^(^\d+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)/) {
#id|operator   |from            |type  |stat|login-time         |idle-time   |num
#1  isadmin     craft            craft  act  1970-01-02:19:04:15 000-00:00:00 leg
#6  isadmin     172.31.212.116   telnet act  1970-01-02:19:00:48 000-00:09:23 leg
      my $sessId = $1;
      my $operator = $2;
      my $initiator = $3;
      my $type = $4;
      my $state = $5;
      my $loginTime = $6;
      my $idleTime = $7;
      my $cliMode = $8;
      $sessionInfo{$sessId}{OPER} = $operator;
      $sessionInfo{$sessId}{FROM} = $initiator;
      $sessionInfo{$sessId}{TYPE} = $type;
      $sessionInfo{$sessId}{STAT} = $state;
      $sessionInfo{$sessId}{LOGIN} = $loginTime;
      $sessionInfo{$sessId}{IDLE} = $idleTime;
      $sessionInfo{$sessId}{MODE} = $cliMode;
    }
  }
  return \%sessionInfo;
}

=head2 WaitForNTSync($;$)

=begin html
Wait for Synchronization of the Redundant NT<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>TELNET session context</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForNTSync($;$) {
  my ($session_isam, $wait_for) = @_; 
  $wait_for = 1 if( !defined $wait_for);
  my $SyncCompleted = 0; 
  my $response; 
  my $PE_State = undef; 
  my $gotSigPipe = 0;
  my $retCode = 0; 
  my $retries = 0;
  my $fh = FamilyCommon::GetWarnLogFile();
  my $timeout = $NT_SYNC_TIMEOUT;  
  my $slept = 0; 
  my $toolConfigData = FamilyCommon::GetConfigData();


  $timeout = $toolConfigData->{NT_SYNC_TIMEOUT} if defined($toolConfigData->{NT_SYNC_TIMEOUT}) ;

  ToolCommon::Log($DEBUG, "Waiting for NE Synchronization ...\n") if defined($DEBUG);
  my( $standby_status, $standby_change_reason ) = ( "", "" ); 
  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$SyncCompleted) {
      $response = IsamCli::RetrieveInfo($session_isam, "show equipment protection-element", undef);
      if (!defined($response)) {
        ToolCommon::Log($DEBUG, sprintf("Failed to retrieve protection info : %s Retries %d\n", IsamCli::GetError(),$retries)) if defined($DEBUG); 
        if ($retries > 4) {
          $ERROR = sprintf("Maximum command retransmissions tried for 'show equipment protection-element'"); 
          return 0; 
        }
        $retries++;
        sleep 2; 
        next;
      } else {
        foreach (@{$response}) {
           ## No need to process when line not matches
           next if( $_ !~ /^1\/1\/([2|3])\s+/ and $_ !~ /^nt\-([a|b])\s+/ );
           my @values = split( /\s+/ );
           $standby_status = defined $values[1] ? $values[1] : "";
           $standby_change_reason =  defined $values[4] ? $values[4] : "";
           next if (uc($standby_status) eq "PROVIDING-SERVICE" );
           if (uc($standby_status) eq "HOT-STANDBY" ) {
               ToolCommon::Log($DEBUG, "NT Sync has completed\n" ) if defined($DEBUG); 
               $SyncCompleted = 1;
               $retCode = 1;
           } elsif( $wait_for == 0 ) {
               ToolCommon::Log($DEBUG, "Received standby status($standby_status) and reason($standby_change_reason)\n") if defined($DEBUG); 
               $SyncCompleted = 1;
           } elsif( $wait_for ) {
              if (uc($standby_status) eq "IDLE" ) {
                  if( uc($standby_change_reason) =~ /^(PROT-GRP-LCKD)$/ ) {
                      ToolCommon::Log($DEBUG, "NT Sync treated as Success, some NT is protection-group-locked\n") if defined($DEBUG); 
                      $SyncCompleted = 1;
                      $retCode = 1;
                  }
              } elsif (uc($standby_status) ne "COLD-STANDBY" ) {
                 $ERROR = "Invalid standby status ($standby_status) found in NT details" ;
                 $SyncCompleted = 1;
              }
           }
         }
      }
      if( $SyncCompleted == 0 ) {
         $slept = $slept + 25;
         if( $slept > $timeout ){
	        my $ERROR = " : Failed due to NT Sync Timeout. Max time was $timeout Seconds";
	        ToolCommon::Log($DEBUG,sprintf("Failed due to NT Sync Timeout. Max time was %s Seconds\n",$timeout)) if defined($DEBUG);
	        $retCode = 0;
	        return $retCode;
         } else {
            ToolCommon::Log($DEBUG, "NT Sync in Progress\n") if defined($DEBUG); 
            sleep 25;
         }
      }
    }
  };
  ToolCommon::Log($DEBUG, "eval block ended ($evalRet)\n") if defined($DEBUG);
  if ($gotSigPipe) {
    ToolCommon::Log($DEBUG, "Local sigpipe handler called\n") if defined($DEBUG);
    $ERROR = "SIGPIPE (WaitForNTSync)";
    kill 13, $$;
    $retCode = 0;
  }
  return $retCode;
}

sub GetVlanUniqHostId($) {
  my ($telnet) = @_;

  my @response = SendCommand($telnet, "info configure vlan vmac-address-format flat", 1);
  my $hostId =  undef; 
  return $hostId if ($ERROR ne "" );
  if (! scalar(@response)) {
    $ERROR = "info configure vlan vmac-address-format flat yields no response\n";
  }
  if ($response[0] =~ /configure vlan vmac-address-format host-id (\d+)/) {
    $hostId = $1;
  } else { # no host id set
    $hostId = 0; 
  }

  return $hostId; 

}

sub SetU2UHairpin($) {
  my ($telnet) = @_;
  my @response = SendCommand($telnet, "configure system user-user-hairpin");
  if( $ERROR ne "" ) {
     return 0;
  } else {
     return 1;
  }
}

sub CheckDBAutoActivate($) {
  my ($telnet) = @_;
  my $cmd = "info configure software-mngt database flat";

  my @response =  IsamCli::SendCommand($telnet,"$cmd");
  return 0 if ($ERROR ne "" );
  foreach my $line (@response) {
        chomp $line;
        if ($line =~ /activate after-download/) {
            ToolCommon::Log($DEBUG, "DB activate after download configured...\n") if defined($DEBUG); 
            return 1; 
        }
  }
  ToolCommon::Log($DEBUG, "DB auto-activate not configured...\n") if defined($DEBUG);
  return 0;
}


sub GetXdslOperDataDetails($$) {
  my ($session, $line ) = @_;
  my %xdsl_operdata = ();;

  my $cmd = "show xdsl operational-data line $line detail";
  my @response =  IsamCli::SendCommand( $session,"$cmd");
  return (0, undef) if ($ERROR ne "" );
  foreach my $line (@response) {
      chomp $line;
      if( $line =~ /\s+:\s+/ ) {
	      $line =~ s/^\s*//; $line =~ s/\s*$//;
	      while ( $line =~ /^([^\s]+)\s+:\s+([^\s]+)/ ) {
	         $xdsl_operdata{$1} = $2;
             $line =~ s/^$1\s+:\s+$2//;
             ### DEVEL NOTE:
             ### Below expression is required to handle
             ### opr-state/tx-rate-ds : up :54783    In that We have to remoce (:54783) value from CLI O/P
             $line =~ s/^\s*:\s*([^\s]+)//;
	         $line =~ s/^\s*//; $line =~ s/\s*$//;
	      }
      }
  }
  $xdsl_operdata{'output-power-down'} = '-';
  $cmd = "show xdsl operational-data near-end line $line detail";
  @response =  IsamCli::SendCommand( $session,"$cmd");
  return (0, undef) if ($ERROR ne "" );
  foreach my $line (@response) {
      if( $line =~ /output-power-down\s+:\s+([^\s]+)\s*/ ) {
          $xdsl_operdata{'output-power-down'} = $1;
      }
  }
  return (1,\%xdsl_operdata);
}


=head2 GetAutoBkpConfig($)

=begin html
Check if Auto DB backup configuration is present or not using the opened telnet
session $telnet.
Returns '1' if prsent otherwise '0'
=end html

=cut
sub GetAutoBkpConfig($) {
  my ($telnet) = @_;
  my ($slot, $type, $id);
      ToolCommon::Log($DEBUG, "calling Auto DB backup\n") if defined ($DEBUG);
  my $response = RetrieveInfo($telnet, "info configure software-mngt database flat", undef);

  if (!defined($response)) {
    return 0;
  }
  foreach (@{$response}) {
    if (/^(configure software-mngt database backup activate).*/) {
      ToolCommon::Log($DEBUG, "backup config present\n") if defined ($DEBUG);
      return 1;
    }
  }
  return 0;
}

sub IsSwReplacementModeLtByLt($) {
  my ($session_isam) = @_;
  my $command_to_exec = "info configure software-mngt flat";
  my $response = RetrieveInfo($session_isam, $command_to_exec, undef);
  if( defined($response) ) {
      foreach (@{$response}) {
         if( /^(configure software-mngt sw-replacement-mode lt-by-lt)/) {
             ToolCommon::Log($DEBUG, "SETUP-CHECK : Setup is an ISSU\n") if defined ($DEBUG);
             return 1 
         }
      }
  }
  ToolCommon::Log($DEBUG, "SETUP-CHECK : Setup isn't an ISSU\n") if defined ($DEBUG);
  return 0;
}

sub IsPortProtectionUp($) {
  my ($session_isam) = @_;

  my ($report_success, $configured_entries ) = (1,0);
  my $command_to_exec = "show port-protection";
  ToolCommon::Log($DEBUG, "Calling $command_to_exec\n") if defined ($DEBUG);
  my $response = RetrieveInfo($session_isam, $command_to_exec, undef);
  if( defined($response) ) {
      my $process_content = 0;
      foreach (@{$response}) {
         if( /^([-+]+)$/ ) {
             $process_content = (not $process_content);
             next;
         }
         next if ( $process_content == 0 );
         ### pon:1/1/7/1      pon:1/1/8/1      inact-up   act-up       no-switch-over     0  
         my @details = split( /\s+/, $_ );
         next if( scalar(@details) < 5 );
         $configured_entries++;
         if( $details[2] !~ /-up$/ or $details[3] !~ /-up$/ ){
            $report_success = 0;
            last;
         }
     }
  }
  if( $report_success ) {
      if( $configured_entries ) {
          ToolCommon::Log($DEBUG, "SETUP-CHECK : port protections count($configured_entries), All are up \n") if defined ($DEBUG);
      } else {
          ToolCommon::Log($DEBUG, "SETUP-CHECK : port protections count($configured_entries) \n") if defined ($DEBUG);
      }
  } else {
      $ERROR = "port protection details not available/port state is not up";
      ToolCommon::Log($DEBUG, "SETUP-CHECK : port protection entries aren't up\n") if defined ($DEBUG);
  }
  return $report_success;
}

sub SetCliConnection($) {
   my ($clisession ) = @_;;
   $CLISession = $clisession;
   return;
}

sub GetCliConnection() {
   return $CLISession;
}

sub reEstablishConnection($) {
  my ($isamConnection) = @_;
  my ($host,$type) = ("","");

  if( !defined $isamConnection ) {
	return undef;
  }

  if( defined($isamConnectionList{$isamConnection}) ) {
      if( defined($isamConnectionList{$isamConnection}{HOST}) ) {
         $host = $isamConnectionList{$isamConnection}{HOST};
      }
      if( defined($isamConnectionList{$isamConnection}{LOGFH}) ) {
         $type = $isamConnectionList{$isamConnection}{TYPE};
      }
      my $logfh = $isamConnectionList{$isamConnection}{LOGFH};
      my $params = $isamConnectionList{$isamConnection}{PARAMS};
      my $timeout = $isamConnectionList{$isamConnection}{TIMEOUT};
      cleanConnection($isamConnection);
      my $clisession = ToolCommon::OpenCli([$host], $logfh, $timeout );
      return $clisession;
  } else{
      return undef;
  }
}

sub GetONTSwVer($) {

  my ($telnet) = @_;
  my %swVer ; 
  my $count = 0; 

  ToolCommon::Log($DEBUG, "Get the ONT Sw versions in OLT\n") if defined ($DEBUG);
  my $response = RetrieveInfo($telnet, "show equipment ont sw-version", undef);

  return undef if (!defined($response)); 
  
  foreach my $line (@{$response}) {
   #================================================================================
   #sw-ver-id|sw-ver        |sw-ver-size
   #---------+--------------+-------------------------------------------------------
   #1         FE53217AOCA33  6787076
   #2         FE53216AOCI96  7311364
   #--------------------------------------------------------------------------------
     chomp $line; 
     if($line =~ /^(\d+)\s+(\w+)\s+(\d+)\s+/){ 
       $swVer{$2} = $3;
       $count = 1; 
     }
  }
  return 1  if(! $count); 

  return \%swVer;
}


sub deleteONTSw($$) {

  my ($telnet, $ontSw) = @_;
  my $response = undef; 

  ToolCommon::Log($DEBUG, "Delete the ONT Sw from the OLT\n") if defined ($DEBUG);
  $response = RetrieveInfo($telnet, "admin equipment ont sw-version delete-version $ontSw", undef);
  ### TODO - use sendCommand

  return 0 if (!defined($response)); 
  
  return 1; 

}

sub getCliResponse($$) {
  my ($telnet, $cliCmd) = @_;
  my $response = undef; 

  ToolCommon::Log($DEBUG, "Get Response for CLI command <$cliCmd>\n") if defined ($DEBUG);
  $response = RetrieveInfo($telnet, $cliCmd, undef);

  return undef if (!defined($response)); 

  return $response; 
}

sub RetriveOntSwCtrl($) {

  my ($telnet) = @_;

  my %ontSwCtrl ; 
  my ($ontCtrl) = {};
  my $cliCmd = "show equipment ont sw-ctrl";
  my $respSwCtrl = getCliResponse($telnet, $cliCmd);

  if (defined ($respSwCtrl)) {
     my ($swCtrlID, $lastUpdtTime, $numRefOnt, $prevPlnSwVer) ;

     foreach my $line (@{$respSwCtrl}) {
        $swCtrlID = ""; $lastUpdtTime = ""; $numRefOnt = ""; $prevPlnSwVer = "";
        if($line =~ /(\d+)\s+([^ ]+)\s+([^ ]+)\s+([^ ]+)/) {
            $swCtrlID = $1; $lastUpdtTime = $2; $numRefOnt = $3; $prevPlnSwVer = $4;
            ###print "line -->  <$line>\n";
            $ontCtrl->{$swCtrlID}{'lastUpdtTime'}  = $lastUpdtTime;
            $ontCtrl->{$swCtrlID}{'numRefOnt'}   = $numRefOnt;
            $ontCtrl->{$swCtrlID}{'prevPlnSwVer'} = $prevPlnSwVer;
        }
     }
  } else {
    return 0;
  }

  $cliCmd = "info configure equipment ont sw-ctrl flat"; 
  ##configure equipment ont sw-ctrl 1 hw-version PAS7401* plnd-sw-version 3FE15776902 plnd-sw-ver-conf 3FE15776902
  my $respInfoSwCtrl = getCliResponse($telnet, $cliCmd);
  if (defined ($respInfoSwCtrl)) {
    foreach my $line (@{$respInfoSwCtrl}) {
        chomp $line;
        if ($line =~ s/configure\s+equipment\s+ont\s+sw\-ctrl\s+([^\s]+)\s+//) {
             my $ctrlId = $1;
             if ($line =~ s/hw\-version\s+([^\s]+)//) {
                $ontCtrl->{$ctrlId}{'hw-version'} = $1;
             }
             if ($line =~ s/plnd\-sw\-version\s+([^\s]+)//) {
                $ontCtrl->{$ctrlId}{'plnd-sw-version'} = $1;
             }
             if ($line =~ s/plnd\-sw\-ver\-conf\s+([^\s]+)//) {
                $ontCtrl->{$ctrlId}{'plnd-sw-ver-conf'} = $1;
             }
             if ($line =~ s/ont\-variant\s+([^\s]+)//) {
                $ontCtrl->{$ctrlId}{'ont-variant'} = $1;
             }
             if ($line =~ s/sw\-dwload\-ver\s+([^\s]+)//) {
                $ontCtrl->{$ctrlId}{'sw-dwload-ver'} = $1;
             }
             $ontCtrl->{$ctrlId}{'ont-variant'} = "NULL"    if(!defined($ontCtrl->{$ctrlId}{'ont-variant'}));
        }
    }
  } else {
    return 0; 
  }

  foreach my $ctrlIdx (keys %{$ontCtrl}){
     my $hwVer = $ontCtrl->{$ctrlIdx}{'hw-version'}; 
     my $varnt = $ontCtrl->{$ctrlIdx}{'ont-variant'}; 
     if(defined($hwVer) && defined($varnt)) {
	     $ontSwCtrl{$hwVer}{$varnt}{'id'}=$ctrlIdx;
	     $ontSwCtrl{$hwVer}{$varnt}{PLNDSWVER}= $ontCtrl->{$ctrlIdx}{'plnd-sw-version'};
	     #$ontSwCtrl{$hwVer}{$varnt}{'plnd-sw-ver-conf'}= $ontCtrl->{$ctrlIdx}{'plnd-sw-ver-conf'};
	     $ontSwCtrl{$hwVer}{$varnt}{DLDSWVER}= $ontCtrl->{$ctrlIdx}{'sw-dwload-ver'};
	     $ontSwCtrl{$hwVer}{$varnt}{NUMREFONTS}= $ontCtrl->{$ctrlIdx}{'numRefOnt'};
     }
  } 

  return (\%ontSwCtrl);
}

sub AddONTSWCTRL($$$$$$){
  my ($telnet, $hw, $var, $mode, $sw, $actsw) = @_;
  #my $cliresponse = undef;;
  my $cliresponse = 0; 
  my %existCtrlId = (); 
  my $avblCtrlId = 0; 
  my $cmd = ""; 

  my $cliCmd = "info configure equipment ont sw-ctrl flat";
  ##configure equipment ont sw-ctrl 1 hw-version PAS7401* plnd-sw-version 3FE15776902 plnd-sw-ver-conf 3FE15776902
  my $respInfoSwCtrl = getCliResponse($telnet, $cliCmd);
  if (defined ($respInfoSwCtrl)) {
     foreach my $line (@{$respInfoSwCtrl}) {
        chomp $line;
        if ($line =~ /configure\s+equipment\s+ont\s+sw\-ctrl\s+(\d+)\s+/) {
             my $ctrlId = $1;
             $existCtrlId{$ctrlId} = 1; 
        }
     }
  }  else {
       #return $cliresponse; 
       return undef; 
  } 

  for(my $i = 1; $i <=250 ; $i++) {
   if(!defined($existCtrlId{$i})) {
      $avblCtrlId = $i;
      last;
   }
  }
 
  if($avblCtrlId == 0) { 
    #return $cliresponse; 
    return undef; 
  }  

  if ($sw eq "UNPLANNED") {
    $cmd = sprintf("configure equipment ont sw-ctrl %s hw-version %s ont-variant %s",$avblCtrlId,$hw,$var) if ($mode == 2);
    $cmd = sprintf("configure equipment ont sw-ctrl %s hw-version %s ont-variant %s",$avblCtrlId,$hw,$var) if ($mode == 3 || $mode ==1);
  } else {
    $cmd = sprintf("configure equipment ont sw-ctrl %s hw-version %s ont-variant %s plnd-sw-version %s plnd-sw-ver-conf %s sw-dwload-ver %s",$avblCtrlId,$hw,$var,$actsw,$actsw,$sw) if ($mode ==1);
    $cmd = sprintf("configure equipment ont sw-ctrl %s hw-version %s ont-variant %s plnd-sw-version %s plnd-sw-ver-conf %s",$avblCtrlId,$hw,$var,$sw,$sw) if ($mode == 2);
    $cmd = sprintf("configure equipment ont sw-ctrl %s hw-version %s ont-variant %s plnd-sw-version %s plnd-sw-ver-conf %s sw-dwload-ver %s",$avblCtrlId,$hw,$var,$sw,$sw,$sw) if ($mode == 3);
    $cmd = sprintf("configure equipment ont sw-ctrl %s hw-version %s ont-variant %s plnd-sw-version %s plnd-sw-ver-conf %s",$avblCtrlId,$hw,$var,"DELAYACTIVATE","DELAYACTIVATE") if ($mode == 4);   ### TODO :check val for PLNDSWVERCONF
  }
  my @response = SendCommand($telnet, $cmd);
  return undef if( $ERROR ne "" );
  my $respStr = "";

  foreach (@response) {
    $respStr .= $_;
  }
  if ($respStr =~ /$ERR/ ||
      $respStr =~ /$INVALID_TOKEN/ ||
      $respStr =~ /$INTERNAL_ERROR/ ||
      $respStr =~ /$GENERAL_ERROR/ || 
      $respStr =~ /$GPONMGNT_ERROR/ || 
          $respStr =~ /$FILE_CLOSED/) {
    $ERROR = $cmd . join("", @response);
    return undef;
    #return 0; 
  }
  return @response;
}




#-- End of library
1;
