package Logger;

#==============================================================================
#-- File:         Logger.pm
#-- Date:         2018/06/20
#-- Company:      Nokia
#-- Author:       Dhineshkumat
#------------------------------------------------------------------------------
#-- Function:     Script to log execution flow message
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Nokia.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Nokia
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0   - 2018/06/20 : First implementation

=head1 VERSION

=begin html
Current version: "Logger.pm - 1.0.1 -- 2018/06/20"

=end html

=cut

my $VERSION = "Logger.pm - 1.0.1 -- 2018/06/20";
use strict;
use warnings 'all';



use vars qw (@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS
             $LogModeMessage $LogModeNotify $LogModeDebug $LogModeTrace $LogModeWarn $LogModeError $LogModeInfo
             );
use Exporter;
@ISA = qw(Exporter);
use File::Copy;
use ErrorCodes qw(:EXIT_CODES);
use PBMTGlobal;
use Net::SNMP;


@EXPORT = qw ( $LogModeMessage $LogModeNotify $LogModeDebug $LogModeTrace $LogModeWarn $LogModeError $LogModeInfo 
               LogEnableDebug LogSetDebug LogSetFH LogCloseFile LogCreateFile LogHandleCreate LogHandleUpdate LogHandleClose 
               LogTrace LogDebug LogError LogWarn LogInfo LogNotify LogWish LogDisplay LogDump 
             );
@EXPORT_OK = qw ( LogEnableDebug LogSetDebug LogSetFH LogCloseFile LogCreateFile LogHandleCreate LogHandleUpdate LogHandleClose 
                  LogTrace LogDebug LogError LogWarn LogInfo LogNotify LogWish LogMessage LogDisplay LogDump
                  $LogModeMessage $LogModeNotify $LogModeDebug $LogModeWarn $LogModeError $LogModeInfo 
              );
%EXPORT_TAGS = ( ALL => [@EXPORT_OK] );

my  $LogModeDebugEnabled = 0;
my  $execLogFilePath = "";
my  $nodeLogFilePath = "";

$LogModeMessage = 1;
$LogModeTrace = 1;
$LogModeDebug = 2;
$LogModeNotify = 3;
$LogModeWarn = 4;
$LogModeInfo = 5;
$LogModeError = 6;

our $LogOnlyHandle = 1;
our $LogOnlyDebug  = 1;
our $LogOnlyMessage = 1;

sub __DumpMessage($$$;$$);
sub LogCreateFile($;$);
sub LogHandleCreate($$$;$);
sub LogHandleUpdate($$$);
sub LogHandleClose($);
sub SelectFH($);
sub SetFH($);
sub GetFH();
sub LogEnableDebug();
sub LogSetFH($);
sub LogTrace($$;$);
sub LogDebug($$;$);
sub LogNotify($$;$);
sub LogError($$;$$);
sub LogWarn($$;$$);
sub LogInfo($$;$$);
sub LogWish($$;$$$);
sub LogDisplay($);



=head2 SelectFH($)

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut
sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

sub LogEnableDebug() {
     $LogModeDebugEnabled  = 1;
}

sub LogSetDebug() {
     $LogModeDebugEnabled  = 1;
}


sub LogCreateFile($;$) {
     my ($logFile,$needToAppend)  = @_;
     my $logFH = undef;
     if( defined $needToAppend and $needToAppend ) {
        if( !open ( $logFH, ">>$logFile" )) {
           #Log("Unable to open new logfile:$logFile:");
           return undef; 
        }
     } else {
        if( !open ( $logFH, ">$logFile" )) {
           #Log("Unable to open new logfile:$logFile:");
           return undef; 
        }
     }
     return $logFH;
}

sub LogCloseFile($) { 
    my ($logFH) = @_;
    if( defined $logFH and fileno $logFH ) {
	    close $logFH;
    }    

}

sub LogHandleClose($) {
    my ($logFH) = @_;
    if( defined $logFH and fileno $logFH ) {
	    close $logFH;
    }    
}

sub LogHandleCreate($$$;$) {
    my( $logExecOrNode, $logFilePath, $logMesg, $logAppend ) = @_;
    my $logFH = undef;
    my $fileCreateMode = '>';
    if( defined $logAppend and $logAppend ) {
        $fileCreateMode = '>>';
    }
    if( !open ( $logFH, "$fileCreateMode$logFilePath" )) {
        print STDERR "Failed to open file '$logFilePath' for $logMesg based logging : $!\n";
        exit FILE_OPEN_FAIL_CODE;
    }
    $logFH->autoflush(1);
    ## 0 - Exec, 1 = Node
    if( $logExecOrNode ) {
        $nodeLogFH = $logFH;
        $nodeLogFilePath = $logFilePath;
    } else {
        $nodeLogFH = $execLogFH = $logFH;
        $execLogFilePath = $logFilePath;
    }
    Net::SNMP->snmp_debug(0x00, $nodeLogFH);
    my $logType = "started";
    my $calledFrom = (caller(1))[3];
    if( defined($calledFrom) and  $calledFrom =~ /LogHandleUpdate/ ) {
          $logType = "moved and started"
    }
    ___DumpMessage( $execLogFH, "INFO", "$logMesg based logging $logType to a file '$logFilePath'\n" );
    ___DumpMessage( *STDOUT, "INFO", "$logMesg based logging $logType to a file '$logFilePath'\n" );
    if( $logExecOrNode ) {
        ___DumpMessage( $nodeLogFH, "TRACE", "$logMesg based logging $logType...\n" );
    }
}

sub LogHandleUpdate($$$) {
    my( $logExecOrNode, $targetPath, $logMesg ) =  @_;
    my $sourcePath = "";
    ## 0 - Exec, 1 = Node
    if( $logExecOrNode ) {
         $sourcePath = $nodeLogFilePath;
         LogCloseFile($nodeLogFH);
    } else {
         $sourcePath = $execLogFilePath;
         LogCloseFile($execLogFH);
    }
    if ( !(move( $sourcePath, $targetPath)) ) {
      print STDERR "Failed to move $sourcePath to $targetPath for $logMesg : $!\n";
      exit CREATE_FILE_FAIL_CODE;
    }
    my $logAppend = 1;
    LogHandleCreate( $logExecOrNode, $targetPath, $logMesg, $logAppend );
}

sub ___DumpMessage($$$;$) {
    my ($logFH, $logType, $logMesg, $logMessageOnly ) = @_;

    if( !defined $logMessageOnly ) {
        my $calledFrom = (caller(2))[3];
        my $lineNo = (caller(1))[2];
        if (defined($calledFrom)) {
          if ($calledFrom =~ /(eval)/) {
            $calledFrom = (caller(3))[3];
          } elsif (($calledFrom =~ /::(ReleaseCompare|ReportStatus|SetAndLogError|SelectFamily|LogHandleUpdate|LogWish)$/)) {
            $calledFrom = (caller(3))[3];
            $lineNo = (caller(2))[2];
          }
        } else {
            $calledFrom = "main::main";
        }
        my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime(time()))[0,1,2,3,4,5];
        $logMesg = sprintf("%04d-%02d-%02d %02d:%02d:%02d %-6s [%s:%d] %s", $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $logType, $calledFrom , $lineNo, $logMesg);
    }
   
    if( defined $logFH and fileno $logFH ) {
        print $logFH "$logMesg";
    } else {
        print STDERR "FIXME : $logMesg";
        return;
    }
}


sub LogTrace($$;$) {
    my ($logFH, $logMesg, $logMessageOnly ) = @_;
    ___DumpMessage( $logFH, "TRACE", $logMesg, $logMessageOnly );
}

sub LogDebug($$;$) {
    my ($logFH, $logMesg, $logMessageOnly ) = @_;
    return if( not defined $params->{DEBUG} or $params->{DEBUG} == 0 );
    ___DumpMessage( $logFH, "DEBUG", $logMesg, $logMessageOnly );
}

sub LogNotify($$;$) {
    my ($logFH, $logMesg, $logMessageOnly ) = @_;
    ___DumpMessage( $logFH, "NOTIFY", $logMesg, $logMessageOnly );
}


################################################################################
### Print Error Message 
################################################################################
sub LogError($$;$$) {
    my( $logFH, $logMesg, $logDebugOnly, $logHandleOnly ) = @_;
    return if( defined $logDebugOnly and not $params->{DEBUG} );
    ___DumpMessage( $logFH, "ERROR", $logMesg );
    return if( defined $logHandleOnly );
    ___DumpMessage( *STDOUT, "ERROR", $logMesg );
}

sub LogWarn($$;$$) {
    my ($logFH, $logMesg, $logDebugOnly, $logHandleOnly ) = @_;
    return if( defined $logDebugOnly and not $params->{DEBUG} );
    ___DumpMessage( $logFH, "WARN", $logMesg );
    return if( defined $logHandleOnly );
    ___DumpMessage( *STDOUT, "WARN", $logMesg );
}

sub LogInfo($$;$$) {
    my ($logFH, $logMesg, $logDebugOnly, $logHandleOnly ) = @_;
    return if( defined $logDebugOnly and not $params->{DEBUG} );
    ___DumpMessage( $logFH, "INFO", $logMesg );
    return if( defined $logHandleOnly );
    ___DumpMessage( *STDOUT, "INFO", $logMesg );
}

sub LogWish($$;$$$) {
    my ($logWishMode, $logFH, $logMesg, $logDebugOnly, $logHandleOnly ) = @_;
    if( !defined $logWishMode or !defined $logFH or !defined $logMesg ) {
        my $calledFrom = (caller(1))[3];
        my $lineNo = (caller(0))[2];
        print STDERR "CRITICAL : LogWish called from $calledFrom at Line no $lineNo is not proper format\n";
        exit -1;
    } elsif( $logWishMode == $LogModeWarn ) {
        LogWarn( $logFH, $logMesg, $logDebugOnly, $logHandleOnly );
    } elsif( $logWishMode == $LogModeError ) {
        LogError( $logFH, $logMesg, $logDebugOnly, $logHandleOnly );
    } elsif( $logWishMode == $LogModeInfo ) {
        LogInfo( $logFH, $logMesg, $logDebugOnly, $logHandleOnly );
    } elsif( $logWishMode == $LogModeDebug ) {
        my $logMessageOnly = $logDebugOnly;
        LogDebug( $logFH, $logMesg, $logMessageOnly);
    } else {
        my $logMessageOnly = $logDebugOnly;
        LogTrace( $logFH, $logMesg, $logMessageOnly);
    }
}

sub LogDisplay($) {
    my (@messages) = @_;
    print STDOUT "@messages";
}

sub LogDump($$) {
    my ($logFH, $logMesg ) = @_;
    if( defined $logFH and fileno $logFH ) {
        print $logFH "$logMesg";
    } else {
        print STDERR "FIXME : $logMesg";
    }
}

1;
