package MigrateIsam7302HiCap;

=head1 HiCap ISAM 7302/7330-specific migration module

=begin html
This library contains the functions dedicated to migrate a HiCap 7302/7330 ISAM from one release to a new one.

=end html

=cut

#==============================================================================
#-- File:         MigrateIsam7302HiCap.pm
#-- Date:         22.09.2008
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Library to migrate HiCap ISAM to a new release
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0  - 2008/09/22 : First implementation
#--   1.0.1  - 2009/04/22 : ANTmt54730.  Do not create empty DB backup in case of activation only.
#--                        Release transaction if TO = 127.0.0.1
#--   1.0.2  - 2009/08/25 : No xVPS for NANT-D required.
#--   1.0.3  - 2009/08/27 : Retrieve access operator ID using SNMP
#--   1.0.4  - 2009/11/09 : ANTmt83235
#--   3.0.00 - 2010/04/21 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01 - 2010/08/06 : ALU00719042 - supprot for --ofldb and sftp
#--   3.0.02 - 2010/11/01 : Lanx restoration failure - when migrating from 4002 to 4.2
#--   3.0.03 - 2011/01/20 : added CDE untar for SIP and MEGACO as as workaround
#--   3.0.04 - 2011/02/08 : ALU01033654 
#--   3.0.05 - 2011/02/10 : ALU01033842
#--   3.0.06 - 2011/02/11 : target release version number check included for range check of vmac host id parameter
#--   3.0.07 - 2011/03/01 : Untar of CDE done via pbmt earlier is commented
#--   3.0.08 - 2011/03/07 : undo vMAC range check for release > R4202 (ALU01033842 reworked due to change in migration tool)
#--   3.0.09 - 2011/03/27 : linux compatible changes
#--   3.0.10 - 2011/03/29 : ALU01031457 - Support for R4220 & R4230
#--   3.0.11 - 2011/05/19 : fix for upgrade in 4220/4230
#--                       : ALU01207252
#--   3.0.12 - 2011/05/27 : ALU01236674 
#--   3.0.13 - 2011/06/14 : ALU01235321 
#--   3.0.14 - 2011/06/24 : Enable Hairpin feature for migration from R4230 to R43
#--   3.0.15 - 2011/07/07 : Check for oflmt version info made more strict
#--   3.0.16 - 2011/11/03 : ALU01595196
#--   3.0.17 - 2011/12/11 : Release convertion corrected in R4230 for offline DB issue 
#--   3.0.18 - 2012/01/02 : for resolving offline migration failure for FANT-F from R4302 to R44
#--   3.0.19 - 2012/04/24 : ALU01777956
#--   3.0.20 - 2012/04/26 : ALU01778953
#--   3.0.21 - 2012/05/11 : ALU01789702 - World writeable directories need sticky bit set 
#--   3.0.22 - 2012/06/28 : ALU01804629
#--   3.0.23 - 2012/08/13 : ALU01815133
#--   3.0.24 - 2012/09/12 : ALU01819262
#--   3.0.25 - 2012/09/27 : ALU01826095
#--   3.0.26 - 2012/10/09 : ALU01828441
#--   3.0.27 - 2012/10/22 : ALU01761938
#--   3.0.28 - 2012/10/30 : ALU01836984, ALU01836986, ALU01837839
#--   3.0.29 - 2012/11/17 : ALU01836902
#--   3.0.30 - 2012/12/07 : ALU01847247
#--   3.0.31 - 2012/12/19 : ALU01844812
#--   3.0.32 - 2013/01/07 : ALU01842744
#--   3.0.33 - 2013/02/06 : enable usage of second password if first one fails for cli over ssh
#--   3.0.34 - 2013/03/26 : ALU01852256
#--   3.0.35 - 2013/05/31 : ALU01881027
#--   3.0.36 - 2013/06/18 : ALU01890986
#--   3.0.37 - 2013/06/27 : determine source sw for offline DB's in R4601
#--   3.0.38 - 2013/08/12 : ALU01903056
#--   3.0.39 - 2013/11/06 : ALU01927702
#--   3.0.40 - 2014/03/25 : support for new db version in R5001 (ALU01963317)
#--   3.0.41 - 2014/06/09 : ALU01976828
#--   3.0.42 - 2014/07/15 : ALU01980121
#--   3.0.43 - 2014/08/19 : ALU01993252
#--   3.0.44 - 2015/02/13 : ALU02026412: Check auto backup progress 
#--   3.0.45 - 2015/05/20 : ALU02082098
#--   3.0.46 - 2015/06/18 : ALU02025800
#--   3.0.47 - 2015/09/01 : ALU02106733
#--   3.0.48 - 2015/09/03 : support for new db version in R5301
#--   3.0.49 - 2015/10/13 : ALU02135551
#--   3.0.50 - 2015/03/10 : ALU02175921
#--   3.0.51 - 2016/03/16 : ALU02190558
#--   3.0.52 - 2016/05/06 : R5203 changes
#--   3.0.53 - 2016/07/18 : ALU02237966
#--   3.0.54 - 2016/09/12 : ALU02271083
#--   3.0.55 - 2016/10/13 : ALU02282385
#--   3.0.56 - 2016/12/15 : ALU02311349
#--   3.0.57 - 2017/10/24 : ALU02400566
#--   3.0.58 - 2018/03/15 : ALU02444650
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.1.1  - 2019/02/28 : ALU02550344


  
=head1 VERSION

=begin html
Current version: "MigrateIsam7302HiCap.pm - 3.1.1 -- 2019/02/28"

=end html

=cut

my $VERSION = "MigrateIsam7302HiCap.pm - 3.1.1 -- 2019/02/28";
#==============================================================================

use strict;
use warnings 'all';
require 5.001;

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2092-DFZZA">DM</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;
use Cwd qw(getcwd);
use Config; 

use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Utils;
require Octopus;
use Capability;
use CapabilityReEngr;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :MIGR);
require FamilyCommon;
use DM;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub MigrateNTDbase($$$$$);
sub CombineDbases($$$$$$);
sub DownloadDbase($$$$$$);
sub MigrateOrUpgrade($$$$);
sub Migrate($$$);
sub CheckEnvironment($$$);
sub LocateCDE($$$$);
sub MigrateXvpsContainer($$$$$$);
sub MigrateXvpsDbase($$$$$$$$$$$);
sub PrepareXvpsContainers($$$$);
sub DetermineXvpsPresence($$);
sub GetXvpsIsamListByIP($$);
sub GetXvpsIsamListByCommunity($$$);
sub GetXvpsDependancyByIP($$);
sub GetXvpsDependancyByCommunity($$$);
sub MakeFakeList($$);
sub GetIvpsIpAddresses($$);
sub CopyXvpsList($$);
sub ModifyDbtoolCfg($$$); 
sub DetermineNtCommunities($$);
sub ModifyXvpsDbtoolCfg($$$$);
sub PrintToolVersions($$);
sub DetermineSwVersion($);
sub SetToolData();
sub UpdateIPDMaxRange($$); 
sub UpdateIPDInterface($$);
sub ResetGlobals();
#==============================================================================
#== Constants
#==============================================================================
my $NTINFILENAME =  "dm_complete.tar";           # input name of the complete NT database
my $NTSMALLFILE =   "dm.tar";                     # input name of the "small" NT database (only used with ofldb).
my $NTOUTFILENAME = "dm_migrated.tar";          # Name of the complete NT database after migration
my $NTMIGFILE =     "migNt.txt";

my $ISNOTISAMV_RE =    "is not an ISAMV";
my $ISISAMV_RE =       ".VPS-(\\d):(\\d+\.\\d+\.\\d+\.\\d+)";
my $IVPSCONTAINER =    "XVPS%d";
my $IVPSDBASE =        "Dbase";
my $CDEACTDIR =        "active";
my $CDECONTNAME =      "CDE";
my $CDETMPDIR   =      "cdetmp";
my $CDEPATHVAR =       "CDE_FILE_PATH";
my $NEWCDEPATHVAR =    "NEW_CDE_FILE_PATH";
my $LOWEST_XVPSNUM =   1;
my $HIGHEST_XVPSNUM =  11;
my $SHUBMIGFILE =      "migShub.txt";
my $XVPSMIGFILE =      "migXvps%d.txt";
my $R4230DBVer =       "C4\.230";
my $R43DBVer =         "A4\.300";
my $R4xDBVer_RE =      "[A-D]4\.[5-9]";
my $DT_ACCOPR =        260;
my $IHUBCONFIG =       "config.cfg";
my $IHUBDIR =          "Ipd";
my $setVlanLabel = 0; 

## for ALU01993252 - changes for core-1 mib range update in R51
my $setMaxIPDCfgRange = 0;   
my $setFastReroute = 0;
my $setIPDCfgIGMPInterface = 0;
my $setIPDCfgDhcp = 0;
my @feltbLts = ();
my $R51_MC_MAX_ROUTES = 4096; 
my $R51_MAX_IPV6_ROUTES = 8192; 
my $R51_MAX_ROUTES = 16384; 
#==============================================================================
#== Vars
#==============================================================================
my %warnLevels;
my $config;
my $toolData;
my $NTOFLMTVERSION = 0;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

sub ResetGlobals() {
   $NTOFLMTVERSION = 0;
   $setVlanLabel = 0;
   $setMaxIPDCfgRange = 0;
   $setFastReroute = 0;
   $setIPDCfgIGMPInterface = 0;
   $setIPDCfgDhcp = 0;
   @feltbLts = ();
}

=head2 MigrateNTDbase($$$$$$)

=begin html
Migration of the NT database.  If requested NT NW-type convertion will be performed once the database has been migrated.  Returns 0 in case of error.
<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>path</td><td>PBMT installation directory name</td></tr>
</table>

=end html

=cut

sub MigrateNTDbase($$$$$) {
  my ($fh, $dir, $from, $to, $path) = @_;
  my $thisPath = getcwd();
  my $ok = 1;
  my $fromRel = $params->{SOURCE_DBVERSION};
  my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $fromRel);
  my $toRel = $params->{DBVER_TGTREL};
  my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $toRel);
  my $cmd = "";
  my $ntDbName;
  my $ntInName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
  my $cdeTmpPath = "";

  if (defined($toRel) && defined($fromRel)) {
      if ($params->{ISAM_HAS_VOICE}) {
         $params->{ISAM_VOICE_MIGR} = 1;

         my $cdeTarFileName = "";
         my $logDir = FamilyCommon::GetNeStorage();
         my $cdeFileName = undef;  

         my $xvpsDbPath = $toolData->{FXRDATA}{HOMEDIR};  # Can be ftp or sftp DB upload
         my $dirName = defined($params->{OFLDB}) ? FamilyCommon::GetNeStorage() : $xvpsDbPath; 

         if ($ok) {
           ToolCommon::Log($fh, "Preparing CDE files for IVPS ...\n") if ($params->{DEBUG});
           if (! -e "$dirName/$CDETMPDIR") {
               $cmd = "cd $dirName;mkdir $CDETMPDIR;cd $CDETMPDIR;$config->{TAR} xf $logDir/$NTINFILENAME $CDECONTNAME >$logDir/MIGRUntarCDE1.txt 2>&1";
           } else {
               $cmd = "cd $dirName/$CDETMPDIR;$config->{RM} -rf * > /dev/null 2>&1;$config->{TAR} xf $logDir/$NTINFILENAME $CDECONTNAME >$logDir/MIGRUntarCDE2.txt 2>&1";
           }
           ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
           my $ret = system($cmd);
           ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});

           if($ret) {
              my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
              my $tMsg = "$cmd failed: $!";
              FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dirName')\n");
              $ok = 0;
           } else {
              ToolCommon::Log($fh, "Preparation of CDE files Successful ...\n") if ($params->{DEBUG});
           }
         }

         $cdeFileName = LocateCDE($fh, "$dirName/$CDETMPDIR/$CDECONTNAME", $config->{CDE_REGEXP}, $dirName);

         if (! defined($cdeFileName)) {
           my $tMsg = "No CDE file was found in the $dir/$CDETMPDIR/$CDECONTNAME directory";
           FamilyCommon::SetAndLogError($fh, $tMsg, " ERROR : $tMsg (check logfiles in working directory '$dir')\n");
           $ok = 0;
         } else {
           $cdeTarFileName = "$dir/$CDETMPDIR/$CDECONTNAME/$cdeFileName";
           $cdeTmpPath = "$dir/$CDETMPDIR/$CDECONTNAME";
           $params->{CDE_TMP_PATH} = $cdeTmpPath; 
         }
         # Workaround for untar of CDE files 
         #if ($ok) {
         #   $cmd = "cd $dir/$CDETMPDIR/$CDECONTNAME; $config->{TAR} xf $cdeTarFileName >$logDir/MIGRUntarCDE1.txt 2>&1";
         #   ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
         #   my $ret = system($cmd);
         #   ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});

         #   if($ret) {
         #     my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
         #     my $tMsg = "$cmd failed: $!";
         #     FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
         #     $ok = 0;
         #   } else {
         #     $cdeTmpPath = "$dir/$CDETMPDIR/$CDECONTNAME"; 
         #   }
         #}
         # Cleanup of CDE source directory done here
         if ($ok) {
           if (-e "$dir/$CDECONTNAME") {
             $cmd = "cd $dir/$CDECONTNAME;$config->{RM} -rf * > $dir/MIGRntCleanCDE.txt 2>&1";
             ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
             my $ret = system($cmd);
             ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
             if($ret) {
                my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
                my $tMsg = "$cmd failed: $!";
                FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg \n");
                $ok = 0;
             }
           }
         }
      }
      if ($from ne $to) {
	if ($params->{CONFOPT82} || $params->{CONFARBERG} || $params->{ACCESSOPRID} || defined($params->{INTL_NW_VLAN})|| $params->{ISAM_HAS_VOICE} || defined($params->{SSM_OUT_VLAN}) || defined($params->{ENABLE_IGMP_GROUPING}) || defined($params->{MAX_GIP_GAP}) || defined($params->{GPON_ONU_RANGE_MODE}) || (defined($params->{IGMP_MAX_UNCFG}))) {
	  if (!ModifyDbtoolCfg($fh, "$path/$toolData->{NTDBCFG}", "$dir/$toolData->{NTDBCFGMOD}")) {
	    # SetAndLogError already done.
	    $ok = 0;
	  }
	} else {
	  if(!copy("$path/$toolData->{NTDBCFG}", "$dir/$toolData->{NTDBCFGMOD}")) {
	    my $t = "Copy of $path/$toolData->{NTDBCFG} to $dir/$toolData->{NTDBCFGMOD} failed: $!";
	    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	    $ok = 0;
	  }
	}
    unlink "$dir/autoswitch_error.log";
    if (-f "$dir/autoswitch_error.log" ) {
        ToolCommon::Log( $fh, "ERROR : Autoswitch error file '$dir/autoswitch_error.log' not removed properly'\n");
    }
	if ($ok) {
	  $cmd = "cd $dir;$path/$toolData->{NTDBTOOL} -in $ntInName -out $NTOUTFILENAME -cfg ./$toolData->{NTDBCFGMOD}" .
	    " -from $fromRel -to $toRel -trc >$NTMIGFILE 2>&1";
	  ToolCommon::Log($fh, "Migrating NT database from $fromRelInfo ($fromRel) to $toRelInfo ($toRel)...");
	  ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
	  my $ret = system($cmd);
	  if($ret) {
	    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	    my $tMsg = "";
	    if ($exitCode == 3) {
	      my $migLogFile;
	      if (open($migLogFile, "$dir/trc.log")) {
		while (my $line = <$migLogFile>) {
		  chomp $line;
		  if ($line =~ /NON_RECOV/) {
		    $line = <$migLogFile>, $line = <$migLogFile>, $line = <$migLogFile>, $line = <$migLogFile>, chomp $line;
		    $line =~ m/error info\s*:(.*)/;
		    $tMsg = "Unrecoverable error occured: $1";
		  }
		}
		close($migLogFile);
	      } else {
		$tMsg = "Opening $dir/trc.log failed: $!";
	      }
	    } elsif ($exitCode == 2) {
	      $tMsg = "Consistency checks failed";
	    } else {
	      $tMsg = "Migration failed";
	    }
	    FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
	    $ok = 0;
	  } else {
	    $ntDbName = $NTOUTFILENAME;
	    ToolCommon::Log( $fh, "  OK\n");
	  }
      if (-f "$dir/autoswitch_error.log" ) {
          ToolCommon::Log( $fh, "ERROR : Autoswitch error file '$dir/autoswitch_error.log' found'\n");
      }
	}
      } else {
	$ntDbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
	ToolCommon::Log($fh, "  NT Migration not needed, same release\n");
      }
  } else {
    FamilyCommon::SetAndLogError($fh, "Migrating NT database from $from to $to not possible: DB version not defined");
    $ok = 0;
  }


  return $ok;
}

=head2 AddCdeFiles($$$$$)

=begin html
Add the CDE files to the new db using some pre-defined variables:
<ul>
  <li>$config->{NEWCDEFILE}: the path and name of the CDE file to be used</li>
  <li>$config->{CDE_REGEXP}: parse the CDE file name syntax to retrieve some parts</li>
</ul>
The CDE tar file is copied to the XVPS CDE directory.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>dirName</td><td>NE output directory pointing the CDE directory in the xVPS container</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
</table>
=end html

=cut

sub AddCdeFiles($$$$) {
  my ($fh, $dirName, $newName, $tmpDir) = @_;
  my $ok = 1;
  my $pwd = getcwd();
  my $logDir = FamilyCommon::GetNeStorage();
  my $cmd = "cd $dirName;$config->{CP} $toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE} $newName >$logDir/MIGRaddCde.txt 2>&1";
  my $ret;

  $ret = system("$cmd");
  ToolCommon::Log($fh, "CMD = $cmd\n") if $params->{DEBUG};
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    $ret = system("cd $pwd;");
  }
  return $ok;
}

=head2 CombineDbases($$$$$$$)

=begin html
Create a combined database from all migrated databases.  Returns 0 in case of error.
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>xvpsConfig</td><td>xVPS configuration.  undef if node is not ISAM-V</td></tr>
</table>
In case of IPVS migration to the following files will be
<ul>
  <li>created in the new backup in case of migration to R3.5</li>
  <li>replaced in the new backup in case of migration from R3.5 to a higher release</li>
</ul>
<ul>
  <li>cdePackName: the name of the new CDE file</li>
  <li>cdeProfileId: the customer ID associated to this CDE file</li>
</ul>

=end html

=cut

sub MakeDir($$$) {
  my ($fh, $dirName, $debug) = @_;
  my $ok = 1;

  if (!mkdir( "$dirName", 0777)) {
    my $t = "Create dir '$dirName' failed: $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "Created '$dirName'\n") if $debug;
    if ($params->{UMASK_CHMOD} and ! chmod(01777, "$dirName")) {
       ToolCommon::Log($fh,  "Failed to chmod 1777 for directory '$dirName'\n") if $debug;
    }
  }
  return $ok;
}

my $MAKETAR_TXT = "MIGRnewdb.txt";

sub CombineDbases($$$$$$) {
  my ($fh, $dir, $from, $to, $xvpsConfig, $fromDir) = @_;
  my $cmd;
  my $ok = 1;
  my $ret;
  my $pwd = getcwd();
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
  my $logDir = FamilyCommon::GetNeStorage();

  my $fromRel = Capability::XvpsDbVersion($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION} );
  my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION});
  my $toRel = Capability::XvpsDbVersion($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});

  ToolCommon::Log($fh, "Combining databases...");
  if (! -d "$dir/newdb") {
    if (! MakeDir($fh, "$dir/newdb", $params->{DEBUG})) {
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Created '$dir/newdb'\n") if $params->{DEBUG};
    }
  } else {
    $cmd = "$config->{RM} -Rf $dir/newdb/* >$logDir/MIGRcombrm.txt 2>&1";
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Emptied '$dir/newdb/*'\n") if $params->{DEBUG};
    }
  }
  if ($ok) {
    $cmd = "cd $dir/newdb;$config->{TAR} xvf ../$NTOUTFILENAME >$logDir/MIGRdbextr.txt 2>&1;cd $pwd";
    ToolCommon::Log( $fh, "CMD = $cmd\n") if $params->{DEBUG};
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    }
  }

  if( defined $fromRel and defined $toRel and ( $fromRel ne $toRel ) ) {
    if (($ok) && defined($xvpsConfig)) {
      # Include xVPS containers in new DB
      my @ivpsIds = (sort keys %{$xvpsConfig});
      my $numOfIvps = @ivpsIds;
      my $ivpsId = 0;
      my $realIvpsId = 0;
      while (($ok) && ($ivpsId < $numOfIvps)) {
	# Create xVPS container in output directory
	$realIvpsId = $ivpsId+1;
	if ($xvpsConfig->{$realIvpsId} ne $toolData->{NO_IP}) {
	  my $newDir = sprintf("$dir/newdb/$IVPSCONTAINER", $realIvpsId);
	  my $source = "";
	  $source = sprintf("$dir/$IVPSCONTAINER/$IVPSDBASE/$NTOUTFILENAME", $realIvpsId);
	  if (! -d "$newDir") {
	    # Creates 'XVPS<n>' under '$dir/newdb'
	    if (! MakeDir($fh, "$newDir", $params->{DEBUG})) {
	      $ok = 0;
	    } else {
	      # Creates 'Dbase' under '$dir/newdb/XVPS<n>'
	      if (! MakeDir($fh, "$newDir/$IVPSDBASE", $params->{DEBUG})) {
		$ok = 0;
	      } else {
		if (Capability::ReleaseCompare($to, Capability::ConvertRelease(3, 5), REGEXP, 0) == 0) {
		  # Creates 'CDE' under 'XVPS<n>'
		  if (! MakeDir($fh, "$newDir/$CDECONTNAME", $params->{DEBUG})) {
		    $ok = 0;
		  } else {
		    # Creates 'active' under 'XVPS<n>/CDE'
		    if (! MakeDir($fh, "$newDir/$CDECONTNAME/$CDEACTDIR", $params->{DEBUG})) {
		      $ok = 0;
		    }
		  }
		}
	      }
	    }
	  } else {
	    # From R3.5 onwards:
	    # XVPS container IS present in old DB. We need to remove all files (also remove the CDE directory).
	    # 1. remove all files from DB
	    $cmd = "cd $newDir/$IVPSDBASE;$config->{RM} -Rf * >$logDir/MIGRemptyXVPS.txt 2>&1";
	    ToolCommon::Log($fh, "Executing '$cmd'...\n") if $params->{DEBUG};
	    $ret = system($cmd);
	    if ($ret) {
	      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    }
	    # 2. Migration to R3.6 or higher: remove CDE directory from XVPS container.
	    if (Capability::ReleaseCompare($to, Capability::ConvertRelease(3, 6), REGEXP, 0) >= 0) {
	      $cmd = "cd $newDir;$config->{RM} -Rf $CDECONTNAME >$logDir/MIGRemptyCDE.txt 2>&1";
	      ToolCommon::Log($fh, "Executing '$cmd'...\n") if $params->{DEBUG};
	      $ret = system($cmd);
	      if ($ret) {
		my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
		my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
		FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
		$ok = 0;
	      }
	    }
	  }
	  if ($ok) {
	    # No IP address present means no xVPS present
	    $cmd = "cd $newDir/$IVPSDBASE;$config->{TAR} xf $source >$logDir/MIGRxvpsextr.txt 2>&1";
	    ToolCommon::Log($fh, "Executing '$cmd'...\n") if $params->{DEBUG};
	    $ret = system($cmd);
	    if ($ret) {
	      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	      my $t = "Extract of xVPS dbase '$source' FAILED (signal $signal, exitcode $exitCode): $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    } 
	  }
	}
	$ivpsId++;
      }
    }
  }

  if ($ok) {
    if (! -d "$dir/newdb/$CDECONTNAME") {
      # Creates 'CDE' under the "root"
      if (! MakeDir($fh, "$dir/newdb/$CDECONTNAME", $params->{DEBUG})) {
        $ok = 0;
      }
    } else {
      my $del1 = unlink <$dir/newdb/$CDECONTNAME/*>;
      ToolCommon::Log($fh, "Emptied '$dir/newdb/$CDECONTNAME', deleted $del1 files\n") if $params->{DEBUG};
    }
    if ($ok) {
      ## for ALU01836986 - migration of ANCP parameters
      if($setVlanLabel == 1) {  
         if(defined($params->{NEW_VLAN_LABEL}) &&  ($params->{NEW_VLAN_LABEL} ne "")) {
            if(defined($params->{ACT_VLAN_LABEL}) &&  ($params->{ACT_VLAN_LABEL} ne "")) {
                my $VLAN_LABEL_TXT = "VlanLbl.txt"; 
                if (-e "$dir/newdb/$IHUBDIR/$IHUBCONFIG") {
                     my $cmdsys = "cd $dir;$config->{PERL} -p -i -e 's/$params->{ACT_VLAN_LABEL}/$params->{NEW_VLAN_LABEL}/g' $dir/newdb/$IHUBDIR/$IHUBCONFIG >$VLAN_LABEL_TXT  2>&1";
                     $ret = system($cmdsys);
                     if ($ret) {
			  my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
                          my $t = "'$cmdsys' FAILED (signal $signal, exitcode $exitCode)";
			  FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
                          $ok = 0;
                     } else {
                        ToolCommon::Log($fh, "Changed Vlan Label from $params->{ACT_VLAN_LABEL} to $params->{NEW_VLAN_LABEL}\n") if $params->{DEBUG};
                     }
                }
            }
         }
      } 

      if( $setFastReroute == 1 ) {
         $ret = UpdateFastReroute($fh, $dir);
         Logger::LogDebug($nodeLogFH, "Update IPD config file for fast reroute\n") if($ret);
      }
      ## for ALU01993252 - changes for core-1 mib range update in R51
      if($setMaxIPDCfgRange == 1) {  
          $ret = UpdateIPDMaxRange($fh, $dir); 
          ToolCommon::Log($fh, "Update IPD config file for MaxRange\n") if $params->{DEBUG};
      }
      ## for ALU02106733 - changes for core-1 igmp interface update upto  R52
      if($setIPDCfgIGMPInterface == 1 ) {
          UpdateIPDInterface($fh,$dir);
          ToolCommon::Log($fh, "Update IPD config file for IGMP Interface\n") if $params->{DEBUG};
      }
      if( $setIPDCfgDhcp == 1) {
        UpdateIPDDhcp($fh,$dir);
        ToolCommon::Log($fh, "Update IPD config file for DHCP lease-populate\n") if $params->{DEBUG};
      }
      
      if( scalar @feltbLts ) {
        ToolCommon::Log($fh, "Update IPD config file for TPID translation\n") if $params->{DEBUG};
	UpdateTPIDTranslation( $fh, $dir, @feltbLts );
      }

      if (-f "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}" && ($toolData->{NEWCDEFILE} ne "")) {
        if (!AddCdeFiles($fh, "$dir/newdb/$CDECONTNAME", ".", $dir)) {
          $ok = 0;
        }
      } else {
          if ($params->{IS_ISAMV_SRV}) {
            my $t = "ISAM is voice server but no CDE file is found";
            FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
            $ok = 0;
          } elsif ($params->{ISAM_HAS_VOICE}) {
            # We will fail the migration in case the ISAM has voice boards
            if (Capability::ReleaseCompare($from, Capability::ConvertRelease(4, 3), REGEXP, 0) >= 0) {
              my $t = "ISAM has voice boards but no CDE file is found";
              FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
              $ok = 0;
            } else {
              ToolCommon::Log($fh, "  ISAM has voice boards but no CDE file will be included in the migrated DB!!");
            }
          } else {
            # Problem, leave the directory empty
          }
      }
    }
  }
  if ($ok) {
    my $readDir;
    if (!defined(opendir($readDir, "$dir/newdb"))) {
      my $t = "Can't opendir $dir/newdb: $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      CORE::exit(OPENDIR_FAIL_CODE);
    }
    my @fileList = readdir($readDir);
    closedir $readDir;
    ToolCommon::Log($fh, "\nTo be included in tar file:\n") if $params->{DEBUG};
    my $copyList = "";
    foreach (@fileList) {
      if (($_ ne ".") && ($_ ne "..")) {
	ToolCommon::Log($fh, "  '$_'\n") if $params->{DEBUG};
	$copyList .= $_ . " ";
      }
    }
    if ($Config{archname} =~ /linux/) {
      $cmd = "cd $dir/newdb;$config->{TAR} --format=gnu -cvf $dbName $copyList >$logDir/$MAKETAR_TXT 2>&1;cd $pwd";
    } else {
      $cmd = "cd $dir/newdb;$config->{TAR} cvf $dbName $copyList >$logDir/$MAKETAR_TXT 2>&1;cd $pwd";
    }
    ToolCommon::Log( $fh, "CMD = $cmd\n") if $params->{DEBUG};
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      ToolCommon::Log( $fh, "  OK\n");
    }
  }
  return $ok;
}

=head2 DownloadDbase($$$$$$$)

=begin html
Download migrated database to the NE using CLI.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>activeRelNum</td><td>the NE release coming from</td></tr>
  <tr><td>thisTftpDir</td><td>directory location relative to tftp-root</td></tr>
  <tr><td>userName</td><td>Unix user name in case of multi-user operation (command line option) empty otherwise</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub DownloadDbase($$$$$$) {
  my ($fh, $cli, $isam, $activeRelNum, $thisTftpDir, $userName) = @_;
  my $ok = 1;
  my $ret;
  my ($time1, $time2);
  my $neId = $params->{NONAME} ? $isam->[0] : $isam->[1];
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;

  ToolCommon::Log($fh, "Download migrated Database...");
  $ret = IsamCli::DownloadCompleteDb($cli, $params->{PRIMTFTPIP}, "$thisTftpDir/newdb"); # -- 1.1.6
  if (!defined($ret) || ! $ret) {
    ToolCommon::Log($fh, "return code not defined\n") if ($params->{DEBUG} && ! defined($ret));
    my $t = sprintf("FAILED (%s)", IsamCli::GetError());
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "  OK\n");
  }
  return $ok;
}

=head2 SwChecksOK($$$$$$$$$$)

=begin html
Perform some basic SW checks before starting the migration process.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>oswpInfo</td><td>pointer of the OSWP information as retrieved from the NE (both active and passive OSWP)</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>path</td><td>Location where the PBMT tools have been installed</td></tr>
  <tr><td>ntType</td><td>The NT type equipped in this ISAM (introduced in support of the NE family concept)</td></tr>
</table>

=end html

=cut

sub SwChecksOK($$$$$$$$$) {
  my ($fh, $cli, $snmpSession, $oswpInfo, $dir, $isam, $path, $ntType) = @_;
  my $ok = 1;

  if (Capability::ReleaseCompare(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'}, $oswpInfo->{'ACTIVE'}{'MINOR'}),
				 Capability::ConvertRelease(1,1), REGEXP, 0) < 0) { # -- 1.1.8
    my $t = "Automatic migrations only supported from R2.0 onwards";
    FamilyCommon::SetAndLogError($fh, $t, "$t !... skipping\n");
    $ok = 0;
  } elsif (($oswpInfo->{'ACTIVE'}{'NAME'} eq basename($params->{OSWP}))) {
    my $t = sprintf("Node already running %s", $oswpInfo->{'ACTIVE'}{'NAME'});
    FamilyCommon::SetAndLogError($fh, $t, "$t ... skipping\n");
    $ok = 0;
  } else {
    if (! $params->{OFLONLY}) {
      ToolCommon::Log($fh, "Checking new NT software is passive...");
      if ($oswpInfo->{'PASSIVE'}{'NAME'} ne basename($params->{OSWP})) {
	my $t = "SWP '$params->{OSWP}' not present as second SW";
	FamilyCommon::SetAndLogError($fh, $t, "  $t... skipping\n");
	$ok = 0;
      } elsif ($oswpInfo->{'PASSIVE'}{'AVAIL'} ne 'enabled') {
	my $t = "Second SWP not enabled";
	FamilyCommon::SetAndLogError($fh, $t, "  $t... skipping\n");
	$ok = 0;
      } else {
	ToolCommon::Log($fh, "  OK\n");
      }
      if ($ok) {
	ToolCommon::Log($fh, "Checking new NT software newer then active software...");
	my $swpInfo = ToolCommon::GetSwpBySnmp($snmpSession, $params->{DEBUG}, $oswpInfo->{'ACTIVE'}{'INDEX'}, $oswpInfo->{'PASSIVE'}{'INDEX'},
					       $toolData->{RETRIES});
	if (defined($swpInfo)) {
	  my $swVersCmp = Capability::ReleaseCompare(Capability::ConvertRelease($swpInfo->{'PASSIVE'}{'MAJOR'},
										$swpInfo->{'PASSIVE'}{'MINOR'}),
						     Capability::ConvertRelease($swpInfo->{'ACTIVE'}{'MAJOR'},
										$swpInfo->{'ACTIVE'}{'MINOR'}), REGEXP, 0);
	  if ($swVersCmp < 0) {
	    my $t = sprintf("New NT release older then active release (%d.%d vs %d.%d)",
			    $swpInfo->{'ACTIVE'}{'MAJOR'}, $swpInfo->{'ACTIVE'}{'MINOR'},
			    $swpInfo->{'PASSIVE'}{'MAJOR'}, $swpInfo->{'PASSIVE'}{'MINOR'});
	    FamilyCommon::SetAndLogError($fh, $t, "  $t !... skipping\n");
	    $ok = 0;
	  } else {
	    ToolCommon::Log($fh, "  OK\n");
	  }
	} else {
	  my $t = sprintf("Failed to determine SWP info: %s", ToolCommon::GetError());
	  FamilyCommon::SetAndLogError($fh, $t, "  $t !... skipping\n");
	  $ok = 0;
	}
      }
    }
  }
  return $ok;
}

=head2 MigrateOrUpgrade($$$$$)

=begin html
This function determines whether we need to migrate of upgrade the node.  True is returned in case of migration, FALSE in case of an upgrade
and the undefined value in case of an error.

=end html

=cut

sub MigrateOrUpgrade($$$$) {
  my ($fh, $config, $dirName, $isam) = @_;
  my $sourceDbVersion = $params->{SOURCE_DBVERSION};
  my $targetDbVersion = $params->{DBVER_TGTREL};
  return ($targetDbVersion eq $sourceDbVersion) ? 0 : 1;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Migrate($$$$)

=begin html
This function is called to perform migration of a node of the ISAM family.  In case of failure the undefined value is returned otherwise a string
of depending holding depending nodes is returned (only in case of ISAM-V, it will be an empty list in case of ISAM).<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>thisTftpDir</td><td>tftp directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Migrate($$$) {
  my ($newLog, $thisTftpDir, $isam) = @_;
  my $cli = FamilyCommon::GetCliSession2Close();
  my $ret;
  my ($newRelNum, $newFullRelNum);
  my $possible = 0;
  my $reason = "";
  my ($swMgntIniSize, $altSwMgntIniSize, $swParSize);
  my $sfpAssignmentTable;
  my $isSfpAssigned = 0;
  my $isamIsIsamV = 0;
  my $xvpsConfig = undef;
  my $xvpsDependancy = undef;
  my $nameList = "";
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $userName = getlogin();
  my $ipAddr;
  my $duplexNT=0;
  my $nanteOrfantf = 0; 
  my $ltActivating = 0;

  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  # Read warning levels
  %warnLevels = Utils::ReadConfig(sprintf("%s/%s", FamilyCommon::GetToolDirName(), $toolData->{WARNINGFILE}));
  if (! defined($params->{OFLDB})) {
    if (! $params->{ACTONLY}) {
      # Keep DB uploaded with previous migration
      #  if (! -e $dir . "/$NTINFILENAME") {
      if (!FamilyCommon::CreateBUFile($parentLogFh, $nodeLogFH, $dir, $NTINFILENAME, $newLog, $childNum, $params->{DEBUG})) {
	CORE::exit(CREATE_FILE_FAIL_CODE);
      }
    }
  }
  if ((defined($params->{OFLDB}) && (! $params->{OFLONLY})) ||
      (! defined($params->{OFLDB}))) {
    if (!defined($cli)) {
      #-- Open Cli Connection
      ToolCommon::Log($nodeLogFH, "Open CLI...");
      $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
      if (!defined($cli)) {
	FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				     EC_OPENCLI_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK\n");
      }
      FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
      ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});
    }
    #-- STEP 0: acquire transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");
  my $oldTransactionTO = IsamCli::GetTransactionTimeout($cli);
  if (!defined($oldTransactionTO)) {
     FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
     	     EC_GET_TRANS_TO_FAILED,
     	     sprintf(": %s", IsamCli::GetError()),
     	     1);
     return undef;
  } else {
     ToolCommon::Log($nodeLogFH, "Transaction timeout: $oldTransactionTO\n") if $params->{DEBUG};
     FamilyCommon::SetOldTransactionTO($oldTransactionTO);
     $ret = IsamCli::SetTransactionTimeout($cli, TRANSACT_TIME);
     if ($ret) {
       ToolCommon::Log($nodeLogFH, " OK\n");
     } else {
     	 FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
     	       sprintf(": %s", IsamCli::GetError()),
     	       EC_SET_TRANS_TO_FAILED,
     	       1);
     	 return undef;
     }
  }
 }

  my $ntInfo;
  my ($activeNt, $passiveNt);
  if (! defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
    #-- STEP 1a                                                         --- 1.2.0 BEGIN
    #-- Get NT type information
    ToolCommon::Log($nodeLogFH, "Obtain extended NT information...");
    $ntInfo = IsamCli::GetNtExtendedInfo($cli);
    if (!defined $ntInfo) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				   EC_GET_NT_INFO_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return undef;
    } else {
      ToolCommon::Log($nodeLogFH, "  OK\n");
      ToolCommon::Log($nodeLogFH, sprintf("  NTA: %s - %s, %s (ICS %s)\n                       NTB: %s - %s, %s (ICS %s)\n",
					  uc($ntInfo->{NTA}{TYPE}), $ntInfo->{NTA}{STATUS}, $ntInfo->{NTA}{ACTIVE}, $ntInfo->{NTA}{ICS},
					  uc($ntInfo->{NTB}{TYPE}), $ntInfo->{NTB}{STATUS}, $ntInfo->{NTB}{ACTIVE}, $ntInfo->{NTB}{ICS}));
    }

    if (($ntInfo->{NTA}{TYPE} ne $ntInfo->{NTB}{TYPE}) &&
	(uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") &&
	(uc($ntInfo->{NTB}{STATUS}) ne "EMPTY")) {
      FamilyCommon::IncrErrorCount(sprintf("Reduncdancy between two incompatible NT types ('%s' and '%s')... skipping\n",
					   $ntInfo->{NTA}{TYPE}, $ntInfo->{NTB}{TYPE}),
				   EC_INCOMPAT_NT,
				   "(NT-A: '$ntInfo->{NTA}{TYPE}', NT-B: '$ntInfo->{NTB}{TYPE}')",
				   1);
      return undef;
    } elsif ((uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") && (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY")) {
		ToolCommon::Log($nodeLogFH, "Duplex System detected !!!\n");
                $duplexNT = 1;
        if ((uc($ntInfo->{NTA}{TYPE}) ne 'NANT-D')) {
              $nanteOrfantf = 1; 
        }

      if($duplexNT) {  ## duplex NT check not done now since NT-B could also be in prot-grp-lckd state during migration
	      ToolCommon::Log($nodeLogFH, "Waiting for NT Sync to happen...\n");
          my $valid_idle_sync_reasons = "prot-grp-lckd";
	      $ret = IsamCli::WaitForNTSync($cli);

	      if(!$ret){
		ToolCommon::Log($nodeLogFH, "NT-A and NT-B are not in Sync, cannot continue ... skipping \n");
		FamilyCommon::IncrErrorCount("Standby NT Not in Sync, Cannot Continue ... skipping\n",
			EC_ISAM_NT_SYNC_FAILED, 
			" ",
			1); 
		return undef;
	      }
      }
   }

    if (! $params->{FORCEMIG}) {
      if ((uc($ntInfo->{NTA}{TYPE}) ne "EMPTY") && (uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") && (! Capability::NtSupported($ntInfo->{NTA}{TYPE}))) {
	FamilyCommon::IncrErrorCount("NT-A type '$ntInfo->{NTA}{TYPE}' not supported for migration... skipping\n",
				     EC_NT_TYPE_UNKNOWN,
				     "($ntInfo->{NTA}{TYPE})",
				     1);
	return undef;
      }
      if ((uc($ntInfo->{NTB}{TYPE}) ne "EMPTY") && (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY") && (! Capability::NtSupported($ntInfo->{NTB}{TYPE}))) {
	FamilyCommon::IncrErrorCount("NT-B type '$ntInfo->{NTB}{TYPE}' not supported for migration... skipping\n",
				     EC_NT_TYPE_UNKNOWN,
				     "($ntInfo->{NTB}{TYPE})",
				     1);
	return undef;
      }
    }
    $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';
    $passiveNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTB' : 'NTA';
    #-- STEP 1a
  }

  my $oswpInfo;
  my $dbVersInfo;
  if (! defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
    #-- STEP 1b: Check if needed files are present on NT
    $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}), undef,
					$toolData->{RETRIES});
    if (!defined($oswpInfo)) {
      FamilyCommon::IncrErrorCount("Obtaining active SW failed.\n",
				   EC_GET_OSWP_FAILED,
				   sprintf(": %s", ToolCommon::GetError()),
				   1);
      return undef;
    }
    ToolCommon::Log($nodeLogFH, "Verify database versions...");
    $dbVersInfo = IsamCli::GetDbVersionInfo($cli);
    if (!defined($dbVersInfo)) {
      FamilyCommon::IncrErrorCount("  FAILED...skipping!!\n",
				   EC_GET_DB_INFO_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return undef;
    } else {
      ToolCommon::Log($nodeLogFH, "  OK\n");
      if ($params->{DEBUG}) {
	foreach my $key (keys %{$dbVersInfo}) {
	  ToolCommon::Log($nodeLogFH, "OSWP '$key': '$dbVersInfo->{$key}'\n");
	}
      }
    }
  }
  my $dbVersionActive = $params->{SOURCE_DBVERSION};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveRel = substr($dbVersionActive,1);
  my $dbVersionTargetRel = substr($dbVersionTarget,1);
  my $activeRelNum;
  my $fullActiveRelNum;
  my $regExp = REGEXP;
  if (!defined($params->{OFLDB})) {
    $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
					       $oswpInfo->{'ACTIVE'}{'MINOR'},
                                               0,
					       $dbVersInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});
    $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						   $oswpInfo->{'ACTIVE'}{'MINOR'},
						   $oswpInfo->{'ACTIVE'}{'SUFFIX'},
                                                   $dbVersInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});
    if ( $params->{SOURCE_DBVERSION} ne $oswpInfo->{'ACTIVE'}{'DBVERSION'}) {
      FamilyCommon::IncrErrorCount(sprintf("DB version not as expected: %s vs %s\n",
					   $params->{SOURCE_DBVERSION}, $oswpInfo->{'ACTIVE'}{'DBVERSION'}),
				   EC_DB_VERS_UNEXPECTED,
				   sprintf("%s vs %s",
					   $params->{SOURCE_DBVERSION}, $oswpInfo->{'ACTIVE'}{'DBVERSION'}),
				   1);
      return undef;
    }
  } else {
    if (basename($params->{SRC_OSWP}) =~ /$regExp/) {
      $activeRelNum = Capability::ConvertRelease($1, $2, 0, $params->{OFL_DB_VERS});
      $fullActiveRelNum = Capability::ConvertRelease($1, $2, $3, $params->{OFL_DB_VERS});
    }
  }
  ToolCommon::Log($nodeLogFH, "Active release info (short and full stream): ($activeRelNum, $fullActiveRelNum)\n") if $params->{DEBUG};
  if( $params->{BTYPE} eq "YANG") {
    ToolCommon::Log($nodeLogFH, "Yang build going to activate directly");
    goto BUILDACTIVATE;
  }
  my $dbVerTargetRel = $params->{DBVER_TGTREL};
  
  if (($params->{OFLONLY}) || defined($params->{OFLDB})) {
    if (basename($params->{SWPNAME}) =~ /$regExp/) {
      $newRelNum = Capability::ConvertRelease($1, $2, 0, $dbVerTargetRel);
      $newFullRelNum = Capability::ConvertRelease($1, $2, $3, $dbVerTargetRel);
    }
  } else {
    $newRelNum = Capability::ConvertRelease($oswpInfo->{'PASSIVE'}{'MAJOR'},
					    $oswpInfo->{'PASSIVE'}{'MINOR'},
                                            0,
                                            $dbVerTargetRel); 
    $newFullRelNum = Capability::ConvertRelease($oswpInfo->{'PASSIVE'}{'MAJOR'},
						$oswpInfo->{'PASSIVE'}{'MINOR'},
						$oswpInfo->{'PASSIVE'}{'SUFFIX'}, $dbVerTargetRel); # -- 1.2.0
  }
  ToolCommon::Log($nodeLogFH, "New release info (short and full stream): ($newRelNum, $newFullRelNum)\n") if $params->{DEBUG};
  
  if (!defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
    #-- STEP: Configure AccessOperatorId (supported starting R3.6).  When going fronm 3.7 need to configure via CLI (OFLMT does not take care)
    if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 7), REGEXP, 0) >= 0) &&
	($params->{ACCESSOPRID})) {
      ToolCommon::Log($nodeLogFH, "Set accessOperatorId...");
      if (! ToolCommon::AccessOprIdConfigure($nodeLogFH, $params->{DEBUG}, $cli, $config->{ACCESS_OPERATOR_ID})) {
	FamilyCommon::IncrErrorCount(sprintf("  Failed...skipping: %s", ToolCommon::GetError()),
				     EC_CONFIG_OPRID_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK\n");
      }
    }
  }

  if ($params->{ACTONLY}) {
    # ACTONLY gets reset in case the ofldb option has been used...
    if ( defined($params->{DBVER_TGTREL}) and ($params->{DBVER_TGTREL} ne $oswpInfo->{'PASSIVE'}{'DBVERSION'} ) ) {
      FamilyCommon::IncrErrorCount(sprintf("New DB version not as expected: %s vs %s\n",
                     $params->{DBVER_TGTREL}, $oswpInfo->{'PASSIVE'}{'DBVERSION'} ),
				   EC_DB_VERS_UNEXPECTED,
				   sprintf("%s vs %s",
                     $params->{DBVER_TGTREL}, $oswpInfo->{'PASSIVE'}{'DBVERSION'} ),
				   1);
      return undef;
    }
  }
  if (! defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
	  if (! SwChecksOK($nodeLogFH, $cli, $snmpSession, $oswpInfo, $dir, $isam, 0, $dirName,
			   defined($params->{OFLDB}) ? $params->{OFL_NT_TYPE} : $ntInfo->{$activeNt}{TYPE})) {
	    FamilyCommon::IncrErrorCount("SW checks failed.\n",
					 EC_SW_CHECKS_NOK,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    return undef;
	  }
  }

  #-- STEP 1c
  #-- Verify migration path
  $params->{IS_ISAMV_SRV} = 0;
  $params->{ISAM_HAS_VOICE} = 0;
  $params->{ISAM_VOICE_MIGR} = 0;
  $params->{CDE_TMP_PATH} = undef; 
  $params->{IGMP_MAX_UNCFG} = undef; 
  if (! $params->{ACTONLY}) {
    if (! defined($params->{OFLDB}) ||
	(defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
      if (! $params->{FORCEMIG}) {
	     ToolCommon::Log($nodeLogFH, "Checking whether migration restrictions apply...");
          if ( ($dbVersionActive eq $dbVersionTarget ) &&
               (! $params->{TOECNTC} && ! $params->{TONANTB} && ! $params->{TOECNTE})) {
                       FamilyCommon::ReportStatus( EC_NEEDS_UPGRADE,
                                 sprintf("Migration from %s (%s) to %s (%s) is not possible, needs upgrade... \n",
                               $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget ), undef );
                       if ($params->{CONTINUE}) {
                         return undef;
                       } else {
                         CORE::exit(UPG_IMPOSSIBLE_CODE);
                       }
          }
          if ($dbVersionActive ne $dbVersionTarget) {
              #-- Verify migration path
              Logger::LogNotify($nodeLogFH, "Migration restriction checks...\n");
              my ($restricted, $restricted_reason) = Capability::IsMigrateRestricted( $ntInfo->{$activeNt}{TYPE},
                                                                           $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                        basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
              if (not $restricted) {
                  Logger::LogTrace($nodeLogFH, "Migrate Restrictions... NONE\n");
              } else {
                  FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                                    $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                                    basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
                  if ($params->{CONTINUE}) {
                    return undef;
                  } else {
                    FamilyCommon::ToolExit(UPG_IMPOSSIBLE_CODE);
                  }
              }
              ($possible, $reason) = CapabilityReEngr::MigrationPossible( $ntInfo->{$activeNt}{TYPE}, $dbVersionActive, $dbVersionTarget);
              if ($possible) {
                  Logger::LogTrace($nodeLogFH, "Migrate Restrictions... NONE\n");
              } else {
                  FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                                  $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason ));
                  if ($params->{CONTINUE}) {
                    return undef;
                  } else {
                    FamilyCommon::ToolExit(UPG_IMPOSSIBLE_CODE);
                  }
              }
         }
      }  else {
	    ToolCommon::Log($nodeLogFH, "Forced migration, continue without checks...\n");
      }

      if (((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 6), REGEXP, 0) >= 0) &&  
           (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(4, 2), REGEXP, 0) < 0)) &&  
          ((Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(4, 2), REGEXP, 0) >= 0)  &&
           (Capability::ReleaseCompare($newFullRelNum, Capability::ConvertRelease(4, 2, 2), REGEXP, 0, 1) < 0))) {   
        ToolCommon::Log($nodeLogFH, "Check Vmac Unique Host-id parameter limitations...");
	if (!ToolCommon::CheckVlanUniqHostId($cli)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", ToolCommon::GetError()),
				       EC_VMAC_HOST_ID_LIMITS_EXCEED,
				       undef,
				       1);
	  return undef;
	} else {
	  ToolCommon::Log($nodeLogFH, "  OK\n");
	}
      }
      ## Migration of IGMP unconfigured multicast channels - ALU01836984
      if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(4, 1, 0), REGEXP, 0) >= 0) &&  
          (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(4, 4), REGEXP, 0) >= 0)) {
          my $accOprId = ToolCommon::GetAccessOperId($nodeLogFH, $snmpSession);
          if (defined($accOprId) && ($accOprId == $DT_ACCOPR)) {
                   $params->{IGMP_MAX_UNCFG} = 1; 
                   ToolCommon::Log($nodeLogFH, "Enable Migration of IGMP unconfigured multicast channels OK\n");
	           
                   $setVlanLabel = 1;   # for migration of ANCP parameters 
          }
      }

      $setIPDCfgDhcp = 0;
      $setIPDCfgIGMPInterface = 0;
      $setMaxIPDCfgRange = 0; 
      $setFastReroute = 0;
      @feltbLts = ();

      if(Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(5,8), REGEXP, 0) >= 0) {
               $setFastReroute = 1;   
      }
      ## for ALU01993252 - changes for core-1 mib range update in R51
      if(Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(5,1), REGEXP, 0) >= 0) {
               $setMaxIPDCfgRange = 1;   
      }
      ## for ALU02106733 - changes for core-1 interface update in or less than R52
      if(Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(5,2), REGEXP, 0) <= 0) {
	       $setIPDCfgIGMPInterface = 1;
      }
      ## for ALU02175921 - changes for core-1 interface update in or greater than R53
      if(Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(5,3), REGEXP, 0) >= 0) {
	       $setIPDCfgDhcp = 1;
      }

      ### for ALU02400566 - Changes to support TPID translation enable/disable support in PBMT's migration
      if(
         (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(5,8), REGEXP, 0) >= 0) and 
         (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(5,8), REGEXP, 0) < 0)
        ) {
               @feltbLts = @{ IsamCli::GetFeltBSlotDetails($cli) };
			ToolCommon::Log($nodeLogFH, "felt-b slot details @feltbLts ");
			my $profileMode = uc(IsamCli::GetProfileSlotNumberMode($cli, $params->{USERNAME}));
			my $shelfType = uc(IsamCli::GetShelfType($cli));
			my $ntType = ToolCommon::DetermineNtType( );
			if(Capability::IsTypeBased($profileMode)) {
				@feltbLts = @{Capability::ConvertLTslot($shelfType,$ntType,@feltbLts)};
				ToolCommon::Log($nodeLogFH, "Since CLI is configured type-based mode, modified felt-b slot details : @feltbLts \n");
			}
      }

      #-- Check if automatic DB activation after download is configured
      if (! defined($params->{OFLDB})) {
         my $retCode =  IsamCli::CheckDBAutoActivate($cli);
	 if (defined($retCode) && $retCode) {
	    FamilyCommon::IncrErrorCount("",
					 EC_DB_AUTO_ACTIVATE_ON_DWLD,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    return undef;
         }
      }

      #-- STEP 1c                                                          --- 1.2.0 END
      if (! defined($params->{OFLDB})) {
            Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
            return undef if( ToolCommon::IsTransferOnGoing() );

      $xvpsConfig = undef;
      $xvpsDependancy = undef;
      $nameList = "";
      # Step 1f check whether ISAM is ISAM-V server and store for later usage
      ToolCommon::Log($nodeLogFH, "Checking whether ISAM is ISAM-V server...");
      $isamIsIsamV = ToolCommon::IsamHasVoiceServer($cli);
      if (!defined($isamIsIsamV)) {
        FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s", ToolCommon::GetError()),
                                     EC_SHOW_EQPT_FAILED,
                                     undef,
                                     1);
        return undef;
      } else {
        ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s server\n", $isamIsIsamV ? "is ISAM-V" : "isn't ISAM-V"));
      }
      $params->{IS_ISAMV_SRV} = $isamIsIsamV;
      # Step 1f check whether ISAM is ISAM-V server and store for later usage

	ToolCommon::Log($nodeLogFH, "Checking whether ISAM has voice boards...");
	my $isamHasV = ToolCommon::IsamHasVoice($cli);
	if (!defined($isamHasV)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s", ToolCommon::GetError()),
				       EC_SHOW_EQPT_FAILED, 
				       undef,
				       1);
	  return undef;
	} else {
	  ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s voice boards\n", $isamHasV ? "has" : "hasn't"));
	}
	$params->{ISAM_HAS_VOICE} = $isamHasV;
        if ($isamIsIsamV) {
          if (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(4, 4), REGEXP, 0) >= 0) {
            if (! $params->{IVPS_MIG_POSSIBLE}) {
              FamilyCommon::IncrErrorCount("",
                                           EC_ISAM_HAS_VOICE,
                                           undef,
                                           1);
              return undef;
            }
          } else {
            if (! -f "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}") {
              FamilyCommon::IncrErrorCount("$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE} not found",
                                           EC_CDE_NOTFOUND,
                                           undef,
                                           1);
              return undef;
            }
          }
        }

    if ( ($dbVersionActiveRel ge "5.800") and ($params->{'BOARD-FAMILY'} eq Capability::IsamHiCapEtsiFamily()) ) {
        my $issu_status = ToolCommon::CheckISSU($cli);
        if( $issu_status ) {
            FamilyCommon::ReportStatus( EC_SETUP_ISSU_FAILED,ToolCommon::GetError());
            return undef;
        }
    }

	#-- STEP 3: Backup NT
	ToolCommon::Log($nodeLogFH, "Backup NT Database...");
	my $dbSize;
	my $time1 = time();

        if(!$params->{NOTFTP}){ 
	  $ret = IsamCli::UploadDb($cli, 'actual-active',
				 $params->{PRIMTFTPIP}, $thisTftpDir, 1);
         } else {
 
          $ret = IsamCli::UploadDb($cli, 'actual-active',
                                  $params->{PRIMTFTPIP}, $toolData->{FXRDATA}{HOMEDIR}, 1);
         }

	if (!defined($ret) || !$ret)  {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED (%s)\n", IsamCli::GetError()),
				       EC_NT_DB_BU_FAIL,
				       sprintf(": %s", IsamCli::GetError()),
				       1);
	  return undef;
	} else {
	  my $realName = "$toolData->{FXRDATA}{HOMEDIR}/$NTINFILENAME";
	  ToolCommon::Log($nodeLogFH, "Uploaded DB into '$realName'\n") if $params->{DEBUG};
	   if (! -e "$toolData->{FXRDATA}{HOMEDIR}/$NTINFILENAME") {
		print "not present\n";
 		FamilyCommon::IncrErrorCount(sprintf("  NT DB not present\n"),
				       EC_NT_DB_NOT_PRESENT,
				       undef,
				       1);
	  	return undef;
       }
	  $dbSize = -s "$realName";
	  ToolCommon::Log($nodeLogFH, sprintf("  OK, transfered $dbSize bytes in %d seconds\n", time() - $time1));
	if ($dbSize == 0) {
	  FamilyCommon::IncrErrorCount(sprintf("  Size of database file 0\n"),
				       EC_NT_DB_EMPTY,
				       undef,
				       1);
	  return undef;
	 }
	}
      } else {
	my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
	my $realName = "$toolData->{FXRDATA}{HOMEDIR}/$dbName";
	ToolCommon::Log($nodeLogFH, "Copy $params->{OFLDB} to $realName...\n") if $params->{DEBUG};
	if (! copy($params->{OFLDB}, $realName)) {
	  FamilyCommon::IncrErrorCount("Copy of '$params->{OFLDB}' to '$realName' failed: $!",
				       EC_COPY_OFLDB_FAILED,
				       undef,
				       1);
	  return undef;
	}
      }
    } else {
      my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
      my $realName = "$toolData->{FXRDATA}{HOMEDIR}/$dbName";
      ToolCommon::Log($nodeLogFH, "Copy $params->{OFLDB} to $realName...\n") if $params->{DEBUG};
      if (! copy($params->{OFLDB}, $realName)) {
	FamilyCommon::IncrErrorCount("Copy of '$params->{OFLDB}' to '$realName' failed: $!",
				     EC_COPY_OFLDB_FAILED,
				     undef,
				     1);
	return undef;
      }
      $isamIsIsamV = DetermineXvpsPresence($nodeLogFH, $dir);
      if (! defined($isamIsIsamV)) {
             FamilyCommon::IncrErrorCount("",
                                           EC_IVPS_DB_FAIL,
                                           sprintf(": %s", FamilyCommon::GetError()),
                                           1);
              return undef;
      } else {
              $params->{IS_ISAMV_SRV} = $isamIsIsamV;
      }
    }

    my $communityStrings = DetermineNtCommunities($nodeLogFH, $dir);
    if (! defined($communityStrings)) {
      FamilyCommon::IncrErrorCount("",
                                   EC_GETCOMMTY_ERROR,
                                   sprintf(": %s", FamilyCommon::GetError()),
                                   1);
      return undef;
    }


    if ($isamIsIsamV &&  (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(4, 4), REGEXP, 0) >= 0)) {
        $xvpsConfig = CopyXvpsList($nodeLogFH, $communityStrings);
        $xvpsDependancy = GetXvpsDependancyByCommunity($nodeLogFH, $communityStrings, $isam->[0]);
        if (! defined($xvpsDependancy)) {
          FamilyCommon::IncrErrorCount("",
                                       EC_IVPS_GETDEP_ERROR,
                                       sprintf(": %s", FamilyCommon::GetError()),
                                       1);
          return undef;
        } else {
          foreach my $ivpsId (sort keys (%{$xvpsDependancy})) {
            foreach my $key (sort keys %{$xvpsDependancy->{$ivpsId}}) {
              if ($xvpsDependancy->{$ivpsId}->{$key} eq "") {
                $nameList .= "??$key??, ";
              } else {
                $nameList .= "$key ($xvpsDependancy->{$ivpsId}->{$key}), ";
              }
            }
          }
          $nameList = substr($nameList, 0, -2) if $nameList ne "";
        }
    }

    #-- STEP 5a: Migrate the NT database
    $ret = MigrateNTDbase($nodeLogFH, $dir, $fullActiveRelNum, $newFullRelNum, $dirName);  # -- 1.5..1.3
    if (!$ret) {
      FamilyCommon::IncrErrorCount("",
				   EC_NT_MIG_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return undef;
    }
    #-- STEP 6: Migrate the IVPS database from R4.3 onwards
    if (((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(4, 3), REGEXP, 0) >= 0)) 
        &&  $isamIsIsamV) {
        $ret = MigrateXvpsDbase($parentLogFh, $nodeLogFH, $isam, $fullActiveRelNum, $newFullRelNum, $xvpsConfig, $toolData->{FXRDATA}{XVPS_HOMEDIR}, $childNum, $userName, $newLog, $dirName);
        if (! $ret) {
          FamilyCommon::IncrErrorCount("",
                                       EC_IVPS_MIG_FAILED,
                                       sprintf(": %s", FamilyCommon::GetError()),
                                       1);
          return undef;
        }
    }

    #-- STEP 7: Combine both databases
    #-- From R4.3: ISAM-V migration, so also CDE needs to be included.
    #$ret = CombineDbases($nodeLogFH, $dir, $activeRelNum, $newRelNum); # -- 1.1.4
    $ret = CombineDbases($nodeLogFH, $dir, $fullActiveRelNum, $newFullRelNum, $xvpsConfig, "$toolData->{TFTPDIR}/$toolData->{FXRDATA}{XVPS_HOMEDIR}"); # -- 1.1.4
    if (!$ret) {
      FamilyCommon::IncrErrorCount("",
				   EC_DB_COMBINE_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return undef;
    }

    if ($params->{OFLONLY}) {
      my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
      ToolCommon::Log($nodeLogFH, "Chosen to only do the offline migration\n", 1);
      ToolCommon::Log($nodeLogFH, "  migrated databases for this ISAM found in\n", 1);
      ToolCommon::Log($nodeLogFH, sprintf("  NT and IHUB: %s/newdb/$dbName\n",
					  $toolData->{FXRDATA}{HOMEDIR}), 1);
      ToolCommon::Log($nodeLogFH, "  Dependant nodes related to this ISAMV: $nameList\n", 1) if ($nameList ne "");
      FamilyCommon::AdvanceToNext("");
      return undef;
    }
    my $ltInfo = IsamCli::GetLtConfigState($cli);
    foreach my $lt ( keys %$ltInfo ) {
       $ltActivating = 1 if( ( defined $ltInfo->{$lt}{TYPE} ) and ( $ltInfo->{$lt}{TYPE} ne "fglt-a" ) and 
           ( $ltInfo->{$lt}{TYPE} =~ /^(fw|fg)lt-[a-z]$/ ) ) ;
    }

    #-- STEP 8a: Download new database to NT
    Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
    return undef if( ToolCommon::IsTransferOnGoing() );

    if(!$params->{NOTFTP}) { 
      $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum, $thisTftpDir, $userName); # -- 1.1.6
    } else {
      $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum,"/". $toolData->{FXRDATA}{HOMEDIR}, $userName);
    } 

    if (!$ret) {
      if (FamilyCommon::GetSigPipeReceived()) {
	# Witnessed at FT:
	# under certain circumstances no more telnet/CLI access is possible due to some problem on the unix side: the pipe
	# carrying the traffic to the node is 'gone' after we have performed:
	# - NT offline migration
	# - SHUB offline migration
	# - execute tar to combine the migrated databases.
	# Trying to write to this pipe results in a SIGPIPE.
	# Workaround: open 2nd CLI session and try again.  The transaction is still taken by 127.0.0.1 at the moment we get the new CLI session.
	ToolCommon::Log($nodeLogFH, "Open new CLI...");
	$cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	if (!defined($cli)) {
	  FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open new cli session for %s: %s\n",
					       FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				       EC_OPENCLI_FAILED,
				       sprintf(": %s", ToolCommon::GetError()),
				       1);
	  return undef;
	} else {
	  ToolCommon::Log($nodeLogFH, "  OK\n");
	}
	FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});
	FamilyCommon::ClearSigPipeReceived();


    Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
    return undef if( ToolCommon::IsTransferOnGoing() );

        if(!$params->{NOTFTP}) {
 	 $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum, $thisTftpDir, $userName); # -- 1.1.6
        } else { 
          $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum, "/".$toolData->{FXRDATA}{HOMEDIR} , $userName);
        }

	if (!$ret) {
	  FamilyCommon::IncrErrorCount("",
				       EC_DB_DL_FAILED,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	  return undef;
	}
      } else {
	FamilyCommon::IncrErrorCount("",
				     EC_DB_DL_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    }
    if ((uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") && (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY")) {
      #-- STEP 8c: Redundant system, wait for stability   Needed?
      ToolCommon::Log($nodeLogFH, "Waiting for stability...");
      sleep(180);   # Only for redundant system?
      ToolCommon::Log($nodeLogFH, " OK\n");
    }
  }
  ToolCommon::Log($nodeLogFH, "Reset transaction timeout and release transaction...");
  $ret = IsamCli::SetTransactionTimeout($cli, FamilyCommon::GetOldTransactionTO());
  if (! $ret) {
    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				 EC_SET_TRANS_TO_FAILED, 
				 sprintf(": %s", IsamCli::GetError()),
				 1);
    return undef;
  }
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

BUILDACTIVATE:
  if( $params->{BTYPE} eq "YANG") {
    ToolCommon::Log($nodeLogFH, "Started Yang build going to activate directly");
  }
### KRAV
  if ($params->{NOACT}) {
    # Save DB info in activation file for later activation purposes.
    # Print info on terminal and to file.
    ToolCommon::Log($nodeLogFH, "Process ends here, continue with migrate.pl --actonly...\n");
  } else {

	#-- STEP : Check if migration interrupt on emergency call feature is enabled
	if ($params->{EMERG_CALL_FEATURE}) {
		my $isActive = 0;
		ToolCommon::Log($nodeLogFH, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
	    $isActive = ToolCommon::AlarmRetrieval($nodeLogFH, $cli, $toolData);
	    if($isActive) {													   ## emergency call alarm is active
			FamilyCommon::IncrErrorCount(sprintf("  FAILED: Emergency call max retries reached\n"),
				     EC_EMERGENCY_CALL_ACTIVE,
				     undef,
				     1);
		    return undef;
		 }
	}
	#-- STEP : End of migration interrupt on emergency call feature.

    # Make sure we do not get an NT reset due to a NON_RECOV at the time of OSWP activation.
    # Workaround for R2.4 problem in OSWP activation for ARAM-D with exp shelves.
    if ($params->{REMAPHOTRESET}) {
      $ret = Octopus::RemapErrRecord($params->{DEBUG}, $dir, $isam, $dirName);
      if (! $ret) {
	FamilyCommon::IncrErrorCount("",
				     EC_MAP_ERR_REC_FAILED,
				     sprintf(": %s", Octopus::GetError()),
				     1);
	return undef;
      }
    }
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  return if( not ToolCommon::ActivateSwp($oswpInfo) );
    #-- NT sw changed, system will reboot, close cli session

    #-- STEP 12: Wait for restart
    if ($toolData->{ACTPINGWAIT} > 0) {
      ToolCommon::Log($nodeLogFH, "Waiting $toolData->{ACTPINGWAIT} seconds for the system to restart...");
      sleep($toolData->{ACTPINGWAIT});
      ToolCommon::Log($nodeLogFH, "  OK\n");
    }

    $ret = FamilyCommon::WaitForRestart($nodeLogFH, $snmpSession, $isam, undef);  # -- 1.0.4
    if (!$ret) {
      if( defined $params->{BTYPE} and ($params->{BTYPE} ne "SNMP") ) {
	ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **ATTENTION** Yang Build activation, Restart of %s failed (timed out), Conditionally success... Please verify\n",
					      FamilyCommon::GetFailedNode()), 1);
          my $statusLogFh = FamilyCommon::GetNodeStatusLog();
          if (defined($statusLogFh)) {
            print $statusLogFh sprintf("%s\n", ErrorCodes::ExitString(CONDITONAL_SUCCESS_CODE));
            FamilyCommon::CloseNodeStatusLog();
          }
          if ($params->{CONTINUE}) {
              return undef;
          } else {
            CORE::exit(CONDITONAL_SUCCESS_CODE);
          }
      } else {
          #-- asam is not back ... start a panic
          FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s... skipping\n", IsamCli::GetError()),
            			   EC_RESTART_FAILED,
            			   undef,
            			   1);
          if ($params->{CONTINUE}) {
            ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **ATTENTION** Restart of %s failed (timed out), please verify %s.\n",
            				      FamilyCommon::GetFailedNode(), $newLog), 1);
            return undef;
          } else {
            ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **FATAL** Restart of %s failed (timed out), please verify %s.\n",
            				      FamilyCommon::GetFailedNode(), "$params->{LOGFILE}_$childNum"), 1);
            CORE::exit(NT_RESTART_FAIL_CODE);
          }
      }
    }

    # Reset transaction owner since this seems to survive a migration (even though the prozone gets erased...
    $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
    if (!defined($cli)) {
      FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
                                   EC_OPENCLI_FAILED,
                                   sprintf(" (re-open after reboot): %s", ToolCommon::GetError()),
                                   1);
      return undef;
    }  else {
      ToolCommon::Log($nodeLogFH, "  OK\n");
      FamilyCommon::ClearCliSession2Close();
    }
   if (IsamCli::GetAutoBkpConfig($cli)) {
      Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
      return undef if( ToolCommon::IsTransferOnGoing() );
   }
    IsamCli::Disconnect($cli);
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

    #-- Open Cli Connection
    ToolCommon::Log($nodeLogFH, "Re-open CLI...");
    $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
    if (!defined($cli)) {
      FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				   EC_OPENCLI_FAILED,
				   sprintf(" (re-open after reboot): %s", ToolCommon::GetError()),
				   1);
      return undef;
    }  else {
      ToolCommon::Log($nodeLogFH, "  OK\n");
      FamilyCommon::ClearCliSession2Close();
    }
  
    #-- STEP 13: Check Correct SWP is active
    my ($restartOswpInfo, $oswpInfoIsEmpty);
    ($ret, $oswpInfoIsEmpty, $restartOswpInfo) = FamilyCommon::CheckActiveSoftwareIsNew($nodeLogFH, $cli, $snmpSession,
											$params->{OSWP}, $dir, $isam,
											$ntInfo->{$activeNt}{TYPE});
    if (! $ret) {
      FamilyCommon::IncrErrorCount("",
				   EC_NEW_SW_NOT_ACT,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return undef;
    }

    if (($config->{ACCESS_OPERATOR_ID} > 0) && ($config->{ACCESS_OPERATOR_ID} <= 65535) && ($params->{ACCESSOPRID})) {
      my $accOperId = ToolCommon::GetAccessOperId($nodeLogFH, $snmpSession);
      if (defined($accOperId)) {
	if ($config->{ACCESS_OPERATOR_ID} != $accOperId) {
	  FamilyCommon::IncrErrorCount(sprintf("Unexepected access operator ID: retrieved %d while expecting %d\n",
					       $accOperId, $config->{ACCESS_OPERATOR_ID}),
				       EC_ACC_OPERID_MISMATCH,
				       sprintf(":  retrieved %d while expecting %d", $accOperId, $config->{ACCESS_OPERATOR_ID}),
				       1);
	  return undef;
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("Failed to retrieve the access-oper-id: %s\n", ToolCommon::GetError()),
				     EC_GET_ACC_OPERID_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      }
    }

    my $dbActVersInfo = IsamCli::GetDbVersionInfo($cli);
    if (!defined($dbActVersInfo)) {
      my $t = sprintf("Obtaining NT Database version information failed: %s", IsamCli::GetError());
      FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
      return undef; 
    } else {
	    my $dbActiveInfo = $dbVersInfo->{$restartOswpInfo->{'ACTIVE'}{'INDEX'}};
	    my $dbPassiveInfo = $dbVersInfo->{$restartOswpInfo->{'PASSIVE'}{'INDEX'}};
            if((($dbActiveInfo ge $R43DBVer) && ($dbPassiveInfo =~ /$R4230DBVer/)) ||
               (($params->{ISAM_HAS_VOICE}) && ($dbActiveInfo =~ /$R4xDBVer_RE/))) {
                ToolCommon::Log($nodeLogFH, "  To Enable User-to-User Hairpin\n");
                IsamCli::SetU2UHairpin($cli); 
	        sleep 3;
                ToolCommon::Log($nodeLogFH, "  OK\n");
            }
    }
  }
  
  IsamCli::AllowAlarms($cli);
  FamilyCommon::CloseCliSession2Close();  # Store for IncrErrorCount
}

=head2 CheckEnvironment($$$$)

=begin html
Check the migration environment.  In case unknown offline migration tools are detected (based on their version number) or files not being executable
the script will abort and not start any activity.

=end html

=cut

sub CheckEnvironment($$$) {
  my ($logFile, $config, $installDir) = @_;
  my $envOk = 1;
  my $toolConfigData = FamilyCommon::GetToolData();
  my $oflVersInfo = Utils::DetermineOFLMTVersions($params->{DEBUG}, $toolData->{NTDBTOOL}, undef, undef, $toolConfigData->{IVPSDBTOOL}, $config, $installDir);

  $params->{IVPS_MIG_POSSIBLE} = 0;
  Logger::LogInfo($logFile, sprintf("NE families: from %s, to %s (%s)\n",
				    defined($toolData->{FROMNEFAMILY}) ? $toolData->{FROMNEFAMILY} : "undef",
				    defined($toolData->{TONEFAMILY}) ? $toolData->{TONEFAMILY} : "undef",
				    defined($toolData->{HOSTS}) ? "defined" : "undef")) if $params->{DEBUG};
  if (! $oflVersInfo->{RET}) {
    if ($oflVersInfo->{NT} ne "") {
      if ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {
	$params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	$params->{NTOFLMTBLDNUM} = $4;
      } elsif ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+).*/) {
	$params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
      } else {
	$params->{NTOFLMTVERS} = 0;
        if($oflVersInfo->{NT} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:.*/) {
	  $NTOFLMTVERSION = $1 * 100 + ($2 * 10) + $3; 
        } elsif ($oflVersInfo->{NT} =~ /.*V(\w)+\s+Build:\s+(\d+\.\d+)/) { 
	  $params->{NTOFLMTBLDNUM} = $2;
          if($params->{NTOFLMTBLDNUM} =~ /(\d{4})\.(\d+)/) {
	      $params->{NTOFLMTBLD} = $1 * 1000 + $2;
          } 
        } else {
	  Logger::LogInfo($logFile, "No NT OFLMT version could be obtained\n", 1) if $params->{DEBUG};
        }
      }
    } else {
      $params->{NTOFLMTVERS} = 0;
      Logger::LogInfo($logFile, "No NT OFLMT version could be obtained\n", 1) if $params->{DEBUG};
    }

    if (-f "$installDir/$toolConfigData->{IVPSDBTOOL}") {
      if ($oflVersInfo->{IVPS} ne "") {
	$params->{IVPS_MIG_POSSIBLE} = 1;
	if ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {
	  $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	  $params->{IVPSOFLMTBLDNUM} = $4;
	} elsif ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+).*/) {
	  $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	} else {
	  $params->{IVPSOFLMTVERS} = 0;
	  Logger::LogInfo($logFile, "   No xVPS OFLMT version could be obtained\n", 1) if $params->{DEBUG};
	}
	if (! -x $toolConfigData->{IVPSINVENTORY}) {
	  Logger::LogInfo($logFile, "   ISAM-V migration warning: no xVPS inventory script found or not executable\n", 1)
	    if $params->{DEBUG};
#	  $params->{IVPS_MIG_POSSIBLE} = 0;   Don't.  We'll see this later.  This script is only needed in case ISAM runs 3.3 or 3.4
#	  $params->{IVPSOFLMTVERS} = 0;
	}
      } else {
	Logger::LogInfo($logFile, "   !!! ISAM-V migration not possible: no xVPS OFLMT version could be obtained\n", 1) if $params->{DEBUG};
      }
    } else {
      $params->{IVPSOFLMTVERS} = 0;
      $params->{IVPS_MIG_POSSIBLE} = 0;
    }

    Logger::LogInfo($logFile, sprintf("NT Versions: %s, %s\n", Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
				      Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion(Capability::IsamHiCapEtsiFamily()))),
		    1) if ($params->{DEBUG});
    if (defined($params->{NTOFLMTVERS}) && ($params->{NTOFLMTVERS} > Capability::HighestNtOflmtVersion(Capability::IsamHiCapEtsiFamily()))) {
      Logger::LogInfo($logFile, 
		      sprintf("Version number of NT OFLMT higher then the one known to this toolset (%s vs %s)... exiting\n",
			      Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
			      Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion(Capability::IsamHiCapEtsiFamily()))),
		      1);
      $envOk = 0;
      exit NT_OFLMT_VERSION_2_HIGH_CODE;
    }

    Logger::LogInfo($logFile, sprintf("NT OFLMT Build : %s, %s\n", Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
				      Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild(Capability::IsamHiCapEtsiFamily()))),
		    1) if (defined($params->{NTOFLMTBLD}) && ($params->{DEBUG}));
    if ((Capability::ReleaseCompare(basename($params->{OSWP_FULL}),Capability::ConvertRelease(4, 5, 5),REGEXP, 0, 1) < 0)) {
     if (defined($params->{NTOFLMTBLD}) && ($params->{NTOFLMTBLD} > Capability::HighestNtOflmtBuild(Capability::IsamHiCapEtsiFamily()))) {
       Logger::LogInfo($logFile, 
		      sprintf("Version number of NT OFLMT higher then the one known to this toolset (%s vs %s)... exiting\n",
			      Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
			      Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild(Capability::IsamHiCapEtsiFamily()))),
		      1);
       $envOk = 0;
       exit NT_OFLMT_VERSION_2_HIGH_CODE;
     }
    }

    Logger::LogInfo($logFile, sprintf("xVPS Versions: %s, %s\n", Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
                                Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion($toolData->{TONEFAMILY}))), 1);

    if (defined($params->{IVPSOFLMTVERS}) && ($params->{IVPSOFLMTVERS} > Capability::HighestXvpsOflmtVersion($toolData->{TONEFAMILY}))) {
	Logger::LogInfo($logFile, 
			sprintf("Version number of xVPS OFLMT higher then the one known to this toolset (%s vs %s)... exiting\n",
				Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
				Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion($toolData->{TONEFAMILY}))),
			1);
	$envOk = 0;
	exit IVPS_OFLMT_VERSION_2_HIGH_CODE;
    }
  } else {
    ToolCommon::Log($logFile, sprintf("Determining version info failed: %s\n", Utils::GetError()), 1);
    $envOk = 0;
    exit $oflVersInfo->{RET};
  }
  my $retC = Utils::VerifyOctopusEnv($params->{DEBUG}, $toolData->{OCTOPUSPATH}, $config, $installDir);
  if ($retC) {
    ToolCommon::Log($logFile, Utils::GetError(), 1);
    $envOk = 0;
    exit $retC;
  }
  return ($oflVersInfo, $envOk);
}

=head2 PrintToolVersions($$$)

=begin html
Print the migration tool versions contained in $oflVersInfo

=end html

=cut

sub PrintToolVersions($$) {
  my ($logFile, $oflVersInfo) = @_;

  if (! $oflVersInfo->{RET}) {
    ToolCommon::Log($logFile, "**** Offline migrationtool version info ****\n", 1);
    if ($params->{NTOFLMTVERS} == 0) {
      if($NTOFLMTVERSION == 0) {
        ToolCommon::Log($logFile, "No version information could be obtained for the NT tool\n", 1);
      } else {
	ToolCommon::Log($logFile, sprintf("NT OFLMT version  : %s\n", Capability::NtOflmtVersionString($NTOFLMTVERSION)), 1);
      }
    } else {
      if (defined($params->{NTOFLMTBLDNUM})) {
	ToolCommon::Log($logFile, sprintf("NT OFLMT version  : %s Build: %s\n", Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
					  $params->{NTOFLMTBLDNUM}), 1);
      } else {
	ToolCommon::Log($logFile, sprintf("NT OFLMT version  : %s\n", Capability::NtOflmtVersionString($params->{NTOFLMTVERS})), 1);
      }
    }
    if ($params->{IVPSOFLMTVERS} == 0) {
      ToolCommon::Log($logFile, "No version information could be obtained for the xVPS tool\n", 1);
    } else {
      if (defined($params->{IVPSOFLMTBLDNUM})) {
        ToolCommon::Log($logFile, sprintf("xVPS OFLMT version  : %s Build: %s\n", Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
                                          $params->{IVPSOFLMTBLDNUM}), 1);
      } else {
        ToolCommon::Log($logFile, sprintf("xVPS OFLMT version: %s\n", Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS})), 1);
      }
    }
  }
}

=head2 DetermineSwVersion($$)

=begin html
Determines the SW version based on the NT type and DB version.

=end html

=cut

sub DetermineSwVersion($) {
  my ($fh) = @_;
  if ($params->{OFL_DB_VERS} =~ /B4.401/) { # Fix for Release R4402
    $params->{SRC_OSWP} = sprintf("L6GP%s44.201", Capability::IsamHiCapEtsiOswpVariant(4400));
  } elsif ($params->{OFL_DB_VERS} =~ /B4.301/) { # Fix for Release R4302
    $params->{SRC_OSWP} = sprintf("L6GP%s43.201", Capability::IsamHiCapEtsiOswpVariant(4300));
  } elsif ($params->{OFL_DB_VERS} =~ /C4.302/) { 
    $params->{SRC_OSWP} = sprintf("L6GP%s43.001", Capability::IsamHiCapEtsiOswpVariant(4320));
  } elsif ($params->{OFL_DB_VERS} =~ /B4.220/) { # Fix for Release R4220
    $params->{SRC_OSWP} = sprintf("L6GP%s42.001", Capability::IsamHiCapEtsiOswpVariant(4220));
  } elsif ($params->{OFL_DB_VERS} =~ /C4.230/) { # Fix for Release R4230
    $params->{SRC_OSWP} = sprintf("L6GP%s42.001", Capability::IsamHiCapEtsiOswpVariant(4230));
  } else {
    my $dbVersionRel = substr( $params->{OFL_DB_VERS}, 1);
    $dbVersionRel =~ s/\.//;
    my $dbVersionMajor = substr( $dbVersionRel, 0, 2);
    my $dbVersionMinor = substr( $dbVersionRel, 2, 2);
    my $buildNo = sprintf( "%03d", (($dbVersionMinor * 200) + 1) );
    $params->{SRC_OSWP} = sprintf("L6GP%s%d.%s", Capability::IsamHiCapEtsiOswpVariant($dbVersionMajor."00"), $dbVersionMajor, $buildNo );
  }

}

=head2 SetToolData()

=begin html
Initializes the common data pointer of the tool

=end html

=cut

sub SetToolData() {
  
  $toolData = FamilyCommon::GetToolData();

}

=head2 ModifyDbtoolCfg($$$$)

=begin html
Modify the NT OFLMT dbtool.cfg file.  Used in case --confopt82 or --arberg option is used.  Depending on the options enabled some lines will be
added to the dbtool.cfg file allowing migration to parameter values as used by the customer.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>inFile</td><td>full file specification of the source config file</td></tr>
  <tr><td>outFile</td><td>full file specification of the new config file</td></tr>
</table>

=end html

=cut

sub ModifyDbtoolCfg($$$) {
  my ($fh, $inFile, $outFile) = @_;
  my $ok = 1;
  my $line;
  my $option82Present = 0;
  my $arbergPresent = 0;
  my $accessOperatorIdPresent = 0;
  my $intlNwVlanPresent = 0; 
  my $ssmOutVlanPresent = 0; 
  my $cdeFilePathPresent = 0;
  my $igmpMaxUncfgPresent = 0;
  my $err;
  my ($input, $output);
  if (open($input, $inFile)) {
    if (open($output, ">" . $outFile)) {
      while ($line = <$input>) {
	if (($params->{CONFOPT82}) && ($line =~ /^SYS_PORT_NUM_MED/)) {
	  $option82Present = 1;
	}
	if (($params->{CONFARBERG}) && ($line =~ /^ARBERGTAG/)) {
	  $arbergPresent = 1;
	}
	if (($params->{ACCESSOPRID}) && ($line =~ /^ACCESS_OPERATOR_ID/)) {
	  $accessOperatorIdPresent = 1;
	}
	if (($params->{INTL_NW_VLAN}) && ($line =~ /^INTL_NETW_VLAN/)) {
          $intlNwVlanPresent = 1; 
	}
	if (($params->{SSM_OUT_VLAN}) && ($line =~ /^SSM_OUT_VLAN/)) {
          $ssmOutVlanPresent = 1; 
	}
        if (($params->{ISAM_HAS_VOICE}) && ($line =~ /^CDE_FILE_PATH/)) {
          $cdeFilePathPresent = 1;
        }
        if ($params->{IGMP_MAX_UNCFG} && ($line =~ /^IGMP_MAX_UNCFG/)) {
           $igmpMaxUncfgPresent = 1;
	}
	ToolCommon::Log($fh, "Read: '$line'") if ($params->{DEBUG});
	print $output $line;
      }
      if ((! $option82Present) && ($params->{CONFOPT82})) {
	$err = "";
	ToolCommon::Log($fh, "  Configure for DHCP option 82...\n") if ($params->{DEBUG});
	if (defined($config->{SYS_PORT_NUM_MED})) {
	  if (($config->{SYS_PORT_NUM_MED} >= 1) && ($config->{SYS_PORT_NUM_MED} <= 4)) {
	    print $output sprintf("SYS_PORT_NUM_MED    $config->{SYS_PORT_NUM_MED}\n");
	  } else {
	    $ok = 0;
	    $err .= "SYS_PORT_NUM_MED has wrong value ($config->{SYS_PORT_NUM_MED}): can only be 1..4";
	  }
	} else {
	  $ok = 0;
	  $err .= "SYS_PORT_NUM_MED not defined";
	}
	if (defined($config->{ATM_BASED_SYNTAX})) {
	  print $output sprintf("ATM_BASED_SYNTAX    $config->{ATM_BASED_SYNTAX}\n");
	} else {
	  $ok = 0;
	  $err .= "ATM_BASED_SYNTAX not defined";
	}
	if (defined($config->{ETH_BASED_SYNTAX})) {
	  print $output sprintf("ETH_BASED_SYNTAX    $config->{ETH_BASED_SYNTAX}\n");
	} else {
	  $ok = 0;
	  $err .= "ETH_BASED_SYNTAX not defined";
	}
	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }
      if ((! $arbergPresent) && ($params->{CONFARBERG})) {
	ToolCommon::Log($fh, "  Configure for Argberg insertion...\n") if ($params->{DEBUG});
	if (defined($config->{ARBERGTAG})) {
	  if (($config->{ARBERGTAG} >= 1) && ($config->{ARBERGTAG} <= 2)) {
	    print $output sprintf("ARBERGTAG    $config->{ARBERGTAG}\n");
	  } else {
	    $ok = 0;
	    $err .= "ARBERGTAG has wrong value ($config->{ARBERGTAG}): can only be 1..2";
	  }
	} else {
	  $ok = 0;
	  $err .= "ARBERGTAG not defined";
	}
	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }
      if ((! $accessOperatorIdPresent) && ($params->{ACCESSOPRID})) {
	ToolCommon::Log($fh, "  Configure for Access OperatorId insertion...\n") if ($params->{DEBUG});
	if (defined($config->{ACCESS_OPERATOR_ID})) {
	  if (($config->{ACCESS_OPERATOR_ID} >= 0) && ($config->{ACCESS_OPERATOR_ID} <= 65535)) {
	    print $output sprintf("ACCESS_OPERATOR_ID    $config->{ACCESS_OPERATOR_ID}\n");
	  } else {
	    $ok = 0;
	    $err .= "ACCESS_OPERATOR_ID has wrong value ($config->{ACCESS_OPERATOR_ID}): can only be 0..65535";
	  }
	} else {
	  $ok = 0;
	  $err .= "ACCESS_OPERATOR_ID not defined";
	}
	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }
      if ((! $intlNwVlanPresent) && defined($params->{INTL_NW_VLAN})) {
	ToolCommon::Log($fh, "  Configure for Internal Network VLAN...\n") if ($params->{DEBUG});
	  if (($params->{INTL_NW_VLAN} > 0) && ($params->{INTL_NW_VLAN} <= 4089)) {
	    print $output sprintf("INTL_NETW_VLAN $params->{INTL_NW_VLAN}\n");
          } else {
	    $ok = 0;
	    $err .= "INTERNAL_NETWORK_VLAN has wrong value ($params->{INTL_NW_VLAN}): can only be 1..4089";
          }

	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }

      if ((! $ssmOutVlanPresent) && defined($params->{SSM_OUT_VLAN})) {
	ToolCommon::Log($fh, "  Configure for SSM Out VLAN...\n") if ($params->{DEBUG});
	  if (($params->{SSM_OUT_VLAN} > 0) && ($params->{SSM_OUT_VLAN} < 4089)) {
	    print $output sprintf("SSM_OUT_VLAN $params->{SSM_OUT_VLAN}\n");
          } else {
	    $ok = 0;
	    $err .= "SSM_OUT_VLAN has wrong value ($params->{SSM_OUT_VLAN}): can only be 1..4088";
          }

	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }

      if(defined($params->{ENABLE_IGMP_GROUPING}) && defined($params->{MAX_GIP_GAP})) {
	  print $output sprintf("ENABLE_IGMP_GROUPING   $params->{ENABLE_IGMP_GROUPING}\n");
          print $output sprintf("MAX_GIP_GAP   $params->{MAX_GIP_GAP}\n");
      }
      if(defined($params->{GPON_ONU_RANGE_MODE})) {
	  print $output sprintf("GPON_ONU_RANGE_MODE $params->{GPON_ONU_RANGE_MODE}\n");
      }
      if((! $igmpMaxUncfgPresent) && (defined($params->{IGMP_MAX_UNCFG}))) {
          print $output sprintf("IGMP_DTAG_UNCFG 1\n");
      }

      if ((! $cdeFilePathPresent) && ($params->{ISAM_HAS_VOICE})) {
        if($params->{ISAM_VOICE_MIGR}) { 
          ToolCommon::Log($fh, "  Configure for CDE File Path ...\n") if ($params->{DEBUG});
          if(defined($params->{CDE_TMP_PATH}) && ($params->{CDE_TMP_PATH} ne "")) {
            print $output "$CDEPATHVAR    $params->{CDE_TMP_PATH}\n";
	    print $output "$NEWCDEPATHVAR    $params->{CDE_TMP_PATH}\n";
          } else {
            $ok = 0;
            $err .= "ERROR: CDE Temp Path is empty.";
          }

          if (! $ok) {
            FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
          }
        }
      }
      close($output), close($input);
    } else {
      $err = "Failed to open $outFile for writing: $!";
      FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
      $ok = 0;
    }
  } else {
    $err = "Failed to open $inFile for reading: $!";
    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
    $ok = 0;
  }
  return $ok;
}

=head2 LocateCDE()

=begin html
Locate the CDE tar file in $path conforming to $regExp and when found, untar the file.  Undef is returned in case no file was foun
d or when
untarring the file failed.
<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>Reference to an opened file for progress and debug message tracing</td></tr>
  <tr><td>path</td><td>path to the CDE tar file</td></tr>
  <tr><td>params</td><td>The command parameters provided to the script</td></tr>
  <tr><td>regExp</td><td>the regular expression to which the CDE file should be conforming to</td></tr>
  <tr><td>dir</td><td>Name of NE output directory (relative to the tftp root)</td></tr>
</table>

=end html

=cut

sub LocateCDE($$$$) {
  my ($fh, $path, $regExp, $dir) = @_;
  my @files = ();

  ToolCommon::Log($fh, "Looking for CDE file in '$path'\n") if $params->{DEBUG};
  opendir(DIR, "$path");
  @files = readdir(DIR);
  closedir(DIR);
  foreach my $file (@files) {
    $file = basename($file);
    ToolCommon::Log($fh, "Found file '$file' in '$path'\n") if $params->{DEBUG};
    return $file if ($file =~ /$regExp/);
  }
  return undef;
}

=head2 MigrateXvpsContainer($$$$$$$)

=begin html
Migrate the xVPS database contained in the ISAM-V backup.  
<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>Reference to an opened file for progress and debug message tracing</td></tr>
  <tr><td>params</td><td>The command parameters provided to the script</td></tr>
  <tr><td>dir</td><td>Name of NE output directory</td></tr>
  <tr><td>ivpsId</td><td>The IVPS container being migrated</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>path</td><td>PBMT installation directory name</td></tr>
</table>

=end html

=cut

sub MigrateXvpsContainer($$$$$$) {
  my ($fh, $dir, $ivpsId, $from, $to, $path) = @_;
  my $ok = 1;
  my $thisPath = getcwd();
  my $fromRel = Capability::XvpsDbVersion($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION} );
  my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION});
  my $toRel = Capability::XvpsDbVersion($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $dirName = sprintf("$IVPSCONTAINER", $ivpsId);
  my $cmd;
  my $cdeTarFileName;
  my $cfgFile = "$path/$toolData->{IVPSDBCFG}";
  my $errLog = "";
  my $errFh;
  my $logDir = FamilyCommon::GetNeStorage();

  ToolCommon::Log($fh, sprintf("Migrating xVPS%d database from %s (%s) to %s (%s)...", $ivpsId, $fromRelInfo, $fromRel, $toRelInfo, $toRel));
  my $cdeFileName = LocateCDE($fh, "$dir/$CDECONTNAME", $config->{CDE_REGEXP}, $dir);
  if (! defined($cdeFileName)) {
	my $tMsg = "No CDE file was found in the $dir/$CDECONTNAME directory";
	FamilyCommon::SetAndLogError($fh, $tMsg, " ERROR : $tMsg (check logfiles in working directory '$dir')\n");
	$ok = 0;
      } else {
	$cdeTarFileName = "$dir/$CDECONTNAME/$cdeFileName";
      }
  if ($ok) {
    if (! -e "$dir/$CDETMPDIR") {
      $cmd = "cd $dir;mkdir $CDETMPDIR;$config->{CP} $cdeTarFileName $dir/$CDETMPDIR >$logDir/MIGRcpCDE1.txt 2>&1";
    } else {
      $cmd = "cd $dir/$CDETMPDIR;$config->{RM} -rf * > /dev/null 2>&1;$config->{CP} $cdeTarFileName $dir/$CDETMPDIR >$logDir/MIGRcpCDE2.txt 2>&1";
    }
    ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
    my $ret = system($cmd);
    ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
    if($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $tMsg = "$cmd failed: $!";
      FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
      $ok = 0;
    } else {
      if ($ok) {
      # Cleanup of CDE source directory done here
        #if (-e "$dir/$CDECONTNAME") {
        #  $cmd = "cd $dir/$CDECONTNAME;$config->{RM} -rf * > $dir/MIGRxvpsCleanCDE.txt 2>&1";
        #  ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
        #  my $ret = system($cmd);
        #  ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
        #  if($ret) {
        #       my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
        #       my $tMsg = "$cmd failed: $!";
        #       FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg \n");
        #       $ok = 0;
        #  }
        #}

	if (Capability::ReleaseCompare($to, Capability::ConvertRelease(4, 3), REGEXP, 0) > 0) {
	  $cfgFile = sprintf("$dir/$IVPSCONTAINER/$toolData->{XVPSDBCFGMOD}", $ivpsId);
	  # Prepare dbtool.cfg for migration  TODO : reqd ??
	  if (!ModifyXvpsDbtoolCfg($fh, "$path/$toolData->{IVPSDBCFG}", $cfgFile, "$dir/$CDETMPDIR")) {
	    # SetAndLogError already done.
	    $ok = 0;
	  }
	}
	my $migTxtFile = sprintf($XVPSMIGFILE, $ivpsId);
	# Generate trace and logfiles in the XVPS containers for possible later retrieval.

          $cmd = "cd $dir/$dirName/$IVPSDBASE >$logDir/MIGRcdXVPS.txt 2>&1;$path/$toolData->{IVPSDBTOOL} -in $NTINFILENAME -out $NTOUTFILENAME " .
            "-cfg ../$toolData->{XVPSDBCFGMOD} -from $fromRel -to $toRel -trc >$logDir/$migTxtFile.txt 2>&1";
          $errLog = "$dir/$dirName/$IVPSDBASE/trc.log";

	ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
	my $ret = system($cmd);
	if($ret) {
	  my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	  my $tMsg = "";
	  if ($exitCode == 3) {
	    if (open($errFh, "$errLog")) {
	      while (my $line = <$errFh>) {
		chomp $line;
		if ($line =~ /NON_RECOV/) {
		  $line = <$errFh>, $line = <$errFh>, $line = <$errFh>, $line = <$errFh>, chomp $line;
		  $line =~ m/error info\s*:(.*)/;
		  $tMsg = "Unrecoverable error occured: $1";
		}
	      }
	      close($errFh);
	    } else {
	      $tMsg = "Opening $errLog failed: $!";
	    }
	  } elsif ($exitCode == 2) {
	    $tMsg = "Consistency checks failed";
	  } else {
	    $tMsg = "Migration failed";
	  }
	  FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir/$dirName/$IVPSDBASE')\n");
	  $ok = 0;
	} else {
	  ToolCommon::Log( $fh, "  OK\n");
	}
      }
    }
  }
  return $ok;
}


=head2 DetermineNtCommunities ($$$)

=begin html
Called when performing ISAM-V migration.  From R3.5 onwards different community strings are used to communicate with the xVPS boards in the system.
The community strings have an xVPS context.  The community strings are retrieved from the NT backup (stored in dmprot/DbCommunity).<br>
The result is hash indexed by an xVPS number and holding the community string as content.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
</table>

=end html

=cut

my $COMMUNITY_FILE = "dmprot/DbCommunity";

sub DetermineNtCommunities($$) {
  my ($fh, $dir) = @_;
  my $ok = 1;
  my $cmd = "cd $dir;$config->{TAR} xf $NTINFILENAME $COMMUNITY_FILE >detcomm.txt 2>&1";
  my $ret = system($cmd);
  my %commStrings;
  my $ctx;

  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "Failed to untar database '$NTINFILENAME' (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    my ($header, $fields, $recs, $recCnt, $actualRecords) = DM::ReadFile("$dir/$COMMUNITY_FILE"); 
    foreach my $rec (@{$recs}) {
      $ctx = uc($rec->{'ctxName'});
      ToolCommon::Log($fh, "Probing community for context '$ctx'\n") if $params->{DEBUG};
      $commStrings{$ctx} = "";
      foreach (split(//, $rec->{'name'})) {
	if (ord($_) != 0) {
	  $commStrings{$ctx} .= $_;
	} else {
	  last;
	}
      }
      ToolCommon::Log($fh, "Community string found for context $ctx found ($commStrings{$ctx})\n") if $params->{DEBUG};
    }
  }
  return $ok ? \%commStrings : undef;
}


=head2 MigrateXvpsDbase($$$$$$$$$$$$)

=begin html
Migration of xVPS database:<br>
-----------------------<br>
1. Loop on all xVPSs<br>
1.1 Create associated xVPS container<br>
1.2 upload the complete xVPS database using SNMP in case xPVS has valid IP address<br>
---------------------------<br>
1. Untar NT DB to extract the XPVS containers<br>
<br>
2. Loop on all xVPSs with valid IP address<br>
2.1 migrate the database<br>
The following parameters are used:
<table>
  <tr><td>parentFh</td><td>the file handle of the parent script.  When fatal errors are occuring logging to the main log file will also be performed</td></tr>
  <tr><td>childFh</td><td>the file handle of the parent script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>xvpsConfig</td><td>A hash containing a list of xVPS boards in the system.</td></tr>
  <tr><td>dir</td><td>NE output directory relative to $TFTPDIR</td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
  <tr><td>userName</td><td>Unix user name in case of multi-user operation (command line option) empty otherwise</td></tr>
  <tr><td>logFile</td><td>the name of the log file associated to this child process</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>path</td><td>PBMT installation directory name</td></tr>
</table>

=end html

=cut

sub MigrateXvpsDbase($$$$$$$$$$$) {
  my ($parentFh, $childFh, $isam, $from, $to, $xvpsConfig, $dir, $count, $userName, $logFile, $path) = @_;
  my $ok = 1;
  my $version = undef;
  my $line;
  my $fromRel = Capability::XvpsDbVersion($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION} );
  my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION});
  my $toRel = Capability::XvpsDbVersion($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $migResult = "";
  my $xvpsDbPath = "";
  my $logDir = FamilyCommon::GetNeStorage();

  $dir =~ s/\/$//;
  ToolCommon::Log($childFh, "Starting XVPS migration...\n") if $params->{DEBUG};
  if (defined($toRel) && defined($fromRel)) {
    if ($from ne $to) {
#      ToolCommon::Log($childFh, "Migrating xVPS database from $fromRelInfo ($fromRel) to $toRelInfo ($toRel)...");
      if (Capability::ReleaseCompare($from, Capability::ConvertRelease(4, 3), REGEXP, 0) >= 0) {
	$xvpsDbPath = $toolData->{FXRDATA}{HOMEDIR};  # Can be ftp or sftp DB upload
	# Extract xVPS backup from NT backup and create tar file from it for migration purpose (for migration of R4.3 to R4.4 or higher).
	if (! PrepareXvpsContainers($childFh, defined($params->{OFLDB}) ? FamilyCommon::GetNeStorage() : $toolData->{FXRDATA}{HOMEDIR},
				    $xvpsConfig, $dir)) {
	  $ok = 0;
	}
      } else {
	FamilyCommon::SetAndLogError($childFh, "Unsupported migration");
	$ok = 0;
      }
      ToolCommon::Log($childFh, "xvps DB path: $xvpsDbPath\n") if $params->{DEBUG};
      if ($ok) {
	foreach my $ivpsId (sort keys %{$xvpsConfig}) {
	  if ($xvpsConfig->{$ivpsId} ne $toolData->{NO_IP}) {
	    if (! MigrateXvpsContainer($childFh, defined($params->{OFLDB}) ? FamilyCommon::GetNeStorage() : $xvpsDbPath, $ivpsId,
				       $from, $to, $path)) {
	      $migResult .= "Migration of xVPS$ivpsId failed\n";
	      $ok = 0;
	    }
	  }
	}
	FamilyCommon::SetAndLogError($childFh, $migResult, $migResult) if ! $ok;
	# Remove the untarred CDE package as we no longer need it.
        if (-e "$toolData->{TFTPDIR}/$dir/$CDETMPDIR") {
	    my $cmd = "$config->{RM} -Rf $toolData->{TFTPDIR}/$dir/$CDETMPDIR > /dev/null 2>&1";
	    my $ret = system($cmd);
	    if ($ret) {
	      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	      my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
	      FamilyCommon::SetAndLogError($childFh, $t, "  $t\n");
	      $ok = 0;
	    }
        }

      # Cleanup of CDE source directory done here
        if (-e "$toolData->{TFTPDIR}/$dir/$CDECONTNAME") {
          my $cmd = "$config->{RM} -rf $toolData->{TFTPDIR}/$dir/$CDECONTNAME/* > $toolData->{TFTPDIR}/$dir/MIGRxvpsCleanCDE.txt 2>&1";
          ToolCommon::Log($childFh, "CMD = $cmd\n") if ($params->{DEBUG});
          my $ret = system($cmd);
          ToolCommon::Log($childFh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
          if($ret) {
               my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
               my $tMsg = "$cmd failed: $!";
               FamilyCommon::SetAndLogError($childFh, $tMsg, "  : $tMsg \n");
               $ok = 0;
          }
        }

      }
    } else {
      ToolCommon::Log($childFh, "  xVPS Migration not needed, same release\n");
    }
  } else {
    FamilyCommon::SetAndLogError($childFh, "Migrating xVPS database from $from to $to not possible: DB version not defined");
    $ok = 0;
  }
  return ($ok);
}

=head2 DetermineXvpsPresence($$$)

=begin html
Called when performing migration using a DB stored offline (-ofldb option).  This procedure checks whether an xVPS board is present.<br>
Returns undef in case of failure, 0 in case no XVPS board was found, 1 in case the ISAM is an ISAM-V.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
</table>

=end html

=cut

sub DetermineXvpsPresence($$) {
  my ($fh, $dir) = @_;
  my $ok = 0;
  my $cmd = "cd $dir;$config->{TAR} xf $params->{OFLDB} >detxvpstarxf.txt 2>&1";
  my $ret = system($cmd);
  my $version = "";
  my $line = "";

  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "Failed to untar database '$params->{OFLDB}' (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = undef;
  } else {
    my ($header, $fields, $recs, $recCnt, $actualRecords) = DM::ReadFile("$dir/dm/EqBoard"); 
    foreach my $rec (@{$recs}) {
      ToolCommon::Log($fh, sprintf("Probing XVPS-type for boardCode 0x%04x\n", $rec->{'Type'})) if $params->{DEBUG};
      if (defined(my $xvpsType = Capability::DetermineXvpsType($rec->{'Type'}))) {
	ToolCommon::Log($fh, " $xvpsType found\n") if $params->{DEBUG};
	$ok = 1;
	last;
      }
    }
  }
  return $ok;
}


=head2 PrepareXvpsContainers($$$$$)

=begin html
Extract the xVPS backups from the NT backup and create individual dm_complete.tar files from it.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>Opened file used for progress and debug message tracing</td></tr>
  <tr><td>params</td><td>The command parameters provided to the script</td></tr>
  <tr><td>longPath</td><td>The unix (complete) path to extract the NT backup and create the individual xVPS backup files</td></tr>
  <tr><td>xvpsConfig</td><td>A hash containing a list of xVPS boards in the system.</td></tr>
</table>
After extraction the following directory structure  will be present (assuming only XVPS1 and 8 are present):<br><tt>
  --+--XVPS1--+--Dbase/config<br>
    |         |<br>
    |         +--Dbase/dm/*<br>
    |         |<br>
    |         +--Dbase/dmprot/*<br>
    |         |<br>
    |         +--CDE/active/CDEtarfile<br>
    :<br>
    :<br>
    |<br>
    +--XVPS8--+--Dbase/config<br>
              |<br>
              +--Dbase/dm/*<br>
              |<br>
              +--Dbase/dmprot/*<br>
              |<br>
              +--CDE/active/CDEtarfile<br>
</tt>

=end html

=cut

my $EXCLUDE = "excl.txt";   # Exclude file for tar.  Only untar the XVPS containers

sub PrepareXvpsContainers($$$$) {
  my ($fh, $longPath, $xvpsConfig, $tmpDir) = @_;
  my $ok = 1;
  my $cmd;
  my $ret;
  my $xvpsNum;
  my $configMM = "";
  my $excl;
  my $logDir = FamilyCommon::GetNeStorage();

  ToolCommon::Log($fh, "Preparing XVPS containers...\n") if $params->{DEBUG};
  for ($xvpsNum = $LOWEST_XVPSNUM; $xvpsNum <= $HIGHEST_XVPSNUM; $xvpsNum++) {
    # Remove the XVPS directories if existing
    if (-e "$longPath/XVPS$xvpsNum") {
      # DB container found, probably from previous migration.  Remove it (recursively) before going on
      $cmd = "$config->{RM} -Rf $longPath/XVPS$xvpsNum >$logDir/MIGRpreCnt.txt 2>&1";
      ToolCommon::Log($fh, "cmd = $cmd\n") if $params->{DEBUG};
      $ret = system($cmd);
      if ($ret) {
	$ok = 0;
	my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      }
    }
  }
  if ($ok) {
    if (open($excl, ">$longPath/$EXCLUDE")) {
      # Use tar with an exclude file to exclude dm dmprot Lanx ODM and config from the untar operation
      if ($Config{archname} =~ /linux/) {
        print $excl "dm/\n" . "dmprot/\n" . "Lanx/\n" . "ODM/\n";
      } else {
       print $excl "config\n" . "dm/\n" . "dmprot/\n" . "Lanx/\n" . "ODM/\n";
      }
      close $excl;
      $cmd = "cd $longPath;$config->{TAR} xfX $NTINFILENAME $EXCLUDE >$logDir/MIGRcrExcl.txt 2>&1";
      ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
      $ret = system($cmd);
      if ($ret) {
	my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      } else {
	for ($xvpsNum = $LOWEST_XVPSNUM; $xvpsNum <= $HIGHEST_XVPSNUM; $xvpsNum++) {
	  if ((-e "$longPath/XVPS$xvpsNum/$IVPSDBASE") && ($xvpsConfig->{$xvpsNum} eq $toolData->{NO_IP})) {
	    # DB container found in DB but not present in XVPS config (no XVPS community string found
	    $configMM .= "DB container found for XVPS$xvpsNum while no community string has been defined\n";
	    $ok = 0;
	  }
	}
	if ($ok) {
	  # Create a dm_complete.tar under XVPSn/Dbase.  This file will be used to do the actual XVPS migration.
	  for ($xvpsNum = $LOWEST_XVPSNUM; $xvpsNum <= $HIGHEST_XVPSNUM; $xvpsNum++) {
	    if (-e "$longPath/XVPS$xvpsNum") {
              if ($Config{archname} =~ /linux/) {
	         $cmd = "cd $longPath/XVPS$xvpsNum/$IVPSDBASE;$config->{TAR} --format=gnu -cf $NTINFILENAME config dm dmprot >$logDir/MIGRtarcf.txt 2>&1";
              } else {
	         $cmd = "cd $longPath/XVPS$xvpsNum/$IVPSDBASE;$config->{TAR} cf $NTINFILENAME config dm dmprot >$logDir/MIGRtarcf.txt 2>&1";
              }
	      ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
	      $ret = system($cmd);
	      if ($ret) {
		my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
		my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
		FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
		$ok = 0;
	      }
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, $configMM, $configMM);
	}
      }
    } else {
      my $t = "Opening exclude file '$longPath/$EXCLUDE' failed: $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    }
  }
  return $ok;
}


=head2 GetIvpsIsamListByIP($$$)

=begin html
Retrieve list of dependant nodes for a specific IVPS present in this ISAM using SNMP.  The list can empty meaning that no remote POLT boards are
controled by this IVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference
to a hash is returned.  Every hash contains of an node name/IP address couple.  In case the hash element is empty no translation was possible.
<br>
Note that this functio takes into account whether we are performing migration using the offline db option or whether we are working with a real
ISAM-V<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>ivpsIp</td><td>IP address of IVPS</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

my $ACCESSEQUIPMENTASAMID = ".1.3.6.1.4.1.637.61.1.55.2.1.1.3";

sub GetXvpsIsamListByIP($$) {
  my ($fh, $ivpsIp) = @_;
  my $result;
  my %couple = ();
  my ($name, $ipAddr, $transOk);

  if (!defined($params->{OFLDB})) {
    my ($session, $snmpError) = Net::SNMP->session(-hostname => $ivpsIp,
						   -version => "SNMPv1",
						   -timeout => 5,
						   -retries => 3,
						   -community => "public");
    if (!defined($session)) {
      FamilyCommon::SetAndLogError($fh, "  ERROR: Could not open session for $ivpsIp: $snmpError");
      return undef;
    } else {
      #-- disable OID translation
      $session->translate(0);
      $result = $session->get_table($ACCESSEQUIPMENTASAMID);
      if (!defined($result)) {
	my $error = $session->error();
	if (!($error =~ /Requested table is empty/)) {
	  FamilyCommon::SetAndLogError($fh, "SNMP table walk '$ACCESSEQUIPMENTASAMID' failed: $error");
	  return undef;
	}
      } else {
	foreach my $entry (Net::SNMP::oid_lex_sort(keys(%{$result}))) {
	  $name = "", $ipAddr = "", $transOk = 1;
	  if ($result->{$entry} =~ /\d+\.\d+\.\d+\.\d+/) {
	    $name = gethostbyaddr(inet_aton($result->{$entry}), AF_INET);
	    if (!defined($name)) {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) no match\n") if $params->{DEBUG};
	    } else {
	      $couple{$name} = $result->{$entry};
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) yields '$name'\n") if $params->{DEBUG};
	    }
	  } else {
	    my @locAddr = gethostbyname($result->{$entry});
	    my $status = $?;
	    if ($status == 0) {
	      my @ip = unpack( "C4", $locAddr[4]);
	      my $ipAddr = join( '.', @ip );
	      $couple{$result->{$entry}} = $ipAddr;
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) yields '$ipAddr'\n") if $params->{DEBUG};
	    } else {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) no match\n") if $params->{DEBUG};
	    }
	  }
	}
	$session->close();
      }
    }
  }
  return \%couple;
}


=head2 GetIvpsIsamListByCommunity($$$$)

=begin html
Retrieve list of dependant nodes for a specific IVPS present in this ISAM using SNMP.  The list can empty meaning that no remote POLT boards are
controled by this IVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference
to a hash is returned.  Every hash contains of an node name/IP address couple.  In case the hash element is empty no translation was possible.
<br>Used from R3.5 onwards: the xVPS no longer has a public IP address but is addressed using the NT IP address in combination with a community
string.<br>
Note that this functio takes into account whether we are performing migration using the offline db option or whether we are working with a real
ISAM-V<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>ivpsIp</td><td>IP address of NT</td></tr>
  <tr><td>community</td><td>The community string associated to the NT</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetXvpsIsamListByCommunity($$$) {
  my ($fh, $ntIp, $community) = @_;
  my $result;
  my %couple = ();
  my ($name, $ipAddr, $transOk);
  my ($xvpsSession, $snmpError);

  if (! defined($params->{OFLDB})) {
    if (defined($params->{SNMP}{V3}{AUTH}{USERNAME}) and ($params->{SNMP}{V3}{AUTH}{USERNAME} ne "")) {
        if ((defined($params->{SNMP}{V3}{AUTH}{PASSWORD}) && ($params->{SNMP}{V3}{AUTH}{PASSWORD} ne "")) &&
            (defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""))) {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME},
                                                      -authprotocol => $params->{SNMP}{V3}{AUTH}{PROTO},
                                                      -authpassword => $params->{SNMP}{V3}{AUTH}{PASSWORD},
                                                      -privpassword => $params->{SNMP}{V3}{PRIV}{PASSWORD},
                                                      -privprotocol => $params->{SNMP}{V3}{PRIV}{PROTO});
        } elsif ((defined($params->{SNMP}{V3}{AUTH}{PASSWORD}) && ($params->{SNMP}{V3}{AUTH}{PASSWORD} ne "")) &&
                 (defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} eq ""))) {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME},
                                                      -authpassword => $params->{SNMP}{V3}{AUTH}{PASSWORD},
                                                      -authprotocol => $params->{SNMP}{V3}{AUTH}{PROTO});
        } elsif ((defined($params->{SNMP}{V3}{AUTH}{PASSWORD}) && ($params->{SNMP}{V3}{AUTH}{PASSWORD} eq "")) &&
                 (defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""))) {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME},
                                                      -privpassword => $params->{SNMP}{V3}{PRIV}{PASSWORD},
                                                      -privprotocol => $params->{SNMP}{V3}{PRIV}{PROTO});
        } else {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME});
        }
    } else {
      if (defined($params->{SNMP}{V2}{NT_COMMUNITY})) {
	($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
							-version => 2,
							-timeout => 5,
							-retries => 3,
							-community => $community);
      } else {
	($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
							-version => 1,
							-timeout => 5,
							-retries => 3,
							-community => $community);
      }
    }
    if (!defined($xvpsSession)) {
      FamilyCommon::SetAndLogError($fh, "  ERROR: Could not open session for $ntIp: $snmpError");
      return undef;
    } else {
      #-- disable OID translation
      ToolCommon::Log($fh, "Opened SNMP session for IP $ntIp and community '$community'\n") if $params->{DEBUG};
      $xvpsSession->translate(0);
      if (defined($params->{SNMP}{V3}{AUTH}{USERNAME}) and ($params->{SNMP}{V3}{AUTH}{USERNAME} ne "")) {
	$result = $xvpsSession->get_table(-baseoid => $ACCESSEQUIPMENTASAMID,
					  -contextname => $community,
					  -maxrepetitions => MAX_SNMPV2_3_REP);
      } else {
	if (defined($params->{SNMP}{V2}{NT_COMMUNITY})) {
	  $result = $xvpsSession->get_table(-baseoid => $ACCESSEQUIPMENTASAMID,
					    -maxrepetitions => MAX_SNMPV2_3_REP);
	} else {
	  $result = $xvpsSession->get_table($ACCESSEQUIPMENTASAMID);
	}
      }
      if (!defined($result)) {
	my $error = $xvpsSession->error();
	if (!($error =~ /Requested table is empty/)) {
	  FamilyCommon::SetAndLogError($fh, "SNMP table walk '$ACCESSEQUIPMENTASAMID' failed: $error");
	  $xvpsSession->close();
	  return undef;
	}
      } else {
	foreach my $entry (Net::SNMP::oid_lex_sort(keys(%{$result}))) {
	  $name = "", $ipAddr = "", $transOk = 1;
	  if ($result->{$entry} =~ /\d+\.\d+\.\d+\.\d+/) {
	    $name = gethostbyaddr(inet_aton($result->{$entry}), AF_INET);
	    if (!defined($name)) {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) no match\n") if $params->{DEBUG};
	    } else {
	      $couple{$name} = $result->{$entry};
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) yields '$name'\n") if $params->{DEBUG};
	    }
	  } else {
	    my @locAddr = gethostbyname($result->{$entry});
	    my $status = $?;
	    if ($status == 0) {
	      my @ip = unpack( "C4", $locAddr[4]);
	      my $ipAddr = join( '.', @ip );
	      $couple{$result->{$entry}} = $ipAddr;
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) yields '$ipAddr'\n") if $params->{DEBUG};
	    } else {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) no match\n") if $params->{DEBUG};
	    }
	  }
	}
	$xvpsSession->close();
      }
    }
  }
  return \%couple;
}


=head2 GetXvpsDependancyByIP($$$)

=begin html
Retrieve list of dependant nodes for every XVPS present in this ISAM.  The list can empty meaning that no remote POLT boards are controled by this
XVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference to a hash is returned.
Every entry consists of a hash consisting of node name pointing to a node IP address (or an empty in case the translation failed.
The key is the xVPS number.
<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>isam</td><td>pointer to the XVPS structure</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetXvpsDependancyByIP($$) {
  my ($fh, $xvpsConfig) = @_;
  my %dependancy;

  ToolCommon::Log($fh, "Determine xVPS dependancy list by IP...");
  foreach my $ivpsId (sort keys %{$xvpsConfig}) {
    if ($xvpsConfig->{$ivpsId} ne $toolData->{NO_IP}) {
    ToolCommon::Log($fh, "  $ivpsId: '$xvpsConfig->{$ivpsId}'\n") if $params->{DEBUG};
      my $isamList = GetXvpsIsamListByIP($fh, $xvpsConfig->{$ivpsId});
      if (defined($isamList)) {
        $dependancy{$ivpsId} = $isamList;
      } else {
        return undef;
      }
    }
  }
  ToolCommon::Log($fh, "  OK!\n");
  return \%dependancy;
}

=head2 GetXvpsDependancyByCommunity($$$$)

=begin html
Retrieve list of dependant nodes for every XVPS present in this ISAM.  The list can empty meaning that no remote POLT boards are controled by this
XVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference to a hash is returned.
Every entry consists of a hash consisting of node name pointing to a node IP address (or an empty in case the translation failed.
The key is the xVPS number.
<br>Used from R3.5 onwards: the xVPS no longer has a public IP address but is addressed using the NT IP address in combination with a community
string.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>isam</td><td>pointer to the XVPS structure containing XVPS community strings</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetXvpsDependancyByCommunity($$$) {
  my ($fh, $communityStrings, $ntIp) = @_;
  my %dependancy;
  my $xvpsNum;

  ToolCommon::Log($fh, "Determine xVPS dependancy list by community...");
  foreach my $context (sort keys %{$communityStrings}) {
    if ($communityStrings->{$context} =~ /xvps(\d)/) {
      $xvpsNum = $1;
      ToolCommon::Log($fh, "  $context ($xvpsNum): '$communityStrings->{$context}'\n") if $params->{DEBUG};
      my $isamList = GetXvpsIsamListByCommunity($fh, $ntIp, $communityStrings->{$context});
      if (defined($isamList)) {
        $dependancy{$xvpsNum} = $isamList;
      } else {
        return undef;
      }
    }
  }
  ToolCommon::Log($fh, "  OK!\n");
  return \%dependancy;
}

=head2 MakeFakeList($$$)

=begin html
In case the AWS functionality is missing to generate an IVPS to IP address mapping a test-help bas been built-in where a pre-defined file can be used.
The name is fixed to IvPsLiSt.txt and has the same format as generated by the AWS:<br>
XVPS<n>:<IP> or 0.0.0.0 when no IVPS is present.  <n>: 1..8<br>
<br>
IMPORTANT NOTE TO PEOPLE READING THIS CODE!!!!!<br>
The fake IvPsLiSt.txt should NEVER be used in fieldconditions.  The AWS is expected to provide the functionality offered by this fake file.<br>
This was added to test the ISAM-V migration when AWS was not yet ready and the required functionality was as such not available.

=end html

=cut

sub MakeFakeList($$) {
  my ($fh, $fileName) = @_;
  my $line;
  my %ivpsConfig = ();
  my $foundIP = 0;
  my $look;

  ToolCommon::Log($fh, "Reading fake IVPS list..");
  if (open($look, $fileName)) {
    while (defined($line = <$look>)) {
      chomp $line;
      ToolCommon::Log($fh, "$line\n") if ($params->{DEBUG});
      if ($line =~ /$ISNOTISAMV_RE/) {
        FamilyCommon::SetAndLogError($fh, "ISAM not known as ISAMV in AWS");
        return undef;
      } elsif ($line =~ /$ISISAMV_RE/) {
        $foundIP = 1 if ($2 ne $toolData->{NO_IP});
        $ivpsConfig{$1} = $2;
        ToolCommon::Log($fh, "Stored IP address $ivpsConfig{$1} for xVPS $1\n") if ($params->{DEBUG});
      } else {
        FamilyCommon::SetAndLogError($fh, "Unexpected line read: '$line'");
        return undef;
      }
    }
    close $look;
    ToolCommon::Log($fh, "  OK\n");
  } else {
    FamilyCommon::SetAndLogError($fh, "Opening '$fileName' failed: $!");
    return undef;
  }
  if (! $foundIP) {
    FamilyCommon::SetAndLogError($fh, "No xVPS IP addresses found in IvPsLiSt.txt");
  }
  return $foundIP ? \%ivpsConfig : undef;
}

=head2 GetIvpsIpAddresses($$$)

=begin html
Retrieve IP addresses associated to an ISAM in case it is an ISAM-V.   The undefined variable is returned in case an error was encountered (not an ISAM-V is considered as an error (this function is not called in case no xVPS boards have been found in the LT configuration) otherise a pointer to a hash is returned in case an IVPS structure is returned by AWS.  The key is the xVPS number, the element its IP address.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>isam</td><td>pointer to the ISAM IP/name structure</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetIvpsIpAddresses($$) {
  my ($fh, $isam) = @_;
  my $cmd = "$toolData->{IVPSINVENTORY} $isam->[0] |";
  my $line;
  my %ivpsConfig = ();
  my $foundIP = 0;
  my $look;

  ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
  if (open($look, $cmd)) {
    while (defined($line = <$look>)) {
      chomp $line;
      ToolCommon::Log($fh, "$line\n") if ($params->{DEBUG});
      if ($line =~ /$ISNOTISAMV_RE/) {
        FamilyCommon::SetAndLogError($fh, "ISAM not known as ISAMV in AWS");
        return undef;
      } elsif ($line =~ /$ISISAMV_RE/) {
	$foundIP = 1 if ($2 ne $toolData->{NO_IP});
        $ivpsConfig{$1} = $2;
        ToolCommon::Log($fh, "Stored IP address $ivpsConfig{$1} for xVPS $1\n") if ($params->{DEBUG});
      } else {
        FamilyCommon::SetAndLogError($fh, "Unexpected line read: '$line'");
        return undef;
      }
    }
    close $look;
  } else {
    my ($exitCode, $signal) = Utils::ConvertExitCode($?);
    FamilyCommon::SetAndLogError($fh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!");
    return undef;
  }
  if(! $foundIP) {
    FamilyCommon::SetAndLogError($fh, "No xVPS IP addresses found in xVPS inventory output");
  }
  return $foundIP ? \%ivpsConfig : undef;
}

=head2 CopyXvpsList($$$)

=begin html
Create a list of xVPS with a non-0.0.0.0 IP address for the xVPS community strings found in the community string input hash.  Used from R3.5 onwards.

=end html

=cut

sub CopyXvpsList($$) {
  my ($fh, $communityStrings) = @_;
  my $xvpsNum;
  my %xvpsConfig = (1 => $toolData->{NO_IP},
		    2 => $toolData->{NO_IP},
		    3 => $toolData->{NO_IP},
		    4 => $toolData->{NO_IP},
		    5 => $toolData->{NO_IP},
		    6 => $toolData->{NO_IP},
		    7 => $toolData->{NO_IP},
		    8 => $toolData->{NO_IP});

  foreach my $context (sort keys %{$communityStrings}) {
#    if ($communityStrings->{$context} =~ /xvps(\d)/) {
    if (uc($context) =~ /XVPS(\d)/) {
      $xvpsNum = $1;
      $xvpsConfig{$xvpsNum} = "1.1.1.1";  # Make it different from $NO_IP
      ToolCommon::Log($fh, "Defining xVPS $xvpsNum in copy list\n") if $params->{DEBUG};
    }
  }
  return \%xvpsConfig;
}


=head2 ModifyXvpsDbtoolCfg($$$$$)

=begin html
Modify the XVPS OFLMT dbtool.cfg file. Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>inFile</td><td>full file specification of the source config file</td></tr>
  <tr><td>outFile</td><td>full file specification of the new config file</td></tr>
  <tr><td>cdePath</td><td>The complete path where the CDE pacakge was untarred.</td></tr>
</table>

=end html

=cut

sub ModifyXvpsDbtoolCfg($$$$) {
  my ($fh, $inFile, $outFile, $cdePath) = @_;
  my $ok = 1;
  my $line;
  my $err;
  my ($input, $output);

  if (open($input, $inFile)) {
    if (open($output, ">" . $outFile)) {
      while ($line = <$input>) {
	ToolCommon::Log($fh, "Read: '$line'") if ($params->{DEBUG});
	print $output $line if (! ($line =~ /$CDEPATHVAR/));
      }
      if (defined($cdePath)) {
        $cdePath =~ s/\/\//\//g;
        print $output "$CDEPATHVAR    $cdePath\n";
      }
      close($output), close($input);
    } else {
      $err = "Failed to open $outFile for writing: $!";
      FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
      $ok = 0;
    }
  } else {
    $err = "Failed to open $inFile for reading: $!";
    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
    $ok = 0;
  }
  return $ok;
}

sub UpdateIPDMaxRange($$)
{
  my ($fh, $dir) = @_;
  my $ok = 1;
  my $NEWIHUBCFG = "ipdconfig_maxrange.cfg"; 
  my $ihubCfgFile = "$dir/newdb/$IHUBDIR/$IHUBCONFIG"; 
  my $newihubCfgFile = "$dir/$NEWIHUBCFG"; 
  #my @lines; 
  #my @newlines; 
  my $tempVar; 
  my $var;
  my $line; 
  my ($infile, $outfile);
  my $cmd; 
  my $err; 
  my $logDir = FamilyCommon::GetNeStorage();

   if (-e "$ihubCfgFile") {
     if (open($infile, $ihubCfgFile)) {
       if (open($outfile, ">" . $newihubCfgFile)) {
          while ($line = <$infile>) {
            $tempVar = $line;
            if($tempVar =~ /mc-maximum-routes[ ]+(\d+)/){
               $var = $1;
               $tempVar =~ s/mc-maximum-routes $var/mc-maximum-routes $R51_MC_MAX_ROUTES/ if($var > $R51_MC_MAX_ROUTES);
             } elsif($tempVar =~ /maximum-ipv6-routes[ ]+(\d+)/){
               $var = $1;
               $tempVar =~ s/maximum-ipv6-routes $var/maximum-ipv6-routes $R51_MAX_IPV6_ROUTES/ if($var > $R51_MAX_IPV6_ROUTES);
             } elsif($tempVar =~ / maximum-routes[ ]+(\d+)/){
               $var = $1;
               $tempVar =~ s/maximum-routes $var/maximum-routes $R51_MAX_ROUTES/ if($var > $R51_MAX_ROUTES);
             }
	    print $outfile $tempVar; 
          }
          close($infile); 
          close($outfile);
 
          $cmd = "cd $dir;$config->{CP} $newihubCfgFile $ihubCfgFile >$logDir/MIGRcpIPDcfg.txt 2>&1";
          my $ret = system($cmd); 
          if($ret) {
             my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
             my $tMsg = "$cmd failed: $!";
             FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
             $ok = 0;
          }
       } else {
         $err = "Failed to open $newihubCfgFile for writing: $!";
         FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
         $ok = 0;
       }
     } else {
       $err = "Failed to open $ihubCfgFile for reading: $!";  
       FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
       $ok = 0; 
     }
   }
  return $ok;
}

sub UpdateFastReroute($$)
{
  my ($fh, $dir) = @_;
  my $ok = 1;
  my $NEWIHUBCFG = "ipdconfig_fastreroute.cfg"; 
  my $ihubCfgFile = "$dir/newdb/$IHUBDIR/$IHUBCONFIG"; 
  my $newihubCfgFile = "$dir/$NEWIHUBCFG"; 
  my $line; 
  my ($infile, $outfile);
  my $cmd; 
  my $err; 
  my $logDir = FamilyCommon::GetNeStorage();

   if (-e "$ihubCfgFile") {
        my $Match = `grep  ' fast-reroute facility' $ihubCfgFile`;
        if( $Match eq "" ) {
              ToolCommon::Log($fh, "Updation not required, Since header is not found\n");
              return 0;
        }
        if (open($infile, $ihubCfgFile)) {
          if (open($outfile, ">" . $newihubCfgFile)) {
             while ($line = <$infile>) {
               if($line =~ /^\s+fast-reroute facility/){
                  $line =~ s/ facility/ one-to-one/;
               }
	           print $outfile $line; 
             }
             close($infile); 
             close($outfile);
 
             $cmd = "cd $dir;$config->{CP} $newihubCfgFile $ihubCfgFile >$logDir/MIGRcpIPDcfg.txt 2>&1";
             my $ret = system($cmd); 
             if($ret) {
                my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
                my $tMsg = "$cmd failed: $!";
                FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
                $ok = 0;
             }
          } else {
            $err = "Failed to open $newihubCfgFile for writing: $!";
            FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
            $ok = 0;
          }
        } else {
          $err = "Failed to open $ihubCfgFile for reading: $!";  
          FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
          $ok = 0; 
        }
   }
  return $ok;
}

sub UpdateIPDInterface($$)
{
  my ($fh, $dir) = @_;
  my $ok = 1;
  my $NEWIHUBCFG = "ipdconfig_interface.cfg"; 
  my $ihubCfgFile = "$dir/newdb/$IHUBDIR/$IHUBCONFIG"; 
  my $newihubCfgFile = "$dir/$NEWIHUBCFG"; 
  #my @lines; 
  #my @newlines; 
  my $tempVar; 
  my $var;
  my $line; 
  my ($infile, $outfile);
  my $cmd; 
  my $err; 
  my $logDir = FamilyCommon::GetNeStorage();


   if ( -e "$ihubCfgFile") {
        my $headerRouter ='^echo "Router (Network Side) Configuration"';
        my $headerIGMP = '^echo "IGMP Configuration"';
        my $routerMatch = `grep  '$headerRouter' $ihubCfgFile`;
        my $igmpMatch = `grep  '$headerIGMP' $ihubCfgFile`;
        if( $igmpMatch eq "" or $routerMatch eq "" ) {
              ToolCommon::Log($fh, "Updation not required, Since header is not found\n");
              return 0;
        }
     if (open($infile, $ihubCfgFile)) {
       if (open($outfile, ">" . $newihubCfgFile)) {
	     my ($isRouterBlock, $isIGMPBlock, $interfaceNeedToRemove ) = ( 0, 0, 0 );
  	     my @routerInterfaces = ();
	     my $interfaceLeadingSpace = "";

          while( my $line=<$infile> ) {
              ### DEV-NOTE :
              ###     Configuration what we read seems to system generated/o/p,
              ###     Anyhow for safety, whenever we get echo, we will initialize all block as 0
              if( $line =~ /^echo\s+/ ) {
                 $isRouterBlock = 0;
                 $isIGMPBlock = 0;
              }
              if( $isRouterBlock == 0 and $isIGMPBlock == 0 ) {
                  if( $line !~ /^echo/ ) {
                      print $outfile $line;
                      next;
                  }
                  print $outfile $line;
                  if( $line =~ /^echo "Router \(Network Side\) Configuration"/ ) {
                      $isRouterBlock = 1;
                  } elsif ( $line =~ /^echo "IGMP Configuration"/ ) {
                      $isIGMPBlock = 1;
                  }
              } elsif ( $isRouterBlock == 1 ) {
                  print $outfile $line;
                  if ( $line =~ /^\s*interface\s*"([^"]+)/ ) {
                        push @routerInterfaces, $1;
                  }
              } elsif ( $isIGMPBlock == 1 ) {
                  ### DEV-NOTE :
                  ###     Removing the matched interface block till next interface block starts
                  if ( $line =~/^(\s*)interface\s*"([^=]+)"/ ) {
                       $interfaceNeedToRemove = 0;
		       $interfaceLeadingSpace = $1;
                       if( grep( /^$2$/, @routerInterfaces) ) {
                          $interfaceNeedToRemove = 1;
                       } else {
                          print $outfile $line;
                       }
                  } elsif ( $interfaceNeedToRemove == 0 ) {
                          print $outfile $line;
                  } elsif ( ( $interfaceNeedToRemove == 1 ) and ( $line =~ /^(\s*)exit(\s*)/ ) ) {
 			 if( "$1" eq "$interfaceLeadingSpace" ){ 
				$interfaceNeedToRemove = 0;
			 }
		  }
              }
            }

          close($infile); 
          close($outfile);
 
          $cmd = "cd $dir;$config->{CP} $newihubCfgFile $ihubCfgFile >>$logDir/MIGRcpIPDcfg.txt 2>&1";
          my $ret = system($cmd); 
          if($ret) {
             my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
             my $tMsg = "$cmd failed: $!";
             FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
             $ok = 0;
          }
       } else {
         $err = "Failed to open $newihubCfgFile for writing: $!";
         FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
         $ok = 0;
       }
     } else {
       $err = "Failed to open $ihubCfgFile for reading: $!";  
       FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
       $ok = 0; 
     }
  }
  return $ok;
}


sub UpdateIPDDhcp($$)
{
  my ($fh, $dir) = @_;
  my $ok = 1;
  my $NEWIHUBCFG = "ipdconfig_dhcp.cfg"; 
  my $ihubCfgFile = "$dir/newdb/$IHUBDIR/$IHUBCONFIG"; 
  my $newihubCfgFile = "$dir/$NEWIHUBCFG"; 
  #my @lines; 
  #my @newlines; 
  my $tempVar; 
  my $var;
  my $line; 
  my ($infile, $outfile);
  my $cmd; 
  my $err; 
  my $logDir = FamilyCommon::GetNeStorage();
  my ($lease_populate, $dhcpUpdate ) = (0,0);
 

   if ( -e "$ihubCfgFile") {
     if (open($infile, $ihubCfgFile)) {
       if (open($outfile, ">" . $newihubCfgFile)) {
	    my ( $exitLeadingSpace, $dhcpLeadingSpace ) = ("", "");
	    my ( $dhcpBlock ) = 0;

          while( my $line=<$infile> ) {
             if( $line =~ /^(\s*)dhcp\s*$/ ) {
		   $dhcpLeadingSpace = $1;
	           $dhcpBlock = 1;
                   $lease_populate = 0;
	     }
             if( $dhcpBlock == 1 ) {
		if(  $line =~ /^(\s*)lease-populate\s+(\d+)/ ) {
		     $lease_populate = $2;
		     if( $lease_populate > 16384 ) {
		          $line =~ s/$lease_populate/16384/;
			  $dhcpUpdate =1;
		     }
                }
                if ( $line =~ /^(\s*)exit\s+/ ) {
	              $exitLeadingSpace = $1;
	              if( "$exitLeadingSpace" eq "$dhcpLeadingSpace" ) {
	           	$dhcpBlock = 0;
	              }
                }
             }
	     print $outfile $line;
          }

          close($infile); 
          close($outfile);
 	  if( $dhcpUpdate == 1 ) {
             ToolCommon::Log($fh, "Updation carried out\n");
          }
 
          $cmd = "cd $dir;$config->{CP} $newihubCfgFile $ihubCfgFile >>$logDir/MIGRcpIPDcfg.txt 2>&1";
          my $ret = system($cmd); 
          if($ret) {
             my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
             my $tMsg = "$cmd failed: $!";
             FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
             $ok = 0;
          }
       } else {
         $err = "Failed to open $newihubCfgFile for writing: $!";
         FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
         $ok = 0;
       }
     } else {
       $err = "Failed to open $ihubCfgFile for reading: $!";  
       FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
       $ok = 0; 
     }
  }
  return $ok;
}


sub UpdateTPIDTranslation($$$)
{
  my ($fh, $dir, $feltblts) = @_;
  my $NEWIHUBCFG = "ipdconfig_translation.cfg";
  my $ihubCfgFile = "$dir/newdb/$IHUBDIR/$IHUBCONFIG";
  my $newihubCfgFile = "$dir/$NEWIHUBCFG";
  my $line;
  my $ltNo = "";
  my ($inFH, $outFH);
  my $logDir = FamilyCommon::GetNeStorage();

  my ($ok, $err ) = (1,"");
  my ($port_block, $port_indent, $port_end_block ) = ( 0, "","");
  my ($ethernet_block, $ethernet_indent, $ethernet_end_block ) = (0, "", "");

  if (open($inFH, $ihubCfgFile)) {
    if (open($outFH, ">" . $newihubCfgFile)) {
       while( $line=<$inFH> ) {
           if( $line =~ /^(\s*)port\s+([^\s]+)\s+$/ ) {
		$ltNo = $2;
 		$port_indent = $1;
 		$port_end_block = $port_indent."exit";
                $port_block = grep { /^$ltNo$/ } @feltbLts;
           }
           if( $port_block ) {
	      if( $line =~ /^$port_end_block\s*$/ ) {
		 $port_block = 0;
	      }elsif( $line =~ /^(\s*)ethernet(\s*)$/ ) {
		  $ethernet_indent = $1;
 		  $ethernet_block = 1;
		  $ethernet_end_block = $ethernet_indent."exit";
              }
	      if( $ethernet_block ) {
                if( $line =~ /^\s*use-vlan-dot1q-etype\s*$/ ) {
		    $line =~ s/use-vlan-dot1q-etype/no use-vlan-dot1q-etype/;
		    $port_block = 0;
		    $ethernet_block = 0;
                } elsif( $line =~ /^$ethernet_end_block\s*$/ ) {
		    $port_block = 0;
		    $ethernet_block = 0;
                }
	      }
           }
           printf $outFH $line;
       }
       close ($inFH);
       close ($outFH);
       my $cmd = "cd $dir;$config->{CP} $newihubCfgFile $ihubCfgFile >>$logDir/MIGRcpIPDcfg.txt 2>&1";
       my $ret = system($cmd); 
       if($ret) {
          my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
          my $tMsg = "$cmd failed: $!";
          FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
          $ok = 0;
       }
     } else {
         $err = "Failed to open $newihubCfgFile for writing: $!";
         FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
         $ok = 0;
     }
   } else {
       $err = "Failed to open $ihubCfgFile for reading: $!";
       FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
       $ok = 0;
   }
   return $ok;

}

1;
