package MigrateIsam7354;

=head1 ISAM 7354-specific migration module

=begin html
This library contains the functions dedicated to migrate a 7354 ISAM from one release to a new one.

=end html

=cut

#==============================================================================
#-- File:         MigrateIsam7354.pm
#-- Date:         22.09.2008
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to migrate ISAM Pizzaboxes to a new release
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/09/22 : First implementation
#--   1.0.1 - 2009/04/22 : ANTmt54730.  Do not create empty DB backup in case of activation only.
#                          Release transaction if TO = 127.0.0.1
#--   1.0.2 - 2011/04/01 : Changes for 7354
#--   1.0.3 - 2011/04/05 : ALU01031457 - Support for R4220 & R4230
#--   1.0.4 - 2016/07/18 : ALU02237966
#--   1.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "MigrateIsam7354.pm - 1.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "MigrateIsam7354.pm - 1.1.0 -- 2018/06/20";
#==============================================================================

use strict;
use warnings 'all';
require 5.001;

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2092-DFZZA">DM</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;
use Cwd qw(getcwd);

use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Utils;
require Octopus;
use Capability;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :MIGR);
require FamilyCommon;
use DM;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub MigrateNTDbase($$$$$);
sub DownloadDbase($$$$$);
sub MigrateOrUpgrade($$$$);
sub Migrate($$$);
sub DetermineSwVersion($);
sub SetToolData();
#==============================================================================
#== Constants
#==============================================================================
my $NTINFILENAME =  "dm_complete.tar";           # input name of the complete NT database
my $NTSMALLFILE =   "dm.tar";                     # input name of the "small" NT database (only used with ofldb).
my $NTOUTFILENAME = "dm_migrated.tar";          # Name of the complete NT database after migration
my $NTMIGFILE =     "migNt.txt";

#==============================================================================
#== Vars
#==============================================================================
my %warnLevels;
my $config;
my $toolData;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 MigrateNTDbase($$$$$$)

=begin html
Migration of the NT database.  If requested NT NW-type convertion will be performed once the database has been migrated.  Returns 0 in case of error.
<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>path</td><td>PBMT installation directory name</td></tr>
</table>

=end html

=cut

sub MigrateNTDbase($$$$$) {
  my ($fh, $dir, $from, $to, $path) = @_;
  my $thisPath = getcwd();
  my $ok = 1;
  return $ok;
}

=head2 DownloadDbase($$$$$$)

=begin html
Download migrated database to the NE using TFTP (&lt;= R2.2) or CLI (&gt; R2.2).  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>activeRelNum</td><td>the NE release coming from</td></tr>
  <tr><td>userName</td><td>Unix user name in case of multi-user operation (command line option) empty otherwise</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub DownloadDbase($$$$$) {
  my ($fh, $cli, $isam, $activeRelNum, $userName) = @_;
  my $ok = 1;
  return $ok;
}

=head2 MigrateOrUpgrade($$$$$)

=begin html
This function determines whether we need to migrate of upgrade the node.  True is returned in case of migration, FALSE in case of an upgrade
and the undefined value in case of an error.

=end html

=cut

sub MigrateOrUpgrade($$$$) {
  my ($fh, $config, $dirName, $isam) = @_;
  my $sourceDbVersion = $params->{SOURCE_DBVERSION};
  my $targetDbVersion = $params->{DBVER_TGTREL};
  return ($targetDbVersion eq $sourceDbVersion) ? 0 : 1;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Migrate($$$$)

=begin html
This function is called to perform migration of a node of the ISAM family.  In case of failure the undefined value is returned otherwise a string
of depending holding depending nodes is returned (only in case of ISAM-V, it will be an empty list in case of ISAM).<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>thisTftpDir</td><td>tftp directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Migrate($$$) {
  my ($newLog, $thisTftpDir, $isam) = @_;
  my $cli = FamilyCommon::GetCliSession2Close();
  my $ret;
  my ($newRelNum, $newFullRelNum);
  my $possible = 0;
  my $reason = "";
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $userName = getlogin();

  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  # Read warning levels
  %warnLevels = Utils::ReadConfig(sprintf("%s/%s", FamilyCommon::GetToolDirName(), $toolData->{WARNINGFILE}));
  if (! defined($params->{OFLDB})) {
    if (! $params->{ACTONLY}) {
      # Keep DB uploaded with previous migration
      # if (! -e $dir . "/$NTINFILENAME") {
      if (!CreateBUFile($parentLogFh, $nodeLogFH, $dir, $NTINFILENAME, $newLog, $childNum, $params->{DEBUG})) {
	CORE::exit(CREATE_FILE_FAIL_CODE);
      }
    }
  }
  if ((defined($params->{OFLDB}) && (! $params->{OFLONLY})) ||
      (! defined($params->{OFLDB}))) {
    if (!defined($cli)) {
      #-- Open Cli Connection
      ToolCommon::Log($nodeLogFH, "Open CLI...");
      $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
      if (!defined($cli)) {
	FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				     EC_OPENCLI_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK\n");
      }
      FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
      ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});
    }
    #-- STEP 0: acquire transaction
    Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
    return undef if( ToolCommon::AcquireTransaction() );
    Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");
    my $oldTransactionTO = IsamCli::GetTransactionTimeout($cli);
    if (!defined($oldTransactionTO)) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
	    			     EC_GET_TRANS_TO_FAILED,
	    			     sprintf(": %s", IsamCli::GetError()),
	    			     1);
	    return undef;
    } else {
	   ToolCommon::Log($nodeLogFH, "Transaction timeout: $oldTransactionTO\n") if $params->{DEBUG};
	   FamilyCommon::SetOldTransactionTO($oldTransactionTO);
	   $ret = IsamCli::SetTransactionTimeout($cli, TRANSACT_TIME);
	   if ($ret) {
	     ToolCommon::Log($nodeLogFH, " OK\n");
	   } else {
	     FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
	   			       sprintf(": %s", IsamCli::GetError()),
	   			       EC_SET_TRANS_TO_FAILED,
	   			       1);
	     return undef;
	   }
    }
  }
  my ($activeNt, $passiveNt);
  if (! defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
    my $oswpInfo;
    my $dbVersInfo;
    if (! defined($params->{OFLDB}) ||
	(defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
      #-- STEP 1b: Check if needed files are present on NT
      $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, Capability::Isam7354RUNtName(), undef,
					  $toolData->{RETRIES});
      if (!defined($oswpInfo)) {
	FamilyCommon::IncrErrorCount("Obtaining active SW failed.\n",
				     EC_GET_OSWP_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      }
      ToolCommon::Log($nodeLogFH, "Verify database versions...");
      $dbVersInfo = IsamCli::GetDbVersionInfo($cli);
      if (!defined($dbVersInfo)) {
	FamilyCommon::IncrErrorCount("  FAILED...skipping!!\n",
				     EC_GET_DB_INFO_FAILED,
				     sprintf(": %s", IsamCli::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK\n");
	if ($params->{DEBUG}) {
	  foreach my $key (keys %{$dbVersInfo}) {
	    ToolCommon::Log($nodeLogFH, "OSWP '$key': '$dbVersInfo->{$key}'\n");
	  }
	}
      }
    }
    my $activeRelNum;
    my $fullActiveRelNum;
    my $regExp = REGEXP;
    my $dbVersionActive = $params->{SOURCE_DBVERSION};
    my $dbVersionTarget = $params->{DBVER_TGTREL};
    my $dbVersionActiveRel = substr($dbVersionActive,1);
    my $dbVersionTargetRel = substr($dbVersionTarget,1);
    if (!defined($params->{OFLDB})) {
      $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
					         $oswpInfo->{'ACTIVE'}{'MINOR'},
                                                 0,
					         $dbVersInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});
      $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						     $oswpInfo->{'ACTIVE'}{'MINOR'},
						     $oswpInfo->{'ACTIVE'}{'SUFFIX'},
                                                     $dbVersInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});

        if ( $dbVersionActive ne $oswpInfo->{'ACTIVE'}{'DBVERSION'}) {
          FamilyCommon::IncrErrorCount(sprintf("DB version not as expected: %s vs %s\n",
	    				   $dbVersionActive, $oswpInfo->{'ACTIVE'}{'DBVERSION'}),
	    			   EC_DB_VERS_UNEXPECTED,
	    			   sprintf("%s vs %s",
	    				   $dbVersionActive, $oswpInfo->{'ACTIVE'}{'DBVERSION'}),
	    			   1);
          return undef;
        }
    } else {
      if (basename($params->{SRC_OSWP}) =~ /$regExp/) {
	$activeRelNum = Capability::ConvertRelease($1, $2);
	$fullActiveRelNum = Capability::ConvertRelease($1, $2, $3);
      }
    }
    ToolCommon::Log($nodeLogFH, "Active release info (short and full stream): ($activeRelNum, $fullActiveRelNum)\n") if $params->{DEBUG};

    if (($params->{OFLONLY}) || defined($params->{OFLDB})) {
      if (basename($params->{SWPNAME}) =~ /$regExp/) {
        my $dbVerTargetRel = $params->{DBVER_TGTREL}; 
        $newRelNum = Capability::ConvertRelease($1, $2, 0, $dbVerTargetRel);
        $newFullRelNum = Capability::ConvertRelease($1, $2, $3, $dbVerTargetRel);
      }
    } else {
      $newRelNum = Capability::ConvertRelease($oswpInfo->{'PASSIVE'}{'MAJOR'},
					      $oswpInfo->{'PASSIVE'}{'MINOR'});
      $newFullRelNum = Capability::ConvertRelease($oswpInfo->{'PASSIVE'}{'MAJOR'},
						  $oswpInfo->{'PASSIVE'}{'MINOR'},
						  $oswpInfo->{'PASSIVE'}{'SUFFIX'}); # -- 1.2.0
    }
    ToolCommon::Log($nodeLogFH, "New release info (short and full stream): ($newRelNum, $newFullRelNum)\n") if $params->{DEBUG};
     Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
     return undef if( ToolCommon::ReleaseTransaction() );
     Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");
    IsamCli::AllowAlarms($cli);
    FamilyCommon::CloseCliSession2Close();  # Store for IncrErrorCount
  }
}

=head2 DetermineSwVersion($$)

=begin html
Determines the SW version based on the NT type and DB version.

=end html

=cut

sub DetermineSwVersion($) {
  my ($fh) = @_;

  # To be completed.
}

=head2 SetToolData()

=begin html
Initializes the common data pointer of the tool

=end html

=cut

sub SetToolData() {
  
  $toolData = FamilyCommon::GetToolData();

}

1;
