package ONT7302HiCapCLI;

=head1 ONT Specific Common modules

=begin html
This library contains the functions dedicated to perform a ONT common operations .

=end html

=cut
#==============================================================================
#-- File        :    ONT7302HiCapCLI.pm
#-- Date        :    29-08-2018
#-- Company     :    Nokia
#-- Author      :    Krishnamurthy Ravichandran    
#------------------------------------------------------------------------------
#-- Function    :    Script to perform ONT common operation across the ONT modules
#------------------------------------------------------------------------------
#-- Disclaimer  :    This script contains references to proprietary information
#--                  which is owned by Nokia.
#------------------------------------------------------------------------------
#-- Copyright:    All rights reserved. Passing on and copying of this document,
#--               use and communication Of its contents not permitted without
#--               written authorization from Nokia
#------------------------------------------------------------------------------
#-- History:
#-- 3.0.1       :     2018/08/29  :  First Implementation  
#-- 3.0.2       :     2018/09/23  :  include check for ont status channel-pair
#-- 3.0.3       :     2018/12/05  :  check to return matching ONT sw from 
#                                    mapping file while finding s/w for particular
#                                    mnemonic/variant.
#------------------------------------------------------------------------------

=head1 VERSION

=begin html
Current version: "ONT7302HiCapCLI.pm - 3.0.3 -- 2018/12/05"

=end html

=cut

my $VERSION = "ONT7302HiCapCLI.pm - 3.0.3 -- 2018/12/05";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;

use FileHandle;
use File::stat;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;

#==============================================================================
#== Find the lib dir
use Cwd;
use File::Basename;

BEGIN {
  my $dir = dirname($0);
  my $path;
  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif ($dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib");
  unshift(@INC, "$path/lib-perl");
}
#==============================================================================

use Net::SNMP;
use IsamTL1;
use IsamCli;
use Crypt::Lite;
use Utils;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :ONT_ERR );
use ToolCommon;
use PBMTGlobal;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetONT($$);
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $STATUSFILE =       "result.txt";
my $toolData;
my $config;
my $init;
my %hw2mnemonic;
my %mnemonic_hw2sw;
my %ontinfo;
my %mnemonic;
my %mnemonic_hw_var2sw;
my %mnemonic_hw_sw2var;
my %mnemonic_var2sw;
my %aid2info;
my %onttype_to_match;  # BDFam48333
my %rel2sw;
my %sw2mnemonic;
my %mnemonic_var_hw;


=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2  mapfile($)

=begin html
 maps the data in the ont release mapping file to associative arrays
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
</table>

=end html

=cut

sub mapfile {
  my($release) = @_;

  my $toolData = FamilyCommon::GetToolData();   # BDFam53774
  my $file = defined ($toolData->{ONTRELDIR}) ? "$toolData->{ONTRELDIR}/$toolData->{ONTRELMAPFILE}" 
                                                : "$toolData->{ONTRELMAPFILE}"; # BDFam53774

  my %col2i;
  my @columns;

  # just to avoid to many hash lookup
  my $col_release   ;
  my $col_hwpartnum ;
  my $col_mnemonic  ;
  my $col_swpartnum ;
  my $col_voiptype  ;

  if (! defined ($release)){
    return undef;
  }

  if ( defined ($init) && $init eq $release ) {
    return $init;
  }

  #rebuild the map file
  if (!open(ONTMAP, $file)) {
 
   FamilyCommon::IncrErrorCount("Failed to retrieve ONT list",
                                 OPEN_MAPPING_FILE_FAILED, undef, 1);		 
    exit (106);
  }

  %mnemonic_hw_var2sw = ();
  %mnemonic_var2sw = ();
  %rel2sw = ();          #BDFam56555

  while(<ONTMAP>) {
    chomp;

    # release            | hwpartnum  | mnemonic   | swpartnum      | bldtype     | voiptype |

    if(/\#\s*release/) {
        s/^\#\s*//;
        s/\s*\|\s*$//;
        @columns = split(/\s*\|\s*/, $_);

        for(my $i = 0; $i < @columns; $i++) {
            $col2i{$columns[$i]} = $i;
        }

        $col_release   = $col2i{release};
        $col_hwpartnum = $col2i{hwpartnum};
        $col_mnemonic  = $col2i{mnemonic};
        $col_swpartnum = $col2i{swpartnum};
        $col_voiptype  = $col2i{voiptype};

        next;
    }

    if(/\|/) {
        s/^\|\s*//;
        s/\s*\|\s*$//;
        my @values = split(/\s*\|\s*/, $_);

        if ($release ne "ALL") {
          if($release ne $values[$col_release]) {
            next;
          }
        }

        if ( ! defined ($init) || ($init ne $release) ){
          $init = $release;
        }

####################################################################################################################################
        #BDFam62885 mapping swverplnd to mnemonic
        $sw2mnemonic{$values[$col_swpartnum]} = $values[$col_voiptype];
        $mnemonic_var_hw{$values[$col_mnemonic]}{$values[$col_voiptype]}{$values[$col_release]}.= $values[$col_hwpartnum] . ":";

####################################################################################################################################

        # mapping hwpartnum to mnemonic
        $hw2mnemonic{$values[$col_hwpartnum]} = $values[$col_mnemonic];

        # mapping mnemonic, hardware, variant to software
        $mnemonic_hw_var2sw{$values[$col_mnemonic]}{$values[$col_hwpartnum]}{$values[$col_voiptype]} = $values[$col_swpartnum];
        # mapping mnemonic, hardware, software to variant
        my $swpartnum_Modified = substr($values[$col_swpartnum],0,9);
        $mnemonic_hw_sw2var{$values[$col_mnemonic]}{$values[$col_hwpartnum]}{$swpartnum_Modified} = $values[$col_voiptype];

        # mapping mnemonic to all
        for(my $i = 0; $i < @columns; $i++) {
            $ontinfo{$values[$col_mnemonic]}{$values[$col_hwpartnum]}{$values[$col_voiptype]}{$columns[$i]} = $values[$i];
        }

        # mapping mnemonic + hardware to software
        $mnemonic_hw2sw{$values[$col_mnemonic]}{$values[$col_hwpartnum]} = $values[$col_swpartnum];

        # mapping mnemonic + Var to software
        $mnemonic_var2sw{$values[$col_mnemonic]}{$values[$col_voiptype]} = $values[$col_swpartnum];

        # mnemonic list to get all ONT Types
        $mnemonic{$values[$col_mnemonic]} = 1;
	
	$rel2sw{$values[$col_swpartnum]} = $values[$col_swpartnum];;
	

	
    }
  } # end of while.
  return $init;
}

sub mnemonic_voip2hw {
    my ($rel,$mnemonic,$vp) =@_;

    my $ret = mapfile($rel);


    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

    if(defined $mnemonic_var_hw{$mnemonic}{$vp}{$rel}) {
        return $mnemonic_var_hw{$mnemonic}{$vp}{$rel};
    } else {
        return undef;
    }
}

sub sw2mnemonic{
    my ($rel,$sw) =@_;

    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

    if(defined $sw2mnemonic{$sw}) {
        return $sw2mnemonic{$sw};
    } else {
        return undef;
    }

#    return (\%sw2mnemonic);
}

=head2  rel2type_hw_var_sw($)

=begin html
 maps the release to hardware variant and software
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
</table>

=end html

=cut

sub rel2type_hw_var_sw{
    my($rel) =@_;
    
  if(defined($rel) && $rel  =~ /[RL]\d{2}\.\d{2}\.\d{2}/){
 
    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

    return (\%mnemonic_hw_var2sw);
  }

 return undef;
}

=head2  gettypes($)

=begin html
 returns the ont types supported in this release
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
</table>

=end html

=cut

sub gettypes {
    my($rel) = @_;

    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

  return (\%mnemonic);
}


=head2  hw2mnemonic($)

=begin html
 returns mapping from harware to mnemonic
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
</table>

=end html

=cut

sub hw2mnemonic {
    my($rel,$t) = @_;

    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

    if(defined $hw2mnemonic{$t}) {
        return $hw2mnemonic{$t};
    } else {
        return undef;
    }
}

=head2  rel2sw($)

=begin html
 returns swpartno array  for that particular release
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
</table>

=end html

=cut

sub rel2sw {
    my($rel) = @_;
#    %rel2sw = ();      #BDFam56555
    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

  return (\%rel2sw);
}

=head2  mnemonic_hw2sw($$$)

=begin html
 returns software for the particular mnemonic and hardware variant
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
  <tr><td>type</td><td>The voip type for which mapping is required</td></tr>
  <tr><td>hardware</td><td>The hardware type for which mapping is required</td></tr>
</table>

=end html

=cut

sub mnemonic_hw2sw {
  my ($rel, $type,$hw) = @_;

  my $ret = mapfile($rel);

  if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
    return undef;
  }

  my $r = \%mnemonic_hw2sw;

  while(@_) {
    my $k = shift(@_);

    if (ref $r && defined $r->{$k}) {
      if(!@_) {
        return ${$r}{$k};
      } else {
        $r = $r->{$k};
        next;
      }
    } else {
      return undef;
    }

  }
}

=head2  mnemonic_var2sw($$$)

=begin html
 returns software for the particular mnemonic and variant
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
  <tr><td>type</td><td>The voip type for which mapping is required</td></tr>
  <tr><td>variant</td><td>The voip variant</td></tr>
</table>

=end html

=cut

sub mnemonic_var2sw{
  my ($rel, $type,$var) = @_;

  my $ret = mapfile($rel);

  if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
    return undef;
  }

  my $r = \%mnemonic_var2sw;

  shift(@_);
  while(@_) {
    my $k = shift(@_);

    if (ref $r && defined $r->{$k}) {
      if(!@_) {
        return ${$r}{$k};
      } else {
        $r = $r->{$k};
        next;
      }
    } else {
      return undef;
    }

  }
}

=head2  mnemonic_var2sw_multi($$$)

=begin html
 returns software for the particular mnemonic and variant
<br>
The following parameters are used:
<table>
  <tr><td>release</td><td>The release for which the data is collected</td></tr>
  <tr><td>type</td><td>The voip type for which mapping is required</td></tr>
  <tr><td>variant</td><td>The voip variant</td></tr>
</table>

=end html

=cut

sub mnemonic_var2sw_multi{
  my ($rel, $type,$var) = @_;
  my @sw_arr;

  my $ret = mapfile($rel);

  if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
    return undef;
  }
   for my $mnemonic ( keys %mnemonic_hw_var2sw ) {
      for my $hw ( keys %{ $mnemonic_hw_var2sw{$mnemonic}} ) {	
       for my $var ( keys %{ $mnemonic_hw_var2sw{$mnemonic}{$hw}} ) {	
        push @sw_arr, $mnemonic_hw_var2sw{$mnemonic}{$hw}{$var} if($mnemonic =~ /$type/); 
       } 
      } 
    } 
   return \@sw_arr;
}

=head2  ontinfo($)

=begin html
 returns an array with info regarding a particular type of ont
<br>
The following parameters are used:
<table>
  <tr><td>t</td><td>The mnemonic for which the particular info is to be retrived</td></tr>
</table>

=end html

=cut

sub ontinfo {
    my($rel, $t) = @_;

    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

    if(defined $ontinfo{$t}) {
        return $ontinfo{$t};
    } else {
        return undef;
    }
}

=head2 aid2info ($)

=begin html
 returns an array with info regarding a particular type of ont
<br>
The following parameters are used:
<table>
  <tr><td>aid</td><td>The aid for which the particular info is to be retrived</td></tr>
</table>

=end html

=cut

sub aid2info {
  my ($t) = @_;

  if (defined $aid2info{$t} ) {
    return $aid2info{$t};
  } else {
    return undef;
  }
}

sub mnemonic_hw_sw2var {
    my($rel,$k1,$k2,$k3) = @_;

    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

    my $r = \%mnemonic_hw_sw2var;
    shift(@_);

    while(@_) {
        my $k = shift(@_);

        if(ref $r && defined $r->{$k}) {
            if(!@_) {
                return $r->{$k};
            } else {
                $r = $r->{$k};
                next;
            }
        } else {
            return undef;
        }
    }

    return undef;
}


=begin html
 returns software for the particular mnemonic , hardware and variant.
<br>
The following parameters are used:
<table>
  <tr><td>rel</td><td>The release for which the data is collected</td></tr>
  <tr><td>k1</td><td>The mnemonic type for which mapping is required</td></tr>
  <tr><td>k2</td><td>The hardware type for which mapping is required</td></tr>
  <tr><td>k3</td><td>The variant  type for which mapping is required</td></tr>
</table>

=end html

=cut

sub mnemonic_hw_var2sw{
    my($rel,$k1,$k2,$k3) = @_;

    my $ret = mapfile($rel);

    if ((! $ret) || (defined ($ret) && ($ret ne $rel)) ) {
      return undef;
    }

    my $r = \%mnemonic_hw_var2sw;

    shift(@_);
    while(@_) {
        my $k = shift(@_);

        if(ref $r && defined $r->{$k}) {
            if(!@_) {
                return $r->{$k};
            } else {
                $r = $r->{$k};
                next;
            }
        } else {
            return undef;
        }
    }

    return undef;
}


# check if a ONT is match a list of types contained in a hash
# The specified types are mnemonic (for example I-020G-F)
# The mnemonic of an ONT can be found with a mapping between hwpartnum and mnemonic
# (see ONT_Release_Mapping.txt($toolData->{ONTRELMAPFILE}), hwpartnum is the first 10 characters of EQPTVERNUM)
# or with the last 6 characters of EQUIPID


sub _ont_type_match_simple {
    my($t, $types) = @_;

    my $nodeLogFh = FamilyCommon::GetWarnLogFile();
    
    if( defined $types->{'ALL'} && $types->{'ALL'} ){
      return 1;
    }
 
    if (! defined($t) ) {
      return 0;
    }

    my $hwpartnum =  substr($t, 0, 10);
    
    my $ont_mnemonic = hw2mnemonic("ALL",uc($hwpartnum));

    if(defined $ont_mnemonic) {
        #if (! (uc($ont_mnemonic) =~ /^[IOB]-(\d+)[A-Z]-[A-Z]$/) ){ # BDFam48335
        #  ToolCommon::Log($nodeLogFh,"Invalid type : $t, $ont_mnemonic\n");
        #  return 0;
        #}

	if(defined $types->{$ont_mnemonic} && $types->{$ont_mnemonic}) {
            $types->{$ont_mnemonic} = 2;
	    return 1;
	} else {
	    return 0;
	}
    }

    return 0;
}

=begin html
 returns list containing details for a particular type of onts.
<br>
The following parameters are used:
<table>
  <tr><td>tl1Session</td><td>handle to the already established tl1 session</td></tr>
  <tr><td>type</td><td>The type(mnemonic)of onts which are to be retrived (ALL will return all types)</td></tr>
</table>

=end html

=cut

sub getontlist {
  #my ($tl1Session, $type) = @_;
  my ($cliSession, $type) = @_;

  my $list;
  my $current_ont;
  my $full_ont;
  my %ontDet = (); 
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();

  if($type eq "ALL") {
    $onttype_to_match{"ALL"} = 1;
  } else {
    $onttype_to_match{"ALL"} = undef;
    my @types = split(':',uc($type));
    foreach my $t (@types) {
      if(! defined($onttype_to_match{uc($t)}) ) {
        $onttype_to_match{uc($t)} = 1;
      }
    }
  }


 # my $response = IsamTL1::getONT($tl1Session);
  my $cliCmd = "show equipment ont status pon"; 
  my $respStPon = IsamCli::getCliResponse($cliSession, $cliCmd);
  my ($ontDet) = {};

  if (!defined($respStPon)) {
    return undef;
  }

    my ($ontID, $serial_num, $admin_st, $opr_st) ; 
    foreach my $line (@{$respStPon}) {
        $ontID = ""; $serial_num = ""; $admin_st = ""; $opr_st = "";
        if($line =~ /([^ ]+)\s+([^ ]+)\s+([^ ]+)\s+(up|down|testing|not-appl)\s+(up|down|testing|not-appl)/) {
            $ontID = $2; $serial_num = $3; $admin_st = $4; $opr_st = $5;
            ###print "line -->  <$line>\n"; 
            $ontDet->{$ontID}{'sernum'}  = $serial_num;
            $ontDet->{$ontID}{'oprstat'}   = $opr_st;
            $ontDet->{$ontID}{'adminst'} = $admin_st;
        }
    }

#------------------
  $cliCmd = "show equipment ont status channel-pair"; 
  my $respStChPair = IsamCli::getCliResponse($cliSession, $cliCmd);

  if (!defined($respStChPair)) {
    return undef;
  }

    foreach my $line (@{$respStChPair}) {
        $ontID = ""; $serial_num = ""; $admin_st = ""; $opr_st = "";
        if($line =~ /([^ ]+)\s+([^ ]+)\s+([^ ]+)\s+(up|down|testing|not-appl)\s+(up|down|testing|not-appl)/) {
            $ontID = $2; $serial_num = $3; $admin_st = $4; $opr_st = $5;
            ###print "line -->  <$line>\n"; 
            $ontDet->{$ontID}{'sernum'}  = $serial_num;
            $ontDet->{$ontID}{'oprstat'}   = $opr_st;
            $ontDet->{$ontID}{'adminst'} = $admin_st;
        }
    }

#------------------

  $cliCmd = "info configure equipment ont interface flat"; 
  my $respOntIntFlat = IsamCli::getCliResponse($cliSession, $cliCmd);

  if (!defined($respOntIntFlat)) {
    return undef;
  }

    foreach my $line (@{$respOntIntFlat}) {
        chomp $line;
        next if ($line =~ s/configure\s+equipment\s+ont\s+interface\s+([^\s]+)\s+admin\-state//) ; 

        if ($line =~ s/configure\s+equipment\s+ont\s+interface\s+([^\s]+)\s+//) {
             #Logger::LogNotify( $nodeLogFH, "Fetching ONT Details \n");
             my $ontId = $1;
             if ($line =~ s/sw\-ver\-pland\s+([^\s]+)//) {
                $ontDet->{$ontId}{'swverplnd'} = $1;
             }
             if ($line =~ s/sernum\s+([^\s]+)//) {
                $ontDet->{$ontId}{'sernum'} = $1;
             }
             if ($line =~ s/sw\-dnload\-version\s+([^\s]+)//) {
                $ontDet->{$ontId}{'dlsw'} = $1;
             }
             if ($line =~ s/plnd\-var\s+([^\s]+)//) {
                $ontDet->{$ontId}{'plnd-var'} = $1;
             } else {
                $ontDet->{$ontId}{'plnd-var'} = ""; 
             }
        }
    }

  $cliCmd = "show equipment ont interface detail"; 
  my $respOntIntDetl = IsamCli::getCliResponse($cliSession, $cliCmd);

  if (!defined($respOntIntDetl)) {
    return undef;
  }

    $full_ont = ""; 

    my (@var_value); 

    foreach my $line (@{$respOntIntDetl}) {
       if($line =~ /[^\s]+\s+:/) {
           chomp $line; 
           $full_ont .= $line; 
       } elsif (($line =~ /^[\-]+/) || ($line =~ /^[\=]+/)) {
           if($full_ont ne "") { 
             my($h_ont) = {};
             $full_ont =~ s/ : /=/g;

             (@var_value) = $full_ont =~ /([^ ]+)=([^ ]*)/g;

             $h_ont->{"nblk"} = {@var_value};
             my $ontid = $h_ont->{nblk}{'ont-idx'};

             $h_ont->{"aid"} = $ontid; 
             $h_ont->{"state"} = $ontDet->{$ontid}{'oprstat'}; 

             if(@var_value % 2 != 0) {
                ToolCommon::Log($nodeLogFh,"Bad information '$full_ont' for ONT '$ontid'\n");
             } 
             #$h_ont->{nblk}{'sernum'}  = $ontDet->{$ontid}{'sernum'}; 
             $h_ont->{nblk}{'sernum'}  = $ontDet->{$ontid}{'sernum'}; 
             $h_ont->{nblk}{'adminst'}  = $ontDet->{$ontid}{'adminst'}; 
             $h_ont->{nblk}{'swverplnd'}  = $ontDet->{$ontid}{'swverplnd'}; 
             $h_ont->{nblk}{'dlsw'}  = $ontDet->{$ontid}{'dlsw'}; 
             $h_ont->{nblk}{'plnd-var'}  = $ontDet->{$ontid}{'plnd-var'}; 

#-----------------

             if ( defined ($h_ont->{nblk}{'version-number'}) ) {
               ToolCommon::Log($nodeLogFh, sprintf("ONT ID : %s and State : %s SWVERPLND: %s\n",$h_ont->{"aid"},$h_ont->{"state"},$h_ont->{nblk}{'swverplnd'} )) ;
               my $hwpartnum =  substr($h_ont->{nblk}{'version-number'}, 0, 10);
               my $ont_mnemonic = hw2mnemonic("ALL",uc($hwpartnum));
               if ($ont_mnemonic) {
                 $h_ont->{nblk}{MNEMONIC} = $ont_mnemonic;
                 my $swveract_Modified = defined($h_ont->{nblk}{'sw-ver-act'})?substr($h_ont->{nblk}{'sw-ver-act'},0,9):" ";
                 my $vp = mnemonic_hw_sw2var("ALL",uc($ont_mnemonic), uc($hwpartnum), uc($swveract_Modified));
                 $h_ont->{nblk}{VOIP}=$vp if (defined $vp);

               }
             }

             else {
               if (defined ($h_ont->{nblk}{'swverplnd'} ) && $h_ont->{nblk}{'swverplnd'} ne "auto") {
                ToolCommon::Log($nodeLogFh, sprintf("ONT ID : %s and State : %s  SWVERPLND: %s\n",$h_ont->{"aid"},$h_ont->{"state"},$h_ont->{nblk}{'swverplnd'} )) ;

                  my $vp = sw2mnemonic("ALL",$h_ont->{nblk}{'swverplnd'});
                  $h_ont->{nblk}{VOIP}=$vp if (defined $vp);
              }
              elsif(defined($h_ont->{nblk}{'swverplnd'} ) && $h_ont->{nblk}{'swverplnd'} eq "auto") {
                ToolCommon::Log($nodeLogFh, sprintf("ONT ID : %s and State : %s SWVERPLND: %s\n",$h_ont->{"aid"},$h_ont->{"state"},$h_ont->{nblk}{'swverplnd'} )) ;
              }
             }

#-----------------

             my ($aid, $h) = ($ontid, $h_ont) ;

            #BDFam55831 - Adding ALL as a onttype_to_match in the if statement
            if((defined ($onttype_to_match{"ALL"}) && $onttype_to_match{"ALL"})||((defined ($h->{nblk}{MNEMONIC})) && 
                (defined ($onttype_to_match{$h->{nblk}{MNEMONIC}}) ) &&
                ($onttype_to_match{$h->{nblk}{MNEMONIC}})) ) {
              push(@{$list}, $h);
            }
### TODO check ont code -->  add here
            $aid2info{$aid} = $h;
              
            $full_ont = ""; 
            Logger::LogNotify( $nodeLogFH, "Generated ONT Details for <$ontid>\n");

           }
       }

    }

  $list ? return $list : return 1;        # BDFam52428
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

sub GetONT($$) {
  my ($isam, $ontInfofh) = @_;
  my ($newMajor, $newMinor, $newSuffix, $newFullRelNum);
  my $shubSwName;
  my $activeNt;
  my $tl1Session = undef;
  my $ret;
  my $dir = FamilyCommon::GetNeStorage();
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();
  my $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();
  my $ontParams = 1;  
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;

  #-- Open TL1 Session
  ToolCommon::Log($nodeLogFh, "Open TL1 Session using port ($toolData->{TL1UDPPORT})\n");
  
  $tl1Session = IsamTL1::Connect($isam->[0], $toolData->{TL1UDPPORT}, $toolData->{TL1USER}, $toolData->{TL1PWD},
                                     $params->{DEBUG} ? $nodeLogFh : undef, $ontParams);
  if (!defined($tl1Session)) {
    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open TL1 session for %s: %s\n", 
                                 FamilyCommon::GetFailedNode(), IsamTL1::GetError()),
                                 EC_OPENTL1_FAILED,undef,
                                 1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFh, "Established TL1 Connection sucessfully\n");
  }
  FamilyCommon::SetTL1Session2Close($tl1Session);  # Store for IncrErrorCount

  #-- STEP 0: Acquire transaction
  ToolCommon::Log($nodeLogFh, "Attempting to Acquire transaction...\n");
  return 0 if( ToolCommon::TransactionReleased() );
  return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'acquire')) );

  #BDFam48393 starts ..
  $ret = IsamTL1::InhibitAlarms($tl1Session);
  if ($ret) {
    ToolCommon::Log($nodeLogFh, " Alarms Inhibited..\n");
  } else {
    ToolCommon::Log($nodeLogFh, " Error in Alarm Inhibit!!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                               EC_ISAM_INH_ALARM_FAILED, undef,
                               1);
    return 0;
  }
  #BDFam48393 ends ..

  my $ont_list;

  # read list of ONT types and prepare a hash
  if($toolData->{ALLTYPES}) {
    $onttype_to_match{"ALL"} = 1;
    $ont_list = getontlist($tl1Session,"ALL");
  } else {
    my @types = split(':',uc($toolData->{ONTTYPES}));
    foreach my $t (@types) {
      if(! defined($onttype_to_match{uc($t)}) ) {
        $onttype_to_match{uc($t)} = 1;
      }
    }

    $ont_list = getontlist($tl1Session,$toolData->{ONTTYPES});
  }

  if (! $ont_list) { # BDFam48273
    FamilyCommon::IncrErrorCount("Failed to retrieve ONT list",
                                 EC_GET_ALL_ONT_FAILED , undef, 0);
    return 0;
  }

  if ( $ont_list == 1 ) { # BDFam52428
        FamilyCommon::IncrWarnCount("ONT Type could not be found",
                                    EC_GET_WARN_TYPE_COULD_NOT_FOUND, undef);

  } else {
#  Logger::Log($nodeStatusLog, '[Update]', 'Get ONT list successful');

    # LRM format
    # <aid>:<PROVVERSION>:<SERNUM>:<SWVERPLND>:<ONTENABLE>:<PLNDVAR>:<DLSW>:<EQPTVERNUM>:
    # <SWVERACT>:<SWVERPSV>:<VENDORID>:<EQUIPID>:<AUTOSWPLNDVER>:<AUTOSWDLDVER>:<state>\n
	
	my @fields_to_print = qw(PROVVERSION
				 SERNUM
				 SWVERPLND
				 ONTENABLE
				 PLNDVAR
				 DLSW
				 EQPTVERNUM
				 SWVERACT
				 SWVERPSV
				 VENDORID
				 EQUIPID
				 AUTOSWPLNDVER
				 AUTOSWDLDVER);
	
	my $separator = ":";
	
        foreach my $ont ( @{$ont_list} ) {
	    
	    print $ontInfofh $ont->{'aid'}, $separator;
	    
	    foreach my $f (@fields_to_print) {
        	if(defined $ont->{'nblk'}{$f}) {
		    print $ontInfofh $ont->{'nblk'}{$f};
		}
		print $ontInfofh $separator;
	    }
	    
	    print $ontInfofh $ont->{'state'};
	    print $ontInfofh "\n";
	}

    my $partialFail = 0;

    if ($onttype_to_match{"ALL"}) {
     ToolCommon::Log($nodeLogFh, "ONT Type 'ALL' is completed\n"); 
    } else {
      foreach my $key (sort keys %onttype_to_match) {
        if ( $onttype_to_match{$key} != 2 ) {
#          Logger::Log($nodeStatusLog,"[Warning]",sprintf("ONT Type %s could not be found",$key));
          $partialFail = 1;
        }
      }

      if ($partialFail) {
        FamilyCommon::IncrWarnCount("ONT Type could not be found",
                                    EC_GET_WARN_TYPE_COULD_NOT_FOUND, undef);
      }
    }
  }

  #BDFam48393 starts here ..
  $ret = IsamTL1::AllowAlarms($tl1Session);
  if ($ret) {
    ToolCommon::Log($nodeLogFh, " Alarms allowed...\n");
  } else {
    ToolCommon::Log($nodeLogFh, " Failed to allow alarms!!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                               EC_ISAM_ALW_ALARM_FAILED,undef,
                               1);
    return 0;
  }
  #BDFam48393 ends here..

  return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'release')) );
  FamilyCommon::CloseTL1Session2Close();
  return 1;
}

1;
