
=head1 PBMT ISAM octopus interface

=begin html
This PERL module implements the T &amp;D accesses made to the ISAM using octopus.

=end html

=cut
#==============================================================================
#-- File:         Octopus.pm
#-- Date:         08.03.2006
#-- Company:      Alcatel Bell - TEC
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Collection of octopus scripts
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel 2006
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2006/03/06 : First implementation
#--   1.2.0 - 2006/03/22 : Introduction of the capability model
#--   1.4.3 - 2006/10/18 : Timestamp logging
#--   1.4.4 - 2007/10/19 : ANTms51632 allow multiple users for same ISAM.
#--   1.4.5 - 2007/08/25 : ANTms55192: Reset NT using T&D in case the NT is still
#--                        running after an OSWP activation.
#--   1.4.6 - 2007/09/05 : ANTms62648: Workaround not OK for redundant systems.
#--   1.4.7 - 2008/02/12 : ANTms83783: - Check for NT crash when parsing T&D output
#--                        ANTms83783: - Remap error record.
#--   1.4.8 - 2008/12/17 : ANTmt36445.
#--   1.4.9 - 2009/06/18 : ANTmt64040.
#--   1.4.10 - 2009/06/24: ANTmt64836: Fix warning generated in ANTmt64040.
#--   1.4.11 - 2009/06/24: Using Capability.pm results in %ntCapabilities to get mixed up resulting in NTs not being 
#--                        supported any longer.
#--   1.4.12 - 2009/11/23: ANTmt86146: Fix for user not able to open Octopus for 2 NE's at a time in same server.
#--   1.4.13 - 2010/03/04: PBMT x86 Changes - ANTmt88407
#--   3.0.01 - 2011/01/12: set the Octopus path to seek the correct executable for the platform 
#--   3.0.02 - 2011/03/27: octopus changes for linux
#--   3.0.02 - 2011/03/27: octopus changes for linux
#--   3.0.03 - 2012/07/18: Sftp functionality for reduce
#--   3.0.04 - 2013/10/14: ALU01913931 Adapted for enable read/write on /Sw/OSWPx 
#--   3.0.05 - 2013/10/34: ALU01913513  
#--   3.0.06 - 2015/05/04: ALU02062224  
#--   3.0.07 - 2015/10/13: ALU02135551
#--   3.0.08 - 2016/03/01: ALU02182211
#--   3.0.09 - 2016/07/15: ALU02237966 
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "Octopus.pm -- 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "Octopus.pm -- 3.1.0 -- 2018/06/20";
#==============================================================================

package Octopus;


=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2090-DFZZA">ReduceSWP</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use Cwd qw(getcwd);
use File::Basename;
use Config; 

require Utils;
use ErrorCodes qw(:EXIT_CODES :OCTO);
use ToolCommon qw(:CONSTANTS);
use FamilyCommon;
use PBMTGlobal;
use Logger;
require ReduceSWP;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub SelectFH($);
sub GetError();
sub SetVars($$$);
sub CheckEnv($);
sub OctopusCommand($$$$;$$$);
sub SwMgntIniSizeOK($$$$$;$);
sub GetSwMgntIniFileSize($$$$$);
sub GetOswpFromFW($$$$$;$);
sub OswpActivationPending($$$$);
sub ShubSwPresent($$$$$$);
sub RemoveNtShubSw($$$$);
sub EnableSwWrite($$$$;$);
sub RemoveSwPkg($$$$;$$);
sub EnableDbaseRead($$$$);
sub RemoveNtShubConf($$$$);
sub SuspendTftp($$$$);
sub ReduceActiveSWP($$$$$$;$);
sub SyncLanxSw($$$$);
sub OctopusActive($$$$);
sub ResetNT($$$$$$$);
sub RemapErrRecord($$$$);
sub SwFilePresent($$$$); # new
#==============================================================================
my $fh = undef;
my $OCTOPUSSCRIPTS = undef;
my $OCTOPUSPATH = undef;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

my $NT_DEAD = "NT dead";
my $ERROR = "";

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}

=head2 SetVars($$$)

=begin html
Sets the required environment to use the octopus interface:
<table>
  <tr><td>scriptsPath</td><td>path containing the octopus (shell) scripts</td></tr>
  <tr><td>octopusPath</td><td>path where to find the octopus executable</td></tr>
  <tr><td>handle</td><td>file handle for progress and debug messages</td></tr>
</table>

=end html

=cut

sub SetVars($$$) {
  my ($scriptsPath, $octopusPath, $handle) = @_;

  $OCTOPUSSCRIPTS = $scriptsPath;
  if ($Config{archname} =~ /sun4/) {
    $OCTOPUSPATH = $octopusPath."/sun4-solaris-64int";
  } elsif ($Config{archname} =~ /i86pc/) {
    $OCTOPUSPATH = $octopusPath."/i86pc-solaris-64int";
  } elsif ($Config{archname} =~ /linux/) {
    $OCTOPUSPATH = $octopusPath."/i386-linux-thread-multi";
  }
  $fh = &SelectFH($handle);
}

=head2 CheckEnv($)

=begin html
Environment check:  A file handle for output of information is required.  Every function in this packacge should call this.<br>
The following parameters are used:
<table>
  <tr><td>caller</td><td>string holding the name of the calling function</td></tr>
</table>
The tool exists in case no init has been done.

=end html

=cut

sub CheckEnv($) {
  my ($caller) = @_;

  if (!defined($OCTOPUSSCRIPTS) || !defined($OCTOPUSPATH) || !defined($fh)) {
    print STDERR "Octopus::$caller - environment not set ('$OCTOPUSSCRIPTS', '$OCTOPUSPATH', '$fh')!!!\n";
    CORE::exit(OCTO_ENV_NOT_SET_CODE);
  }
}

=head2 OctopusCommand($$$$;$$$)

=begin html
Launch octopus command script.<br>The following parameters are used:
<table>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>script</td><td>name of the octopus script</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>log</td><td>name of an logfile containing  which will hold the output produced by the octopus script</td></tr>
  <tr><td>p1</td><td>first parameter supplied to the octopus script</td></tr>
  <tr><td>p2</td><td>second parameter supplied to the octopus script</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns the Unix exit code.

=end html

=cut

my $cmdNum = 0;
sub OctopusCommand($$$$;$$$) {
  my ($dir, $script, $isam, $log, $p1, $p2, $path) = @_;
  $path = getcwd() if (!defined($path));
  my $cmd = "";
  my $par1 = defined($p1) ? $p1 : "";
  my $par2 = defined($p2) ? $p2 : "";

  $cmdNum++;
  my $debugFile = sprintf("octCmd%d.txt", $cmdNum);
  return defined $log ? "cd $dir;$path/$OCTOPUSSCRIPTS/$script -o $path/$OCTOPUSPATH -g $log $par1 $par2 $isam> $debugFile 2>&1" :
    "cd $dir;$path/$OCTOPUSSCRIPTS/$script -o $path/$OCTOPUSPATH $isam $par1 $par2> $debugFile 2>&1";
}

=head2 SwMgntIniSizeOK($$$$$;$)

=begin html
Check the size of the SwMgnt.ini file (required on R2.2 systems).<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>criticalSize</td><td>The critical file size in case the board was flashed with R2.2</td></tr>
  <tr><td>altSize</td><td>The critical file size in case the board was flashed with SW > R2.2</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 0 in case size is lower then the critical size, 0 in case when it is bigger.

=end html

=cut

sub SwMgntIniSizeOK($$$$$;$) {
  my ($debug, $dir, $isam, $criticalSize, $altSize, $path) = @_;
  my ($fileSize, $stringFound);
  my $ok = 1;

  $ERROR = "";
  CheckEnv("SwMgntIniSizeOK");
  Logger::LogDebug($nodeLogFH, "Checking the SwMgnt.ini file size (criticalSize = $criticalSize or $altSize)...");
  ($fileSize, $stringFound) = GetSwMgntIniFileSize($debug, $dir, $isam, "ProtGroupLocked", $path);
  if ($fileSize < 0) {
    $ok = 0;
  } else {
    $criticalSize = $altSize if $stringFound;
    if ($fileSize > $criticalSize) {
      $ERROR = "Migration or rollback (after migration) of this ISAM isn't safe (filesize: $fileSize)";
      Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
      $ok = 0;
    } else {
      Logger::LogDebug($nodeLogFH, "  OK ($fileSize bytes < $criticalSize bytes)!\n");
    }
  }
  return $ok;
}

=head2 GetSwMgntIniFileSize($$$$$)

=begin html
Retrieve the size of the SwMgnt.ini file from the ISAM (required on R2.2 systems).<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>string</td><td>String identifying whether the board was flash with R2.2 or higher</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns the filesize and the presence of the string.  In case of error a filesize of -1 is returned.

=end html

=cut

sub GetSwMgntIniFileSize($$$$$) {
  my ($debug, $dir, $isam, $string, $path) = @_;
  my $logFile = "readSwMgntIni.log";
  my $cmd = OctopusCommand($dir, "readSwMgntIni.sh", $isam->[0], $logFile, undef, undef, $path);
  my $fileSize = -1;
  my $stringFound = 0;
  my $sizeFound = 0;

  CheckEnv("GetSwMgntIniFileSize");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n");
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $line;
    while (defined($line = <RESULT>) && (! $sizeFound || ! $stringFound)) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n");
                    #      45   2   0   0     1    920   100777 SwMgnt.ini
      if (($line =~ m/^\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+SwMgnt.ini.*/) &&
	  (! $sizeFound)) {
	$sizeFound = 1;
	$fileSize = $6;
#      } elsif (($line =~ /$string/) && (! $stringFound)) {
      } elsif (($line =~ /ProtGroupLocked/) && (! $stringFound)) {
	$stringFound = 1;
      }
    }
    close RESULT;
  }
  Logger::LogDebug($nodeLogFH, "GetSwMgntIniFileSize::fileSize: '$fileSize', stringfound: $stringFound\n");
  return ($fileSize, $stringFound);
}

=head2 GetOswpFromFW($$$$$;$)

=begin html
Determine OSWP info using firmware settings.  Will be called in case the CLI command returned no information (e.g. SW bug or in case of migration
from R2.2 to R2.3 or higher)<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>string</td><td>String possibly holding the OSWP reference</td></tr>
  <tr><td>regExp</td><td>Regular expression used to determine the release number information</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: "ACTIVE" and "PASSIVE".  For each
entry the following is given:
<ul>
  <li>INDEX: the associated OSWP index</li>
  <li>NAME: the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>COMMITED: the OSWP SW commited status</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>

=end html

=cut

sub GetOswpFromFW($$$$$;$) {
  my ($debug, $dir, $isam, $string, $regExp, $path) = @_;
  my $logFile = "readOswpFW.log";
  my $cmd = OctopusCommand($dir, "readOswpFW.sh", $isam->[0], $logFile, undef, undef, $path);
  my $activeOswp = -1;
  my ($oswpNameAct, $oswpNamePas);
  my $oswpNameActFound = 0;
  my $oswpNamePasFound = 0;
  my $loggedIn = 0;
  my $loggedOut = 0;
  my $ecntaPresent = 0;
#SW management runstate and event
  my $runState = "";
  my $runEvent = "";
  my $initState = "";
  my $errRecRelease = "";
  my $actRelNum = 0;
  my %oswpInfo = ('ACTIVE' => { 'INDEX' => 0,
				'NAME' => NO_OSWP,
				'COMMITED' => "",
				'MAJOR' => 0,
				'MINOR' => 0,
				'SUFFIX' => 0,
				'MAINT' => 0,
				'REDUCED' => 0},
		  'PASSIVE' => { 'INDEX' => 0,
				 'NAME' => NO_OSWP,
				 'COMMITED' => "",
				 'MAJOR' => 0,
				 'MINOR' => 0,
				 'SUFFIX' => 0,
				 'MAINT' => 0,
				 'REDUCED' => 0},
		  'FRMW_OSWP' => 0,
		  'ACT_PENDING' => -1);

  CheckEnv("GetOswpFromFW");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n");
  my $ret = system($cmd);
  if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
      Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
      return undef;
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $line;
    while (defined($line = <RESULT>)) {
# &&
#	   (($activeOswp == -1) || ($oswpInfo{'ACTIVE'}{'NAME'} eq "NO_OSWP") || ($oswpInfo{'PASSIVE'}{'NAME'} eq "NO_OSWP"))) {
# read complete file
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n");
      $loggedIn = 1 if ($line =~ /T\&D console USR\d+ login\./);
      $loggedOut = 1 if ($line =~ /\{nt\d\d\d\d\}\[USR\d+\-shell\]\>exit/);
      if ($line =~ /\d\d\d\d\s*:\s*ECNT\-A/) {
	$ecntaPresent = 1;
	Logger::LogDebug($nodeLogFH, "ECNT-A equipped\n");
      } elsif ($line =~ /running oswp=(\d)/) {
	$oswpInfo{'FRMW_OSWP'} = $1;
	Logger::LogDebug($nodeLogFH, "Stored '$oswpInfo{FRMW_OSWP}' as active FW OSWP number\n");
      } elsif ($line =~ /ActivateOswpNr\s=\s(\d)/) {
	$oswpInfo{'ACT_PENDING'} = $1;
	Logger::LogDebug($nodeLogFH, "Stored activation pending for OSWP '$oswpInfo{ACT_PENDING}'\n");
      } elsif ($line =~ /ActiveOswp\s=\s(\d)/) {
	$activeOswp = $1;
	if ($ecntaPresent) {
	  # In case of wreset when actviating a new OSWP we run into the situation where the NT (on ECNT-A) is restarted with the new SW
	  # while the SHUB remains running with the old release.  When the system comes up, the SwMgnt.ini file indicates that the old
	  # OSWP is still active while firmware management indicates that the other OSWP is active.  In this case we use the information
	  # provided by firmware management to continue the parsing of the SwMgnt.ini file.
	  $activeOswp = $oswpInfo{'FRMW_OSWP'} if ($activeOswp != $oswpInfo{'FRMW_OSWP'});
	}
      } elsif ($line =~ /RunState\s=\s(\w+)/) {
	$runState = $1;
      } elsif ($line =~ /RunEvent\s=\s(\w+)/) {
	$runEvent = $1;
      } elsif ($line =~ /InitState\s=\s(\w+)/) {
	$initState = $1;
	$oswpInfo{'ACTIVE'}{'INDEX'} = $activeOswp;
	$oswpInfo{'PASSIVE'}{'INDEX'} = $activeOswp == 1 ? 2 : 1;
	$oswpNameAct = sprintf("\\[Oswp%d\\]", $activeOswp);
	$oswpNamePas = $activeOswp == 1 ? "\\[Oswp2\\]" : "\\[Oswp1\\]";
	Logger::LogDebug($nodeLogFH, "oswpNameAct, oswpNamePas: '$oswpNameAct', '$oswpNamePas'\n");
      } elsif ($line =~ /build name\s+:\s$regExp/) {
	# Regular error record
	my $thisRelNum = ($1*10 + $2) * 10 + $3;
	if ($thisRelNum > $actRelNum) {
	  $errRecRelease = "$1$2.$3$4";
	  Logger::LogDebug($nodeLogFH, "Stored '$errRecRelease as last active release (err rec build name)\n");
	} else {
	  Logger::LogDebug($nodeLogFH, "Error record from same or previous release\n");
	}
      } elsif ($line =~ /Application\s\/Sw\/$regExp\sstarted/) {
	# Application restart: is usually another release then the one found in the last error record (unless there was a prozone resize).
	$errRecRelease = "$1$2.$3$4";
	$actRelNum = ($1*10 + $2) * 10 + $3;
	Logger::LogDebug($nodeLogFH, "Stored $errRecRelease as last active release (restart notification)\n");
      }
      if ($activeOswp != -1) {
		if (defined($oswpNameAct) && ($line =~ /$oswpNameAct/)) {
		  Logger::LogDebug($nodeLogFH, "Found active OSWP\n");
		  $oswpNameActFound = 1;
	    } elsif (defined($oswpNamePas) && ($line =~ /$oswpNamePas/)) {
		  Logger::LogDebug($nodeLogFH, "Found passive OSWP\n");
		  $oswpNamePasFound = 1;
		 }	elsif ($oswpNameActFound) {
	  if ($line =~ /$string\s=\s(.*)/) {
	    $oswpInfo{'ACTIVE'}{'NAME'} = $1;
	    if ($oswpInfo{'ACTIVE'}{'NAME'} =~ /$regExp/) {
	      $oswpInfo{'ACTIVE'}{'MAJOR'} = $1;
	      $oswpInfo{'ACTIVE'}{'MINOR'} = $2;
	      $oswpInfo{'ACTIVE'}{'SUFFIX'} = $3;
	      $oswpInfo{'ACTIVE'}{'MAINT'} = $4;
	    }
	  } elsif ($line =~ /CommitState\s=\s(.*)/) {
	    $oswpInfo{'ACTIVE'}{'COMMITED'} = $1;
	  }
	  $oswpNameActFound = 0 if (($oswpInfo{'ACTIVE'}{'MAJOR'} != 0) && ($oswpInfo{'ACTIVE'}{'COMMITED'} ne ""));
	} elsif ($oswpNamePasFound) {
	  if ($line =~ /$string\s=\s(.*)/) {
	    $oswpInfo{'PASSIVE'}{'NAME'} = $1;
	    if ($oswpInfo{'PASSIVE'}{'NAME'} =~ /$regExp/) {
	      $oswpInfo{'PASSIVE'}{'MAJOR'} = $1;
	      $oswpInfo{'PASSIVE'}{'MINOR'} = $2;
	      $oswpInfo{'PASSIVE'}{'SUFFIX'} = $3;
	      $oswpInfo{'PASSIVE'}{'MAINT'} = $4;
	    }
	  } elsif ($line =~ /CommitState\s=\s(.*)/) {
	    $oswpInfo{'PASSIVE'}{'COMMITED'} = $1;
	  }
	  $oswpNamePasFound = 0 if (($oswpInfo{'PASSIVE'}{'MAJOR'} != 0) && ($oswpInfo{'PASSIVE'}{'COMMITED'} ne ""));
	}
      }
    }
    close RESULT;
    # Check release from error records (if one was found versus the active OSWP as dertermined along the line
    if ($errRecRelease ne "") {
      if (!($oswpInfo{'ACTIVE'}{'NAME'} =~ /$errRecRelease/)) {
	# The release running as determined from the error records does not seem to be the active release... so it can only be the 'passive' one
	# which still seems to be active
	if ($oswpInfo{'PASSIVE'}{'NAME'} =~ /$errRecRelease/) {
	  # Swap active and passive - should normally not occur since this means that SW management is "in a mess"
	  Logger::LogNotify($nodeLogFH, "***ALERT*** active and passive switched: SwMgnt.ini not in line with active SW as determined from error recs\n");
	  my $prevMaj = $oswpInfo{'ACTIVE'}{'MAJOR'};
	  my $prevMin = $oswpInfo{'ACTIVE'}{'MINOR'};
	  my $prevSfx = $oswpInfo{'ACTIVE'}{'SUFFIX'};
	  my $prevMnt = $oswpInfo{'ACTIVE'}{'MAINT'};
	  my $prevNme = $oswpInfo{'ACTIVE'}{'NAME'};
	  $oswpInfo{'ACTIVE'}{'INDEX'} = $oswpInfo{'PASSIVE'}{'INDEX'};
	  $oswpInfo{'ACTIVE'}{'NAME'} = $oswpInfo{'PASSIVE'}{'NAME'};
	  $oswpInfo{'ACTIVE'}{'MAJOR'} = $oswpInfo{'PASSIVE'}{'MAJOR'};
	  $oswpInfo{'ACTIVE'}{'MINOR'} = $oswpInfo{'PASSIVE'}{'MINOR'};
	  $oswpInfo{'ACTIVE'}{'SUFFIX'} = $oswpInfo{'PASSIVE'}{'SUFFIX'};
	  $oswpInfo{'ACTIVE'}{'MAINT'} = $oswpInfo{'PASSIVE'}{'MAINT'};
	  $oswpInfo{'PASSIVE'}{'INDEX'} = $oswpInfo{'ACTIVE'}{'INDEX'} == 1 ? 2 : 1;
	  $oswpInfo{'PASSIVE'}{'NAME'} = $prevNme;
	  $oswpInfo{'PASSIVE'}{'MAJOR'} = $prevMaj;
	  $oswpInfo{'PASSIVE'}{'MINOR'} = $prevMin;
	  $oswpInfo{'PASSIVE'}{'SUFFIX'} = $prevSfx;
	  $oswpInfo{'PASSIVE'}{'MAINT'} = $prevMnt;
	}
      }
    }
  }
  Logger::LogDebug($nodeLogFH,
		  sprintf("(string %s) - activeOswp: '%s', activeOswpName: %s\n",
			  $string, $oswpInfo{ACTIVE}{INDEX}, $oswpInfo{ACTIVE}{NAME}));
  Logger::LogDebug($nodeLogFH,
		  sprintf("(string %s) - passiveOswp: '%s', passiveOswpName: %s\n",
			  $string, $oswpInfo{PASSIVE}{INDEX}, $oswpInfo{PASSIVE}{NAME}));
  $ERROR = "";
  if ($loggedIn) {
    $ERROR = $NT_DEAD if ! $loggedOut;
  } else {
    $ERROR = $NT_DEAD;
  }
  return \%oswpInfo;
}

=head2 OswpActivationPending($$$$)

=begin html
Determine whether OSWP activation is still pending in the NE.  Use the ActivateOswpNr field of the SwMgnt.ini file.  When set to -1 no more
activation is pending.  This function returns:<br>
<ul>
  <li>undefined value in case of error
  <li>0: no more activation pending
  <li>&gt; 0: activation pending.  The returned value indicates the OSWP to be activated
</ul>

<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>

=end html

=cut

sub OswpActivationPending($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "oswpActivation.log";
  my $cmd = OctopusCommand($dir, "readOswpFW.sh", $isam->[0], $logFile, undef, undef, $path);
  my $activatePending = 0;
  my $loggedIn = 0;
  my $loggedOut = 0;

  CheckEnv("OswpActivationPending");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n");
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    return undef;
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $line;
    while (defined($line = <RESULT>)) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n");
      $loggedIn = 1 if ($line =~ /T\&D console USR\d+ login\./);
      $loggedOut = 1 if ($line =~ /\{nt\d\d\d\d\}\[USR\d+\-shell\]\>exit/);
      if ($line =~ /ActivateOswpNr\s=\s(\d)/) {
	$activatePending = $1;
      }
   }
   close RESULT;
  }
  Logger::LogDebug($nodeLogFH, "activation pending: $activatePending\n");
  $ERROR = "";
  if ($loggedIn) {
    $ERROR = $NT_DEAD if ! $loggedOut;
  } else {
    $ERROR = $NT_DEAD;
  }
  return $activatePending;
}

=head2 ShubSwPresent($$$$$$)

=begin html
Check presence of Lanx.Z in /Sw of the ISAM (used when migrating away from R2.0/R2.1)<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>script</td><td>name of the script to be executed</td></tr>
  <tr><td>shubSwSize</td><td>Expected size of the SHUB SW file</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns
<ul>
  <li>&lt; 0: failed to execute shell script</li>
  <li>= 0: file not found or file found but does not have the expected size</li>
  <li>&gt; 0: file found and size OK</li>
</ul>

=end html

=cut

sub ShubSwPresent($$$$$$) {
  my ($debug, $dir, $isam, $script, $shubSwSize, $path) = @_;
  my $logFile = $script . ".log";
  my $cmd = OctopusCommand($dir, $script, $isam->[0], $logFile, undef, undef, $path);
  my $ret = 0;
  my $sizeRead = 0;

  $ERROR = "";
  CheckEnv("ShubSwPresent");
  Logger::LogDebug($nodeLogFH, "Checking for SHUB SW on NT...");
  Logger::LogDebug($nodeLogFH,  "CMD = $cmd\n");
  if (system($cmd)) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ret = -1;
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $found = 0;
    my $line;
    while (defined($line = <RESULT>) && ! $found) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n");
      # 58   2   0   0     1    598   100777 lanx.cfg
      if ($line =~ m/^\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+(\d+)\s+\d+\s+Lanx.Z.*/) {
	$sizeRead = 0 + $1;
	Logger::LogDebug($nodeLogFH, "Lanx.Z found, size: $sizeRead bytes\n");
	if ($sizeRead != $shubSwSize) {
	  Logger::LogDebug($nodeLogFH, "Size of file present differs ($sizeRead vs $shubSwSize)!!\n");
	} else {
	  $found = 1;
	  $ret = 1;
	  Logger::LogDebug($nodeLogFH, "  OK\n");
	}
      }
    }
    close RESULT;
    if (! $ret) {
      $ERROR = "SHUB file not found or not expected size (size read: $sizeRead)";
      Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    }
  }
  return $ret;
}


=head2 ShubSwPresent($$$$$$)

=begin html
Check presence of any given SW file in /Sw of the ISAM The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns
<ul>
  <li>&lt; 0: failed to execute shell script</li>
  <li>= 0: file not found or file found but does not have the expected size</li>
  <li>&gt; 0: file found and size OK</li>
</ul>

=end html

=cut

sub SwFilePresent($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my @swArray=(); 
  my $logFile = "SwPkg.log";
  my $cmd = OctopusCommand($dir, "shubSwCheck.sh", $isam->[0], $logFile, undef, undef, $path);
  my $ret = -1;

  $ERROR = "";
  CheckEnv("SwFilePresent");
  Logger::LogDebug($nodeLogFH, "Checking for SW on NT...");
  Logger::LogDebug($nodeLogFH,  "CMD = $cmd\n");
  if (system($cmd)) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
      Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    #$ret = -1;
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $found = 0;
    my $line;
    while (defined($line = <RESULT>) && ! $found) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n");
      # 58   2   0   0     1    598   100777 MTW8ANxx.xxx
       if ($line =~ m/(^\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+)(\w+.*)/)
      {
       push @swArray, $2;
      }
    }
    close RESULT;
    $ret = 1;
  }
  return ($ret, \@swArray);
}

=head2 RemoveNtShubSw($$$$)

=begin html
Remove Lanx.Z file from /Sw and /pub/Sw/Download and /Lanx.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub RemoveNtShubSw($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "delLanxZ.log";
  my $cmd = OctopusCommand($dir, "delLanxZ.sh", $isam->[0], $logFile, undef, undef, $path);
  my $ok = 1;

  CheckEnv("RemoveNtShubSw");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Remove SHUB SW from /pub/Sw/Download...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n");
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ok = 0;
  } else {
    Logger::LogDebug($nodeLogFH, "  OK\n");
    if ($debug) {
      open RESULT, "<" . $dir . "/" . $logFile;
      my $line;
      while (defined($line = <RESULT>)) {
	chomp $line;
	$line =~ s/\r//g;
	Logger::LogDebug($nodeLogFH, "'$line'\n");
      }
      close RESULT;
    }
  }
  return $ok;
}

=head2 EnableSwWrite($$$$)

=begin html
Enable read and write access on /Sw.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
  <tr><td>oswp</td><td>Optional:Indication to read/write /Sw/OSWPx</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub EnableSwWrite($$$$;$) {
  my ($debug, $dir, $isam, $path, $oswp) = @_;
  my $swOswpx = defined ($oswp) ? "-s $oswp" : undef;
  my $logFile = "enableSwWrite.log";
  $logFile = "enableSwOSWPXWrite.log" if $swOswpx; #Introdcued for ALU01913931
  my $cmd = OctopusCommand($dir, "enableSw_RW.sh", $isam->[0], $logFile, $swOswpx, undef, $path);
  my $ok = 1;

  if ( $oswp ) {
  	CheckEnv("EnableSwOSWPXWrite");
  	$ERROR = "";
  	Logger::LogDebug($nodeLogFH, "Enable read/write on /Sw/OSWPx...");
  } else {
  	CheckEnv("EnableSwWrite");
  	$ERROR = "";
  	Logger::LogDebug($nodeLogFH, "Enable read/write on /Sw...");
  }
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n");
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ok = 0;
  } else {
    Logger::LogDebug($nodeLogFH, "  OK\n");
    if ($debug) {
      open RESULT, "<" . $dir . "/" . $logFile;
      my $line;
      while (defined($line = <RESULT>)) {
	chomp $line;
	$line =~ s/\r//g;
	Logger::LogDebug($nodeLogFH, "'$line'\n");
      }
      close RESULT;
    }
  }
  return $ok;
}

=head2 RemoveSwPkg($$$$;$$)

=begin html
Enable read and write access on /Sw.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT install tool install directory</td></tr>
  <tr><td>oswp</td><td>OSWP pkg to be removed</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub RemoveSwPkg($$$$;$$) {
  my ($debug, $dir, $isam, $path,$swp, $oswp) = @_;
  my $logFile = "removeSwPkg.log";
  my $var1 = defined ($swp) ? "-s $swp" : undef;
  my $var2 = defined ($oswp) ? "-f $oswp" : undef;
  my $cmd = OctopusCommand($dir, "removeSwp.sh", $isam->[0], $logFile, $var1, $var2, $path);
  my $ok = 1;

  CheckEnv("RemoveSwPkg");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Remove Swp pkg on /Sw...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n");
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ok = 0;
  } else {
    Logger::LogDebug($nodeLogFH, "  OK\n");
    if ($debug) {
      open RESULT, "<" . $dir . "/" . $logFile;
      my $line;
      while (defined($line = <RESULT>)) {
        chomp $line;
        $line =~ s/\r//g;
        Logger::LogDebug($nodeLogFH, "'$line'\n");
      }
      close RESULT;
    }
  }
  return $ok;
}


=head2 EnableDbaseRead($$$$)

=begin html
Enable read and write access on /Dbase.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

#------------------------------------------------------------------------------
#-- Enable read and on /Dbase
#------------------------------------------------------------------------------
sub EnableDbaseRead($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "enableDbaseRead.log";
  my $cmd = OctopusCommand($dir, "enableDbase_R.sh", $isam->[0], $logFile, undef, undef, $path);
  my $ok = 1;

  CheckEnv("EnableDbaseRead");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Enable read on /Dbase...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n");
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
      Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ok = 0;
  } else {
    Logger::LogDebug($nodeLogFH, "  OK\n");
    if ($debug) {
      open RESULT, "<" . $dir . "/" . $logFile;
      my $line;
      while (defined($line = <RESULT>)) {
	chomp $line;
	$line =~ s/\r//g;
	Logger::LogDebug($nodeLogFH, "'$line'\n");
      }
      close RESULT;
    }
  }
  return $ok;
}

=head2 RemoveNtShubConf($$$$)

=begin html
Remove iss.conf file from /Sw and /pub/Sw/Download and /Lanx.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub RemoveNtShubConf($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "delIssconf.log";
  my $cmd = OctopusCommand($dir, "delIssconf.sh", $isam->[0], $logFile, undef, undef, $path);
  my $ok = 1;

  CheckEnv("RemoveNtShubConf");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Remove SHUB database from /pub/Sw/Download...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if $debug;
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ok = 0;
  } else {
    Logger::LogDebug($nodeLogFH, "  OK\n");
    if ($debug) {
      open RESULT, "<" . $dir . "/" . $logFile;
      my $line;
      while (defined($line = <RESULT>)) {
	chomp $line;
	$line =~ s/\r//g;
	Logger::LogDebug($nodeLogFH, "'$line'\n");
      }
      close RESULT;
    }
  }
  return $ok;
}

=head2 SuspendTftp($$$$)

=begin html
Suspend tftp.  Restart of a R2.2 NT also results in a restart of the SHUB.  When the tftp has been started by SHUB before the NT actually died there
is a risk that the LANx will decompress a partially received file and start it.  Suspending the tftp on the NT prevents this from happening.  This is
a workaround for a bug in the Lanx boot code.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>ISAM IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub SuspendTftp($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $suspended = 0;
  my $logFile = "suspendTftp.log";
  my $cmd = OctopusCommand($dir, "suspendTftp.sh", $isam->[0], $logFile, undef, undef, $path);  # -- 1.1.6

  CheckEnv("SuspendTftp");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Suspending tftp...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if ($debug);
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $found = 0;
    my $line;
    while (defined($line = <RESULT>) && ! $found) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n") if ($debug);
      if ($line =~ m/^result\s*\=\s*(\d*).*/) {
	$found = 1;
	if ($1 == 0) {
	  $suspended = 1;
	  Logger::LogDebug($nodeLogFH, "  OK\n");
	} else {
	  $ERROR = "No 'result=0' found";
      Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
	}
      }
    }
    close RESULT;
  }
  return $suspended;
}

=head2 ReduceActiveSWP($$$$$$$;$)

=begin html
Reduce active OSWP to make room in the SW partition.<br>The following parameters are used:
<table>
  <tr><td>isam</td><td>the IP address</td></tr>
  <tr><td>snmpSession</td><td>Reference to an opened SNMP session</td></tr>
  <tr><td>params</td><td>A reference to the command parameter environment of the PBMT tool</td></tr>
  <tr><td>count</td><td>the identification of the child process calling this function</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>redId</td><td>Reduction character string (holds the 2 characters to be overwritten on position 5 and 6 of the OSWP/SWP file</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub ReduceActiveSWP($$$$$$;$) {
  my ($isam, $snmpSession, $count, $dir, $redId, $path, $cli) = @_;
  my $ret = undef;
  my $cmd;
#  my $path = getcwd();

  CheckEnv("ReduceActiveSWP");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Reduce active SWP...");
  $ret = ReduceSWP::ReduceActiveSWP($isam, $snmpSession, $dir, $fh, $path, $redId, undef,
				    defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} : $params->{PRIMTFTPIP},defined($cli) ? $cli : undef);
  if ($ret) {
    my $exitCode = ErrorCodes::ErrorString($ret);
    my $err = ReduceSWP::GetError();
    $ERROR = sprintf("SW reduction ended prematurely: %s (%s)", $exitCode, $err);
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ret = 0;
  } else {
    $ret = 1;
  }
  return $ret;
}

=head2 SyncLanxSw($$$$)

=begin html
Syncing Lanx SW for systems > 2.2: in case NT runs 2.3 or higher (redundant systems).<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub SyncLanxSw($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "syncLanxZ.log";
  my $cmd = OctopusCommand($dir, "syncLanxZ.sh", $isam->[0], $logFile, undef, undef, $path);  # -- 1.1.6

  CheckEnv("SyncLanxSw");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Syncing Lanx.Z...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if ($debug);
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $line;
    while (defined($line = <RESULT>)) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n") if ($debug);
      if ($line =~ /failed/) {
	Logger::LogDebug($nodeLogFH, "  FAILED\n");
	$ERROR = "line read: '$line'";
	return 0;
      }
    }
    close RESULT;
  }
  return 1;
}

=head2 OctopusActive($$$$)

=begin html
Check whether an octopus session to this ISAM is acive.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns
<ul>
  <li>0: No octopus session active on NE</li>
  <li>&lt;&gt; 0: octopus session active to NE</li>
  <li>undef: script could not be executed</li>
</ul>

=end html

=cut

sub OctopusActive($$$$) {
  my ($debug, $isam, $grepCmd, $psCmd) = @_;
  my $cmd = "$psCmd | $grepCmd -v grep | grep octopus | grep '$isam->[0]:udp'";
  my $active = 0;
  my $line = "";
  my $pCount = 0;
  
  CheckEnv("OctopusActive");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if $debug;
  my $childPid = open(GREP, "$cmd |");
  if ( !defined($childPid)) {
     return FamilyCommon::ReportStatus( EC_OCTO_PS_ERROR, undef, undef );
  }
  while (defined($line = <GREP>)) {
    chomp $line;
    Logger::LogDebug($nodeLogFH, "$line\n") if $debug;
    $active++;
  }
  close GREP;
  if( $active ) {
      return FamilyCommon::ReportStatus( EC_OCTO_ACTIVE, undef, undef );
  }
  return 0;
}

=head2 ResetNT($$$$$$$)

=begin html
Reset NT.  R2.4 has a problem in OSWP activation where SW management prepares the ISAM for restart with the new SW but the NT does not restart.
Any sub-seqquent CLI SW management commands fail maining that only an err wreset can bring rescue.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
  <tr><td>passiveUp</td><td>Is passive NT up: 1=yes, 0=no ?</td></tr>
  <tr><td>tftpIp</td><td>TFTP IP address</td></tr>
  <tr><td>relFileName</td><td>NT reset command file (relaive to TFTP root directory)</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

#------------------------------------------------------------------------------
#-- Reset NT and SHUB.
#--            R2.4 has a problem in OSWP activation where SW management prepares
#--            the ISAM for restart with the new SW but the NT does not restart.
#--            Any sub-seqquent CLI SW management commands fail maining that only
#--            an err wreset can bring rescue.
#------------------------------------------------------------------------------
sub ResetNT($$$$$$$) {
  my ($debug, $dir, $isam, $path, $passiveUp, $tftpIp, $relFileName) = @_;
  my $ok = 1;
  my $logFile = "resetNT.log";
  my $cmd = OctopusCommand($dir, defined($passiveUp) ? "resetNTred.sh" : "resetNT.sh", $isam->[0], $logFile,
			   "-a $tftpIp", "-f $relFileName", $path);

  CheckEnv("resetNT");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, sprintf("Resetting NT and SHUB %s...", defined($passiveUp) ? "(Redundant)" : ""));
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if ($debug);
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    $ok = 0;
  } else {
    Logger::LogDebug($nodeLogFH, "  OK\n");
  }
  return $ok;
}

=head2 NtIsUp()

=begin html
This function returns true in case the NT is up or 0 in case the NT is dead.

=end html

=cut

sub NtIsUp() {
  return $ERROR eq $NT_DEAD ? 0 : 1;
}

=head2 RemapErrRecord($$$$)

=begin html
Map a non-recoverable error record onto a notification error record to prevent the NT from going through a hot reset.<br>
The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the IP address</td></tr>
  <tr><td>path</td><td>PBMT tool install directory</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub RemapErrRecord($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "mapNonRecov.log";
  my $cmd = OctopusCommand($dir, "mapNonRecov.sh", $isam->[0], $logFile, undef, undef, $path);  # -- 1.1.6

  CheckEnv("RemapErrRecord");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Map error record...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if ($debug);
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
    return 0;
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $line;
    while (defined($line = <RESULT>)) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n") if ($debug);
    }
    close RESULT;
  }
  Logger::LogDebug($nodeLogFH, "  OK\n");
  return 1;
}

sub ReduceSwFs($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "reduce_sw_fs.log";
  my $cmd = OctopusCommand($dir, "reduceSw_FW.sh", $isam->[0], $logFile, undef, undef, $path);  # -- 1.1.6

  CheckEnv("ReduceSwFs");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Reducing the /Sw FileSystem through T&D...");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if ($debug);
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $line;
    while (defined($line = <RESULT>)) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n") if ($debug);
      if ($line =~ /failed/) {
        Logger::LogDebug($nodeLogFH, "  FAILED\n");
        $ERROR = "line read: '$line'";
        return 0;
      }
    }
    close RESULT;
  }
  return 1;
}

sub UpdateNewFs($$$$) {
  my ($debug, $dir, $isam, $path) = @_;
  my $logFile = "update_fs_bp.log";
  my $cmd = OctopusCommand($dir, "updateNewFS.sh", $isam->[0], $logFile, undef, undef, $path);  # -- 1.1.6

  CheckEnv("UpdateNewFs");
  $ERROR = "";
  Logger::LogDebug($nodeLogFH, "Updating newfs BP flag..");
  Logger::LogDebug($nodeLogFH, "CMD = $cmd\n") if ($debug);
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
    Logger::LogError($nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
  } else {
    open RESULT, "<" . $dir . "/" . $logFile;
    my $line;
    while (defined($line = <RESULT>)) {
      chomp $line;
      $line =~ s/\r//g;
      Logger::LogDebug($nodeLogFH, "'$line'\n") if ($debug);
      if ($line =~ /failed/) {
        Logger::LogDebug($nodeLogFH, "  FAILED\n");
        $ERROR = "line read: '$line'";
        return 0;
      }
    }
    close RESULT;
  }
  return 1;
}



1;
#-- End of library
