package PreCheckIsam;

=head1 ISAM 7302/7330-specific pre-check module

=begin html
This library contains the functions dedicated to perform prechecks for a HiCap 7302/7330 ISAM.

=end html

=cut

#==============================================================================
#-- File:         PreCheckIsam.pm
#-- Date:         2018/07/09
#-- Company:      Nokia
#-- Author:       Dhineshkumar Manikannan
#------------------------------------------------------------------------------
#-- Function:     Supporting library carrying out the ISAM HiCap NE-family prechecks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   3.0.0  - 2018/07/09 : First implementation (Combined existing common libraries )
#--   3.0.1  - 2018/08/06 : UpgradeteRestricted implemented

=head1 VERSION

=begin html
Current version: "PreCheckIsam.pm - 3.0.1 -- 2018/08/06"

=end html

=cut

my $VERSION = "PreCheckIsam.pm - 3.0.1 -- 2018/08/06";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
require 5.001;

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;
use Config;
use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Crypt::Lite;
use Utils;
require Octopus;
use Capability;
use PbmtConfig;
require FamilyCommon;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :PRCK);
use version();
use CapabilityReEngr;
use vars qw(%version);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Check($$$);
sub CheckEnvironment($$$);
sub PrintToolVersions($$);
sub UploadBackUpDB($$$);
sub CreateRestoreDB($$$);
sub ResetGlobals();

#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;

#==============================================================================
#== Vars
#==============================================================================
my $warnLogFile;
my %config;
my $warnLevels;

my $config;
my $toolData;
my $NTOFLMTVERSION = 0; 
my $possibleBoardFamily = "";

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

sub ResetGlobals() {
   $NTOFLMTVERSION = 0;
}

=head2 CreateRestoreDB($$$)

=begin html
Creates 'RestoreDB' directory on /tftpboot/tmp/isam path. Creates an empty dm_complete.tar file inside the RestoreDB directory.
The dm_complete.tar is overwritten each time preCheck.pl is run.

=end html

=cut


sub CreateRestoreDB($$$) {
    my ($isamNode,$toolData,$configData) = @_;
    my $RESFILENAME = "dm_complete.tar";
    my $dir1 = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamNode";
    if(! -d "$dir1/RestoreDB") {
         if(!mkdir( "$dir1/RestoreDB", 0777)) {
            Logger::LogError($nodeLogFH, "Failed to create directory : $dir1/RestoreDB\n", undef, $Logger::LogOnlyHandle );
            return 0;
         }
         Logger::LogTrace($nodeLogFH, "Created RestoreDB '$dir1/RestoreDB' directory\n")
    }else{
       my $cmd = "$configData->{RM} -rf $dir1/RestoreDB/$RESFILENAME >$dir1/PRCKremove.txt 2>&1";
       my $sysret = system($cmd);
       if($sysret) {
          my ($exitCode, $signal) = Utils::ConvertExitCode($sysret);
          Logger::LogError($nodeLogFH, , "$cmd FAILED (signal $signal, exitcode $exitCode): $!", undef, $Logger::LogOnlyHandle);
          return 0;
       }
    }
    my $RESTOREDIR = "$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$isamNode/RestoreDB";
    my $cmd1 = "$configData->{TOUCH} $RESTOREDIR/$RESFILENAME >$dir1/PRCKtouch.txt 2>&1";
    my $sysret = system($cmd1);
    if($sysret) {
         my ($exitCode, $signal) = Utils::ConvertExitCode($sysret);
         Logger::LogError($nodeLogFH, "$cmd1 FAILED (signal $signal, exitcode $exitCode): $!", undef, $Logger::LogOnlyHandle);
         return 0;
    }else {
        ### File require full permision so upload from ISAM will succeed
        if($params->{UMASK_CHMOD} and ! chmod(01777, "$RESTOREDIR/$RESFILENAME")) {
           Logger::LogError($nodeLogFH, "Failed to chmod 1777 for directory '$RESTOREDIR/$RESFILENAME'\n", undef, $Logger::LogOnlyHandle);
           return 0;
        }
    }
    return 1;
}

=head2 UploadBackUpDB($$$$)

=begin html
Uploads the active DB in isam to 'RestoreDB' directory.

=end html

=cut

sub UploadBackUpDB($$$)
{
    my ($isamCliSession,$toolData,$isamNode) = @_;
    my $possibleBoards = Capability::IsamHiCapEtsiFamily() . "|" .  Capability::IsamHiCapAnsiFamily(); 
    return 1 if ($params->{'BOARD-FAMILY'} !~ /^($possibleBoards)$/ ); 
    my $thisTftpDir = $toolData->{TFTPWORKDIR}."/".$isamNode;
    my $RESFILENAME = "dm_complete.tar";
    my( $name, $dir, $returnValue ) = ("",  "", 0 );
    Logger::LogTrace($nodeLogFH,"Upload active build...\n");
	if(!$params->{NOTFTP}){
        $dir = "$thisTftpDir/RestoreDB";
	    $name = "$toolData->{TFTPDIR}/$thisTftpDir/RestoreDB/$RESFILENAME";
	} else {
        $dir = "$toolData->{FXRDATA}{HOMEDIR}/RestoreDB";
	    $name = "$toolData->{FXRDATA}{HOMEDIR}/RestoreDB/$RESFILENAME";
    }
	my $status = IsamCli::UploadDb($isamCliSession, 'actual-active', $params->{PRIMTFTPIP}, $dir, 1);
    if ( defined $status and $status ) {
         if( -f $name ) {
              Logger::LogTrace($nodeLogFH, "Uploaded DB into '$name'\n");
	          my $dbSize = -s "$name";
	          Logger::LogTrace($nodeLogFH,"OK, transfered $dbSize bytes\n");
              $returnValue = 1;
         } else {
              Logger::LogError($nodeLogFH, "Uploaded DB into '$name' is not exists\n", undef, $Logger::LogOnlyHandle );
         }
	}
    if( not $returnValue ) {
        return 0 if( FamilyCommon::ReportStatus( EC_UPLOAD_DB_FAILED, IsamCli::GetError() ) );
    }
	return 1;  
}

sub PerformVoiceCheck($$) {
    my ($isamCliSession, $dbVersionTargetNum) = @_;
    # Step 1f check whether ISAM is ISAM-V server
    ToolCommon::Log($nodeLogFH, "Checking whether ISAM is ISAM-V server...");
    my $isamIsIsamv = ToolCommon::IsamHasVoiceServer($isamCliSession);
    if (not defined($isamIsIsamv)) {
        return undef if( FamilyCommon::ReportStatus( EC_SHOW_EQPT_FAILED, ToolCommon::GetError() ) );
    } else {
        ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s server\n", $isamIsIsamv ? "is ISAM-V" : "isn't ISAM-V"));
    }

    # Step 1f check whether ISAM has voice
    ToolCommon::Log($nodeLogFH, "Checking whether ISAM has voice boards...");
    my $isamHasV = ToolCommon::IsamHasVoice($isamCliSession);
    if (not defined($isamHasV)) {
        return undef if( FamilyCommon::ReportStatus( EC_SHOW_EQPT_FAILED,ToolCommon::GetError() ) );
    } else {
        ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s voice boards\n", $isamHasV ? "has" : "hasn't"));
    }

    if ( $isamIsIsamv &&  ($dbVersionTargetNum ge "3.500") ) {
      if (not $params->{IVPS_MIG_POSSIBLE}) {
          return undef if( FamilyCommon::ReportStatus( EC_ISAM_HAS_VOICE, "" ) );
      }
    }

    if ( ($isamIsIsamv &&  ($dbVersionTargetNum ge "3.500")) ||
         ($isamHasV &&  ($dbVersionTargetNum ge "3.600")) 
       ){
          if (not -f "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}") {
              return undef if( FamilyCommon::ReportStatus( EC_CDE_NOTFOUND, "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}" ) );
          }
    }
    return 1;
}


#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Check($$$$)

=begin html
This function is called to perform pre-checks of a node of the ISAM family. In case of failure the undefined value is returned.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>userName</td><td>the UNIX username of the user who launched the script</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#

sub Check($$$) {
  my ($newLog, $isam, $userName) = @_;
  my ($swMgntIniSize, $altSwMgntIniSize, $swParSize);
  my ($newMajor, $newMinor, $newSuffix, $newRelNum, $newFullRelNum, $activeRelNum, $fullActiveRelNum);
  my $activeNt;
  my $possible = 0;
  my $reason = "";
  my $isamCliSession = undef;
  my $ret;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $warningLevels = FamilyCommon::GetWarningLevels();
  my $HiCapLTEnabled = 0; 
  my $gponLTExists = 0; 
  my $possibleBoards = "";

  $config = $configData;
  $toolData = $toolConfigData;
  $warnLevels = $warningLevels;
  my $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1];
  FamilyCommon::SetFailedNode($isamNode); # Store for IncrErrorCount
 
  #-- Open CLI Session
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $isamCliSession = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($isamCliSession)) {
     return undef if( FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, ToolCommon::GetError(), undef ) );
  } else {
     ToolCommon::Log($nodeLogFH, "  OK\n");
  }
  FamilyCommon::SetCliSession2Close($isamCliSession);
  ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $isamCliSession->errmode)) if ($params->{DEBUG});

  #-- STEP 0: Acquire transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  #-- STEP : Check if migration interrupt on emergency call feature is enabled
  if ($params->{EMERG_CALL_FEATURE}) {
       my $isActive = 0;
       ToolCommon::Log($nodeLogFH, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
       $isActive = ToolCommon::AlarmRetrieval($nodeLogFH, $isamCliSession, $toolData);
       if ($isActive) {													   ## emergency call alarm is active
           return undef if( FamilyCommon::ReportStatus( EC_EMERGENCY_CALL_ACTIVE, ToolCommon::GetError(), undef ) );
       }
  }

  #-- STEP : Check if automatic DB activation after download is configured
  my $retCode =  IsamCli::CheckDBAutoActivate($isamCliSession);
  if (defined($retCode) && $retCode) {
        return undef if( FamilyCommon::ReportStatus( EC_DB_AUTO_ACTIVATE_ON_DWLD, ToolCommon::GetError(), undef ) );
  }

  #-- STEP 1: NT type information and check Active OSWP is committed
  ToolCommon::Log($nodeLogFH, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($isamCliSession);
  return undef if (!defined($ntInfo));
  $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';

  my $safeIcs = Capability::SafeNtIcs($ntInfo->{NTA}{TYPE});
  if (defined($safeIcs)) {
      if (defined($ntInfo->{NTA}{ICS}) && ($ntInfo->{NTA}{ICS} ne "") ) {
           if ($ntInfo->{NTA}{ICS} < $safeIcs) {
                return undef if( FamilyCommon::ReportStatus( EC_NT_ICS_ISSUE, 
                                 sprintf("NT-A ICS %s too low than expect %d\n", $ntInfo->{NTA}{ICS}, $safeIcs) ) );
           }
      }
      if (defined($ntInfo->{NTB}{ICS}) && ($ntInfo->{NTB}{ICS} ne "") )  {
          if ($ntInfo->{NTB}{ICS} < $safeIcs) {
                return undef if( FamilyCommon::ReportStatus( EC_NT_ICS_ISSUE, 
                                 sprintf("NT-B ICS %s too low than expect %d\n", $ntInfo->{NTB}{ICS}, $safeIcs)) );
          } 
      }
  }

  # Check whether the NT type is present in the SWP 
  my $ntType = uc($ntInfo->{$activeNt}{TYPE}); 
  if (!FamilyCommon::BoardInSwp($nodeLogFH, $ntType)) {
       return undef if( FamilyCommon::ReportStatus( EC_NT_NOT_IN_SWP, FamilyCommon::GetError() ) );
  }
  
  ### No need to perform Board Availability check for 7367 and 7367_SX
  $possibleBoards = Capability::Isam7367_SXFamily() ."|". Capability::Isam7367Family();
  my $ltConfig = IsamCli::GetSlotConfig($isamCliSession);
  if (not defined($ltConfig)) {
      return undef if( FamilyCommon::ReportStatus( EC_GET_LT_FAILED, IsamCli::GetError() ) );
  } elsif ($params->{'BOARD-FAMILY'} !~ /^($possibleBoards)$/) {
      my %ltHandled = ();
      my $swMissing = "";
      foreach my $ltNum (sort keys %$ltConfig) {
         next if ( defined($ltHandled{$ltConfig->{$ltNum}}) );
         next if ( FamilyCommon::BoardInSwp($nodeLogFH, uc($ltConfig->{$ltNum})));
         $swMissing .= sprintf(" %s", uc($ltConfig->{$ltNum}));
         $ltHandled{$ltConfig->{$ltNum}} = 1;
      }
      if ($swMissing ne "") {
         return undef if( FamilyCommon::ReportStatus( EC_LT_NOT_SUPPORTED, $swMissing ) );
      }
  }

  my $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}), undef,
					 $toolData->{RETRIES});
  if (! defined($oswpInfo)) {
      return undef if( FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, ToolCommon::GetError() ) );
  } 

  my $dbVersionActive = $oswpInfo->{'ACTIVE'}{'DBVERSION'};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveNum = substr($dbVersionActive,1);
  my $dbVersionTargetNum = substr($dbVersionTarget,1);

  if ($dbVersionActive ne $dbVersionTarget) {
      #-- Verify migration path
     my ($restricted, $restricted_reason) = Capability::IsMigrateRestricted( $ntInfo->{$activeNt}{TYPE},
                                                                  $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                               basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
     if (not $restricted) {
         Logger::LogTrace($nodeLogFH, "Migrate Restrictions... NONE\n");
     } else {
         FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                           $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                           basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
         if ($params->{CONTINUE}) {
           return undef;
         } else {
           FamilyCommon::ToolExit(UPG_IMPOSSIBLE_CODE);
         }
     }
      Logger::LogNotify($nodeLogFH, "Checking whether migration restrictions apply...\n");
      ($possible, $reason) = CapabilityReEngr::MigrationPossible( $ntInfo->{$activeNt}{TYPE}, $dbVersionActive, $dbVersionTarget);
      if (not $possible) {
          return undef if( FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                             $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason )) );
      }
  } else {
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntInfo->{$activeNt}{TYPE}, $dbVersionActive, $dbVersionTarget,
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          return undef if( FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                             $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason )) );
      }
  }

  ToolCommon::Log($nodeLogFH, "Checking Target OSWP already present...");
  if (basename($oswpInfo->{ACTIVE}{NAME}) eq (basename($params->{OSWP})) || # -- 1.2.3
      (basename($oswpInfo->{PASSIVE}{NAME}) eq basename($params->{OSWP}) && $oswpInfo->{PASSIVE}{AVAIL} eq 'enabled')) { #-- 1.2.3
          return undef if( FamilyCommon::ReportStatus( EC_NEW_SW_PRESENT, sprintf(": Target OSWP '%s' already exists",  basename($params->{OSWP}) )) );
  }
  my $possibleBoardFamily = Capability::IsamEtsiFamily() ."|". Capability::IsamEtsiExtendedFamily();
  $possibleBoardFamily   .= "|". Capability::IsamFamily() ."|". Capability::IsamAnsiFamily();
  if ($params->{'BOARD-FAMILY'} =~ /^($possibleBoardFamily)$/ ) {
     my $shubStatus = ShubChecks($isamCliSession, $snmpSession, $ntInfo, $dbVersionActiveNum);
     return undef if (!defined $shubStatus );
  }

  my $voiceChecks = PerformVoiceCheck($isamCliSession, $dbVersionTarget);
  return undef if (not defined $voiceChecks );
  
   if( (($dbVersionActiveNum ge "3.600") and ($dbVersionActiveNum lt "4.200"))  and
       (($dbVersionTargetNum ge "4.200") and ($dbVersionTargetNum lt "4.202"))  ) {
           $possibleBoardFamily =  Capability::Isam7367Family();
           if ($params->{'BOARD-FAMILY'} !~ /^($possibleBoardFamily)$/ ) {
               ToolCommon::Log($nodeLogFH, "Check Vmac Unique Host-id parameter limitations...");
               if (not ToolCommon::CheckVlanUniqHostId($isamCliSession)) {
                      return undef if( FamilyCommon::ReportStatus( EC_VMAC_HOST_ID_LIMITS_EXCEED, ToolCommon::GetError() ) );
               }
           }
   }


   $possibleBoardFamily = Capability::IsamHiCapEtsiFamily() . "|" .  Capability::IsamHiCapAnsiFamily(); 
   if ($params->{'BOARD-FAMILY'} =~ /^($possibleBoardFamily)$/ ) {
       ### ISSU Check from 5.800
       if( $dbVersionActiveNum ge "5.800" ) {
           if( ToolCommon::CheckISSU( $isamCliSession ) ) {
              return undef if( FamilyCommon::ReportStatus( EC_SETUP_ISSU_FAILED, IsamCli::GetError() ) );
           }
       }
       ### Create respective directory and upload DB contents to that directory
       if( not CreateRestoreDB($isamNode,$toolData,$config) ) {
           return undef if( FamilyCommon::ReportStatus(EC_CREATE_RESTOREDIR_FAILED, "Creating RestoreDB failed...", undef ) );
       }
 
       if( not UploadBackUpDB($isamCliSession,$toolData,$isamNode) ) {
           return undef;
       }
   }

   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

   IsamCli::AllowAlarms($isamCliSession);
   FamilyCommon::CloseCliSession2Close();
}

sub ShubChecks($$$) {
    my ($isamCliSession, $snmpSession, $ntInfo, $dbVersionActiveNum) = @_;

   my $activeConfig = IsamCli::GetLanxActiveOSWP($isamCliSession, "config-file");
   if (not defined($activeConfig) || ($activeConfig eq "")) {
        return undef if( FamilyCommon::ReportStatus( EC_GET_SHUB_CONF_FAILED, IsamCli::GetError() ) );
   }

   my $swParSize = ToolCommon::GetSwParSize();
   if( not defined $swParSize or $swParSize < 0 ) {
      return undef if( FamilyCommon::ReportStatus( EC_SW_RED_FAILED, SNMPComm::GetError(), undef ) );
   } elsif ($swParSize < MINIMALSWPARSIZE)  {
        return undef if( FamilyCommon::ReportStatus( EC_PAR_SIZE_TOO_SMALL, "(SW partition size: $swParSize)" ) );
   }

   if( $dbVersionActiveNum ge "3.400" ) {
     # Possible issue with the number of packages present in a MC source: from R3.4 onwards only 20 are possible...
     my $activeRelNum = $dbVersionActiveNum;
     $activeRelNum =~ s/\.//g;
     ToolCommon::Log($nodeLogFH, "Verifying number of packages in the IGMP multicast sources...");
     if (not ToolCommon::CheckMcPackages($snmpSession, $params->{DEBUG}, $activeRelNum)) {
          return undef if( FamilyCommon::ReportStatus( EC_TOO_MANY_MC_SRC_PKG, ToolCommon::GetError() ) );
     }
   }

   if( $dbVersionActiveNum ge "3.300" ) {

       my $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';
       my $profileMode = IsamCli::GetProfileSlotNumberMode($isamCliSession, $params->{USERNAME});
       if (not defined($profileMode)) {
           return undef if( FamilyCommon::ReportStatus( EC_GET_CLIMODE_FAILED, IsamCli::GetError() ) );
       } else {
         Logger::LogDebug($nodeLogFH, "Profile mode '$profileMode' detected for NT type '$ntInfo->{$activeNt}{TYPE}'\n");
       }
 
       ToolCommon::Log($nodeLogFH, "Check MAC address limitations...");
       if (not ToolCommon::CheckUnicastMacLimits($snmpSession, uc($profileMode), uc($ntInfo->{$activeNt}{TYPE}), $params->{DEBUG})) {
           return undef if( FamilyCommon::ReportStatus( EC_MAC_LIMITS_EXCEEDED, ToolCommon::GetError() ) );
       }
   }
   return 1;

}

=head2 CheckEnvironment($$$$)

=begin html
Check the migration environment.  In case unknown offline migration tools are detected (based on their version number) or files not being executable
the script will abort and not start any activity.

=end html

=cut

sub CheckEnvironment($$$) {
  my ($logFile, $config, $installDir) = @_;
  my $toolConfigData = FamilyCommon::GetToolData();
  my ($label, $envOk ) = ("", 1 );


  $params->{IVPS_MIG_POSSIBLE} = 0;
  my $oflVersInfo = Utils::DetermineOFLMTVersions($params->{DEBUG}, $toolConfigData->{NTDBTOOL}, $toolConfigData->{LANXDBTOOLDIR},
						  $toolConfigData->{LANXDBTOOL},
						  $toolConfigData->{IVPSDBTOOL}, $config, $installDir);
  if ( $oflVersInfo->{RET}) {
    Logger::LogError($nodeLogFH, sprintf("Determining OFLMT Version info failed: %s\n", Utils::GetError()));
    $envOk = 0;
    exit $oflVersInfo->{RET};
    return $oflVersInfo;
  }


  $label = "NT OFLMT Version";
  if ($oflVersInfo->{NT} ne "") {
	  if ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {		  
	      $params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	      $params->{NTOFLMTBLDNUM} = $4;
      } elsif ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+).*/) {
	      $params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
      } else {
          $params->{NTOFLMTVERS} = 0;
          if($oflVersInfo->{NT} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:.*/) {
	          $NTOFLMTVERSION = $1 * 100 + ($2 * 10) + $3; 
          } elsif ($oflVersInfo->{NT} =~ /.*V(\w)+\s+Build:\s+(\d+\.\d+)/) { 
	          $params->{NTOFLMTBLDNUM} = $2;
              if($params->{NTOFLMTBLDNUM} =~ /(\d{4})\.(\d+)/) {
	                $params->{NTOFLMTBLD} = $1 * 1000 + $2;
              } 
          } else {
              Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "Not in expected format to obtain"));
          }
      }
    } else {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "No information could be obtained"));
    }
    if( defined $params->{NTOFLMTVERS} and $params->{NTOFLMTVERS} >  0 ) {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n",
                                          $label, Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
	    			                      Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion(Capability::IsamEtsiFamily()))
                                     ), $Logger::LogOnlyDebug);
        if ( $params->{NTOFLMTVERS} > Capability::HighestNtOflmtVersion(Capability::IsamEtsiFamily()) ) {
              Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
                                                   $label, 
	            	                               Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
	            	                               Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion(Capability::IsamEtsiFamily()))
                                            ));
              $envOk = 0;
              exit NT_OFLMT_VERSION_2_HIGH_CODE;
        }
    } elsif ( $NTOFLMTVERSION > 0 ) {
	    Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, Capability::NtOflmtVersionString($NTOFLMTVERSION)));
    }
    if (defined($params->{NTOFLMTBLDNUM})) {
         $label = "NT OFLMT Build";

         Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n", 
                                       $label, Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
	     		                      Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild(Capability::IsamEtsiFamily()))
                                     ), $Logger::LogOnlyDebug) if (defined($params->{NTOFLMTBLD}));
         if (defined($params->{NTOFLMTBLD}) && ($params->{NTOFLMTBLD} > Capability::HighestNtOflmtBuild(Capability::IsamEtsiFamily()))) {
             Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
                                                $label, 
               	                               Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
               	                               Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild(Capability::IsamEtsiFamily()))
                                           ));
             $envOk = 0;
             exit NT_OFLMT_VERSION_2_HIGH_CODE;
         }
    }
    $label = "xVPS OFLMT Exec";
	if ( (defined $toolConfigData->{IVPSINVENTORY}) and ( not -x $toolConfigData->{IVPSINVENTORY}) ) {
      Logger::LogWarn($nodeLogFH, sprintf("%-20s : %s\n", $label, "$toolConfigData->{IVPSINVENTORY} not found/executable"), undef, $Logger::LogOnlyHandle);
    }
    $label = "xVPS OFLMT Version";
    if ($oflVersInfo->{IVPS} ne "") {
        $params->{IVPS_MIG_POSSIBLE} = 1;
        if ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {
          $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
          $params->{IVPSOFLMTBLDNUM} = $4;
        } elsif ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+).*/) {
          $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
        } else {
          $params->{IVPSOFLMTVERS} = 0;
          Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "No information could be obtained"));
        }
    } else {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "No information could be obtained"));
    }
    if( defined $params->{IVPSOFLMTVERS} and $params->{IVPSOFLMTVERS} > 0 ) {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n",
                                          $label, Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
	    			                      Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion(Capability::IsamEtsiFamily()))
                                     ), $Logger::LogOnlyDebug);
        if ($params->{IVPSOFLMTVERS} > Capability::HighestXvpsOflmtVersion(Capability::IsamEtsiNttFamily())) {
             Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
	    		      Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
	    		      Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion(Capability::IsamEtsiNttFamily()))) );
             $envOk = 0;
             exit IVPS_OFLMT_VERSION_2_HIGH_CODE;
        }
    }
    $label = "SHUB OFLMT Version";
    my $possibleBoardFamily = Capability::IsamEtsiFamily() ."|". Capability::IsamEtsiExtendedFamily();
    $possibleBoardFamily   .= "|". Capability::IsamFamily() ."|". Capability::IsamAnsiFamily();
    return $oflVersInfo if ($params->{'BOARD-FAMILY'} !~ /^($possibleBoardFamily)$/ );
    if ($oflVersInfo->{SHUB} =~ /.*(\d+)\.(\d+).*/) {
      $params->{SHUBOFLMTVERS} = ($1 * 100) + $2;
    } else {
       Logger::LogError($nodeLogFH, "No SHUB OFLMT version could be obtained...\n" );
       $envOk = 0;
       exit NO_SHUB_OFLMT_VER_CODE;
    }
    Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n",
                                      $label, $oflVersInfo->{SHUB},
                                      Capability::ShubOflmtVersionString(Capability::HighestShubOflmtVersion(Capability::IsamEtsiNttFamily()))
                                 ), $Logger::LogOnlyDebug);
    if ( $params->{SHUBOFLMTVERS} > Capability::HighestShubOflmtVersion(Capability::IsamEtsiNttFamily())) {
             Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
			      $oflVersInfo->{SHUB},
			      Capability::ShubOflmtVersionString(Capability::HighestShubOflmtVersion(Capability::IsamEtsiNttFamily()))) );
      $envOk = 0;
      exit SHUB_OFLMT_VERSION_2_HIGH_CODE;
    }
    ### Starting shub version 1.67 only Lanx Checksum exe CalcIpCheckSum_BE is available for sparc platform. 
    if (($Config{archname} =~ /sun4/) && ($params->{SHUBOFLMTVERS} < 167)) { 
       $toolConfigData->{LANXIPCKSUM} = "CalcIpCheckSum"; 
    }
    if (! -x "$installDir/$toolConfigData->{LANXDBTOOLDIR}/$toolConfigData->{LANXIPCKSUM}") {
      $envOk = 0;
      Logger::LogError($nodeLogFH, "$installDir/$toolConfigData->{LANXDBTOOLDIR}/$toolConfigData->{LANXIPCKSUM} is not executable\n" );
      exit CALC_IP_CKSUM_NO_EXEC_CODE;
    }
    return $oflVersInfo;
}
1;
