package PreCheckIsam1540;

=head1 ISAM 1540-specific pre-check module

=begin html
This library contains the functions dedicated to perform prechecks for a 1540 ETSI ISAM.

=end html

=cut

#==============================================================================
#-- File:         PreCheckIsam1540.pm
#-- Date:         01.09.2008
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Supporting library carrying out the ISAM NE-family prechecks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2009/05/11 : First implementation (based on copy of PreCheckIsam7302Etsi.pm)
#--   1.0.1 - 2009/07/14 : ANTmt67832 - the IVPS inventory script is only required for migrations till R3.5
#--   3.0.1 - 2011/03/29 : ALU01031457 - Support for R4220 & R4230
#--   3.0.2 - 2011/07/26 : Check for oflmt version info made more strict
#--   3.0.3 - 2012/05/31 : Ensure backward compatibility for Lanx Checksum tool CalcIpCheckSum 
#--   3.0.4 - 2016/03/16 : ALU02190558
#--   3.0.5 - 2016/07/15 : ALU02237966
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.1.1  - 2018/08/06 : ALU02460863 & ALU02471697


=head1 VERSION

=begin html
Current version: "PreCheckIsam1540.pm - 3.1.1 -- 2018/08/06"

=end html

=cut

my $VERSION = "PreCheckIsam1540.pm - 3.1.1 -- 2018/08/06";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
require 5.001;

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;

#==============================================================================
#== Find the lib dir
use Cwd;
use Config; 
use File::Basename;

BEGIN {
  my $dir = dirname($0);
  my $path;
  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif( $dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib");
  unshift(@INC, "$path/lib-perl");
}
#==============================================================================
use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Crypt::Lite;
use Utils;
require Octopus;
use Capability;
use PbmtConfig;
require FamilyCommon;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :PRCK);
use version();
use vars qw(%version);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Check($$$);
sub CheckEnvironment($$$);
sub PrintToolVersions($$);
sub ResetGlobals();
#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;

my $SHUBSWCHECK = "shubSwCheck.sh";

#==============================================================================
#== Vars
#==============================================================================
my $warnLogFile;
my %config;
my $warnLevels;

my $config;
my $toolData;
my $NTOFLMTVERSION = 0;

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

sub ResetGlobals() {
   $NTOFLMTVERSION = 0;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Check($$$$)

=begin html
This function is called to perform pre-checks of a node of the ISAM family. In case of failure the undefined value is returned.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>userName</td><td>the UNIX username of the user who launched the script</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#

sub Check($$$) {
  my ($newLog, $isam, $userName) = @_;
  my ($swMgntIniSize, $altSwMgntIniSize, $swParSize);
  my ($newMajor, $newMinor, $newSuffix, $newRelNum, $newFullRelNum, $activeRelNum, $fullActiveRelNum);
  my ($new1540RelNum, $newFull1540RelNum, $active1540RelNum, $fullActive1540RelNum);
  my $activeNt;
  my $possible = 0;
  my $reason = "";
  my $cli = undef;
  my $ret;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $warningLevels = FamilyCommon::GetWarningLevels();

  $config = $configData;
  $toolData = $toolConfigData;
  $warnLevels = $warningLevels;
  #-- Open CLI Session
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($cli)) {
    if (! ErrorCodes::WarningLevelIsIgnore($warnLevels->{&EC_OPENCLI_FAILED})) {
      if (ErrorCodes::WarningLevelIsError($warnLevels->{&EC_OPENCLI_FAILED})) {
	FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				     EC_OPENCLI_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				     EC_OPENCLI_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     undef);
      }
    } else {
      ToolCommon::Log($nodeLogFH, sprintf("   Open CLI failed: %s\n\t\tCheck set to be ignored\n",
					  ToolCommon::GetError()));
    }
  } else {
    ToolCommon::Log($nodeLogFH, "  OK\n");
  }
  FamilyCommon::SetCliSession2Close($cli);
  ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});

  #-- STEP 0: Acquire transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");
  ToolCommon::Log($nodeLogFH, "Acquire transaction...");


  #-- STEP : Check if migration interrupt on emergency call feature is enabled
  if ($params->{EMERG_CALL_FEATURE}) {
    my $isActive = 0;
    ToolCommon::Log($nodeLogFH, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
    $isActive = ToolCommon::AlarmRetrieval($nodeLogFH, $cli, $toolData);
    if ($isActive) {													   ## emergency call alarm is active
       return undef if( FamilyCommon::ReportStatus( EC_EMERGENCY_CALL_ACTIVE, "Emergency call max retries reached") );
    }
  }

  #-- STEP : Check if automatic DB activation after download is configured
  my $retCode =  IsamCli::CheckDBAutoActivate($cli);
  if (defined($retCode) && $retCode) {
     return undef if( FamilyCommon::ReportStatus( EC_DB_AUTO_ACTIVATE_ON_DWLD, FamilyCommon::GetError() ) );
  }


  #-- STEP 1: NT type information and check Active OSWP is committed
  #-- Get NT type information
  ToolCommon::Log($nodeLogFH, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($cli);
  return 0 if (!defined($ntInfo));
  $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';

  my $safeIcs = Capability::SafeNtIcs($ntInfo->{NTA}{TYPE});
  if (defined($safeIcs)) {
      if (defined($ntInfo->{NTA}{ICS}) && ($ntInfo->{NTA}{ICS} ne "") ) {
           if ($ntInfo->{NTA}{ICS} < $safeIcs) {
                return undef if( FamilyCommon::ReportStatus( EC_NT_ICS_ISSUE, 
                                 sprintf("NT-A ICS %s too low than expect %d\n", $ntInfo->{NTA}{ICS}, $safeIcs) ) );
           }
      }
      if (defined($ntInfo->{NTB}{ICS}) && ($ntInfo->{NTB}{ICS} ne "") )  {
          if ($ntInfo->{NTB}{ICS} < $safeIcs) {
                return undef if( FamilyCommon::ReportStatus( EC_NT_ICS_ISSUE, 
                                 sprintf("NT-B ICS %s too low than expect %d\n", $ntInfo->{NTB}{ICS}, $safeIcs)) );
          } 
      }
  }

  my $ltConfig = IsamCli::GetSlotConfig($cli);
  if (!defined($ltConfig)) {
     return undef if( FamilyCommon::ReportStatus( EC_GET_LT_FAILED, IsamCli::GetError() ) );
  } else {
    my %ltHandled = ();
    my $swMissing = "";
    foreach my $ltNum (sort keys %$ltConfig) {
      if (! defined($ltHandled{$ltConfig->{$ltNum}})) {
	      if (! FamilyCommon::BoardInSwp($nodeLogFH, uc($ltConfig->{$ltNum}))) {
	        $swMissing .= sprintf(" %s", uc($ltConfig->{$ltNum}));
	      }
	      $ltHandled{$ltConfig->{$ltNum}} = 1;
      }
    }
    if ($swMissing ne "") {
       return undef if( FamilyCommon::ReportStatus( EC_LT_NOT_SUPPORTED, $swMissing ) );
    }
  }

  my $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}), undef,
					 $toolData->{RETRIES});
  if (not defined($oswpInfo)) {
       return undef if( FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, ToolCommon::GetError() ) );
  }

  if (Capability::ReleaseCompare(Capability::IsamLS2IsamRelease(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
											   $oswpInfo->{'ACTIVE'}{'MINOR'})),
				 Capability::ConvertRelease(2,2), REGEXP, 0) < 0) {
       my ($shubSwName, $shubSwSize);
       if (uc($ntInfo->{NTA}{TYPE}) eq 'ECNT-A') {  # 1.2.4.2
         # $shubSwSize only used for releases in the context of ECNT-A when directly migrating from R2.0/R2.1 to R2.2
         # This does not apply for ECNT-C.
         $shubSwName = Capability::FindShubSwName($params->{SWPNAME}, REGEXP, $toolData->{TFTPDIR}, $toolData->{SHUBSWPATH},
	   				  $toolData->{SHUBSWNAME}, Capability::IsamLSFamily());
         $shubSwSize = -s $toolData->{TFTPDIR} . "/" . $shubSwName;
       } else {                                                                                         # 1.2.4.2
         $shubSwSize = 0;                                                                               # 1.2.4.2
       }                                                                                                # 1.2.4.2
       ToolCommon::Log($nodeLogFH, "Expected shub SW size: $shubSwSize\n") if $params->{DEBUG};
       $ret = Octopus::ShubSwPresent($params->{DEBUG}, $dir, $isam, $SHUBSWCHECK, $shubSwSize, $dirName);
       if ($ret <= 0) {
          return undef if( FamilyCommon::ReportStatus( EC_GET_SHUB_SWSIZE_FAILED, Octopus::GetError() ) );
       }
  }
  my $dbVersionActive = $oswpInfo->{'ACTIVE'}{'DBVERSION'};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveNum = substr($dbVersionActive,1);
  my $dbVersionTargetNum = substr($dbVersionTarget,1);
  
  $fullActiveRelNum = Capability::IsamLS2IsamRelease(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
										$oswpInfo->{'ACTIVE'}{'MINOR'},
										$oswpInfo->{'ACTIVE'}{'SUFFIX'}));
  $fullActive1540RelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						     $oswpInfo->{'ACTIVE'}{'MINOR'},
						     $oswpInfo->{'ACTIVE'}{'SUFFIX'});
  $activeRelNum = Capability::IsamLS2IsamRelease(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
									    $oswpInfo->{'ACTIVE'}{'MINOR'}));
  $active1540RelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'});

  my $newSwpIndex = $oswpInfo->{PASSIVE}{INDEX};
  my $new = basename($params->{SWPNAME});
  my $regExp = REGEXP;
  if ($new =~ /$regExp/) {   # -- Start 1.0.4
    $newMajor = $1;
    $newMinor = $2;
    $newSuffix = $3;
    $newFullRelNum = Capability::ConvertRelease($newMajor, $newMinor, $newSuffix); # -- 1.2.0
    $newFull1540RelNum = Capability::IsamLS2IsamRelease(Capability::ConvertRelease($newMajor, $newMinor, $newSuffix)); # -- 1.2.0
    $newRelNum = Capability::ConvertRelease($newMajor, $newMinor);
    $new1540RelNum = Capability::IsamLS2IsamRelease(Capability::ConvertRelease($newMajor, $newMinor));
  }                          # -- End 1.0.4

  if ($dbVersionActive ne $dbVersionTarget) {
      #-- Verify migration path
     my ($restricted, $restricted_reason) = Capability::IsMigrateRestricted( $ntInfo->{$activeNt}{TYPE},
                                                                  $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                               basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
     if (not $restricted) {
         Logger::LogTrace($nodeLogFH, "Migrate Restrictions... NONE\n");
     } else {
         FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                           $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                           basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
         if ($params->{CONTINUE}) {
           return undef;
         } else {
           FamilyCommon::ToolExit(UPG_IMPOSSIBLE_CODE);
         }
     }
      Logger::LogNotify($nodeLogFH, "Checking whether migration restrictions apply...\n");
      ($possible, $reason) = CapabilityReEngr::MigrationPossible( $ntInfo->{$activeNt}{TYPE}, $dbVersionActive, $dbVersionTarget);
      if (not $possible) {
          return undef if( FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                             $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason )) );
      }
  } else {
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntInfo->{$activeNt}{TYPE}, $dbVersionActive, $dbVersionTarget,
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          return undef if( FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                             $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason )) );
      }
  }

  ToolCommon::Log($nodeLogFH, "Checking new software already present...");
  if (basename($oswpInfo->{ACTIVE}{NAME}) eq (basename($params->{OSWP})) || # -- 1.2.3
      (basename($oswpInfo->{PASSIVE}{NAME}) eq basename($params->{OSWP}) && $oswpInfo->{PASSIVE}{AVAIL} eq 'enabled')) { #-- 1.2.3
       return undef if( FamilyCommon::ReportStatus( EC_NEW_SW_PRESENT, "Already present" ) );
  }

  ToolCommon::Log($nodeLogFH, sprintf("ISAM running R%d.%d, get active osp...", $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{MINOR}));
  my $showShubConf = Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(1,1), REGEXP, 0) == 0
    ? "" : "config-file";
  my $activeConfig = IsamCli::GetLanxActiveOSWP($cli, $showShubConf);
  
  if (!defined($activeConfig) || ($activeConfig eq "")) {
       return undef if( FamilyCommon::ReportStatus( EC_GET_SHUB_CONF_FAILED, IsamCli::GetError() ) );
  }
  if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) == 0) &&
      ($activeConfig ne "tftp")) {
       return undef if( FamilyCommon::ReportStatus( EC_SHUB_NO_TFTP_BOOT, "Current config $activeConfig)" ) );
  }

  #-- STEP 1d get SAN disk partitioning size                           --- 1.1.7 BEGIN
  #-- SW download process has reduced active package so just print warning
  if (!defined($swMgntIniSize = Capability::SwMgntIniSize($ntInfo->{$activeNt}{TYPE}, $activeRelNum))) {
       return undef if( FamilyCommon::ReportStatus( EC_NO_NT_CAP, sprintf("Failed to get SwMgntIni size for NT type '%s' and release '%s'... exiting\n",
                                                   $ntInfo->{$activeNt}{TYPE}, $activeRelNum)) );
  }
  if (!defined($altSwMgntIniSize = Capability::AltSwMgntIniSize($ntInfo->{$activeNt}{TYPE}, $activeRelNum))) {
       return undef if( FamilyCommon::ReportStatus( EC_NO_ALT_NT_CAP, sprintf("Failed to get (alternative) SwMgntIni size for NT type '%s' and release '%s'... exiting\n",
                                                   $ntInfo->{$activeNt}{TYPE}, $activeRelNum)) );
  }

  if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) < 0) {
    # Running R2.2 potential rollback issue so check it.
    if (!FamilyCommon::DetermineSwMgntSize($nodeLogFH, $params->{DEBUG}, $dir, $isam, $swMgntIniSize, $altSwMgntIniSize,
					   $oswpInfo->{'PASSIVE'}{'INDEX'},
					   $params->{OSWP}, $dirName)) {
       return undef if( FamilyCommon::ReportStatus( EC_SW_MGNT_SIZE_NOK,FamilyCommon::GetError() ) ); 
    }
  }

  $swParSize = ToolCommon::GetSwParSize();
  if( not defined $swParSize or $swParSize < 0 ) {
     return undef if( FamilyCommon::ReportStatus( EC_SW_RED_FAILED, ToolCommon::GetError(), undef ) );
  }
  if ($swParSize < MINIMALSWPARSIZE)  {
       return undef if( FamilyCommon::ReportStatus( EC_PAR_SIZE_TOO_SMALL,"(SW partition size: $swParSize)" ) ); 
  }

  # Step 1f check whether ISAM is ISAM-V server
  ToolCommon::Log($nodeLogFH, "Checking whether ISAM is ISAM-V server...");
  my $isamIsIsamv = ToolCommon::IsamHasVoiceServer($cli);
  if (!defined($isamIsIsamv)) {
       return undef if( FamilyCommon::ReportStatus( EC_SHOW_EQPT_FAILED, ToolCommon::GetError() ) ); 
  } else {
    ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s server\n", $isamIsIsamv ? "is ISAM-V" : "isn't ISAM-V"));
  }

  # Step 1f check whether ISAM has voice
  ToolCommon::Log($nodeLogFH, "Checking whether ISAM has voice boards...");
  my $isamHasV = ToolCommon::IsamHasVoice($cli);
  if (!defined($isamIsIsamv)) {
       return undef if( FamilyCommon::ReportStatus( EC_SHOW_EQPT_FAILED, ToolCommon::GetError() ) ); 
  } else {
    ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s voice boards\n", $isamHasV ? "has" : "hasn't"));
  }

  if ($isamIsIsamv && (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) >= 0)) {
    if ((! $params->{IVPS_MIG_POSSIBLE}) && (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0)) {
       return undef if( FamilyCommon::ReportStatus( EC_ISAM_HAS_VOICE, "" ) ); 
    } else {
      if (! -f "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}") {
          return undef if( FamilyCommon::ReportStatus( EC_CDE_NOTFOUND, "ISAM is voice server but $toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE} not found" ) ); 
      }
    }
  }

  if ($isamHasV &&  (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 6), REGEXP, 0) >= 0)) {
    if (! -f "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}") {
          return undef if( FamilyCommon::ReportStatus( EC_CDE_NOTFOUND, "ISAM is voice server but $toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE} not found" ) ); 
    }
  }

  my $relCmp = Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 4), REGEXP, 0);
  if ($relCmp >= 0) {
    # Possible issue with the number of packages present in a MC source: from R3.4 onwards only 20 are possible...
    ToolCommon::Log($nodeLogFH, "Verifying number of packages in the IGMP multicast sources...");
    if (! ToolCommon::CheckMcPackages($snmpSession, $params->{DEBUG}, $activeRelNum)) {
          return undef if( FamilyCommon::ReportStatus( EC_TOO_MANY_MC_SRC_PKG, ToolCommon::GetError() ) );
    }
  }
  if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 3), REGEXP, 0) == 0) {
    my $profileMode = IsamCli::GetProfileSlotNumberMode($cli, $params->{USERNAME});
    if (! defined($profileMode)) {
          return undef if( FamilyCommon::ReportStatus( EC_GET_CLIMODE_FAILED, IsamCli::GetError() ) );
    } else {
      ToolCommon::Log($nodeLogFH, "Profile mode '$profileMode' detected for NT type '$ntInfo->{$activeNt}{TYPE}'\n") if $params->{DEBUG};
    }

    ToolCommon::Log($nodeLogFH, "Check MAC address limitations...");
    if (!ToolCommon::CheckUnicastMacLimits($snmpSession, uc($profileMode), uc($ntInfo->{$activeNt}{TYPE}), $params->{DEBUG})) {
          return undef if( FamilyCommon::ReportStatus( EC_MAC_LIMITS_EXCEEDED, "" ) );
    }
  }

   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  IsamCli::AllowAlarms($cli);
  FamilyCommon::CloseCliSession2Close();
}

=head2 CheckEnvironment($$$$)

=begin html
Check the migration environment.  In case unknown offline migration tools are detected (based on their version number) or files not being executable
the script will abort and not start any activity.

=end html

=cut
sub CheckEnvironment($$$) {
  my ($logFile, $config, $installDir) = @_;
  my $toolConfigData = FamilyCommon::GetToolData();
  my ($label, $envOk ) = ("", 1 );

  $params->{IVPS_MIG_POSSIBLE} = 0;
  my $oflVersInfo = Utils::DetermineOFLMTVersions($params->{DEBUG}, $toolConfigData->{NTDBTOOL}, $toolConfigData->{LANXDBTOOLDIR},
						  $toolConfigData->{LANXDBTOOL},
						  $toolConfigData->{IVPSDBTOOL}, $config, $installDir);
  if ( $oflVersInfo->{RET}) {
    Logger::LogError($nodeLogFH, sprintf("Determining OFLMT Version info failed: %s\n", Utils::GetError()));
    $envOk = 0;
    exit $oflVersInfo->{RET};
    return $oflVersInfo;
  }


  $label = "NT OFLMT Version";
  if ($oflVersInfo->{NT} ne "") {
	  if ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {		  
	      $params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	      $params->{NTOFLMTBLDNUM} = $4;
      } elsif ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+).*/) {
	      $params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
      } else {
          $params->{NTOFLMTVERS} = 0;
          if($oflVersInfo->{NT} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:.*/) {
	          $NTOFLMTVERSION = $1 * 100 + ($2 * 10) + $3; 
          } elsif ($oflVersInfo->{NT} =~ /.*V(\w)+\s+Build:\s+(\d+\.\d+)/) { 
	          $params->{NTOFLMTBLDNUM} = $2;
              if($params->{NTOFLMTBLDNUM} =~ /(\d{4})\.(\d+)/) {
	                $params->{NTOFLMTBLD} = $1 * 1000 + $2;
              } 
          } else {
              Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "Not in expected format to obtain"));
          }
      }
    } else {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "No information could be obtained"));
    }
    if( defined $params->{NTOFLMTVERS} and $params->{NTOFLMTVERS} >  0 ) {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n",
                                          $label, Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
	    			                      Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion(Capability::IsamEtsiFamily()))
                                     ), $Logger::LogOnlyDebug);
        if ( $params->{NTOFLMTVERS} > Capability::HighestNtOflmtVersion(Capability::IsamEtsiFamily()) ) {
              Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
                                                   $label, 
	            	                               Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
	            	                               Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion(Capability::IsamEtsiFamily()))
                                            ));
              $envOk = 0;
              exit NT_OFLMT_VERSION_2_HIGH_CODE;
        }
    } elsif ( $NTOFLMTVERSION > 0 ) {
	    Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, Capability::NtOflmtVersionString($NTOFLMTVERSION)));
    }
    if (defined($params->{NTOFLMTBLDNUM})) {
         $label = "NT OFLMT Build";

         Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n", 
                                       $label, Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
	     		                      Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild(Capability::IsamEtsiFamily()))
                                     ), $Logger::LogOnlyDebug) if (defined($params->{NTOFLMTBLD}));
         if (defined($params->{NTOFLMTBLD}) && ($params->{NTOFLMTBLD} > Capability::HighestNtOflmtBuild(Capability::IsamEtsiFamily()))) {
             Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
                                                $label, 
               	                               Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
               	                               Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild(Capability::IsamEtsiFamily()))
                                           ));
             $envOk = 0;
             exit NT_OFLMT_VERSION_2_HIGH_CODE;
         }
    }
    $label = "xVPS OFLMT Exec";
	if ( (defined $toolConfigData->{IVPSINVENTORY}) and ( not -x $toolConfigData->{IVPSINVENTORY}) ) {
      Logger::LogWarn($nodeLogFH, sprintf("%-20s : %s\n", $label, "$toolConfigData->{IVPSINVENTORY} not found/executable"), undef, $Logger::LogOnlyHandle);
    }
    $label = "xVPS OFLMT Version";
    if ($oflVersInfo->{IVPS} ne "") {
        $params->{IVPS_MIG_POSSIBLE} = 1;
        if ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {
          $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
          $params->{IVPSOFLMTBLDNUM} = $4;
        } elsif ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+).*/) {
          $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
        } else {
          $params->{IVPSOFLMTVERS} = 0;
          Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "No information could be obtained"));
        }
    } else {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s\n", $label, "No information could be obtained"));
    }
    if( defined $params->{IVPSOFLMTVERS} and $params->{IVPSOFLMTVERS} > 0 ) {
        Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n",
                                          $label, Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
	    			                      Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion(Capability::IsamEtsiFamily()))
                                     ), $Logger::LogOnlyDebug);
        if ($params->{IVPSOFLMTVERS} > Capability::HighestXvpsOflmtVersion(Capability::IsamEtsiNttFamily())) {
             Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
	    		      Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
	    		      Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion(Capability::IsamEtsiNttFamily()))) );
             $envOk = 0;
             exit IVPS_OFLMT_VERSION_2_HIGH_CODE;
        }
    }
    $label = "SHUB OFLMT Version";
    my $possibleBoardFamily = Capability::IsamEtsiFamily() ."|". Capability::IsamEtsiExtendedFamily();
    $possibleBoardFamily   .= "|". Capability::IsamFamily() ."|". Capability::IsamAnsiFamily();
    return $oflVersInfo if ($params->{'BOARD-FAMILY'} !~ /^($possibleBoardFamily)$/ );
    if ($oflVersInfo->{SHUB} =~ /.*(\d+)\.(\d+).*/) {
      $params->{SHUBOFLMTVERS} = ($1 * 100) + $2;
    } else {
       Logger::LogError($nodeLogFH, "No SHUB OFLMT version could be obtained...\n" );
       $envOk = 0;
       exit NO_SHUB_OFLMT_VER_CODE;
    }
    Logger::LogInfo($nodeLogFH, sprintf("%-20s : %s, %s\n",
                                      $label, $oflVersInfo->{SHUB},
                                      Capability::ShubOflmtVersionString(Capability::HighestShubOflmtVersion(Capability::IsamEtsiNttFamily()))
                                 ), $Logger::LogOnlyDebug);
    if ( $params->{SHUBOFLMTVERS} > Capability::HighestShubOflmtVersion(Capability::IsamEtsiNttFamily())) {
             Logger::LogError($nodeLogFH, sprintf("%-20s : Toolset version '%s' is higher than Max version '%s'\n",
			      $oflVersInfo->{SHUB},
			      Capability::ShubOflmtVersionString(Capability::HighestShubOflmtVersion(Capability::IsamEtsiNttFamily()))) );
      $envOk = 0;
      exit SHUB_OFLMT_VERSION_2_HIGH_CODE;
    }
    ### Starting shub version 1.67 only Lanx Checksum exe CalcIpCheckSum_BE is available for sparc platform. 
    if (($Config{archname} =~ /sun4/) && ($params->{SHUBOFLMTVERS} < 167)) { 
       $toolConfigData->{LANXIPCKSUM} = "CalcIpCheckSum"; 
    }
    if (! -x "$installDir/$toolConfigData->{LANXDBTOOLDIR}/$toolConfigData->{LANXIPCKSUM}") {
      $envOk = 0;
      Logger::LogError($nodeLogFH, "$installDir/$toolConfigData->{LANXDBTOOLDIR}/$toolConfigData->{LANXIPCKSUM} is not executable\n" );
      exit CALC_IP_CKSUM_NO_EXEC_CODE;
    }
    return $oflVersInfo;
}


1;
