package PreCheckIsam7354;

=head1 ISAM 7354-specific pre-check module

=begin html
This library contains the functions dedicated to perform prechecks for a 7354 ISAM.

=end html

=cut

#==============================================================================
#-- File:         PreCheckIsam7302.pm
#-- Date:         01.09.2008
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Supporting library carrying out the ISAM PIZZA NE-family prechecks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/10/10 : First implementation
#--   1.0.1 - 2009/06/11
#--   1.0.2 - 2011/04/01 : Changes for 7354
#--   1.0.3 - 2011/04/05 : ALU01031457 - Support for R4220 & R4230
#--   1.0.4 - 2015/07/15 : ALU02237966
#--   1.1.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   1.1.1  - 2018/08/06 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "PreCheckIsam7354.pm - 1.1.1 - 2018/08/06"

=end html

=cut

my $VERSION = "PreCheckIsam7354.pm - 1.1.1 -- 2018/08/06";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
require 5.001;

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;

#==============================================================================
use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Crypt::Lite;
use Utils;
require Octopus;
use Capability;
use PbmtConfig;
require FamilyCommon;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :PRCK);
use version();
use vars qw(%version);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Check($$$$);
#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;

#==============================================================================
#== Vars
#==============================================================================
my $warnLogFile;
my %config;
my $warnLevels;

my $config;
my $toolData;

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Check($$$$$)

=begin html
This function is called to perform pre-checks of a node of the ISAM family. In case of failure the undefined value is returned.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>userName</td><td>the UNIX username of the user who launched the script</td></tr>
  <tr><td>ntType</td><td>NT type name</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#

sub Check($$$$) {
  my ($newLog, $isam, $userName, $ntType) = @_;
  my ($swMgntIniSize, $altSwMgntIniSize, $swParSize);
  my ($newMajor, $newMinor, $newSuffix, $newRelNum, $newFullRelNum, $activeRelNum, $fullActiveRelNum);
  my $activeNt;
  my $possible = 0;
  my $reason = "";
  my $isamCliSession = undef;
  my $ret;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $warningLevels = FamilyCommon::GetWarningLevels();

  $config = $configData;
  $toolData = $toolConfigData;
  $warnLevels = $warningLevels;
  #-- Open CLI Session
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $isamCliSession = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($isamCliSession)) {
     return undef if( FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, ToolCommon::GetError(), undef ) );
  } else {
     ToolCommon::Log($nodeLogFH, "  OK\n");
  }
  FamilyCommon::SetCliSession2Close($isamCliSession);
  ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $isamCliSession->errmode)) if ($params->{DEBUG});

  #-- STEP 0: Acquire transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");


#-- STEP : Check if migration interrupt on emergency call feature is enabled
  if ($params->{EMERG_CALL_FEATURE}) {
       my $isActive = 0;
       ToolCommon::Log($nodeLogFH, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
       $isActive = ToolCommon::AlarmRetrieval($nodeLogFH, $isamCliSession, $toolData);
       if ($isActive) {													   ## emergency call alarm is active
           return undef if( FamilyCommon::ReportStatus( EC_EMERGENCY_CALL_ACTIVE, ToolCommon::GetError(), undef ) );
       }
  }

  #-- STEP : Check if automatic DB activation after download is configured
  my $retCode =  IsamCli::CheckDBAutoActivate($isamCliSession);
  if (defined($retCode) && $retCode) {
        return undef if( FamilyCommon::ReportStatus( EC_DB_AUTO_ACTIVATE_ON_DWLD, ToolCommon::GetError(), undef ) );
  }

  #-- STEP 1: NT type information and check Active OSWP is committed
  my $oswpInfo = ToolCommon::GetOswpInfo($isamCliSession, $snmpSession, $dir, $isam, uc($ntType), undef,
					 $toolData->{RETRIES});
  if (! defined($oswpInfo)) {
      return undef if( FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, ToolCommon::GetError() ) );
  } 
  my $dbVersionActive = $oswpInfo->{'ACTIVE'}{'DBVERSION'};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveNum = substr($dbVersionActive,1);
  my $dbVersionTargetNum = substr($dbVersionTarget,1);


  $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'}, $oswpInfo->{'ACTIVE'}{'SUFFIX'}, $dbVersionActive );

  $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
		                             $oswpInfo->{'ACTIVE'}{'MINOR'}, 0, $dbVersionActive );

  my $newSwpIndex = $oswpInfo->{PASSIVE}{INDEX};
  my $new = basename($params->{SWPNAME});
  my $dbVerTargetRel = $params->{DBVER_TGTREL};
  my $regExp = REGEXP;
  if ($new =~ /$regExp/) {   # -- Start 1.0.4
    $newMajor = $1;
    $newMinor = $2;
    $newSuffix = $3;
    $newFullRelNum = Capability::ConvertRelease($newMajor, $newMinor, $newSuffix, $dbVerTargetRel); # -- 1.2.0
    $newRelNum =  Capability::ConvertRelease($newMajor, $newMinor, 0, $dbVerTargetRel);
  }                          # -- End 1.0.4

  if ($dbVersionActive ne $dbVersionTarget) {
      #-- Verify migration path
     my ($restricted, $restricted_reason) = Capability::IsMigrateRestricted( $ntType,
                                                                  $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                               basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
     if (not $restricted) {
         Logger::LogTrace($nodeLogFH, "Migrate Restrictions... NONE\n");
     } else {
         FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                           $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                           basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
         if ($params->{CONTINUE}) {
           return undef;
         } else {
           FamilyCommon::ToolExit(UPG_IMPOSSIBLE_CODE);
         }
     }
      Logger::LogNotify($nodeLogFH, "Checking whether migration restrictions apply...\n");
      ($possible, $reason) = CapabilityReEngr::MigrationPossible( $ntType, $dbVersionActive, $dbVersionTarget);
      if (not $possible) {
          return undef if( FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                             $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason )) );
      }
  } else {
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntType, $dbVersionActive, $dbVersionTarget,
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          return undef if( FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                             $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason )) );
      }
  }

  ToolCommon::Log($nodeLogFH, "Checking new software already present...");
  if (basename($oswpInfo->{ACTIVE}{NAME}) eq (basename($params->{OSWP})) || # -- 1.2.3
      (basename($oswpInfo->{PASSIVE}{NAME}) eq basename($params->{OSWP}) && $oswpInfo->{PASSIVE}{AVAIL} eq 'enabled')) { #-- 1.2.3
          return undef if( FamilyCommon::ReportStatus( EC_NEW_SW_PRESENT, "Already present") );
  }

  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
  return undef if( ToolCommon::ReleaseTransaction() );
  Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  IsamCli::AllowAlarms($isamCliSession);
  FamilyCommon::CloseCliSession2Close();
}

1;
