package RollbackIsam; 

=head1 HiCap ISAM 7302 Specific Rollback Module 

=begin html
This library contains the functions dedicated to perform a Rollback of 7302/7330 ISAM

=end html

=cut
#------------------------------------------------------------------------------
#-- File          :RollbackIsam7302HiCap.pm
#-- Date          :23-07-2014
#-- Author        :K Ravi
#------------------------------------------------------------------------------
#-- Function:     Script for ISAM7302/7330 Rollback
#------------------------------------------------------------------------------
#-- Copyright:    All rights reserved. Passing on and copying of this document,
#--               use and communication Of its contents not permitted without
#--               written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- History:
#--    1.0.1    :    23-07-2014 - First Implementation for RollbackIsam7302HiCap.pm 
#--    1.0.2    :    26-09-2014 - Defensive mechanism to avoid rollback on higher releases 
#--    1.0.3    :    07-10-2014 - ALU02010669
#--    1.0.4    :    19-11-2014 - ALU02028145
#--    1.0.5    :    13-02-2015 - ALU02046572
#--    1.0.6    :    16-03-2016 - ALU02190558
#--    1.0.6    :    15-07-2016 - ALU02237966
#--    1.1.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--    1.1.1  - 2019/04/09 : ALU02568045
#------------------------------------------------------------------------------

=head1 VERSION

=begin html
Current version: "RollbackIsam.pm - 1.1.1 -- 2019/04/03"

=end html

=cut

my $VERSION = "RollbackIsam.pm - 1.1.1 -- 2019/04/03";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;

use FileHandle;
use File::stat;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;

#==============================================================================
#== Find the lib dir
use Cwd;
use File::Basename;

#==============================================================================
use Data::Dumper;
use Net::SNMP;
use Crypt::Lite;
use Utils;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :RLBK);
use ToolCommon;
use PBMTGlobal;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
#use Logger;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub DownloadDbase($$$$$$);
sub Rollback($$);
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
##my $STATUSFILE =       "result.txt";
my $toolData;
my $config;
my $RESFILENAME = "dm_complete.tar";

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#-- sub DownloadDbase($$$$$$);
#------------------------------------------------------------------------------

=head2 DownloadDbase($$$$$$$)

=begin html
Downloads the Restored DB to ISAM.

=end html

=cut



sub DownloadDbase($$$$$$) {
  my ($fh, $cli, $failedNode, $activeRelNum, $thisTftpDir, $userName) = @_;
  my $ret;
  my $ok = 1;
  ToolCommon::Log($fh, "Downloading the Restored Database...\n",1);
  $ret = IsamCli::DownloadCompleteDb($cli, $params->{PRIMTFTPIP}, "$thisTftpDir/RestoreDB"); 
  if ( defined($ret) and $ret ) {
     Logger::LogNotify($nodeLogFH, "Restored DB Download successful...\n");
     return $ok;
  }
  $ok = 0;
  my $t = sprintf("DB Download FAILED (%s)", IsamCli::GetError());
  FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
  FamilyCommon::ReportStatus( EC_DOWNLOAD_DB_FAILED, undef, "$t\n" );
  return $ok;
}

    
 
=head2 Rollback($$$)

=begin html
This function is called to perform rollback of a node of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>ntType</td><td>NT family or NT Type</td></tr>
</table>

=end html

=cut

sub Rollback($$) {
  my ($isam, $ntType) = @_;
  my ($newMajor, $newMinor, $newSuffix, $newFullRelNum);
  my ($newRelNum, $fullActiveRelNum, $activeRelNum, $fromDbVers, $toDbVers);

  my $activeNt;
  my $cliSession = undef;
  my $ret;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $dir = FamilyCommon::GetNeStorage();
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $childNum = FamilyCommon::GetChildNum();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();
  my $ltActivating = 0;
  
  my  $nodeStatusLog = FamilyCommon::GetNodeStatusLog();

  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  my $failedNode = $params->{NONAME} ? $isam->[0] : $isam->[1]; # Store for IncrErrorCount
  FamilyCommon::SetFailedNode($failedNode);

  #-- Open CLI Session
  ToolCommon::Log($nodeLogFh, "Open CLI...");
  $cliSession = ToolCommon::OpenCliWrap($isam, $nodeLogFh, $toolData->{TELNETTIMEOUT});
  if (!defined($cliSession)) {
    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
                                 EC_OPENCLI_FAILED,
                                 sprintf(": %s", ToolCommon::GetError()),
                                 1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFh, "  OK\n");
  }
  FamilyCommon::SetCliSession2Close($cliSession);  # Store for IncrErrorCount
  ToolCommon::Log($nodeLogFh, sprintf("Telnet errmode: %s\n", $cliSession->errmode)) if ($params->{DEBUG});
  my $ltInfo = IsamCli::GetLtConfigState($cliSession);
  foreach my $lt ( keys %$ltInfo ) {
     if( defined $ltInfo->{$lt}{TYPE} and ($ltInfo->{$lt}{TYPE} eq "fwlt-a" or $ltInfo->{$lt}{TYPE}  eq "fglt-b") ) {
      $ltActivating = 1;
     }
  }

  Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership... DONE\n");

  #-- Stop receiving alarms (Inhibit alarms)   


  
   #--Step 6: Get Nt Ex Information to check for compatibility in redundent NT.
   ToolCommon::Log($nodeLogFh, "Obtain extended NT information...");
   ###my $ntInfo = Isam7342TL1::GetNtExInfo($tl1Session);
   my $ntInfo = IsamCli::GetNtExtendedInfo($cliSession);
   return 0 if (not defined($ntInfo) ); 
    
   ### Step 7 : Collect OSWP information
   ToolCommon::Log($nodeLogFh, "Attempting to retrieve OSWP Info..\n");
   my $oswpInfo = ToolCommon::GetOswpInfo($cliSession, $snmpSession, $dir, $isam, $ntType, undef, $toolData->{RETRIES});
   if (! defined($oswpInfo)) {
        FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED, ToolCommon::GetError(), undef );
        return 0;
   }

   ### Step 7A : OSWP related validations
   my $base = basename($params->{OSWP}); 
   (my $oswp) = $base =~ /(\d+\.\d+)/;
   if(!($oswpInfo->{'ACTIVE'}{'NAME'} =~ m/$oswp/)) {		
        FamilyCommon::ReportStatus(EC_TARGET_DIFF_FROM_ACTIVE_OSWP, "Target SWP is not same as Active OSWP", undef );
        return 0;	
   }
  
   ToolCommon::Log($nodeLogFh, "Check Passive OSWP is empty \n");
   if (uc($oswpInfo->{'PASSIVE'}{'NAME'}) eq "NO_OSWP") {
         FamilyCommon::ReportStatus(EC_PASSIVE_OSWP_EMPTY, "Passive OSWP not present", undef );
         return 0;
   } 

   if (uc($oswpInfo->{'PASSIVE'}{'AVAIL'}) ne "ENABLED") {
         FamilyCommon::ReportStatus(EC_NEW_SWP_NOT_ENABLED, "Passive OSWP not enabled", undef );
         return 0;
   }

   if (defined($oswpInfo->{'PASSIVE'}{'COMMITED'}) && (uc($oswpInfo->{'PASSIVE'}{'COMMITED'}) ne "COMMITTED")) {   
         FamilyCommon::ReportStatus(EC_PASSIVE_SWP_NOT_COMMITTED, "Passive OSWP not committed", undef );
         return 0;
   }

 
 
  my $retval =  Capability::ReleaseCompare(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
                                                                 $oswpInfo->{'ACTIVE'}{'MINOR'},
                                                                 $oswpInfo->{'ACTIVE'}{'SUFFIX'}),
                                     Capability::ConvertRelease($oswpInfo->{'PASSIVE'}{'MAJOR'},
                                                                 $oswpInfo->{'PASSIVE'}{'MINOR'},
                                                                 $oswpInfo->{'PASSIVE'}{'SUFFIX'}),
                                     REGEXP, 0);
  my $buildNoActive = "$oswpInfo->{'ACTIVE'}{'SUFFIX'}$oswpInfo->{'ACTIVE'}{'MAINT'}";
  my $buildNoPassive = "$oswpInfo->{'PASSIVE'}{'SUFFIX'}$oswpInfo->{'PASSIVE'}{'MAINT'}";
  if( ($retval <= 0) && ( $buildNoActive < $buildNoPassive ) ) {
       FamilyCommon::ReportStatus(EC_HIGHER_PASSIVE_RESTRICTED, "Passive OSWP release is greater", undef );
       return 0;
  }
  
#  my $filesize; 
#  if ($toolData->{"NOTFTP"} == 1) { #TFTP BDFam63384
#    $filesize = -s "$toolData->{FTPDIR}/$toolData->{RESTORE_FILE_NAME}";
#    ToolCommon::Log($nodeLogFh, "Restore file Path $toolData->{FTPDIR}/$toolData->{RESTORE_FILE_NAME} \n");
#  } else { 
#    $filesize = -s "$toolData->{RESTORE_FILE_NAME}";
#    ToolCommon::Log($nodeLogFh, "Restore file Path $toolData->{RESTORE_FILE_NAME} \n");
#  }


# if ($filesize) { 
#   $ret =  Isam7342TL1::RestoreDB($tl1Session, $params->{PRIMTFTPIP}, $params->{ISAM}, $toolData->{RESTORE_FILE_NAME},1);
#   if (! $ret) {
#     FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", Isam7342TL1::GetError()),
#                                 ISAM_RESTORE_DB_FAILED, undef,
#                                 1);
#     return ISAM_RESTORE_DB_FAILED;
#     ToolCommon::Log($nodeLogFh, "DB Restore failed Attempting to restore with linked DB..\n");
#   }
#} else { 
#  ToolCommon::Log($nodeLogFh, "DB Restore file either not found or zero in size!!!\n");
#  ToolCommon::Log($nodeLogFh, "Attempting to restore with linked DB..\n");
#}


  ToolCommon::Log($nodeLogFh, "Attempting to check DB Info..\n");
  my $dbInfo = IsamCli::GetDbVersionInfo($cliSession);
  if (! defined($dbInfo)) {
    FamilyCommon::IncrErrorCount("Failed to get NT Database version information.",
                   EC_PASSIVE_DB_FAILED,undef,1);
    return 0;
  }

   $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'},
						 $oswpInfo->{'ACTIVE'}{'SUFFIX'},
                                                 $dbInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});

   Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership...\n");
   return undef if( ToolCommon::AcquireTransaction() );
   Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership... DONE\n");

   if ( $params->{'BOARD-FAMILY'} eq Capability::IsamHiCapEtsiFamily() ) {
        my $userName = getlogin();
        my $thisTftpDir = $toolData->{TFTPWORKDIR};
        if(!$params->{NOTFTP}){
          	$ret = DownloadDbase($nodeLogFh, $cliSession, $failedNode, $fullActiveRelNum, "$thisTftpDir/$failedNode", $userName); 
        }else {
          	$ret = DownloadDbase($nodeLogFh, $cliSession, $failedNode, $fullActiveRelNum,"/".$toolData->{FXRDATA}{HOMEDIR}, $userName); 
        }
        return undef if(!$ret);
   }
 
#-- STEP : Check if migration interrupt on emergency call feature is enabled
if ($params->{EMERG_CALL_FEATURE}) {
  my $isActive = 0;
  ToolCommon::Log($nodeLogFh, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
  $isActive = ToolCommon::AlarmRetrieval($nodeLogFh, $cliSession, $toolData);
  if($isActive) {													   ## emergency call alarm is active
	 FamilyCommon::IncrErrorCount(sprintf("  FAILED: Emergency call max retries reached\n"),
				     EC_EMERGENCY_CALL_ACTIVE,
				     undef,
				     1);
	 return undef;
 }
}
#-- STEP : End of migration interrupt on emergency call feature.


    #---Step 14: Release CLI transaction Actiation is done using SNMP.
    return if( not ToolCommon::ActivateSwp($oswpInfo) );

  if ($toolData->{ACTPINGWAIT}>0) {
    ToolCommon::Log($nodeLogFh, "Waiting $toolData->{ACTPINGWAIT} seconds for the system to restart...");
    sleep($toolData->{ACTPINGWAIT});
    ToolCommon::Log($nodeLogFh, "  OK\n");
  }
  #BDFam54607 starts here 


  ##$ret = FamilyCommon::WaitForRestartConnect($nodeLogFh, $isam, undef); 
  $ret = FamilyCommon::WaitForRestart($nodeLogFh, $snmpSession, $isam, undef);  

  if (!$ret) {
    ToolCommon::Log($nodeLogFh, "NE Lost after Restart!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s... skipping\n", IsamCli::GetError()),
				 EC_RESTART_FAILED, undef, 0);
    return 0;  #BDFam56126
  }
  

  $ret = FamilyCommon::WaitForNePingAble($nodeLogFh, $isam); 
  if (!$ret) {
    ToolCommon::Log($nodeLogFh, "NE not pingable after Rollback!!!\n");
  
    FamilyCommon::IncrErrorCount(sprintf(" NE not pingable after Rollback!!!"),
				 EC_RESTART_FAILED, undef, 1);
    return 0;
  }

  #$ret = FamilyCommon::WaitForSnmpConnect($nodeLogFh); 
  #if (!$ret) {
  #  ToolCommon::Log($nodeLogFh, "SNMP connection failed after Rollback!!!\n");
 # 
 #   FamilyCommon::IncrErrorCount(sprintf(" SNMP connection failed  after Rollback!!!"),
#				 EC_RESTART_FAILED, undef, 1);
#    return 0;
#  }
#  else {  #Close the session,since we only opened to make sure the NE is up after upgrade
#     $ret->close();
#     $ret = undef;
#  }

  sleep (60); #Just to be on the safer side waiting for a minute


  #-- Open CLI  Connection to verify OSWP
  ToolCommon::Log($nodeLogFh, "Re-open CLI connection \n");
  $cliSession = ToolCommon::OpenCliWrap($isam, $nodeLogFh, $toolData->{TELNETTIMEOUT});
  #$tl1Session = Isam7342TL1::Connect($isam->[0], $toolData->{TL1UDPPORT}, $toolData->{TL1USERNAME}, $toolData->{TL1PASSWORD},
  #                                   $params->{DEBUG} ? $nodeLogFh : undef);
  if (!defined($cliSession)) {
    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open CLI session for %s: %s\n", 
                                 FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
                                 EC_OPENCLI_FAILED,undef,
                                 0);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFh, "CLI connection opened sucessfully\n");
  }
  FamilyCommon::SetCliSession2Close($cliSession);  
  
  #-- Stop receiving alarms (Inhibit alarms)   

  #-- Stop receiving alarms
  #$ret = Isam7342TL1::InhibitAlarms($tl1Session);
  #if ($ret) {
  #  ToolCommon::Log($nodeLogFh, "Alarms Inhibited Sucessfully\n");
  #} else {
  #  FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", Isam7342TL1::GetError()),
  #                             ISAM_INH_ALARM_FAILED,undef,
  #                             1);
  #  return 0;
  #}

 
  #BDFam54607 ends here 

  ToolCommon::Log($nodeLogFh, "Attempting to retrieve OSWP Info after activation..\n");
  ##my $oswpInfoAfterRollBack = ToolCommon::GetOswpInfo($cliSession, $snmpSession, $dir, $isam, $ntType, undef, $toolData->{RETRIES});
  my $oswpInfoAfterRollBack = ToolCommon::GetOswpInfoCLI($cliSession, $snmpSession, $dir, $isam, $ntType, undef, $toolData->{RETRIES});
  if (! defined($oswpInfoAfterRollBack)) {
    FamilyCommon::IncrErrorCount("Failed to get OSWP information using CLI",
                   EC_GET_OSWP_FAILED,
                   sprintf("Failed for ISAM: %s", $isam),
                   1);
    return 0;
  }
  
  
  if (uc($oswpInfo->{PASSIVE}{NAME}) ne uc($oswpInfoAfterRollBack->{ACTIVE}{NAME})) {
    FamilyCommon::IncrErrorCount("Error",
  				 EC_NEW_SW_NOT_ACT,
  				 "Passive bank did not rollback",
  				 1); 
    return 0;
  } 
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership... DONE");

  FamilyCommon::CloseCliSession2Close();
  return 1;
}

1;
