
=head1 PBMT module for SNMP Communication

=begin html
This PERL module implements the SNMP Communication of PBMT.

=end html

=cut

#==============================================================================
#-- File:         SNMPComm.pm
#-- Date:         2018-04-11
#-- Company:      Nokia Corporation
#-- Author:       Dhineshkumar Manikannan
#------------------------------------------------------------------------------
#-- Function:     Commons for the toolset
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel 2006
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0  -  2018/04/11  : Introduction of SNMp Commmunication library
#--   3.0.1  -  2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "SNMPComm.pm -- 1.0.1 -- 2018/06/20"

=end html

=cut

my $VERSION = "SNMPComm.pm -- 1.0.1 -- 2018/06/20";
#==============================================================================

package SNMPComm;

use strict;
use warnings 'all';
use PBMTGlobal;
use Net::SNMP;
use Logger qw(:ALL);
#use FamilyCommon;
use Config;
use ToolCommon qw(:CONSTANTS);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetError();
sub SetSession($);
sub ClearSession();
sub IsSessionActive();
sub EstablishSNMP($);
sub GetValue($$;$$);
sub ResetGlobals();
sub SetDebug($);
sub IsSHA2SupportValid();


my %snmpConsts  = (
        'NO_OSWP'                       => 'NO_OSWP',
		'UPLOADDBSELECTION'             => {'ACTUALACTIVE' => 1},
		'UPLOADPROGRESS'    => {   'ONGOING' => 1,
						           'FINISHEDOK' => 2,
						           'FINISHEDERR' => 3 
                               },
		'FXRSETTINGS'       => {   'tftp' => 1,
						           'ftp' => 2,
						           'sftp' => 3
                               },
		'FXRNAMES'          => {    1 => 'tftp',
						            2 => 'ftp',
						            3 => 'sftp'
                               },
		'OSWPPROGRESS'      => {    'EMPTY' => 1,
						            'ENABLED' => 2,
						            'DISABLED' => 3,
						            'DOWNLOADING' => 4,
						            'ABORTING' => 5
                               },
		'UPLOADERRORS'      => {    0 => 'Undefined',
						            1 => 'File not found',
						            2 => 'Access violation',
						            3 => 'Disk full/allocation exceeded',
						            4 => 'Illegal TFTP operation',
						            5 => 'Unknown transfer ID',
						            6 => 'File already exists',
						            7 => 'No such user',
						            8 => 'Selected DB not available',
						            9 => 'System restart',
						            10 => 'No error'
                              },
		'SYSUPTIME'                     => '.1.3.6.1.2.1.1.3.0',
		'MAXSNMPMSGSIZE'                => 65534,
		'NT_CONTEXT'                    => "nt",
        'QUERY_WAITTIME'                => 30,
        'RETRY_COUNT'                   => 2,
        'RETRY_COUNT_TABLE'             => 5,
        'RETRY_WAITTIME'                => 5,
   );
sub SYSUPTIME()                     { $snmpConsts{"SYSUPTIME"} }
sub MAXSNMPMSGSIZE()                { $snmpConsts{"MAXSNMPMSGSIZE"} }
sub NT_CONTEXT()                    { $snmpConsts{"NT_CONTEXT"} }
sub SNMP_RETRY_COUNT()              { $snmpConsts{"RETRY_COUNT"} }

my $logMesgOnly = 1;
my ( $snmpSession, $snmpError ) = (undef, "" );
my %snmpParams = ();

################################################################################
## To return version of this library/module
################################################################################
sub GetVersion() {
  return $VERSION;
}

################################################################################
## To return Error Information observed from SNMP 
################################################################################
sub GetError() {
   return $snmpError;
}

sub SetSession($) {
  my( $snmpSessionToSet ) = ( @_ );
  if( !defined $snmpSession ) {
      $snmpSession  = $snmpSessionToSet;
  }
}

sub ResetGlobals() {
    $snmpSession = undef;
    $snmpError = "";
    %snmpParams = ();
}

sub IsSessionActive() {
  if( !defined $snmpSession ) {
      $snmpError = "SNMP Session is not active to process SNMP related queries";
      LogError( $nodeLogFH, "$snmpError\n" );
      return 0;
  }
  return 1;
}

sub ClearSession() {
  if( defined $snmpSession ) {
      $snmpSession = undef;
  }
}

sub SetDebug($) {
    my( $snmpDebugFlag ) = @_;
    if( defined $snmpDebugFlag and $snmpDebugFlag ) {
         Net::SNMP->snmp_debug(0x3e, $nodeLogFH);
    } else {
         Net::SNMP->snmp_debug(0x0, $nodeLogFH);
    }
}

sub IsSHA2SupportValid() {
  ### Linux x86 (perl v5.16) and sunOS/sun4-solaris-64int (perl v5.8) is supported right now
  if ($Config{archname} =~ /sun4/) {
	return 1;
  } elsif ( $Config{archname} =~ /linux/ ) {
  	my $perlVersion = 0;
        if ($Config{version} =~ /(\d+)\.(\d{1,2})/) {
            $perlVersion = ($1 *100 + $2);
        }
        if ( $perlVersion >= 516 ) {
	    return 1;
        }
  }
  return 0;
}
################################################################################
## Function   : EstablishSNMP
## Usage      : Function to establish SNMP Communication with ISAM
## Input Param: (1) iSAM; 
## Return     : 0 - Success; 1 - Failure
################################################################################
sub EstablishSNMP($) {
    my( $isam ) = (@_);
    LogTrace($nodeLogFH, "SNMP communication with ISAM ($isam)...\n");
    ($snmpSession, $snmpError) = ( undef, "" );
    %snmpParams = (
                    hostname => $isam,
                    ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
                    timeout => ( defined $params->{SNMP}{QUERY_TIMEOUT} ) ? $params->{SNMP}{QUERY_TIMEOUT} : $snmpConsts{'QUERY_WAITTIME'},
                    retries => 1,
                  );
    LogTrace($nodeLogFH, "SNMP Query waittime : $snmpParams{timeout} seconds\n");
    ### Check username is defined to consider SNMP V3
    if (defined($params->{SNMP}{V3}{AUTH}{USERNAME}) and ($params->{SNMP}{V3}{AUTH}{USERNAME} ne "")) {
        LogTrace($nodeLogFH, "Conn-Type : SNMP V3");
        $snmpParams{ 'version' }  = 3;
        $snmpParams{ 'username' } = $params->{SNMP}{V3}{AUTH}{USERNAME};
        if (defined($params->{SNMP}{V3}{AUTH}{PASSWORD}) && ($params->{SNMP}{V3}{AUTH}{PASSWORD} ne "")) {
             LogTrace($nodeLogFH, " - AUTH", $logMesgOnly );
             $snmpParams{ 'authprotocol' } = $params->{SNMP}{V3}{AUTH}{PROTO};
             $snmpParams{ 'authpassword' } = $params->{SNMP}{V3}{AUTH}{PASSWORD};
             if ($snmpParams{ 'authprotocol' } =~ /^(SHA256|SHA512)$/ ) {
                  my $validForSHA2 = IsSHA2SupportValid();
                  if ( $validForSHA2 == 0 ) {
                      $snmpError = "Authentication Protocol - SHA2(SHA256|SHA512) is not supported for this environment";
                      return (1, $snmpSession);
                  }
             }
        }
        if (defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne "")) {
             LogTrace($nodeLogFH, " - PRIV", $logMesgOnly);
             $snmpParams{ 'privpassword' } = $params->{SNMP}{V3}{PRIV}{PASSWORD};
             $snmpParams{ 'privprotocol' } = $params->{SNMP}{V3}{PRIV}{PROTO};
        }
        LogTrace($nodeLogFH, " Provided..\n", $logMesgOnly);
    } elsif (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
        LogTrace($nodeLogFH, "Conn-Type : SNMP V1\n");
        $snmpParams{ 'version' }   = 1;
        $snmpParams{ 'community' } = $params->{SNMP}{V1}{NT_COMMUNITY};
    } elsif (defined($params->{SNMP}{V2}{NT_COMMUNITY})) {
        LogTrace($nodeLogFH, "Conn-Type : SNMP V2\n");
        $snmpParams{ 'version' }   = 1;
        $snmpParams{ 'community' } = $params->{SNMP}{V2}{NT_COMMUNITY};
    } else {
        $snmpError = "Unable to determine SNMP communication type";
        return (1, $snmpSession);
    }
    ### Enable the debug, While establish the session. It helps lot to debug error faced while establishing SNMP session
    SetDebug(1);
    ($snmpSession, $snmpError) = Net::SNMP->session(%snmpParams);
    SetDebug(0);
    if (!defined($snmpSession)) {
         $snmpError .= " (username/password mismatch?)" if ($snmpError =~ /usmStatsWrongDigests/);
         return (1, $snmpSession);
    } else {
         LogTrace($nodeLogFH, "SNMP communication with isam ($isam) esblished...\n");
         LogTrace($nodeLogFH, "SNMP Details : Session ($snmpSession), Version(".$snmpParams{ 'version' }."), Error ($snmpError)\n");
         my $response = GetValue( 'SYSTEM UPTIME', SYSUPTIME ); ## "SYSUPTIME"
         # Get the sysuptime and check for a reasonable value: a get returns a non-empty string.
         # An empty string is returned in case we open an SNMPv3 session while the node does not have the SNMP user configured.
         # The session setup succeeds but all sub-sequent SNMP operation do not have any effect.
         if ( !defined($response) or ($response eq "") ) {
                $snmpError = "SNMP V". $snmpParams{ 'version' } . " Configuration mismatch; Check SNMP version and appropriate SNMP details/credentials (if applicable)";
                return (1, $snmpSession);
	     }
	     LogDebug($nodeLogFH, "Uptime: '$response'\n");
         FamilyCommon::SetSnmpSession2Close($snmpSession); # Store for IncrErrorCount
         #-- disable OID translation
         $snmpSession->translate(0);
    }
    return (0, $snmpSession);
}

sub TrapError() {
	my $error_code = $snmpSession->error_status();
    my $error_mesg = $snmpSession->error();
    if( $error_mesg ne "" or $error_code != 0 ) {
       $snmpError = "SNMP Error Mesg($error_code) : $error_mesg";
       Logger::LogError( $nodeLogFH, "$snmpError\n", undef, $Logger::LogOnlyHandle);
    }
}


sub MaximizeMsgSize() {
    return 1 if( ! IsSessionActive() );
    #-- increase maximum message size
    my $oldSize = $snmpSession->max_msg_size();
    if( !defined($snmpSession->max_msg_size(MAXSNMPMSGSIZE)) ) {
        $snmpError = $snmpSession->error();
        return 1;
    } else{
        LogDebug( $nodeLogFH, sprintf("Increased maximum SNMP message size from %d to %d\n", $oldSize, MAXSNMPMSGSIZE));
        return 0;
    }
}

################################################################################
## Function   : GetValue 
## Usage      : Get OID value through SNMP session 
## Input Param: (1) Label to log, (2) SNMP OID ; 
## Return     : 0 - Success; 1 - Failure
################################################################################
sub GetValue($$;$$) {
  my ($snmpLabel, $snmpOid, $maxRetryCount,$retryWaitTime) = @_;
  return undef if( ! IsSessionActive() );
  $maxRetryCount = $snmpConsts{'RETRY_COUNT'} if( not defined $maxRetryCount );
  $retryWaitTime = $snmpConsts{'RETRY_WAITTIME'}  if( not defined $retryWaitTime );
  Logger::LogTrace( $nodeLogFH,"\n", 1);
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $snmpOid (Retry:$maxRetryCount)\n");
  my %snmpRequestParam = ( varbindlist => [ $snmpOid ] );
  if( $snmpParams{ 'version' } == 3 ) {
       $snmpRequestParam{ 'contextname' } = NT_CONTEXT;
  }
  my( $snmpResponse, $snmpResponseValue ) = (undef, undef);
  my $curRetryCount = 0;
  SetDebug(0);
  $snmpError = "";
  while ( $curRetryCount < $maxRetryCount ) {
       $snmpResponse = $snmpSession->get_request( %snmpRequestParam );
       if( defined $snmpResponse and defined $snmpResponse->{$snmpOid} ) {
             $snmpResponseValue = $snmpResponse->{$snmpOid};
             $snmpResponseValue =~ s/\s//g;
             last;
       } else {
          TrapError();
       }
       sleep $retryWaitTime;
       $curRetryCount++;
       SetDebug(1);
  }
  SetDebug(0);
  ### For SNMP V-3, Sometimes it happens it return NULL
  if( defined $snmpResponseValue and $snmpResponseValue eq "" and $snmpParams{ 'version' } == 3 ) {
      $snmpResponseValue = undef;
      $snmpError = "Received NULL for V3, SNMP Version mismatched?";
  }
  my $valuemesg = "#NULL#";
  if (defined $snmpResponseValue){
      $valuemesg = "'$snmpResponseValue'";
      $snmpError = "";
  }
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $valuemesg \n\n");
  return $snmpResponseValue;
}

################################################################################
## Function   : GetValueNext 
## Usage      : Get OID value through SNMP session 
## Input Param: (1) Label to log, (2) SNMP OID ; 
## Return     : 0 - Success; 1 - Failure
################################################################################
sub GetValueNext($$;$$) {
  my ($snmpLabel, $snmpOid, $maxRetryCount,$retryWaitTime) = @_;
  return undef if( ! IsSessionActive() );
  $maxRetryCount = $snmpConsts{'RETRY_COUNT'} if( !defined $maxRetryCount );
  $retryWaitTime = $snmpConsts{'RETRY_WAITTIME'}  if( !defined $retryWaitTime );
  Logger::LogTrace( $nodeLogFH,"\n", 1);
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $snmpOid (Retry:$maxRetryCount)\n");
  my %snmpRequestParam = ( varbindlist => [ $snmpOid ] );
  if( $snmpParams{ 'version' } == 3 ) {
       $snmpRequestParam{ 'contextname' } = NT_CONTEXT;
  }
  my( $snmpResponse, $snmpResponseValue ) = (undef, undef);
  my $curRetryCount = 0;
  SetDebug(0);
  while ( $curRetryCount < $maxRetryCount ) {
       $snmpResponse = $snmpSession->get_next_request( %snmpRequestParam );
       if (defined $snmpResponse ) {
            foreach my $snmpOid ( keys %{$snmpResponse} ) {
                Logger::LogDebug( $nodeLogFH, sprintf( "%s : %s - %s\n", 
                                 $snmpLabel, $snmpOid, (defined $snmpResponse->{$snmpOid}) ? $snmpResponse->{$snmpOid} : "##NULL##") );
                if( $snmpResponse->{$snmpOid} ) {
                    $snmpResponseValue = $snmpResponse->{$snmpOid};
                }
            }
            last if( defined $snmpResponseValue );
       } else {
           TrapError();
       }
       sleep $retryWaitTime;
       SetDebug(1);
       $curRetryCount++;
  }
  SetDebug(0);
  ### For SNMP V-3, Sometimes it happens it return NULL
  if( defined $snmpResponseValue and $snmpResponseValue eq "" and $snmpParams{ 'version' } == 3 ) {
      $snmpResponseValue = undef;
      $snmpError = "Received NULL for V3, SNMP Version mismatched?";
  }
  my $valuemesg = "#NULL#";
  if (defined $snmpResponseValue){
      $valuemesg = "'$snmpResponseValue'";
      $snmpError = "";
  }
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $valuemesg \n\n");
  return $snmpResponseValue;
}
################################################################################
## Function   : GetValueList 
## Usage      : Get OID value through SNMP session 
## Input Param: (1) Label to log, (2) SNMP OID ; 
## Return     : 0 - Success; 1 - Failure
################################################################################
sub GetValueList($$;$$) {
  my ($snmpLabel, $snmpOidDetails, $maxRetryCount,$retryWaitTime) = @_;
  return undef if( ! IsSessionActive() );
  $maxRetryCount = $snmpConsts{'RETRY_COUNT'} if( !defined $maxRetryCount );
  $retryWaitTime = $snmpConsts{'RETRY_WAITTIME'}  if( !defined $retryWaitTime );
  Logger::LogTrace( $nodeLogFH,"\n", 1);
  $" = " -- ";
  while (my ($key,$value) = each %$snmpOidDetails ) {
      Logger::LogTrace( $nodeLogFH,"$snmpLabel : $value - $key (Retry:$maxRetryCount)\n");
  }
  SetDebug(0);
  my @snmpOidList = keys %$snmpOidDetails;
  my %snmpRequestParam = ( varbindlist => \@snmpOidList  );
  if( $snmpParams{ 'version' } == 3 ) {
       $snmpRequestParam{ 'contextname' } = NT_CONTEXT;
  }
  my( $snmpResponse, $snmpResponseValue ) = (undef, undef);
  my $curRetryCount = 0;
  while ( $curRetryCount < $maxRetryCount ) {
       $snmpResponse = $snmpSession->get_request( %snmpRequestParam );
       if (defined $snmpResponse ) {
            foreach my $snmpOid ( keys %$snmpResponse ) {
                Logger::LogTrace( $nodeLogFH, sprintf( "%s : %s - %s\n",
                                 $snmpLabel, $snmpOidDetails->{$snmpOid}, $snmpResponse->{$snmpOid} ) );
            }
            last;
       } else {
           TrapError();
       }
       sleep $retryWaitTime;
       $curRetryCount++;
       SetDebug(1);
  }
  SetDebug(0);
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : ##NULL##\n") if( not defined $snmpResponse );
  return $snmpResponse;
}
################################################################################
## Function   : GetTable 
## Usage      : Get OID value through SNMP session 
## Input Param: (1) Label to log, (2) SNMP OID ; 
## Return     : 0 - Success; 1 - Failure
################################################################################
sub GetTable($$;$$) {
  my ($snmpLabel, $snmpOid, $maxRetryCount,$retryWaitTime) = @_;
  return undef if( ! IsSessionActive() );
  $maxRetryCount = $snmpConsts{'RETRY_COUNT'} if( !defined $maxRetryCount );
  $retryWaitTime = $snmpConsts{'RETRY_WAITTIME'}  if( !defined $retryWaitTime );
  Logger::LogTrace( $nodeLogFH,"\n", 1);
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $snmpOid (Retry:$maxRetryCount)\n");
  my %snmpRequestParam = ( baseoid => $snmpOid );
  if( $snmpParams{ 'version' } > 1 ) {
       $snmpRequestParam{ 'maxrepetitions' } = $snmpConsts{'RETRY_COUNT_TABLE'};
  }
  if( $snmpParams{ 'version' } == 3 ) {
       $snmpRequestParam{ 'contextname' } = NT_CONTEXT;
  }
  my( $snmpResponse, $snmpResponseValue ) = (undef, undef);
  my $curRetryCount = 0;
  SetDebug(0);
  while ( $curRetryCount < $maxRetryCount ) {
       $snmpResponse = $snmpSession->get_table( %snmpRequestParam );
       last if( defined $snmpResponse );
       TrapError();
       sleep $retryWaitTime;
       SetDebug(1);
       $curRetryCount++;
  }
  SetDebug(0);
  my $valuemesg = "#NULL#";
  if (defined $snmpResponseValue){
      $valuemesg = "VALUE-RETRIVED";
      $snmpError = "";
  }
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $valuemesg \n\n");
  return $snmpResponse;
}

################################################################################
## Function   : SetValue 
## Usage      : Get OID value through SNMP session 
## Input Param: (1) iSAM; 
## Return     : 0 - Success; 1 - Failure
################################################################################
sub SetValue($$$;$$) {
  my ($snmpLabel, $snmpOid, $valueToSet, $maxRetryCount,$retryWaitTime) = @_;
  return undef if( ! IsSessionActive() );
  $maxRetryCount = $snmpConsts{'RETRY_COUNT'} if( !defined $maxRetryCount );
  $retryWaitTime = $snmpConsts{'RETRY_WAITTIME'}  if( !defined $retryWaitTime );
  my @a_setarr;
  if ($valueToSet =~ /^.*[A-Z]|[a-z].*$/) {  
    push(@a_setarr,($snmpOid, OCTET_STRING, $valueToSet));
  } elsif ($valueToSet =~ /\d+\.\d+\.\d+\.\d+/) {
    push(@a_setarr,($snmpOid, IPADDRESS, $valueToSet));
  } else {   
    push(@a_setarr,($snmpOid, INTEGER, $valueToSet));
  }
  Logger::LogTrace( $nodeLogFH,"\n", 1);
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $snmpOid => $valueToSet (Retry:$maxRetryCount)\n");
  my %snmpRequestParam = ( varbindlist => \@a_setarr  );
  if( $snmpParams{ 'version' } == 3 ) {
       $snmpRequestParam{ 'contextname' } = NT_CONTEXT;
  }
  my( $snmpResponse, $snmpResponseValue ) = (undef, undef);
  my $curRetryCount = 0;
  SetDebug(0);
  while ( $curRetryCount < $maxRetryCount ) {
       $snmpResponse = $snmpSession->set_request( %snmpRequestParam );
       if( defined $snmpResponse ) {
             $snmpResponseValue = $snmpResponse->{$snmpOid};
             last;
       } else {
           TrapError();
       }
       sleep $retryWaitTime;
       $curRetryCount++;
       SetDebug(1);
  }
  SetDebug(0);
  my $valuemesg = "FAILURE";
  my $status = 0;
  if (defined $snmpResponse  ) {
      $valuemesg = "SUCCEED";
      $snmpError = "";
      $status = 1;
  }
  Logger::LogTrace( $nodeLogFH,"$snmpLabel : $valuemesg\n\n");
  return $status;
}

1;
