
=head1 PBMT Secure T& D interface

=begin html
This PERL module implements the T &amp;D accesses made to the ISAM using octopus.

=end html

=cut
#==============================================================================
#-- File:         SecureDebugTAndD.pm
#-- Date:         20.03.2015
#-- Company:      Alcatel Bell - TEC
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Collection of octopus scripts
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel 2015
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.01 - 2015/03/20 : First implementation
#--   1.0.02 - 2015/05/12 : ALU02077835 
#--   1.0.03 - 2015/08/10 : ALU02111830
#--   1.0.04 - 2015/10/13 : ALU02135551
#--   1.0.05 - 2016/07/15 : ALU02237966

=head1 VERSION

=begin html
Current version: "SecureDebugTAndD.pm -- 1.0.05 - 2016/07/15"

=end html

=cut

my $VERSION = "SecureDebugTAndD.pm -- 1.0.05 - 2016/07/15";
#==============================================================================

package SecureDebugTAndD;


=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2090-DFZZA">ReduceSWP</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use Cwd qw(getcwd);
use File::Basename;
use Config; 

require Utils;
use ErrorCodes qw(:EXIT_CODES);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
require ReduceSWP;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub SelectFH($);
sub GetError();
sub SetVars($);
sub SshEnableSwWrite($$;$); 
sub SshRemoveSwPkg($$$$); 
sub SshGetOswpFromFW($$$$$$); 
sub SetFileSystemBPF($$);
sub ReduceSW($$) ;
#==============================================================================
my $fh = undef;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

my $NT_DEAD = "NT dead";
my $ERROR = "";

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut
sub GetError() {
  return $ERROR;
}

=head2 NtIsUp()

=begin html
This function returns true in case the NT is up or 0 in case the NT is dead.

=end html

=cut


sub NtIsUp() {
  return $ERROR eq $NT_DEAD ? 0 : 1;
}

=head2 SetVars($$$)

=begin html
Sets the required environment to use the Debug via SSH interface:
<table>
  <tr><td>handle</td><td>file handle for progress and debug messages</td></tr>
</table>

=end html

=cut

sub SetVars($) {
  my ($handle) = @_;
  $fh = &SelectFH($handle);
}

=head2 SshEnableSwWrite($$;$)

=begin html
Enable read and write access on /Sw.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>oswp</td><td>Optional:Indication to read/write /Sw/OSWPx</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub SshEnableSwWrite($$;$) {
  my ($ssh, $dir,$oswp)=@_;

  my $swOswpx = defined ($oswp) ? "$oswp" : undef;
  my $logFile = "$dir/enableSwWrite.log";
  $logFile = "$dir/enableSwOSWPXWrite.log" if $swOswpx;
  my $fhOut;
  my $usr;
  my $cmd;
  my @out;


  if (!open($fhOut, '>', $logFile)) { 
    print STDERR "Cannot open  '$logFile' for writing: $!\n "; 
    return CREATE_FILE_FAIL_CODE;
  }
  if ( $oswp ) {
  	ToolCommon::Log($fh, "Enable read/write on /Sw/OSWPx...\n");
        $usr = GetDebugUserIdentity($ssh);

	if (defined($usr)) {
        $cmd = "dbg suspend $usr";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(30); 
	}

        $cmd = "tapi showaccess /Sw/OSWP1";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "tapi enable read /Sw/OSWP1";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "tapi enable write /Sw/OSWP1";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "tapi showaccess /Sw/OSWP1";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "tapi showaccess /Sw/OSWP2";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(5); 

        $cmd = "tapi enable read /Sw/OSWP2";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "tapi enable write /Sw/OSWP2";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "tapi showaccess /Sw/OSWP2";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(5); 

        $cmd = "dbg resume EPRT";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(1); 
  } else {
  	ToolCommon::Log($fh, "Enable read/write on /Sw...");
        $usr = GetDebugUserIdentity($ssh);

	if (defined($usr)) {
        $cmd = "dbg suspend $usr";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        print $fhOut "Response received:@out\n";
        sleep(2); 
	}

        $cmd = "tapi enable read /Sw";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "tapi enable write /Sw";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "dbg resume EPRT";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 
  }
      close $fhOut;
      return 0;
}

=head2 SshRemoveSwPkg($$$$)

=begin html
Enable read and write access on /Sw.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>swp</td><td>SW package to be removed</td></tr>
  <tr><td>oswp</td><td>OSWP package to be removed</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub SshRemoveSwPkg($$$$) {
  my ($ssh, $dir,$swp,$oswp)=@_;
  my $logFile = "$dir/removeSwWrite.log";
  my $fhOut;
  my $usr;
  my $cmd;
  my @out;


  if (!open($fhOut, '>', $logFile)) { 
    print STDERR "Cannot open  '$logFile' for writing: $!\n "; 
    return CREATE_FILE_FAIL_CODE;
  }
        $usr = GetDebugUserIdentity($ssh);

	if (defined($usr)) {
        $cmd = "dbg suspend $usr";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(30); 
	}

  if ($swp) {
  	ToolCommon::Log($fh, "Remove Software pkg $swp ...\n");

        $cmd = "remove $swp";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "dir $swp";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 
  }
  if ($oswp) {
  	ToolCommon::Log($fh, "Remove OSWP $oswp ...\n");

        my $cmd = "remove $oswp";
        print $fhOut "$cmd\n";
        my @out = $ssh->cmd("$cmd");
        if (!@out) {
         print "empty res..\n"; 
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 

        $cmd = "dir $oswp";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
         print "empty res..\n"; 
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2); 
  }
        $cmd = "dbg resume EPRT";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
         print "empty res..\n"; 
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(1); 

  close $fhOut;
  return 0;
} 

=head2 SshGetOswpFromFW($$$$$)

=begin html
Enable read and write access on /Sw.<br>The following parameters are used:
<table>
  <tr><td>debug</td><td>Secuted T&D connection</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>swp</td><td>ISAM detail</td></tr>
  <tr><td>oswp</td><td>'isam' string</td></tr>
  <tr><td>oswp</td><td>Regular expression</td></tr>
</table>
Returns oswpInfo 

=end html

=cut

sub SshGetOswpFromFW($$$$$$) {
  my ($ssh, $debug, $dir, $isam, $string,$regExp)=@_;
  my $logFile = "$dir/readOswpFW.log";
  my $fhOut;
  my $usr;
  my $cmd;
  my @out;
  my $activeOswp = -1;
  my ($oswpNameAct, $oswpNamePas);
  my $oswpNameActFound = 0;
  my $oswpNamePasFound = 0;
  my $loggedIn = 0;
  my $loggedOut = 0;
  my $ecntaPresent = 0;
#SW management runstate and event
  my $runState = "";
  my $runEvent = "";
  my $initState = "";
  my $errRecRelease = "";
  my $actRelNum = 0;
  my %oswpInfo = ('ACTIVE' => { 'INDEX' => 0,
				'NAME' => NO_OSWP,
				'COMMITED' => "",
				'MAJOR' => 0,
				'MINOR' => 0,
				'SUFFIX' => 0,
				'MAINT' => 0,
				'REDUCED' => 0},
		  'PASSIVE' => { 'INDEX' => 0,
				 'NAME' => NO_OSWP,
				 'COMMITED' => "",
				 'MAJOR' => 0,
				 'MINOR' => 0,
				 'SUFFIX' => 0,
				 'MAINT' => 0,
				 'REDUCED' => 0},
		  'FRMW_OSWP' => 0,
		  'ACT_PENDING' => -1);

  if (!open($fhOut, ">>", $logFile)) { 
    print STDERR "Cannot open  '$logFile' for writing: $!\n "; 
    return CREATE_FILE_FAIL_CODE;
  }
        my @out1 = $ssh->cmd("");
        $usr = GetDebugUserIdentity($ssh);

	if (defined($usr)) {
        $cmd = "dbg suspend $usr";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         exit 0;
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(15);
	}

        $cmd = "dir /Dbase";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(3);

        $cmd = "eqpt displayAsam s";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(3);

        $cmd = "frmw get_oswp";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(2);

        $cmd = "dump /Dbase/SwMgnt.ini";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out\n";
        sleep(5);

        $cmd = "dbg resume EPRT";
        print $fhOut "$cmd\n";
        @out = $ssh->cmd("$cmd");
        if (!@out) {
  	 ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
         sleep(2);
         $ssh->buffer_empty();
         @out = $ssh->cmd("$cmd");
        }
        print $fhOut "Response received:@out";
        print $fhOut "@out";
        $cmd = "exit";
        print $fhOut "$cmd";
        sleep(1);
    close $fhOut;

    my $fhOut1;
    if (!open $fhOut1, "<" . $logFile) {
    print STDERR "Cannot open  '$logFile' for reading: $!\n "; 
    return FILE_OPEN_FAIL_CODE;
  }
    my $line;
    while (defined($line = <$fhOut1>)) {
    # read complete file
      chomp $line;
      #print "line:$line\n";
      $line =~ s/\r//g;
      ToolCommon::Log($fh, "'$line'\n") if $debug;
      $loggedIn = 1 if ($line =~ /\{nt\d\d\d\d\}\[USR\d+\-shell\]/);
      $loggedOut = 1 if ($line =~ /\{nt\d\d\d\d\}\[USR\d+\-shell\]exit/);
      if ($line =~ /\d\d\d\d\s*:\s*ECNT\-A/) {
	$ecntaPresent = 1;
	ToolCommon::Log($fh, "ECNT-A equipped\n") if $debug;
      } elsif ($line =~ /running oswp=(\d)/) {
	$oswpInfo{'FRMW_OSWP'} = $1;
	ToolCommon::Log($fh, "Stored '$oswpInfo{FRMW_OSWP}' as active FW OSWP number\n") if $debug;
      } elsif ($line =~ /ActivateOswpNr\s=\s(\d)/) {
	$oswpInfo{'ACT_PENDING'} = $1;
	ToolCommon::Log($fh, "Stored activation pending for OSWP '$oswpInfo{ACT_PENDING}'\n") if $debug;
      } elsif ($line =~ /ActiveOswp\s=\s(\d)/) {
	$activeOswp = $1;
	if ($ecntaPresent) {
	  # In case of wreset when actviating a new OSWP we run into the situation where the NT (on ECNT-A) is restarted with the new SW
	  # while the SHUB remains running with the old release.  When the system comes up, the SwMgnt.ini file indicates that the old
	  # OSWP is still active while firmware management indicates that the other OSWP is active.  In this case we use the information
	  # provided by firmware management to continue the parsing of the SwMgnt.ini file.
	  $activeOswp = $oswpInfo{'FRMW_OSWP'} if ($activeOswp != $oswpInfo{'FRMW_OSWP'});
	}
      } elsif ($line =~ /RunState\s=\s(\w+)/) {
	$runState = $1;
      } elsif ($line =~ /RunEvent\s=\s(\w+)/) {
	$runEvent = $1;
      } elsif ($line =~ /InitState\s=\s(\w+)/) {
	$initState = $1;
	$oswpInfo{'ACTIVE'}{'INDEX'} = $activeOswp;
	$oswpInfo{'PASSIVE'}{'INDEX'} = $activeOswp == 1 ? 2 : 1;
	$oswpNameAct = sprintf("\\[Oswp%d\\]", $activeOswp);
	$oswpNamePas = $activeOswp == 1 ? "\\[Oswp2\\]" : "\\[Oswp1\\]";
	ToolCommon::Log($fh, "oswpNameAct, oswpNamePas: '$oswpNameAct', '$oswpNamePas'\n") if $debug;
      } elsif ($line =~ /build name\s+:\s$regExp/) {
	# Regular error record
	my $thisRelNum = ($1*10 + $2) * 10 + $3;
	if ($thisRelNum > $actRelNum) {
	  $errRecRelease = "$1$2.$3$4";
	  ToolCommon::Log($fh, "Stored '$errRecRelease as last active release (err rec build name)\n") if $debug;
	} else {
	  ToolCommon::Log($fh, "Error record from same or previous release\n") if $debug;
	}
      } elsif ($line =~ /Application\s\/Sw\/$regExp\sstarted/) {
	# Application restart: is usually another release then the one found in the last error record (unless there was a prozone resize).
	$errRecRelease = "$1$2.$3$4";
	$actRelNum = ($1*10 + $2) * 10 + $3;
	ToolCommon::Log($fh, "Stored $errRecRelease as last active release (restart notification)\n") if $debug;
      }
      if ($activeOswp != -1) {
	if ($oswpNameActFound) {
	  if ($line =~ /$string\s=\s(.*)/) {
	    $oswpInfo{'ACTIVE'}{'NAME'} = $1;
	    if ($oswpInfo{'ACTIVE'}{'NAME'} =~ /$regExp/) {
	      $oswpInfo{'ACTIVE'}{'MAJOR'} = $1;
	      $oswpInfo{'ACTIVE'}{'MINOR'} = $2;
	      $oswpInfo{'ACTIVE'}{'SUFFIX'} = $3;
	      $oswpInfo{'ACTIVE'}{'MAINT'} = $4;
	    }
	  } elsif ($line =~ /CommitState\s=\s(.*)/) {
	    $oswpInfo{'ACTIVE'}{'COMMITED'} = $1;
	  }
	  $oswpNameActFound = 0 if (($oswpInfo{'ACTIVE'}{'MAJOR'} != 0) && ($oswpInfo{'ACTIVE'}{'COMMITED'} ne ""));
	} elsif ($oswpNamePasFound) {
	  if ($line =~ /$string\s=\s(.*)/) {
	    $oswpInfo{'PASSIVE'}{'NAME'} = $1;
	    if ($oswpInfo{'PASSIVE'}{'NAME'} =~ /$regExp/) {
	      $oswpInfo{'PASSIVE'}{'MAJOR'} = $1;
	      $oswpInfo{'PASSIVE'}{'MINOR'} = $2;
	      $oswpInfo{'PASSIVE'}{'SUFFIX'} = $3;
	      $oswpInfo{'PASSIVE'}{'MAINT'} = $4;
	    }
	  } elsif ($line =~ /CommitState\s=\s(.*)/) {
	    $oswpInfo{'PASSIVE'}{'COMMITED'} = $1;
	  }
	  $oswpNamePasFound = 0 if (($oswpInfo{'PASSIVE'}{'MAJOR'} != 0) && ($oswpInfo{'PASSIVE'}{'COMMITED'} ne ""));
	} else {
	  if (defined($oswpNameAct) && ($line =~ /$oswpNameAct/)) {
	    ToolCommon::Log($fh, "Found active OSWP\n") if $debug;
	    $oswpNameActFound = 1;
	  } elsif (defined($oswpNamePas) && ($line =~ /$oswpNamePas/)) {
	    ToolCommon::Log($fh, "Found passive OSWP\n") if $debug;
	    $oswpNamePasFound = 1;
	  }
	}
      }
    }
    close $fhOut1;
    # Check release from error records (if one was found versus the active OSWP as dertermined along the line
    if ($errRecRelease ne "") {
      if (!($oswpInfo{'ACTIVE'}{'NAME'} =~ /$errRecRelease/)) {
	# The release running as determined from the error records does not seem to be the active release... so it can only be the 'passive' one
	# which still seems to be active
	if ($oswpInfo{'PASSIVE'}{'NAME'} =~ /$errRecRelease/) {
	  # Swap active and passive - should normally not occur since this means that SW management is "in a mess"
	  ToolCommon::Log($fh, "***ALERT*** active and passive switched: SwMgnt.ini not in line with active SW as determined from error recs\n");
	  my $prevMaj = $oswpInfo{'ACTIVE'}{'MAJOR'};
	  my $prevMin = $oswpInfo{'ACTIVE'}{'MINOR'};
	  my $prevSfx = $oswpInfo{'ACTIVE'}{'SUFFIX'};
	  my $prevMnt = $oswpInfo{'ACTIVE'}{'MAINT'};
	  my $prevNme = $oswpInfo{'ACTIVE'}{'NAME'};
	  $oswpInfo{'ACTIVE'}{'INDEX'} = $oswpInfo{'PASSIVE'}{'INDEX'};
	  $oswpInfo{'ACTIVE'}{'NAME'} = $oswpInfo{'PASSIVE'}{'NAME'};
	  $oswpInfo{'ACTIVE'}{'MAJOR'} = $oswpInfo{'PASSIVE'}{'MAJOR'};
	  $oswpInfo{'ACTIVE'}{'MINOR'} = $oswpInfo{'PASSIVE'}{'MINOR'};
	  $oswpInfo{'ACTIVE'}{'SUFFIX'} = $oswpInfo{'PASSIVE'}{'SUFFIX'};
	  $oswpInfo{'ACTIVE'}{'MAINT'} = $oswpInfo{'PASSIVE'}{'MAINT'};
	  $oswpInfo{'PASSIVE'}{'INDEX'} = $oswpInfo{'ACTIVE'}{'INDEX'} == 1 ? 2 : 1;
	  $oswpInfo{'PASSIVE'}{'NAME'} = $prevNme;
	  $oswpInfo{'PASSIVE'}{'MAJOR'} = $prevMaj;
	  $oswpInfo{'PASSIVE'}{'MINOR'} = $prevMin;
	  $oswpInfo{'PASSIVE'}{'SUFFIX'} = $prevSfx;
	  $oswpInfo{'PASSIVE'}{'MAINT'} = $prevMnt;
	}
      }
    }
  ToolCommon::Log($fh,
		  sprintf("(string %s) - activeOswp: '%s', activeOswpName: %s\n",
			  $string, $oswpInfo{ACTIVE}{INDEX}, $oswpInfo{ACTIVE}{NAME})) if $debug;
  ToolCommon::Log($fh,
		  sprintf("(string %s) - passiveOswp: '%s', passiveOswpName: %s\n",
			  $string, $oswpInfo{PASSIVE}{INDEX}, $oswpInfo{PASSIVE}{NAME})) if $debug;
  $ERROR = "";
  if ($loggedIn) {
    $ERROR = $NT_DEAD if ! $loggedOut;
  } else {
    $ERROR = $NT_DEAD;
  }
  return \%oswpInfo;
}

sub GetDebugUserIdentity($) {
  my($ssh) = @_;
  my @out = $ssh->cmd("");
  my $usr;
  if($out[0] =~ /([^\[])\[(\w+\d+)\-/) {
     $usr = $2;
  }
  
 return $usr;
}


sub UpdateNewFs($$) {
  my ($ssh, $dir)=@_;
  my $logFile = "$dir/newfs_bpf.log";
  my $fhOut;
  my $cmd;
  my @out;


  if (!open($fhOut, '>', $logFile)) {
    print STDERR "Cannot open  '$logFile' for writing: $!\n ";
    return CREATE_FILE_FAIL_CODE;
  }
  $cmd = "newfs info";
  print $fhOut "$cmd\n";
  @out = $ssh->cmd("$cmd");
  if (!@out) {
   ToolCommon::Log($fhOut, "Empty response for the command : $cmd ...\n");
   sleep(2);
   $ssh->buffer_empty();
   @out = $ssh->cmd("$cmd");
  }
  print $fhOut "Response received:@out\n";
  my $respone_lineno = scalar @out;
  my ($header, $BPA_zone, $BPA_property, $BPB_zone, $BPB_property ) = ( "", 0, 0,0,0 );
  my $bp_fs_details = "";
  my ($flag_recover, $flag_boot_package, $flag_emer, $flag_rest_count ) = (0,0,0,0);
  my ($flag_setbp ) =  ( 1 );
  for (my $index=0; $index < $respone_lineno; $index++ ) {
    if( $out[$index] =~ /^===([^=]+)===$/ ) {
        $header = "$1";
        $header =~ s/^\s+//; $header =~ s/\s+$//;
    } elsif( $header ne "" ) {
        if( $header eq "BPA" ) {
	   if( $out[$index] =~ /^zone\s*(\d+)/ ) {
		$BPA_zone = $1;
	   }
           $BPA_property++;
        } elsif( $header eq "BPB" ) {
	   if( $out[$index] =~ /^zone\s*(\d+)/ ) {
		$BPB_zone = $1;
	   }
           $BPB_property++;
        } elsif( $header eq "RUNNING BP/FS" ) {
	   $bp_fs_details = "$out[$index]\n$out[$index+1]\n";
	   if( $bp_fs_details =~ /recover flag=(\w)\s*/ ) {
                $flag_recover=$1;
           }
	   if( $bp_fs_details =~ /boot package flag=(\w)\s*/ ) {
                $flag_boot_package=$1;
           }
	   if( $bp_fs_details =~ /emer flag=(\w)\s*/ ) {
		$flag_emer=$1;
           }
	   if( $bp_fs_details =~ /reset count=(\w)\s*/ ) {
		$flag_rest_count=$1;
           }
           last;
	}
   }
 }
 if( $BPB_property > $BPA_property ) {
    $flag_setbp = 2;
 } else {
    $flag_setbp = 1;
 }
 ToolCommon::Log($fhOut, "BP/FS details:- recover($flag_recover), boot package($flag_boot_package), emer($flag_emer), reset count($flag_rest_count)\n");
 if( $flag_boot_package eq "Unknown" ) {
     ToolCommon::Log($fhOut, "Boot package is set to Unknown, Going to clear now...\n");
     #clear (common area: recovery flag, bootpackage flag, emer flag, reset cnt)
     $cmd = "newfs clear $flag_recover $flag_setbp $flag_emer $flag_rest_count";
     print $fhOut "$cmd\n";
     @out = $ssh->cmd("$cmd");
     if (!@out) {
      ToolCommon::Log($fhOut, "Empty response for the command : $cmd ...\n");
      sleep(2);
      $ssh->buffer_empty();
      @out = $ssh->cmd("$cmd");
     }
     print $fhOut "Response received:@out\n";

 } 
 sleep(10);
}

sub ReduceSwFs($$) {
  my ($ssh, $dir)=@_;
  my $logFile = "$dir/reduce_sw.log";
  my $fhOut;
  my $cmd;
  my @out;

  if (!open($fhOut, '>', $logFile)) {
    print STDERR "Cannot open  '$logFile' for writing: $!\n ";
    return CREATE_FILE_FAIL_CODE;
  }

  $cmd = "swm swpktmgnt";
  print $fhOut "$cmd\n";
  @out = $ssh->cmd("$cmd");
  if (!@out) {
   ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
   sleep(2);
   $ssh->buffer_empty();
   @out = $ssh->cmd("$cmd");
  }
  print $fhOut "Response received:@out\n";
  sleep(2);

  $cmd = "swm rdcAoswp";
  print $fhOut "$cmd\n";
  @out = $ssh->cmd("$cmd");
  if (!@out) {
   ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
   sleep(2);
   $ssh->buffer_empty();
   @out = $ssh->cmd("$cmd");
  }
  sleep(2);

  $cmd = "swm back";
  print $fhOut "$cmd\n";
  @out = $ssh->cmd("$cmd");
  if (!@out) {
   ToolCommon::Log($fh, "Empty response for the command : $cmd ...\n");
   sleep(2);
   $ssh->buffer_empty();
   @out = $ssh->cmd("$cmd");
  }
  sleep(5);
}




1;
#-- End of library
