package StatusCheckCmpIsam7363;

=head1 HiCap ISAM 7302/7330 specific status check comparison module

=begin html
This script will compare the results from the status health-checks run prior to and after a SW upgrade or migration.  Differences are compared
using thresholds.  As long as the number of differences for a certain class remains below the threshold the migration is regarded as successful.
In case the threshold is crossed the error level associated to this check will influence the comparison result.  It can be regarded as either:
<ul>
  <li>To be ignored</li>
  <li>to be raised as a warning (migration/upgrade regarded as succesfull but operator needs to verify</li>
  <li>To be treated as an error.  Even if the node is running the new release, these errors may be a reason to declare a migration as failed
      (e.g. a(n) (number of) LT(s) is not operational after the migration/upgrade)</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         StatusCheckCmpIsam7363.pm
#-- Date:         06.25.2014
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#--               Thiruvalluvan 
#------------------------------------------------------------------------------
#-- Function:     Script to perform a series of stand-alone PBMT pre- and post-checks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2014
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   3.1.1  - 2008/10/08 : ALU01991113 - First implementation
#--   3.1.2  - 2015/12/02 : ALU02151998
#--   3.1.3  - 2016/07/15 : ALU02237966
#--   3.1.4  - 2017/05/12 : ALU02346977
#--   3.2.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.2.1  - 2018/12/06 : eliminate warnings generated in alarm section comparison

=head1 VERSION

=begin html
Current version: "StatusCheckCmpIsam7363.pm - 3.2.1 -- 2018/12/06"

=end html

=cut

my $VERSION = "StatusCheckCmpIsam7363.pm - 3.2.1 -- 2018/12/06";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';

require 5.001;

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;

use Utils;
use Capability;
use ErrorCodes qw(:EXIT_CODES :CHKC);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use PbmtConfig;
use version();
use vars qw(%version);
use ONT7302HiCap;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub StoreCheckResults($$);
sub CompareCheckResults($$);
sub Compare();
sub ResetGlobals();

#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;

my $LTSECTION   = "LT;Type;Status";
my $LINESECTION = "Line;SrvP;SpcP;adm-state;opr-state;tx-rate-us;max-tx-rate-us;tx-rate-ds;max-tx-rate-ds;inp-up;inp-dn;output-power-down";
my $BRDGSECTION = "PVC-ID;BridgeInfo";
my $AUTHSECTION = "PVC;auth-pae-state;auth-backend-state;port-status";
my $ALRMSECTION = "Outstanding alarms";
my $ONTSECTION  = "ONT Status";
my $PRE         = "PRE";
my $POST        = "POST";
my $NOSECTIONSTARTED = 0;
my $LTSECTIONSTARTED = 1;
my $LINESECTIONSTARTED = 2;
my $BRDGSECTIONSTARTED = 3;
my $AUTHSECTIONSTARTED = 4;
my $ALRMSECTIONSTARTED = 5;
my $ONTSECTIONSTARTED = 6;
my @section = ("No section", "LT section", "Line section", "Bridge section", "Authentication section", "Alarm section", "Ont section" );
my $ontInfoPresent = 0;
#==============================================================================
#== Vars
#==============================================================================
my $warnLevels;
my $toolData;

my %ltStates = ();
my %ltStateMM = ("configured" => 0,
		 "down_before" => 0,
		 "down_after" => 0);
my %lineStates = ();
my %lineStateMM = ("configured" => 0,
		   "down_before" => 0,
		   "down_after" => 0);
my %authStates = ();
my %authStateMM = ("configured" => 0,
		   "no_auth_before" => 0,
		   "no_auth_after" => 0);
my %alarms = ();
my %bridgeStates = ();
my %bridgeConfMM = ("configured" => 0,
		    "mm" => 0);
my %ontStatusPre;
my %ontStatusPost;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

sub ResetGlobals() {
   $ontInfoPresent = 0;
}


=head2 StoreCheckResults($$$)

=begin html
Store the check results in a hash table for later comparison.  The file is parsed on the different checks that can be present and is storing the
parsed information appropriately.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>fileName</td><td>the name of the file to be read</td></tr>
  <tr><td>params</td><td>the command options as passed to the script</td></tr>
</table>

=end html

=cut

sub StoreCheckResults($$) {
  my ($fh, $fileName) = @_;
  my $err = "";
  my $ok = 1;
  my $preOperCheck = PREOPERCHECKFILETAG; 
  my $state = $fileName =~ /$preOperCheck/ ? $PRE : $POST;
  my $section = $NOSECTIONSTARTED;

  if (open(FILE, $fileName)) {
    ToolCommon::Log($fh, "Reading from '$fileName'\n") if $params->{DEBUG};
    while (defined(my $line = <FILE>) && $ok) {
      chomp $line;
      ToolCommon::Log($fh, "[$state] Current section: $section[$section]\n") if $params->{DEBUG};
      if ($line =~ /$LTSECTION/) {
	$section = $LTSECTIONSTARTED;
      } elsif ($line =~ /$LINESECTION/) {
	$section = $LINESECTIONSTARTED;
      } elsif ($line =~ /$BRDGSECTION/) {
	$section = $BRDGSECTIONSTARTED;
      } elsif ($line =~ /$AUTHSECTION/) {
	$section = $AUTHSECTIONSTARTED;
      } elsif ($line =~ /$ALRMSECTION/) {
	$section = $ALRMSECTIONSTARTED;
      } elsif ($line =~ /$ONTSECTION/) {
        $section = $ONTSECTIONSTARTED;
        $ontStatusPre{RELEASE}=undef if $state eq "PRE";
        $ontStatusPost{RELEASE}=undef if $state eq "POST";

      } else {
	if ($section == $LTSECTIONSTARTED) {
	  if ($line =~ /(\d+\/\d+\/\d+);\w+\-\w;(.*)/) {
	    my $lt = $1;
	    my $ltState = $2;
	    $ltStates{$lt}{$state} = $ltState;
	    if ($state eq $PRE) {
	      $ltStateMM{configured}++;
	      $ltStateMM{down_before}++ if ($ltState =~ /DOWN/);
	    } else {
	      $ltStateMM{down_after}++ if ($ltState =~ /DOWN/);
	    }
	    ToolCommon::Log($fh, "[$state] Storing state '$ltState' for LT '$lt'\n") if $params->{DEBUG};
	  } else {
	    $err = "LT section not according to expected syntax ($line)";
	    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	    $ok = 0;
	  }
	} elsif ($section == $LINESECTIONSTARTED) {
	  if ($line =~ /(\d+\/\d+\/\d+)\/(\d+);(.*)/) {
	    # 1/1/6/33;1;5;no admin-up;Down;0;0;0;0;0;0;0
	    my $lt = $1;
	    my $line = $2;
	    my $info = $3;
	    my $lineId = "$lt/$line";
            if (($info =~ /\d+;\d+;(admin-up|no admin-up);(Up|Down);.*/) || ($info =~ /\d+;\d+;(admin-up|no admin-up);(-);.*/)) {
	      my $adminState = $1;
	      my $operState = $2;
              $operState = "Down" if($operState =~ /^-/); 
	      $lineStates{$lineId}{$state} = $info;
              if ($state eq $PRE) {
                if($params->{INCLLTOOS}) {
                        $lineStateMM{configured}++  if($adminState !~  /no admin-up/);
                        $lineStateMM{down_before}++ if (($operState =~ /Down/) && ($adminState =~ /^admin-up/));
                } else {
                        if(defined($ltStates{$lt}{$state}) && ($ltStates{$lt}{$state} =~ /UP/)) {
                                $lineStateMM{down_before}++ if (($operState =~ /Down/) && ($adminState =~ /^admin-up/));
                                $lineStateMM{configured}++ if($adminState !~  /no admin-up/);
                        }
                }
              } else {
                if($params->{INCLLTOOS}) {
                        $lineStateMM{down_after}++ if (($operState =~ /Down/) && ($adminState =~ /^admin-up/));
                } else {
                        if(defined($ltStates{$lt}{$state}) && ($ltStates{$lt}{$state} =~ /UP/)) {
                                $lineStateMM{down_after}++ if (($operState =~ /Down/) && ($adminState =~ /^admin-up/));
                        } elsif(defined($ltStates{$lt}{$state}) && ($ltStates{$lt}{$state} =~ /DOWN/)) {
                                if(defined($ltStates{$lt}{$PRE}) && ($ltStates{$lt}{$PRE} =~ /UP/)) {
                                     $lineStateMM{down_after}++ if (($operState =~ /Down/) && ($adminState =~ /^admin-up/));
                                }
                        }
                }
              }

	      ToolCommon::Log($fh, "[$state] Storing info '$info' for line '$lineId'\n") if $params->{DEBUG};
	    } else {
	      $err = "Line info section not according to expected syntax ($info)";
	      FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	      $ok = 0;
	    }
	  } else {
	    $err = "Line section not according to expected syntax ($line)";
	    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	    $ok = 0;
	  }
	} elsif ($section == $ALRMSECTIONSTARTED) {
          if (index($line,',') != -1) {
            my @alarm_values = split(/,/, $line );
            my $mandate_field = join( ',', @alarm_values[0 .. 3]);
	    $alarms{$mandate_field}{$state} = $line;
	    ToolCommon::Log($fh, "[$state] Storing '$mandate_field' -  '$line'\n") if $params->{DEBUG};
          } else {
            $alarms{$line}{$state} = 1; 
            ToolCommon::Log($fh, "[$state] Storing '$line'\n") if $params->{DEBUG};
          }
	} elsif ($section == $AUTHSECTIONSTARTED) {
	  if (($line =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+);(\w+);\w+;(\w+)/) ||  ($line =~ /(\d+\/\d+\/\d+\/\d+);(\w+);\w+;(\w+)/) || ($line =~ /(\d+\/\d+\/\d+\/\d+);(\w+\-\w+);\w+;(\w+)/)) {
	    my $pvcId = $1;
	    my $authState = $2;
	    my $portState = $3;
	    my $authenticated = ((uc($authState) eq uc("authenticated")) && (uc($portState) eq uc("authorized"))) ? 1 : 0;
	    $authStates{$pvcId}{$state} = $authenticated;
	    if ($state eq $PRE) {
	      $authStateMM{configured}++;
	      $authStateMM{no_auth_before}++ if ((! $authenticated) && ($params->{INCLLTOOS}));
	    } else {
	      $authStateMM{no_auth_after}++ if ((! $authenticated) && ($params->{INCLLTOOS}));
	    }
	    ToolCommon::Log($fh, "[$state] Storing authentication state '$authenticated' for bridge '$pvcId'\n") if $params->{DEBUG};
	  } else {
	    $err = "Authentication section not according to expected syntax ($line)";
	    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	  }
        }elsif( $section == $ONTSECTIONSTARTED) {
          $ontInfoPresent = 1; 
          if($line =~ /^(RELEASE)/) {
            $ok=1;
            ToolCommon::Log($fh, "[$state] Storing '$line' \n") if $params->{DEBUG};
            my($rel,$value)=split(/:/, $line);
            $rel =~s/ //g;
            $value=~s/ //g;

           if(defined($rel) && (uc($rel) eq "RELEASE") ){
             if (defined($value) && ($value =~ /R\d{2}\.\d{2}\.\d{2}/) ){
               if ($state eq "PRE") { $ontStatusPre{RELEASE} = $value; } else { $ontStatusPost{RELEASE} = $value; }
               }
             }
          } elsif ($line =~ /^#/ || $line eq "" ){
            next;
          } else {
            my($aid_ont,@end) = split(/:/, $line);
            ToolCommon::Log($fh, "[$state] Storing '$line' \n") if $params->{DEBUG}; 
            foreach my $str (@end) {
              my ($par, $value) = split(/=/, $str);
              if (defined($value) || defined($par) ne "") {
                if ($state eq "PRE") { $ontStatusPre{$aid_ont}{$par} = $value; } else { $ontStatusPost{$aid_ont}{$par} = $value; }
                }
              }
            }
	} else {
	  # $section == $BRDGSECTIONSTARTED
	  if (($line =~ /(\d+\/\d+\/\d+\/\d+:\d+:\d+);(.*)/) ||
	      ($line =~ /(\d+\/\d+\/\d+\/\d+);(.*)/)) {
	    my $bridgeId = $1;
	    my $info = $2;
	    $bridgeConfMM{configured}++ if ($state eq $PRE);
	    $bridgeStates{$bridgeId}{$state} = $info;
	    ToolCommon::Log($fh, "[$state] Storing info '$info' for bridge '$bridgeId'\n") if $params->{DEBUG};
	  } else {
	    $err = "Bridge section not according to expected syntax ($line)";
	    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	  }
	}
      }
    }
    close FILE;
  } else {
    $err = "Failed to open $fileName for reading: $!";
    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
    $ok = 0;
  }
  return($ok);
}

=head2 CompareCheckResults($$$)

=begin html
Compare the check results as stored in a hash table.  In case a threshold is exceeded the mismatch is either ignored, treated as a warning or
treated as an error depending on the error level as defined in the warning-level file.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>the command options as passed to the script</td></tr>
  <tr><tD>isam</td><td>the ISAM identification (IP address or name)</td></tr>
</table>

=end html

=cut

sub CompareCheckResults($$) {
  my ($fh, $isam) = @_;
  my $ok = 1;
  my ($downAfterPct, $downBeforePct);
  my $str = "";
  my $card_pulled = undef;
  my $lt_down_identifier = undef;
  my $state_change_identifier = undef;
  my $have_lt_cards = 0; #ALU01917501

  foreach my $lt (sort { my $a1 = $a; $a1 =~ s/\d+\/\d+\///g;
			 my $b1 = $b; $b1 =~ s/\d+\/\d+\///g; $a1 <=> $b1 }
		  keys %ltStates) {
    if (defined($ltStates{$lt}{$PRE}) && defined($ltStates{$lt}{$POST})){
      if ($ltStates{$lt}{$PRE} ne $ltStates{$lt}{$POST}) {
	if ($ltStates{$lt}{$PRE} eq "UP") {
	  ToolCommon::Log($fh, "  LT $lt not operational after migration\n");
          $lt_down_identifier = 1;
	}
      } else {
	 $state_change_identifier = 1;
      }
    } else {
      if(defined($ltStates{$lt}{$PRE})) {
	ToolCommon::Log($fh,"LT conf mismatch for $lt: '$ltStates{$lt}{$PRE}' before vs. not present after\n"); 
	#$ltStateMM{"down_after"}++ if($ltStates{$lt}{$PRE} eq "UP"); 
	$ltStateMM{"down_after"}++; 
        $card_pulled = 1;
      } else {
	# Unlikely to happen.
	if(defined($ltStates{$lt}{$POST})) {
	  ToolCommon::Log($fh,"LT conf mismatch for $lt: 'not present' before vs. '$ltStates{$lt}{$POST}' after\n"); 
	  $ltStateMM{"down_before"}++ if($ltStates{$lt}{$POST} eq "UP"); 
	}
      }
    }
  $have_lt_cards = 1; 
  }
  $downAfterPct = $ltStateMM{"configured"} != 0 ? ($ltStateMM{"down_after"} * 100) / $ltStateMM{"configured"} : 0;
  $downBeforePct = $ltStateMM{"configured"} != 0 ? ($ltStateMM{"down_before"} * 100) / $ltStateMM{"configured"} : 0;  
  $str = sprintf("%.2f%% LTs more OOS after migration (%d out of %d or %.2f%% after vs. %d out of %d or %.2f%% before)",
		 abs($downBeforePct - $downAfterPct),
		 $ltStateMM{"down_after"}, $ltStateMM{"configured"}, $downAfterPct,
		 $ltStateMM{"down_before"}, $ltStateMM{"configured"}, $downBeforePct);
  if ($downAfterPct >= $downBeforePct) {
   if ( $have_lt_cards ) {
    if ( !defined($state_change_identifier) || defined ($card_pulled) || defined ($lt_down_identifier) ) {
     if ((!($downAfterPct eq 0) || !($downBeforePct eq 0)) && (($downAfterPct - $downBeforePct) > $toolData->{LTOOSPCT} || (($downAfterPct - $downBeforePct) eq 0))) {
      if (! ErrorCodes::WarningLevelIsIgnore($warnLevels->{&EC_LTOOS_THR_EXCEEDED()})) {
	if (ErrorCodes::WarningLevelIsError($warnLevels->{&EC_LTOOS_THR_EXCEEDED()})) {
	  FamilyCommon::IncrErrorCount("ERROR: $str\n",
				       EC_LTOOS_THR_EXCEEDED,
				       $str,
				       undef);
	  $ok = 0;
	} else {
	  FamilyCommon::IncrErrorCount("WARNING: $str\n",
				       EC_LTOOS_THR_EXCEEDED,
				       $str,
				       undef);
	}
      } else {
	ToolCommon::Log($fh, "IGNORED: $str\n");
      }
     }
    }
   }
  } else {
  ToolCommon::Log($fh, "$str\n");
  }
  foreach my $line (sort { my $a1 = $a; $a1 =~ s/\//\./g;
			   my $b1 = $b; $b1 =~ s/\//\./g;
			     pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
		    keys %lineStates) {
    if (defined($lineStates{$line}{$PRE}) &&  defined($lineStates{$line}{$POST})) {
      if (($lineStates{$line}{$PRE} =~ /Up/) && ! ($lineStates{$line}{$POST} =~ /Up/)) {
        ToolCommon::Log($fh, "  Line $line not up after migration\n") if defined($params->{DETAILED});
      }
    }
  }
  $downAfterPct = $lineStateMM{"configured"} != 0 ? ($lineStateMM{"down_after"} * 100) / $lineStateMM{"configured"} : 0;
  $downBeforePct = $lineStateMM{"configured"} != 0 ? ($lineStateMM{"down_before"} * 100) / $lineStateMM{"configured"} : 0;
  $str = sprintf("%.2f%% lines more OOS after migration (%d out of %d or %.2f%% after vs. %d out of %d or %.2f%% before)",
		 abs($downBeforePct - $downAfterPct),
		 $lineStateMM{"down_after"}, $lineStateMM{"configured"}, $downAfterPct,
		 $lineStateMM{"down_before"}, $lineStateMM{"configured"}, $downBeforePct);
  if ($downBeforePct > $downAfterPct) {
    if (($downBeforePct - $downAfterPct) > $toolData->{LINEOOSPCT}) {
      if (! ErrorCodes::WarningLevelIsIgnore($warnLevels->{&EC_LINEOOS_THR_EXCEEDED()})) {
	if (ErrorCodes::WarningLevelIsError($warnLevels->{&EC_LINEOOS_THR_EXCEEDED()})) {
	  FamilyCommon::IncrErrorCount("ERROR: $str\n",
				       EC_LINEOOS_THR_EXCEEDED,
				       $str,
				       undef);
	  $ok = 0;
	} else {
	  FamilyCommon::IncrErrorCount("WARNING: $str\n",
				       EC_LINEOOS_THR_EXCEEDED,
				       $str,
				       undef);
	}
      } else {
	ToolCommon::Log($fh, "IGNORED: $str\n");
      }
    }
  } else {
    ToolCommon::Log($fh, "$str\n");
  }

  foreach my $line (sort { my $a1 = $a; $a1 =~ s/\//\./g;
			   my $b1 = $b; $b1 =~ s/\//\./g;
			     pack('C4', $a1 =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
		    keys %authStates) {
    if (defined($authStates{$line}{$PRE}) && defined($authStates{$line}{$POST})) {
      if (($authStates{$line}{$PRE}) && ! ($authStates{$line}{$POST})) {
        ToolCommon::Log($fh, "  Line $line not authenticated after migration\n") if defined($params->{DETAILED});
      }
    }
  }
  $downAfterPct = $authStateMM{"configured"} != 0 ? ($authStateMM{"no_auth_after"} * 100) / $authStateMM{"configured"} : 0;
  $downBeforePct = $authStateMM{"configured"} != 0 ? ($authStateMM{"no_auth_before"} * 100) / $authStateMM{"configured"} : 0;
  $str = sprintf("%.2f%% lines more not authenticated after migration (%d out of %d or %.2f%% after vs. %d out of %d or %.2f%% before)",
		 abs($downBeforePct - $downAfterPct),
		 $authStateMM{"no_auth_after"}, $authStateMM{"configured"}, $downAfterPct,
		 $authStateMM{"no_auth_before"}, $authStateMM{"configured"}, $downBeforePct);
  if ($downAfterPct > $downBeforePct) {
    if (($downAfterPct - $downBeforePct) > $toolData->{AUTHOOSPCT}) {
      if (! ErrorCodes::WarningLevelIsIgnore($warnLevels->{&EC_AUTHOOS_THR_EXCEEDED()})) {
	if (ErrorCodes::WarningLevelIsError($warnLevels->{&EC_AUTHOOS_THR_EXCEEDED()})) {
	  FamilyCommon::IncrErrorCount("ERROR: $str\n",
				       EC_AUTHOOS_THR_EXCEEDED,
				       $str,
				       undef);
	  $ok = 0;
	} else {
	  FamilyCommon::IncrErrorCount("WARNING: $str\n",
				       EC_AUTHOOS_THR_EXCEEDED,
				       "WARNING: $str",
				       undef);
	}
      } else {
	ToolCommon::Log($fh, "IGNORED: $str\n");
      }
    }
  } else {
    ToolCommon::Log($fh, "$str\n");
  }

  foreach my $line (sort { my $a1 = $a; $a1 =~ s/\//\./g;
			   my $b1 = $b; $b1 =~ s/\//\./g;
			     pack('C4', $a1 =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
		    keys %bridgeStates) {
    if (defined($bridgeStates{$line}{$PRE}) && defined($bridgeStates{$line}{$POST})) {
      if (($bridgeStates{$line}{$PRE}) ne ($bridgeStates{$line}{$POST})) {
		  if(!(($bridgeStates{$line}{$PRE} =~ /network-vlan/) && ($bridgeStates{$line}{$POST} =~ /l2fwder-vlan/))) {
	ToolCommon::Log($fh, "  Bridge conf mismatch for $line: '$bridgeStates{$line}{$PRE}' before vs. '$bridgeStates{$line}{$POST}' after\n");
	$bridgeConfMM{mm}++;
		}
	  }
    } else {
      if (defined($bridgeStates{$line}{$PRE})) {
	ToolCommon::Log($fh, "  Bridge conf mismatch for $line: '$bridgeStates{$line}{$PRE}' before vs. not present after\n");
      } else {
	ToolCommon::Log($fh, "  Bridge conf mismatch for $line: 'not present' before vs. '$bridgeStates{$line}{$POST}' after\n");
      }
      $bridgeConfMM{mm}++;
    }
  }
  $downAfterPct = $bridgeConfMM{"configured"} != 0 ? ($bridgeConfMM{"mm"} * 100) / $bridgeConfMM{"configured"} : 0;
  $str = sprintf("%.2f%% bridges have configuration mismatch", $downAfterPct);
  if ($downAfterPct > $toolData->{BRIDGEMMPCT}) {
    if (! ErrorCodes::WarningLevelIsIgnore($warnLevels->{&EC_BRIDGECONFMM_THR_EXCEEDED()})) {
      if (ErrorCodes::WarningLevelIsError($warnLevels->{&EC_BRIDGECONFMM_THR_EXCEEDED()})) {
	FamilyCommon::IncrErrorCount("ERROR: $str\n",
				     EC_BRIDGECONFMM_THR_EXCEEDED,
				     $str,
				     undef);
	$ok = 0;
      } else {
	FamilyCommon::IncrErrorCount("WARNING: $str\n",
				     EC_BRIDGECONFMM_THR_EXCEEDED,
				     "WARNING: $str",
				     undef);
      }
    } else {
      ToolCommon::Log($fh, "IGNORED: $str\n");
    }
  } else {
    ToolCommon::Log($fh, "$str\n");
  }


  my $newAlarms = 0;
  foreach my $alarm (keys %alarms) {
    if (defined($alarms{$alarm}{$POST}) && ! defined($alarms{$alarm}{$PRE})) {
      ToolCommon::Log($fh, "  New alarm popped up after migration: $alarms{$alarm}{$POST}\n");
      $newAlarms++;
      $ok = 0;
    }
  }
  if ($newAlarms) {
    $str = "$newAlarms new alarm(s)";
    if (! ErrorCodes::WarningLevelIsIgnore($warnLevels->{&EC_NEW_ALARM()})) {
      if (ErrorCodes::WarningLevelIsError($warnLevels->{&EC_NEW_ALARM()})) {
	FamilyCommon::IncrErrorCount("ERROR: $str\n",
				     EC_NEW_ALARM,
				     $str,
				     undef);
	$ok = 0;
      } else {
	FamilyCommon::IncrErrorCount("WARNING: $str\n",
				     EC_NEW_ALARM,
				     $str,
				     undef);
      }
    } else {
      ToolCommon::Log($fh, "IGNORED: $str\n");
    }
  }
  #ont related comparison
  my $warncount = 0;
  my $cnt =0;
  my $ontSwInfoPost;
  my $statusfile = FamilyCommon::GetFailontListFile();
  my $ontConfMismatch = 0;
  
  foreach my $aid (sort keys %ontStatusPre)
  {
    if($aid ne "RELEASE"){
      $cnt++;
    }
  }
   
      
  if(defined($ontStatusPost{RELEASE})){
    my $ontSwInfo=ONT7302HiCap::rel2type_hw_var_sw($ontStatusPost{RELEASE});
    if(defined($ontSwInfo)){
      foreach my $mnem (sort keys(%$ontSwInfo)){
        my $mnemMod=$mnem;
        $ontSwInfoPost->{$mnemMod}=$ontSwInfo->{$mnem};
      }
    } else {
      FamilyCommon::IncrErrorCount("No information in ONT_Release_Mapping.txt for RELEASE : $ontStatusPost{RELEASE}\n",
                                    EC_RTRV_REL_INFO_FAILED,undef,1);
      return 0;
    }
  } else {
    if ($ontInfoPresent) {
      #Report error only in case ONT section is present in the post file.
      FamilyCommon::IncrErrorCount("Missing in post file [RELEASE]\n", EC_RTRV_REL_FAILED, undef, 1);
      return 0;
    }
  }
  
  foreach my $aid (sort keys %ontStatusPre) {
    if (exists($ontStatusPost{$aid}) && $aid ne "RELEASE") {
      foreach my $prm (sort keys %{$ontStatusPre{$aid}}) {
        if(uc($prm) eq "SWVERACT" ) {
          if (exists($ontStatusPost{$aid}{$prm}) ){
            my $matches=0;
            my $mnemonicPost= (defined($ontStatusPost{$aid}{MNEMONIC}))?$ontStatusPost{$aid}{MNEMONIC}:undef;
            my $eqptPost= (defined($ontStatusPost{$aid}{EQPTVERNUM}))?substr($ontStatusPost{$aid}{EQPTVERNUM},0,10):undef;
            my $voipType= (defined($ontStatusPre{$aid}{VOIP}))?$ontStatusPre{$aid}{VOIP}:undef;
            if(defined($mnemonicPost) && defined($voipType) && defined($eqptPost)){
              if(uc($prm) eq "SWVERACT"){
                if(defined($ontSwInfoPost->{$mnemonicPost}{$eqptPost}{$voipType})){
                  if($ontSwInfoPost->{$mnemonicPost}{$eqptPost}{$voipType} eq $ontStatusPost{$aid}{$prm})
                  {
                      $matches=1;
                  }
                }
              }
            }
            if(!$matches) {
              print $statusfile "No Predictable Difference ["  . $aid . "] : pre value for " .  $prm ."=" . 
              $ontStatusPre{$aid}{$prm}. " post value for " . $prm . "=" .$ontStatusPost{$aid}{$prm} ."\n";
              $ontConfMismatch=1;
            }
          }
        }
        elsif(uc($prm) eq "SWVERPSV")
        {
          if(exists($ontStatusPost{$aid}{$prm})){
             if($ontStatusPre{$aid}{$prm} eq $ontStatusPost{$aid}{$prm}){
                if(defined($ontStatusPre{$aid}{SWVERACT}) && defined($ontStatusPost{$aid}{SWVERACT})
                   && ($ontStatusPre{$aid}{SWVERACT} ne $ontStatusPost{$aid}{SWVERACT})){
                   print $statusfile "No Predictable Difference ["  . $aid . "] : pre value for " .  $prm ."=" . 
                   $ontStatusPre{$aid}{$prm} . " post value for " . $prm . "=" .$ontStatusPost{$aid}{$prm}."\n";
                   $ontConfMismatch=1;
               }
             } else {
               if(defined($ontStatusPre{$aid}{SWVERACT}) && ($ontStatusPost{$aid}{$prm} ne $ontStatusPre{$aid}{SWVERACT}) ) {
                 print $statusfile  "No Predictable Difference [" . $aid ."] : pre value for " .  $prm ."=" . 
                 $ontStatusPre{$aid}{$prm} . " post value for " . $prm . "=" .$ontStatusPost{$aid}{$prm} ."\n";
                 $ontConfMismatch=1;    
               }
             }
          }
        } else {                 
          if (uc($prm) eq "STATE") {
            if ( uc(($ontStatusPre{$aid}{$prm}))  ne uc(($ontStatusPost{$aid}{$prm}))  ) {
              $warncount++;
              $ontConfMismatch=1;
              print $statusfile "Mismatch occurred ["  . $aid . "] : pre value for " .  $prm ."=" .
                                     $ontStatusPre{$aid}{$prm} . " post value for "
                                     . $prm . "=" .$ontStatusPost{$aid}{$prm} ."\n";
            }
          } elsif(uc($prm) ne "VOIP") {
            if (exists($ontStatusPost{$aid}{$prm}) ){
              if ( uc(($ontStatusPre{$aid}{$prm}))  ne uc(($ontStatusPost{$aid}{$prm}))  ) {
                print $statusfile "Mismatch occurred ["  . $aid . "] : pre value for " .  $prm ."=" .
                                      $ontStatusPre{$aid}{$prm} . " post value for "
                                      . $prm . "=" .$ontStatusPost{$aid}{$prm} ."\n";
                $ontConfMismatch=1;
              }
            }
          }
        }      
      }
      if($ontConfMismatch == 1){
        ToolCommon::Log($fh, "Configuration mismatch for ont : $aid\n");
        $ontConfMismatch = 0; 
      } 
    } else {
      if($aid ne "RELEASE"){
        print $statusfile "Missing in post file aid  ["  . $aid . "]\n";
      }
    }
  }

  my $ontoutage = $cnt > 0 ? ($warncount/$cnt)*100 : 0;
  $str = sprintf("%.2f%% ONTs found to be varied between pre and post results ", $ontoutage);
  if ($ontoutage > $params->{WARNTHRESHOLD} ) {
    if (! ErrorCodes::WarningLevelIsIgnore($warnLevels->{&EC_LTOOS_THR_EXCEEDED()})) {
      if (ErrorCodes::WarningLevelIsError($warnLevels->{&EC_LTOOS_THR_EXCEEDED()})) {
            FamilyCommon::IncrErrorCount("ERROR: $str\n",
                                         EC_ONTOOS_THR_EXCEEDED,
                                         $str,
                                         undef);
            $ok = 0;
      } else {
      FamilyCommon::IncrErrorCount("WARNING: $str\n",
                                    EC_ONTOOS_THR_EXCEEDED,
                                    $str,
                                    undef);
      }
    } else {
    ToolCommon::Log($fh, "IGNORED: $str\n");
    }
  } else {
    ToolCommon::Log($fh, "$str\n");
  }
  return $ok;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Compare($)

=begin html
The main body of the scipt defining the complete status-check process:
<ul>
  <li>Read, parse and store pre-check results</li>
  <li>Read, parse and store post-check results</li>
  <li>Compare pre- and post-checks results and report analysis as success, warning or error.</li>
</ul>
<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub Compare() {
  my $dir = FamilyCommon::GetNeStorage();
  my $failedNode = FamilyCommon::GetFailedNode();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $toolWarningData = FamilyCommon::GetWarningLevels();

  $toolData = $toolConfigData;
  $warnLevels = $toolWarningData;

  %ltStates = ();  
  %lineStates = (); 
  %authStates = (); 
  %alarms = (); 
  %bridgeStates = (); 

  Utils::ResetHash(\%ltStateMM, HASHDEFVAL); 
  Utils::ResetHash(\%lineStateMM, HASHDEFVAL); 
  Utils::ResetHash(\%authStateMM, HASHDEFVAL); 
  Utils::ResetHash(\%bridgeConfMM, HASHDEFVAL);

  if (StoreCheckResults($nodeLogFH, sprintf("%s/%s-%s.txt", $dir, PREOPERCHECKFILETAG, Capability::IsamHiCapEtsiFamily()))) {
    if (StoreCheckResults($nodeLogFH, sprintf("%s/%s-%s.txt", $dir, POSTOPERCHECKFILETAG, Capability::IsamHiCapEtsiFamily()))) {
      CompareCheckResults($nodeLogFH, $failedNode);
    } else {
      FamilyCommon::IncrErrorCount(sprintf("Failed to store post-check results: %s\n", FamilyCommon::GetError()),
				   EC_OPEN_POST_CHECK_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
    }
  } else {
    FamilyCommon::IncrErrorCount(sprintf("Failed to store pre-check results: %s\n", FamilyCommon::GetError()),
				 EC_OPEN_PRE_CHECK_FAILED,
				 sprintf(": %s", FamilyCommon::GetError()),
				 1);
  } 
}

1;
