package StatusCheckIsam7302;

=head1 ISAM 7302/7330-specific status check module

=begin html
This library contains the functions dedicated to perform status checks for a 7302/7330 ISAM.

=end html

=cut

#==============================================================================
#-- File:         StatusCheckIsam7302.pm
#-- Date:         25.10.2006
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to perform a series of stand-alone PBMT pre- and post-checks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2006/10/25 : ANTms14531, ANTms23976 - First implementation
#--   1.0.1 - 2007/04/25 : ANTms44293 - include LT status information.
#--   1.0.2 - 2007/05/04 : Small correction in Ctrl-c handling (don't kill pid
#--                        that have already exited)
#--   1.0.3 - 2007/06/18 : ANTms51632 allow multiple users for same ISAM.
#--   1.0.4 - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.0.5 - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.0.6 - 2007/08/22 : ANTms60869: Make NT community string configurable
#--   1.0.7 - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.0.8 - 2008/05/29 : ANTmt09138.  LT numbering starts from 1 in type-based
#--                        operator profiles.  Incorrect LT number is used to calculate
#--                        an ifIndex.
#--   1.0.9 - 2008/06/13 : ANTmt09138.  Solution not complete: on FD the first LT is always 1
#--                                     whatever CLI command-mode is used.
#--   1.0.10 - 2008/08/18: Remove call to retrieve first LT offset.
#--                        Clearer trace when determining CLI LT slotnumber mode.
#--   1.1.0
#--   1.1.1 - 2009/04/27 : Use ToolCommon::GetError (not IsamCli::GetError) after calling ToolCommon::OpenCli
#--   1.1.2 - 2009/05/07 : Prevent an empty line for bridge ports in case none are configured.
#--   1.1.3 - 2009/05/25 : ANTmt59034, ANTmt59036
#--   1.1.4 - 2009/05/25 : ANTmt60011
#--   1.1.5 - 2009/06/11 : Disable debug logging if no debug option was provided (for TL1/SSH).
#--   1.1.6 - 2010/12/06 : ALU00926734
#--   1.1.7 - 2011/09/22 : ALU01393253
#--   1.1.8 - 2013/09/23 : ALU01903046
#--   1.1.9 - 2015/08/05 : ALU02110266
#--   1.1.10 - 2015/01/07 : ALU02160776
#--   1.1.11 - 2016/02/15 : ALU02174145
#--   1.1.12 - 2016/07/15 : ALU02237966
#--   2.0.0  - 2018/06/20 : ALU02460863 & ALU02471697



=head1 VERSION

=begin html
Current version: "StatusCheckIsam7302.pm - 2.0.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "StatusCheckIsam7302.pm - 2.0.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2102-DFZZA">perlTL1::UdpChannel</a></li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
require 5.001;

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;

use Net::SNMP;
use Net::TFTP;
use perlTL1::UdpChannel;
use perlTL1::SshChannel;

use IsamCli;
use Crypt::Lite;
use Utils;
use Capability;
use PbmtConfig;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :SCHK);
use version();
use vars qw(%version);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub StoreLtStatus($$$$);
sub GetXdslOperData($$$$$$);
sub StoreXdslLineStatus($$$$$$$);
sub StoreBridgeConfig($$$$);
sub StoreAuthenticationInfo($$$$);
sub SendTL1Command($$$$$$);
sub StoreOutstandingAlarms($$$);
sub Check($$);
#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;

my %stateStrings = ('1' => 'Up',
		    '2' => 'Down',
		    '3' => 'Testing');
#==============================================================================
#== Vars
#==============================================================================
my $toolData;
my $config;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 StoreLtStatus($$$$)

=begin html
Retrieve and store LT status information.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>pointer to an opened CLI session</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>statusFile</td><td>reference to the result file</td></tr>
</table>

=end html

=cut

sub StoreLtStatus($$$$) {
  my ($fh, $cli, $debug, $statusFile) = @_;
  my $ltInfo = IsamCli::GetLtConfigState($cli);
  my $ok = 1;
  if (defined($ltInfo)) {
    print $statusFile "LT;Type;Status\n";
    foreach my $lt (sort { my $a1 = $a; $a1 =~ s/\d+\/\d+\///g;
			   my $b1 = $b; $b1 =~ s/\d+\/\d+\///g; $a1 <=> $b1 }
		    keys %{$ltInfo}) {
      print $statusFile sprintf("%s;%s;%s\n", $lt, uc($ltInfo->{$lt}{TYPE}), $ltInfo->{$lt}{ENABLED} eq "NO" ? "DOWN" : "UP");
    }
  } else {
    FamilyCommon::SetAndLogError($fh, sprintf("CLI error: '%s'", IsamCli::GetError()), sprintf("  FAILED: '%s'", IsamCli::GetError()));
    $ok = 0;
  }
  return $ok;
}

=head2 GetXdslOperData($$$$$$)

=begin html
Retrieve XDSL operational data using SNMP.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>snmpSession</td><td>pointer to an opened SNMP session (not used)</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>lineId</td><td>line identification (CLI notation)</td></tr>
  <tr><td>profMode</td><td>CLI profile mode</td></tr>
  <tr><td>ntType</td><td>NT type</td></tr>
</table>
Returns undef in case of failure otherwise a reference to a hash holding the retrieved states is returned
=end html

=cut

sub GetXdslOperData($$$$$$) {
  my ($fh, $snmpSession, $debug, $lineId, $profMode, $ntType) = @_;
  my @oids;
  my ($rack, $shelf, $slot, $line) = split('/', $lineId);
  my $intLtNum = Capability::GetLtInternalNum($profMode, $ntType, $slot);
  my $lineItfIdx;
  my $chnlItfIdx;
  my $result = undef;
  my %states = ();

  if (defined($intLtNum)) {
    $lineItfIdx = ($rack << 28) + ($shelf << 24) + (($intLtNum-1) << 16) + ($line-1);
    $chnlItfIdx = $lineItfIdx + 0x1000;
    ToolCommon::Log($fh, sprintf("Calculated XDSL line ifIndex 0x%08x for $lineId\n", $lineItfIdx)) if $debug;
    $oids[0] = IFOPERSTATUS_OID . ".$lineItfIdx";
    $oids[1] = XDSLLNPWRDS_OID . ".$lineItfIdx";
    $oids[2] = XDSLCHFEACTBRDS_OID . ".$chnlItfIdx";
    $oids[3] = XDSLCHFEATTBRDS_OID . ".$chnlItfIdx";
    $oids[4] = XDSLCHFEACTIMPNUS_OID . ".$chnlItfIdx";
    $oids[5] = XDSLCHACTBRUS_OID . ".$chnlItfIdx";
    $oids[6] = XDSLCHATTBRUS_OID . ".$chnlItfIdx";
    $oids[7] = XDSLCHIMPNDS_OID  . ".$chnlItfIdx";
    if ($snmpSession->version() < 2) {
      # SNMPv1/SNMPv2
      $result = $snmpSession->get_request(-varbindlist => \@oids);
    } else {
      # SNMPv3
      $result = $snmpSession->get_request(-varbindlist => \@oids,
					  -contextname => NT_CONTEXT);
    }
    if (defined($result)) {
      $states{IFOPERSTATUS} = $result->{$oids[0]};
      $states{XDSLLNPWRDS} = $result->{$oids[1]};
      $states{XDSLCHFEACTBRDS} = $result->{$oids[2]};
      $states{XDSLCHFEATTBRDS} = $result->{$oids[3]};
      $states{XDSLCHFEACTIMPNUS} = $result->{$oids[4]};
      $states{XDSLCHACTBRUS} = $result->{$oids[5]};
      $states{XDSLCHATTBRUS} = $result->{$oids[6]};
      $states{XDSLCHIMPNDS} = $result->{$oids[7]};
    } else {
      ToolCommon::Log($fh, sprintf("SNMP (%s, %s) get failed: %s\n", $oids[0], $oids[1], $snmpSession->error()));
      return undef;
    }
  } else {
    ToolCommon::Log($fh, sprintf("Could not calculate internal LT number for profile mode %s, NT type %s and LT %d\n",
				 $profMode, $ntType, $slot));
    return undef;
  }
  return \%states;
}

=head2 StoreXdslLineStatus($$$$$$$)

=begin html
Retrieve and store XDSL line and status information.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>pointer to an opened CLI session</td></tr>
  <tr><td>snmpSession</td><td>pointer to an opened SNMP session (not used)</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>statusFile</td><td>reference to the result file</td></tr>
  <tr><td>profMode</td><td>CLI profile mode</td></tr>
  <tr><td>ntType</td><td>NT type</td></tr>
</table>

=end html

=cut

sub StoreXdslLineStatus($$$$$$$) {
  my ($fh, $cli, $snmpSession, $debug, $statusFile, $profMode, $ntType) = @_;
  my $ifTypes;
  my $ifOperStates;
  my $ifAdminStates;
  my $ok = 1;
  my $ifIndex;
  my $linesCounted = 0;
  my $totalIfIndexes = 0;
  my @cmdResponse;
  my %xdslLines = ();

  ToolCommon::Log($fh, "Retrieve XDSL line configuration...");
  @cmdResponse = IsamCli::SendCommand($cli, "info configure xdsl line flat");
  if (scalar(@cmdResponse > 0)) {
    ToolCommon::Log($fh, "  OK\n");
    foreach my $line (@cmdResponse) {
      chomp $line;
      if ($line =~ /^configure\sxdsl\sline\s(\d+\/\d+\/\d+\/\d+)\sservice-profile\s(\d+)\sspectrum-profile\s(\d+)/) {
	my $lineId = $1;
	my $servProf = $2;
	my $specProf = $3;
	my $adminUp = 1;
	$adminUp = 0 if (!($line =~ /admin-up/));
	$xdslLines{$lineId}{SRVPROF} = $servProf;
	$xdslLines{$lineId}{SPCPROF} = $specProf;
	$xdslLines{$lineId}{ADMIN} = $adminUp ? "admin-up" : "no admin-up";
        my ($status, $states) = IsamCli::GetXdslOperDataDetails( $cli, $lineId);
        #my $states = GetXdslOperData($fh, $snmpSession, $debug, $lineId, $profMode, $ntType);
        if (defined($states)) {
          $xdslLines{$lineId}{IFOPERSTATUS} = defined $states->{'opr-state/tx-rate-ds'} ? ucfirst($states->{'opr-state/tx-rate-ds'}) : '-';
          $xdslLines{$lineId}{XDSLLNPWRDS} = defined $states->{'output-power-down'} ? $states->{'output-power-down'} : '-';
          $xdslLines{$lineId}{XDSLCHFEACTBRDS} = defined $states->{'tx-rate/ds'} ? $states->{'tx-rate/ds'} : '-';
          $xdslLines{$lineId}{XDSLCHFEATTBRDS} = defined $states->{'max-tx-rate-ds'} ? $states->{'max-tx-rate-ds'} : '-';
          $xdslLines{$lineId}{XDSLCHFEACTIMPNUS} = defined $states->{'inp-up'} ? $states->{'inp-up'} : '-';
          $xdslLines{$lineId}{XDSLCHACTBRUS} = defined $states->{'tx-rate/us'} ? $states->{'tx-rate/us'} : '-';
          $xdslLines{$lineId}{XDSLCHATTBRUS} = defined $states->{'max-tx-rate-us'} ? $states->{'max-tx-rate-us'} : '-';
          $xdslLines{$lineId}{XDSLCHIMPNDS} = defined $states->{'inp-dn'} ? $states->{'inp-dn'} : '-';
	} else {
	  $xdslLines{$lineId}{IFOPERSTATUS} = '-';
	  $xdslLines{$lineId}{XDSLLNPWRDS} = '-';
	  $xdslLines{$lineId}{XDSLCHFEACTBRDS} = '-';
	  $xdslLines{$lineId}{XDSLCHFEATTBRDS} = '-';
	  $xdslLines{$lineId}{XDSLCHFEACTIMPNUS} = '-';
	  $xdslLines{$lineId}{XDSLCHACTBRUS} = '-';
	  $xdslLines{$lineId}{XDSLCHATTBRUS} = '-';
	  $xdslLines{$lineId}{XDSLCHIMPNDS} = '-';
	}
      } else {
	ToolCommon::Log($fh, "Ignoring '$line'\n") if ($debug);
      }
    }
    print $statusFile "Line;SrvP;SpcP;adm-state;opr-state;tx-rate-us;max-tx-rate-us;tx-rate-ds;max-tx-rate-ds;inp-up;inp-dn;output-power-down\n";
    foreach my $line (sort { my $a1 = $a; $a1 =~ s/\//\./g;
			     my $b1 = $b; $b1 =~ s/\//\./g;
			     pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
		      keys %xdslLines) {
      print $statusFile sprintf("%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s\n", $line,
				$xdslLines{$line}{SRVPROF},
				$xdslLines{$line}{SPCPROF},
				$xdslLines{$line}{ADMIN},
				$xdslLines{$line}{IFOPERSTATUS},
				$xdslLines{$line}{XDSLCHACTBRUS},
				$xdslLines{$line}{XDSLCHATTBRUS},
				$xdslLines{$line}{XDSLCHFEACTBRDS},
				$xdslLines{$line}{XDSLCHFEATTBRDS},
				$xdslLines{$line}{XDSLCHFEACTIMPNUS},
				$xdslLines{$line}{XDSLCHIMPNDS},
				$xdslLines{$line}{XDSLLNPWRDS});
    }
  }  else {
    FamilyCommon::SetAndLogError($fh, sprintf("CLI error: '%s'", IsamCli::GetError()), sprintf("  FAILED: '%s'", IsamCli::GetError()));
    $ok = 0;
  }
  Utils::EmptyHash(\%xdslLines);
  return $ok;
}

=head2 StoreBridgeConfig($$$$)

=begin html
Retrieve and store bridge configuration information.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>pointer to an opened CLI session</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>statusFile</td><td>reference to the result file</td></tr>
</table>

=end html

=cut

sub StoreBridgeConfig($$$$) {
  my ($fh, $cli, $debug, $statusFile) = @_;
  my @cmdResponse;
  my $prevPvcId = "";
  my $pvcId = "";
  my $bridgeConfig = "";
  my $ok = 1;

  ToolCommon::Log($fh, "Retrieve bridge configuration...");
  @cmdResponse = IsamCli::SendCommand($cli, "info configure bridge flat");
  if (scalar(@cmdResponse) > 0) {
    print $statusFile "PVC-ID;BridgeInfo\n";
    foreach my $line (@cmdResponse) {
      chomp $line;
      if (($line =~ /^configure\sbridge\sport\s(\d+\/\d+\/\d+\/\d+:\d+:\d+)/) ||
	  ($line =~ /^configure\sbridge\sport\s(\d+\/\d+\/\d+\/\d+)/)) {
	$pvcId = $1;
	if (($pvcId ne $prevPvcId) && ($prevPvcId ne "")) {
	  print $statusFile "$prevPvcId;$bridgeConfig\n";
          $bridgeConfig = "";
	  $line =~ s/configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+:\d+:\d+\s//g if ($line =~ /configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+:\d+:\d+/);
	  $line =~ s/configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+\s//g if ($line =~ /configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+/);
	  $bridgeConfig = $line;
	} else {
	  $line =~ s/configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+:\d+:\d+\s//g if ($line =~ /configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+:\d+:\d+/);
	  $line =~ s/configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+\s//g if ($line =~ /configure\sbridge\sport\s\d+\/\d+\/\d+\/\d+/);
	  if ($line =~ /vlan-id\s\d+\suntagged/) {
	    $line =~ s/\suntagged//;
	  }
	  $bridgeConfig = $bridgeConfig eq "" ? $line : "$bridgeConfig $line";
	}
	$prevPvcId = $pvcId;
      } else {
	ToolCommon::Log($fh, "Ignoring '$line'\n") if ($debug);
      }
    }
    print $statusFile "$pvcId;$bridgeConfig\n" if ($pvcId ne "");
    ToolCommon::Log($fh, "  OK\n");
  } else {
    FamilyCommon::SetAndLogError($fh, sprintf("CLI error: '%s'", IsamCli::GetError()), sprintf("  FAILED: '%s'", IsamCli::GetError()));
    $ok = 0;
  }
  return $ok;
}

=head2 StoreAuthenticationInfo($$$$)

=begin html
Retrieve and store authentication status information.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>pointer to an opened CLI session</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>statusFile</td><td>reference to the result file</td></tr>
</table>

=end html

=cut

sub StoreAuthenticationInfo($$$$) {
  my ($fh, $cli, $debug, $statusFile) = @_;
  my @cmdResponse;
  my $ok = 1;
  my ($pvcId, $status);

  ToolCommon::Log($fh, "Retrieve authentication status...");
  @cmdResponse = IsamCli::SendCommand($cli, "show security pae authenticator");
  if (scalar(@cmdResponse > 0)) {
    print $statusFile "PVC;auth-pae-state;auth-backend-state;port-status\n";
    foreach my $line (@cmdResponse) {
      chomp $line;
      if (($line =~ /^(\d+\/\d+\/\d+\/\d+:\d+:\d+)(.*)/) || ($line =~ /^(\d+\/\d+\/\d+\/\d+)(.*)/)) {
	$pvcId = $1;
	$status = $2;
	$status =~ s/^\s+//;
	$status =~ s/\s+/;/g;
	print $statusFile "$pvcId;$status\n";
      } else {
	ToolCommon::Log($fh, "Ignoring '$line'\n") if ($debug);
      }
    }
    ToolCommon::Log($fh, "  OK\n");
  } else {
    FamilyCommon::SetAndLogError($fh, sprintf("CLI error: '%s'", IsamCli::GetError()), sprintf("  FAILED: '%s'", IsamCli::GetError()));
    $ok = 0;
  }
  return $ok;
}

=head2 SendTL1Command($$$$$$)

=begin html
Send a TL-1 command.<br>
The following parameters are used:
<table>
  <tr><td>logFile</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>comm</td><td>pointer to an opened TL-1 session</td></tr>
  <tr><td>cmd</td><td>TL-1 command to be sent</td></tr>
  <tr><td>expectedResult</td><td>The expected result to the TL-1 command</td></tr>
  <tr><td>ignore</td><td>in case of failure (response is not as expected) ignore (1) or not (0)</td></tr>
</table>

=end html

=cut

sub SendTL1Command($$$$$$) {
  my ($logFile, $debug, $comm, $cmd, $expectedResult, $ignore) = @_;
  my $t0;
  my $t1;
  my $response = undef;
  my $ok = 1;

  $cmd =~ s/\#/\\\#/;
  ToolCommon::Log($logFile, "Sending $cmd...") if $debug;
  $response = $comm->SendTL1Command($cmd);
  if (!defined($response)) {
    my $t = sprintf("  Sending $cmd failed: %s", $comm->GetError());
    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
    $ok = 0;
  } else {
    ToolCommon::Log($logFile, "Response: $response\n") if $debug;
    if (defined($expectedResult)) {
      if ($expectedResult ne "") {
	if (!($response =~ /$expectedResult/)) {
	  my $t = "  Executing $cmd failed: response does not match $expectedResult";
	  FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
	  $ok = 0;
	}
      }
    } else {
      if (!($response =~ /COMPLD/)) {
	if (! $ignore) {
	  my $t = "  $cmd failed: $response";
	  FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
	  $ok = 0;
	}
      }
    }
  }
  $response =~ s/\r//g;
  return ($ok, $response);
}

=head2 StoreOutstandingAlarms($$$$)

=begin html
Retrieve the list of outstanding alarms using TL-1.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>pointer to an opened CLI session</td></tr>
  <tr><td>debug</td><td>Additional debugging required?</td></tr>
  <tr><td>statusFile</td><td>reference to the result file</td></tr>
</table>

=end html

=cut

sub StoreOutstandingAlarms($$$) {
  my ($fh, $isam, $statusFile) = @_;
  my $cmdResponse;
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $snmpAccBits;
  my $comm;
  my $ok = 1;
  my @splitted;

  ToolCommon::Log($fh, "Retrieve outstanding alarms...");
  $snmpAccBits = ToolCommon::GetAccessettings($snmpSession, $params->{DEBUG});
  if (defined($snmpAccBits)) {
    print $statusFile "Outstanding alarms\n";
    if (ToolCommon::TL1AccessEnabled($snmpAccBits, $params->{DEBUG})) {
      if (! ToolCommon::TL1ViaUDP($snmpAccBits, $params->{DEBUG})) {
	$comm = SshChannel::ConnectSSH($isam, $toolData->{TL1USER}, $toolData->{TL1PWD}, $toolData->{TELNETTIMEOUT},
				       $params->{DEBUG} ? $fh : undef, $config->{TL1SSHPORT});
	if (!defined($comm)) {
	  my $t = sprintf("Could not open TL1/SSH: %s", SshChannel::GetError());
	  FamilyCommon::SetAndLogError($fh, $t, "$t\n");
	  $ok = 0;
	} else {
	  $cmdResponse = SshChannel::SendCommand($comm, "RTRV-ALM-ALL::;");
	  if (! defined($cmdResponse)) {
	    my $t = sprintf("Retrieve alarm list using SSH failed: %s", SshChannel::GetError());
	    FamilyCommon::SetAndLogError($fh, $t, "$t\n");
	    $ok = 0;
	  } else {
	    ToolCommon::Log($fh, "Response recvd: '$cmdResponse'\n") if $params->{DEBUG};
	    @splitted = split(/\n/, $cmdResponse);
	    SshChannel::Disconnect($comm);
	  }
	}
      } else {
	$comm = new UdpChannel;
	$ok = $comm->OpenChannel($isam, $config->{TL1UDPPORT});
	if ($ok) {
	  ($ok, $cmdResponse) = SendTL1Command($fh, $params->{DEBUG}, $comm, "LOGOFF;", undef, 1);
	  if ($ok) {
	    ($ok, $cmdResponse) = SendTL1Command($fh, $params->{DEBUG}, $comm,
						 "ACT-USER::".$toolData->{TL1USER}.":::$toolData->{TL1PWD};", "COMPLD", 0);
	    if (!defined($cmdResponse) || ! $ok) {
	      my $t = sprintf("Login failed:Incorrect TL1 username or password specified: %s", $comm->GetError());
	      FamilyCommon::SetAndLogError($fh, $t, "$t\n");
	      $ok = 0;
	    } else {
	      ($ok, $cmdResponse) = SendTL1Command($fh, $params->{DEBUG}, $comm, "RTRV-ALM-ALL::;", "COMPLD", 0);
	      if ($ok) {
		@splitted = split(/\n/, $cmdResponse);
		SendTL1Command($fh, $params->{DEBUG}, $comm, "LOGOFF;", "COMPLD", 0); # Ignore failures
		$comm->CloseChannel();
	      } else {
		my $t = sprintf("Retrieve alarm list using UDP failed: %s", $comm->GetError());
		FamilyCommon::SetAndLogError($fh, $t, "$t\n");
		$ok = 0;
	      }
	    }
	  } else {
	    my $t = sprintf("Logoff failed: %s", $comm->GetError());
	    FamilyCommon::SetAndLogError($fh, $t, "$t\n");
	    $ok = 0;
	  }
	} else {
	  my $t = sprintf("UDP open failed: %s", $comm->GetError());
	  FamilyCommon::SetAndLogError($fh, $t, "$t\n");
	  $ok = 0;
	}
      }
      my $alarm = "";
      my $cmdFound = 0;
      foreach my $line (@splitted) {
	ToolCommon::Log($fh, "$line\n") if $params->{DEBUG};
	if ($cmdFound) {
	  $line =~ s/^\s+//;
	  if ($line =~ /^\"/) {
	    $alarm = $line;
	  } else {
	    $alarm .= $line;
	  }
	  if ($alarm =~ /\"$/) {
	    ToolCommon::Log($fh, "'$alarm'\n") if $params->{DEBUG};
	    printf $statusFile "$alarm\n";
	  }
	} else {
	  if (uc($line) =~ /RTRV-ALM-ALL/) {
	    $cmdFound = 1;
	  }
	}
      }
      ToolCommon::Log($fh, "  OK\n");
    }
  } else {
    my $t = ToolCommon::GetError();
    FamilyCommon::SetAndLogError($fh, $t, "$t\n");
    $ok = 0;
  }
  return $ok;
}
#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 Check($$$)

=begin html
This function is called to perform status checks on a node of the ISAM family. <br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>ntType</td><td>The NT type detected.  Allows to determine the equipment type</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Check($$) {
  my ($isam, $ntType) =  @_;
  my $checkLog;
  my $cli;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();

  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  #-- Open CLI Session
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $params->{TELNETTIMEOUT_SCHK});
  if (!defined($cli)) {
    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				 EC_OPENCLI_FAILED,
				 sprintf(": %s", ToolCommon::GetError()),
				 1);
    return;
  } else {
    ToolCommon::Log($nodeLogFH, "  OK\n");
  }
  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount

  ToolCommon::Log($nodeLogFH, "Set CLI session timeout...");
  if (!IsamCli::SetSessionTimeout($cli, 120)) {
    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				 EC_SET_SESSION_TO_FAILED,
				 undef,
				 1);
    return;
  } else {
    ToolCommon::Log($nodeLogFH, "  OK\n");
  }

  ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});

  my $prev = $cli->max_buffer_length($params->{BUFSIZE});
  ToolCommon::Log($nodeLogFH, sprintf("Increased input buffer length from %d to %d bytes\n", $prev, $params->{BUFSIZE})) if ($params->{DEBUG});

  my $fileName; 

  my $shelfType = IsamCli::GetShelfType($cli); 
  if (Capability::IsANSIShelf($shelfType)) { 
    $fileName = sprintf("$dir/%s-%s.txt", $params->{PRE_OR_POST} == 0 ? PREOPERCHECKFILETAG : POSTOPERCHECKFILETAG, Capability::IsamAnsiFamily());
  } else {
    $fileName = sprintf("$dir/%s-%s.txt", $params->{PRE_OR_POST} == 0 ? PREOPERCHECKFILETAG : POSTOPERCHECKFILETAG, Capability::IsamFamily());
  } 

  if (! open($checkLog, ">$fileName")) {
    ToolCommon::Log($parentLogFh, sprintf("Child $childNum: cannot create %s for %s: %s\n",
					  $fileName, FamilyCommon::GetFailedNode(), $!), 1);
    CORE::exit(CREATE_FILE_FAIL_CODE);
  }
  $checkLog->autoflush(1);

  my $profileMode = IsamCli::GetProfileSlotNumberMode($cli, $params->{USERNAME});
  if (! defined($profileMode)) {
    FamilyCommon::IncrErrorCount(sprintf("Failed to determine CLI slot-number mode: %s\n", IsamCli::GetError()),
				 EC_GET_CLIMODE_FAILED,
				 sprintf(": %s", IsamCli::GetError()),
				 1);
    return;
  }

  if (index(uc($params->{CHECKS}), "LTSTS") != -1) {
    if (! StoreLtStatus($nodeLogFH, $cli, $params->{DEBUG}, $checkLog)) {
      FamilyCommon::IncrErrorCount(sprintf("Failed to retrieve LT config and operational states: %s\n", FamilyCommon::GetError()),
				   EC_GET_LT_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return;
    }
  }
  if (index(uc($params->{CHECKS}), "XDSL") != -1) {
    # Read out XDSL config and status data
    if (! StoreXdslLineStatus($nodeLogFH, $cli, $snmpSession, $params->{DEBUG}, $checkLog, $profileMode, uc($ntType))) {
      FamilyCommon::IncrErrorCount(sprintf("Failed to retrieve XDSL line config and operational states: %s\n", FamilyCommon::GetError()),
				   EC_GET_XDSL_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return;
    }
  }
  if (index(uc($params->{CHECKS}), "BRIDGE") != -1) {
    # Read out bridge config data
    if (! StoreBridgeConfig($nodeLogFH, $cli, $params->{DEBUG}, $checkLog)) {
      FamilyCommon::IncrErrorCount(sprintf("Failed to retrieve Bridge config data: %s\n", FamilyCommon::GetError()),
				   EC_GET_BRIDGE_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return;
    }
  }
  if (index(uc($params->{CHECKS}), "AUTH") != -1) {
    # Read out authentication info
    if (! StoreAuthenticationInfo($nodeLogFH, $cli, $params->{DEBUG}, $checkLog)) {
      FamilyCommon::IncrErrorCount(sprintf("Failed to authentication information: %s\n", FamilyCommon::GetError()),
				   EC_GET_AUTHENT_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return;
    }
  }
  if (index(uc($params->{CHECKS}), "ALARM") != -1) {
    # Read outstanding alarms
    if (! StoreOutstandingAlarms($nodeLogFH, $isam->[0], $checkLog)) {
      FamilyCommon::IncrErrorCount(sprintf("Failed to retrieve outstanding alarms: %s\n", FamilyCommon::GetError()),
				   EC_GET_ALARM_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return;
    }
  }
  FamilyCommon::CloseCliSession2Close();
  close $checkLog;
}

1;
