#==============================================================================
#-- Company:      Alcatel Bell - RSC
#-- Author:       Pieter Blommaert
#-- function:     Generic Trace and Debug access over UDP (bypass octopus)
#------------------------------------------------------------------------------
#-- disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel. The script is for TAC use only.
#--               There is no warranty about it's running except in a
#--               controlled manner by qualified personnel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent 2008
#--               All rights reserved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 First version
#--   1.1.0 minor fixes brought in from other script fixing
#--   1.2.0 (timeout=... option to sendMsg for variable timeout) (internal)
#--         Added waitForTimeout ($$$) to wait until there is no further output 
#--   1.3.0 Adapted to PBMT context.
#--   3.0.1 Resolve subroutine call and variable declaration issue
#--   3.0.2 15/07/2016 ALU02237966
#==============================================================================

package TAD;

use strict;
use warnings 'all';
use IO::Socket;
use File::Basename;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetError();
sub SendMsg($$$);
sub NewSession($$);
sub CloseSession($);
sub Cleanup();
sub RetrieveResult($$$);
sub WaitForString($$$);
sub WaitForTimeout($$$);
sub RetrieveInfo($$$);
sub SelectFH($);
sub SetDebug($);
sub GetUSRnr($);
sub SuspendUSR($);
sub ResumeUSR($);
#==============================================================================
#== Variables
#==============================================================================
my ($VERSION) = "TAD.pm - Framework v3.0.2";
my ($sessions);
my ($errortxt) = "";
my ($DEBUG) = undef;
my ($cmdCounter) = 0;
my ($DETAILLOGFH);

#For exporting the Namespace
#By default not exporting anything, but the pragma :ALL will export everything
BEGIN {
  use Exporter();
  our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
  # set the version for version checking
  $VERSION     = 1.00;
  @ISA         = qw(Exporter);
  #@EXPORT      = qw(); #nothiing to export by default
  
  # exported package globals go here,
  # as well as any optionally exported functions
  @EXPORT_OK   = qw(GetVersion NewSession CloseSession Cleanup RetrieveResult RetrieveInfo GetError SuspendUSR ResumeUSR);
  %EXPORT_TAGS = (ALL => \@EXPORT_OK);     # eg: TAG => [ qw!name1 name2! ],

}

use ToolCommon;
use PBMTGlobal;

sub GetVersion() {
  return $VERSION;
}

sub GetError() {
  return $errortxt;
}

sub SendMsg($$$) {
  # Does a send and receive until prompt or newline
  my ($msg, $sData, $options) = @_;
  $options = "" unless defined($options);
  my ($handle) = $sData->{handle};
  my ($ipaddr) = $sData->{ipaddr};
  my ($portnr) = $sData->{portnr};
  $portnr = 23 unless (defined($portnr)); # Startup phase
  my ($result) = {};
  $result->{portnr} = $portnr; #overruled later as appropriate
  my ($timeout) = 10;
  if ($options =~ /timeout=(\d+)/) {
    $timeout = $1;
  }

  if (defined($sData->{dead})) {
    $result->{portnr} = 0;
    $result->{message} = "";
    $result->{failure} = 1;
    return; # when the remote is dead, don't do anything any more
  }

  if (not(defined($ipaddr))) {
    ToolCommon::Log($DEBUG, "SENDMSG request without ip address\n") if defined($DEBUG);
    return;
  }

  my ($maxlen) = 1500;

  $ipaddr = gethostbyname($ipaddr); 
  my ($portaddr) = pack_sockaddr_in($portnr, $ipaddr);

  unless ($msg eq "") {
    unless ($msg =~ /[\r\n]\s*$/) {
      $msg .= "\n"; #make sure there is a newline: otherwise, there will be no prompt...
    }
    ToolCommon::Log($DEBUG, "SEND: $msg\n") if defined($DEBUG);
    if ($options =~ /noSendLog/) {
      ToolCommon::Log($DEBUG, "SENDING *** (hidden string)\n") if defined($DEBUG);
    } else {
      ToolCommon::Log($DEBUG, "TAD: SENDING $msg\n") if defined($DEBUG);
    }
    unless (send($handle, $msg, 0, $portaddr) == length($msg)) {
      $errortxt = "FAILED TO SEND: $!\n";
      ToolCommon::Log($DEBUG, "TAD: failed to send data to isam: $!\n") if defined($DEBUG);
      $result->{failure} = 1;
    }
  }

  my ($totalmsg) = "";
  $msg = "";
 MSGLOOP:
  until($msg =~ /\n/ or $totalmsg=~/\[USR\d-shell\]/) {
    if (not(defined($sData->{USRnr}))) {
      if ($totalmsg =~ /\[USR(\d)-shell\]/) {
	$sData->{USRnr} = $1;
      }
    }
    my ($recieveRes);
    $recieveRes = eval {
      local $SIG{ALRM} = sub { die "alarm time out" };
      alarm $timeout; # timeout in seconds
      $portaddr = recv($handle, $msg, $maxlen, 0) or die "recv: $!";
      alarm 0;
      1;  # return value from eval on normalcy
    };
    unless ($portaddr and $recieveRes) {
      if ($options =~ /timeout/) {
	$result->{timeout} = 1;
	last MSGLOOP;
      } else {
	ToolCommon::Log($DEBUG, "TIMEOUT no data received from isam. Move on to next command\n") if defined($DEBUG);
	my $errCode = $!;
	$errortxt = "FAILED TO RECEIVE: $!\n";
	$sData->{receiveFailures} ++;
	$result->{failure} = 1;
	if ($sData->{receiveFailures} > 5) {
	  $errortxt = "FAILED to receive: '$errCode'.  Too many failures in receiving from socket\n";
	  ToolCommon::Log($DEBUG, "TAD: Too many timeouts. Consider isam dead, ignore all future requests\n") if defined($DEBUG);
	  $sData->{dead} = 1;
	  $result->{portnr} = 0;
	}
	last MSGLOOP;
      }
    } else {
      ($portnr, $ipaddr) = unpack_sockaddr_in($portaddr);
      if (defined($sData->{portnr}) and ($portnr ne $sData->{portnr})) {
	$sData->{wrongport} ++;
	ToolCommon::Log($DEBUG, "RX from wrong port $portnr: $msg") if defined($DEBUG);
	$totalmsg .= $msg;
      } else {
	$totalmsg .= $msg;
      }
      $result->{portnr} = $portnr;
    }
  }
  ToolCommon::Log($DEBUG, "RX from port $portnr : \n$totalmsg\n********************************\n") if defined($DEBUG);
  $result->{message} = $totalmsg;
  return $result;
}

###############################################
#
#  Start a new session
sub NewSession($$) {
  my ($ipaddr, $sessionid) = @_;

  if (defined($sessionid) and defined($sessions->{$sessionid})) {
    $errortxt = "Failed to create TAD session: session ID '$sessionid' allready in use.\n";
    return 0;
  }

  ### START WITH LOGIN VIA PORT 23
  my ($sData) = {}; # Session data
  $sData->{ipaddr} = $ipaddr;
  $sData->{wrongport} = 0;
  my ($portnr) = 23;
  my ($handle) = IO::Socket::INET->new(Proto => 'udp') or die "socket: $@";     # yes, it uses $@ here
  $sData->{handle} = $handle;

  my ($answer);
  $answer = sendMsg("\r", $sData);
  return 0 if ($answer->{portnr} == 0);

  until ($answer->{message} =~ /Login/) {
    $answer = sendMsg("\r", $sData);
    return 0 if ($answer->{portnr} == 0);
  }
  return 0 if ($answer->{portnr} == 0);
  $answer = sendMsg("shell\r", $sData);

  until ($answer->{message} =~ /Password/) {
    $answer = sendMsg("", $sData);
    return 0 if ($answer->{portnr} == 0);
  }

  $answer = sendMsg("nt\r", $sData, "noSendLog");
  return 0 if ($answer->{portnr} == 0);

  until ($answer->{message} =~ /console/) {
    $answer = sendMsg("", $sData);
    return 0 if ($answer->{portnr} == 0);
  }

  ### NOW WE SHOULD HAV GOTTEN A DEDICATED PORT. ACCEPT ONLY FROM THAT PORT
  $sData->{portnr} = $answer->{portnr};
  if (defined($sessionid) and $sessionid ne "") {
    $sessions->{$sessionid} = $sData;
  } elsif ($sData->{portnr} and ($sData->{portnr} != 23)) {
    $sessionid = $sData->{portnr};
    $sessions->{$sessionid} = $sData;
  } else {
    return 0; # Failed to open a new session
  }
  ### WAIT FOR PROMPT. THEN WE ARE READY TO TAKE IN COMMANDS
  until ($answer->{message} =~ /\[USR\d-shell\]/) {
    $answer = sendMsg("", $sData);
    return 0 if ($answer->{portnr} == 0 or defined($sData->{dead}));
  }
  return $sessionid
}

sub CloseSession($) {
  my ($sessionid) = @_;
  my ($sData) = $sessions->{$sessionid};
  my ($answer) = sendMsg("exit\r\n",$sData);

  ToolCommon::Log($DEBUG, "Closing session '$sessionid' with $sData->{wrongport} messages received for the wrong port\n") if defined($DEBUG);
  delete $sessions->{$sessionid};
  return;
}

## FOR SAFETY: ALWAYS CALL THIS UPON SIG{INT} OR ANY REGULER SCRIPT END
sub Cleanup() {
  my ($sessionid);
  foreach $sessionid (keys %{$sessions}) {
    closeSession($sessionid);
  }
}


sub RetrieveResult($$$) {
  my ($sessionid, $msg, $retry) = @_;
  $retry = 1 unless (defined($retry));

  if ($retry < 1) {
    $errortxt = "Requested number of attempts is negative";
    return "";
  }

  unless (defined($sessions->{$sessionid})) {
    $errortxt = "Unknown sessionid: '$sessionid'";
    return "";
  }

  unless (defined($msg)) {
    $errortxt = "No commands provided";
    return "";
  }

  my ($sData) = $sessions->{$sessionid};
  my ($totalMsg) = "";
  my ($count);
 RETRYLOOP:
  for ($count = 1; $count <= $retry; $count++) {
    my ($res) = sendMsg($msg,$sData);

    if ($res->{portnr} == 0 or ($sData->{dead})) {
      my ($reason) = $!;
      $errortxt = "Failed to send message: ISAM considered dead: $reason";
      ToolCommon::Log($DEBUG, "RetrieveResult failed: ISAM DEAD\n") if defined($DEBUG);
      return "";
    }
    $totalMsg .= $res->{message} if (defined($res->{message}));

  WAITFORPROMPT:
    until ($res->{message} =~ /\[USR\d-shell\]/) {
      $res = sendMsg("",$sData);
      $totalMsg .= $res->{message} if (defined($res->{message}));

      if ($res->{portnr} == 0 or ($sData->{dead})) {
	my ($reason) = $!;
	ToolCommon::Log($DEBUG, "RetrieveResult failed: ISAM DEAD\n") if defined($DEBUG);
	return "";
      }
      last WAITFORPROMPT if ($res->{failure});
    }

    $totalMsg .= $res->{message};

    last RETRYLOOP unless ($res->{failure});

    ToolCommon::Log($DEBUG, "Retry (nr. $count) after timeout\n") if defined($DEBUG);
    $totalMsg .= ">>>> FAILED ATTEMPT NR $count. WILL RETRY NOW <<<<\n";
  }
  return $totalMsg;
}

sub WaitForString($$$) {
  my ($sessionid, $result, $searchRe) = @_;
  $result = "" unless (defined($result));

  return $result unless (defined($sessions->{$sessionid}));
  my ($sData) = $sessions->{$sessionid};

  my ($waitFinished) = 1 if ($result =~ /$searchRe/s); # must be there at least twice
  $waitFinished = 1 if ($sData->{dead});

  while (not($waitFinished)) {
    my ($rXres) = sendMsg("",$sData);
    if ($rXres->{failure}) {
      # Nothing received any more 
      ToolCommon::Log($DEBUG, "Timeout waiting for data from the isam\n") if defined($DEBUG);
      $waitFinished = 1;
    }
    $result .= $rXres->{message} if (defined($rXres->{message}));
    if ($result =~ /$searchRe/s) {
      $waitFinished = 1;
    }
  }
  return $result;    
}

sub WaitForTimeout($$$) {
  my ($sessionid, $result, $timeout) = @_;
  $result = "" unless (defined($result));

  return $result unless (defined($sessions->{$sessionid}));
  return $result unless ($timeout > 0);
  my ($sData) = $sessions->{$sessionid};

  my ($waitFinished) = 0;
  $waitFinished = 1 if ($sData->{dead});
  while (not($waitFinished)) {
    my ($rXres) = sendMsg("", $sData, "timeout=$timeout");
    if ($rXres->{timeout}) {
      # Nothing received any more
      ToolCommon::Log($DEBUG, "Awaited timeout ($timeout) reached\n") if defined($DEBUG);
      $waitFinished = 1;
    }
    $result .= $rXres->{message} if (defined($rXres->{message}));
  }
  return $result;    
}
    
####################################
#
#   Depreciated. Only for backward compatibility
sub RetrieveInfo($$$) {
  my ($isamip, $cmd, $options) = @_;
  $options = "" unless (defined($options));

  my ($sessionid) = newSession($isamip);
  if (not(defined($sessionid)) or $sessionid == 0) {
    ToolCommon::Log($DEBUG, "Failed to setup session\n") if defined($DEBUG);
    return [];
  }

  my ($sData) = $sessions->{$sessionid}; 
  sleep 2;

  my ($totalresult) = [];
  my ($msg);
  my ($res) = " ";
  my ($rcomCounter) = 0;

 CMDLOOP:
  while ($cmd =~ s/^([^\n]+\n)//) {
    $msg = $1;
    if ($msg =~ /trc_sleep/) {
      ToolCommon::Log($DEBUG, "Skipping $msg\n") if defined($DEBUG);
    } elsif ($msg =~ /trc_realsleep\s+(\d+)/) {
      my ($secs) = $1;
      ToolCommon::Log($DEBUG, "Sleeping $secs\n") if defined($DEBUG);
      sleep $secs;
    } elsif ($msg =~ s/trc_echo //) {
      $cmdCounter++;
      my ($res) = retrieveResult($sessionid, $msg,1);
      if (not($res)) {
	last CMDLOOP;
      }
      if ($msg =~ /(rcom exec -b \S+)/ and not ($msg =~ /rcom output 0/)) {
	#          detailLog("TAD: RCOM start string: $res***\n");
	ToolCommon::Log($DEBUG, "rcom command detcted, adding additional check step\n") if defined($DEBUG);
	my ($rcommsg) = $1;
	$rcomCounter++;
	sleep 1 if ($options =~ /slow/);
	$res .= retrieveResult($sessionid, "$rcommsg -c echo RCOM_TRACE_DONE_$$-$sessionid-$rcomCounter-$cmdCounter\n",1);
	#          detailLog("TAD: RCOM check string: $res***\n");
	my ($rcomTraceDone) = 0;
	$rcomTraceDone = 1 if
	  ($res =~ /RCOM_TRACE_DONE_$$-$sessionid-$rcomCounter-$cmdCounter.*RCOM_TRACE_DONE_$$-$sessionid-$rcomCounter-$cmdCounter/s); # must be there at least twice
	$rcomTraceDone = 1 if ($sData->{dead});
	while (not($rcomTraceDone)) {
	  my ($rXres) = sendMsg("", $sData);
	  if ($rXres->{failure}) {
	    # Nothing received any more
	    ToolCommon::Log($DEBUG, "Timeout waiting for data from the ISAM\n") if defined($DEBUG);
	    $rcomTraceDone = 1;
	  }
	  $res .= $rXres->{message} if (defined($rXres->{message}));
	  #            detailLog("TAD: RCOM added string: $rXres->{message}***\n");
	  if ($res =~ /RCOM_TRACE_DONE_$$-$sessionid-$rcomCounter-$cmdCounter.*RCOM_TRACE_DONE_$$-$sessionid-$rcomCounter-$cmdCounter/s) {
	    $rcomTraceDone = 1;
	  }
	}
      }
      $res =~ s/\s+$/\n/; #make sure it ends on a newline
      #        detailLog("TAD: COLLECTED string: $res***\n");
      my (@resArr) = $res =~ /[^\n\r]*[\r\n]{1,2}/g;
      push @{$totalresult}, @resArr;
      if ($sData->{dead}) {
	ToolCommon::Log($DEBUG, "ISAM considered dead. Stopping here.\n") if defined($DEBUG);
	last CMDLOOP;
      }
    } else {
      ToolCommon::Log($DEBUG, "Skipping $msg\n");
    }
  }
  retrieveResult($sessionid,"dbg fvt PRNT") unless ($sData->{dead});
  closeSession($sessionid);
  return $totalresult;
}

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

sub SetDebug($) {
  my $logFile = @_;
  $DETAILLOGFH = &selectFH($logFile);
}

sub GetUSRnr($) {
  my ($sessionid) = @_;

  if (not(defined($sessionid)) or not(defined($sessions->{$sessionid}))) {
    $errortxt = "Not-existing sessionid: '$sessionid'\n";
    return;
  }

  my ($sData) = $sessions->{$sessionid};
  if (not(defined($sData->{USRnr}))) {
    $errortxt = "USR nr for $sessionid not yet detected\n";
    return;
  }
  my ($USRnr) = $sData->{USRnr};
  return $USRnr;
}
 
sub SuspendUSR($) {
  my ($sessionid) = @_;
  my ($USRnr) = getUSRnr ($sessionid);
  if (defined($USRnr)) {
    retrieveResult($sessionid,"dbg suspend USR$USRnr");
  }
}

sub ResumeUSR($) {
  my ($sessionid) = @_;
  my ($USRnr) = getUSRnr ($sessionid);
  if (defined($USRnr)) {
    retrieveResult($sessionid,"dbg resume USR$USRnr");
  }
}

1;


