package UpgradeIsam7354;

=head1 ISAM 7354-specific SW upgrade module

=begin html
This library contains the functions dedicated to upgrade a 7302/7330 ISAM from one release to a maintenance release.

=end html

=cut

#==============================================================================
#-- File:         UpgradeIsam7354.pm
#-- Date:         17.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to upgrade ISAMs pizza boxes to a new minor release
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/10/10 : First implementation
#--   1.0.1 - 2009/04/24 : Release transaction if TO = 127.0.0.1
#--                        ANTmt55561
#--   1.0.2 - 2009/08/26 : Call ToolCommon::ReleaseTransaction after node restart.
#--                        CheckActiveSoftwareIsNew returns a list and not a single value.
#--   1.0.3 - 2009/09/22 : ANTmt75743
#--   1.0.4 - 2009/11/13 : ANTmt86148: 7354RU fixes
#--   1.0.5 - 2010/03/09 : ANTmt92967.   Proper Handling of software rollback for upgradation.
#--   1.0.6 - 2011/04/01 : Changes for 7354
#--   1.0.7 - 2011/04/05 : ALU01031457 - Support for R4220 & R4230
#--   3.0.1 - 2012/03/29 : 7354 changes for SSH failure 
#--   3.0.2 - 2012/04/30 : ALU01781801
#--   3.0.3 - 2016/03/16 : ALU02190558
#--   3.0.4 - 2016/07/15 : ALU02237966
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "UpgradeIsam7354.pm - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "UpgradeIsam7354.pm - 3.1.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h"</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;

#==============================================================================

use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Crypt::Lite;
use Utils;
use Capability;
require Octopus;
use PbmtConfig;
use ErrorCodes qw(:EXIT_CODES :UPGR);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use version();
use vars qw(%version);

#==============================================================================
#== Comments:
#==
#==   General steps of the script
#==     1a) Check if new SWP is on NT board (and find its index)
#==     1b) Check old software release is same major release
#==     3) Activate new SWP
#==     4) Wait for NT to become active
#==     5) Verify correct SWP is active
#==============================================================================

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Upgrade($$$$);
#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;

my $NTINFILENAME = "dm_complete.tar";           # input name of the complete NT database

#==============================================================================
#== Vars  -- mod 1.0.2
#==============================================================================
my $toolData;
my $config;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 UpgradeIsamFamily($$$$$$$$$$$$)

=begin html
This function is called to perform SW upgrade of a node of the ISAM family..<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>thisTftpDir</td><td>tftp directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Upgrade($$$$) {
  my ($newLog, $thisTftpDir, $isam, $ntType) = @_;
  my $ret;
  my ($newFullRelNum, $newRelNum, $fullActiveRelNum, $activeRelNum, $fromDbVers, $toDbVers);
  my $possible = 0;
  my $reason = "";
  my $cliTypeSettingErr = "";
  my $cli = undef;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();

  #-- Open Cli Connection
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  $params->{ISAM7354RU} = ""; 
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($cli)) {
    if(defined($params->{PASSWORD2})) {
            sleep(30);
            my $cliError = IsamCli::GetError(); 
	    if($cliError =~ /Login failed/i) {
		    ToolCommon::Log($nodeLogFH, "Open CLI failed with first password, Trying again with second password ...");
		    my $oldCliPassword = $params->{PASSWORD}; 
		    $params->{PASSWORD} = $params->{PASSWORD2}; 
		    $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
		    $params->{PASSWORD} = $oldCliPassword; 
		 if (!defined($cli)) {
		    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s:\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
						 EC_OPENCLI_FAILED,
						 sprintf(": %s", ToolCommon::GetError()),
						 1);
		    return;
		 } else {
		    ToolCommon::Log($nodeLogFH, "  OK\n");
		 } 
            }
    } else {
            FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s:\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
					 EC_OPENCLI_FAILED,
					 sprintf(": %s", ToolCommon::GetError()),
					 1);
	    return;
   }
  }
  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");


  #-- STEP 1: Check if needed files are present on NT
  #-- Get NT type information
  ToolCommon::Log($nodeLogFH, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($cli);
  return undef if( not defined($ntInfo) );
  my ($activeNt, $passiveNt);
  my $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, $ntType, undef,
					 $toolData->{RETRIES});
  if (!defined($oswpInfo)) {
    FamilyCommon::IncrErrorCount("Obtaining active SW failed.\n",
				 EC_GET_OSWP_FAILED,
				 sprintf(": %s", ToolCommon::GetError()),
				 1);
    return;
  }

  ToolCommon::Log($nodeLogFH, "Checking Active Software <> new Software...");
  if ($oswpInfo->{'ACTIVE'}{'NAME'} eq basename($params->{OSWP})) {
    FamilyCommon::AdvanceToNext("  Node already running $oswpInfo->{ACTIVE}{NAME} ... skipping\n");
    return;
  }
  ToolCommon::Log($nodeLogFH, "  OK\n");

  $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'},
						 $oswpInfo->{'ACTIVE'}{'SUFFIX'});
  $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
					     $oswpInfo->{'ACTIVE'}{'MINOR'});
  my $regExp = REGEXP;
  if (basename($params->{SWPNAME}) =~ /$regExp/) {
    $newFullRelNum = Capability::ConvertRelease($1, $2, $3); # -- 1.2.0
    $newRelNum = Capability::ConvertRelease($1, $2); # -- 1.2.0
  } else {
    FamilyCommon::IncrErrorCount("Could not determine release version from basename($params->{SWPNAME}\n",
				 EC_SW_NAME_ERROR,
				 sprintf(" (%s)", basename($params->{SWPNAME})),
				 1);
    return;
  }
  if (! $params->{FORCEUPG}) {
      if ($oswpInfo->{'ACTIVE'}{'DBVERSION'} ne $params->{DBVER_TGTREL} ) {
          FamilyCommon::ReportStatus( EC_NEEDS_MIGRATION, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                                                          $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                                                          basename($params->{OSWP}), $params->{DBVER_TGTREL}, "Needs migration" ));
	      return 0 if ($params->{CONTINUE});
	      CORE::exit(UPG_IMPOSSIBLE_CODE);
      }
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntType,
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
  } else {
    ToolCommon::Log($nodeLogFH, "Forced upgrade, continue without checks...\n");
  }
  ToolCommon::Log($nodeLogFH, "Checking New Software is passive...");
  if ($oswpInfo->{'PASSIVE'}{'NAME'} ne basename($params->{OSWP})) {
    FamilyCommon::IncrErrorCount("  SWP not present... skipping\n",
				 EC_NEW_SWP_NOT_PRESENT,
				 undef,
				 1);   # -- ANTms31758
    return;
  }
  if ($oswpInfo->{'PASSIVE'}{'AVAIL'} ne 'enabled') {
    FamilyCommon::IncrErrorCount("  SWP not enabled... skipping\n",
				 EC_NEW_SWP_NOT_ENABLED,
				 undef,
				 1);   # -- ANTms31758
    return;
  }
  ToolCommon::Log($nodeLogFH, "  OK\n");

  #-- STEP 2: Backup database
  if (!FamilyCommon::CreateBUFile($parentLogFh, $nodeLogFH, $dir, $NTINFILENAME, $newLog, $childNum, $params->{DEBUG})) {
    CORE::exit(CREATE_FILE_FAIL_CODE);
  }
  my $time1 = time();
  my $dbSize;
  ToolCommon::Log($nodeLogFH, "Backup NT Database...");
  $ret = IsamCli::UploadDb($cli, 'actual-active',
			   $params->{PRIMTFTPIP}, $thisTftpDir, 1);
  if (!defined($ret) || !$ret)  {
    FamilyCommon::IncrErrorCount(sprintf("  FAILED (%s)\n", IsamCli::GetError()),
				 EC_NT_DB_BU_FAIL,
				 sprintf(": %s", IsamCli::GetError()),
				 1);
    return;
  } else {
    my $realName = "$toolData->{FXRDATA}{HOMEDIR}/$NTINFILENAME";
    $dbSize = -s "$realName";
    ToolCommon::Log($nodeLogFH, sprintf("  OK, transfered $dbSize bytes in %d seconds\n", time() - $time1));
  }

#-- STEP : Check if migration interrupt on emergency call feature is enabled
if ($params->{EMERG_CALL_FEATURE}) {
  my $isActive = 0;
  ToolCommon::Log($nodeLogFH, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
  $isActive = ToolCommon::AlarmRetrieval($nodeLogFH, $cli, $toolData);
  if($isActive) {													   ## emergency call alarm is active
	 FamilyCommon::IncrErrorCount(sprintf("  FAILED: Emergency call max retries reached\n"),
				     EC_EMERGENCY_CALL_ACTIVE,
				     undef,
				     1);
	 return undef;
 }
}
#-- STEP : End of migration interrupt on emergency call feature.

  #-- STEP 11: Activate SWP

  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  return if( not ToolCommon::ActivateSwp($oswpInfo) );
  #-- NT sw changed, system will reboot, close cli session

  #-- STEP 12: Wait for restart
  if ($toolData->{ACTPINGWAITRU} > 0) {
    ToolCommon::Log($nodeLogFH, "Waiting $toolData->{ACTPINGWAITRU} seconds for the system to restart...");
    sleep($toolData->{ACTPINGWAITRU});
    ToolCommon::Log($nodeLogFH, "  OK\n");
  }

  $ret = FamilyCommon::WaitForRestart($nodeLogFH, $snmpSession, $isam, undef);  # -- 1.0.4
  if (!$ret) {
    #-- asam is not back ... start a panic
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s... skipping\n", IsamCli::GetError()),
				 EC_RESTART_FAILED,
				 undef,
				 1);
    if ($params->{CONTINUE}) {
      ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **ATTENTION** Restart of %s failed (timed out), please verify %s.\n",
					    FamilyCommon::GetFailedNode(), $newLog), 1);
      return;
    } else {
      ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **FATAL** Restart of %s failed (timed out), please verify %s.\n",
					    FamilyCommon::GetFailedNode(), "$params->{LOGFILE}_$childNum"), 1);
      CORE::exit(NT_RESTART_FAIL_CODE);
    }
  }

  # Reset transaction owner since this seems to survive a migration (even though the prozone gets erased...
  my $delay = 75;     # delay of 75secs introduced since otherwise the release transaction does not happen (only for 7354)
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  #-- Open Cli Connection
  ToolCommon::Log($nodeLogFH, "Re-open CLI...");
  $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($cli)) {
      if(defined($params->{PASSWORD2})) {
            sleep(30);
            my $cliError = IsamCli::GetError(); 
	    if($cliError =~ /Login failed/i) {
		    ToolCommon::Log($nodeLogFH, "Open CLI failed with first password, Trying again with second password ...");
		    my $oldCliPassword = $params->{PASSWORD}; 
		    $params->{PASSWORD} = $params->{PASSWORD2}; 
		    $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
		    $params->{PASSWORD} = $oldCliPassword; 
		 if (!defined($cli)) {
   		    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				 EC_OPENCLI_FAILED,
				 sprintf(" (re-open after reboot): %s", ToolCommon::GetError()),
				 1);
   		   return;
  		}  else {
    		  ToolCommon::Log($nodeLogFH, "  OK\n");
    		  #FamilyCommon::SetCliSession2Close($cli);
  		}
	   }
      } else {
            FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s:\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
					 EC_OPENCLI_FAILED,
					 sprintf("(re-open after reboot): %s", ToolCommon::GetError()),
					 1);
	    return;
      }
  }
  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount		
  
  #-- STEP 13: Check Correct SWP is active
  my ($restartOswpInfo, $oswpInfoIsEmpty);
  ($ret, $oswpInfoIsEmpty, $restartOswpInfo) = FamilyCommon::CheckActiveSoftwareIsNew($nodeLogFH, $cli, $snmpSession,
										      $params->{OSWP}, $dir, $isam,
										      $ntType);

    if (!$ret) {
      if ($oswpInfoIsEmpty == -1) {
	# Node has possible gone down, so wait until back.  Will fall trough if up.
	$ret = FamilyCommon::WaitForRestart($nodeLogFH, $snmpSession, $isam, undef); # -- 1.1.6
	if (!$ret) {
	  #-- isam is not back ... start a panic
	  my $t = sprintf(" FAILED: %s... skipping", IsamCli::GetError());
	  FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	  if ($params->{CONTINUE}) {
	    ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			    sprintf("Child %d: **ATTENTION** Restart of %s failed (timed out), please verify %s.\n",
				   $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	    FamilyCommon::IncrErrorCount("",
				 EC_NEW_SW_NOT_ACT,
				 sprintf(": %s", FamilyCommon::GetError()),
				 1);
		return; 
	  } else {
	    # Here's the exit from the script...
	    ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			    sprintf("Child %d: **FATAL** Restart of %s failed (timed out), please verify %s.\n",
				   $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	    FamilyCommon::ToolExit(NT_RESTART_FAIL_CODE);
	  }
	} else {
	  if (! IsamCli::ConnectionStillOpen($cli)) {
	    # Wait till telnet open before checking a pending OSWP activation.
	    $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	    if (!defined($cli)) {
	       if(defined($params->{PASSWORD2})) {
                  sleep(30);
                  my $cliError = IsamCli::GetError(); 
	          if($cliError =~ /Login failed/i) {
		    ToolCommon::Log($nodeLogFH, "Open CLI failed with first password, Trying again with second password ...");
		    my $oldCliPassword = $params->{PASSWORD}; 
		    $params->{PASSWORD} = $params->{PASSWORD2}; 
		    $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
		    $params->{PASSWORD} = $oldCliPassword; 
		    if (!defined($cli)) {
 			my $t = sprintf("  ERROR: Could not open cli for %s (checking new active SW): %s\n", FamilyCommon::GetFailedNode(),
			    ToolCommon::GetError());
	    		FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	    		FamilyCommon::IncrErrorCount("",
				 EC_NEW_SW_NOT_ACT,
				 sprintf(": %s", FamilyCommon::GetError()),
				 1);
		        return; 
		    } else {
		        ToolCommon::Log($nodeLogFH, "  OK\n");
		    } 
               }
	       my $t = sprintf("  ERROR: Could not open cli for %s (checking new active SW): %s\n", FamilyCommon::GetFailedNode(),
			      ToolCommon::GetError());
	       FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	       FamilyCommon::IncrErrorCount("",
				 EC_NEW_SW_NOT_ACT,
				 sprintf(": %s", FamilyCommon::GetError()),
				 1);
		  return; 
	    } else {
	      ToolCommon::Log($nodeLogFH, "  OK\n");
	    }
	    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
          }
	 }
	}
      } elsif (basename($restartOswpInfo->{'ACTIVE'}{'NAME'}) ne basename($params->{OSWP})) {
	    
		FamilyCommon::IncrErrorCount("",
				 EC_NEW_SW_NOT_ACT,
				 sprintf(": %s", FamilyCommon::GetError()),
				 1);
		return;   
      } else {
	if (! defined($restartOswpInfo)) {
	  # No OSWP information could be retrieved
	  $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	  if (!defined($cli)) {
	    my $t = sprintf("  ERROR: Could not open cli for %s (checking new active SW): %s\n", FamilyCommon::GetFailedNode(),
			    ToolCommon::GetError());
	    FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	    FamilyCommon::IncrErrorCount("",
				 EC_NEW_SW_NOT_ACT,
				 sprintf(": %s", FamilyCommon::GetError()),
				 1);
		return; 

	  } else {
	    ToolCommon::Log($nodeLogFH, "  OK\n");
	  }
	  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	}
      }   
    } elsif($ret == 2) {
    FamilyCommon::IncrErrorCount("",
				 EC_NEW_SW_NOT_ACT,
				 sprintf(": %s", FamilyCommon::GetError()),
				 1);
    return;
    }

  FamilyCommon::CloseCliSession2Close();
}

1;
