package UpgradeIsamIHUB;

=head1 ISAM 7302/7330-specific SW upgrade module

=begin html
This library contains the functions dedicated to upgrade a HiCap 7302/7330 ISAM from one release to a maintenance release.

=end html

=cut

#==============================================================================
#-- File:         UpgradeIsamIHUB.pm
#-- Date:         28.11.2018  (Derived from UpgradeIsam7302HiCap.pm )
#-- Company:      Nokia
#-- Author:       Dhineshkumar Manikannan
#------------------------------------------------------------------------------
#-- Function:     Script to upgrade HiCap ISAMs to a new minor release
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0  - 2008/09/22 : First implementation
#--   1.0.1  - 2009/04/24 : Release transaction if TO = 127.0.0.1
#--                         ANTmt55561
#--   1.0.2  - 2009/08/11 : ANTmt70461: solve warnings generated while running upgrade
#--   1.0.3  - 2009/08/26 : Call ToolCommon::RreleaseTransaction after node restart.
#--                         CheckActiveSoftwareIsNew returns a list and not a single value.
#--   1.0.4  - 2009/09/22 : ANTmt75743
#--   1.0.5  - 2010/01/29 : ANTmt92967. Proper Handling of software rollback for upgradation.(Corrected by Kupuswamy)
#--   3.0.00 - 2010/04/21 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01 - 2011/03/29 : ALU01031457 - Support for R4220 & R4230
#--   3.0.02 - 2011/10/03 : R430x upgrade changes
#--   3.0.03 - 2012/12/29 : SW Upgrade via StandBy NT
#--   3.0.04 - 2013/02/06 : enable usage of second password if first one fails for cli over ssh
#--   3.0.05 - 2013/08/12 : ALU01903056
#--   3.0.06 - 2014/02/01 : ALU01944957
#--   3.0.07 - 2014/07/09 : ALU01995383
#--   3.0.08 - 2016/03/16 : ALU02190558
#--   3.0.09 - 2016/07/15 : ALU02237966
#--   3.0.10 - 2016/09/12 : ALU02271083
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "UpgradeIsamIHUB.pm - 3.0.10 -- 2016/09/12"

=end html

=cut

my $VERSION = "UpgradeIsamIHUB.pm - 3.1.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h"</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;

#==============================================================================

use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Crypt::Lite;
use Utils;
use Capability;
require Octopus;
use ErrorCodes qw(:EXIT_CODES :UPGR);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use version();
use vars qw(%version);

#==============================================================================
#== Comments:
#==
#==   General steps of the script
#==     1a) Check if new SWP is on NT board (and find its index)
#==     1b) Check old software release is same major release
#==     3) Activate new SWP
#==     4) Wait for NT to become active
#==     5) Verify correct SWP is active
#==============================================================================

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Upgrade($$$);
#==============================================================================
#== Constants
#==============================================================================
*version = \%VERSION::version;
my $NTINFILENAME = "dm_complete.tar";           # input name of the complete NT database

#==============================================================================
#== Vars  -- mod 1.0.2
#==============================================================================
my $toolData;
my $config;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 UpgradeIsamIpdFamily($$$$$$$$$$$$)

=begin html
This function is called to perform SW upgrade of a node of the ISAM family..<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>thisTftpDir</td><td>tftp directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Upgrade($$$) {
  my ($newLog, $thisTftpDir, $isam) = @_;
  my $ret;
  my ($newFullRelNum, $newRelNum, $fullActiveRelNum, $activeRelNum, $fromDbVers, $toDbVers);
  my $possible = 0;
  my $reason = "";
  my $cliTypeSettingErr = "";
  my $cli = undef;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $configData = FamilyCommon::GetConfigData();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $duplexNT = 0; 
  my $nanteOrfantf = 0; 
  my $ltActivating = 0;

  #-- Open Cli Connection
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($cli)) {
       FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, ToolCommon::GetError() );
       return;
  }
  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount

  my $ltInfo = IsamCli::GetLtConfigState($cli);
  foreach my $lt ( keys %$ltInfo ) {
       $ltActivating = 1 if( ( defined $ltInfo->{$lt}{TYPE} ) and ( $ltInfo->{$lt}{TYPE} ne "fglt-a" ) and 
           ( $ltInfo->{$lt}{TYPE} =~ /^(fw|fg)lt-[a-z]$/ ) ) ;
  }


  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  #-- STEP 1: Check if needed files are present on NT
  #-- Get NT type information
  ToolCommon::Log($nodeLogFH, "Obtain extended NT information...");
  my $ntInfo = IsamCli::GetNtExtendedInfo($cli);
  return undef if( not defined($ntInfo) );

  my $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';
  my $passiveNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTB' : 'NTA';
  my $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}), undef,
					 $toolData->{RETRIES});
  if (!defined($oswpInfo)) {
       FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, ToolCommon::GetError() ); 
       return;
  }

  my $dbVersionActive = $oswpInfo->{'ACTIVE'}{'DBVERSION'};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveRel = substr($dbVersionActive,1);
  my $dbVersionTargetRel = substr($dbVersionTarget,1);

  ToolCommon::Log($nodeLogFH, "Checking Active Software <> new Software...");
  if ($oswpInfo->{'ACTIVE'}{'NAME'} eq basename($params->{OSWP})) {
    FamilyCommon::AdvanceToNext("  Node already running $oswpInfo->{ACTIVE}{NAME} ... skipping\n");
    return;
  }
  ToolCommon::Log($nodeLogFH, "  OK\n");

### DHINI
  if( $params->{BTYPE} eq "YANG") {
    ToolCommon::Log($nodeLogFH, "Yang build going to activate directly");
    goto BUILDACTIVATE;
  }

  my $forceUpg = $params->{FORCEUPG}; 
  if ( (($dbVersionTargetRel eq "4.300") and ($dbVersionActiveRel eq "4.300" )) ||
       (($dbVersionTargetRel eq "4.600") and ($dbVersionActiveRel eq "4.600" )) ||
       (($dbVersionTargetRel eq "5.000") and ($dbVersionActiveRel eq "5.000" )) ) {
         $params->{FORCEUPG} = 1;  
  }

  if (! $params->{FORCEUPG}) {
      if ($oswpInfo->{'ACTIVE'}{'DBVERSION'} ne $params->{DBVER_TGTREL} ) {
          FamilyCommon::ReportStatus( EC_NEEDS_MIGRATION, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                                                          $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                                                          basename($params->{OSWP}), $params->{DBVER_TGTREL}, "Needs migration" ));
	      return 0 if ($params->{CONTINUE});
	      CORE::exit(UPG_IMPOSSIBLE_CODE);

      }
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntInfo->{$activeNt}{TYPE}, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
  } else {
    ToolCommon::Log($nodeLogFH, "Forced upgrade, continue without checks...\n");
    $params->{FORCEUPG} = $forceUpg; 
  }
  ToolCommon::Log($nodeLogFH, "Checking New Software is passive...");
  if ($oswpInfo->{'PASSIVE'}{'NAME'} ne basename($params->{OSWP})) {
       FamilyCommon::ReportStatus( EC_NEW_SWP_NOT_PRESENT, "SWP not present to proceed\n" ); 
       return;
  }
  if ($oswpInfo->{'PASSIVE'}{'AVAIL'} ne 'enabled') {
       FamilyCommon::ReportStatus( EC_NEW_SWP_NOT_PRESENT, "SWP not enabled to proceed\n" ); 
       return;
  }
  ToolCommon::Log($nodeLogFH, "  OK\n");

  #-- STEP: Configure AccessOperatorId (supported starting R3.6)
  if ( ($dbVersionActiveRel ge "3.600") and ($params->{ACCESSOPRID}) ) {
     if (!ToolCommon::AccessOprIdConfigure($nodeLogFH, $params->{DEBUG}, $cli, $config->{ACCESS_OPERATOR_ID})) {
       FamilyCommon::ReportStatus( EC_CONFIG_OPRID_FAILED,ToolCommon::GetError());
       return;
     }
  }

  if ( ($dbVersionActiveRel ge "5.800") and ($params->{'BOARD-FAMILY'} eq Capability::IsamHiCapEtsiFamily()) ) {
      my $issu_status = ToolCommon::CheckISSU($cli);
      if( $issu_status ) {
          FamilyCommon::ReportStatus( EC_SETUP_ISSU_FAILED,ToolCommon::GetError());
          return undef;
      }
  }

  #-- STEP 2: Save dbase
  if (!FamilyCommon::CreateBUFile($parentLogFh, $nodeLogFH, $dir, $NTINFILENAME, $newLog, $childNum, $params->{DEBUG})) {
    CORE::exit(CREATE_FILE_FAIL_CODE);
  }
  my $time1 = time();
  my $dbSize;
  ToolCommon::Log($nodeLogFH, "Backup NT Database...");
  $ret = IsamCli::UploadDb($cli, 'actual-active',
			   $params->{PRIMTFTPIP}, $thisTftpDir, 1);
  if (!defined($ret) || !$ret)  {
       FamilyCommon::ReportStatus( EC_NT_DB_BU_FAIL, IsamCli::GetError());
       return undef;
  } else {
       $dbSize = -s "$toolData->{FXRDATA}{HOMEDIR}/$NTINFILENAME";
       ToolCommon::Log($nodeLogFH, sprintf("  OK, transfered $dbSize bytes in %d seconds\n", time() - $time1));
  }

  if ((uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") && (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY")) {
    #-- STEP 8c: Redundant system, wait for stability   Needed?
    ToolCommon::Log($nodeLogFH, "Waiting for stability...");
    sleep(180);   # Only for redundant system?
    ToolCommon::Log($nodeLogFH, " OK\n");
  } else {
    sleep $toolData->{SECUREDBSAVE} if ( $dbVersionActiveRel lt "2.300" );
  }

BUILDACTIVATE:
  if( $params->{BTYPE} eq "YANG") {
    ToolCommon::Log($nodeLogFH, "Started Yang build going to activate directly");
  }
#-- STEP : Check if migration interrupt on emergency call feature is enabled
if ($params->{EMERG_CALL_FEATURE}) {
  my $isActive = 0;
  ToolCommon::Log($nodeLogFH, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
  $isActive = ToolCommon::AlarmRetrieval($nodeLogFH, $cli, $toolData);
  if($isActive) {													   ## emergency call alarm is active
       FamilyCommon::ReportStatus( EC_EMERGENCY_CALL_ACTIVE, "Emergency call max retries reached\n");
	   return undef;
 }
}
#-- STEP : End of migration interrupt on emergency call feature.


  #-- STEP 11: Activate SWP
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");


  # Make sure we do not get an NT reset due to a NON_RECOV at the time of OSWP activation.
  # Workaround for R2.4 problem in OSWP activation for ARAM-D with exp shelves.
  if ($params->{REMAPHOTRESET}) {
    $ret = Octopus::RemapErrRecord($params->{DEBUG}, $dir, $isam, $dirName);
    if (! $ret) {
       FamilyCommon::ReportStatus( EC_MAP_ERR_REC_FAILED,  Octopus::GetError());
       return;
    }
  }

  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  return if( not ToolCommon::ActivateSwp($oswpInfo) );
  #-- NT sw changed, system will reboot, close cli session

  #-- STEP 12: Wait for restart
  if ($toolData->{ACTPINGWAIT} > 0) {
    ToolCommon::Log($nodeLogFH, "Waiting $toolData->{ACTPINGWAIT} seconds for the system to restart...");
    sleep($toolData->{ACTPINGWAIT});
    ToolCommon::Log($nodeLogFH, "  OK\n");;
  }


  $ret = FamilyCommon::WaitForRestart($nodeLogFH, $snmpSession, $isam, undef);  # -- 1.0.4
  if (!$ret) {
      if( defined $params->{BTYPE} and ($params->{BTYPE} ne "SNMP") ) {
        ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **ATTENTION** Yang Build activation, Restart of %s failed (timed out), Conditionally success... Please verify\n",
                                              FamilyCommon::GetFailedNode()), 1);
          my $statusLogFh = FamilyCommon::GetNodeStatusLog();
          if (defined($statusLogFh)) {
            print $statusLogFh sprintf("%s\n", ErrorCodes::ExitString(CONDITONAL_SUCCESS_CODE));
            FamilyCommon::CloseNodeStatusLog();
          }
          if ($params->{CONTINUE}) {
              return undef;
          } else {
            CORE::exit(CONDITONAL_SUCCESS_CODE);
          }
      } else {
          #-- asam is not back ... start a panic
           FamilyCommon::ReportStatus( EC_RESTART_FAILED,  FamilyCommon::GetError());
          if ($params->{CONTINUE}) {
            ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **ATTENTION** Restart of %s failed (timed out), please verify %s.\n",
              				    FamilyCommon::GetFailedNode(), $newLog), 1);
            return;
          } else {
            ToolCommon::Log($parentLogFh, sprintf("Child $childNum: **FATAL** Restart of %s failed (timed out), please verify %s.\n",
              				    FamilyCommon::GetFailedNode(), "$params->{LOGFILE}_$childNum"), 1);
            CORE::exit(NT_RESTART_FAIL_CODE);
          }
      }
  }

  # Reset transaction owner since this seems to survive a migration (even though the prozone gets erased...
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  #-- Open Cli Connection
  ToolCommon::Log($nodeLogFH, "Re-open CLI...");
  $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($cli)) {
        FamilyCommon::ReportStatus( EC_OPENCLI_FAILED,  " (re-open after reboot): %s". ToolCommon::GetError());
        return;
  }  else {
        ToolCommon::Log($nodeLogFH, "  OK\n");
        FamilyCommon::SetCliSession2Close($cli);
  }
  
  #-- STEP 13: Check Correct SWP is active
  my ($restartOswpInfo, $oswpInfoIsEmpty);
  ($ret, $oswpInfoIsEmpty, $restartOswpInfo) = FamilyCommon::CheckActiveSoftwareIsNew($nodeLogFH, $cli, $snmpSession,
										      $params->{OSWP}, $dir, $isam,
										      $ntInfo->{$activeNt}{TYPE});

     if (!$ret) {
      if ($oswpInfoIsEmpty == -1) {
	# Node has possible gone down, so wait until back.  Will fall trough if up.
	$ret = FamilyCommon::WaitForRestart($nodeLogFH, $snmpSession, $isam, undef); # -- 1.1.6
	if (!$ret) {
	  #-- isam is not back ... start a panic
	  my $t = sprintf(" FAILED: %s... skipping", IsamCli::GetError());
	  FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	  if ($params->{CONTINUE}) {
	       ToolCommon::Log(FamilyCommon::GetParentLogFile(),
		   	    sprintf("Child %d: **ATTENTION** Restart of %s failed (timed out), please verify %s.\n",
		   		   $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
           FamilyCommon::ReportStatus( EC_NEW_SW_NOT_ACT, FamilyCommon::GetError());
		   return; 
	  } else {
	    # Here's the exit from the script...
	    ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			    sprintf("Child %d: **FATAL** Restart of %s failed (timed out), please verify %s.\n",
				   $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	    FamilyCommon::ToolExit(NT_RESTART_FAIL_CODE);
	  }
	} else {
	  if (! IsamCli::ConnectionStillOpen($cli)) {
	    # Wait till telnet open before checking a pending OSWP activation.
	    $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	    if (!defined($cli)) {
	      my $t = sprintf("  ERROR: Could not open cli for %s (checking new active SW): %s\n", FamilyCommon::GetFailedNode(),
			      ToolCommon::GetError());
	      FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
          FamilyCommon::ReportStatus( EC_NEW_SW_NOT_ACT, FamilyCommon::GetError() ); 
		  return; 
	    } else {
	      ToolCommon::Log($nodeLogFH, "  OK\n");
	    }
	    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	  }
	}
      } elsif (basename($restartOswpInfo->{'ACTIVE'}{'NAME'}) ne basename($params->{OSWP})) {
	    
           FamilyCommon::ReportStatus( EC_NEW_SW_NOT_ACT, FamilyCommon::GetError());
		   return;   
      } else {
	if (! defined($restartOswpInfo)) {
	  # No OSWP information could be retrieved
	  $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	  if (!defined($cli)) {
	       my $t = sprintf("  ERROR: Could not open cli for %s (checking new active SW): %s\n", FamilyCommon::GetFailedNode(),
			    ToolCommon::GetError());
	       FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
           FamilyCommon::ReportStatus( EC_NEW_SW_NOT_ACT, FamilyCommon::GetError());
		   return; 

	  } else {
	    ToolCommon::Log($nodeLogFH, "  OK\n");
	  }
	  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	}
      }   
    } elsif($ret == 2) {
           FamilyCommon::ReportStatus( EC_NEW_SW_NOT_ACT, FamilyCommon::GetError());
           return;
    }

  if (($config->{ACCESS_OPERATOR_ID} > 0) && ($config->{ACCESS_OPERATOR_ID} <= 65535) && ($params->{ACCESSOPRID})) {
    my $accOperId = ToolCommon::GetAccessOperId($nodeLogFH, $snmpSession);
    if (defined($accOperId)) {
      if ($config->{ACCESS_OPERATOR_ID} != $accOperId) {
           FamilyCommon::ReportStatus( EC_ACC_OPERID_MISMATCH, sprintf("Retrieved %d while expecting %d\n", $accOperId, $config->{ACCESS_OPERATOR_ID}) ); 
	       return undef;
      }
    } else {
           FamilyCommon::ReportStatus( EC_GET_ACC_OPERID_FAILED, sprintf("Failed to retrieve the access-oper-id: %s\n", ToolCommon::GetError())); 
           return undef;
    }
  }
  FamilyCommon::CloseCliSession2Close();
}

1;
