
=head1 PBMT utilities

=begin html
This PERL module implements a number of utilities.

=end html

=cut

#==============================================================================
#-- File:         Utils.pm
#-- Date:         18.02.2001
#-- Company:      Alcatel Bell - TEC
#-- Author:       Donny Janssens, WD25
#------------------------------------------------------------------------------
#-- Function:     Provide some basic functions for handling of config files
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel CID 2001
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 -   1999/02/18 : First implementation
#--   1.1.0 -   2001/04/17 : Cleanup
#--   1.1.1 -   2006/03/06 : Added ISAM stuff
#--   1.2.5.1 - 2006/09/26 : Support for CLI replay script (empty hash).
#--   1.4.0   - 2006/10/06 : ANTms17249: R3.1/R3.2 maintream support
#--   1.4.3   - 2006/10/18 : Timestamp logging
#--   1.4.4   - 2007/03/20 : Handle fatal returns when retrieving NT
#--                          OFLMT version
#--   1.4.5   - 2007/06/18 : ANTms51632 allow multiple users for same ISAM.
#--   1.8.0   - 2007/09/12 : ISAM-V offline migration.
#--   1.8.1   - 2008/02/07 : extra robustness
#--   1.8.2   - 2009/03/12 : ANTmt47452
#--   1.8.3   - 2009/06/11.
#--   1.8.4   - 2009/06/18 : ANTmt64040 - readOswpFw.sh is modified.  Results in new cksum output.
#--   3.0.00  - 2010/04/21 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01  - 2010/05/25 : Converged PBMT Changes
#--   3.0.02  - 2010/09/29 : for ALU00747293 - added ResetHash routine to reset a hash structure to a value
#--   3.0.03  - 2011/01/12 : Octopus path verification check changed for different platforms.  
#--   3.0.04  - 2011/03/27 : Linux changes for octopus
#--   3.0.05  - 2013/10/14: ALU01913931: Modified for the change in cksum of enableSw_RW.sh
#--   3.0.06  - 2013/10/24: ALU01913513
#--   3.0.07  - 2013/12/20: ALU01935279: Replace octopus binary for Linux apart from cksum update for other scripts
#--   3.0.08  - 2015/04/03: ALU02059235
#--   3.0.09  - 2015/05/04: ALU02072681
#--   3.0.10  - 2015/09/01: ALU02119959
#--   3.0.11  - 2015/11/25: ALU02148679
#--   3.0.12  - 2016/04/28: ALU02206611 
#--   3.0.13  - 2016/06/24: ALU02237966
#--   3.0.14  - 2016/07/15: ALU02250240
#--   3.0.15  - 2016/07/27: ALU02237966
#--   3.0.16  - 2016/09/12: ALU02271083

=head1 VERSION

=begin html
Current version: "Utils.pm -- 3.0.16 - 2016/09/12"

=end html

=cut

my $VERSION = "Utils.pm -- 3.0.16 - 2016/09/12";
#==============================================================================

package Utils;

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use File::Basename;
use Cwd qw(getcwd);
use Config;

use ToolCommon qw(:CONSTANTS);
use ToolCommon;
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetError();
sub SelectFH($);
sub GetVersion();
sub SetDebug($);
sub ReadConfig($);
sub FindTftpInfo($$);
sub ConvertExitCode($);
sub GetShubOflmtVersion($$$$$);
sub GetNtOflmtVersion($$);
sub DetermineOFLMTVersions($$$$$$$);
sub VerifyOctopusEnv($$;$$);
sub EmptyHash($);
sub ResetHash($$);
sub UpdateOSWPFile($$$$);
sub FetchNTDescriptor($$);
#==============================================================================

# Update below each time a shell script has been added or modified.
my %scriptCksum = ('delIssconf.sh' =>    [74002112, 7721],
		   'delLanxZ.sh' =>      [1172459907, 7625],
		   'enableSw_RW.sh' =>   [2346708564, 8400],
		   'removeSwp.sh' =>     [1431162872, 7803],
		   'readOswpFW.sh' =>    [3874764402, 7441],
		   'readSwMgntIni.sh' => [3548593732, 7404],
		   'resetNT.sh' =>       [322234074, 7880],
		   'resetNTred.sh' =>    [2730972673, 8368],
		   'shubLanxCopy.sh' =>  [3812708010, 7426],
		   'shubSwCheck.sh' =>   [2632811513, 7342],
		   'suspendTftp.sh' =>   [193414235, 7572],
		   'syncLanxZ.sh' =>     [1007437847, 7395],
		   'octopus' =>          [1848108053, 69516],
		   'resetShell.cmd' =>   [2217056099, 85],
		   'updateNewFS.sh' =>   [2640953848, 8947],
		   'reduceSw_FW.sh' =>   [2290815830, 7360],
		   'mapNonRecov.sh' =>   [3620859998, 7407]);

 if ($Config{archname} =~ /i86pc/) {
    $scriptCksum{'octopus'} = [4085184329, 53900];
 } elsif ($Config{archname} =~ /linux/) {
    $scriptCksum{'octopus'} = [668106997, 47254]; 
 }

my $ERROR = "";

=head1 FUNCTIONS

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}

my $DEBUG = undef;

=head2 SelectFH($)

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($fh) = @_;

  select((select($fh), $|=1)[$[]);
  $fh;
}

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 SetDebug($)

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
#  print $DEBUG "Testing...\n";
}

=head2 ReadConfig($)

=begin html
Read the configuration file $file and store contents as a hashtable with key is name of variable.

=end html

=cut

sub ReadConfig($) {
  my ($file) = @_;
  my %configtable;
  my $line;
  my $cleanline;

  if (open(CONFIG, $file)) {
    while (<CONFIG>) {
      #-- Remove comment-lines
      if (/^[^\s\#]/) {
	#-- Remove trailing comments
	($cleanline) = split( /[^\\]\#/);
	$cleanline =~ s/\\\#/\#/g;
	#-- Break up line into variable and value
	my ($variable,@value) = split( /=/, $cleanline);
	my $value=join('=', @value);
	chomp($value);
	#-- Strip leading and trailing whitespace
	$variable =~ s/^\s*(.*?)\s*$/$1/;
	$value =~ s/^\s*(.*?)\s*$/$1/;
	#print STDERR "variable: '$variable', value: '@value'\n";
	#-- Substitute variables in the values.
	$_ = $value;
	if ( /^\$(\w*)/) {
	  if( ! defined $configtable{$1}) {
	    print STDERR "Undefined variable: $1\n";
	    $value =~ s/^\$(\w*)/$1/g;
	  } else {
	    $value =~ s/^\$(\w*)/$configtable{$1}/g;
	  }
	}
	if ( /([^\\])\$(\w*)/) {
	  if( ! defined $configtable{$2}) {
	    print STDERR "Undefined variable: $2\n";
	    $value =~ s/([^\\])\$(\w*)/$1$2/g;
	  } else {
	    $value =~ s/([^\\])\$(\w*)/$1$configtable{$2}/g;
	  }
	}
	$value =~ s/\\\$/\$/g;
	$value =~ s/^\"(.*)\"$/$1/g;
	#print STDERR "variable: '$variable', value: '$value'\n";
	#-- Put into hashtable
	$configtable{$variable} = $value;
      }
    }
  } else {
    $ERROR = "failed to open '$file': $!";
    return;
  }
  #  addConfig($file, \%configtable);
  return %configtable;
}


=head2 FindTftpInfo($$)

=begin html
Return TFTP related info from the OSWP file specified by $tftpDir and $oswp.<br>
Returns the TFTP server IP address and the name of the SW package file.

=end html

=cut

sub FindTftpInfo($$) {
  my ($tftpDir, $oswp) = @_;
  my ($dummy1, $colon, $swp, $ip, $secip);
  my $line;
  my $oswpFile;
  my $fileName = defined($tftpDir)? "$tftpDir/$oswp" : "$oswp";
  my $packagePath = undef;
  my %tftpInfo = ('NAME' => "",
		  'PRIMIP' => "",
		  'SECIP' => "");

  $ERROR = "";
  if (open($oswpFile, "$fileName")) {
    $swp = undef;
    while ($line = <$oswpFile>) {
#      print STDERR "$line";
      if ($line =~ /ASAM-CORE/) {
	# Regular ISAM OSWP
	$line =~ s/^\s+//;
	$line =~ s/;$//;
	($dummy1, $colon, $swp, $ip, $secip) = split(/\s+/, $line);
	$secip =~ s/;//g;  # remove any ';' if included.
	$tftpInfo{NAME} = $swp;
	$tftpInfo{PRIMIP} = $ip;
	$tftpInfo{SECIP} = $secip;
	last;  # Got it all so quit from the loop
      } elsif ($line =~ /SWP_NAME\s=\s([^\s]+)\s;/) {
	$swp = $1;
	# Accton PIZZA box OSWP file: Get SWP name ...
	if (defined($packagePath)) {
	  $swp = sprintf("%s%s", $packagePath, $swp);
	}
#	print STDERR "pkgp: '$swp'\n";
      } elsif ($line =~ /PACKAGE_PATH\s=\s([^\s]+)\s;/) {
	$packagePath = $1;
	if (defined($swp)) {
	  $swp = sprintf("%s%s", $packagePath, $swp);
	}
#	print STDERR "swp: '$swp'\n";
	$tftpInfo{NAME} = $swp;      
      } elsif ($line =~ /SWP_PRIMARY_FILE_SERVER_ID\s*=\s*(\d+\.\d+\.\d+\.\d+)\s*/) {
	# Accton PIZZA box: ... tftp IP address
	$tftpInfo{PRIMIP} = $1;
      } elsif ($line =~ /SWP_SECONDARY_FILE_SERVER_ID\s*=\s*(\d+\.\d+\.\d+\.\d+)\s*/) {
	# Accton PIZZA box: ... tftp IP address
	$tftpInfo{SECIP} = $1;
      }
    }
    close($oswpFile);
  } else {
    $ERROR = "Could not open $fileName\n";
  }
  return (\%tftpInfo);
}                                                                    # -- 1.1.1 END


=head2 UpdateOSWPFile($$$$)

=begin html

=end html

=cut

sub UpdateOSWPFile($$$$) {
  my ($tftpDir, $sourceFile, $destFile,$NTOswpDescriptor) = @_;
  my $line;
  my $oswpFile;
  my $fileName = defined($tftpDir)? "$tftpDir/$sourceFile" : "$sourceFile";

  $ERROR = "";
  my ($dir,$DFh,$swp,$baseName) = ("",undef,undef,""); 
  my ($dummy1, $colon,$ip, $secip);
  $dir = dirname($destFile);
  if( ! -d $dir ) {
     mkdir $dir or $ERROR = "Unable to create directory $dir :$!:";
     return (1,$ERROR) if( $ERROR ne "" );
  }
  if( !( open($DFh, '>', "$destFile")) ) {
   $ERROR = "Unable to open file $destFile :$!:";
   return (1,$ERROR);
  }
  my ($actual_line,$basename) = ("","");
  if (open($oswpFile, "$sourceFile")) {
    while ($line = <$oswpFile>) {
      $actual_line = $line;
#      print STDERR "$line";
      if ($line =~ /ASAM-CORE/) {
	# Regular ISAM OSWP
	$line =~ s/^\s+//;
	$line =~ s/;$//;
	($dummy1, $colon, $swp, $ip, $secip) = split(/\s+/, $line);
        $baseName = basename($swp);
        $line = $actual_line;
        $line =~ s/$baseName /$NTOswpDescriptor /;
      } elsif ($line =~ /SWP_NAME\s=\s([^\s]+)\s;/) {
	$swp = $1;
        $baseName = basename($swp);
        $line =~ s/$baseName /$NTOswpDescriptor /;
      }
      print $DFh $line;
    }
    close($oswpFile);
  } else {
    $ERROR = "Could not open $fileName\n";
    return (1,$ERROR);
  }
  close ($DFh);
  return (0,"");
}                                                                    # -- 1.1.1 END


sub FetchNTDescriptor($$) {
   my( $ntDescriptorMapFile, $ntType ) = @_;

   my $baseSWP = basename($params->{SWPNAME});
   my $regexpVariantRelease = VARIANT_RELEASE_REGEXP;
   if( $baseSWP =~ /$regexpVariantRelease/ ) {
	$baseSWP = "($1$2|Z$2|A$2)(\\d\\d.\\d\\d\\d)";
   }
#################################################################################
## DEVEL-NOTE :
##    from ATC/LRM : btype is argument to specify build type, Else SNMP is default one
##    from AMS     : They wont pass btype option at all, But they will pass exact SW Descriptor in OSWP
##		     Through SW Descriptor, We have to identify the respective btype for that SW Descriptor
#################################################################################
   my $buildType = "";
   if ( defined($params->{BTYPE}) ) {
	$buildType = $params->{BTYPE};
   }
   if( ! -f $ntDescriptorMapFile) {
       return ( 1, "File :$ntDescriptorMapFile: not exists" );
   }
   my ($status,$nt,$ntDescriptorFile,$btype,$dummy) = (1,"","","","");
   my ($oswpFile, $line, $snmpDesFile ) = (undef, "", "" ); 
   if (open($oswpFile, "$ntDescriptorMapFile")) {
    while ($line = <$oswpFile>) {
      next if( $line =~ /^\s*$/ );
      $line =~ s/^\s+//;
      $line =~ s/\s+$//;
      ($nt, $ntDescriptorFile,$btype,$dummy ) = split( /\s+/, $line );
      $btype = "SNMP" if( !defined $btype );
      if( "$nt" ne "" and "$nt" eq "$ntType" ) {
          if( $buildType eq "" and $ntDescriptorFile =~ /$baseSWP/ ) {
	      $params->{BTYPE} = $btype;
	      $status = 0;
	      last;
	  } elsif( $buildType ne "" and $buildType eq $btype ) {
	      $params->{BTYPE} = $btype;
	      $status = 0;
	      last;
	  } elsif( $btype eq "" or $btype eq "SNMP" ) {
	      $snmpDesFile = $ntDescriptorFile;
          }
      }
    }
    close($oswpFile);
    if( $status == 0 ) {
	return ($status,$ntDescriptorFile);
    } else {
	if( $buildType ne "" and $buildType ne "SNMP" ) {
            return ($status,"XDescriptor mapping for $ntType, $buildType is not exists");
        } else {
	    $buildType = "SNMP";
	    if( $snmpDesFile ne "" ) {
	       $params->{BTYPE} = $buildType;
	       return (0,$snmpDesFile);
            } else {
               return ($status,"XDescriptor mapping for $ntType, $buildType is not exists");
	    }
        }
    }
  } else {
    $ERROR = "Could not open $ntDescriptorMapFile\n";
    return (1,$ERROR);
  }
}

=head2 ConvertExitCode($)

=begin html
Convert a Unix return code into a exitcode and signal value.

=end html

=cut

sub ConvertExitCode($) {
  my ($retCode) = @_;
  my $exitCode = $retCode >> 8;
  my $signal = $retCode & 255;

  return ($exitCode, $signal);
}


=head2 GetShubOflmtVersion($$$$$)

=begin html
Determine the version of the SHUB offline migration tool.
In case of success 0 and the OFLMT version is returned, a non-zero value is returned in case of error.

=end html

=cut

sub GetShubOflmtVersion($$$$$) {
  my ($debug, $lanxDbToolDir, $lanxDbTool, $config, $path) = @_;
  my $cmd = "cd $path/$lanxDbToolDir;$config->{PERL} $lanxDbTool -v |";
  my $ret = SUCCESS_CODE;
  my $version = undef;
  my $line;
  my $found = 0;

  ToolCommon::Log($DEBUG, "CMD = $cmd\n") if ($debug);
  if (open(LOOK, $cmd)) {
    while (defined($line = <LOOK>) && ! $found) {
      chomp $line;
      ToolCommon::Log($DEBUG, "$line\n") if ($debug);
      if ($line =~ /.*VERSION\s(\d+\.\d+)/) {
	$version = $1;
	$found = 1;
      }
    }
    close LOOK;
    system("cd $path;");
  } else {
    my ($exitCode, $signal) = Utils::ConvertExitCode($?);
    $ERROR = "$cmd FAILED (signal $signal, exitcode $exitCode): $!\n";
    $ret = VERS_DET_FAILED_CODE;
  }
  return ($ret, $version);
}

=head2 GetNtOflmtVersion($$)

=begin html
Determine the version of the NT offline migration tool.
An undefined value is returned in case of error, the tool version is returned in the other case.

=end html

=cut

sub GetNtOflmtVersion($$) {
  my ($debug, $file) = @_;
  my $cmd = "$file -ver 2>&1";
  my $version = undef;
  my $line;
  my $found = 0;
  my $pwd = getcwd();

  ToolCommon::Log($DEBUG, "CMD = '$cmd', cwd = '$pwd'\n") if ($debug);
  $version = `$cmd`;
  my $files = unlink <prozone*>;
  ToolCommon::Log($DEBUG, "Deleted $files files (version: '$version')\n") if ($debug);
  chomp $version;
#  if (open(LOOK, "$cmd")) {
#    $line = <LOOK>;
#    chomp $line;
#    Log($fh, "$line\n") if ($debug);
#    $version = $line;
#    $found = 1;
#    close LOOK;
#  } else {
#    my ($exitCode, $signal) = Utils::ConvertExitCode($?);
#    Log($fh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!\n");
#  }
  if ($version =~ /fatal/) {
    $ERROR = $version;
    $version = undef;
  }
  return $version;
}

=head2 DetermineOFLMTVersions($$$$$$$)

=begin html
Return version information w.r.t. the offline tools being used during migration.  A hash is returned to the caller with the following entries:
<ul>
  <li>RET: sucess (0) or failure (<> 0)</li>
  <li>NT: NT OFLMT version (with expected syntax OFLMT_3EC15254BAAA_Vnn.n.n)</li>
  <li>IVPS: IVPS OFLMT version (with expected syntax OFLMT_3EC15254BAAA_Vnn.n.n)</li>
  <li>SHUB: SHUB OFLMT (with expected syntax Migrationtool.pl VERSION n.nn)</li>
</ul>

=end html

=cut

sub DetermineOFLMTVersions($$$$$$$) {
  my ($debug, $ntDbTool, $lanxDbToolDir, $lanxDbTool, $ivpsDbTool, $config, $dirName) = @_;
  my $shubOflmtVer = "";
  my $ntOflmtVer = "";
  my $ivpsOflmtVer = "";
  my $ret = SUCCESS_CODE;
  my $path = $dirName;
  my %versInfo = ('RET' => SUCCESS_CODE,
		  'NT' => "",
		  'IVPS' => "",
		  'SHUB' => "");

  $ERROR = "";
  if (defined($lanxDbTool)) {
    ($ret, $shubOflmtVer) = GetShubOflmtVersion($debug, $lanxDbToolDir, $lanxDbTool, $config, $path);
  }
  if (defined($ntDbTool)) {
    if (-e "$path/$ntDbTool") {
      if (-x "$path/$ntDbTool") {
	$ntOflmtVer = GetNtOflmtVersion($debug, "$path/$ntDbTool");
	if (!defined($ntOflmtVer)) {
	  # $ERROR set in GetNtOflmtVersion
	  $ret = NO_NT_OFLMT_VER_CODE;
	}
      } else {
	$ret = NT_OFLMT_NO_EXE_CODE;
	$ERROR = "$path/$ntDbTool not executable... exiting!!\n";
      }
    } else {
      $ret = NO_NT_OFLMT_CODE;
      $ERROR = "$path/$ntDbTool not found... exiting!!\n";
    }
  }
  if (defined($ivpsDbTool)) {
    if (-e "$path/$ivpsDbTool") {
      if (-x "$path/$ivpsDbTool") {
	$ivpsOflmtVer = GetNtOflmtVersion($debug, "$path/$ivpsDbTool");
	if (!defined($ivpsOflmtVer)) {
	  # $ERROR set in GetNtOflmtVersion
	  $ret = NO_IVPS_OFLMT_VER_CODE;
	}
      } else {
	$ret = IVPS_OFLMT_NO_EXE_CODE;
	$ERROR = "$path/$ivpsDbTool not executable... exiting!!\n";
      }
    } else {
      $ivpsOflmtVer = "NOT FOUND";
    }
  }
  $versInfo{NT} = defined($ntOflmtVer) ? $ntOflmtVer : "";
  $versInfo{SHUB} = defined($shubOflmtVer) ? $shubOflmtVer : "";
  $versInfo{IVPS} =  defined($ivpsOflmtVer) ? $ivpsOflmtVer : "";
  $versInfo{RET} = $ret;
  return \%versInfo;
}

=head2 VerifyOctopusEnv($$;$$)

=begin html
Verifies whether the environment is OK for running octopus scripts.<br>
The following parameters are used:
<table>
  <tr><td>debug</td><td>Debuggihg enabled or not</td></tr>
  <tr><td>octopusPath</td><td>location where to find the PBMT octopus scripts (relative to the installation directory)</td></tr>
  <tr><td>config</td><td>Not used</td></tr>
  <tr><td>runFrom</td><td>The installation directory of PBMT when specified</td></tr>
</table>
0 is returned in case of success, a non-zero value in case of failure.

=end html

=cut

sub VerifyOctopusEnv($$;$$) {
  my ($debug, $octopusPath, $config, $runFrom) = @_;
  my $path = defined($runFrom) ? "$runFrom" : getcwd();
  my $cmd = "cd $path/$octopusPath;ls |";
  my $line;
  my $goOn = 1;
  my $ret = SUCCESS_CODE;
  my $ckSum;
  my $arch = "sun4";
  if ($Config{archname} =~ /i86pc/) {
    $arch = "i86pc"; 
  } elsif ($Config{archname} =~ /linux/) {
    $arch = "linux"; 
  }

  $ERROR = "";
  ToolCommon::Log($DEBUG, "CMD = $cmd\n") if ($debug);
  if (open(LOOK, $cmd)) {
    while ((defined($line = <LOOK>)) && $goOn) {
      chomp $line;
      ToolCommon::Log($DEBUG, "$path/$octopusPath/$line\n") if ($debug);
      if (! -x "$path/$octopusPath/$line") {
	if (!($line =~ /\.cmd/)) {
	  $ERROR .= "$path/$octopusPath/$line is not executable...\n";
	  $ret = OCTOPUS_NO_EXE_CODE;
	}
      }

      if(-d "$path/$octopusPath/$line") {
	  if($line =~ /$arch/) { 
      		if (! -x "$path/$octopusPath/$line/octopus") {
		  $ERROR .= "$path/$octopusPath/$line/octopus is not executable...\n";
		  $ret = OCTOPUS_NO_EXE_CODE;
		}
	  } else {
		next; 
  	  }
      }
      if($line =~ /$arch/) { 
      	$ckSum = `cksum $path/$octopusPath/$line/octopus`;
      } else {
      $ckSum = `cksum $path/$octopusPath/$line`;
      }

      chomp $ckSum;

      if ($ckSum =~ /(\d+)\s+(\d+)\s+(.*)/) {
	my $name = basename($3);
	my $sum1 = $1;
	my $sum2 = $2;
	if (defined($scriptCksum{$name})) {
	  if (($scriptCksum{$name}[0] != $sum1) || ($scriptCksum{$name}[1] != $sum2)) {
	    $ERROR .= "$name has bad cksum [$scriptCksum{$name}[0], $scriptCksum{$name}[1]] expected versus [$sum1, $sum2] detected\n";
	    $ret = BAD_CKSUM_CODE;
	  }
	} else {
	  $ERROR .= "$name found but unknown\n";
	  $ret = UNKNOWN_SCRIPT_CODE;
	}
      } else {
	$ERROR .= "Failed to determine checksum of '$path/$octopusPath/$line' ($ckSum)\n";
	$ret = BAD_CKSUM_SYNTAX_CODE;
      }
    }
    close LOOK;
    system("cd $path;");
  } else {
    my ($exitCode, $signal) = Utils::ConvertExitCode($?);
    $ERROR = "$cmd FAILED (signal $signal, exitcode $exitCode): $!\n";
    $ret = VERS_DET_FAILED_CODE;
  }  
  return $ret;
}

# Empty hash
sub EmptyHash($) { my ($table) = @_; foreach my $key (keys %{$table}) { delete $table->{$key}; } }

# Reset hash
sub ResetHash($$) { my ($table, $value) = @_; foreach my $key (keys %{$table}) { $table->{$key} = $value; } }

#-- End of library
1;
