#==============================================================================
#-- File:         SshChannel.pm
#-- Date:         20.05.2009
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert, JA54
#------------------------------------------------------------------------------
#-- Function:     Module for SSH communication to the Asam Craft
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#--               There is no warranty about it, use of this script is at own
#--               risk
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel CID 2001
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0   - 50052009   : First implementation
#--   3.0.00  - 2010/04/21 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01  - 2010/05/21 : Converged PBMT Changes
#--   3.0.02  - 2011/01/12 : updated lib-perl path for LRM support
#--   3.0.03  - 2011/03/27 : linux compatible changes
#--   3.0.04  - 2011/05/05 : ALU01178887 
#--   3.0.05  - 2013/04/29 : ALU01880147
#--   3.0.06  - 2013/06/14 : ALU01877212
#--   3.0.07  - 2015/10/19 : ALU02136122
#--   3.0.08  - 2015/01/07 : ALU02160776
#--   3.0.09  - 2016/01/18 : ALU02166695 - PBMT v3.2.61
#--   3.0.10  - 2016/07/15 : ALU02237966
#--   3.0.11  - 2016/09/28 : ALU02278259
#==============================================================================

=head1 VERSION

=begin html
Current version: "SshChannel.pm - 3.0.11 -- 2016/09/28"

=end html

=cut

my $VERSION = "perlTL1::SshChannel.pm - 3.0.11 -- 2016/09/28";
#==============================================================================
package SshChannel;

use strict;
#-- Includes
use Cwd;
use File::Basename;
use Config;

use vars qw ($tmpLogFile);

#-- 1.2.4.1 -- BEGIN
BEGIN {
  my $dir = dirname($0);
  my $path;
  my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif( $dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib");
  unshift(@INC, "$path/lib-perl/$perlLibLocation");

  if ($Config{archname} =~ /sun4/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$path/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
}

END {
  unlink($tmpLogFile) if (defined($tmpLogFile) && ($tmpLogFile ne ""));
  $tmpLogFile = "";
}
require Net::Telnet;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub SelectFH($);
sub SetDebug($);
sub GetError();
sub Spawn(@);
sub ConnectSSH($$$;$$$);
sub Disconnect($);
sub SendCommand($$;$);
#==============================================================================
#== Constants
#==============================================================================
my $CONN_RESET = "Connection reset by peer";
my $FILE_CLOSED = "write error: filehandle isn\'t open";
my $INVALID_TOKEN = "invalid token";
my $INTERNAL_ERROR = "Internal processing error";
my $TELNET_CLOSED = "telnet session is closed/no longer opened";
my $ERR = "Error : ";
#==============================================================================
#== Variables
#==============================================================================
my $DEBUG = undef;
my $tmpLogFile = "";
my $ERROR = "";
my $responseEmpty = 0;
my $connected = 0;     # Set to 1 when IsamCli::Connect has been called, 0 when disconnect was called.
my $sshPid = undef;
my $sshTty = undef;
my $MAXEMPTYRESP = 10;
our $neId = ""; ## NE's ID

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 SetDebug()

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
#  print $DEBUG "Testing...\n";
}

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}

=head2 Spawn(@)

=begin html
Spawn a program.  The function returns the pty associated to the spawned program or undef in case of failure.

=end html

=cut

sub Spawn(@) {
  my(@cmd) = @_;
  my($pid, $pty, $tty, $tty_fd);

  $pty = undef;
  ## Create a new pseudo terminal.
  use IO::Pty ();
  $^W = 0;
  $pty = new IO::Pty;
  if ($pty) {
    ## Execute the program in another process.
    $pid = fork();
    if (defined($pid)) {
      if (! $pid) {
	# In child process
	## Disassociate process from existing controlling terminal.
	use POSIX ();
	if (POSIX::setsid) {
	  ## Associate process with a new controlling terminal.
	  $tty = $pty->slave;
	  $pty->make_slave_controlling_terminal();
	  $tty_fd = $tty->fileno;
	  close $pty;
	  ## Make stdio use the new controlling terminal.
	  if (open STDIN, "<&$tty_fd") {
	    if (open STDOUT, ">&$tty_fd") {
	      if (open STDERR, ">&STDOUT") {
		close $tty;
		$sshTty = $tty;
		## Execute requested program.
		if (! exec @cmd) {
		  $ERROR = "Spawn failure: problem executing $cmd[0]: $!";
		  $pty = undef;
		}
	      } else {
		$ERROR = "Spawn failure: Redirecting STDERR failed: $!";
		$pty = undef;
	      }
	    } else {
	      $ERROR = "Spawn failure: Redirecting STDOUT failed: $!";
	      $pty = undef;
	    }
	  } else {
	    $ERROR = "Spawn failure: Redirecting STDIN failed: $!";
	    $pty = undef;
	  }
	} else {
	  $ERROR = "Spawn failure: setsid failed: $!";
	}
      } else {
	# In parent.
	$sshPid = $pid;
	ToolCommon::Log($DEBUG, "Spawned '@cmd' as $pid\n") if defined($DEBUG);
      }
    } else {
      $ERROR = "Spawn failure: failed to create child process";
      $pty = undef;
    }
  } else {
    $ERROR = "Spawn failure: Allocating pty failed: $!";
  }
  $^W = 1;
  return $pty;
}

=head2 ConnectSSH($$$$;$$$)

=begin html
Open an ssh session using a telnet socket on the local machine and establish CLI login.
The opened CLI session is returned to the caller or undef in case of failure.<br>
The following parameters are used:
<table>
  <tr><td>ip</td><td>IP address of the NE</td></tr>
  <tr><td>user</td><td>TL1 user name</td></tr>
  <tr><td>passwd</td><td>TL1 password</td></tr>
  <tr><td>params</td><td>pointer to a hash of command paramater values</td></tr>
  <tr><td>timeout</td><td>TELNET timeout value</td></tr>
  <tr><td>logFile</td><td>file handle for logging of TELNET activity (undef if not needed)</td></tr>
  <tr><td>portId</td><td>TL1/SSH port</td></tr>
</table>

=end html

=cut

sub ConnectSSH($$$;$$$) {
  my ($ip, $user, $passwd, $timeout, $logFile, $portId) = @_;
  my ($ssh, $pty);
  my $port = defined($portId) ? $portId : 1022;
  my $hostAutoUpd = 0;
  my $homeDir = $ENV{HOME};
  my $config = FamilyCommon::GetConfigData();
  $neId = $ip;

  $hostAutoUpd = 1 if($params->{UPD_KNOWN_HOST_FILE}); # indicates known hosts file should be updated automatically.
  my $sysKnownHosts = $params->{SYS_KNOWNHOSTS} if defined($params->{SYS_KNOWNHOSTS});
  my $locKnownHosts = $params->{LOC_KNOWNHOSTS} if defined($params->{LOC_KNOWNHOSTS});
  if ($sysKnownHosts !~ m/^[\/]/) {
	  $sysKnownHosts = "$homeDir/$sysKnownHosts";
  }
  if ($locKnownHosts !~ m/^[\/]/) {
	  $locKnownHosts = "$homeDir/$locKnownHosts";
  }
 
  my ($retLocal, $retSystem, $errLocal, $errSystem ) = ( 0, 0, "", "" );
  ToolCommon::Log($DEBUG, "KnownHostCheck of user specific known hosts file\n") if defined($DEBUG);
  ($retLocal, $errLocal) = ToolCommon::KnownHostCheck($hostAutoUpd, $locKnownHosts);
  if ($retLocal) {
	  ToolCommon::Log($DEBUG, "KnownHostCheck of system specific known hosts file\n") if defined($DEBUG);
	  ($retSystem, $errSystem) = ToolCommon::KnownHostCheck($hostAutoUpd, $sysKnownHosts);
	   if ($retSystem) {
		   $ERROR = ( $errLocal ne "" ) ? "User specific known hosts : $errLocal\n" : "System specific known hosts : $errSystem\n";
		   return undef;
	   }
	}

  ## Start ssh program.
  $ERROR = "";
  $pty = &Spawn("ssh", "-l", $user, "-o UserKnownHostsFile=$locKnownHosts", "-o GlobalKnownHostsFile=$sysKnownHosts", "-p", $port, $ip);
  ToolCommon::Log($DEBUG, "Spawned ssh...\n") if defined($DEBUG);
    
  ## Create a Net::Telnet object to perform I/O on ssh's tty.
  $ssh = new Net::Telnet (-fhopen => $pty,
			  -timeout => defined($timeout) ? $timeout : 10,
			  -prompt => '/<\s$/',
			  -telnetmode => 0,
			  -cmd_remove_mode => 1,
			  -errmode => "return",
			  -input_log => defined($logFile) ? $logFile : "/tmp/telnet_ssh_$ip.trace",
			  -output_record_separator => "\r");
  if ($ssh) {
    ## Login to remote host.
    $tmpLogFile = defined($logFile) ? "" : "/tmp/telnet_$ip.trace";
    if (! $ssh->waitfor(-match => '/password: $/i',
			-errmode => "return")) {
      my $bufferdata = $ssh->get(Timeout=>0);
      if ( $bufferdata =~ /Are you sure you want to continue connecting \(yes\/no\)\?\s*$/ ) {
		 if ($hostAutoUpd) {
		  ToolCommon::Log($DEBUG, "Known hosts file automatic update of NE $neId\n") if defined($DEBUG);
		  $ssh->print("yes\n");
		  sleep(2);
		 } else {
#           $ERROR = $ssh->lastline;
            my $msg = "fingerprint not available on client";
            ToolCommon::Log($DEBUG, sprintf("No is selected to ensure network security\n")) if defined($DEBUG); 
            $ERROR = sprintf("Problem connecting to %s: %s (%s)", $ip, $ssh->errmsg, $msg);
            return undef;
		  }
      } else {
        my $msg = $ssh->lastline; $msg =~ s/\r//g; $msg =~ s/\n//g;
        $ERROR = sprintf("Problem connecting to %s: %s (%s)", $ip, $ssh->errmsg, $msg);
        return undef;
      }
    }
    ToolCommon::Log($DEBUG, sprintf("last line: '%s'\n", $ssh->lastline)) if defined($DEBUG);
    $ssh->print($passwd);
   my ($freeBufferContent, $readIteration) = (0, 0);
    ToolCommon::Log($DEBUG, "Read complete usage text\n") if defined($DEBUG);
    while ( ! $freeBufferContent ) {
		if (! $ssh->waitfor(-match => $ssh->prompt, -errmode => "return")) {
			 if ( $ssh->timed_out() ) {
				 if( $readIteration ) {
					  $freeBufferContent = 1;
					  last;
				 } else {
         			 $ERROR = "Timed-out after sending password";
                     return undef;
				 }
			 }
	         my $msg = $ssh->lastline; $msg =~ s/\r//g; $msg =~ s/\n//g;
	         $ERROR = sprintf("Login failed: %s (%s)", $ssh->errmsg, $msg);
	         $ssh->close();
	         close($sshTty);
	         $sshTty = undef;
	         unlink($tmpLogFile) if ($tmpLogFile ne "");
	         $pty = undef;
	         $ssh = undef;
		}
	  ToolCommon::Log($DEBUG, sprintf("Last line: '%s'\n", $ssh->lastline())) if defined($DEBUG);
	  if ($ssh->lastline() =~ /\*\/\s*/) {
				$freeBufferContent = 1;
				last;
	  }
	  $readIteration++;
    }
	ToolCommon::Log($DEBUG, sprintf("Freed the buffer content\n")) if defined($DEBUG);
	$ssh->buffer_empty();
  }
  return($ssh);
}

=head2 Disconnect($)

=begin html
Logout from the NE and close the telnet session specified by $telnet.

=end html

=cut

sub Disconnect($) {
  my ($telnet) = @_;
  my $deleted = -1;

  if ($connected) {
    $connected--;
    $telnet->cmd("LOGOFF;") if ! defined($sshPid);
    $telnet->close();
    if (defined($sshTty)) {
      close($sshTty);
      $sshTty = undef;
    }
    if ($tmpLogFile ne "") {
      $deleted = unlink($tmpLogFile);
      if ($deleted == 1) {
	$tmpLogFile = "";
      } else {
	$ERROR = "FAILED to delete '$tmpLogFile': $!";
      }
    }
  } else {
    $ERROR = "Not (yet) connected";
  }
}

=head2 SendCommand($$;$)

=begin html
Send a TL1 command to the ISAM.<br>
The following parameters are used:
<table>
  <tr><td>telnet</td><td>telnet session context</td></tr>
  <tr><td>cmd</td><td>CLI command to be sent</td></tr>
  <tr><td>checkError</td><td>check the response when defined.  In this case undef will be returned to the caller otherwise the parsing for errors
          is left to the caller</td></tr>
</table>
The command is retransmitted in case an alarm has been received during the command execution.<br>
The response received is returned to the caller.

=end html

=cut

sub SendCommand($$;$) {
  my ($telnet, $cmd, $checkError) = @_;
  my $rspReceived = 0;
  my $line;
  my $goOn = 1;
  my $rspLine = "";
  my $ors = $telnet->ors("");

  $ERROR = "";
  $responseEmpty = 0;
  $telnet->buffer_empty();
  $telnet->print($cmd);

  my $cmdtmp = $cmd;
  chop($cmdtmp); #This is to remove the semi-colon at the end.
  sleep (2);
  while ($goOn) {
    $line = $telnet->getline();
    if (defined($line)) {
      if ($line =~ /^;$/) {
        if ($rspLine =~ /REPT ALM/){
          ToolCommon::Log($DEBUG, "Alarm Ignored \n") if defined($DEBUG);
          $rspLine = undef;
        } else { 
          my $rspLinetmp = $rspLine;
          $rspLinetmp =~ s/\n//g;
          if ($cmdtmp ne $rspLinetmp && $rspLinetmp =~ /\Q$cmdtmp/){  #BDFam61079
            $goOn = 0;
          }elsif ($rspLinetmp =~ /SARB|SSTP/){
             ToolCommon::Log($DEBUG, sprintf("SARB/SSTP Busy error code received !!! for $cmdtmp \n"));
          } else {
            ToolCommon::Log($DEBUG, "Wrong Response recieved:'$rspLinetmp'\n") if defined($DEBUG);
            if($rspLinetmp !~ $cmdtmp) {
               $rspLine = undef;
            }
          }
        }
      } else {
        $line =~ s/^<.*\r\n$//mg;
        $line =~ s/^;//mg;	
        $rspLine .= $line;
      }
    } else {
      $goOn = 0;
      if ($telnet->timed_out()) {
	     $ERROR = "$cmd timed-out";
      } else {
	     $ERROR = sprintf("'%s' failed: %s", $cmd, $telnet->errmsg);
      }
    }
  }
  $telnet->ors($ors);
  return $ERROR eq "" ? $rspLine : undef;
}

#-- End of Module

1;
