#==============================================================================
#-- File:         UdpChannel.pm
#-- Date:         20.02.2001
#-- Company:      Alcatel Bell - TEC
#-- Author:       Donny Janssens, WD25
#------------------------------------------------------------------------------
#-- Function:      Module for Udp communication to the Asam Craft
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#--               There is no warranty about it, use of this script is at own
#--               risk
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel CID 2001
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0   - 20022001   : First implementation
#--   1.0.1   - 12042001   : Change timeout mechanism
#--                          Added ReadTidList function
#--   1.0.2   - 30052001   : Change Binding to IP address -> INADDR_ANY
#--   1.0.3   - 22092001   : Change ReadResponse to discard response if not
#--                          complete
#--                          Increase recv buffer to 64000
#--                          Add retransmissions
#--   3.0.00  - 2010/04/21 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01  - 2010/05/21 : Converged PBMT Changes
#--   3.0.02  - 2011/01/12 : Updated lib-perl path for LRM
#--   3.0.03  - 2011/03/27 : linux compatible changes
#--   3.0.04  - 2011/05/05 : ALU01178887 
#--   3.0.05  - 2013/04/29 : ALU01880147
#--   3.0.06  - 2016/07/15 : ALU02237966
#--   3.0.07  - 2016/09/28 : ALU02278259
#==============================================================================

my $VERSION = "perlTL1::UdpChannel.pm - 3.0.07 -- 2016/09/28";

package UdpChannel;

use strict;

#-- Includes
use Socket;
use Cwd;
use Sys::Hostname;
use File::Basename;
use Config;

#-- 1.2.4.1 -- BEGIN
BEGIN {
  my $dir = dirname($0);
  my $path;
  my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif( $dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib-perl/$perlLibLocation");

  if ($Config{archname} =~ /sun4/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$path/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
}

require ToolCommon;
use PBMTGlobal;
use Logger qw(:ALL);

#-- Constants
my $ATTEMPTS=4000;
my $SLEEP_PERIOD=0.05; #-- in seconds
my $RETRANSMISSIONS=5;

my $ctag = 11000;
my $rtrv_cnt = 0;
my $rtrvHdr  = 0;


=head1 NAME

UdpChannel - Library for Udp TL1 communication.

=head1 SYNOPSIS

use UdpChannel;

This library contains a number of functions, to send TL1 commands to an ASAM,
using a particular communication mechanism. In this case a Udp Port.

=head1 COMMANDS

=cut


=head2 UdpChannel::new

UdpChannel::new();

Constructor of this object.

Example:
  my $channel = new UdpChannel;

=cut

#----------------------------------------------------------------------------
#-- Constructor
#----------------------------------------------------------------------------
sub new {
  my $self = {};
  return bless $self;
}

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 UdpChannel::OpenChannel

UdpChannel::Openchannel( $self, $host, $port [, $logFile]);

$host is the hostname/ip of the asam
$port is the port where the asam is listening for TL1 (normally 13001)

Example:
  $channel->OpenChannel( "asam1", 13001);

Opens the connection to the ASAM.

returns 1 if successful, 0 if failed.

=cut

#----------------------------------------------------------------------------
#-- Open the Port
#----------------------------------------------------------------------------
sub OpenChannel {
  my ($self, $host, $port, $logFile) = @_;
  my $iaddr = gethostbyname( '0.0.0.0');
  my $proto = getprotobyname( 'udp');
  my $paddr = sockaddr_in(0, $iaddr);
  my $asam_iaddr = inet_aton( $host);
  socket( SOCKET, PF_INET, SOCK_DGRAM, $proto) || return 0;
  bind( SOCKET, $paddr) || return 0;
  #-- Keep the Socket & Asam address
  $self->{SOCKET}= \*SOCKET;
  $self->{ASAM_PADDR} = sockaddr_in( $port, $asam_iaddr);
  $self->{HOST} = $host;
  $self->{DEBUG} = defined($logFile) ? SelectFH($logFile) : undef;

  ToolCommon::Log($self->{DEBUG}, "host -> '$host', port -> '$port'\n") if defined($self->{DEBUG});
  return 1;
}


=head2 UdpChannel::CloseChannel

UdpChannel::Closechannel( $self);

Example:
  $channel->CloseChannel();

Closes the connection to the ASAM.

returns 1 if successful, 0 if failed.

=cut

#----------------------------------------------------------------------------
#-- Close the Port
#----------------------------------------------------------------------------
sub CloseChannel {
  my ($self) = @_;

  shutdown( $self->{SOCKET}, 2);
  $self->{SOCKET} = undef;
  $self->{ASAM_PADDR} = undef;
  $self->{HOST} = undef;
  $self->{ERROR} = "";
  $self->{DEBUG} = undef;
  return 1;
  }

#----------------------------------------------------------------------------
#-- Read from the port until ; appears (end of TL1 message)
#----------------------------------------------------------------------------
sub ReadResponse {
  my ($self) = @_;
  my $str="";
  my $rin = "";
  my $rout;
  my $str2;
  my $count = 0;
  my $recvaddr;

  vec ($rin, fileno( $self->{SOCKET}), 1) = 1;
  while (($count < $ATTEMPTS) && (index( $str, ';') == -1)&& ($str ne "No Resources.... Maximum sessions (6) exceeded") ) {
    if (select( $rout = $rin, undef, undef, $SLEEP_PERIOD)) {
      ($recvaddr = recv( $self->{SOCKET}, $str2, 64000, 0)) || return "";
      if (length( $str2)) {
	$str .= $str2;
	#-- if we receive something, reset the timeout period
	$count = 0;
      }
    }
    $count++;
  }

 if (defined($str) && ($str =~ /IP/) && !(length($str2)) && (index( $str, ';') == -1) && ($str ne "No Resources.... Maximum sessions (6) exceeded") ) {
    $ATTEMPTS = 10000;
    $count = 0;
    LogDebug( $nodeLogFH, "Partial response received till now : $str\n");
    while ( ($count < $ATTEMPTS) && (index( $str, ';') == -1) ) {
     if (select( $rout = $rin, undef, undef, $SLEEP_PERIOD)) {
      ($recvaddr = recv( $self->{SOCKET}, $str2, 64000, 0)) || return "";
      if (length( $str2)) {
        $str .= $str2;
        $count = 0;
      }
     }
    $count++;
    }
   }
  #-- if there is no end of command (;), discard response
  if ((index( $str, ';') == -1) && ($str ne "No Resources.... Maximum sessions (6) exceeded")) {
    return "";
  } else {
    return $str;
  }
}

=head2 UdpChannel::SendTL1Command

UdpChannel::SendTL1Command( $self, $command);

Example:
  my $response = $channel->SendTL1Command( "RTRV-EQPT:TEST-ASAM:ALL;");

Sends the commandstr which is given to the ASAM and returns the unparsed,
raw response.

returns undef if  send or receive failed.

=cut

#----------------------------------------------------------------------------
#-- Send one command and return the response
#----------------------------------------------------------------------------
sub SendTL1Command {
  my ($self, $command) = @_;
  #-- Send the command
  #ToolCommon::Log($self->{DEBUG}, "$command\n") if defined($self->{DEBUG});

  my $retries = $RETRANSMISSIONS;
  my $response = "";
  while ($retries != 0) {
    if (!defined( send($self->{SOCKET}, $command, 0, $self->{ASAM_PADDR}))) {
      $self->{ERROR} = "Send failed: $!";
      return undef;
    }
    

    my @actualCmdArray;
    my $actualCmd = undef;
    @actualCmdArray=split(/:/, $command);
    $actualCmd = $actualCmdArray[0];

    #-- Read the actual response
    my $readretries = 2;  #BDFam63711 BDFam63688   #ALU01823269
    
    if ($actualCmd =~ /ACT-USER/) {
        $readretries = 10;   # Work around only for ACT-User in the case of upgrade delays....
    }
	if ($actualCmd =~ /CANC-USER/) {
        $readretries = 1;   # Work around only for CANC-User....
    }

    while ($readretries > 0) {
      $response = $self->ReadResponse();
      if ($response ne "") {
		 my ($almflag, $evtflag ) = (0, 0);
         if ($response =~ /REPT ALM/) {
			 LogDebug( $nodeLogFH, "Alarm received \n");
			 $almflag = 1;
		 }
		 if ($response =~ /REPT EVT/) {
			 LogDebug( $nodeLogFH, "Event received \n");
			 $evtflag = 1;
		 }
		 if (($almflag ne 1) && ($evtflag ne 1)) {
			 $readretries = 0;
		 }
      } else { 
        LogDebug( $nodeLogFH, "NO RESPONSE RECEIVED Reading:$readretries\n");
        sleep(60) if($readretries != 1);
        $readretries--;
      }
    }
	if (defined $response and $response ne "") {
      $retries = 0;
    } else {
      LogDebug( $nodeLogFH,"INVALID RESPONSE FOUND\n");
      if ( uc($command)=~/ED-OSWP/ ) {
		return undef if($response eq "");          #ALU0178891
	  }
	  if ( uc($command)=~/CANC-USER/ ) {
        return undef if($response eq "");          #ALU0178891
      }

	  if ( ($command =~ /STP-CMDSSN/i) || ($command =~ /alw-msg-ALL/i) || ($command =~ /CANC-USER/i) ) {
         LogDebug( $nodeLogFH, "No RTRV-HDR retry\n");
		 $retries = 0;
	  } else {
        LogDebug( $nodeLogFH, "Try RTRV-HDR before sending the command again\n");
        my ($t, $prevCmdResponse) = RtrvHdr($self,$actualCmd);
        if($t) {
         if (defined($prevCmdResponse)) {
            LogDebug( $nodeLogFH, "Response received for the actual command. \n");
            $response = $prevCmdResponse;
         } else {
            LogDebug( $nodeLogFH, "Response received for RTRV-HDR re-send the previous command again\n");
         }
          $retries = 0;
		} else {
         LogDebug( $nodeLogFH, "RTRV-HDR no response recieved!!! \n");
         $retries--;
		}
	  }
	}
  } #while 
  if($response eq "") {
    $self->{ERROR} = "No response received";
    return undef;
  } else {
    #-- OK, we got a valid response, remove the prompts which could be there
    $response =~ s/^<.*\r\n$//mg;
    $response =~ s/^;//mg;
    return $response;
  }
}
sub SetRetrieveHeaderCount($) {
    my ( $count) = @_;
    $rtrv_cnt = $count;
}

sub GetRetrieveHeaderCount() {
    return $rtrv_cnt;
}

##############################
#ALU01823269 seperated teh rtrvhdr from the send command function 
#############################

sub RtrvHdr {
  my ($self, $prevCmd) = @_;
  my $cnt =5;
  my $tmpCmd ="";
  my $hdrresponse;
  #my $rtrvHdr =0;
  my $actualCmd = undef;
  my @actualCmdArray;
  my $prevCmdResponse = undef;
  $rtrv_cnt=0;
  for ($cnt=5; $cnt>=1; $cnt--) {
      my $ctagvalue = IsamTL1::GetCtagValue() +1;
      IsamTL1::SetCtagValue($ctagvalue);
      $tmpCmd="RTRV-HDR:::".$ctagvalue.":::;";
      @actualCmdArray=split(/;/, $tmpCmd);
      $actualCmd = $actualCmdArray[0];
	  $rtrv_cnt++;
      if (!defined( send($self->{SOCKET}, $tmpCmd, 0, $self->{ASAM_PADDR}))) {
             $self->{ERROR} = "Send failed: $!";
             return undef;
      }
      LogDebug( $nodeLogFH, "Command Sent: $tmpCmd \n");

      $hdrresponse = $self->ReadResponse();
      if ($hdrresponse ne "") {
		  LogDebug( $nodeLogFH, "Alarm received \n") if ($hdrresponse =~ /REPT ALM/);
		  LogDebug( $nodeLogFH, "Event received \n") if ($hdrresponse =~ /REPT EVT/);
      } else {
        LogDebug( $nodeLogFH, "NO RESPONSE RECEIVED for RTRV-HDR \n");
      }

      if ( ($hdrresponse =~ /SARB|SSTP/) ){
		  LogDebug( $nodeLogFH, "SARB|SSTP Busy recieved... \n");
		  LogDebug( $nodeLogFH, "Response recieved: $hdrresponse \n");
      } elsif ( ($hdrresponse =~ /DENY/) ){
		  LogDebug( $nodeLogFH, "DENY recieved... \n");
		  LogDebug( $nodeLogFH, "Response recieved: $hdrresponse \n");
      } elsif (($hdrresponse =~/COMPLD/) && ($hdrresponse =~/$actualCmd/)) {
          LogDebug( $nodeLogFH, sprintf("Response received: %s \n",$hdrresponse));
          LogDebug( $nodeLogFH, "RTRV HDR responded COMPLD...\n");
          $rtrvHdr =1;
          last;
      } elsif (($hdrresponse =~/COMPLD/) &&  ($hdrresponse !~/$actualCmd/)) {
           LogDebug( $nodeLogFH, sprintf("Response received is not for %s: %s \n",$tmpCmd,$hdrresponse));
           if ($hdrresponse =~/$prevCmd/) {
              LogDebug( $nodeLogFH, sprintf("Response received is for %s: %s \n",$prevCmd,$hdrresponse));
              $prevCmdResponse = $hdrresponse;
			  #since original command response recieved read resonse for rtrv hdr
			  $rtrvHdr = ReadRemaining($actualCmd,$hdrresponse,$self,$cnt);
			  $rtrvHdr = 1;
              last;
           }

           if ($hdrresponse !~/$actualCmd/) {
			#since original rtrv-hdr command response recieved read resonse for remaining rtrv hdr
			$rtrvHdr = ReadRemaining($actualCmd,$hdrresponse,$self,$cnt);
			if ($rtrvHdr) {
                last;
			  }
		   }
	     sleep($cnt * 60);
		
      } elsif(($hdrresponse =~/PLNA/) &&  ($hdrresponse =~/$prevCmdResponse/)) {
		  LogDebug( $nodeLogFH, sprintf("Reading Response : %s \n",$hdrresponse));
		  $prevCmdResponse = $hdrresponse;
          $rtrvHdr = 1;
		  sleep(60);
      } else {
		   if ($hdrresponse =~ /REPT ALM/) {
			  LogDebug( $nodeLogFH, "Alarm received\n");
	       } elsif ($hdrresponse =~ /REPT EVT/) {
		      LogDebug( $nodeLogFH, "Event received\n");
		   } else {
              LogDebug( $nodeLogFH, sprintf("Response received: %s \n",$hdrresponse));
		      sleep(60);
		  }
        }
  }  #for
  ###Added for shutdown of port 13001###
  if( $prevCmd =~/ACT-USER/) {
	LogDebug( $nodeLogFH, "Close the socket \n");
	CloseChannel($self);
}
  if (defined($prevCmdResponse)) {
     return ($rtrvHdr, $prevCmdResponse);
  } else {
     return $rtrvHdr;
  }
}

### FR-ALU02074307 Added ReadRemaining functionality to read the remaining response of RTRV-HDR
sub ReadRemaining {
 my ($cmd,$response,$self,$cnt) = @_;
 my $HdrFlag = 0;
 my $recCtag;
 my $local = 0;
 my $tmp = 1;
 my $tmpCtag = substr($cmd,11,5);  # to get the command for which we got the response
 if ($response =~/RTRV-HDR/) {
     #$tmpCtag = substr($cmd,11,5);
   my @tmpresponse = split(/\n/, $response);
                    foreach my $line (@tmpresponse) {
                        if($line=~/RTRV-HDR/) {
                            $line=~ s/^\s+//;
                            $recCtag = substr($line,14,5);
                            $local = $tmpCtag - $recCtag;
                            LogDebug( $nodeLogFH, sprintf("recCtag : %s, tmpCtag : %s, local : %s\n",$recCtag, $tmpCtag, $local));
                            last;
                        }
                    }
              } else {
                  if($response =~ /[^\s]+\s+(\d+)\sCOMPLD/){
                    $recCtag = $1;
                  }
                $local = $tmpCtag - $recCtag;
                LogDebug( $nodeLogFH, sprintf("recCtag : %s, tmpCtag : %s, local : %s\n",$recCtag, $tmpCtag, $local));
              }

              for ($tmp=1; $tmp<=$local; $tmp++) {
                LogDebug( $nodeLogFH, sprintf("Reading Remaining Response : %s \n",$tmp));
                $response = $self->ReadResponse();  #
                LogDebug( $nodeLogFH, sprintf("Response : %s \n",$response));
                if ($response =~ /$tmpCtag/) {
                  LogDebug( $nodeLogFH, sprintf("Response Received for : %s \n",$tmpCtag));
                  $local = $cnt+1;
                  $HdrFlag =1;
                  last;
                } else {
                  sleep (60);
                }
           }
           return $HdrFlag;
}
=head2 UdpChannel::ReadTidList

UdpChannel::ReadTidList( $command);

Example:
  my $tids = $channel->ReadTidList();

Reads the list of managed TIDs, for Udp communication this will be only one
ASAM.

returns undef if  send or receive failed.

=cut

#----------------------------------------------------------------------------
#-- Send the list of managed TIDs
#----------------------------------------------------------------------------
sub ReadTidList {
  my ($self) = @_;
  my $response = SendTL1Command($self, "RTRV-HDR;");

  return undef if (!defined( $response));
  return ($response->{SID});
}

sub GetError {
  my ($self) = @_;

  return $self->{ERROR};
}

#-- End of Module

1;
