#!/usr/bin/perl
#==============================================================================
#-- File:         mapVoiceIp2Oam.pl
#-- Date:         09.09.2009
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to map voice IP address to NT OAM IP address
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel-Lucent.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent 2009
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2009/09/09 : First implementation
#--   3.0.1 - 2011/01/11 : LRM updates for lib-perl path
#--   3.0.2 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.3 - 2011/05/02 : ALU01178887
#--   3.0.4 - 2012/10/22 : ALU01761938
#--   3.0.5 - 2012/11/06 : ALU01837839
#--   3.0.6 - 2013/04/29 : ALU01880147
#--   3.0.7 - 2015/10/28 : ALU02139820
#--   3.0.8 - 2016/07/15 : ALU02237966
#--   3.0.9 - 2016/09/28 : ALU02278259


my $VERSION = "mapVoiceIp1Oam.pl - 3.0.9 - 2016/09/28";
#==============================================================================

use strict;
use warnings 'all';
require 5.001;

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Cwd qw( realpath getcwd);
use File::Basename;
use Config; 

my $scriptDir; 

BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
    unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
    unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}


use lib "$scriptDir/lib"; 
use Utils;
use PbmtConfig;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :IPINV);
use version qw(:ALL);
use Crypt::Lite;
use SNMPComm;
use IsamCli;
require FamilyCommon;
use Net::SNMP qw(:asn1);

#==============================================================================
#== Vars
#==============================================================================
my %toolData = ('HOSTSFILE' => "/etc/hosts",
		'TFTPDIR' => "/global/aws_var/aws/equipment",
		'TFTPWORKDIR' => "tmp",
		'USERNAME' => "",
		'PASSWORD' => "",
		'TELNETTIMEOUT' => 10,
		'NTDBTOOLDIR' => "tools/NT",
		'NTDBTOOL' => "tools/NT/MJSH.exe",
		'NTDBCFG' => "tools/NT/dbtool.cfg",
		'NTDBCFGMOD' => "dbtoolMOD.cfg",
		'NODEPREFIX' => "AS-",                          # -- 1.1.6
		'RETRIES' => 5,
		'MAXWAITTIME' => 1800, #-- 30 Minutes
		'FXRDATA' => undef);
my %config;

#------------------------------------------------------------------------------
#-- Display the usage
#------------------------------------------------------------------------------
sub Usage {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <IP> | --hosts <file> --logfile <file> [--username <user>] [--password <password>] [--debug] [--v] \\ \n";
  print STDERR "                [--sessions <n>] [--comm <SNMP community string>] [--v2] \\ \n";
  print STDERR "                [--v3auth <user>:<password>:<authProtocol>] [--v3priv <privProtocol>:<password>] [--noname]\n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         isam     : IP address of ISAM to be converted\n";
  print STDERR "         username : ISAM username\n";
  print STDERR "         password : ISAM password\n";
  print STDERR "         sessions : the number of parallel sessions being invoked.\n";
  print STDERR "         noname   : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "       SNMP settings:\n";
  print STDERR "         comm     : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is 'public'\n";
  print STDERR "         v2       : Use SNMPv2c\n";
  print STDERR "         v3auth   : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                    The password should be at least 8 characters long.\n";
  print STDERR "                    Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                    All fields are mandatory when an option is used.\n";
  print STDERR "                    e.g. --v3auth user:password:: will result in MD5 as protocol.\n";
  print STDERR "         v3priv   : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                    Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  exit(CMD_INCOMPL_CODE);
}

#------------------------------------------------------------------------------
#-- Parse all command line parameters
#------------------------------------------------------------------------------
sub ParseCommandLine {
  my ($config) = @_;
  my ($isam, $hosts, $debug, $logfile, $version,  $username, $password, $sessions, $communityString, $v3Auth, $v3Priv, $snmpV2, $noName);
  my %params;
  my ($dirName1, $dirName2);
  my $options = GetOptions('isam=s', \$isam,
			   'hosts=s', \$hosts,
			   'logfile=s', \$logfile,
			   'username=s', \$username,
			   'password=s', \$password,
			   'v', \$version,
			   'v2', \$snmpV2,
			   'debug', \$debug,
			   'noname', \$noName,
			   'sessions=i', \$sessions);
  #-- check options
  if (! $options) {
    Usage();
  }
  if (defined($version)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logfile)) {
    print STDERR "ERROR: no logfile given\n";
    Usage();
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }
  $params{NONAME} = defined($noName) ? 1 : 0;
  if (defined($username)) {
    $params{USERNAME} = $username;
  } else {
    $params{USERNAME} = $toolData{USERNAME};
  }
  if (defined($password)) {
    $params{PASSWORD} = $password;
  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }
  $params{DEBUG} = defined($debug) ? 1 : 0;
  $params{ISAM} = $isam;
  $params{LOGFILE} = $logfile;
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    $params{SESSIONS} = $sessions;
  }
  if (!defined($snmpV2)) {
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }
  if (defined($v3Auth)) {
    my @snmpV3AuthParms = split(':', $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA256|SHA512|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
      if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
         $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
      }
	} else {
	  print STDERR "SNMPv3 authentication protocol can only be SHA or SHA256 or SHA512 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else { 
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
    $params{SNMP}{V3}{AUTH}{USERNAME} = $toolData{SNMPV3_USERNAME};
    $params{SNMP}{V3}{AUTH}{PASSWORD} = $toolData{SNMPV3_AUTH_PASSWORD};
    $params{SNMP}{V3}{AUTH}{PROTO} = $toolData{SNMPV3_AUTH_PROTOCOL};
    if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
       $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  }
  if (defined($v3Priv)) {
    my @snmpV3PrivParms = split(':', $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
    $params{SNMP}{V3}{PRIV}{PROTO} = $toolData{SNMPV3_PRIV_PROTOCOL};
    $params{SNMP}{V3}{PRIV}{PASSWORD} = $toolData{SNMPV3_PRIV_PASSWORD};    
  }
  if (($params{PASSWORD} eq "") || ($params{USERNAME} eq "")) {
    print STDERR "No username or password defined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }
  $params{SEPLOG} = 1;
  return \%params;
}

#------------------------------------------------------------------------------
#-- Print the command line parameters
#------------------------------------------------------------------------------
sub PrintLibVersions {
  my ($fh, $out) = @_;

  if (defined($fh)) {
    ToolCommon::Log($fh, "Library versions:\n", $out);
    ToolCommon::Log($fh, sprintf("  %s\n", Utils::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ErrorCodes::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ToolCommon::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", PbmtConfig::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", FamilyCommon::GetVersion()), $out);
  } else {
    print STDERR "Library versions:\n";
    print STDERR sprintf("  %s\n", Utils::GetVersion());
    print STDERR sprintf("  %s\n", ErrorCodes::GetVersion());
    print STDERR sprintf("  %s\n", ToolCommon::GetVersion());
    print STDERR sprintf("  %s\n", PbmtConfig::GetVersion());
    print STDERR sprintf("  %s\n", FamilyCommon::GetVersion());
  }
}

sub PrintOptionsSummary {
  my ($fh, $out) = @_;

  ToolCommon::Log($fh, sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION), $out);
  ToolCommon::Log($fh, "Options:\n", $out);
  ToolCommon::Log($fh, sprintf("   working dir           : %s%s\n", $toolData{TFTPDIR},
			       $toolData{TFTPWORKDIR} =~ /^\// ? $toolData{TFTPWORKDIR} : "/$toolData{TFTPWORKDIR}"), $out);
  ToolCommon::Log($fh, "   SNMP settings:\n");
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      ToolCommon::Log($fh, "   Using SNMPv1\n", $out);
      ToolCommon::Log($fh, "     NT community string  : $params->{SNMP}{V1}{NT_COMMUNITY}\n", $out) if $params->{DEBUG};
    } else {
      ToolCommon::Log($fh, "   Using SNMPv2c\n", $out);
      ToolCommon::Log($fh, "     NT community string  : $params->{SNMP}{V2}{NT_COMMUNITY}\n", $out) if $params->{DEBUG};
    }
  } else {
    ToolCommon::Log($fh, "   Using SNMPv3\n", $out);
    if ($params->{DEBUG}) {
      ToolCommon::Log($fh, "            Authentication :\n", $out);
      ToolCommon::Log($fh, "                  username : $params->{SNMP}{V3}{AUTH}{USERNAME}\n", $out);
      ToolCommon::Log($fh, "                  password : *****\n", $out) if(defined($params->{SNMP}{V3}{AUTH}{PASSWORD}));
      ToolCommon::Log($fh, "                  protocol : $params->{SNMP}{V3}{AUTH}{PROTO}\n", $out) if(defined($params->{SNMP}{V3}{AUTH}{PROTO}));
      if (defined($params->{SNMP}{V3}{PRIV}{PROTO})) {
	ToolCommon::Log($fh, "                  Privacy :\n", $out);
	ToolCommon::Log($fh, "                  protocol : $params->{SNMP}{V3}{PRIV}{PROTO}\n", $out) if(defined($params->{SNMP}{V3}{PRIV}{PROTO}));
	ToolCommon::Log($fh, "                  password : *****\n", $out) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}));
      }
    }
  }
  ToolCommon::Log($fh, "   Debugging Enabled\n", $out) if (defined($params->{DEBUG}) && $params->{DEBUG});
  PrintLibVersions($fh, $out);
}

sub GetVoiceVlan($$) {
  my ($logFile, $cli) = @_;
  my @response = IsamCli::SendCommand($cli, "info configure vlan shub flat | match exact \"voice\-vlan\"");
  my $voiceVlan = undef;

  if (IsamCli::CommandError(join("", @response))) {
    my $t = sprintf("Retrieving SHUB voice VLAN info failed: %s", IsamCli::GetError());
    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
  } else {
    $voiceVlan = 0;
    foreach my $line (@response) {
      chomp $line;
      ToolCommon::Log($logFile, "line read: $line\n") if $params->{DEBUG};
      if ($line =~ /configure\s+vlan\s+shub\s+id\s+(\d+)/) {
	$voiceVlan = $1;
      }
    }
  }
  return $voiceVlan;
}

sub GetVoiceIp($$$) {
  my ($logFile, $cli, $vlan) = @_;
  my @response = IsamCli::SendCommand($cli, "info configure interface shub ip flat | match exact $vlan");
  my $voiceIp = undef;
  my $multipleIp = 0;

  if (IsamCli::CommandError(join("", @response))) {
    my $t = sprintf("Retrieving SHUB voice IP info failed: %s", IsamCli::GetError());
    FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
  } else {
    $voiceIp = "";
    foreach my $line (@response) {
      if ($line =~ /configure\s+interface\s+shub\s+ip\s+$vlan\s+ip-addr\s+(\d+\.\d+\.\d+\.\d+)\/\d+/) {
	my $thisIp = $1;
	if ($voiceIp eq "") {
	  $voiceIp = $thisIp;
	} else {
	  my $t = "Multiple voice IP addresses found: $thisIp (already found $voiceIp as first)";
	  FamilyCommon::SetAndLogError($logFile, $t, "$t\n");
	  $multipleIp = 1;
	}
      }
    }
    $voiceIp = "" if $multipleIp;
  }
  return $voiceIp;
}
		   

sub MakeInventory($$$$) {
  my ($fh, $isams, $count, $config) = @_;
  my $dirName = dirname($0);
  my $ret = SUCCESS_CODE;
  my $tarFileName = "";
  my $dbVersion;
  my $relNum;
  my $cli;
  my $isam;
  my $newLog = "$params->{LOGFILE}_$count";    # Name of child logfile
  my ($startTime, $endTime);
  my ($dir, $newfh, $nodeLogFile, $nodeLogFh, $thisTftpDir, $fxrData, $session);
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $failedNode;
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $oswpInfo;
  my $childNum = FamilyCommon::GetChildNum();

  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetChildNum($count);
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetConfigData($config);
  FamilyCommon::SetFailedNode($params->{ISAM});
  FamilyCommon::SetChildLogFile($fh);

  IsamCli::SetDebug($fh) if ($params->{DEBUG});
  ToolCommon::SetDebug($fh);           # -- 1.2.5 END

  $dirName = getcwd() if ($dirName eq ".");

  if (! open($newfh, ">$newLog")) {
    ToolCommon::Log($fh, "Child $count: **FATAL** cannot open $newLog for writing: $!\n", 1);
    CORE::exit(CREATE_FILE_FAIL_CODE)
  }
  $newfh->autoflush(1);
  ToolCommon::Log($fh, "Child $count: Logging continuous in $newLog...\n", 1);
  ToolCommon::Log($newfh, sprintf("Child %d [PID %s] started...\n", $count, FamilyCommon::GetOwnPid()));
  my $invFh;
  my $ipInvLogFile = sprintf("$dirName/IPinv_%d.csv", $count);
  if (! open($invFh, ">$ipInvLogFile")) {
    ToolCommon::Log($fh, "Child $count: **FATAL** cannot open $ipInvLogFile for writing: $!\n", 1);
    CORE::exit(CREATE_FILE_FAIL_CODE)
  }
  ToolCommon::Log($fh, " Inventory logging to               : '$ipInvLogFile\n", 1);

  PrintOptionsSummary($newfh, undef, $config);

  foreach $isam (@{$isams}) {
    $failedNode = $params->{NONAME} ? $isam->[0] : $isam->[1];
    FamilyCommon::SetFailedNode($failedNode); # Store for IncrErrorCount
    FamilyCommon::SetWarnLogFile($newfh);   # Make sure warnings and 
    FamilyCommon::SetChildLogFile($newfh);  # logging is sent to the child process' logfile.
    # Switch debugging (if any) to an opened filehandle...
    IsamCli::SetDebug($newfh) if ($params->{DEBUG});
    Utils::SetDebug($newfh) if ($params->{DEBUG});
    ToolCommon::SetDebug($newfh);           # -- 1.2.5 END
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();
    Net::SNMP->snmp_debug(0x3e, $newfh) if ($params->{DEBUG});

    ToolCommon::Log($newfh, "#===================================================\n");
    ToolCommon::Log($newfh, "#== $failedNode\n");
    ToolCommon::Log($newfh, "#===================================================\n");

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Started $failedNode...\n",
		    $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second), 1);
    #-- open snmp connection
    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
        FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError(), undef );
        next;
    }

    ToolCommon::Log($newfh, "Prepare logging directories...");
    ($dir, $thisTftpDir, $fxrData) = FamilyCommon::PrepareLogging($session, $newfh, $userName, $failedNode);

    if (not defined($fxrData)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_DETERMINE_LOG_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      next;
    } else {
      $toolData{FXRDATA} = $fxrData;
      ToolCommon::Log($newfh, "  OK\n");
    }

    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g if defined($thisTftpDir);
    ToolCommon::Log($newfh, "Creating workspace for '$dir'\n") if $params->{DEBUG};
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($fh, $newfh, $dir, $userName, $newLog, $count, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};

    FamilyCommon::SetNeStorage($dir);

    if ($params->{SEPLOG}) {
      $nodeLogFile = sprintf("h248toSip_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
      if (! open($nodeLogFh, ">$dir/$nodeLogFile")) {  # -- 1.2.5 START
	ToolCommon::Log($fh, "Child $count: cannot open $dir/$nodeLogFile for writing: $!\n", 1);
	CORE::exit(CREATE_NODE_FILE_FAIL_CODE)
      }
      $nodeLogFh->autoflush(1);
      IsamCli::SetDebug($nodeLogFh) if ($params->{DEBUG});
      Utils::SetDebug($nodeLogFh) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFh);           # -- 1.2.5 END
      PrintOptionsSummary($nodeLogFh, undef, $config);
      FamilyCommon::SetWarnLogFile($nodeLogFh);   # Adjust warning and child logfile handles.
      FamilyCommon::SetChildLogFile($nodeLogFh);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
      Net::SNMP->snmp_debug(0x3e, $nodeLogFh) if ($params->{DEBUG});
      ToolCommon::Log($newfh, "Divert NE specific logging to $dir/$nodeLogFile now\n");
    } else {
      $nodeLogFh = $newfh;
    }
    ToolCommon::Log($nodeLogFh, "Child $count: Using the following file storage locations:\n", 1);
    ToolCommon::Log($nodeLogFh, sprintf(" Script specific logfiles stored in : '%s'\n", getcwd()), 1);
    ToolCommon::Log($nodeLogFh, " NE specific logging to             : '$dir/$nodeLogFile'\n", 1) if $params->{SEPLOG};

    $cli = ToolCommon::OpenCli($isam, $nodeLogFh, $toolData{TELNETTIMEOUT});
    if (!defined($cli)) {
      FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for $isam->[0]: %s\n", IsamCli::GetError()),
				   EC_OPENCLI_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    my $voiceVlan = GetVoiceVlan($nodeLogFh, $cli);
    if (defined($voiceVlan)) {
      if ($voiceVlan != 0) {
	my $voiceIp = GetVoiceIp($nodeLogFh, $cli, $voiceVlan);
	print $invFh "$isam->[0];$voiceIp\n";
	if (defined($voiceIp)) {
	} else {
	  FamilyCommon::IncrErrorCount(FamilyCommon::GetError(),
				       EC_GET_VOICEIP_FAILED,
				       FamilyCommon::GetError(),
				       1);
	  next;
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(FamilyCommon::GetError(),
				   EC_GET_VVLAN_FAILED,
				   FamilyCommon::GetError(),
				   1);
      next;
    }
    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
  
    FamilyCommon::CloseCliSession2Close();
    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Finished $failedNode... in %d seconds\n",
		     $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $endTime - $startTime), 1);
    FamilyCommon::CloseChildLogFile() if ($params->{SEPLOG});
  }
  close($invFh);
  if (FamilyCommon::GetErrorCount()) {
    ToolCommon::Log($fh, sprintf("Child %d: %d error(s) occured for the following ISAMs, please verify %s\n",
				 $count, FamilyCommon::GetErrorCount(),
				 defined($nodeLogFile) ? $nodeLogFile : $newLog), 1);
    my $failedNodes = FamilyCommon::GetFailedNodeList();
    foreach my $node (sort keys %$failedNodes) {
      ToolCommon::Log($fh, "  $node\n", 1);
    }
  }
  FamilyCommon::CloseChildLogFile() if (! $params->{SEPLOG});
  ToolCommon::Log($fh, "Child $count: finished...\n", 1);
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}
#------------------------------------------------------------------------------
#-- Adjust Tool defaults if needed (i.e. when defined in the parameter config
#-- file
#-- Modif BB
#------------------------------------------------------------------------------
sub AdjustToolVars {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'});                   # -- 1.1.6
  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
										 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
										($config->{'DEFAULTCLIPWD'} ne ""));
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT});            # -- 1.1.7
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{NTDBTOOLDIR} = $config->{'NTDBTOOLDIR'} if defined($config->{'NTDBTOOLDIR'});
  $toolData{NTDBTOOL} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTOFLMT'} if defined($config->{'NTOFLMT'});
  $toolData{NTDBCFG} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
										 ($config->{NT_COMMUNITY} ne ""));
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});
  $toolData{ACTPINGWAIT} = $config->{ACT_PING_WAIT} if defined($config->{ACT_PING_WAIT});
  $toolData{MAXWAITTIME} = $config->{'WAITTIME'} if defined($config->{'WAITTIME'});
}

#==============================================================================
#== Main program
#==============================================================================
my $count = 0;
my @pids;
my $pid;
my $warnCount = 0;

sub CleanUp {
  my ($signal, $exitCode) = @_;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string;
  ToolCommon::Log($warnLogFile, $string) if defined($warnLogFile);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      ToolCommon::Log($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR "  Aborting children...\n";
    ToolCommon::Log($warnLogFile, "  Aborting children...\n") if defined($warnLogFile);
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	ToolCommon::Log($warnLogFile, "Killing process $pid...\n") if defined($warnLogFile);
	kill($pid);
      }
    }
  }
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();

  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  if (!($msgStr =~ /Zlib/)) {
    print $warnLogFile "Warning: $msgStr" if (defined($warnLogFile));
    print STDERR "Warning: $msgStr";
    $warnCount++;
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  ToolCommon::Log($warnLogFile, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};

STDOUT->autoflush(1);
STDERR->autoflush(1);
#-- Read parameter config file and adjust tool defaults if needed
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
%config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars(\%config);
#-- Handle commandline
$params = ParseCommandLine(\%config);

my $logFile;
if (!open($logFile, ">$params->{LOGFILE}")) {
  print STDERR "Cannot open $params->{LOGFILE} for writing: $!\n";
  exit CREATE_FILE_FAIL_CODE;
}
$logFile->autoflush(1);
FamilyCommon::SetWarnLogFile($logFile);
ToolCommon::Log($logFile, sprintf("Parent started as %d\n", FamilyCommon::GetOwnPid()));

PrintOptionsSummary($logFile, 1);

if (defined($params->{SIPCFG})) {
  if (!ReadSipConfig($logFile, "$params->{SIPCFG}")) {
    exit -1;
  }
}

Utils::SetDebug($logFile) if $params->{DEBUG};
#-- Create work dir
my $tmpDir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}";
$tmpDir =~ s/\/\//\//;
if (! -d $tmpDir) {
  if (!mkdir( $tmpDir, 0777)) {
    ToolCommon::Log($logFile, "Cannot create directory $tmpDir: $!\n", 1);
    exit CREATE_DIR_FAILED_CODE;
  }
  $tmpDir .= "/$params->{ISAM}";
  if (!mkdir( $tmpDir, 0777)) {
    ToolCommon::Log($logFile, "Cannot create directory $tmpDir: $!\n", 1);
    exit CREATE_DIR_FAILED_CODE;
  }
  $toolData{FXRDATA}{HOMEDIR} = $tmpDir;
}

my $isamsRef;
if (defined( $params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    ToolCommon::Log($logFile, "Translate host-IP to hostname\n") if $params->{DEBUG};
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    ToolCommon::Log($logFile, "Translate hostname to host-IP\n") if $params->{DEBUG};
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}] ];
  }
} else {
  ToolCommon::Log($logFile, "Reading Hostfile $params->{HOSTS}...\n", 1);
  $isamsRef = ToolCommon::ReadHostsFile($logFile, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

my $isamsPerSession = int(scalar(@isams) / $params->{SESSIONS});
if (scalar(@isams) - $isamsPerSession*$params->{SESSIONS} >= $params->{SESSIONS}/2) {
  $isamsPerSession++;
}

while ($count < $params->{SESSIONS}) {
  if ($pid = fork()) {
    ToolCommon::Log($logFile, "Child $count: started as $pid\n", 1);
    push(@pids, $pid);
    splice(@isams, 0, $isamsPerSession);
  } else {
    if ($count+1 == $params->{SESSIONS} ) {
      MakeInventory($logFile, \@isams, $count, \%config);
    } else {
      my @isamPart = splice( @isams , 0, $isamsPerSession);
      MakeInventory($logFile, \@isamPart, $count, \%config);
    }
    exit SUCCESS_CODE;
  }
  $count++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
my %ipInventory = ();
my $invFh;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      ToolCommon::Log($logFile, sprintf("Child %d: exit code: '%d' (%s), signal: '%d'\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal), 1);
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  ToolCommon::Log($logFile, "All childs finished, consolidate IP address mapping files in IPinventory.csv\n", 1);
  if (open($invFh, "<IPinventory.csv")) {
    # Read inventory so far
    my $line;
    while (defined($line = <$invFh>)) {
      if ($line =~ /^\#/) {
	# Skip comments
      } elsif ($line =~ /([^;]+);([^;]+)/) {
	$ipInventory{$1} = $2;
      } else {
	# ignore
      }
    }
    close($invFh);
  }
}
ToolCommon::Log($logFile, "All childs finished, exiting\n", 1);
my $ipInvLogFh;
my $dir = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
for (my $i = 0; $i < $count; $i++) {
  my $ipInvLogFile = sprintf("$dir/IPinv_%d.csv", $i);
  if (! open($ipInvLogFh, "<$ipInvLogFile")) {  # -- 1.2.5 START
    ToolCommon::Log($logFile, "Cannot open  for reading: $!\n", 1);
    CORE::exit(CREATE_FILE_FAIL_CODE);
  } else {
    while (defined(my $line = <$ipInvLogFh>)) {
      chomp $line;  #  remove '\n'
      if ($line =~ /^\#/) {
	# Ignore comments
      } elsif ($line =~ /([^;]+);([^;]+)/) {
	my $oamIp = $1;
	my $voiceIp = $2;
	$ipInventory{$oamIp} = $voiceIp;
      } else {
	# Unknown, skip.
      }
    }
    close $ipInvLogFh;
  }
}
if (open($invFh, ">$dir/3HH046964014.csv")) {
  print $invFh "#OAM IP;voice IP\n";
  foreach my $oamIp (sort keys %ipInventory) {
    print $invFh sprintf("%s;%s\n", $oamIp, $ipInventory{$oamIp});
  }
  close $invFh;
} else {
  ToolCommon::Log($logFile, "Cannot open IPinventory.csv for writing: $!\n", 1);
  CORE::exit(CREATE_FILE_FAIL_CODE);
}
FamilyCommon::CloseWarnLogFile();
print STDERR "$warnCount warnings encountered...\n" if ($warnCount);

exit $overAllExitCode;
