#!/usr/bin/perl

=head1 ISAM MIGRATION

=begin html
This script will migrate an ISAM from one release to the other taking care of all the steps needed in the process (helicopter view):
<ul>
  <li>upload the database from the life system</li>
  <li>perform the offline migration of the database using the various offline migration tools provided seprately from this package</li>
  <li>combine all migrated databases into one new consolidated database and download this to the ISAM</li>
  <li>restart the ISAM with this new database</li>
</ul>

The operations of the script are influenced by:
<ul>
  <li>command line options</li>
  <li>a parameter configuration file</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         migrate.pl
#-- Date:         05.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Donny Janssens (for the very first version)
#--               Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to migrate ISAMs to a new release
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2005/11/05 : First implementation
#--   1.0.1 - 2005/11/06 : Bug fixing after first test
#--   1.1.0 - 2005/11/18 : Added octopus script for SwMngt.ini > 1024
#--                        Added octopus script for disabling tftp daemon
#--                        Added Config file
#--                        Added check on 'enabled' new software
#--                        Added SHUB dbase save
#--   1.1.1 - 2005/11/23 : Added extraction of IGMPXmlfile.txt file for
#--                        migration from R2.2 to a higher release
#--   1.1.2 - 2005/12/14 : Added stability waits:
#--                          + wait for cli to be available after restart
#--                          + wait 2 mins before activating
#--   1.1.3 - 2005/12/15 : Added octopus to tools directory, hence path to
#--                        octopus was adjusted.
#--                        Added unix ping in stead of udp ping: caused by
#--                        bug in NT: BDFhw70276
#--                        Added latest LANX migration tool: has lined up
#--                        command parameters with NT OFLMT
#--                        Adopt for R2.4
#--                        Added secureDbSave option: pre-R2.3 releases may
#--                        need to wait some time after issueing the
#--                        SHUB database save to secure a successful upload
#--                        of dm_complete.tar following the db save.
#--   1.1.4 - 2006/01/05 : Some minor fixes
#--                        Included the ODM sub-directory when combining the
#--                        databases for migration from R2.3 to higher releases.
#--   1.1.5 - 2006/01/18 : Indicate in main log file error occurence for one
#--                        the children processes.
#--   1.1.6 - 2006/01/20 : Node prefix configurable.
#--                        AMS can't do node name to IP address convertion
#--                        (ip address/node name not stored in /etc/hosts)
#--   1.1.7 - 2006/02/02 : Include migration from R2.1
#--   1.1.8 - 2006/02/27 : Include migration from R2.0
#--   1.1.9 - 2006/03/13 : use tftp for bacup of NT database on releases prior
#                          to R2.2
#--   1.2.0 - 2006/03/16 : Capability model
#--   1.2.1 - 2006/05/19 : Verify environment
#--   1.2.3 - 2006/06/08 : ECNT-C NT SW file has other prefix the ENCT-A
#--   1.2.4 - 2006/06/13 : - When migrating to R3.1 again the IGMPXmlfile.txt
#--                          needs to be generated.  The 'bwreduce' option
#--                          will be applied by the SHUB OFLMT.
#--                        - Included v1.22 of SHUB OFLMT
#--                        - Introduction of R2.4.20 having a different DB
#--                          version in order to allow a migration to R3.1.
#--                        - Make sure we don't call migration for SHUB when
#--                          from and to release are equal.
#--                        - Perform DB download using tftp for releases < R2.3
#--                        - OSWP activation using SNMP i.s.o. CLI
#--                        - Check on DB version (expected vs read).
#--   1.2.4.1 - 2006/07/25 : ANTms06883 - remove CLI logfile frmo /tmp on exit
#--                          of emergency exit
#--                          Upgrade of R2.2: wait 30 seconds (i.s.o. after
#--                          tftp of SHUB SW to /pub/Sw/Download
#--   1.2.4.2 - 2006/08/21 : ANTms08417 - do not check on SHUB SW size for releases
#--                          after R2.2
#--   1.2.5.1 - 2006/08/23 : ANTms10868 - create log file per NE in stead of
#--                          appending all in the child logfile.
#--                          ANTms10885 - perform some retries when activating
#--                          new OSWP via SNMP: craft can have taken transaction.
#--             2006/09/25 : Set to type-based CLI positioning.
#--             2006/09/27 : ANTms15715: take transaction before backing up
#--                          NT database.
#--             2006/09/28 : ANTms09983 - Add option to continue in case of some
#--                          fatal errors rather then exiting the child: --continue
#--             2006/09/29 : Safety check on database size after upload (should be
#--                          OK after fix for ANTms15715 but it does not harm to
#--                          check this anyway.
#--             2006/09/29 : Include program warnings in log files.
#--   1.4.1 -   2006/10/10 : ANTms17446: Copy IGMPXmlfile for correct location.
#--   1.4.2   - 2006/10/09 : ANTms17315: info about migration or upgrade
#--                          issues.
#--                          ANTms17319: allow migration of ECNT-A to ECNT-C for
#--                          SHUB OFLMT
#--             2006/10/11 : ANTmsxxxxx: - Allow script to end after DB download
#--                                      - Allow script to start from migrated DB
#--                                        has been downloaded, now only active OSWP.
#--   1.4.3   - 2006/10/18 : - Timestamp logging
#--                          - AMS/AWS integration
#--   1.4.3.4 - 2006/11/13 : ANTms22321 - transaction timeout unit different between
#--                          pre- and post-R2.4.
#--   1.4.3.5 - 2006/11/15 : Prepare for R3.2
#--   1.4.3.6 - 2006/11/20 : ANTms23105: due to SW bug NT takes more then 15 seconds
#--                          to reboot meaning that the WaitForRestart does not work
#--                          very well: introduce a configurable sleep.
#--                          ANTms23395: set transaction owner before doing SNMP set
#--   1.4.3.8 - 2006/11/23 : DB version check disabled --toecntc
#--   1.4.3.8 - 2006/11/24 : ANTms24159: a transaction survives an NT reset.  Release
#--                          it after the NT reachable again.
#--   1.5.0   - 2006/12/07 : ANTms26421 - Prepare for R3.3:
#--                          - SHUB OFLMT change: parameters via config file
#--                          - NT OFLMT change for Telefonica OAM config (--loopback)
#--   1.5.1   - 2006/12/19 : Call capability accesor to determine SHUB NT type for
#--                          offline migration.
#--   1.5.1.1 - 2007/01/08 : Skip to next ISAM when pre- or post-upgrade script fails.
#--   1.5.1.2 - 2007/01/22 : Error in migration from R2.1 (BackupShubDB: case swCmp < 0)
#--   1.5.1.3 - 2007/02/07 : ANTms33348, allow configurable dhcp option 82: confopt82
#--                                      allow configurable arberg insert: confarberg
#--   1.5.1.4 - 2007/02/15 : ANTms34745 bw=yes is the default when migrating SHUB DB
#--   1.6.0   - 2007/02/21 : ANTms26421 - SHUB support for 1.28
#--   1.6.1   - 2007/02/28 : ANTms35659 - Added code to handel SFP provisioning when
#--                          migrating from R 2.4 or 2.5 to R 3.3 only
#--                          [Implemented by Haq Furqan]
#--   1.6.2   - 2007/03/05 : ANTms37972 - SHUB OFLMT now on 1.30.
#--   1.6.3   - 2007/03/14 : ANTms38590 Also extract IGMPXmlfile for migration to R3.2
#--   1.6.3.1 - 2007/03/14 : ANTms38662 - Add environment check option.
#--   1.6.3.2 - 2007/03/20 : - ANTms40019 - Minor error in child exit monitoring
#--                          - Handle fatal returns when retrieving NT OFLMT version
#--   1.6.3.3 - 2007/03/29 : ANTms40631: NT OFLMT version 1.0.10 -> 1.0.12
#--                          - Always extract IGMP XML file when new release > 2.2
#--   1.6.3.4 - 2007/04/16 : ANTms42842: Include version 1.31 of SHUB OFLMT.
#--   1.6.3.5 - 2007/04/18 : ANTms43293: release transaction prior to re-establishing
#--                          CLI connection.  In case this would fail, transaction
#--                          will not be released (ISAM NT has a bug of not restarting
#--                          the transaction timeout after NT restart (transaction
#--                          survives a reset).
#--   1.6.3.6 - 2007/04/19 : ANTms43447.  Include support for SHUB OFLMT version 1.32
#--   1.6.3.7 - 2007/04/27 : ANTms44768: NT oflmt version is 10.1.3
#--   1.6.3.8 - 2007/05/09 : ANTms46024 - sfp reconf only for migrations from 2.4/2.5
#--                          to 3.3
#--                          ANTms46026 - SHUB OFLMT 1.33
#--   1.6.3.9 - 2007/05/16 : ANTms47166 - SHUB OFLMT 1.34
#--   1.6.3.10- 2007/05/29 ; ANTms48845 - check chmod result.
#--   1.6.3.11- 2007/06/04 : ANTms49515 - NT OFLMT version is now 11.1.0
#--   1.7.0   - 2007/05/02 : ANTms45206: prepare for R3.4/R3.5
#--                          ANTms49896: Enable forwarding of TTL=0 packet for a VRF.
#--             2007/05/04 : ANTms49911: Small correction in Ctrl-c handling (don't kill pid
#--                          that have already exited)
#--             2007/06/04 : Allow to start migration from an already uploaded DB
#--   1.7.1   - 2007/06/11 : ANTms50664 - NT OFLMT version 11.1.1 support
#--   1.7.2   - 2007/06/12 : ANTms50447: wait 3 minutes after SHUB DB save on redundant
#--                          systems before activating new OSWP.
#--   1.7.3   - 2007/06/14 : ANTms51245 - Fix flaw in transaction release at the end of the
#--                          operations.
#--             2007/06/18 : ANTms51632 - allow multiple users for same ISAM.
#--   1.7.4   - 2007/06/22 : ANTms52362 - Make sure node status result file is never
#--                          empty (Added AdvanceToNext).
#--   1.7.5   - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.7.6   - 2007/07/30 : ANTms55014 NT oflmt version is now 11.2.0
#--                          Latest SHUB OFLMT is 1.36
#--             2007/08/01 : ANTms56107 - SFP reconfiguration for R3.3 or higher.
#--             2007/08/01 : ANTms55243 - Logical error when using OFLDB resulted in an
#--                          empty database.
#--   1.7.7   - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.7.8   - 2007/08/07 : ANTms12345: NT OFLMT framework version upgraded to 11.2.1
#--   1.7.9   - 2007/08/10 : ANTms59351: 1.36 of SHUB OFLMT expects new parameter.
#--   1.7.10  - 2007/08/17 : ANTms58361: "Stray handler 18".
#--             2007/08/20 : ANTms60348: NT OFLMT FW version is now 11.2.4
#--   1.7.11  - 2007/08/22 : ANTms54982: no longer perform an SHUB DB save before uploading
#--                          the NT DB when the active release is R2.3 or higher.
#--             2007/08/22 : ANTms60869: Make NT community string configurable
#--             2007/08/25 : ANTms55192: Reset NT using T&D in case the NT is still
#--                          running after an OSWP activation.
#--   1.7.12  - 2007/09/03 : ANTms62454: When the node is up too soon, the workaround
#--                          is triggered but the Telnet session is closed.  We now
#--                          specifically check on this.
#--   1.7.13  - 2007/09/05 : ANTms62648: Workaround not OK for redundant systems.
#--   1.7.14  - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.7.15  - 2007/10/02 : ANTms66036: new NT OFLMT FW version (12.1.2)
#--   1.7.16  - 2007/10/11   ANTms66037: the correct board code for NANT-B is 12396 in stead
#--                          of 12361.
#--   1.7.17  - 2007/10/17 : ANTms68417: check the number of packages in the multicast sources
#--                          ANTms68358: The NANT-B board code originally used in PBMT was correct after all.
#--                          ANTms67685: NT OFLMT version is now 12.1.3
#--   1.7.18  - 2007/10/23 : ANTms45206 NANT-A to NANT-B convertion.  SHUB OFLMT param file
#--                          was missing 1 parameter.  Moved file creation to common part.
#--   1.7.19  - 2007/11/14 : ANTms72085: SHUB OFLMT is now 1.38
#--   1.7.20  - 2007/12/18 : ANTms77027.  NT OFLMT 3.4.01 converts 64 packages as member into
#--                                       1024 packages on the condition that the highest package
#--                                       member is 64.  As such the check in PBMT is adapted to
#--                                       accept this too.
#--   1.8.0   - 2007/09/12 : IVPS offline migration for R3.5
#--   1.8.1   - 2008/01/09 : ANTms78942.  OSWP activation accompanied by an NT reset makes the OSWP activation
#--                          'fail': only the NT is activated (with the previous load).
#--   1.8.2   - 2008/01/14 : ANTms80071.  Use param.cfg for the NT community string.
#--   1.8.3   - 2008/01/10 : ANTms63550. Added functionality to configure Access OperatorId during 
#--                                      offline migration. ACCESS_OPERATOR_ID key gets written to
#--                                      dbtool.cfg which is read by OFLMT during migration and 
#--                                      stored in the DB. This functionality is available in R36 OFLMT.
#--   1.8.4   - 2008/01/20 : ANTms80082: migration of R3.6V
#--   1.8.5   - 2008/02/07 : ANTms83783: Check whether both NT and SHUB are running the new SW
#--   1.8.6   - 2008/02/29 : Ignore unknown stdby NT when status is not operational.
#--                          ANTms86893, error in IP regular expression
#--   1.8.7   - 2008/03/13:  ANTms88054.  Remove warning when Waiting for restart times out (warning in print statement)
#--   1.8.8   - 2008/04/08 : ANTmt02100.  SHUB SW version retrieval resulted in migration failure while node was running
#--                                       the new release when the NT (and the system) had restarted.
#--   1.8.9   - 2008/04/29 : ANTmt04599.  NANT-A TYPE-B SW name changed between R3.2 and R3.3
#--                                       Implement logic for SW name retrieval in case of
#--                                       'reduced' SW
#--   1.8.10  - 2008/05/06 : ANTmt05829.  Check ip address immediately after parsing the OSWP file
#--                                       in ParseCommandLine.
#--   1.8.11  - 2008/05/13 : ANTms05084.  Issue in XVPS DB migration:
#--                                       - Remove XVPS directories possibly present from previous migrations
#--                                       - Assure correct settings of source and target directories when combining
#--                                         the migrated databases.
#--   1.8.12  - 2008/05/13 : ANTmt06468.  CDE directory still in XVPS DB container.
#--   1.8.13  - 2008/05/27 : ANTmt08912.  CDE filename to be included in backup is not CDE.tar
#--   1.8.14  - 2008/05/28 : ANTmt08500.  All DB migrated to R3.6 must include the CDE directory.
#--   1.8.15  - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.8.16  - 2008/06/24 : ANTmt13345.  Starting from R3.6 the CDE file is in a different location in the backup.
#--   1.8.17  - 2008/06/27 : ANTmt14021.  Include on CDE file presence depending on the source release.
#--   1.8.18  - 2008/07/03 : Typo in IS_ISAM_SRV.  Must be IS_ISAMV_SRV
#--   1.8.19  - 2008/07/22 : ANTmt17241.  No IgmpXml extraction when migrating from R3.1 or higher.
#--   1.8.20  - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--             2008/07/25 : Add check on file transfer protocol being used in the ISAM.
#--   1.8.21  - 2008/09/04 : ANTmt22054
#--   1.8.22  - 2008/09/24 : ANTmt24857 (and clones ANTmt24872, ANTmt24874)
#--   1.8.23  - 2008/10/02 : ANTmt23353.  Implement workaround for random initvalues in vlanStaTbl definition.
#--   1.8.24  - 2008/10/28 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.8.25  - 2009/02/04 : ANTmt40831.  Include the usage of DBCHECK option in case of vclPortTbl corruptions.  We use dbCheck.pl
#--                                       to deterine these errors.
#--                          Use '|' as separator when using ofldb option.  The automation team is using the ':' char in the path name.
#--   1.9.0   - 2008/10/06 : ANTmt44696. R3.7.10 and IPD ISAM family.
#--   1.9.1   - 2009/03/10 : ANTmt18506.
#--   1.9.2   - 2009/03/18 : ANTmt48688.
#--   1.9.3   - 2009/04/30 : ANTmt55717.  Introduced NE famile ISAM_BOTH to indicate NTs that can be part of both the ETSI and ANSI stream.
#--   1.9.4   - 2009/05/19 : ANTmt59034: Change $config{USESSH} to $config{USE_SSH}
#--   1.9.5   - 2009/05/25 : ANTmt59036.  Check security access settings and force usage of SSH if no telnet access for CLI possible.
#--   1.9.6   - 2009/05/26 : ANTmt60011.
#--   1.9.7   - 2009/06/02 : ANTmt60891.
#--   1.9.8   - 2009/06/02 : ANTmt61239 : Added 1540LS NE family.
#--   1.9.9   - 2009/06/09 : ANTmt62387.  When upgrading an ESTI ISAM call the ESTI upgrade and migrate plugs, not the 7302 plugs.
#--   1.9.10  - 2009/06/10 : ANTmt62662, allow public-key authentication with SSH
#--   1.9.11  - 2009/06/26 : ANTmt65332.  Introduction of the NTT ETSI family
#--   1.9.12  - 2009/07/13 : ANTmt65332
#--   1.9.13  - 2009/07/24 : ANTmt71238: ECNT-A/ECNT-C to ECNTE conversion 
#--   1.9.14  - 2009/09/24 : Suppress XVPS migration possible since it is not established at the moment of
#--                          displaying this info.
#--   1.9.15  - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.9.16  - 2009/11/04 : 
#--   1.9.17  - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session 
#--   1.9.18  - 2009/12/01 : 7354RU fixes
#--   3.0.00  - 2010/04/27 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01  - 2010/04/7  : Converged PBMT Changes
#--                        : x86 Changes
#--   3.0.02  - 2010/07/30 : ALU00441701  Support added for FORCE_OPERATOR_ID in param.cfg
#--                          which will be the default for --oprid 
#--   3.0.03  - 2010/09/29 : ALU00441703. Provide path to the OFLMT tools as command line parameters
#--   3.0.04  - 2010/10/19 : ALU00808688 : new LRM option --id (suppresses the --logfile option).  Must not be used in combination with --hosts
#--                          ALU00804222 - Update default names for tools
#--   3.0.05  - 2010/11/01 : ALU00825495 - Lanx restoration failure when migrating from 4002 to 4.2 
#--   3.0.06  - 2010/12/14 : ALU00954124 - Revert back to old naming conventions (with .exe extension)  for tools
#--   3.0.07  - 2011/01/07 : LRM updates for lib-perl path 
#--   3.0.08  - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.09  - 2011/04/01 : Changes for 7354RU
#--   3.0.10  - 2011/04/05 : ALU01184513
#--   3.0.11  - 2011/04/07 : ALU01031457 - Support for R4220 & R4230
#--   3.0.12  - 2011/05/20 : ALU01207252
#--   3.0.13  - 2011/05/25 : ALU01178887
#--   3.0.14  - 2011/06/07 : ALU01246291
#--                        : Changes for RLAB issue of not able to open descriptor file
#--   3.0.15  - 2011/07/25 : ALU01348790
#--   3.0.16  - 2011/09/19 : ALU01497319
#--   3.0.17  - 2011/09/24 : Support for IVPS solaris x86 tool 
#--   3.0.18  - 2011/10/11 : I386 parameters renamed since AMS only supports I386_NTOFLMT type formats
#--   3.0.19  - 2011/10/24 : logfile command line parameter can include a directory
#--   3.0.20  - 2012/01/05 : ALU01733408
#--   3.0.21  - 2012/03/26 : ALU01748774
#--   3.0.22  - 2012/05/14 : ALU01789702
#--   3.0.23  - 2012/05/30 : to correct the display of snmp option used in logfile
#--   3.0.24  - 2012/08/23 : ALU01797328
#--   3.0.25  - 2012/09/12 : ALU01819262
#--   3.0.26  - 2012/10/17 : ALU01761938
#--   3.0.27  - 2012/10/30 : ALU01836984 & ALU01836986
#--   3.0.28  - 2013/02/07 : enable usage of second password if first one fails for cli over ssh
#--   3.0.29  - 2013/02/08 : corrected issue related to picking of second password from config file
#--   3.0.30  - 2013/02/15 : undef second password in params when it is set as empty
#--   3.0.31  - 2013/04/25 : ALU01880147
#--   3.0.32  - 2013/05/21 : set linux oflmt path from configuration file
#--   3.0.33  - 2013/07/09 : ALU01893590
#--   3.0.34  - 2013/10/17 : ALU01913431
#--   3.0.35  - 2014/02/07 : ofmlt support on Linux in ANSI stream 
#--   3.0.36  - 2014/06/12 : OFLMT tool path in Linux updated properly 
#--   3.0.37  - 2014/07/06 : ALU01991130 Full support for ANSI VSRM 7367_SX
#--   3.0.38  - 2014/11/30 : ALU02028455
#--   3.0.39  - 2015/04/13 : ALU02073720
#--   3.0.40  - 2015/05/21 : ALU02078593
#--   3.0.41  - 2015/05/25 : ALU02060588
#--   3.0.42  - 2015/06/10 : ALU02083077
#--   3.0.43  - 2015/06/18 : ALU02025800
#--   3.0.44  - 2015/07/14 : ALU02031458
#--   3.0.45  - 2015/08/04 : ALU02104915
#--   3.0.46  - 2015/10/28 : ALU02139820
#--   3.0.47  - 2015/11/02 : ALU02142016
#--   3.0.48  - 2015/12/14 : ALU02156544
#--   3.0.49  - 2015/12/30 : ALU02160793
#--   3.0.50  - 2016/03/16 : ALU02190558
#--   3.0.51  - 2016/07/15 : ALU02237966
#--   3.0.52  - 2016/08/08 : ALU02251693
#--   3.0.53  - 2016/08/17 : ALU02253040, ALU02257963
#--   3.0.54  - 2016/09/12 : ALU02271083
#--   3.0.55  - 2016/09/28 : ALU02278259
#--   3.0.56  - 2016/11/21 : ALU02285181
#--   3.0.57  - 2017/09/25 : Changes in post migration to support RemoteUplinkScript
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "migrate.pl - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "migrate.pl - 3.1.0 -- 2018/06/20";
#==============================================================================

use strict;
require 5.6.0;
use warnings 'all';

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2219-DFZZA">User::pwent</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2092-DFZZA">DM</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2201-FDZZA">MigrateIsam7302.pm</li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2202-FDZZA">MigrateIsam7302HiCap.pm</li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2203-FDZZA">MigrateIsam7354.pm</li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2200-FDZZA">PbmtConfig.pm</li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use File::Spec;
use Config;

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
                #unshift(@INC, "$scriptDir/lib-perl/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 

use Net::SNMP qw(:asn1);
use Net::TFTP;
use User::pwent;

use Logger qw(:ALL);
use version qw(:ALL);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :MIGR);
require FamilyCommon;
require MigrateIsam7302;
require MigrateIsam7302Ansi;
require MigrateIsam7302Etsi;
require MigrateIsam7302EtsiNtt;
use MigrateIsam7354;
use MigrateIsam7367; 
use MigrateIsam7363; 
use MigrateIsam7367_SX; 
use MigrateIsam7302HiCap;
require MigrateIsam1540;
use UpgradeIsam7354;
use UpgradeIsam1540;
use UpgradeIsamIHUB;
use UpgradeIsamSHUB;
use IsamCli;
use Crypt::Lite;
use Utils;
use Octopus;
use Capability;
use PbmtConfig;
use SNMPComm;
use DM;

#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$$);
sub DetermineTypeAndDbVersion($);
sub MigrateIsams($$$);
sub AdjustToolVars($);
sub HandleSigPipe();
sub CleanUp($;$);
#==============================================================================
#== Constants
#==============================================================================

my $NTINFILENAME = "dm_complete.tar";           # input name of the complete NT database
my $NTSMALLFILE = "dm.tar";                     # input name of the "small" NT database (only used with ofldb).

my %toolData = ('HOSTSFILE' => "/etc/hosts",
		'MAXWAITTIME' => 1800, #-- 30 Minutes
		'MAXSESSIONS' => 3,
		'SHUBSWPATH' => "",
		'SHUBSWNAME' => "",
		'TFTPDIR' => "/global/aws_var/aws/equipment",
		'TFTPWORKDIR' => "tmp",
		'USERNAME' => "",
		'PASSWORD' => "",
		'PASSWORD2' => "",
		'SECUREDBSAVE' => 30,
		'TFTPTIMEOUT' => 5,
		'TELNETTIMEOUT' => 10,
		'EMERG_ALRM_POLL_TIME' => 120,
		'ACTPINGWAIT' => 0,
		'NTDBTOOLDIR' => "tools/NT",
		'NTDBTOOL' => "tools/NT/MJSH.exe",
		'NTDBTOOL_I386' => "tools/NT/MJSH_i386",     # oflmt tool for Solaris x86 platform
		'NTDBTOOL_LINUX' => "tools/NT/MJSH_l.exe",       # oflmt tool for Linux platform
		'ANSI_NTDBTOOL_LINUX' => "tools/NT_ANSI/MJSH_l.exe",       # ANSI oflmt tool for Linux platform
		'NTDBCFG' => "tools/NT/dbtool.cfg",
		'NTDBCFGMOD' => "dbtoolMOD.cfg",
		'XVPSDBCFGMOD' => "dbtoolXVPSMOD.cfg",
		'LANXDBTOOLDIR' => "tools/Lanx",
		'SHUBDBCFG' => "tools/Lanx/migration.dat",
		'LANXDBTOOL' => "Migrationtool.pl",
		'LANXIPCKSUM' =>  "CalcIpCheckSum_BE",
		'IVPSDBTOOLDIR' => "tools/IVPS",
		'IVPSDBTOOL' => "tools/IVPS/NBTW.exe",
		'IVPSDBTOOL_I386' => "tools/IVPS/NBTW_i386",
		'IVPSDBTOOL_LINUX' => "tools/IVPS/NBTW_l.exe",
		'IVPSDBCFG' => "tools/IVPS/dbtool.cfg",
		'$IVPSINVENTORY' => "/opt/aws/isamv/voipim/script/getIVPSBoardInNE.pl",
		'CDEDIR' => "",
		'NEWCDEFILE' => "",
		'ACTCDEFILE' => "",
		'PINGWAIT' => 10,                             # seconds -- 1.0.3
		'NODEPREFIX' => "AS-",                          # -- 1.1.6
		'NT_COMMUNITY' => "public",
		'REMAP_HOTRESET' => "N",
		'NO_IP' => "0.0.0.0",
		'OCTOPUSSCRIPTS' =>   "tools/octopus",
		'OCTOPUSPATH' => "tools/octopus",
		'WARNINGFILE' => "PRCKwarnings.txt",
		'RETRIES' => 5,
		'FROMNEFAMILY' => undef,
		'TONEFAMILY' => undef,
		# FXRDATA is a pointer to hash being filled in at a later stage.  The hash contains the following:
		# 'PROTOCOL': set to the filetransfer protocol being used by the ISAM
		# 'SERVERS': the filetransfer server used by the ISAM.  Only set when the protocol is different from tftp.
		#            Is a hash consisting of a list of IP-addresses and usernames (only username per filetransfer server is possible).
		# 'HOMEDIR': the home directory for NT DB backup and restore.  Depending on the file transfer protocol this setting will be
		#            different
		# 'XVPS_HOMEDIR': the home directory fox xPS DB backups.  xVPS only support TFTP, so this setting is relative to the TFTPDIR setting
		'FXRDATA' =>        undef,
		'SSMOUT_VLAN' =>    4088,
		'ENABLE_IGMP_GROUPING' =>    1,
		'MAX_GIP_GAP' =>    0,
		'GPON_ONU_RANGE_MODE' =>    0,
		'IGMP_MC_RANGE_SETTING' =>    'UNU',
		'ACT_VLAN_LABEL' =>    undef,
		'NEW_VLAN_LABEL' =>    undef,
                'FORCE_OPERATOR_ID' => undef);

 if ($Config{archname} =~ /i86pc/) {
    $toolData{LANXIPCKSUM} = "CalcIpCheckSum_LE";
 } elsif ($Config{archname} =~ /linux/) {
    $toolData{LANXIPCKSUM} = "CalcIpCheckSum_LE_linux";
 }

my $STATUSFILE =       "result.txt";
my $ID_FILENAME =      "MIGR";
my $MAINSTREAM =       99;
my $sessionCount = 0;
my $logExecOrNode = 0;

=head1 USAGE

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] migrate.pl --isam &lt;ip&gt; | --hosts &lt;file&gt; --logfile &lt;file&gt; --swp &lt;path-to-oswp-descriptor&gt; \ <br>
                      [--sessions &lt;nr&gt;] [--cliusername &lt;user&gt;] [--clipassword &lt;password&gt;] \ <br>
                      [--oflonly] [--singleRestart] [--secureDbSave &lt;seconds&gt;] \ <br>
                      [--force] [--bwreduce] [--seplog | --noseplog] [--continue] [--toecntc legacy | native] [--toecnte legacy | native] \ <br>
                      [--noact] [--actonly] [--confopt82] [--confarberg] [--intlnwvlan] [--ssmoutvlan] [--confmcastgrp] [--confgapsize] [--v] [--debug] \ <br>
                      [--cap] [--tonantb] [--igmpmcrange] [--gpononurngmd] [--ttl0fwd vrf-id] [--ofldb [NT-type|source_release_name|]db_path/name] \ <br>
                      [--multi] [--noname] [--comm <SNMP community string>] [--haip &lt;ip&gt;] [--oprid] [--remap] [--noping] [--v2]\ <br>
                      [--v3Auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;] [-v3priv &lt;privProtocol&gt;:&lt;password&gt;]</tt>

=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>swp</td><td>path to oswp descriptor file (relative to TFTP root directory)</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>cliusername</td><td>ISAM CLI username</td></tr>
  <tr><td>clipassword</td><td>ISAM CLI password</td></tr>
  <tr><td>oflonly</td><td>Stop after the offline migration of the databases</td></tr>
  <tr><td>singleRestart</td>
      <td>No restart from NT SAN disk will be attempted, only SHUB restart from own flash.  Only for releases &lt; R2.2</td>
  </tr>
  <tr><td>secureDbSave</td><td>number of seconds (default 30) to wait after SHUB database save - only for releases &lt; R2.3</td></tr>
  <tr><td>force</td><td>skip migration path checks, migrate anyhow</td></tr>
  <tr><td>bwreduce</td><td>Reduce sustained bitrate to 85% (only when migrating to R3.1)</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>;
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>continue</td><td>continue migrating other ISAMs in stead of exiting the current child process</td></tr>
  <tr><td valign="top">toecntc</td>
      <td>Migration of ECNT-A to ECNT-C (will halt migration process after migrating databases):
        <ul>
          <li>legacy : System MAC address assigned following ECNT-A rules (MAC address of SHUB port 17)</li>
          <li>native : System MAC address assigned following ECNT-C rules</li>
        </ul>
      </td>
  </tr>
  <tr><td>tonantb</td><td>Migration of NANT-A to NANT-B (will halt migration process after migrating databases)</td></tr>
  <tr><td>noact</td><td>download migrated database but do not activate new OSWP</td></tr>
  <tr><td>actonly</td><td>activate OSWP using previously downloaded database (no new DB migration takes place)</td></tr>
  <tr><td>confopt82</td><td>configure DHCP option 82 different from default values (configurable via param.cfg)</td></tr>
  <tr><td>confarberg</td><td>configure Arberg tag different from default values (configurable via param.cfg)</td></tr>
  <tr><td>check</td><td>only check migration environment (tool version numbers) and exit</td></tr>
  <tr><td>ttl0fwd</td><td>ID of VRF for which forwarding of packets with TTL=0 has to be done (from R3.5 onwards)</td></tr>
  <tr><td valign="top">ofldb</td>
      <td>NT type, name of source release and path/name of database previously uploaded separated by '|'
        <ul>
          <li>NT type and source release will be determined from DB if only a source DB is provided,</li>
          <li>All on-line actions prior to DB upload will be skipped, except in case the migrated DB is to be downloaded</li>
        </ul>
        ONLY dm_complete.tar allowed.
      </td>
  </tr>
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td></tr>
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>cap</td><td>Print tool capabilities</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>haip</td><td>High available system IP address associated to the physical interface to the DSLAMs</td></tr>
  <tr><td>oprid</td><td>Configure access operatorId assigned by Nokia (configurable via param.cfg)</td></tr>
  <tr><td>remap</td><td>Disable the NT hot reset (in case of no firewall issues because of opening up all UDP ports)</td></tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed.</td></tr>
  <tr><td valign="top">v3auth</td><td>username, password and authentication protocol when using SNMPv3.
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                             <li>All fields are mandatory when option is used.</li>
                         </ul>
                         e.g. --v3auth user:password:: will result in DM5 as protocol.</td></tr>
  <tr><td valign="top">v3priv</td><td>optional setting when using SNMPv3 privacy protocol providing protocol and password.
                         <ul><li>Privacy protocol is DES, 3DES or AES, DES is default protocol.</li>
                         </ul></td></tr>
</table>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

#------------------------------------------------------------------------------
#-- Display the usage
#------------------------------------------------------------------------------
sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> | -id <id> --swp <path-to-oswp-descriptor> \\ \n";
  print STDERR "       [--sessions <nr>] [--cliusername <user>] [--clipassword <password>] [--clipassword2 <password>] \\ \n";
  print STDERR "       [--oflonly] [--intlnwvlan] [--ssmoutvlan] [--singleRestart] [--secureDbSave <seconds>] \\ \n";
  print STDERR "       [--force] [--bwreduce] [--seplog | --noseplog] [--continue] [--toecntc legacy | native] [--igmpmcrange] [--gpononurngmd]\\ \n";
  print STDERR "       [--toecnte legacy | native] [--noact] [--actonly] [--confopt82] [--confarberg] [--confmcastgrp] [--confgapsize] [--v] [--debug] \\ \n";
  print STDERR "       [--ntdbtool <path of the NT DB Offline Migration Tool>] \\ \n";
  print STDERR "       [--lanxtool <path of the Lanx Migration Tool>] \\ \n";
  print STDERR "       [--ivpstool <path of the IVPS Migration Tool>] \\ \n";
  print STDERR "       [--cap] [--tonantb] [--ttl0fwd vrf-id] [--ofldb [NT-type|source_release_name|]db_path/name] \\ \n";
  print STDERR "       [--multi] [--noname] [--comm <SNMP community string>] [--v2] [--haip <ip>] [--oprid] [--remap] [--noping] \\ \n";
  print STDERR "       [--v3auth <user>:<password>:<authProtocol>] [-v3priv <privProtocol>:<password>] \n";
  print STDERR "       [--btype <password>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         id           : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                        The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         v            : print version and exit\n";
  print STDERR "         swp          : path to oswp descriptor file \n";
  print STDERR "                        (relative to TFTP root - $toolData{TFTPDIR})\n";
  print STDERR "         sessions     : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         cliusername  : ISAM CLI username\n";
  print STDERR "         clipassword  : ISAM CLI password\n";
  print STDERR "         clipassword2 : Second ISAM CLI password (option to use a second password if first password fails).\n";
  print STDERR "                        The second password option will be used only when CLI over SSH is configured.\n";
  print STDERR "         oflonly      : Stop after the offline migration of the databases\n";
  print STDERR "         ntdbtool     : NT DB Offline migration tool path\n";
  print STDERR "         lanxtool     : Lanx Offline migration tool path\n";
  print STDERR "         ivpstool     : IVPS Offline migration tool path\n";
  print STDERR "         singleRestart: No restart from NT SAN disk will be attempted, only SHUB restart from own flash\n";
  print STDERR "         secureDbSave : number of seconds (default 30) to wait after SHUB database save - only for releases < R2.3\n";
  print STDERR "         force        : skip migration path checks, migrate anyhow\n";
  print STDERR "         bwreduce     : Reduce sustained bitrate to 85% (only when migrating to R3.1)\n";
  print STDERR "         seplog       : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noseplog     : keep all NE logging in a single child log file (default is a single logfile per NE)\n";
  print STDERR "         continue     : continue migrating other ISAMs in stead of exiting the current child process\n";
  print STDERR "         toecntc      : Migration of ECNT-A to ECNT-C (will halt migration process after migrating databases)\n";
  print STDERR "               legacy : System MAC address assigned following ECNT-A rules (MAC address of SHUB port 17)\n";
  print STDERR "               native : System MAC address assigned following ECNT-C rules\n";
  print STDERR "         toecnte      : Migration of ECNT-A/ECNT-C to ECNT-E (will halt migration process after migrating databases)\n";
  print STDERR "               legacy : System MAC address assigned following ECNT-A/ECNT-C rules (MAC address of SHUB port 17)\n";
  print STDERR "               native : System MAC address assigned following ECNT-E rules\n";
  print STDERR "         tonantb      : Migration of NANT-A to NANT-B (will halt migration process after migrating databases)\n";
  print STDERR "         noact        : download migrated database but do not activate new OSWP\n";
  print STDERR "         actonly      : activate OSWP using previously downloaded database (no new DB migration takes place)\n";
  print STDERR "         confopt82    : configure DHCP option 82 different from default values (configurable via param.cfg)\n";
  print STDERR "         confarberg   : configure Arberg tag different from default values (configurable via param.cfg)\n";
  print STDERR "         confmcastgrp : allows grouping of IGMP multicast entries (default 1 - enabled) - used for ETSI stream\n";
  print STDERR "         confgapsize  : allowed gap size between multicast group ip-addresses while grouping (default 0 - consecutive entries)\n";
  print STDERR "         ttl0fwd      : ID of VRF for which forwarding of packets with TTL=0 has to be done\n";
  print STDERR "         intlnwvlan   : unused vlan value which can be used as internal network vlan for NANT-D migration\n"; 
  print STDERR "                        (intlnwvlan option can be given when 4089 is already in use)\n";
  print STDERR "         ssmoutvlan   : unused vlan value which can be used as ssm out vlan during migration\n"; 
  print STDERR "                        (ssmoutvlan option can be given when 4088 is already in use)\n";
  print STDERR "         igmpmcrange  : IGMP multicast tables setting (Selective[SEL]/Forced[FRC]/Unused[UNU]) used for ANSI stream\n"; 
  print STDERR "         gpononurngmd : allow GPON onu to range in presence of Alien onu (default 0 - disabled)\n";
  print STDERR "         ofldb        : NT type, name of source release and path/name of database previously uploaded separated by '|'\n";
  print STDERR "                        NT type and source release will be determined from DB if only a source DB is provided.\n";
  print STDERR "                        All on-line actions prior to DB upload will be skipped,\n";
  print STDERR "                        except in case the migrated DB is to be downloaded\n";
  print STDERR "                        ONLY dm_complete.tar allowed\n";
  print STDERR "         noname       : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         multi        : include Unix username in NE logging directory path\n";
  print STDERR "         cap          : Print capabilities\n";
  print STDERR "         haip         : High available systems' IP address associated to the physical interface to the DSLAMs\n";
  print STDERR "         oprid        : Configure access operatorId assigned by Nokia (configurable via param.cfg)\n";
  print STDERR "         remap        : Disable the NT hot reset (in case of no firewall issues because of opening up all UDP ports)\n";
  print STDERR "         noping       : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "       SNMP settings:\n";
  print STDERR "         comm         : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is \"public\"\n";
  print STDERR "         v2           : Use SNMPv2c\n";
  print STDERR "         v3auth       : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                        The password should be at least 8 characters long.\n";
  print STDERR "                        Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                        All fields are mandatory when option is used.\n";
  print STDERR "                        e.g. --v3auth user:password:: will result in DM5 as protocol.\n";
  print STDERR "         v3priv       : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                        Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  print STDERR "         btype    : Build type to choose XDescriptor; Possible values are SNMP, YANG\n";
  exit(CMD_INCOMPL_CODE);
}


=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $noaction, $swp, $sessions, $debug,
      $hosts, $logFile, $cliUserName, $cliPassword, $cliPassword2, $rlabOpt,
      $secureDbSave, $oflOnly, $singleRestart, $forceMigUpg, $version, $bwReduce, $sepLog, $noSepLog, $continue, $toEcntC, $toEcntE,
      $noAct, $actOnly, $confOpt82, $confArberg, $capability, $toNantB, $ttl0VrfId, $oflDb, $multi, $wipe, $noName, $communityString,
      $haIp, $accessOperatorId, $btype, $myTest, $remap, $noPing, $vlanStaticTbl, $v3Auth, $v3Priv, $useSsh, $snmpV2, $ntdbTool, $lanxTool, $ivpsTool, $id, $intlNwVlan, $ssmOutVlan, $confMcastGrp, $confGapSize, $igmpMCRange, $gponOnuRngMd);

  my $regExp;
  my %params;
  my $dbVersion;
  my $oswp2Consider;
  my $dirName = getcwd();
  my $options = GetOptions('isam=s', \$isam,
			   'noaction', \$noaction,
			   'swp=s', \$swp,
			   'btype=s', \$btype,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'logfile=s', \$logFile,
			   'username|cliusername=s', \$cliUserName,
			   'password|clipassword=s', \$cliPassword,
                           'clipassword2=s', \$cliPassword2,  
			   'secureDbSave=i', \$secureDbSave,
			   'oflonly', \$oflOnly,
                           'ntdbtool=s', \$ntdbTool,
                           'lanxtool=s', \$lanxTool,
                           'ivpstool=s', \$ivpsTool,
			   'singleRestart', \$singleRestart,
			   'force', \$forceMigUpg,
			   'v', \$version,
			   'v2', \$snmpV2,
			   'bwreduce', \$bwReduce,
			   'seplog', \$sepLog,
			   'noseplog', \$noSepLog,
			   'continue', \$continue,
			   'toecntc=s', \$toEcntC,
			   'toecnte=s', \$toEcntE,
			   'noact', \$noAct,
			   'actonly', \$actOnly,
			   'confopt82', \$confOpt82,
			   'confarberg', \$confArberg,
			   'cap', \$capability,
			   'tonantb', \$toNantB,
			   'ttl0fwd=i', \$ttl0VrfId,
			   'ofldb=s', \$oflDb,
			   'multi', \$multi,
			   'wipe', \$wipe,
			   'noname', \$noName,
			   'comm=s', \$communityString,
			   'haip=s', \$haIp,
		           'oprid', \$accessOperatorId,
			   'mytest', \$myTest,
                           'rlab', \$rlabOpt,                  # undocumented for RLAB
			   'remap', \$remap,
			   'noping', \$noPing,
			   'vlanStaTbl', \$vlanStaticTbl,
			   'v3auth=s', \$v3Auth,
			   'v3priv=s', \$v3Priv,
			   'ssh', \$useSsh,
                           'id=s', \$id,
                           'intlnwvlan=i', \$intlNwVlan,
                           'ssmoutvlan=i', \$ssmOutVlan,
                           'confmcastgrp=i', \$confMcastGrp,
			   'confgapsize=i', \$confGapSize,
			   'igmpmcrange=s', \$igmpMCRange,
			   'gpononurngmd=i', \$gponOnuRngMd);
                          
  #-- check options
  if (! $options) {
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (defined($capability)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    print STDERR sprintf("Highest SHUB OFLMT version supported:\t%s\n",
			 Capability::ShubOflmtVersionString(Capability::HighestShubOflmtVersion(Capability::IsamFamily())));
    print STDERR sprintf("Highest NT OFLMT version supported:\t%s\n",
			 Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion(Capability::IsamFamily())));
    print STDERR sprintf("Highest xVPS OFLMT version supported:\t%s\n",
			 Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion(Capability::IsamFamily())));
    Capability::PrintStreamInfo();
    exit(SUCCESS_CODE);
  }
  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }

  if (defined($intlNwVlan) && (($intlNwVlan < 1) || ($intlNwVlan > 4089))) {
    print STDERR "ERROR: --intlnwvlan value unacceptable. Please specify unused Network Vlan value between 1 and 4089\n";
    Usage();
  } 
  $params{INTL_NW_VLAN} = defined($intlNwVlan) ? $intlNwVlan  : undef;


  if(defined($ssmOutVlan)) {
     $params{SSM_OUT_VLAN} = $ssmOutVlan; 
  } else {
     $params{SSM_OUT_VLAN} = (defined($toolData{SSMOUT_VLAN}) && ($toolData{SSMOUT_VLAN} != 0)) ? $toolData{SSMOUT_VLAN} : undef; 
  }

  if ((defined($params{SSM_OUT_VLAN}) && (($params{SSM_OUT_VLAN} < 1) || ($params{SSM_OUT_VLAN} > 4088))) || (defined($intlNwVlan) && ($params{SSM_OUT_VLAN} == $intlNwVlan))) {
    print STDERR "ERROR: --ssmoutvlan value unacceptable. Please specify unused Vlan value between 1 and 4087 (both inclusive)\n";
    Usage();
  } 
  if( (defined $hosts) and (!defined $btype ) ) {
     print STDERR "ERROR: --btype value must be passed for --hosts\n";
     Usage();
  }

  if( defined($btype) ) {
      if( $btype ne "SNMP" and $btype ne "YANG" ) {
         print STDERR "ERROR: Invalid value passed for --bytpe; possible values are SNMP, YANG\n";
         Usage();
      } else {
         $params{BTYPE} = $btype;
      }
  } elsif( defined $hosts ) {
      $btype = "SNMP";
  }

##  $params{SSM_OUT_VLAN} = defined($ssmOutVlan) ? $ssmOutVlan : undef;

# if(!defined($confMcastGrp)) {
#   $confMcastGrp = 1;
# }
  
# if(!defined($confGapSize)) {
#   $confGapSize = 0;
# }
  
  if(defined($confMcastGrp)) {
     $params{ENABLE_IGMP_GROUPING} = $confMcastGrp;
  } else {
     $params{ENABLE_IGMP_GROUPING} = (defined($toolData{ENABLE_IGMP_GROUPING})) ? $toolData{ENABLE_IGMP_GROUPING} : 1; 
  }
  if(defined($params{ENABLE_IGMP_GROUPING}) && (($params{ENABLE_IGMP_GROUPING} < 0) || ($params{ENABLE_IGMP_GROUPING} > 1))) {
    print STDERR "Invalid value for confmcastgrp. Please specify either 0 for disabling grouping of mcast entries or 1 for enabling\n";
    Usage();
  }

  if(defined($confGapSize)) {
     $params{MAX_GIP_GAP} = $confGapSize;  
  } else {
     $params{MAX_GIP_GAP} = (defined($toolData{MAX_GIP_GAP})) ? $toolData{MAX_GIP_GAP} : 0; 
  }

  if(defined($igmpMCRange)) {
     $params{IGMP_MC_RANGE_SETTING} = $igmpMCRange;
  } else {
     $params{IGMP_MC_RANGE_SETTING} = (defined($toolData{IGMP_MC_RANGE_SETTING})) ? $toolData{IGMP_MC_RANGE_SETTING} : 'UNU'; 
  }
  if(defined($params{IGMP_MC_RANGE_SETTING}) && ((uc($params{IGMP_MC_RANGE_SETTING}) ne "UNU") && 
                                                 (uc($params{IGMP_MC_RANGE_SETTING}) ne "SEL") &&
                                                 (uc($params{IGMP_MC_RANGE_SETTING}) ne "FRC")))  
  {
    print STDERR "Invalid value for igmpmcrange. Please specify either UNU (Unused) or SEL (Selective) or FRC (Forced)\n";
    Usage();
  }

  
  if(defined($gponOnuRngMd)) {
     $params{GPON_ONU_RANGE_MODE} = $gponOnuRngMd;
  } else {
     $params{GPON_ONU_RANGE_MODE} = (defined($toolData{GPON_ONU_RANGE_MODE})) ? $toolData{GPON_ONU_RANGE_MODE} : 0; 
  }
  if(defined($params{GPON_ONU_RANGE_MODE}) && (($params{GPON_ONU_RANGE_MODE} < 0) || ($params{GPON_ONU_RANGE_MODE} > 1))) {
    print STDERR "Invalid value for gpononurngmd. Please specify either 0 for disabling gpon onu range mode or 1 for enabling\n";
    Usage();
  }

  $params{ACT_VLAN_LABEL} = (defined($toolData{ACT_VLAN_LABEL})) ? $toolData{ACT_VLAN_LABEL} : undef;  
  $params{NEW_VLAN_LABEL} = (defined($toolData{NEW_VLAN_LABEL})) ? $toolData{NEW_VLAN_LABEL} : undef;  
 

  if(defined($params{MAX_GIP_GAP}) && (($params{MAX_GIP_GAP} < 0) || ($params{MAX_GIP_GAP} > 64))) {
    print STDERR "Invalid value for confgapsize. Please specify the mcast group ip-address gap in the range 0 to 64\n";
    Usage();
  }
  
  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }
  if (!defined($swp)) {
    print STDERR "ERROR: no OSWP specified via --swp\n";
    Usage();
  }
  $oswp2Consider = $swp;
  $params{NOTFTP} = 0;
  # First try the file as specfied (could be an absolute path when using ftp e.g.
  if (! -r $oswp2Consider) {
    # Next try the SWP relative to the login home directory
    $oswp2Consider = "$ENV{HOME}/$swp";
    $oswp2Consider =~ s/\/\//\//g;
    if (! -r $oswp2Consider) {
      # Finally check as tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $oswp2Consider =~ s/\/\//\//g;
      if (! -r  $oswp2Consider) {
	print STDERR "ERROR: Accessing OSWP '$oswp2Consider' failed: $!\n";
	Usage();
      } else {
	$params{OSWP_HOME} = $toolData{TFTPDIR};
      }
    } else {
      $params{OSWP_HOME} = $ENV{HOME};
      $params{NOTFTP} = 1;
    }
  } else {
    if (index($toolData{TFTPDIR}, $dirName) != -1) {
      # PBMT installed relative to TFTP directory so still tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $params{OSWP_HOME} = $toolData{TFTPDIR};
    } else {
      $params{OSWP_HOME} = "";
      $params{NOTFTP} = 1;
    }
  }
  if(defined($rlabOpt) && ($params{OSWP_HOME} eq "")) {
        $params{OSWP_HOME} = $toolData{TFTPDIR}; 
        $params{NOTFTP} = 0;
  }

  $params{OSWP_FULL} = $oswp2Consider;
  $regExp = SWPSYNTAX;
  if ($swp =~ /$regExp/) {
    print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_INV_CHARS_CODE));
    exit OSWP_FILE_INV_CHARS_CODE;
  }
  if (defined($oflDb)) {
    if ($oflDb =~ /(\w+\-\w)\|(\w+\.\w+)\|(.+)/) {
      my $ntType = $1;
      my $srcOswp = $2;
      my $dbName = $3;
      if (! -r $dbName) {
	print STDERR "ERROR: accessing database '$dbName' failed: $!\n";
	Usage();
      }
      $regExp = REGEXP;
      if (!(basename($srcOswp) =~ /$regExp/)) {  # -- 1.1.7
	print STDERR sprintf("Software package name (%s) not conforming to regexp (%s)\n",
			     $srcOswp, REGEXP);
	exit(PARAM_CFG_NOT_OK_CODE);
      }
      if (! ($ntType =~ /\w\w\w\w\-\w/)) {
	print STDERR "Incorrect NT type syntax: $ntType\n";
	exit(PARAM_CFG_NOT_OK_CODE);
      }
      $params{OFL_NT_TYPE} = uc($ntType);
      $params{SRC_OSWP} = $srcOswp;
      $params{OFLDB} = $dbName;
    } elsif ($oflDb =~ /(.+)/) {
      if (! -r $oflDb) {
	print STDERR "ERROR: accessing '$oflDb' failed: $!\n";
	Usage();
      } else {
	if ((uc(basename($oflDb)) ne uc($NTINFILENAME)) && (uc(basename($oflDb)) ne uc($NTSMALLFILE))) {
	  print STDERR sprintf("ERROR: only $NTINFILENAME or $NTSMALLFILE allowed for migration, provided %s!\n", basename($oflDb));
	  $params{DMTAR} = 1 if (uc(basename($oflDb)) eq uc($NTSMALLFILE));
	  Usage();
	}
      }
      $params{OFL_NT_TYPE} = undef;
      $params{SRC_OSWP} = undef;
      $params{OFLDB} = $oflDb;
    } else {
      print STDERR "ERROR: please specify NT type, source release name and DB path/name separated by a colon.\n";
      print STDERR "       This info is needed to allow PBMT to perform the checks it normally carries out\n";
      Usage();
    }
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }

  #-- Support for OFLMT tool on x86-Solaris and Linux platforms

  if ($Config{archname} =~ /i86pc/) {
    $toolData{NTDBTOOL} = $toolData{NTDBTOOL_I386}; 
    $toolData{ANSI_NTDBTOOL} = $toolData{ANSI_NTDBTOOL_I386}; 
  } elsif ($Config{archname} =~ /linux/) {
    $toolData{NTDBTOOL} = $toolData{NTDBTOOL_LINUX};
    $toolData{ANSI_NTDBTOOL} = $toolData{ANSI_NTDBTOOL_LINUX}; 
  }
  if ($Config{archname} =~ /i86pc/) {
    $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOL_I386}; 
  } elsif ($Config{archname} =~ /linux/) {
    $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOL_LINUX};
  }

  if(defined($ntdbTool)){                       
      if (-d $ntdbTool) { 	

        my $relDir = File::Spec->abs2rel($ntdbTool, $scriptDir); 
        # ETSI tool path
        $toolData{NTDBTOOLDIR} = $ntdbTool; 
        $toolData{NTDBTOOL} = $relDir . "/" . $config->{'NTOFLMT'} if defined($config->{'NTOFLMT'});
        $toolData{NTDBCFG}  = $relDir . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'}); 

        # ANSI tool paths 
        $toolData{ANSI_NTDBTOOLDIR} = $ntdbTool; 
        $toolData{ANSI_NTDBTOOL} = $relDir . "/" . $config->{'ANSI_NTOFLMT'} if defined($config->{'ANSI_NTOFLMT'});
        $toolData{ANSI_NTDBCFG}  = $relDir . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});

        #print STDERR  "NTDBTool path given as $ntdbTool\n"; 

        if ($Config{archname} =~ /i86pc/) {
           $toolData{NTDBTOOL} = $relDir . "/" . $config->{'I386_NTOFLMT'} if defined($config->{'I386_NTOFLMT'});
           $toolData{ANSI_NTDBTOOL} = $relDir . "/" . $config->{'I386_ANSI_NTOFLMT'} if defined($config->{'I386_ANSI_NTOFLMT'});
        } elsif ($Config{archname} =~ /linux/) {
           $toolData{NTDBTOOL} = $relDir . "/" . $config->{'LINUX_NTOFLMT'} if defined($config->{'LINUX_NTOFLMT'});
           $toolData{ANSI_NTDBTOOL} = $relDir . "/" . $config->{'LINUX_ANSI_NTOFLMT'} if defined($config->{'LINUX_ANSI_NTOFLMT'});
        }

      } else { 
          print STDERR "ERROR: NT Offline Migration toolpath $ntdbTool does not exist\n";  
          Usage();
      }
  }

  if(defined($lanxTool)){                       
   if (-d $lanxTool) { 	

     my $relDir = File::Spec->abs2rel($lanxTool, $dirName); 
     $toolData{LANXDBTOOLDIR} = $relDir; 
     $toolData{SHUBDBCFG} = "$relDir/migration.dat"; 
     
     $toolData{ANSI_LANXDBTOOLDIR} = $relDir; 

     #print STDERR  "LanxTool path given as $lanxTool\n"; 
   } else { 
       print STDERR "ERROR: Lanx Offline Migration toolpath $lanxTool does not exist\n";  
       Usage();
   }
  }

  if(defined($ivpsTool)){                       
   if (-d $ivpsTool) { 	

     my $relDir = File::Spec->abs2rel($ivpsTool, $dirName); 
     $toolData{IVPSDBTOOLDIR} = $ivpsTool; 
     $toolData{IVPSDBTOOL} = $relDir . "/" . $config->{'IVPSOFLMT'} if defined($config->{'IVPSOFLMT'});
     $toolData{IVPSDBCFG}  = $relDir . "/" . $config->{'IVPSDBCFG'} if defined($config->{'IVPSDBCFG'});
     #print STDERR  "IVPS toolpath given as $ivpsTool\n"; 

     if ($Config{archname} =~ /i86pc/) {
        $toolData{IVPSDBTOOL} = $relDir . "/" . $config->{'I386_IVPSOFLMT'} if defined($config->{'I386_IVPSOFLMT'});
     } elsif ($Config{archname} =~ /linux/) {
        $toolData{IVPSDBTOOL} = $relDir . "/" . $config->{'LINUX_IVPSOFLMT'} if defined($config->{'LINUX_IVPSOFLMT'});
     }

   } else { 
       print STDERR "ERROR: IVPS Offline Migration toolpath $ivpsTool does not exist\n";  
       Usage();
   }
  }


  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
	"($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }
  if (defined($cliUserName)) {
    $params{USERNAME} = $cliUserName;
  } else {
    $params{USERNAME} = $toolData{USERNAME};
  }
  if (defined($cliPassword)) {
    $params{PASSWORD} = $cliPassword;
  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }
  if (defined($cliPassword2)) {
    $params{PASSWORD2} = $cliPassword2;
  } else {
    $params{PASSWORD2} = ($toolData{PASSWORD2} ne "") ? $toolData{PASSWORD2} : undef ;
  }
  if (defined($secureDbSave)) {  #-- 1.1.3
    $params{SECUREDBSAVE} = $secureDbSave;
  } else {
    $params{SECUREDBSAVE} = $toolData{SECUREDBSAVE};
  }  
  
#ALU01524313-indhu-15/12/2011
  if (defined($communityString) && defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  if (defined($communityString) && defined($v3Priv)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }
#ALU01524313-indhu-15/12/2011
                            #-- 1.1.3
  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if (defined($isam));
  $params{OFLONLY} = defined($oflOnly) ? 1 : 0;
  $params{SINGLERESTART} = 0;
  $params{SINGLERESTART} = 1 if (defined($singleRestart));
  $params{NOACTION} = 1 if (defined($noaction));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if (defined($debug));
  LogSetDebug() if(defined $debug);
  $params{OSWP} = $swp if (defined($swp));
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
  } else {
    $params{LOGFILE} = $logFile;
    if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
      my $logdir = $1;
      if (! -d $logdir) {
        if (!mkdir( $logdir, 0777)) {
          print STDERR "Cannot create directory '$logdir': $!\n";
          exit CREATE_DIR_FAILED_CODE;
        }
      }
    }
  }
  my $tftpInfo = Utils::FindTftpInfo(undef, $oswp2Consider);
  # Statement below stored major release number in $1 and minor release number in $2.
  # Both set to 9 means mainstream load.
  if (!defined($tftpInfo->{PRIMIP}) || ($tftpInfo->{PRIMIP} eq "")) {
    print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_NO_IP_CODE));
    exit OSWP_FILE_NO_IP_CODE;
  }
  my $baseName = basename($tftpInfo->{NAME});
  $regExp = REGEXP;
  if (!($baseName =~ /$regExp/)) {  # -- 1.1.7
    print STDERR sprintf("Software package name (%s, read from %s) not conforming to regexp (%s).  Check param.cfg!!!\n",
			 $baseName, $tftpInfo->{NAME}, REGEXP);
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  $params{PRIMTFTPIP} = $tftpInfo->{PRIMIP};
  $params{SECTFTPIP} = $tftpInfo->{SECIP};
  $params{SWPNAME} = $tftpInfo->{NAME};
  $params{FORCEMIG} = $params{FORCEUPG} = defined($forceMigUpg) ? 1 : 0;
  
  $params{BWREDUCE} = defined($bwReduce) ? "no" : "yes";
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
  $params{CONTINUE} = defined($continue) ? 1 : 0;
  $params{TOECNTC} = defined($toEcntC) ? 1 : 0;
  $params{TOECNTE} = defined($toEcntE) ? 1 : 0;
  $params{TONANTB} = defined($toNantB) ? 1 : 0;
  $params{NOACT} = defined($noAct) ? 1 : 0;
  $params{ACTONLY} = defined($actOnly) ? 1 : 0;
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{WIPE} = defined($wipe) ? 1 : 0;
  $params{OFLONLY} = 1 if (($params{TOECNTC}) || $params{TONANTB} || $params{TOECNTE});
  if ($params{TOECNTC} && $params{TONANTB}) {
    print STDERR "ECNT-C and NANT-B conversion is mutually exclusice, select either 'toecntc' or 'tonantb' only\n";
    exit(PARAM_BOARD_CONV_ERR_CODE);
  }
  if ($params{TOECNTE} && $params{TONANTB}) {
    print STDERR "ECNT-E and NANT-B conversion is mutually exclusice, select either 'toecnte' or 'tonantb' only\n";
    exit(PARAM_BOARD_CONV_ERR_CODE);
  }
  if ($params{TOECNTE} && $params{TOECNTC}) {
    print STDERR "ECNT-E and ECNT-C conversion is mutually exclusice, select either 'toecnte' or 'toecntc' only\n";
    exit(PARAM_BOARD_CONV_ERR_CODE);
  }
  if ($params{TOECNTC} || $params{TOECNTE}) {
    if (defined($toEcntC)) {
      if ((uc($toEcntC) eq 'NATIVE') || (uc($toEcntC) eq 'LEGACY')) {
	$params{MACMODE} = uc($toEcntC) eq 'NATIVE' ? 'Native' : 'Legacy';
      } else {
	print STDERR "Invalid MAC-mode when migrating from ECNT-A to ECNT-C, only native or legacy allowed\n";
	exit(PARAM_INV_MACMODE_CODE);
      }
    }
    if (defined($toEcntE)) {
	if ((uc($toEcntE) eq 'NATIVE') || (uc($toEcntE) eq 'LEGACY')) {
	  $params{MACMODE} = uc($toEcntE) eq 'NATIVE' ? 'Native' : 'Legacy';
	} else {
	  print STDERR "Invalid MAC-mode when migrating from ECNT-A/ECNT-C to ECNT-E, only native or legacy allowed\n";
          exit(PARAM_INV_MACMODE_CODE);
	}
    } 
  }
  $params{MYTEST} = defined($myTest) ? 1 : 0;
  if (defined($accessOperatorId)) {
    $params{ACCESSOPRID} = 1;
  } else {
    $params{ACCESSOPRID} = uc($toolData{FORCE_OPERATOR_ID}) eq "Y" ? 1 : 0;
  }
  $params{CONFOPT82} = defined($confOpt82) ? 1 : 0;
  $params{CONFARBERG} = defined($confArberg) ? 1 : 0;
  $params{REMAPHOTRESET} = $toolData{REMAP_HOTRESET} eq "N" ? 0 : 1;
  if (defined($remap)) {
    $params{REMAPHOTRESET} = 1;
  }
  if ($params{OFLONLY} && $params{NOACT}) {
    print STDERR "'noact' option will be ignored, offline migration only has precedence\n";
    $params{NOACT} = 0;
  }
  if ($params{OFLONLY} && $params{ACTONLY}) {
    print STDERR "'actonly' option will be ignored, offline migration only has precedence\n";
    $params{ACTONLY} = 0;
  }
  if ($params{ACTONLY} && $params{NOACT}) {
    print STDERR "'actonly' option will be ignored, no OSWP activation has precedence\n";
    $params{ACTONLY} = 0;
  }
  $params{TTL0FWD} = $ttl0VrfId if (defined($ttl0VrfId));
  if (defined($oflDb)) {
    if ($params{ACTONLY}) {
      $params{ACTONLY} = 0;
      print STDERR "Activation only option disabled -- incompatible with the ofldb option\n";
      exit(PARAM_CFG_NOT_OK_CODE);
    }
  }
  $params{PHYSICAL_HOST_IP} = $config->{PHYSICAL_HOST_IP} if (defined($config->{PHYSICAL_HOST_IP}) &&
							    ($config->{PHYSICAL_HOST_IP} ne $toolData{NO_IP}));
  if (defined($haIp)) {
    if (ToolCommon::ValidIP($haIp)) {
      $params{PHYSICAL_HOST_IP} = $haIp;
    } else {
      print STDERR "'$haIp' is not a valid IP address\n";
      exit(PARAM_CFG_NOT_OK_CODE);
    }
  }
  $params{NONAME} = defined($noName) ? 1 : 0;
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : ""; #ALU02021411

  if (!defined($snmpV2)) { #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }
  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA512|SHA256|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
          if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
             $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
	} else {
	  print STDERR "SNMPv3 authentication protocol can only be SHA  or SHA256 or SHA512 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
    $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
    if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
       $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {  
     my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    $params{SNMP}{V3}{PRIV}{PASSWORD} = defined($toolData{SNMPV3_PRIV_PASSWORD}) ? $toolData{SNMPV3_PRIV_PASSWORD} : "";
  } #ALU01524313-indhu-15/12/2011
  }
  $params{VLANSTATBL} = defined($vlanStaticTbl) ? 1 : 0;
  if (($params{PASSWORD} eq "") || ($params{USERNAME} eq "")) {
    print STDERR "No username or passworddefined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }
  $params{NOPING} = 0;
  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'});
  $params{NOPING} = 1 if defined($noPing);

  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";

  $params{UPD_KNOWN_HOST_FILE} = 0;
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});

  $params{EMERG_CALL_FEATURE} = 0;
  $params{EMERG_CALL_FEATURE} = uc($config->{'EMERG_CALL_FEATURE'}) eq "Y" ? 1 : 0 if defined($config->{'EMERG_CALL_FEATURE'});


  my $oswpPath;
  $oswpPath = "$params{OSWP_HOME}/$params{SWPNAME}";
  if($params{NOTFTP} == 1) {
    if($params{OSWP_HOME} ne "") {
        $oswpPath = "$params{SWPNAME}";
        $params{OSWP_HOME} = "";
    }
  }
  $params{SWPPATH} = $oswpPath;
  $dbVersion = ToolCommon::GetDBVersion($oswpPath);
  if(!defined($dbVersion)) {
      print STDERR "DB Version is not defined in file $oswpPath \n";
      exit FILE_OPEN_FAIL_CODE;
  }
  $params{DBVER_TGTREL} = $dbVersion;

  return \%params;
}

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut

#------------------------------------------------------------------------------
#-- Print the command line parameters
#------------------------------------------------------------------------------

sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw(
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli IsamTL1 Octopus ReduceSWP
                       MigrateIsam7302 MigrateIsam7302Etsi MigrateIsam7302EtsiNtt MigrateIsam7302Ansi MigrateIsam7354 MigrateIsam7302HiCap 
                       MigrateIsam7367 MigrateIsam7363 MigrateIsam7367_SX MigrateIsam1540
                       UpgradeIsam1540 UpgradeIsam7354 UpgradeIsamIHUB UpgradeIsamSHUB
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

=head2 PrintOptionsSummary($$$$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$$) {
  my ($argLogFH, $logOnScreen, $config) = @_;
  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Interrupt on emergency call" , 
                $params->{EMERG_CALL_FEATURE} ? "Enabled" : "Disabled" ) ) if defined($params->{EMERG_CALL_FEATURE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Internal NW Vlan", $params->{INTL_NW_VLAN}) ) if defined($params->{INTL_NW_VLAN});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "SSM Out Vlan", $params->{SSM_OUT_VLAN}) ) if defined($params->{SSM_OUT_VLAN});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Telnet timeout in seconds", $toolData{TELNETTIMEOUT} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "XDescriptor build type" , $params->{BTYPE} ) ) if defined($params->{BTYPE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "OSWP Descriptor file" , $params->{OSWP} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- SWP Name" , $params->{SWPNAME} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- DB Version" , $params->{DBVER_TGTREL} ) ) if defined($params->{DBVER_TGTREL});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "DB & Release version mappping" , "$toolData{NTDBCFG}") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ONT-SW Descriptor file" , $params->{ONT} ) ) if defined($params->{ONT});
  if ($config->{SHUBSWPATH} ne "") {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "SHUB SW Location" , $config->{SHUBSWPATH}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "SHUB SW file" , $config->{SHUBSWNAME}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Transaction IP Address" , 
			       defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} :  $params->{PRIMTFTPIP}) );
            
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "FTP Details :-") );
  if (! $params->{NOTFTP}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "TFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Timeout in seconds" , $toolData{TFTPTIMEOUT}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- TFTP (HOME) Directory" , $toolData{TFTPDIR}) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "SFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Bandwidth reduction" , $params->{BWREDUCE} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Secure DB save" , $params->{SECUREDBSAVE} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "IGMP Mcast group :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Channel Entry" , 
                 $params->{ENABLE_IGMP_GROUPING} ? "Enabled" : "Disabled" ) ) if defined($params->{ENABLE_IGMP_GROUPING});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Allowed Gap size" , "$params->{MAX_GIP_GAP} (between address)" ) ) if defined($params->{MAX_GIP_GAP});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Range setting" , "$params->{IGMP_MC_RANGE_SETTING} (used in ANSI stream)" ) ) if defined($params->{IGMP_MC_RANGE_SETTING});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "GPON ONU Range mode" , $params->{GPON_ONU_RANGE_MODE} ) ) if defined($params->{GPON_ONU_RANGE_MODE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "CDE File" , "$toolData{TFTPDIR}/$toolData{CDEDIR}/$toolData{NEWCDEFILE}" ) ) if (defined($toolData{NEWCDEFILE}) && ($toolData{NEWCDEFILE} ne ""));
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Actions to be perform" , $params->{NOACTION} ? "No, Checks only" : "Yes, Action after checks") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Restart SHUB using SW TFTP" , $params->{MAX_GIP_GAP} ) ) if defined($params->{MAX_GIP_GAP});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Debug Hooks" , "Enabled" ) ) if ($params->{MYTEST});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "HOT RESET Remap" , 
                 $params->{REMAPHOTRESET} ? "Enabled" : "Disabled" ) );
  if ($params->{TOECNTC}) {
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Migration Mode" , "ECNT-A to ECNT-C" ) );
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "MAC-MODE" , $params->{MACMODE} ) ) if (defined($params->{MACMODE}));
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "HALT AFTER MIGRATING DB" , "YES" ) );
  } elsif ($params->{TONANTB}) {
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Migration Mode" , "NANT-A to NANT-B" ) );
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "MAC-MODE" , $params->{MACMODE} ) ) if (defined($params->{MACMODE}));
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "HALT AFTER MIGRATING DB" , "YES" ) );
  } elsif ($params->{TOECNTE}) {
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Migration Mode" , "ECNT-A/C to ECNT-C" ) );
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "MAC-MODE" , $params->{MACMODE} ) ) if (defined($params->{MACMODE}));
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "HALT AFTER MIGRATING DB" , "YES" ) );
  } elsif ( $params->{OFLONLY} ) {
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Migration Mode" , "Through OffLine" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Force Migration" , "Enabled" ) ) if ($params->{FORCEMIG});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Force Ugradation" , "Enabled" ) ) if ($params->{FORCEUPG});
  if( $params->{NOACT} ) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Operation Mode" , "No activation; Only download migrated DB" ) );
  }
  if( $params->{ACTONLY} ) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Operation Mode" , "Activation Only with previously download migrated DB" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Settings from param.cfg" , "Configure DCHP 82" ) ) if ($params->{CONFOPT82});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Settings from param.cfg" , "Configure Arbergtag" ) ) if ($params->{CONFARBERG});
  Logger::LogWish( $logMode, $argLogFH, "Access operatorId assigned by Nokia configured\n" ) if ($params->{ACCESSOPRID} == 1);
  Logger::LogWish( $logMode, $argLogFH, "TTL=0 packet forwarding for VRF $params->{TTL0FWD}\n" ) if (defined($params->{TTL0FWD})); 
  if (defined($params->{OFLDB})) {
    Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Source DB path" , $params->{OFLDB} ) );
    if ( defined($params->{OFL_NT_TYPE})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Source DB Upload", "No need to upload" ) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Source DB Used", "For $params->{OFL_NT_TYPE}" ) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Source SW", $params->{SRC_OSWP} ) );
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Source DB Used", "To Determine source release and NT-Type " ) );
    }
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Remap non-recov error", $config->{REMAP_HOT_RESET} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Enable second OSWP act", $config->{SECOND_RUN} ) );

  if( defined($config->{PRE_MIG_ACTION}) && $config->{PRE_MIG_ACTION} ne "" ) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Pre-Migration Activity", $config->{PRE_MIG_ACTION} ) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Pre-Migration Activity", "Not Configured" ) );
  }
  if( defined($config->{POST_MIG_ACTION}) && $config->{POST_MIG_ACTION} ne "" ) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Post-Migration Activity", $config->{POST_MIG_ACTION} ) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Post-Migration Activity", "Not Configured" ) );
  }
  if ($params->{CONTINUE}) {
    Logger::LogWish( $logMode, $argLogFH, "!!!WARNING!!!  USE AT OWN RISK!!!\n" );
    Logger::LogWish( $logMode, $argLogFH, "    Migration process will continue even in case an ISAM is lost in the process of migration" );
    Logger::LogWish( $logMode, $argLogFH, "    Hit Ctrl-C now if you do not want to continue\n!!!WARNING!!!\n" );
  }
}

=head2 DetermineTypeAndDbVersion($)

=begin html
Called when performing migration using a DB stored offline (-ofldb option).  NT type and DB version will be determined from the DB.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
</table>

=end html

=cut

sub DetermineTypeAndDbVersion($) {
  my ($dir) = @_;
  my $ok = 1;
  my $cmd = "cd $dir;tar xf $params->{OFLDB} 2>&1";
  my $version = "";
  my $line = "";
  my $conf;
  my $ntType;

  Logger::LogDebug( $nodeLogFH, "Extract OFLDB ($cmd)...\n" );
  my @cmd_op = `$cmd`;
  my $cmd_status = $?;
  foreach ( @cmd_op ) {
     Logger::LogDebug( $nodeLogFH, "$_" );
  }
  my $logMesg = sprintf( "Extract OFLDB(%s)... %s\n", $cmd, ( $cmd_status ) ? "Failed" : "Succeed" );
  Logger::LogNotify( $nodeLogFH, $logMesg );
  if( $cmd_status ) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($cmd_status);
    my $t = "Failed to untar database '$params->{OFLDB}' (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($execLogFH, $t, "  $t\n");
    $ok = 0;
  } else {
    if (-e "$dir/dm/EqBoard") {
      my ($header, $fields, $recs, $recCnt, $actualRecords) = DM::ReadFile("$dir/dm/EqBoard"); 
      foreach my $rec (@{$recs}) {
	      Logger::LogDebug($nodeLogFH, sprintf("Probing NT-type for boardCode 0x%04x\n", $rec->{'Type'}));
	      if (defined($ntType = Capability::DetermineNtType($rec->{'Type'}))) {
	        Logger::LogDebug($nodeLogFH, "NT $ntType found\n");
	        $params->{OFL_NT_TYPE} = $ntType;
	        last;
	      }
      }
      if (!defined($params->{OFL_NT_TYPE})) {
	FamilyCommon::SetAndLogError($execLogFH, "No NT type found in database '$params->{OFLDB}'");
	$ok = 0;
      }
      if (open ($conf, "<$dir/config")) {
	      while (defined($line = <$conf>) && ($version eq "")) {
	        chomp $line;
	        $line =~ s/\r//g;
	        Logger::LogDebug($nodeLogFH, "$line\n");
	        if ($line =~ /^version=(.*)/) {
	          $version = $1;
	          $params->{OFL_DB_VERS} = $version;
	          last;
	        }
	      }
	      close($conf);
	      Logger::LogDebug($nodeLogFH, "  $version\n");
      } else {
	      my $t = "Could not open '$dir/config for reading: $!";
	      FamilyCommon::SetAndLogError($execLogFH, $t, "  $t\n");
	      $ok = 0;
      }
    } elsif (-e "$dir/dbVersion.dat") {
      # Pizza box
      $params->{OFL_NT_TYPE} = Capability::Isam7354RUNtName();
      if (open ($conf, "<$dir/dbVersion.dat")) {
	     while (defined($line = <$conf>) && ($version eq "")) {
	       chomp $line;
	       $line =~ s/\r//g;
	       Logger::LogDebug($nodeLogFH, "$line\n");
	       if ($line =~ /^SW_DB_VERSION = (.*);/) {
	         $version = $1;
	         $params->{OFL_DB_VERS} = $version;
	         last;
	       }
	     }
	     close($conf);
	     Logger::LogNotify($nodeLogFH, "  $version\n");
       } else {
	     my $t = "Could not open '$dir/dbVersion.dat for reading: $!";
	     FamilyCommon::SetAndLogError($execLogFH, $t, "  $t\n");
	     $ok = 0;
       }
    } else {
      my $t = "Unknown database file, could not determine NT type";
      FamilyCommon::SetAndLogError($execLogFH, $t, "  $t\n");
      $ok = 0;
    }
  }
  return $ok;
}

sub DynamicOSWPVariant($$$) {
    my ($nodeLogFH, $config, $ntType) = @_;
    if( defined $params->{SWPPATH} and $params->{SWPPATH} ne "" ) {
        if( defined $config->{NT_SWP_MAPFILE} and $config->{NT_SWP_MAPFILE} ne "" ) {
           my $ntDescriptorMapFile = dirname($params->{SWPPATH}) . "/" . $config->{NT_SWP_MAPFILE};
           if( ! ( -f $ntDescriptorMapFile ) ) {
	            $params->{BTYPE} = "SNMP";
                Logger::LogNotify($nodeLogFH, "NT SWP Mapping file :$ntDescriptorMapFile: not exists\n");
                return 0;
           }
           Logger::LogTrace($nodeLogFH, "NT Descriptor mapping file($ntDescriptorMapFile) process started..\n");
           my( $status, $ntDescriptorFile) = Utils::FetchNTDescriptor($ntDescriptorMapFile, $ntType);
           Logger::LogTrace($nodeLogFH, "Details : $status, $ntType, $ntDescriptorFile\n");
           if( $status != 0 ) {
              FamilyCommon::IncrErrorCount("NTDescriptor Mapfile : $ntDescriptorFile \n",
                                     EC_NTTYPE_MAPFILE_FAILED,
                                     undef,
                                     1);
              return 1;
           }
           if( defined $params->{BTYPE} ) {
               if( ($params->{BTYPE} ne "SNMP") and (defined $config->{SNMP_YANG_ACTIVATE_TIMEOUT}) ) { 
 	               $toolData{MAXWAITTIME} = $config->{SNMP_YANG_ACTIVATE_TIMEOUT};
               }
               Logger::LogNotify($nodeLogFH, "XDescriptor build type:$params->{BTYPE}:, Max wait time:$toolData{MAXWAITTIME}:\n");
	       }
       }
   } 
   return 0;
}

=head2 MigrateIsams($$$)

=begin html
The main body of the script defining the complete migration process.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by this child process.  Every
      array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
</table>

=end html

=cut

#
# This is a rather long function because we prefered to clearly show the different steps of the process
#
sub MigrateIsams($$$) {
  my ( $isamsRef, $sessionCount, $config) = @_;
  my $isam;
  my $cli = undef;
  my $ret;
  my $dir;
  my $dirName = dirname($0);
  my $startTime;
  my $nodeLogFile;
  my $newLog = "$params->{LOGFILE}_$sessionCount";
  my $userName = $params->{MULTI} ? getlogin() : "";
  my $thisTftpDir;
  my $nameList = "";
  my $ntType = "";
  my $isamNode;
  my $mibVersion;
  my $fxrData;
  my $xvpsDir;
  my ( $snmpStatus, $snmpSession );

  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetToolData(\%toolData);
  Capability::BuildNtOflCap($toolData{NTDBCFG});
  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetConfigData($config);

  Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Started as '%d'\n", $sessionCount, FamilyCommon::GetOwnPid()));
  ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );

      ### Switch debugging (if any) to an opened filehandle...
      FamilyCommon::SetFailedNode($isamNode);
      FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and
      FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
      # Switch debugging (if any) to an opened filehandle...
      Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
      Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
      IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
      Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});   # send SNMP debug info to the child logfile first

      PrintOptionsSummary($nodeLogFH, undef, $config);
  }
  my $isamCount  = -1;
  foreach $isam ( @{$isamsRef} ) {
    $isamCount++;

    $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1];

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

    ##### Create ResultSpace directory #####
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );

    if ( defined($params->{ID})) {
        $logExecOrNode = 0;
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif ( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );

        PrintOptionsSummary($nodeLogFH, undef, $config);
    }


    FamilyCommon::SetFailedNode($isamNode);
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and 
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    # Switch debugging (if any) to an opened filehandle...
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
    IsamCli::ResetGlobals();
    MigrateIsam7363::ResetGlobals();
    MigrateIsam7302HiCap::ResetGlobals();
    MigrateIsam7302Etsi::ResetGlobals();
    MigrateIsam7302EtsiNtt::ResetGlobals();
    MigrateIsam1540::ResetGlobals();
    MigrateIsam7367::ResetGlobals();
    MigrateIsam7367_SX::ResetGlobals();
    $params->{USESSH} = 0; #for each isam the value is reset
    Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});

    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    my $nodeStatusLogFH = undef;
    if (! open($nodeStatusLogFH, ">$toolData{RESULTSPACE}/$isamNode/$STATUSFILE")) {
      my $str = "Session $sessionCount: cannot create $toolData{RESULTSPACE}/$isamNode/$STATUSFILE: $!\n";
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$STATUSFILE' for $isamNode: $!\n");
      Logger::LogError($execLogFH, $str, 1);
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE" ) );
      print $nodeStatusLogFH sprintf("%s '%s'\n", ErrorCodes::ExitString(START_ACTIVITY_CODE), "MIGR");
    }
    $nodeStatusLogFH->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLogFH);
    
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    if (defined($params->{OFLDB}) && ! defined($params->{OFL_NT_TYPE})) {
      if (! DetermineTypeAndDbVersion( dirname($params->{OFLDB}))) {
	FamilyCommon::IncrErrorCount(FamilyCommon::GetError(),
				     EC_INPUTDB_PROBLEM,
				     undef,
				     1);
	next;
      }else {
	$ntType = $params->{OFL_NT_TYPE};
	$mibVersion = $params->{OFL_DB_VERS};
    $params->{SOURCE_DBVERSION} = $params->{OFL_DB_VERS};
      }
      if (Capability::IsIsamFamily($ntType, $mibVersion)) {
	MigrateIsam7302::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsamEtsiFamily($ntType, $mibVersion)) {
	MigrateIsam7302Etsi::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsamEtsiExtendedFamily($ntType, $mibVersion)) {
	MigrateIsam7302Etsi::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsamAnsiFamily($ntType, $mibVersion)) {
	MigrateIsam7302Ansi::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsamBothFamily($ntType, $mibVersion)) {
	MigrateIsam7302::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsamHiCapEtsiFamily($ntType, $mibVersion)) {
	MigrateIsam7302HiCap::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsamHiCapAnsiFamily($ntType, $mibVersion)) {
	MigrateIsam7302HiCap::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::Is7354RUFamily($ntType, $mibVersion)) {
	MigrateIsam7354::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsam7367Family($ntType, $mibVersion)) {
	MigrateIsam7367::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsam7363Family($ntType, $mibVersion)) {
	MigrateIsam7363::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsam7367_SXFamily($ntType, $mibVersion)) {
	MigrateIsam7367_SX::DetermineSwVersion($nodeLogFH);
      } elsif (Capability::IsIsamLSFamily($ntType, $mibVersion)) {
 	MigrateIsam1540::DetermineSwVersion($nodeLogFH);
      } else {
	FamilyCommon::IncrErrorCount(sprintf("NT type '%s' not supported in NE family '%s' for this release\n",
					     $ntType, Capability::SelectFamily($ntType, $params->{SWPNAME})),
				     EC_NT_NOT_SUPPORTED,
				     undef,
				     1);
	next;
      }
    }

    #-- reachability test
    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }
    # Check whether an octopus session to this ISAM is open

    if ((defined($params->{OFLDB}) && (! $params->{OFLONLY})) || (! defined($params->{OFLDB}))) {
         # Check whether an octopus session to this ISAM is open
         next if( Octopus::OctopusActive($params->{DEBUG}, $isam, $config->{GREP}, $config->{PS}) );

         ( $snmpStatus, $snmpSession ) = SNMPComm::EstablishSNMP( $isam->[0] );
         if( $snmpStatus ) {
               FamilyCommon::IncrErrorCount( "FAILED : ".SNMPComm::GetError(),
                                             EC_SNMP_SESS_ERR, ": ".SNMPComm::GetError(), 1);
               next;
         }
         $snmpStatus = SNMPComm::MaximizeMsgSize();
         if( $snmpStatus ) {
              FamilyCommon::IncrErrorCount( "Failed to set maximum SNMP message size : ". SNMPComm::GetError(),
                                  EC_SNMP_MAX_SIZE_FAIL, " : ".SNMPComm::GetError(), undef);
              next;
         }
         $mibVersion = SNMPComm::GetValue( 'MIB VERSION', MIBVERSIONOID );
         if( !defined($mibVersion) ) {
              FamilyCommon::IncrErrorCount(sprintf("  FAILED to obtain MIB version: %s\n", SNMPComm::GetError() ),
                        EC_SNMP_NO_GET, sprintf(": %s", SNMPComm::GetError() ), 1);
              next;
         }
         ToolCommon::GetSourceDetails();
         if( !defined $params->{SOURCE_OSWPNAME} ) {
              FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED,ToolCommon::GetError(), undef );
              next;
         }
    
         Logger::LogTrace($nodeLogFH, "Prepare logging directories...");
         ($dir, $thisTftpDir, $fxrData) = FamilyCommon::PrepareLogging($snmpSession, $nodeLogFH, $userName, $isamNode);

         if (not defined($fxrData)) {
	          FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
	          			     EC_DETERMINE_LOG_FAILED,
	          			     sprintf(": %s", FamilyCommon::GetError()),
	          			     1);
	          next;
         } else {
	        Logger::LogTrace($nodeLogFH, "  OK\n", 1);
	        $toolData{FXRDATA} = $fxrData;
         }
    } else {
         $dir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isamNode";
         $thisTftpDir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isamNode";
         $toolData{FXRDATA}{HOMEDIR} = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isamNode";
         $toolData{FXRDATA}{XVPS_HOMEDIR} = "";
         $toolData{FXRDATA}->{PROTOCOL} = 1;
    }
	$xvpsDir = (!($toolData{FXRDATA}->{PROTOCOL} == TFTP_FXR)) ? "$toolData{FXRDATA}->{XVPS_HOMEDIR}" : "$toolData{TFTPDIR}/$toolData{FXRDATA}->{XVPS_HOMEDIR}";

    Logger::LogDebug($nodeLogFH, sprintf("Homedir: '%s', fxr->homedir: '%s'\n",
				    defined($toolData{FXRDATA}{HOMEDIR}) ? $toolData{FXRDATA}{HOMEDIR} : "undef",
				    defined($fxrData->{HOMEDIR}) ? $fxrData->{HOMEDIR} : "undef"));

    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g if defined($thisTftpDir);
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $newLog, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};
    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile= sprintf("migrate_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
        Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
        Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});

    }

    FamilyCommon::SetNeStorage($dir);
    Logger::LogInfo($nodeLogFH, "Uploaded and migrated NT database in  : '$dir'\n");
    if( defined($toolData{FXRDATA}->{XVPS_HOMEDIR}) ) {
       Logger::LogInfo($nodeLogFH, sprintf("Uploaded xVPS databases (if any) in   : '%s'\n", $xvpsDir) );
    }


    # Determine NE family: retrieve NT-A and NT-B type and status using SNMP.
    if (!defined($params->{OFLDB}) && ! defined($params->{OFL_NT_TYPE})) {
         $ntType = ToolCommon::DetermineNtType( );
         next if (not defined($ntType));
    } else {
         $ntType = $params->{OFL_NT_TYPE};
         $mibVersion = $params->{OFL_DB_VERS};
         $params->{SOURCE_DBVERSION} = $params->{OFL_DB_VERS};
    }
    $toolData{FROMNEFAMILY} = Capability::SelectFamily($ntType, $params->{DBVER_TGTREL});
    $toolData{TONEFAMILY} = Capability::SelectFamily($ntType, $params->{DBVER_TGTREL});


    if (! defined($toolData{FROMNEFAMILY})) { 
	FamilyCommon::IncrErrorCount(Capability::GetError(),     
				     EC_UNKNOWN_NE_FAM,   
				     sprintf("Unable to retrieve FROM NE Family : %s", Capability::GetError()),
				     1);
	next;
    } 
    if (! defined($toolData{TONEFAMILY})) { 
	FamilyCommon::IncrErrorCount(Capability::GetError(),     
				     EC_UNKNOWN_NE_FAM,  
				     sprintf("Unable to retrieve TO NE Family : %s", Capability::GetError()),
				     1);
	next;
    }
    Logger::LogDebug($nodeLogFH, sprintf("From NE Family: %s, To NE Family: %s\n",
					$toolData{FROMNEFAMILY}, $toolData{TONEFAMILY})) ;

    if ((defined($params->{OFLDB}) && (! $params->{OFLONLY})) ||
	(! defined($params->{OFLDB}))) {
      Logger::LogTrace($nodeLogFH, "Check access security settings...\n");
      my $accBits = ToolCommon::GetAccessettings($snmpSession, $params->{DEBUG});
      if (! defined($accBits)) {
	       FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
	       			     EC_GET_ACC_BITS_FAILED,
	       			     undef,
	       			     1);
	       next;
      } else {
	     Logger::LogTrace($nodeLogFH, "  OK\n");
      }

      if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
        $params->{USESSH} = 1;
        Logger::LogInfo($nodeLogFH, "CLI Communication through SSH\n");
      }
    }

    my $ntBoardFamily = Capability::LogNeFamily($ntType, $params->{DBVER_TGTREL});


    if ((defined($params->{OFLDB}) && (! $params->{OFLONLY})) ||
	(! defined($params->{OFLDB}))) {
      if (defined($config->{PRE_MIG_ACTION}) && $config->{PRE_MIG_ACTION} ne "") {
	      Logger::LogTrace($nodeLogFH, "Executing pre-migration actions...\n");
	      if (system("$config->{PRE_MIG_ACTION} -ip $isam->[0] -logdir $dir")) {
	        FamilyCommon::IncrErrorCount("non-successful exit code... skipping\n",
	      			       EC_PRE_UPGR_ERROR,
	      			       undef,
	      			       1);
	        next;
	      } else {
	        Logger::LogTrace($nodeLogFH, "  OK\n");
	      }
      }
    }

###DHINI
    my $oswpDynamicVariant = DynamicOSWPVariant($nodeLogFH, $config, $ntType );
    if( $oswpDynamicVariant ) {
         next;
    }

    my ($oflInfo, $envOk, $needsMigration);
    if (Capability::IsIsamFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7302::SetToolData();
      ($oflInfo, $envOk) = MigrateIsam7302::CheckEnvironment($execLogFH, $config, $dirName);
      MigrateIsam7302::PrintToolVersions($execLogFH, $oflInfo);
      $needsMigration = MigrateIsam7302::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsamBothFamily($ntType, $params->{DBVER_TGTREL})) {
      my $realFamily = Capability::FamilyFromOswp(basename($params->{OSWP}), 1);
      if (defined($realFamily)) {
	if ($realFamily eq Capability::IsamAnsiFamily()) {
	  MigrateIsam7302Ansi::SetToolData();
	  if ($toolData{FROMNEFAMILY} eq Capability::IsamBothFamily()) {
	    $toolData{FROMNEFAMILY} = Capability::IsamAnsiFamily();
	  }
	  if ($toolData{TONEFAMILY} eq Capability::IsamBothFamily()) {
	    $toolData{TONEFAMILY} = Capability::IsamAnsiFamily();
	  }
	  Logger::LogTrace($nodeLogFH, sprintf("From NE Family: %s, To NE Family: %s\n",
					      $toolData{FROMNEFAMILY}, $toolData{TONEFAMILY})) if ($params->{DEBUG});
	  ($oflInfo, $envOk) = MigrateIsam7302Ansi::CheckEnvironment($execLogFH, $config, $dirName);
	  MigrateIsam7302Ansi::PrintToolVersions($execLogFH, $oflInfo);
	  $needsMigration = MigrateIsam7302Ansi::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
	} elsif ($realFamily eq Capability::IsamEtsiFamily()) {
	  MigrateIsam7302Etsi::SetToolData();
	  if ($toolData{FROMNEFAMILY} eq Capability::IsamBothFamily()) {
	    $toolData{FROMNEFAMILY} = Capability::IsamEtsiFamily();
	  }
	  if ($toolData{TONEFAMILY} eq Capability::IsamBothFamily()) {
	    $toolData{TONEFAMILY} = Capability::IsamEtsiFamily();
	  }
	  Logger::LogTrace($nodeLogFH, sprintf("From NE Family: %s, To NE Family: %s\n",
					      $toolData{FROMNEFAMILY}, $toolData{TONEFAMILY})) if ($params->{DEBUG});
	  ($oflInfo, $envOk) = MigrateIsam7302Etsi::CheckEnvironment($execLogFH, $config, $dirName);
	  MigrateIsam7302Etsi::PrintToolVersions($execLogFH, $oflInfo);
	  $needsMigration = MigrateIsam7302Etsi::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
	} elsif ($realFamily eq Capability::IsamEtsiNttFamily()) {
	  MigrateIsam7302EtsiNtt::SetToolData();
	  if ($toolData{FROMNEFAMILY} eq Capability::IsamBothFamily()) {
	    $toolData{FROMNEFAMILY} = Capability::IsamEtsiNttFamily();
	  }
	  if ($toolData{TONEFAMILY} eq Capability::IsamBothFamily()) {
	    $toolData{TONEFAMILY} = Capability::IsamEtsiNttFamily();
	  }
	  Logger::LogDebug($nodeLogFH, sprintf("From NE Family: %s, To NE Family: %s\n",
					      $toolData{FROMNEFAMILY}, $toolData{TONEFAMILY}));
	  ($oflInfo, $envOk) = MigrateIsam7302EtsiNtt::CheckEnvironment($execLogFH, $config, $dirName);
	  MigrateIsam7302EtsiNtt::PrintToolVersions($execLogFH, $oflInfo);
	  $needsMigration = MigrateIsam7302EtsiNtt::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("NT type '%s' not supported in NE family '%s' for this release\n",
					       $ntType, Capability::SelectFamily($ntType, $params->{DBVER_TGTREL})),
				       EC_NT_NOT_SUPPORTED,
				       undef,
				       1);
	  next;
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  Unknown NE family: %s... skipping\n", Capability::GetError()),
				     EC_UNKNOWN_NE_FAM,
				     sprintf(": %s", Capability::GetError()),
				     1);
	next;
      }
    } elsif (Capability::IsIsamEtsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7302Etsi::SetToolData();
      ($oflInfo, $envOk) = MigrateIsam7302Etsi::CheckEnvironment($execLogFH, $config, $dirName);
      MigrateIsam7302Etsi::PrintToolVersions($execLogFH, $oflInfo);
      $needsMigration = MigrateIsam7302Etsi::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsamEtsiExtendedFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      my $realFamily = Capability::FamilyFromOswp(basename($params->{OSWP}), 1);
      if (defined($realFamily)) {
	if ($realFamily eq Capability::IsamEtsiFamily()) {
	  MigrateIsam7302Etsi::SetToolData();
	  if ($toolData{FROMNEFAMILY} eq Capability::IsamEtsiExtendedFamily()) {
	    $toolData{FROMNEFAMILY} = Capability::IsamEtsiFamily();
	  }
	  if ($toolData{TONEFAMILY} eq Capability::IsamEtsiExtendedFamily()) {
	    $toolData{TONEFAMILY} = Capability::IsamEtsiFamily();
	  }
	  Logger::LogDebug($nodeLogFH, sprintf("From NE Family: %s, To NE Family: %s\n",
					      $toolData{FROMNEFAMILY}, $toolData{TONEFAMILY}));
	  ($oflInfo, $envOk) = MigrateIsam7302Etsi::CheckEnvironment($execLogFH, $config, $dirName);
	  MigrateIsam7302Etsi::PrintToolVersions($execLogFH, $oflInfo);
	  $needsMigration = MigrateIsam7302Etsi::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
	} elsif ($realFamily eq Capability::IsamEtsiNttFamily()) {
	  MigrateIsam7302EtsiNtt::SetToolData();
	  if ($toolData{FROMNEFAMILY} eq Capability::IsamEtsiExtendedFamily()) {
	    $toolData{FROMNEFAMILY} = Capability::IsamEtsiNttFamily();
	  }
	  if ($toolData{TONEFAMILY} eq Capability::IsamEtsiExtendedFamily()) {
	    $toolData{TONEFAMILY} = Capability::IsamEtsiNttFamily();
	  }
	  Logger::LogDebug($nodeLogFH, sprintf("From NE Family: %s, To NE Family: %s\n",
					      $toolData{FROMNEFAMILY}, $toolData{TONEFAMILY}));
	  ($oflInfo, $envOk) = MigrateIsam7302EtsiNtt::CheckEnvironment($execLogFH, $config, $dirName);
	  MigrateIsam7302EtsiNtt::PrintToolVersions($execLogFH, $oflInfo);
	  $needsMigration = MigrateIsam7302EtsiNtt::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("NT type '%s' not supported in NE family '%s' for this release\n",
					       $ntType, Capability::SelectFamily($ntType, $params->{DBVER_TGTREL})),
				       EC_NT_NOT_SUPPORTED,
				       undef,
				       1);
	  next;
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("  Unknown NE family: %s... skipping\n", Capability::GetError()),
				     EC_UNKNOWN_NE_FAM,
				     sprintf(": %s", Capability::GetError()),
				     1);
	next;
      }
    } elsif (Capability::IsIsamAnsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7302Ansi::SetToolData();
      ($oflInfo, $envOk) = MigrateIsam7302Ansi::CheckEnvironment($execLogFH, $config, $dirName);
      MigrateIsam7302Ansi::PrintToolVersions($execLogFH, $oflInfo);
      $needsMigration = MigrateIsam7302Ansi::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsamLSFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam1540::SetToolData();
      ($oflInfo, $envOk) = MigrateIsam1540::CheckEnvironment($execLogFH, $config, $dirName);
      MigrateIsam1540::PrintToolVersions($execLogFH, $oflInfo);
      $needsMigration = MigrateIsam1540::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsamHiCapEtsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7302HiCap::SetToolData();
      ($oflInfo, $envOk) = MigrateIsam7302HiCap::CheckEnvironment($execLogFH, $config, $dirName);
      MigrateIsam7302HiCap::PrintToolVersions($execLogFH, $oflInfo);
      $needsMigration = MigrateIsam7302HiCap::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsamHiCapAnsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7302HiCap::SetToolData();
      ($oflInfo, $envOk) = MigrateIsam7302HiCap::CheckEnvironment($execLogFH, $config, $dirName);
      MigrateIsam7302HiCap::PrintToolVersions($execLogFH, $oflInfo);
      $needsMigration = MigrateIsam7302HiCap::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::Is7354RUFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7354::SetToolData();
      ($oflInfo, $envOk) = MigrateIsam7354::CheckEnvironment($execLogFH, $config, $dirName);
      MigrateIsam7354::PrintToolVersions($execLogFH, $oflInfo);
      $needsMigration = MigrateIsam7354::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsam7367Family($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7367::SetToolData();
      ### ($oflInfo, $envOk) = MigrateIsam7367::CheckEnvironment($execLogFH, $config, $dirName);
      ### MigrateIsam7367::PrintToolVersions($execLogFH, $oflInfo);
      $envOk = 1; 
      $needsMigration = MigrateIsam7367::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsam7363Family($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7363::SetToolData();
      ### ($oflInfo, $envOk) = MigrateIsam7367::CheckEnvironment($execLogFH, $config, $dirName);
      ### MigrateIsam7367::PrintToolVersions($execLogFH, $oflInfo);
      $envOk = 1; 
      $needsMigration = MigrateIsam7363::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } elsif (Capability::IsIsam7367_SXFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
      MigrateIsam7367_SX::SetToolData();
      ### ($oflInfo, $envOk) = MigrateIsam7367::CheckEnvironment($execLogFH, $config, $dirName);
      ### MigrateIsam7367::PrintToolVersions($execLogFH, $oflInfo);
      $envOk = 1; 
      $needsMigration = MigrateIsam7367_SX::MigrateOrUpgrade($execLogFH, $config, $dirName, $isam);
    } else {
      # Already captured above.
    }

    if (!defined($needsMigration)) {
      FamilyCommon::IncrErrorCount(sprintf(": %s\n", FamilyCommon::GetError()),
				   EC_DETERMINE_OPER_FAILED,
				   undef,
				   1);
      next;
    } elsif( ($needsMigration == 0) and (defined $params->{OFLDB}) ) {
		FamilyCommon::IncrErrorCount("Offline DB Migration is not required...\n",
		EC_OFLDB_NOTREQ,
		undef,
		1);
      next;
    }

    Logger::LogInfo($execLogFH, sprintf("Environment %s\n", $envOk ? "OK" : "NOT OK"));

    if(defined($params->{INTL_NW_VLAN})) {
       if(!Capability::IsIsamHiCapEtsiFamily($ntType, $params->{DBVER_TGTREL})) { 
	       Logger::LogInfo($nodeLogFH, "--intlnwvlan option not applicable for this Board... Will be Ignored \n" );
          $params->{INTL_NW_VLAN} = undef; 
       } else {
           if(!(($mibVersion =~ /^3FE.*_E4\.0.*/) || ($mibVersion =~ /^3FE.*_V4\.0.*/))) {  ## --intlnwvlan option invoke only applicable for R4.0 NANT-D 
              Logger::LogInfo($nodeLogFH, "--intlnwvlan option not applicable for this NANT-D Release... Will be ignored\n");
              $params->{INTL_NW_VLAN} = undef; 
           } else {
              Logger::LogInfo($nodeLogFH, "Internal Network Vlan value to be set : $params->{INTL_NW_VLAN} \n");
           }
       }
    }

    if(defined($params->{SSM_OUT_VLAN})) {
       if((Capability::IsIsamHiCapEtsiFamily($ntType, $params->{DBVER_TGTREL})) || (Capability::IsIsamEtsiFamily($ntType, $params->{DBVER_TGTREL})) || (Capability::IsIsamEtsiExtendedFamily($ntType, $params->{DBVER_TGTREL}))) { 
           my $regExp = REGEXP;  
           if (basename($params->{OSWP}) =~ /$regExp/) {
              my $releaseNum = Capability::ConvertRelease($1, $2);
              if (Capability::ReleaseCompare($releaseNum, Capability::ConvertRelease(4, 3), REGEXP, 0) < 0) {
		          Logger::LogInfo($nodeLogFH, "--ssmoutvlan option not applicable for this Release... Will be ignored\n");
		          $params->{SSM_OUT_VLAN} = undef; 
              } else {
                 Logger::LogInfo($nodeLogFH, "SSM Out Vlan value to be set : $params->{SSM_OUT_VLAN} \n");
              }
           }
       } else {
	      Logger::LogInfo($nodeLogFH, "--ssmoutvlan option not applicable for this Board... Will be ignored \n");
          $params->{SSM_OUT_VLAN} = undef; 
       }
    }

    if ($envOk) {
      $nameList = "" if (! $needsMigration);
      if (Capability::IsIsamFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	if ($needsMigration) {
	  $nameList = MigrateIsam7302::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
	  UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}
      } elsif (Capability::IsIsamEtsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	if ($needsMigration) {
	  $nameList = MigrateIsam7302Etsi::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
	  UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}
      } elsif (Capability::IsIsamEtsiExtendedFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	my $realFamily = Capability::FamilyFromOswp(basename($params->{OSWP}), 1);
	if (defined($realFamily)) {
	  if ($realFamily eq Capability::IsamEtsiFamily()) {
	    if ($needsMigration) {
	      $nameList = MigrateIsam7302Etsi::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	    } else {
	      UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	    }
	  } elsif ($realFamily eq Capability::IsamEtsiNttFamily()) {
	    if ($needsMigration) {
	      $nameList = MigrateIsam7302EtsiNtt::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	    } else {
	      UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	    }
	  } else {
	    # Captured above.
	  }
	} else {
	  # Should not occur - captured above
	  FamilyCommon::IncrErrorCount(sprintf("  Unknown NE family: %s... skipping\n", Capability::GetError()),
				       EC_UNKNOWN_NE_FAM,
				       sprintf(": %s", Capability::GetError()),
				       1);
	  next;
	}
      } elsif (Capability::IsIsamAnsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	if ($needsMigration) {
	  $nameList = MigrateIsam7302Ansi::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
	  UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}
      } elsif (Capability::IsIsamBothFamily($ntType, $params->{DBVER_TGTREL})) {
	my $realFamily = Capability::FamilyFromOswp(basename($params->{OSWP}), 1);
	if (defined($realFamily)) {
	  if ($realFamily eq Capability::IsamAnsiFamily()) {
	    if ($needsMigration) {
	      $nameList = MigrateIsam7302Ansi::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	    } else {
	      UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	    }
	  } elsif ($realFamily eq Capability::IsamEtsiFamily()) {
	    if ($needsMigration) {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}
		        $nameList = MigrateIsam7302Etsi::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);

	    } else {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;	
		}
		      UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	    }
	  } elsif ($realFamily eq Capability::IsamEtsiNttFamily()) {
	    if ($needsMigration) {
    		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}

	      $nameList = MigrateIsam7302EtsiNtt::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	    } else {
    		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}

	      UpgradeIsamSHUB::Upgrade($newLog, $thisTftpDir, $isam);
	    }
	  } else {
	    # Already captured above
	  }
	} else {
	  # Should not occur - captured above
	  FamilyCommon::IncrErrorCount(sprintf("  Unknown NE family: %s... skipping\n", Capability::GetError()),
				       EC_UNKNOWN_NE_FAM,
				       sprintf(": %s", Capability::GetError()),
				       1);
	  next;
	}
      } elsif (Capability::IsIsamLSFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	if ($needsMigration) {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}

 	  $nameList = MigrateIsam1540::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
 	} else {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
 	  UpgradeIsam1540::Upgrade($newLog, $thisTftpDir, $isam);
 	}
      } elsif (Capability::IsIsamHiCapEtsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	$nameList = "";
	if ($needsMigration) {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}
	  MigrateIsam7302HiCap::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  UpgradeIsamIHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}
      } elsif (Capability::IsIsamHiCapAnsiFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	$nameList = "";
	if ($needsMigration) {
	  MigrateIsam7302HiCap::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
	  UpgradeIsamIHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}
      } elsif (Capability::Is7354RUFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	$nameList = "";
	if ($needsMigration) {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  MigrateIsam7354::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  UpgradeIsam7354::Upgrade($newLog, $thisTftpDir, $isam, $ntType);
	}
      } elsif (Capability::IsIsam7367Family($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	$nameList = "";
	if ($needsMigration) {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  MigrateIsam7367::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  UpgradeIsamIHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}

     } elsif (Capability::IsIsam7363Family($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	$nameList = "";
	if ($needsMigration) {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  MigrateIsam7363::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  UpgradeIsamIHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}		
     } elsif (Capability::IsIsam7367_SXFamily($ntType, $params->{DBVER_TGTREL})) { # was $mibVersion
	$nameList = "";
	if ($needsMigration) {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  MigrateIsam7367_SX::Migrate($params->{SEPLOG} ? "$dir/$nodeLogFile" : $newLog, $thisTftpDir, $isam);
	} else {
		if($params->{TOECNTE}) {
			FamilyCommon::IncrErrorCount("ECNT-E conversion called for wrong NT type ... impossible... skipping\n",
			EC_NT_NOT_ECNT_A_OR_C,
			undef,
			1);
			next;
		}	
	  UpgradeIsamIHUB::Upgrade($newLog, $thisTftpDir, $isam);
	}		
      }

      FamilyCommon::CloseSnmpSession2Close();

      if ((! $params->{NOACT}) && (defined($nameList))) {
	      if (defined($config->{POST_MIG_ACTION}) && $config->{POST_MIG_ACTION} ne "") {
	          Logger::LogInfo($nodeLogFH, "Executing post-migration actions...\n");
              if (system("$config->{POST_MIG_ACTION} -ip $isam->[0] -logdir $dir")) {
	               FamilyCommon::IncrErrorCount("non-successful exit code... skipping\n",
		           			 EC_POST_UPGR_ERROR,
		           			 undef,
		           			 1);
	               next;
	          } else {
	            Logger::LogTrace($nodeLogFH, "  OK\n");
	          }
          }
      }
      my $endTime = time();
      ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
      Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );
      Logger::LogInfo($nodeLogFH, "  Dependant nodes related to this ISAMV: $nameList\n") if (defined($nameList) && ($nameList ne ""));
      Logger::LogTrace($nodeLogFH, "Done!\n");
      FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
      if ($params->{SEPLOG}) {
          FamilyCommon::CloseChildLogFile();
          LogCloseFile($nodeLogFH);
      }
      last if (defined($params->{OFLDB}));
    } else {
      # exits if not checking
    }
  }
  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }
  if (!defined($params->{ID})) {
      if (!$params->{SEPLOG}) {
          FamilyCommon::CloseChildLogFile();
          LogCloseFile($nodeLogFH);
      }
  }
  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file.

=end html

=cut

sub AdjustToolVars($) {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPDIR} =~ s/\/\///g; # remove any '//'
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'});                   # -- 1.1.6
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{MAXSESSIONS} = $config->{'MAXDLSESSION'} if defined($config->{'MAXDLSESSION'});
  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
										 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
										($config->{'DEFAULTCLIPWD'} ne ""));
  $toolData{PASSWORD2} = $crypt->decrypt($config->{'SECONDCLIPWD'}, CYPHER) if (defined($config->{'SECONDCLIPWD'}) &&
										($config->{'SECONDCLIPWD'} ne ""));
  $toolData{MAXWAITTIME} = $config->{'WAITTIME'} if defined($config->{'WAITTIME'});
  $toolData{NTDBTOOLDIR} = $config->{'NTDBTOOLDIR'} if defined($config->{'NTDBTOOLDIR'});
  $toolData{ANSI_NTDBTOOLDIR} = $config->{'ANSI_NTDBTOOLDIR'} if defined($config->{'ANSI_NTDBTOOLDIR'});
  $toolData{LS_NTDBTOOLDIR} = $config->{'LS_NTDBTOOLDIR'} if defined($config->{'LS_NTDBTOOLDIR'});
  $toolData{GPON_NTDBTOOLDIR} = $config->{'GPON_NTDBTOOLDIR'} if defined($config->{'GPON_NTDBTOOLDIR'});
  $toolData{LANXDBTOOLDIR} = $config->{'LANXDBTOOLDIR'} if defined($config->{'LANXDBTOOLDIR'});
  $toolData{ANSI_LANXDBTOOLDIR} = $config->{'ANSI_LANXDBTOOLDIR'} if defined($config->{'ANSI_LANXDBTOOLDIR'});
  $toolData{LS_LANXDBTOOLDIR} = $config->{'LS_LANXDBTOOLDIR'} if defined($config->{'LS_LANXDBTOOLDIR'});
  $toolData{GPON_LANXDBTOOLDIR} = $config->{'GPON_LANXDBTOOLDIR'} if defined($config->{'GPON_LANXDBTOOLDIR'});
  $toolData{NTDBTOOL} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTOFLMT'} if defined($config->{'NTOFLMT'});
  $toolData{NTDBTOOL_I386} = $toolData{NTDBTOOLDIR} . "/" . $config->{'I386_NTOFLMT'} if defined($config->{'I386_NTOFLMT'});
  $toolData{NTDBTOOL_LINUX} = $toolData{NTDBTOOLDIR} . "/" . $config->{'LINUX_NTOFLMT'} if defined($config->{'LINUX_NTOFLMT'});
  $toolData{ANSI_NTDBTOOL} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'ANSI_NTOFLMT'} if defined($config->{'ANSI_NTOFLMT'});
  $toolData{ANSI_NTDBTOOL_I386} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'I386_ANSI_NTOFLMT'} if defined($config->{'I386_ANSI_NTOFLMT'});
  $toolData{ANSI_NTDBTOOL_LINUX} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'LINUX_ANSI_NTOFLMT'} if defined($config->{'LINUX_ANSI_NTOFLMT'});
  $toolData{LS_NTDBTOOL} = $toolData{LS_NTDBTOOLDIR} . "/" . $config->{'LS_NTOFLMT'} if defined($config->{'LS_NTOFLMT'});
  $toolData{GPON_NTDBTOOL} = $toolData{GPON_NTDBTOOLDIR} . "/" . $config->{'GPON_NTOFLMT'} if defined($config->{'GPON_NTOFLMT'});
  $toolData{NTDBCFG} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{ANSI_NTDBCFG} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{LS_NTDBCFG} = $toolData{LS_NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{GPON_NTDBCFG} = $toolData{GPON_NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{LANXDBTOOL} = $config->{'LANXOFLMT'} if defined($config->{'LANXOFLMT'});
  $toolData{OCTOPUSPATH} = $config->{'OCTOPUSPATH'} if defined($config->{'OCTOPUSPATH'});
  $toolData{SECUREDBSAVE} = $config->{'SECUREDBSAVE'} if defined($config->{'SECUREDBSAVE'});
  $toolData{TFTPTIMEOUT} = $config->{'TFTPTIMEOUT'} if defined($config->{TFTPTIMEOUT});                  # -- 1.1.7
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT});            # -- 1.1.7
  $toolData{EMERG_ALRM_POLL_TIME} = $config->{'EMERG_ALRM_POLL_TIME'} if defined($config->{EMERG_ALRM_POLL_TIME});            
  $toolData{SHUBSWPATH} = $config->{'SHUBSWPATH'} if (defined($config->{'SHUBSWPATH'}));
  $toolData{SHUBSWNAME} = $config->{'SHUBSWNAME'} if (defined($config->{'SHUBSWNAME'}));
  $toolData{IVPSDBTOOLDIR} = $config->{'IVPSDBTOOLDIR'} if defined($config->{'IVPSDBTOOLDIR'});
  $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'IVPSOFLMT'} if defined($config->{'IVPSOFLMT'});
  $toolData{IVPSDBTOOL_LINUX} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'LINUX_IVPSOFLMT'} if defined($config->{'LINUX_IVPSOFLMT'});
  $toolData{IVPSDBTOOL_I386} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'I386_IVPSOFLMT'} if defined($config->{'I386_IVPSOFLMT'});
  $toolData{IVPSDBCFG} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'IVPSDBCFG'} if defined($config->{'IVPSDBCFG'});
  $toolData{IVPSINVENTORY} = $config->{'IVPSINVENTORY'} if defined($config->{'IVPSINVENTORY'});
  $toolData{CDEDIR} = $config->{'CDEDIR'} if defined($config->{'CDEDIR'});
  $toolData{NEWCDEFILE} = $config->{'NEWCDEFILE'} if defined($config->{'NEWCDEFILE'});
  $toolData{ACTCDEFILE} = $config->{'ACTCDEFILE'} if defined($config->{'ACTCDEFILE'});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
										 ($config->{NT_COMMUNITY} ne ""));
  $toolData{ACTPINGWAIT} = $config->{ACT_PING_WAIT} if defined($config->{ACT_PING_WAIT});
  $toolData{ACTPINGWAITRU} = $config->{ACT_PING_WAIT_RU} if defined($config->{ACT_PING_WAIT_RU});
  $toolData{REMAP_HOTRESET} = uc($config->{REMAP_HOT_RESET}) if defined($config->{REMAP_HOT_RESET});
  $toolData{RETRIES} = $config->{RETRIES} if (defined($config->{RETRIES}));
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});
  $STATUSFILE = $config->{MIGR_STATUS_FILE} if defined($config->{MIGR_STATUS_FILE});
  $toolData{FORCE_OPERATOR_ID}  = $config->{'FORCE_OPERATOR_ID'} if defined($config->{'FORCE_OPERATOR_ID'});
  $toolData{SSMOUT_VLAN}  = $config->{'SSMOUT_VLAN'} if defined($config->{'SSMOUT_VLAN'});
  $toolData{ENABLE_IGMP_GROUPING}  = $config->{'ENABLE_IGMP_GROUPING'} if defined($config->{'ENABLE_IGMP_GROUPING'});
  $toolData{IGMP_MC_RANGE_SETTING}  = $config->{'IGMP_MC_RANGE_SETTING'} if defined($config->{'IGMP_MC_RANGE_SETTING'});
  $toolData{MAX_GIP_GAP}  = $config->{'MAX_GIP_GAP'} if defined($config->{'MAX_GIP_GAP'});
  $toolData{GPON_ONU_RANGE_MODE}  = $config->{'GPON_ONU_RANGE_MODE'} if defined($config->{'GPON_ONU_RANGE_MODE'});
  $toolData{ACT_VLAN_LABEL}  = $config->{'ACT_VLANLABEL'} if defined($config->{'ACT_VLANLABEL'});
  $toolData{NEW_VLAN_LABEL}  = $config->{'NEW_VLANLABEL'} if defined($config->{'NEW_VLANLABEL'});
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});

}


#==============================================================================
#== Main program
#==============================================================================
my @pids;
my $pid;
my $warnCount = 0;

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub HandleSigPipe() {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();

  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  FamilyCommon::SetSigPipeReceived();

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    Logger::LogTrace($warnLogFile, "SIGPIPE received (ignored): $msgStr") if defined($warnLogFile);
    print STDERR "SIGPIPE received (ignored): $msgStr" if !defined($warnLogFile);
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
	print STDERR sprintf("StackFrame $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
    IsamCli::TelnetConnectionLost(FamilyCommon::GetCliSession2Close());
    $SIG{"PIPE"} = \&HandleSigPipe;
    FamilyCommon::CloseCliSession2Close();
    # Don't exist, go-on
  }
}

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string if !defined($warnLogFile);
  Logger::LogTrace($warnLogFile, $string) if defined($warnLogFile);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;
  my $nodeStatusLogFH = FamilyCommon::GetNodeStatusLog();

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      if (defined($nodeStatusLogFH)) {
	      print $nodeStatusLogFH sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
				   defined($ownPackage) ? $ownPackage : "undefined",
				   defined($ownFileName) ? $ownFileName: "undefined",
				   defined($ownLine) ? $ownLine : "undefined",
				   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      }
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR "  Aborting children...\n";
    Logger::LogTrace($warnLogFile, "  Aborting children...\n") if defined($warnLogFile);
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	Logger::LogTrace($warnLogFile, "Killing process $pid...\n") if defined($warnLogFile);
	kill($pid);
      }
    }
  }
  FamilyCommon::UpdateNodeStatusLog( $exitCode, $signal );
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);

$SIG{"INT"} = \&CleanUp;
$SIG{"__WARN__"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();
  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($warnLogFile, "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr") if (defined($warnLogFile));
    print STDERR "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]: $msgStr\n";
    $warnCount++;
    my $framePresent = 1;
    my $i = 0;
    my $ownPackage;
    my $ownFileName;
    my $ownLine;
    my $ownRoutine;
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
	print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
  }
};
$SIG{"__DIE__"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "[PID $ownPid] Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "[PID $ownPid] Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "[PID $ownPid] Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "[PID $ownPid] Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};
$SIG{"PIPE"} = \&HandleSigPipe;

#-- Read parameter config file and adjust tool defaults if needed -- BB--
my $dirName = dirname($0);
FamilyCommon::SetToolDirName($dirName);
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR sprintf("%s (%s::%s): $dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n", basename($0),
		       PBMT_VERSION, $VERSION);
  exit PARAM_CFG_MISSING_CODE;
}
FamilyCommon::SetConfigData(\%config);
AdjustToolVars(\%config);
# -- End modif BB

#-- Handle commandline
$params = ParseCommandLine(\%config);
STDOUT->autoflush(1);
STDERR->autoflush(1);
$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

Utils::SetDebug($execLogFH) if $params->{DEBUG};
PrintOptionsSummary($execLogFH, 1, \%config);

#-- Create work dir
#umask(0); # Make sure directory remains 0777
umask(0) if ( $params->{UMASK_CHMOD} );
### Create work-space directory
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
Logger::LogDebug( $execLogFH, "Workspace directory : $toolData{TFTPDIR}/$toolData{TFTPWORKDIR}\n" );

ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
Logger::LogInfo( $execLogFH, "Resultspace directory : $toolData{RESULTSPACE}\n", $Logger::LogOnlyDebug );

my $status_dbtool_parse = CapabilityReEngr::ParseDBToolConfig( "$scriptDir/$toolData{NTDBCFG}" );
if( $status_dbtool_parse ) {
  Logger::LogError($nodeLogFH, "Issue in parsing DB & Release version mapping file '$scriptDir/$toolData{NTDBCFG}'\n" );
  exit FILE_OPEN_FAIL_CODE;
}
#-- read asams
my $isamsRef;
if (defined($params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    if (ToolCommon::ValidIP($params->{ISAM}))
    {
      Logger::LogDebug($execLogFH, "Translate host-IP to hostname\n");
      my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
      if (! defined($name)) {
        $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
      } else {
        $isamsRef = [ [$params->{ISAM}, $name] ];
      }
    } else {
      print STDERR "ERROR: Invalid ISAM IP\n";
      exit INVALID_ISAM_IP_CODE;
    }
  } else {
    Logger::LogDebug($execLogFH, "Translate hostname to host-IP\n");
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  Logger::LogInfo($execLogFH, "Reading Hostfile...\n");
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}
exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      push(@pids, $pid);
    } else {
      MigrateIsams(\@isamPart, $sessionCount, \%config);
      if( defined $params->{BTYPE} and ( $params->{BTYPE} ne "SNMP" ) ) {
	 exit CONDITONAL_SUCCESS_CODE;
      }
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    $exitCode = $? >> 8;
    $signal = $? & 255;
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      Logger::LogInfo( $execLogFH, sprintf("Session %d : ends with exit code : %d - (%s); signal (%d)\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal ) );
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode == 5) {
	$newExitCode =  CONDITONAL_SUCCESS_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
  FamilyCommon::CloseWarnLogFile();
}
exit $overAllExitCode;
