#!/usr/bin/perl
#==============================================================================
#-- File:         ppp2pppipoe.pl
#-- Date:         28.01.2008
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to PVCs using automatic:ppp into automatic:ipoe-ppp
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent AND 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/01/24 : First implementation (RCR ANTms80330)
#--   1.0.1 - 2008/02/22 : ANTms85515: - One more command line parameter: the max mac address
#--                        that can be learned on the bridge
#--                                    - only reconfigure pvc's involved in CC client-ports.
#--   1.0.2 - 2008/06/03 : AANTmt09938 - include set and release transaction
#--   1.0.3 - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.0.4 - 2008/10/28 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.0.5 - 2008/03/10 : ANTmt18506
#--   1.0.6 - 2009/06/11 : Allow SSH for CLI
#--   1.0.7 - 2009/06/29 : Some cleanup and best practises.
#--   1.0.8 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.9 - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session
#--   1.0.10 - 2010/03/04 : ANTmg01050. Total_MiB label change from Ex.x.y.z to Vx.x.x.y.z
#--   1.0.11 - 2010/03/15 : PBMT x86 Changes - ANTmt88407
#--   1.0.12 - 2011/01/11 : LRM updates for lib-perl path
#--   1.0.13 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   1.0.14 - 2011/05/02 : ALU01178887
#--   1.0.15 - 2012/05/14 : ALU01789702
#--   1.0.16 - 2012/06/13 : Change permissions for backup db file
#--   1.0.17 - 2013/04/29 : ALU01880147
#--   1.0.18 - 2016/07/15 : ALU02237966
#--   1.0.19 - 2016/09/28 : ALU02278259

my $VERSION = "ppp2pppipoe.pl - 1.0.19 - 2016/09/28";
#==============================================================================
use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 
#==============================================================================
use Crypt::Lite;
use Capability;
use IsamCli;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :PPPC);
use Utils;
use version qw(:ALL);
#==============================================================================
#== Constants
#==============================================================================
my $HOSTSFILE =               "/etc/hosts";
my $TFTPDIR =                 "/var/opt/aws/common/tftpd/data";
my $TFTPWORKDIR =             "tmp";
my $MAXSESSIONS =             5;
my $USERNAME =                "";
my $PASSWORD =                "";
my $NODEPREFIX =              "AS-";
my $TELNETTIMEOUT =           10;
my $STATUSFILE =              "PPPCresult.txt";
my $NT_COMMUNITY =            "public";
my $RETRIES =                 5;
my $NO_PING =                 0;

#==============================================================================
#== Vars
#==============================================================================
my $warnLogFile;
my %config;

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub Usage {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> --pppvlan <vlanid> -ipvlan <vlanid> --maxmac <nr> \\ \n";
  print STDERR "       [--sessions <nr>] [--seplog] [--noaction] [--confinfo] \\ \n";
  print STDERR "       [--rollback]  [--telnetto <minutes>] [--username <user>] \\ \n";
  print STDERR "       [--password <password>] [--v] [--debug] [--bufsize n] [--noping]\n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         pppvlan  : id of the network VLAN used for PPP\n";
  print STDERR "         ipvlan   : id of the network VLAN used to IP\n";
  print STDERR "         maxmac   : maximum number of MAC addresses that be learned on a bridge (1..64)\n";
  print STDERR "         sessions : number of parallel ISAMs to be treated (MAX=$MAXSESSIONS)\n";
  print STDERR "         username : ISAM username\n";
  print STDERR "         password : ISAM password\n";
  print STDERR "         seplog   : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noaction : just check, do not change the type\n";
  print STDERR "         debug    : extra logging for debugging purposes\n";
  print STDERR "         confinfo : print out all retrieved configration info\n";
  print STDERR "         rollback : roll back to original configuration\n";
  print STDERR "         bufsize  : size of CLI command return buffer (default is 15Mb)\n";
  print STDERR "         telnetto : timeout value for telnet session (in minutes)\n";
  print STDERR "         noping   : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  exit CMD_INCOMPL_CODE;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub ParseCommandLine {
  my ($isam, $nwVlanIdPPP, $nwVlanIdIP, $maxMac, $username, $password, $sessions, $debug, $hosts, $logfile, $version, $sepLog, $noaction, $confInfo,
      $rollBack, $bufSize, $telnetTO, $noPing);
  my %params;
  my $options = GetOptions('isam=s', \$isam,
			   'noaction', \$noaction,
			   'pppvlan=i', \$nwVlanIdPPP,
			   'ipvlan=i', \$nwVlanIdIP,
			   'maxmac=i', \$maxMac,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'username=s', \$username,
			   'password=s', \$password,
			   'logfile=s', \$logfile,
			   'v', \$version,
			   'seplog', \$sepLog,
			   'confinfo', \$confInfo,
			   'rollback', \$rollBack,
			   'telnetto=i', \$telnetTO,
			   'bufsize=i', \$bufSize,
			   'noping', \$noPing);

  #-- check options
  if (! $options) {
    Usage();
  }
  if (defined($version)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logfile)) {
    print STDERR "ERROR: no logfile given\n";
    Usage();	
  }
  if (!defined($nwVlanIdPPP)) {
    print STDERR "ERROR: No network VLAN ID for PPP provided\n";
    Usage();
  }
  if (!defined($nwVlanIdIP)) {
    print STDERR "ERROR: No network VLAN ID for IP provided\n";
    Usage();
  }
  if (!defined($maxMac)) {
    print STDERR "ERROR: No maximum number of MAC addresses specified\n";
    Usage();
  } else {
    if (($maxMac <1) || ($maxMac > 64)) {
      print STDERR "ERROR: maximum number of MAC addresses must be between 1 and 64\n";
      Usage();
    }
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: Hostsfile $hosts does not exist\n";
    Usage();
  }
  if (defined($username)) {
    $params{USERNAME} = $username;
  } else {
    $params{USERNAME} = $USERNAME;
  }
  if (defined($password)) {
    $params{PASSWORD} = $password;
  } else {
    $params{PASSWORD} = $PASSWORD;
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $MAXSESSIONS) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
	"($sessions > $MAXSESSIONS)\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $HOSTSFILE;
  }
  $params{UMASK_CHMOD} = ( (! defined $config{UMASK_OVERRIDE}) or $config{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = defined($debug) ? 1 : 0;
  $params{NWVLANIDPPP} = $nwVlanIdPPP;
  $params{NWVLANIDIP} = $nwVlanIdIP;
  $params{MAXMAC} = $maxMac;
  $params{LOGFILE} = $logfile;
  $params{SEPLOG} = defined($sepLog) ? 1 : 0;
  $params{NOACTION} = defined($noaction) ? 1 : 0;
  $params{CONFINFO} = defined($confInfo) ? 1 : 0;
  $params{ROLLBACK} = defined($rollBack) ? 1 : 0;
  $params{BUFSIZE} = defined($bufSize) ? $bufSize : 15;
  $params{BUFSIZE} = 100 if ($params{BUFSIZE} > 100);
  $params{BUFSIZE} *= 1048576; # Convert to MBytes
  $params{TELNETTIMEOUT} = defined($telnetTO) ? $telnetTO * 60 : 600; # default 10 minutes
  $params{NOPING} = $NO_PING;
  $params{NOPING} = 1 if defined($noPing);
  return \%params;
}

#------------------------------------------------------------------------------
#--
#------------------------------------------------------------------------------
sub PrintLibVersions {
  my ($fh, $out) = @_;

  if (defined($fh)) {
    ToolCommon::Log($fh, "Library versions:\n", $out);
    ToolCommon::Log($fh, sprintf("  %s\n", IsamCli::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ToolCommon::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", FamilyCommon::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", Capability::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", ErrorCodes::GetVersion()), $out);
    ToolCommon::Log($fh, sprintf("  %s\n", Utils::GetVersion()), $out);
  } else {
    print STDERR "Library versions:\n";
    print STDERR sprintf("  %s\n", IsamCli::GetVersion());
    print STDERR sprintf("  %s\n", ToolCommon::GetVersion());
    print STDERR sprintf("  %s\n", FamilyCommon::GetVersion());
    print STDERR sprintf("  %s\n", Capability::GetVersion());
    print STDERR sprintf("  %s\n", ErrorCodes::GetVersion());
    print STDERR sprintf("  %s\n", Utils::GetVersion());
  }
}

sub PrintOptionsSummary {
  my($fh, $out) = @_;

  ToolCommon::Log($fh, sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION), $out);
  ToolCommon::Log($fh, "Options:\n", $out);
  ToolCommon::Log($fh, "   Nr Sessions                              : $params->{SESSIONS}\n", $out);
  ToolCommon::Log($fh, "   Hosts file                               : $params->{HOSTS}\n", $out) if (!defined($params->{ISAM}));
  ToolCommon::Log($fh, "   Only One Isam                            : $params->{ISAM}\n", $out) if (defined($params->{ISAM}));
  ToolCommon::Log($fh, sprintf("   Ping                                     : %s\n", $params->{NOPING} ? "not allowed" : "allowed"), $out);
  ToolCommon::Log($fh, "   Network VLAN ID for PPP                  : $params->{NWVLANIDPPP}\n", $out);
  ToolCommon::Log($fh, "   Network VLAN ID for IP                   : $params->{NWVLANIDIP}\n", $out);
  ToolCommon::Log($fh, "   Maximum number of MAC addresses on bridge: $params->{MAXMAC}\n", $out);
  ToolCommon::Log($fh, sprintf("   Telnet CLI command buffer size           : %d Mbytes\n", $params->{BUFSIZE} / 1048576), $out);
  ToolCommon::Log($fh, sprintf("   Log file per node                        : %s\n", $params->{SEPLOG} ? "Yes" : "No"), $out);
  ToolCommon::Log($fh, sprintf("   Telnet timeout                           : %d minutes\n", $params->{TELNETTIMEOUT} / 60), $out);
  ToolCommon::Log($fh, "   Debugging Enabled\n", $out) if $params->{DEBUG};
  ToolCommon::Log($fh, "   No reconfiguration, only testing\n", $out) if $params->{NOACTION};
  ToolCommon::Log($fh, "   Print retrieved CLI configuration information\n", $out) if $params->{CONFINFO};
  ToolCommon::Log($fh, "   Rollback original configuration\n", $out) if $params->{ROLLBACK};
  PrintLibVersions($fh, $out);
}

#------------------------------------------------------------------------------
#-- Parse PVC config.
#-- Hash structure as follows:
#-- %pvcConfig{$pvc}{ADMINDOWN}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to CLI response
#-- $configStorage: pointer to a hash for storing the ATM PVC configuration.
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParsePvcConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseNum) = @_;
  my $rspStr = "";

  foreach my $line (@{$response}) {
    chomp ($line);
    if ($line =~ /configure/) {
      ToolCommon::Log($fh, "[ParsePvcConfig] line read: $line\n") if ($params->{DEBUG});
      if ($line =~ s/configure\s+atm\s+pvc\s+(\d+\/\d+\/\d+\/\d+:\d+:\d+)//) {
	my $pvc = $1;
	push @{$cliResponseLines}, $line;
	$configStorage->{$pvc}{ADMINDOWN} = 0;
	if ($line =~ s/admin\-down//) {
	  $configStorage->{$pvc}{ADMINDOWN} = 1;
	  ToolCommon::Log($fh, sprintf("Stored admin-down for %s\n", $pvc)) if ($params->{DEBUG});
	}
	if ($line =~ s/aal5\-encap\-type\s+([^\s]+)//) {
	  my $aal5EncapsType = $1;
	  if (uc($aal5EncapsType) ne "AUTOMATIC:PPP") {
	    FamilyCommon::SetAndLogError($fh, "[ParsePvcConfig] Invalid encapsulation type returned: $aal5EncapsType");
	    return 0;
	  }
	}
      }
      $line =~ s/\s//g;
      if ($line ne "") {
	FamilyCommon::SetAndLogError($fh, "[ParsePvcConfig] Command not completely parsed, left: $line");
	return 0;
      }
    } else {
      if (! ($line =~ /\#[-]+$/) &&
	  ! ($line =~ /[\$%\#>]\s*$/)) {
	FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	return 0;
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse ATM-PVC interface.
#-- Hash structure as follows:
#-- %pvcConfig{$pvc}{CUSTID}
#-- $fh: pointer to a file handle for logging purposes
#-- $cli: pointer to opened TELNET session
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to CLI response
#-- $configStorage: pointer to a hash for storing the ATM PVC configuration.
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParsePvcItfConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseNum) = @_;
  my $rspStr = "";

  foreach my $line (@{$response}) {
    chomp ($line);
    if ($line =~ /configure/) {
      if ($line =~ s/configure\s+interface\s+atm\-pvc\s+(\d+\/\d+\/\d+\/\d+:\d+:\d+)//) {
	my $pvc = $1;
	ToolCommon::Log($fh, "[ParsePvcItfConfig] line read: $line\n") if ($params->{DEBUG});
	if (defined($configStorage->{$pvc})) {
	  push @{$cliResponseLines}, $line;
	  if ($line =~ s/customer\-id\s+([^\s]+)//) {
	    my $custId = $1;
	    $configStorage->{$pvc}{CUSTID} = $custId;
	    ToolCommon::Log($fh, sprintf("Stored customer ID for %s as %s\n", $pvc, $custId)) if ($params->{DEBUG});
	  }
	  $line =~ s/\s//g;
	  if ($line ne "") {
	    FamilyCommon::SetAndLogError($fh, "[ParsePvcItfConfig] Command not completely parsed, left: $line");
	    return 0;
	  }
	}
      } else {
	if (! ($line =~ /\#[-]+$/) &&
	    ! ($line =~ /[\$%\#>]\s*$/)) {
	  FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	  return 0;
	}
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
#-- Parse PPPoX-relay CC client-port configuration.
#-- Hash structure as follows:
#-- %pppCcClientPorts{bridgeport}{VLANID}
#--                              {DEFPRIO}
#--                              {MAXCC}
#--                              {QOSPROF}
#--
#-- There is one line per client-port in the response.
#--
#-- $fh: pointer to a file handle for logging purposes
#-- $params: pointer to a hash containing all script command options.
#-- $response: pointer to array of CLI response lines
#-- $cliResponseLines: pointer to array for holding the CLI response lines in the
#--                    order how the complete configuration was retrieved
#-- $configStorage: pointer to a hash for storing the complete VLAN structure.
#-- $releaseId: release indentification (for possible future usage).
#------------------------------------------------------------------------------
sub ParseClientPortConfig {
  my ($fh, $response, $cliResponseLines, $configStorage, $releaseId) = @_;
  my $bridgePort = "";
  my $ok = 1;

  foreach my $line (@{$response}) {
    chomp $line;
    if ($line =~ /configure/) {
      if ($line =~ s/configure\s+pppox\-relay\s+cross\-connect\s+client\-port\s+([^\s]+)//) {
	ToolCommon::Log($fh, "[ParseClientPortConfig] line read: $line\n") if ($params->{DEBUG});
	$bridgePort = $1;
	push @{$cliResponseLines}, $line;
	if ($line =~ s/vlan\-id\s+([^\s]+)//) {
	  $configStorage->{$bridgePort}{VLANID} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{VLANID} as '%s'\n",
				       $configStorage->{$bridgePort}{VLANID})) if ($params->{DEBUG});
	}
	if ($line =~ s/default\-priority\s+(\d+)//) {
	  $configStorage->{$bridgePort}{DEFPRIO} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{DEFPRIO} as '%s'\n",
				       $configStorage->{$bridgePort}{DEFPRIO})) if ($params->{DEBUG});
	}
	if ($line =~ s/max\-cc\s+(\d+)//) {
	  $configStorage->{$bridgePort}{MAXCC} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{MAXCC} as '%s'\n",
				       $configStorage->{$bridgePort}{MAXCC})) if ($params->{DEBUG});
	}
	if ($line =~ s/qos\-profile\s+[^\s]+//) {
	  $configStorage->{$bridgePort}{QOSPROF} = $1;
	  ToolCommon::Log($fh, sprintf("  Stored pppCcClientPort->{$bridgePort}{QOSPROF} as '%s'\n",
				       $configStorage->{$bridgePort}{QOSPROF})) if ($params->{DEBUG});
	}
	$line =~ s/\s//g;
	if ($line ne "") {
	  FamilyCommon::SetAndLogError($fh, "[ParseClientPortConfig] Command not completely parsed, left: $line");
	  $ok = 0;
	}
      } else {
	if (! ($line =~ /\#[-]+$/) &&
	    ! ($line =~ /[\$%\#>]\s*$/)) {
	  FamilyCommon::SetAndLogError($fh, "Line not according to expected syntax: $line");
	  $ok = 0;
	}
      }
    }
  }
  return $ok;
}

#------------------------------------------------------------------------------
# Generate PPPoX CC clientport deletion rollback command using the client-port
# configuration previously retrieved.
#------------------------------------------------------------------------------
sub RollBackCcPortDeletion {
  my ($clientPorts, $pvc, $releaseNum) = @_;

  my $rollBackCmd = "configure pppox-relay cross-connect client-port $pvc vlan-id $clientPorts->{$pvc}{VLANID}";
  $rollBackCmd .= " default-priority $clientPorts->{$pvc}{DEFPRIO}" if (defined($clientPorts->{$pvc}{DEFPRIO}));
  $rollBackCmd .= " max-cc $clientPorts->{$pvc}{MAXCC}" if (defined($clientPorts->{$pvc}{MAXCC}));
  $rollBackCmd .= " qos-profile $clientPorts->{$pvc}{QOSPROF}" if (defined($clientPorts->{$pvc}{QOSPROF}));
  return $rollBackCmd;
}

#------------------------------------------------------------------------------
# Prepare the PVC reconfiguration:
# Delete all PVCs
#
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $ccEngine: pointer to a hash of PVC configuration.
# $clientPorts: pointer to a hash of all the clientports associated with that engine (only needed for rollback).
# $dhcpAgentStorage: pointer to a hash of the complete DHCP relay-agent structure.
# $pvcStorage: pointer to a hash of the ATM PVC configuration.
# $cliCommandStorage: pointer to an array holding all the unconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollkbac commands of the unconfiguration.
#------------------------------------------------------------------------------
sub PrepareDeletion {
  my ($fh, $pvcConfig, $clientPorts, $cliCommandStorage, $rollBackCmds, $releaseId) = @_;
  my $cmd = "";

  ToolCommon::Log($fh, "[PrepareDeletion]\n") if ($params->{DEBUG});
  foreach my $pvc (sort { my $a1 = $a; $a1 =~ s/\//\./g;
			  my $b1 = $b; $b1 =~ s/\//\./g;
			  pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) }
		   keys %{$pvcConfig}) {
    if (defined($clientPorts->{$pvc})) {
      $cmd = "configure atm no pvc $pvc";
      push @{$cliCommandStorage}, $cmd;
      ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
      # Start rollback action:
      # 1. Rollback PVC deletion
      push @{$rollBackCmds->{$cmd}}, "configure atm pvc $pvc aal5-encap-type automatic:ppp";
      if (defined($pvcConfig->{$pvc}{CUSTID})) {
	# 2. Rollback customer ID PVC-delete propagation
	push @{$rollBackCmds->{$cmd}}, "configure interface atm-pvc $pvc customer-id \"$pvcConfig->{$pvc}{CUSTID}\"";
      }
      # 3. Rollback PPPoX client-port PVC-delete propagation
      push @{$rollBackCmds->{$cmd}}, RollBackCcPortDeletion($clientPorts, $pvc, $releaseId);
      # From R3.5 cross-connect client-port creation no longer results in auto-creation of a bridge port.  Recreate it.
      # 4. Rollback bridge port PVC-delete propagation - part 1
      #push @{$rollBackCmds->{$cmd}}, "configure bridge port $pvc";
      # 5. Rollback bridge port PVC-delete propagation - part 2
      #push @{$rollBackCmds->{$cmd}}, "configure bridge port $pvc vlan-id $params->{NWVLANIDPPP}";
      # No need to re-configure pvid: seems to be set as part of CC client-port configuration.
      if ($params->{DEBUG}) {
	foreach my $rollBackCmd (@{$rollBackCmds->{$cmd}}) {
	  ToolCommon::Log($fh, "    Stored rollback command '$rollBackCmd'\n");
	}
      }
    }
  }
}

sub CreatePppoxCcClientport {
  my ($pvc, $clientPorts) = @_;
  my $cmd = "configure pppox-relay cross-connect client-port $pvc vlan-id $clientPorts->{$pvc}{VLANID}";

  $cmd .= " default-priority $clientPorts->{$pvc}{DEFPRIO}" if defined($clientPorts->{$pvc}{DEFPRIO});
  $cmd .= " max-cc $clientPorts->{$pvc}{MAXCC}" if defined($clientPorts->{$pvc}{MAXCC});
  $cmd .= " qos-profile $clientPorts->{$pvc}{QOSPROF}" if defined($clientPorts->{$pvc}{QOSPROF});
  return $cmd;
}

#------------------------------------------------------------------------------
# Prepare creation of ATM-PVC using automatic:ipoe-ppp.
# 1. Create PVC
# 2. Re-assign customer-id
# 3. Create PPPoX CC client-port
# 4. Create bridge port for PPP
# 5. Create VLAN port for PPP
# 7. Create VLAN port for IP
# $fh: pointer to a file handle for logging purposes
# $params: pointer to a hash containing all script command options.
# $pvcConfig: pointer to a hash of the ATM PVC configuration.
# $clientPorts pointer to a hash of the PPPoX CC client ports.
# $cliCommandStorage: pointer to an array holding all the reconfiguration commands
# $rollBackCmds: pointer to an array holding all the rollkbac commands of the reconfiguration.
#------------------------------------------------------------------------------
sub PrepareCreation {
  my ($fh, $pvcConfig, $clientPorts, $cliCommandStorage, $rollBackCmds, $releaseId) = @_;
  my $cmd = "";
  my $ok = 1;
  my $rollBackCmd = "";

  foreach my $pvc (sort { my $a1 = $a; $a1 =~ s/\//\./g;
			  my $b1 = $b; $b1 =~ s/\//\./g;
			  pack('C4', $a1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) cmp pack('C4', $b1 =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) } keys %{$pvcConfig}) {
    ToolCommon::Log($fh, "[PrepareCreation] $releaseId\n") if ($params->{DEBUG});
    if (defined($clientPorts->{$pvc})) {
      $cmd = "configure atm pvc $pvc aal5-encap-type automatic:ipoe-ppp";
      push @{$cliCommandStorage}, $cmd;
      ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
      # Only 1 command to rollback these 2 commands
      $rollBackCmd = "configure atm no pvc $pvc";
      push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
      ToolCommon::Log($fh, "    Stored rollback command '$rollBackCmd'\n")if ($params->{DEBUG});
      if (defined($pvcConfig->{$pvc}{CUSTID})) {
	$cmd = "configure interface atm-pvc $pvc customer-id \"$pvcConfig->{$pvc}{CUSTID}\"";
	push @{$cliCommandStorage}, $cmd;
	ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
      }
      $cmd = CreatePppoxCcClientport($pvc, $clientPorts);
      ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
      push @{$cliCommandStorage}, $cmd;
      $rollBackCmd = "configure pppox-relay cross-connect no client-port $pvc";
      push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
      ToolCommon::Log($fh, "    Stored rollback command '$rollBackCmd'\n") if ($params->{DEBUG});
      $cmd = "configure bridge port $pvc";
      push @{$cliCommandStorage}, $cmd;
      ToolCommon::Log($fh, "  Stored '$cmd'\n")if ($params->{DEBUG});
      push @{$cliCommandStorage}, "configure bridge port $pvc max-unicast-mac $params->{MAXMAC}";
      push @{$cliCommandStorage}, "configure bridge port $pvc vlan-id $params->{NWVLANIDPPP}";
      push @{$cliCommandStorage}, "configure vlan port-protocol $pvc protocol-group pppoe vlan-id $params->{NWVLANIDPPP} priority 0";
      push @{$cliCommandStorage}, "configure bridge port $pvc vlan-id $params->{NWVLANIDIP}";
      push @{$cliCommandStorage}, "configure vlan port-protocol $pvc protocol-group ipoe vlan-id $params->{NWVLANIDIP} priority 0";
      $rollBackCmd = "configure bridge no port $pvc";
      push @{$rollBackCmds->{$cmd}}, $rollBackCmd;
      ToolCommon::Log($fh, "    Stored rollback command '$rollBackCmd'\n") if ($params->{DEBUG});
    }
  }
  return $ok;
}

#------------------------------------------------------------------------------
#  Backup system database for safety (may be needed if reconfiguration failed
# dramatically and CLI restoration failed.
#------------------------------------------------------------------------------
sub BackupDatabase {
  my ($fh, $cli, $isam, $debug, $parentFh, $logfileName, $count) = @_;
  my $time1 = time();
  my $oswpInfo;
  my $ok = 1;
  my $tftpIpAddress = "";
  my $fileName = "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
  my $cmd = "$config{RM} $fileName > PPPCrm.txt 2>&1;$config{TOUCH} $fileName";
  my $sysret = system($cmd);

  ToolCommon::Log($fh, "\nCMD = $cmd\n") if ($debug);
  if ($sysret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($sysret);
    FamilyCommon::SetAndLogError($fh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** $cmd FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "Touching $fileName OK\n") if $debug;
  }
  if ($params->{UMASK_CHMOD} and ! chmod(0666, "$fileName")) {
    FamilyCommon::SetAndLogError($fh, "chmod(666,$fileName) FAILED: $!", "  FAILED: $!\n");
    ToolCommon::Log($parentFh, "Child $count: **FATAL** chmod(666,$fileName) FAILED, please verify $logfileName.\n", 1);
    $ok = 0;
  }
  if ($ok) {
    $oswpInfo = IsamCli::GetOswpInfo($cli, REGEXP);
    if (!defined($oswpInfo)) {
      FamilyCommon::SetAndLogError($fh, sprintf("Failed to retrieve OSWP info: %s", IsamCli::GetError()));
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Active OSWP: $oswpInfo->{ACTIVE}{INDEX}\n") if $debug;
      my @response = IsamCli::SendCommand($cli, "info configure software-mngt oswp flat", undef);
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::SetAndLogError($fh, sprintf("Retrieving oswp configuration failed: %s", IsamCli::GetError()));
	$ok = 0;
      } else {
	foreach my $line (@response) {
	  chomp $line;
	  ToolCommon::Log($fh, "Line read: $line\n") if $debug;
	  if ($line =~ /configure\s+software\-mngt\s+oswp\s+(\d+)\s+primary\-file\-server\-id\s+([^\s]+)/) {
	    my $oswpNum = $1;
	    my $ipAddress = $2;
	    if ($oswpInfo->{ACTIVE}{INDEX} == $oswpNum) {
	      $tftpIpAddress = $ipAddress;
	      last;
	    }
	  } else {
	    if (! ($line =~ /\#[-]+$/) &&
		! ($line =~ /[\$%\#>]\s*$/)) {
	      # Should not happen
	      FamilyCommon::SetAndLogError($fh, "OSWP config info not according to expected syntax: $line");
	      $ok = 0;
	      last;
	    }
	  }
	}
	if (($tftpIpAddress ne "") && ($tftpIpAddress ne "0.0.0.0")) {
	  my $ret = IsamCli::UploadDb($cli, 'actual-active',
				      $tftpIpAddress, "$TFTPWORKDIR/$isam", 1);
	  if (!defined($ret) || !$ret)  {
	    FamilyCommon::SetAndLogError($fh, sprintf("  FAILED (%s)\n", IsamCli::GetError()));
	    $ok = 0;
	  } else {
	    my $dbSize = -s "$TFTPDIR/$TFTPWORKDIR/$isam/dm_complete.tar";
	    if ($dbSize == 0) {
	      FamilyCommon::SetAndLogError($fh, sprintf("  Size of database file 0\n"));
	      $ok = 0;
	    } else {
	      ToolCommon::Log($fh, sprintf("  OK, transferred %d bytes in %d seconds\n", $dbSize, time() - $time1));
	      ToolCommon::Log($fh, "Database backup stored as $fileName\n", 1);
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, "No TFTP IP address could be obtained\n");
	  $ok = 0;
	}
      }
    }
  }
  my @response = IsamCli::SendCommand($cli, "exit all");  # Set prompt to top node
  return $ok;
}

#------------------------------------------------------------------------------
# Rollback configuration in case of error (or forced as command option)
#------------------------------------------------------------------------------
sub RollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;
  my @response;

  if ($params->{ROLLBACK}) {
    ToolCommon::Log($fh, "==================================\n");
    ToolCommon::Log($fh, "The following will be rolled back:\n");
    ToolCommon::Log($fh, "==================================\n");
  } else {
    ToolCommon::Log($fh, "Start rollback...\n");
  }
  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "Rolling back '$rollBackCmd'...\n") if $params->{DEBUG};
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	if ($params->{ROLLBACK}) {
	  ToolCommon::Log($fh, "  '$cmd'\n");
	} else {
	  ToolCommon::Log($fh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating" : "Sending", $cmd));
	}
	if (! $params->{NOACTION}) {
	  @response = IsamCli::SendCommand($cli, $cmd);
	  if (IsamCli::CommandError(join("", @response))) {
	    FamilyCommon::SetAndLogError($fh, sprintf("Rollback failed: %s", IsamCli::GetError()));
	    return 0;
	  }
#	  sleep 2;
	}
      }
    }
  }
  return 1;
}

#------------------------------------------------------------------------------
# Debugging: dump the content of the rollback command buffer.
#------------------------------------------------------------------------------
sub DumpRollBack {
  my ($fh, $cli, $rollBackCommands, $rollBackCommandConfig) = @_;
  my $rollBackCmd;
  my $cmd;

  ToolCommon::Log($fh, "==================================\n");
  ToolCommon::Log($fh, "CLI commands in case of roll back:\n");
  ToolCommon::Log($fh, "==================================\n");

  while (defined($rollBackCmd = pop @{$rollBackCommands})) {
    ToolCommon::Log($fh, "Rolling back '$rollBackCmd'...\n");
    foreach $cmd (@{$rollBackCommandConfig->{$rollBackCmd}}) {
      if (defined($cmd) && ($cmd ne "")) {
	ToolCommon::Log($fh, sprintf("  Simulating '%s'\n", $cmd));
      }
    }
  }
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------
sub HandleIsams {
  my ($fh, $isams, $count, $config) = @_;
  my $isam;
  my ($time1, $time2);
  my $startTime;
  my $nodeLogFh;
  my $nodeLogFile;
  my $nodeStatusLog;
  my $cli = undef;
  my $releaseNum;
  my %pvcConfig;
  my %pppCcClientPorts;
  my %bridgeConfig;
  my @configToDelete;
  my @configToCreate;
  my @configRetrieved = ();
  my %rollBackCommandConfig = ();
  my @rollBackCommands = ();
  my $cliCommandsSent = 0;
  my $reconfigStart;
  my $ret;

  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetParentLogFile($fh);
  FamilyCommon::SetChildNum($count);
  my $newLog = "$params->{LOGFILE}_$count";
  my $newfh;
  if (! open($newfh, ">$newLog")) {
    ToolCommon::Log($fh, "Child $count: cannot open $newLog for writing: $!\n", 1);
    CORE::exit(CREATE_FILE_FAIL_CODE)
  }
  $newfh->autoflush(1);
  ToolCommon::Log($fh, "Child $count: Logging continuous in $newLog...\n", 1);

  PrintOptionsSummary($newfh, undef);

  foreach $isam (@{$isams}) {
    FamilyCommon::SetFailedNode($isam->[1]);
    FamilyCommon::ClearError();
    # Switch debugging (if any) to an opened filehandle...
    Utils::SetDebug($newfh) if ($params->{DEBUG});
    IsamCli::SetDebug($newfh)  if ($params->{DEBUG});
    Capability::SetDebug($newfh) if ($params->{DEBUG});
    ToolCommon::SetDebug($newfh);

    ToolCommon::Log($newfh, "#===================================================\n");
    ToolCommon::Log($newfh, "#== $isam->[1]\n");
    ToolCommon::Log($newfh, "#===================================================\n");
    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Started $isam->[1]...\n",
		    $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second), 1);

    my $dir = "$TFTPDIR/$TFTPWORKDIR/$isam->[1]";
    $dir =~ s/\/\//\//g;
    if ( ! -d $dir) {
      ToolCommon::Log($newfh, "Creating working dir $dir...");
      if (!mkdir( $dir, 0777)) {
	ToolCommon::Log($newfh, "  FAILED\n");
	ToolCommon::Log($fh, "Child $count: **FATAL** Creating working dir $dir for $isam->[1] failed, please verify $newLog.\n", 1);
	CORE::exit(CREATE_DIR_FAILED_CODE);
      }
      ToolCommon::Log($newfh, "  OK\n");
    }
    if (! chmod(01777, "$dir")) {
       ToolCommon::Log($fh,  "Failed to chmod 1777 for directory '$dir'\n",1);
    }

    if ($params->{SEPLOG}) {
      $nodeLogFile = sprintf("$dir/commit_%04d-%02d-%02d_%02d:%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
      if (! open($nodeLogFh, ">$nodeLogFile")) {  # -- 1.2.5 START
	ToolCommon::Log($fh, "Child $count: cannot open $nodeLogFile for writing: $!\n", 1);
	CORE::exit(CREATE_FILE_FAIL_CODE)
      } else {
	ToolCommon::Log($fh, "Child $count: NE specific logging to $nodeLogFile...\n", 1);
      }
      $nodeLogFh->autoflush(1);
      Utils::SetDebug($nodeLogFh) if ($params->{DEBUG});
      IsamCli::SetDebug($nodeLogFh)  if ($params->{DEBUG});
      Capability::SetDebug($nodeLogFh) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFh);
    } else {
      $nodeLogFh = $newfh;
    }
    FamilyCommon::SetWarnLogFile($nodeLogFh);
    FamilyCommon::SetChildLogFile($nodeLogFh);

    if (! open($nodeStatusLog, ">$dir/$STATUSFILE")) {
      ToolCommon::Log($fh, "Child $count: cannot create $dir/$STATUSFILE for $isam->[1]: $!\n", 1);
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    }
    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }
    #-- open snmp connection
    my ($session, $snmpError) = Net::SNMP->session(-hostname => $isam->[0],
						   -version => "SNMPv1",
						   -timeout => 5,
						   -retries => 3,
						   -community => $NT_COMMUNITY);
    if (!defined($session)) {
      FamilyCommon::IncrErrorCount("  ERROR: Could not open session for $isam->[1]: $snmpError\n",
				   EC_SNMP_SESS_ERR,
				   ": $snmpError",
				   1);
      next;
    }
    #-- disable OID translation
    $session->translate(0);
    FamilyCommon::SetSnmpSession2Close($session);

    ToolCommon::Log($nodeLogFh, "SNMP connectivity...");
    my $mibVersion = ToolCommon::DoSafeGet($session, MIBVERSIONOID, $RETRIES);
    if (!defined($mibVersion)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", $session->error()),
				   EC_SNMP_NO_GET,
				   sprintf(": %s", $session->error()),
				   1);
      next;
    } else {

	if ($mibVersion eq "") {
		  my $version = $session->version();
		  $version++ if ($version < 3);
		  my $t = sprintf("SNMP configuration mismatch: check SNMP version (locally SNMPv%d is configure) and SNMP userna me/password (if applicable)", $version);
		  FamilyCommon::IncrErrorCount("  FAILED: $t",
					       EC_SNMP_NO_GET,
					       ": $t",
					       1);
		  next;
        }

     if (($mibVersion =~ /3FE21961EAAA_E(\d+)\.(\d+)\.\d+\.\d+/)      ||  # ETSI
          ($mibVersion =~ /3FE21961EAAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/) ||  # ETSI (ISAM MIB label Extn)
	  ($mibVersion =~ /3FE21961ANAA_E(\d+)\.(\d+)\.\d+\.\d+/)      ||  # ANSI
	  ($mibVersion =~ /3FE21961ANAA_V(\d+)\.(\d+)\.\d+\.\d+\.\d+/)) {  # ANSI (ISAM MIB label Extn)
	my $major = $1; my $minor = $2;
	$releaseNum = Capability::ConvertRelease($major, $minor);
	if (Capability::ReleaseCompare($releaseNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
	  FamilyCommon::IncrErrorCount(sprintf("  ERROR: Need at least R3.5 to do this reconfiguration (current release : %s)\n",
					       Capability::NtDbVersionInfo(Capability::IsamFamily(), $releaseNum)),
				       EC_RELEASE_TOO_LOW,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      } else {
	FamilyCommon::IncrErrorCount("  ERROR: unsupported MIB version ($mibVersion)\n",
				     EC_INV_MIB_VERSION,
				     "($mibVersion)",
				     1);
	next;
      }
    }

    ToolCommon::Log($nodeLogFh, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
				   EC_GET_ACC_BITS_FAILED,
				   undef,
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
      # Check whether this IP address is found in the know hosts file(s).
      if (! ToolCommon::SshPossible($isam->[0], $params->{DEBUG}, $config)) {
	FamilyCommon::IncrErrorCount(sprintf("No SSH access possible by PBMT: %s", ToolCommon::GetError()),
				     EC_NO_SSH_POSS,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	next;
      } else {
	$params->{USESSH} = 1;
	ToolCommon::Log($nodeLogFh, "No CLI access via Telnet possible, forced usage of SSH\n", 1);
      }
    }

    #-- Open CLI Session
    $cli = ToolCommon::OpenCli($isam, $nodeLogFh, $params->{TELNETTIMEOUT});
    if (!defined($cli)) {
      FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for $isam->[1]: %s\n", IsamCli::GetError()),
				   EC_OPENCLI_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    }
    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount

    my $prev = $cli->max_buffer_length($params->{BUFSIZE});
    ToolCommon::Log($nodeLogFh, sprintf("Increased input buffer length from %d to %d bytes\n", $prev, $params->{BUFSIZE}));

    if (!IsamCli::SetSessionTimeout($cli, 120)) {
      FamilyCommon::IncrErrorCount(sprintf("  Set session timeout FAILED: %s\n", IsamCli::GetError()),
				   EC_SET_TIMEOUT_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      next;
    } else {
      ToolCommon::Log($nodeLogFh, "  OK\n");
    }

    # Here we go....
    @configRetrieved = ();
    ToolCommon::Log($nodeLogFh, "Check VLANs $params->{NWVLANIDPPP} and $params->{NWVLANIDIP}...");
    my @response = IsamCli::SendCommand($cli, "info configure vlan id $params->{NWVLANIDPPP} flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") != -1) {
	  FamilyCommon::IncrErrorCount("  FAILED: VLAN (PPP) does not exist\n",
				       EC_NO_SUCH_PPPVLAN,
				       "($params->{NWVLANIDPPP})",
				       1);
	  next;
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_VLAN_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	@response = IsamCli::SendCommand($cli, "info configure vlan id $params->{NWVLANIDIP} flat");
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    if (index(IsamCli::GetError(), "does not exist") != -1) {
	      FamilyCommon::IncrErrorCount("  FAILED: VLAN (IP) does not exist\n",
					   EC_NO_SUCH_IPVLAN,
					   "($params->{NWVLANIDIP})",
					   1);
	      next;
	    } else {
	      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					   EC_INFO_CONF_VLAN_ERROR,
					   undef,
					   1);
	      next;
	    }
	  } else {
	    ToolCommon::Log($nodeLogFh, "  OK\n");
	  }
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				       EC_EMPTY_RESPONSE,
				       undef,
				       1);
	  next;
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    ToolCommon::Log($nodeLogFh, "Backup NT Database...");
    if (! BackupDatabase($nodeLogFh, $cli, $isam->[1], $params->{DEBUG}, FamilyCommon::GetParentLogFile(), $newLog, $count)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_BACKUPDB_FAILED,
				   undef,
				   1);
      next;
    }

    ToolCommon::Log($nodeLogFh, "Retrieve auto-detect PPP ATM PVCs...");
    @response = IsamCli::SendCommand($cli, "info configure atm pvc flat | match exact:\"automatic:ppp\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_PVCCONFIG_ERROR,
				     undef,
				     1);
	next;
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	%pvcConfig = ();
	ToolCommon::Log($nodeLogFh, "Parsing ATM PVC configuration...");
	if ( !ParsePvcConfig($nodeLogFh, \@response, \@configRetrieved, \%pvcConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_PVCCONFIG_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }
    ToolCommon::Log($nodeLogFh, "Retrieve ATM PVC customer IDs...");
    @response = IsamCli::SendCommand($cli, "info configure interface atm-pvc flat");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				     EC_INFO_CONF_INTERFACE_ERROR,
				     undef,
				     1);
	next;
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	ToolCommon::Log($nodeLogFh, "Parsing ATM PVC interface configuration...");
	if ( !ParsePvcItfConfig($nodeLogFh, \@response, \@configRetrieved, \%pvcConfig, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_ITFCONFIG_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }
    
    ToolCommon::Log($nodeLogFh, "Retrieve PPPoX-relay CC client-port info...");
    @response = IsamCli::SendCommand($cli,
				     "info configure pppox-relay cross-connect client-port flat | match exact:\"vlan-id $params->{NWVLANIDPPP}\"");
    if (defined($response[0])) {
      if (IsamCli::CommandError(join("", @response))) {
	if (index(IsamCli::GetError(), "does not exist") != -1) {
	  FamilyCommon::IncrErrorCount("  FAILED: does not exist\n",
				       EC_NO_SUCH_CLIENTPORT,
				       "($params->{NWVLANIDPPP})",
				       1);
	  next;
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_INFO_CONF_CLIENTPORT_ERROR,
				       undef,
				       1);
	  next;
	}
      } else {
	ToolCommon::Log($nodeLogFh, "  OK\n");
	%pppCcClientPorts = ();
	ToolCommon::Log($nodeLogFh, "Parsing client port configuration...");
	if ( !ParseClientPortConfig($nodeLogFh, \@response, \@configRetrieved, \%pppCcClientPorts, $releaseNum)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_CLIENTPORT_PARSE_ERROR,
				       undef,
				       1);
	  next;
	} else {
	  ToolCommon::Log($nodeLogFh, "  OK\n");
	}
      }
    } else {
      FamilyCommon::IncrErrorCount(sprintf("  0 lines returned - command ended prematurely: '%s'", IsamCli::GetError()),
				   EC_EMPTY_RESPONSE,
				   undef,
				   1);
      next;
    }

    if ($params->{CONFINFO}) {
      ToolCommon::Log($nodeLogFh, "====================================================================\n");
      ToolCommon::Log($nodeLogFh, "The following configuration info has been read, parsed and stored...\n");
      ToolCommon::Log($nodeLogFh, "====================================================================\n");
      foreach (@configRetrieved) {
	ToolCommon::Log($nodeLogFh, "  $_\n");
      }
    }
    @configToDelete = ();
    %rollBackCommandConfig = ();
    PrepareDeletion($nodeLogFh, \%pvcConfig, \%pppCcClientPorts, \@configToDelete, \%rollBackCommandConfig, $releaseNum);
    PrepareCreation($nodeLogFh, \%pvcConfig, \%pppCcClientPorts, \@configToCreate, \%rollBackCommandConfig, $releaseNum);
    ToolCommon::Log($nodeLogFh, "==============================\n");
    ToolCommon::Log($nodeLogFh, "The following will be deleted:\n");
    ToolCommon::Log($nodeLogFh, "==============================\n");
    foreach (@configToDelete) {
      ToolCommon::Log($nodeLogFh, "  $_\n");
    }
    ToolCommon::Log($nodeLogFh, "==============================\n");
    ToolCommon::Log($nodeLogFh, "The following will be created:\n");
    ToolCommon::Log($nodeLogFh, "==============================\n");
    foreach (@configToCreate) {
      ToolCommon::Log($nodeLogFh, "  $_\n");
    }
    my $deletionOk = 1;
    my $creationOk = 1;
    @rollBackCommands = ();

    # Now start the reconfiguration... first acquire a transaction
    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership...\n");
    return undef if( ToolCommon::AcquireTransaction() );
    Logger::LogTrace($nodeLogFh, "Acquire config-lock ownership... DONE\n");

    # Now start the reconfiguration... first delete what has to be deleted.
    ToolCommon::Log($nodeLogFh, "Start deletion of resources...") if (! $params->{NOACTION});
    $reconfigStart = time();
    foreach my $cliCmd (@configToDelete) {
      ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd)) if $params->{DEBUG};
      if (! $params->{NOACTION}) {
	$cliCommandsSent++;
	@response = IsamCli::SendCommand($cli, $cliCmd);
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    $deletionOk = 0;
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_DEL_CONFIG_ERROR,
					 undef,
					 undef);
	    last;
	  } else {
	    if (defined($rollBackCommandConfig{$cliCmd})) {
	      push @rollBackCommands, $cliCmd;
	      ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						  join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	    }
	    #	  sleep 2;
	  }
	} else {
	  $deletionOk = 0;
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_DEL_CONFIG_ERROR,
				       undef,
				       undef);
	  if (IsamCli::ConnectionReset()) {
	    ToolCommon::Log($fh,
			    "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	    sleep 1;
	    CORE::exit(EXIT_WARN_CODE);
	  }
	  last;
	}
      } else {
	if (defined($rollBackCommandConfig{$cliCmd})) {
	  push @rollBackCommands, $cliCmd;
	  ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd'\ -> '%s'n",
					      join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	}
      }
    }
    if (! $params->{NOACTION}) {
      if (! $deletionOk) {
	if (!RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				       EC_ROLLBACK_ERROR,
				       undef,
				       1);
	}
	next;
      }
    }
    ToolCommon::Log($nodeLogFh, "  OK\n") if (! $params->{NOACTION});
    sleep 10;

    ToolCommon::Log($nodeLogFh, "Start creation of resources...") if (! $params->{NOACTION});
    foreach my $cliCmd (@configToCreate) {
      ToolCommon::Log($nodeLogFh, sprintf("  %s '%s'\n", $params->{NOACTION} ? "Simulating": "Sending", $cliCmd)) if $params->{DEBUG};
      if (! $params->{NOACTION}) {
	$cliCommandsSent++;
	@response = IsamCli::SendCommand($cli, $cliCmd);
	if (defined($response[0])) {
	  if (IsamCli::CommandError(join("", @response))) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
					 EC_CREATE_CONFIG_ERROR,
					 undef,
					 undef);
	    $creationOk = 0;
	    last;
	  } else {
	    if (defined($rollBackCommandConfig{$cliCmd})) {
	      push @rollBackCommands, $cliCmd;
	      ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
						  join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	    }
	    #sleep 2;
	  }
	} else {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				       EC_CREATE_CONFIG_ERROR,
				       undef,
				       undef);
	  $creationOk = 0;
	  if (IsamCli::ConnectionReset()) {
	    ToolCommon::Log($fh,
			    "Child $count: Restore backup as communication to NT was lost and configuration is left in undefined state!!!\n", 1);
	    sleep 1;
	    CORE::exit(EXIT_WARN_CODE);
	  }
	  last;
	}
      } else {
	if (defined($rollBackCommandConfig{$cliCmd})) {
	  push @rollBackCommands, $cliCmd;
	  ToolCommon::Log($nodeLogFh, sprintf("Stored for rollback: '$cliCmd' -> '%s'\n",
					      join(" | ", @{$rollBackCommandConfig{$cliCmd}})))  if $params->{DEBUG};
	}
      }
    }
    if (! $creationOk) {
      if (! RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				     EC_ROLLBACK_ERROR,
				     undef,
				     1);
      }
      next;
    }
    ToolCommon::Log($nodeLogFh, "  OK\n") if (! $params->{NOACTION});
    ToolCommon::Log($nodeLogFh, sprintf("%d CLI commands sent in %d seconds to accomplish this reconfiguration\n",
					$cliCommandsSent, time() - $reconfigStart));

    # We're done, see what's next
    if ($params->{ROLLBACK}) {
      if (! RollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				     EC_ROLLBACK_ERROR,
				     undef,
				     undef);
      }
    } else {
      DumpRollBack($nodeLogFh, $cli, \@rollBackCommands, \%rollBackCommandConfig)  if $params->{DEBUG};
    }

    # Release transaction
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFh, "Release configuration-lock ownership... DONE");

    IsamCli::Disconnect($cli);
    $session->close();
    if ($params->{SEPLOG}) {
      close $nodeLogFh;
      FamilyCommon::ClearChildLogFile();
    }
  
    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    ToolCommon::Log($fh, sprintf("Child $count: %4d-%02d-%02d %02d:%02d:%02d: Finished $isam->[1]... in %d seconds\n",
				 $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second, $endTime - $startTime), 1);
    $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
    if (defined($nodeStatusLog)) {
      print $nodeStatusLog
	sprintf("%s\n",
		FamilyCommon::GetErrorCount() ? ErrorCodes::ExitString(EXIT_WARN_CODE) : ErrorCodes::ExitString(SUCCESS_CODE));
      close $nodeStatusLog;
      FamilyCommon::ClearNodeStatusLog();
    }
  }
  if (FamilyCommon::GetErrorCount()) {
    ToolCommon::Log($fh, sprintf("Child %d: %d error(s) occured for the following ISAMs, please verify %s\n",
				 $count, FamilyCommon::GetErrorCount(),
				 $params->{SEPLOG} ? $nodeLogFile : "$params->{LOGFILE}_$count."), 1);
    my $failedNodes = FamilyCommon::GetFailedNodeList();
    foreach my $node (sort keys %$failedNodes) {
      ToolCommon::Log($fh, "  $node\n", 1);
    }
  }
  if (! $params->{SEPLOG}) {
    close $nodeLogFh;
    FamilyCommon::ClearChildLogFile();
  }
  ToolCommon::Log($fh, "Child $count: finished...\n", 1);
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}

#------------------------------------------------------------------------------
#-- Adjust Tool defaults if needed (i.e. when defined in the parameter config
#-- file
#------------------------------------------------------------------------------
sub AdjustToolVars {
  my $crypt = Crypt::Lite->new('debug' => 0);

  $HOSTSFILE = $config{'HOSTSFILE'} if defined ($config{'HOSTSFILE'});
  $TFTPDIR = $config{'TFTPDIR'} if defined($config{'TFTPDIR'});
  $TFTPWORKDIR = $config{'TFTPWORKDIR'} if defined($config{'TFTPWORKDIR'});
  $MAXSESSIONS = $config{'MAXTFTPSESSION'} if defined($config{'MAXTFTPSESSION'});
  $USERNAME = $crypt->decrypt($config{'DEFAULTCLIUSER'}, CYPHER) if (defined($config{'DEFAULTCLIUSER'}) &&
								     ($config{'DEFAULTCLIUSER'} ne ""));
  $PASSWORD = $crypt->decrypt($config{'DEFAULTCLIPWD'}, CYPHER) if (defined($config{'DEFAULTCLIPWD'}) &&
								    ($config{'DEFAULTCLIPWD'} ne ""));
  $NT_COMMUNITY = $crypt->decrypt($config{NT_COMMUNITY}, CYPHER) if (defined($config{NT_COMMUNITY}) &&
								     ($config{NT_COMMUNITY} ne ""));
  $NODEPREFIX = $config{'NODEPREFIX'} if defined($config{'NODEPREFIX'});
  $TELNETTIMEOUT = $config{'TELNETTIMEOUT'} if defined($config{TELNETTIMEOUT});
  $STATUSFILE = $config{CCCE_STATUS_FILE} if defined($config{CCCE_STATUS_FILE});
  $RETRIES = $config{RETRIES} if defined($config{RETRIES});
  $NO_PING = uc($config{'NO_PING'}) eq "N" ? 0 : 1 if defined($config{'NO_PING'});
}

#==============================================================================
#== Main program
#==============================================================================
my $count = 0;
my @pids;
my $pid;
my $warnCount = 0;

sub CleanUp {
  if (scalar @pids) {
    print STDERR sprintf("%s::%s\n", PBMT_VERSION, $VERSION);
    print STDERR "\nCaught Interupt (^C), aborting children...";
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	kill($pid);
      }
    }
  }
  exit(EXIT_SIGNAL_CODE);
}

$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }

  if (!($msgStr =~ /Zlib/)) {
    print $warnLogFile "Warning: $msgStr" if (defined($warnLogFile));
    print STDERR "Warning: $msgStr";
    $warnCount++;
  }
};

#-- Read parameter config file and adjust tool defaults if needed 
%config = Utils::ReadConfig('param.cfg');
AdjustToolVars();

#-- Handle command line
$params = ParseCommandLine();

STDOUT->autoflush(1);
my $logFile;
if (!open($logFile, ">$params->{LOGFILE}")) {
  print STDERR "Cannot open $params->{LOGFILE} for writing: $!\n";
  exit CREATE_FILE_FAIL_CODE;
}
$logFile->autoflush(1);
FamilyCommon::SetWarnLogFile($logFile);

#-- Print summary of options
PrintOptionsSummary($logFile, 1);

Utils::SetDebug($logFile) if ($params->{DEBUG});  # -- 1.2.2

#-- Create work dir
my $tmpDir = "$TFTPDIR/$TFTPWORKDIR";
if (! -d $tmpDir) {
  if (!mkdir( $tmpDir, 0777)) {
    ToolCommon::Log($logFile, "Cannot create directory $tmpDir: $!\n", 1);
    exit CREATE_DIR_FAILED_CODE;
  }
}
if (! chmod(01777, "$tmpDir")) {
    ToolCommon::Log($logFile,  "Failed to chmod 1777 for directory '$tmpDir'\n",1);
}

#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  my @addr = gethostbyname($params->{ISAM});
  my @ip = unpack( "C4", $addr[4]);
  my $ipAddr = join( '.', @ip );
  $isamsRef = [ [$ipAddr, $params->{ISAM}] ];
} else {
  ToolCommon::Log($logFile, "Reading Hostfile...\n", 1);
  $isamsRef = ToolCommon::ReadHostsFile($logFile, $params->{DEBUG}, $params->{HOSTS}, $NODEPREFIX);
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

while ($count < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $count) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      ToolCommon::Log($logFile, "Child $count: started as $pid\n", 1);
      push(@pids, $pid);
    } else {
      HandleIsams($logFile, \@isamPart, $count, \%config);
      exit SUCCESS_CODE;
    }
  }
  $count++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    $exitCode = $? >> 8;
    $signal = $? & 255;
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      ToolCommon::Log($logFile, sprintf("Child %d: exit code: '%d' (%s), signal: '%d'\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal), 1);
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  ToolCommon::Log($logFile, "All childs finished, exiting\n", 1);
  close $logFile;
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
