#!/usr/bin/perl

=head1 ISAM PRE-CHECKS

=begin html
This script will perform a number of pre-checks to determine whether migration/SW upgrade will not encounter problems that need to be fixed up-front.
This script will typically be called from a higher layer application to build a list of candidate nodes.

The operations of the script are influenced by:
<ul>
  <li>command line options</li>
  <li>a parameter configuration file</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         preCheck.pl
#-- Date:         25.10.2006
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to perform a series of stand-alone PBMT pre-checks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2006/10/25 : First implementation
#--   1.0.1 - 2007/03/29 : ANTms40202
#--   1.0.2 - 2007/06/18 : ANTms51632 allow multiple users for same ISAM.
#--   1.0.3 - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.0.4 - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.0.5 - 2007/08/22 : ANTms60869: Make NT community string configurable
#--   1.0.6 - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.0.7 - 2007/10/19 : ANTms68417: Additional precheck on maximum number of packages in a
#--                                    multicast source.  From R3.4 onwards this can only be
#--                                    maximum 20 (or all)
#--   1.0.8 - 2007/10/23 : ANTms69045: an empty IGMP multicast source table generates an SNMP
#--                                    error when walked (the error is not an "error").
#--   1.0.9 - 2007/12/18 : ANTms77027. NT OFLMT 3.4.01 converts 64 packages as member into
#--                                    1024 packages on the condition that the highest package
#--                                    member is 64.  As such the check in PBMT is adapted to
#--                                    accept this too.
#--   1.1.0 - 2007/12/18 : ISAM-V
#--   1.1.1 - 2008/04/29 : ANTmt04599.  NANT-A TYPE-B SW name changed between R3.2 and R3.3
#--                                     Implement logic for SW name retrieval in case of
#--                                     'reduced' SW
#--   1.1.2 - 2008/05/06 : ANTmt05829.  Check ip address immediately after parsing the OSWP file
#--                                     in ParseCommandLine.
#--   1.1.3 - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.1.4 - 2008/07/25 : Add check on file transfer protocol being used in the ISAM.
#--           2008/08/20 : ANTmt16940.  Check on static unicast MAC@ and verify that the amount of
#--                                     max mac addresses per line on a LT does not exceed 1024 (per LT).
#--   1.1.5 - 2008/10/10 : ANTmt26931.  Include a success code in case of warnings: LRM displays the warnings
#--                                     and expects a pass (success) of fail (error).
#--   1.1.6 - 2008/10/27 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.2.0 - 2008/09/01 : Introduction of IPD ISAM in R3.7.10
#--   1.2.1 - 2009/03/10 : ANTmt18506.
#--   1.2.2 - 2009/03/18 : ANTmtxxxxx.  Increase the maximum message size for SNMP packets.
#--   1.2.3 - 2009/04/30 : ANTmt55717.  Introduced NE famile ISAM_BOTH to indicate NTs that can be part of both the ETSI and ANSI stream.
#--   1.2.4 - 2009/05/19 : ANTmt59034: Change $config{USESSH} to $config{USE_SSH}
#--   1.2.5 - 2009/05/25 : ANTmt59036.  Check security access settings and force usage of SSH if no telnet access for CLI possible.
#--   1.2.6 - 2009/05/26 : ANTmt60011.
#--   1.2.7 - 2009/06/02 : ANTmt60891.
#--   1.2.8 - 2009/06/02 : ANTmt61239.  Added 1540LS NE Family
#--   1.2.9 - 2009/06/10 : ANTmt62662, allow public-key authentication with SSH
#--   1.2.10 - 2009/08/11: csv file name changed
#--   1.2.11 - 2009/08/25: Update toolData to reflect correct settings for OFLMT for the various ISAM streams.
#--   1.2.12 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.2.13 - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session
#--   3.0.01 - 2010/04/12 : Converged PBMT Changes
#--                         x86 Changes 
#--   3.0.02 - 2010/09/29 : ALU00441703. Provide path to the OFLMT tools as command line parameters
#--   3.0.03 - 2010/10/15 : ALU00808688 - new LRM option --id (suppresses the --logfile option).  Must not be used 
#--                         in combination with --hosts
#--                         ALU00804222 - Update default names for tools
#--   3.0.04 - 2010/10/29 : set ANSI_NTDBTOOLDIR to point to folder holding ANSI Migration tools
#--   3.0.05 - 2010/11/25 : to resolve problem of child log file being written to parent log. 
#--   3.0.06 - 2010/12/14 : ALU00954124 - Revert back to old naming conventions (with .exe extension)  for tools
#--   3.0.07 - 2011/01/07 : LRM updates for lib-perl path
#--   3.0.08 - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.09 - 2011/04/01 : Changes for 7354RU
#--   3.0.10 - 2011/04/05 : Support for R4220/R4230
#--   3.0.11 - 2011/05/02 : ALU01178887
#--   3.0.12 - 2011/07/27 : ALU01348790
#--   3.0.13 - 2011/09/19 : ALU01497319
#--   3.0.14 - 2011/09/22 : ALU01492708
#--   3.0.15 - 2011/09/24 : Support for IVPS solaris x86 tool
#--   3.0.16 - 2011/10/11 : I386 parameters renamed since AMS only supports I386_NTOFLMT type formats
#--   3.0.17 - 2011/10/24 : logfile command line parameter can include a directory
#--   3.0.18 - 2011/12/11 : ALU01643367
#--   3.0.19 - 2012/01/05 : ALU01733408
#--   3.0.20 - 2012/03/26 : ALU01748774
#--   3.0.21 - 2012/05/14 : ALU01789702
#--   3.0.22 - 2012/05/30 : to correct the display of snmp option used in logfile
#--   3.0.23 - 2012/10/22 : ALU01761938
#--   3.0.24 - 2012/11/06 : ALU01837839
#--   3.0.25 - 2013/02/07 : enable usage of second password if first one fails for cli over ssh
#--   3.0.26 - 2013/02/08 : corrected issue related to picking of second password from config file
#--   3.0.27 - 2013/02/15 : undef second password in params when it is set as empty
#--   3.0.28 - 2013/04/29 : ALU01880147
#--   3.0.29 - 2013/05/21 : set linux oflmt path from configuration file
#--   3.0.30 - 2013/12/09 : precheck functionality for 7367
#--   3.0.31 - 2014/02/07 : ofmlt support on Linux in ANSI stream
#--   3.0.32 - 2014/07/06 : ALU01991130 Full support for ANSI VSRM 7367_SX
#--   3.0.33 - 2014/11/30 : ALU02028455
#--   3.0.34 - 2015/02/10 : ALU02045085
#--   3.0.35 - 2015/05/25 : ALU02060588
#--   3.0.36 - 2015/06/10 : ALU02083077
#--   3.0.37 - 2015/08/04 : ALU02104915
#--   3.0.38 - 2015/10/28 : ALU02139820
#--   3.0.39 - 2015/11/02 : ALU02142016
#--   3.0.40 - 2015/12/14 : ALU02156544
#--   3.0.41 - 2015/12/30 : ALU02160793
#--   3.0.42 - 2016/03/16 : ALU02190558
#--   3.0.43 - 2016/05/23 : ALU02216697
#--   3.0.44 - 2016/07/15 : ALU02237966
#--   3.0.45 - 2016/09/28 : ALU02278259
#--   3.1.0 -- 2018/06/20 : ALU02285181
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697


=head1 VERSION

=begin html
Current version: "preCheck.pl - 3.1.0 -- 2018/06/20";

=end html

=cut

my $VERSION = "preCheck.pl - 3.1.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2209-FDZZA">PreCheckIsam7354.pm</li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2200-FDZZA">PbmtConfig.pm</li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 
#==============================================================================
use Net::SNMP;
use Net::TFTP;
use Logger qw(:ALL);
use IsamCli;
use FamilyCommon;
use PreCheckIsam;
use PreCheckIsam1540;
use PreCheckIsam7354;
use Crypt::Lite;
use Utils;
use Octopus;
use Capability;
use CapabilityReEngr;
use PbmtConfig;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :PRCK);
use SNMPComm;
use version qw(:ALL);

#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$);
sub CheckIsams($$$$);
sub AdjustToolVars($);
sub CheckEnvironment($$$$);
sub NtOflmtVersionString($);
sub HandleSigPipe($);
sub CleanUp($;$);

#==============================================================================
#== Constants
#==============================================================================
my $config;
my %toolData = (
		"SHUBSWCHECK" => "shubSwCheck.sh",
		"HOSTSFILE" => "/etc/hosts",
		"TFTPDIR" => "/var/opt/aws/equipment",
		"SHUBSWPATH" => "",
		"SHUBSWNAME" => "",
		"TFTPWORKDIR" => "tmp",
		"MAXSESSIONS" => 5,
		"USERNAME" => "",
		"PASSWORD" => "",
		"PASSWORD2" => "",
		"RETRIES" => 5,
		"PINGWAIT" => 10,
		"OCTOPUSPATH" => "tools/octopus",
		"OCTOPUSSCRIPTS" => "tools/octopus",
		"NODEPREFIX" => "AS-",
		"TFTPTIMEOUT" => 5,
		"TELNETTIMEOUT" => 10,
		"EMERG_ALRM_POLL_TIME" => 120,
		"WARNINGFILE" => "PRCKwarnings.txt",
		"NT_COMMUNITY" => "public",
		"NTDBTOOLDIR" => "tools/NT",
		"NTDBTOOL" => "tools/NT/MJSH.exe",
		'NTDBTOOL_I386' => "tools/NT/MJSH_i386",         # oflmt tool for Solaris x86 platform
		'NTDBTOOL_LINUX' => "tools/NT/MJSH_l.exe",       # oflmt tool for Linux platform
		'ANSI_NTDBTOOL_LINUX' => "tools/NT_ANSI/MJSH_l.exe",       # ANSI oflmt tool for Linux platform
		"LANXDBTOOLDIR" => "tools/Lanx",
		"LANXDBTOOL" => "Migrationtool.pl",
		"IVPSDBTOOLDIR" => "tools/IVPS",
		"IVPSDBTOOL" => "tools/IVPS/NBTW.exe",
		"IVPSDBTOOL_I386" => "tools/IVPS/NBTW_i386",
		"IVPSDBTOOL_LINUX" => "tools/IVPS/NBTW_l.exe",
		"IVPSINVENTORY" => "/opt/aws/isamv/voipim/script/getIVPSBoardInNE.pl",
		"LANXIPCKSUM" => "CalcIpCheckSum_BE",
		"NEWCDEFILE" => "",
		"CDEDIR" => "",
		"ECNTA_ICSDATA" => {});

 if ($Config{archname} =~ /i86pc/) {
    $toolData{LANXIPCKSUM} = "CalcIpCheckSum_LE";
 } elsif ($Config{archname} =~ /linux/) {
    $toolData{LANXIPCKSUM} = "CalcIpCheckSum_LE_linux";
 }

my $STATUSFILE = "result.txt";
my $ID_FILENAME = "PRCK";
$nodeLogFH = undef;
$execLogFH = undef;
my $logExecOrNode = 0;

#==============================================================================
#== Vars
#==============================================================================

=head1 USAGE

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] preCheck.pl --isam &lt;ip&gt; | --hosts &lt;file&gt; --swp &lt;path-to-oswp-descriptor&gt; --logfile &lt;file&gt; \ <br>
                    [--sessions &lt;nr&gt;] [--seplog | --noseplog] [--multi] [--noname] [--comm &lt;SNMP community string&gt;] [--v2] \ <br>
                    [--cliusername &lt;user&gt;] [--clipassword &lt;password&gt;] [--sftpusername &lt;sftpusername&gt;] [--v] [--debug] [--noping] \ <br>
                    [--v3Auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;] [-v3priv &lt;privProtocol&gt;:&lt;password&gt;] </tt>

=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>swp</td><td>path to oswp descriptor file (relative to TFTP root directory)</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>cliusername</td><td>ISAM CLI username</td></tr>
  <tr><td>clipassword</td><td>ISAM CLI password</td></tr>
  <tr><td>sftpusername</td><td>SFTP username. Required only for SFTP protocol</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td></tr>
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed.</td></tr>
  <tr><td valign="top">v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                             <li>All fields are mandatory when option is used.</li>
                         <ul<br>
                         e.g. --v3auth user:password:: will result in DM5 as protocol.</td></tr>
  <tr><td valign="top">v3priv</td><td>optional setting when using SNMPv3 privacy protocol providing protocol and password<br>
                         <ul><li>Privacy protocol is DES, 3DES or AES, DES is default protocol.</li>
                         </ul></td></tr>
</table>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file>  --logfile <file> | -id <id> --swp <path-to-oswp-descriptor>\\ \n";
  print STDERR "       [--sessions <nr>] [--seplog | --noseplog] [--multi] [--noname] [--comm <SNMP community string>] [--v2] \\ \n";
  print STDERR "       [--cliusername <user>] [--clipassword <password>] [--clipassword2 <password>] [--sftpusername <username>] \\ \n";
  print STDERR "       [--ntdbtool <path of the NT DB Offline Migration Tool>] \\ \n";
  print STDERR "       [--lanxtool <path of the Lanx Migration Tool>] \\ \n";
  print STDERR "       [--ivpstool <path of the IVPS Migration Tool>] \\ \n";
  print STDERR "       [--v] [--debug] [--noping] [--v3auth <user>:<password>:<authProtocol>] \\ \n";
  print STDERR "       [-v3priv <privProtocol>:<password>] [--bytpe  <value>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         id       : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                    The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         v             : print version and exit\n";
  print STDERR "         swp           : path to oswp descriptor file \n";
  print STDERR "                         (relative to TFTP root - $toolData{TFTPDIR})\n";
  print STDERR "         sessions      : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         cliusername   : ISAM CLI username\n";
  print STDERR "         clipassword   : ISAM CLI password\n";
  print STDERR "         clipassword2  : Second ISAM CLI password (option to use a second password if first password fails). \n";
  print STDERR "                         The second password option will be used only when CLI over SSH is configured. \n";
  print STDERR "         sftpusername  : SFTP client username. Mandatory for SFTP protocol\n";
  print STDERR "         ntdbtool      : NT DB Offline migration tool path\n";
  print STDERR "         lanxtool      : Lanx Offline migration tool path\n";
  print STDERR "         ivpstool      : IVPS Offline migration tool path\n";
  print STDERR "         seplog        : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noseplog      : keep all NE logging in a single child log file (default is a single logfile per NE)\n";
  print STDERR "         noname        : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         multi         : include Unix username in NE logging directory path\n";
  print STDERR "         noping        : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "         SNMP settings :\n";
  print STDERR "         comm          : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is \"public\"\n";
  print STDERR "         v2            : Use SNMPv2c\n";
  print STDERR "         v3auth        : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                         The password should be at least 8 characters long.\n";
  print STDERR "                         Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                         All fields are mandatory when an option is used.\n";
  print STDERR "                         e.g. --v3auth user:password:: will result in MD5 as protocol.\n";
  print STDERR "         v3priv        : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                         Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  print STDERR "         btype    : Build type to choose XDescriptor; Possible values are SNMP, YANG\n";
  exit(CMD_INCOMPL_CODE);
}




=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $noaction, $swp, $sessions, $debug, $hosts, $logFile, $btype, $cliUserName, $cliPassword,  $cliPassword2,
      $sftpUserName, $version, $sepLog, $makeFile, $multi, $wipe, $noName, 
      $communityString, $noPing, $v3Auth, $v3Priv, $useSsh, $noSepLog, $snmpV2, $ntdbTool, $lanxTool, $ivpsTool, $id);
  my $regExp;
  my %params;
  my $dbVersion;
  my $dirName = getcwd();
  my $options = GetOptions('isam=s', \$isam,
			   'noaction', \$noaction,
			   'swp=s', \$swp,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'logfile=s', \$logFile,
			   'btype=s', \$btype,
			   'username|cliusername=s', \$cliUserName,
			   'password|clipassword=s', \$cliPassword,
                           'clipassword2=s', \$cliPassword2,
			   'sftpusername=s', \$sftpUserName,
                           'ntdbtool=s', \$ntdbTool,
                           'lanxtool=s', \$lanxTool,
                           'ivpstool=s', \$ivpsTool,
			   'v', \$version,
			   'v2', \$snmpV2,
			   'seplog', \$sepLog,
			   'noseplog', \$noSepLog,
			   'make', \$makeFile,
			   'multi', \$multi,
			   'wipe', \$wipe,
			   'noname', \$noName,
			   'comm=s', \$communityString,
			   'noping', \$noPing,
			   'v3auth=s', \$v3Auth,
			   'v3priv=s', \$v3Priv,
			   'ssh', \$useSsh,
                           'id=s', \$id);
  #-- check options
  if (! $options) {
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  if (defined($makeFile)) {
    ErrorCodes::MakeWarningFile("PRCK", dirname($0), $toolData{WARNINGFILE});
    exit(SUCCESS_CODE);
  }
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }

  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }

  if(!defined($swp)){
     print STDERR "ERROR: No OSWP specified via --swp\n"; 
     Usage(); 
  } 

  if( (defined $hosts) and (!defined $btype ) ) {
     print STDERR "ERROR: --btype value must be passed for --hosts\n";
     Usage();
  }

  if( defined($btype) ) {
     if( $btype ne "SNMP" and $btype ne "YANG" ) {
        print STDERR "ERROR: Invalid value passed for --bytpe; possible values are SNMP, YANG\n";
        Usage();
     } else {
        $params{BTYPE} = $btype;
     }
  } elsif( defined $hosts ) {
     $btype = "SNMP";
  }
  $params{NOTFTP} = 0;
  # First try the file as specfied (could be an absolute path when using ftp e.g.
  my $oswp2Consider = $swp;
  if (! -r $oswp2Consider) {
    # Next try the SWP relative to the login home directory
    $oswp2Consider = "$ENV{HOME}/$swp";
    $oswp2Consider =~ s/\/\//\//g;
    if (! -r $oswp2Consider) {
      # Finally check as tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $oswp2Consider =~ s/\/\//\//g;
      if (! -r  $oswp2Consider) {
	print STDERR "ERROR: Accessing OSWP '$oswp2Consider' failed: $!\n";
	Usage();
      } else {
	$params{OSWP_HOME} = $toolData{TFTPDIR};
      }
    } else {
      $params{OSWP_HOME} = $ENV{HOME};
      $params{NOTFTP} = 1;
    }
  } else {
    if (index($toolData{TFTPDIR}, $dirName) != -1) {
      # PBMT installed relative to TFTP directory so still tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $params{OSWP_HOME} = $toolData{TFTPDIR};
    } else {
      $params{OSWP_HOME} = "";
      $params{NOTFTP} = 1;
    }
  }

  if ($params{NOTFTP}) {
    if (defined($sftpUserName)) {
      $params{SFTPUSERNAME} = $sftpUserName;
    } else {
      $params{SFTPUSERNAME} = $toolData{SFTPUSERNAME};
    }

    if (!defined ($params{SFTPUSERNAME})) {
      print STDERR "ERROR: SFTP client user name must specified SFTP protocol \n";
      Usage();   
    }
    if ($params{SFTPUSERNAME} eq "") {
      print STDERR "Error : Invalid value specified for SFTP user name:'$params{SFTPUSERNAME}' \n";
      Usage();
    }
  }

  $params{OSWP_FULL} = $oswp2Consider;
  $regExp = SWPSYNTAX;
  if ($swp =~ /$regExp/) {
    print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_INV_CHARS_CODE));
    exit OSWP_FILE_INV_CHARS_CODE;
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
	"($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }

  #-- Support for OFLMT tool on x86-Solaris and Linux platforms

  if ($Config{archname} =~ /i86pc/) {
    $toolData{NTDBTOOL} = $toolData{NTDBTOOL_I386}; 
    $toolData{ANSI_NTDBTOOL} = $toolData{ANSI_NTDBTOOL_I386}; 
  } elsif ($Config{archname} =~ /linux/) {
    $toolData{NTDBTOOL} = $toolData{NTDBTOOL_LINUX};
    $toolData{ANSI_NTDBTOOL} = $toolData{ANSI_NTDBTOOL_LINUX}; 
  }

  if ($Config{archname} =~ /i86pc/) {
    $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOL_I386}; 
  } elsif ($Config{archname} =~ /linux/) {
    $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOL_LINUX};
  }


 if(defined($ntdbTool)){
      if (-d $ntdbTool) {

        my $relDir = File::Spec->abs2rel($ntdbTool, $scriptDir);
        # ETSI tool path
        $toolData{NTDBTOOLDIR} = $ntdbTool;
        $toolData{NTDBTOOL} = $relDir . "/" . $config->{'NTOFLMT'} if defined($config->{'NTOFLMT'});
        $toolData{NTDBCFG}  = $relDir . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});

        # ANSI tool paths
        $toolData{ANSI_NTDBTOOLDIR} = $ntdbTool;
        $toolData{ANSI_NTDBTOOL} = $relDir . "/" . $config->{'ANSI_NTOFLMT'} if defined($config->{'ANSI_NTOFLMT'});
        $toolData{ANSI_NTDBCFG}  = $relDir . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});

        #print STDERR  "NTDBTool path given as $ntdbTool\n";

        if ($Config{archname} =~ /i86pc/) {
           $toolData{NTDBTOOL} = $relDir . "/" . $config->{'I386_NTOFLMT'} if defined($config->{'I386_NTOFLMT'});
           $toolData{ANSI_NTDBTOOL} = $relDir . "/" . $config->{'I386_ANSI_NTOFLMT'} if defined($config->{'I386_ANSI_NTOFLMT'});
        } elsif ($Config{archname} =~ /linux/) {
           $toolData{NTDBTOOL} = $relDir . "/" . $config->{'LINUX_NTOFLMT'} if defined($config->{'LINUX_NTOFLMT'});
           $toolData{ANSI_NTDBTOOL} = $relDir . "/" . $config->{'LINUX_ANSI_NTOFLMT'} if defined($config->{'LINUX_ANSI_NTOFLMT'});
        }

      } else {
          print STDERR "ERROR: NT Offline Migration toolpath $ntdbTool does not exist\n";
          Usage();
      }
  }

  my $toolDirectory = "";
  if(defined($lanxTool)){
   if (-d $lanxTool) {

     $toolDirectory = File::Spec->abs2rel($lanxTool, $dirName);
     $toolData{LANXDBTOOLDIR} = $toolDirectory;
     $toolData{SHUBDBCFG} = "$toolDirectory/migration.dat";

     $toolData{ANSI_LANXDBTOOLDIR} = $toolDirectory;

     #print STDERR  "LanxTool path given as $lanxTool\n";
   } else {
       print STDERR "ERROR: Lanx Offline Migration toolpath $lanxTool does not exist\n";
       Usage();
   }
  }

  if(defined($ivpsTool)){
   if (-d $ivpsTool) {

     $toolDirectory = File::Spec->abs2rel($ivpsTool, $dirName);
     $toolData{IVPSDBTOOLDIR} = $ivpsTool;
     $toolData{IVPSDBTOOL} = $toolDirectory . "/" . $config->{'IVPSOFLMT'} if defined($config->{'IVPSOFLMT'});
     $toolData{IVPSDBCFG}  = $toolDirectory . "/" . $config->{'IVPSDBCFG'} if defined($config->{'IVPSDBCFG'});
     #print STDERR  "IVPS toolpath given as $ivpsTool\n";

     if ($Config{archname} =~ /i86pc/) {
        $toolData{IVPSDBTOOL} = $toolDirectory . "/" . $config->{'I386_IVPSOFLMT'} if defined($config->{'I386_IVPSOFLMT'});
     } elsif ($Config{archname} =~ /linux/) {
        $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOL_LINUX};
     }

   } else {
       print STDERR "ERROR: IVPS Offline Migration toolpath $ivpsTool does not exist\n";
       Usage();
   }
  }

  if (defined($cliUserName)) {
    $params{USERNAME} = $cliUserName;
  } else {
    $params{USERNAME} = $toolData{USERNAME};
  }
  if (defined($cliPassword)) {
    $params{PASSWORD} = $cliPassword;
  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }
  if (defined($cliPassword2)) {
    $params{PASSWORD2} = $cliPassword2;
  } else {
    $params{PASSWORD2} = ($toolData{PASSWORD2} ne "") ? $toolData{PASSWORD2} : undef ;
  }
  
#ALU01524313-indhu-15/12/2011
  if (defined($communityString) && defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  if (defined($communityString) && defined($v3Priv)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }
#ALU01524313-indhu-15/12/2011
  
  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if (defined($isam));
  $params{NOACTION} = 1 if (defined($noaction));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if (defined($debug));
  LogSetDebug() if(defined $debug);
  $params{OSWP} = $swp if (defined($swp));
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
  } else {
    $params{LOGFILE} = $logFile;
    if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
      my $logdir = $1;
      if (! -d $logdir) {
        if (!mkdir( $logdir, 0777)) {
          print STDERR "Cannot create directory '$logdir': $!\n";
          exit CREATE_DIR_FAILED_CODE;
        }
      }
    }
  }

  my $tftpInfo = Utils::FindTftpInfo(undef, $oswp2Consider);
  if (!defined($tftpInfo->{PRIMIP}) || ($tftpInfo->{PRIMIP} eq "")) {
    print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_NO_IP_CODE));
    exit OSWP_FILE_NO_IP_CODE;
  }
  my $baseName = basename($tftpInfo->{NAME});
  $regExp = REGEXP;
  if (!($baseName =~ /$regExp/)) {  # -- 1.1.7
    print STDERR sprintf("Software package name (%s, read from %s) not conforming to regexp (%s).  Check param.cfg!!!\n",
			 $baseName, $tftpInfo->{NAME}, REGEXP);
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  $params{PRIMTFTPIP} = $tftpInfo->{PRIMIP};
  $params{SECTFTPIP} = $tftpInfo->{SECIP};
  $params{SWPNAME} = $tftpInfo->{NAME};
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{WIPE} = defined($wipe) ? 1 : 0;
  $params{NONAME} = defined($noName) ? 1 : 0;
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : ""; #ALU02021411

  if (!defined($snmpV2)) { #ALU2028455 #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }
  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA256|SHA512|MD5)$/ ) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
          if ($params{SNMP}{V3}{AUTH}{PROTO} eq "SHA") {
              $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
	} else {
	  print STDERR "SNMPv3 authentication protocol only be SHA or SHA256 or SHA512 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
    $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
    if ($params{SNMP}{V3}{AUTH}{PROTO} eq "SHA") {
        $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {  
     my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    $params{SNMP}{V3}{PRIV}{PASSWORD} = defined($toolData{SNMPV3_PRIV_PASSWORD}) ? $toolData{SNMPV3_PRIV_PASSWORD} : "";    
  } #ALU01524313-indhu-15/12/2011
  }
  if (($params{PASSWORD} eq "") || ($params{USERNAME} eq "")) {
    print STDERR "No username or password defined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }
  $params{NOPING} = 0;
  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'});
  $params{NOPING} = 1 if defined($noPing);

  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";

  $params{UPD_KNOWN_HOST_FILE} = 0;
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});
 
  $params{EMERG_CALL_FEATURE} = 0;
  $params{EMERG_CALL_FEATURE} = uc($config->{'EMERG_CALL_FEATURE'}) eq "Y" ? 1 : 0 if defined($config->{'EMERG_CALL_FEATURE'});

  my $oswpPath = "$params{OSWP_HOME}/$params{SWPNAME}";
  $params{SWPPATH} = $oswpPath;
  $dbVersion = ToolCommon::GetDBVersion($oswpPath);
  if(!defined($dbVersion)) {
      print STDERR "DB Version is not defined in file $oswpPath \n";
      exit FILE_OPEN_FAIL_CODE;
  }
  $params{DBVER_TGTREL} = $dbVersion;

  return \%params;
}

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut

sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw(
                       PBMTGlobal Utils Capability ErrorCodes ToolCommon FamilyCommon SNMPComm IsamCli IsamTL1 Octopus
                       PreCheckIsam PreCheckIsam1540 PreCheckIsam7354 PreCheckIsam 
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( sort @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

=head2 PrintOptionsSummary($$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$) {
  my ($argLogFH, $logOnScreen) = @_;
  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }
  PrintLibVersions($argLogFH, $logMode);
  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "XDescriptor build type" , $params->{BTYPE} ) ) if defined($params->{BTYPE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "OSWP Descriptor file" , $params->{OSWP} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- SWP Name" , $params->{SWPNAME} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- DB Version" , $params->{DBVER_TGTREL} ) ) if defined($params->{DBVER_TGTREL});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "DB & Release version mappping" , "$toolData{NTDBCFG}") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ONT-SW Descriptor file" , $params->{ONT} ) ) if defined($params->{ONT});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Interrupt on emergency call" , 
                $params->{EMERG_CALL_FEATURE} ? "Enabled" : "Disabled" ) ) if defined($params->{EMERG_CALL_FEATURE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "FTP Details :-") );
  if (! $params->{NOTFTP}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "TFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Timeout in seconds" , $toolData{TFTPTIMEOUT}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- TFTP (HOME) Directory" , $toolData{TFTPDIR}) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "SFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "xVPS Migration" , $params->{IVPS_MIG_POSSIBLE} ? "Yes" : "No") );
  if( $params->{IVPS_MIG_POSSIBLE} ) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "CDE File" , "$toolData{TFTPDIR}/$toolData{CDEDIR}/$toolData{NEWCDEFILE}" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Actions to be perform" , $params->{NOACTION} ? "No, Checks only" : "Yes, Action after checks") );
  Logger::LogWish( $logMode, $argLogFH, "\n" );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Current Work Directory" , getcwd() ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Work space Directory" , "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Result space Directory" , "$toolData{RESULTSPACE}" ) );
  Logger::LogWish( $logMode, $argLogFH, "\n" );
}

################################################################################
### Set Debugging for libraries, which required logfile to log the message
### Goal to eliminate this SetDebug and use Global logfile varaiables
################################################################################
sub SetDebugging() {
    # Switch debugging (if any) to an opened filehandle...
    FamilyCommon::SetWarnLogFile($nodeLogFH); 
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
}
################################################################################
### reset the global variables used
################################################################################
sub ResetGlobals() {
    IsamCli::ResetGlobals();
    SNMPComm::ResetGlobals();
    PreCheckIsam1540::ResetGlobals();
    PreCheckIsam::ResetGlobals();
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    $params->{USESSH} = 0; #for each isam the value is reset
}

=head2 CheckIsams($$$$$)

=begin html
The main body of the script defining the complete pre-check process.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by this child process.  Every
      array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
</table>

=end html

=cut

sub CheckIsams($$$$) {
  my ($isams, $sessionCount, $config, $warnLevels) = @_;
  my $isam;
  my $ret;
  my ($time1, $time2);
  my $startTime;
  my $nodeLogFile = undef;
  my $statusLogFh;
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $dirName = dirname($0);
  my $ntType;
  my $errorCount = 0;
  my $warningCount = 0;
  my $dir;
  my $thisTftpDir;
  my $fxrData;
  my $prevErrorCount = 0;

  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetOwnPid($$);
  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetWarnLogFile($execLogFH);   # Make sure warnings and
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetConfigData($config);
  FamilyCommon::SetWarningLevels($warnLevels);

  Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Started as '%d'\n", $sessionCount, FamilyCommon::GetOwnPid()));
  ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );
      PrintOptionsSummary($nodeLogFH, undef);
      SetDebugging();


  }
  my $isamCount = -1;
  foreach $isam (@{$isams}) {
    $isamCount++;
    $startTime = time();
    $prevErrorCount = FamilyCommon::GetErrorCount();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];

    my $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");
    FamilyCommon::SetFailedNode($isamNode); # Store for IncrErrorCount

    ResetGlobals();

    ##### Create ResultSpace directory #####
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );
    ### Create Satus file
    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    FamilyCommon::CreateNodeStatusLog( "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE", "Session $sessionCount: Node ($isamCount - $isamNode )" );

    if ( defined($params->{ID})) {
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        $logExecOrNode = 0;
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif ( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
        PrintOptionsSummary($nodeLogFH, undef);
    }
    # Reset handles
    SetDebugging();
    
    ### Is Isam reachable through Ping command
    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        next if( FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef ) );
    }

    # Check whether an octopus session to this ISAM is open
    next if( Octopus::OctopusActive($params->{DEBUG}, $isam, $config->{GREP}, $config->{PS}) );

    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
        next if( FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError(), undef ) );
    }

    $snmpStatus = SNMPComm::MaximizeMsgSize();
    if( $snmpStatus ) {
        next if( FamilyCommon::ReportStatus( EC_SNMP_MAX_SIZE_FAIL, SNMPComm::GetError(), undef ) );
    }

    my $mibVersion = SNMPComm::GetValue( 'MIB VERSION', MIBVERSIONOID );
    if (!defined($mibVersion)) {
        next if( FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError(), "Failed to obtain MIB Version" ) );
    }


    Logger::LogTrace($nodeLogFH, "Prepare logging directories...\n");
    ($dir, $thisTftpDir, $fxrData) = FamilyCommon::PrepareLogging($session, $nodeLogFH, $userName, $isamNode);
    if (not defined($fxrData)) {
        next if( FamilyCommon::ReportStatus( EC_DETERMINE_LOG_FAILED, FamilyCommon::GetError(), undef ) );
    } else {
      $toolData{FXRDATA} = $fxrData;
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

    if ($fxrData->{PROTOCOL} != TFTP_FXR) {
      Logger::LogTrace($nodeLogFH, "Checking SFTP credentials...\n");
      my $mibSftpClientUsername = SNMPComm::GetValue( 'SFTP USER NAME', SFTPCLIENTUSERNAMEOID );
      if (!defined($mibSftpClientUsername)) {
           next if( FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError(), "Failed to obtain SFTP User name" ) );
      } elsif ($params->{SFTPUSERNAME} ne $mibSftpClientUsername) {
           next if( FamilyCommon::ReportStatus( EC_SNMP_CLIENT_CREDENTIAL_MISMATCH, undef, undef ) );
      }
    }

    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g;
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $nodeLogFH, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};
    FamilyCommon::SetNeStorage($dir);

    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile= sprintf("precheck_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );
        SetDebugging();
    }

    # Determine NE family: retrieve NT-A and NT-B type and status using SNMP.  Only then we continue (GPON does not have full
    # CLI capability...

    $ntType = ToolCommon::DetermineNtType( );
    next if (not defined($ntType)) ;
    if ( not Capability::NtSupported( $ntType ) ) {
	     FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, sprintf("NT type '%s' not supported in the Tool set", $ntType ) );
         next;
    }

    Capability::ResetGlobals();
    my $oswpDynamicVariant = DynamicOSWPVariant($nodeLogFH, $config, $ntType );
    if( $oswpDynamicVariant ) {
         next;
    }

    Logger::LogTrace($nodeLogFH, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
         FamilyCommon::ReportStatus(EC_GET_ACC_BITS_FAILED, ToolCommon::GetError(), undef );
         next;
    } else {
         Logger::LogTrace($nodeLogFH, "  OK\n");
         if (! ToolCommon::CreateTmpKeyFile($dir, $accBits, $params->{DEBUG})) {
	          FamilyCommon::ReportStatus( EC_MAKE_KEYFILE_FAILED, sprintf("Creating temp key file failed: %s", ToolCommon::GetError()), undef );
	          next;
         }
    }

   if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
	   $params->{USESSH} = 1;
	   Logger::LogTrace($nodeLogFH, "No CLI access via Telnet possible, forced usage of SSH\n");
   }

   ToolCommon::GetSourceDetails();
   if( !defined $params->{SOURCE_OSWPNAME} ) {
        FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED,ToolCommon::GetError(), undef );
        next;
   }

    $params->{'BOARD-FAMILY'} = "";
    Capability::LogNeFamily($ntType, $params->{DBVER_TGTREL});
    if ( $params->{'BOARD-FAMILY'} eq "" ) {
      FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, undef, 
                                  sprintf("Board family not found for NT-Type '%s', DB-Version '%s'\n",
                                      $ntType, $params->{DBVER_TGTREL} ) );
      next;
    }
    if( $params->{'BOARD-FAMILY'} eq Capability::IsamLSFamily() ) {
         PreCheckIsam1540::CheckEnvironment($nodeLogFH, $config, $dirName);
         PreCheckIsam1540::Check($nodeLogFH, $isam, $userName);
    } elsif( $params->{'BOARD-FAMILY'} eq Capability::RU7354Family() ) {
         PreCheckIsam7354::Check($nodeLogFH, $isam, $userName, uc($ntType));
    } else {
         PreCheckIsam::CheckEnvironment($nodeLogFH, $config, $dirName);
  	     PreCheckIsam::Check($nodeLogFH, $isam, $userName);
    }

    FamilyCommon::CloseSnmpSession2Close();
    my $endTime = time();
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );
    FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
    $errorCount = FamilyCommon::GetErrorCount();
    $warningCount = FamilyCommon::GetWarningCount();
    if ( $params->{SEPLOG}) {
        LogCloseFile($nodeLogFH);
        FamilyCommon::CloseChildLogFile() 
    }
  } #end of for loop
  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }
  if (FamilyCommon::GetWarningCount()) {
      my $isamNodes = FamilyCommon::GetWarnedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d warning(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }

  if (!defined($params->{ID})) {
    if (! $params->{SEPLOG}) {
        LogCloseFile($nodeLogFH);
        FamilyCommon::CloseChildLogFile() 
    }
  }
  Logger::LogInfo($execLogFH, "Child $sessionCount: finished...\n");
  if (FamilyCommon::GetErrorCount()) {
    CORE::exit(EXIT_CHILD_FATAL_CODE);
# ANTmt26931
#  } elsif ($warning) {
#    CORE::exit($exitCodes{EXIT_WARN}[0]);
# ANTmt26931
  } else {
    CORE::exit(SUCCESS_CODE);
  }
}


sub DynamicOSWPVariant($$$) {
    my ($nodeLogFH, $config, $ntType) = @_;
    if( defined $params->{SWPPATH} and $params->{SWPPATH} ne "" ) {
        if( defined $config->{NT_SWP_MAPFILE} and $config->{NT_SWP_MAPFILE} ne "" ) {
           my $ntDescriptorMapFile = dirname($params->{SWPPATH}) . "/" . $config->{NT_SWP_MAPFILE};
           if( ! ( -f $ntDescriptorMapFile ) ) {
	        Logger::LogNotify($nodeLogFH, "NT SWP Mapping file :$ntDescriptorMapFile: not exists\n");
		return 0;
           }
           Capability::SetVariantCheckSkip(1);
           Logger::LogTrace($nodeLogFH, "NT Descriptor mapping file($ntDescriptorMapFile) process started..\n");
           my( $status, $ntDescriptorFile) = Utils::FetchNTDescriptor($ntDescriptorMapFile, $ntType);
           Logger::LogTrace($nodeLogFH, "Details : $status, $ntType, $ntDescriptorFile\n");
           if( $status != 0 ) {
              FamilyCommon::ReportStatus(EC_NTTYPE_MAPFILE_FAILED, "NTDescriptor Mapfile : $ntDescriptorFile", undef);
	          return 1;
           }
           Logger::LogTrace($nodeLogFH, "XDescriptor build type:$params->{BTYPE}:\n") if defined($params->{BTYPE});
           my $regExp = REGEXP;
           if (!($ntDescriptorFile =~ /$regExp/)) {  # -- 1.0.4
                 FamilyCommon::ReportStatus( PARAM_CFG_NOT_OK_CODE, "SW package name ($ntDescriptorFile) from NT Descriptor Mapfile is not as per regexp (".REGEXP.") from param.cfg", undef );
		         return 1;
           }
           Logger::LogTrace($nodeLogFH, "NT DescriptorFile($ntDescriptorFile) found..\n");
           my( $oswpVariantU,$oswpVariantL, $oswpRelease ) = ("","","");
           my( $ntDescriptorVariantU,$ntDescriptorVariantL, $ntDescriptorRelease ) = ("","","","");
           my( $oswpRegexpMatch, $ntDescriptorRegexpMatch ) = (0,0);
           my $baseOSWP = basename($params->{OSWP});
           my $regexpVariantRelease = VARIANT_RELEASE_REGEXP;
           if( $baseOSWP =~ /$regexpVariantRelease/ ) {
                $oswpVariantU = $1; 
                $oswpVariantL = $2; 
                $oswpRelease = $3;
                $oswpRegexpMatch = 1;
           }
           if( $ntDescriptorFile =~ /$regexpVariantRelease/ ) {
                $ntDescriptorVariantU = $1;
                $ntDescriptorVariantL = $2;
                $ntDescriptorRelease = $3;
                $ntDescriptorRegexpMatch = 1;
           }
           if( $oswpRegexpMatch and $ntDescriptorRegexpMatch ) {
              if( $baseOSWP !~ /(AA|AN|ZA|ZN|$ntDescriptorVariantU$ntDescriptorVariantL|Z$ntDescriptorVariantL)(\d\d\.\d\d\d)$/ ) {
                  FamilyCommon::ReportStatus(EC_NTTYPE_OSWP_FAILED, "ntDescriptor($ntDescriptorFile), OSWP($baseOSWP)Variant Release Details are not same",undef );
	              return 1;
              } else {
	          Logger::LogTrace($nodeLogFH, "OSWP($baseOSWP), ntDescriptor($ntDescriptorFile) match the regex..\n");
              }
           }
           my( $swpVariantU,$swpVariantL, $swpRelease, $swpRegexpMatch, ) = ("","", 0);
           ($ntDescriptorVariantU,$ntDescriptorVariantL, $ntDescriptorRelease,$ntDescriptorRegexpMatch ) = ("","", 0);
           my $baseSWP = basename($params->{SWPNAME});
           if( $baseSWP =~ /$regexpVariantRelease/ ) {
                $swpVariantU = $1;
                $swpVariantL = $2;
                $swpRelease = $3;
                $swpRegexpMatch = 1;
           }
           if( $ntDescriptorFile =~ /$regexpVariantRelease/ ) {
                $ntDescriptorVariantU = $1;
                $ntDescriptorVariantL = $2;
                $ntDescriptorRelease = $3;
                $ntDescriptorRegexpMatch = 1;
           }
           if( $swpRegexpMatch and $ntDescriptorRegexpMatch ) {
               if( $baseSWP =~ /($ntDescriptorVariantU$ntDescriptorVariantL)(\d\d\.\d\d\d)$/ ) {
	             Logger::LogTrace($nodeLogFH, "NT-SWP DescriptorFile($ntDescriptorFile) and SWP basename($baseSWP) are same, No need to update..\n");
               } elsif ( $baseSWP =~ /(Z$ntDescriptorVariantL)$ntDescriptorRelease$/ ) {
	             Logger::LogTrace($nodeLogFH, "NT-SWP DescriptorFile($ntDescriptorFile) and SWP basename($baseSWP) are same as per MINIMIZED SWP, No need to update..\n");
               } elsif( $swpVariantU eq "Z" ) {
                  FamilyCommon::ReportStatus(EC_NTTYPE_SWPNAME_FAILED, "ntDescriptor($ntDescriptorFile), SWP($params->{SWPNAME})Variant Release Details are not same and it is Minimized SWP", undef );
	              return 1;
               } else {
	          Logger::LogTrace($nodeLogFH, "SWPNAME($baseSWP), ntDescriptor($ntDescriptorFile) are not same, Need to generate new file..\n");
                  my $newSWPNAME = "";
                  if( dirname($params->{SWPNAME}) ne "" and dirname($params->{SWPNAME}) ne "." ) {
	            $newSWPNAME = dirname($params->{SWPNAME}) . "/";
                  }
                  $newSWPNAME .=  $ntDescriptorFile;
                  my $newSWPFile = $newSWPNAME;
                  if( $params->{NOTFTP} == 0 ) {
 			$newSWPFile = $toolData{TFTPDIR} . "/" . $newSWPNAME;
                  }
                  if( ! -e $newSWPFile ) {
                       FamilyCommon::ReportStatus(EC_NTTYPE_SWPNAME_FAILED, "NT-SWP Map, SWP file ($newSWPFile) not exists", undef );
	                   return 1;
		          }
                  if( defined $params->{OSWP_FULL} and $params->{OSWP_FULL} ne "" ) {
                     Logger::LogTrace($nodeLogFH, "Updated new OSWP NT:$ntType:, based:$newSWPNAME:\n");
                     #$params->{OSWP} = $newOSWP;
                     $params->{SWPNAME} = $newSWPNAME;
                     Logger::LogTrace($nodeLogFH, "NT-SWP Map : SWP name updated as :$params->{SWPNAME}:\n", 1 ); 
                     return 0;
                  }
             
              }
          } else {
             ### DEVEL-NOTE : This check is not required, For safe guard we log this message
             Logger::LogTrace($nodeLogFH, "NT DescriptorFile internal regular expression failed, So not going to perform NT-SWP checks...");
 	     Capability::SetVariantCheckSkip(0);
          }

        }
    }
    return 0;
}

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file).

=end html

=cut

sub AdjustToolVars($) {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{MAXSESSIONS} = $config->{'MAXTFTPSESSION'} if defined($config->{'MAXTFTPSESSION'});
  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
										 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
										($config->{'DEFAULTCLIPWD'} ne ""));
  $toolData{PASSWORD2} = $crypt->decrypt($config->{'SECONDCLIPWD'}, CYPHER) if (defined($config->{'SECONDCLIPWD'}) &&
										($config->{'SECONDCLIPWD'} ne ""));
  $toolData{SFTPUSERNAME} = $crypt->decrypt($config->{'DEFAULTSFTPUSER'}, CYPHER) if (defined($config->{'DEFAULTSFTPUSER'}) &&
										($config->{'DEFAULTSFTPUSER'} ne ""));
  $toolData{TL1UDPPORT} = $config->{TL1UDPPORT} if defined($config->{TL1UDPPORT});
  $toolData{OCTOPUSPATH} = $config->{'OCTOPUSPATH'} if defined($config->{'OCTOPUSPATH'});
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'}); # -- 1.0.3
  $toolData{TFTPTIMEOUT} = $config->{'TFTPTIMEOUT'} if defined($config->{TFTPTIMEOUT});                  # -- 1.0.5
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT});            # -- 1.0.5
  $toolData{EMERG_ALRM_POLL_TIME} = $config->{'EMERG_ALRM_POLL_TIME'} if defined($config->{EMERG_ALRM_POLL_TIME});            
  $toolData{SHUBSWPATH} = $config->{'SHUBSWPATH'} if (defined($config->{'SHUBSWPATH'}));
  $toolData{SHUBSWNAME} = $config->{'SHUBSWNAME'} if (defined($config->{'SHUBSWNAME'}));
  $toolData{WARNINGFILE} = $config->{PRCK_WARN_FILE} if defined($config->{PRCK_WARN_FILE});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
										 ($config->{NT_COMMUNITY} ne ""));
  $toolData{NTDBTOOLDIR} = $config->{'NTDBTOOLDIR'} if defined($config->{'NTDBTOOLDIR'});
  $toolData{NTDBTOOL} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTOFLMT'} if defined($config->{'NTOFLMT'});
  $toolData{NTDBTOOL_I386} = $toolData{NTDBTOOLDIR} . "/" . $config->{'I386_NTOFLMT'} if defined($config->{'I386_NTOFLMT'});
  $toolData{NTDBTOOL_LINUX} = $toolData{NTDBTOOLDIR} . "/" . $config->{'LINUX_NTOFLMT'} if defined($config->{'LINUX_NTOFLMT'});
  $toolData{HICAP_NTDBTOOL} = $toolData{NTDBTOOLDIR} . "/" . $config->{'HICAP_NTOFLMT'} if defined($config->{'HICAP_NTOFLMT'});
  $toolData{ANSI_NTDBTOOLDIR} = $config->{'ANSI_NTDBTOOLDIR'} if defined($config->{'ANSI_NTDBTOOLDIR'});
  $toolData{ANSI_NTDBTOOL} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'ANSI_NTOFLMT'} if defined($config->{'ANSI_NTOFLMT'});
  $toolData{ANSI_NTDBTOOL_I386} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'I386_ANSI_NTOFLMT'} if defined($config->{'I386_ANSI_NTOFLMT'});
  $toolData{ANSI_NTDBTOOL_LINUX} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'LINUX_ANSI_NTOFLMT'} if defined($config->{'LINUX_ANSI_NTOFLMT'});
  $toolData{LS_NTDBTOOLDIR} = $config->{'LS_NTDBTOOLDIR'} if defined($config->{'LS_NTDBTOOLDIR'});
  $toolData{GPON_NTDBTOOLDIR} = $config->{'GPON_NTDBTOOLDIR'} if defined($config->{'GPON_NTDBTOOLDIR'});
  $toolData{LS_NTDBTOOL} = $toolData{LS_NTDBTOOLDIR} . "/" . $config->{'LS_NTOFLMT'} if defined($config->{'LS_NTOFLMT'});
  $toolData{GPON_NTDBTOOL} = $toolData{GPON_NTDBTOOLDIR} . "/" . $config->{'GPON_NTOFLMT'} if defined($config->{'GPON_NTOFLMT'});
  $toolData{LANXDBTOOLDIR} = $config->{'LANXDBTOOLDIR'} if defined($config->{'LANXDBTOOLDIR'});
  $toolData{ANSI_LANXDBTOOLDIR} = $config->{'ANSI_LANXDBTOOLDIR'} if defined($config->{'ANSI_LANXDBTOOLDIR'});
  $toolData{LS_LANXDBTOOLDIR} = $config->{'LS_LANXDBTOOLDIR'} if defined($config->{'LS_LANXDBTOOLDIR'});
  $toolData{GPON_LANXDBTOOLDIR} = $config->{'GPON_LANXDBTOOLDIR'} if defined($config->{'GPON_LANXDBTOOLDIR'});
  $toolData{IVPSDBTOOLDIR} = $config->{'IVPSDBTOOLDIR'} if defined($config->{'IVPSDBTOOLDIR'});
  $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'IVPSOFLMT'} if defined($config->{'IVPSOFLMT'});
  $toolData{IVPSDBTOOL_I386} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'I386_IVPSOFLMT'} if defined($config->{'I386_IVPSOFLMT'});
  $toolData{IVPSINVENTORY} = $config->{'IVPSINVENTORY'} if defined($config->{'IVPSINVENTORY'});
  $toolData{NEWCDEFILE} = $config->{'NEWCDEFILE'} if defined($config->{'NEWCDEFILE'});
  $toolData{CDEDIR} = $config->{'CDEDIR'} if defined($config->{'CDEDIR'});
  $toolData{RETRIES} = $config->{'RETRIES'} if defined($config->{'RETRIES'});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});
  $STATUSFILE = $config->{PRCK_STATUS_FILE} if defined($config->{PRCK_STATUS_FILE});
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});
  $toolData{NTDBCFG} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{ANSI_NTDBCFG} = $toolData{ANSI_NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{LS_NTDBCFG} = $toolData{LS_NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{GPON_NTDBCFG} = $toolData{GPON_NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
}

#==============================================================================
#== Main program
#==============================================================================
my $sessionCount = 0;
my @pids;
my $pid;
my $warnCount = 0;

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub HandleSigPipe($) {
  my @msgs = @_;
  my $msgStr = "";

  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  FamilyCommon::SetSigPipeReceived();

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    Logger::LogTrace($execLogFH, "SIGPIPE received (ignored): $msgStr") if defined($execLogFH);
    print STDERR "SIGPIPE received (ignored): $msgStr" if !defined($execLogFH);
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	Logger::LogTrace($execLogFH, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($execLogFH);
	print STDERR sprintf("StackFrame $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($execLogFH);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
    CleanUp("PIPE", EXIT_DIED_CODE);
  }
}

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;
  my $ownPid = FamilyCommon::GetOwnPid();

  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string;
  Logger::LogTrace($execLogFH, $string) if defined($execLogFH);

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($execLogFH, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($execLogFH);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($execLogFH);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR "  Aborting children...\n";
    Logger::LogTrace($execLogFH, "  Aborting children...\n") if defined($execLogFH);
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	Logger::LogTrace($execLogFH, "Killing process $pid...\n") if defined($execLogFH);
	kill($pid);
      }
    }
  }
  FamilyCommon::UpdateNodeStatusLog( $exitCode, $signal );
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $ownPid = FamilyCommon::GetOwnPid();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();
  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($execLogFH, "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr") if (defined($execLogFH));
    print STDERR "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]: $msgStr\n";
    $warnCount++;
    my $framePresent = 1;
    my $i = 0;
    my $ownPackage;
    my $ownFileName;
    my $ownLine;
    my $ownRoutine;
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	Logger::LogTrace($execLogFH, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($execLogFH);
	print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($execLogFH);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($execLogFH, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($execLogFH));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($execLogFH, "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($execLogFH));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};
$SIG{"PIPE"} = \&HandleSigPipe;

#-- Read parameter config file and adjust tool defaults if needed -- BB--
STDOUT->autoflush(1);
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
FamilyCommon::SetConfigData(\%config);
AdjustToolVars(\%config);
# -- End modif BB

# Read warning levels
my %warnLevels = Utils::ReadConfig("$dirName/$toolData{WARNINGFILE}");
FamilyCommon::SetWarningLevels(\%warnLevels);

my $ecntIcsData;
# Read ECNT-A ICS 11/12 data file
if (open($ecntIcsData, "<$dirName/3EC-36420-3056-DGZZA.csv")) {
  my $line;
  while ($line = <$ecntIcsData>) {
    if ($line =~ /^(\w\w\d+);(.+)/) {
      $toolData{ECNTA_ICSDATA}{$1} = $2;
#      print STDERR "Stored $1 as $2\n";
    }
  }
  close($ecntIcsData);
} else {
  print STDERR "Cannot open '$dirName/3EC-36420-3056-DGZZA.csv' for read: $!\n";
  exit FILE_OPEN_FAIL_CODE;
}
#-- Handle command line
$params = ParseCommandLine(\%config);
$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1);

Utils::SetDebug($execLogFH) if ($params->{DEBUG}); # 1.2.2
my $retC = Utils::VerifyOctopusEnv($params->{DEBUG}, $toolData{OCTOPUSPATH}, \%config, $dirName);
if ($retC) {
  Logger::LogError($execLogFH, Utils::GetError());
  exit $retC;
}


my $status_dbtool_parse = CapabilityReEngr::ParseDBToolConfig( "$scriptDir/$toolData{NTDBCFG}" );
if( $status_dbtool_parse ) {
  Logger::LogError($nodeLogFH, "Issue in parsing DB & Release version mapping file '$scriptDir/$toolData{NTDBCFG}'\n" );
  exit FILE_OPEN_FAIL_CODE;
}


my $isamsRef;
if (defined( $params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  Logger::LogNotify($execLogFH, "Reading Hostfile...\n");
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

### Make sure directory remains 0777
umask(0) if ( $params->{UMASK_CHMOD} );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );

my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      push(@pids, $pid);
    } else {
      CheckIsams( \@isamPart, $sessionCount, \%config, \%warnLevels);
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1, $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      Logger::LogInfo( $execLogFH, sprintf("Session %d : ends with exit code : %d - (%s); signal (%d)\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal ) );
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
