#!/usr/bin/perl

=head1 ISAM STATUS CHECK

=begin html
This script will perform a number of "health" status checks.  This script can be run prior to and after a SW upgrade or migration.

The operations of the script are influenced by:
<ul>
  <li>command line options</li>
  <li>a parameter configuration file</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         statusCheck.pl
#-- Date:         25.10.2006
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to perform a series of stand-alone PBMT pre- and post-checks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2006/10/25 : ANTms14531, ANTms23976 - First implementation
#--   1.0.1 - 2007/04/25 : ANTms44293 - include LT status information.
#--   1.0.2 - 2007/05/04 : Small correction in Ctrl-c handling (don't kill pid
#--                        that have already exited)
#--   1.0.3 - 2007/06/18 : ANTms51632 allow multiple users for same ISAM.
#--   1.0.4 - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.0.5 - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.0.6 - 2007/08/22 : ANTms60869: Make NT community string configurable
#--   1.0.7 - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.0.8 - 2008/05/29 : ANTmt09138.  LT numbering starts from 1 in type-based
#--                        operator profiles.  Incorrect LT number is used to calculate
#--                        an ifIndex.
#--   1.0.9 - 2008/06/13 : ANTmt09138.  Solution not complete: on FD the first LT is always 1
#--                                     whatever CLI command-mode is used.
#--   1.0.10 - 2008/08/18: Remove call to retrieve first LT offset.
#--                        Clearer trace when determining CLI LT slotnumber mode.
#--   1.0.11 - 2008/10/27 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.0.12 - 2009/03/10 : ANTmt18506
#--   1.0.13 - 2009/03/18 : ANTmt48688.
#--   1.0.14 - 2009/04/22 : ANTmt54584
#--   1.0.15 - 2009/04/30 : ANTmt55717.  Introduced NE famile ISAM_BOTH to indicate NTs that can be part of both the ETSI and ANSI stream.
#--   1.0.16 - 2009/05/19 : ANTmt59034: Change $config{USESSH} to $config{USE_SSH}.  Skip alamr retrieval if no TL1 access possible.
#--                         ANTmt59036.  Check security access settings and force usage of SSH if no telnet access for CLI possible.
#--   1.0.17 - 2009/05/26 : ANTmt60011.
#--   1.0.18 - 2009/06/02 : ANTmt60891.
#--   1.0.19 - 2009/06/02 : ANTmt61239 - Added 1540LS NE family
#--   1.0.20 - 2009/06/10 : ANTmt62662, allow public-key authentication with SSH
#--   1.0.21 - 2009/06/26 : ANTmt65332.  Introduction of the NTT ETSI family
#--   1.0.22 - 2009/10/09 : ANTmt77854.  Typo in function call for Hicap NT (must be HiCap)
#--                       : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.22 - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session
#--   3.0.1  - 2010/05/24 : First Implementation for converged PBMT - JanakiRaman and Janith
#--                       : x86 Changes
#--   3.0.2  - 2010/08/03 : ALU00441702 Addition of support for --bufsize in param.cfg 
#--   3.0.3  - 2010/10/15 : ALU00808688 new LRM option --id (suppresses the --logfile option).  Must not be used in combination with --hosts
#--   3.0.4  - 2010/11/25 : to resolve problem of child log file being written to parent log.
#--   3.0.5  - 2010/12/03 : ALU00926734 Delete older pre/postOperCheck files during respective pre/post run 
#--   3.0.6  - 2011/01/11 : LRM updates for lib-perl path
#--   3.0.7  - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.8  - 2011/04/01 : Changes for 7354RU
#--   3.0.9  - 2011/05/02 : ALU01178887
#--   3.0.10 - 2011/10/24 : logfile command line parameter can include a directory
#--   3.0.11 - 2011/11/01 : ALU01584510 
#--   3.0.12 - 2012/03/26 : ALU01748774
#--   3.0.13 - 2012/05/14 : ALU01789702 - World writeable directories need sticky bit set 
#--   3.0.14 - 2012/05/30 : to correct the display of snmp option used in logfile
#--   3.0.15 - 2012/10/22 : ALU01761938
#--   3.0.16 - 2012/11/06 : ALU01837839
#--   3.0.17 - 2012/12/20 : ALU01849616
#--   3.0.18 - 2013/02/07 : enable usage of second password if first one fails for cli over ssh
#--   3.0.19 - 2013/02/08 : corrected issue related to picking of second password from config file
#--   3.0.20 - 2013/02/15 : undef second password in params when it is set as empty
#--   3.0.21 - 2013/04/29 : ALU01880147
#--   3.0.22 - 2013/12/09 : statuscheck functionality added for 7367
#--   3.0.23 - 2014/06/09 : ALU01937911
#--   3.0.24 - 2014/07/06 : ALU01991130 Full support for ANSI VSRM 7367_SX
#--   3.0.25 - 2014/11/30 : ALU02028455
#--   3.0.26 - 2015/05/25 : ALU02060588
#--   3.0.27 - 2015/06/10 : ALU02083077
#--   3.0.28 - 2015/10/28 : ALU02139820
#--   3.0.29 - 2015/11/02 : ALU02142016
#--   3.0.30 - 2015/12/14 : ALU02156544
#--   3.0.31 - 2015/12/30 : ALU02160793
#--   3.0.32 - 2016/02/15 : ALU02174145
#--   3.0.33 - 2016/07/15 : ALU02237966
#--   3.0.34 - 2016/09/28 : ALU02278259
#--   3.0.35 - 2016/11/21 : ALU02285181
#--   3.1.0 -- 2018/06/20 : ALU02285181
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.2.0  - 2018/09/24 : ONT Migration via CLI changes


=head1 VERSION

=begin html
Current version: "statusCheck.pl - 3.2.0 -- 2018/09/24"

=end html

=cut

my $VERSION = "statusCheck.pl - 3.2.0 -- 2018/09/24";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2102-DFZZA">perlTL1::UdpChannel</a></li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use File::Copy;
use File::Glob;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use Config;

my $scriptDir; 

BEGIN {
$scriptDir = dirname(realpath($0));
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}


use lib "$scriptDir/lib"; 
#==============================================================================
use Net::SNMP;
use Net::TFTP;
use perlTL1::UdpChannel;

use Logger qw(:ALL);
use IsamCli;
require  perlTL1::SshChannel;
use Crypt::Lite;
use Utils;
use Capability;
use PbmtConfig;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use StatusCheckIsam7302;
use StatusCheckIsam7302Etsi;
use StatusCheckIsam7302EtsiNtt;
use StatusCheckIsam7302Ansi;
use StatusCheckIsam7302HiCap;
use StatusCheckIsam1540;
use StatusCheckIsam7354;
use StatusCheckIsam7367;
use StatusCheckIsam7363;
use StatusCheckIsam7367_SX;
use ErrorCodes qw(:EXIT_CODES :SCHK);
use SNMPComm;
use version qw(:ALL);

#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$;$);
sub CheckIsams($$$);
sub AdjustToolVars($);
sub CleanUp($;$);

#==============================================================================
#== Constants
#==============================================================================

#my $TELNETTIMEOUT = 10;
my $STATUSFILE     = "result.txt";
my $ID_FILENAME =      "SCHK";
my @CHECKOPTIONS   = ("LTSTS","XDSL","BRIDGE","AUTH","ALARM","ONT");
$execLogFH = undef;
$nodeLogFH = undef;
my $logExecOrNode = 0;

#==============================================================================
#== Vars
#==============================================================================
my %toolData = (HOSTSFILE => "/etc/hosts",
		TFTPDIR => "/var/opt/aws/equipment",
		TFTPWORKDIR => "tmp",
		MAXSESSIONS => 5,
		USERNAME => "",
		PASSWORD => "",
		PASSWORD2 => "",
		TL1USER => "",
		TL1PWD => "",
                ONTRELDIR => "tools",
                ONTRELMAPFILE => "ONT_Release_Mapping.txt",
		OCTOPUSPATH => "tools/octopus",
		OCTOPUSSCRIPTS =>   "tools/octopus",
		NODEPREFIX => "AS-",
		TFTPTIMEOUT => 5,
		NT_COMMUNITY => "public",
		RETRIES => 5,
                BUFSIZE => 15);

=head1 USAGE

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] statusCheck.pl --isam &lt;ip&gt; | --hosts &lt;file&gt; --pre|--post --logfile &lt;file&gt; [--checks &lt;checks&gt;] \ <br>
                    [--sessions &lt;nr&gt;] [--seplog | --noseplog] [--multi] [--noname]  [--bufsize &lt;megs&gt;] [--telnetto &lt;mins&gt;] \ <br>
                    [--cliusername &lt;user&gt;] [--clipassword &lt;password&gt;] [--v] [--debug] [--comm &lt;SNMP community string&gt; [--noping] \ <br>
                    [--tl1username &lt;user&gt;] [--tl1password &lt;password&gt;] [--release &lt;text&gt;][--ontreleasemap &lt;path&gt;]\ <br>
                    [--v2] [--v3Auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;] [-v3priv &lt;privProtocol&gt;:&lt;password&gt;] </tt>

=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>pre</td><td>run pre-operation status checks</td></tr>
  <tr><td>post</td><td>run post-operation status checks</td></tr>
  <tr><td valign="top">checks</td><td>the checks to be carried out: a ':' delimited string containing one or more of the following:
         <ul>
           <li>ltsts: LT status information (up/down)</li>
           <li>xdsl: read out XDSL line config and some operational data</li>
           <li>bridge: read out bridge config data</li>
           <li>auth: read the authentication status</li>
           <li>alarm: read outstanding alarms</li>
           <li>ont: ont status</li>
         </ul>
         When not specifcied all checks will be carried out.
      </td></tr>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>cliusername</td><td>ISAM CLI username</td></tr>
  <tr><td>clipassword</td><td>ISAM CLI password</td></tr>
  <tr><td>tl1username</td><td>ISAM TL1 username</td></tr>
  <tr><td>tl1password</td><td>ISAM TL1 password</td></tr>
  <tr><td>release</td><td>target release info </td></tr>
  <tr><td>ontreleasemap</td><td>path of the ont release mapping file</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td></tr>
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed.</td></tr>
  <tr><td>v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                             <li>All fields are mandatory when option is used.</li>
                         <ul<br>
                         e.g. --v3auth user:password:: will result in DM5 as protocol.</td></tr>
  <tr><td>v3priv</td><td>optional setting when using SNMPv3 privacy protocol providing protocol and password<br>
                         <ul><li>Privacy protocol is DES, 3DES or AES, DES is default protocol.</li>
                         </ul></td></tr>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

#------------------------------------------------------------------------------
#-- Display the usage
#------------------------------------------------------------------------------
sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> | --id <id> --pre|--post [--sessions <nr>] [--seplog | --noseplog]\\ \n";
  print STDERR "       [--checks <checks>] [--cliusername <user>] [--clipassword <password>] [--clipassword2 <password>] [--bufsize <megs>] \\ \n";
  print STDERR "       [--telnetto <mins>] [--release <text>] [--ontreleasemap <path>]\\ \n";
  print STDERR "       [--tl1username <username>] [--tl1password <password>]  \n";
  print STDERR "       [--v] [--debug] [--multi] [--noname] [--comm <SNMP community string>] [--v2] [--noping] \\ \n";
  print STDERR "       [--v3auth <user>:<password>:<authProtocol>] [-v3priv <privProtocol>:<password>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         id       : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                    The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         pre:          : run pre-operation status checks\n";
  print STDERR "         post:         : run post-operation status checks\n";
  print STDERR "         telnetto      : timeout value for telnet session (in seconds)\n";
  print STDERR "         checks        : the checks to be carried out: a ':' delimited string containing one or more of the following:\n";
  print STDERR "                         - ltsts: LT status information (up/down)\n";
  print STDERR "                         - xdsl: read out XDSL line config and some operational data\n";
  print STDERR "                         - bridge: read out bridge config data\n";
  print STDERR "                         - auth: read the authentication status\n";
  print STDERR "                         - alarm: read outstanding alarms\n";
  print STDERR "                         - ont: read ont status\n";
  print STDERR "                         When not specifcied all checks will be carried out.\n";
  print STDERR "         v             : print version and exit\n";
  print STDERR "         sessions      : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         release       : target release info in case if ont is selected \n";
  print STDERR "         ontreleasemap : path of directory where ONT Release Mapping file is located\n";
  print STDERR "         tl1username   : TL1 username\n";
  print STDERR "         tl1password   : TL1 password\n";
  print STDERR "         cliusername   : CLI username\n";
  print STDERR "         clipassword   : CLI password\n";
  print STDERR "         clipassword2 : Second ISAM CLI password (option to use a second password if first password fails).\n";
  print STDERR "                        The second password option will be used only when CLI over SSH is configured. \n";
  print STDERR "         seplog        : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noseplog      : keep all NE logging in a single child log file (default is a single logfile per NE)\n";
  print STDERR "         bufsize       : size of CLI command return buffer (default is 15Mb)\n";
  print STDERR "         noname        : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         multi         : include Unix username in NE logging directory path\n";
  print STDERR "         noping        : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "       SNMP settings:\n";
  print STDERR "         comm          : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is \"public\"\n";
  print STDERR "         v2            : Use SNMPv2c\n";
  print STDERR "         v3auth        : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                         The password should be at least 8 characters long.\n";
  print STDERR "                         Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                         All fields are mandatory when an option is used.\n";
  print STDERR "                         e.g. --v3auth user:password:: will result in MD5 as protocol.\n";
  print STDERR "         v3priv        : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                         Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  exit(CMD_INCOMPL_CODE);
}


=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $noaction, $sessions, $debug, $hosts, $logFile, $cliUserName, $cliPassword, $cliPassword2, $version, $sepLog, $noSepLog, $preCheck, $postCheck, $bufSize, $checks, $telnetTO, $multi, $wipe, $noName, $communityString, $noPing, $v3Auth, $v3Priv, $useSsh, $snmpV2, $id);
  my ($tl1username, $tl1password, $release,$relMapDirName,$checked, @checklist);
  my %params;
  my $options = GetOptions('isam=s', \$isam,
			   'noaction', \$noaction,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'logfile=s', \$logFile,
                           'ontreleasemap=s', \$relMapDirName,
                           'release=s', \$release,
                           'tl1username=s', \$tl1username,
                           'tl1password=s', \$tl1password,
			   'username|cliusername=s', \$cliUserName,
			   'password|clipassword=s', \$cliPassword,
                           'clipassword2=s', \$cliPassword2,     # undocumented, for 7354
			   'bufSize=i', \$bufSize,
			   'v', \$version,
			   'v2', \$snmpV2,
			   'seplog', \$sepLog,
			   'noseplog', \$noSepLog,
			   'checks=s', \$checks,
			   'telnetto=i', \$telnetTO,
			   'pre', \$preCheck,
			   'post', \$postCheck,
			   'multi', \$multi,
			   'wipe', \$wipe,
			   'noname', \$noName,
			   'comm=s', \$communityString,
			   'noping', \$noPing,
			   'v3auth=s', \$v3Auth,
			   'v3priv=s', \$v3Priv,
			   'ssh', \$useSsh,
                           'id=s', \$id);
  #-- check options
  if (! $options) {
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }

  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  if ((!defined($preCheck) && !defined($postCheck)) ||
      (defined($preCheck) && defined ($postCheck))) {
    print STDERR "ERROR: please select either the pre or post option (only one)\n";
    Usage();
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }
  if (defined($cliUserName)) {
    $params{USERNAME} = $cliUserName;
  } else {
    $params{USERNAME} = $toolData{USERNAME};
  }
  if (defined($cliPassword)) {
    $params{PASSWORD} = $cliPassword;
  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }
  if (defined($cliPassword2)) {
    $params{PASSWORD2} = $cliPassword2;
  } else {
    $params{PASSWORD2} = ($toolData{PASSWORD2} ne "") ? $toolData{PASSWORD2} : undef ;
  }
  if (defined($tl1username)) {
    $params{TL1USER} = $tl1username;
    $toolData{TL1USER} = $tl1username;
  } else {
    $params{TL1USER} = $toolData{TL1USER};
  }
  if (defined($tl1password)) {
    $params{TL1PWD} = $tl1password;
    $toolData{TL1PWD} = $tl1password;
  } else {
    $params{TL1PWD} = $toolData{TL1PWD};
  }
  if (defined($checks)) {
    @checklist = split(':', $checks);
    foreach $checked (@checklist) {
      if (!grep(/^$checked$/i,@CHECKOPTIONS)){
        print STDERR "ERROR : Wrong option provided for --checks : $checked\n";
        Usage();
      }
    }
    $params{CHECKS} = $checks;
  } else {
    $params{CHECKS} = $toolData{CHECKS};
  }

  if (defined($relMapDirName)) {
    if(!($params{CHECKS} =~ /ont/i)) {
      print STDERR "ERROR : --ontreleasemap need to be provided only when \"ont\" is added in --checks option.\n";
      Usage();
    } 
    if($relMapDirName ne ""){
        $toolData{ONTRELDIR} = $relMapDirName;
    } 
  }
  
  if(($params{CHECKS} =~ /ont/i)) {
    if(! -r "$toolData{ONTRELDIR}/$toolData{ONTRELMAPFILE}"){
      print STDERR "ERROR: ONT Release mapping file not found in the location $toolData{ONTRELDIR}. \n";
      Usage();
    }
  }
  
  if (!defined($release)) {
    if($params{CHECKS} =~ /ont/i) {   
      print  STDERR "ERROR: --release mandatory when \"ont\" is added in --checks option. \n";
      Usage();
    }
  } else {
    if(!($params{CHECKS} =~ /ont/i)) {
      print STDERR "ERROR : --release need to be provided only when \"ont\" is added in --checks option.\n";
      Usage();
    }
    if ( uc($release) =~ /^[LR][0-9][1-9]\.[0-9][0-9]\.[0-9][0-9]$/ || uc($release) =~ /^[LR][0-9][1-9]\.[0-9][0-9]\.[0-9][0-9][A-Z
]$/ ) {
      $toolData{RELEASE} = $release;
    } else {
      print STDERR "ERROR : Invalid release info - '$release'\n";
      Usage (); # not sure about exit code.
    }
  }

#ALU01524313-indhu-15/12/2011
  if (defined($communityString) && defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  if (defined($communityString) && defined($v3Priv)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }
#ALU01524313-indhu-15/12/2011

  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if (defined($isam));
  $params{NOACTION} = 1 if (defined($noaction));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if (defined($debug));
  LogSetDebug() if(defined $debug);
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
  } else {
    $params{LOGFILE} = $logFile;
    if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
      my $logdir = $1;
      if (! -d $logdir) {
        if (!mkdir( $logdir, 0777)) {
          print STDERR "Cannot create directory '$logdir': $!\n";
          exit CREATE_DIR_FAILED_CODE;
        }
      }
    }
  }
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
  $params{PRE_OR_POST} = defined($preCheck) ? 0 : 1;
  if (defined($bufSize)) {
    $params{BUFSIZE} = $bufSize;
  } else {
    $params{BUFSIZE} = $toolData{BUFSIZE};
  }
  $params{BUFSIZE} = 100 if ($params{BUFSIZE} > 100);
  $params{BUFSIZE} *= 1048576; # Convert to MBytes
  if (defined($telnetTO)) {
   $params{TELNETTIMEOUT_SCHK} = $telnetTO;
   $toolData{TELNETTIMEOUT_SCHK} = $telnetTO;
  } else {
   $params{TELNETTIMEOUT_SCHK} = $toolData{TELNETTIMEOUT_SCHK};
  }
  $STATUSFILE = defined($preCheck) ? "pre_" . $STATUSFILE : "post_" . $STATUSFILE;
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{WIPE} = defined($wipe) ? 1 : 0;
  $params{NONAME} = defined($noName) ? 1 : 0;
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : ""; #ALU02028455
  
  if (!defined($snmpV2)) { #ALU02028455 #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }
  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA256|SHA512|MD5)$/ ) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
          if ($params{SNMP}{V3}{AUTH}{PROTO} eq "SHA" ) {
              $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
	} else {
	  print STDERR "SNMPv3 authentication protocol can only be SHA or SHA256 or SHA512 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
    $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
    if ($params{SNMP}{V3}{AUTH}{PROTO} eq "SHA" ) {
        $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {  
    my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
 	$params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    $params{SNMP}{V3}{PRIV}{PASSWORD} = defined($toolData{SNMPV3_PRIV_PASSWORD}) ? $toolData{SNMPV3_PRIV_PASSWORD} : "";
  } #ALU01524313-indhu-15/12/2011
  }
  if (!defined($params{PASSWORD}) || (!defined($params{USERNAME} )) || (!defined($toolData{TL1USER})) || (!defined($toolData{TL1PWD}) )) {
    print STDERR "No CLI or TL1 username, password defined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }
  $params{NOPING} = 0;
  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'});
  $params{NOPING} = 1 if defined($noPing);

  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";

  $params{UPD_KNOWN_HOST_FILE} = 0;
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});

  return \%params;
}

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut


sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw(
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli IsamTL1 Octopus 
                       UdpChannel SshChannel
                       StatusCheckIsam7302 StatusCheckIsam7302Etsi StatusCheckIsam7302EtsiNtt StatusCheckIsam7302Ansi StatusCheckIsam7302HiCap 
                       StatusCheckIsam1540 StatusCheckIsam7354 StatusCheckIsam7367 StatusCheckIsam7363 StatusCheckIsam7367_SX
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

=head2 PrintOptionsSummary($$$;$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$;$) {
  my ($argLogFH, $logOnScreen, $config) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Status check type" , 
			       $params->{PRE_OR_POST} ? "Post operation" :  "Pre operation") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Checks to be perform" , $params->{CHECKS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Telnet session timeout in seconds" , $params->{TELNETTIMEOUT_SCHK} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Actions to be perform" , $params->{NOACTION} ? "No, Checks only" : "Yes, Action after checks") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "CLI buffer size" , ($params->{BUFSIZE} / 1048576) ) );

  Logger::LogWish( $logMode, $argLogFH, "\n" );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Current Work Directory" , getcwd() ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Work space Directory" , "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Result space Directory" , "$toolData{RESULTSPACE}" ) );
  Logger::LogWish( $logMode, $argLogFH, "\n" );
}

=head2 CheckIsams($$$$)

=begin html
The main body of the script defining the complete status-check process.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by this child process.  Every
      array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
</table>
Depending on the checks specified the script will retrieve and store:
<ul>
  <li>LT status information</li>
  <li>XDSL configuration and status information</li>
  <li>bridge port configuration data</li>
  <li>authentication status information</li>
  <li>outstanding alarms</li>
</ul>
Running the script before the upgrade/migration (using --pre) and after (using --post) allows comparison of the results to determine how succesfull
the migration was.   Allow some time for the system/users to stabilize after the migration/upgrade.

=end html

=cut

sub CheckIsams($$$) {
  my ( $isams, $sessionCount, $config) = @_;
  my $isam;
  my $ret;
  my ($swMgntIniSize, $altSwMgntIniSize, $swParSize);
  my ($newMajor, $newMinor, $newSuffix, $newRelNum, $newFullRelNum, $activeRelNum, $fullActiveRelNum);
  my $shubSwName;
  my ($time1, $time2);
  my $startTime;
  my $activeNt;
  my $nodeLogFile;
  my $nodeStatusLog;
  my $possible = 0;
  my $reason = "";
  my $cli = undef;
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $isamNode;
  my ($session, $snmpError);
  my $dirName = dirname($0);
  my $dir;
  my $thisTftpDir;
  my $fxrData;

  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetOwnPid($$);
  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetConfigData($config);

  Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Started as '%d'\n", $sessionCount, FamilyCommon::GetOwnPid()));
  ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );

      ### Switch debugging (if any) to an opened filehandle...
      FamilyCommon::SetWarnLogFile($nodeLogFH);   # Make sure warnings and
      FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
      IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
      IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
      SshChannel::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
      Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
      PrintOptionsSummary($nodeLogFH, undef, $config);
  }

  my $isamCount = -1;
  foreach $isam (@{$isams}) {
    $isamCount++;
    $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1]; # Store for IncrErrorCount

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

    ##### Create ResultSpace directory #####
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );

    if (defined($params->{ID})) {
        $logExecOrNode = 0;
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif ( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
        PrintOptionsSummary($nodeLogFH, undef, $config);
    }
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and 
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    FamilyCommon::SetFailedNode($isamNode);
    # Switch debugging (if any) to an open file handle.
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    SshChannel::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
    Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    IsamTL1::ResetGlobals();
    $params->{USESSH} = 0; #for each isam the value is reset

    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    if (! open($nodeStatusLog, ">$toolData{RESULTSPACE}/$isamNode/$STATUSFILE")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$STATUSFILE' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE);
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE" ) );
    }
    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }

    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
          FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError() );
	      next;
    }
    $snmpStatus = SNMPComm::MaximizeMsgSize();
    if( $snmpStatus ) {
         FamilyCommon::ReportStatus( EC_SNMP_MAX_SIZE_FAIL, SNMPComm::GetError() );
         next;
    }
    my $mibVersion = SNMPComm::GetValue('MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError() , "Failed to obtain MIB version" );
         next;
    }

    Logger::LogTrace($nodeLogFH, "Attempt to get file transfer data...\n");
    $fxrData = ToolCommon::GetFxrSettings($session, $toolData{RETRIES});
    if (not defined($fxrData)) {
      Logger::LogNotify($nodeLogFH, "Obtained empty file transfer protocol details\n");
      FamilyCommon::ReportStatus( EC_DETERMINE_LOG_FAILED, "Obtained empty file transfer protocol details" );
      next;
    } else {
      if($fxrData->{PROTOCOL} != TFTP_FXR) {
        $params->{NOTFTP}=1;
      }
    }

    #umask(0); #directory remains 0777
    umask(0) if ( $params->{UMASK_CHMOD} );
    my $tmpDir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}";
    ToolCommon::CreateDirectory( $execLogFH, $tmpDir );
    $dir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isamNode";
    $thisTftpDir = "$toolData{TFTPWORKDIR}/$isamNode";
    
    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g;
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $nodeLogFH, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};
    Logger::LogNotify( $nodeLogFH, "Node ($isamNode) : Storage Directory '$dir'\n");
    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile = sprintf("statuscheck_%s_%04d-%02d-%02d_%02d_%02d.log",
                             $params->{PRE_OR_POST} == 0 ? "pre" : "post", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
        Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        SshChannel::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
        Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    }

    FamilyCommon::SetNeStorage($dir);

    # Cleanup existing pre/postOperCheck files if any 
    my $fileDelete = -1; 
    my $preOperCheck  = PREOPERCHECKFILETAG . "*"; 
    my $postOperCheck = POSTOPERCHECKFILETAG . "*"; 
    my $tmpFileName = ($params->{PRE_OR_POST} == 0) ? $preOperCheck : $postOperCheck; 
    my @fileExist = glob("$dir/$tmpFileName"); 
    if(@fileExist != 0) {
      $fileDelete = unlink <$dir/$tmpFileName>;
      if($fileDelete) {
         Logger::LogInfo($nodeLogFH, "Cleaned up old files '$tmpFileName' from '$dir'\n"); 
      } else {
         Logger::LogWarn($nodeLogFH, "Failed to Clean up old files '$tmpFileName' from '$dir'\n"); 
      }
    } else {
         Logger::LogInfo($nodeLogFH, "No older files '$tmpFileName' from '$dir'\n"); 
    }

    # Determine NE family: retrieve NT-A and NT-B type and status using SNMP.  Only then we continue (GPON does not have full
    # CLI capability...
    my $ntType = ToolCommon::DetermineNtType( );
    next if (not defined($ntType));

    my $oswpName = ToolCommon::GetActiveOswpInfoBySnmp($session, $params->{DEBUG});
    if (!defined($oswpName)) {
      FamilyCommon::ReportStatus( EC_GET_OSWP_FAILED, ToolCommon::GetError() );
      next;
    }

    Logger::LogTrace($nodeLogFH, "Check access security settings...\n");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      FamilyCommon::ReportStatus( EC_GET_ACC_BITS_FAILED, ToolCommon::GetError() );
      next;
    } else {
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

    $params->{NOTL1} = 0;
    if( ! ToolCommon::TL1ViaUDP($accBits, $params->{DEBUG}) && ! ToolCommon::TL1ViaSSH($accBits, $params->{DEBUG}) ) {
        $params->{NOTL1} = 1;
             Logger::LogInfo($nodeLogFH, "TL1 Communication is disabled (No UDP/SSH)\n");
    }
	    if (! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
	        $params->{USESSH} = 1;
	        Logger::LogInfo($nodeLogFH, "CLI Communication through SSH\n");
            }
    

    ToolCommon::GetSourceDetails();
    if( !defined $params->{SOURCE_OSWPNAME} ) {
         FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED,ToolCommon::GetError(), undef );
         next;
    }

    $params->{'BOARD-FAMILY'} = "";
    Capability::LogNeFamily($ntType, $params->{SOURCE_DBVERSION});
    if ( $params->{'BOARD-FAMILY'} eq "" ) {
      FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, undef, 
                                  sprintf("Board family not found for NT-Type '%s', DB-Version '%s'\n",
                                      $ntType, $params->{SOURCE_DBVERSION} ) );
      next;
    }
    Capability::LogNeFamily($ntType, $params->{SOURCE_DBVERSION});

    if (Capability::IsIsamFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7302::Check($isam, uc($ntType));
    } elsif (Capability::IsIsamEtsiFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7302Etsi::Check($isam, uc($ntType));
    } elsif (Capability::IsIsamEtsiExtendedFamily($ntType, $params->{SOURCE_DBVERSION})) {
      my $realFamily = Capability::FamilyFromOswp(basename($oswpName));
      if ($realFamily eq Capability::IsamEtsiFamily()){
	StatusCheckIsam7302Etsi::Check($isam, uc($ntType));
      } elsif ($realFamily eq Capability::IsamEtsiNttFamily()){
	StatusCheckIsam7302EtsiNtt::Check($isam, uc($ntType));
      } else {
	    FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, "NT-Type : $ntType, DB-Version : $params->{SOURCE_DBVERSION}");
	    next;
      }
    } elsif (Capability::IsIsamAnsiFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7302Ansi::Check($isam, uc($ntType));
    } elsif (Capability::IsIsamBothFamily($ntType, $params->{SOURCE_DBVERSION})) {
      my $realFamily = Capability::FamilyFromOswp(basename($oswpName));
      if ($realFamily eq Capability::IsamAnsiFamily()) {
	StatusCheckIsam7302Ansi::Check($isam, uc($ntType));
      } elsif ($realFamily eq Capability::IsamEtsiFamily()){
	StatusCheckIsam7302Etsi::Check($isam, uc($ntType));
      } elsif ($realFamily eq Capability::IsamEtsiNttFamily()){
	StatusCheckIsam7302EtsiNtt::Check($isam, uc($ntType));
      } else {
	    FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, "NT-Type : $ntType, DB-Version : $params->{SOURCE_DBVERSION}");
	    next;      
      }
    } elsif (Capability::IsIsamHiCapEtsiFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7302HiCap::Check($isam, uc($ntType));
    } elsif (Capability::IsIsamHiCapAnsiFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7302HiCap::Check($isam, uc($ntType));
    } elsif (Capability::IsIsamLSFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam1540::Check($isam, uc($ntType));
    } elsif (Capability::Is7354RUFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7354::Check($isam, uc($ntType));
    } elsif (Capability::IsIsam7367Family($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7367::Check($isam, uc($ntType));
    } elsif (Capability::IsIsam7363Family($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7363::Check($isam, uc($ntType));
    } elsif (Capability::IsIsam7367_SXFamily($ntType, $params->{SOURCE_DBVERSION})) {
      StatusCheckIsam7367_SX::Check($isam, uc($ntType));
    } else {
	   FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, "NT-Type : $ntType, DB-Version : $params->{SOURCE_DBVERSION}");
       next;      
    }
    Logger::LogTrace($nodeLogFH, "Done!\n");

    my $endTime = time();
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );
    if (not FamilyCommon::NodeIsFailed() ){
        FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
    }
    if ($params->{SEPLOG}) {
        FamilyCommon::CloseChildLogFile();
        LogCloseFile($nodeLogFH);
    }
  }
  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }

  if (!defined($params->{ID})) {
    if (!$params->{SEPLOG}) {
        FamilyCommon::CloseChildLogFile();
        LogCloseFile($nodeLogFH);
    }
  }
  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file).

=end html

=cut

sub AdjustToolVars($) {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);
  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{REGEXP} = $config->{'REGEXP'} if defined ($config->{'REGEXP'});
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{MAXSESSIONS} = $config->{'MAXTFTPSESSION'} if defined($config->{'MAXTFTPSESSION'});
  $toolData{CHECKS} = $config->{'STATUS_CHECKS'} if defined($config->{'STATUS_CHECKS'});
  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
										 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
										($config->{'DEFAULTCLIPWD'} ne ""));
  $toolData{PASSWORD2} = $crypt->decrypt($config->{'SECONDCLIPWD'}, CYPHER) if (defined($config->{'SECONDCLIPWD'}) &&
										($config->{'SECONDCLIPWD'} ne ""));
  $toolData{TL1USER} = $crypt->decrypt($config->{'DEFAULTTL1USER'}, CYPHER) if (defined($config->{'DEFAULTTL1USER'}) &&
										($config->{'DEFAULTTL1USER'} ne ""));
  $toolData{TL1PWD} = $crypt->decrypt($config->{'DEFAULTTL1PWD'}, CYPHER) if (defined($config->{'DEFAULTTL1PWD'}) &&
									      ($config->{'DEFAULTTL1PWD'} ne ""));
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
										 ($config->{NT_COMMUNITY} ne ""));
  $toolData{OCTOPUSPATH} = $config->{'OCTOPUSPATH'} if defined($config->{'OCTOPUSPATH'});
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'}); # -- 1.0.3
  $toolData{TFTPTIMEOUT} = $config->{'TFTPTIMEOUT'} if defined($config->{'TFTPTIMEOUT'});                  # -- 1.0.5
  $toolData{RETRIES} = $config->{'RETRIES'} if defined($config->{'RETRIES'});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{TL1UDPPORT} = $config->{'TL1UDPPORT'} if defined($config->{'TL1UDPPORT'});
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{'TELNETTIMEOUT'});      
  $toolData{TELNETTIMEOUT_SCHK} = $config->{'TELNETTIMEOUT_SCHK'} if defined($config->{'TELNETTIMEOUT_SCHK'});            # -- 1.0.5
  $STATUSFILE = $config->{SCHK_STATUS_FILE} if defined($config->{SCHK_STATUS_FILE});
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});
  $toolData{ONTRELDIR} = $config->{'ONTRELDIR'} if defined($config->{'ONTRELDIR'});
  $toolData{ONTRELMAPFILE} = $config->{'ONTRELMAPFILE'} if defined($config->{'ONTRELMAPFILE'});
  $toolData{BUFSIZE} = $config->{'BUFSIZE'} if defined($config->{'BUFSIZE'});
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});

}


#==============================================================================
#== Main program
#==============================================================================
my $sessionCount = 0;
my %pids;
my $pid;
my $warnCount = 0;

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $ownPid = FamilyCommon::GetOwnPid();

  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  print STDERR sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR "\nCaught signal '$signal'\n";
  if (keys %pids) {
    print STDERR "  Aborting children...\n";
    foreach my $pid (keys %pids) {
      if ($pids{$pid} > 0) {
	print STDERR "Killing process $pids{$pid}...\n";
	kill($pids{$pid});
      }
    }
  }
  FamilyCommon::UpdateNodeStatusLog( $exitCode, $signal );
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();

  if (!($msgStr =~ /Zlib/)) {
    print $warnLogFile "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr" if (defined($warnLogFile));
    print STDERR "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr";
    $warnCount++;
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  print $warnLogFile "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr" if (defined($warnLogFile));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  print $warnLogFile "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr" if (defined($warnLogFile));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};


#-- Read parameter config file and adjust tool defaults if needed -- BB--
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars(\%config);
FamilyCommon::SetConfigData(\%config);
# -- End modif BB

#-- Handle command line
$params = ParseCommandLine(\%config);

STDOUT->autoflush(1);
$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1, \%config);

Utils::SetDebug($execLogFH) if ($params->{DEBUG}); # 1.2.2
my $retC = Utils::VerifyOctopusEnv($params->{DEBUG}, $toolData{OCTOPUSPATH}, \%config, $dirName);
if ($retC) {
  Logger::LogError($execLogFH, Utils::GetError() );
  exit $retC;
}


#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  Logger::LogInfo($execLogFH, "Reading Hostfile...\n");
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

#-- Create work dir
#umask(0); # Make sure directory remains 0777
umask(0) if ( $params->{UMASK_CHMOD} );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );

my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      $pids{$pid} = $sessionCount; 
    } else {
      CheckIsams( \@isamPart, $sessionCount, \%config);
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
if (keys %pids) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = (defined($pids{$kid})) ? $pids{$kid} : -1;
      Logger::LogInfo( $execLogFH, sprintf("Session %d : ends with exit code : %d - (%s); signal (%d)\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal ) );
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode = EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
