#!/usr/bin/perl

=head1 ISAM STATUS CHECK COMPARISON

=begin html
This script will compare the results from the status health-checks run prior to and after a SW upgrade or migration.  Differences are compared
using thresholds.  As long as the number of differences for a certain class remains below the threshold the migration is regarded as successful.
In case the threshold is crossed the error level associated to this check will influence the comparison result.  It can be regarded as either:
<ul>
  <li>To be ignored</li>
  <li>to be raised as a warning (migration/upgrade regarded as succesfull but operator needs to verify</li>
  <li>To be treated as an error.  Even if the node is running the new release, these errors may be a reason to declare a migration as failed
      (e.g. a(n) (number of) LT(s) is not operational after the migration/upgrade)</li>
</ul>

The operations of the script are influenced by:
<ul>
  <li>command line options</li>
  <li>a parameter configuration file</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         statusCheckCmp.pl
#-- Date:         25.10.2006
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to perform a series of stand-alone PBMT pre- and post-checks.
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2007/05/16 : ANTms45965 - First implementation
#--   1.0.1 - 2007/06/18 : ANTms51632 allow multiple users for same ISAM.
#--   1.0.2 - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.0.3 - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.0.4 - 2007/09/24 : ANTms64938: remove all traces of readable username and passwords.
#--   1.0.5 - 2009/03/10 : ANTmt18506
#--   1.0.6 - 2009/04/22 : ANTmt54584
#--   1.0.7 - 2009/04/30 : ANTmt55717.  AGNT-A belongs to both ISAM streams so had to introduce ISAM_ETSI and ISAM_ANSI.
#--   1.0.8 - 2009/05/19 : Print correct SNMP settings at program startup.
#--   1.0.9 - 2009/06/02 : ANTmt61239 - Added 1540LS NE family.
#--   1.0.10 - 2009/06/12
#--   1.0.11 - 2009/06/26 : ANTmt65332.  Introduction of the NTT ETSI family
#--   1.0.12 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.13 - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session
#--   3.0.1  - 2010/05/24 : First Implementation for converged PBMT - JanakiRaman and Janith
#--                       : x86 Changes
#--   3.0.2  - 2010/10/19 : ALU00808688 new LRM option --id (suppresses the --logfile option).  Must not be used in combination with --hosts
#--                         ALU00813833 ONT Release mapping file missing error
#--   3.0.3  - 2010/11/25 : solve problem of child log file being written to parent log.
#--   3.0.4  - 2010/12/06 : ALU00926734
#--   3.0.5  - 2011/01/11 : LRM updates for lib-perl path
#--   3.0.6  - 2011/02/02 : changing error returned for SNMP errors
#--   3.0.7  - 2011/02/11 : resolve issue of community string not accepted by the script
#--   3.0.8  - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.9  - 2011/05/02 : ALU01178887
#--   3.0.10 - 2011/10/24 : ALU01421954
#--   3.0.11 - 2011/11/01 : ALU01584510 
#--   3.0.12 - 2012/05/14 : ALU01789702 - World writeable directories need sticky bit set 
#--   3.0.13 - 2012/05/30 : to correct the display of snmp option used in logfile
#--   3.0.14 - 2012/10/22 : ALU01761938
#--   3.0.15 - 2013/04/29 : ALU01880147
#--   3.0.16 - 2013/08/12 : ALU01901723
#--   3.0.17 - 2013/12/09 : statuscheckCmp functionality for 7367
#--   3.0.18 - 2014/07/06 : ALU01991130: Full support for ANSI VSRM 7367_SX
#--   3.0.19 - 2014/11/30 : ALU02028455
#--   3.0.20 - 2015/05/25 : ALU02060588
#--   3.0.21 - 2015/06/10 : ALU02083077
#--   3.0.22 - 2015/10/28 : ALU02139820
#--   3.0.23 - 2015/01/18 : ALU02167211
#--   3.0.24 - 2016/07/15 : ALU02237966
#--   3.0.25 - 2016/08/16 : ALU02255980
#--   3.0.26 - 2016/09/28 : ALU02278259
#--   3.1.0  - 2018/06/20 : ALU02285181
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "statusCheckCmp.pl - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "statusCheckCmp.pl - 3.1.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use Config;

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 

use PBMTGlobal;
use Logger qw(:ALL);
use Utils;
use Capability;
use ErrorCodes qw(:EXIT_CODES :CHKC);
use StatusCheckCmpIsam7302;
use StatusCheckCmpIsam7302Etsi;
use StatusCheckCmpIsam7302EtsiNtt;
use StatusCheckCmpIsam7302Ansi;
use StatusCheckCmpIsam7302HiCap;
use StatusCheckCmpIsam7354;
use StatusCheckCmpIsam7367;
use StatusCheckCmpIsam7367_SX;
use StatusCheckCmpIsam1540;
use StatusCheckCmpIsam7363;
use FamilyCommon;
use ToolCommon qw(:CONSTANTS);
use PbmtConfig;
use SNMPComm;
use version qw(:ALL);
use Crypt::Lite;
#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$;$);
sub CompareIsamChecks($$$$);
sub AdjustToolVars($);
sub CleanUp($;$);

#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars
#==============================================================================

my $ID_FILENAME = "CHKC";
my %toolData = ('HOSTSFILE' => "/etc/hosts",
		'TFTPDIR' => "/var/opt/aws/equipment",
		'TFTPWORKDIR' => "tmp",
                'ONTRELDIR' => "tools/",
                'ONTRELMAPFILE' => "ONT_Release_Mapping.txt",
		'MAXSESSIONS' => 5,
		'STATUSFILE' => "result.txt",
                'OCTOPUSPATH' => "tools/octopus",
                'OCTOPUSSCRIPTS' =>   "tools/octopus",
		'WARNINGFILE' => "CHKCwarnings.txt",
		'NODEPREFIX' => "AS-",
		'LINEOOSPCT' => 5,
		'LTOOSPCT' => 5,
		'AUTHOOSPCT' => 5,
		'BRIDGEMMPCT' => 0);
my $ontRelMapOpt = 0; 
my $logExecOrNode = 0;

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] statusCheckCmp.pl --isam &lt;ip&gt; | --hosts &lt;file&gt; --logfile &lt;file&gt; [--detail] [--incltoos] \ <br>
                    [--sessions &lt;nr&gt;] [--seplog | --noseplog] [--multi] [--noname] [--v] [--debug]</tt>
                    [--ontreleasemap <path>] [--warn &ltThreshold&gt]\ <br 
                    [--comm &lt;SNMP community string&gt; [--noping] [--v2] [--v3Auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;] \ <br>
                    [-v3priv &lt;privProtocol&gt;:&lt;password&gt;]</tt>

=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>detail</td><td>print the mismatches</td></tr>
  <tr><td>incltoos</td><td>include Lines OOS of LTs OOS (default excluded)</td></tr>
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td></tr>
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>ontreleasemap</td><td>path of the ont release mapping file</td></tr>
  <tr><td>warn</td><td>Maximum warning threshold to indicate OOS onts </td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                             <li>All fields are mandatory when option is used.</li>
                         <ul<br>
                         e.g. --v3auth user:password:: will result in DM5 as protocol.</td></tr>
  <tr><td>v3priv</td><td>optional setting when using SNMPv3 privacy protocol providing protocol and password<br>
                         <ul><li>Privacy protocol is DES, 3DES or AES, DES is default protocol.</li>
                         </ul></td></tr>
</table>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> | --id <id> [--sessions <nr>] [--seplog | --noseplog]]\\ \n";
  print STDERR "       [--v] [--debug] [--detail] [--incltoos] [--multi] [--noname] [--warn <Threshold>]\\ \n";
  print STDERR "       [--ontreleasemap <path>] [--comm <SNMP community string>] [--v2]\\ \n";
  print STDERR "       [--v3auth <user>:<password>:<authProtocol>] [-v3priv <privProtocol>:<password>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         id       : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                    The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         v             : print version and exit\n";
  print STDERR "         sessions      : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         seplog        : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noseplog      : keep all NE logging in a single child log file (default is a single logfile per NE)\n";
  print STDERR "         detail        : print the mismatches\n";
  print STDERR "         incltoos      : include Lines OOS of LTs OOS (default excluded)\n";
  print STDERR "         noname        : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         multi         : include Unix username in NE logging directory path\n";
  print STDERR "         warn          : Maximum warning threshold to indicate OOS onts\n";
  print STDERR "         ontreleasemap : path of directory where ONT Release Mapping file is located\n";
  print STDERR "         SNMP settings :\n";
  print STDERR "         comm          : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is \"public\"\n";
  print STDERR "         v2            : Use SNMPv2c\n";
  print STDERR "         v3auth        : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                         The password should be at least 8 characters long.\n";
  print STDERR "                         Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                         All fields are mandatory when an option is used.\n";
  print STDERR "                         e.g. --v3auth user:password:: will result in MD5 as protocol.\n";
  print STDERR "         v3priv        : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                         Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  exit(CMD_INCOMPL_CODE);
}

=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $sessions, $debug, $hosts, $logFile, $version, $sepLog, $detailed, $makeFile, $includeLtOos, $multi, $warnThreshold, $noName, $v3Auth, $v3Priv, $snmpV2, $communityString, $noSepLog, $relMapDirName, $id);
  my %params;
  my $options = GetOptions('isam=s', \$isam,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'logfile=s', \$logFile,
			   'v', \$version,
                           'v2', \$snmpV2,
			   'seplog', \$sepLog,
                           'noseplog', \$noSepLog,
			   'detail', \$detailed,
			   'make', \$makeFile,
                           'ontreleasemap=s', \$relMapDirName,
                           'warn=i', \$warnThreshold,
			   'incltoos', \$includeLtOos,
			   'multi', \$multi,
                           'v3auth=s', \$v3Auth,
                           'v3priv=s', \$v3Priv,
			   'noname', \$noName,
		           'comm=s', \$communityString,
                           'id=s', \$id);
  #-- check options
  if (! $options) {
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  if (defined($makeFile)) {
    ErrorCodes::MakeWarningFile("CHKC", dirname($0), $toolData{WARNINGFILE});
    exit(SUCCESS_CODE);
  }
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }
  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
	"($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }
  if ( defined($relMapDirName) ) {
     if($relMapDirName ne ""){
        $toolData{ONTRELDIR} = $relMapDirName;
     } 

     if(! -r "$toolData{ONTRELDIR}/$toolData{ONTRELMAPFILE}"){
       print STDERR "ERROR: ONT Release mapping file not found in the location $toolData{ONTRELDIR}. \n";
       Usage();
     }
     $ontRelMapOpt = 1; 
  }

  if (defined($warnThreshold) && (0 > $warnThreshold || $warnThreshold > 100)) {
    print STDERR "Error: Invalid value specified for warn : $warnThreshold\n";
    Usage();
  } elsif (defined($warnThreshold)) {
    $params{WARNTHRESHOLD} = $warnThreshold;
  } else {
    $params{WARNTHRESHOLD} = 0;                 
  }


  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if (defined($isam));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if (defined($debug));
  $params{DETAILED} = 1 if (defined($detailed));
  $params{INCLLTOOS} = 1 if (defined($includeLtOos));
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
    $params{ONT_STATUSFILE} = $id . "_" . $ID_FILENAME . ".status";
  } else {
    $params{LOGFILE} = $logFile;
    $params{ONT_STATUSFILE} = $logFile . ".status";
    if($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
     my $logdir = $1;
     if (! -d $logdir) {
      if (!mkdir( $logdir, 0777)) {
	print STDERR "Cannot create directory '$logdir': $!\n";
        exit CREATE_DIR_FAILED_CODE;
      } 
     }
    }
  }
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{NONAME} = defined($noName) ? 1 : 0;
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : ""; #ALU02028455
  if (!defined($snmpV2)) { #ALU02028455 #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }
  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA256|SHA512|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
          if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
             $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
	} else {
	  print STDERR "SNMPv3 authentication protocol can only be SHA or SHA256 or SHA512 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = $toolData{SNMPV3_USERNAME};
    $params{SNMP}{V3}{AUTH}{PASSWORD} = $toolData{SNMPV3_AUTH_PASSWORD};
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
    if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
       $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {  
     my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{PRIV}{PROTO} = $toolData{SNMPV3_PRIV_PROTOCOL};
    $params{SNMP}{V3}{PRIV}{PASSWORD} = $toolData{SNMPV3_PRIV_PASSWORD};    
  } #ALU01524313-indhu-15/12/2011
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }

  return \%params;
}

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut

sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw( 
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli IsamTL1 Octopus ReduceSWP 
                       StatusCheckCmpIsam7302 StatusCheckCmpIsam7302Etsi StatusCheckCmpIsam7302EtsiNtt StatusCheckCmpIsam7302Ansi
                       StatusCheckCmpIsam7302HiCap StatusCheckCmpIsam1540 StatusCheckCmpIsam7354 StatusCheckCmpIsam7367 StatusCheckCmpIsam7367_SX 
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

=head2 PrintOptionsSummary($$$;$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$;$) {
  my ($argLogFH, $logOnScreen, $config) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
            
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Warning threshold" , $params->{WARNTHRESHOLD} ) ) if (defined($params->{WARNTHRESHOLD}));
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Detailed reporting (mismatch)" , "Enabled" ) )  if defined($params->{DETAILED});;
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "LTs OOS in line OOS count" , "Include" ) )  if defined($params->{INCLLTOOS});;

  Logger::LogWish( $logMode, $argLogFH, "\n" );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Current Work Directory" , getcwd() ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Work space Directory" , "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Result space Directory" , "$toolData{RESULTSPACE}" ) );
  Logger::LogWish( $logMode, $argLogFH, "\n" );
}

=head2 CheckONTRelMapFileExist($)

=begin html
Check whether the ONT Release Map File exists when the ONT Release Map file option is not given.  

=end html

=cut

sub CheckONTRelMapFileExist($)
{
  my ($OperFileName) = @_;
  my  $ONT ="ONT Status";
  my $match = `grep "$ONT" $OperFileName`;
  if($match) {
    return 0  if(! -r "$toolData{ONTRELDIR}/$toolData{ONTRELMAPFILE}");
  }
  return 1; 
}


#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 CompareIsamChecks($$$$$)

=begin html
The main body of the scipt defining the complete status-check process:
<ul>
  <li>Read, parse and store pre-check results</li>
  <li>Read, parse and store post-check results</li>
  <li>Compare pre- and post-checks results and report analysis as success, warning or error.</li>
</ul>
<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by this child process.  Every
      array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
</table>

=end html

=cut

sub CompareIsamChecks($$$$) {
  my ( $isams, $sessionCount, $config, $warnLevels) = @_;
  my $isam;
  my $ret;
  my ($time1, $time2);
  my $startTime;
  my $statfile;
  my $nodeStatusLog;
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $errorCount;
  my $warningCount;
  my $dir;
  my $thisTftpDir;
  my $fxrData;
  my $prevErrorCount = 0;
  my $prevWarningCount = 0;
  my $nodeLogFile = undef;


  FamilyCommon::SetOwnPid($$);
  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetConfigData($config);
  FamilyCommon::SetWarningLevels($warnLevels);

 ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );

      FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Adjust warning and child logfile handles.
      FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
      StatusCheckCmpIsam7302HiCap::ResetGlobals();
      StatusCheckCmpIsam7363::ResetGlobals();
      ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
      Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});

      PrintOptionsSummary($nodeLogFH, undef,$config);
  }

  my $isamCount = -1;
  foreach $isam (@{$isams}) {
    $isamCount++;

    my $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1]; # Store for IncrErrorCount
    FamilyCommon::SetFailedNode($isamNode);

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

    ##### Create ResultSpace directory #####
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );

    if ( defined($params->{ID})) {
        $logExecOrNode = 0;
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif ( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
        PrintOptionsSummary($nodeLogFH, undef,$config);
    }


    FamilyCommon::SetFailedNode($isamNode);
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Adjust warning and child logfile handles.
    FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
    StatusCheckCmpIsam7302HiCap::ResetGlobals();
    StatusCheckCmpIsam7363::ResetGlobals();
    ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
    Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    $params->{USESSH} = 0; #for each isam the value is reset
    PrintOptionsSummary($nodeLogFH, undef);

    $prevErrorCount = FamilyCommon::GetErrorCount();
    $prevWarningCount = FamilyCommon::GetWarningCount();

    my $ontStatusFile = "$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}"; 
    if($params->{LOGFILE} =~ /([^\/]+)\/(.*)/) {
      my $logdir = $1;
      my $dirName = "$toolData{RESULTSPACE}/$isamNode/$logdir"; 
      if (! -d $dirName) {
       if (!mkdir( $dirName, 0777)) {
         my $str = "Cannot create directory '$dirName': $!\n";
         Logger::LogError($execLogFH, $str);
         exit CREATE_DIR_FAILED_CODE;
       } else {
         $ontStatusFile = "$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}"; 
       } 
      }
    }

    if (!open($statfile, ">$ontStatusFile")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create ONT status file '$ontStatusFile' for $isamNode: $!\n");
      exit CREATE_FILE_FAIL_CODE;
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : ONT status file '%s%s'\n",
                                ( $ontStatusFile =~ /^\//) ? "" : getcwd()."/",
                                $ontStatusFile ) );
    }
    $statfile->autoflush(1);
    FamilyCommon::SetFailontlistFile($statfile);

    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
	      FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError(), undef );
	      next;
    }
    $snmpStatus = SNMPComm::MaximizeMsgSize();
    if( $snmpStatus ) {
         FamilyCommon::ReportStatus( EC_SNMP_MAX_SIZE_FAIL, SNMPComm::GetError(), undef );
         next;
    }
    my $mibVersion = SNMPComm::GetValue( 'MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError(), "Failed to obtain MIB version" );
         next;
    }
    
    Logger::LogTrace($nodeLogFH, "Attempt to get file transfer data...\n");
    
    $fxrData = ToolCommon::GetFxrSettings($session, $toolData{RETRIES});
    if (not defined($fxrData)) {
      Logger::LogTrace($nodeLogFH, "obtained empty  file transfer protocol details\n");
      FamilyCommon::ReportStatus( EC_DETERMINE_LOG_FAILED, undef, undef );
      next;
    } else {
      if($fxrData->{PROTOCOL} != TFTP_FXR) {
        $params->{NOTFTP}=1;
	    Logger::LogTrace($nodeLogFH, "  OK\n");
      }
    }
    
    #-- Create work dir
  #  umask(0); #directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
    my $tmpDir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}";

    if (! -d $tmpDir) {
      if (!mkdir( $tmpDir, 0777)) {
        Logger::LogError($execLogFH, "Cannot create directory $tmpDir: $!\n" );
        exit CREATE_DIR_FAILED_CODE;
      } else {
        Logger::LogDebug($execLogFH, "Logging to $tmpDir\n");
      }
    }
    $dir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isamNode";
    $thisTftpDir = "$toolData{TFTPWORKDIR}/$isamNode";
    
    
    $dir .= "/$userName" if $params->{MULTI};
    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
     if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $nodeLogFH, $sessionCount, 0));
    if ( ! -d $dir) {
         FamilyCommon::ReportStatus( EC_NO_INPUT_DIR, undef, "Directory '$dir'");
         next;
    } else {
      if (! -W $dir) {
	     Logger::LogError($execLogFH, sprintf("Session %d: **FATAL** working dir '%s' for '%s' can't be written by user %s\n",
	     			     $sessionCount, $dir, $isamNode, getpwuid($<)) );
	     CORE::exit(WORK_NOT_WRITEABLE_CODE);
      }
    }
    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile = sprintf("checkCmp_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
        Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        PrintOptionsSummary($nodeLogFH, undef, $config);
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
        Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});

    }
    FamilyCommon::SetNeStorage($dir);

    $toolData{STATUSFILE} = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    if (! open($nodeStatusLog, ">$toolData{RESULTSPACE}/$isamNode/$toolData{STATUSFILE}")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$toolData{STATUSFILE}' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE);
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$toolData{STATUSFILE}" ) );
    }
    Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Ont configuration mismatch file '%s%s'\n",
                              ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                              "$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}" ) );

    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    if (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamFamily())) {
      StatusCheckCmpIsam7302::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamEtsiFamily())) {
      StatusCheckCmpIsam7302Etsi::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamEtsiNttFamily())) {
      StatusCheckCmpIsam7302EtsiNtt::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamLSFamily())) {
      StatusCheckCmpIsam1540::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamAnsiFamily())) {
      StatusCheckCmpIsam7302Ansi::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamHiCapEtsiFamily())) {
         if(!$ontRelMapOpt) { 
            my $OperCheckFile = sprintf(("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamHiCapEtsiFamily())); 
            if(!CheckONTRelMapFileExist($OperCheckFile)) {
              my $t = sprintf("ONT Release Map File does not exist in $toolData{ONTRELDIR}/$toolData{ONTRELMAPFILE}\n"); 
	          FamilyCommon::ReportStatus( EC_ONT_RELMAP_MISSING, "$t", undef );
              next; 
            } 
         }
      StatusCheckCmpIsam7302HiCap::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::IsamHiCapAnsiFamily())) {
      StatusCheckCmpIsam7302HiCap::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::RU7354Family())) {
      StatusCheckCmpIsam7354::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::Isam7367Family())) {
      StatusCheckCmpIsam7367::Compare();
    } elsif (-e sprintf("$dir/%s-%s.txt", PREOPERCHECKFILETAG, Capability::Isam7367_SXFamily())) {
      StatusCheckCmpIsam7367_SX::Compare();
    } else {
      FamilyCommon::ReportStatus(EC_NO_STATUS_FILE,undef, sprintf("No status file ($dir/%s-%s.txt) found for this ISAM\n",
                                   PREOPERCHECKFILETAG,
                                   Capability::IsamFamily()) );
      Logger::LogTrace($nodeLogFH, "Dumping directory contents:\n");
      my $dirHandle;
      opendir($dirHandle, $dir);
      my @fileList = readdir($dirHandle);
      closedir($dirHandle);
      foreach my $fileName (@fileList) {
        Logger::LogTrace($nodeLogFH, "  '$fileName'\n");
      }
      next;
    }

    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );

    $errorCount = FamilyCommon::GetErrorCount();
    $warningCount = FamilyCommon::GetWarningCount();
    if ((!($errorCount - $prevErrorCount)) && (!($warningCount - $prevWarningCount))) {
       FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
    }

    if ($params->{SEPLOG}){
        FamilyCommon::CloseChildLogFile();
        LogCloseFile($nodeLogFH);
    }
  }
  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }

  if (FamilyCommon::GetWarningCount()) {
      my $isamNodes = FamilyCommon::GetWarnedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d warning(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetWarningCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }

  if (!defined($params->{ID})) {
    if (! $params->{SEPLOG}) {
        FamilyCommon::CloseChildLogFile();  
        LogCloseFile($nodeLogFH);
    }
  }

  close $statfile if (defined($statfile));
  $statfile = undef;

  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  if ($errorCount) {
    CORE::exit(EXIT_CHILD_FATAL_CODE);
  } elsif ($warningCount) {
    CORE::exit(EXIT_WARN_CODE);
  } else {
    CORE::exit(SUCCESS_CODE);
  }
}

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file).

=end html

=cut

sub AdjustToolVars($) {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{MAXSESSIONS} = $config->{'MAXCMPSESSION'} if defined($config->{'MAXCMPSESSION'});
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'}); # -- 1.0.3
  $toolData{STATUSFILE} = $config->{CHKC_STATUS_FILE} if defined($config->{CHKC_STATUS_FILE});
  $toolData{LINEOOSPCT} = $config->{LINE_MM_PCT} if defined($config->{LINE_MM_PCT});
  $toolData{LTOOSPCT} = $config->{LT_MM_PCT} if defined($config->{LT_MM_PCT});
  $toolData{AUTHOOSPCT} = $config->{AUTH_MM_PCT} if defined($config->{AUTH_MM_PCT});
  $toolData{BRIDGEMMPCT} = $config->{BRDG_CONFMM_PCT} if defined($config->{BRDG_CONFMM_PCT});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
										 ($config->{NT_COMMUNITY} ne ""));
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});
  $toolData{STATUSFILE} = $config->{CHKC_STATUS_FILE} if defined($config->{CHKC_STATUS_FILE});
  $toolData{WARNINGFILE} = $config->{CHKC_WARN_FILE} if defined($config->{CHKC_WARN_FILE});
  $toolData{ONTRELDIR} = $config->{'ONTRELDIR'} if defined($config->{'ONTRELDIR'});
  $toolData{ONTRELMAPFILE} = $config->{'ONTRELMAPFILE'} if defined($config->{'ONTRELMAPFILE'});

}

#==============================================================================
#== Main program
#==============================================================================
my $sessionCount = 0;
my %pids;
my $pid;
my $warnCount = 0;

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string if !defined($warnLogFile);
  Logger::LogTrace($warnLogFile, $string) if defined($warnLogFile);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (keys %pids) {
    print STDERR "  Aborting children...\n";
    Logger::LogTrace($warnLogFile, "  Aborting children...\n") if defined($warnLogFile);
    foreach my $pid (keys %pids) {
      if ($pids{$pid} > 0) {
	print STDERR "Killing process $pids{$pid}...\n";
	Logger::LogTrace($warnLogFile, "Killing process $pids{$pid}...\n") if defined($warnLogFile);
	kill($pids{$pid});
      }
    }
  }
  FamilyCommon::UpdateNodeStatusLog( $exitCode, $signal );
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();
  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($warnLogFile, "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr") if (defined($warnLogFile));
    print STDERR "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]: $msgStr\n";
    $warnCount++;
    my $framePresent = 1;
    my $i = 0;
    my $ownPackage;
    my $ownFileName;
    my $ownLine;
    my $ownRoutine;
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
	print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  print $warnLogFile "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr" if (defined($warnLogFile));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my ($package, $fileName, $line) = caller();
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  print $warnLogFile "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr" if (defined($warnLogFile));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};

#-- Read parameter config file and adjust tool defaults if needed -- BB--
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit;
}
AdjustToolVars(\%config);
FamilyCommon::SetConfigData(\%config);
# -- End modif BB

my %warnLevels = Utils::ReadConfig("$dirName/$toolData{WARNINGFILE}");
FamilyCommon::SetWarningLevels(\%warnLevels);

#-- Handle command line
$params = ParseCommandLine(\%config);

FamilyCommon::SetOwnPid($$);
STDOUT->autoflush(1);
$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1, \%config);


#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  Logger::LogTrace($execLogFH, "Reading Hostfile...\n", 1);
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

#umask(0);
umask(0) if ( $params->{UMASK_CHMOD} );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );

my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      $pids{$pid} = $sessionCount; 
    } else {
      CompareIsamChecks( \@isamPart, $sessionCount, \%config, \%warnLevels);
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
if (keys %pids) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = (defined($pids{$kid})) ? $pids{$kid} : -1;
      Logger::LogInfo($execLogFH, sprintf("Session %d: exit code: '%d' (%s), signal: '%d'\n",
				     $childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal), 1);  
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogTrace($execLogFH, "All childs finished, exiting\n", 1);
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
