#!/usr/bin/perl

=head1 ISAM SW download

=begin html
This script will download a new SW package onto the ISAM taking care of all the steps needed in the process (helicopter view):
<ul>
  <li>perform a number of pre-checks to make sure NT HW is OK to accomodate new SW</li>
  <li>perform online SW reduction if required</li>
  <li>download new OSWP onto the node</li>
</ul>

The operations of the script are influenced by:
<ul>
  <li>command line options</li>
  <li>a parameter configuration file</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         swdl.pl
#-- Date:         06.11.2005
#-- Company:      Alcatel-Lucent
#-- Authors:      Donny Janssens for the initial version
#--               Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to download new OSWP to ISAMs
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2005/11/06 : First implementation
#--   1.0.1 - 2005/12/15 : Added unix ping in stead of udp ping.  Caused by
#--                        BDFhw70276
#--                        Added a regexp for determining major and minor
#--                        release ids.
#--   1.0.2 - 2006/01/16 : Include a call to ReduceSwp.pl for R2.4 upgrades
#--   1.0.3 - 2006/01/20 : Node prefix configurable.
#--                        AMS can't do node name to IP address convertion (
#--                        ip address/node name not stored in /etc/hosts)
#--   1.0.4 - 2006/02/13 : SW DL from active SW R2.1 possible.
#--   1.0.5 - 2006/02/27 : SW DL from active SW R2.0 possible.
#--   1.2.0 - 2006/03/16 : Intoduction of the capability model
#--   1.2.1 - 2006/03/19 : SW reduction needs to be done in case active release
#--                        is R2.3 and new release is R3.0 or higher (BDFhw92378)
#--                        Verify environment
#--   1.2.2 - 2006/05/31 : Using --debug results in failure (undefined value).
#--   1.2.3 - 2006/06/08 : ECNT-C NT SW file has other prefix the ENCT-A
#--                        Compare basenames of OSWP instead of full name.
#--   1.2.4.1 - 2006/07/25 : ANTms06883 - remove CLI logfile frmo /tmp on exit
#--                          of emergency exit
#--   1.2.4.2 - 2006/08/21 : ANTms08417 - do not check on SHUB SW size for releases
#--                          after R2.2
#--   1.2.5.1 - 2006/08/23 : ANTms10868 - create log file per NE in stead of
#--                          appending all in the child logfile.
#--             2009/09/25 : Set to type-based CLI positioning.
#--             2006/09/27 : ANTms15715: take transaction before backing up
#--                          NT database.
#--             2006/09/29 : Include program warnings in log files.
#--   1.4.2   - 2006/10/09 : ANTms17315: info about migration or upgrade
#--                          issues.
#--   1.4.3   - 2006/10/18 : Timestamp logging
#--   1.5 1.1 - 2007/01/09 : ANTms30214 - Do not abort passive OSWP is set to NO_OSWP.
#--   1.5.1.2 - 2007/01/29 : Include --force.  Force an online SW reduction.
#--   1.5.1.3 - 2007/02/02 : Allow download of the same OSWP from R3.1 onwards.
#--   1.5.1.4 - 2007/05/04 : ANTms49911: Small correction in Ctrl-c handling (don't kill pid
#--                          that have already exited)
#--   1.5.2 - 2007/06/18   : ANTms51632 allow multiple users for same ISAM.
#--   1.5.3   - 2007/06/22 : ANTms52362 - Make sure node status result file is never
#--                          empty (Added AdvanceToNext).
#--   1.5.4   - 2007/06/26 : ANTms51632 allow multiple users for same ISAM.
#--                          Corrected incorrect check.
#--   1.5.5   - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.5.6   - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.5.7   - 2007/08/22 : ANTms60869: Make NT community string configurable
#--   1.5.8   - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.5.9   - 2008/03/04 : - ANTms86893, error in IP regular expression
#--                          - Ignore unknown stdby NT when status is not operational.
#--   1.5.10  - 2008/04/29 : ANTmt04599.  NANT-A TYPE-B SW name changed between R3.2 and R3.3
#--                                       Implement logic for SW name retrieval in case of
#--                                       'reduced' SW
#--   1.5.11  - 2008/05/06 : ANTmt05829.  Check ip address immediately after parsing the OSWP file
#--                                       in ParseCommandLine.
#--   1.5.12  - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.5.13  - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--             2008/07/25 : Add check on file transfer protocol being used in the ISAM.
#--   1.5.14  - 2008/10/01 : ANTmt25573.  Include a workaround for files marked as not available after SW download.
#--   1.5.15  - 2008/10/27 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.6.0   - 2008/09/05 : ANTmt44696.  R3.7.10 and IPD ISAM family.
#--   1.6.1   - 2009/03/10 : ANTmt18506.
#--   1.6.2   - 2009/03/18 : ANTmt48688.
#--   1.6.3   - 2009/04/30 : ANTmt55717.  Introduced NE famile ISAM_BOTH to indicate NTs that can be part of both the ETSI and ANSI stream.
#--   1.6.4   - 2009/05/19 : ANTmt59034: Change $config{USESSH} to $config{USE_SSH}
#--   1.6.5   - 2009/05/25 : ANTmt59036.  Check security access settings and force usage of SSH if no telnet access for CLI possible.
#--   1.6.6   - 2009/05/26 : ANTmt60011
#--   1.6.7   - 2009/06/02 : ANTmt60891.
#--   1.6.8   - 2009/06/02 : ANTmt61239 - Added 1540LS NE family.
#--   1.6.9   - 2009/06/10 : ANTmt62662, allow public-key authentication with SSH
#--   1.6.10  - 2009/06/26 : ANTmt65332.  Introduction of the NTT ETSI family
#--   1.6.11  - 2009/08/25 : Remove ssh option from command-line option help (and option; accept it but ignore)
#--   1.6.12  - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.6.13  - 2009/11/12 : ANTmt86148: 7354RU changes
#--   3.0.00  - 2010/04/21 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01  - 2010/04/21 : Converged PBMT Changes
#--                        : x86 Changes
#--   3.0.02  - 2010/10/15 : ALU00808688 new LRM option --id (suppresses the --logfile option).  Must not be used in combination with --hosts
#--   3.0.03  - 2010/11/25 : to resolve problem of child log file being written to parent log.
#--   3.0.04  - 2011/01/11 : LRM updates for lib-perl path
#--   3.0.05  - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.06  - 2011/04/01 : Changes for 7354RU
#--   3.0.07  - 2011/04/05 : ALU01031457 - Support for R4220 & R4230
#--   3.0.08  - 2011/05/02 : ALU01178887
#--   3.0.09  - 2011/06/10 : Changes for RLAB issue of not able to open descriptor file
#--   3.0.10  - 2011/10/24 : logfile command line parameter can include a directory
#--   3.0.11  - 2012/02/22 : ALU01755206
#--   3.0.12  - 2012/03/26 : ALU01748774
#--   3.0.13  - 2012/05/14 : ALU01789702 - World writeable directories need sticky bit set 
#--   3.0.14  - 2012/05/30 : to correct the display of snmp option used in logfile
#--   3.0.15  - 2012/07/23 : ALU01788919 
#--   3.0.16  - 2012/10/22 : ALU01761938
#--   3.0.17  - 2012/11/06 : ALU01837839
#--   3.0.18  - 2013/02/01 : enable usage of second password if first one fails for cli over ssh
#--   3.0.19  - 2013/02/08 : corrected issue related to picking of second password from config file
#--   3.0.20  - 2013/02/15 : undef second password in params when it is set as empty
#--   3.0.21  - 2013/04/29 : ALU01880147
#--   3.0.22  - 2013/07/09 : ALU01893590
#--   3.0.23  - 2013/08/20 : NGVR support 
#--   3.0.24  - 2013/11/30 : ALU02028455 
#--   3.0.25  - 2015/05/25 : ALU02060588
#--   3.0.26  - 2015/06/10 : ALU02083077
#--   3.0.27  - 2015/08/04 : ALU02104915
#--   3.0.28  - 2015/10/28 : ALU02139820
#--   3.0.29  - 2015/11/02 : ALU02142016
#--   3.0.30  - 2015/12/14 : ALU02156544
#--   3.0.31  - 2015/12/30 : ALU02160793
#--   3.0.32  - 2016/04/28 : ALU02206611
#--   3.0.33  - 2016/05/13 : ALU02202972, ALU02213657
#--   3.0.34  - 2016/07/15 : ALU02237966
#--   3.0.35  - 2016/09/28 : ALU02278259
#--   3.1.0   - 2018/06/20 : ALU02285181
#--   3.1.0   - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.1.1   - 2019/03/24 : ALU02559738

=head1 VERSION

=begin html
Current version: "swdl.pl - 3.1.1 -- 2019/03/24"

=end html

=cut

my $VERSION = "swdl.pl - 3.1.1 -- 2019/03/24";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP<a/></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2156-DFZZA">Isam7354RUCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use File::stat;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$|=1;
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
  STDOUT->autoflush(1);
}


use lib "$scriptDir/lib"; 
use PBMTGlobal;

use Net::TFTP;

use Logger;
use IsamCli;
use IsamTL1;
use Isam7354RUCli;
use Crypt::Lite;
use Utils;  # -- BB --
use Octopus;
use Capability;
use ToolCommon qw(:CONSTANTS);
use PbmtConfig;
use ErrorCodes qw(:EXIT_CODES :SWDL);
use FamilyCommon;
use DownloadIsamIHUB;
use DownloadIsamSHUB;
use DownloadIsam7354;
use DownloadIsam1540;
use ToolCommon();
use PBMTGlobal;
use SNMPComm;
use version qw(:ALL);

### END BLOCK to have smooth closure
END {
  Logger::LogHandleClose($nodeLogFH);
  Logger::LogHandleClose($execLogFH);
}

#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$$);
sub HandleIsams($$$);
sub AdjustToolVars($);
sub CleanUp($;$);
sub DynamicOSWPVariant($$$);

#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my %toolData = (HOSTSFILE =>       "/etc/hosts",
		TFTPDIR =>         "/var/opt/aws/equipment",
		SHUBSWPATH =>      "",
		SHUBSWNAME =>      "",
		TFTPWORKDIR =>     "tmp",
		MAXSESSIONS =>     5,
		USERNAME =>        "",
		PASSWORD =>        "",
		PASSWORD2 =>        "",
		TL1USER =>     "",
		TL1PWD =>     "",
		RETRIES =>         5,
		PINGWAIT =>        10, # seconds -- 1.0.1
		OCTOPUSPATH =>     "tools/octopus", # -- 1.0.2
		OCTOPUSSCRIPTS =>  "tools/octopus",
		NODEPREFIX =>      "AS-",  # -- 1.0.3
		TFTPTIMEOUT =>     5,
		TELNETTIMEOUT =>   10,
		NT_COMMUNITY =>    "public",
		TL1UDPPORT =>      13001,
		# FXRDATA is a pointer to hash being filled in at a later stage.  The hash contains the following:
		# 'PROTOCOL': set to the filetransfer protocol being used by the ISAM
		# 'SERVERS': the filetransfer server used by the ISAM.  Only set when the protocol is different from tftp.
		#            Is a hash consisting of a list of IP-addresses and usernames (only username per filetransfer server is possible).
		FXRDATA =>         undef);

my $STATUSFILE =       "result.txt";
my $ID_FILENAME =      "SWDL";
my $oswpDynamicDir = "";
my $logExecOrNode = 0;
my $logScreenPrefix= "";

=head1 USAGE

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] swdl.pl --isam &lt;ip&gt; | --hosts &lt;file&gt; --swp &lt;path-to-oswp-descriptor&gt; --logfile &lt;file&gt; \ <br>
                    [--sessions &lt;nr&gt;] [--force] [--seplog | --noseplog] [--continue] [--reduce] [--multi] [--noname] [--haip &lt;ip&gt;] [--notftp] \ <br>
                    [--username &lt;user&gt;] [--password &lt;password&gt;] [--v] [--debug] [--comm &lt;SNMP community string&gt;] [--v2] [--noping] \ <br>
                    [--v3Auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;] [-v3priv &lt;privProtocol&gt;:&lt;password&gt;] </tt>

=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>swp</td><td>path to oswp descriptor file (relative to TFTP root directory)</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>cliusername</td><td>ISAM CLI username</td></tr>
  <tr><td>clipassword</td><td>ISAM CLI password</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td></tr>
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>force</td><td>skip migration path checks, download SW anyhow</td></tr>
  <tr><td>reduce</td><td>force an online SW reduction</td></tr>
  <tr><td>continue</td><td>continue SW download for other ISAMs in stead of exiting the current child process</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>haip</td><td>High available system IP address associated to the physical interface to the DSLAMs</td></tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed.</td></tr>
  <tr><td>v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                             <li>All fields are mandatory when option is used.</li>
                         <ul<br>
                         e.g. --v3auth user:password:: will result in DM5 as protocol.</td></tr>
  <tr><td>v3priv</td><td>optional setting when using SNMPv3 privacy protocol providing protocol and password<br>
                         <ul><li>Privacy protocol is DES, 3DES or AES, DES is default protocol.</li>
                         </ul></td></tr>
</table>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --swp <path-to-oswp-descriptor> [--ont <path-to-ont-sw>] --logfile <file> | --id <id> \\\n";
  print STDERR "       [--sessions <nr>] [--force] [--verify] [--seplog | --noseplog] [--continue] [--reduce] [--noname]  [--haip <ip>] [--noping] \\ \n";
  print STDERR "       [--cliusername <user>] [--clipassword <password>]  [--clipassword2 <password>] [--v] [--debug] [--multi] \\ \n";
  print STDERR "       [--comm <SNMP community string>] [--v2][--v3auth <user>:<password>:<authProtocol>] [-v3priv <privProtocol>:<password>] \\ \n";
  print STDERR "       [--bytpe  <value>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         id       : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                    The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         swp      : path to oswp descriptor file (either relative or absolute)\n";
  print STDERR "         ont      : path to ONT SW (either relative or absolute)\n";
  print STDERR "         sessions : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         cliusername : ISAM CLI username\n";
  print STDERR "         clipassword : ISAM CLI password\n";
  print STDERR "         clipassword2 : Second ISAM CLI password (option to use a second password if first password fails).\n";
  print STDERR "                        The second password option will be used only when CLI over SSH is configured.\n";
  print STDERR "         seplog   : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noseplog : keep all NE logging in a single child log file (default is a single logfile per NE)\n";
  print STDERR "         force    : skip migration path checks, download SW anyhow\n";
  print STDERR "         verify   : Auto-verify the OSWP after download\n";
  print STDERR "         reduce   : force an online SW reduction\n";
  print STDERR "         continue : continue SW download for other ISAMs in stead of exiting the current child process\n";
  print STDERR "         noname   : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         multi    : include Unix username in NE logging directory path\n";
  print STDERR "         haip     : High available systems' IP address associated to the physical interface to the DSLAMs\n";
  print STDERR "         noping   : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "       SNMP settings:\n";
  print STDERR "         comm     : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is 'public'\n";
  print STDERR "         v2       : Use SNMPv2c\n";
  print STDERR "         v3auth   : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                    The password should be at least 8 characters long.\n";
  print STDERR "                    Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                    All fields are mandatory when an option is used.\n";
  print STDERR "                    e.g. --v3auth user:password:: will result in MD5 as protocol.\n";
  print STDERR "         v3priv   : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                    Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  print STDERR "         btype    : Build type to choose XDescriptor; Possible values are SNMP, YANG\n";
#  print STDERR "         noreduce : do not call script to reduce current active OSWP\n";
  exit CMD_INCOMPL_CODE;
}

=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $swp, $btype, $ont, $cliUserName, $cliPassword, $cliPassword2, $rlabOpt);
  my ($sessions, $debug, $hosts, $execLogFile, $noReduce, $forceDl, $dlShub, $version, $sepLog, $continue, $forceReduce, $multi, $wipe, $noName,
      $communityString, $haIp, $noPing, $v3Auth, $v3Priv, $noSepLog, $useTL1, $snmpV2, $useSsh, $id, $verify);
  my $regExp;
  my $dirName = getcwd();
  my $realPath = dirname(realpath($0));
  my %params;
  my $dbVersion;
  my $options = GetOptions('isam=s', \$isam,
			   'swp=s', \$swp,
			   'btype=s', \$btype,
			   'ont=s', \$ont,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'username|cliusername=s', \$cliUserName,
			   'password|clipassword=s', \$cliPassword,
			   'clipassword2=s', \$cliPassword2,     # undocumented, for 7354
			   'logfile=s', \$execLogFile,
			   'noreduce', \$noReduce,
			   'reduce', \$forceReduce,
			   'force', \$forceDl,
               'verify', \$verify,
			   'dlshub', \$dlShub,
			   'v', \$version,
			   'v2', \$snmpV2,
			   'seplog', \$sepLog,
			   'noseplog', \$noSepLog,
			   'continue', \$continue,
			   'multi', \$multi,
			   'wipe', \$wipe,
			   'noname', \$noName,
			   'comm=s', \$communityString,
			   'haip=s', \$haIp,
			   'noping', \$noPing,
			   'rlab', \$rlabOpt,                  # undocumented for RLAB
			   'v3auth=s', \$v3Auth,
			   'v3priv=s', \$v3Priv,
			   'ssh', \$useSsh,
			   'tl1', \$useTL1,
               'id=s', \$id);

  #-- check options
  if (! $options) {
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
#  print STDERR "getcwd: '$dirName', realPath: '$realPath'\n";
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($execLogFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();	
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }
  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }
  if (!defined($swp) && !defined($ont)) {
    print STDERR "ERROR: neither OSWP or ONT specified.  At least one needs to be provided\n";
    Usage();
  }

  if( (defined $hosts) and (!defined $btype ) ) {
     print STDERR "ERROR: --btype value must be passed for --hosts\n";
     Usage();
  }

  if( defined($btype) ) {
      if( $btype ne "SNMP" and $btype ne "YANG" ) {
         print STDERR "ERROR: Invalid value passed for --bytpe; possible values are SNMP, YANG\n";
         Usage();
      } else {
         $params{BTYPE} = $btype;
      }
  } elsif( defined $hosts ){
      $btype = "SNMP";
  }

  ### For (--verify input parameter ) auto-verify after download
  if (defined $verify) {
    $params{AUTOVERIFY} = 1;
  } else {
    $params{AUTOVERIFY} = ( (defined $config->{DOWNLOAD_VERIFY}) and $config->{DOWNLOAD_VERIFY} eq "Y" ) ? 1 : 0;
  }

 
  my $oswp2Consider = $swp;
  my $ont2Consider = $ont;
  $params{NOTFTP} = 0;
  if (defined($swp)) {
    # First try the file as specfied (could be an absolute path when using ftp e.g.
    if (! -r $oswp2Consider) {
      # Next try the SWP relative to the login home directory
      $oswp2Consider = "$ENV{HOME}/$swp";
      $oswp2Consider =~ s/\/\//\//g;
      if (! -r $oswp2Consider) {
	# Finally check as tftp
	$oswp2Consider = "$toolData{TFTPDIR}/$swp";
	$oswp2Consider =~ s/\/\//\//g;
	if (! -r  $oswp2Consider) {
	  print STDERR "ERROR: Accessing OSWP '$oswp2Consider' failed: $!\n";
	  Usage();
	} else {
	  $params{OSWP_HOME} = $toolData{TFTPDIR};
	}
      } else {
	$params{OSWP_HOME} = $ENV{HOME};
	$params{NOTFTP} = 1;
      }
    } else {
      if (index($toolData{TFTPDIR}, $dirName) != -1) {
	# PBMT installed relative to TFTP directory so still tftp
	$oswp2Consider = "$toolData{TFTPDIR}/$swp";
	$params{OSWP_HOME} = $toolData{TFTPDIR};
      } else {
	$params{OSWP_HOME} = "";
	$params{NOTFTP} = 1;
      }
    }
    if(defined($rlabOpt) && ($params{OSWP_HOME} eq "")) {
        $params{OSWP_HOME} = $toolData{TFTPDIR}; 
	$params{NOTFTP} = 0;
    }
    $params{OSWP_FULL} = $oswp2Consider;
    $regExp = SWPSYNTAX;
    if ($swp =~ /$regExp/) {
      print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_INV_CHARS_CODE));
      exit OSWP_FILE_INV_CHARS_CODE;
    }
    my $tftpInfo = Utils::FindTftpInfo(undef, $oswp2Consider);
    if (!defined($tftpInfo->{PRIMIP}) || ($tftpInfo->{PRIMIP} eq "")) {
      print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_NO_IP_CODE));
      exit OSWP_FILE_NO_IP_CODE;
    }
    my $baseName = basename($tftpInfo->{NAME});
    $regExp = REGEXP;
    if (!($baseName =~ /$regExp/)) {  # -- 1.0.4
      print STDERR sprintf("Software package name (%s, read from $swp) not conforming to regexp (%s).  Check param.cfg!!!\n",
			   $baseName, $tftpInfo->{NAME}, REGEXP);
      exit PARAM_CFG_NOT_OK_CODE;
    }
    $params{PRIMTFTPIP} = $tftpInfo->{PRIMIP};
    $params{SECTFTPIP} = $tftpInfo->{SECIP};
    $params{SWPNAME} = $tftpInfo->{NAME};
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  if (defined($cliUserName)) {
    $params{USERNAME} = $cliUserName;
  } else {
    $params{USERNAME} = $toolData{USERNAME};
  }
  if (defined($cliPassword)) {
    $params{PASSWORD} = $cliPassword;
  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }
  if (defined($cliPassword2)) {
    $params{PASSWORD2} = $cliPassword2;
  } else {
    $params{PASSWORD2} = ($toolData{PASSWORD2} ne "") ? $toolData{PASSWORD2} : undef ; 
  }
  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
	"($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }
  
#ALU01524313-indhu-15/12/2011
  if (defined($communityString) && defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  if (defined($communityString) && defined($v3Priv)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }
#ALU01524313-indhu-15/12/2011
  $params{UMASK_CHMOD} = ( (!defined $config->{UMASK_OVERRIDE}) or  $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if( defined($debug));
  LogEnableDebug() if(defined $debug);
  $params{OSWP} = $swp if (defined($swp));
  $params{ONT} = $ont if (defined($ont));
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
  } else {
    $params{LOGFILE} = $execLogFile;
    if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
       my $logdir = $1;
       if (! -d $logdir) {
         if (!mkdir( $logdir, 0777)) {
           print STDERR "Cannot create directory '$logdir': $!\n";
           exit CREATE_DIR_FAILED_CODE;
         }
       }
    }
  }
  $params{NOREDUCE} = defined($noReduce) ? 1 : 0;
  $params{FORCEDL} = defined($forceDl) ? 1 : 0;
  $params{FORCE}  = $params{FORCEDL};
  $params{DLSHUB} = defined($dlShub) ? 1 : 0;
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
  $params{FORCEREDUCE} = defined($forceReduce) ? 1 : 0;
  if (($params{FORCEREDUCE}) && ($params{NOREDUCE})) {
    print STDERR "WARNING: Disabling noreduce option.  Forced online SW reduction overrules\n";
    $params{NOREDUCE} = 0;
  }
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{WIPE} = defined($wipe) ? 1 : 0;
  $params{PHYSICAL_HOST_IP} = $config->{PHYSICAL_HOST_IP} if (defined($config->{PHYSICAL_HOST_IP}) &&
							      ($config->{PHYSICAL_HOST_IP} ne "0.0.0.0"));
  if (defined($haIp)) {
    if (ToolCommon::ValidIP($haIp)) {
      $params{PHYSICAL_HOST_IP} = $haIp;
    } else {
      print STDERR "'$haIp' is not a valid IP address\n";
      exit(PARAM_CFG_NOT_OK_CODE);
    }
  }
  $params{NONAME} = defined($noName) ? 1 : 0;
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : ""; #ALU02028455

  if (!defined($snmpV2)) { #ALU02028455 #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }
  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA512|SHA256|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
          if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
             $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
	} else {
	  print STDERR "SNMPv3 authentication protocol can only be SHA or SHA256 or SHA512 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
    $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
    if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
       $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {  
     my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    $params{SNMP}{V3}{PRIV}{PASSWORD} = defined($toolData{SNMPV3_PRIV_PASSWORD}) ? $toolData{SNMPV3_PRIV_PASSWORD} : "";
  } #ALU01524313-indhu-15/12/2011
  }
  if (($params{PASSWORD} eq "") || ($params{USERNAME} eq "")) {
    print STDERR "No username or password defined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }
  $params{NOPING} = 0;
  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'});
  $params{NOPING} = 1 if defined($noPing);

  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";

  $params{UPD_KNOWN_HOST_FILE} = 0;
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});

  my $oswpPath; 
  $oswpPath = "$params{OSWP_HOME}/$params{SWPNAME}"; 
  if($params{NOTFTP} == 1) {
    if($params{OSWP_HOME} ne "") {
        $oswpPath = "$params{SWPNAME}"; 
        $params{OSWP_HOME} = ""; 
    }
  }
  $params{SWPPATH} = $oswpPath;
  $dbVersion = ToolCommon::GetDBVersion($oswpPath); 
  if(!defined($dbVersion)) {
      print STDERR "DB Version is not defined in file $oswpPath \n";     
      exit FILE_OPEN_FAIL_CODE;
  } 
  $params{DBVER_TGTREL} = $dbVersion; 

  return \%params;
}

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut

sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  ### Library Category :-
  ### LS    : DownloadIsam1540.pm 
  ### PIZZA : DownloadIsam7354.pm
  ### SHUB  : DownloadIsamSHUB.pm (DownloadIsam7302Ansi.pm, DownloadIsam7302EtsiNtt.pm, DownloadIsam7302Etsi.pm, DownloadIsam7302.pm)
  ### IHUB  : DownloadIsamIHUB.pm (DownloadIsam7302HiCap.pm, DownloadIsam7363.pm, DownloadIsam7367.pm, DownloadIsam7367_SX.pm)

  my @libraryList = qw( 
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli IsamTL1 Isam7354RUCli Octopus ReduceSWP 
                       DownloadIsamIHUB DownloadIsamSHUB DownloadIsam7354 DownloadIsam1540
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

=head2 PrintOptionsSummary($$$$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$$) {
  my($argLogFH, $logOnScreen, $config) = @_;
  
  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "XDescriptor build type" , $params->{BTYPE} ) ) if defined($params->{BTYPE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "OSWP Descriptor file" , $params->{OSWP} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- SWP Name" , $params->{SWPNAME} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- DB Version" , $params->{DBVER_TGTREL} ) ) if defined($params->{DBVER_TGTREL});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ONT-SW Descriptor file" , $params->{ONT} ) ) if defined($params->{ONT});
  if ($config->{SHUBSWPATH} ne "") {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "SHUB SW Location" , $config->{SHUBSWPATH}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "SHUB SW file" , $config->{SHUBSWNAME}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Transaction IP Address" , 
			       defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} :  $params->{PRIMTFTPIP}) );
            
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "FTP Details :-") );
  if (! $params->{NOTFTP}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "TFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Timeout in seconds" , $toolData{TFTPTIMEOUT}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- TFTP (HOME) Directory" , $toolData{TFTPDIR}) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "SFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Active SW Reduction" , $params->{FORCEREDUCE} ? "Force" : "No" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Force SW Download" , $params->{FORCEDL} ? "Yes" : "No") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Verify After Download" , "Yes" ) ) if defined $params->{AUTOVERIFY} and $params->{AUTOVERIFY};
  if ($params->{CONTINUE}) {
    Logger::LogWish( $logMode, $argLogFH, "!!!WARNING!!!  USE AT OWN RISK!!!\n" );
    Logger::LogWish( $logMode, $argLogFH, "    Migration process will continue even in case an ISAM is lost in the process of migration" );
    Logger::LogWish( $logMode, $argLogFH, "    Hit Ctrl-C now if you do not want to continue\n!!!WARNING!!!\n" );
  }
  Logger::LogWish( $logMode, $argLogFH, "\n" );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Current Work Directory" , getcwd() ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Work space Directory" , "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Result space Directory" , "$toolData{RESULTSPACE}" ) );
  Logger::LogWish( $logMode, $argLogFH, "\n" );
}

sub DynamicOSWPVariant($$$) {
    my ($nodeLogFH, $config, $ntType) = @_;
    if( defined $params->{SWPPATH} and $params->{SWPPATH} ne "" ) {
        if( defined $config->{NT_SWP_MAPFILE} and $config->{NT_SWP_MAPFILE} ne "" ) {
           my $ntDescriptorMapFile = dirname($params->{SWPPATH}) . "/" . $config->{NT_SWP_MAPFILE};
           if( ! ( -f $ntDescriptorMapFile ) ) {
	        LogNotify($nodeLogFH, "NT SWP Mapping file :$ntDescriptorMapFile: not exists\n");
		    return 0;
           }
           Capability::SetVariantCheckSkip(1);
           Logger::LogDebug($nodeLogFH, "NT DescriptorFile mapping process started..:$ntDescriptorMapFile:\n");
           my( $status, $ntDescriptorFile) = Utils::FetchNTDescriptor($ntDescriptorMapFile, $ntType);
           Logger::LogTrace($nodeLogFH, "Details : $status, $ntType, $ntDescriptorFile\n");
           if( $status != 0 ) {
                 FamilyCommon::ReportStatus( EC_NTTYPE_MAPFILE_FAILED, undef, "NTDescriptor Mapfile : $ntDescriptorFile" );
	             return 1;
           }
           Logger::LogTrace($nodeLogFH, "XDescriptor build type:$params->{BTYPE}:\n") if defined($params->{BTYPE});
           my $regExp = REGEXP;
           if (!($ntDescriptorFile =~ /$regExp/)) {  # -- 1.0.4
                 FamilyCommon::ReportStatus( PARAM_CFG_NOT_OK_CODE, undef, 
                        "Software package name ($ntDescriptorFile, read from NT Descriptor Mapfile) not conforming to regexp (".REG     EXP.") as per param.cfg" );
	             return 1;
           }
           LogNotify($nodeLogFH, "NT DescriptorFile($ntDescriptorFile) found..\n");
           my( $oswpVariantU,$oswpVariantL, $oswpRelease ) = ("","","");
           my( $ntDescriptorVariantU,$ntDescriptorVariantL, $ntDescriptorRelease ) = ("","","","");
           my( $oswpRegexpMatch, $ntDescriptorRegexpMatch ) = (0,0);
           my $baseOSWP = basename($params->{OSWP});
           my $regexpVariantRelease = VARIANT_RELEASE_REGEXP;
           if( $baseOSWP =~ /$regexpVariantRelease/ ) {
                $oswpVariantU = $1; 
                $oswpVariantL = $2; 
                $oswpRelease = $3;
                $oswpRegexpMatch = 1;
           }
           if( $ntDescriptorFile =~ /$regexpVariantRelease/ ) {
                $ntDescriptorVariantU = $1;
                $ntDescriptorVariantL = $2;
                $ntDescriptorRelease = $3;
                $ntDescriptorRegexpMatch = 1;
           }
           if( $oswpRegexpMatch and $ntDescriptorRegexpMatch ) {
              if( $baseOSWP !~ /(AA|AN|ZA|ZN|$ntDescriptorVariantU$ntDescriptorVariantL|Z$ntDescriptorVariantL)(\d\d\.\d\d\d)$/ ) {
                 FamilyCommon::ReportStatus( EC_NTTYPE_OSWP_FAILED, undef, 
                                            "ntDescriptor($ntDescriptorFile), OSWP($baseOSWP)Variant Release Details are not same" );
	             return 1;
              } else {
	             Logger::LogTrace($nodeLogFH, "OSWP($baseOSWP), ntDescriptor($ntDescriptorFile) match the regex..\n");
              }
           }
           my( $swpVariantU,$swpVariantL, $swpRelease, $swpRegexpMatch, ) = ("","", 0);
           ($ntDescriptorVariantU,$ntDescriptorVariantL, $ntDescriptorRelease,$ntDescriptorRegexpMatch ) = ("","", 0);
           my $baseSWP = basename($params->{SWPNAME});
           if( $baseSWP =~ /$regexpVariantRelease/ ) {
                $swpVariantU = $1;
                $swpVariantL = $2;
                $swpRelease = $3;
                $swpRegexpMatch = 1;
           }
           if( $ntDescriptorFile =~ /$regexpVariantRelease/ ) {
                $ntDescriptorVariantU = $1;
                $ntDescriptorVariantL = $2;
                $ntDescriptorRelease = $3;
                $ntDescriptorRegexpMatch = 1;
           }
           if( $swpRegexpMatch and $ntDescriptorRegexpMatch ) {
               if( $baseSWP =~ /($ntDescriptorVariantU$ntDescriptorVariantL)(\d\d\.\d\d\d)$/ ) {
	             Logger::LogDebug($nodeLogFH, "NT-SWP DescriptorFile($ntDescriptorFile) and SWP basename($baseSWP) are same as per FULL DOWNLOAD, No need to generate new file..\n");
               } elsif ( $baseSWP =~ /(Z$ntDescriptorVariantL)$ntDescriptorRelease$/ ) {
	             Logger::LogDebug($nodeLogFH, "NT-SWP DescriptorFile($ntDescriptorFile) and SWP basename($baseSWP) are same as per MINIMIZED DOWNLOAD, No need to generate new file..\n");
               } elsif( $swpVariantU eq "Z" ) {
                   FamilyCommon::ReportStatus( EC_NTTYPE_SWPNAME_FAILED, undef, 
                                              "ntDescriptor($ntDescriptorFile), SWP($params->{SWPNAME})Variant Release Details are not same and it is Minimized SWP" );
	               return 1;
               } else {
	              LogNotify($nodeLogFH, "SWPNAME($baseSWP), ntDescriptor($ntDescriptorFile) are not same, Need to generate new file..\n");
                  my $newSWPNAME = "";
                  if( dirname($params->{SWPNAME}) ne "" and dirname($params->{SWPNAME}) ne "." ) {
	            $newSWPNAME = dirname($params->{SWPNAME}) . "/";
                  }
                  $newSWPNAME .=  $ntDescriptorFile;
                  my $newSWPFile = $newSWPNAME;
                  if( $params->{NOTFTP} == 0 ) {
 			$newSWPFile = $toolData{TFTPDIR} . "/" . $newSWPNAME;
                  }
                  if( ! -e $newSWPFile ) {
                         FamilyCommon::ReportStatus( EC_NTTYPE_SWPNAME_FAILED, undef, 
                                                    "NT-SWP Map, SWP file ($newSWPFile) not exists" );
	                     return 1;
		          }
             
                  if( defined $params->{OSWP_FULL} and $params->{OSWP_FULL} ne "" ) {
                     #my $OSWPDescriptorFile = dirname($params->{OSWP_FULL}) . "/pbmt_" . $$ ."/" . basename($params->{OSWP_FULL});
                     my $OSWPDescriptorFile = $oswpDynamicDir ."/" . basename($params->{OSWP_FULL});
                     Logger::LogTrace($nodeLogFH, "UpdateOSWPFile :  undef, $params->{OSWP_FULL}, $OSWPDescriptorFile, $ntDescriptorFile");
                     my ($updateStatus,$mesg)  = Utils::UpdateOSWPFile(undef,$params->{OSWP_FULL}, $OSWPDescriptorFile, $ntDescriptorFile);
                     if( $updateStatus != 0 ) {
                         FamilyCommon::ReportStatus( EC_NEW_OSWP_FILE_FAILED, undef, "OSWP Descriptor New file : $mesg" );
	                     return 1;
	                 } 
                     if( $params->{NOTFTP} == 0 ) {
                        $OSWPDescriptorFile =~ s/^$toolData{TFTPDIR}\///;
                     }
                     Logger::LogTrace($nodeLogFH, "Updated new OSWP NT:$ntType: based $OSWPDescriptorFile\n");
                     $params->{OSWP} = $OSWPDescriptorFile;
                     $params->{SWPNAME} = $newSWPNAME;
                     Logger::LogInfo($nodeLogFH, "NT-SWP Map : SWP name updated as :$params->{SWPNAME}:\n", 1 ); 
                     return 0;
                  } else {
                         FamilyCommon::ReportStatus( EC_NEW_OSWP_FILE_FAILED, undef, "Failed to generate new OSWP file" );
	                     return 1;
                  }
              }
          } else {
             ### DEVEL-NOTE : This check is not required, For safe guard we log this message
             LogNotify($nodeLogFH, "NT DescriptorFile (internal) regular expression failed; Continue normal process\n");
 	     Capability::SetVariantCheckSkip(0);
          }

        }
    }
    return 0;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 HandleIsams($$$)

=begin html
The main body of the script defining the complete SW download process.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by this child process.
      Every array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
</table>

=end html

=cut

sub HandleIsams($$$) {
  my ($isams, $sessionCount, $config) = @_;
  my $isam;
  my $ret;
  my ($time1, $time2);
  my $startTime;
  my $nodeLogFile;                             # Name of node specific log
  my $nodeStatusLog;                           # Filehandle of the node status result file
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $dirName = dirname($0);
  my $ntType;
  my $isamNode;
  my $dir;
  my $thisTftpDir;
  my $fxrData;

  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetOwnPid($$);
  umask(0) if ($params->{UMASK_CHMOD});
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetConfigData($config);
  SNMPComm::ClearSession();

  Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Started as '%d'\n", $sessionCount, FamilyCommon::GetOwnPid()));
  ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );
      PrintOptionsSummary($nodeLogFH, undef, $config);

      ### Switch debugging (if any) to an opened filehandle...
      FamilyCommon::SetWarnLogFile($nodeLogFH);   # Make sure warnings and
      FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
      IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Isam7354RUCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
      IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
      Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
  }

  my $paramsSWPNAME = $params->{SWPNAME};
  my $paramsOSWPNAME = $params->{OSWP};

  my $isamCount = -1;
  foreach $isam (@{$isams}) {
    $isamCount++;
    $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1];

    $params->{SWPNAME} = $paramsSWPNAME;
    $params->{OSWP} = $paramsOSWPNAME;
    ReduceSWP::ResetGlobals();
    Capability::ResetGlobals();
    IsamCli::ResetGlobals();
    SNMPComm::ResetGlobals();
    $oswpDynamicDir= "";
    FamilyCommon::SetFailedNode($isamNode); # Store for IncrErrorCount
    $params->{USESSH} = 0; #for each isam the value is reset

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

    ##### Create ResultSpace directory #####
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );
    if ( defined($params->{ID})) {
        $logExecOrNode = 0;
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif ( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
        PrintOptionsSummary($nodeLogFH, undef, $config);
    }
    # Switch debugging (if any) to an opened filehandle...
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and 
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Isam7354RUCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END


    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    if (! open($nodeStatusLog, ">$toolData{RESULTSPACE}/$isamNode/$STATUSFILE")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$STATUSFILE' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE" ) );
    }
    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }
    # Check whether an octopus session to this ISAM is open
    next if( Octopus::OctopusActive($params->{DEBUG}, $isam, $config->{GREP}, $config->{PS}) );


    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
        FamilyCommon::ReportStatus( EC_SNMP_SESS_ERR, SNMPComm::GetError(), undef );
        next;
    }
    $snmpStatus = SNMPComm::MaximizeMsgSize();
    if( $snmpStatus ) {
        next if( FamilyCommon::ReportStatus( EC_SNMP_MAX_SIZE_FAIL, SNMPComm::GetError(), undef ) );
    }
    my $mibVersion = SNMPComm::GetValue('MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         next if( FamilyCommon::ReportStatus( EC_SNMP_NO_GET, SNMPComm::GetError(), undef ) );
    }
    Logger::LogTrace($nodeLogFH, "  OK\n");
    Logger::LogTrace($nodeLogFH, "Prepare logging directories...");
    ($dir, $thisTftpDir, $fxrData) = FamilyCommon::PrepareLogging($session, $nodeLogFH, $userName, $isamNode);

    if( defined($fxrData) ) {
        $toolData{FXRDATA} = $fxrData;
        Logger::LogTrace($nodeLogFH, "  OK\n");
    } else {
        next if( FamilyCommon::ReportStatus( EC_DETERMINE_LOG_FAILED, FamilyCommon::GetError(), undef ) );
    }

    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g;
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    ## Example : /tftpboot/tmp/<ISAM-IP>/
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $execLogFH, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};

    FamilyCommon::SetNeStorage($dir);
    ### Store current working directory as OSWP Dynamic Directory
    $oswpDynamicDir = $dir;

    if( $params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile= sprintf("swdl_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Isam7354RUCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
        Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
    }


    # Determine NE family: retrieve NT-A and NT-B type and status using SNMP.  Only then we continue (GPON does not have full
    # CLI capability...
    $ntType = ToolCommon::DetermineNtType( );
    next if (not defined($ntType));

    next if( DynamicOSWPVariant($nodeLogFH, $config, $ntType ) );

    Logger::LogTrace($nodeLogFH, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
        next if( FamilyCommon::ReportStatus( EC_GET_ACC_BITS_FAILED, ToolCommon::GetError(), undef ) );
    } else {
        Logger::LogTrace($nodeLogFH, "  OK\n");
    }

   if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
	 $params->{USESSH} = 1;
	 Logger::LogInfo($nodeLogFH, "CLI Communication through SSH\n");
   }
   ### Library Category :-
   ### LS    : DownloadIsam1540.pm 
   ### PIZZA : DownloadIsam7354.pm
   ### SHUB  : DownloadIsamSHUB.pm (DownloadIsam7302Ansi.pm, DownloadIsam7302EtsiNtt.pm, DownloadIsam7302Etsi.pm, DownloadIsam7302.pm)
   ### IHUB  : DownloadIsamIHUB.pm (DownloadIsam7302HiCap.pm, DownloadIsam7363.pm, DownloadIsam7367.pm, DownloadIsam7367_SX.pm)
   $params->{'BOARD-FAMILY'} = "";
   Capability::LogNeFamily($ntType, $params->{DBVER_TGTREL});
   if ( $params->{'BOARD-FAMILY'} eq "" ) {
     next if (FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, undef, 
                                 sprintf("Board family not found for NT-Type '%s', DB-Version '%s'\n",
                                     $ntType, $params->{DBVER_TGTREL} ) ) );
   }
   my $shubBoardFamily = Capability::IsamEtsiFamily() ."|". Capability::IsamEtsiExtendedFamily();
   $shubBoardFamily .= "|". Capability::IsamFamily() ."|". Capability::IsamAnsiFamily();
   $shubBoardFamily .= "|". Capability::IsamBothFamily();
   my $ihubBoardFamily = Capability::IsamHiCapEtsiFamily() . "|" .  Capability::IsamHiCapAnsiFamily(); 
   $ihubBoardFamily .= "|". Capability::Isam7367_SXFamily() ."|". Capability::Isam7367Family();
   $ihubBoardFamily .= "|". Capability::Isam7363Family();
   if ($params->{'BOARD-FAMILY'} =~ /^($ihubBoardFamily)$/ ) {
       $ret = DownloadIsamIHUB::Download($isam, $ntType);
   } elsif ($params->{'BOARD-FAMILY'} =~ /^($shubBoardFamily)$/ ) {
       $ret = DownloadIsamSHUB::Download($isam, $ntType);
   } elsif( $params->{'BOARD-FAMILY'} eq Capability::IsamLSFamily() ) {
      $ret = DownloadIsam1540::Download($isam, $ntType);
   } elsif( $params->{'BOARD-FAMILY'} eq Capability::RU7354Family() ) {
      $ret = DownloadIsam7354::Download($isam, $ntType);
   } else {
      next if( FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, undef, 
                             sprintf("Operation not permitted for NT-Type '%s', DB-Version '%s' and Family '%s'\n",
                                      $ntType, $params->{DBVER_TGTREL}, $params->{'BOARD-FAMILY'} ) ) );
   }
    Logger::LogTrace($nodeLogFH, "Done!\n");
    FamilyCommon::CloseSnmpSession2Close();

    my $endTime = time();
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );
    if($ret) {
       FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
    }
    if ($params->{SEPLOG}) {
        FamilyCommon::CloseChildLogFile();
        Logger::LogHandleClose($nodeLogFH);
    }
  }
  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }
  if (!defined($params->{ID})) {
    if (! $params->{SEPLOG}) {
        FamilyCommon::CloseChildLogFile(); 
        Logger::LogHandleClose($nodeLogFH);
    }
  }
  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file.

=end html

=cut

sub AdjustToolVars($) {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{MAXSESSIONS} = $config->{'MAXTFTPSESSION'} if defined($config->{'MAXTFTPSESSION'});
  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
										 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
										($config->{'DEFAULTCLIPWD'} ne ""));
  $toolData{PASSWORD2} = $crypt->decrypt($config->{'SECONDCLIPWD'}, CYPHER) if (defined($config->{'SECONDCLIPWD'}) &&
										($config->{'SECONDCLIPWD'} ne ""));
  $toolData{TL1USER} =  $crypt->decrypt($config->{'DEFAULTTL1USER'}, CYPHER) if (defined($config->{'DEFAULTTL1USER'}) &&
										     ($config->{'DEFAULTTL1USER'} ne ""));
  $toolData{TL1PWD} =  $crypt->decrypt($config->{'DEFAULTTL1PWD'}, CYPHER) if (defined($config->{'DEFAULTTL1PWD'}) &&
										     ($config->{'DEFAULTTL1PWD'} ne ""));
  $toolData{OCTOPUSPATH} = $config->{'OCTOPUSPATH'} if defined($config->{'OCTOPUSPATH'});
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'}); # -- 1.0.3
  $toolData{TFTPTIMEOUT} = $config->{'TFTPTIMEOUT'} if defined($config->{TFTPTIMEOUT});                  # -- 1.0.5
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT});            # -- 1.0.5
  $toolData{SHUBSWPATH} = $config->{'SHUBSWPATH'} if (defined($config->{'SHUBSWPATH'}));
  $toolData{SHUBSWNAME} = $config->{'SHUBSWNAME'} if (defined($config->{'SHUBSWNAME'}));
  $toolData{RETRIES} = $config->{'RETRIES'} if defined($config->{'RETRIES'});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{RESULTSPACE} =~ s/\/$//;
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
										 ($config->{NT_COMMUNITY} ne ""));
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});
  $toolData{TL1UDPPORT} = $config->{TL1UDPPORT} if defined($config->{TL1UDPPORT});
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});
  $STATUSFILE = $config->{SWDL_STATUS_FILE} if defined($config->{SWDL_STATUS_FILE});
}

#==============================================================================
#== Main program
#==============================================================================
my $sessionCount = 0;
my @pids;
my $pid;
my $warnCount = 0;

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $warnLogFH = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("Execution (pid :%d) ends, Due to signal '%s'\n", $ownPid, $signal);
  if( fileno $execLogFH ) {
      Logger::LogNotify($warnLogFH, $string ) if( (defined $warnLogFH) and (fileno $warnLogFH) );
      Logger::LogError($execLogFH, $string );
  } else {
     print STDERR $string;
  }

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($warnLogFH, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFH);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFH);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR "  Aborting children...\n";
    Logger::LogTrace($warnLogFH, "  Aborting children...\n") if defined($warnLogFH);
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	Logger::LogTrace($warnLogFH, "Killing process $pid...\n") if defined($warnLogFH);
	kill($pid);
      }
    }
  }
  FamilyCommon::UpdateNodeStatusLog( $exitCode, $signal );
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFH = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();

  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($warnLogFH, "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr") if (defined($warnLogFH));
    print STDERR "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr";
    $warnCount++;
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFH = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFH, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFH));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFH = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFH, "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFH));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};
$SIG{"PIPE"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $warnLogFH = FamilyCommon::GetWarnLogFile();

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    print STDERR "SIGPIPE received (ignored): $msgStr" if !defined($warnLogFH);
    Logger::LogTrace($warnLogFH, "SIGPIPE received (ignored): $msgStr") if defined($warnLogFH);
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	print STDERR sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFH);
	Logger::LogTrace($warnLogFH, sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFH);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
    CleanUp("PIPE", EXIT_DIED_CODE);
  }
};

#-- Read parameter config file and adjust tool defaults if needed -- BB--
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars(\%config);
FamilyCommon::SetConfigData(\%config);
# -- End modif BB

#-- Handle command line
$params = ParseCommandLine(\%config);

STDOUT->autoflush(1);

$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

PrintOptionsSummary($execLogFH, 1, \%config);
Utils::SetDebug($execLogFH) if ($params->{DEBUG}); # 1.2.2
my $retC = Utils::VerifyOctopusEnv($params->{DEBUG}, $toolData{OCTOPUSPATH}, \%config, $dirName);
if ($retC) {
  LogError( $execLogFH, Utils::GetError() );
  exit $retC;
}


#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  Logger::LogInfo( $execLogFH, "Reading Hostfile...\n" );
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

umask(0) if ($params->{UMASK_CHMOD});
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );

my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      push(@pids, $pid);
    } else {
      HandleIsams( \@isamPart, $sessionCount, \%config);
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      Logger::LogInfo( $execLogFH, sprintf("Session %d : ends with exit code : %d - (%s); signal (%d)\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal ) );
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;

