#!/usr/bin/perl

=head1 ISAM SW upgrade

=begin html
This script will upgrade an ISAM to a new SW release (upgrade means a SW change within the same release stream and thus not requiring a DB
migration.

The operations of the script are influenced by:
<ul>
  <li>command line options</li>
  <li>a parameter configuration file</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         upgrade.pl
#-- Date:         17.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Donny Janssens for the initial version.
#--               Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to upgrade ISAMs to a new minor release
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2005/11/17 : First implementation
#--   1.0.1 - 2005/11/18 : Add shub database save
#--                        Add check for enabled software
#--
#--   1.0.2 - 2005/12/15 : Added config file usage
#--                        Usage of unix ping in stead of UDP ping .  Caused
#--                        by DBFhw70276
#--   1.0.3 - 2006/01/18 : Indicate in main log file error occurence for one
#--                        the children processes.
#--   1.0.4 - 2006/01/20 : Node prefix configurable.
#                          AMS can't do node name to IP address convertion (
#--                        ip address/node name not stored in /etc/hosts)
#--   1.0.5 - 2006/03/09 : Included exit codes.
#--   1.2.0 - 2006/03/16 : Introduction of the capability model.
#--   1.2.1 - 2006/05/19 : Verify environment
#--   1.2.2 - 2006/05/31 : Using --debug results in failure (undefined value).
#--   1.2.3 - 2006/06/08 : ECNT-C NT SW file has other prefix the ENCT-A
#--   1.2.4 - 2006/06/30 : Change migrate or upgrade check: now uses NtDbVersion
#--                        from Capability.pm
#--   1.2.4.1 - 2006/07/25 : ANTms06883 - remove CLI logfile frmo /tmp on exit
#--                          of emergency exit
#--   1.2.5.1 - 2006/08/23 : ANTms10868 - create log file per NE in stead of
#--                          appending all in the child logfile.
#--                          ANTms10885 - perform some retries when activating
#--                          new OSWP via SNMP: craft can have taken transaction.
#--             2009/09/25 : Set to type-based CLI positioning.
#--             2006/09/29 : Include program warnings in log files.
#--   1.4.1   - 2006/10/09 : ANTms17315: info about migration or upgrade
#--                          issues.
#--   1.4.3   - 2006/10/18 : Timestamp logging
#--                          Integration with AMS
#--                          When upgrading a R2.2 system: wait after saving SHUB
#--                          database
#--   1.4.3.6 - 2006/11/20 : ANTms23105: due to SW bug NT takes more then 15 seconds
#--                          to reboot meaning that the WaitForRestart does not work
#--                          very well: introduce a configurable sleep.
#--                          ANTms23395: set transaction owner before doing SNMP set
#--   1.4.3.8 - 2006/11/24 : ANTms24159: a transaction survives an NT reset.  Release
#--                          it after the NT reachable again.
#--   1.4.3.9 - 2007/01/08 : ANTms28838, typo's for pre- and post-upgrade actions.
#--                          Skip to next ISAM when pre- or post-upgrade script fails.
#--   1.5.1.3 - 2007/01/26 : ANTms31758.  Call IncrErrorCount with correct parameters.
#--   1.5.1.4 - 2007/02/01 : SECUREDBSAVE handling incorrect.
#--   1.5.1.5 - 2007/05/04 : ANTms49911: Small correction in Ctrl-c handling (don't kill pid
#--                          that have already exited)
#--   1.5.1.6 - 2007/06/12 : ANTms50447: wait 3 minutes after SHUB DB save on redundant
#--                          systems before activating new OSWP.
#--   1.5.1.7 - 2007/06/14 : ANTms51245 - Fix flaw in transaction release at the end of the
#--                          operations.
#--   1.5.2   - 2007/06/18 : ANTms51632 allow multiple users for same ISAM.
#--   1.5.3   - 2007/06/22 : ANTms52362 - Make sure node status result file is never
#--                          empty (Added AdvanceToNext).
#--   1.5.4   - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.5.5   - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.5.6   - 2007/08/22 : ANTms60869: Make NT community string configurable
#--             2007/08/25 : ANTms55192: Reset NT using T&D in case the NT is still
#--                          running after an OSWP activation.
#--   1.5.7   - 2007/09/03 : ANTms62454: When the node is up too soon, the workaround
#--                          is triggered but the Telnet session is closed.  We now
#--                          specifically check on this.
#--   1.5.8   - 2007/09/05 : ANTms62648: Workaround not OK for redundant systems.
#--   1.5.9   - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.5.10  - 2007/12/14 : ANTms76707, tool reports success on Unix level even if a failure
#--                          occured.  Reason: the child process was not returning an exit
#--                          code
#--   1.5.11  - 2008/01/10 : ANTms63550: Added functionality to configure access operatorId
#--                          through CLI before software upgrade.
#--   1.5.12  - 2008/02/29 : Ignore unknown stdby NT when status is not operational.
#--   1.5.13  - 2008/03/04 : ANTms86893, error in IP regular expression
#--   1.5.14  - 2008/04/10 : ANTmt02497.  Node reset (rescue path) call when upgrading between
#--                          the online reduced and non-reduced package of the same release/
#--   1.5.15  - 2008/04/29 : ANTmt04599.  NANT-A TYPE-B SW name changed between R3.2 and R3.3
#--                                       Implement logic for SW name retrieval in case of
#--                                       'reduced' SW
#--   1.5.16  - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.5.17  - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--   1.5.18  - 2008/10/27 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.6.0   - 2008/09/08 : R3.7.10 and IPD ISAM family.
#--   1.6.1   - 2009/03/10 : ANTmt18506.
#--   1.6.2   - 2009/03/18 : ANTmt48688.
#--   1.6.3   - 2009/04/30 : ANTmt55717.  Introduced NE famile ISAM_BOTH to indicate NTs that can be part of both the ETSI and ANSI stream.
#--   1.6.4   - 2009/05/06.  Correct SNMP settings print out at program startup.
#--   1.6.5   - 2009/05/19 : ANTmt59034: Change $config{USESSH} to $config{USE_SSH}
#--   1.6.6   - 2009/05/25 : ANTmt59036.  Check security access settings and force usage of SSH if no telnet access for CLI possible.
#--   1.6.7   - 2009/05/26 : ANTmt60011
#--   1.6.8   - 2009/06/02 : ANTmt60891.
#--   1.6.9   - 2009/06/02 : ANTmt61239
#--   1.6.10  - 2009/06/10 : ANTmt62662, allow public-key authentication with SSH
#--   1.6.11  - 2009/06/26 : ANTmt65332.  Introduction of the NTT ETSI family
#--   1.6.12  - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.6.13  - 2009/11/10 : Check for empty response on DoSafeGet for SNMPv3 session  
#--   1.6.14  - 2009/11/12 : ANTmt86148: 7354RU changes
#--   3.0.00  - 2010/05/05 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01  - 2010/05/05 : Converged PBMT changes
#--                        : x86 Changes
#--   3.0.02  - 2010/07/30 : ALU00441701  Support added for FORCE_OPERATOR_ID in param.cfg
#--                          which will be the default for --oprid
#--   3.0.03  - 2010/10/15 : new LRM option --id (suppresses the --logfile option).  Must not be used in combination with --hosts
#--   3.0.04  - 2010/11/25 : to resolve problem of child log file being written to parent log.
#--   3.0.05  - 2010/12/14 : ALU00954124 - Revert back to old naming conventions (with .exe extension)  for tools
#--   3.0.06  - 2011/01/11 : LRM updates for lib-perl path
#--   3.0.07  - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.08  - 2011/04/01 : Changes for 7354RU
#--   3.0.09  - 2011/04/05 : ALU01031457 - Support for R4220 & R4230
#--   3.0.10  - 2011/05/02 : ALU01178887
#--   3.0.11  - 2011/10/24 : logfile command line parameter can include a directory
#--   3.0.12  - 2012/01/05 : ALU01733408
#--   3.0.13  - 2012/03/26 : ALU01748774
#--   3.0.14  - 2012/05/14 : ALU01789702 - World writeable directories need sticky bit set
#--   3.0.15  - 2012/05/30 : to correct the display of snmp option used in logfile
#--   3.0.16  - 2012/10/22 : ALU01761938
#--   3.0.17  - 2012/11/06 : ALU01837839
#--   3.0.18  - 2013/02/01 : enable usage of second password if first one fails for cli over ssh
#--   3.0.19  - 2013/02/08 : corrected issue related to picking of second password from config file
#--   3.0.20  - 2013/02/15 : undef second password in params when it is set as empty
#--   3.0.21  - 2013/04/29 : ALU01880147
#--   3.0.22  - 2013/07/09 : ALU01893590
#--   3.0.23  - 2013/08/20 : NGVR Support 
#--   3.0.24  - 2015/04/28 : ALU02057368 
#--   3.0.25  - 2015/05/25 : ALU02060588
#--   3.0.26  - 2015/06/10 : ALU02083077
#--   3.0.27  - 2015/10/28 : ALU02139820
#--   3.0.28  - 2015/11/02 : ALU02142016
#--   3.0.29  - 2015/12/14 : ALU02156544
#--   3.0.30  - 2015/12/30 : ALU02160793
#--   3.0.31  - 2016/03/16 : ALU02190558
#--   3.0.32  - 2016/07/15 : ALU02237966
#--   3.0.33  - 2016/08/08 : ALU02251693
#--   3.0.34  - 2016/09/12 : ALU02271083
#--   3.0.35  - 2016/09/28 : ALU02278259
#--   3.0.36 -- 2016/11/21 : ALU02285181
#--   3.1.0   - 2018/06/20 : ALU02460863 & ALU02471697


=head1 VERSION

=begin html
Current version: "upgrade.pl - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "upgrade.pl - 3.1.0 -- 2018/06/20";
#==============================================================================
=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h"</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 

use Net::SNMP;
use Net::TFTP;

use Logger qw(:ALL);
use IsamCli;
use Crypt::Lite;
use Utils;
use Capability;
use Octopus;
use PbmtConfig;
use ErrorCodes qw(:EXIT_CODES :UPGR);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use UpgradeIsam1540;
use UpgradeIsam7354;
use UpgradeIsamIHUB;
use UpgradeIsamSHUB;
use SNMPComm;
use version qw(:ALL);

#==============================================================================
#== Comments:
#==
#==   General steps of the script
#==     1a) Check if new SWP is on NT board (and find its index)
#==     1b) Check old software release is same major release
#==     3) Activate new SWP
#==     4) Wait for NT to become active
#==     5) Verify correct SWP is active
#==============================================================================

#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$$);
sub UpgradeIsams($$$);
sub AdjustToolVars($);
sub CleanUp($;$);

#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- mod 1.0.2
#==============================================================================
my $logExecOrNode = 0;
my $STATUSFILE = "result.txt";
my $ID_FILENAME = "UPGR";
my %toolData = ('HOSTSFILE' => "/etc/hosts",
		'MAXWAITTIME' => 1800, #-- 30 Minutes
		'MAXSESSIONS' => 5,
		'TFTPDIR' => "/global/aws_var/aws/equipment",
		'TFTPWORKDIR' => "tmp",
		'USERNAME' => "",
		'PASSWORD' => "",
		'PASSWORD2' => "",
		'OCTOPUSSCRIPTS' => "tools/octopus",
		'OCTOPUSPATH' => "tools/octopus",
		'TELNETTIMEOUT' => 10,
        'EMERG_ALRM_POLL_TIME' => 120,
        'ACTPINGWAIT' => 0,
		'NTDBTOOLDIR' => "tools/NT",
		'NTDBTOOL' => "tools/NT/MJSH.exe",
		'NTDBCFG' => "tools/NT/dbtool.cfg",
		'NTDBCFGMOD' => "dbtoolMOD.cfg",
		'XVPSDBCFGMOD' => "dbtoolXVPSMOD.cfg",
		'LANXDBTOOLDIR' => "tools/Lanx",
		'SHUBDBCFG' => "tools/Lanx/migration.dat",
		'LANXDBTOOL' => "Migrationtool.pl",
		'LANXIPCKSUM' =>  "CalcIpCheckSum_BE",
		'IVPSDBTOOLDIR' => "tools/IVPS",
		'IVPSDBTOOL' => "tools/IVPS/NBTW.exe",
		'IVPSDBCFG' => "tools/IVPS/dbtool.cfg",
		'$IVPSINVENTORY' => "/opt/aws/isamv/voipim/script/getIVPSBoardInNE.pl",
		'CDEDIR' => "",
		'NEWCDEFILE' => "",
		'ACTCDEFILE' => "",
		'PINGWAIT' => 10, # seconds -- 1.0.2
		'NODEPREFIX' => "AS-",  # -- 1.0.4
		'NT_COMMUNITY' => "public",
		'SECUREDBSAVE' => 30,
		'REMAP_HOTRESET' => "N",
		'NO_IP' => "0.0.0.0",
		'OCTOPUSSCRIPTS' =>   "tools/octopus",
		'OCTOPUSPATH' => "tools/octopus",
		'WARNINGFILE' => "PRCKwarnings.txt",
		'RETRIES' => 5,
		# FXRDATA is a pointer to hash being filled in at a later stage.  The hash contains the following:
		# 'PROTOCOL': set to the filetransfer protocol being used by the ISAM
		# 'SERVERS': the filetransfer server used by the ISAM.  Only set when the protocol is different from tftp.
		#            Is a hash consisting of a list of IP-addresses and usernames (only username per filetransfer server is possible).
		# 'HOMEDIR': the home directory for NT DB backup and restore.  Depending on the file transfer protocol this setting will be
		#            different
		# 'XVPS_HOMEDIR': the home directory fox xPS DB backups.  xVPS only support TFTP, so this setting is relative to the TFTPDIR setting
		'FXRDATA' => undef,
                'FORCE_OPERATOR_ID' => undef);


=head1 USAGE

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] upgrade.pl --isam &lt;ip&gt; --hosts &lt;file&gt; --swp &lt;path-to-oswp-descriptor&gt; --logfile &lt;file&gt; \ <br>
                    [--sessions &lt;nr&gt;] [--force] [--seplog | --noseplog] [--continue] [--multi] [--noname] [--haip &lt;ip&gt;] [--noping] \ <br>
                    [--cliusername &lt;user&gt;] [--clipassword &lt;password&gt;] [--v] [--debug] [--comm &lt;SNMP community string&gt;] [--oprid] [--remap] \ <br>
                    [--v2] [--v3Auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;] [-v3priv &lt;privProtocol&gt;:&lt;password&gt;] </tt>

=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>swp</td><td>path to oswp descriptor file (relative to TFTP root directory)</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>cliusername</td><td>ISAM CLI username</td></tr>
  <tr><td>cliPassword</td><td>ISAM CLI password</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td></tr>
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>force</td><td>skip migration path checks, download SW anyhow</td></tr>
  <tr><td>continue</td><td>continue SW download for other ISAMs in stead of exiting the current child process</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>haip</td><td>High available system IP address associated to the physical interface to the DSLAMs</td></tr>
  <tr><td>oprid</td><td>Configure access operatorId assigned by Nokia before upgrade (configurable via param.cfg)</td></tr>
  <tr><td>remap</td><td>Disable the NT hot reset (in case of no firewall issues because of opening up all UDP ports)</td></tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed.</td></tr>
  <tr><td>v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                             <li>All fields are mandatory when option is used.</li>
                         <ul<br>
                         e.g. --v3auth user:password:: will result in DM5 as protocol.</td></tr>
  <tr><td>v3priv</td><td>optional setting when using SNMPv3 privacy protocol providing protocol and password<br>
                         <ul><li>Privacy protocol is DES, 3DES or AES, DES is default protocol.</li>
                         </ul></td></tr>
</table>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> | --id <id> --swp <path-to-oswp-descriptor> [--noping] \\ \n";
  print STDERR "       [--sessions <nr>] [--force] [--continue] [--seplog | --noseplog] [--cliusername <user>] [--clipassword <password>]\\ \n";
  print STDERR "       [--clipassword2 <password>] [--v] [--debug] [--multi] [--noname] [--comm <SNMP community string>] [--v2] [--haip <ip>] \\ \n";
  print STDERR "       [--oprid] [--remap] [--v3auth <user>:<password>:<authProtocol>] [-v3priv <privProtocol>:<password>] \n";
  print STDERR "       [--btype <btype>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         id       : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                    The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         v        : print version and exit\n";
  print STDERR "         swp      : path to oswp descriptor file \n";
  print STDERR "                   (relative to TFTP root - $toolData{TFTPDIR})\n";
  print STDERR "         sessions : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         cliusername  : ISAM CLI username\n";
  print STDERR "         cliPassword  : ISAM CLI password\n";
  print STDERR "         clipassword2 : Second ISAM CLI password (option to use a second password if first password fails). \n";
  print STDERR "                        The second password option will be used only when CLI over SSH is configured. \n";
  print STDERR "         force    : skip upgrade path checks, upgrade anyhow\n";
  print STDERR "         seplog   : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noseplog : keep all NE logging in a single child log file (default is a single logfile per NE)\n";
  print STDERR "         continue : continue migrating other ISAMs in stead of exiting the current child process\n";
  print STDERR "         noname   : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         multi    : include Unix username in NE logging directory path\n";
  print STDERR "         haip     : High available systems' IP address associated to the physical interface to the DSLAMs\n";
  print STDERR "         oprid    : Configure access operatorId assigned by Nokia before upgrade (configurable via param.cfg)\n";
  print STDERR "         remap    : Disable the NT hot reset (in case of no firewall issues because of opening up all UDP ports)\n";
  print STDERR "         noping   : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "       SNMP settings:\n";
  print STDERR "         comm     : NT SNMP community string when using SNMPv1 or SNMPv2c.  Default is \"public\"\n";
  print STDERR "         v2       : Use SNMPv2c\n";
  print STDERR "         v3auth   : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                    The password should be at least 8 characters long.\n";
  print STDERR "                    Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                    All fields are mandatory when option is used.\n";
  print STDERR "                    e.g. --v3auth user:password:: will result in DM5 as protocol.\n";
  print STDERR "         v3priv   : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                    Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  print STDERR "         btype    : Build type to choose XDescriptor; Possible values are SNMP, YANG\n";
  exit(CMD_INCOMPL_CODE);
}

=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut
sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $noaction, $swp, $sessions, $debug, $hosts, $logFile, $cliUserName, $cliPassword, $cliPassword2, $forceUpg, $version, $sepLog, $continue, $multi, $wipe,
      $noName, $btype, $communityString, $haIp, $accessOperatorId, $remap, $noPing, $v3Auth, $v3Priv, $useSsh, $noSepLog, $snmpV2, $id);
  my $regExp;
  my %params;
  my $dbVersion;
  my $dirName = getcwd();
  my $options = GetOptions('isam=s', \$isam,
			   'noaction', \$noaction,
			   'btype=s', \$btype,
			   'swp=s', \$swp,
			   'sessions=i', \$sessions,
			   'debug', \$debug,
			   'hosts=s', \$hosts,
			   'logfile=s', \$logFile,
			   'username|cliusername=s', \$cliUserName,
			   'password|clipassword=s', \$cliPassword,
                           'clipassword2=s', \$cliPassword2,   
			   'force', \$forceUpg,
			   'v', \$version,
			   'v2', \$snmpV2,
			   'seplog', \$sepLog,
			   'noseplog', \$noSepLog,
			   'continue', \$continue,
			   'multi', \$multi,
			   'wipe', \$wipe,
			   'noname', \$noName,
			   'comm=s', \$communityString,
			   'haip=s', \$haIp,
                           'oprid', \$accessOperatorId,
			   'remap', \$remap,
			   'noping', \$noPing,
			   'v3auth=s', \$v3Auth,
			   'v3priv=s', \$v3Priv,
			   'ssh', \$useSsh,
                           'id=s', \$id);
  
  ##--- check options
  if (! $options) {
    Usage();
  }
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }
  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }
  if (!defined($swp)) {
    print STDERR "ERROR: swp not set\n";
    Usage();
  }
  my $oswp2Consider = $swp;
  $params{NOTFTP} = 0;
  # First try the file as specfied (could be an absolute path when using ftp e.g.
  if (! -r $oswp2Consider) {
    # Next try the SWP relative to the login home directory
    $oswp2Consider = "$ENV{HOME}/$swp";
    $oswp2Consider =~ s/\/\//\//g;
    if (! -r $oswp2Consider) {
      # Finally check as tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $oswp2Consider =~ s/\/\//\//g;
      if (! -r  $oswp2Consider) {
	print STDERR "ERROR: Accessing OSWP '$oswp2Consider' failed: $!\n";
	Usage();
      } else {
	$params{OSWP_HOME} = $toolData{TFTPDIR};
      }
    } else {
      $params{OSWP_HOME} = $ENV{HOME};
      $params{NOTFTP} = 1;
    }
  } else {
    if (index($toolData{TFTPDIR}, $dirName) != -1) {
      # PBMT installed relative to TFTP directory so still tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $params{OSWP_HOME} = $toolData{TFTPDIR};
    } else {
      $params{OSWP_HOME} = "";
      $params{NOTFTP} = 1;
    }
  }

  if( (defined $hosts) and (!defined $btype ) ) {
     print STDERR "ERROR: --btype value must be passed for --hosts\n";
     Usage();
  }

  if( defined($btype) ) {
      if( $btype ne "SNMP" and $btype ne "YANG" ) {
         print STDERR "ERROR: Invalid value passed for --bytpe; possible values are SNMP, YANG\n";
         Usage();
      } else {
         $params{BTYPE} = $btype;
      }
  } elsif( defined $hosts ) {
      $btype = "SNMP";
  }

  $params{OSWP_FULL} = $oswp2Consider;
  $regExp = SWPSYNTAX;
  if ($swp =~ /$regExp/) {
    print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_INV_CHARS_CODE));
    exit OSWP_FILE_INV_CHARS_CODE;
  }
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  #-- fill hash, set defaults if needed
  if (!defined( $sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }
  if (defined($cliUserName)) {
    $params{USERNAME} = $cliUserName;
  } else {
    $params{USERNAME} = $toolData{USERNAME};
  }
  if (defined($cliPassword)) {
    $params{PASSWORD} = $cliPassword;
  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }
  if (defined($cliPassword2)) {
    $params{PASSWORD2} = $cliPassword2;
  } else {
    $params{PASSWORD2} = ($toolData{PASSWORD2} ne "") ? $toolData{PASSWORD2} : undef ; 
  }
  
#ALU01524313-indhu-15/12/2011
  if (defined($communityString) && defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  if (defined($communityString) && defined($v3Priv)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }
#ALU01524313-indhu-15/12/2011

  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if (defined($isam));
  $params{NOACTION} = 1 if (defined($noaction));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if (defined($debug));
  $params{OSWP} = $swp if (defined($swp));
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
  } else {
    $params{LOGFILE} = $logFile;
    if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
      my $logdir = $1;
      if (! -d $logdir) {
        if (!mkdir( $logdir, 0777)) {
          print STDERR "Cannot create directory '$logdir': $!\n";
          exit CREATE_DIR_FAILED_CODE;
        }
      }
    }
  }
  my $tftpInfo = Utils::FindTftpInfo(undef, $oswp2Consider);
  if (!defined($tftpInfo->{PRIMIP}) || ($tftpInfo->{PRIMIP} eq "")) {
    print STDERR sprintf("%s\n", OSWP_FILE_NO_IP_CODE);
    exit OSWP_FILE_NO_IP_CODE;
  }
  my $baseName = basename($tftpInfo->{NAME});
  $regExp = REGEXP;
  if (!($baseName =~ /$regExp/)) {  # -- 1.0.4
    print STDERR sprintf("Software package name (%s, read from $swp) not conforming to regexp (%s).  Check param.cfg!!!\n",
			 $baseName, $tftpInfo->{NAME}, REGEXP);
    exit PARAM_CFG_NOT_OK_CODE;
  }
  $params{PRIMTFTPIP} = $tftpInfo->{PRIMIP};
  $params{SECTFTPIP} = $tftpInfo->{SECIP};
  $params{SWPNAME} = $tftpInfo->{NAME};
  $params{FORCEUPG} = defined($forceUpg) ? 1 : 0;
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
  $params{CONTINUE} = defined($continue) ? 1 : 0;
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{WIPE} = defined($wipe) ? 1 : 0;
  $params{PHYSICAL_HOST_IP} = $config->{PHYSICAL_HOST_IP} if (defined($config->{PHYSICAL_HOST_IP}) &&
							      ($config->{PHYSICAL_HOST_IP} ne "0.0.0.0"));
  if (defined($haIp)) {
    if (ToolCommon::ValidIP($haIp)) {
      $params{PHYSICAL_HOST_IP} = $haIp;
    } else {
      print STDERR "'$haIp' is not a valid IP address\n";
      exit(PARAM_CFG_NOT_OK_CODE);
    }
  }
  $params{NONAME} = defined($noName) ? 1 : 0;
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : ""; #ALU02057368 

  #if (!defined($snmpV2) && !(defined($communityString))) { #ALU01524313-indhu-15/12/2011
  if (!defined($snmpV2)) { #ALU02057368#ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }
  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }
  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA256|SHA512|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
          if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
             $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
	} else {
	  print STDERR "SNMPv3 authentication protocol can only be SHA or SHA256 or SHA512 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
    if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
    $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
    if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
       $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {  
     my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    $params{SNMP}{V3}{PRIV}{PASSWORD} = defined($toolData{SNMPV3_PRIV_PASSWORD}) ? $toolData{SNMPV3_PRIV_PASSWORD} : "";
  } #ALU01524313-indhu-15/12/2011
  }
  if (($params{PASSWORD} eq "") || ($params{USERNAME} eq "")) {
    print STDERR "No username, password or SNMP community string defined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }
  if (defined($accessOperatorId)) {
    $params{ACCESSOPRID} = 1;
  } else {
    $params{ACCESSOPRID} = uc($toolData{FORCE_OPERATOR_ID}) eq "Y" ? 1 : 0;
  }
  $params{REMAPHOTRESET} = $toolData{REMAP_HOTRESET} eq "N" ? 0 : 1;
  if (defined($remap)) {
    $params{REMAPHOTRESET} = 1;
  }
  $params{NOPING} = 0;
  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'});
  $params{NOPING} = 1 if defined($noPing);

  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";

  $params{UPD_KNOWN_HOST_FILE} = 0;
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});

  $params{EMERG_CALL_FEATURE} = 0;
  $params{EMERG_CALL_FEATURE} = uc($config->{'EMERG_CALL_FEATURE'}) eq "Y" ? 1 : 0 if defined($config->{'EMERG_CALL_FEATURE'});

  my $oswpPath = "$params{OSWP_HOME}/$params{SWPNAME}";
  $dbVersion = ToolCommon::GetDBVersion($oswpPath);
  if(!defined($dbVersion)) {
      print STDERR "DB Version is not defined in file $oswpPath \n";
      exit FILE_OPEN_FAIL_CODE;
  }
  $params{DBVER_TGTREL} = $dbVersion;      # Holds the DB version of Target Release
  if($params{NOTFTP} == 1) {
    if($params{OSWP_HOME} ne "") {
        $oswpPath = "$params{SWPNAME}";
        $params{OSWP_HOME} = "";
    }
  }
  $params{SWPPATH} = $oswpPath;

  return \%params;
}

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut

sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw(
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli IsamTL1 Octopus ReduceSWP
                       UpgradeIsam1540 UpgradeIsam7354 UpgradeIsamIHUB UpgradeIsamSHUB
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

=head2 PrintOptionsSummary($$$$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$$) {
  my ($argLogFH, $logOnScreen, $config) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Interrupt on emergency call" , 
                $params->{EMERG_CALL_FEATURE} ? "Enabled" : "Disabled" ) ) if defined($params->{EMERG_CALL_FEATURE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "XDescriptor build type" , $params->{BTYPE} ) ) if defined($params->{BTYPE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "OSWP Descriptor file" , $params->{OSWP} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- SWP Name" , $params->{SWPNAME} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- DB Version" , $params->{DBVER_TGTREL} ) ) if defined($params->{DBVER_TGTREL});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ONT-SW Descriptor file" , $params->{ONT} ) ) if defined($params->{ONT});
  if ($config->{SHUBSWPATH} ne "") {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "SHUB SW Location" , $config->{SHUBSWPATH}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "SHUB SW file" , $config->{SHUBSWNAME}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Transaction IP Address" , 
			       defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} :  $params->{PRIMTFTPIP}) );
            
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "FTP Details :-") );
  if (! $params->{NOTFTP}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "TFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Timeout in seconds" , $toolData{TFTPTIMEOUT}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- TFTP (HOME) Directory" , $toolData{TFTPDIR}) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "SFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Actions to be perform" , $params->{NOACTION} ? "No, Checks only" : "Yes, Action after checks") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "HOT RESET Remap" , 
                 $params->{REMAPHOTRESET} ? "Enabled" : "Disabled" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Force Ugradation" , "Enabled" ) ) if ($params->{FORCEUPG});
  if( defined($config->{PRE_UPG_ACTION}) && $config->{PRE_UPG_ACTION} ne "" ) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Pre-Upgrade Activity", $config->{PRE_UPG_ACTION} ) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Pre-Upgrade Activity", "Not Configured" ) );
  }
  if( defined($config->{POST_UPG_ACTION}) && $config->{POST_UPG_ACTION} ne "" ) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Post-Upgrade Activity", $config->{POST_UPG_ACTION} ) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Post-Upgrade Activity", "Not Configured" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, "Access operatorId assigned by Nokia configured\n" ) if ($params->{ACCESSOPRID} == 1);
  if ($params->{CONTINUE}) {
    Logger::LogWish( $logMode, $argLogFH, "!!!WARNING!!!  USE AT OWN RISK!!!\n" );
    Logger::LogWish( $logMode, $argLogFH, "    Migration process will continue even in case an ISAM is lost in the process of migration" );
    Logger::LogWish( $logMode, $argLogFH, "    Hit Ctrl-C now if you do not want to continue\n!!!WARNING!!!\n" );
  }
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 UpgradeIsams($$$$)

=begin html
The main body of the script defining the complete SW upgrade process.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by this child process.  Every
      array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
</table>

=end html

=cut

sub UpgradeIsams($$$) {
  my ($isamsRef, $sessionCount, $config) = @_;
  my $isam;
  my $ret;
  my $startTime;
  my $nodeLogFile;
  my $nodeStatusLog;
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $dirName = dirname($0);
  my $ntType;
  my $isamNode;
  my $dir;
  my $fxrData;
  my $thisTftpDir;
  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );

  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetConfigData($config);

  Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Started as '%d'\n", $sessionCount, FamilyCommon::GetOwnPid()));
  ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );
      FamilyCommon::SetFailedNode($isamNode);
      FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;
      FamilyCommon::SetChildLogFile($nodeLogFH);
      # Switch debugging (if any) to an opened filehandle...
      IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});   # -- 1.2.0
      Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
      ToolCommon::SetDebug($nodeLogFH);
      Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
      PrintOptionsSummary($nodeLogFH, undef, $config);
  }
  my $isamCount = -1;
  foreach $isam (@{$isamsRef}) {
    $isamCount++;
    $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1]; # Store for IncrErrorCount

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );

   if ( defined($params->{ID})) {
        $logExecOrNode = 0;
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
        PrintOptionsSummary($nodeLogFH, undef, $config);
    }

    FamilyCommon::SetFailedNode($isamNode);
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;
    FamilyCommon::SetChildLogFile($nodeLogFH);
    # Switch debugging (if any) to an opened filehandle...
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});   # -- 1.2.0
    Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
    ToolCommon::SetDebug($nodeLogFH);
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    IsamCli::ResetGlobals();
    $params->{USESSH} = 0; #for each isam the value is reset

    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    if (! open($nodeStatusLog, ">$toolData{RESULTSPACE}/$isamNode/$STATUSFILE")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$STATUSFILE' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE" ) );
    }
    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();

    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }

    # Check whether an octopus session to this ISAM is open
    next if( Octopus::OctopusActive($params->{DEBUG}, $isam, $config->{GREP}, $config->{PS}) );

    if (defined($config->{PRE_UPG_ACTION}) && $config->{PRE_UPG_ACTION} ne "") {
      Logger::LogInfo($nodeLogFH, "Executing pre-upgrade actions...");
      if (system("$config->{PRE_UPG_ACTION} -ip $isam->[0] -logdir $dir")) {
       	   FamilyCommon::IncrErrorCount("non-successful exit code... skipping\n",
       	   			     EC_PRE_UPGR_ERROR,
       	   			     undef,
       	   			     1);
       	   next;
      } else {
	       Logger::LogTrace($nodeLogFH, "  OK\n");
      }
    }
    my ( $snmpStatus, $snmpSession ) = SNMPComm::EstablishSNMP( $isam->[0] );
    if( $snmpStatus ) {
          FamilyCommon::IncrErrorCount( "FAILED : ".SNMPComm::GetError(),
                                        EC_SNMP_SESS_ERR, ": ".SNMPComm::GetError(), 1);
          next;
    }
    $snmpStatus = SNMPComm::MaximizeMsgSize();
    if( $snmpStatus ) {
         FamilyCommon::IncrErrorCount( "Failed to set maximum SNMP message size : ". SNMPComm::GetError(),
                             EC_SNMP_MAX_SIZE_FAIL, " : ".SNMPComm::GetError(), undef);
         next;
    }
    my $mibVersion = SNMPComm::GetValue('MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         FamilyCommon::IncrErrorCount(sprintf("  FAILED to obtain MIB version: %s\n", SNMPComm::GetError() ),
                   EC_SNMP_NO_GET, sprintf(": %s", SNMPComm::GetError() ), 1);
         next;
    }

    Logger::LogTrace($nodeLogFH, "Prepare logging directories...");
    ($dir, $thisTftpDir, $fxrData) = FamilyCommon::PrepareLogging($snmpSession, $nodeLogFH, $userName, $isamNode);

    if (not defined($fxrData)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_DETERMINE_LOG_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      next;
    } else {
      $toolData{FXRDATA} = $fxrData;
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g;
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $nodeLogFH, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};

    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile = sprintf("upgrade_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
        Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
        Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    }
    FamilyCommon::SetNeStorage($dir);

    # Determine NE family: retrieve NT-A and NT-B type and status using SNMP.  Only then we continue (GPON does not have full
    #     # CLI capability...
    #
    $ntType = ToolCommon::DetermineNtType( );
    next if (not defined($ntType));

    ToolCommon::GetSourceDetails();
    if( !defined $params->{SOURCE_OSWPNAME} ) {
         FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED,ToolCommon::GetError(), undef );
         next;
    }

    Logger::LogTrace($nodeLogFH, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($snmpSession, $params->{DEBUG});
    if (! defined($accBits)) {
      FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
				   EC_GET_ACC_BITS_FAILED,
				   undef,
				   1);
      next;
    } else {
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

   if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
	    $params->{USESSH} = 1;
        Logger::LogInfo($nodeLogFH, "CLI Communication through SSH\n");
   }

    my $ntBoardFamily = Capability::LogNeFamily($ntType, $params->{DBVER_TGTREL});
    my $oswpDynamicVariant = DynamicOSWPVariant($nodeLogFH, $config, $ntType );
    if( $oswpDynamicVariant ) {
         next;
    }
    my $shubBoardFamily = Capability::IsamEtsiFamily() ."|". Capability::IsamEtsiExtendedFamily();
    $shubBoardFamily .= "|". Capability::IsamFamily() ."|". Capability::IsamAnsiFamily();
    $shubBoardFamily .= "|". Capability::IsamBothFamily();
    my $ihubBoardFamily = Capability::IsamHiCapEtsiFamily() . "|" .  Capability::IsamHiCapAnsiFamily(); 
    $ihubBoardFamily .= "|". Capability::Isam7367_SXFamily() ."|". Capability::Isam7367Family();
    $ihubBoardFamily .= "|". Capability::Isam7363Family();
   if ( $ntBoardFamily =~ /^($ihubBoardFamily)$/ ) {
       $ret = UpgradeIsamIHUB::Upgrade($params->{SEPLOG} ? "$dir/$nodeLogFile" : $nodeLogFH, $thisTftpDir, $isam);
   } elsif ( $ntBoardFamily =~ /^($shubBoardFamily)$/ ) {
       $ret = UpgradeIsamSHUB::Upgrade($params->{SEPLOG} ? "$dir/$nodeLogFile" : $nodeLogFH, $thisTftpDir, $isam);
   } elsif( $ntBoardFamily eq Capability::IsamLSFamily() ) {
       $ret = UpgradeIsam1540::Upgrade($params->{SEPLOG} ? "$dir/$nodeLogFile" : $nodeLogFH, $thisTftpDir, $isam);
   } elsif( $ntBoardFamily eq Capability::RU7354Family() ) {
       $ret = UpgradeIsam7354::Upgrade($params->{SEPLOG} ? "$dir/$nodeLogFile" : $nodeLogFH, $thisTftpDir, $isam, $ntType);
   } else {
       FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, undef, 
                             sprintf("Operation not permitted for NT-Type '%s', DB-Version '%s' and Family '%s'\n",
                                      $ntType, $params->{DBVER_TGTREL}, $ntBoardFamily ) );
       next;
   }
    FamilyCommon::CloseSnmpSession2Close(); # Reset for IncrErrorCount in case the post migration script would fail.
    
    if (defined($config->{POST_UPG_ACTION}) && ($config->{POST_UPG_ACTION} ne "")) {
      Logger::LogInfo($nodeLogFH, "Executing post-upgrade actions...\n");
      if (system("$config->{POST_UPG_ACTION} -ip $isam->[0] -logdir $dir")) {
	        FamilyCommon::IncrErrorCount("non-successful exit code... skipping\n",
	        			     EC_POST_UPGR_ERROR,
	        			     undef,
	        			     1);
	        next;
      } else {
	        Logger::LogTrace($nodeLogFH, "  OK\n");
      }
    }
    Logger::LogTrace($nodeLogFH, "Done!\n");
    FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );
    if ($params->{SEPLOG}) {
       FamilyCommon::CloseChildLogFile();
       LogCloseFile($nodeLogFH);
    }
  }
  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }
  if (!defined($params->{ID})) {
    if (! $params->{SEPLOG}) {
        LogCloseFile($nodeLogFH);
        FamilyCommon::CloseChildLogFile();
    }
  }
  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}

sub DynamicOSWPVariant($$$) {
    my ($nodeLogFH, $config, $ntType) = @_;
    if( defined $params->{SWPPATH} and $params->{SWPPATH} ne "" ) {
        if( defined $config->{NT_SWP_MAPFILE} and $config->{NT_SWP_MAPFILE} ne "" ) {
           my $ntDescriptorMapFile = dirname($params->{SWPPATH}) . "/" . $config->{NT_SWP_MAPFILE};
           if( ! ( -f $ntDescriptorMapFile ) ) {
                $params->{BTYPE} = "SNMP";
                Logger::LogTrace($nodeLogFH, "NT SWP Mapping file :$ntDescriptorMapFile: not exists\n");
                return 0;
           }
           Logger::LogTrace($nodeLogFH, "NT Descriptor mapping file($ntDescriptorMapFile) process started..\n");
           my( $status, $ntDescriptorFile) = Utils::FetchNTDescriptor($ntDescriptorMapFile, $ntType);
           Logger::LogTrace($nodeLogFH, "Details : $status, $ntType, $ntDescriptorFile\n");
           if( $status != 0 ) {
              FamilyCommon::IncrErrorCount("NTDescriptor Mapfile : $ntDescriptorFile \n",
                                     EC_NTTYPE_MAPFILE_FAILED,
                                     undef,
                                     1);
              return 1;
           }
           if( defined $params->{BTYPE} ) {
               if( ($params->{BTYPE} ne "SNMP") and (defined $config->{SNMP_YANG_ACTIVATE_TIMEOUT}) ) {
                   $toolData{MAXWAITTIME} = $config->{SNMP_YANG_ACTIVATE_TIMEOUT};
               }
               Logger::LogTrace($nodeLogFH, "XDescriptor build type:$params->{BTYPE}:, Max wait time:$toolData{MAXWAITTIME}:\n");
           }
       }
   }
   return 0;
}


=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file.

=end html

=cut

sub AdjustToolVars($) {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{MAXSESSIONS} = $config->{'MAXDLSESSION'} if defined($config->{'MAXDLSESSION'});
  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
										 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
										($config->{'DEFAULTCLIPWD'} ne ""));
  $toolData{PASSWORD2} = $crypt->decrypt($config->{'SECONDCLIPWD'}, CYPHER) if (defined($config->{'SECONDCLIPWD'}) &&
										($config->{'SECONDCLIPWD'} ne ""));
  $toolData{MAXWAITTIME} = $config->{'WAITTIME'} if defined($config->{'WAITTIME'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'}); # -- 1.0.4
  $toolData{NTDBTOOLDIR} = $config->{'NTDBTOOLDIR'} if defined($config->{'NTDBTOOLDIR'});
  $toolData{LANXDBTOOLDIR} = $config->{'LANXDBTOOLDIR'} if defined($config->{'LANXDBTOOLDIR'});
  $toolData{NTDBTOOL} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTOFLMT'} if defined($config->{'NTOFLMT'});
  $toolData{NTDBCFG} = $toolData{NTDBTOOLDIR} . "/" . $config->{'NTDBCFG'} if defined($config->{'NTDBCFG'});
  $toolData{OCTOPUSPATH} = $config->{'OCTOPUSPATH'} if defined($config->{'OCTOPUSPATH'});
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT});
  $STATUSFILE = $config->{UPGR_STATUS_FILE} if defined($config->{UPGR_STATUS_FILE});
  $toolData{SECUREDBSAVE} = $config->{'SECUREDBSAVE'} if defined($config->{'SECUREDBSAVE'});
  $toolData{TFTPTIMEOUT} = $config->{'TFTPTIMEOUT'} if defined($config->{TFTPTIMEOUT});                  # -- 1.1.7
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT});            # -- 1.1.7
  $toolData{EMERG_ALRM_POLL_TIME} = $config->{'EMERG_ALRM_POLL_TIME'} if defined($config->{EMERG_ALRM_POLL_TIME});            
  $toolData{SHUBSWPATH} = $config->{'SHUBSWPATH'} if (defined($config->{'SHUBSWPATH'}));
  $toolData{SHUBSWNAME} = $config->{'SHUBSWNAME'} if (defined($config->{'SHUBSWNAME'}));
  $toolData{IVPSDBTOOLDIR} = $config->{'IVPSDBTOOLDIR'} if defined($config->{'IVPSDBTOOLDIR'});
  $toolData{IVPSDBTOOL} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'IVPSOFLMT'} if defined($config->{'IVPSOFLMT'});
  $toolData{IVPSDBCFG} = $toolData{IVPSDBTOOLDIR} . "/" . $config->{'IVPSDBCFG'} if defined($config->{'IVPSDBCFG'});
  $toolData{IVPSINVENTORY} = $config->{'IVPSINVENTORY'} if defined($config->{'IVPSINVENTORY'});
  $toolData{CDEDIR} = $config->{'CDEDIR'} if defined($config->{'CDEDIR'});
  $toolData{NEWCDEFILE} = $config->{'NEWCDEFILE'} if defined($config->{'NEWCDEFILE'});
  $toolData{ACTCDEFILE} = $config->{'ACTCDEFILE'} if defined($config->{'ACTCDEFILE'});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
										 ($config->{NT_COMMUNITY} ne ""));
  $toolData{ACTPINGWAIT} = $config->{ACT_PING_WAIT} if defined($config->{ACT_PING_WAIT});
  $toolData{ACTPINGWAITRU} = $config->{ACT_PING_WAIT_RU} if defined($config->{ACT_PING_WAIT_RU});
  $toolData{REMAP_HOTRESET} = uc($config->{REMAP_HOT_RESET}) if defined($config->{REMAP_HOT_RESET});
  $toolData{RETRIES} = $config->{RETRIES} if defined($config->{RETRIES});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});
  $toolData{FORCE_OPERATOR_ID}  = $config->{'FORCE_OPERATOR_ID'} if defined($config->{'FORCE_OPERATOR_ID'});
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});

}
#==============================================================================
#== Main program
#==============================================================================

my $sessionCount = 0;
my @pids;
my $pid;
my $warnCount = 0;

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $execLogFH = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string;
  Logger::LogTrace($execLogFH, $string) if defined($execLogFH);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($execLogFH, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($execLogFH);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($execLogFH);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR "  Aborting children...\n";
    Logger::LogTrace($execLogFH, "  Aborting children...\n") if defined($execLogFH);
    foreach my $pid (@pids) {
        if ($pid > 0) {
	        print STDERR "Killing process $pid...\n";
	        Logger::LogTrace($execLogFH, "Killing process $pid...\n") if defined($execLogFH);
	        kill($pid);
        }
    }
  }
  FamilyCommon::UpdateNodeStatusLog( $exitCode, $signal );
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $execLogFH = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();
  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($execLogFH, "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr") if (defined($execLogFH));
    print STDERR "[PID $ownPid] Warning [package '$package', file: '$fileName', line:$line]: $msgStr\n";
    $warnCount++;
    my $framePresent = 1;
    my $i = 0;
    my $ownPackage;
    my $ownFileName;
    my $ownLine;
    my $ownRoutine;
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	        Logger::LogTrace($execLogFH, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
	        				      defined($ownPackage) ? $ownPackage : "undefined",
	        				      defined($ownFileName) ? $ownFileName: "undefined",
	        				      defined($ownLine) ? $ownLine : "undefined",
	        				      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($execLogFH);
	        print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
	        		     defined($ownPackage) ? $ownPackage : "undefined",
	        		     defined($ownFileName) ? $ownFileName: "undefined",
	        		     defined($ownLine) ? $ownLine : "undefined",
	        		     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($execLogFH);
	        $i++;
      } else {
	        $framePresent = 0;
      }
    }
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $execLogFH = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($execLogFH, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($execLogFH));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $execLogFH = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($execLogFH, "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($execLogFH));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};
$SIG{"PIPE"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $execLogFH = FamilyCommon::GetWarnLogFile();
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    print STDERR "SIGPIPE received (ignored): $msgStr" if !defined($execLogFH);
    Logger::LogTrace($execLogFH, "SIGPIPE received (ignored): $msgStr") if defined($execLogFH);
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	print STDERR sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($execLogFH);
	Logger::LogTrace($execLogFH, sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($execLogFH);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
    CleanUp("PIPE", EXIT_DIED_CODE);
  }
};

#-- Read parameter config file and adjust tool defaults if needed -- mod 1.0.2
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars(\%config);
FamilyCommon::SetConfigData(\%config);
# -- End mod 1.0.2
#-- Handle commandline
$params = ParseCommandLine(\%config);

STDOUT->autoflush(1);
$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

PrintOptionsSummary($execLogFH, 1, \%config);

Utils::SetDebug($execLogFH) if ($params->{DEBUG});  # -- 1.2.2
my $retC = Utils::VerifyOctopusEnv($params->{DEBUG}, $toolData{OCTOPUSPATH}, \%config, $dirName);
if ($retC) {
  Logger::LogError($execLogFH, Utils::GetError(), 1);
  exit $retC;
}

#-- Create work dir
#umask(0); # Make sure directory remains 0777
umask(0) if ( $params->{UMASK_CHMOD} );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
Logger::LogDebug( $execLogFH, "Workspace directory : $toolData{TFTPDIR}/$toolData{TFTPWORKDIR}\n" );

ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
Logger::LogInfo( $execLogFH, "Resultspace directory : $toolData{RESULTSPACE}\n", $LogModeDebug );

#-- read asams
my $isamsRef;
if (defined($params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  Logger::LogInfo($execLogFH, "Reading Hostfile...\n");
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      push(@pids, $pid);
    } else {
      UpgradeIsams( \@isamPart, $sessionCount, \%config);
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    $exitCode = $? >> 8;
    $signal = $? & 255;
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      Logger::LogInfo( $execLogFH, sprintf("Session %d : ends with exit code : %d - (%s); signal (%d)\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal ) );
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode == 5) {
	$newExitCode =  CONDITONAL_SUCCESS_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
