package MigrateIsam7302Etsi;

=head1 ISAM 7302/7330-specific migration module

=begin html
This library contains the functions dedicated to migrate a 7302/7330 ETSI ISAM from one release to a new one.

=end html

=cut

#==============================================================================
#-- File:         MigrateIsam7302Etsi.pm
#-- Date:         05.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Library to migrate ETSI ISAMs to a new release
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2005/11/05 : First implementation
#--   1.0.1 - 2005/11/06 : Bug fixing after first test
#--   1.1.0 - 2005/11/18 : Added octopus script for SwMngt.ini > 1024
#--                        Added octopus script for disabling tftp daemon
#--                        Added Config file
#--                        Added check on 'enabled' new software
#--                        Added SHUB dbase save
#--   1.1.1 - 2005/11/23 : Added extraction of IGMPXmlfile.txt file for
#--                        migration from R2.2 to a higher release
#--   1.1.2 - 2005/12/14 : Added stability waits:
#--                          + wait for cli to be available after restart
#--                          + wait 2 mins before activating
#--   1.1.3 - 2005/12/15 : Added octopus to tools directory, hence path to
#--                        octopus was adjusted.
#--                        Added unix ping in stead of udp ping: caused by
#--                        bug in NT: BDFhw70276
#--                        Added latest LANX migration tool: has lined up
#--                        command parameters with NT OFLMT
#--                        Adopt for R2.4
#--                        Added secureDbSave option: pre-R2.3 releases may
#--                        need to wait some time after issueing the
#--                        SHUB database save to secure a successful upload
#--                        of dm_complete.tar following the db save.
#--   1.1.4 - 2006/01/05 : Some minor fixes
#--                        Included the ODM sub-directory when combining the
#--                        databases for migration from R2.3 to higher releases.
#--   1.1.5 - 2006/01/18 : Indicate in main log file error occurence for one
#--                        the children processes.
#--   1.1.6 - 2006/01/20 : Node prefix configurable.
#--                        AMS can't do node name to IP address convertion
#--                        (ip address/node name not stored in /etc/hosts)
#--   1.1.7 - 2006/02/02 : Include migration from R2.1
#--   1.1.8 - 2006/02/27 : Include migration from R2.0
#--   1.1.9 - 2006/03/13 : use tftp for bacup of NT database on releases prior
#                          to R2.2
#--   1.2.0 - 2006/03/16 : Capability model
#--   1.2.1 - 2006/05/19 : Verify environment
#--   1.2.3 - 2006/06/08 : ECNT-C NT SW file has other prefix the ENCT-A
#--   1.2.4 - 2006/06/13 : - When migrating to R3.1 again the IGMPXmlfile.txt
#--                          needs to be generated.  The 'bwreduce' option
#--                          will be applied by the SHUB OFLMT.
#--                        - Included v1.22 of SHUB OFLMT
#--                        - Introduction of R2.4.20 having a different DB
#--                          version in order to allow a migration to R3.1.
#--                        - Make sure we don't call migration for SHUB when
#--                          from and to release are equal.
#--                        - Perform DB download using tftp for releases < R2.3
#--                        - OSWP activation using SNMP i.s.o. CLI
#--                        - Check on DB version (expected vs read).
#--   1.2.4.1 - 2006/07/25 : ANTms06883 - remove CLI logfile frmo /tmp on exit
#--                          of emergency exit
#--                          Upgrade of R2.2: wait 30 seconds (i.s.o. after
#--                          tftp of SHUB SW to /pub/Sw/Download
#--   1.2.4.2 - 2006/08/21 : ANTms08417 - do not check on SHUB SW size for releases
#--                          after R2.2
#--   1.2.5.1 - 2006/08/23 : ANTms10868 - create log file per NE in stead of
#--                          appending all in the child logfile.
#--                          ANTms10885 - perform some retries when activating
#--                          new OSWP via SNMP: craft can have taken transaction.
#--             2006/09/25 : Set to type-based CLI positioning.
#--             2006/09/27 : ANTms15715: take transaction before backing up
#--                          NT database.
#--             2006/09/28 : ANTms09983 - Add option to continue in case of some
#--                          fatal errors rather then exiting the child: --continue
#--             2006/09/29 : Safety check on database size after upload (should be
#--                          OK after fix for ANTms15715 but it does not harm to
#--                          check this anyway.
#--             2006/09/29 : Include program warnings in log files.
#--   1.4.1 -   2006/10/10 : ANTms17446: Copy IGMPXmlfile for correct location.
#--   1.4.2   - 2006/10/09 : ANTms17315: info about migration or upgrade
#--                          issues.
#--                          ANTms17319: allow migration of ECNT-A to ECNT-C for
#--                          SHUB OFLMT
#--             2006/10/11 : ANTmsxxxxx: - Allow script to end after DB download
#--                                      - Allow script to start from migrated DB
#--                                        has been downloaded, now only active OSWP.
#--   1.4.3   - 2006/10/18 : - Timestamp logging
#--                          - AMS/AWS integration
#--   1.4.3.4 - 2006/11/13 : ANTms22321 - transaction timeout unit different between
#--                          pre- and post-R2.4.
#--   1.4.3.5 - 2006/11/15 : Prepare for R3.2
#--   1.4.3.6 - 2006/11/20 : ANTms23105: due to SW bug NT takes more then 15 seconds
#--                          to reboot meaning that the WaitForRestart does not work
#--                          very well: introduce a configurable sleep.
#--                          ANTms23395: set transaction owner before doing SNMP set
#--   1.4.3.8 - 2006/11/23 : DB version check disabled --toecntc
#--   1.4.3.8 - 2006/11/24 : ANTms24159: a transaction survives an NT reset.  Release
#--                          it after the NT reachable again.
#--   1.5.0   - 2006/12/07 : ANTms26421 - Prepare for R3.3:
#--                          - SHUB OFLMT change: parameters via config file
#--                          - NT OFLMT change for Telefonica OAM config (--loopback)
#--   1.5.1   - 2006/12/19 : Call capability accesor to determine SHUB NT type for
#--                          offline migration.
#--   1.5.1.1 - 2007/01/08 : Skip to next ISAM when pre- or post-upgrade script fails.
#--   1.5.1.2 - 2007/01/22 : Error in migration from R2.1 (BackupShubDB: case swCmp < 0)
#--   1.5.1.3 - 2007/02/07 : ANTms33348, allow configurable dhcp option 82: confopt82
#--                                      allow configurable arberg insert: confarberg
#--   1.5.1.4 - 2007/02/15 : ANTms34745 bw=yes is the default when migrating SHUB DB
#--   1.6.0   - 2007/02/21 : ANTms26421 - SHUB support for 1.28
#--   1.6.1   - 2007/02/28 : ANTms35659 - Added code to handel SFP provisioning when
#--                          migrating from R 2.4 or 2.5 to R 3.3 only
#--                          [Implemented by Haq Furqan]
#--   1.6.2   - 2007/03/05 : ANTms37972 - SHUB OFLMT now on 1.30.
#--   1.6.3   - 2007/03/14 : ANTms38590 Also extract IGMPXmlfile for migration to R3.2
#--   1.6.3.1 - 2007/03/14 : ANTms38662 - Add environment check option.
#--   1.6.3.2 - 2007/03/20 : - ANTms40019 - Minor error in child exit monitoring
#--                          - Handle fatal returns when retrieving NT OFLMT version
#--   1.6.3.3 - 2007/03/29 : ANTms40631: NT OFLMT version 1.0.10 -> 1.0.12
#--                          - Always extract IGMP XML file when new release > 2.2
#--   1.6.3.4 - 2007/04/16 : ANTms42842: Include version 1.31 of SHUB OFLMT.
#--   1.6.3.5 - 2007/04/18 : ANTms43293: release transaction prior to re-establishing
#--                          CLI connection.  In case this would fail, transaction
#--                          will not be released (ISAM NT has a bug of not restarting
#--                          the transaction timeout after NT restart (transaction
#--                          survives a reset).
#--   1.6.3.6 - 2007/04/19 : ANTms43447.  Include support for SHUB OFLMT version 1.32
#--   1.6.3.7 - 2007/04/27 : ANTms44768: NT oflmt version is 10.1.3
#--   1.6.3.8 - 2007/05/09 : ANTms46024 - sfp reconf only for migrations from 2.4/2.5
#--                          to 3.3
#--                          ANTms46026 - SHUB OFLMT 1.33
#--   1.6.3.9 - 2007/05/16 : ANTms47166 - SHUB OFLMT 1.34
#--   1.6.3.10- 2007/05/29 ; ANTms48845 - check chmod result.
#--   1.6.3.11- 2007/06/04 : ANTms49515 - NT OFLMT version is now 11.1.0
#--   1.7.0   - 2007/05/02 : ANTms45206: prepare for R3.4/R3.5
#--                          ANTms49896: Enable forwarding of TTL=0 packet for a VRF.
#--             2007/05/04 : ANTms49911: Small correction in Ctrl-c handling (don't kill pid
#--                          that have already exited)
#--             2007/06/04 : Allow to start migration from an already uploaded DB
#--   1.7.1   - 2007/06/11 : ANTms50664 - NT OFLMT version 11.1.1 support
#--   1.7.2   - 2007/06/12 : ANTms50447: wait 3 minutes after SHUB DB save on redundant
#--                          systems before activating new OSWP.
#--   1.7.3   - 2007/06/14 : ANTms51245 - Fix flaw in transaction release at the end of the
#--                          operations.
#--             2007/06/18 : ANTms51632 - allow multiple users for same ISAM.
#--   1.7.4   - 2007/06/22 : ANTms52362 - Make sure node status result file is never
#--                          empty (Added AdvanceToNext).
#--   1.7.5   - 2007/06/29 : Fix in ANTms51632 did not work properly due to a non-0 umask.
#--   1.7.6   - 2007/07/30 : ANTms55014 NT oflmt version is now 11.2.0
#--                          Latest SHUB OFLMT is 1.36
#--             2007/08/01 : ANTms56107 - SFP reconfiguration for R3.3 or higher.
#--             2007/08/01 : ANTms55243 - Logical error when using OFLDB resulted in an
#--                          empty database.
#--   1.7.7   - 2007/08/06 : ANTms57361: LRM expects the IP@ in the NE output directory naming.
#--   1.7.8   - 2007/08/07 : ANTms12345: NT OFLMT framework version upgraded to 11.2.1
#--   1.7.9   - 2007/08/10 : ANTms59351: 1.36 of SHUB OFLMT expects new parameter.
#--   1.7.10  - 2007/08/17 : ANTms58361: "Stray handler 18".
#--             2007/08/20 : ANTms60348: NT OFLMT FW version is now 11.2.4
#--   1.7.11  - 2007/08/22 : ANTms54982: no longer perform an SHUB DB save before uploading
#--                          the NT DB when the active release is R2.3 or higher.
#--             2007/08/22 : ANTms60869: Make NT community string configurable
#--             2007/08/25 : ANTms55192: Reset NT using T&D in case the NT is still
#--                          running after an OSWP activation.
#--   1.7.12  - 2007/09/03 : ANTms62454: When the node is up too soon, the workaround
#--                          is triggered but the Telnet session is closed.  We now
#--                          specifically check on this.
#--   1.7.13  - 2007/09/05 : ANTms62648: Workaround not OK for redundant systems.
#--   1.7.14  - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.7.15  - 2007/10/02 : ANTms66036: new NT OFLMT FW version (12.1.2)
#--   1.7.16  - 2007/10/11   ANTms66037: the correct board code for NANT-B is 12396 in stead
#--                          of 12361.
#--   1.7.17  - 2007/10/17 : ANTms68417: check the number of packages in the multicast sources
#--                          ANTms68358: The NANT-B board code originally used in PBMT was correct after all.
#--                          ANTms67685: NT OFLMT version is now 12.1.3
#--   1.7.18  - 2007/10/23 : ANTms45206 NANT-A to NANT-B convertion.  SHUB OFLMT param file
#--                          was missing 1 parameter.  Moved file creation to common part.
#--   1.7.19  - 2007/11/14 : ANTms72085: SHUB OFLMT is now 1.38
#--   1.7.20  - 2007/12/18 : ANTms77027.  NT OFLMT 3.4.01 converts 64 packages as member into
#--                                       1024 packages on the condition that the highest package
#--                                       member is 64.  As such the check in PBMT is adapted to
#--                                       accept this too.
#--   1.8.0   - 2007/09/12 : IVPS offline migration for R3.5
#--   1.8.1   - 2008/01/09 : ANTms78942.  OSWP activation accompanied by an NT reset makes the OSWP activation
#--                          'fail': only the NT is activated (with the previous load).
#--   1.8.2   - 2008/01/14 : ANTms80071.  Use param.cfg for the NT community string.
#--   1.8.3   - 2008/01/10 : ANTms63550. Added functionality to configure Access OperatorId during 
#--                                      offline migration. ACCESS_OPERATOR_ID key gets written to
#--                                      dbtool.cfg which is read by OFLMT during migration and 
#--                                      stored in the DB. This functionality is available in R36 OFLMT.
#--   1.8.4   - 2008/01/20 : ANTms80082: migration of R3.6V
#--   1.8.5   - 2008/02/07 : ANTms83783: Check whether both NT and SHUB are running the new SW
#--   1.8.6   - 2008/02/29 : Ignore unknown stdby NT when status is not operational.
#--                          ANTms86893, error in IP regular expression
#--   1.8.7   - 2008/03/13:  ANTms88054.  Remove warning when Waiting for restart times out (warning in print statement)
#--   1.8.8   - 2008/04/08 : ANTmt02100.  SHUB SW version retrieval resulted in migration failure while node was running
#--                                       the new release when the NT (and the system) had restarted.
#--   1.8.9   - 2008/04/29 : ANTmt04599.  NANT-A TYPE-B SW name changed between R3.2 and R3.3
#--                                       Implement logic for SW name retrieval in case of
#--                                       'reduced' SW
#--   1.8.10  - 2008/05/06 : ANTmt05829.  Check ip address immediately after parsing the OSWP file
#--                                       in ParseCommandLine.
#--   1.8.11  - 2008/05/13 : ANTms05084.  Issue in XVPS DB migration:
#--                                       - Remove XVPS directories possibly present from previous migrations
#--                                       - Assure correct settings of source and target directories when combining
#--                                         the migrated databases.
#--   1.8.12  - 2008/05/13 : ANTmt06468.  CDE directory still in XVPS DB container.
#--   1.8.13  - 2008/05/27 : ANTmt08912.  CDE filename to be included in backup is not CDE.tar
#--   1.8.14  - 2008/05/28 : ANTmt08500.  All DB migrated to R3.6 must include the CDE directory.
#--   1.8.15  - 2008/06/14 : TransactionReleased may return undef as result.  Catch it.
#--   1.8.16  - 2008/06/24 : ANTmt13345.  Starting from R3.6 the CDE file is in a different location in the backup.
#--   1.8.17  - 2008/06/27 : ANTmt14021.  Include on CDE file presence depending on the source release.
#--   1.8.18  - 2008/07/03 : Typo in IS_ISAM_SRV.  Must be IS_ISAMV_SRV
#--   1.8.19  - 2008/07/22 : ANTmt17241.  No IgmpXml extraction when migrating from R3.1 or higher.
#--   1.8.20  - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--             2008/07/25 : Add check on file transfer protocol being used in the ISAM.
#--   1.8.21  - 2008/09/04 : ANTmt22054
#--   1.8.22  - 2008/09/24 : ANTmt24857 (and clones ANTmt24872, ANTmt24874)
#--   1.8.23  - 2008/10/02 : ANTmt23353.  Implement workaround for random initvalues in vlanStaTbl definition.
#--   1.8.24  - 2008/10/28 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.9.0   - 2008/09/05 : ANTmt44696.  Introduction of IPD ISAM in R3.7.10
#--   1.9.1   - 2009/03/13 : ANTmt47687.  SHUB OFLMT 1.49 required for migrationm to R4.0.
#--   1.9.2   - 2009/03/19 : ANTmt48958.  Use fixed public for xVPS SNMP sessions.
#--   1.9.3   - 2009/04/01 : ANTmt50504.  filetransfer settings can be different for NT and xVPS (< R3.5)
#--   1.9.4   - 2009/04/22 : ANTmt54730.  Do not create empty DB backup in case of activation only.
#                            Release transaction if TO = 127.0.0.1
#--   1.9.5   - 2009/04/30 : ANTmt55717.  Introduced NE famile ISAM_BOTH to indicate NTs that can be part of both the ETSI and ANSI stream.
#--   1.9.6   - 2009/05/06 : ANTmt54684.
#--   1.9.7   - 2009/06/02
#--   1.9.8   - 2009/06/12 : ANTmt63037 - Take maxrepetitions into account for SNMPv2/SNMPv3.  If not specified it gets calculated
#                            by the SNMP library.  For big responses the resulting message can become too big.
#--   1.9.9   - 2009/07/13 : ANTmt65332
#--   1.9.10  - 2009/07/14 : ANTmt67832 - the IVPS inventory script is only required for migrations till R3.5
#--   1.9.11  - 2009/08/12 : ANTmt71238: ECNTE conversion Changes
#--   1.9.12  - 2009/08/27 : retrieve access operator ID using SNMP
#--                          Include issprot.conf if active release is R4.0
#--   1.9.13  - 2009/09/22 : ANTmt75743
#--   1.9.14  - 2009/09/25 : ANTmt77273.  ISAM-V migration problems when using --ofldb option.
#--   1.9.15  - 2009/09/28 : Error in correction 1.9.14
#--   1.9.16  - 2009/11/10 : Fix for ANTmt82666. Also introduced check for empty response 
#--                          on DoSafeGet for SNMPv3 session
#--   1.9.17  - 2009/11/24 : ANTmt84679 & ANTmt83962 
#--   1.9.18  - 2010/03/05 : Config file name too long for XVPS migration
#--   1.9.19  - 2010/08/06 : ALU00719042 - supprot for --ofldb and sftp
#--   3.0.01  - 2010/10/15 : ALU00820717
#--   3.0.02  - 2010/10/19 : correct print log statement when wrong SWP is present as second SW in the node
#--   3.0.03  - 2010/12/08 : ALU00865237
#--   3.0.04  - 2010/12/13 : rearrange code for CDE untar related to SIP migration
#--   3.0.05  - 2010/12/17 : ALU00975861 - added CDE untar for SIP and MEGACO as as workaround
#--   3.0.06  - 2011/02/08 : ALU01033654 & ALU01033842 
#--   3.0.07  - 2011/02/11 : target release version number check included for range check of vmac host id parameter
#--   3.0.08  - 2011/02/23 : Untar of CDE done via pbmt earlier is commented
#                          : Cleanup of CDE source directory before xvps migration, untar CDE file removed for xvps
#--   3.0.09  - 2011/03/07 : undo vMAC range check for release > R4202 (ALU01033842 reworked due to change in migration tool)
#--   3.0.10  - 2011/03/27 : linux compatibility changes
#--   3.0.11  - 2011/04/05 : change directory for created log file
#--   3.0.12  - 2011/04/08 : ALU01031457 - Support for R4220 & R4230
#--   3.0.13  - 2011/05/19 : fix for upgrade in R4220/R4230
#--                        : ALU01207252
#--   3.0.14  - 2011/05/27 : ALU01236674 
#--   3.0.15  - 2011/05/31 : ALU01235324 
#--   3.0.16  - 2011/06/14 : ALU01235321 & ALU01242257
#--   3.0.17  - 2011/07/07 : Check for oflmt version info made more strict
#--   3.0.18  - 2011/11/03 : ALU01595196
#--   3.0.19  - 2011/11/14 : ALU01605960
#--   3.0.20  - 2011/12/11 : Release convertion corrected in R4230 for offline DB issue
#--   3.0.21  - 2012/01/11 : for resolving offline migration failure from R4302 to R44
#--   3.0.22  - 2012/02/03 : Suppress warnings occuring in Hairpin enabling via script for Offline db migration
#--   3.0.23  - 2012/04/30 : ALU01783098
#--   3.0.24  - 2012/05/11 : ALU01789702 - World writeable directories need sticky bit set 
#--   3.0.25  - 2012/05/25 : Ensure backward compatibility for Lanx Checksum tool CalcIpCheckSum 
#--   3.0.26  - 2012/06/28 : ALU01804629
#--   3.0.27  - 2012/09/12 : ALU01819262
#--   3.0.28  - 2012/09/27 : ALU01826095
#--   3.0.29  - 2012/10/22 : ALU01761938
#--   3.0.30  - 2012/10/30 : ALU01836984, ALU01836986, ALU01837839
#--   3.0.31  - 2012/11/17 : ALU01836902
#--   3.0.32  - 2012/12/19 : ALU01844812
#--   3.0.33  - 2013/01/07 : ALU01842744
#--   3.0.34  - 2013/02/06 : enable usage of second password if first one fails for cli over ssh
#--   3.0.35  - 2013/06/18 : ALU01890986
#--   3.0.36  - 2013/06/27 : determine source sw for offline DB's in R4601
#--   3.0.37  - 2013/08/12 : ALU01903056
#--   3.0.38  - 2013/09/23 : ALU01901387
#--   3.0.39  - 2013/11/06 : ALU01927702
#--   3.0.40  - 2014/03/25 : support for new db version in R5001 (ALU01963317)
#--   3.0.41  - 2014/06/09 : ALU01976828
#--   3.0.42  - 2014/07/15 : ALU01980121
#--   3.0.43  - 2014/10/17 : ALU02013702
#--   3.0.44  - 2015/05/20 : ALU02082098
#--   3.0.45  - 2015/09/03 : support for new db version in R5301 
#--   3.0.46  - 2015/10/13 : ALU02135551
#--   3.0.47  - 2016/03/16 : ALU02190558
#--   3.0.48  - 2016/05/06 : R5203 changes
#--   3.0.49  - 2016/07/18 : ALU02237966
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "MigrateIsam7302Etsi.pm - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "MigrateIsam7302Etsi.pm - 3.1.0 -- 2018/06/20";
#==============================================================================

use strict;
use warnings 'all';
require 5.001;

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>ss
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2092-DFZZA">DM</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;
use File::Basename;
use Cwd qw(getcwd);
use Config; 

use Net::SNMP;
use Net::TFTP;

use IsamCli;
use Utils;
require Octopus;
use Capability;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :MIGR :PRCK);
require FamilyCommon;
use DM;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetActiveLanxConf($$);
sub BackupShub($$$$$$);
sub UploadCompleteDbTftp($$$$$);
sub ModifyDbtoolCfg($$$);
sub ModifyXvpsDbtoolCfg($$$$);
sub MigrateNTDbase($$$$$);
sub ExtractIGMPXmlFile($$$$);
sub ChangeNtType($$$$$$);
sub PrepareVlanStaTbl($$$$);
sub MigrateSHUBDbase($$$$$$;$);
sub GetIvpsBackup($$$$);
sub LocateCDE($$$$);
sub MigrateXvpsContainer($$$$$$);
sub PrepareXvpsContainers($$$$);
sub MigrateXvpsDbase($$$$$$$$$$$);
sub GetXvpsIsamListByIP($$);
sub GetXvpsIsamListByCommunity($$$);
sub GetXvpsDependancyByIP($$);
sub GetXvpsDependancyByCommunity($$$);
sub MakeFakeList($$);
sub GetIvpsIpAddresses($$);
sub CopyXvpsList($$);
sub CopyNTDbase($$);
sub AddCdeFiles($$$$);
sub MakeDir($$$);
sub CombineDbases($$$$$$);
sub DownloadDbase($$$$$$);
sub DownloadSHUBDbase($$$$);
sub SwChecksOK($$$$$$$$$);
sub SaveShubDbase($$$);
sub SetAndCheckNewLanxConf($$$);
sub RebootAndChangeLanxToTftp($$$$$$$);
sub DetermineNtCommunities($$);
sub DetermineXvpsPresence($$);
sub IsamSystemRestart($$$$$$$$$$$$$$);
sub MigrateOrUpgrade($$$$);
sub Migrate($$$);
sub CheckEnvironment($$$);
sub PrintToolVersions($$);
sub DetermineSwVersion($);
sub SetToolData();
sub ResetGlobals();
#==============================================================================
#== Constants
#==============================================================================

my $NTINFILENAME = "dm_complete.tar";           # input name of the complete NT database
my $NTSMALLFILE = "dm.tar";                     # input name of the "small" NT database (only used with ofldb).
my $NTOUTFILENAME = "dm_migrated.tar";          # Name of the complete NT database after migration
my $SHUBNVRAMFILENAME = "issnvram.txt";         # Name of R2.0/R2.1 issnram file
my $SHUBINFILENAMER1 = "iss.conf";              # input name of the SHUB R2.2 database
my $SHUBPROTINFILENAME = "issprot.conf";
my $SHUBOUTFILENAME = "iss.conf.new";           # input name of the SHUB R2.2 database
my $SHUBINFILENAMER2 = "dmlanx.tar";            # input name of the SHUB R2.2 database
my $SHUBSWCHECK = "shubSwCheck.sh";
my $SHUBLANXCHECK = "shubLanxCopy.sh";
my $SHUBMIGPARS = "shubmigpars.cfg";

my %SYNCVALUES = (0 , 'disabled',
		  1 , 'enabled');

my $ISNOTISAMV_RE =    "is not an ISAMV";
my $ISISAMV_RE =       ".VPS-(\\d):(\\d+\.\\d+\.\\d+\.\\d+)";
my $IVPSCONTAINER =    "XVPS%d";
my $IVPSDBASE =        "Dbase";
my $CDECONTNAME =      "CDE";
my $CDEACTDIR =        "active";
my $CDETMPDIR =        "cdetmp";
my $CDEPATHVAR =       "CDE_FILE_PATH";
my $NEWCDEPATHVAR =    "NEW_CDE_FILE_PATH";
my $ECNTABRDCODE =     12355;                          # -- 1.4.2
my $ECNTCBRDCODE =     12359;                          # -- 1.4.2
my $NANTABRDCODE =     12360;
my $NANTBBRDCODE =     12361;
my $R2_4_RESCUE =      "resetShell.cmd";
my $LOWEST_XVPSNUM =   1;
my $HIGHEST_XVPSNUM =  11;
my $NTMIGFILE =        "migNt.txt";
my $SHUBMIGFILE =      "migShub.txt";
my $XVPSMIGFILE =      "migXvps%d.txt";
my $R3604G      =      "36\.559";   # 3604g & 3604k emergency build user-to-user hairpin feature enabled
my $R3604K      =      "36\.564";  
my $DT_ACCOPR =      260;
#==============================================================================
#== Vars
#==============================================================================
my %warnLevels;
my $config;
my $toolData;
my $NTOFLMTVERSION = 0; 

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

sub ResetGlobals() {
   $NTOFLMTVERSION = 0;
}

=head2 GetActiveLanxConf($$)

=begin html
Determine active LANX configuration (conf1, conf2 or tftp or undef in case of failure).<br>
The following parameters are used:
<table>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>relNum</td><td>the release number of the active SW</td></tr>
</table>

=end html

=cut

sub GetActiveLanxConf($$) {
  my ($cli, $relNum) = @_;
  my $showShubConf = Capability::ReleaseCompare($relNum, Capability::ConvertRelease(1,1), REGEXP, 0) == 0 ? "" : "config-file";
  my $activeConf = "";
  my $trials = 1;

  while (($activeConf eq "") && ($trials < 5)) {
    # Determining active LANX conf after an SHUB restart may involve more then a single command
    # since alarms are being printed on the console...
    $activeConf = IsamCli::GetLanxActiveOSWP($cli, $showShubConf);
    $trials = 5 if (defined($activeConf) && ($activeConf ne ""));
    $trials++;
  }
  return $activeConf;
}

=head2 BackupShub($$$$$$$)

=begin html
Backup the SHUB database.  Depending on the active release:
<ul>
<li>R2.2: Upload dmlanx.tar</li>
<li>&lt; R2.2: copy the database from SHUB config1 or config2 and change LANX config to new config</li>
<li>&gt; R2.2: included in the NT backup</li>
</ul>
Returns 0 in case of error.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>swpInfo</td><td>pointer of the SWP information as retrieved from the NE (both active and passive OSWP)</td></tr>
  <tr><td>ip</td><td>TFTP server IP address</td></tr>
  <tr><td>userName</td><td>Unix user name in case of multi-user operation (command line option) empty otherwise</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub BackupShub($$$$$$) {
  my ($fh, $cli, $isam, $swpInfo, $ip, $userName) = @_;
  my $dir = "";
  my $ok = 1;
  my $swRelCmp = Capability::ReleaseCompare(Capability::ConvertRelease($swpInfo->{ACTIVE}{MAJOR}, $swpInfo->{ACTIVE}{MINOR}),
					    Capability::ConvertRelease(2, 2), REGEXP, 0);
  my ($time1, $time2);
  my $ret;
  my $bytes = 0;

  ToolCommon::Log($fh, "Backup SHUB database...");
  if ($swRelCmp == 0) {
    my $tftp = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});  # -- 1.1.6
    if (defined($tftp)) {
      $tftp->binary();
      # In case of active SW running R2.2
      $dir = sprintf("$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/%s/$userName", $params->{NONAME} ? $isam->[0] : $isam->[1]);  # -- 1.1.6
      $time1 = time();
      if ($tftp->get("/pub/Lanx/Dbase/Upload/$SHUBINFILENAMER2", "$dir/$SHUBINFILENAMER2") ne "") {
	my $tftpErr = $tftp->error();
	FamilyCommon::SetAndLogError($fh, "tftp get /pub/Lanx/Dbase/Upload/$SHUBINFILENAMER2 failed: $tftpErr",
				     "  FAILED tftp get /pub/Lanx/Dbase/Upload/$SHUBINFILENAMER2 failed: $tftpErr");
	$ok = 0;
      } else {
	$time2 = time();
	$bytes = $tftp->bytesTransfered();
	if ($bytes > 0) {
	  ToolCommon::Log($fh, sprintf("  OK transfered %d bytes in %d seconds\n", $tftp->bytesTransfered(), $time2 - $time1));
	  ToolCommon::Log($fh, "  Filename = $dir/$SHUBINFILENAMER2\n") if $params->{DEBUG};
	} else {
	  FamilyCommon::SetAndLogError($fh, "Empty file $dir/$SHUBINFILENAMER2 retrieved");
	  $ok = 0;
	}
      }
    } else {
      FamilyCommon::SetAndLogError("Failed to open tftp session", "  FAILED - could not open tftp session\n");
      $ok = 0;
    }
  } elsif ($swRelCmp < 0) {  # -- Added 1.1.7
    my $activeConf = GetActiveLanxConf($cli, $swpInfo->{ACTIVE}{NAME});
    my $showShubConf = Capability::ReleaseCompare(Capability::ConvertRelease($swpInfo->{ACTIVE}{MAJOR}, $swpInfo->{ACTIVE}{MINOR}),
						  Capability::ConvertRelease(1,1), REGEXP, 0) == 0 ? "" : "config-file";
    my $cliConfig = $activeConf eq "config-1" ? "config1" : "config2";
    my $newConfig = $activeConf eq "config-1" ? "config-2" : "config-1";
    $dir = sprintf("$toolData->{TFTPWORKDIR}/%s/$userName", $params->{NONAME} ? $isam->[0] : $isam->[1]);
    if ($activeConf eq "") {
      my $t = sprintf("FAILED to get active SHUB OSWP: %s", IsamCli::GetError());
      FamilyCommon::SetAndLogError ($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      $ret = IsamCli::GetLanxDbFile($cli, "$cliConfig/$SHUBINFILENAMER1", "$ip:$dir/$SHUBINFILENAMER1", $showShubConf);
      if (defined($ret) && $ret) {
	$ret = IsamCli::GetLanxDbFile($cli, $SHUBNVRAMFILENAME, "$ip:$dir/$SHUBNVRAMFILENAME", $showShubConf);
        if (defined($ret) && $ret) {
	  $bytes = -s "$toolData->{TFTPDIR}/$dir/$SHUBNVRAMFILENAME";   # 1.5.1.2 add $TFTPDIR
	  if ($bytes > 0) {
	    ToolCommon::Log($fh, "  OK\n");
	  } else {
	    FamilyCommon::SetAndLogError($fh, "Empty file $toolData->{TFTPDIR}/$dir/$SHUBNVRAMFILENAME retrieved",         # 1.5.1.2 add $TFTPDIR
					 "  FAILED: empty file $toolData->{TFTPDIR}/$dir/$SHUBNVRAMFILENAME retrieved");   # 1.5.1.2 add $TFTPDIR
	    $ok = 0;
	  }
        } else {
	  $ok = 0;
	  my $t = sprintf("Could not retrieve $SHUBNVRAMFILENAME: %s", IsamCli::GetError());
	  FamilyCommon::SetAndLogError($fh, $t, "  FAILED - $t\n");
        }
      } else {
        $ok = 0;
	my $t =  sprintf("Could not retrieve $SHUBINFILENAMER1: %s", IsamCli::GetError());
	FamilyCommon::SetAndLogError($fh, $t, "  FAILED - $t\n");
      }
    }
  } else {
    my $t = "Non-supported SHUB sw version: $swpInfo->{ACTIVE}{MAJOR}.$swpInfo->{ACTIVE}{MINOR}";
    FamilyCommon::SetAndLogError($fh, $t, "  FAILED - $t\n");
    $ok = 0;
  }
  return $ok;
}

=head2 UploadCompleteDbTftp($$$$$)

=begin html
Upload the NT database using tftp (so NOT a CLI backup).  Returns 0 in case of error.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>tftpIp</td><td>TFTP server IP address</td></tr>
  <tr><td>path</td><td>directory where the backup needs to be uploaded</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

#------------------------------------------------------------------------------
#-- 1.1.9: 
#------------------------------------------------------------------------------
sub UploadCompleteDbTftp($$$$$) {
  my ($fh, $debug, $tftpIp, $path, $isam) = @_;
  my $ok = 1;
  my $sourceFile = "/pub/Dbase/Upload/ActActive/$NTINFILENAME";
  my $destFile = "$path/$NTINFILENAME";
  my ($time1, $time2);

  my $tftp = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});
  if (defined($tftp)) {
    $tftp->binary();
    ToolCommon::Log($fh, sprintf("\ntftp %s: get %s %s\n", $isam->[0], $sourceFile, $destFile)) if $debug;
    $time1 = time();
    if ($tftp->get($sourceFile, $destFile) ne "") {
      my $t = sprintf("tftp get %s %s failed: %s", $sourceFile, $destFile, $tftp->error());
      FamilyCommon::SetAndLogError($fh, $t, "  FAILED - $t\n");
      $ok = 0;
    } else {
      $time2 = time();
      ToolCommon::Log($fh, sprintf("  OK, retrieved %d bytes in %d seconds\n", $tftp->bytesTransfered(), $time2 - $time1));
    }
  } else {
    my $t = "Failed to open tftp session";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  }
  return $ok;
}

=head2 ModifyDbtoolCfg($$$$)

=begin html
Modify the NT OFLMT dbtool.cfg file.  Used in case --confopt82 or --arberg option is used.  Depending on the options enabled some lines will be
added to the dbtool.cfg file allowing migration to parameter values as used by the customer.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>inFile</td><td>full file specification of the source config file</td></tr>
  <tr><td>outFile</td><td>full file specification of the new config file</td></tr>
</table>

=end html

=cut

sub ModifyDbtoolCfg($$$) {
  my ($fh, $inFile, $outFile) = @_;
  my $ok = 1;
  my $line;
  my $option82Present = 0;
  my $arbergPresent = 0;
  my $accessOperatorIdPresent = 0;
  my $ssmOutVlanPresent = 0; 
  my $cdeFilePathPresent = 0;
  my $igmpMaxUncfgPresent = 0;
  my $err;
  my ($input, $output);
  if (open($input, $inFile)) {
    if (open($output, ">" . $outFile)) {
      while ($line = <$input>) {
	if (($params->{CONFOPT82}) && ($line =~ /^SYS_PORT_NUM_MED/)) {
	  $option82Present = 1;
	}
	if (($params->{CONFARBERG}) && ($line =~ /^ARBERGTAG/)) {
	  $arbergPresent = 1;
	}
	if (($params->{ACCESSOPRID}) && ($line =~ /^ACCESS_OPERATOR_ID/)) {
	  $accessOperatorIdPresent = 1;
	}
	if (($params->{SSM_OUT_VLAN}) && ($line =~ /^SSM_OUT_VLAN/)) {
          $ssmOutVlanPresent = 1; 
	}
        if (($params->{ISAM_HAS_VOICE}) && ($line =~ /^CDE_FILE_PATH/)) {
          $cdeFilePathPresent = 1;
	}
        if ($params->{IGMP_MAX_UNCFG} && ($line =~ /^IGMP_MAX_UNCFG/)) {
           $igmpMaxUncfgPresent = 1;
	}
	ToolCommon::Log($fh, "Read: '$line'") if ($params->{DEBUG});
	print $output $line;
      }
      if ((! $option82Present) && ($params->{CONFOPT82})) {
	$err = "";
	ToolCommon::Log($fh, "  Configure for DHCP option 82...\n") if ($params->{DEBUG});
	if (defined($config->{SYS_PORT_NUM_MED})) {
	  if (($config->{SYS_PORT_NUM_MED} >= 1) && ($config->{SYS_PORT_NUM_MED} <= 4)) {
	    print $output sprintf("SYS_PORT_NUM_MED    $config->{SYS_PORT_NUM_MED}\n");
	  } else {
	    $ok = 0;
	    $err .= "SYS_PORT_NUM_MED has wrong value ($config->{SYS_PORT_NUM_MED}): can only be 1..4";
	  }
	} else {
	  $ok = 0;
	  $err .= "SYS_PORT_NUM_MED not defined";
	}
	if (defined($config->{ATM_BASED_SYNTAX})) {
	  print $output sprintf("ATM_BASED_SYNTAX    $config->{ATM_BASED_SYNTAX}\n");
	} else {
	  $ok = 0;
	  $err .= "ATM_BASED_SYNTAX not defined";
	}
	if (defined($config->{ETH_BASED_SYNTAX})) {
	  print $output sprintf("ETH_BASED_SYNTAX    $config->{ETH_BASED_SYNTAX}\n");
	} else {
	  $ok = 0;
	  $err .= "ETH_BASED_SYNTAX not defined";
	}
	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }
      if ((! $arbergPresent) && ($params->{CONFARBERG})) {
	ToolCommon::Log($fh, "  Configure for Argberg insertion...\n") if ($params->{DEBUG});
	if (defined($config->{ARBERGTAG})) {
	  if (($config->{ARBERGTAG} >= 1) && ($config->{ARBERGTAG} <= 2)) {
	    print $output sprintf("ARBERGTAG    $config->{ARBERGTAG}\n");
	  } else {
	    $ok = 0;
	    $err .= "ARBERGTAG has wrong value ($config->{ARBERGTAG}): can only be 1..2";
	  }
	} else {
	  $ok = 0;
	  $err .= "ARBERGTAG not defined";
	}
	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }
      if ((! $accessOperatorIdPresent) && ($params->{ACCESSOPRID})) {
	ToolCommon::Log($fh, "  Configure for Access OperatorId insertion...\n") if ($params->{DEBUG});
	if (defined($config->{ACCESS_OPERATOR_ID})) {
	  if (($config->{ACCESS_OPERATOR_ID} >= 0) && ($config->{ACCESS_OPERATOR_ID} <= 65535)) {
	    print $output sprintf("ACCESS_OPERATOR_ID    $config->{ACCESS_OPERATOR_ID}\n");
	  } else {
	    $ok = 0;
	    $err .= "ACCESS_OPERATOR_ID has wrong value ($config->{ACCESS_OPERATOR_ID}): can only be 0..65535";
	  }
	} else {
	  $ok = 0;
	  $err .= "ACCESS_OPERATOR_ID not defined";
	}
	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }
      if ((! $ssmOutVlanPresent) && defined($params->{SSM_OUT_VLAN})) {
	ToolCommon::Log($fh, "  Configure for SSM Out VLAN...\n") if ($params->{DEBUG});
	  if (($params->{SSM_OUT_VLAN} > 0) && ($params->{SSM_OUT_VLAN} < 4089)) {
	    print $output sprintf("SSM_OUT_VLAN $params->{SSM_OUT_VLAN}\n");
          } else {
	    $ok = 0;
	    $err .= "SSM_OUT_VLAN has wrong value ($params->{SSM_OUT_VLAN}): can only be 1..4088";
          }

	if (! $ok) {
	  FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	}
      }

      if(defined($params->{ENABLE_IGMP_GROUPING}) && defined($params->{MAX_GIP_GAP})) {
          print $output sprintf("ENABLE_IGMP_GROUPING $params->{ENABLE_IGMP_GROUPING}\n");
          print $output sprintf("MAX_GIP_GAP $params->{MAX_GIP_GAP}\n");
      }

      if((! $igmpMaxUncfgPresent) && (defined($params->{IGMP_MAX_UNCFG}))) {
          print $output sprintf("IGMP_DTAG_UNCFG 1\n");
      }

      if ((! $cdeFilePathPresent) && ($params->{ISAM_HAS_VOICE})) {
        if($params->{ISAM_VOICE_MIGR}) { 
	  ToolCommon::Log($fh, "  Configure for CDE File Path ...\n") if ($params->{DEBUG});
          if(defined($params->{CDE_TMP_PATH}) && ($params->{CDE_TMP_PATH} ne "")) {
            print $output "$CDEPATHVAR    $params->{CDE_TMP_PATH}\n";
			print $output "$NEWCDEPATHVAR    $params->{CDE_TMP_PATH}\n";
          } else {
	    $ok = 0;
	    $err .= "ERROR: CDE Temp Path is empty.";
          }

	  if (! $ok) {
	    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
	  }
        }
      }
      close($output), close($input);
    } else {
      $err = "Failed to open $outFile for writing: $!";
      FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
      $ok = 0;
    }
  } else {
    $err = "Failed to open $inFile for reading: $!";
    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
    $ok = 0;
  }
  return $ok;
}

=head2 ModifyXvpsDbtoolCfg($$$$$)

=begin html
Modify the XVPS OFLMT dbtool.cfg file.  Used in case of migration to R3.6V where a path to the untarred CDE files needs to be included to allow
migration from CDE files to the database.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>inFile</td><td>full file specification of the source config file</td></tr>
  <tr><td>outFile</td><td>full file specification of the new config file</td></tr>
  <tr><td>cdePath</td><td>The complete path where the CDE pacakge was untarred.</td></tr>
</table>

=end html

=cut

sub ModifyXvpsDbtoolCfg($$$$) {
  my ($fh, $inFile, $outFile, $cdePath) = @_;
  my $ok = 1;
  my $line;
  my $option82Present = 0;
  my $arbergPresent = 0;
  my $err;
  my ($input, $output);

  if (open($input, $inFile)) {
    if (open($output, ">" . $outFile)) {
      while ($line = <$input>) {
	ToolCommon::Log($fh, "Read: '$line'") if ($params->{DEBUG});
	print $output $line if (! ($line =~ /$CDEPATHVAR/));
      }
      if (defined($cdePath)) {
        $cdePath =~ s/\/\//\//g;
        print $output "$CDEPATHVAR    $cdePath\n";
      }
      close($output), close($input);
    } else {
      $err = "Failed to open $outFile for writing: $!";
      FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
      $ok = 0;
    }
  } else {
    $err = "Failed to open $inFile for reading: $!";
    FamilyCommon::SetAndLogError($fh, $err, "  $err\n");
    $ok = 0;
  }
  return $ok;
}

=head2 MigrateNTDbase($$$$$$)

=begin html
Migration of the NT database.  If requested NT NW-type convertion will be performed once the database has been migrated.  Returns 0 in case of error.
<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>path</td><td>PBMT installation directory name</td></tr>
</table>

=end html

=cut

sub MigrateNTDbase($$$$$) {
  my ($fh, $dir, $from, $to, $path) = @_;
  my $thisPath = getcwd();
  my $ok = 1;
  my $fromRel = $params->{SOURCE_DBVERSION};
  my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $fromRel);
  my $toRel = $params->{DBVER_TGTREL};
  my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $toRel);
  my $cmd = "";
  my $ntDbName;
  my $ntInName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
  my $cdeTmpPath = "";

  if (defined($toRel) && defined($fromRel)) {
      if ($params->{ISAM_HAS_VOICE}) {
         $params->{ISAM_VOICE_MIGR} = 1;

         my $cdeTarFileName = "";
         my $logDir = FamilyCommon::GetNeStorage();
         my $cdeFileName = undef;  

         my $xvpsDbPath = $toolData->{FXRDATA}{HOMEDIR};  # Can be ftp or sftp DB upload
         my $dirName = defined($params->{OFLDB}) ? FamilyCommon::GetNeStorage() : $xvpsDbPath; 

         if ($ok) {
           ToolCommon::Log($fh, "Preparing CDE files for IVPS ...\n") if ($params->{DEBUG});
           if (! -e "$dirName/$CDETMPDIR") {
               $cmd = "cd $dirName;mkdir $CDETMPDIR;cd $CDETMPDIR;$config->{TAR} xf $logDir/$NTINFILENAME $CDECONTNAME >$logDir/MIGRUntarCDE1.txt 2>&1";
           } else {
               $cmd = "cd $dirName/$CDETMPDIR;$config->{RM} -rf * > /dev/null 2>&1;$config->{TAR} xf $logDir/$NTINFILENAME $CDECONTNAME >$logDir/MIGRUntarCDE2.txt 2>&1";
           }
           ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
           my $ret = system($cmd);
           ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});

           if($ret) {
              my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
              my $tMsg = "$cmd failed: $!";
              FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dirName')\n");
              $ok = 0;
           } else {
              ToolCommon::Log($fh, "Preparation of CDE files Successful ...\n") if ($params->{DEBUG});
           }
         }

         $cdeFileName = LocateCDE($fh, "$dirName/$CDETMPDIR/$CDECONTNAME", $config->{CDE_REGEXP}, $dirName);

         if (! defined($cdeFileName)) {
           my $tMsg = "No CDE file was found in the $dir/$CDETMPDIR/$CDECONTNAME directory";
           FamilyCommon::SetAndLogError($fh, $tMsg, " ERROR : $tMsg (check logfiles in working directory '$dir')\n");
           $ok = 0;
         } else {
           $cdeTarFileName = "$dir/$CDETMPDIR/$CDECONTNAME/$cdeFileName";
           $cdeTmpPath = "$dir/$CDETMPDIR/$CDECONTNAME"; 
           $params->{CDE_TMP_PATH} = $cdeTmpPath; 
         }
         # Workaround for untar of CDE files 
         #if ($ok) {
         #   $cmd = "cd $dir/$CDETMPDIR/$CDECONTNAME; $config->{TAR} xf $cdeTarFileName >$logDir/MIGRUntarCDE1.txt 2>&1";
         #   ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
         #   my $ret = system($cmd);
         #   ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});

         #   if($ret) {
         #     my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
         #     my $tMsg = "$cmd failed: $!";
         #     FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
         #     $ok = 0;
         #   } else {
         #     $cdeTmpPath = "$dir/$CDETMPDIR/$CDECONTNAME"; 
         #   }
         #}
         # Cleanup of CDE source directory done here
         if ($ok) {
           if (-e "$dir/$CDECONTNAME") {
             $cmd = "cd $dir/$CDECONTNAME;$config->{RM} -rf * > $dir/MIGRntCleanCDE.txt 2>&1";
             ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
             my $ret = system($cmd);
             ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
             if($ret) {
                my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
                my $tMsg = "$cmd failed: $!";
                FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg \n");
                $ok = 0;
             }
           }
         }
      }
      if ($from ne $to) {
	if ($params->{CONFOPT82} || $params->{CONFARBERG} || $params->{ACCESSOPRID} || $params->{ISAM_HAS_VOICE} || defined($params->{SSM_OUT_VLAN}) || defined($params->{ENABLE_IGMP_GROUPING}) || defined($params->{MAX_GIP_GAP})  || defined($params->{IGMP_MAX_UNCFG}))  {
	  if (!ModifyDbtoolCfg($fh, "$path/$toolData->{NTDBCFG}", "$dir/$toolData->{NTDBCFGMOD}")) {
	    # SetAndLogError already done.
	    $ok = 0;
	  }
	} else {
	  if(!copy("$path/$toolData->{NTDBCFG}", "$dir/$toolData->{NTDBCFGMOD}")) {
	    my $t = "Copy of $path/$toolData->{NTDBCFG} to $dir/$toolData->{NTDBCFGMOD} failed: $!";
	    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	    $ok = 0;
	  }
	}
    unlink "$dir/autoswitch_error.log";
    if (-f "$dir/autoswitch_error.log" ) {
        ToolCommon::Log( $fh, "ERROR : Autoswitch error file '$dir/autoswitch_error.log' not removed properly'\n");
    }
	if ($ok) {
	  $cmd = "cd $dir;$path/$toolData->{NTDBTOOL} -in $ntInName -out $NTOUTFILENAME -cfg ./$toolData->{NTDBCFGMOD}" .
	    " -from $fromRel -to $toRel -trc >$NTMIGFILE 2>&1";
	  ToolCommon::Log($fh, "Migrating NT database from $fromRelInfo ($fromRel) to $toRelInfo ($toRel)...");
	  ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
	  my $ret = system($cmd);
	  if($ret) {
	    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	    my $tMsg = "";
	    if ($exitCode == 3) {
	      my $migLogFile;
	      if (open($migLogFile, "$dir/trc.log")) {
		while (my $line = <$migLogFile>) {
		  chomp $line;
		  if ($line =~ /NON_RECOV/) {
		    $line = <$migLogFile>, $line = <$migLogFile>, $line = <$migLogFile>, $line = <$migLogFile>, chomp $line;
		    $line =~ m/error info\s*:(.*)/;
		    $tMsg = "Unrecoverable error occured: $1";
		  }
		}
		close($migLogFile);
	      } else {
		$tMsg = "Opening $dir/trc.log failed: $!";
	      }
	    } elsif ($exitCode == 2) {
	      $tMsg = "Consistency checks failed";
	    } else {
	      $tMsg = "Migration failed";
	    }
	    FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
	    $ok = 0;
	  } else {
	    $ntDbName = $NTOUTFILENAME;
	    ToolCommon::Log( $fh, "  OK\n");
	  }
      if (-f "$dir/autoswitch_error.log" ) {
          ToolCommon::Log( $fh, "ERROR : Autoswitch error file '$dir/autoswitch_error.log' found'\n");
      }
	}
      } else {
	$ntDbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
	ToolCommon::Log($fh, "  NT Migration not needed, same release\n");
      }
      if ($ok) {
	# -- 1.4.2
	if ($params->{TOECNTC} || ($params->{TONANTB})) {
	  $ok = ChangeNtType($fh, $ntDbName, $params->{DEBUG}, $dir, $params->{TOECNTC} ? $ECNTABRDCODE : $NANTABRDCODE, $path);
	}
	# -- 1.4.2
      }
  } else {
    FamilyCommon::SetAndLogError($fh, "Migrating NT database from $from to $to not possible: DB version not defined");
    $ok = 0;
  }
  return $ok;
}

=head2 ExtractIGMPXmlFile($$$$$)

=begin html
Extract the IgmpMcTable from the NT backup.  This file is required when migrating the SHUB database.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>fromNum</td><td>the NE release coming from</td></tr>
  <tr><td>path</td><td>directory holding the NT OFLMT configuration file</td></tr>
</table>

=end html

=cut

sub ExtractIGMPXmlFile($$$$) {
  my($fh, $dir, $fromNum, $path) = @_;
  my $cmd = "cd $dir;$config->{RM} -rf $dir/data >MIGRextrRM.txt 2>&1;$config->{MKDIR} $dir/data";
  my $ok = 1;
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;

  ToolCommon::Log($fh, "CMD = $cmd\n") if $params->{DEBUG};
  my $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "Failed to create data directory (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "  data directory created\n") if $params->{DEBUG};
    $cmd = "cd $dir;($config->{SLEEP} 5;$config->{ECHO} dm table2xml IgmpMcTable;$config->{SLEEP} 5;$config->{ECHO} exit) | " .
      "$path/$toolData->{NTDBTOOL} -in $dbName -dbg >MIGRextrIGMP.txt 2>&1";
    ToolCommon::Log($fh, "  CMD = $cmd\n") if $params->{DEBUG};
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t, check log file in '$dir'...\n");
      $ok = 0;
    } else {
      if (-e "$dir/data/IgmpMcTable") {
	if (-d "$dir/Lanx") {
	  if (Capability::ReleaseCompare($fromNum,  Capability::ConvertRelease(2, 3), REGEXP, 0) >= 0) {
	    if (!copy("$dir/data/IgmpMcTable", "$dir/Lanx/IGMPXmlfile.txt")) {
	      my $t = " Copy of '$dir/data/IgmpMcTable' to '$dir/Lanx/IGMPXmlfile.txt' failed: $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    }
	  } else {
	    if (! copy("$dir/data/IgmpMcTable", "$dir/IGMPXmlfile.txt")) {
	      my $t = " Copy of '$dir/data/IgmpMcTable' to '$dir/IGMPXmlfile.txt' failed: $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    }
	  }
	} else {
	  if (Capability::ReleaseCompare($fromNum,  Capability::ConvertRelease(2, 3), REGEXP, 0) >= 0) {
	    if (! MakeDir($fh, "$dir/Lanx", $params->{DEBUG})) {
	      $ok = 0;
	    } else {
	      if (! copy("$dir/data/IgmpMcTable", "$dir/Lanx/IGMPXmlfile.txt")) {
		my $t = " Copy of '$dir/data/IgmpMcTable' to '$dir/Lanx/IGMPXmlfile.txt' failed: $!";
		FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
		$ok = 0;
	      }
	    }
	  } else {
	    if (!copy("$dir/data/IgmpMcTable", "$dir/IGMPXmlfile.txt")) {
	      my $t = " Copy of '$dir/data/IgmpMcTable' to '$dir/IGMPXmlfile.txt' failed: $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    }
	  }
	}
	if ($ok) { 
	  ToolCommon::Log($fh, "  OK\n");
	}
      } else {
	my $t = "No IgmpMcTable file created";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      }
      my $filesDel = unlink <$dir/prozone*>;
      ToolCommon::Log($fh, "  $filesDel prozone files deleted from $dir\n") if $params->{DEBUG};
      $filesDel = unlink <$path/prozone*>;
      ToolCommon::Log($fh, "  $filesDel prozone files deleted from $path\n") if $params->{DEBUG};
      $filesDel = unlink <$path/dmModified.tar>;
      ToolCommon::Log($fh, "  $filesDel dmModified.tar deleted from $path\n") if $params->{DEBUG};
    }
  }
  return $ok;
}
#-- End BB 1.1.1

=head2 ChangeNtType($$$$$$)

=begin html
Convert the NT HW type.  Currently the following convertions are supported:
<ul>
  <li>ECNT-A to ECNT-C (from R2.4.01 onwards)</li>
  <li>NANT-A to NANT-B (from R3.5 onwards)</li>
</ul>
Returns 0 in case of error.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>ntDbName</td><td>database filename</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>oldBoardCode</td><td>Board code of the NT to be converted (all occurences)</td></tr>
  <tr><td>path</td><td>directory holding the NT OFLMT configuration file</td></tr>
</table>

=end html

=cut

#------------------------------------------------------------------------------
#-- Added 1.4.2
#------------------------------------------------------------------------------
sub ChangeNtType($$$$$$) {
  my($fh, $ntDbName, $debug, $dir, $oldBoardCode, $path) = @_;
  my $cmd = "";
  my $ok = 1;
  my $ret;
  my $newBoardCode = $oldBoardCode == $ECNTABRDCODE ? $ECNTCBRDCODE : $NANTBBRDCODE;
  my ($fd1, $fd2);

  ToolCommon::Log($fh, "Changing NT type in DB...");
  $cmd = "cd $dir;($config->{SLEEP} 5;$config->{ECHO} dm table2dml eqB /a EqBoard;$config->{SLEEP} 5;$config->{ECHO} exit ) | " .
    "$path/$toolData->{NTDBTOOL} -in $ntDbName -dbg -out t.tar >MIGRchgNtTyp.txt 2>&1";
  ToolCommon::Log($fh, "  CMD = $cmd\n") if ($debug);
  $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t, check log file in '$dir'...\n");
    $ok = 0;
  } else {
    if (-e "$dir/eqB") {
      if (open($fd1, "$dir/eqB")) {
	if (open($fd2, ">$dir/neweqB")) {
	  my $line;
	  while ($line = <$fd1>) {
	    if ($line =~ /Type=$oldBoardCode/) {
	      $line =~ s/$oldBoardCode/$newBoardCode/;
	    }
	    print $fd2 $line;
	  }
	  close($fd1), close($fd2);
	  $cmd = "cd $dir;($config->{SLEEP} 5;$config->{ECHO} dm loaddml neweqB;$config->{SLEEP} 5;$config->{ECHO} exit) | " .
	    "$path/$toolData->{NTDBTOOL} -in $ntDbName -out t.tar -dbg >MIGRload.txt 2>&1";
	  $ret = system($cmd);
	  ToolCommon::Log($fh, "  CMD = $cmd\n") if ($debug);
	  if ($ret) {
	    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	    my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
	    FamilyCommon::SetAndLogError($fh, $t, "  $t, check log file in '$dir'...\n");
	    $ok = 0;
	  } else {
	    if (!rename("$dir/t.tar", "$dir/$NTOUTFILENAME")) {
	      $ok = 0;
	      my $t = "Renaming temp tar file to $dir/$NTOUTFILENAME failed: $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	    } else {
	      ToolCommon::Log($fh, "  OK!\n");
	      unlink "$dir/eqB", unlink "$dir/neweqB";
	      my $filesDel = unlink <$dir/prozone*>;
	      ToolCommon::Log($fh, "  $filesDel prozone files deleted from $dir\n") if ($debug);
	      $filesDel = unlink <$path/prozone*>;
	      ToolCommon::Log($fh, "  $filesDel prozone files deleted from $path\n") if ($debug);
	    }
	  }
	} else {
	  my $t = "Failed to open $dir/neweqB for writing: $!";
	  FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	  $ok = 0;
	}
      } else {
	my $t = "Failed to open $dir/eqB: $!";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      }
    } else {
      my $t = "Failed: no equipment table file file created";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    }
  }
  return $ok;
}
#-- End 1.4.2

=head2 PrepareVlanStaTbl($$$$$)

=begin html
Prepare the VLAN static stable for migration.  In certain conditions incorrect init values are used for some fields.
We will set these fields to 0. Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>fromNum</td><td>the NE release coming from</td></tr>
  <tr><td>path</td><td>directory holding the NT OFLMT configuration file</td></tr>
</table>

=end html

=cut

sub PrepareVlanStaTbl($$$$) {
  my($fh, $dir, $fromNum, $path) = @_;
  my $ok = 1;
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
  my $reload = 0;
  my $ret;
  my $cmd;

  $cmd = "cd $dir;($config->{SLEEP} 5;$config->{ECHO} dm table2dml vlanT1.txt /x vlanStaTbl;$config->{SLEEP} 5;$config->{ECHO} exit) | " .
    "$path/$toolData->{NTDBTOOL} -in $dbName -dbg >MIGRvlan1.txt 2>&1";
  ToolCommon::Log($fh, "  CMD = $cmd\n") if $params->{DEBUG};
  $ret = system($cmd);
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t, check log file in '$dir'...\n");
    $ok = 0;
  } else {
    if (-e "$dir/vlanT1.txt") {
      my ($fd1, $fd2);
      if (open($fd1, "$dir/vlanT1.txt")) {
	if (open($fd2, ">$dir/vlanT2.txt")) {
	  my $line;
	  while ($line = <$fd1>) {
	    ToolCommon::Log($fh, "Line read: '$line'\n") if $params->{DEBUG};
	    if (($line =~ /(.+FieldName=vSysMod, FieldType=.+, FieldLength=\d+, FieldInit=0x)([0-9a-fA-F]+)(.+)/) ||
		($line =~ /(.+FieldName=vStaPri, FieldType=.+, FieldLength=\d+, FieldInit=0x)([0-9a-fA-F]+)(.+)/) ||
		($line =~ /(.+FieldName=vVerNbr, FieldType=.+, FieldLength=\d+, FieldInit=0x)([0-9a-fA-F]+)(.+)/)) {
	      my $v1 = $1; my $v2 = $2; my $v3 = $3;
	      if (hex($2) != 0)  {
		$reload = 1;
		$line = $1 . "0" . $3;
		ToolCommon::Log($fh, "Line to be written: '$line'\n") if $params->{DEBUG};
	      }
	    }
	    print $fd2 "$line";
	  }
	  close($fd1), close($fd2);
	  if ($reload) {
	    $cmd = "cd $dir;($config->{SLEEP} 5;$config->{ECHO} dm loaddml vlanT2.txt;$config->{SLEEP} 5;$config->{ECHO} exit) | " .
	      "$path/$toolData->{NTDBTOOL} -in $dbName -out t.tar -dbg >MIGRvlan2.txt 2>&1";
	    $ret = system($cmd);
	    ToolCommon::Log($fh, "  CMD = $cmd\n") if ($params->{DEBUG});
	    if ($ret) {
	      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	      my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t, check log file in '$dir'...\n");
	      $ok = 0;
	    } else {
	      if (copy("$dir/$dbName", "$dir/$dbName" . ".sav")) {
		if (!rename("$dir/t.tar", "$dir/$dbName")) {
		  $ok = 0;
		  my $t = "Renaming temp tar file to $dir/$dbName failed: $!";
		  FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
		} else {
		  ToolCommon::Log($fh, "Modified and renamed NT DB\n") if $params->{DEBUG};
		  #		unlink "$dir/vlanT1.txt", unlink "$dir/vlanT2.txt";
		  my $filesDel = unlink <$dir/prozone*>;
		  ToolCommon::Log($fh, "  $filesDel prozone files deleted from $dir\n") if ($params->{DEBUG});
		  $filesDel = unlink <$path/prozone*>;
		  ToolCommon::Log($fh, "  $filesDel prozone files deleted from $path\n") if ($params->{DEBUG});
		}
	      } else {
		$ok = 0;
		my $t = "Copying original DB tar file to $dir/$dbName.sav failed: $!";
		FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      }
	    }
	  }
	} else {
	  my $t = "Failed to open $dir/vlanT2.txt for writing: $!";
	  FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	  $ok = 0;
	}
      } else {
	my $t = "Failed to open $dir/vlanT1.txt: $!";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      }
    } else {
      my $t = "Failed: no vlanStatTbl table file created";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    }
  }
  return $ok;
}

=head2 MigrateSHUBDbase($$$$$$$;$)

=begin html
Build the SHUB OFLMT parameter file and migrate the SHUB database.  Some details:
<ul>
  <li>Only Extract iss.conf</li>
  <li>R2.2: lanx db file is dmlanx.tar</li>
  <li>R2.3 and later: lanx db file is in dm_complete.tar under Lanx</li>
</ul>
Returns 0 in case of error.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>ntType</td><td>NT type</td></tr>
  <tr><td>path</td><td>directory holding the SHUB OFLMT configuration file</td></tr>
</table>

=end html

=cut

sub MigrateSHUBDbase($$$$$$;$) {
  my ($fh, $dir, $from, $to, $ntType, $path, $u2uhairpin) = @_;
  my $ret;
  my $fromRel = Capability::ShubDbVersion($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION} );
  #my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION});
  my $toRel = Capability::ShubDbVersion($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  #my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $cmd;
  my $file;
  my $ok = 1;
  my $shubNtType = Capability::ShubBoardType($ntType);
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;

  if (defined($fromRel) && defined($toRel)) {
    ToolCommon::Log($fh, "Migrating SHUB database from $fromRel to $toRel...");
    my $swCmpRes = Capability::ReleaseCompare($from, Capability::ConvertRelease(2, 2), REGEXP, 0);
    if ($swCmpRes == 0) {
      # Case R2.2
      $cmd = "cd $dir;$config->{TAR} xvf $SHUBINFILENAMER2 $SHUBINFILENAMER1 >MIGRtar1.txt 2>&1";
      $file = $SHUBINFILENAMER1;
    } elsif ($swCmpRes > 0) {
      # Case R2.3 or higher
      $cmd = "cd $dir;$config->{TAR} xvf $dbName Lanx/$SHUBINFILENAMER1 >MIGRtar2.txt 2>&1";
      $file = "Lanx/$SHUBINFILENAMER1";
    } else {               # -- Added 1.1.7
      # Case R2.0 or R2.1
      $cmd = "";
      $file = $SHUBINFILENAMER1;
    }
    if ($cmd ne "") { # -- 1.1.7 START
      ToolCommon::Log($fh, "CMD = $cmd\n") if($params->{DEBUG});
      $ret = system($cmd);
      if ($ret) {
	my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
	FamilyCommon::SetAndLogError( $fh, $t, "  $t\n");
	$ok = 0;
      }
    }                 # -- 1.1.7 END
    # -- mod 1.1.3
    if (Capability::ReleaseCompare($from, Capability::ConvertRelease(2, 2), REGEXP, 0) < 0) {
      # Coming from R2.0.  Recalc checksum in order to avoid a corrupted database.
      $cmd = "cd $dir;$config->{CP} $file $file" . ".sav;$path/$toolData->{LANXDBTOOLDIR}/$toolData->{LANXIPCKSUM} $file 0 >shubcheck1.txt 2>&1";
      ToolCommon::Log($fh, "CMD - '$cmd'\n") if ($params->{DEBUG});
      $ret = system($cmd);
    } else {
      # Verify whether SHUB DB checksum is OK.
      $cmd = "cd $dir;$path/$toolData->{LANXDBTOOLDIR}/$toolData->{LANXIPCKSUM} $file 1 >shubcheck2.txt 2>&1";
      ToolCommon::Log($fh, "CMD - '$cmd'\n") if ($params->{DEBUG});
      if (system($cmd)) {
	my $t = "SHUB database has incorrect checksum";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      }
    }
    if ($ok) {
      if (($fromRel ne $toRel) || # -- 1.2.4
	  ($params->{TOECNTC}) ||
	  ($params->{TONANTB})) { # -- 1.4.2
	if ($params->{SHUBOFLMTVERS} >= 127) {
	  if (ToolCommon::BuildShubConfigFile($fh, $dir, $fromRel, $toRel, $path, $file, $ntType,
					      $SHUBMIGPARS, $toolData->{SHUBDBCFG}, $SHUBOUTFILENAME, $u2uhairpin)) {
	    $cmd = "cd $dir;$config->{PERL} $path/$toolData->{LANXDBTOOLDIR}/$toolData->{LANXDBTOOL} -f $SHUBMIGPARS >$SHUBMIGFILE 2>&1";
	  } else {
	    $cmd = "";
	    $ok = 0;
	    my $t = ToolCommon::GetError();
	    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	  }
	} elsif ($params->{SHUBOFLMTVERS} >= 121) {
	  # Extra parameter added from V1.21 onwards
	  if ($params->{SHUBOFLMTVERS} > 125) {
	    if ($params->{TOECNTC}) {
	      $cmd = "cd $dir;$config->{PERL} $path/$toolData->{LANXDBTOOLDIR}/$toolData->{LANXDBTOOL} -in $file -out ".
		"$SHUBOUTFILENAME -from $fromRel -to $toRel -cfg $path/$toolData->{LANXDBTOOLDIR}/migration.dat -trc -log " .
		  "lanx_migration.log -bw $params->{BWREDUCE} -FromMode ECNTA -ToMode ECNTC >$SHUBMIGFILE 2>&1";
	    } else {
	      $cmd = "cd $dir;$config->{PERL} $path/$toolData->{LANXDBTOOLDIR}/$toolData->{LANXDBTOOL} -in $file -out ".
		"$SHUBOUTFILENAME -from $fromRel -to $toRel -cfg $path/$toolData->{LANXDBTOOLDIR}/migration.dat -trc -log " .
		  "lanx_migration.log -bw $params->{BWREDUCE} -FromMode $shubNtType -ToMode $shubNtType >$SHUBMIGFILE 2>&1";
	    }
	  } else {
	    $cmd = "cd $dir;$config->{PERL} $path/$toolData->{LANXDBTOOLDIR}/$toolData->{LANXDBTOOL} -in $file -out " .
	      "$SHUBOUTFILENAME -from $fromRel -to $toRel -cfg $path/$toolData->{LANXDBTOOLDIR}/migration.dat -trc -log " .
		"lanx_migration.log -bw $params->{BWREDUCE} >$SHUBMIGFILE 2>&1";
	  }
	} else {
	  $cmd = "cd $path/$toolData->{LANXDBTOOLDIR};$config->{PERL} $toolData->{LANXDBTOOL} -in $dir/$file -out $dir/$SHUBOUTFILENAME " .
	    "-from $fromRel -to $toRel -cfg migration.dat -trc -log $dir/lanx_migration.log >$SHUBMIGFILE 2>&1";
	}
	ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
	if ($cmd ne "") {
	  $ret = system($cmd);
	  if ($ret) {
	    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	    my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
	    FamilyCommon::SetAndLogError($fh, $t, "   $t, (check logfiles in working directory $dir)\n");
	    $ok = 0;
	  } else {
	    ToolCommon::Log($fh, "  OK\n");
	  }
	}
      } else { # -- 1.2.4
	if (!copy("$dir/$file", "$dir/$SHUBOUTFILENAME")) {
	  my $t = "Copy of $dir/$file to $dir/$SHUBOUTFILENAME failed: $!";
	  FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	  $ok = 0;
	} else {
	  ToolCommon::Log($fh, "  not needed, DB identical\n");
	}
      }
    }
  } else {
    FamilyCommon::SetAndLogError($fh, "Migrating SHUB database from $from to $to not possible: DB version not defined");
    $ok = 0;
  }
  return $ok;
}

=head2 GetIvpsBackup($$$$$)

=begin html
Retrieve the xVPS DB using SNMP.  Returns 0 in case of error.  Only required for R3.3 and R3.4.
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>ivpsIp</td><td>The xVPS IP address</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>shortPath</td><td>file transfer short directory name (excluding the tftp top directory path)</td></tr>
  <tr><td>fileName</td><td>DB backup filename</td></tr>
</table>

=end html

=cut

sub GetIvpsBackup($$$$) {
  my ($fh, $ivpsIp, $shortPath, $xvpsNum) = @_;
  my $ok = 1;
  my @oids;
  my $goon = 1;

  #-- open snmp connection
  ToolCommon::Log($fh, sprintf("Retrieve xVPS%d backup of %s...", $xvpsNum, $ivpsIp));
  my ($session, $snmpError) = Net::SNMP->session(-hostname => $ivpsIp,
						 -version => "snmpv1",
						 -timeout => 5,
						 -retries => 3,
						 -community => "public");   # XVPS only supports SNMPv1 and public up till R3.4
  if (!defined($session)) {
    FamilyCommon::SetAndLogError($fh, "  ERROR: Could not open session for $ivpsIp: $snmpError");
    $ok = 0;
  } else {
    #-- disable OID translation
    $session->translate(0);
    push(@oids, (ASAMDBMUPLOADSERVERID, IPADDRESS, $params->{PRIMTFTPIP}));
    push(@oids, (ASAMDBMUPLOADPATH, OCTET_STRING, "$shortPath/$NTINFILENAME"));
    ToolCommon::Log($fh, "  Set TFTP IP and path to '$params->{PRIMTFTPIP}' and '$shortPath/$NTINFILENAME'\n") if $params->{DEBUG};
    my $reply = $session->set_request(@oids);
    if (!defined($reply)) {
      FamilyCommon::SetAndLogError($fh, sprintf("Configure DB upload setting failed: %s", $session->error()));
      $ok = 0;
    } else {
      @oids = ();
      push(@oids, (ASAMDBMUPLOADDBSELECTION, INTEGER, UPLOADACTACTDB));
      ToolCommon::Log($fh, "  Start upload... ") if $params->{DEBUG};
      my $time1 = time();
      $reply = $session->set_request(@oids);
      if (!defined($reply)) {
	FamilyCommon::SetAndLogError($fh, sprintf("Configure DB selection setting failed: %s", $session->error()));
	$ok = 0;
      } else {
	@oids = ();
	$oids[0] = ASAMDBMUPLOADPROGRESS;
	$oids[1] = ASAMDBMUPLOADERROR;
	my $version = $session->version();
	while ($goon) {
	  if ($version < 2) {
	    # SNMPv1/SNMPv2
	    $reply = $session->get_request(-varbindlist => \@oids);
	  } else {
	    # SNMPv3
	    $reply = $session->get_request(-varbindlist => \@oids,
					   -contextname => NT_CONTEXT);
	  }
	  if (!defined($reply->{ASAMDBMUPLOADPROGRESS()})) {
	    FamilyCommon::SetAndLogError($fh, sprintf("Unable to read DB upload progress: %s", $session->error()));
	    $goon = 0, $ok = 0;
	  } else {
	    if ($reply->{ASAMDBMUPLOADPROGRESS()} != DBUPLOADONGOING) {
	      $goon = 0;
	      if ($reply->{ASAMDBMUPLOADPROGRESS()} != DBUPLOADOK) {
		if (defined($reply->{ASAMDBMUPLOADERROR()})) {
		  FamilyCommon::SetAndLogError($fh, sprintf("DB upload finished with error: %s",
							    DBUPLOADERROR($reply->{ASAMDBMUPLOADERROR()})));
		} else {
		  FamilyCommon::SetAndLogError($fh, "DB upload finished with undefined upload error");
		}
		$ok = 0;
	      } else {
		my $uploadSize = -s "$toolData->{TFTPDIR}/$shortPath/$NTINFILENAME";
		ToolCommon::Log($fh, sprintf("  OK: uploaded %d bytes in %d seconds\n", $uploadSize, time() - $time1));
	      }
	    } else {
	      ToolCommon::Log($fh, ".") if $params->{DEBUG};
	      sleep 5;
	    }
	  }
	}
      }
    }
    $session->close();
  }
  return $ok;
}

=head2 LocateCDE()

=begin html
Locate the CDE tar file in $path conforming to $regExp and when found, untar the file.  Undef is returned in case no file was found or when 
untarring the file failed.
<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>Reference to an opened file for progress and debug message tracing</td></tr>
  <tr><td>path</td><td>path to the CDE tar file</td></tr>
  <tr><td>params</td><td>The command parameters provided to the script</td></tr>
  <tr><td>regExp</td><td>the regular expression to which the CDE file should be conforming to</td></tr>
  <tr><td>dir</td><td>Name of NE output directory (relative to the tftp root)</td></tr>
</table>

=end html

=cut

sub LocateCDE($$$$) {
  my ($fh, $path, $regExp, $dir) = @_;
  my @files = ();

  ToolCommon::Log($fh, "Looking for CDE file in '$path'\n") if $params->{DEBUG};
  opendir(DIR, "$path");
  @files = readdir(DIR);
  closedir(DIR);
  foreach my $file (@files) {
    $file = basename($file);
    ToolCommon::Log($fh, "Found file '$file' in '$path'\n") if $params->{DEBUG};
    return $file if ($file =~ /$regExp/);
  }
  return undef;
}

=head2 MigrateXvpsContainer($$$$$$$)

=begin html
Migrate the xVPS database contained in the ISAM-V backup.  For migration to R3.6V the CDE file will be untarred in a temporary location.  The CDE
file to be used depends on the 'from' release:<br>
<ul>
  <li>&lt; R3.5: the CDE file as stored on the ftp server</li>
  <li>&gt; R3.5: the CDE file as contained in the XVPS container</li>
</ul>
<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>Reference to an opened file for progress and debug message tracing</td></tr>
  <tr><td>params</td><td>The command parameters provided to the script</td></tr>
  <tr><td>dir</td><td>Name of NE output directory</td></tr>
  <tr><td>ivpsId</td><td>The IVPS container being migrated</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>path</td><td>PBMT installation directory name</td></tr>
</table>

=end html

=cut

sub MigrateXvpsContainer($$$$$$) {
  my ($fh, $dir, $ivpsId, $from, $to, $path) = @_;
  my $ok = 1;
  my $thisPath = getcwd();
  my $fromRel = Capability::XvpsDbVersion($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION} );
  my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION});
  my $toRel = Capability::XvpsDbVersion($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $dirName = sprintf("$IVPSCONTAINER", $ivpsId);
  my $cmd;
  my $cdeTarFileName;
  my $cfgFile = "$path/$toolData->{IVPSDBCFG}";
  my $errLog = "";
  my $errFh;
  my $logDir = FamilyCommon::GetNeStorage();

  ToolCommon::Log($fh, sprintf("Migrating xVPS%d database from %s (%s) to %s (%s)...", $ivpsId, $fromRelInfo, $fromRel, $toRelInfo, $toRel));
  if (Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
    $cdeTarFileName = "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{ACTCDEFILE}";
  } else {
    if (Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 5), REGEXP, 0) == 0) {
      my $cdeFileName = LocateCDE($fh, "$dir/$dirName/$CDECONTNAME/$CDEACTDIR", $config->{CDE_REGEXP}, $dir);
      if (! defined($cdeFileName)) {
	my $tMsg = "No CDE file was found in the $dir/$dirName/$CDECONTNAME/$CDEACTDIR directory";
	FamilyCommon::SetAndLogError($fh, $tMsg, " ERROR : $tMsg (check logfiles in working directory '$dir')\n");
	$ok = 0;
      } else {
	$cdeTarFileName = "$dir/$dirName/$CDECONTNAME/$CDEACTDIR/$cdeFileName";
      }
    } else {
      my $cdeFileName = LocateCDE($fh, "$dir/$CDECONTNAME", $config->{CDE_REGEXP}, $dir);
      if (! defined($cdeFileName)) {
	my $tMsg = "No CDE file was found in the $dir/$CDECONTNAME directory";
	FamilyCommon::SetAndLogError($fh, $tMsg, " ERROR : $tMsg (check logfiles in working directory '$dir')\n");
	$ok = 0;
      } else {
	$cdeTarFileName = "$dir/$CDECONTNAME/$cdeFileName";
      }
    }
  }
  if ($ok) {
    if (! -e "$dir/$CDETMPDIR") {
      $cmd = "cd $dir;mkdir -m 1775 $CDETMPDIR;$config->{CP} $cdeTarFileName $dir/$CDETMPDIR >$logDir/MIGRcpCDE1.txt 2>&1";
    } else {
      $cmd = "cd $dir/$CDETMPDIR;$config->{RM} -rf * > /dev/null 2>&1;$config->{CP} $cdeTarFileName $dir/$CDETMPDIR >$logDir/MIGRcpCDE2.txt 2>&1";
    }
    ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
    my $ret = system($cmd);
    ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
    if($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $tMsg = "$cmd failed: $!";
      FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir')\n");
      $ok = 0;
    } else {
      if ($ok) {
      # Cleanup of CDE source directory done here
        #if (-e "$dir/$CDECONTNAME") {
        #  $cmd = "cd $dir/$CDECONTNAME;$config->{RM} -rf * > $dir/MIGRxvpsCleanCDE.txt 2>&1";
        #  ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
        #  my $ret = system($cmd);
        #  ToolCommon::Log($fh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
        #  if($ret) {
        #       my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
        #       my $tMsg = "$cmd failed: $!";
        #       FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg \n");
        #       $ok = 0;
        #  }
        #}

	if (Capability::ReleaseCompare($to, Capability::ConvertRelease(3, 5), REGEXP, 0) > 0) {
	  $cfgFile = sprintf("$dir/$IVPSCONTAINER/$toolData->{XVPSDBCFGMOD}", $ivpsId);
	  # Prepare dbtool.cfg for migration to R3.6V
	  if (!ModifyXvpsDbtoolCfg($fh, "$path/$toolData->{IVPSDBCFG}", $cfgFile, "$dir/$CDETMPDIR")) {
	    # SetAndLogError already done.
	    $ok = 0;
	  }
	}
	my $migTxtFile = sprintf($XVPSMIGFILE, $ivpsId);
	# Generate trace and logfiles in the XVPS containers for possible later retrieval.
	if (Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 5), REGEXP, 0) >= 0) {
	  $cmd = "cd $dir/$dirName/$IVPSDBASE >$logDir/MIGRcdXVPS.txt 2>&1;$path/$toolData->{IVPSDBTOOL} -in $NTINFILENAME -out $NTOUTFILENAME " .
            "-cfg ../$toolData->{XVPSDBCFGMOD} -from $fromRel -to $toRel -trc >$logDir/$migTxtFile.txt 2>&1";
	  $errLog = "$dir/$dirName/$IVPSDBASE/trc.log";
	} else {
	  $cmd = "cd $dir/$dirName >$logDir/MIGRcdXVPS.txt 2>&1;$path/$toolData->{IVPSDBTOOL} -in $NTINFILENAME -out $NTOUTFILENAME " .
            "-cfg ./$toolData->{XVPSDBCFGMOD} -from $fromRel -to $toRel -trc >$logDir/$migTxtFile.txt 2>&1";
	  $errLog = "$dir/$dirName/trc.log";
	}
	ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
	my $ret = system($cmd);
	if($ret) {
	  my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	  my $tMsg = "";
	  if ($exitCode == 3) {
	    if (open($errFh, "$errLog")) {
	      while (my $line = <$errFh>) {
		chomp $line;
		if ($line =~ /NON_RECOV/) {
		  $line = <$errFh>, $line = <$errFh>, $line = <$errFh>, $line = <$errFh>, chomp $line;
		  $line =~ m/error info\s*:(.*)/;
		  $tMsg = "Unrecoverable error occured: $1";
		}
	      }
	      close($errFh);
	    } else {
	      $tMsg = "Opening $errLog failed: $!";
	    }
	  } elsif ($exitCode == 2) {
	    $tMsg = "Consistency checks failed";
	  } else {
	    $tMsg = "Migration failed";
	  }
	  FamilyCommon::SetAndLogError($fh, $tMsg, "  : $tMsg (check logfiles in working directory '$dir/$dirName/$IVPSDBASE')\n");
	  $ok = 0;
	} else {
	  ToolCommon::Log( $fh, "  OK\n");
	}
      }
    }
  }
  return $ok;
}

=head2 PrepareXvpsContainers($$$$$)

=begin html
Extract the xVPS backups from the NT backup and create individual dm_complete.tar files from it.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>Opened file used for progress and debug message tracing</td></tr>
  <tr><td>params</td><td>The command parameters provided to the script</td></tr>
  <tr><td>longPath</td><td>The unix (complete) path to extract the NT backup and create the individual xVPS backup files</td></tr>
  <tr><td>xvpsConfig</td><td>A hash containing a list of xVPS boards in the system.</td></tr>
</table>
After extraction the following directory structure  will be present (assuming only XVPS1 and 8 are present):<br><tt>
  --+--XVPS1--+--Dbase/config<br>
    |         |<br>
    |         +--Dbase/dm/*<br>
    |         |<br>
    |         +--Dbase/dmprot/*<br>
    |         |<br>
    |         +--CDE/active/CDEtarfile<br>
    :<br>
    :<br>
    |<br>
    +--XVPS8--+--Dbase/config<br>
              |<br>
              +--Dbase/dm/*<br>
              |<br>
              +--Dbase/dmprot/*<br>
              |<br>
              +--CDE/active/CDEtarfile<br>
</tt>

=end html

=cut

my $EXCLUDE = "excl.txt";   # Exclude file for tar.  Only untar the XVPS containers

sub PrepareXvpsContainers($$$$) {
  my ($fh, $longPath, $xvpsConfig, $tmpDir) = @_;
  my $ok = 1;
  my $cmd;
  my $ret;
  my $xvpsNum;
  my $configMM = "";
  my $excl;
  my $logDir = FamilyCommon::GetNeStorage();

  ToolCommon::Log($fh, "Preparing XVPS containers...\n") if $params->{DEBUG};
  for ($xvpsNum = $LOWEST_XVPSNUM; $xvpsNum <= $HIGHEST_XVPSNUM; $xvpsNum++) {
    # Remove the XVPS directories if existing
    if (-e "$longPath/XVPS$xvpsNum") {
      # DB container found, probably from previous migration.  Remove it (recursively) before going on
      $cmd = "$config->{RM} -Rf $longPath/XVPS$xvpsNum >$logDir/MIGRpreCnt.txt 2>&1";
      ToolCommon::Log($fh, "cmd = $cmd\n") if $params->{DEBUG};
      $ret = system($cmd);
      if ($ret) {
	$ok = 0;
	my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      }
    }
  }
  if ($ok) {
    if (open($excl, ">$longPath/$EXCLUDE")) {
      # Use tar with an exclude file to exclude dm dmprot Lanx ODM and config from the untar operation
  if ($Config{archname} =~ /linux/) {
      print $excl "dm/\n" . "dmprot/\n" . "Lanx/\n" . "ODM/\n";
  } else {
      print $excl "config\n" . "dm/\n" . "dmprot/\n" . "Lanx/\n" . "ODM/\n";
  }
      close $excl;
      $cmd = "cd $longPath;$config->{TAR} xfX $NTINFILENAME $EXCLUDE >$logDir/MIGRcrExcl.txt 2>&1";
      ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
      $ret = system($cmd);
      if ($ret) {
	my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      } else {
        if(! defined($params->{OFLDB})) {
		for ($xvpsNum = $LOWEST_XVPSNUM; $xvpsNum <= $HIGHEST_XVPSNUM; $xvpsNum++) {
		  if ((-e "$longPath/XVPS$xvpsNum/$IVPSDBASE") && ($xvpsConfig->{$xvpsNum} eq $toolData->{NO_IP})) {
		    # DB container found in DB but not present in XVPS config (no XVPS community string found
		    $configMM .= "DB container found for XVPS$xvpsNum while no community string has been defined\n";
		    $ok = 0;
		  }
		}
        }
	if ($ok) {
	  # Create a dm_complete.tar under XVPSn/Dbase.  This file will be used to do the actual XVPS migration.
	  for ($xvpsNum = $LOWEST_XVPSNUM; $xvpsNum <= $HIGHEST_XVPSNUM; $xvpsNum++) {
	    if (-e "$longPath/XVPS$xvpsNum") {
	      if (-e "$longPath/XVPS$xvpsNum/$IVPSDBASE/dmprot") {
                 if ($Config{archname} =~ /linux/) {
	           $cmd = "cd $longPath/XVPS$xvpsNum/$IVPSDBASE;$config->{TAR} --format=gnu -cf $NTINFILENAME config dm dmprot >$logDir/MIGRtarcf.txt 2>&1";
                 } else {
	           $cmd = "cd $longPath/XVPS$xvpsNum/$IVPSDBASE;$config->{TAR} cf $NTINFILENAME config dm dmprot >$logDir/MIGRtarcf.txt 2>&1";
                 }
              } else {
                 if ($Config{archname} =~ /linux/) {
	           $cmd = "cd $longPath/XVPS$xvpsNum/$IVPSDBASE;$config->{TAR} --format=gnu -cf $NTINFILENAME config dm >$logDir/MIGRtarcf.txt 2>&1";
                 } else {
	           $cmd = "cd $longPath/XVPS$xvpsNum/$IVPSDBASE;$config->{TAR} cf $NTINFILENAME config dm >$logDir/MIGRtarcf.txt 2>&1";
                 }
              }
	      ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
	      $ret = system($cmd);
	      if ($ret) {
		my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
		my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
		FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
		$ok = 0;
	      }
	    }
	  }
	} else {
	  FamilyCommon::SetAndLogError($fh, $configMM, $configMM);
	}
      }
    } else {
      my $t = "Opening exclude file '$longPath/$EXCLUDE' failed: $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    }
  }
  return $ok;
}

=head2 MigrateXvpsDbase($$$$$$$$$$$$)

=begin html
Migration of xVPS database:<br>
Case act release &lt; R3.5<br>
-----------------------<br>
1. Loop on all xVPSs<br>
1.1 Create associated xVPS container<br>
1.2 upload the complete xVPS database using SNMP in case xPVS has valid IP address<br>
Case active release &gt;= R3.5<br>
---------------------------<br>
1. Untar NT DB to extract the XPVS containers<br>
<br>
2. Loop on all xVPSs with valid IP address<br>
2.1 migrate the database<br>
The following parameters are used:
<table>
  <tr><td>parentFh</td><td>the file handle of the parent script.  When fatal errors are occuring logging to the main log file will also be performed</td></tr>
  <tr><td>childFh</td><td>the file handle of the parent script.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>xvpsConfig</td><td>A hash containing a list of xVPS boards in the system.</td></tr>
  <tr><td>dir</td><td>NE output directory relative to $TFTPDIR</td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
  <tr><td>userName</td><td>Unix user name in case of multi-user operation (command line option) empty otherwise</td></tr>
  <tr><td>logFile</td><td>the name of the log file associated to this child process</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>path</td><td>PBMT installation directory name</td></tr>
</table>

=end html

=cut

sub MigrateXvpsDbase($$$$$$$$$$$) {
  my ($parentFh, $childFh, $isam, $from, $to, $xvpsConfig, $dir, $count, $userName, $logFile, $path) = @_;
  my $ok = 1;
  my $version = undef;
  my $line;
  my $fromRel = Capability::XvpsDbVersion($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION} );
  my $fromRelInfo = Capability::NtDbVersionInfo($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION});
  my $toRel = Capability::XvpsDbVersion($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $toRelInfo = Capability::NtDbVersionInfo($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
  my $migResult = "";
  my $xvpsDbPath = "";
  my $logDir = FamilyCommon::GetNeStorage();

  $dir =~ s/\/$//;
  ToolCommon::Log($childFh, "Starting XVPS migration...\n") if $params->{DEBUG};
  if (defined($toRel) && defined($fromRel)) {
    if ($from ne $to) {
#      ToolCommon::Log($childFh, "Migrating xVPS database from $fromRelInfo ($fromRel) to $toRelInfo ($toRel)...");
      if ((Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) &&
	  (Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 3), REGEXP, 0) >= 0)) {
	foreach my $ivpsId (sort keys %{$xvpsConfig}) {
	  if ($xvpsConfig->{$ivpsId} ne $toolData->{NO_IP}) {
	    my $fullContainer = sprintf("$toolData->{TFTPDIR}/$dir/$IVPSCONTAINER", $ivpsId);  # Only TFTP for xVPS
	    my $shortContainer = sprintf("$dir/$IVPSCONTAINER", $ivpsId);
	    $xvpsDbPath = "$toolData->{TFTPDIR}/$dir";  # R3.3 or R3.4: can only be tftp DB upload from xVPS.
	    # Empty directory (if existing)
	    if (-d $fullContainer) {
	      system("cd $fullContainer;$config->{RM} -Rf * >$logDir/MIGRxvpsRm.txt 2>&1");
	    } else {
	      # Create backup containers
	      CORE::exit(CREATE_DIR_FAILED_CODE) if (! FamilyCommon::CreateTmpDirFile($parentFh, $childFh, $fullContainer,
										      $userName, $logFile, $count, 1));
	    }
	    if (! -e "$fullContainer/$NTINFILENAME") {
	      if (!FamilyCommon::CreateBUFile($parentFh, $childFh, $fullContainer, $NTINFILENAME, $logFile, $count, $params->{DEBUG})) {
		CORE::exit(CREATE_FILE_FAIL_CODE);
	      }
	    }
	    # Get backup
	    if (! GetIvpsBackup($childFh, $xvpsConfig->{$ivpsId}, $shortContainer, $ivpsId)) {
	      return 0;  # End the processing here to allow debugging
	    }
	  }
	}
      } elsif (Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 5), REGEXP, 0) >= 0) {
	$xvpsDbPath = $toolData->{FXRDATA}{HOMEDIR};  # Can be ftp or sftp DB upload
	# Extract xVPS backup from NT backup and create tar file from it for migration purpose (for migration of R3.5 to R3.6 or higher).
	if (! PrepareXvpsContainers($childFh, defined($params->{OFLDB}) ? FamilyCommon::GetNeStorage() : $toolData->{FXRDATA}{HOMEDIR},
				    $xvpsConfig, $dir)) {
	  $ok = 0;
	}
      } else {
	FamilyCommon::SetAndLogError($childFh, "Unsupported migration");
	$ok = 0;
      }
      ToolCommon::Log($childFh, "xvps DB path: $xvpsDbPath\n") if $params->{DEBUG};
      if ($ok) {
	foreach my $ivpsId (sort keys %{$xvpsConfig}) {
	  if ($xvpsConfig->{$ivpsId} ne $toolData->{NO_IP}) {
	    if (! MigrateXvpsContainer($childFh, defined($params->{OFLDB}) ? FamilyCommon::GetNeStorage() : $xvpsDbPath, $ivpsId,
				       $from, $to, $path)) {
	      $migResult .= "Migration of xVPS$ivpsId failed\n";
	      $ok = 0;
	    }
	  }
	}
	FamilyCommon::SetAndLogError($childFh, $migResult, $migResult) if ! $ok;
	# Remove the untarred CDE package as we no longer need it.
        if (-e "$toolData->{TFTPDIR}/$dir/$CDETMPDIR") {
	    my $cmd = "$config->{RM} -Rf $toolData->{TFTPDIR}/$dir/$CDETMPDIR > /dev/null 2>&1";
	    my $ret = system($cmd);
	    if ($ret) {
	      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	      my $t = "'$cmd' failed (signal $signal, exitcode $exitCode): $!";
	      FamilyCommon::SetAndLogError($childFh, $t, "  $t\n");
	      $ok = 0;
	    }
        }

      # Cleanup of CDE source directory done here
        if (-e "$toolData->{TFTPDIR}/$dir/$CDECONTNAME") {
          my $cmd = "$config->{RM} -rf $toolData->{TFTPDIR}/$dir/$CDECONTNAME/* > $toolData->{TFTPDIR}/$dir/MIGRxvpsCleanCDE.txt 2>&1";
          ToolCommon::Log($childFh, "CMD = $cmd\n") if ($params->{DEBUG});
          my $ret = system($cmd);
          ToolCommon::Log($childFh, " Result of command: $ret, $!\n") if ($params->{DEBUG});
          if($ret) {
               my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
               my $tMsg = "$cmd failed: $!";
               FamilyCommon::SetAndLogError($childFh, $tMsg, "  : $tMsg \n");
               $ok = 0;
          }
        }

      }
    } else {
      ToolCommon::Log($childFh, "  xVPS Migration not needed, same release\n");
    }
  } else {
    FamilyCommon::SetAndLogError($childFh, "Migrating xVPS database from $from to $to not possible: DB version not defined");
    $ok = 0;
  }
  return ($ok);
}

=head2 GetIvpsIsamListByIP($$$)

=begin html
Retrieve list of dependant nodes for a specific IVPS present in this ISAM using SNMP.  The list can empty meaning that no remote POLT boards are
controled by this IVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference
to a hash is returned.  Every hash contains of an node name/IP address couple.  In case the hash element is empty no translation was possible.
<br>
Note that this functio takes into account whether we are performing migration using the offline db option or whether we are working with a real
ISAM-V<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>ivpsIp</td><td>IP address of IVPS</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

my $ACCESSEQUIPMENTASAMID = ".1.3.6.1.4.1.637.61.1.55.2.1.1.3";

sub GetXvpsIsamListByIP($$) {
  my ($fh, $ivpsIp) = @_;
  my $result;
  my %couple = ();
  my ($name, $ipAddr, $transOk);

  if (!defined($params->{OFLDB})) {
    my ($session, $snmpError) = Net::SNMP->session(-hostname => $ivpsIp,
						   -version => "SNMPv1",
						   -timeout => 5,
						   -retries => 3,
						   -community => "public");
    if (!defined($session)) {
      FamilyCommon::SetAndLogError($fh, "  ERROR: Could not open session for $ivpsIp: $snmpError");
      return undef;
    } else {
      #-- disable OID translation
      $session->translate(0);
      $result = $session->get_table($ACCESSEQUIPMENTASAMID);
      if (!defined($result)) {
	my $error = $session->error();
	if (!($error =~ /Requested table is empty/)) {
	  FamilyCommon::SetAndLogError($fh, "SNMP table walk '$ACCESSEQUIPMENTASAMID' failed: $error");
	  return undef;
	}
      } else {
	foreach my $entry (Net::SNMP::oid_lex_sort(keys(%{$result}))) {
	  $name = "", $ipAddr = "", $transOk = 1;
	  if ($result->{$entry} =~ /\d+\.\d+\.\d+\.\d+/) {
	    $name = gethostbyaddr(inet_aton($result->{$entry}), AF_INET);
	    if (!defined($name)) {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) no match\n") if $params->{DEBUG};
	    } else {
	      $couple{$name} = $result->{$entry};
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) yields '$name'\n") if $params->{DEBUG};
	    }
	  } else {
	    my @locAddr = gethostbyname($result->{$entry});
	    my $status = $?;
	    if ($status == 0) {
	      my @ip = unpack( "C4", $locAddr[4]);
	      my $ipAddr = join( '.', @ip );
	      $couple{$result->{$entry}} = $ipAddr;
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) yields '$ipAddr'\n") if $params->{DEBUG};
	    } else {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) no match\n") if $params->{DEBUG};
	    }
	  }
	}
	$session->close();
      }
    }
  }
  return \%couple;
}

=head2 GetIvpsIsamListByCommunity($$$$)

=begin html
Retrieve list of dependant nodes for a specific IVPS present in this ISAM using SNMP.  The list can empty meaning that no remote POLT boards are
controled by this IVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference
to a hash is returned.  Every hash contains of an node name/IP address couple.  In case the hash element is empty no translation was possible.
<br>Used from R3.5 onwards: the xVPS no longer has a public IP address but is addressed using the NT IP address in combination with a community
string.<br>
Note that this functio takes into account whether we are performing migration using the offline db option or whether we are working with a real
ISAM-V<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>ivpsIp</td><td>IP address of NT</td></tr>
  <tr><td>community</td><td>The community string associated to the NT</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetXvpsIsamListByCommunity($$$) {
  my ($fh, $ntIp, $community) = @_;
  my $result;
  my %couple = ();
  my ($name, $ipAddr, $transOk);
  my ($xvpsSession, $snmpError);

  if (! defined($params->{OFLDB})) {
    if (defined($params->{SNMP}{V3}{AUTH}{USERNAME}) and ($params->{SNMP}{V3}{AUTH}{USERNAME} ne "")) {
        if ((defined($params->{SNMP}{V3}{AUTH}{PASSWORD}) && ($params->{SNMP}{V3}{AUTH}{PASSWORD} ne "")) &&
            (defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""))) {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME},
                                                      -authprotocol => $params->{SNMP}{V3}{AUTH}{PROTO},
                                                      -authpassword => $params->{SNMP}{V3}{AUTH}{PASSWORD},
                                                      -privpassword => $params->{SNMP}{V3}{PRIV}{PASSWORD},
                                                      -privprotocol => $params->{SNMP}{V3}{PRIV}{PROTO});
        } elsif ((defined($params->{SNMP}{V3}{AUTH}{PASSWORD}) && ($params->{SNMP}{V3}{AUTH}{PASSWORD} ne "")) &&
                 (defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} eq ""))) {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME},
                                                      -authpassword => $params->{SNMP}{V3}{AUTH}{PASSWORD},
                                                      -authprotocol => $params->{SNMP}{V3}{AUTH}{PROTO});
        } elsif ((defined($params->{SNMP}{V3}{AUTH}{PASSWORD}) && ($params->{SNMP}{V3}{AUTH}{PASSWORD} eq "")) &&
                 (defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""))) {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME},
                                                      -privpassword => $params->{SNMP}{V3}{PRIV}{PASSWORD},
                                                      -privprotocol => $params->{SNMP}{V3}{PRIV}{PROTO});
        } else {
	  ($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
                                                      -version => 3,
                                                      -timeout => 5,
                                                      -retries => 3,
                                                      -username => $params->{SNMP}{V3}{AUTH}{USERNAME});
        }
    } else {
      if (defined($params->{SNMP}{V2}{NT_COMMUNITY})) {
	($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
							-version => 2,
							-timeout => 5,
							-retries => 3,
							-community => $community);
      } else {
	($xvpsSession, $snmpError) = Net::SNMP->session(-hostname => $ntIp,
							-version => 1,
							-timeout => 5,
							-retries => 3,
							-community => $community);
      }
    }
    if (!defined($xvpsSession)) {
      FamilyCommon::SetAndLogError($fh, "  ERROR: Could not open session for $ntIp: $snmpError");
      return undef;
    } else {
      #-- disable OID translation
      ToolCommon::Log($fh, "Opened SNMP session for IP $ntIp and community '$community'\n") if $params->{DEBUG};
      $xvpsSession->translate(0);
      if (defined($params->{SNMP}{V3}{AUTH}{USERNAME}) and ($params->{SNMP}{V3}{AUTH}{USERNAME} ne "")) {
	$result = $xvpsSession->get_table(-baseoid => $ACCESSEQUIPMENTASAMID,
					  -contextname => $community,
					  -maxrepetitions => MAX_SNMPV2_3_REP);
      } else {
	if (defined($params->{SNMP}{V2}{NT_COMMUNITY})) {
	  $result = $xvpsSession->get_table(-baseoid => $ACCESSEQUIPMENTASAMID,
					    -maxrepetitions => MAX_SNMPV2_3_REP);
	} else {
	  $result = $xvpsSession->get_table($ACCESSEQUIPMENTASAMID);
	}
      }
      if (!defined($result)) {
	my $error = $xvpsSession->error();
	if (!($error =~ /Requested table is empty/)) {
	  FamilyCommon::SetAndLogError($fh, "SNMP table walk '$ACCESSEQUIPMENTASAMID' failed: $error");
	  $xvpsSession->close();
	  return undef;
	}
      } else {
	foreach my $entry (Net::SNMP::oid_lex_sort(keys(%{$result}))) {
	  $name = "", $ipAddr = "", $transOk = 1;
	  if ($result->{$entry} =~ /\d+\.\d+\.\d+\.\d+/) {
	    $name = gethostbyaddr(inet_aton($result->{$entry}), AF_INET);
	    if (!defined($name)) {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) no match\n") if $params->{DEBUG};
	    } else {
	      $couple{$name} = $result->{$entry};
	      ToolCommon::Log($fh, "gethostbyaddr($result->{$entry}) yields '$name'\n") if $params->{DEBUG};
	    }
	  } else {
	    my @locAddr = gethostbyname($result->{$entry});
	    my $status = $?;
	    if ($status == 0) {
	      my @ip = unpack( "C4", $locAddr[4]);
	      my $ipAddr = join( '.', @ip );
	      $couple{$result->{$entry}} = $ipAddr;
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) yields '$ipAddr'\n") if $params->{DEBUG};
	    } else {
	      $couple{$result->{$entry}} = "";
	      ToolCommon::Log($fh, "gethostbyname($result->{$entry}) no match\n") if $params->{DEBUG};
	    }
	  }
	}
	$xvpsSession->close();
      }
    }
  }
  return \%couple;
}

=head2 GetXvpsDependancyByIP($$$)

=begin html
Retrieve list of dependant nodes for every XVPS present in this ISAM.  The list can empty meaning that no remote POLT boards are controled by this
XVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference to a hash is returned.
Every entry consists of a hash consisting of node name pointing to a node IP address (or an empty in case the translation failed.
The key is the xVPS number.
<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>isam</td><td>pointer to the XVPS structure</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetXvpsDependancyByIP($$) {
  my ($fh, $xvpsConfig) = @_;
  my %dependancy;

  ToolCommon::Log($fh, "Determine xVPS dependancy list by IP...");
  foreach my $ivpsId (sort keys %{$xvpsConfig}) {
    if ($xvpsConfig->{$ivpsId} ne $toolData->{NO_IP}) {
    ToolCommon::Log($fh, "  $ivpsId: '$xvpsConfig->{$ivpsId}'\n") if $params->{DEBUG};
      my $isamList = GetXvpsIsamListByIP($fh, $xvpsConfig->{$ivpsId});
      if (defined($isamList)) {
        $dependancy{$ivpsId} = $isamList;
      } else {
        return undef;
      }
    }
  }
  ToolCommon::Log($fh, "  OK!\n");
  return \%dependancy;
}

=head2 GetXvpsDependancyByCommunity($$$$)

=begin html
Retrieve list of dependant nodes for every XVPS present in this ISAM.  The list can empty meaning that no remote POLT boards are controled by this
XVPS.  The name(s) retrieved are checked versus the /etc/config file to determine whether these are known names.  A reference to a hash is returned.
Every entry consists of a hash consisting of node name pointing to a node IP address (or an empty in case the translation failed.
The key is the xVPS number.
<br>Used from R3.5 onwards: the xVPS no longer has a public IP address but is addressed using the NT IP address in combination with a community
string.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>isam</td><td>pointer to the XVPS structure containing XVPS community strings</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetXvpsDependancyByCommunity($$$) {
  my ($fh, $communityStrings, $ntIp) = @_;
  my %dependancy;
  my $xvpsNum;

  ToolCommon::Log($fh, "Determine xVPS dependancy list by community...");
  foreach my $context (sort keys %{$communityStrings}) {
    if ($communityStrings->{$context} =~ /xvps(\d)/) {
      $xvpsNum = $1;
      ToolCommon::Log($fh, "  $context ($xvpsNum): '$communityStrings->{$context}'\n") if $params->{DEBUG};
      my $isamList = GetXvpsIsamListByCommunity($fh, $ntIp, $communityStrings->{$context});
      if (defined($isamList)) {
        $dependancy{$xvpsNum} = $isamList;
      } else {
        return undef;
      }
    }
  }
  ToolCommon::Log($fh, "  OK!\n");
  return \%dependancy;
}

=head2 MakeFakeList($$$)

=begin html
In case the AWS functionality is missing to generate an IVPS to IP address mapping a test-help bas been built-in where a pre-defined file can be used.
The name is fixed to IvPsLiSt.txt and has the same format as generated by the AWS:<br>
XVPS<n>:<IP> or 0.0.0.0 when no IVPS is present.  <n>: 1..8<br>
<br>
IMPORTANT NOTE TO PEOPLE READING THIS CODE!!!!!<br>
The fake IvPsLiSt.txt should NEVER be used in fieldconditions.  The AWS is expected to provide the functionality offered by this fake file.<br>
This was added to test the ISAM-V migration when AWS was not yet ready and the required functionality was as such not available.

=end html

=cut

sub MakeFakeList($$) {
  my ($fh, $fileName) = @_;
  my $line;
  my %ivpsConfig = ();
  my $foundIP = 0;
  my $look;

  ToolCommon::Log($fh, "Reading fake IVPS list..");
  if (open($look, $fileName)) {
    while (defined($line = <$look>)) {
      chomp $line;
      ToolCommon::Log($fh, "$line\n") if ($params->{DEBUG});
      if ($line =~ /$ISNOTISAMV_RE/) {
        FamilyCommon::SetAndLogError($fh, "ISAM not known as ISAMV in AWS");
        return undef;
      } elsif ($line =~ /$ISISAMV_RE/) {
        $foundIP = 1 if ($2 ne $toolData->{NO_IP});
        $ivpsConfig{$1} = $2;
        ToolCommon::Log($fh, "Stored IP address $ivpsConfig{$1} for xVPS $1\n") if ($params->{DEBUG});
      } else {
        FamilyCommon::SetAndLogError($fh, "Unexpected line read: '$line'");
        return undef;
      }
    }
    close $look;
    ToolCommon::Log($fh, "  OK\n");
  } else {
    FamilyCommon::SetAndLogError($fh, "Opening '$fileName' failed: $!");
    return undef;
  }
  if (! $foundIP) {
    FamilyCommon::SetAndLogError($fh, "No xVPS IP addresses found in IvPsLiSt.txt");
  }
  return $foundIP ? \%ivpsConfig : undef;
}

=head2 GetIvpsIpAddresses($$$)

=begin html
Retrieve IP addresses associated to an ISAM in case it is an ISAM-V.   The undefined variable is returned in case an error was encountered (not an ISAM-V is considered as an error (this function is not called in case no xVPS boards have been found in the LT configuration) otherise a pointer to a hash is returned in case an IVPS structure is returned by AWS.  The key is the xVPS number, the element its IP address.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>isam</td><td>pointer to the ISAM IP/name structure</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub GetIvpsIpAddresses($$) {
  my ($fh, $isam) = @_;
  my $cmd = "$toolData->{IVPSINVENTORY} $isam->[0] |";
  my $line;
  my %ivpsConfig = ();
  my $foundIP = 0;
  my $look;

  ToolCommon::Log($fh, "CMD = $cmd\n") if ($params->{DEBUG});
  if (open($look, $cmd)) {
    while (defined($line = <$look>)) {
      chomp $line;
      ToolCommon::Log($fh, "$line\n") if ($params->{DEBUG});
      if ($line =~ /$ISNOTISAMV_RE/) {
        FamilyCommon::SetAndLogError($fh, "ISAM not known as ISAMV in AWS");
        return undef;
      } elsif ($line =~ /$ISISAMV_RE/) {
	$foundIP = 1 if ($2 ne $toolData->{NO_IP});
        $ivpsConfig{$1} = $2;
        ToolCommon::Log($fh, "Stored IP address $ivpsConfig{$1} for xVPS $1\n") if ($params->{DEBUG});
      } else {
        FamilyCommon::SetAndLogError($fh, "Unexpected line read: '$line'");
        return undef;
      }
    }
    close $look;
  } else {
    my ($exitCode, $signal) = Utils::ConvertExitCode($?);
    FamilyCommon::SetAndLogError($fh, "$cmd FAILED (signal $signal, exitcode $exitCode): $!");
    return undef;
  }
  if(! $foundIP) {
    FamilyCommon::SetAndLogError($fh, "No xVPS IP addresses found in xVPS inventory output");
  }
  return $foundIP ? \%ivpsConfig : undef;
}

=head2 CopyXvpsList($$$)

=begin html
Create a list of xVPS with a non-0.0.0.0 IP address for the xVPS community strings found in the community string input hash.  Used from R3.5 onwards.

=end html

=cut

sub CopyXvpsList($$) {
  my ($fh, $communityStrings) = @_;
  my $xvpsNum;
  my %xvpsConfig = (1 => $toolData->{NO_IP},
		    2 => $toolData->{NO_IP},
		    3 => $toolData->{NO_IP},
		    4 => $toolData->{NO_IP},
		    5 => $toolData->{NO_IP},
		    6 => $toolData->{NO_IP},
		    7 => $toolData->{NO_IP},
		    8 => $toolData->{NO_IP});

  foreach my $context (sort keys %{$communityStrings}) {
#    if ($communityStrings->{$context} =~ /xvps(\d)/) {
    if (uc($context) =~ /XVPS(\d)/) {
      $xvpsNum = $1;
      $xvpsConfig{$xvpsNum} = "1.1.1.1";  # Make it different from $NO_IP
      ToolCommon::Log($fh, "Defining xVPS $xvpsNum in copy list\n") if $params->{DEBUG};
    }
  }
  return \%xvpsConfig;
}

=head2 CopyNTDbase($$$)

=begin html
Copy the R2.2 (or lower) NT database to the same directory structure as maintained for R2.3 or higher.  Keeps the NT download similar.
Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
</table>

=end html

=cut

sub CopyNTDbase($$) {
  my ($fh, $dir) = @_;
  my $ok = 1;
  my $ret;
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;

  ToolCommon::Log($fh, "Copy migrated NT database...");
  if (! -d "$dir/newdb") {
    if (!mkdir( "$dir/newdb", 0777)) {
      my $t = "Create newdb dir failed: $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    }
  }
  if ($ok) {
    if ($params->{UMASK_CHMOD} and ! chmod(01777, "$dir/newdb")) {
       ToolCommon::Log($fh,  "Failed to chmod 1777 for directory '$dir/newdb'\n");
    }
    my $cmd = "cp $dir/$NTOUTFILENAME $dir/newdb/$dbName >ntdbcopy.txt 2>&1";
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "  OK\n");
    }
  }
  return $ok;
}

=head2 AddCdeFiles($$$$$)

=begin html
Add the CDE files to the new db using some pre-defined variables:
<ul>
  <li>$config->{NEWCDEFILE}: the path and name of the CDE file to be used</li>
  <li>$config->{CDE_REGEXP}: parse the CDE file name syntax to retrieve some parts</li>
</ul>
The CDE tar file is copied to the XVPS CDE directory.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>dirName</td><td>NE output directory pointing the CDE directory in the xVPS container</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
</table>
=end html

=cut

sub AddCdeFiles($$$$) {
  my ($fh, $dirName, $newName, $tmpDir) = @_;
  my $ok = 1;
  my $pwd = getcwd();
  my $logDir = FamilyCommon::GetNeStorage();
  my $cmd = "cd $dirName;$config->{CP} $toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE} $newName >$logDir/MIGRaddCde.txt 2>&1";
  my $ret;

  $ret = system("$cmd");
  ToolCommon::Log($fh, "CMD = $cmd\n") if $params->{DEBUG};
  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "$cmd FAILED (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    $ret = system("cd $pwd;");
  }
  return $ok;
}

=head2 CombineDbases($$$$$$$)

=begin html
Create a combined database from all migrated databases.  Returns 0 in case of error.
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>xvpsConfig</td><td>xVPS configuration.  undef if node is not ISAM-V</td></tr>
</table>
In case of IPVS migration to the following files will be
<ul>
  <li>created in the new backup in case of migration to R3.5</li>
  <li>replaced in the new backup in case of migration from R3.5 to a higher release</li>
</ul>
<ul>
  <li>cdePackName: the name of the new CDE file</li>
  <li>cdeProfileId: the customer ID associated to this CDE file</li>
</ul>

=end html

=cut

sub MakeDir($$$) {
  my ($fh, $dirName, $debug) = @_;
  my $ok = 1;

  if (!mkdir( "$dirName", 0777)) {
    my $t = "Create dir '$dirName' failed: $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    ToolCommon::Log($fh, "Created '$dirName'\n") if $debug;
    if ( $params->{UMASK_CHMOD} and ! chmod(01777, "$dirName")) {
       ToolCommon::Log($fh,  "Failed to chmod 1777 for directory '$dirName'\n") if $debug;
    }
  }
  return $ok;
}

sub CombineDbases($$$$$$) {
  my ($fh, $dir, $from, $to, $xvpsConfig, $fromDir) = @_;
  my $cmd;
  my $ok = 1;
  my $ret;
  my $pwd = getcwd();
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
  my $logDir = FamilyCommon::GetNeStorage();

  my $fromRel = Capability::XvpsDbVersion($toolData->{FROMNEFAMILY}, $params->{SOURCE_DBVERSION} );
  my $toRel = Capability::XvpsDbVersion($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});

  ToolCommon::Log($fh, "Combining databases...");
  if (! -d "$dir/newdb") {
    if (! MakeDir($fh, "$dir/newdb", $params->{DEBUG})) {
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Created '$dir/newdb'\n") if $params->{DEBUG};
    }
  } else {
    $cmd = "$config->{RM} -Rf $dir/newdb/* >$logDir/MIGRcombrm.txt 2>&1";
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "Emptied '$dir/newdb/*'\n") if $params->{DEBUG};
    }
  }
  if ($ok) {
    $cmd = "cd $dir/newdb;$config->{TAR} xf ../$NTOUTFILENAME >$logDir/combineDBtarxf.txt 2>&1;cd $pwd >$logDir/MIGRdbextr.txt 2>&1";
    ToolCommon::Log( $fh, "CMD = $cmd\n") if $params->{DEBUG};
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      #-- Add Lanx directory
      if (! -d "$dir/newdb/Lanx") {
	if (! MakeDir($fh, "$dir/newdb/Lanx", $params->{DEBUG})) {
	  $ok = 0;
	}
      } else {
	# From R2.3 onwards the Lanx subdirectory is present in the migrated DB so it is present... clear it
	my $del1 = unlink <$dir/newdb/Lanx/*>;
	ToolCommon::Log($fh, "Emptied '$dir/newdb/Lanx', deleted $del1 files\n") if $params->{DEBUG};
      }
      if ($ok) {
	#-- Copy Lanx files
	if (!copy("$dir/$SHUBOUTFILENAME", "$dir/newdb/Lanx/$SHUBINFILENAMER1")) {
	  my $t = " Copy of $SHUBINFILENAMER1 failed: $!";
	  FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	  $ok = 0;
	} else {
	  ToolCommon::Log($fh, "Copied from '$dir/$SHUBOUTFILENAME' to '$dir/newdb/Lanx/$SHUBINFILENAMER1'\n") if $params->{DEBUG};
	}
	# If active SW is R4.0 also copy iss.conf to issprot.conf since DB download otherwise fails.
	if (Capability::ReleaseCompare($from, Capability::ConvertRelease(4, 0), REGEXP, 0) == 0) {
	  if (!copy("$dir/$SHUBOUTFILENAME", "$dir/newdb/Lanx/$SHUBPROTINFILENAME")) {
	    my $t = " Copy of $SHUBPROTINFILENAME failed: $!";
	    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	    $ok = 0;
	  } else {
	    ToolCommon::Log($fh, "Copied from '$dir/$SHUBOUTFILENAME' to '$dir/newdb/Lanx/$SHUBPROTINFILENAME'\n") if $params->{DEBUG};
	  }
	}
      }
    }
  }
  if( defined $fromRel and defined $toRel and ( $fromRel ne $toRel ) ) {
    if (($ok) && defined($xvpsConfig)) {
      # Include xVPS containers in new DB
      my @ivpsIds = (sort keys %{$xvpsConfig});
      my $numOfIvps = @ivpsIds;
      my $ivpsId = 0;
      my $realIvpsId = 0;
      while (($ok) && ($ivpsId < $numOfIvps)) {
	# Create xVPS container in output directory
	$realIvpsId = $ivpsId+1;
	if ($xvpsConfig->{$realIvpsId} ne $toolData->{NO_IP}) {
	  my $newDir = sprintf("$dir/newdb/$IVPSCONTAINER", $realIvpsId);
	  my $source = "";
	  if (Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
	    $source = sprintf("$fromDir/$IVPSCONTAINER/$NTOUTFILENAME", $realIvpsId);
	  } else {
	    $source = sprintf("$dir/$IVPSCONTAINER/$IVPSDBASE/$NTOUTFILENAME", $realIvpsId);
	  }
	  if (! -d "$newDir") {
	    # Creates 'XVPS<n>' under '$dir/newdb'
	    if (! MakeDir($fh, "$newDir", $params->{DEBUG})) {
	      $ok = 0;
	    } else {
	      # Creates 'Dbase' under '$dir/newdb/XVPS<n>'
	      if (! MakeDir($fh, "$newDir/$IVPSDBASE", $params->{DEBUG})) {
		$ok = 0;
	      } else {
		if (Capability::ReleaseCompare($to, Capability::ConvertRelease(3, 5), REGEXP, 0) == 0) {
		  # Creates 'CDE' under 'XVPS<n>'
		  if (! MakeDir($fh, "$newDir/$CDECONTNAME", $params->{DEBUG})) {
		    $ok = 0;
		  } else {
		    # Creates 'active' under 'XVPS<n>/CDE'
		    if (! MakeDir($fh, "$newDir/$CDECONTNAME/$CDEACTDIR", $params->{DEBUG})) {
		      $ok = 0;
		    }
		  }
		}
	      }
	    }
	  } else {
	    # From R3.5 onwards:
	    # XVPS container IS present in old DB. We need to remove all files (also remove the CDE directory).
	    # 1. remove all files from DB
	    $cmd = "cd $newDir/$IVPSDBASE;$config->{RM} -Rf * >$logDir/MIGRemptyXVPS.txt 2>&1";
	    ToolCommon::Log($fh, "Executing '$cmd'...\n") if $params->{DEBUG};
	    $ret = system($cmd);
	    if ($ret) {
	      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    }
	    # 2. Migration to R3.6 or higher: remove CDE directory from XVPS container.
	    if (Capability::ReleaseCompare($to, Capability::ConvertRelease(3, 6), REGEXP, 0) >= 0) {
	      $cmd = "cd $newDir;$config->{RM} -Rf $CDECONTNAME >$logDir/MIGRemptyCDE.txt 2>&1";
	      ToolCommon::Log($fh, "Executing '$cmd'...\n") if $params->{DEBUG};
	      $ret = system($cmd);
	      if ($ret) {
		my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
		my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!";
		FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
		$ok = 0;
	      }
	    }
	  }
	  if ($ok) {
	    # No IP address present means no xVPS present
	    $cmd = "cd $newDir/$IVPSDBASE;$config->{TAR} xf $source >$logDir/MIGRxvpsextr.txt 2>&1";
	    ToolCommon::Log($fh, "Executing '$cmd'...\n") if $params->{DEBUG};
	    $ret = system($cmd);
	    if ($ret) {
	      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
	      my $t = "Extract of xVPS dbase '$source' FAILED (signal $signal, exitcode $exitCode): $!";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    } else {
	      if (Capability::ReleaseCompare($to, Capability::ConvertRelease(3, 6), REGEXP, 0) < 0) {
		# Migrate to R3.5: Copy CDE tar file in XVPS container, migrate to R3.6 or higher copy in NT CDE directory
		if (!AddCdeFiles($fh, "$newDir/$CDECONTNAME/$CDEACTDIR", ".", $dir)) {
		  $ok = 0;
		}
	      }
	    }
	  }
	}
	$ivpsId++;
      }
    }
  }
  if ($ok) {
    if (Capability::ReleaseCompare($to, Capability::ConvertRelease(3, 6), REGEXP, 0) >= 0) {
      if (! -d "$dir/newdb/$CDECONTNAME") {
	# Creates 'CDE' under the "root"
	if (! MakeDir($fh, "$dir/newdb/$CDECONTNAME", $params->{DEBUG})) {
	  $ok = 0;
	}
      } else {
	my $del1 = unlink <$dir/newdb/$CDECONTNAME/*>;
	ToolCommon::Log($fh, "Emptied '$dir/newdb/$CDECONTNAME', deleted $del1 files\n") if $params->{DEBUG};
      }
      if ($ok) {
	# Migrate to R3.6 or higher copy in NT CDE directory if the CDE file can be found.
	if ((-f "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}") && ($toolData->{NEWCDEFILE} ne "")) {
	  if (!AddCdeFiles($fh, "$dir/newdb/$CDECONTNAME", ".", $dir)) {
	    $ok = 0;
	  }
	} else {
	  # We will continue but will however write a warning in case the ISAM has voice boards or we will fail the migration in case
	  # the isam has voice servers we distinguish 2 cases:
	  # - migration from R3.5: in this case we generate a warning if no CDE file is found
	  # - migration from R3.6: in this case we fail the migration
	  if ($params->{IS_ISAMV_SRV}) {
	    my $t = "ISAM is voice server but no CDE file is found";
	    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	    $ok = 0;
	  } elsif ($params->{ISAM_HAS_VOICE}) {
	    if (Capability::ReleaseCompare($from, Capability::ConvertRelease(3, 6), REGEXP, 0) >= 0) {
	      my $t = "ISAM has voice boards but no CDE file is found";
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      $ok = 0;
	    } else {
	      ToolCommon::Log($fh, "  ISAM has voice boards but no CDE file will be included in the migrated DB!!");
	    }
	  } else {
	    # Problem, leave the directory empty
	  }
	}
      }
    }
  }
  if ($ok) {
    my $readDir;
    if (!defined(opendir($readDir, "$dir/newdb"))) {
      my $t = "Can't opendir $dir/newdb: $!";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      CORE::exit(OPENDIR_FAIL_CODE);
    }
    my @fileList = readdir($readDir);
    closedir $readDir;
    ToolCommon::Log($fh, "\nTo be included in tar file:\n") if $params->{DEBUG};
    my $copyList = "";
    foreach (@fileList) {
      if (($_ ne ".") && ($_ ne "..")) {
	ToolCommon::Log($fh, "  '$_'\n") if $params->{DEBUG};
	$copyList .= $_ . " ";
      }
    }
    if ($Config{archname} =~ /linux/) {
      $cmd = "cd $dir/newdb;$config->{TAR} --format=gnu -cf $dbName $copyList;cd $pwd >$logDir/MIGRnewdb.txt 2>&1";
    } else {
      $cmd = "cd $dir/newdb;$config->{TAR} cf $dbName $copyList;cd $pwd >$logDir/MIGRnewdb.txt 2>&1";
    }
    ToolCommon::Log( $fh, "CMD = $cmd\n") if $params->{DEBUG};
    $ret = system($cmd);
    if ($ret) {
      my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
      my $t = "'$cmd' FAILED (signal $signal, exitcode $exitCode)";
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      ToolCommon::Log( $fh, "  OK\n");
    }
  }
  return $ok;
}

=head2 DownloadDbase($$$$$$$)

=begin html
Download migrated database to the NE using TFTP (&lt;= R2.2) or CLI (&gt; R2.2).  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>activeRelNum</td><td>the NE release coming from</td></tr>
  <tr><td>userName</td><td>Unix user name in case of multi-user operation (command line option) empty otherwise</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

#------------------------------------------------------------------------------
#-- Use tftp till R2.2, CLI from R2.3 onwards (alarms can be suppressed).
#------------------------------------------------------------------------------
sub DownloadDbase($$$$$$) {
  my ($fh, $cli, $isam, $activeRelNum, $thisTftpDir, $userName) = @_;
  my $ok = 1;
  my $ret;
  my ($time1, $time2);
  my $neId = $params->{NONAME} ? $isam->[0] : $isam->[1];
  my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;

  ToolCommon::Log($fh, "Download migrated Database...");
  if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) >= 0) { 
    $ret = IsamCli::DownloadCompleteDb($cli, $params->{PRIMTFTPIP}, "$thisTftpDir/newdb"); # -- 1.1.6
    if (!defined($ret) || ! $ret) {
      ToolCommon::Log($fh, "return code not defined\n") if ($params->{DEBUG} && ! defined($ret));
      my $t = sprintf("FAILED (%s)", IsamCli::GetError());
      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
      $ok = 0;
    } else {
      ToolCommon::Log($fh, "  OK\n");
    }
  } else {
    my $tftp = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});  # -- 1.1.6
    if (defined($tftp)) {
      $tftp->binary();
      $time1 = time();
      if ($tftp->put("$toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$neId/$userName/newdb/$dbName", "/pub/Dbase/Download/$dbName") ne "") {
	my $t = sprintf("FAILED (Filename = $toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$neId/$userName/newdb/$dbName), error: %s",
			$tftp->error());
	SetAndLogError($fh, $t, "  $t\n");
	$ok = 0;
      } else {
	$time2 = time();
	ToolCommon::Log($fh, sprintf("  OK transfered %d bytes in %d seconds\n", $tftp->bytesTransfered(), $time2 - $time1));
	ToolCommon::Log($fh, "  Filename = $toolData->{TFTPDIR}/$toolData->{TFTPWORKDIR}/$neId/$userName/newdb/$dbName\n") if $params->{DEBUG};
      }
    } else {
      my $t = "Could not open tftp session";
      FamilyCommon::SetAndLogError($fh, $t, "  FAILED - $t\n");
      $ok = 0;
    }
  }
  return $ok;
}

=head2 DownloadSHUBDbase($$$$)

=begin html
Download SHUB database.  Only used when migrating to R2.2.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
</table>

=end html

=cut

sub DownloadSHUBDbase($$$$) {
  my ($fh, $debug, $isam, $dir) = @_;
  my $ok = 1;
  my $sourceFile1 =  $dir . "/" . $SHUBOUTFILENAME;
  my ($time1, $time2);

  ToolCommon::Log($fh, "Download migrated SHUB Database...");
  my $tftp = Net::TFTP->new($isam->[0], Timeout => $toolData->{TFTPTIMEOUT});
  if (defined($tftp)) {
    $tftp->binary();
    ToolCommon::Log($fh, sprintf("\ntftp %s: put %s /pub/Sw/Download/$SHUBINFILENAMER1\n", $isam->[0], $sourceFile1)) if $debug;
    $time1 = time();
    if ($tftp->put($sourceFile1, "/pub/Sw/Download/$SHUBINFILENAMER1") ne "") {
      my $t = sprintf("Failed to download migrated database: %s", $tftp->error());
      FamilyCommon::SetAndLogError($fh, $t, "  $t... skipping\n");
      $ok = 0;
    } else {
      $time2 = time();
      ToolCommon::Log($fh, sprintf("  OK, transfered %d bytes in %d seconds\n", $tftp->bytesTransfered(), $time2 - $time1));
    }
  } else {
    my $t = "Failed to open tftp session";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  }
  return $ok;
}

=head2 SwChecksOK($$$$$$$$$$)

=begin html
Perform some basic SW checks before starting the migration process.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>oswpInfo</td><td>pointer of the OSWP information as retrieved from the NE (both active and passive OSWP)</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>shubSwSize</td><td>Expected size of the SHUB SW file</td></tr>
  <tr><td>path</td><td>Location where the PBMT tools have been installed</td></tr>
  <tr><td>ntType</td><td>The NT type equipped in this ISAM (introduced in support of the NE family concept)</td></tr>
</table>

=end html

=cut

sub SwChecksOK($$$$$$$$$) {
  my ($fh, $cli, $snmpSession, $oswpInfo, $dir, $isam, $shubSwSize, $path, $ntType) = @_;
  my $ok = 1;

  if (Capability::ReleaseCompare(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'}, $oswpInfo->{'ACTIVE'}{'MINOR'}),
				 Capability::ConvertRelease(1,1), REGEXP, 0) < 0) { # -- 1.1.8
    my $t = "Automatic migrations only supported from R2.0 onwards";
    FamilyCommon::SetAndLogError($fh, $t, "$t !... skipping\n");
    $ok = 0;
  } elsif (($oswpInfo->{'ACTIVE'}{'NAME'} eq basename($params->{OSWP})) && (! $params->{TOECNTC}) && (! $params->{TONANTB}) && (! $params->{TOECNTE}) ) {
    my $t = sprintf("Node already running %s", $oswpInfo->{'ACTIVE'}{'NAME'});
    FamilyCommon::SetAndLogError($fh, $t, "$t ... skipping\n");
    $ok = 0;
  } else {
    if (! $params->{OFLONLY}) {
      ToolCommon::Log($fh, "Checking new NT software is passive...");
      if ($oswpInfo->{'PASSIVE'}{'NAME'} ne basename($params->{OSWP})) {
	my $t = "SWP '$params->{OSWP}' not present as second SW";
	FamilyCommon::SetAndLogError($fh, $t, "  $t... skipping\n");
	$ok = 0;
      } elsif ($oswpInfo->{'PASSIVE'}{'AVAIL'} ne 'enabled') {
	my $t = "Second SWP not enabled";
	FamilyCommon::SetAndLogError($fh, $t, "  $t... skipping\n");
	$ok = 0;
      } else {
	ToolCommon::Log($fh, "  OK\n");
	if (Capability::ReleaseCompare(Capability::ConvertRelease($oswpInfo->{'PASSIVE'}{'MAJOR'},
								  $oswpInfo->{'PASSIVE'}{'MINOR'}),
				       Capability::ConvertRelease(2,2), REGEXP, 0) == 0) {
	  my $ret = Octopus::ShubSwPresent($params->{DEBUG}, $dir, $isam, $SHUBSWCHECK, $shubSwSize, $path);
	  if ($ret <= 0) {
	    FamilyCommon::SetAndLogError($fh, sprintf("SHUB SW not found: %s", Octopus::GetError()));
	    $ok = 0;
	  }
	}
      }
      if ($ok) {
	ToolCommon::Log($fh, "Checking new NT software newer then active software...");
	my $swpInfo = ToolCommon::GetSwpBySnmp($snmpSession, $params->{DEBUG}, $oswpInfo->{'ACTIVE'}{'INDEX'}, $oswpInfo->{'PASSIVE'}{'INDEX'},
					       $toolData->{RETRIES});
	if (defined($swpInfo)) {
	  my $swVersCmp = Capability::ReleaseCompare(Capability::ConvertRelease($swpInfo->{'PASSIVE'}{'MAJOR'},
										$swpInfo->{'PASSIVE'}{'MINOR'}),
						     Capability::ConvertRelease($swpInfo->{'ACTIVE'}{'MAJOR'},
										$swpInfo->{'ACTIVE'}{'MINOR'}), REGEXP, 0);
	  if ($swVersCmp < 0) {
	    my $t = sprintf("New NT release older then active release (%d.%d vs %d.%d)",
			    $swpInfo->{'ACTIVE'}{'MAJOR'}, $swpInfo->{'ACTIVE'}{'MINOR'},
			    $swpInfo->{'PASSIVE'}{'MAJOR'}, $swpInfo->{'PASSIVE'}{'MINOR'});
	    FamilyCommon::SetAndLogError($fh, $t, "  $t !... skipping\n");
	    $ok = 0;
	  } else {
	    ToolCommon::Log($fh, "  OK\n");
	  }
	} else {
	  my $t = sprintf("Failed to determine SWP info: %s", ToolCommon::GetError());
	  FamilyCommon::SetAndLogError($fh, $t, "  $t !... skipping\n");
	  $ok = 0;
	}
      }
    }
  }
  return $ok;
}

=head2 SaveShubDbase($$$)

=begin html
Save the SHUB database.<br>The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
</table>

=end html

=cut

#------------------------------------------------------------------------------
#-- Save the SHUB database
#------------------------------------------------------------------------------
sub SaveShubDbase($$$) {
  my ($fh, $debug, $cli) = @_;
  my $ok = 1;

  ToolCommon::Log( $fh, "Saving SHUB database...");
  my $ret = IsamCli::ShubDbaseSave($cli);
  if (!$ret) {
    my $t = sprintf("Failed to save SHUB DB: %s", IsamCli::GetError());
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    ToolCommon::Log( $fh, "  OK\n");
  }
  return $ok;
}

=head2 SetAndCheckNewLanxConf($$$)

=begin html
Change the SHUB boot parameters to TFTP.<br>
The following parameters are used:
<table>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>oswpInfo</td><td>pointer of the OSWP information as retrieved from the NE (both active and passive OSWP)</td></tr>
  <tr><td>newConf</td><td>the new SHUB configuraton setting</td></tr>
</table>

=end html

=cut

sub SetAndCheckNewLanxConf($$$) {
  my ($cli, $oswpInfo, $newConf) = @_;
  my $ret = IsamCli::ChangeShubConfig($cli, $newConf);

  return $ret;
}

=head2 RebootAndChangeLanxToTftp($$$$$$$$)

=begin html
From R2.2 onwards, the SHUB retrieves its SW from the NT flash disk.  SHUBs running R2.1 will be downloaded with R2.2 in the passive OSWP but also
the SW is downloaded to the NT.<br>
Changing the boot params of the SHUB to tftp makes sure that the SHUB - once restarted in R2.2 will fetch its SW from the NT disk (the database will
always be retrieved from the NT disk once SHUB R2.2 has been activated (as such there is no need to also download the SHUB R2.2 database to the
passive OSWP.<br>
Two SHUB reboots are needed to achieve this:
<ul>
  <li>a first reboot to have the SHUB running the new release from its own configuration</li>
  <li>a second reboot after the boot parameters have been changed.</li>
</ul>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>asam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>oswpInfo</td><td>pointer of the OSWP information as retrieved from the NE (both active and passive OSWP)</td></tr>
  <tr><td>expectedRelease</td><td>the release identification of the new OSWP</td></tr>
  <tr><td>singleRestart</td><td>Perform a single SHUB restart or execute the complete procedure involving 2 SHUB restarts</td></tr>
</table>

=end html

=cut

sub RebootAndChangeLanxToTftp($$$$$$$) {
  my ($fh, $cli, $session, $asam, $oswpInfo, $expectedRelease, $singleRestart) = @_;
  my $ret;
  my $swVers = "";
  my $trials = 1;
  my $activeConf;
  my ($shubMaj, $shubMin);

  ToolCommon::Log($fh, "Reboot and change SHUB boot parameter to tftp, first reboot SHUB to new release...");
  $ret = IsamCli::RebootShub($cli);
  if (!$ret) {
    my $t = sprintf("  First SHUB reboot FAILED: %s!!\n", IsamCli::GetError());
    FamilyCommon::SetAndLogError($fh, $t, "  $t !!\n");
  } else {
    $ret = FamilyCommon::WaitForRestart($fh, $session, $asam, 1);
    sleep(5);
    if ($ret) {
      while (($swVers eq "") && ($trials < 5)) {
	$swVers = IsamCli::GetLanxSwVersion($cli);
	$trials = 5 if ($swVers ne "");
	$trials++;
      }
      if (defined($swVers) && ($swVers ne "")) {
	if (($swVers =~ /LANX_R(\d*)\.(\d*).*/) || ($swVers =~ /^R(\d*)\.(\d*).*/)) {
	  $shubMaj = $1, $shubMin = $2;
	  ToolCommon::Log($fh, "SHUB runs '$swVers'\n") if ($params->{DEBUG});
	  if (! $singleRestart) {
	    if (Capability::ReleaseCompare(Capability::ConvertRelease($shubMaj, $shubMin), $expectedRelease, REGEXP, 0) == 0) {
	      $ret = SetAndCheckNewLanxConf($cli, $oswpInfo, "tftp");
	      if (defined($ret) && $ret) {
		ToolCommon::Log($fh, "Now again reboot SHUB and wait for restart with tftp settings...");
		$ret = IsamCli::RebootShub($cli);
		if ($ret) {
		  $ret = FamilyCommon::WaitForRestart($fh, $session, $asam, 1);
		  if ($ret) {
		    # Determine the new active conf on SHUB.  This command needs to be tried a couple
		    # of times since there is no way to suppress alarms and these clutter the response.
		    $activeConf = GetActiveLanxConf($cli, $expectedRelease);
		    if ($activeConf eq "tftp") {
		      ToolCommon::Log($fh, "SHUB now running from NT disk\n");
		    } else {
		      $ret = 0;
		      my $t = sprintf("SHUB not booting using tftp settings: %s", IsamCli::GetError());
		      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
		    }
		  } else {
		    my $t = "ISAM not back after restart SHUB using tftp";
		    FamilyCommon::SetAndLogError($fh, $t, "  FAILED - $t\n");
		    # Failed.  Error set in WaitForRestart
		  }
		} else {
		  $ret = 0;
		  my $t = sprintf("Failed to boot SHUB using tftp settings: %s", IsamCli::GetError());
		  FamilyCommon::SetAndLogError($fh, $t, "  $t!!\n");
		}
	      }  else {
		$ret = 0;
		my $t = sprintf("FAILED to change boot params to tftp: %s", IsamCli::GetError());
		FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	      }
	    } else {
	      $ret = 0;
	      my $t = sprintf("FAILED to restart SHUB with %d.%d while expecting %s\n", $shubMaj, $shubMin,
			      Utils::DotRelease($expectedRelease));
	      FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	    }
	  }
	} else {
	  ToolCommon::Log($fh, sprintf("  FAILED to determine SHUB release: '%s'\n", $swVers));
	  $ret = 0;
	}
      } else {
	my $t = sprintf("Failed to get SHUB SW version: %s\n", IsamCli::GetError());
	FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
	$ret = 0;
      }
    } else {
      my $t = "ISAM not back after restart SHUB using other config";
      FamilyCommon::SetAndLogError($fh, $t, "  FAILED - $t\n");
      $ret = 0;
    }
  }
  return $ret;
}

=head2 DetermineNtCommunities ($$$)

=begin html
Called when performing ISAM-V migration.  From R3.5 onwards different community strings are used to communicate with the xVPS boards in the system.
The community strings have an xVPS context.  The community strings are retrieved from the NT backup (stored in dmprot/DbCommunity).<br>
The result is hash indexed by an xVPS number and holding the community string as content.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
</table>

=end html

=cut

my $COMMUNITY_FILE = "dmprot/DbCommunity";

sub DetermineNtCommunities($$) {
  my ($fh, $dir) = @_;
  my $ok = 1;
  my $cmd = "cd $dir;$config->{TAR} xf $NTINFILENAME $COMMUNITY_FILE >detcomm.txt 2>&1";
  my $ret = system($cmd);
  my %commStrings;
  my $ctx;

  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "Failed to untar database '$NTINFILENAME' (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = 0;
  } else {
    my ($header, $fields, $recs, $recCnt, $actualRecords) = DM::ReadFile("$dir/$COMMUNITY_FILE"); 
    foreach my $rec (@{$recs}) {
      $ctx = uc($rec->{'ctxName'});
      ToolCommon::Log($fh, "Probing community for context '$ctx'\n") if $params->{DEBUG};
      $commStrings{$ctx} = "";
      foreach (split(//, $rec->{'name'})) {
	if (ord($_) != 0) {
	  $commStrings{$ctx} .= $_;
	} else {
	  last;
	}
      }
      ToolCommon::Log($fh, "Community string found for context $ctx found ($commStrings{$ctx})\n") if $params->{DEBUG};
    }
  }
  return $ok ? \%commStrings : undef;
}

=head2 DetermineXvpsPresence($$$)

=begin html
Called when performing migration using a DB stored offline (-ofldb option).  This procedure checks whether an xVPS board is present.<br>
Returns undef in case of failure, 0 in case no XVPS board was found, 1 in case the ISAM is an ISAM-V.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
</table>

=end html

=cut

sub DetermineXvpsPresence($$) {
  my ($fh, $dir) = @_;
  my $ok = 0;
  my $cmd = "cd $dir;$config->{TAR} xf $params->{OFLDB} >detxvpstarxf.txt 2>&1";
  my $ret = system($cmd);
  my $version = "";
  my $line = "";

  if ($ret) {
    my ($exitCode, $signal) = Utils::ConvertExitCode($ret);
    my $t = "Failed to untar database '$params->{OFLDB}' (signal $signal, exitcode $exitCode): $!";
    FamilyCommon::SetAndLogError($fh, $t, "  $t\n");
    $ok = undef;
  } else {
    my ($header, $fields, $recs, $recCnt, $actualRecords) = DM::ReadFile("$dir/dm/EqBoard"); 
    foreach my $rec (@{$recs}) {
      ToolCommon::Log($fh, sprintf("Probing XVPS-type for boardCode 0x%04x\n", $rec->{'Type'})) if $params->{DEBUG};
      if (defined(my $xvpsType = Capability::DetermineXvpsType($rec->{'Type'}))) {
	ToolCommon::Log($fh, " $xvpsType found\n") if $params->{DEBUG};
	$ok = 1;
	last;
      }
    }
  }
  return $ok;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 IsamSystemRestart($$$$$$$$$$$$$$$)

=begin html
This function deals with all conditions to restart the system with the new SW and DB.  Dedicated workarounds have been included to increase the
probability to a level as high as possible that the system is running the new release when it has come up after the restart.  The following
parameters are provided:<br>
<table>
  <tr><td>nodeLogFH</td><td>A file handle to log migration progress for this ISAM</td></tr>
  <tr><td>params</td><td>a pointer to the a hash containing parameters used when calling the script</td></tr>
  <tr><td>cli</td><td>a pointer to an opened CLI session.  This session is closed and re-opened during this restart, consequently
                      the new session is returned to the caller</td></tr>
  <tr><td>snmpSession</td><td>a pointer to an opened SNMP session</td></tr>
  <tr><td>dir</td><td>directory name (including path) to the temporary storage for  this ISAM</td></tr>
  <tr><td>isam</td><td>a pointer to an isam identification structure</td></tr>
  <tr><td>ntInfo</td><td>pointer to a hash containing NT information</td></tr>
  <tr><td>thisTftpDir</td><td>tftp directory</td></tr>
  <tr><td>fullActiveRelNum</td><td>The full release number currently active</td></tr>
  <tr><td>newRelNum</td><td>The shortened release number of the new release</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>dirName</td><td>directory name where PBMT was installed</td></tr>
  <tr><td>oswpPar</td><td>Pointer to a hash containing OSWP information</td></tr>
  <tr><td>secondRun</td><td>Indication whether this is the first (0) or second (1) call to the ISAM restart function.</td></tr>
  <tr><td>childNum</td><td>The number of the child process carrying out the migration(used for printing error information in case of fatal error)</td></tr>
</table>
The function returns a list of 3 values:
<ul>
  <li>an exit status: 0 indicates failure, 1 means success</li>
  <li>a retry possible indication:<ul>
      <li>-1: no more retries and SHUB and NT are not running the same SW version</li>
      <li>0: no retry possible</li>
      <li>1: retry still possible</li>
      </ul></li>
  <li>a cli session: either the old one in case of failure or a new one in case of NT restart</li>
</ul>
<br>
Some coding information:
<ul>
  <li>This function contains an exit statement in case of migration failure and we lost the system (and if the script is not instructed to
continue execution</li>
  <li>we return at various places in order to prevent too deep nesting of if-then-else structures (or sub-routine call cascades).  This means that
      a return is executed as soon as an error condition is encountered.</li>
</ul>

=end html

=cut

my $MAX_RETRIALS = 2;

# This function contains an exit statement in case migration failed and we lost the system (and if the script is not instructed to
# continue execution.
# Some deviations from coding rules:
# - This function used the common variables:
#   $childCount
#   $parentFh
#   $failedNode
#   $cliSession2Close
# - we return at various places in order to prevent too deep nesting of if-then-else structures (or sub-routine call cascades).  This means that
#   a return is executed as soon as an error condition is encountered.
# 3 values are returned:
# - an exit status (0 indicates failure, 1 means success)
# - a retry possible indication:
#   -1: no more retries and SHUB and NT are not running same SW version
#    0: no retry possible
#    1: retry still possible.
# a CLI session; either the old one in case of failure or a new one in case of NT restart.

sub IsamSystemRestart($$$$$$$$$$$$$$) {
  my ($nodeLogFH, $cli, $snmpSession, $dir, $isam, $ntInfo, $thisTftpDir, $fullActiveRelNum, $newRelNum, $newLog, $dirName, $oswpInfo,
     $secondRun, $childNum) = @_;
  my $ret;
  my $restartTrials = 0;
  my $shubSwVers = undef;
  my $shubMaj = undef;
  my $shubMin = undef;
  my $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';
  my $passiveNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTB' : 'NTA';
  my $oswpInfoIsEmpty = 0;
  my $ntRestarted = 0;  # Controls SHUB SW version retrieval failure: the restart loop is executed twice.  Due to a bug in R2.4 the NT restart
                        # is unpredictable and may occur at any time.  This results in failing CLI commands.  This variable will be set to 1
                        # After we have waited for the NT to restart

  #-- STEP 9: Activate SWP
  # Make sure we do not get an NT reset due to a NON_RECOV at the time of OSWP activation.
  # Workaround for R2.4 problem in OSWP activation for ARAM-D with exp shelves.
  # Only if not explicitely forbidden by the requestor
  if ($params->{REMAPHOTRESET}) {
    ToolCommon::Log($nodeLogFH, "Remap error record...");
    $ret = Octopus::RemapErrRecord($params->{DEBUG}, $dir, $isam, $dirName);
    if (! $ret) {
      my $t = sprintf("  FAILED: %s... skipping\n", Octopus::GetError());
      FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
      return (0, 0, $cli);
    } else {
      ToolCommon::Log($nodeLogFH, "  OK\n");
    }
  }

  if ($params->{MYTEST}) {
    ToolCommon::Log($nodeLogFH, "You can reset LTs now, sleeping for 10 seconds...\n", 1);
    sleep 10;
  }

  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return (0, 0, $cli) if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  return if( not ToolCommon::ActivateSwp($oswpInfo) );
  
  #-- STEP 10: Wait for restart
  if ($toolData->{ACTPINGWAIT} > 0) {
    ToolCommon::Log($nodeLogFH, "Waiting $toolData->{ACTPINGWAIT} seconds for the system to restart...");
    sleep($toolData->{ACTPINGWAIT});
    ToolCommon::Log($nodeLogFH, "  OK\n");
  }
  # R2.4 workaround for ANTms55912: Check whether NT has actually performed a reset and kick it if not (bug also in R3.4)
  my $systemUp;
  while ($restartTrials < $MAX_RETRIALS) {
    if ($params->{NOPING}) {
      my $uptime = ToolCommon::DoSafeGet($snmpSession, SYSUPTIME, 0);
      if (defined($uptime)) {
        $systemUp = 1;

        if ($uptime eq "") {
          my $version = $snmpSession->version();
          $version++ if ($version < 3);
          my $t = sprintf("SNMP configuration mismatch: check SNMP version (locally SNMPv%d is configure) and SNMP username/password (if applicable)", $version);
          FamilyCommon::IncrErrorCount("  FAILED: $t",
				       EC_SNMP_SESS_ERR,
                                       ": $t",
                                       1);
	  $systemUp = 0;
        } 

      } else {
	$systemUp = 0;
      }
    } else {
      $systemUp = ToolCommon::Ping($config->{PING}, $isam->[0]);
    }
    ### Found to be system would be in ISSU
    $systemUp = 0;
    if ($systemUp) {
	  $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	  if (!defined($cli)) {
	    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s (2nd node-restart run): %s\n",
						 FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
					 EC_OPENCLI_FAILED,
					 sprintf(": %s", ToolCommon::GetError()),
					 1);
	    return undef;
	  } else {
	    ToolCommon::Log($nodeLogFH, "  OK\n");
	  }
	  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
      $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}),
					  \$oswpInfoIsEmpty, $toolData->{RETRIES});
      if (!defined($oswpInfo)) {
	if (! IsamCli::ConnectionReset()) {
	  if (ToolCommon::NtIsUp()) {
	    if (! IsamCli::EmptyCliResponse()) {
	      my $t = sprintf("Obtaining active SW failed (R2.4 rescue): %s", ToolCommon::GetError());
	      FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	      return (0, 0, $cli);
	    } else {
	      # Empty response received, try the Octopus way.
	      $oswpInfo = Octopus::GetOswpFromFW($params->{DEBUG}, $dir, $isam, "Name", REGEXP);
	      if (!defined($oswpInfo)) {
		my $t = "Empty CLI response received and failed to get OSWP info from firmware";
		FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		return (0, 0, $cli);
	      } else {
		if (($oswpInfo->{'ACTIVE'}{'MAJOR'} == 0) && ($oswpInfo->{'PASSIVE'}{'MAJOR'} == 0)) {
		  if (Octopus::NtIsUp()) {
		    my $t = "Empty CLI response received and failed to obtain FW version info";
		    FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		    return (0, 0, $cli);
		  } else {
		    # NT died in the process, just wait till node is back.
		  }
		}
	      }
	    }
	  } else {
	    # NT died in the process of obtaining the active SW.  Rescue path not needed and continue with rest of loop.
	    # A second attempt will be made.
	  }
	} else {
	  # CLI connection was lost, means NT has gone for reset
	}
      } else {
	$shubSwVers = IsamCli::GetLanxSwVersion($cli) if !defined($shubSwVers);  # in case $shubSwVers is defined means that we're going for a retry
	if (defined($shubSwVers) && ($shubSwVers ne "")) {
	  if (($shubSwVers =~ /LANX_R(\d*)\.(\d*).*/) || ($shubSwVers =~ /^R(\d*)\.(\d*).*/)) {
	    $shubMaj = $1;
	    $shubMin = $2;
	    ToolCommon::Log($nodeLogFH, "SHUB runs '$shubSwVers'\n") if ($params->{DEBUG});
	    if ((Capability::ReleaseCompare(Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
								       $oswpInfo->{'ACTIVE'}{'MINOR'},
								       $oswpInfo->{'ACTIVE'}{'SUFFIX'}),
					    $fullActiveRelNum, REGEXP, 0) == 0) &&
		(($oswpInfoIsEmpty > 0) && ($oswpInfo->{FRMW_OSWP} != $oswpInfo->{ACT_PENDING}))) {
	      if ($oswpInfoIsEmpty) {
		# Node did not complete a successfull node restart
		unlink "$dir/$R2_4_RESCUE";
		if (copy("$dirName/$toolData->{OCTOPUSSCRIPTS}/$R2_4_RESCUE", "$dir/$R2_4_RESCUE")) {
		  if (!Octopus::ResetNT($params->{DEBUG}, $dir, $isam, $dirName,
					$ntInfo->{$passiveNt}{STATUS} eq "EMPTY" ? undef : $activeNt,
					$params->{PRIMTFTPIP}, "$thisTftpDir/$R2_4_RESCUE")) {
		    my $t = sprintf("System reset via rescue path failed: %s", Octopus::GetError());
		    FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		    return (0, 0, $cli);
		  } else {
		    ToolCommon::Log($nodeLogFH, "NT/SHUB reset rescue path triggered!\n");
		  }
		} else {
		  my $t = "Copy of '$dirName/$toolData->{OCTOPUSSCRIPTS}/$R2_4_RESCUE' to '$dir/$R2_4_RESCUE' failed: $!";
		  FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		  return (0, 0, $cli);
		}
	      } else {
		if (! $secondRun) {
		  ToolCommon::Log($nodeLogFH, "Node still on old release, initiate a complete retry\n");
		  return (0, $config->{SECOND_RESTART} ? 1 : 0, $cli);  # Allow the complete sequence to restart
		} else {
		  my $t = "Failed to activate new release after 2 attempts";
		  FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		  return (0, 0, $cli);
		}
	      }
	    } elsif (Capability::ReleaseCompare(Capability::ConvertRelease($shubMaj, $shubMin), $newRelNum, REGEXP, 0) != 0) {
	      # This one is also executed when the active release is still the old one, the CLI response was empty but the 
	      # active OSWP as returned by firmware management is the same as the "to-be-activated" OSWP.  This in fact means that
	      # the new NT SW is active but the NT is waiting for the SHUB to come up.  It will however not come up and we have
	      # to reboot it.
	      # Only SHUB restart required
	      if (uc($ntInfo->{$activeNt}{TYPE}) eq "ECNT-A") {
		if (!IsamCli::RebootShub($cli)) {
		  my $t = sprintf("SHUB reboot failed: %s", IsamCli::GetError());
		  FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		  return (0, 0, $cli);
		}
	      } else {
		my $t = "NT and SHUB on different releases and SHUB reboot only possible on ECNT-A";
		FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		return (0, 0, $cli);
	      }
	    } else {
	      ToolCommon::Log($nodeLogFH, "NT/SHUB reset rescue path not required (NT not running previous release)!\n");
	    }
	  } else {
	    my $t = "Unxepected LANX SW version syntax: $shubSwVers";
	    FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	    return (0, 0, $cli);
	  }
	} else {
	  if ($ntRestarted) {
	    my $t = sprintf("Obtaining SHUB SW version failed: %s", IsamCli::GetError());
	    FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	    return (0, 0, $cli);
	  } else {
	    # We're still in the first iteration of the loop where we've given the OSWP activation command.  Due to a bug in R2.4.9* this 
	    # restart may take some time.  We are executing a number of CLI commands to determine what needs to be done.  As such it can happen
	    # that reading the OSWP info via the T&D function calls returns that the old SW is still running but that the NT crashes at the time
	    # we try to obtain the SHUB SW version.  As such we continue as if the node is restarting.
	  }
	}
      }
    } else {
      ToolCommon::Log($nodeLogFH, "NT/SHUB reset rescue path not required!\n");
    }
    FamilyCommon::CloseCliSession2Close();
    $ntRestarted = 1;
    # Telnet session realy ends when the NT resets and comes up again.
    $ret = FamilyCommon::WaitForRestart($nodeLogFH, $snmpSession, $isam, undef); # -- 1.1.6
    if (!$ret) {
      #-- isam is not back ... start a panic
#      my $t = sprintf(" FAILED: %s... skipping", IsamCli::GetError());
#      FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
#      Logging has already been done in WaitForRestart
      if ($params->{CONTINUE}) {
	ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			sprintf("Child %d: **ATTENTION** Restart of %s failed (timed out), please verify %s.\n",
			       $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	return (0, 0, $cli);
      } else {
	# Here's the exit from the script...
	ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			sprintf("Child %d: **FATAL** Restart of %s failed (timed out), please verify %s.\n",
			       $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	FamilyCommon::IncrErrorCount(sprintf(" ERROR: %s is not reachable!!\n",FamilyCommon::GetFailedNode()),
                        									EC_NO_PING,
                        									undef,
                        									1);
	FamilyCommon::ToolExit(NT_RESTART_FAIL_CODE);
      }
    }

    # Reset transaction owner since this seems to survive a migration (even though the prozone gets erased...)
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return (0,0,$cli) if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");
    #-- Open Cli Connection
    ToolCommon::Log($nodeLogFH, "Re-open CLI...");
    $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
    if (!defined($cli)) {
      my $t = sprintf("  ERROR: Could not open cli for %s: %s", FamilyCommon::GetFailedNode(), ToolCommon::GetError());
      FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
      return (0, 0, $cli);  # no more second call to us: we lost the CLI...
    }
    ToolCommon::Log($nodeLogFH, "  OK\n");
    FamilyCommon::SetCliSession2Close($cli);

    #-- STEP 11: Check Correct SWP is active
    my $restartOswpInfo;
    ($ret, $oswpInfoIsEmpty, $restartOswpInfo) = FamilyCommon::CheckActiveSoftwareIsNew($nodeLogFH, $cli, $snmpSession,
											$params->{OSWP},
											$dir, $isam, $ntInfo->{$activeNt}{TYPE});
    if (!$ret) {
      if ($oswpInfoIsEmpty == -1) {
	# Node has possible gone down, so wait until back.  Will fall trough if up.
	$ret = FamilyCommon::WaitForRestart($nodeLogFH, $snmpSession, $isam, undef); # -- 1.1.6
	if (!$ret) {
	  #-- isam is not back ... start a panic
	  my $t = sprintf(" FAILED: %s... skipping", IsamCli::GetError());
	  FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	  if ($params->{CONTINUE}) {
	    ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			    sprintf("Child %d: **ATTENTION** Restart of %s failed (timed out), please verify %s.\n",
				   $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	    return (0, 0, $cli);
	  } else {
	    # Here's the exit from the script...
	    ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			    sprintf("Child %d: **FATAL** Restart of %s failed (timed out), please verify %s.\n",
				   $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	    FamilyCommon::ToolExit(NT_RESTART_FAIL_CODE);
	  }
	} else {
	  if (! IsamCli::ConnectionStillOpen($cli)) {
	    # Wait till telnet open before checking a pending OSWP activation.
	    $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	    if (!defined($cli)) {
	      my $t = sprintf("  ERROR: Could not open cli for %s (checking new active SW): %s\n", FamilyCommon::GetFailedNode(),
			      ToolCommon::GetError());
	      FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	      return (0, 0, $cli);
	    } else {
	      ToolCommon::Log($nodeLogFH, "  OK\n");
	    }
	    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	  }
	}
      } else {
	if (! defined($restartOswpInfo)) {
	  # No OSWP information could be retrieved
	  $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	  if (!defined($cli)) {
	    my $t = sprintf("  ERROR: Could not open cli for %s (checking new active SW): %s\n", FamilyCommon::GetFailedNode(),
			    ToolCommon::GetError());
	    FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	    return (0, 0, $cli);
	  } else {
	    ToolCommon::Log($nodeLogFH, "  OK\n");
	  }
	  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	}
      }
      if ($oswpInfoIsEmpty > 0) {  # use > 0 to cope with a non-empty response to show software-mgnt command but still the old SW is running...
	                           # In this case $oswpInfoIsEmpty is set to -1.  We only try this one if the CLI response is empty.
	if ($restartTrials < ($MAX_RETRIALS - 1)) {
	  $restartTrials++;
	  ToolCommon::Log($nodeLogFH, "OSWP info retrieval returned empty list but old SW (NT and/or SHUB) still active, one more retry...\n");
	  ToolCommon::Log($nodeLogFH, "Time to complete node migration will be extended due to this retry, be patient...\n");
	  $ntRestarted = 0;    # We're going for a new retry (including NT restart so clear this indicator).
	} else {
	  # Set and log has been done already.  Old SW still running, a new OSWP activation is need as retry.
	  return (0, uc($config->{SECOND_RUN}) eq "Y" ? 1 : 0, $cli);
	}
      } else {
	# Set and log has been done already
	return (0, uc($config->{SECOND_RUN}) eq 'Y' ? 1 : 0, $cli);
      }
    } else {
      if ($ret != 2) {   ### ADDED BBOG
      $shubSwVers = IsamCli::GetLanxSwVersion($cli);
      if (defined($shubSwVers) && ($shubSwVers ne "")) {
	if (($shubSwVers =~ /LANX_R(\d*)\.(\d*).*/) || ($shubSwVers =~ /^R(\d*)\.(\d*).*/)) {
	  $shubMaj = $1;
	  $shubMin = $2;
	  my $swRelNum = Capability::ShubReleaseNum($toolData->{TONEFAMILY}, $params->{DBVER_TGTREL});
	  if (!defined($swRelNum)) {
	    my $t = "Internal error, unable to determin SHUB SW version";
	    FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	    return (0, 0, $cli);
	  } else {
	    ToolCommon::Log($nodeLogFH, "SHUB runs '$shubSwVers'\n") if ($params->{DEBUG});
	    if (Capability::ReleaseCompare(Capability::ConvertRelease($shubMaj, $shubMin), $swRelNum, REGEXP, 0) == 0) {
	      $restartTrials = $MAX_RETRIALS;
	    } else {
	      if ($restartTrials < ($MAX_RETRIALS - 1)) {
		$restartTrials++;
		ToolCommon::Log($nodeLogFH, "SHUB not running the active release while NT does, one more retry...\n");
		ToolCommon::Log($nodeLogFH, "Time to complete node migration will be extended due to this retry, be patient...\n");
	      } else {
		my $t = "SHUB still runs the old release even after rebooting.  NT/SHUB SW version mismatch";
		FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
		return (0, -1, $cli);
	      }
	    }
	  }
	} else {
	  my $t = "SHUB SW version not according to expected syntax: $shubSwVers";
	  FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	  return (0, 0, $cli);
	}
      } else {
	my $t = sprintf("Obtaining SHUB SW version failed: %s", IsamCli::GetError());
	FamilyCommon::SetAndLogError($nodeLogFH, $t, "  $t\n");
	return (0, uc($config->{SECOND_RUN}) eq 'Y' ? 1 : 0, $cli);
	}
      } else {
	##### ADDED BBOG
	if ($restartTrials < ($MAX_RETRIALS - 1)) {
	  $restartTrials++;
	  ToolCommon::Log($nodeLogFH, "OSWP info retrieval returned empty list but old SW (NT and/or SHUB) still active, one more retry...\n");
	  ToolCommon::Log($nodeLogFH, "Time to complete node migration will be extended due to this retry, be patient...\n");
	  $ntRestarted = 0;    # We're going for a new retry (including NT restart so clear this indicator).
	} else {
	  # Set and log has been done already.  Old SW still running, a new OSWP activation is need as retry.
	  return (0, uc($config->{SECOND_RUN}) eq "Y" ? 1 : 0, $cli);
	}
	##### ADDED BBOG
      }
    }
  }
  return (1, 0, $cli);
}

=head2 MigrateOrUpgrade($$$$$)

=begin html
This function determines whether we need to migrate of upgrade the node.  True is returned in case of migration, FALSE in case of an upgrade
and the undefined value in case of an error.

=end html

=cut

sub MigrateOrUpgrade($$$$) {
  my ($fh, $config, $dirName, $isam) = @_;
  my $sourceDbVersion = $params->{SOURCE_DBVERSION};
  my $targetDbVersion = $params->{DBVER_TGTREL};
  if (! $params->{TOECNTC} && ! $params->{TONANTB} && ! $params->{TOECNTE}) {
     return ($targetDbVersion eq $sourceDbVersion) ? 0 : 1;
  } else {
     return 1;
  }
}

=head2 Migrate($$$$$)

=begin html
This function is called to perform migration of a node of the ISAM family.  In case of failure the undefined value is returned otherwise a string
of depending holding depending nodes is returned (only in case of ISAM-V, it will be an empty list in case of ISAM).<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>newLog</td><td>Name of the logfile of this child (used for printing error information in case of fatal error)</td></tr>
  <tr><td>thisTftpDir</td><td>tftp directory</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Migrate($$$) {
  my ($newLog, $thisTftpDir, $isam) = @_;
  my $cli = FamilyCommon::GetCliSession2Close();
  my $ret;
  my ($newRelNum, $newFullRelNum);
  my $possible = 0;
  my $reason = "";
  my ($swMgntIniSize, $altSwMgntIniSize, $swParSize);
  my $sfpAssignmentTable;
  my $isSfpAssigned = 0;
  my $isamIsIsamV = 0;
  my $xvpsConfig = undef;
  my $xvpsDependancy = undef;
  my $nameList = "";
  my $parentLogFh = FamilyCommon::GetParentLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childNum = FamilyCommon::GetChildNum();
  my $userName = getlogin();
  my $U2UHairPin = 0;     ## U2U hairpin disabled (0)

  FamilyCommon::ClearError();
  $config = FamilyCommon::GetConfigData();
  # Read warning levels
  %warnLevels = Utils::ReadConfig(sprintf("%s/%s", $dirName, $toolData->{WARNINGFILE}));
  if (not %warnLevels) {
    FamilyCommon::IncrErrorCount(sprintf("  Error reading warning file: %s\n", Utils::GetError()),
				 EC_OPEN_WARN_FILE_FAILED,
				 sprintf(": %s", Utils::GetError()),
				 1);
    return undef;
  }

  if (! defined($params->{OFLDB})) {
    if (! $params->{ACTONLY}) {
      # Keep DB uploaded with previous migration
#    if (! -e $dir . "/$NTINFILENAME") {
      if (!FamilyCommon::CreateBUFile($parentLogFh, $nodeLogFH, $dir, $NTINFILENAME, $newLog, $childNum, $params->{DEBUG})) {
	CORE::exit(CREATE_FILE_FAIL_CODE);
      }
    }
  }
  if ((defined($params->{OFLDB}) && (! $params->{OFLONLY})) ||
      (! defined($params->{OFLDB}))) {
    if (!defined($cli)) {
      #-- Open Cli Connection
      ToolCommon::Log($nodeLogFH, "Open CLI...");
      $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
      if (!defined($cli)) {
	FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				     EC_OPENCLI_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK\n");
      }
      FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
      ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});
    }
    #-- STEP 0: acquire transaction
    Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
    return undef if( ToolCommon::AcquireTransaction() );
    Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");
  }

  if (! defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
      ToolCommon::Log($nodeLogFH, "Set CLI session timeout value to two hours ...");
      if (!IsamCli::SetSessionTimeout($cli, 120)) { 
          ToolCommon::Log($nodeLogFH, sprintf("Set CLI session timeout FAILED: %s\n", IsamCli::GetError()));
      } else {
          ToolCommon::Log($nodeLogFH, "  OK\n");
      }
  }

  my $ntInfo = undef;
  my ($activeNt, $passiveNt);
  if (! defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
    #-- STEP 1a                                                         --- 1.2.0 BEGIN
    #-- Get NT type information
    ToolCommon::Log($nodeLogFH, "Obtain extended NT information...");
    $ntInfo = IsamCli::GetNtExtendedInfo($cli);
    if (!defined $ntInfo) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				   EC_GET_NT_INFO_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return undef;
    } else {
      ToolCommon::Log($nodeLogFH, "  OK\n");
      ToolCommon::Log($nodeLogFH, sprintf("  NTA: %s - %s, %s (ICS %s)\n                       NTB: %s - %s, %s (ICS %s)\n",
					  uc($ntInfo->{NTA}{TYPE}), $ntInfo->{NTA}{STATUS}, $ntInfo->{NTA}{ACTIVE}, $ntInfo->{NTA}{ICS},
					  uc($ntInfo->{NTB}{TYPE}), $ntInfo->{NTB}{STATUS}, $ntInfo->{NTB}{ACTIVE}, $ntInfo->{NTB}{ICS}));
    }

    if (($ntInfo->{NTA}{TYPE} ne $ntInfo->{NTB}{TYPE}) &&
	(uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") &&
	(uc($ntInfo->{NTB}{STATUS}) ne "EMPTY")) {
      FamilyCommon::IncrErrorCount(sprintf("Reduncdancy between two incompatible NT types ('%s' and '%s')... skipping\n",
					   $ntInfo->{NTA}{TYPE}, $ntInfo->{NTB}{TYPE}),
				   EC_INCOMPAT_NT,
				   "(NT-A: '$ntInfo->{NTA}{TYPE}', NT-B: '$ntInfo->{NTB}{TYPE}')",
				   1);
      return undef;
    }
    if (! $params->{FORCEMIG}) {
      if ((uc($ntInfo->{NTA}{TYPE}) ne "EMPTY") && (uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") && (! Capability::NtSupported($ntInfo->{NTA}{TYPE}))) {
	FamilyCommon::IncrErrorCount("NT-A type '$ntInfo->{NTA}{TYPE}' not supported for migration... skipping\n",
				     EC_NT_TYPE_UNKNOWN,
				     "($ntInfo->{NTA}{TYPE})",
				     1);
	return undef;
      }
      if ((uc($ntInfo->{NTB}{TYPE}) ne "EMPTY") && (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY") && (! Capability::NtSupported($ntInfo->{NTB}{TYPE}))) {
	FamilyCommon::IncrErrorCount("NT-B type '$ntInfo->{NTB}{TYPE}' not supported for migration... skipping\n",
				     EC_NT_TYPE_UNKNOWN,
				     "($ntInfo->{NTB}{TYPE})",
				     1);
	return undef;
      }
    }
    $activeNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTA' : 'NTB';
    $passiveNt = $ntInfo->{NTA}{ACTIVE} eq "ACTIVE" ? 'NTB' : 'NTA';
    #-- STEP 1a
  }

  my $oswpInfo;
  my $dbVersInfo;
  if (! defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
    #-- STEP 1b: Check if needed files are present on NT
    $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}), undef,
					$toolData->{RETRIES});
    if (!defined($oswpInfo)) {
      FamilyCommon::IncrErrorCount("Obtaining active SW failed.\n",
				   EC_GET_OSWP_FAILED,
				   sprintf(": %s", ToolCommon::GetError()),
				   1);
      return undef;
    }
    
    ToolCommon::Log($nodeLogFH, "Verify database versions...");
    $dbVersInfo = IsamCli::GetDbVersionInfo($cli);
    if (!defined($dbVersInfo)) {
      FamilyCommon::IncrErrorCount("  FAILED...skipping!!\n",
				   EC_GET_DB_INFO_FAILED,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return undef;
    } else {
      ToolCommon::Log($nodeLogFH, "  OK\n");
      if ($params->{DEBUG}) {
	foreach my $key (keys %{$dbVersInfo}) {
	  ToolCommon::Log($nodeLogFH, "OSWP '$key': '$dbVersInfo->{$key}'\n");
	}
      }
    }
  }
  my $dbVersionActive = $params->{SOURCE_DBVERSION};
  my $dbVersionTarget = $params->{DBVER_TGTREL};
  my $dbVersionActiveRel = substr($dbVersionActive,1);
  my $dbVersionTargetRel = substr($dbVersionTarget,1);
  my $activeRelNum;
  my $fullActiveRelNum;
  my $regExp = REGEXP;
  if (!defined($params->{OFLDB})) {
    $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
					       $oswpInfo->{'ACTIVE'}{'MINOR'},
                                               0,
					       $dbVersInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});
    $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						   $oswpInfo->{'ACTIVE'}{'MINOR'},
						   $oswpInfo->{'ACTIVE'}{'SUFFIX'},
                                                   $dbVersInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});

    if ( $dbVersionActive ne $oswpInfo->{'ACTIVE'}{'DBVERSION'}) {
      FamilyCommon::IncrErrorCount(sprintf("DB version not as expected: %s vs %s\n",
					   $dbVersionActive, $oswpInfo->{'ACTIVE'}{'DBVERSION'}),
				   EC_DB_VERS_UNEXPECTED,
				   sprintf("%s vs %s",
					   $dbVersionActive, $oswpInfo->{'ACTIVE'}{'DBVERSION'}),
				   1);
      return undef;
    }
  } else {
    if (basename($params->{SRC_OSWP}) =~ /$regExp/) {
      $activeRelNum = Capability::ConvertRelease($1, $2, 0, $params->{OFL_DB_VERS});
      $fullActiveRelNum = Capability::ConvertRelease($1, $2, $3, $params->{OFL_DB_VERS});
    }
  }
  ToolCommon::Log($nodeLogFH, "Active release info (short and full stream): ($activeRelNum, $fullActiveRelNum)\n") if $params->{DEBUG};
  if (basename($params->{SWPNAME}) =~ /$regExp/) {
    my $dbVerTargetRel = $params->{DBVER_TGTREL}; 
    $newRelNum = Capability::ConvertRelease($1, $2, 0, $dbVerTargetRel);
    $newFullRelNum = Capability::ConvertRelease($1, $2, $3, $dbVerTargetRel);
  }
  ToolCommon::Log($nodeLogFH, "New release info (short and full stream): ($newRelNum, $newFullRelNum)\n") if $params->{DEBUG};
  
  if ((defined($params->{OFLDB}) && (! $params->{OFLONLY})) ||
      (! defined($params->{OFLDB}))) {
    ToolCommon::Log($nodeLogFH, "Setting the Transaction Timeout Value ...");

    my $oldTransactionTO = IsamCli::GetTransactionTimeout($cli);
    if (!defined($oldTransactionTO)) {
          FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
                                     EC_GET_TRANS_TO_FAILED,
                                     sprintf(": %s", IsamCli::GetError()),
                                     1);
          return undef;
    } else {
        if($oldTransactionTO == 0) {
            $oldTransactionTO = Capability::GetTransactValue($activeRelNum);
        }
        ToolCommon::Log($nodeLogFH, "Transaction timeout : $oldTransactionTO\n") if $params->{DEBUG};
        FamilyCommon::SetOldTransactionTO($oldTransactionTO);
    }

    $ret = IsamCli::SetTransactionTimeout($cli, Capability::GetTransactValue($activeRelNum));

    if ($ret) {
          ToolCommon::Log($nodeLogFH, " OK\n");
    } else {
          FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
                                       sprintf(": %s", IsamCli::GetError()),
                                       EC_SET_TRANS_TO_FAILED,
                                       1);
          return undef;
    }
  }

  if (!defined($params->{OFLDB}) ||
      (defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
    # Get SFP Assignments from the ISAM only if migrating from (R2.4 OR R2.5) --->  R3.3
    if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) > 0) &&
	(Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 1), REGEXP, 0) < 0) &&
	(Capability::ReleaseCompare($newFullRelNum, Capability::ConvertRelease(3, 3), REGEXP, 0) >= 0)) {
      ToolCommon::Log($nodeLogFH, "Obtain extended SFP Assignment info...");
      $sfpAssignmentTable =  IsamCli::GetSfpAssignment($cli);
      if (!defined($sfpAssignmentTable)) {
	FamilyCommon::IncrErrorCount("  Unable to get SFP assignments...skipping!!\n",
				     EC_GET_SFP_ASSIGNMENT_FAILED,
				     sprintf(": %s", IsamCli::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK\n");
	IsamCli::PrintSfpAssignmentTable($sfpAssignmentTable);
	$isSfpAssigned = 1 if $sfpAssignmentTable->{TOTAL} > 0;
      }
    }

    #-- STEP: Configure AccessOperatorId (supported starting R3.6).  When going fronm 3.7 need to configure via CLI (OFLMT does not take care)
    if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 7), REGEXP, 0) >= 0) &&
	($params->{ACCESSOPRID})) {
      ToolCommon::Log($nodeLogFH, "Set accessOperatorId...");
      if (! ToolCommon::AccessOprIdConfigure($nodeLogFH, $params->{DEBUG}, $cli, $config->{ACCESS_OPERATOR_ID})) {
	FamilyCommon::IncrErrorCount(sprintf("  Failed...skipping: %s", ToolCommon::GetError()),
				     EC_CONFIG_OPRID_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK\n");
      }
    }
  }

  # 1.4.2 ECNT-A to ECNT-C only possible from R2.4 onwards
  if ($params->{TOECNTC}) {
    my $ntTypeFound;
    if (!defined($params->{OFLDB})) {
      $ntTypeFound = $ntInfo->{$activeNt}{TYPE};
    } else {
      $ntTypeFound = $params->{OFL_NT_TYPE};
    }
    if (uc($ntTypeFound) eq "ECNT-A") {
      # -- 1.4.2.2: $activeRelNum -> $newFullRelNum
      if (Capability::ReleaseCompare($newFullRelNum, Capability::ConvertRelease(2, 4, 1), REGEXP, 1) < 0) {
	FamilyCommon::IncrErrorCount("ECNT-A to ECNT-C conversion only possible from R2.4.01 onwards... skipping\n",
				     EC_INVALID_2_ECNT_C,
				     undef,
				     1);
	return undef;
      }
      if ($params->{SHUBOFLMTVERS} < 126) {
	FamilyCommon::IncrErrorCount(sprintf("Incorrect SHUB OFLMT for ECNT-A to ECNT-C migration need 1.26 or higher (v%d.%d found)... skipping\n",
					     $params->{SHUBOFLMTVERS} / 100, $params->{SHUBOFLMTVERS} % 100),
				     EC_INCORR_SHUB_OFLMT,
				     sprintf("v%d.%d found", $params->{SHUBOFLMTVERS} / 100, $params->{SHUBOFLMTVERS} % 100),
				     1);
	return undef;
      }
    } else {
      FamilyCommon::IncrErrorCount("ECNT-A to ECNT-C conversion called for $ntTypeFound: impossible... skipping\n",
				   EC_NT_NOT_ECNT_A,
				   undef,
				   1);
      return undef;
    }
  }
  # End 1.4.2
  if ($params->{TONANTB}) {
    if (!defined($params->{OFLDB})) {
      if (uc($ntInfo->{$activeNt}{TYPE}) eq "NANT-A") {
	if (Capability::ReleaseCompare($newFullRelNum, Capability::ConvertRelease(3, 5), REGEXP, 1) < 0) {
	  FamilyCommon::IncrErrorCount("NANT-A to NANT-B conversion only possible from R3.5 onwards... skipping\n",
				       EC_INVALID_2_NANT_B,
				       undef,
				       1);
	  return undef;
	}
      } else {
	FamilyCommon::IncrErrorCount("NANT-A to NANT-B conversion called for $ntInfo->{$activeNt}{TYPE}: impossible... skipping\n",
				     EC_NT_NOT_NANT_A,
				     undef,
				     1);
	return undef;
      }
      if ($params->{SHUBOFLMTVERS} < 132) {
	FamilyCommon::IncrErrorCount(sprintf("Incorrect SHUB OFLMT for NANT-A to NANT-B migration need 1.32 or higher (v%d.%d found)... skipping\n",
					     $params->{SHUBOFLMTVERS} / 100, $params->{SHUBOFLMTVERS} % 100),
				     EC_INCORR_SHUB_OFLMT_NANT_B,
				     sprintf("v%d.%d found", $params->{SHUBOFLMTVERS} / 100, $params->{SHUBOFLMTVERS} % 100),
				     1);
	return undef;
      }
    }
  }
  if (defined($params->{TTL0FWD})) {
    if (Capability::ReleaseCompare($newFullRelNum, Capability::ConvertRelease(3, 5), REGEXP, 1) < 0) {
      FamilyCommon::IncrErrorCount("TTL=0 packet forwarding only possible from R3.5 onwards... skipping\n",
				   EC_NO_TTL0_FWD,
				   undef,
				   1);
      return undef;
    }
  }
  if ($params->{ACTONLY}) {
    # ACTONLY gets reset in case the ofldb option has been used...
    if ( $dbVersionTarget ne $oswpInfo->{'PASSIVE'}{'DBVERSION'}) {
      FamilyCommon::IncrErrorCount(sprintf("New DB version not as expected: %s vs %s\n",
                       $dbVersionTarget, $oswpInfo->{'PASSIVE'}{'DBVERSION'}),
				   EC_DB_VERS_UNEXPECTED,
				   sprintf("%s vs %s", $dbVersionTarget, $oswpInfo->{'PASSIVE'}{'DBVERSION'}),
				   1);
      return undef;
    }
  }

  my ($shubSwName, $shubSwSize);
  if (! defined($params->{OFLDB})) {
    if (uc($ntInfo->{NTA}{TYPE}) eq 'ECNT-A') {  # 1.2.4.2
      # $shubSwSize only used for releases in the context of ECNT-A when directly migrating from R2.0/R2.1 to R2.2
      # This does not apply for ECNT-C.
      $shubSwName = Capability::FindShubSwName($params->{SWPNAME}, REGEXP, $toolData->{TFTPDIR}, $toolData->{SHUBSWPATH},
					       $toolData->{SHUBSWNAME}, Capability::IsamEtsiFamily());
      $shubSwSize = -s $toolData->{TFTPDIR} . "/" . $shubSwName;
    } else {                                                                                         # 1.2.4.2
      $shubSwSize = 0;                                                                               # 1.2.4.2
    }                                                                                                # 1.2.4.2
    ToolCommon::Log($nodeLogFH, "Expected shub SW size: $shubSwSize\n") if ($params->{DEBUG} && defined($shubSwSize));

    if (! $params->{TOECNTC} && ! $params->{TONANTB} && ! $params->{TOECNTE} ) {
      # -- 1.4.2.2
      if (! SwChecksOK($nodeLogFH, $cli, $snmpSession, $oswpInfo, $dir, $isam, $shubSwSize, $dirName,
		       defined($params->{OFLDB}) ? $params->{OFL_NT_TYPE} : $ntInfo->{$activeNt}{TYPE})) {
	FamilyCommon::IncrErrorCount("SW checks failed.\n",
				     EC_SW_CHECKS_NOK,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    }
  }

  $params->{IS_ISAMV_SRV} = 0;
  $params->{ISAM_HAS_VOICE} = 0;
  $params->{ISAM_VOICE_MIGR} = 0;
  $params->{CDE_TMP_PATH} = undef; 
  $params->{IGMP_MAX_UNCFG} = undef; 
  #-- STEP 1c
  #-- Verify migration path
  if (! $params->{ACTONLY}) {
    if (! defined($params->{OFLDB}) ||
	(defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
      if (! $params->{FORCEMIG}) {
	       ToolCommon::Log($nodeLogFH, "Checking whether migration restrictions apply...");
	       if (($dbVersionActive eq $dbVersionTarget) && (! $params->{TOECNTC} && ! $params->{TONANTB} && ! $params->{TOECNTE})) {
                  FamilyCommon::ReportStatus( EC_NEEDS_UPGRADE,
                            sprintf("Migration from %s (%s) to %s (%s) is not possible, needs upgrade... \n",
                          $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget ), undef );
	              if ($params->{CONTINUE}) {
	                return undef;
	              } else {
	                CORE::exit(UPG_IMPOSSIBLE_CODE);
	              }
	       }

          if ($dbVersionActive ne $dbVersionTarget) {
              #-- Verify migration path
              Logger::LogNotify($nodeLogFH, "Migration restriction checks...\n");
              my ($restricted, $restricted_reason) = Capability::IsMigrateRestricted( $ntInfo->{$activeNt}{TYPE},
                                                                           $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                        basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
              if (not $restricted) {
                  Logger::LogTrace($nodeLogFH, "Migrate Restrictions... NONE\n");
              } else {
                  FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                                    $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                                    basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
                  if ($params->{CONTINUE}) {
                    return undef;
                  } else {
                    FamilyCommon::ToolExit(UPG_IMPOSSIBLE_CODE);
                  }
              }
              ($possible, $reason) = CapabilityReEngr::MigrationPossible( $ntInfo->{$activeNt}{TYPE}, $dbVersionActive, $dbVersionTarget);
              if ($possible) {
                  Logger::LogTrace($nodeLogFH, "Migrate Restrictions... NONE\n");
              } else {
                  FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s",
                                  $oswpInfo->{'ACTIVE'}{'NAME'}, $dbVersionActive, basename($params->{OSWP}), $dbVersionTarget, $reason ));
                  if ($params->{CONTINUE}) {
                    return undef;
                  } else {
                    FamilyCommon::ToolExit(UPG_IMPOSSIBLE_CODE);
                  }
              }
         }
      }  else {
	     ToolCommon::Log($nodeLogFH, "Forced migration, continue without checks...\n");
      }
      #-- STEP 1c                                                          --- 1.2.0 END

      #-- STEP 1d get SAN disk partitioning size                           --- 1.1.7 BEGIN
      #-- SW download process has reduced active package so just print warning
      if (!defined($swMgntIniSize = Capability::SwMgntIniSize($ntInfo->{$activeNt}{TYPE}, $params->{SOURCE_DBVERSION}))) {
           FamilyCommon::ReportStatus( EC_NO_NT_CAP, sprintf("Unable to find SwMgntIniSize for NT-Type '%s' and DB-Version '%s'\n",
                                        $ntInfo->{$activeNt}{TYPE}, $params->{SOURCE_DBVERSION} ) );
	       if ($params->{CONTINUE}) {
	         return undef;
	       } else {
	         FamilyCommon::ToolExit(NT_CAP_NOTSET_CODE);
	       }
      }
      if (!defined($altSwMgntIniSize = Capability::AltSwMgntIniSize($ntInfo->{$activeNt}{TYPE}, $params->{SOURCE_DBVERSION}))) {
           FamilyCommon::ReportStatus( EC_NO_ALT_NT_CAP, sprintf("Unable to find AltSwMgntIniSize for NT-Type '%s' and DB-Version '%s'\n",
                                        $ntInfo->{$activeNt}{TYPE}, $params->{SOURCE_DBVERSION} ) );
	       if ($params->{CONTINUE}) {
	         return undef;
	       } else {
	         FamilyCommon::ToolExit(NT_CAP_ALT_NOTSET_CODE);
	       }
      }
      #-- STEP 1d                                                          --- 1.1.7 END

      my $relCmp = Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 4), REGEXP, 0);
      if ($relCmp >= 0) {
	# Possible issue with the number of packages present in a MC source: from R3.4 onwards only 20 are possible...
	ToolCommon::Log($nodeLogFH, "Verifying number of packages in the IGMP multicast sources...");
	if (! ToolCommon::CheckMcPackages($snmpSession, $params->{DEBUG}, $activeRelNum)) {
	  # Re-using a precheck error level here!!!  so EC_TOO_MANY_MC_SRC_PKG is NOT an error!!
	  my $code = EC_TOO_MANY_MC_SRC_PKG;
	  if (ErrorCodes::WarningLevelIsError($warnLevels{&EC_TOO_MANY_MC_SRC_PKG})) {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", ToolCommon::GetError()),
					 EC_TOO_MANY_MC_SRC_PKG,
					 undef,
					 1);
	    return undef;
	  } else {
	    ToolCommon::Log($nodeLogFH, "  Check set to be discarded (functionality is lost...)\n");
	  }
	} else {
	  ToolCommon::Log($nodeLogFH, "  OK\n");
	}
      }

      if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 3), REGEXP, 0) == 0) {
	my $profileMode = IsamCli::GetProfileSlotNumberMode($cli, $params->{USERNAME});
	if (! defined($profileMode)) {
	  FamilyCommon::IncrErrorCount(sprintf("Failed to determine CLI profile mode: %s", IsamCli::GetError()),
				       EC_GET_CLIMODE_FAILED,
				       sprintf(": %s", IsamCli::GetError()),
				       1);
	  return undef;
	} else {
	  ToolCommon::Log($nodeLogFH, "Profile mode '$profileMode' detected for NT type '$ntInfo->{$activeNt}{TYPE}'\n") if $params->{DEBUG};
	}
	ToolCommon::Log($nodeLogFH, "Check MAC address limitations...");
	if (!ToolCommon::CheckUnicastMacLimits($snmpSession, uc($profileMode), uc($ntInfo->{$activeNt}{TYPE}), $params->{DEBUG})) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", ToolCommon::GetError()),
				       EC_MAC_LIMITS_EXCEEDED,
				       undef,
				       1);
	  return undef;
	} else {
	  ToolCommon::Log($nodeLogFH, "  OK\n");
	}
      }

      if (((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 6), REGEXP, 0) >= 0) &&  
           (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(4, 2), REGEXP, 0) < 0)) &&  
          ((Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(4, 2), REGEXP, 0) >= 0)  &&
           (Capability::ReleaseCompare($newFullRelNum, Capability::ConvertRelease(4, 2, 2), REGEXP, 0, 1) < 0))) {   
        ToolCommon::Log($nodeLogFH, "Check Vmac Unique Host-id parameter limitations...");
	if (!ToolCommon::CheckVlanUniqHostId($cli)) {
	  FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", ToolCommon::GetError()),
				       EC_VMAC_HOST_ID_LIMITS_EXCEED,
				       undef,
				       1);
	  return undef;
	} else {
	  ToolCommon::Log($nodeLogFH, "  OK\n");
	}
         
      }

      ## Migration of IGMP unconfigured multicast channels - ALU01836984  
      if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(4, 1, 0), REGEXP, 0) >= 0) &&  
          (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(4, 4), REGEXP, 0) >= 0)) {
          my $accOprId = ToolCommon::GetAccessOperId($nodeLogFH, $snmpSession);
          if (defined($accOprId) && ($accOprId == $DT_ACCOPR)) {
		  $params->{IGMP_MAX_UNCFG} = 1; 
		  ToolCommon::Log($nodeLogFH, "Enable Migration of IGMP unconfigured multicast channels OK\n");
          }
      }

      #-- STEP 1e
      if (! $params->{TOECNTC} && ! $params->{TONANTB} && ! $params->{TOECNTE}) {
	# -- 1.4.2.2
	if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) < 0) {  # -- 1.1.3 -- 1.1.7
	  # Running R2.2 potential rollback issue so check it.
	  if (!Octopus::SwMgntIniSizeOK($params->{DEBUG}, $dir, $isam, $swMgntIniSize, $altSwMgntIniSize, $dirName)) {
	    FamilyCommon::IncrErrorCount("",
					 EC_SW_MGNT_SIZE_NOK,
					 sprintf(": %s", Octopus::GetError()),
					 1);
	    return undef;
	  }
	}
      }
      $xvpsConfig = undef;
      $xvpsDependancy = undef;
      $nameList = "";
      # Step 1f check whether ISAM is ISAM-V server and store for later usage
      ToolCommon::Log($nodeLogFH, "Checking whether ISAM is ISAM-V server...");
      $isamIsIsamV = ToolCommon::IsamHasVoiceServer($cli);
      if (!defined($isamIsIsamV)) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s", ToolCommon::GetError()),
				     EC_SHOW_EQPT_FAILED,
				     undef,
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s server\n", $isamIsIsamV ? "is ISAM-V" : "isn't ISAM-V"));
      }
      $params->{IS_ISAMV_SRV} = $isamIsIsamV;
      # Step 1f check whether ISAM is ISAM-V server and store for later usage
      ToolCommon::Log($nodeLogFH, "Checking whether ISAM has voice boards...");
      my $isamHasV = ToolCommon::IsamHasVoice($cli);
      if (!defined($isamHasV)) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s", ToolCommon::GetError()),
				     EC_SHOW_EQPT_FAILED, 
				     undef,
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, sprintf("  ISAM %s voice boards\n", $isamHasV ? "has" : "hasn't"));
      }
      $params->{ISAM_HAS_VOICE} = $isamHasV;

      if ($isamIsIsamV) {
	if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
	  if (! -e "$toolData->{TFTPDIR}/$toolData->{FXRDATA}->{XVPS_HOMEDIR}") {
	    my $checkDir = $toolData->{TFTPDIR};
	    my @dirList = split('/', $toolData->{FXRDATA}->{XVPS_HOMEDIR});
	    foreach my $entry (@dirList) {
	      $checkDir .= "/$entry";
	      ToolCommon::Log($nodeLogFH, "Creating '$checkDir'\n") if $params->{DEBUG};
	      if (! -e $checkDir) {
		if (! mkdir($checkDir, 0777)) {
		  FamilyCommon::IncrErrorCount("  Failed to create '$checkDir': $!",
					       EC_MKDIR_FAILED, 
					       undef,
					       1);
		  return undef;
		}
                if ($params->{UMASK_CHMOD} and ! chmod(01777, "$checkDir")) {
                    ToolCommon::Log($nodeLogFH,  "Failed to chmod 1777 for directory '$checkDir'\n") if $params->{DEBUG};
                }
	      }
	    }
	  }
	}
	if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) &&
	    (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) >= 0)) {
	  if (! $params->{IVPS_MIG_POSSIBLE}) {
	    FamilyCommon::IncrErrorCount("",
					 EC_ISAM_HAS_VOICE,
					 undef,
					 1);
	    return undef;
	  }
	} else {
	  if (! -f "$toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE}") {
	    FamilyCommon::IncrErrorCount("New CDE file $toolData->{TFTPDIR}/$toolData->{CDEDIR}/$toolData->{NEWCDEFILE} not found or format not defined properly.!! Check param.cfg\n",
					 EC_NEW_CDE_NOTFOUND,
					 undef,
					 1);
	    return undef;
	  }
	}
      }

      #-- Check if automatic DB activation after download is configured
      if (! defined($params->{OFLDB})) {
         my $retCode =  IsamCli::CheckDBAutoActivate($cli);
	 if (defined($retCode) && $retCode) {
	    FamilyCommon::IncrErrorCount("",
					 EC_DB_AUTO_ACTIVATE_ON_DWLD,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    return undef;
         }
      }

      #-- STEP 2: Save SHUB Database
      if (! defined($params->{OFLDB})) {
	if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) < 0) {
	  # From R2.3 onwards there seems to be a possible issue with an SHUB DB save followed by an NT DB upload.
	  # Since an internal SHUB DB save is done starting from R2.3 onwards we will no longer do this.
	  if (!SaveShubDbase($nodeLogFH, $params->{DEBUG}, $cli)) {
	    FamilyCommon::IncrErrorCount("",
					 EC_SHUB_DB_SAVE_FAIL,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    return undef;
	  }
	  sleep $params->{SECUREDBSAVE};
	}

    Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
    return undef if( ToolCommon::IsTransferOnGoing() );

	#-- STEP 3: Backup NT
	ToolCommon::Log($nodeLogFH, "Backup NT Database...");
	my $dbSize;
	if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) < 0) {
	  $ret = UploadCompleteDbTftp($nodeLogFH, $params->{DEBUG}, $params->{PRIMTFTPIP}, $toolData->{TFTPDIR} . "/" . $thisTftpDir, $isam);
	  if (!$ret) {
	    FamilyCommon::IncrErrorCount("",
					 EC_NT_DB_BU_FAIL,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    return undef;
	  }
	} else {
	  my $time1 = time();
          if(!$params->{NOTFTP}){
            $ret = IsamCli::UploadDb($cli, 'actual-active',
                                 $params->{PRIMTFTPIP}, $thisTftpDir, 1);
          } else {
            $ret = IsamCli::UploadDb($cli, 'actual-active',
                                  $params->{PRIMTFTPIP}, $toolData->{FXRDATA}{HOMEDIR}, 1);
          }
	  if (!defined($ret) || !$ret)  {
	    FamilyCommon::IncrErrorCount(sprintf("  FAILED (%s)\n", IsamCli::GetError()),
					 EC_NT_DB_BU_FAIL,
					 sprintf(": %s", IsamCli::GetError()),
					 1);
	    return undef;
	  } else {
	    my $realName = "$toolData->{FXRDATA}{HOMEDIR}/$NTINFILENAME";
	    ToolCommon::Log($nodeLogFH, "Uploaded DB into '$realName'\n") if $params->{DEBUG};
       if (! -e "$toolData->{FXRDATA}{HOMEDIR}/$NTINFILENAME") {
 		FamilyCommon::IncrErrorCount(sprintf("  NT DB not present\n"),
				       EC_NT_DB_NOT_PRESENT,
				       undef,
				       1);
	  	return undef;
       }
	    $dbSize = -s "$realName";
	    ToolCommon::Log($nodeLogFH, sprintf("  OK, transfered $dbSize bytes in %d seconds\n", time() - $time1));
	  if ($dbSize == 0) {
	 	 FamilyCommon::IncrErrorCount(sprintf("  Size of database file 0\n"),
				       EC_NT_DB_EMPTY,
				       undef,
				       1);
	 	 return undef;	
	  }	
	}
      }
	  
	if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 3), REGEXP, 0) < 0) {
	  #-- STEP 4: Backup SHUB
	  #-- 1.1.3: Only needed for R2.2 or lower.  From R2.3 onwards Lanx DB is present in dm_complete.tar
	  if (! FamilyCommon::CreateBUFile($parentLogFh, $nodeLogFH, $dir, $SHUBINFILENAMER1, $newLog, $childNum, $params->{DEBUG})) {
	    FamilyCommon::IncrErrorCount("",
					 EC_CREATE_ISS_FAIL,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    FamilyCommon::ToolExit(CREATE_ISS_CONF_FAIL_CODE);
	  }
	  if (! FamilyCommon::CreateBUFile($parentLogFh, $nodeLogFH, $dir, $SHUBNVRAMFILENAME, $newLog, $childNum, $params->{DEBUG})) {
	    FamilyCommon::IncrErrorCount("",
					 EC_CREATE_NVR_FAIL,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    FamilyCommon::ToolExit(CREATE_NVRAM_FAIL_CODE);
	  }
	  $ret = BackupShub($nodeLogFH, $cli, $isam, $oswpInfo, $params->{PRIMTFTPIP}, $userName);  # -- 1.1.6, 1.1.7
	  if (!$ret) {
	    FamilyCommon::IncrErrorCount("",
					 EC_SHUB_BU_FAIL,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    return undef;
	  }
	}
      } else {
	my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
	my $realName = "$toolData->{FXRDATA}{HOMEDIR}/$dbName";
	ToolCommon::Log($nodeLogFH, "Copy $params->{OFLDB} to $realName...\n") if $params->{DEBUG};
	if (! copy($params->{OFLDB}, $realName)) {
	  FamilyCommon::IncrErrorCount("Copy of '$params->{OFLDB}' to '$realName' failed: $!",
				       EC_COPY_OFLDB_FAILED,
				       undef,
				       1);
	  return undef;
	}
      }
    } else {
      my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
      my $realName = "$toolData->{FXRDATA}{HOMEDIR}/$dbName";
      ToolCommon::Log($nodeLogFH, "Copy $params->{OFLDB} to $realName...\n") if $params->{DEBUG};
      if (! copy($params->{OFLDB}, $realName)) {
	FamilyCommon::IncrErrorCount("Copy of '$params->{OFLDB}' to '$realName' failed: $!",
				     EC_COPY_OFLDB_FAILED,
				     undef,
				     1);
	return undef;
      }
      $isamIsIsamV = DetermineXvpsPresence($nodeLogFH, $dir);
      if (! defined($isamIsIsamV)) {
             FamilyCommon::IncrErrorCount("",
                                           EC_IVPS_DB_FAIL,
                                           sprintf(": %s", FamilyCommon::GetError()),
                                           1);
              return undef;
      } else {
              $params->{IS_ISAMV_SRV} = $isamIsIsamV;
      }
    }

    my $communityStrings = DetermineNtCommunities($nodeLogFH, $dir);
    if (! defined($communityStrings)) {
      FamilyCommon::IncrErrorCount("",
				   EC_GETCOMMTY_ERROR,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return undef;
    }

    if ($isamIsIsamV &&  (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) >= 0)) {
      if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0) {
	if (-e "$dirName/IvPsLiSt.txt") {
	  $xvpsConfig = MakeFakeList($nodeLogFH, "$dirName/IvPsLiSt.txt");
	} else {
	  if (! defined($params->{OFLDB})) {  # ANTmt22054
	    $xvpsConfig = GetIvpsIpAddresses($nodeLogFH, $isam);
	  } else {
	    FamilyCommon::IncrErrorCount("Migration with offline DB requested for ISAM-V, please create IvPsLiSt.txt\n",
					 EC_ISAMV_OFLMIG_NO_CONF,
					 undef,
					 1);
	    return undef;
	  }
	}
	if (! defined($xvpsConfig)) {
	  FamilyCommon::IncrErrorCount("",
				       EC_IVPS_GETIP_ERROR,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	  return undef;
	} else {
	  $xvpsDependancy = GetXvpsDependancyByIP($nodeLogFH, $xvpsConfig);
	  if (! defined($xvpsDependancy)) {
	    FamilyCommon::IncrErrorCount("",
					 EC_IVPS_GETDEP_ERROR,
					 sprintf(": %s", FamilyCommon::GetError()),
					 1);
	    return undef;
	  } else {
	    foreach my $ivpsId (sort keys (%{$xvpsDependancy})) {
	      foreach my $key (sort keys %{$xvpsDependancy->{$ivpsId}}) {
		if ($xvpsDependancy->{$ivpsId}->{$key} eq "") {
		  $nameList .= "??$key??, ";
		} else {
		  $nameList .= "$key ($xvpsDependancy->{$ivpsId}->{$key}), ";
		}
	      }
	    }
	    $nameList = substr($nameList, 0, -2) if $nameList ne "";
	  }
	}
      } else {
	$xvpsConfig = CopyXvpsList($nodeLogFH, $communityStrings);
	$xvpsDependancy = GetXvpsDependancyByCommunity($nodeLogFH, $communityStrings, $isam->[0]);
	if (! defined($xvpsDependancy)) {
	  FamilyCommon::IncrErrorCount("",
				       EC_IVPS_GETDEP_ERROR,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	  return undef;
	} else {
	  foreach my $ivpsId (sort keys (%{$xvpsDependancy})) {
	    foreach my $key (sort keys %{$xvpsDependancy->{$ivpsId}}) {
	      if ($xvpsDependancy->{$ivpsId}->{$key} eq "") {
		$nameList .= "??$key??, ";
	      } else {
		$nameList .= "$key ($xvpsDependancy->{$ivpsId}->{$key}), ";
	      }
	    }
	  }
	  $nameList = substr($nameList, 0, -2) if $nameList ne "";
	}
      }
    }

    if ($params->{VLANSTATBL}) {
      # Check vlanStaTbl for randam init-values: ANTmt23353
      if (! PrepareVlanStaTbl($nodeLogFH, $dir, $activeRelNum, $dirName)) {
	FamilyCommon::IncrErrorCounf("",
				     EC_CORR_VLAN_TBL_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    }

    #-- STEP 5a: Migrate the NT database
    $ret = MigrateNTDbase($nodeLogFH, $dir, $fullActiveRelNum, $newFullRelNum, $dirName);  # -- 1.5..1.3
    if (!$ret) {
      FamilyCommon::IncrErrorCount("",
				   EC_NT_MIG_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return undef;
    }

    #-- Added BB 1.1.1
    #-- STEP 5b: Extract IGMPXmlfile.txt in case of migrating from R2.2 to a higher releaseUploadCompleteDb.
    #            Not required when migrating from R3.1 or higher
    if ((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) >= 0) && 
	(Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 1), REGEXP, 0) < 0) && 
	(Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) > 0)) {
      ToolCommon::Log($nodeLogFH, "Extracting IGMPXmlfile.txt...");
      $ret = ExtractIGMPXmlFile($nodeLogFH, $dir, $activeRelNum, $dirName); # -- 1.4.2.2 $newRelNum -> $activeRelNum
      if (!$ret) {
	# Error already reported in ExtractIGMPXmlFile routine.
	FamilyCommon::IncrErrorCount("",
				     EC_EXTR_MCAST_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    }
    #-- Enable Hairpin feature only for 36g and 36k releases.
    if (! defined($params->{OFLDB}) ||
	(defined($params->{OFLDB}) && ! $params->{OFLONLY})) {
	    if(($oswpInfo->{'ACTIVE'}{'NAME'} =~ /$R3604G/) || ($oswpInfo->{'ACTIVE'}{'NAME'} =~ /$R3604K/)) {
	       $U2UHairPin = 1;     ## U2U hairpin enabled for R36 emergency releases
	    } 
    }
    #-- STEP 6: Migrate the SHUB database
    # SHUB migration filas in case we go to the same release
    $ret = MigrateSHUBDbase($nodeLogFH, $dir, $fullActiveRelNum, $newFullRelNum,
			    defined($params->{OFLDB}) ? $params->{OFL_NT_TYPE} : $ntInfo->{$activeNt}{TYPE}, $dirName, $U2UHairPin);  # -- 1.1.3
    if (!$ret) {
      FamilyCommon::IncrErrorCount("",
				   EC_SHUB_MIG_FAILED,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      return undef;
    }

    #-- STEP 6 bis: migration IVPS database, from R3.5 onwards
    if (((Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 3), REGEXP, 0) >= 0))
	&&
	(Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) >= 0)
	&&
	$isamIsIsamV) {
      if (defined($params->{OFLDB}) && (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3, 5), REGEXP, 0) < 0)) {
	# No migration of an offline stored DB if backup is not from a R3.5 system.
      } else {
	$ret = MigrateXvpsDbase($parentLogFh, $nodeLogFH, $isam, $fullActiveRelNum, $newFullRelNum, $xvpsConfig, $toolData->{FXRDATA}{XVPS_HOMEDIR},
				$childNum, $userName, $newLog, $dirName);
	if (! $ret) {
	  FamilyCommon::IncrErrorCount("",
				       EC_IVPS_MIG_FAILED,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	  return undef;
	}
      }
    }

    #-- STEP 7: Combine both databases
    #-- From R3.5: ISAM-V migration, so also CDE needs to be included.
    if (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) > 0) {
      $ret = CombineDbases($nodeLogFH, $dir,
#			   $activeRelNum / 100 . "." . ($activeRelNum % 100) / 10,
#			   $newRelNum / 100 . "." . ($newRelNum % 100) /10, $xvpsConfig); # -- 1.1.4
			   $fullActiveRelNum, $newFullRelNum, $xvpsConfig, "$toolData->{TFTPDIR}/$toolData->{FXRDATA}{XVPS_HOMEDIR}"); # -- 1.1.4
      if (!$ret) {
	FamilyCommon::IncrErrorCount("",
				     EC_DB_COMBINE_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    } else {
      $ret = CopyNTDbase($nodeLogFH, $dir);
      if (! $ret) {
	FamilyCommon::IncrErrorCount("",
				     EC_NT_DB_COPY_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    }                                                    # -- 1.1.7

    if ($params->{OFLONLY}) {
      my $dbName = defined($params->{DMTAR}) ? $NTSMALLFILE : $NTINFILENAME;
      ToolCommon::Log($nodeLogFH, "Chosen to only do the offline migration\n", 1);
      ToolCommon::Log($nodeLogFH, "  migrated databases for this ISAM found in\n", 1);
      if (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) > 0) {
	ToolCommon::Log($nodeLogFH, sprintf("  NT and SHUB: %s/newdb/$dbName\n",
					    defined($params->{OFLDB}) ? FamilyCommon::GetNeStorage() : $toolData->{FXRDATA}{HOMEDIR}), 1);
      } else {
	ToolCommon::Log($nodeLogFH, "  NT: $toolData->{TFTPDIR}/$thisTftpDir/newdb/$dbName\n", 1);
	ToolCommon::Log($nodeLogFH, "  SHUB: $dir/$SHUBOUTFILENAME\n", 1);
      }
      ToolCommon::Log($nodeLogFH, "  Dependant nodes related to this ISAMV: $nameList\n", 1) if ($nameList ne "");
      FamilyCommon::AdvanceToNext("");
      return undef;
    }

    #-- STEP 8a: Download new database to NT
    Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
    return undef if( ToolCommon::IsTransferOnGoing() );

    if(!$params->{NOTFTP}) {
      $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum, $thisTftpDir, $userName); # -- 1.1.6
    } else {
      $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum,"/". $toolData->{FXRDATA}{HOMEDIR}, $userName);
    }
    if (!$ret) {
      if (FamilyCommon::GetSigPipeReceived()) {
	# Witnessed at FT:
	# under certain circumstances no more telnet/CLI access is possible due to some problem on the unix side: the pipe
	# carrying the traffic to the node is 'gone' after we have performed:
	# - NT offline migration
	# - SHUB offline migration
	# - execute tar to combine the migrated databases.
	# Trying to write to this pipe results in a SIGPIPE.
	# Workaround: open 2nd CLI session and try again.  The transaction is still taken by 127.0.0.1 at the moment we get the new CLI session.
	ToolCommon::Log($nodeLogFH, "Open new CLI...");
	$cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	if (!defined($cli)) {
	  FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open new cli session for %s: %s\n",
					       FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				       EC_OPENCLI_FAILED,
				       sprintf(": %s", ToolCommon::GetError()),
				       1);
	  return undef;
	} else {
	  ToolCommon::Log($nodeLogFH, "  OK\n");
	}
	FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});
	FamilyCommon::ClearSigPipeReceived();

    Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
    return undef if( ToolCommon::IsTransferOnGoing() );

        if(!$params->{NOTFTP}) {
         $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum, $thisTftpDir, $userName); # -- 1.1.6
        } else {
         $ret = DownloadDbase($nodeLogFH, $cli, $isam, $activeRelNum,"/". $toolData->{FXRDATA}{HOMEDIR}, $userName);
        }
	if (!$ret) {
	  FamilyCommon::IncrErrorCount("",
				       EC_DB_DL_FAILED,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	  return undef;
	}
      } else {
	FamilyCommon::IncrErrorCount("",
				     EC_DB_DL_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    }

    #-- STEP 8b: Download SHUB database in case of migration to R2.2 -- 1.1.7 START
    if (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) == 0) {
      if (! Octopus::RemoveNtShubConf($params->{DEBUG}, $dir, $isam, $dirName)) {
	FamilyCommon::IncrErrorCount("",
				     EC_DEL_SHUB_SW_FAILED,
				     sprintf(": %s", Octopus::GetError()),
				     1);
	return undef;
      }
      sleep(10);
      if (!DownloadSHUBDbase($nodeLogFH, $params->{DEBUG}, $isam, $dir)) {
	FamilyCommon::IncrErrorCount("",
				     EC_SHUB_DB_DL_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	return undef;
      }
    }                                                                  # -- 1.1.7 END

    if ((uc($ntInfo->{NTA}{STATUS}) ne "EMPTY") && (uc($ntInfo->{NTB}{STATUS}) ne "EMPTY")) {
      #-- STEP 8c: Redundant system, wait for stability   Needed?
      ToolCommon::Log($nodeLogFH, "Waiting for stability...");
      sleep(180);   # Only for redundant system?
      ToolCommon::Log($nodeLogFH, " OK\n");
    }
  }

  if (! $params->{NOACT}) {
    #-- STEP 8d: suspend the tftp task to guarantee upgrade away from R2.2
    if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) == 0) {
      #-- 1.1.3 Only needed when going away from R2.2
      $ret = Octopus::SuspendTftp($params->{DEBUG}, $dir, $isam, $dirName);  # -- 1.1.6
      if (!$ret) {
	FamilyCommon::IncrErrorCount("",
				     EC_TFTP_SUSPEND_FAILED,
				     sprintf(": %s", Octopus::GetError()),
				     1);
	return undef;
      }
    }
  }
  ToolCommon::Log($nodeLogFH, "Reset transaction timeout and release transaction...");
  $ret = IsamCli::SetTransactionTimeout($cli, FamilyCommon::GetOldTransactionTO());
  if (! $ret) {
    FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", IsamCli::GetError()),
				 EC_SET_TRANS_TO_FAILED, 
				 sprintf(": %s", IsamCli::GetError()),
				 1);
    return undef;
  }
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  if ($params->{NOACT}) {
    # Save DB info in activation file for later activation purposes.
    # Print info on terminal and to file.
    ToolCommon::Log($nodeLogFH, "Process ends here, continue with migrate.pl --actonly...\n");
  } else {

	#-- STEP : Check if migration interrupt on emergency call feature is enabled
	if ($params->{EMERG_CALL_FEATURE}) {
		my $isActive = 0;
		ToolCommon::Log($nodeLogFH, "Retrieving the emergency-call alarms status...\n") if ($params->{DEBUG});
	    $isActive = ToolCommon::AlarmRetrieval($nodeLogFH, $cli, $toolData);
	    if($isActive) {													   ## emergency call alarm is active
			FamilyCommon::IncrErrorCount(sprintf("  FAILED: Emergency call max retries reached\n"),
				     EC_EMERGENCY_CALL_ACTIVE,
				     undef,
				     1);
		    return undef;
		 }
	}
	#-- STEP : End of migration interrupt on emergency call feature.

    my $canRetry = 0;
    ($ret, $canRetry, $cli) = IsamSystemRestart($nodeLogFH, $cli, $snmpSession, $dir, $isam, $ntInfo, $thisTftpDir,
						$fullActiveRelNum, $newRelNum, $newLog, $dirName, $oswpInfo, 0, $childNum);
    FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
    if (!$ret) {
      if ($canRetry > 0) {
	# Give it a second try but first check whether we still have a CLI
	if (! IsamCli::ConnectionStillOpen($cli)) {
	  $cli = ToolCommon::OpenCliWrap($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
	  if (!defined($cli)) {
	    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s (2nd node-restart run): %s\n",
						 FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
					 EC_OPENCLI_FAILED,
					 sprintf(": %s", ToolCommon::GetError()),
					 1);
	    return undef;
	  } else {
	    ToolCommon::Log($nodeLogFH, "  OK\n");
	  }
	  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
	}
	ToolCommon::Log($nodeLogFH, "Second run of system restart.  Node down time extended.  Please be patient...\n");
	($ret, $canRetry, $cli) = IsamSystemRestart($nodeLogFH, $cli, $snmpSession, $dir, $isam, $ntInfo, $thisTftpDir,
						    $fullActiveRelNum, $newRelNum, $newLog, $dirName, $oswpInfo, 1, $childNum);
	# Don't do a retry when hot reset was disabled at the first OSWP activation.  Any other NT reset should bring the new SW up
	if (! $ret) {
	  FamilyCommon::IncrErrorCount("",
				       EC_NEW_SW_NOT_ACT,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	  return undef;
	}
      } else {
	if ($canRetry < 0) {
	  FamilyCommon::IncrErrorCount("",
				       EC_NT_SHUB_SW_MM,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	} else {
	  FamilyCommon::IncrErrorCount("",
				       EC_NEW_SW_NOT_ACT,
				       sprintf(": %s", FamilyCommon::GetError()),
				       1);
	}
	return undef;
      }
    }

    #-- Re-configure SFP Assignments
    if ($isSfpAssigned == 1) {
      ToolCommon::Log($nodeLogFH, "Re-configure SFP Assignment...");
      $ret = IsamCli::ConfigureSfpAssignment ($cli, $sfpAssignmentTable);
      if (!defined($ret) || !$ret) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED (%s)\n", IsamCli::GetError()),
				     EC_SET_SFP_ASSIGNMENT_FAILED,
				     sprintf(": %s", IsamCli::GetError()),
				     1);
	return undef;
      } else {
	ToolCommon::Log($nodeLogFH, "  OK!\n");
      }
    }

    if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) < 0) {
      #-- 1.1.7 STEP 12: Change LANx settings to tftp
      #-- First check whether the Lanx SW is found where expected (only for R2.2)
      if (Capability::ReleaseCompare($newRelNum, Capability::ConvertRelease(2, 2), REGEXP, 0) == 0) {
	if (Octopus::ShubSwPresent($params->{DEBUG}, $dir, $isam, $SHUBLANXCHECK, $shubSwSize, $dirName) <= 0) {
	  FamilyCommon::IncrErrorCount("",
				       EC_SHUB_SW_NOT_FOUND,
				       sprintf(": %s", Octopus::GetError()),
				       1);
	  return undef;
	}
      }
	
      #-- Next change active conf
      my $activeConf = GetActiveLanxConf($cli, $newRelNum);
      my $newConf =  $activeConf eq "config-1" ? "config-2" : "config-1";
      ToolCommon::Log($nodeLogFH, "Select $newConf for SHUB...");
      $ret = SetAndCheckNewLanxConf($cli, $oswpInfo, $newConf);
      if (!defined($ret) || ! $ret) {
	FamilyCommon::IncrErrorCount("Select $newConf for SHUB failed.\n",
				     EC_SHUB_SET_NEW_CONF_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	if ($params->{CONTINUE}) {
	  return undef;
	} else {
	  ToolCommon::Log(FamilyCommon::GetParentLogFile(),
			  sprintf("Child %d: **FATAL** Restart of %s SHUB failed, please verify %s.\n",
				 $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	  FamilyCommon::ToolExit(SHUB_CONF_SELECT_FAIL_CODE);
	}
      }
      ToolCommon::Log($nodeLogFH, "  OK\n");
      $ret = RebootAndChangeLanxToTftp($nodeLogFH, $cli, $snmpSession, $isam, $oswpInfo, $newRelNum, $params->{SINGLERESTART});
      if (! $ret) {
	FamilyCommon::IncrErrorCount(sprintf("**FATAL** Restart of SHUB of %s failed.\n", FamilyCommon::GetFailedNode()),
				     EC_SHUB_REBOOT_FAILED,
				     sprintf(": %s", FamilyCommon::GetError()),
				     1);
	if ($params->{CONTINUE}) {
	  return undef;	  
	} else {
	  ToolCommon::Log($nodeLogFH,
			  sprintf("Child $childNum: **FATAL** Restart of SHUB of %s failed, please verify %s.\n",
				  $childNum, FamilyCommon::GetFailedNode(), $newLog), 1);
	  FamilyCommon::ToolExit(SHUB_RESTART_FAIL_CODE);
	}
      }

      $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, uc($ntInfo->{$activeNt}{TYPE}), undef,
					  $toolData->{RETRIES});
      if (! defined($oswpInfo)) {
	FamilyCommon::IncrErrorCount("",
				     EC_GET_OSWP_FAILED,
				     sprintf("(after SHUB tftp reboot): %s", ToolCommon::GetError()),
				     1);
	return undef;
      }
    }

    if (defined($params->{TTL0FWD})) {
      ToolCommon::Log($nodeLogFH, "Forward TTL=0 packets for VRF $params->{TTL0FWD}...");
      if (! IsamCli::ForwardTtl0Packets($cli, $params->{TTL0FWD})) {
	FamilyCommon::IncrErrorCount(sprintf("  FAILED (%s)\n", IsamCli::GetError()),
				     EC_TTL_0_FWD_FAILED,
				     sprintf(": %s", IsamCli::GetError()),
				     1);
	return undef;
      }
    }
    if (($config->{ACCESS_OPERATOR_ID} > 0) && ($config->{ACCESS_OPERATOR_ID} <= 65535) && ($params->{ACCESSOPRID})) {
      my $accOperId = ToolCommon::GetAccessOperId($nodeLogFH, $snmpSession);
      if (defined($accOperId)) {
	if ($config->{ACCESS_OPERATOR_ID} != $accOperId) {
	  FamilyCommon::IncrErrorCount(sprintf("Unexepected access operator ID: retrieved %d while expecting %d\n",
					       $accOperId, $config->{ACCESS_OPERATOR_ID}),
				       EC_ACC_OPERID_MISMATCH,
				       sprintf(":  retrieved %d while expecting %d", $accOperId, $config->{ACCESS_OPERATOR_ID}),
				       1);
	  return undef;
	}
      } else {
	FamilyCommon::IncrErrorCount(sprintf("Failed to retrieve the access-oper-id: %s\n", ToolCommon::GetError()),
				     EC_GET_ACC_OPERID_FAILED,
				     sprintf(": %s", ToolCommon::GetError()),
				     1);
	return undef;
      }
    }
  }
  IsamCli::AllowAlarms($cli);
  FamilyCommon::CloseCliSession2Close();  # Store for IncrErrorCount
  return $nameList;
}

=head2 CheckEnvironment($$$$)

=begin html
Check the migration environment.  In case unknown offline migration tools are detected (based on their version number) or files not being executable
the script will abort and not start any activity.

=end html

=cut

sub CheckEnvironment($$$) {
  my ($logFile, $config, $installDir) = @_;
  my $envOk = 1;
  my $toolConfigData = FamilyCommon::GetToolData();
  my $oflVersInfo = Utils::DetermineOFLMTVersions($params->{DEBUG}, $toolConfigData->{NTDBTOOL}, $toolConfigData->{LANXDBTOOLDIR},
						  $toolConfigData->{LANXDBTOOL}, $toolConfigData->{IVPSDBTOOL}, $config, $installDir);

  $params->{IVPS_MIG_POSSIBLE} = 0;
  ToolCommon::Log($logFile, sprintf("NE families: from %s, to %s (%s)\n",
				    defined($toolData->{FROMNEFAMILY}) ? $toolData->{FROMNEFAMILY} : "undef",
				    defined($toolData->{TONEFAMILY}) ? $toolData->{TONEFAMILY} : "undef",
				    defined($toolData->{HOSTS}) ? "defined" : "undef")) if $params->{DEBUG};
  if (! $oflVersInfo->{RET}) {
    if ($oflVersInfo->{NT} ne "") {
      if ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {
	$params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	$params->{NTOFLMTBLDNUM} = $4;
      } elsif ($oflVersInfo->{NT} =~ /.*V(\d{2})\d{0,2}\.(\d+)\.(\d+).*/) {
	$params->{NTOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
      } else {
	$params->{NTOFLMTVERS} = 0;
        if($oflVersInfo->{NT} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:.*/) {
	  $NTOFLMTVERSION = $1 * 100 + ($2 * 10) + $3; 
        } elsif ($oflVersInfo->{NT} =~ /.*V(\w)+\s+Build:\s+(\d+\.\d+)/) { 
	  $params->{NTOFLMTBLDNUM} = $2;
          if($params->{NTOFLMTBLDNUM} =~ /(\d{4})\.(\d+)/) {
	      $params->{NTOFLMTBLD} = $1 * 1000 + $2;
          } 
        } else { 
          ToolCommon::Log($logFile, "No NT OFLMT version could be obtained\n", 1) if $params->{DEBUG};
        }
      }
    } else {
      $params->{NTOFLMTVERS} = 0;
      ToolCommon::Log($logFile, "No NT OFLMT version could be obtained\n", 1) if $params->{DEBUG};
    }
    if (-f "$installDir/$toolConfigData->{IVPSDBTOOL}") {
      if ($oflVersInfo->{IVPS} ne "") {
	$params->{IVPS_MIG_POSSIBLE} = 1;
	if ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+)\s+Build:\s+(\d+\.\d+)/) {
	  $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	  $params->{IVPSOFLMTBLDNUM} = $4;
	} elsif ($oflVersInfo->{IVPS} =~ /.*V(\d+)\.(\d+)\.(\d+).*/) {
	  $params->{IVPSOFLMTVERS} = $1 * 100 + ($2 * 10) + $3;
	} else {
	  $params->{IVPSOFLMTVERS} = 0;
	  ToolCommon::Log($logFile, "   No xVPS OFLMT version could be obtained\n", 1) if $params->{DEBUG};
	}
	if (! -x $toolConfigData->{IVPSINVENTORY}) {
	  ToolCommon::Log($logFile, "   ISAM-V migration warning: no xVPS inventory script found or not executable\n", 1)
	    if $params->{DEBUG};
#	  $params->{IVPS_MIG_POSSIBLE} = 0;   Don't.  We'll see this later.  This script is only needed in case ISAM runs 3.3 or 3.4
#	  $params->{IVPSOFLMTVERS} = 0;
	}
      } else {
	ToolCommon::Log($logFile, "   !!! ISAM-V migration not possible: no xVPS OFLMT version could be obtained\n", 1) if $params->{DEBUG};
      }
    } else {
      $params->{IVPSOFLMTVERS} = 0;
      $params->{IVPS_MIG_POSSIBLE} = 0;
    }
    if ($oflVersInfo->{SHUB} =~ /.*(\d+)\.(\d+).*/) {
      $params->{SHUBOFLMTVERS} = ($1 * 100) + $2;
      if ((Capability::ReleaseCompare(basename($params->{OSWP_FULL}),
				      Capability::ConvertRelease(4, 0),
				      REGEXP, 0, undef) >= 0) &&
	  ($params->{SHUBOFLMTVERS} < 149)) {
	if ($params->{FORCEMIG}) {
	  ToolCommon::Log($logFile, "1.49 of the SHUB OFLMT is at least required, forced to continue.  Migrated DB may not be usable~\n", 1);
	} else {
	  exit SHUB_OFLMT_VER_TOO_LOW;
	}
      }
    } else {
      ToolCommon::Log($logFile, "No SHUB OFLMT version could be obtained... exiting\n", 1) if $params->{DEBUG};
      $envOk = 0;
      exit NO_SHUB_OFLMT_VER_CODE;
    }
    if (!defined($params->{MAINSTREAM})) {
      if ($params->{SHUBOFLMTVERS} > Capability::HighestShubOflmtVersion($toolData->{TONEFAMILY})) {
	ToolCommon::Log($logFile, 
			sprintf("Version number of SHUB OFLMT higher then the one known to this toolset (%s vs %s)... exiting\n",
				$oflVersInfo->{SHUB},
				Capability::ShubOflmtVersionString(Capability::HighestShubOflmtVersion($toolData->{TONEFAMILY}))),
			1);
	$envOk = 0;
	exit SHUB_OFLMT_VERSION_2_HIGH_CODE;
      }
    } else {
      ToolCommon::Log($logFile, "Running on mainstream, no SHUB OFLMT version check done...\n", 1) if ($params->{DEBUG});
    }
    ToolCommon::Log($logFile, sprintf("NT Versions: %s, %s\n", Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
				      Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion($toolData->{TONEFAMILY}))),
		    1) if ($params->{DEBUG});
    if (!defined($params->{MAINSTREAM})) {
      if (defined($params->{NTOFLMTVERS}) && ($params->{NTOFLMTVERS} > Capability::HighestNtOflmtVersion($toolData->{TONEFAMILY}))) {
	ToolCommon::Log($logFile, 
			sprintf("Version number of NT OFLMT higher then the one known to this toolset (%s vs %s)... exiting\n",
				Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
				Capability::NtOflmtVersionString(Capability::HighestNtOflmtVersion($toolData->{TONEFAMILY}))),
			1);
	$envOk = 0;
	exit NT_OFLMT_VERSION_2_HIGH_CODE;
      }
    ToolCommon::Log($logFile, sprintf("NT Build : %s, %s\n", Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
				      Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild($toolData->{TONEFAMILY}))),
		    1) if (defined($params->{NTOFLMTBLD}) && ($params->{DEBUG}));

      if ((Capability::ReleaseCompare(basename($params->{OSWP_FULL}),Capability::ConvertRelease(4, 5, 5),REGEXP, 0, 1) < 0)) {
       if (defined($params->{NTOFLMTBLD}) && ($params->{NTOFLMTBLD} > Capability::HighestNtOflmtBuild($toolData->{TONEFAMILY}))) {
	 ToolCommon::Log($logFile, 
			sprintf("Build number of NT OFLMT higher then the one known to this toolset (%s vs %s)... exiting\n",
				Capability::NtOflmtBuildString($params->{NTOFLMTBLD}),
				Capability::NtOflmtBuildString(Capability::HighestNtOflmtBuild($toolData->{TONEFAMILY}))),
			1);
	 $envOk = 0;
	 exit NT_OFLMT_VERSION_2_HIGH_CODE;
       }
      }
    } else {
      ToolCommon::Log($logFile, "Running on mainstream or no version could be obtained, no NT OFLMT version check done...\n", 1)
	if ($params->{DEBUG});
    }
    ToolCommon::Log($logFile, sprintf("xVPS Versions: %s, %s\n", Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
				      Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion($toolData->{TONEFAMILY}))), 1)
      if ($params->{DEBUG});
    if (!defined($params->{MAINSTREAM})) {
      if (defined($params->{IVPSOFLMTVERS}) && ($params->{IVPSOFLMTVERS} > Capability::HighestXvpsOflmtVersion($toolData->{TONEFAMILY}))) {
	ToolCommon::Log($logFile, 
			sprintf("Version number of xVPS OFLMT higher then the one known to this toolset (%s vs %s)... exiting\n",
				Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
				Capability::NtOflmtVersionString(Capability::HighestXvpsOflmtVersion($toolData->{TONEFAMILY}))),
			1);
	$envOk = 0;
	exit IVPS_OFLMT_VERSION_2_HIGH_CODE;
      }
    } else {
      ToolCommon::Log($logFile, "Running on mainstream or no version could be obtained, no xVPS OFLMT version check done...\n", 1)
	if ($params->{DEBUG});
    }
  } else {
    ToolCommon::Log($logFile, sprintf("Determining version info failed: %s\n", Utils::GetError()), 1);
    $envOk = 0;
    exit $oflVersInfo->{RET};
  }
  ### Starting shub version 1.67 only Lanx Checksum exe CalcIpCheckSum_BE is available for sparc platform.
  if (($Config{archname} =~ /sun4/) && ($params->{SHUBOFLMTVERS} < 167)) { 
     $toolConfigData->{LANXIPCKSUM} = "CalcIpCheckSum"; 
  }
  if (! -x "$installDir/$toolConfigData->{LANXDBTOOLDIR}/$toolConfigData->{LANXIPCKSUM}") {
    $envOk = 0;
    ToolCommon::Log($logFile, "$installDir/$toolConfigData->{LANXDBTOOLDIR}/$toolConfigData->{LANXIPCKSUM} is not executable\n", 1);
    exit CALC_IP_CKSUM_NO_EXEC_CODE;
  }
  
  my $retC = Utils::VerifyOctopusEnv($params->{DEBUG}, $toolConfigData->{OCTOPUSPATH}, $config, $installDir);
  if ($retC) {
    ToolCommon::Log($logFile, Utils::GetError(), 1);
    $envOk = 0;
    exit $retC;
  }
  return ($oflVersInfo, $envOk);
}

=head2 PrintToolVersions($$$)

=begin html
Print the migration tool versions contained in $oflVersInfo

=end html

=cut

sub PrintToolVersions($$) {
  my ($logFile, $oflVersInfo) = @_;

  if (! $oflVersInfo->{RET}) {
    ToolCommon::Log($logFile, "**** Offline migrationtool version info ****\n", );
    if ($params->{NTOFLMTVERS} == 0) {
      if($NTOFLMTVERSION == 0) {
        ToolCommon::Log($logFile, "No version information could be obtained for the NT tool\n", 1);
      } else {
	ToolCommon::Log($logFile, sprintf("NT OFLMT version  : %s\n", Capability::NtOflmtVersionString($NTOFLMTVERSION)), 1);
      }
    } else {
      if (defined($params->{NTOFLMTBLDNUM})) {
	ToolCommon::Log($logFile, sprintf("NT OFLMT version  : %s Build: %s\n", Capability::NtOflmtVersionString($params->{NTOFLMTVERS}),
					  $params->{NTOFLMTBLDNUM}),
			1);
      } else {
	ToolCommon::Log($logFile, sprintf("NT OFLMT version  : %s\n", Capability::NtOflmtVersionString($params->{NTOFLMTVERS})),
			1);
      }
    }
    if ($params->{IVPSOFLMTVERS} == 0) {
      ToolCommon::Log($logFile, "No version information could be obtained for the xVPS tool\n", 1);
    } else {
      if (defined($params->{IVPSOFLMTBLDNUM})) {
	ToolCommon::Log($logFile, sprintf("xVPS OFLMT version  : %s Build: %s\n", Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS}),
					  $params->{IVPSOFLMTBLDNUM}), 1);
      } else {
	ToolCommon::Log($logFile, sprintf("xVPS OFLMT version: %s\n", Capability::NtOflmtVersionString($params->{IVPSOFLMTVERS})),
			1);
      }
    }
    ToolCommon::Log($logFile, "SHUB OFLMT version: $oflVersInfo->{SHUB}\n", 1);
  }
}

=head2 DetermineSwVersion($$)

=begin html
Determines the SW version based on the NT type and DB version.

=end html

=cut

sub DetermineSwVersion($) {
  my ($fh) = @_;
  my $relNum;

  if ($params->{OFL_DB_VERS} =~ /C4.302/) { # R4302g 
    $params->{SRC_OSWP} = sprintf("L6GP%s43.001", Capability::IsamEtsiOswpVariant(4320));
  } elsif ($params->{OFL_DB_VERS} =~ /B4.220/) {
    $params->{SRC_OSWP} = sprintf("L6GP%s42.001", Capability::IsamEtsiOswpVariant(4220));
  } elsif ($params->{OFL_DB_VERS} =~ /C4.230/) {
    $params->{SRC_OSWP} = sprintf("L6GP%s42.001", Capability::IsamEtsiOswpVariant(4230));
  } elsif ($params->{OFL_DB_VERS} =~ /B3.350/) {
    # Fix in starting from R3.3.04
    $params->{SRC_OSWP} = sprintf("L6GP%s33.400", Capability::IsamOswpVariant(3300));
  } elsif ($params->{OFL_DB_VERS} =~ /B3.450/) {
    # Version is B3.450
    $params->{SRC_OSWP} = sprintf("L6GP%s34.200", Capability::IsamOswpVariant(3400));
  } else {
    my $dbVersionRel = substr( $params->{OFL_DB_VERS}, 1);
    $dbVersionRel =~ s/\.//;
    my $dbVersionMajor = substr( $dbVersionRel, 0, 2);
    my $dbVersionMinor = substr( $dbVersionRel, 2, 2);
    my $buildNo = sprintf( "%03d", (($dbVersionMinor * 200) + 1) );
    $params->{SRC_OSWP} = sprintf("L6GP%s%d.%s", Capability::IsamEtsiOswpVariant($dbVersionMajor."00"), $dbVersionMajor, $buildNo );
  }
}

=head2 SetToolData()

=begin html
Initializes the common data pointer of the tool

=end html

=cut

sub SetToolData() {
  
  $toolData = FamilyCommon::GetToolData();

}

1;
